﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Search;
using Azteca.Kernel.Business;
using Azteca.Kernel.Printing;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Contabilidad;
using System.Data;

namespace Azteca.Web.Controllers.Contabilidad
{
    [Route("api/contabilidad/comprobantes")]
    public class ComprobantesController : DocumentController<ComprobanteFiscalInfo, ComprobanteFiscalDetail, ComprobanteFiscalService, ComprobanteFiscalFacade>
    {
        public ComprobantesController()
            : base(new ComprobanteFiscalFacade())
        {

        }

        [HttpGet]
        [Route("load-list-addendas")]
        public DataTable LoadListAddendas(int IdEmpresa)
        {
            try
            {
                return Facade.LoadListAddendas(this.SessionID, IdEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener el listado de addendas.", ex);
                return null;
            }
        }

        [HttpGet]
        [Route("search-comprobantes")]
        public List<ComprobanteFiscalItem> SearchKeyword(int idBranch, string keyword, int idContribuyente)
        {
            List<ComprobanteFiscalItem> dtRet = null;
            try
            {
                ComprobanteFiscalFacade Facade = new ComprobanteFiscalFacade();
                dtRet = Facade.BuscarComprobantes(SessionID, idBranch, keyword, idContribuyente);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener comprobantes", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("cancelar")]
        public string Cancelar(string idComprobanteFiscal)
        {
            try
            {
                string Acuse = Facade.Cancelar(SessionID, idComprobanteFiscal);
                return Acuse;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("cancelar-folio")]
        public string CancelarFolio(int idSucursal, string uuid, string qrString, bool validaCancelacion)
        {
            try
            {
                byte[] bytes = System.Convert.FromBase64String(qrString);
                qrString = System.Text.Encoding.UTF8.GetString(bytes);

                string Acuse = Facade.Cancelar(this.SessionID, idSucursal, uuid, qrString, validaCancelacion);
                return Acuse;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("pdf-temporal")]
        public FileLink GenerarPdfTemporal(string idComprobanteFiscal)
        {
            try
            {
                FileLink Uri = Facade.GenerarPdfTemporal(SessionID, idComprobanteFiscal);
                return Uri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("regenerar-pdf-temporal")]
        public FileLink ReGenerarPdfTemporal(string idComprobanteFiscal)
        {
            try
            {
                FileLink Uri = Facade.ReGenerarPdfTemporal(SessionID, idComprobanteFiscal);
                return Uri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("pdf-temporal-multiple")]
        public List<string> GenerarPdfTemporalList([FromBody]List<string> comprobantes)
        {
            try
            {
                List<string> Uri = Facade.GenerarPdfTemporal(SessionID, comprobantes);
                return Uri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("pdf-print")]
        public byte[] GenerarPrintPdf(string idComprobanteFiscal)
        {
            try
            {
                byte[] pdf = Facade.GenerarPrintPdf(SessionID, idComprobanteFiscal);
                return pdf;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("pdf-temporal-arr")]
        public FileLink GenerarPdfTemporalArr([FromBody] List<string> idComprobanteFiscal)
        {
            try
            {

                FileLink Uri = Facade.GenerarPdfTemporalArr(SessionID, idComprobanteFiscal);
                return Uri;

            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


        [HttpGet]
        [Route("xml")]
        public string GetXml(string idComprobanteFiscal)
        {
            try
            {
                string xml = Facade.GetXml(SessionID, idComprobanteFiscal);
                return xml;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return string.Empty;
            }
        }

        [HttpGet]
        [Route("xml-file")]
        public FileLink GetXmlFile(string idComprobanteFiscal)
        {
            try
            {
                FileLink uri = Facade.GetXmlFile(SessionID, idComprobanteFiscal);
                return uri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("xml-file-arr")]
        public FileLink GetXmlFileArr([FromBody] List<string> idComprobanteFiscal)
        {
            try
            {
                FileLink uri = Facade.GetXmlFileArr(SessionID, idComprobanteFiscal);
                return uri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }
        

        [HttpPost]
        [Route("descargar-lote")]
        public LoteImpresionInfo LoteDescargaGenerar([FromBody] List<string> idComprobantes)
        {
            try
            {
                LoteImpresionInfo lote = Facade.LoteDescargaGenerar(SessionID, idComprobantes);
                return lote;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


        [HttpPost]
        [Route("exportar-lote")]
        public int LoteExportarGenerar([FromBody] List<string> idComprobantes)
        {
            int IdLote = 0;
            try
            {
                IdLote=Facade.LoteExportarGenerar(SessionID, idComprobantes);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return IdLote;
        }

        [HttpPost]
        [Route("exportar-lote-status")]
        public Azteca.Kernel.Queue.Job LoteExportarStatus(int IdLote)
        {
            try
            {
                return Facade.LoteExportarStatus(SessionID, IdLote);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


        [HttpGet]
        [Route("timbrar")]
        public void Timbrar(string idComprobanteFiscal)
        {
            try
            {
                Facade.Timbrar(SessionID, idComprobanteFiscal);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("send")]
        public bool Enviar(string idComprobanteFiscal, string correo, string mensaje)
        {
            try
            {
                Facade.Enviar(SessionID, idComprobanteFiscal, correo, mensaje);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpGet]
        [Route("envio-facturas-list")]
        public DataTable GetEnvioFacturas(string idEmpresa, DateTimeOffset fechaIni, DateTimeOffset fechaFin)
        {
            try
            {
                return Facade.EnvioFacturasLST(SessionID, idEmpresa, fechaIni, fechaFin);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("comprobantes-destino-list")]
        public DataTable ComprobantesDestinosList()
        {
            try
            {
                return Facade.ComprobantesDestinosList(SessionID);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("reset-envio-facturas")]
        public void ResetEnvioFacturas([FromBody] List<ResetComprobantesItem> jobs)
        {
            try
            {
                Facade.ResetEnvioFacturas(SessionID, jobs);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }



        [HttpGet]
        [Route("portal-clientes")]
        public DataTable PortalClientes(int idCliente, DateTimeOffset fechaIni, DateTimeOffset fechaFin)
        {
            DataTable dtRet = null;
            try
            {
                ComprobanteFiscalFacade Facade = new ComprobanteFiscalFacade();
                dtRet = Facade.PortalClientes(SessionID, idCliente, fechaIni, fechaFin);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener comprobantes", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("validar-comprobante-sat")]
        public bool ValidarComprobanteSAT(string idComprobanteFiscal)
        {
            try
            {
                ComprobanteFiscalFacade Facade = new ComprobanteFiscalFacade();
                Facade.ValidarComprobatenSAT(this.SessionID, idComprobanteFiscal);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener comprobantes", ex);
            }
            return false;
        }

        [HttpGet]
        [Route("validar-comprobante-sat-by-idComprobante")]
        public bool ValidarComprobanteByIdComprobante(string idComprobanteFiscal)
        {
            bool result = false;
            try
            {
                ComprobanteFiscalFacade Facade = new ComprobanteFiscalFacade();
                result= Facade.ValidarComprobanteByIdComprobante(this.SessionID, idComprobanteFiscal);
                
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener comprobantes", ex);
            }
            return result;
        }


        [HttpGet]
        [Route("get-comprobante-serie-folio")]
        public ComprobanteFiscalInfo GetComprobanteBySerieFolio(string serie, long folio, int idEmpresa)
        {
            try
            {
                ComprobanteFiscalFacade Facade = new ComprobanteFiscalFacade();
                return Facade.GetComprobanteBySerieFolio(this.SessionID, serie, folio, idEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Error al consultar el comprobante fiscal", ex);
            }
            return null;
        }
        
        [HttpGet]
        [Route("get-addenda")]
        public string GetAddenda(string idComprobanteFiscal)
        {
            string sRet = string.Empty;

            try
            {
                ComprobanteFiscalFacade Facade = new ComprobanteFiscalFacade();
                sRet= Facade.GetAddenda(this.SessionID, idComprobanteFiscal);
            }
            catch(Exception ex)
            {
                HandleException("Error al obtener addenda", ex);
            }
            return sRet;
        }

        [HttpPost]
        [Route ("reemplazar-addenda")]
        public void ReemplazarAddenda(string idComprobanteFiscal, [FromBody] List<string> addenda)
        {
            try
            {
                ComprobanteFiscalFacade Facade = new ComprobanteFiscalFacade();
                Facade.ReemplazarAddenda(this.SessionID, idComprobanteFiscal, addenda[0]);
            }
            catch(Exception ex)
            {
                HandleException("Error al reemplazar addenda", ex);
            }
            
        }


        //[HttpPost]
        //[Route("search")]
        //public new SearchResult Search(int idSucursal, [FromBody] SearchRequest request)
        //{
        //    try
        //    {
        //        SearchResult Result = new SearchResult();
        //        Result.Rows = Facade.Search(SessionID, idSucursal, request.Conditions, request.Order, 0, 0);
        //    }
        //    catch (Exception ex)
        //    {
        //        HandleException("Error al consultar el comprobante fiscal", ex);
        //    }
        //    return null;
        //}


    }
}
