﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Contabilidad;
using System.Data;


namespace Azteca.Web.Controllers.Contabilidad
{
    [Route("api/contabilidad/divisas")]
    public class DivisasController : CatalogController<DivisaFacade, DivisaService, DivisaInfo>
    {
        public DivisasController()
            : base(new DivisaFacade())
        {

        }

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadList(int IdEmpresa)
        {
            DataTable dtRet = new DataTable();

            try
            {
                DivisaFacade Facade = new DivisaFacade();
                dtRet = Facade.LoadList(SessionID, IdEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("divisa-by-empresa")]
        public DivisaInfo GetDetailByEmpresa(int idEmpresa)
        {
            try
            {
                DivisaInfo Divisa = Facade.GetDetailByEmpresa(SessionID, idEmpresa);
                return Divisa;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        //Tipos de Cambio

        [HttpGet]
        [Route("load-tipos-cambio")]
        public DataTable LoadTiposCambioLST(int IdEmpresa)
        {
            DataTable dtRet = new DataTable();

            try
            {
                DivisaFacade Facade = new DivisaFacade();
                dtRet = Facade.LoadTiposCambioLST(SessionID, IdEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("update-tipos-cambio")]
        public Boolean UpdateTiposCambio([FromBody] TipoCambioInfo[] info)
        {
            Boolean flag = new Boolean();

            try
            {

                
                DivisaFacade Facade = new DivisaFacade();
                foreach (TipoCambioInfo p in info)
                {
                    if (p.State == "ADDED" || p.State == "CHANGED")
                    {
                        flag = Facade.SaveTipoCambio(SessionID, p.IdEmpresa, p.IdDivisaOrigen, p.IdDivisaDestino, p.PrecioCompra, p.PrecioVenta, p.PrecioOtro);
                    }
                }
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return flag;
        }
    }
}