﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Contabilidad;
using System.Data;


namespace Azteca.Web.Controllers.Contabilidad
{
    [Route("api/contabilidad/empresas")]
    public class EmpresasController : CatalogController<EmpresaFacade, EmpresaService, EmpresaInfo>
    {
        public EmpresasController()
            :base(new EmpresaFacade())
        {

        }

        [HttpGet]
        [Route("load-regimenes-fiscales")]
        public List<RegimenFiscal> LoadRegimenesFiscales(int IdEmpresa)
        {            
            try
            {                
                List<RegimenFiscal> RetList = Facade.LoadRegimenesFiscales(SessionID, IdEmpresa);
                return RetList;
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
                return null;
            }
        }

        [HttpGet]
        [Route("get-periodo-actual")]
        public DataTable getPeriodoActual(int IdEmpresa)
        {
            DataTable dtRet = new DataTable();

            try
            {
                EmpresaFacade Facade = new EmpresaFacade();
                dtRet = Facade.getPeriodoActual( SessionID, IdEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-list-ejercicios")]
        public DataTable EjerciciosList(int IdEmpresa, bool SoloAbiertos)
        {
            DataTable dtRet = new DataTable();

            try
            {                
                dtRet = Facade.EjerciciosList(SessionID, IdEmpresa, SoloAbiertos);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("nuevo-ejercicio")]
        public void nuevoEjercicio(int IdEmpresa)
        {
            try
            {
                Facade.nuevoEjercicio(SessionID, IdEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Error error al generar un nuevo ejercicio fiscal.", ex);
            }
        }

        [HttpPost]        
        [Route("abrir-ejercicio")]
        public void abrirEjercicio(string IdEjercicio)
        {
            try
            {                
                Facade.abrirEjercicio(SessionID, IdEjercicio);
               
            }
            catch (Exception ex)
            {
                HandleException("Error error al intentar abir el ejercicio fiscal.", ex);
            }
        }

        [HttpPost]
        [Route("cerrar-ejercicio")]
        public void cerrarEjercicio(string IdEjercicio)
        {
            try
            {                
                Facade.cerrarEjercicio(SessionID, IdEjercicio);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }            
        }

        [HttpGet]
        [Route("cargar-periodos")]
        public DataTable cargaPeriodo( string IdEjercicio, bool SoloAbiertos)
        {
            DataTable dtRet = new DataTable();
            try
            {               
                dtRet = Facade.cargaPeriodo(SessionID, IdEjercicio, SoloAbiertos);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("abrir-periodo")]
        public void abrirPeriodo(string IdPeriodo)
        {
            try
            {                
                Facade.abrirPeriodo(SessionID, IdPeriodo);                
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
        }

        [HttpPost]
        [Route("cerrar-periodo")]
        public void cerrarPeriodo(string IdPeriodo)
        {
            try
            {
                EmpresaFacade EmpresaFac = new EmpresaFacade();
                EmpresaFac.cerrarPeriodo(SessionID, IdPeriodo);
               
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
        }

    }
}