﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Contabilidad;
using Azteca.Kernel.Business;
using System.Data;
using Azteca.Kernel.Queue;

namespace Azteca.Web.Controllers.Contabilidad
{
    [Route("api/contabilidad/impuestos")]
    public class ImpuestosController : CatalogController<ImpuestoFacade, ImpuestoService, ImpuestoInfo>
    {
        public ImpuestosController()
            :base(new ImpuestoFacade())
        {

        }

        [HttpGet]
        [Route("load-tipos")]
        public DataTable LoadList(int IdEmpresa)
        {
            DataTable dtRet = new DataTable();

            try
            {
                ImpuestoFacade Facade = new ImpuestoFacade();
                dtRet = Facade.LoadTipos(SessionID, IdEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("ieps-preview")]
        public List<IEPSPreview> GetIepsPreview(int idEmpresa, DateTime fechaIni, DateTime fechaFin)
        {

            List<IEPSPreview> vRet = null;

            try
            {
                ImpuestoFacade Facade = new ImpuestoFacade();
                vRet = Facade.getIepsPreview(SessionID,fechaIni,fechaFin, idEmpresa);

            }
            catch (Exception ex)
            {
                HandleException("Error al obtener el la vista previa del IEPS", ex);
            }

            return vRet;
        }

        [HttpPost]
        [Route("file-ieps")]
        public FileLink GenerarArchivoIEPS(int idEmpresa, string nombreArchivo, DateTime fechaIni, DateTime fechaFin, [FromBody] List<IEPSPreview> rfcs)
        {
            try
            {
                FileLink fRet = Facade.ArchivoIEPS(SessionID, fechaIni, fechaFin, nombreArchivo,idEmpresa, rfcs);
                return fRet;
            }
            catch(Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("get-async-ieps-preview")]
        public AsyncTaskInfo AsyncGetIepsPreview(int idEmpresa, DateTime fechaIni, DateTime fechaFin)
        {
            try
            {
                ImpuestoFacade Facade = new ImpuestoFacade();
                return Facade.GetAsyncIepsPreview(SessionID, fechaIni, fechaFin, idEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener el la vista previa del IEPS", ex);
            }
            return null;
        }

    }
}