﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Contabilidad;
using System.Data;
using System.Reflection;
using Azteca.Kernel.Business;

namespace Azteca.Web.Controllers.Contabilidad
{
    [Route("api/contabilidad/mapas-distribuciones")]
    public class MapasDistribucionesController : CatalogController<MapaDistribucionFacade, MapaDistribucionService, MapaDistribucionInfo>
    {
        public MapasDistribucionesController()
            :base(new MapaDistribucionFacade())
        {

        }

        [HttpGet]
        [Route("get-element-by-esquema")]
        public List<MapElementInfo> GetElementByEsquma(int idEsquema)
        {

            EsquemaFacade EsquemaFac = new EsquemaFacade();
            EsquemaInfo esquema = EsquemaFac.GetDetailFromCache(SessionID, idEsquema);
            object element = CreateInstance(esquema.Ensamblado.AssemblyName, esquema.Clase);
            return LoadElements(element);
        }

        [HttpPost]
        [Route("get-element-childrens")]
        public List<MapElementInfo> GetElementChidlrens([FromBody] MapElementInfo element)
        {

            Type type = Type.GetType(element.TypeElement);
            List<MapElementInfo> elements = LoadElements(type, element.Path);

            if (elements.Count == 0)
            {
                object newElement = CreateInstance(element.BaseAssembly, element.BaseTypeName);
                PropertyInfo property = newElement.GetType().GetProperty(element.Name);
                elements = LoadElements(property, element.Path);
            }

            return elements;
        }

        private object CreateInstance(string ensamblado, string clase)
        {
            object element;
            try
            {
                System.Reflection.Assembly CustomAssembly = System.Reflection.Assembly.Load(ensamblado);
                element = CustomAssembly.CreateInstance(clase);
                if (element == null)
                {
                    throw new Exception("No se encontró una clase con el nombre [" + clase + "] en el ensamblado [" + ensamblado + "]");
                }
            }
            catch (Exception ex)
            {
                throw new Exception("Error al crear la instancia de la clase: " + ensamblado + ":" + clase, ex);
            }
            return element;
        }

        public List<MapElementInfo> LoadElements(PropertyInfo property, string basePath = null)
        {
            List<MapElementInfo> elements = new List<MapElementInfo>();
            if (property.CustomAttributes.Count() > 0)
            {
                EntityAttribute custom;
                if ((custom = property.GetCustomAttribute(typeof(EntityAttribute)) as EntityAttribute) != null)
                {
                    object element = Activator.CreateInstance(custom.Info);
                    elements.AddRange(LoadElements(element, basePath));
                }
            }
            return elements;
        }

        public List<MapElementInfo> LoadElements(Type type, string basePath = null)
        {
            List<MapElementInfo> elements = new List<MapElementInfo>();
            foreach (var cons in type.GetConstructors())
            {
                if (cons.GetParameters().Length == 0)
                {
                    object element = Activator.CreateInstance(type);
                    elements.AddRange(LoadElements(element, basePath));
                }
            }
            return elements;
        }

        public List<MapElementInfo> LoadElements(object element, string basePath = "")
        {
            List<MapElementInfo> elements = new List<MapElementInfo>();
            Type type = element.GetType();
            PropertyInfo[] properties = type.GetProperties();
            foreach (var property in properties)
            {
                if (property.DeclaringType.IsGenericType && typeof(System.Collections.Generic.List<>) == property.DeclaringType.GetGenericTypeDefinition())
                {
                    var item = property.DeclaringType.GetGenericArguments().FirstOrDefault();
                    if (item != null)
                    {
                        elements.AddRange(LoadElements(item, basePath));
                        break;
                    }
                }
                else
                {
                    MapElementInfo info = new MapElementInfo();
                    info.BaseAssembly = element.GetType().Assembly.GetName().Name;
                    info.BaseTypeName = element.GetType().FullName;
                    info.BasePath = basePath;
                    info.Name = property.Name;
                    info.Property = property.ToString();
                    info.TypeElement = property.PropertyType.ToString();
                    elements.Add(info);
                }
            }
            return elements;
        }

    }
}