﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Contabilidad;
using System.Data;

namespace Azteca.Web.Controllers.Contabilidad
{
    [Route("api/contabilidad/polizas-tit")]
    public class PolizasTitController: DocumentController<PolizaInfo, PolizaDetail, PolizaService, PolizaFacade>
    {
        public PolizasTitController()
            :base(new PolizaFacade())
        {

        }

        [HttpPost]
        [Route("marcar-exportacion")]
        public bool MarcarExportacion(string IdPolizaTit, ExportacionStatusEnum status, string mensaje)
        {
            bool result = false;
            try
            {
                PolizaFacade Facade = new PolizaFacade();
                Facade.MarcarExportacion(SessionID, IdPolizaTit, status, mensaje);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return result;
        }

        [HttpGet]
        [Route("load-log")]
        public DataTable LoadList(string IdPolizaTit)
        {
            DataTable dtRet = new DataTable();

            try
            {
                PolizaFacade Facade = new PolizaFacade();
                dtRet = Facade.LoadLog(SessionID, IdPolizaTit);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-asociaciones")]
        public DataTable LoadAsociaciones(string IdPolizaTit)
        {
            DataTable dtRet = new DataTable();

            try
            {
                PolizaFacade Facade = new PolizaFacade();
                dtRet = Facade.LoadAsociaciones(SessionID, IdPolizaTit);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-perfiles-by-lote")]
        public DataTable LoadPerfilesByLote(int idSucursal, int IdUsuario)
        {
            DataTable dtRet = new DataTable();

            try
            {
                PolizaPerfilFacade Facade = new PolizaPerfilFacade();
                dtRet = Facade.loadPolizasByLote(SessionID, idSucursal, IdUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-perfiles")]
        public DataTable LoadPerfiles(int idSucursal, int IdUsuario)
        {
            DataTable dtRet = new DataTable();

            try
            {
                PolizaPerfilFacade Facade = new PolizaPerfilFacade();
                dtRet = Facade.loadPolizasByLote(SessionID, idSucursal, IdUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        //Carga de Tipos de Poliza Test
        [HttpGet]
        [Route("load-types")]
        public DataTable Loadtypes()
            {
            DataTable dtRet = new DataTable();
            try
            {
                PolizaFacade Facade = new PolizaFacade();
                dtRet = Facade.loadPolizaTipos(SessionID);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-contabilizacion-by-lote")]
        public PolizaInfo LoadContabilizacionByLote(int IdSucursal, int IdPerfil, DateTimeOffset FechaIni , DateTimeOffset FechaFin)
        {
            PolizaInfo Poliza = new PolizaInfo();

            try
            {
                PolizaFacade Facade = new PolizaFacade();
                Poliza = Facade.ContabilizarByLote(SessionID, IdSucursal, IdPerfil, FechaIni, FechaFin);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return Poliza;
        }

        [HttpGet]
        [Route("load-contabilizacion-by-mapa")]
        public PolizaInfo LoadContabilizacionByMapa(int IdSucursal, int IdPerfil, DateTimeOffset FechaIni, DateTimeOffset FechaFin)
        {
            PolizaInfo Poliza = new PolizaInfo();
            try
            {
                PolizaFacade Facade = new PolizaFacade();
                return Facade.ContabilizarMapaByPerfil(SessionID, IdSucursal, IdPerfil, FechaIni.Date);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return Poliza;
        }



        [HttpGet]
        [Route("get-poliza-by-perfil")]
        public PolizaInfo getPolizaByPerfil(int IdSucursal, int IdPerfil, DateTimeOffset Fecha)
        {
            PolizaInfo Poliza = new PolizaInfo();

            try
            {
                PolizaFacade Facade = new PolizaFacade();
                Poliza = Facade.getPolizaByPerfil(SessionID, IdSucursal, IdPerfil, Fecha);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return Poliza;
        }

        [HttpGet]
        [Route("get-poliza-by-documento")]
        public PolizaInfo LoadPolizaByDocumento(string idDocumento, int idPolizaPerfil, int idSucursal)
        {
            try
            {
                PolizaFacade Facade = new PolizaFacade();
                return Facade.ContabilizarMapaByDocumento(SessionID, idDocumento, idPolizaPerfil, idSucursal);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost]
        [Route("add-asociacion")]
        public bool AddAsociacion(string IdPolizaTit, DateTime FechaSys, int IdCuenta, int Consecutivo, string uuid, int IdCFDSerie, long Folio)
        {
            bool result = false;
            try
            {
                PolizaFacade Facade = new PolizaFacade();
                Facade.AddAsociacion(SessionID, IdPolizaTit, FechaSys, IdCuenta, Consecutivo, uuid, IdCFDSerie, Folio);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return result;
        }

        [HttpPost]
        [Route("remove-asociacion")]
        public bool RemoveAsociacion(string IdPolizaTit, DateTime FechaSys, int IdCuenta, int Consecutivo, string uuid)
        {
            bool result = false;
            try
            {
                PolizaFacade Facade = new PolizaFacade();
                Facade.RemoveAsociacion(SessionID, IdPolizaTit, FechaSys, IdCuenta, Consecutivo, uuid);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return result;
        }


    }
}
