﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Credito;
using System.Data;
using Azteca.Kernel.Security;
using Azteca.Kernel.Business;

namespace Azteca.Web.Controllers.Credito
{
    [Route("api/credito/cajas")]
    public class CajasController : CatalogController<CajaFacade, CajaService, CajaInfo>
    {
        public CajasController()
            : base(new CajaFacade())
        {

        }

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadList(int IdSucursal)
        {
            {
                DataTable dtRet = new DataTable();
                try
                {
                    CajaFacade Facade = new CajaFacade();
                    dtRet = Facade.LoadList(SessionID, IdSucursal);
                }
                catch (Exception ex)
                {
                    HandleException(ex);
                }
                return dtRet;
            }
        }

        [HttpGet]
        [Route("load-list-by-user")]
        public DataTable LoadListByUser(int IdSucursal)
        {
            {
                DataTable dtRet = new DataTable();
                try
                {
                    CajaFacade Facade = new CajaFacade();
                    dtRet = Facade.LoadListByUser(SessionID, IdSucursal);
                }
                catch (Exception ex)
                {
                    HandleException(ex);
                }
                return dtRet;
            }
        }

        [HttpGet]
        [Route("members")]
        public List<UsuarioItem> GetMembers(int idCaja)
        {
            try
            {
                List<UsuarioItem> Users = Azteca.Kernel.Common.DataTableUtils.ToList<UsuarioItem>(Facade.LoadCajasAccesos(SessionID, idCaja));
                return Users;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-members")]
        public void AddMembers(int idCaja, [FromBody] List<UsuarioItem> users)
        {
            try
            {
                users.ForEach(user =>
                {
                    Facade.InsertarAcceso(SessionID, idCaja, user.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-member")]
        public void DeleteMember(int idCaja, int idUser)
        {
            try
            {
                Facade.RemoverAcceso(SessionID, idCaja, idUser);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("cortes")]
        public DataTable LoadCortes(int IdSucursal)
        {
            {
                DataTable dtRet = new DataTable();
                try
                {
                    CajaFacade Facade = new CajaFacade();
                    dtRet = Facade.LoadCortes(SessionID, IdSucursal);
                }
                catch (Exception ex)
                {
                    HandleException(ex);
                }
                return dtRet;
            }
        }

        [HttpGet]
        [Route("abrir")]
        public void abrir(int idCaja, int idCajero, int idTurno)
        {
            try
            {
                Facade.Abrir(SessionID, idCaja, idCajero, idTurno);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("cerrar")]
        public void cerrar(Guid idCorte)
        {
            try
            {
                Facade.Cerrar(SessionID, idCorte);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("load-cortes-historico")]
        public DataTable LoadCortesHistorico(int IdCaja, DateTime FechaIni, DateTime FechaFin)
        {
            {
                DataTable dtRet = new DataTable();
                try
                {
                    CajaFacade Facade = new CajaFacade();
                    dtRet = Facade.LoadCortes(SessionID, IdCaja, FechaIni, FechaFin);
                }
                catch (Exception ex)
                {
                    HandleException(ex);
                }
                return dtRet;
            }
        }

        [HttpGet]
        [Route("pdf-corte")]
        public FileLink GenerarPdfCorte(string idCorte)
        {
            try
            {
                FileLink FileUri = Facade.GenerarPdfCorte(SessionID, idCorte);
                return FileUri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

    }
}