﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Credito;


namespace Azteca.Web.Controllers.Credito
{
    [Route("api/credito/cobranzas-tit")]
    public class CobranzasTitController: DocumentController<CobranzaInfo, CobranzaDetail, CobranzaService, CobranzaFacade>
    {
        public CobranzasTitController()
            :base(new CobranzaFacade())
        { 

        }

        [HttpGet]
        [Route("load-cxc-by-grupo-deudor")]
        public List<CobranzaDetail> LoadCxCbyIdGrupoDeudor(DateTime FechaIni, DateTime FechaFin, int IdGrupoDeudor, string FechaFiltro)
        {
            try
            {
                List<CobranzaDetail> CXC = Azteca.Kernel.Common.DataTableUtils.ToList<CobranzaDetail>(Facade.LoadCxCbyIdGrupoDeudor(SessionID, FechaIni, FechaFin, IdGrupoDeudor, FechaFiltro)).OrderBy(order => (order.Serie, order.FolioFactura, order.Vencimiento)).ToList();                
                return CXC;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("load-cxc-by-deudor")]
        public List<CobranzaDetail> LoadCxCbyIdDeudor(DateTime FechaIni, DateTime FechaFin, int IdDeudor, string FechaFiltro)
        {
            try
            {
                List<CobranzaDetail> CXC = Azteca.Kernel.Common.DataTableUtils.ToList<CobranzaDetail>(Facade.LoadCxCbyIdDeudor(SessionID,  FechaIni,  FechaFin, IdDeudor, FechaFiltro)).OrderBy(order => (order.Serie, order.FolioFactura, order.Vencimiento)).ToList();
                return CXC;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("load-cxc-by-rfc")]
        public List<CobranzaDetail> LoadCxCbyRFC(DateTime FechaIni, DateTime FechaFin, string RFC)
        {
            try
            {
                List<CobranzaDetail> CXC = Azteca.Kernel.Common.DataTableUtils.ToList<CobranzaDetail>(Facade.LoadCxCbyRFC(SessionID, FechaIni, FechaFin, RFC));
                return CXC;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("pdf-comprobante")]
        public FileLink GenerarPdfComprobante(string idCobranza)
        {
            try
            {
                FileLink FileUri = Facade.GenerarPdfComprobante(SessionID, idCobranza);
                return FileUri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("xml-comprobante")]
        public FileLink GetXmlComprobante(string idCobranza)
        {
            try
            {
                FileLink FileUri = Facade.GetXmlComprobante(SessionID, idCobranza);
                return FileUri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("reprocesar")]
        public void Reprocesar()
        {
            try
            {

                Facade.Reprocesar(SessionID);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpPost]
        [Route("cobranza-por-redondeo")]
        public void PorRedondeo()
        {
            try
            {                
                Facade.PorRedondeo(SessionID);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpPost]
        [Route("sustituir-comprobante-pago")]
        public void Sustituir(string IdCobranza)
        {

            try
            {
                CobranzaFacade CobranzaFac = new CobranzaFacade();
                //CobranzaInfo Cobranza = CobranzaFac.GetDetail(SessionID, IdCobranza);
                
                //CuentaPorCobrarFacade Facade = new CuentaPorCobrarFacade();
                Facade.Sustituir(SessionID, IdCobranza);
            }
            catch (Exception ex)
            {
                HandleException("Error al regenerar la factura.", ex);
            }
            //return Factura;
        }

        [HttpPost]
        [Route("generar-comprobante-pago-sin-condiciones")]
        public void GenerarComprobantePagoFiscalSinCondiciones(string IdCobranza)
        {
            try
            {
                Facade.GenerarComprobantePagoFiscalSinCondiciones(SessionID, IdCobranza);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }
        
    }
}
