﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Credito;
using System.Data;

namespace Azteca.Web.Controllers.Credito
{
    [Route("api/credito/cuentas-por-cobrar-tit")]
    public class CuentasPorCobrarTitController: DocumentController<CuentaPorCobrarInfo, CuentaPorCobrarDetail, CuentaPorCobrarService, CuentaPorCobrarFacade>
    {
        public CuentasPorCobrarTitController()
            :base(new CuentaPorCobrarFacade())
        {

        }

        [HttpGet]
        [Route("search-cpc")]
        public List<CuentaPorCobrarItem> SearchKeyword(int idBranch, string keyword, int idDeudor, int idGrupoDeudor)
        {
            List<CuentaPorCobrarItem> dtRet = null;
            try
            {
                CuentaPorCobrarFacade Facade = new CuentaPorCobrarFacade();

                dtRet = Facade.SearchCuentasPorCobrar(SessionID, idBranch, keyword, idDeudor, idGrupoDeudor);

            }
            catch (Exception ex)
            {
                HandleException("Error al obtener la lista de cuentas por cobrar", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-facturacion-pendiente")]
        public DataTable LoadFacturacionPendiente(int IdSucursal, string TipoActor, int IdActor,  DateTime FechaIni, DateTime FechaFin)
        {
            DataTable dtRet = new DataTable();
            try
            {
                CuentaPorCobrarFacade Facade = new CuentaPorCobrarFacade();
                dtRet = Facade.LoadFacturacionPendiente(SessionID, IdSucursal, TipoActor, IdActor, FechaIni, FechaFin);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los conceptos.", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("genera-factura")]
        public FacturaInfo GeneraFactura(int IdCfdSerie, string CodigoMetodoPago, string CuentaBanco, int IdSucursal, int IdContribuyente, [FromBody] List<CuentaPorCobrarItem> cxc)
        {
            FacturaInfo Factura = null;
            List<string> CxCList = new List<string>(); 
            cxc.ForEach(item => {
                CxCList.Add(item.id);
            });

            try
            {
                CuentaPorCobrarFacade Facade = new CuentaPorCobrarFacade();
                Factura= Facade.GeneraFactura(SessionID, IdCfdSerie, CodigoMetodoPago, CuentaBanco, IdSucursal, IdContribuyente, CxCList);
            }
            catch (Exception ex)
            {
                HandleException("Error al generar la factura.", ex);                
            }
            return Factura;
        }

        [HttpPost]
        [Route("modificar-addenda")]
        public DataTable ModificarAddenda(string IdCuentaPorCobrar, string OrdenCompra, DateTime FechaOrdenCompra, string NumRemision, DateTime FechaRemision, DateTime FechaEntrega)
        {
            DataTable data = new DataTable();
            try
            {
                data = Facade.ModificarAddenda(SessionID, IdCuentaPorCobrar, OrdenCompra, FechaOrdenCompra, NumRemision, FechaRemision, FechaEntrega);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return data;
        }


        [HttpPost]
        [Route("requerir-cancelacion")]
        public string RequerirCancelacion(string idCuentaPorCobrar)
        {
            try
            {
                Facade.RequerirCancelacion(this.SessionID, idCuentaPorCobrar);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost]
        [Route("timbrar-comprobante-pago")]
        public string TimbrarComprobantePago(string idComprobantePago)
        {
            try
            {
                ComprobantePagoFacade facade = new ComprobantePagoFacade();
                facade.GenerarXML(this.SessionID, idComprobantePago);
                return idComprobantePago+" Timbrado";
            }
            catch (Exception ex)
            {
                return "Error en el ID: "+idComprobantePago+", " +ex.Message;
            }
        }

        [HttpPost]
        [Route("verifica-comprobante-timbrado")]
        public bool VerificaComprobanteTimbrado([FromBody]string[] idCuentaPorCobrarTit)
        {
            bool error = false;
            try
            {
                CuentaPorCobrarFacade facade = new CuentaPorCobrarFacade();
                error= facade.VerificarComprobantetimbrado(SessionID, idCuentaPorCobrarTit);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return error;
        }

       



    }
}
