﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Distribucion;
using System.Data;

namespace Azteca.Web.Controllers.Distribucion
{
    [Route("api/distribucion/clientes")]
    public class ClientesController : CatalogController<ClienteFacade, ClienteService, ClienteInfo>
    {
        public ClientesController()
            :base(new ClienteFacade())
        {

        }

        [HttpGet]
        [Route("impresion-lote-cfdi")]
        public void GenerarImpresionLoteCFDI(DateTimeOffset fechaComprobantes, int idSucursal, string rfc, string condicionesPago)
        {
            try
            {
                if (rfc == null)
                    rfc = string.Empty;
                Facade.GenerarImpresionLoteCFDI(SessionID, fechaComprobantes, idSucursal, rfc, condicionesPago);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("cancelar-facturas")]
        public void CancelarFacturas(DateTime fecha, int idSucursal )
        {
            try
            {
                Facade.CancelarFacturasRechazadas(SessionID, fecha, idSucursal);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }


        [HttpGet]
        [Route("lotes-cfdi")]
        public List<LoteImpresionCfdiItem> LoadLotesImpresionCFDI(int idSucursal, DateTimeOffset fechaLotes)
        {
            try
            {
                List<LoteImpresionCfdiItem> Lotes = Facade.LoadLotesImpresionCFDI(SessionID, idSucursal, fechaLotes);
                return Lotes;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("archivos-lote-impresion")]
        public List<string> GetArchivosLoteImpresion(int idLote)
        {
            try
            {
                List<string> Archivos = Facade.GetArchivosLoteImpresion(SessionID, idLote);
                return Archivos;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("archivos-lote-impresion-multiple")]
        public List<string> GetArchivosLoteImpresionMultiple([FromBody]List<int> lotes)
        {
            try
            {
                List<string> Archivos = Facade.GetArchivosLoteImpresion(SessionID, lotes);
                return Archivos;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("facturas-impresion")]
        public DataTable GetFacturasPeriodo(int idSucursal, DateTime fechaIni, DateTime fechaFin)
        {
            try
            {
                DataTable dtFacturas = Facade.GetFacturasPeriodo(SessionID, idSucursal, fechaIni, fechaFin);
                return dtFacturas;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("clientes-rutas")]
        public DataTable GetClientesRutas(int idCliente)
        {
            try
            {
                DataTable dtFacturas = Facade.GetClientesRutas(SessionID, idCliente);
                return dtFacturas;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


    }
}