﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Distribucion;

namespace Azteca.Web.Controllers.Distribucion
{
    [Route("api/distribucion/conteos")]
    public class ConteosController: DocumentController<ConteoInfo, ConteoDetail, ConteoService, ConteoFacade>
    {
        public ConteosController()
            :base(new ConteoFacade())
        {

        }

        [HttpGet]
        [Route("load-documentos-conteo")]
        public List<ConteoDocumento> LoadDocumentosConteo(int idRutaReparto, DateTime fecha, int idEmpresa)
        {
            List<ConteoDocumento> dtRet = new List<ConteoDocumento>();

            try
            {
                ConteoFacade Facade = new ConteoFacade();
                dtRet = Facade.LoadDocumentosConteo(SessionID, idRutaReparto, fecha, idEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("ruta-procesada")]
        public bool IsRutaProcesada(int idRutaReparto, DateTime fechaCobro)
        {
            bool bRet = false;
            try
            {
                ConteoFacade facade = new ConteoFacade();
                bRet = facade.IsRutaProcesada(SessionID, idRutaReparto, fechaCobro);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return bRet;

        }

        [HttpPost]
        [Route("to-conteo-detail")]
        public List<ConteoDetail> ToConteoDetail([FromBody] List<ConteoDocumento> conteoDocumentos)
        {

            List<ConteoDetail> dtRet = new List<ConteoDetail>();

            try
            {
                ConteoFacade Facade = new ConteoFacade();
                dtRet = Facade.ToConteoDetail(SessionID, conteoDocumentos);
                return dtRet;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("cerrar")]
        public bool Cerrar(DateTime fecha, int idSucursal)
        {
            try
            {
                ConteoFacade Facade = new ConteoFacade();
                Facade.CerrarConteo(SessionID, fecha, idSucursal);
                return true;
            }
            catch(Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("cancelar-cierre")]
        public bool CancelarCierre(DateTime fecha, int idSucursal)
        {
            try
            {
                ConteoFacade Facade = new ConteoFacade();
                Facade.CancelarCierre(SessionID, fecha, idSucursal);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost("enqueue-conteo")]
        public ConteoInfo EnqueueConteo([FromBody] ConteoInfo conteo)
        {
            try
            {
                Facade = new ConteoFacade();
                return Facade.EnqueueConteo(this.SessionID, conteo);
            }
            catch(Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost]
        [Route("to-pdf")]
        public Byte[] ToPdf ([FromBody] ConteoInfo conteo)
        {
            try
            {
                Byte[] Formato = Facade.ToPdf(SessionID, conteo);
                return Formato;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

    }
}
