﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Distribucion;
using Azteca.Business.Inventario;
using System.Data;


namespace Azteca.Web.Controllers.Distribucion
{
    [Route("api/distribucion/productos")]
    public class ProductosController : CatalogController<ProductosFacade, ProductosService, ProductosInfo>
    {
        public ProductosController()
            : base(new ProductosFacade())
        {

        }

        [HttpGet]
        [Route("create")]
        public override ProductosInfo Create()
        {
            ProductosInfo Productos = base.Create();
            //Cargar clasificaciones del articulo stock
            ArticuloStockFacade articuloStockFacade = new ArticuloStockFacade();
            Productos.Articulo.ArticuloStock.Clasificaciones = articuloStockFacade.LoadClassifications(SessionID, 0);
            return Productos;
        }

        // metodos para agregar un producto nuevo

        [HttpGet]
        [Route("get-articulo-stock")]
        public StockItem GetArticuloStock(int idSucursal, string codigo)
        {
            try
            {
                if (codigo != null && codigo.Trim() != string.Empty)
                {
                    ArticuloStockFacade articuloStockFacade = new ArticuloStockFacade();
                    StockItem articuloStockItem = articuloStockFacade.GetArticuloStock(SessionID, idSucursal, codigo);
                    return articuloStockItem;
                }
                else
                {
                    return new StockItem();
                }
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("articulo-asignado-check")]
        public Boolean ArticuloAsignadoCheck(int idArticuloStock)
        {
            Boolean asigCheck = new Boolean();

            try
            {
                asigCheck = Facade.ArticuloAsignadoCheck(SessionID, idArticuloStock);
            }
            catch (Exception ex)
            {
                HandleException("Error en al revisar si el articulo stock ya esta asignado a un producto.", ex);
            }
            return asigCheck;
        }

    }
}