﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
//using Azteca.Business.Distribucion;
using System.Data;
using Azteca.Web.Controllers.Kernel;
using Azteca.Kernel.Search;
using ConsorcioAga.Custom.Distribucion;

namespace ConsorcioAga.Custom.Distribucion
{
    [Route("api/distribucion/reestructura-distribucion")]
    public class ReestructuraDistribucionController : BaseController
    {
        protected IDbConnection Connection;
        protected IDbTransaction Transaction;
        protected List<ReestructuraDistributionInfo> resInfo = new List<ReestructuraDistributionInfo>();

        /// <summary>
        /// Encola proceso de transpiler
        /// </summary>
        /// <param name="infoFormato"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("encolar-transpiler")]
        public bool EncolarTranspiler([FromBody] ReestructuraDistribucionFormato infoFormato)
        {
            try
            {
                if(infoFormato.File.OriginalFileName.Contains("Pixcua") == true || infoFormato.File.OriginalFileName.Contains("pixcua") == true || infoFormato.File.OriginalFileName.Contains("PIXCUA") == true)
                {
                    ConsorcioAga.Custom.Distribucion.ReestructuraDistribucionFacade reestructuraFacade = new ConsorcioAga.Custom.Distribucion.ReestructuraDistribucionFacade();
                    reestructuraFacade.EncolarTranspiler(this.SessionID, infoFormato);
                }
                else
                {
                    throw new ArgumentException("Archivo no indica destino  (Pixcua)");
                }
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }


        /// <summary>
        /// Obtiene información del título de la Reestructuras.
        /// </summary>
        [HttpGet]
        [Route("get-reestructuraTit")]
        public List<ReestructuraDistributionInfo> ConsultaReestructuraTitulo(DateTime fechaIni,DateTime fechaFin)
        {
            List<ReestructuraDistributionInfo> tituloList = new List<ReestructuraDistributionInfo>();
            try
            {
                ReestructuraDistribucionFacade facade = new ReestructuraDistribucionFacade();
                tituloList = facade.ReestructuraTituloGET(this.SessionID,fechaIni, fechaFin);
                return tituloList;
                //return facade.ReestructuraTituloGET();
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }
        
        /// <summary>
        /// Obtiene el detalle de la reestructura.
        /// 
        /// </summary>
        /// <param name="infoFormato"></param>
        [HttpGet]
        [Route ("get-reestructuraDet")]
        public List<ReestructuraDistribucionDetail> ConsultaReestructuraDetalle(int id)
        {
            List<ReestructuraDistribucionDetail> detalleList = new List<ReestructuraDistribucionDetail>();
            try
            {
                ReestructuraDistribucionFacade facade = new ReestructuraDistribucionFacade();
                detalleList = facade.ReestructuraDetalleGET(this.SessionID,id);
                return detalleList;
                //return facade.ReestructuraDetalleGET(id);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        /// <summary>
        ///Actualiza información del detalle de la reestructura [Distribution.ReestructuraDet]
        /// </summary>
        /// <param name="infoFormato"></param>
        [HttpPost]
        [Route("put-reestructura")]
        public bool ReestructuraDetalleUPD([FromBody]  List<ReestructuraDistribucionDetail> resInfo)
        {
            try
            {
                ReestructuraDistribucionFacade facade = new ReestructuraDistribucionFacade();
                facade.ActualizarReestructura(this.SessionID,resInfo);
                return true;
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return false;
        }

        [HttpPost]
        [Route("search-reestructura-jobs")]
        public SearchResult SearchJobs(int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    ReestructuraDistribucionFacade Facade = new ReestructuraDistribucionFacade();
                    Result.TotalRows = Facade.GetTotalRecorsJobs(this.SessionID, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchJobs(this.SessionID, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    ReestructuraDistribucionFacade  Facade = new ReestructuraDistribucionFacade();
                    Result.TotalRows = Facade.GetTotalRecorsJobs(this.SessionID, idBranch, null);
                    Result.Rows = Facade.SearchJobs(this.SessionID, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda.", ex);
                return null;
            }
        }



    }
}
