﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Distribucion;
using System.Data;
using Newtonsoft.Json;

namespace Azteca.Web.Controllers.Distribucion
{
    [Route("api/distribucion/rutas-preventa")]
    public class RutasPreventaController : CatalogController<RutasPreventaFacade, RutasPreventaService, RutasPreventaInfo>
    {
        public RutasPreventaController()
            :base(new RutasPreventaFacade())
        {

        }

        [HttpGet]
        [Route("load-celulas-list")]
        public DataTable LoadCelulasList(int idRutaPreventa)
        {
            DataTable dtRet = new DataTable();

            try
            {
                dtRet = Facade.LoadCelulasList(SessionID, idRutaPreventa);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los celulas de la ruta de preventa.", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("add-members")]
        public void AddMembers(int idRutaPreventa, [FromBody] List<CelulasItem> celulas)
        {
            try
            {
                celulas.ForEach(celula =>
                {
                    Facade.AddMembers(SessionID, idRutaPreventa, celula.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException("Error al agregar celulas a la ruta de preventa.", ex);
            }
        }

        [HttpDelete]
        [Route("remove-member")]
        public void RemoveMember(int idRutaPreventa, int idCelula)
        {
            try
            {
                Facade.RemoveMember(SessionID, idRutaPreventa, idCelula);
            }
            catch (Exception ex)
            {
                HandleException("Error al quitar la celula a la ruta de preventa.", ex);
            }
        }
        
        [HttpGet]
        [Route("load-list-by-user")]
        public List<RutasPreventaItem> LoadList(int idSucursal, int idUsuario)
        {
            try
            {
                DataTable Rutas = Facade.LoadList(SessionID, idSucursal);
                List<RutasPreventaItem> result =Azteca.Kernel.Common.DataTableUtils.ToList<RutasPreventaItem>(Rutas);
                return result;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("load-catalogs")]
        public string[] LoadCatalogos(int idRutaPreventa, String fecha)
        {
            string[] resultado = null;//  string.Empty;

            DateTime dateFecha2 = Convert.ToDateTime(fecha);
            DateTime  dateFecha =  DateTime.Parse(fecha);

            RutasPreventaFacade Ruta = new RutasPreventaFacade();
            try
            {
                CatalogosCargaCollection Catalogos = Ruta.GenerarCatalogos(SessionID, idRutaPreventa, dateFecha, TipoPDA.WorkAboutPro);//, "P");
                //Definir array de cadenas que conendrá los catálogos                
                String[] CatalogosArray = new String[12];
                CatalogosArray[0] = Catalogos.Item("Clientes");
                CatalogosArray[1] = Catalogos.Item("Productos");
                CatalogosArray[2] = Catalogos.Item("Eventos");
                CatalogosArray[3] = Catalogos.Item("Firmas");
                CatalogosArray[4] = Catalogos.Item("Rutas");
                CatalogosArray[5] = Catalogos.Item("TiposPedido");
                CatalogosArray[6] = Catalogos.Item("TiposVale");
                CatalogosArray[7] = Catalogos.Item("Promociones");
                CatalogosArray[8] = Catalogos.Item("Prestamos");
                CatalogosArray[9] = Catalogos.Item("PromocionesArticulos");
                CatalogosArray[10] = Catalogos.Item("ListaPrecios");
                CatalogosArray[11] = Catalogos.Item("Comodatos");

                resultado = CatalogosArray;// JsonConvert.SerializeObject( CatalogosArray);

            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los catalogos de la ruta de preventa.", ex);
            }
            return resultado;
        }



    }
}