﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Distribucion;
using System.Data;


namespace Azteca.Web.Controllers.Distribucion
{
    [Route("api/distribucion/rutas-reparto")]
    public class RutasRepartoController : CatalogController<RutasRepartoFacade, RutasRepartoService, RutasRepartoInfo>
    {
        public RutasRepartoController()
            :base(new RutasRepartoFacade())
        {

        }

        [HttpGet]
        [Route("load-celulas-list")]
        public DataTable LoadCelulasList(int idRutaReparto)
        {
            DataTable dtRet = new DataTable();

            try
            {
                dtRet = Facade.LoadCelulasList(SessionID, idRutaReparto);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los celulas de la ruta de reparto.", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("add-members")]
        public void AddMembers(int idRutaReparto, [FromBody] List<CelulasItem> celulas)
        {
            try
            {
                celulas.ForEach(celula =>
                {
                    Facade.AddMembers(SessionID, idRutaReparto, celula.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException("Error al agregar celulas a la ruta de reparto.", ex);
            }
        }

        [HttpDelete]
        [Route("remove-member")]
        public void RemoveMember(int idRutaReparto, int idCelula)
        {
            try
            {
                Facade.RemoveMember(SessionID, idRutaReparto, idCelula);
            }
            catch (Exception ex)
            {
                HandleException("Error al quitar la celula a la ruta de reparto.", ex);
            }
        }

        [HttpGet]
        [Route("rutas-conteo-pendientes")]
        public List<RutaConteoItem> RutasPendientesConteo(int idSucursal, DateTime fecha)
        {
            List<RutaConteoItem> dtRet = new List<RutaConteoItem>();

            try
            {
                dtRet = Facade.RutasPendientesConteo(SessionID,idSucursal, fecha);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los celulas de la ruta de reparto.", ex);
            }
            return dtRet;
        }

    }
}