﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Inventario;
using Azteca.Kernel.General;
using System.Data;
using Azteca.Kernel.Search;

namespace Azteca.Web.Controllers.Inventario
{
    [Route("api/inventario/almacenes")]
    public class AlmacenesController : CatalogController<AlmacenFacade, AlmacenService, AlmacenInfo>
    {
        public AlmacenesController()
            : base(new AlmacenFacade())
        {

        }

        #region Usuarios Miembro

        [HttpGet]
        [Route("load-access-users-list")]
        public DataTable LoadUserList(int idAlmacen)
        {
            DataTable dtRet = new DataTable();

            try
            {
                AlmacenFacade Facade = new AlmacenFacade();
                dtRet = Facade.LoadAlmacenAccess(SessionID, idAlmacen);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los usuarios por almacén.", ex);
            }
            return dtRet;
        }

        [HttpDelete]
        [Route("remove-member")]
        public void RemoveMember(int idAlmacen, int idUsuario)
        {
            try
            {
                Facade.RemoveUser(SessionID, idAlmacen, idUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Error al quitar el usuario del almacén.", ex);
            }
        }

        [HttpPost]
        [Route("add-members")]
        public void AddMembers(int idAlmacen, [FromBody] List<AlmacenUsuarioItem> usuarios)
        {
            try
            {
                usuarios.ForEach(user =>
                {
                    Facade.AddUser(SessionID, idAlmacen, user.ID, user.Entrada, user.Salida);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpPost]
        [Route("update-members")]
        public void UpdateMembers(int idAlmacen, [FromBody] List<AlmacenUsuarioItem> usuarios)
        {
            try
            {
                usuarios.ForEach(user =>
                {
                    Facade.UpdateUser(SessionID, idAlmacen, user.ID, user.Entrada, user.Salida);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        #endregion

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadList(int idArea)
        {
            DataTable dtRet = new DataTable();

            try
            {
                dtRet = Facade.LoadList(SessionID, idArea);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-list-by-user")]
        public DataTable LoadList(int idArea, int idUsuario)
        {
            try
            {
                DataTable dtRet = Facade.LoadAlmacenesByUsuario(SessionID, idArea, idUsuario);
                return dtRet;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        ///////////Articulos Permitidos///////////

        [HttpGet]
        [Route("load-clasificaciones-articulos")]
        public DataTable ClasificacionesList()
        {
            DataTable dtRet = new DataTable();

            try
            {
                dtRet = Facade.ClasificacionesList(SessionID);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-clasificaciones-permitidas")]
        public List<ClasificacionValor> ClasificacionesPermitidas(int idAlmacen)
        {
            try
            {
                List<ClasificacionValor> RetList = Facade.LoadClasificacionesPermitidas(SessionID, idAlmacen);
                return RetList;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-clasificaciones-articulos")]
        public void AddClasificacionesArticulos(int idAlmacen, [FromBody] List<ClasificacionesArticulosItem> clasificacionesArticulos)
        {
            try
            {
                clasificacionesArticulos.FindAll(c => c.Selected).ForEach(clasificacion =>
                {
                    Facade.AgregarClasificacionPermitida(SessionID, idAlmacen, clasificacion.IdClasificacion);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-clasificaciones-articulos")]
        public void RemoveClasificacionesArticulos(int idAlmacen, int idClasificacion)
        {
            try
            {
                Facade.QuitarClasificacionPermitida(SessionID, idAlmacen, idClasificacion);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        ///////////Existencias///////////

        [HttpGet]
        [Route("load-existencias")]
        public DataTable Existencia(int idAlmacen)
        {
            DataTable dtRet = new DataTable();
            try
            {
                dtRet = Facade.Existencia(SessionID, idAlmacen);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dtRet;
        }


        [HttpGet]
        [Route("load-existencias-by-article")]
        public DataTable ExistenciaArticulo(int idArticuloStock, int idSucursal)
        {
            DataTable dtRet = new DataTable();

            try
            {
                dtRet = Facade.ExistenciaArticulo(SessionID, idArticuloStock, idSucursal);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dtRet;
        }

        ///////////Articulos Stock///////////

        [HttpGet]
        [Route("stock")]
        public StockItem LoadStock(int idAlmacen, int idArticuloStock)
        {
            try
            {
                StockItem Stock = Facade.LoadStock(SessionID, idAlmacen, idArticuloStock);
                return Stock;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("stock-almacen")]
        public List<StockItem> LoadStockAlmacen(int idAlmacen)
        {
            try
            {
                List<StockItem> RetList = Facade.LoadStockAlmacen(SessionID, idAlmacen);
                return RetList;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("stock-articulo")]
        public List<StockItem> LoadStockArticulo(int idArticulo, int idSucursal)
        {
            List<StockItem> RetList = null;
            try
            {
                RetList = Facade.LoadStockArticulo(SessionID, idArticulo, idSucursal);
                return RetList;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


        [HttpPost]
        [Route("update-stock")]
        public void UpdateStock(int idAlmacen, [FromBody] StockItem Stock)
        {
            try
            {
                if (Stock.MetodoValuacion == null || Stock.MetodoValuacion == string.Empty)
                    Stock.MetodoValuacion = "N";
                Facade.UpdateStock(SessionID, Stock.IdArticuloStock, idAlmacen, Stock.StockMaximo, Stock.StockMinimo, Stock.PuntoReorden, Stock.Reserva, Stock.ManejaSeries, Stock.ManejaCaducidad,
                    Stock.ManejaLotes, Stock.MetodoValuacion, Stock.IdUbicacion, Stock.IdTipoInventario);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }


        [HttpPost]
        [Route("update-stock-by-article")]
        public void UpdateStockbyArt(int idArticulo, [FromBody] StockItem Stock)
        {
            try
            {
                AlmacenFacade AlmacenFac = new AlmacenFacade();
                AlmacenFac.UpdateStock(SessionID, idArticulo, Stock.IdAlmacen, Stock.StockMaximo, Stock.StockMinimo, Stock.PuntoReorden, Stock.Reserva, Stock.ManejaSeries, Stock.ManejaCaducidad, Stock.ManejaLotes,
                    Stock.MetodoValuacionDspl, Stock.IdUbicacion, Stock.IdTipoInventario);

            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }


        #region Almacener Provision

        [HttpGet]
        [Route("load-almacenes-provisiones")]
        public DataTable LoadAlmacenProvisiones(int idAlmacen)
        {
            DataTable dtRet = new DataTable();

            try
            {
                AlmacenFacade Facade = new AlmacenFacade();
                dtRet = Facade.LoadAlmacenProvisiones(SessionID, idAlmacen);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los almacén para provisión.", ex);
            }
            return dtRet;
        }

        [HttpDelete]
        [Route("remove-almacen-provision")]
        public void RemoveAlmacenProvision(int idAlmacen, int idAlmacenProvision)
        {
            try
            {
                Facade.RemoveAlmacenProvision(SessionID, idAlmacen, idAlmacenProvision);
            }
            catch (Exception ex)
            {
                HandleException("Error al quitar el almacén de provisión.", ex);
            }
        }

        [HttpPost]
        [Route("add-almacen-provision")]
        public void AddAlmacenProvision(int idAlmacen, [FromBody] List<AlmacenProvisionItem> almacenes)
        {
            try
            {
                almacenes.ForEach(almacen =>
                {
                    Facade.AddAlmacenProvision(SessionID, idAlmacen, almacen.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        //[HttpPost]
        //[Route("update-almacenes-provisiones")]
        //public void UpdateAlmacenesProvisiones(int idAlmacen, [FromBody] List<AlmacenUsuarioItem> usuarios)
        //{
        //    try
        //    {
        //        usuarios.ForEach(user =>
        //        {
        //            Facade.AddAlmacenProvision(SessionID, idAlmacen, user.ID, user.Entrada, user.Salida);
        //        });
        //    }
        //    catch (Exception ex)
        //    {
        //        HandleException(ex);
        //    }
        //}

        #endregion



    }
}