﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Inventario;
using Azteca.Kernel.Security;
using System.Data;


namespace Azteca.Web.Controllers.Inventario
{
    [Route("api/inventario/areas")]
    public class AreasController : CatalogController<AreaFacade, AreaService, AreaInfo>
    {
        public AreasController()
            : base(new AreaFacade())
        {

        }

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadList(int idSucursal)
        {
            DataTable dtRet = new DataTable();

            try
            {
                AreaFacade Facade = new AreaFacade();
                dtRet = Facade.LoadList(SessionID, idSucursal);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los usuarios por área.", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-list-by-user")]
        public DataTable LoadList(int idSucursal, int idUsuario)
        {
            DataTable dtRet = new DataTable();

            try
            {
                AreaFacade Facade = new AreaFacade();
                dtRet = Facade.LoadList(SessionID, idSucursal, idUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los usuarios por área.", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-almacenes")]
        public DataTable LoadAlmacenes(int idArea)
        {
            DataTable dtRet = new DataTable();

            try
            {
                AreaFacade Facade = new AreaFacade();
                dtRet = Facade.LoadAlmacenes(SessionID, idArea);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los almacenes por área.", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-almacenes-by-user")]
        public DataTable LoadAlmacenes(int idArea, int idUsuario)
        {
            DataTable dtRet = new DataTable();

            try
            {
                AreaFacade Facade = new AreaFacade();
                dtRet = Facade.LoadAlmacenes(SessionID, idArea, idUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los almacenes por área.", ex);
            }
            return dtRet;
        }


        [HttpGet]
        [Route("load-access-users-list")]
        public DataTable LoadAreaAccess(int idArea)
        {
            DataTable dtRet = new DataTable();

            try
            {
                AreaFacade Facade = new AreaFacade();
                dtRet = Facade.LoadAreaAccess(SessionID, idArea);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los usuarios por área.", ex);
            }
            return dtRet;
        }

        #region Usuarios Miembro

        [HttpDelete]
        [Route("remove-member")]
        public void RemoveMember(int idArea, int idUsuario)
        {
            try
            {
                Facade.RemoveUser(SessionID, idArea, idUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Error al quitar el usuario del área.", ex);
            }
        }

        [HttpPost]
        [Route("add-members")]
        public void AddMembers(int idArea, [FromBody] List<UsuarioItem> usuarios)
        {
            try
            {
                usuarios.ForEach(user =>
                {
                    Facade.AddUser(SessionID, idArea, user.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException("Error al agregar miembros al área.", ex);
            }
        }

        #endregion

        #region Turnos

        [HttpGet]
        [Route("load-turnos")]
        public List<TurnoItem> LoadTurnos(int idArea)
        {
            List<TurnoItem> dtRet = new List<TurnoItem>();

            try
            {
                AreaFacade Facade = new AreaFacade();
                dtRet = Facade.LoadTurnos(SessionID, idArea);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los turnos.", ex);
            }
            return dtRet;
        }

        #endregion

        #region Cortes
        [HttpGet]
        [Route("load-corte")]
        public CorteInfo LoadCorte(Guid idCorte)
        {
            CorteInfo dtRet = new CorteInfo();

            try
            {
                AreaFacade Facade = new AreaFacade();
                dtRet = Facade.LoadCorte(SessionID, idCorte);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar la información del corte.", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("load-corte-actual")]
        public CorteInfo LoadCorteActual(int idArea)
        {
            CorteInfo dtRet = new CorteInfo();

            try
            {
                AreaFacade Facade = new AreaFacade();
                dtRet = Facade.LoadCorteActual(SessionID, idArea);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar la información del corte Actual.", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("abrir-corte")]
        public CorteInfo AbrirCorte(int idArea, int idTurno, int idUsuario)
        {
            CorteInfo dtRet = new CorteInfo();
            try
            {
                dtRet = Facade.AbrirCorte(SessionID, idArea, idTurno, idUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Error al abrir el corte.", ex);
            }
            return dtRet;
        }

        [HttpDelete]
        [Route("cerrar-corte")]
        public void CerrarCorte(Guid idCorte, bool ajustarInventario, int idUsuario)
        {
            try
            {
                Facade.CerrarCorte(SessionID, idCorte, ajustarInventario, idUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Error al cerrar el corte.", ex);
            }
        }

        #endregion

    }
}