﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Kernel.Security;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Inventario;
using System.Data;

namespace Azteca.Web.Controllers.Inventario
{
    [Route("api/inventario/conceptos")]
    public class ConceptosController : CatalogController<ConceptoFacade, ConceptoService, ConceptoInfo>
    {
        public ConceptosController()
            :base(new ConceptoFacade())
        {

        }

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadList(int idEmpresa, string tipo)
        {
            try
            {
                DataTable RetList = Facade.LoadList(SessionID, idEmpresa, tipo);
                return RetList;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("load-list-by-user")]
        public DataTable LoadList(int idEmpresa, int idUsuario, string tipo)
        {
            try
            {
                DataTable RetList = Facade.LoadList(SessionID, idEmpresa, idUsuario, tipo);
                return RetList;
            }
            catch(Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("users")]
        public List<UsuarioItem> LoadUsers(int idConcepto)
        {
            try
            {
                List<UsuarioItem> Users = Facade.LoadUserAccess(SessionID, idConcepto);
                return Users;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-users")]
        public bool AddUsers(int idConcepto, [FromBody] List<UsuarioItem> users)
        {
            try
            {
                users.ForEach(user =>
                {
                    Facade.AddUserAccess(SessionID, idConcepto, user.ID);
                });
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpGet]
        [Route("remove-user")]
        public bool RemoveUser(int idConcepto, int idUsuario)
        {
            try
            {
                Facade.RemoveUserAccess(SessionID, idConcepto, idUsuario);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

    }
}