﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Mantenimiento;
using Azteca.Kernel.Business;
using System.Data;


namespace Azteca.Web.Controllers.Mantenimiento
{
    [Route("api/mantenimiento/equipos")]
    public class EquiposController : CatalogController<EquipoFacade, EquipoService, EquipoInfo>
    {
        public EquiposController()
            :base(new EquipoFacade())
        {

        }


        [HttpGet]
        [Route("load-documentos")]
        public List<Documento> LoadDocumentos(int idEquipo)
        {
            List<Documento> dtRet = null;

            try
            {
                EquipoFacade Facade = new EquipoFacade();
                dtRet = Facade.LoadDocumentos(SessionID, idEquipo);
            }
            catch(Exception ex)
            {
                HandleException("Error al obtener documentos del equipo", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("agregar-documento")]
        public void AgregarDocumento(int idEquipo, int idTipoDocumento,[FromBody] FileInfo documento)
        {
            try
            {
                EquipoFacade Facade = new EquipoFacade();
                Facade.AgregarDocumento(SessionID, idEquipo, idTipoDocumento, documento);
            }
            catch(Exception ex)
            {
                HandleException("Error al guardar documento del equipo", ex);
            }
        }

        [HttpDelete]
        [Route("eliminar-documento")]
        public void EliminarDocumento(int idEquipo, int idTipoDocumento, Guid idArchivo)
        {
            try
            {
                EquipoFacade Facade = new EquipoFacade();
                Facade.EliminarDocumento(SessionID, idEquipo, idTipoDocumento, idArchivo);
            }
            catch(Exception ex)
            {
                HandleException("Error al eliminar documento del equipo", ex);
            }
        }


        [HttpGet]
        [Route("load-fotografias")]
        public List<FotografiaEquipo> LoadFotografías(int idEquipo)
        {
            List<FotografiaEquipo> dtRet = null;
            try
            {
                EquipoFacade Facade = new EquipoFacade();
                dtRet = Facade.CargarFotografias(SessionID, idEquipo);
            }
            catch(Exception ex)
            {
                HandleException("Error al obtener fotografías del equipo", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("agregar-fotografia")]
        public void AgregarFotografia(int idEquipo, [FromBody] FotografiaEquipo fotografia)
        {
            try
            {
                EquipoFacade Facade = new EquipoFacade();
                Facade.AgregarFotografia(SessionID, idEquipo, fotografia);
            }
            catch (Exception ex)
            {
                HandleException("Error al guardar documento del equipo", ex);
            }
        }

        [HttpDelete]
        [Route("eliminar-fotografia")]
        public void EliminarFotografia(int idEquipo, Guid idArchivo)
        {
            try
            {
                EquipoFacade Facade = new EquipoFacade();
                Facade.EliminarFotografia(SessionID, idEquipo, idArchivo);
            }
            catch (Exception ex)
            {
                HandleException("Error al eliminar fotografía del equipo", ex);
            }
        }

        [HttpGet]
        [Route("search-equipo")]
        public List<EquipoItem> SearchKeyword(int idBranch, string keyword, Int32 idServicioTipo)
        {
            List<EquipoItem> dtRet = null;
            try
            {
                EquipoFacade Facade = new EquipoFacade();
                dtRet = Facade.BuscarEquipos(SessionID, idBranch, keyword, idServicioTipo);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener listado de equipos", ex);
            }
            return dtRet;
        }

    }
}