﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;
using System.Data;
using Azteca.Kernel.Search;

namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/articulos")]
    public class ArticulosController : CatalogController<ArticuloFacade, ArticuloService, ArticuloInfo>
    {
        public ArticulosController()
            : base(new ArticuloFacade())
        {

        }
        
        [HttpPost]
        [Route("search-articulo-proveedor")]
        public SearchResult SearchArticuloProveedor(int idProveedor, int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ArticuloFacade();
                    Result.TotalRows = Facade.GetTotalRecordsArticulosProveedor(this.SessionID, idProveedor, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchArticulosProveedor(this.SessionID, idProveedor, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ArticuloFacade();
                    Result.TotalRows = Facade.GetTotalRecordsArticulosProveedor(this.SessionID, idProveedor, idBranch, null);
                    Result.Rows = Facade.SearchArticulosProveedor(this.SessionID, idProveedor, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda de atículos.", ex);
                return null;
            }
        }


        [HttpGet]
        [Route("costo-proveedor")]
        public CostoInfo CalcularCosto(int idArticulo, int idUnidadMedida, int idProveedor, int idEmpresa)
        {
            try
            {
                CostoInfo Costo = Facade.CalcularCosto(SessionID, idArticulo, idUnidadMedida, idProveedor, idEmpresa);
                return Costo;
            } 
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("get-articulo-impuestos")]
        public List<Azteca.Business.Contabilidad.ImpuestoAsignacionInfo> LoadImpuestos(int idArticulo)
        {
            try
            {
                Facade = new ArticuloFacade();
                return Facade.LoadImpuestos(this.SessionID, idArticulo);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }


    }
}