﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;
using System.Data;
using Azteca.Kernel.Security;

namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/centrales")]
    public class CentralesController : CatalogController<CentralFacade, CentralService, CentralInfo>
    {
        public CentralesController()
            :base(new CentralFacade())
        {

        }

        [HttpGet]
        [Route("load-security-context")]
        public DataTable LoadSecurityContext(int IdEmpresa)
        {
            {
                DataTable dtRet = new DataTable();
                try
                {
                    dtRet = Facade.LoadSecurityContext(SessionID, IdEmpresa);
                }
                catch (Exception ex)
                {
                    HandleException(ex);
                }
                return dtRet;
            }
        }

        #region "Accesos"

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadListByUser(int IdSucursal)
        {
            {
                DataTable dtRet = new DataTable();
                try
                {                    
                    dtRet = Facade.LoadList(SessionID, IdSucursal);
                }
                catch (Exception ex)
                {
                    HandleException(ex);
                }
                return dtRet;
            }
        }

        [HttpGet]
        [Route("members")]
        public List<UsuarioItem> GetMembers(int idCentral)
        {
            try
            {
                List<UsuarioItem> Users = Azteca.Kernel.Common.DataTableUtils.ToList<UsuarioItem>(Facade.LoadCentralAccesos(SessionID, idCentral));
                return Users;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-members")]
        public void AddMembers(int idCentral, [FromBody] List<UsuarioItem> users)
        {
            try
            {
                users.ForEach(user =>
                {
                    Facade.InsertaAcceso(SessionID, idCentral, user.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-member")]
        public void DeleteMember(int idCentral, int idUser)
        {
            try
            {
                Facade.RemoverAcceso(SessionID, idCentral, idUser);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        #endregion

        #region "centros recepcion"

        [HttpGet]
        [Route("get-centros-recepcion")]
        public List<CentroRecepcionItem> GetCentrosRecepcion(int idCentral)
        {
            try
            {
                List<CentroRecepcionItem> CentrosRecepcion = Azteca.Kernel.Common.DataTableUtils.ToList<CentroRecepcionItem>(Facade.GetCentrosRecepcion(SessionID, idCentral));
                return CentrosRecepcion;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-centro-recepcion")]
        public void AddDepartamentos(int idCentral, [FromBody] List<CentroRecepcionItem> CentrosRecepcion)
        {
            try
            {
                CentrosRecepcion.ForEach(item =>
                {
                    Facade.AddCentroRecepcion(SessionID, idCentral, item.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-centro-recepcion")]
        public void DeleteCentroRecepcion(int idCentral, int idCentroRecepcion)
        {
            try
            {
                Facade.RemoveCentroRecepcion(SessionID, idCentral, idCentroRecepcion);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        #endregion

        #region "departamentos"

        [HttpGet]
        [Route("get-central-departamentos")]
        public List<DepartamentoItem> GetDepartamentos(int idCentral)
        {
            try
            {
                List<DepartamentoItem> Departamentos = Azteca.Kernel.Common.DataTableUtils.ToList<DepartamentoItem>(Facade.LoadCentralDepartamentos(SessionID, idCentral));
                return Departamentos;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }
        
        [HttpPost]
        [Route("add-departamento")]
        public void AddDepartamentos(int idCentral, [FromBody] List<DepartamentoItem> departamentos)
        {
            try
            {
                departamentos.ForEach(depto =>
                {
                    Facade.InsertaDepartamento(SessionID, idCentral, depto.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-departamento")]
        public void DeleteDepartamento(int idCentral, int idDepartamento)
        {
            try
            {
                Facade.RemoverDepartamento(SessionID, idCentral, idDepartamento);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        #endregion

    }
}