﻿using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Kernel.Serialization;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;

namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/comprobantes")]
    public class ComprobantesController: DocumentController<ComprobanteFiscalInfo, Azteca.Business.Contabilidad.ComprobanteFiscalDetail, ComprobanteFiscalService, ComprobanteFiscalFacade>
    {
        public ComprobantesController()
            :base(new ComprobanteFiscalFacade())
        {

        }     
       
        
        [HttpPost]
        [Route("validate")]
        public List<CFDValidationResult> Validate(int idEmpresa, [FromBody] JObject file)
        {
            List<CFDValidationResult> Result = new List<CFDValidationResult>();
            try
            {                
                FileInfo Archivo = JsonConvert.DeserializeObject<FileInfo>(file.ToString(), new FileConverter());
                List<CFDValidationResult> Comprobantes = new List<CFDValidationResult>();
                //Verificar si se trata de un comprobante individual (xml) o un archivo comprimido con varios comprobantes (zip)
                if (Archivo.Extension.ToLower() == "zip")
                {
                    FileService FileSvc = new FileService(SessionID);
                    List<FileInfo> Archivos = FileSvc.ExtractFiles(Archivo, "*.xml");                    
                    //Validar cada uno de los comprobantes fiscales
                    foreach (FileInfo extractedFile in Archivos)
                    {
                        string Xml = GetFileXml(extractedFile);
                        CFDValidationResult ArchivoResult = Facade.Validar(SessionID, idEmpresa, Xml);
                        ArchivoResult.FileName = extractedFile.OriginalFileName;
                        Result.Add(ArchivoResult);
                    }
                }
                else if (Archivo.Extension.ToLower() == "xml")
                {
                    string Xml = GetFileXml(Archivo);
                    CFDValidationResult ArchivoResult =Facade.Validar(SessionID, idEmpresa, Xml);
                    ArchivoResult.FileName = Archivo.OriginalFileName;
                    Result.Add(ArchivoResult);
                }
                else
                {
                    throw new BusinessException("Se especificó un archivo no soportado para validación de comprobantes fiscales.");
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("save")]
        public ComprobanteFiscalInfo Save(int idProveedor, int idEmpresa, int idSucursal, [FromBody] string xml)
        {            
            try
            {
                ComprobanteFiscalInfo Comprobante = Facade.Save(SessionID, idProveedor, idEmpresa, idSucursal, xml);
                return Comprobante;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("list-proveedor")]
        public DataTable LoadListByProveedor(int idEmpresa, int idProveedor, DateTimeOffset fechaInicial, DateTimeOffset fechaFinal)
        {
            try
            {
                DataTable dtList = Facade.LoadListByProveedor(SessionID, idEmpresa, idProveedor, fechaInicial, fechaFinal);
                return dtList;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        private string GetFileXml(FileInfo file)
        {
            UTF8Encoding Encoder = new UTF8Encoding(false);
            string xml = Encoder.GetString(file.Content);
            if (xml.Length > 0 && (int)xml[0] == 65279) xml = xml.Substring(1);
            return xml;
        }

        [HttpGet]
        [Route("marcar-documento-rel")]
        public void MarcarDocumentoRel(string IdComprobante, string IdDocumentoRel)
        {
            try
            {
                Facade.MarcarDocumentoRelacionado(SessionID, IdComprobante, IdDocumentoRel);                
            }
            catch (Exception ex)
            {
                HandleException(ex);                
            }
        }

    }
}
