﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;
using System.Data;
using Azteca.Kernel.Business;
using Azteca.Kernel.Security;


namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/departamentos")]
    public class DepartamentosController : CatalogController<DepartamentoFacade, DepartamentoService, DepartamentoInfo>
    {
        public DepartamentosController()
            :base(new DepartamentoFacade())
        {

        }


        #region "Accesos"

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadListByUser(int IdSucursal)
        {
            {
                DataTable dtRet = new DataTable();
                try
                {
                    dtRet = Facade.LoadList(SessionID, IdSucursal);
                }
                catch (Exception ex)
                {
                    HandleException(ex);
                }
                return dtRet;
            }
        }

        [HttpGet]
        [Route("load-list-by-central-user")]
        public List<DepartamentoItem> LoadListByCentralUser(int idCentral, int idSucursal)
        {
            try
            {
                List<DepartamentoItem> Departamentos = Azteca.Kernel.Common.DataTableUtils.ToList<DepartamentoItem>(Facade.LoadListByCentralUser(SessionID, idCentral, idSucursal));
                return Departamentos;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("members")]
        public List<UsuarioItem> GetMembers(int IdDepartamento)
        {
            try
            {
                List<UsuarioItem> Users = Azteca.Kernel.Common.DataTableUtils.ToList<UsuarioItem>(Facade.LoadAccess(SessionID, IdDepartamento));
                return Users;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-members")]
        public void AddMembers(int IdDepartamento, [FromBody] List<UsuarioItem> users)
        {
            try
            {
                users.ForEach(user =>
                {
                    Facade.AddUser(SessionID, IdDepartamento, user.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-member")]
        public void DeleteMember(int IdDepartamento, int IdUser)
        {
            try
            {
                Facade.RemoveUser(SessionID, IdDepartamento, IdUser);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        #endregion

        #region "Centros de Recepcion"

        [HttpGet]
        [Route("load-centros-recepcion")]
        public List<UsuarioItem> GetCentrosRecepcion(int IdDepartamento)
        {
            try
            {
                List<UsuarioItem> Users = Azteca.Kernel.Common.DataTableUtils.ToList<UsuarioItem>(Facade.LoadCentrosRecepcion(SessionID, IdDepartamento));
                return Users;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-centros-recepcion")]
        public void AddCentrosRecepcion(int IdDepartamento, [FromBody] List<CentroRecepcionItem> centrosRecepcion)
        {
            try
            {
                centrosRecepcion.ForEach(centroRecepcion =>
                {
                    Facade.AddCentroRecepcion(SessionID, IdDepartamento, centroRecepcion.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-centro-recepcion")]
        public void DeleteCentroRecepcion(int IdDepartamento, int IdCentroRecepcion)
        {
            try
            {
                Facade.RemoveCentroRecepcion(SessionID, IdDepartamento, IdCentroRecepcion);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        #endregion


        #region "Categorias Presupuestales"

        [HttpGet]
        [Route("load-categorias-presupuestales")]
        public List<CategoriasPresupuestalesItem> GetCategoriasPresupuestales(int IdDepartamento)
        {
            try
            {
                List<CategoriasPresupuestalesItem> CategoriasPres = Azteca.Kernel.Common.DataTableUtils.ToList<CategoriasPresupuestalesItem>(Facade.LoadCategoriasPresupuestales(SessionID, IdDepartamento));
                return CategoriasPres;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-categoria-presupuestal")]
        public void AddCategoriaPresupuestal(int IdDepartamento, [FromBody] List<CategoriasPresupuestalesItem> categoriasPresupuestales)
        {
            try
            {
                categoriasPresupuestales.ForEach(categoriaPresupuestal =>
                {
                    Facade.AddCategoriaPresupuestal(SessionID, IdDepartamento, categoriaPresupuestal.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-categoria-presupuestal")]
        public void DeleteCategoriaPresupuestal(int IdDepartamento, int IdCategoriaPresupuestal)
        {
            try
            {
                Facade.RemoveCategoriaPresupuestal(SessionID, IdDepartamento, IdCategoriaPresupuestal);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        #endregion



    }
}