﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;


namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/ordenes")]
    public class OrdenesController: DocumentController<OrdenInfo, OrdenDetail, OrdenService, OrdenFacade>
    {
        public OrdenesController()
            :base(new OrdenFacade())
        {

        }

        [HttpPost]
        [Route("save")]
        public OrdenInfo Save([FromBody] JObject ordenJson)
        {
            try
            {
                OrdenInfo Orden = Facade.Deserialize(SessionID, ordenJson);
                OrdenInfo UpdatedOrden = Facade.Update(SessionID, Orden);
                return UpdatedOrden;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("pdf-orden")]
        public FileLink GenerarPdfOrden(string idOrden)
        {
            try
            {
                FileLink FileUri = Facade.GenerarPdf(SessionID, idOrden);
                return FileUri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("requisiciones-pendientes")]
        public List<OrdenRequisicionDetail> RequisicionesPendientesList(int IdCentral, DateTime FechaIni, DateTime FechaFin)
        {
            try
            {
                List<OrdenRequisicionDetail> Requisiones = Facade.RequisicionesPendientesList(SessionID, IdCentral, FechaIni, FechaFin);
                return Requisiones;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("generar-desde-requisiciones")]
        public bool GenerarDesdeRequisiciones(int IdCentral, int IdDepartamento, [FromBody] List<OrdenRequisicionDetail> Requisiciones, int IdSucursal, int IdEmpresa)
        {
            try
            {                
                Facade.GenerarDesdeRequisiciones(this.SessionID, IdCentral, IdDepartamento, Requisiciones,IdSucursal, IdEmpresa);
                return true;
            }
            catch (BusinessException ex)
            {
                HandleException(ex);                
            }
            catch (Exception ex)
            {
                HandleException(ex);                
            }
            return false;
        }
                       
        [HttpGet]
        [Route("get-comprobante-relacionado")]
        public ComprobanteFiscalInfo GetComprobanteRelacionado(string IdOrden)
        {
            try
            {
                ComprobanteFiscalInfo Comprobante = Facade.GetComprobanteRelacionado(SessionID, IdOrden);
                return Comprobante;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

    }
}
