﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;

namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/presupuestos")]
    public class PresupuestosController: DocumentController<PresupuestosInfo, PresupuestosDetail, PresupuestosService, PresupuestosFacade>
    {
        public PresupuestosController()
            :base(new PresupuestosFacade())
        {

        }


        [HttpGet]
        [Route("load-categorias-presupuestales")]
        public List<PresupuestoCategoriaPresupuestalItem> GetCategoriasPresupuestales(int IdDepartamento,string IdPresupuesto)
        {
            try
            {
                List<PresupuestoCategoriaPresupuestalItem> CategoriasPres = Azteca.Kernel.Common.DataTableUtils.ToList<PresupuestoCategoriaPresupuestalItem>(Facade.LoadCategoriasPresupuestales(SessionID, IdDepartamento,IdPresupuesto));
                return Facade.LoadArticulosServicios(SessionID,CategoriasPres,IdPresupuesto);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("load-categorias-presupuestales-con-articulo")]
        public List<CategoriasPresupuestalesItem> GetCategoriasPresupuestalesConArticulo(string IdPresupuesto, int IdArticulo)
        {
            try
            {
                List<CategoriasPresupuestalesItem> CategoriasPres = Azteca.Kernel.Common.DataTableUtils.ToList<CategoriasPresupuestalesItem>(Facade.LoadCategoriasConArticulo(SessionID,  IdPresupuesto, IdArticulo));
                return CategoriasPres;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("load-categorias-presupuestales-con-servicio")]
        public List<CategoriasPresupuestalesItem> GetCategoriasPresupuestalesConServicio(string IdPresupuesto, int IdServicio)
        {
            try
            {
                List<CategoriasPresupuestalesItem> CategoriasPres = Azteca.Kernel.Common.DataTableUtils.ToList<CategoriasPresupuestalesItem>(Facade.LoadCategoriasConServicio(SessionID, IdPresupuesto, IdServicio));
                return CategoriasPres;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("load-presupuestos")]
        public List<PresupuestoItem>LoadPresupuestos(int IdDepartamento)
        {
            try
            {
                List<PresupuestoItem> presupuestos = new List<PresupuestoItem>();
                return Facade.LoadListByDepartamento(SessionID, IdDepartamento);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("unlock")]
        public void Unlock(string IdPresupuesto)
        {
            try
            {
                Facade.Unlock(SessionID, IdPresupuesto);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpPost]
        [Route("lock")]

        public void Lock(string IdPresupuesto)
        {
            try
            {
                Facade.Lock(SessionID, IdPresupuesto);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpPost]
        [Route("change")]
        public void Cambiar([FromBody]PresupuestosInfo Presupuesto)
        {
            try
            {
                Facade.Cambiar(SessionID, Presupuesto);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        
        [HttpGet]
        [Route("pdf-presupuesto")]
        public FileLink GenerarPdfPresupuesto(string idPresupuesto)
        {
            try
            {
                FileLink FileUri = Facade.GenerarPdf(SessionID, idPresupuesto);
                return FileUri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

    }
}
