﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;
using System.Data;
using Azteca.Kernel.Search;
using Azteca.Kernel.General;

namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/proveedores")]
    public class ProveedoresController : CatalogController<ProveedorFacade, ProveedorService, ProveedorInfo>
    {
        public ProveedoresController()
            : base(new ProveedorFacade())
        {

        }

        [HttpGet]
        [Route("enviar-registro")]
        public void EnviarMailRegistro(int idProveedor)
        {
            try
            {
                Facade.EnviarMailRegistro(SessionID, idProveedor);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("registrar")]
        public void Registrar(int idProveedor, string password)
        {
            try
            {
                Facade.Registrar(SessionID, idProveedor, password);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("detail-by-user")]
        public ProveedorInfo GetDetailByUser(int idUsuario, int idEmpresa)
        {
            try
            {
                ProveedorInfo proveedor = Facade.GetDetailByUser(SessionID, idUsuario, idEmpresa);
                return proveedor;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("empresas")]
        public DataTable LoadEmpresas(int idUsuario)
        {
            try
            {
                DataTable dtEmpresas = Facade.LoadEmpresas(SessionID, idUsuario);
                return dtEmpresas;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("search-proveedor-by-zona")]
        public SearchResult SearchProveedorByZona(int idZonaCompra, int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorByZona(this.SessionID, idZonaCompra, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchProveedorByZona(this.SessionID, idZonaCompra, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorByZona(this.SessionID, idZonaCompra, idBranch, null);
                    Result.Rows = Facade.SearchProveedorByZona(this.SessionID, idZonaCompra, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda de atículos.", ex);
                return null;
            }
        }

        /////////////////////// Articulos ///////////////////////
        [HttpPost]
        [Route("update-proveedor-articulo")] 
        public bool UpdateProveedArticulo([FromBody] ProveedorArticulo proveedorArticulo)
        {
            try
            {
                Facade = new ProveedorFacade();
                Facade.UpdateProveedorArticulo(this.SessionID, proveedorArticulo);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }

        [HttpGet]
        [Route("get-detail-proveedor-articulo")]
        public ProveedorArticulo GetDetailProveedorArticulo(int idProveedorArticulo)
        {
            try
            {
                Facade = new ProveedorFacade();
                return Facade.GetDetailProveedorArticulo(this.SessionID, idProveedorArticulo);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost]
        [Route("delete-proveedor-articulo")]
        public bool DeleteProveedorArticulo(int idProveedorArticulo)
        {
            try
            {
                Facade = new ProveedorFacade();
                Facade.DeleteProveedorArticulo(this.SessionID, idProveedorArticulo);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }

        [HttpPost]
        [Route("search-proveedor-articulo")]
        public SearchResult SearchProveedorArticulo(int idArticulo, int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorArticulo(this.SessionID, idArticulo, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchProveedorArticulo(this.SessionID, idArticulo, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorArticulo(this.SessionID, idArticulo, idBranch, null);
                    Result.Rows = Facade.SearchProveedorArticulo(this.SessionID, idArticulo, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda de atículos.", ex);
                return null;
            }
        }

        [HttpPost]
        [Route("search-proveedor-articulo-by-zona")]
        public SearchResult SearchProveedorArticuloByZona(int idZonaCompra, int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorArticuloByZona(this.SessionID, idZonaCompra, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchProveedorArticuloByZona(this.SessionID, idZonaCompra, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorArticuloByZona(this.SessionID, idZonaCompra, idBranch, null);
                    Result.Rows = Facade.SearchProveedorArticuloByZona(this.SessionID, idZonaCompra, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda de atículos.", ex);
                return null;
            }
        }


        /////////////////////// Servicios ///////////////////////

        [HttpPost]
        [Route("update-proveedor-servicio")]
        public bool UpdateProveedServicio([FromBody] ProveedorServicio proveedorServicio)
        {
            try
            {
                Facade = new ProveedorFacade();
                Facade.UpdateProveedorServicio(this.SessionID, proveedorServicio);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }

        [HttpGet]
        [Route("get-detail-proveedor-servicio")]
        public ProveedorServicio GetDetailProveedorServicio(int idProveedorServicio)
        {
            try
            {
                Facade = new ProveedorFacade();
                return Facade.GetDetailProveedorServicio(this.SessionID, idProveedorServicio);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost]
        [Route("delete-proveedor-servicio")]
        public bool DeleteProveedorServicio(int idProveedorServicio)
        {
            try
            {
                Facade = new ProveedorFacade();
                Facade.DeleteProveedorServicio(this.SessionID, idProveedorServicio);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }
        
        [HttpPost]
        [Route("search-proveedor-servicio")]
        public SearchResult SearchProveedorServicio(int idServicio, int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorServicio(this.SessionID, idServicio, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchProveedorServicio(this.SessionID, idServicio, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorServicio(this.SessionID, idServicio, idBranch, null);
                    Result.Rows = Facade.SearchProveedorServicio(this.SessionID, idServicio, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda de servicios.", ex);
                return null;
            }
        }

        [HttpPost]
        [Route("search-proveedor-servicio-by-zona")]
        public SearchResult SearchProveedorServicioByZona(int idZonaCompra, int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorServicioByZona(this.SessionID, idZonaCompra, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchProveedorServicioByZona(this.SessionID, idZonaCompra, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ProveedorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsProveedorServicioByZona(this.SessionID, idZonaCompra, idBranch, null);
                    Result.Rows = Facade.SearchProveedorServicioByZona(this.SessionID, idZonaCompra, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda de servicio.", ex);
                return null;
            }
        }

        /////////////////////// Pre Registro ///////////////////////

        [HttpGet]
        [Route("registrar-de-preregistro")]
        public void Registrar(int idProveedor, string password, int idPreRegistro, string correo)
        {
            try
            {
                Facade.Registrar(SessionID, idProveedor, password, idPreRegistro, correo);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }

        }

        [HttpPost]
        [Route("pre-registrar")]
        public void PreRegistrar([FromBody] ProveedorPreRegistro preRegistro)
        {
            try
            {
                Facade.PreRegistrar(SessionID, preRegistro);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpPost]
        [Route("pre-registro-delete")]
        public void PreRegistroEliminar(int idPreRegistro)
        {
            try
            {
                Facade.PreRegistroEliminar(SessionID, idPreRegistro);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("pre-registros")]
        public List<ProveedorPreRegistro> PreRegistros()
        {
            try
            {
                List<ProveedorPreRegistro> pRegistros = (Facade.PreRegistros(SessionID));
                return pRegistros;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("load-contactos")]
        public List<PersonaInfo> LoadContactos(int idProveedor)
        {
            try
            {
                List<PersonaInfo> Contacts = Azteca.Kernel.Common.DataTableUtils.ToList<PersonaInfo>(Facade.LoadListContacto(SessionID, idProveedor));
                return Contacts;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


        [HttpPost]
        [Route("add-contacto")]
        public void AddMembers(int idProveedor, [FromBody] PersonaInfo contacto)
        {
            try
            {
                {
                    Facade.UpdateContacto(SessionID, idProveedor, contacto.Identity, contacto.Nombre, contacto.ApellidoPaterno, contacto.ApellidoMaterno, contacto.TelefonoOficina, contacto.Celular, contacto.Correo);
                }
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-contacto")]
        public void RemoveContacto(int idProveedor, int idPersona)
        {
            try
            {
                Facade.RemoverContacto(SessionID, idProveedor, idPersona);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }
    }
}