﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;
using System.Data;
using Azteca.Kernel.Search;


namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/servicios")]
    public class ServiciosController : CatalogController<ServicioFacade, ServicioService, ServicioInfo>
    {
        public ServiciosController()
            :base(new ServicioFacade())
        {

        }

        [HttpPost]
        [Route("search-servicio-proveedor")]
        public SearchResult SearchServicioProveedor(int idProveedor, int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ServicioFacade();
                    Result.TotalRows = Facade.GetTotalRecordsServiciosProveedor(this.SessionID, idProveedor, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchServiciosProveedor(this.SessionID, idProveedor, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ServicioFacade();
                    Result.TotalRows = Facade.GetTotalRecordsServiciosProveedor(this.SessionID, idProveedor, idBranch, null);
                    Result.Rows = Facade.SearchServiciosProveedor(this.SessionID, idProveedor, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda de atículos.", ex);
                return null;
            }
        }


        [HttpGet]
        [Route("costo-proveedor")]
        public CostoInfo CalcularCosto(int idServicio, int idProveedor, int idEmpresa, decimal costoBruto)
        {
            try
            {
                CostoInfo Costo = Facade.CalcularCosto(SessionID, idServicio, idProveedor, idEmpresa, costoBruto);
                return Costo;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

    }
}