﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Compras;
using System.Data;
using Azteca.Kernel.Security;
using Azteca.Kernel.Search;

namespace Azteca.Web.Controllers.Compras
{
    [Route("api/compras/zonas-compra")]
    public class ZonasCompraController : CatalogController<ZonaCompraFacade, ZonaCompraService, ZonaCompraInfo>
    {
        public ZonasCompraController()
            :base(new ZonaCompraFacade())
        {

        }

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadListByUser(int IdSucursal)
        {
            {
                DataTable dtRet = new DataTable();
                try
                {
                    dtRet = Facade.LoadList(SessionID, IdSucursal);
                }
                catch (Exception ex)
                {
                    HandleException(ex);
                }
                return dtRet;
            }
        }

        [HttpPost]
        [Route("add-user")]
        public void AddUser(int IdZonaCompra, [FromBody] List<UsuarioItem> usuarios)
        {
            try
            {
                foreach (UsuarioItem user in usuarios)
                {
                    Facade.AddUser(SessionID, user.ID, IdZonaCompra);
                }                
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("load-access-users-list")]
        public DataTable LoadZonasCompraAccess(int IdZonaCompra)
        {
            DataTable dtAccess = new DataTable();
            try
            {
                dtAccess = Facade.LoadZonasCompraAccess(SessionID,IdZonaCompra);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dtAccess;
        }

        [HttpPost]
        [Route("remove-user")]
        public void RmoveUser(int IdZonaCompraAcceso)
        {
            try
            {
                Facade.RemoveUser(SessionID,IdZonaCompraAcceso);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("search-zona-compra")]
        public DataTable SearchZonaCompra(int idSucursal, int idUsuario,string keyword)
        {
            DataTable dtAccess = new DataTable();
            try
            {
                dtAccess = Facade.SearchZonaCompra(SessionID, idSucursal, keyword, idUsuario);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dtAccess;
        }

        [HttpGet]
        [Route("load-resticciones-costos")]
        public DataTable LoadRestriccionesCostos()
        {
            DataTable dt = new DataTable();
            try
            {
                dt = Facade.LoadRestriccionesCostos(SessionID);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dt;
        }

        [HttpPost]
        [Route("search2")]
        public SearchResult Search2(int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                    //
                    Result.TotalRows = Facade.GetTotalRecords2(SessionID, idBranch, request.Conditions);
                    Result.Rows = Facade.Search2(SessionID, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Result.TotalRows = Facade.GetTotalRecords2(SessionID, idBranch, null);
                    Result.Rows = Facade.Search2(SessionID, idBranch, null, null,1,20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda.", ex);
                return null;
            }
        }

        [HttpGet]
        [Route("has-access")]
        public bool HasAccessZonaCompra(int idUsuario,int idZonaCompra)
        {
            try
            {
                ZonaCompraFacade facade = new ZonaCompraFacade();
                return facade.HasAccessZonaCompra(SessionID, idUsuario, idZonaCompra);
            }
            catch (Exception ex)
            {
                HandleException("Error al verificar el acceso del usuario a la zona de compra.", ex);
            }
            return false;
        }
    }
}