﻿using Azteca.Business.RH;
using Azteca.Web.Controllers.Kernel;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Threading.Tasks;

namespace Azteca.Web.Controllers.RH
{

    [Route("api/rh/adam")]
    public class AdamController : BaseController
    {

        [HttpGet]
        [Route("load-compania")]
        public DataTable LoadCompania(int idEmpresaAdam)
        {
            DataTable dtResult = null;
            try
            {
                ADAMFacade ADAMFac = new ADAMFacade();
                dtResult = ADAMFac.LoadCompaniasAdam(this.SessionID, idEmpresaAdam);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar las compañías de Adam", ex);
            }
            return dtResult;
        }


        [HttpGet]
        [Route("load-clase-nomina")]
        public DataTable LoadClaseNomina(int idEmpresaAdam)
        {
            DataTable dtResult = null;
            try
            {
                ADAMFacade ADAMFac = new ADAMFacade();
                dtResult = ADAMFac.LoadClaseNominaAdam(this.SessionID, idEmpresaAdam);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar las clases de nómina", ex);
            }
            return dtResult;
        }

        [HttpGet]
        [Route("load-tipo-nomina")]
        public DataTable LoadTipoNomina(int idEmpresaAdam, string claseNomina)
        {
            DataTable dtResult = null;
            try
            {
                ADAMFacade ADAMFac = new ADAMFacade();
                dtResult = ADAMFac.LoadTipoNominaAdam(this.SessionID, idEmpresaAdam, claseNomina);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los tipos de nómina", ex);
            }
            return dtResult;
        }

        [HttpGet]
        [Route("load-anio")]
        public DataTable LoadAnio(int idEmpresaAdam, string compania, string tipoNomina)
        {
            DataTable dtResult = null;
            try
            {
                ADAMFacade ADAMFac = new ADAMFacade();
                dtResult = ADAMFac.LoadAnios(this.SessionID, idEmpresaAdam, compania, tipoNomina);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los años", ex);
            }
            return dtResult;
        }

        [HttpGet]
        [Route("load-periodo")]
        public DataTable LoadPeriodo(int idEmpresaAdam, string compania, string tipoNomina, int anio)
        {
            DataTable dtResult = null;
            try
            {
                ADAMFacade ADAMFac = new ADAMFacade();
                dtResult = ADAMFac.LoadPeriodos(this.SessionID, idEmpresaAdam, compania, tipoNomina, anio);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los periodos", ex);
            }
            return dtResult;
        }

        [HttpPost]
        [Route("interfas-poliza-adam")]
        public bool InterfasPolizaAdam(int numero_liquidacion,string tipoPoliza, [FromBody] AdamNominaInfo adamNomina)
        {
            try
            {
                ADAMFacade Facade = new ADAMFacade();
                Facade.ImportarPolizas(SessionID, numero_liquidacion, tipoPoliza, adamNomina);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al guardar documento del equipo", ex);
                return false;
            }
        }

        [HttpGet]
        [Route("load-cuentas-faltantes")]
        public List<CuentaFaltanteADAM> LoadCuentasFaltantes(int idEmpresaAdam, string compania, string claseNomina, string tipoNomina, int anio, int periodo)
        {
            List<CuentaFaltanteADAM> lResult = null;
            try
            {
                ADAMFacade Fac = new ADAMFacade();
                lResult = Fac.CuentasFaltantes(SessionID, idEmpresaAdam, compania, claseNomina, tipoNomina, anio.ToString(), periodo, 0.ToString(), "N");
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar las cuentas faltantes", ex);
            }
            return lResult;
        }

    }
}
