﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.RH;
using System.Data;


namespace Azteca.Web.Controllers.RH
{
    [Route("api/rh/cedulasfiniquitos")]
    public class CedulasFiniquitosController : CatalogController<CedulaFiniquitoFacade, CedulaFiniquitoService, CedulaFiniquitoInfo>
    {
        public CedulasFiniquitosController()
            : base(new CedulaFiniquitoFacade())
        {

        }

        [HttpGet]
        [Route("load-ingreso-informacion")]
        public CedulaFiniquitoInfo LoadIngresoInformacion(int idEmpleado, bool calcualr)
        {
            try
            {
                CedulaFiniquitoFacade CedulaFac = new CedulaFiniquitoFacade();
                return CedulaFac.LoadIngresoInformacion(this.SessionID, idEmpleado, calcualr);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al cargar la información de la cedula", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("load-rescisiones")]
        public List<RescisionLaboral> LoadRescisiones()
        {
            try
            {
                CedulaFiniquitoFacade CedulaFac = new CedulaFiniquitoFacade();
                return CedulaFac.LoadRescisiones(this.SessionID);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al cargar la información de la cedula", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("get-tarifa-provicional")]
        public TarifaPagoProvicional GetTarifaPagoProvicional(int idCompania, decimal importe)
        {
            try
            {
                CedulaFiniquitoFacade CedulaFac = new CedulaFiniquitoFacade();
                return CedulaFac.GetTarifaPagoProvicional(this.SessionID, idCompania, importe);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al cargar la tarifa provicional", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("get-subsidio-empleo")]
        public SubsidioEmpleo GetSubsidioEmpleo(int idCompania, decimal importe)
        {
            try
            {
                CedulaFiniquitoFacade CedulaFac = new CedulaFiniquitoFacade();
                return CedulaFac.GetSubsidioEmpleo(this.SessionID, idCompania, importe);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al cargar la tarifa provicional", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("generar-cedula-finiquito")]
        public byte[] GenerarCedulaFiniquito(int idCedulaFiniquito)
        {
            try
            {
                CedulaFiniquitoFacade CedulaFiniquitoFac = new CedulaFiniquitoFacade();
                return CedulaFiniquitoFac.GenerarCedulaFiniquito(this.SessionID, idCedulaFiniquito);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al generar la cédula de finquito", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("load-cedula-finiquito-adeudo")]
        public byte[] LoadCedulaFiniquitoAdeudo(int idEmpleado)
        {
            try
            {
                CedulaFiniquitoFacade CedulaFiniquitoFac = new CedulaFiniquitoFacade();
                return CedulaFiniquitoFac.LoadCedulaFiniquitoAdeudo(this.SessionID, idEmpleado);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al generar la cédula de finquito", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("load-kardex-asistencia")]
        public byte[] LoadKardexAsistencia(string empleado, DateTime fechaIni, DateTime fechaFin)
        {
            try
            {
                CedulaFiniquitoFacade CedulaFiniquitoFac = new CedulaFiniquitoFacade();
                return CedulaFiniquitoFac.LoadKardexAsistencia(this.SessionID, empleado, fechaIni, fechaFin);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al generar el Kardex de asistencia", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("get-task-comision")]
        public AsyncTaskInfo GetTaskComision(int idTask)
        {
            try
            {
                CedulaFiniquitoFacade CedulaFac = new CedulaFiniquitoFacade();
                return CedulaFac.GetTaskComision(this.SessionID, idTask);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al consultar la comsión del empleado", ex);
            }
            return null;
        }


        [HttpPost]
        [Route("update-cedula")]
        public CedulaFiniquitoInfo UpdateCedula([FromBody] CedulaFiniquitoInfo catalogInfo)
        {
            try
            {
                CedulaFiniquitoFacade CedulaFac = new CedulaFiniquitoFacade();
                return CedulaFac.Update(this.SessionID, catalogInfo);
            }
            catch (Exception ex)
            {
                HandleException("Ocurrió un problema al acualizar la cédula", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("get-adeudo-empleado")]
        public decimal LoadAdedudosEmpleado(Guid sessionID, string compania, string numEmpleado, DateTime fechaBaja)
        {
            try
            {
                CedulaFiniquitoFacade Facade = new CedulaFiniquitoFacade();
                return Facade.LoadAdedudosEmpleado(this.SessionID, compania, numEmpleado, fechaBaja);
            }
            catch (Exception ex)
            {
                HandleException("Error al consultar el adeudo del empleado", ex);
            }
            return 0;
        }

    }
}