﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.RH;
using System.Data;
using Azteca.Kernel.Search;
using Azteca.Kernel.Security;

namespace Azteca.Web.Controllers.RH
{
    [Route("api/rh/checadores")]
    public class ChecadoresController : CatalogController<ChecadorFacade, ChecadorService, ChecadorInfo>
    {
        public ChecadoresController()
            :base(new ChecadorFacade())
        {

        }

        [HttpPost]
        [Route("test-connection")]
        public bool TestConnection([FromBody] ChecadorInfo checador)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.TestConnection(this.SessionID, checador);
            }
            catch(Exception ex)
            {
                HandleException(ex);                
            }
            return false;
        }

        [HttpGet]
        [Route("import-users")]
        public List<ChecadorUsuarioItem> ImportUsers(int idChecador)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.ImportUser(this.SessionID, idChecador);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost]
        [Route("export-users")]
        public List<ChecadorUsuarioItem> ExportUsers(int idChecador, bool sincronizarChecador, [FromBody] List<ChecadorUsuarioItem> usuarios)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.ExportUsers(this.SessionID, idChecador, sincronizarChecador, usuarios);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost]
        [Route("export-employees-to-checks")]
        public List<ChecadorUsuarioItem> ExportEmployeesToChecks([FromBody] ChecadoresUsuariosCollections ChecadoresUsuarios)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.ExportEmployeesToChecks(this.SessionID, ChecadoresUsuarios);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }
        

       [HttpPost]
        [Route("download-transacciones")]
        public bool DownloadTransacciones(int idChecador)
        {
            try
            {
                Facade = new ChecadorFacade();
                Facade.DownloadTransacciones(this.SessionID, idChecador);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }

        [HttpPost]
        [Route("update-transacciones")]
        public bool UpdateTransacciones(int idChecador, List<TransaccionInfo> transacciones)
        {
            try
            {
                Facade = new ChecadorFacade();
                Facade.UpdateTransacciones(this.SessionID, idChecador, transacciones);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("import-user")]
        public ChecadorUsuarioItem ImportUser(int idChecador, [FromBody] ChecadorUsuarioItem item)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.ImportUser(SessionID, idChecador, item);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return item;
        }

        [HttpPost]
        [Route("import-new-user")]
        public List<ChecadorUsuarioItem> ImportNewUser(int idChecador, [FromBody] List<ChecadorUsuarioItem> items)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.ImportUser(SessionID, idChecador, items, true);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return items;
        }

        [HttpPost]
        [Route("export-fingers")]
        public bool ExportFingers(int idChecador, [FromBody] ChecadorUsuarioItem item)
        {
            try
            {
                Facade = new ChecadorFacade();
                Facade.ExportFinger(SessionID, idChecador, item);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("delete-user")]
        public bool DeleteUser(int idChecador, [FromBody] ChecadorUsuarioItem item)
        {
            try
            {
                Facade = new ChecadorFacade();
                Facade.DeleteUser(SessionID, idChecador, item);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }


        [HttpGet]
        [Route("get-transacciones-empleado")]
        public List<TransaccionItem> GetTransaccionesEmpleado(DateTime fechaIni, DateTime fechaFin, int idEmpleado)
        {
            try
            {
                return Facade.GetTransaccionesEmpleado(SessionID, fechaIni, fechaFin, idEmpleado);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost]
        [Route("update-transacciones-empleado")]
        public bool UpdateTransaccionesEmpleado([FromBody] List<TransaccionItem> transacciones)
        {
            try
            {
                Facade.UpdateTransaccion(SessionID, transacciones);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("procesar-transacciones-empleado")]
        public bool ProcesarTransaccionesEmpleado([FromBody] List<TransaccionItem> transacciones)
        {
            try
            {
                Facade.UpdateTransaccion(SessionID, transacciones, true);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("search-checadores-eventos")]
        public DataTable SearchChecadoresEventos(int idBranch, DateTime FechaIni, DateTime FechaFin)
        {
            DataTable dt= null;
            try
            {
                ChecadorFacade Facade = new ChecadorFacade();
                dt = Facade.SearchEventos(this.SessionID, idBranch,FechaIni,FechaFin);
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda.", ex);
            }

            return dt;
        }

        [HttpPost]
        [Route("evento-status")]
        public bool EventoStatus(int idEventoUsuario, bool status, int idUsuario)
        {
            try
            {
                ChecadorFacade Facade = new ChecadorFacade();
                Facade.EventoStatus(this.SessionID, idEventoUsuario, status, idUsuario);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al cambiar el estatus del evento.", ex);
                return false;
            }
        }

        [HttpPost]
        [Route("save-evento-usuario")]
        public bool SaveEventoUsuario([FromBody] EventosUsuarioInfo eventoUsuario)
        {
            try
            {
                ChecadorFacade Facade = new ChecadorFacade();
                return Facade.SaveEventoUsuario(SessionID, eventoUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Error al guardar el Evento", ex);
                return false;
            }
        }

        [HttpGet]
        [Route("get-evento-usuario")]
        public DataTable GetEventoUsuario(int idEventoUsuario)
        {
            try
            {
                ChecadorFacade Facade = new ChecadorFacade();
                return Facade.GetEventosUsuarioInfo(SessionID, idEventoUsuario);
            }
            catch (Exception ex)
            {
                HandleException("Error al consultar el evento", ex);
                return null; 
            }
        }

        [HttpGet]
        [Route("get-id-checador")]
        public int GetIdChecador(string numSerie)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.GetIdChecador(SessionID, numSerie);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return 0;
            }
        }

        [HttpGet]
        [Route("get-empleados-checador")]
        public DataTable GetEmpleadosChecador(int idChecador)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.GetEmpleadosChecador(SessionID, idChecador);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("get-empleado-info")]
        public DataTable GetEmpleadoInfo(int idEmpleado)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.GetEmpleadoInfo(SessionID, idEmpleado);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("get-huellas-empleado")]
        public DataTable GetHuellasEmpleado(int idEmpleado,int idChecador)
        {
            try
            {
                Facade = new ChecadorFacade();
                return Facade.GetHuellasEmpleado(SessionID, idEmpleado,idChecador);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
                throw;
            }
        }

        [HttpPost]
        [Route("save-empleado-checador")]
        public bool SaveEmpleadoChecador(int idChecador,[FromBody]ChecadorUsuarioCel empleado)
        {
            try
            {
                Facade = new ChecadorFacade();
                Facade.SetUsuarioChecador(SessionID, idChecador, empleado);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("save-huellas-empleado")]
        public bool SaveHuellasEmpleado(int idChecador, [FromBody]List<HuellaUsuarioCel> huellas)
        {
            try
            {
                Facade = new ChecadorFacade();
                Facade.SaveHuellaUsuario(SessionID, idChecador, huellas);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("verify-finger")]
        public Dictionary<string, object> VerifyFinger( int idChecador,string  fecha, int timeZone,bool isMenu, [FromBody]string huella)
        {
            try
            {
                DateTime time = Convert.ToDateTime(fecha);
                Facade = new ChecadorFacade();
                return Facade.VerifyFinger(SessionID, huella, idChecador, time, timeZone, isMenu);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-user")]
        public void AddUser(int idChecador, int userUpdate, [FromBody] List<UsuarioItem> usuarios)
        {
            try
            {
                foreach (UsuarioItem user in usuarios)
                {
                    Facade.AddUser(SessionID, idChecador,user.ID,userUpdate);
                }
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("load-access-users-list")]
        public DataTable LoadZonasCompraAccess(int idChecador)
        {
            DataTable dtAccess = new DataTable();
            try
            {
                dtAccess = Facade.LoadAcceos(SessionID, idChecador);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dtAccess;
        }

        [HttpPost]
        [Route("remove-user")]
        public void RemoveUser(int idChecadorAcceso)
        {
            try
            {
                Facade.RemoveUser(SessionID, idChecadorAcceso);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("get-periodos-trabajo-empleado")]
        public DataTable GetPeriodoTrabajoEmpleados(int idEmpresa,int idSucursal, DateTime fecha)
        {
            DataTable dt = null;
            try
            {
                ChecadorFacade facade = new ChecadorFacade();
                dt= facade.GetPeriodosTrabajoEmpleados(SessionID, idEmpresa, fecha, idSucursal);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return dt;
        }

        [HttpPost]
        [Route("planchar-checadas")]
        public bool PlancharChecadas([FromBody] DataTable empleados, int idMotivo,int idUsuario,string Usuario)
        {
            try
            {
                ChecadorFacade facade = new ChecadorFacade();
                facade.PlancharChecadas(SessionID, empleados, idMotivo, idUsuario, Usuario);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("search-admin-checadores")]
        public SearchResult SearchAdminChecadores(int idUsuario, int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ChecadorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsAdminChecadores(this.SessionID, idUsuario, idBranch, request.Conditions);
                    Result.Rows = Facade.SearchAdminChecadores(this.SessionID, idUsuario, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ChecadorFacade();
                    Result.TotalRows = Facade.GetTotalRecordsAdminChecadores(this.SessionID, idUsuario, idBranch, null);
                    Result.Rows = Facade.SearchAdminChecadores(this.SessionID, idUsuario, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda de checadores.", ex);
                return null;
            }
        }


    }
}