﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.RH;
using System.Data;
using Azteca.Kernel.Business;

namespace Azteca.Web.Controllers.RH
{
    [Route("api/rh/empleados")]
    public class EmpleadosController : CatalogController<EmpleadoFacade, EmpleadoService, EmpleadoInfo>
    {
        public EmpleadosController()
            :base(new EmpleadoFacade())
        {

        }

        [HttpGet]
        [Route("load-employe-code-company")]
        public DataTable LoadEmpleadoCodigoCompania(string codigo, int idEmpresa)
        {
            DataTable dtResult = null;
            try
            {
                dtResult = Facade.LoadEmpleadoCodigoCompania(this.SessionID, codigo, idEmpresa);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar la información del empleado", ex);
            }
            return dtResult;
        }

        [HttpGet]
        [Route("load-documentos")]
        public List<DocumentoEmpleadoItem> LoadDocumentos(int idEmpleado)
        {
            try
            {
                return Facade.GetDocumentos(this.SessionID, idEmpleado);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los documentos del empleado", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("get-documento")]
        public FileInfo GetDocumento(string idArchivo)
        {
            try
            {
                return Facade.GetDocumento(this.SessionID, idArchivo);
            }
            catch (BusinessException ex)
            {
                HandleException("Error al cargar el documento del empleado", ex);
            }
            return null;
        }

        [HttpPost]
        [Route("update-documento-empleado")]
        public DocumentoEmpleadoItem UpdateDocumento(int idEmpleado, [FromBody] DocumentoEmpleadoItem item)
        {
            try
            {
                return Facade.UpdateDocumento(SessionID, idEmpleado, item);
            }
            catch (BusinessException ex)
            {
                HandleException("Error al guardar el documento del empleado", ex);
            }
            return null;
        }

        [HttpDelete]
        [Route("delete-documento-empleado")]
        public bool DeleteDocumento(int idDocumentoEmpleado)
        {
            try
            {
                Facade.DeleteDocumento(SessionID, idDocumentoEmpleado);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al eliminar el documento del empleado", ex);
            }
            return false;
        }

        [HttpGet]
        [Route("importar-empleados-adam")]
        public Azteca.Kernel.Queue.AsyncTaskInfo ImportarEmpladosAdam(int idSucursal)
        {
            try
            {
                return Facade.GetAsyncImportarEmpleadosAdam(SessionID, idSucursal);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener datos a conciliar", ex);
            }
            return null;
        }

        [HttpPost]
        [Route("programar-cambio-turno")]
        public void ProgramarCambioTurno(int IdEmpleado, int IdTurnoActual, int IdTurnoNuevo, DateTime Fecha)
        {
            try
            {
                Facade.ProgramarCambioTurno(SessionID,  IdEmpleado,  IdTurnoActual,  IdTurnoNuevo,  Fecha);
            }
            catch (Exception ex)
            {
                HandleException("Error al programar el cambio de turno", ex);
            }            
        }


        [HttpPost]
        [Route("establecer-password")]
        public void EstablecerPassword(int IdEmpleado, string Password)
        {
            try
            {
                Facade.EstablecerPassword(SessionID, IdEmpleado, Password);
            }
            catch (Exception ex)
            {
                HandleException("Error al programar el cambio de turno", ex);
            }
        }

        [HttpGet]
        [Route("get-empleados-empresa")]
        public List<EmpleadoItem> GetEmpleadosEmpresa(int idSucursal)
        {
            try
            {
                return Facade.GetEmpleadosEmpresa(this.SessionID, idSucursal);
            }
            catch (BusinessException ex)
            {
                HandleException("Error al obtener listado de empleados", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("search-empleados-empresa")]
        public List<EmpleadoItem> SearchKeyword(int idBranch, string keyword)
        {
            List<EmpleadoItem> dtRet = null;
            try
            {
                EmpleadoFacade Facade = new EmpleadoFacade();
                dtRet = Facade.BuscarEmpleados(SessionID, idBranch, keyword);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener listado de empleados", ex);
            }
            return dtRet;
        }



    }
}