﻿using Azteca.Business.RH;
using Azteca.Kernel.Search;
using Azteca.Web.Controllers.Kernel;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Threading.Tasks;

namespace Azteca.Web.Controllers.RH
{

    [Route("api/rh/kadex-asistencia")]
    public class KardexAsistenciaController : BaseController
    {

        [HttpPost]
        [Route("search-incidencia-empleado")]
        public SearchResult Search(int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                    Result.TotalRows = Facade.GetTotalRecords(this.SessionID, idBranch, request.Conditions);
                    Result.Rows = Facade.Search(this.SessionID, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                    Result.TotalRows = Facade.GetTotalRecords(this.SessionID, idBranch, null);
                    Result.Rows = Facade.Search(this.SessionID, idBranch, null, null, 1, 20);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda.", ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-incidencia-empleado")]
        public bool UpdateIncidenciaEmpleado([FromBody] IncidenciaEmpleadoInfo info)
        {
            try
            {
                KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                Facade.UpdateIncidenciaEmpleado(this.SessionID, info);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al actualizar la incidencia del empleado", ex);
            }
            return false;
        }

        [HttpPost]
        [Route("delete-incidencia-empleado")]
        public bool DeleteIncidenciaEmpleado(int identity)
        {
            try
            {
                KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                Facade.DeleteIncidenciaEmpleado(this.SessionID, identity);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al eliminar la incidencia del empleado", ex);
            }
            return false;
        }

        [HttpGet]
        [Route("print-tarjeta-asistencias")]
        public byte[] PrintKardexAsistencia(int idEmpleado, int idDepartamento, DateTime fechaIni, DateTime fechaFin, bool activo, string pie)
        {
            try
            {
                KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                return Facade.PrintKardexAsistencia(this.SessionID, idEmpleado, idDepartamento, fechaIni, fechaFin, activo, pie);
            }
            catch (Exception ex)
            {
                HandleException("Error al imprimir el kardex de asistencia", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("print-transacciones")]
        public byte[] PrintTransacciones(int idEmpleado, int idDepartamento, DateTime fechaIni, DateTime fechaFin, bool activo, string pie)
        {
            try
            {
                KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                return Facade.PrintTransacciones(this.SessionID, idEmpleado, idDepartamento, fechaIni, fechaFin, activo, pie);
            }
            catch (Exception ex)
            {
                HandleException("Error al imprimir el kardex de asistencia", ex);
            }
            return null;
        }


        [HttpGet]
        [Route("get-kardex-asistencia")]
        public List<KardexAsistenciaInfo> GetKardexAsistencia(int idEmpleado, DateTime fechaIni, DateTime fechaFin)
        {
            try
            {
                KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                return Facade.GetKardexAsistencia(this.SessionID, idEmpleado, fechaIni, fechaFin);
            }
            catch (Exception ex)
            {
                HandleException("Error al consultar el kardex de asistencia del empleado", ex);
            }
            return null;
        }

        [HttpPost]
        [Route("update-kardex-asistencia")]
        public bool UpdateKardexAsistencia([FromBody] List<KardexAsistenciaInfo> incidencias)
        {
            try
            {
                KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                Facade.UpdateKardexAsistencia(SessionID, incidencias);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al actualizar el kardex de asistencia del empleado", ex);
            }
            return false;
        }

        [HttpGet]
        [Route("get-kardex-estatus")]
        public DataTable GetKardexEstatus()
        {
            try
            {
                KardexAsistenciaFacade Facade = new KardexAsistenciaFacade();
                return Facade.GetKardexEstatus(SessionID);
            }
            catch (Exception ex)
            {
                HandleException("Error al consultar los estatus del kardex", ex);
            }
            return null;
        }

        [HttpGet]
        [Route("get-incidencias-fecha")]
        public DataTable GetIncidenciasFecha(int idSucursal,DateTime fecha)
        {
            DataTable dt = new DataTable();
            try
            {
                KardexAsistenciaFacade facade = new KardexAsistenciaFacade();
                dt = facade.GetIncidenciasFecha(SessionID, idSucursal,fecha);
            }
            catch (Exception ex)
            {
                HandleException("Error al consultar los estatus del kardex", ex);
            }
            return dt;
        }

        [HttpGet]
        [Route("get-incidencias-by-empleado")]
        public DataTable GetIncidenciasByEmpleado(int idEmpleado, DateTime fechaIni, DateTime fechaFin)
        {
            DataTable dt = null;
            try
            {
                KardexAsistenciaFacade facade = new KardexAsistenciaFacade();
                dt = facade.GetIncidenciasByEmpleado(SessionID, idEmpleado, fechaIni, fechaFin);
            }
            catch (Exception ex)
            {
                HandleException("Error al consultar los estatus del kardex", ex);
            }
            return dt;
        }

        [HttpPost]
        [Route("update-incidencias-by-empleado")]
        public bool UpdateIncidenciasByEmpleado([FromBody] DataTable incidencias)
        {
            try
            {
                KardexAsistenciaFacade facade = new KardexAsistenciaFacade();
                facade.UpdateIncidenciasByEmpleado(SessionID, incidencias);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al guardar las incidencias", ex);
                return false;
            }
        }
    }
}
