﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.RH;
using Azteca.Kernel.Search;
using Azteca.Kernel.Queue;

namespace Azteca.Web.Controllers.RH
{
    [Route("api/rh/programaciones-recibos-adam")]
    public class ProgramacionesRecibosAdamController: DocumentController<ProgramacionReciboAdamInfo, ProgramacionReciboAdamDetail, ProgramacionReciboAdamService, ProgramacionReciboAdamFacade>
    {
        public ProgramacionesRecibosAdamController()
            :base(new ProgramacionReciboAdamFacade())
        {

        }

        [HttpPost]
        [Route("generar-programacion")]
        public bool GenerarProgramacion(int idSucursal, [FromBody] NominaAdamInfo nominaAdam)
        {
            try
            {
                Facade = new ProgramacionReciboAdamFacade();
                Facade.GenerarProgramacion(this.SessionID, idSucursal, nominaAdam);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpGet]
        [Route("generar-comparativo-recibo")]
        public byte[] GenerarComparativoPDF(string idProgramacion)
        {
            try
            {
                Facade = new ProgramacionReciboAdamFacade();
                return Facade.GenerarComparativoPDF(this.SessionID, idProgramacion);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("generar-descarga-comprobante")]
        public JsonResult GenerarDescargaComprobantes(string idProgramacion)
        {
            try
            {
                Facade = new ProgramacionReciboAdamFacade();
                return Json(Facade.GenerarDescargaComprobantes(this.SessionID, idProgramacion));
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("get-task-descarga-comprobante")]
        public Job GetJobDescargaComprobante(string idProgramacion)
        {
            try
            {
                Facade = new ProgramacionReciboAdamFacade();
                return Facade.GetProgramacionReciboAdmaJob(this.SessionID, idProgramacion);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("cancelar-progrmacion")]
        public bool CancelarProgramacion(string idProgramacion)
        {
            try
            {
                Facade = new ProgramacionReciboAdamFacade();
                Facade.CancelarProgramacion(this.SessionID, idProgramacion);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpGet]
        [Route("recibos-to-download")]
        public byte[] GetReciboToDownload(string idProgramacion)
        {
            try
            {
                //Facade = new ProgramacionReciboAdamFacade();
                //return Facade.GetRecibosToDownload(this.SessionID, idProgramacion);
                return null;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("recibos-to-download-path")]
        public string GetFileToDownload(string idProgramacion)
        {
            try
            {
                Facade = new ProgramacionReciboAdamFacade();
                return Facade.GetFileToDownload(this.SessionID, idProgramacion);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

    }
}
