﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.RH;
using Azteca.Kernel.Search;
using System.Data;
using Azteca.Business.RH.Catalogos.Facades;

namespace Azteca.Web.Controllers.RH
{
    [Route("api/rh/recibos-nominas-adam")]
    public class RecibosNominasAdamController: DocumentController<ReciboNominaAdamInfo, ReciboNominaAdamDetail, ReciboNominaAdamService, ReciboNominaAdamFacade>
    {
        public RecibosNominasAdamController()
            :base(new ReciboNominaAdamFacade())
        {

        }

        [HttpPost]
        [Route("search-recibos")]
        public SearchResult SearchRecibos( int idBranch, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    int pageIndex = 0;
                    if (request.Paged) pageIndex = request.PageIndex + 1;//Search manager inicia la paginación desde 1, los view desde 0
                                                                         //
                    Facade = new ReciboNominaAdamFacade();
                    Result.TotalRows = Facade.GetTotalRecords(this.SessionID, idBranch, request.Conditions);
                    Result.Rows = Facade.Search(this.SessionID, idBranch, request.Conditions, request.Order, pageIndex, request.PageSize);
                }
                else
                {
                    Facade = new ReciboNominaAdamFacade();
                    Result.TotalRows = Facade.GetTotalRecords(this.SessionID, idBranch, null);
                    Result.Rows = Facade.Search(this.SessionID, idBranch, null, null);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException("Error al ejecutar la búsqueda.", ex);
                return null;
            }
        }

        [HttpPost]
        [Route("generar-recibo")]
        public bool GenerarRecibo(int idSucursal, [FromBody] NominaAdamInfo nominaAdam)
        {
            try
            {
                ReciboNominaAdamFacade ReciboFac = new ReciboNominaAdamFacade();
                ReciboFac.GenerarRecibo(this.SessionID, idSucursal, nominaAdam);
                return true;
            }
            catch (Exception ex)
            {
                HandleException("Error al generar el recibo.", ex);
                return false;
            }
        }

        [HttpGet("create-nomin-adam")]
        public NominaAdamInfo CreateNominaAdam()
        {
            try
            {
                ReciboNominaAdamFacade Fac = new ReciboNominaAdamFacade();
                return Fac.CreateNominaAdam(this.SessionID);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }

        [HttpPost("timbrar-recibo")]
        public bool TimbrarRecibo(string idReciboNominaAdam)
        {
            try
            {
                ReciboNominaAdamFacade Fac = new ReciboNominaAdamFacade();
                Fac.TimbrarRecibo(this.SessionID, idReciboNominaAdam);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }

        [HttpGet("enviar-recibo")]
        public bool EnviarRecibo(string idReciboNominaAdam, int idBranch)
        {
            try
            {
                ReciboNominaAdamFacade Fac = new ReciboNominaAdamFacade();
                Fac.EnviarRecibo(this.SessionID, idReciboNominaAdam, idBranch);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }

        [HttpPost("cancelar-recibo")]
        public bool CancelarRecibo(string idReciboNominaAdam)
        {
            try
            {
                ReciboNominaAdamFacade Fac = new ReciboNominaAdamFacade();
                Fac.CancelarRecibo(this.SessionID, idReciboNominaAdam);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }

        [HttpGet("get-comprobante-empleado")]
        public DataTable LoadComprobanteEmpleado(string numEmpleado, DateTime fecha)
        {
            try
            {
                ReciboNominaAdamFacade Fac = new ReciboNominaAdamFacade();
                return Fac.LoadComprobanteEmpleado(this.SessionID, numEmpleado, fecha);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }


        [HttpGet("consultar-asistencias")]
        public DataTable consultarAsistencias(string codigo, DateTime fechaInicial , DateTime fechaFinal)
        {
            try
            {
                ConsultaAsistenciaEmpleadoFacade Fa = new ConsultaAsistenciaEmpleadoFacade();
                //ReciboNominaAdamFacade Fac = new ReciboNominaAdamFacade();
                return Fa.LoadList(this.SessionID, codigo, fechaInicial,fechaFinal);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return null;
        }
    }
}
