﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Inventario;
using Azteca.Business.Ventas;
using System.Data;


namespace Azteca.Web.Controllers.ventas
{
    [Route("api/ventas/articulos")]
    public class ArticulosController : CatalogController<ArticuloFacade, ArticuloService, ArticuloInfo>
    {
        public ArticulosController()
            : base(new ArticuloFacade())
        {

        }

        [HttpGet]
        [Route("create")]
        public override ArticuloInfo Create()
        {
            ArticuloInfo ArticuloVentas = base.Create();
            //Cargar clasificaciones del articulo stock
            ArticuloStockFacade ArticuloFac = new ArticuloStockFacade();
            ArticuloVentas.ArticuloStock.Clasificaciones = ArticuloFac.LoadClassifications(SessionID, 0);
            return ArticuloVentas;
        }

        [HttpGet]
        [Route("get-articulo-stock")]
        public StockItem GetArticuloStock(int idSucursal, string codigo)
        {
            try
            {
                if (codigo != null && codigo.Trim() != string.Empty)
                {
                    ArticuloStockFacade articuloStockFacade = new ArticuloStockFacade();
                    StockItem articuloStockItem = articuloStockFacade.GetArticuloStock(SessionID, idSucursal, codigo);
                    return articuloStockItem;
                }
                else
                {
                    return new StockItem();
                }
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("articulo-asignado-check")]
        public Boolean ArticuloAsignadoCheck(int idArticuloStock)
        {
            Boolean asigCheck = new Boolean();

            try
            {
                asigCheck = Facade.ArticuloAsignadoCheck(SessionID, idArticuloStock);
            }
            catch (Exception ex)
            {
                HandleException("Error en al revisar si el articulo stock ya esta asignado a un articulo de venta.", ex);
            }
            return asigCheck;
        }

        [HttpGet]
        [Route("get-articulo-by-articulo-stock")]
        public Int32 GetArticuloByArticuloStock(int idArticuloStock)
        {
            try
            {
                Int32 IdArticulo = Facade.GetArticuloByArticuloStock(SessionID, idArticuloStock);
                return IdArticulo;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return 0;
            }
        }

        //Lista de Precios

        [HttpGet]
        [Route("load-articulo-lista-precios")]
        public DataTable LoadArticuloListaPrecios(int IdEmpresa, int IdArticulo)
        {
            try
            {
                DataTable articuloListaPrecios = Facade.LoadArticuloListaPrecios(SessionID, IdEmpresa, IdArticulo);
                return articuloListaPrecios;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-articulo-lista-precios")]
        public Boolean UpdateArticuloListaPrecios([FromBody] ArticuloListaPrecios[] info, int IdArticulo)
        {
            Boolean flag = new Boolean();
            try
            {
                foreach (ArticuloListaPrecios p in info)
                {
                    if (p.State == "CHANGED")
                    {
                        flag = Facade.UpdateArticuloListaPrecios(SessionID, p.ID, IdArticulo, p.Precio, p.IdDivisa);
                    }
                }
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return flag;
        }

        //Grupos de Descuentos

        [HttpGet]
        [Route("load-articulo-grupo-descuento")]
        public DataTable LoadArticuloGrupoDescuento(int IdEmpresa, int IdArticulo)
        {
            try
            {
                DataTable articuloGrupoDescuento = Facade.LoadArticuloGrupoDescuento(SessionID, IdEmpresa, IdArticulo);
                return articuloGrupoDescuento;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-articulo-grupo-descuento")]
        public Boolean UpdateArticuloGrupoDescuento([FromBody] ArticuloGrupoDescuentos[] info, int IdArticulo)
        {
            Boolean flag = new Boolean();
            try
            {
                foreach (ArticuloGrupoDescuentos p in info)
                {
                    if (p.State == "CHANGED")
                    {
                        flag = Facade.UpdateArticuloGrupoDescuento(SessionID, p.ID, IdArticulo, p.PorcentajeDescuento1, p.PorcentajeDescuento2, p.PorcentajeDescuento3, p.PorcentajeDescuento4,
                                p.PorcentajeDescuento5);
                    }
                }
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return flag;
        }

        [HttpGet]
        [Route("precio")]
        public PrecioInfo CalcularPrecio(int idArticulo, int idListaPrecios, int idGrupoDescuentos, int idEmpresa, int idZonaVenta)
        {
            try
            {
                PrecioInfo Precio = Facade.CalcularPrecio(SessionID, idArticulo, idListaPrecios, idGrupoDescuentos, idEmpresa, idZonaVenta);
                return Precio;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("precio-cliente")]
        public PrecioInfo CalcularPrecio(int idArticulo, int idCliente, int idEmpresa, int idZonaVenta)
        {
            try
            {
                PrecioInfo Precio = Facade.CalcularPrecio(SessionID, idArticulo, idCliente, idEmpresa, idZonaVenta);
                return Precio;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


    }
}