﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Ventas;
using System.Data;
using Azteca.Kernel.General;
using Azteca.Business.Credito;
using Azteca.Business.Contabilidad;

namespace Azteca.Web.Controllers.Ventas
{
    [Route("api/ventas/clientes")]
    public class ClientesController : CatalogController<ClienteFacade, ClienteService, ClienteInfo>
    {
        public ClientesController()
            :base(new ClienteFacade())
        {

        }

        [HttpGet]
        [Route("load-contactos")]
        public List<PersonaInfo> LoadContactos(int idCliente)
        {
            try
            {
                List<PersonaInfo> Contacts = Azteca.Kernel.Common.DataTableUtils.ToList<PersonaInfo>(Facade.LoadListContacto(SessionID, idCliente));
                return Contacts;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


        [HttpPost]
        [Route("add-contacto")]
        public void AddMembers(int idCliente, [FromBody] PersonaInfo contacto)
        {
            try
            {                
                {
                    Facade.InsertarContacto(SessionID, idCliente, contacto);
                }
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-contacto")]
        public void RemoveContacto(int idCliente, int idPersona)
        {
            try
            {
                Facade.RemoverContacto(SessionID, idCliente, idPersona);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("enviar-registro")]
        public void EnviarMailRegistro(int idCliente)
        {
            try
            {
                Facade.EnviarMailRegistro(SessionID, idCliente);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("registrar")]
        public void Registrar(int idCliente, string password)
        {
            try
            {
                Facade.Registrar(SessionID, idCliente, password);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }

        }




        [HttpGet]
        [Route("detail-by-user")]
        public ClienteInfo GetDetailByUser(int idUsuario, int idEmpresa)
        {
            try
            {
                ClienteInfo cliente = Facade.GetDetailByUser(SessionID, idUsuario, idEmpresa);
                return cliente;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


        [HttpGet]
        [Route("empresas")]
        public DataTable LoadEmpresas(int idUsuario)
        {
            try
            {
                DataTable dtEmpresas = Facade.LoadEmpresas(SessionID, idUsuario);
                return dtEmpresas;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("cliente-clear-cache")]
        public void ClienteClearCache(int idCliente, int idDeudor, int idContribuyente)
        {
            try
            {
                Facade.ClienteClearCache(SessionID, idCliente);

                DeudorFacade deudorFacade = new DeudorFacade();
                deudorFacade.DeudorClearCache(SessionID, idDeudor);

                ContribuyenteFacade contribuyenteFacade = new ContribuyenteFacade();
                contribuyenteFacade.ContribuyenteClearCache(SessionID, idContribuyente);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        /////////////////////// Pre Registro ///////////////////////

        [HttpGet]
        [Route("registrar-de-preregistro")]
        public void Registrar(int idCliente, string password, int idPreRegistro, string correo)
        {
            try
            {
                Facade.Registrar(SessionID, idCliente, password, idPreRegistro, correo);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }

        }


        [HttpPost]
        [Route("pre-registrar")]
        public void PreRegistrar([FromBody] ClientePreRegistro preRegistro)
        {
            try
            {
                Facade.PreRegistrar(SessionID, preRegistro);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpPost]
        [Route("pre-registro-delete")]
        public void PreRegistroEliminar(int idPreRegistro)
        {
            try
            {
                Facade.PreRegistroEliminar(SessionID, idPreRegistro);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }


        [HttpGet]
        [Route("pre-registros")]
        public List<ClientePreRegistro> PreRegistros()
        {
            try
            {
                List<ClientePreRegistro> pRegistros = (Facade.PreRegistros(SessionID));
                return pRegistros;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


    }
}