﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Ventas;
using System.Data;
using Azteca.Kernel.Search;

namespace Azteca.Web.Controllers.Ventas
{
    [Route("api/ventas/gruposdescuentos")]
    public class GruposDescuentosController : CatalogController<GrupoDescuentoFacade, GrupoDescuentoService, GrupoDescuentoInfo>
    {
        public GruposDescuentosController()
            :base(new GrupoDescuentoFacade())
        {

        }

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadList(int idSucursal)
        {
            DataTable dtRet = new DataTable();

            try
            {                
                dtRet = Facade.LoadList(SessionID, idSucursal);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar los grupos de descuentos.", ex);
            }
            return dtRet;
        }

        [HttpPost]
        [Route("articulos")]
        public SearchResult LoadArticulos(int idGrupoDescuentos, int idSucursal, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    Result.Rows = Facade.LoadArticulos(SessionID, idGrupoDescuentos, idSucursal, request);
                    Result.TotalRows = Facade.GetTotalArticulos(SessionID, idGrupoDescuentos, idSucursal, request.Conditions);
                }
                else
                {
                    Result.Rows = Facade.LoadArticulos(SessionID, idGrupoDescuentos, idSucursal, new SearchRequest());
                    Result.TotalRows = Facade.GetTotalArticulos(SessionID, idGrupoDescuentos, idSucursal, null);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-articulo")]
        public bool UpdateArticulo(int idGrupoDescuentos, [FromBody] ArticuloGrupoDescuentos articulo)
        {
            try
            {
                Facade.UpdateArticulo(SessionID, idGrupoDescuentos, articulo);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("servicios")]
        public SearchResult LoadServicios(int idGrupoDescuentos, int idSucursal, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    Result.Rows = Facade.LoadServicios(SessionID, idGrupoDescuentos, idSucursal, request);
                    Result.TotalRows = Facade.GetTotalServicios(SessionID, idGrupoDescuentos, idSucursal, request.Conditions);
                }
                else
                {
                    Result.Rows = Facade.LoadServicios(SessionID, idGrupoDescuentos, idSucursal, new SearchRequest());
                    Result.TotalRows = Facade.GetTotalServicios(SessionID, idGrupoDescuentos, idSucursal, null);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-servicio")]
        public bool UpdateServicio(int idGrupoDescuentos, [FromBody] ServicioGrupoDescuentos servicio)
        {
            try
            {
                Facade.UpdateServicio(SessionID, idGrupoDescuentos, servicio);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("clientes")]
        public SearchResult GetClientes(int idGrupoDescuentos, int idEmpresa, int idUsuario, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    Result.Rows = Facade.LoadClientes(SessionID, idGrupoDescuentos, idEmpresa, idUsuario, request);
                    Result.TotalRows = Facade.GetTotalClientes(SessionID, idGrupoDescuentos, idEmpresa, idUsuario, request.Conditions);
                }
                else
                {
                    Result.Rows = Facade.LoadClientes(SessionID, idGrupoDescuentos, idEmpresa, idUsuario, null);
                    Result.TotalRows = Facade.GetTotalClientes(SessionID, idGrupoDescuentos, idEmpresa, idUsuario, null);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-clientes")]
        public void AddClientes(int idGrupoDescuentos, [FromBody] List<ClienteItem> clientes)
        {
            try
            {
                clientes.ForEach(cliente =>
                {
                    Facade.AddCliente(SessionID, idGrupoDescuentos, cliente.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }


    }
}