﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Ventas;
using Azteca.Kernel.Search;
using System.Data;


namespace Azteca.Web.Controllers.Ventas
{
    [Route("api/ventas/listas-precios")]
    public class ListasPreciosController : CatalogController<ListaPreciosFacade, ListaPreciosService, ListaPreciosInfo>
    {
        public ListasPreciosController()
            :base(new ListaPreciosFacade())
        {

        }

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadList(int idSucursal)
        {
            DataTable dtRet = new DataTable();

            try
            {
                ListaPreciosFacade Facade = new ListaPreciosFacade();
                dtRet = Facade.LoadList(SessionID, idSucursal);
            }
            catch (Exception ex)
            {
                HandleException("Error al cargar las Listas de Precios.", ex);
            }
            return dtRet;
        }



        [HttpPost]
        [Route("clientes")]
        public SearchResult GetClientes(int idListaPrecios, int idEmpresa, int idUsuario, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    Result.Rows = Facade.LoadClientes(SessionID, idListaPrecios, idEmpresa, idUsuario, request);
                    Result.TotalRows = Facade.GetTotalClientes(SessionID, idListaPrecios, idEmpresa, idUsuario, request.Conditions);
                }
                else
                {
                    Result.Rows = Facade.LoadClientes(SessionID, idListaPrecios, idEmpresa, idUsuario, null);
                    Result.TotalRows = Facade.GetTotalClientes(SessionID, idListaPrecios, idEmpresa, idUsuario, null);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-clientes")]
        public void AddClientes(int idListaPrecios, [FromBody] List<ClienteItem> clientes)
        {
            try
            {
                clientes.ForEach(cliente =>
                {
                    Facade.AddCliente(SessionID, idListaPrecios, cliente.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpPost]
        [Route("articulos")]
        public SearchResult LoadArticulos(int idListaPrecios, int idSucursal, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    Result.Rows = Facade.LoadArticulos(SessionID, idListaPrecios, idSucursal, request);
                    Result.TotalRows = Facade.GetTotalArticulos(SessionID, idListaPrecios, idSucursal, request.Conditions);
                }
                else
                {
                    Result.Rows = Facade.LoadArticulos(SessionID, idListaPrecios, idSucursal, new SearchRequest());
                    Result.TotalRows = Facade.GetTotalArticulos(SessionID, idListaPrecios, idSucursal, null);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-articulo")]
        public bool UpdateArticulo(int idListaPrecios, [FromBody] ArticuloListaPrecios articulo)
        {
            try
            {
                Facade.UpdateArticulo(SessionID, idListaPrecios, articulo);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }

        [HttpPost]
        [Route("servicios")]
        public SearchResult LoadServicios(int idListaPrecios, int idSucursal, [FromBody] SearchRequest request)
        {
            try
            {
                SearchResult Result = new SearchResult();
                if (request != null)
                {
                    Result.Rows = Facade.LoadServicios(SessionID, idListaPrecios, idSucursal, request);
                    Result.TotalRows = Facade.GetTotalServicios(SessionID, idListaPrecios, idSucursal, request.Conditions);
                }
                else
                {
                    Result.Rows = Facade.LoadServicios(SessionID, idListaPrecios, idSucursal, new SearchRequest());
                    Result.TotalRows = Facade.GetTotalServicios(SessionID, idListaPrecios, idSucursal, null);
                }
                return Result;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-servicio")]
        public bool UpdateServicio(int idListaPrecios, [FromBody] ServicioListaPrecios servicio)
        {
            try
            {
                Facade.UpdateServicio(SessionID, idListaPrecios, servicio);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return false;
            }
        }



    }

}