﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Ventas;
using System.Data;


namespace Azteca.Web.Controllers.Ventas
{
    [Route("api/ventas/servicios")]
    public class ServiciosController : CatalogController<ServicioFacade, ServicioService, ServicioInfo>
    {
        public ServiciosController()
            :base(new ServicioFacade())
        {

        }

        //Lista de Precios
        [HttpGet]
        [Route("load-servicio-lista-precios")]
        public DataTable LoadServicioListaPrecios(int IdEmpresa, int IdServicio)
        {
            try
            {
                DataTable servicioListPrecios = Facade.LoadServicioListaPrecios(SessionID, IdEmpresa, IdServicio);
                return servicioListPrecios;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-servicio-lista-precios")]
        public Boolean UpdateServicioListaPrecios([FromBody] ServicioListaPrecios[] info, int IdServicio)
        {
            Boolean flag = new Boolean();
            try
            {
                foreach (ServicioListaPrecios p in info)
                {
                    if (p.State == "CHANGED")
                    {
                        flag = Facade.UpdateServicioListaPrecios(SessionID, p.ID, IdServicio, p.Precio, p.IdDivisa);
                    }
                }
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return flag;
        }

        //Grupo de Descuentos
        [HttpGet]
        [Route("load-servicio-grupo-descuento")]
        public DataTable LoadServicioGrupoDescuento(int IdEmpresa, int IdServicio)
        {
            try
            {
                DataTable servicioGrupoDescuento = Facade.LoadServicioGrupoDescuento(SessionID, IdEmpresa, IdServicio);
                return servicioGrupoDescuento;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("update-servicio-grupo-descuento")]
        public Boolean UpdateServicioGrupoDescuento([FromBody] ServicioGrupoDescuentos[] info, int IdServicio)
        {
            Boolean flag = new Boolean();
            try
            {
                foreach (ServicioGrupoDescuentos p in info)
                {
                    if (p.State == "CHANGED")
                    {
                        flag = Facade.UpdateServicioGrupoDescuento(SessionID, p.ID, IdServicio, p.PorcentajeDescuento1, p.PorcentajeDescuento2, p.PorcentajeDescuento3, p.PorcentajeDescuento4, p.PorcentajeDescuento5);
                    }
                }
            }
            catch (Exception ex)
            {
                HandleException("Mensaje de error", ex);
            }
            return flag;
        }

        [HttpGet]
        [Route("precio")]
        public PrecioInfo CalcularPrecio(int idServicio, int idListaPrecios, int idGrupoDescuentos, int idEmpresa, int idZonaVenta, decimal precioBruto)
        {
            try
            {
                PrecioInfo Precio = Facade.CalcularPrecio(SessionID, idServicio, idListaPrecios, idGrupoDescuentos, idEmpresa, idZonaVenta, precioBruto);
                return Precio;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("precio-cliente")]
        public PrecioInfo CalcularPrecio(int idServicio, int idCliente, int idEmpresa, int idZonaVenta, decimal precioBruto)
        {
            try
            {
                PrecioInfo Precio = Facade.CalcularPrecio(SessionID, idServicio, idCliente, idEmpresa, idZonaVenta, precioBruto);
                return Precio;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


    }
}