﻿using System;
using System.Collections.Generic;
using Microsoft.AspNetCore.Mvc;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Ventas;
using System.Data;
using Azteca.Kernel.Security;

namespace Azteca.Web.Controllers.Ventas
{
    [Route("api/ventas/unidades-negocio")]
    public class UnidadesNegocioController : CatalogController< UnidadNegocioFacade, UnidadNegocioService, UnidadNegocioInfo>
    {
        public UnidadesNegocioController()
            :base(new UnidadNegocioFacade())
        {

        }

        [HttpGet]
        [Route("user-access")]
        public List<UnidadNegocioItem> LoadList(int idSucursal, int idUsuario)
        {
            try
            {
                List<UnidadNegocioItem> Unidades = Facade.LoadList(SessionID, idSucursal, idUsuario);
                return Unidades;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("members")]
        public List<UsuarioItem> GetMembers(int idUnidadNegocio)
        {
            try
            {
                List<UsuarioItem> Users = Azteca.Kernel.Common.DataTableUtils.ToList<UsuarioItem>(Facade.LoadAccesos(SessionID, idUnidadNegocio));
                return Users;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("add-members")]
        public void AddMembers(int idUnidadNegocio, [FromBody] List<UsuarioItem> users)
        {
            try
            {
                users.ForEach(user =>
                {
                    Facade.SaveAcceso(SessionID, idUnidadNegocio, user.ID);
                });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpDelete]
        [Route("remove-member")]
        public void DeleteMember(int idUnidadNegocio, int idUser)
        {
            try
            {
                Facade.RemoveAcceso(SessionID, idUnidadNegocio, idUser);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        [HttpGet]
        [Route("load-list")]
        public DataTable LoadList(int idSucursal)
        {
            try
            {
                DataTable Unidades = Facade.LoadList(SessionID, idSucursal);
                return Unidades;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

    }
}