﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Data;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Ventas;
using Azteca.Business.Credito;

namespace Azteca.Web.Controllers.Ventas
{
    [Route("api/ventas/ventas")]
    public class VentasController : DocumentController<VentaInfo, VentaDetail, VentaService, VentaFacade>
    {
        public VentasController()
            : base(new VentaFacade())
        {

        }

        [HttpPost]
        [Route("save")]
        public VentaInfo Save([FromBody] JObject ventaJson)
        {
            try
            {
                VentaInfo Venta = Facade.Deserialize(SessionID, ventaJson);
                VentaInfo UpdatedVenta = Facade.Update(SessionID, Venta);
                return UpdatedVenta;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("search-ventas")]
        public List<VentaItem> SearchKeyword(int idBranch, string keyword)
        {
            List<VentaItem> dtRet = null;
            try
            {
                VentaFacade Facade = new VentaFacade();
                dtRet = Facade.BuscarVentas(SessionID, idBranch, keyword);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener ventas", ex);
            }
            return dtRet;
        }

        [HttpGet]
        [Route("pdf-factura")]
        public FileLink GenerarPdfFactura(string idVenta)
        {
            try
            {
                FileLink FileUri = Facade.GenerarPdfFactura(SessionID, idVenta);
                return FileUri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("xml-factura")]
        public FileLink GetXmlFactura(string idVenta)
        {
            try
            {
                FileLink FileUri = Facade.GetXmlFactura(SessionID, idVenta);
                return FileUri;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("estadistica-periodo")]
        public DataTable LoadEstadisticaPeriodo(int idSucursal, DateTimeOffset fechaInicial, DateTimeOffset fechaFinal)
        {
            try
            {
                DataTable dt = Facade.LoadEstadisticaPeriodo(SessionID, idSucursal, fechaInicial, fechaFinal);
                return dt;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpGet]
        [Route("formato-venta")]
        public Byte[] GenerarFormatoVenta(string idVenta)
        {
            try
            {
                Byte[] Formato = Facade.GetFormatoVenta(SessionID, idVenta);
                return Formato;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("regenerar-factura")]
        public void RegeneraFactura(string IdVenta)
        {
           
            try
            {
                VentaFacade VentaFac = new VentaFacade();
                VentaInfo Venta = VentaFac.GetDetail(SessionID, IdVenta);

                CuentaPorCobrarFacade Facade = new CuentaPorCobrarFacade();
                Facade.RegenerarFactura(SessionID, Venta.CuentaPorCobrar.Identity);
            }
            catch (Exception ex)
            {
                HandleException("Error al regenerar la factura.", ex);
            }
            //return Factura;
        }

    }
}
