﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Tesoreria;
using Azteca.Kernel.Queue;


namespace Azteca.Web.Controllers.Tesoreria
{
    [Route("api/tesoreria/estados-cuenta-bancarios")]
    public class EstadosCuentaBancariosController: DocumentController<EstadoCuentaBancarioInfo, EstadoCuentaBancarioDetail, EstadoCuentaBancarioService, EstadoCuentaBancarioFacade>
    {
        public EstadosCuentaBancariosController()
            :base(new EstadoCuentaBancarioFacade())
        {

        }

        [HttpPost]
        [Route("encolar-transpiler")]
        public int EncolarTranspiler([FromBody] EstadoCuentaBancarioFormato info)
        {
            int idLote = 0;
            try
            {
                EstadoCuentaBancarioFacade estadoCuentaBancarioFacade = new EstadoCuentaBancarioFacade();
                idLote = estadoCuentaBancarioFacade.EncolarTranspiler(this.SessionID, info);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return idLote;
        }
        
        [HttpPost]
        [Route("transpiler-status")]
        public Azteca.Kernel.Queue.Job TranspilerStatus(int idLote)
        {
            try
            {
                return Facade.jobTranspilerStatus(SessionID, idLote);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("conciliar-ingresos-set")]
        public bool ConciliarIngresosSet([FromBody] List<EstadoCuentaBancarioConciliarItem> conciliado)
        {
            try
            {
                EstadoCuentaBancarioFacade estadoCuentaBancarioFacade = new EstadoCuentaBancarioFacade();
                estadoCuentaBancarioFacade.ConciliarDetalle(this.SessionID, conciliado);
                return true;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return false;
        }
        

        [HttpGet]
        [Route("conciliar-ingresos")]
        public List<EstadoCuentaBancarioConciliarItem> ConciliarIngresos(string idEstadoCuenta, string origen, string cuenta)
        {
            List<EstadoCuentaBancarioConciliarItem> lstRet = new List<EstadoCuentaBancarioConciliarItem>();
            try
            {
                lstRet = Facade.ConciliarIngresos(SessionID, idEstadoCuenta, origen, cuenta);
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
            return lstRet;
        }

        [HttpGet]
        [Route("get-async-conciliar-ingresos")]
        public AsyncTaskInfo AsyncGetConciliarIngresos(string idEstadoCuenta, string origen, string cuenta)
        {
            try
            {
                EstadoCuentaBancarioFacade Facade = new EstadoCuentaBancarioFacade();
                return Facade.GetAsyncConciliarIngresos(SessionID, idEstadoCuenta, origen, cuenta);
            }
            catch (Exception ex)
            {
                HandleException("Error al obtener datos a conciliar", ex);
            }
            return null;
        }

        [HttpPost]
        [Route("to-pdf")]
        public Byte[] ToPdf(string idEstadoCuenta,  decimal saldoEmpresa, decimal saldoEstadoCuentaBancario)
        {
            try
            {
                Byte[] Formato = Facade.ToPdf(SessionID, idEstadoCuenta, saldoEmpresa, saldoEstadoCuentaBancario);
                return Formato;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }


    }
}
