﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Azteca.Kernel.Business;
using Azteca.Web.Controllers.Kernel;
using Azteca.Business.Tesoreria;
using Azteca.Kernel.Printing;

namespace Azteca.Web.Controllers.Tesoreria
{
    [Route("api/tesoreria/pagos")]
    public class PagosController: DocumentController<PagoInfo, PagoDetail, PagoService, PagoFacade>
    {
        public PagosController()
            :base(new PagoFacade())
        {

        }

        [HttpGet]
        [Route("load-cxp-by-beneficiario")]
        public List<PagoDetail> LoadCxPbyIdBeneficiario(DateTime fechaIni, DateTime fechaFin, int idBeneficiario, string fechaFiltro, int idDivisa, int idSucursal)
        {
            try
            {
                List<PagoDetail> CXC = Azteca.Kernel.Common.DataTableUtils.ToList<PagoDetail>(Facade.LoadCxPbyIdBeneficiario(SessionID, fechaIni, fechaFin, idBeneficiario, fechaFiltro, idDivisa ,idSucursal)).OrderBy(order => (order.FechaCuentaPorPagar)).ToList();
                return CXC;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }
        
        [HttpPost]
        [Route("to-cheques")]
        public List<string> toCheques([FromBody]List<string> idPagos)
        {
            List<string> chequesRet = new List<string>();
            try
            {
                chequesRet = Facade.toCheques(SessionID, idPagos);
                return chequesRet;
            }
            catch(Exception ex)
            {
                HandleException(ex);
                return null;
            }
        }

        [HttpPost]
        [Route("lote-cheques")]
        public LoteImpresionInfo LoteChequesGenerar([FromBody] List<string> idPagos)
        {
            try
            {
                LoteImpresionInfo lote = Facade.LoteChequesGenerar(SessionID, idPagos);
                return lote;
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return null;
            }

        }


    }
}
