﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Globalization;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.SpaServices.Webpack;
using Microsoft.AspNetCore.Rewrite;
using Azteca.Web.Middleware;


namespace Onili2Web
{
    public class Startup
    {
        private void ConfigureFramework()
        {
            //Default
            Azteca.Data.DataSource dataSource = new Azteca.Data.DataSource("Onili");
            dataSource.Provider = Azteca.Data.DataProvider.Sql;
            dataSource.ConnectionString = Configuration["Data:DefaultConnection:ConnectionString"];
            dataSource.PageSize = 20;
            //Contapq Esp
            Azteca.Data.DataSource dataSourceContpaq = new Azteca.Data.DataSource("ContpaqEsp");
            dataSourceContpaq.Provider = Azteca.Data.DataProvider.Sql;
            dataSourceContpaq.ConnectionString = Configuration["Data:ContpaqEsp:ConnectionString"];
            dataSourceContpaq.PageSize = 20;
            //Pixcua Aga
            Azteca.Data.DataSource dataSourcePixcuaAga = new Azteca.Data.DataSource("PixcuaAga");
            dataSourcePixcuaAga.Provider = Azteca.Data.DataProvider.Sql;
            dataSourcePixcuaAga.ConnectionString = Configuration["Data:PixcuaAga:ConnectionString"];
            dataSourcePixcuaAga.PageSize = 20;
            //Pixcua Refresquera
            Azteca.Data.DataSource dataSourcePixcuaRef = new Azteca.Data.DataSource("PixcuaRefresquera");
            dataSourcePixcuaRef.Provider = Azteca.Data.DataProvider.Sql;
            dataSourcePixcuaRef.ConnectionString = Configuration["Data:PixcuaRefresquera:ConnectionString"];
            dataSourcePixcuaRef.PageSize = 20;
            //Pixcua Tijuana
            Azteca.Data.DataSource dataSourcePixcuaTij = new Azteca.Data.DataSource("PixcuaTijuana");
            dataSourcePixcuaTij.Provider = Azteca.Data.DataProvider.Sql;
            dataSourcePixcuaTij.ConnectionString = Configuration["Data:PixcuaTijuana:ConnectionString"];
            dataSourcePixcuaTij.PageSize = 20;
            //
            Azteca.Data.DataServices.AddDataSource(dataSource);
            Azteca.Data.DataServices.AddDataSource(dataSourceContpaq);
            Azteca.Data.DataServices.AddDataSource(dataSourcePixcuaAga);
            Azteca.Data.DataServices.AddDataSource(dataSourcePixcuaRef);
            Azteca.Data.DataServices.AddDataSource(dataSourcePixcuaTij);
            //
            Azteca.Kernel.ServerConfiguration serverConfiguration = new Azteca.Kernel.ServerConfiguration();
            serverConfiguration.ServerName = "Onili";
            serverConfiguration.HostingUrl = Configuration["AztecaFramework:WebServer:HostUrl"];
            if (serverConfiguration.HostingUrl == string.Empty) serverConfiguration.HostingUrl = "http://localhost:8080";
            serverConfiguration.FileServerPath = Configuration["AztecaFramework:Paths:FileServerLocation"];
            if (serverConfiguration.FileServerPath == string.Empty) serverConfiguration.FileServerPath = "wwwroot/file-server";
            
           // serverConfiguration.FileServerPath = @"\\192.168.250.103\H$\FileServer";
            serverConfiguration.FileServerPath = @"\\192.168.254.51\FileServer";


            serverConfiguration.ApplicationDataFolder = Configuration["AztecaFramework:Paths:AppDataFolder"];
            if (serverConfiguration.ApplicationDataFolder == string.Empty) serverConfiguration.ApplicationDataFolder = serverConfiguration.FileServerPath + "/app-data";
            serverConfiguration.AllowMultipleSessions = bool.Parse(Configuration["AztecaFramework:Session:AllowMultiple"]);
            serverConfiguration.SessionTimeout = int.Parse(Configuration["AztecaFramework:Session:Timeout"]);
            serverConfiguration.SessionStorage = Configuration["AztecaFramework:Session:Storage"];
            serverConfiguration.LogExceptions = bool.Parse(Configuration["AztecaFramework:Logging:LogExceptions"]);
            serverConfiguration.TraceEnabled = bool.Parse(Configuration["AztecaFramework:Logging:TraceEnabled"]);
            //
            Azteca.Kernel.Server.Initialize(serverConfiguration);
        }

        public Startup(IHostingEnvironment env)
        {
            var builder = new ConfigurationBuilder()
                .SetBasePath(env.ContentRootPath)
                .AddJsonFile("appsettings.json", optional: true, reloadOnChange: true)
                .AddJsonFile($"appsettings.{env.EnvironmentName}.json", optional: true)
                .AddEnvironmentVariables();
            Configuration = builder.Build();
            //
            ConfigureFramework();
        }

        public IConfigurationRoot Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            // Add framework services.
            services.AddMvc()
                .AddJsonOptions(jsonOptions =>
                {
                    jsonOptions.SerializerSettings.NullValueHandling = Newtonsoft.Json.NullValueHandling.Ignore;
                })
                .AddApplicationPart(typeof(Azteca.Web.Controllers.Kernel.BaseController).Assembly);

        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env, ILoggerFactory loggerFactory)
        {
            var cultureInfo = new CultureInfo("es-MX");
            CultureInfo.DefaultThreadCurrentCulture = cultureInfo;
            CultureInfo.DefaultThreadCurrentUICulture = cultureInfo;

            //loggerFactory.AddConsole(Configuration.GetSection("Logging"));
            //loggerFactory.AddDebug();

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
                app.UseWebpackDevMiddleware(new WebpackDevMiddlewareOptions
                {
                    HotModuleReplacement = true,
                    EnvironmentVariables = new Dictionary<string, string> {
                        { "NODE_OPTIONS", "--max-old-space-size=8192 " }
                    }
                });
            }


            app.UseRewriter(new RewriteOptions()
                .AddIISUrlRewrite(env.ContentRootFileProvider, "rewrite-rules.xml"));


            app.UseDefaultFiles();

            app.UseStaticFiles(new StaticFileOptions()
            {
                OnPrepareResponse = (context) =>
                {
                    // Disable caching for all static files.
                    context.Context.Response.Headers["Cache-Control"] = Configuration["StaticFiles:Headers:Cache-Control"];
                    context.Context.Response.Headers["Pragma"] = Configuration["StaticFiles:Headers:Pragma"];
                    context.Context.Response.Headers["Expires"] = Configuration["StaticFiles:Headers:Expires"];
                }
            });

            //Si el servidor de archivos se encuentra en una ubicacion externa, habilitar middleware de mapeo
            if (Azteca.Kernel.Server.FileServerPath != string.Empty)
            {
                app.MapWhen(
                        context => context.Request.Path.ToString().Contains("file-server"),
                        appBranch =>
                        {
                            appBranch.UseFileServerHandler();
                        });
            }

            app.UseMiddleware(typeof(Azteca.Web.Middleware.ErrorHandlingMiddleware));
            app.UseMvc();
        }
    }
}
