import { QueryList, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { FormBuilder } from '@angular/forms';
import { Location } from '@angular/common';
import { MenuItemDirective } from 'framework3/controls/directives/menu-item.directive';
import { ModalDirective } from 'ngx-bootstrap';
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import { Menu } from 'wijmo/wijmo.input';
import { Context } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { MenuItem } from "framework3/controls";
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
import { CedulaFiniquitoService } from '../services/cedulafiniquito.service';
import { CedulaFiniquitoInfo, CedulaConcepto, RescisionLaboral, TarifaPagoProvicional, SubsidioEmpleo, AcumuladoBajaTit } from '../models/cedulafiniquito';
export declare class CedulasFiniquitosEditor extends CatalogEditor {
    protected router: Router;
    protected route: ActivatedRoute;
    private formBuilder;
    protected changeDetector: ChangeDetectorRef;
    protected context: Context;
    private location;
    private cedulaFiniquitoService;
    cedulaFiniquito: CedulaFiniquitoInfo;
    rescisionesLaborales: Array<RescisionLaboral>;
    conceptosCedulaNegosiacion: wjcCore.CollectionView;
    conceptosApoyo: wjcCore.CollectionView;
    conceptosIsr: wjcCore.CollectionView;
    conceptosPrevio: wjcCore.CollectionView;
    calcularComision: boolean;
    calcular: boolean;
    timerInterval: any;
    idConceptoParent: number;
    lastCedulaConcepto: CedulaConcepto;
    newCedulaConcepto: CedulaConcepto;
    menuItemsConcepto: MenuItem[];
    tileDowmload: string;
    nameDownload: string;
    fechaAdeudo: Date;
    maxDate: Date;
    detailMode: string;
    detailModeMulti: string;
    menuItems: QueryList<MenuItemDirective>;
    dlgConfigirmacionCalculo: ModalDirective;
    dlgConcepto: ModalDirective;
    dlgAdeudo: ModalDirective;
    reportViewer: ContentViewerComponent;
    constructor(router: Router, route: ActivatedRoute, formBuilder: FormBuilder, changeDetector: ChangeDetectorRef, context: Context, location: Location, cedulaFiniquitoService: CedulaFiniquitoService);
    ngAfterViewInit(): void;
    createMenu(): void;
    onMenuItemClicked(menu: Menu, args: any): void;
    onConfigureCatalog(options: CatalogOptions): void;
    onCreatingItem(): void;
    onViewingItem(id: number): void;
    onCloningItem(id: number): void;
    onLoadingItem(id: number): void;
    onSavingItem(args: EditorEventArgs): void;
    onEmpleadoSelected(args: any): void;
    loadIngresoInformacion(idEmpleado: number, comision: boolean): void;
    onCambioSindicalizado(args: any): void;
    onSeleccionRescision(args: any): void;
    onCambioFechaAntiguedad(args: any): void;
    onCambioUltimoDiaLaborado(args: any): void;
    onCambioFechaBaja(args: any): void;
    onCambioFechaUltimoAguinaldo(args: any): void;
    onCambioDiasAguinaldoContrato(args: any): void;
    onCambioDisfrutePrevioAguinaldo(args: any): void;
    onCambioPrimaVacacionalPendiente(args: any): void;
    onCambioPirmaVacExentaPagada(args: any): void;
    onCambioPorcentajePrimaVacacional(args: any): void;
    onCambioVacacionesPendiente(args: any): void;
    onCambioVacacionesGeneradas(args: any): void;
    onCambioTotalDiasVacaciones(args: any): void;
    onCambioPorcentajePrimaAntiguedad(args: any): void;
    onCambioPorcentajeVeinteDiasAnio(args: any): void;
    onCambioSueldoDiario(args: any): void;
    onCambioSueldoPromedio(args: any): void;
    onCambioSueldoDiarioIntegrado(args: any): void;
    onCambioAusentismo(args: any): void;
    onCambioSueldoDias(args: any): void;
    onCambioSueldoImporte(args: any): void;
    onCambioComisiones(args: any): void;
    onCambioIncentivos(args: any): void;
    onCambioHorasExtraImporte(args: any): void;
    onCambioPrimaVacacionalPendienteEntrega(args: any): void;
    onCambioNumDomingos(args: any): void;
    onCambioPrimaDominicalImporte(args: any): void;
    onCambioRepartoUtilidades(args: any): void;
    onCambioDiasNegociadosIndemnizacion(args: any): void;
    onCambioIndemnizacionAdicional(args: any): void;
    onCambioOtros(args: any): void;
    onCambioSalarioMinimoZona(args: any): void;
    onCambioInfonavit(args: any): void;
    onCambioFonacot(args: any): void;
    onCambioAdeudo(args: any): void;
    consultarCalculo(calcular: boolean): void;
    onConceptoCancelar(): void;
    onConceptoAgregar(): void;
    /**
    * Calcula el concepto de adeudo sueldo
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) negosiación
    */
    cedulaConceptoAdeudoSueldo(idClasificacion: number): void;
    /**
    * Calcula las concepto de comisiones
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoComisiones(idClasificacion: number): void;
    /**
    * Calcula las concepto de incentivos
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoIncentivos(idClasificacion: number): void;
    /**
    * Calcula las concepto de Horas Extra
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoHorasExtra(idClasificacion: any): void;
    /**
    * Calcula las concepto de Reparto de Utilidades
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoRepartoUtilidades(idClasificacion: number): void;
    /**
    * Calculo del concepto Otros (En otros todo es gravado)
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoOtros(idClasificacion: number): void;
    /**
    * Calcula el concepto de aguninaldo
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) negosiación
    */
    cedulaConceptoAguinaldo(idClasificacion: number): void;
    /**
    * Calcula el concepto de vacaciones
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) negosiación
    */
    cedulaConceptoVacaciones(idClasificacion: number): void;
    /**
    * Calcula el concepto de Prima Dominical
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) negosiación
    */
    cedulaConceptoPrimaDominical(idClasificacion: number): void;
    /**
    * Calcula el concepto de Prima Vacacional
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) apoyo previo finiquito
    */
    cedulaConceptoPrimaVacacional(idClasificacion: number): void;
    /**
    * Calcula el concepto de Prima Antiguedad
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (3) negosiacion o (8) negosiación
    */
    cedulaConceptoPrimaAntiguedad(idClasificacion: number): void;
    /**
    * Calcula el concepto de Indemnizacion Ley(20d x Año)
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (3) negosiacion o (8) negosiación
    */
    cedulaConceptoIndeminzacionLeyDiasAnio(idClasificacion: number): void;
    /**
    * Calcula el concepto de Indemnizacion Ley(Días)
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (3) negosiacion o (8) negosiación
    */
    cedulaConceptoIndeminzacionLeyDias(idClasificacion: number): void;
    /**
    * Calcula el concepto de Indemnizacion Adicional Acordada
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (3) negosiacion o (8) negosiación
    */
    cedulaConceptoIndeminzacionAdicionalAcordada(idClasificacion: number): void;
    cedulaGravadoVsExento(idClasificacion: number): void;
    /**
     * Actualiza el ISR
     * @param idClasificacion
     */
    cedulaDeduccionesIsr(idClasificacion: any): void;
    calcularSeguros(idClasificacion: number): void;
    /**
    * ISR Mensual Ordinario - Sueldo Mensual Ordinario
    */
    calcularImporteAdeudo(idClasificacion: number): void;
    /**
     * Calcula el concepto de Total Pagos Separación
     */
    cedulaConceptoTotalPagosSeparacion(): void;
    /**
     * Calcula el concepto de Total Pagos Separación (Injust, Negocia)
     */
    cedulaConceptoTotalPagosSeparacionInjust(): void;
    /**
    * ISR Mensual Ordinario - Sueldo Mensual Ordinario
    */
    cedulaIsrSueldoMensualOrdinario(): void;
    cedulaIsrInjustificado(): void;
    cedulaIsrJustificado(): void;
    cedulaIsrPartesProporcionales(): void;
    cedulaIsrPercepcionesAdeudadas(): void;
    cedulaPrevioFiniquito(): void;
    cedulaInfonavitFonacot(): void;
    onTabSelectedPrevios(flex: wjcGrid.FlexGrid): void;
    onCambioComision(event: any): void;
    /*************************************************************************/
    getConcepto(idClasificacion: number, concepto: string): CedulaConcepto;
    getImporteCantidad(idClasificacion: number): number;
    getImporteTotal(idClasificacion: number): number;
    getImporteGravado(idClasificacion: number): number;
    getImporteExento(idClasificacion: number): number;
    getImporte(idClasificacion: number, tipo: string): number;
    getImporteAcumulado(concepto: CedulaConcepto): CedulaConcepto;
    onTabSelectedFlexGrid(flex: wjcGrid.FlexGrid): void;
    getDaysBetweenTwoDates(dateIni: Date, dateFin: Date): number;
    onRowDblclicked(s: wjcGrid.FlexGrid, e: any): void;
    printKardexAsistencia(): void;
    getSubsidioEmpleo(importe: number): SubsidioEmpleo;
    getSubsidioEmpleoByDia(dias: number, importe: number): SubsidioEmpleo;
    getTarifaPagoProvicional(importe: number): TarifaPagoProvicional;
    verificarTaskComision(): void;
    onMenuItemClick(option: MenuItem): void;
    onConsultarAdeudo(): void;
    calcularPercepciones(acumulado: AcumuladoBajaTit): string;
    calcularDeducciones(acumulado: AcumuladoBajaTit): string;
    calcularTotales(acumulado: AcumuladoBajaTit): string;
}
