﻿import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

//////////////////////////// Guards //////////////////////////////////
import { AuthGuard } from 'framework3/core';
import { EditorDeactivateGuard } from 'framework3/editors';

//////////////////////////// Catalogos ////////////////////////////////
import { CategoriasArticulosManager } from './catalogs/categorias-articulos.manager';
import { CategoriasArticulosEditor } from './catalogs/categorias-articulos.editor';
import { CategoriasUnidadesMedidaManager } from './catalogs/categorias-unidades-medida.manager';
import { CategoriasUnidadesMedidaEditor } from './catalogs/categorias-unidades-medida.editor';
import { DivisasManager } from './catalogs/divisas.manager';
import { DivisasEditor } from './catalogs/divisas.editor';
import { PACsManager } from './catalogs/pacs.manager';
import { PACsEditor } from './catalogs/pacs.editor';
import { ContribuyentesManager } from './catalogs/contribuyentes.manager';
import { ContribuyentesEditor } from './catalogs/contribuyentes.editor';
import { EmpresasManager } from './catalogs/empresas.manager';
import { EmpresasEditor } from './catalogs/empresas.editor';
import { SucursalesManager } from './catalogs/sucursales.manager';
import { SucursalesEditor } from './catalogs/sucursales.editor';
import { CuentasManager } from './catalogs/cuentas.manager';
import { FlujoManager } from './catalogs/flujo.manager';
import { ImpuestosManager } from './catalogs/impuestos.manager';
import { ImpuestosEditor } from './catalogs/impuestos.editor';
import { RetencionesManager } from './catalogs/retenciones.manager';
import { RetencionesEditor } from './catalogs/retenciones.editor';
import { CFDCertificadosManager } from './catalogs/cfdcertificados.manager';
import { CFDCertificadosEditor } from './catalogs/cfdcertificados.editor';
import { CFDSeriesManager } from './catalogs/cfdseries.manager';
import { CFDSeriesEditor } from './catalogs/cfdseries.editor';
import { MetodosPagoManager } from './catalogs/metodos-pago.manager';
import { MetodosPagoEditor } from './catalogs/metodos-pago.editor';
import { SegmentosNegocioEditor } from './catalogs/segmentos-negocio.editor';
import { SegmentosNegocioManager } from './catalogs/segmentos-negocio.manager';
import { NotasConceptosEditor } from './catalogs/notas-conceptos.editor';
import { NotasConceptosManager } from './catalogs/notas-conceptos.manager';
import { BancosManager } from './catalogs/bancos.manager';
import { BancosEditor } from './catalogs/bancos.editor';
import { CuentasBancariasManager } from './catalogs/cuentas-bancarias.manager';
import { CuentasBancariasEditor } from './catalogs/cuentas-bancarias.editor';
import { AddendasEditor } from './catalogs/addendas.editor';
import { AddendasManager } from './catalogs/addendas.manager';
import { PlantillasComprobantesEditor } from './catalogs/plantillas-comprobantes.editor';
import { PlantillasComprobantesManager } from './catalogs/plantillas-comprobantes.manager';
import { RiesgosPuestosEditor } from './catalogs/riesgos-puestos.editor';
import { RiesgosPuestosManager } from './catalogs/riesgos-puestos.manager';
import { JornadasEditor } from './catalogs/jornadas.editor';
import { JornadasManager } from './catalogs/jornadas.manager';
import { TiposRegimenesEditor } from './catalogs/tipos-regimenes.editor';
import { TiposRegimenesManager } from './catalogs/tipos-regimenes.manager';
import { PeriodosPagosEditor } from './catalogs/periodos-pagos.editor';
import { PeriodosPagosManager } from './catalogs/periodos-pagos.manager';
import { TiposContratosEditor } from './catalogs/tipos-contratos.editor';
import { TiposContratosManager } from './catalogs/tipos-contratos.manager';
import { ListaNegraManager } from './catalogs/lista-negra.manager';
import { ListaNegraEditor } from './catalogs/lista-negra.editor';
import { EjerciciosTiposEditor } from './catalogs/ejercicios-tipos.editor';
import { EjerciciosTiposManager } from './catalogs/ejercicios-tipos.manager';
import { EsquemasEditor } from './catalogs/esquemas.editor';
import { EsquemasManager } from './catalogs/esquemas.manager';
import { MapasDistribucionesEditor } from './catalogs/mapas-distribuciones.editor';
import { MapasDistribucionesManager } from './catalogs/mapas-distribuciones.manager';
import { MapaDistribucionCodeEditor } from './components/mapa-distribucion-code.editor';


//////////////////////////// Mantenimientos ////////////////////////////////
import { ComprobanteMaintenance } from './maintenances/comprobante.maintenance';
import { IepsArchivoGenerador } from './maintenances/ieps-archivo-generador.maintenance';
import { AdmonCancelacion } from './maintenances/admon-cancelacion.maintenance';

///////////////////////////// Documentos /////////////////////////////////
import { PolizasEditor } from './documents/polizas.editor';
import { PolizasManager } from './documents/polizas.manager';
import { PolizasByLoteEditor } from './documents/parts/polizas-by-lote.editor';

import { ComprobantesManager } from './documents/comprobantes.manager';
import { ComprobantesLoaderComponent } from './documents/comprobantes.loader';

import { NotasManager } from './documents/notas.manager';
import { NotasEditor } from './documents/notas.editor';
import { NotaDetailEditor } from './documents/parts/notas-detail.editor';

import { FichasManager } from './documents/fichas.manager';
import { FichasEditor } from './documents/fichas.editor';
import { FichaDetailEditor } from './documents/parts/fichas-detail.editor';


import { EjerciciosManager } from './documents/ejercicios.manager';
import { EjerciciosEditor } from './documents/ejercicios.editor';
import { PeriodosEditor } from './documents/periodos.editor';

import { ComprobantesCancelacionesManager } from './documents/comprobantes-cancelaciones.manager';
import { ComprobantesCancelacionesEditor } from './documents/comprobantes-cancelaciones.editor';

///////////////////////////// Componentes /////////////////////////////////
import { TipoCambioComponent } from './components/tipos-cambio.component';
import { TipoCambioEditorComponent } from './components/tipos-cambio-editor.component';
import { CuentaBancariaMiembrosComponent } from './components/cuenta-bancaria-miembros.component';

const routes: Routes = [
    {
        path: '',
        canActivateChild: [AuthGuard],
        children: [
            { path: 'articulos-fiscales/:proceso', component: CategoriasArticulosManager },
            { path: 'articulos-fiscales/:proceso/:id', component: CategoriasArticulosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'unidades-fiscales/:proceso', component: CategoriasUnidadesMedidaManager },
            { path: 'unidades-fiscales/:proceso/:id', component: CategoriasUnidadesMedidaEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'contribuyentes/:proceso', component: ContribuyentesManager },
            { path: 'contribuyentes/:proceso/:id', component: ContribuyentesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'empresas-contabilidad/:proceso', component: EmpresasManager },
            { path: 'empresas-contabilidad/:proceso/:id', component: EmpresasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'sucursales-contabilidad/:proceso', component: SucursalesManager },
            { path: 'sucursales-contabilidad/:proceso/:idEmpresa/:id', component: SucursalesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'cuentas/:proceso', component: CuentasManager },
            { path: 'flujo/:proceso', component: FlujoManager },
            { path: 'divisas/:proceso', component: DivisasManager },
            { path: 'divisas/:proceso/:id', component: DivisasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'pacs/:proceso', component: PACsManager },
            { path: 'pacs/:proceso/:id', component: PACsEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'impuestos/:proceso', component: ImpuestosManager },
            { path: 'impuestos/:proceso/:id', component: ImpuestosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'retenciones/:proceso', component: RetencionesManager },
            { path: 'retenciones/:proceso/:id', component: RetencionesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'certificados/:proceso', component: CFDCertificadosManager },
            { path: 'certificados/:proceso/:id', component: CFDCertificadosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'series/:proceso', component: CFDSeriesManager },
            { path: 'series/:proceso/:IdCFDCertificado/:id', component: CFDSeriesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'metodosPago/:proceso', component: MetodosPagoManager },
            { path: 'metodosPago/:proceso/:id', component: MetodosPagoEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'banco/:proceso', component: BancosManager },
            { path: 'banco/:proceso/:id', component: BancosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'cuentas-bancarias/:proceso', component: CuentasBancariasManager },
            { path: 'cuentas-bancarias/:proceso/:idBanco/:id', component: CuentasBancariasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'banco/:proceso/:idBanco/:id/members', component: CuentaBancariaMiembrosComponent },
            { path: 'cuentas-bancarias/:proceso/:idBanco/:id/members', component: CuentaBancariaMiembrosComponent },

            { path: 'notas-conceptos/:proceso', component: NotasConceptosManager },
            { path: 'notas-conceptos/:proceso/:id', component: NotasConceptosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'tipos-cambio/:proceso', component: TipoCambioComponent },
            { path: 'tipos-cambio-editor/:proceso', component: TipoCambioEditorComponent },
            { path: 'addendas/:proceso', component: AddendasManager },
            { path: 'addendas/:proceso/:id', component: AddendasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'plantilla-comprobante/:proceso', component: PlantillasComprobantesManager },
            { path: 'plantilla-comprobante/:proceso/:id', component: PlantillasComprobantesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'riesgo-puestos/:proceso', component: RiesgosPuestosManager },
            { path: 'riesgo-puestos/:proceso/:id', component: RiesgosPuestosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'jornadas/:proceso', component: JornadasManager },
            { path: 'jornadas/:proceso/:id', component: JornadasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'tipos-regimen/:proceso', component: TiposRegimenesManager },
            { path: 'tipos-regimen/:proceso/:id', component: TiposRegimenesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'periodos-pago/:proceso', component: PeriodosPagosManager },
            { path: 'periodos-pago/:proceso/:id', component: PeriodosPagosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'tipos-contrato/:proceso', component: TiposContratosManager },
            { path: 'tipos-contrato/:proceso/:id', component: TiposContratosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'lista-negra/:proceso', component: ListaNegraManager },
            { path: 'lista-negra/:proceso/:id', component: ListaNegraEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'ejercicios-tipos/:proceso', component: EjerciciosTiposManager },
            { path: 'ejercicios-tipos/:proceso/:id', component: EjerciciosTiposEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'esquemas/:proceso', component: EsquemasManager },
            { path: 'esquemas/:proceso/:id', component: EsquemasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'mapa-distribucion/:proceso', component: MapasDistribucionesManager },
            { path: 'mapa-distribucion/:proceso/:id', component: MapasDistribucionesEditor, canDeactivate: [EditorDeactivateGuard] },

            //DOCUMENTS
            { path: 'polizas/:proceso', component: PolizasManager },
            { path: 'polizas/:proceso/:id', component: PolizasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'polizas-by-lote/:proceso', component: PolizasByLoteEditor },

            { path: 'comprobantes-loader/:proceso', component: ComprobantesLoaderComponent },
            { path: 'comprobantes/:proceso', component: ComprobantesManager },
            { path: 'comprobantes/:proceso/:sentido', component: ComprobantesManager },


            { path: 'ejercicios/:proceso', component: EjerciciosManager },
            { path: 'ejercicios/:proceso/:id', component: EjerciciosEditor },
            { path: 'periodosEditor/:proceso/:idEjercicio/:id', component: PeriodosEditor },

            { path: 'segmentosNegocio/:proceso', component: SegmentosNegocioManager },
            { path: 'segmentosNegocio/:proceso/:id', component: SegmentosNegocioEditor, canDeactivate: [EditorDeactivateGuard] },
            
            { path: 'notas/:proceso', component: NotasManager },
            { path: 'notas/:proceso/:id', component: NotasEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'fichas/:proceso', component: FichasManager },
            { path: 'fichas/:proceso/:id', component: FichasEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'series/:proceso/:idCFDCertificado/:id', component: CFDSeriesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'comprobante/:proceso', component: ComprobanteMaintenance },

            { path: 'ieps-archivo-generador/:proceso', component: IepsArchivoGenerador },

            { path: 'comprobante-cancelaciones/:proceso', component: ComprobantesCancelacionesManager },
            { path: 'comprobante-cancelaciones/:proceso/:id', component: ComprobantesCancelacionesEditor },

            // MAINTENANCE
            { path: 'admon-cancelacion/:proceso', component: AdmonCancelacion },
            { path: 'mapa-code-editor/:proceso', component: MapaDistribucionCodeEditor }
        ]
    },
];

@NgModule({
    imports: [
        RouterModule.forChild(routes)
    ],
    exports: [
        RouterModule
    ]
})
export class AccountingRoutingModule { }

