﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { BancoService} from '../services/banco.service';
import { BancoInfo, BancoItem } from '../models/banco';
import { CuentaBancariaItem} from '../models/cuenta-bancaria';
import { CuentaBancariaService } from '../services/cuenta-bancaria.service';
import { Control } from 'wijmo/wijmo';

@Component({
    selector: 'contabilidad-banco-editor',
    templateUrl: './bancos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => BancosEditor)
        },
        {
            provide: CatalogService, useExisting: BancoService
        }
    ]
})
export class BancosEditor extends CatalogEditor {
    banco: BancoInfo = new BancoInfo(); 
    @Input() cuentasBancarias: CuentaBancariaItem[] = [];

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private bancoService: BancoService, private cuentaBancariaService: CuentaBancariaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'codigoFiscal': ['', Validators.required],
            'cuenta': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.bancoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.banco = info;       
                this.loadCuentasBancarias();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.bancoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.banco = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.bancoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();                
                this.banco = info;             
                this.loadCuentasBancarias();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.bancoService.update(this.banco)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.banco.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.banco = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadCuentasBancarias() {
        let request: SearchRequest = new SearchRequest();
        request.paged = false;
        //Agregar condición de empresa actual
        let condicion: SearchCondition = { field: "IdBanco", operator: "=", valueType: "int", value: this.banco.identity.toString() };
        request.conditions.push(condicion);
        //
        this.context.app.showSpinner();
        this.cuentaBancariaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.cuentasBancarias = response.rows;
            })
            .catch(error => this.handleError(error));
    }

    onSelectedTab(arg: any) {
        Control.invalidateAll();
    }

}