﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { CFDCertificadoService} from '../services/cfd-certificado.service';
import { CFDCertificadoInfo, CFDCertificadoItem } from '../models/cfd-certificado';
import { SerieFiscalService } from '../services/serie-fiscal.service';
import { SerieFiscalItem } from '../models/serie-fiscal';
import { Control } from 'wijmo/wijmo';



@Component({
    selector: 'contabilidad-cfdcertificados-editor',
    templateUrl: './cfdcertificados.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CFDCertificadosEditor)
        },
        {
            provide: CatalogService, useExisting: CFDCertificadoService
        }
    ]
})
export class CFDCertificadosEditor extends CatalogEditor {
    cfdCertificado: CFDCertificadoInfo = new CFDCertificadoInfo();
    @Input() series: SerieFiscalItem[] = [];        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private cFDCertificadoService: CFDCertificadoService, private serieFiscalService: SerieFiscalService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],            
            'passwordPK': ['', Validators.required],
            'certificado': ['', Validators.required],
            'llavePrivada': ['', Validators.required]

        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.cFDCertificadoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cfdCertificado = info;                
                this.loadSeries();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.cFDCertificadoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cfdCertificado = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.cFDCertificadoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cfdCertificado = info;            
                this.loadSeries();    
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.cFDCertificadoService.update(this.cfdCertificado)
            .then(info => {
                this.context.app.hideSpinner();
                this.cfdCertificado = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    onSelectedTab(arg: any) {
        Control.invalidateAll();
    }

    loadSeries() {
        let request: SearchRequest = new SearchRequest();
        request.paged = false;
        //Agregar condición de empresa actual
        //let condicionServidorSMTP: SearchCondition = { field: "IdCFDCertificado", operator: "=", valueType: "int", value: this.cfdCertificado.identity.toString() };
        //request.conditions.push(condicionServidorSMTP);
        //
        this.context.app.showSpinner();
        this.serieFiscalService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.series = response.rows;
            })
            .catch(error => this.handleError(error));
    }
}