﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { SerieFiscalService} from '../services/serie-fiscal.service';
import { SerieFiscalInfo, SerieFiscalItem} from '../models/serie-fiscal';

@Component({
    selector: 'contabilidad-cfdseries-editor',
    templateUrl: './cfdseries.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CFDSeriesEditor)
        },
        {
            provide: CatalogService, useExisting: SerieFiscalService
        }
    ]
})
export class CFDSeriesEditor extends CatalogEditor {
    seriefiscal: SerieFiscalInfo = new SerieFiscalInfo();        
    IdCFDCertificado: number;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private serieFiscalService: SerieFiscalService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'serie': ['', Validators.required],
            'ultimoFolio': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.route.params.forEach((params: Params) => {
            this.IdCFDCertificado = +params['IdCFDCertificado'];
        }); 
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        this.seriefiscal.IdCFDCertificado = this.IdCFDCertificado;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.serieFiscalService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.seriefiscal = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.serieFiscalService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.seriefiscal = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.serieFiscalService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.seriefiscal = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.serieFiscalService.update(this.seriefiscal)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.seriefiscal.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.seriefiscal = info;
            })
            .catch(error => this.handleError(error));        
    }

    onCancelingEdit() {
        this.location.back();
    }


}