﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, Output, EventEmitter } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { SerieFiscalService} from '../services/serie-fiscal.service';
import { SerieFiscalInfo, SerieFiscalItem} from '../models/serie-fiscal';

@Component({    
    selector: 'contabilidad-cfdseries-manager',
    templateUrl: './cfdseries.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CFDSeriesManager)
        },
        {
            provide: CatalogService, useExisting: SerieFiscalService
        }
	]
})
export class CFDSeriesManager extends CatalogManager {
    @Input() series: SerieFiscalItem[] = [];  
    @Input() IdCFDCertificado: number;  
    @Output() refreshListParent: EventEmitter<void> = new EventEmitter<void>(); 
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private serieFiscalService: SerieFiscalService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        //this.manager.serverSide = options.busquedasServidor;
        //this.manager.pageSize = options.tamanioPagina;        
        //this.refreshList();
    }

    doSearch(request: SearchRequest) {
        //this.context.app.showSpinner();

        //this.serieFiscalService.search(this.context.app.defaultSite.identity, request)
        //    .then(response => {
        //        this.context.app.hideSpinner();
        //        //
        //        this.series = response.rows;
        //        this.totalRows = response.totalRows;
        //        this.pageIndex = request.pageIndex;                                
        //    })
        //    .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['accounting/series', this.process.id, this.IdCFDCertificado, 0]);
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate(['accounting/series', this.process.id, this.IdCFDCertificado, item.id]);
    }

    onDeletingItem(item: CatalogItem) {
        this.serieFiscalService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
                this.refreshListParent.emit();
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.serieFiscalService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
                this.refreshListParent.emit();
            })
            .catch(error => this.handleError(error));
    }

}
