﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { ContribuyenteService} from '../services/contribuyente.service';
import { ContribuyenteInfo, ContribuyenteItem } from '../models/contribuyente';
import { MenuItem } from 'framework3/controls';

@Component({    
    selector: 'contabilidad-contribuyentes-manager',
    templateUrl: './contribuyentes.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ContribuyentesManager)
        },
        {
            provide: CatalogService, useExisting: ContribuyenteService
        }
	]
})
export class ContribuyentesManager extends CatalogManager {
    contribuyentes: ContribuyenteItem[]=[];    
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private contribuyenteService: ContribuyenteService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.contribuyenteService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.contribuyentes = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.contribuyenteService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.contribuyenteService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'VALIDATE') {
            this.context.app.showSpinner();
            this.contribuyenteService.consultarRfcInscrito(this.manager.currentItem.id)
                .then(result => {
                    this.context.app.hideSpinner();
                    if (result) {
                        this.showAlert("RFC " + this.manager.currentItem.codigo + " válido", "success");
                    }
                    else {
                        this.showAlert("El RFC " + this.manager.currentItem.codigo + " no es válido según la lista de RFC inscritos no cancelados en el SAT", "danger");
                    }
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
    }

}
