﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { FormatoTipoService, FormatoItem } from 'framework3/general';

//Features
import { CuentaBancariaService} from '../services/cuenta-bancaria.service';
import { CuentaBancariaInfo, CuentaBancariaItem} from '../models/cuenta-bancaria';

@Component({
    selector: 'contabilidad-cuenta-bancaria-editor',
    templateUrl: './cuentas-bancarias.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CuentasBancariasEditor)
        },
        {
            provide: CatalogService, useExisting: CuentaBancariaService
        }
    ]
})
export class CuentasBancariasEditor extends CatalogEditor {
    cuentaBancaria: CuentaBancariaInfo = new CuentaBancariaInfo(); 
    idBanco: number;  
    formatosImpresion: FormatoItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private cuentaBancariaService: CuentaBancariaService, private formatoTipoService: FormatoTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuenta': ['', Validators.required],
            'clabe': ['', Validators.required],
            'idFormato': [''],
            'idCuenta': [''],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    loadFormatosImpresion() {
        this.formatoTipoService.loadFormatos('CHECKS', this.context.app.defaultCompany.identity)
            .then(formatos => {
                this.formatosImpresion = formatos;
                this.formatosImpresion.unshift({ id: 0, nombre: '[NINGUNO]', activo: true, codigo: '' });
            })
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.route.params.forEach((params: Params) => {
            this.idBanco = +params['idBanco'];
        }); 
        this.loadFormatosImpresion();
        this.cuentaBancaria.idBanco = this.idBanco;
    }

    onCreatingItem() {        
        this.cuentaBancaria.idBanco = this.idBanco;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.cuentaBancariaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cuentaBancaria = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.cuentaBancariaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cuentaBancaria = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.cuentaBancariaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cuentaBancaria = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.cuentaBancariaService.update(this.cuentaBancaria)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.cuentaBancaria.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
                    if (args.closeEditor)
                        this.location.back();
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');             
                }
				this.cuentaBancaria = info;
            })
            .catch(error => this.handleError(error));        
    }

    onCancelingEdit() {
        this.location.back();
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['members'], { relativeTo: this.route });
        }
    }

}