﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, Output, EventEmitter  } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';

//Features
import { CuentaBancariaService} from '../services/cuenta-bancaria.service';
import { CuentaBancariaInfo, CuentaBancariaItem} from '../models/cuenta-bancaria';

@Component({    
    selector: 'contabilidad-cuentas-bancarias-manager',
    templateUrl: './cuentas-bancarias.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CuentasBancariasManager)
        },
        {
            provide: CatalogService, useExisting: CuentaBancariaService
        }
	]
})
export class CuentasBancariasManager extends CatalogManager {
    @Input()cuentasBancarias: CuentaBancariaItem[] = [];        
    @Input() idBanco: number;
    @Output() refreshListParent: EventEmitter<void> = new EventEmitter<void>();
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private cuentaBancariaService: CuentaBancariaService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        //this.manager.serverSide = options.busquedasServidor;
        //this.manager.pageSize = options.tamanioPagina;        
        //this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        //this.context.app.showSpinner();

        //this.cuentaBancariaService.search(this.context.app.defaultSite.identity, request)
        //    .then(response => {
        //        this.context.app.hideSpinner();
        //        //
        //        this.cuentasBancarias = response.rows;
        //        this.totalRows = response.totalRows;
        //        this.pageIndex = request.pageIndex;                                
        //    })
        //    .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['accounting/cuentas-bancarias', this.process.id, this.idBanco, 0]);
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate(['accounting/cuentas-bancarias', this.process.id, this.idBanco, item.id]);
    }

    onDeletingItem(item: CatalogItem) {
        this.cuentaBancariaService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
                this.refreshListParent.emit();
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate(['accounting/cuentas-bancarias', this.process.id, this.idBanco, 0, { cloneId: item.id }]);
    }

    onChangingItemState(item: CatalogItem) {
        this.cuentaBancariaService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
                this.refreshListParent.emit();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate([this.manager.currentItem.id.toString() + '/members'], { relativeTo: this.route });
        }
     
    }
}
