﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { DivisaService} from '../services/divisa.service';
import { DivisaInfo, DivisaItem} from '../models/divisa';

@Component({
    selector: 'contabilidad-divisas-editor',
    templateUrl: './divisas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DivisasEditor)
        },
        {
            provide: CatalogService, useExisting: DivisaService
        }
    ]
})
export class DivisasEditor extends CatalogEditor {
    divisa: DivisaInfo = new DivisaInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private divisaService: DivisaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'abreviatura': ['', Validators.required],
            'tipo': [''],
            'serieBanxico': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.divisaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.divisa = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.divisaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.divisa = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.divisaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.divisa = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.divisaService.update(this.divisa)
            .then(info => {
                this.context.app.hideSpinner();
                this.divisa = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

}