﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';

import { EnsambladoService } from 'framework3/core/services/ensamblado.service';
import { DocumentTypeService } from 'framework3/core/services/document-type.service';

import { EnsambladoItem } from 'framework3/core/models/ensamblado';




//Features
import { EsquemaService} from '../services/esquema.service';
import { EsquemaInfo, EsquemaItem} from '../models/esquema';

@Component({
    selector: 'contabilidad-esquema-editor',
    templateUrl: './esquemas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EsquemasEditor)
        },
        {
            provide: CatalogService, useExisting: EsquemaService
        }
    ]
})
export class EsquemasEditor extends CatalogEditor {
    esquema: EsquemaInfo = new EsquemaInfo();        

    origenes = [{ "id": "D", "nombre": "Documento" }, { "id": "C", "nombre": "Clase" }];

    ensamblados: EnsambladoItem[] = [];
    documentsTypes: any[] = [];

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private esquemaService: EsquemaService, private ensambladoSvc: EnsambladoService, private documentTypeSvc: DocumentTypeService) {
        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'origen': ['', Validators.required],
            'documentType': [''],
            'ensamblado': [''],
            'clase': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        setTimeout(_ => {
            this.cargarEnsamblados();
            this.cargarDocumentsTypes();
        });
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.esquemaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.esquema = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.esquemaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.esquema = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.esquemaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.esquema = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.esquemaService.update(this.esquema)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.esquema.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.esquema = info;
            })
            .catch(error => this.handleError(error));        
    }

    ///////////////////////// METODOS ///////////////////////////////////////
    cargarEnsamblados() {
        this.ensambladoSvc.loadList()
            .then(ensamblados => {
                this.ensamblados = ensamblados;
                this.ensamblados.unshift({ id: 0, codigo: '', nombre: '[NINGUNO]', activo: true });
            })
            .catch(error => this.handleError(error));
    }

    cargarDocumentsTypes() {
        this.documentTypeSvc.loadList()
            .then(result => {
                this.documentsTypes = result;
                this.documentsTypes.unshift({ id: 0, nombre: '[NINGUNO]' });
            })
            .catch(error => this.handleError(error));
    }

}