﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { FlujoInfo } from '../models/flujo';
import { FlujoService } from '../services/flujo.service';

@Component({    
    selector: 'contabilidad-flujo-manager',
    templateUrl: './flujo.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FlujoManager)
        },
        {
            provide: CatalogService, useExisting: FlujoService
        }
    ]
})
export class FlujoManager extends HierarchyManager {
    flujo: FlujoInfo[] = [];
    flujoEdicion: FlujoInfo = new FlujoInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;

    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private flujoService: FlujoService, protected router: Router, protected route: ActivatedRoute) {
        super(context,router, route, changeDetector);
       
        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required]
        });
    }

	////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.flujoService.loadHierarchy(this.context.app.defaultCompany.identity,0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.flujo = nodes;
            })
            .catch(error => this.handleError(error));
    }


	//////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadHierarchy();
        //Hacer llamadas de carga inicial de datos aquí

    }


    onNodesChanged(flujo: FlujoInfo[]) {
        this.flujo = flujo;        
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.flujoEdicion = new FlujoInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.flujoEdicion = new FlujoInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.flujoEdicion.idBranch = this.context.app.defaultSite.identity;
        this.flujoEdicion.idCompany = this.context.app.defaultCompany.identity;
		//
        this.currentParent = parentNode;        
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {        
        this.flujoEdicion = this.flujoService.copy(node.data);
        this.showEditor();        
    }

    onSavingNode() {        
        //Si es un nuevo nodo
        if (this.flujoEdicion.clientID == 0) {
            this.addNode(this.flujoEdicion, this.currentParent);            
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.flujoEdicion);
        }
		this.flujoEdicion = new FlujoInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<FlujoInfo>) {
        this.context.app.showSpinner();
        this.flujoService.saveHierarchy(changes)
            .then(flujo => {
                this.flujo = flujo;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));            
    }

}