﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ImpuestoService} from '../services/impuesto.service';
import { ImpuestoInfo, ImpuestoItem, ImpuestoTipoItem} from '../models/impuesto';

@Component({
    selector: 'contabilidad-impuestos-editor',
    templateUrl: './impuestos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ImpuestosEditor)
        },
        {
            provide: CatalogService, useExisting: ImpuestoService
        }
    ]
})
export class ImpuestosEditor extends CatalogEditor {
    impuesto: ImpuestoInfo = new ImpuestoInfo();        
    //
    impuestosTipo: ImpuestoTipoItem[] = [];

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private impuestoService: ImpuestoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'abreviatura': ['', Validators.required],
            'porcentaje': ['', Validators.required],
            'variable': [''],
            'local': [''],
            'tipoImpuesto': ['', Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadTipos();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.impuestoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.impuesto = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.impuestoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.impuesto = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.impuestoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.impuesto = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.impuestoService.update(this.impuesto)
            .then(info => {                
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.impuesto.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.impuesto = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadTipos() {
        this.impuestoService.loadTipos(this.context.app.defaultCompany.identity.toString())
            .then(tipos => this.impuestosTipo = tipos)
            .catch(error => this.handleError(error));
    }


}