﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ListaNegraService} from '../services/lista-negra.service';
import { ListaNegraInfo, ListaNegraItem} from '../models/lista-negra';
import { now } from 'moment';

@Component({
    selector: 'azteca.business.contabilidad-lista-negra-editor',
    templateUrl: './lista-negra.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ListaNegraEditor)
        },
        {
            provide: CatalogService, useExisting: ListaNegraService
        }
    ]
})
export class ListaNegraEditor extends CatalogEditor {
    listaNegra: ListaNegraInfo = new ListaNegraInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private listaNegraService: ListaNegraService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'rfc': ['', Validators.required],
            'situacion':['',Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.listaNegraService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.listaNegra = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.listaNegraService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.listaNegra = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number)
    {
        this.context.app.showSpinner();

        this.listaNegraService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.listaNegra = info;            
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs)
    {
        this.context.app.showSpinner();
        this.listaNegra.fechaSys = new Date(now());
        this.listaNegraService.update(this.listaNegra)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.listaNegra.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.listaNegra = info;
            })
            .catch(error => this.handleError(error));        
    }

}