﻿//Frameworks
import { Component, ViewChild, forwardRef, ContentChildren, QueryList } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { ListaNegraService} from '../services/lista-negra.service';
import { ListaNegraInfo, ListaNegraItem} from '../models/lista-negra';
import { ModalDirective } from 'ngx-bootstrap';
import { MenuItem } from 'framework3/controls';
import { JSDocCommentStmt } from '@angular/compiler';
import { FileInfo } from 'framework3/general';
import { error } from 'util';

@Component({    
    selector: 'azteca.business.contabilidad-lista-negra-manager',
    templateUrl: './lista-negra.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ListaNegraManager)
        },
        {
            provide: CatalogService, useExisting: ListaNegraService
        }
	]
})
export class ListaNegraManager extends CatalogManager {
    file: FileInfo = new FileInfo();
    textMenu: string = '';
    currentItem: CatalogItem = null;
    listaNegra: ListaNegraItem[] = [];    
    @ViewChild('dialogFile') dialogFile: ModalDirective;
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private listaNegraService: ListaNegraService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
        this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.listaNegraService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.listaNegra = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.listaNegraService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.showAlert('Acción no permitida', 'success');     
        //this.listaNegraService.changeStatus(item.id, !item.activo)
        //    .then(success => {
        //        this.refreshList();
        //        this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');                
        //    })
        //    .catch(error => this.handleError(error));
    }

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
        if (item.activo)
            this.textMenu = 'Bloquear EFO';
        else
            this.textMenu = 'Permitir EFO';
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == "UPLOADFILE") {
            this.dialogFile.config.backdrop = false;
            this.dialogFile.show();
        }
        else if (option.name == "PERMITTED") {
            this.listaNegraService.permitirEFO(this.currentItem.id, this.context.session.user.identity, this.context.app.defaultCompany.idCompany, !this.currentItem.activo)
                .then(success => {
                    this.refreshList();
                    this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == 'DELETEFILE') {
            this.context.app.showSpinner();
            this.listaNegraService.eliminarFirmas()
                .then(success => {
                    this.context.app.hideSpinner();
                    this.showAlert("Se eliminaron las imagenes", 'success');
                })
                .catch(error => this.handleError(error));
        }
    }
    onSelectFile() {
        if (this.file.extension == 'csv') {
            this.context.app.showSpinner();

            this.listaNegraService.listaFromFile(this.file,this.context.app.defaultCompany.idCompany,this.context.app.defaultCompany.idBranch)
                .then(success => {
                    this.context.app.hideSpinner();
                    this.dialogFile.hide();
                    this.refreshList();
                    this.showAlert("Se insertaron correctamente", 'success');
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.showAlert('Tiene que ser un archivo CSV','error');
        }
    }

}
