﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { MetodoPagoService} from '../services/metodo-pago.service';
import { MetodoPagoInfo, MetodoPagoItem} from '../models/metodo-pago';

@Component({
    selector: 'contabilidad-metodospago-editor',
    templateUrl: './metodos-pago.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => MetodosPagoEditor)
        },
        {
            provide: CatalogService, useExisting: MetodoPagoService
        }
    ]
})
export class MetodosPagoEditor extends CatalogEditor {
    metodopago: MetodoPagoInfo = new MetodoPagoInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private metodoPagoService: MetodoPagoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuenta': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.metodoPagoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.metodopago = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.metodoPagoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.metodopago = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.metodoPagoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.metodopago = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.metodoPagoService.update(this.metodopago)
            .then(info => {
                this.context.app.hideSpinner();
                this.metodopago = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

}