﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, TrackableItem  } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { NotaConceptoService} from '../services/nota-concepto.service';
import { NotaConceptoInfo, NotaConceptoItem } from '../models/nota-concepto';
import { ImpuestosAsignacionEditor } from '../components/impuesto-asignacion.editor';
import { ImpuestoAsignacionInfo } from '../models/impuesto-asignacion';

@Component({
    selector: 'contabilidad-nota-concepto-editor',
    templateUrl: './notas-conceptos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => NotasConceptosEditor)
        },
        {
            provide: CatalogService, useExisting: NotaConceptoService
        }
    ]
})
export class NotasConceptosEditor extends CatalogEditor {
    notaConcepto: NotaConceptoInfo = new NotaConceptoInfo();        
    conceptoImpuestos: ImpuestoAsignacionInfo[] = [];
    conceptoImpCollection: TrackableCollection = new TrackableCollection();

    //
    @ViewChild(ImpuestosAsignacionEditor) conceptoImpuestosDialog: ImpuestosAsignacionEditor;

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private notaConceptoService: NotaConceptoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'conceptoLibre': [0, Validators.required],
            'categoriaArticulo': [0, Validators.required],
            'idCuenta': [0, Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    ngAfterViewChecked() {
        this.conceptoImpCollection.sourceCollection = this.notaConcepto.impuestos;
    }


    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.notaConceptoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.notaConcepto = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.notaConceptoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.notaConcepto = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.notaConceptoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.notaConcepto = info;                
            })
            .catch(error => this.handleError(error));
    }


    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.notaConceptoService.update(this.notaConcepto)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.notaConcepto.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.notaConcepto = info;
            })
            .catch(error => this.handleError(error));        
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.conceptoImpuestosDialog.show();
        }
    }


    onAgregarImpuesto(args: ImpuestoAsignacionInfo) {
        let impuestoExists: boolean = false;
        this.notaConcepto.impuestos.forEach(articuloImp => {
            if (articuloImp.idImpuesto == args.idImpuesto) {
                this.showAlert("El impuesto ya fue agregado", "información");
                impuestoExists = true;
            }
        })
        if (!impuestoExists) {
            this.notaConcepto.impuestos.push(args);
            this.conceptoImpCollection.refresh();
        }
    }

    onRemoveItem(item) {
        this.conceptoImpCollection.remove(item);
        this.conceptoImpCollection.refresh();
    }


}