﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { PACService} from '../services/pac.service';
import { PACInfo, PACItem} from '../models/pac';

@Component({
    selector: 'contabilidad-pacs-editor',
    templateUrl: './pacs.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PACsEditor)
        },
        {
            provide: CatalogService, useExisting: PACService
        }
    ]
})
export class PACsEditor extends CatalogEditor {
    pac: PACInfo = new PACInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private pACService: PACService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'clase': ['', Validators.required],
            'ensamblado': ['', Validators.required],
            'usuario': ['', Validators.required],
            'password': ['', Validators.required],
            'urlLogin': [''],
            'urlTimbrado': [''],
            'urlCancelacion':['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
       //this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.pACService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.pac = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.pACService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.pac = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.pACService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.pac = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.pACService.update(this.pac)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.pac.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.pac = info;
            })
            .catch(error => this.handleError(error));        
    }

}