﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';

import { MenuItem } from 'framework3/controls';
//Features
import { PlantillaComprobanteService} from '../services/plantilla-comprobante.service';
import { PlantillaComprobanteInfo, PlantillaComprobanteItem} from '../models/plantilla-comprobante';

@Component({    
    selector: 'accounting-plantillas-comprobantes-manager',
    templateUrl: './plantillas-comprobantes.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PlantillasComprobantesManager)
        },
        {
            provide: CatalogService, useExisting: PlantillaComprobanteService
        }
	]
})
export class PlantillasComprobantesManager extends CatalogManager {
    plantillasComprobantes: PlantillaComprobanteItem[]=[];    
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private plantillaComprobanteService: PlantillaComprobanteService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'DOWNLOAD') {
            this.context.app.showSpinner();
            this.plantillaComprobanteService.downloadPlantilla(this.manager.currentItem.id)
                .then(lote => {
                    this.context.app.hideSpinner();
                    this.showAlert("Se ha descargado la plantilla", "success");
                })
                .catch(error => this.handleError(error));
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.plantillaComprobanteService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.plantillasComprobantes = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.plantillaComprobanteService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.plantillaComprobanteService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
