﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { RetencionService} from '../services/retencion.service';
import { RetencionInfo, RetencionItem, RetencionTipoItem} from '../models/retencion';

@Component({
    selector: 'contabilidad-retenciones-editor',
    templateUrl: './retenciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RetencionesEditor)
        },
        {
            provide: CatalogService, useExisting: RetencionService
        }
    ]
})
export class RetencionesEditor extends CatalogEditor {
    retencion: RetencionInfo = new RetencionInfo();        
    //
    retencionesTipo: RetencionTipoItem[] = [];

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private retencionService: RetencionService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'abreviatura': ['', Validators.required],
            'porcentaje': ['', Validators.required],
            'variable': [''],
            'local': [''],
            'tipoRetencion': ['', Validators.required]

        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadTipos();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.retencionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.retencion = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.retencionService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.retencion = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.retencionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.retencion = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.retencionService.update(this.retencion)
            .then(info => {
                this.context.app.hideSpinner();
                this.retencion = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    loadTipos() {
        this.retencionService.loadTipos(this.context.app.defaultCompany.identity.toString())
            .then(tipos => this.retencionesTipo = tipos)
            .catch(error => this.handleError(error));
    }

}