﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { SegmentoNegocioService} from '../services/segmento-negocio.service';
import { SegmentoNegocioInfo, SegmentoNegocioItem} from '../models/segmento-negocio';

@Component({
    selector: 'contabilidad-segmentos-negocio-editor',
    templateUrl: './segmentos-negocio.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SegmentosNegocioEditor)
        },
        {
            provide: CatalogService, useExisting: SegmentoNegocioService
        }
    ]
})
export class SegmentosNegocioEditor extends CatalogEditor {
    segmentonegocio: SegmentoNegocioInfo = new SegmentoNegocioInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private segmentoNegocioService: SegmentoNegocioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.segmentoNegocioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.segmentonegocio = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.segmentoNegocioService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.segmentonegocio = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.segmentoNegocioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.segmentonegocio = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.segmentoNegocioService.update(this.segmentonegocio)
            .then(info => {
                this.context.app.hideSpinner();
                this.segmentonegocio = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

}