﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { BancoInfo, BancoItem } from '../models/banco';
import { BancoService } from '../services/banco.service';

@Component({
    selector: 'accounting-banco-selector',
    templateUrl: './banco-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => BancoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => BancoSelectorComponent),
        multi: true
    }]
})
export class BancoSelectorComponent extends CatalogSelector {
    banco: any[];
    selectedItem: BancoInfo = new BancoInfo;

    
    @Output() bancoUpdated = new EventEmitter();
    @Input() bancoSelector: boolean = true;

    constructor(protected context: Context, private bancoService: BancoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        }).pipe(mergeMap((keyword: string) => this.findBanco(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.bancoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + "-" + info.nombre ; //+ ", " + info.municipio.estado.nombre;
                    this.bancoUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.bancoUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.bancoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.banco = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findBanco(keyword: string): Observable<CatalogItem[]> {
        return this.bancoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

