﻿//Angular
import { Component, forwardRef, ViewChild, ChangeDetectorRef, Input, Output, EventEmitter } from '@angular/core';
import { Route, Router, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Framework 3
import { BaseComponent, CatalogService, Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor } from 'framework3/editors';
//Features
import { ComprobanteCorreo } from '../models/comprobante-correo'

@Component({
    selector: 'accounting-comprobante-correo',
    templateUrl: './comprobante-correo.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ComprobantesCorreoEditor)
        }
    ]
})

export class ComprobantesCorreoEditor extends CatalogEditor {
    //variables
    editorForm: FormGroup;
    tiposComprobantes: { id: number, name: string }[] = [
        { "id": 1, "name": "Ingreso" },
        { "id": 2, "name": "Egreso" },
        { "id": 3, "name": "Traslado" },
        { "id": 4, "name": "Nómina" },
        { "id": 5, "name": "Pago" }
    ];
   
    //Inputs
    @Input() data: ComprobanteCorreo = new ComprobanteCorreo();
    //Outputs
    @Output() dataUpdater: EventEmitter<ComprobanteCorreo> = new EventEmitter<ComprobanteCorreo>();

    //viewChilds
    @ViewChild('dialog') dialog: ModalDirective;
 

    constructor(protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetectorRef: ChangeDetectorRef) {
        super(context, router, route, changeDetectorRef);
        this.editorForm = formBuilder.group({
            'idComprobanteTipo': ['', AztecaValidators.selected],
            'correo': ['',Validators.required],
        });
    }

    onTipoComprobanteChange(id:number) {
       this.data.comprobanteTipo = this.tiposComprobantes.find(item=> item.id== id).name
    }

    show() {
        this.data = new ComprobanteCorreo();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => this.editorForm.reset());
        Control.invalidateAll();
    }

    onUpdate() {
        this.dataUpdater.emit(this.data);
        this.dialog.hide();
    }
        
}