﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem, PolicyInfo, UsuarioItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { UsuariosListComponent } from 'framework3/security';
//
import { CuentaBancariaService } from '../services/cuenta-bancaria.service';
import { CuentaBancariaInfo, CuentaBancariaItem } from '../models/cuenta-bancaria';


@Component({
    selector: 'accounting-cuenta-bancaria-miembros',
    templateUrl: './cuenta-bancaria-miembros.component.html'
})
export class CuentaBancariaMiembrosComponent extends BaseComponent {
    title: string = '';
    usuarios: UsuarioItem[];
    idCuentaBancaria: number;
    cuentaBancaria: CuentaBancariaInfo;
    currentItem: CatalogItem;

    //
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private cuentaBancariaService: CuentaBancariaService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idCuentaBancaria = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadCuentaBancaria());

    }

    ///////////////// METODOS ///////////////////////////////

    loadCuentaBancaria() {
        this.cuentaBancariaService.getDetailByID(this.idCuentaBancaria)
            .then(info => {
                this.cuentaBancaria = info;
                this.title = 'Miembros de la Cuenta-Bancaria - ' + this.cuentaBancaria.nombre;
            })
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.cuentaBancariaService.getMembers(this.idCuentaBancaria)
            .then(usuarios => {
                this.usuarios = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ///////////////// HANDLERS ///////////////////////////////
    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.cuentaBancariaService.addMembers(this.idCuentaBancaria, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser() {
        this.context.app.showSpinner();
        this.cuentaBancariaService.removeMember(this.idCuentaBancaria, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}