﻿import { Component, Input, Output, ViewChild, ContentChildren, OnInit, EventEmitter, QueryList, forwardRef, Optional, SkipSelf } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SearchCondition, SortOrder, HierarchySelector, ITreeHost  } from 'framework3/search/index'; //'../../search/index';
//
//
//
import { CuentaInfo, CuentaItem } from '../models/cuenta';
import { CuentaService } from '../services/cuenta.service';

@Component({
    selector: 'accounting-cuenta-selector',
    templateUrl: './cuenta-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => CuentaSelectorComponent)
    },
    {
        provide: ITreeHost,
        useExisting: forwardRef(() => CuentaSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => CuentaSelectorComponent),
        multi: true
    }]
})
export class CuentaSelectorComponent extends HierarchySelector implements ITreeHost {
    cuentas: CuentaItem[];
    nodes: CuentaInfo[];
    selectedItem: CuentaInfo = null;
    //Inputs
    @Input() idDocumento: number = 0;
    @Input() version: number = 0;
    @Input() idPadre: number = 0;
    //Outputs
    @Output() selectedItemChanged: EventEmitter<CuentaInfo> = new EventEmitter<CuentaInfo>();

    constructor(protected context: Context, private cuentaService: CuentaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        }).pipe(mergeMap((keyword: string) => this.findKeyword(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.cuentaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + "-" + info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = '';
            this.selectedItemChanged.emit(this.selectedItem);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.cuentaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.cuentas = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findKeyword(keyword: string): Observable<CatalogItem[]> {
        return this.cuentaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

    onLoadingHierarchy() {
        //Cargar nodos raíz al abrir diálogo de jerarquía
        this.cuentaService.loadChildren(this.context.app.defaultCompany.identity, 0)
            .then(rootNodes => this.nodes = rootNodes)
            .catch(error => this.handleError(error));
    }


    loadChildren(parent: CuentaInfo): Promise<CuentaInfo[]> {
        return this.cuentaService.loadChildren(this.context.app.defaultCompany.identity, parent.identity);
    }


}