﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { FlujoInfo, FlujoItem } from '../models/flujo';
import { FlujoService } from '../services/flujo.service';

@Component({
    selector: 'accounting-flujo-selector',
    templateUrl: './flujo-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => FlujoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => FlujoSelectorComponent),
        multi: true
    }]
})
export class FlujoSelectorComponent extends CatalogSelector {
    flujo: any[];
    selectedItem: FlujoInfo = new FlujoInfo;

    constructor(protected context: Context, private flujoService: FlujoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findFlujo(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.flujoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre; //+ ", " + info.municipio.estado.nombre;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.flujoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.flujo = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findFlujo(keyword: string): Observable<CatalogItem[]> {
        return this.flujoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

