﻿//Angular
import { Component, forwardRef, ViewChild, ChangeDetectorRef, Input, Output, EventEmitter } from '@angular/core';
import { Route, Router, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Framework 3
import { BaseComponent, CatalogService, Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor } from 'framework3/editors';
//Features
import { ImpuestoAsignacionInfo } from '../models/impuesto-asignacion';
import { ImpuestoService } from '../services/impuesto.service';
import { ImpuestoInfo } from '../models/impuesto';
import { ImpuestosSelectorComponent } from '../controls/impuestos-selector.component';

@Component({
    selector: 'accounting-impuesto-asignacion',
    templateUrl: './impuesto-asignacion.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ImpuestosAsignacionEditor)
        }
    ]
})

export class ImpuestosAsignacionEditor extends CatalogEditor {
    //variables
    editorForm: FormGroup;
    tiposFactor: { id: number, name: string }[] = [
        { "id": 1, "name": "Tasa" },
        { "id": 2, "name": "Cuota" }
    ];
    tiposCalculo: { id: number, name: string }[] = [
        { "id": 1, "name": "Base" },
        { "id": 2, "name": "Cascada" }
    ];
    tiposAplicacion: { id: number, nombre: string }[] = [
        { id: 1, nombre: 'Traslado' },
        { id: 2, nombre: 'Retención' }
    ];


    //Inputs
    @Input() data: ImpuestoAsignacionInfo = new ImpuestoAsignacionInfo();
    //Outputs
    @Output() dataUpdater: EventEmitter<ImpuestoAsignacionInfo> = new EventEmitter<ImpuestoAsignacionInfo>();

    //viewChilds
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild('selectorImpuestos') selectorImpuestos: ImpuestosSelectorComponent;

    constructor(protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetectorRef: ChangeDetectorRef) {
        super(context, router, route, changeDetectorRef);
        this.editorForm = formBuilder.group({
            'impuestos': ['', AztecaValidators.selected],
            'valorTasa': [1],
            'valorCuota': [1],
            'base': [1],
            'tipoCalculo': [0, AztecaValidators.selected],
            'ordenCalculo': [1],
            'tipoAplicacion': [0]
        });
    }

    onTipoCalculoChange() {
        if (this.editorForm.controls.tipoCalculo.value == 1) {
            this.data.tipoCalculo = 'B';
            this.data.ordenCalculo = 1;
        }
        else
            this.data.tipoCalculo = 'C';
        this.data.ordenCalculo = 2;
    }

    show() {
        this.data = new ImpuestoAsignacionInfo();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => this.editorForm.reset());
        Control.invalidateAll();
    }

    onUpdate() {
        if (this.editorForm.controls['tipoAplicacion'].value == 1) {
            this.data.tipoAplicacion = 'T';
        }
        else {
            this.data.tipoAplicacion = 'R';
        }
        this.dataUpdater.emit(this.data);
        this.dialog.hide();
    }

    onItemChanged(args: ImpuestoInfo) {
        this.data.impuesto = args;

        if (!args.rango) {
            this.data.tasaOCuota = args.valorFijo;
        }
        else {
            this.data.tasaOCuota = 0;
            this.data.base = 0;
        }

        this.tiposAplicacion = [];
        if (this.data.impuesto.traslado)
            this.tiposAplicacion.push({ id: 1, nombre: 'Traslado' });
        if (this.data.impuesto.retencion)
            this.tiposAplicacion.push({ id: 2, nombre: 'Retencion' });

        if (args.factor == 'C')
            this.editorForm.controls.base.setValidators(Validators.min(0.01));
        else
            this.editorForm.controls.base.clearValidators();
    }

}