﻿import { Component } from "@angular/core";
import { Context } from "framework3/core";
import { FormGroup, FormBuilder } from "@angular/forms";

//Tree
import { ITreeOptions, IActionMapping, TREE_ACTIONS, TreeNode } from 'angular-tree-component';

// Features
import { EsquemaInfo } from '../models/esquema';
import { MapElementInfo } from "../models/mapa-distribucion";

import { MapaDistribucionService } from '../services/mapa-distribucion.service';

@Component({
    selector: 'accounting-mapa-distribucion-editor',
    templateUrl: './mapa-distribucion-code.editor.html'
})
export class MapaDistribucionCodeEditor {

    formGroup: FormGroup;

    esquema: EsquemaInfo;
    idEsquema: number = 0;
    statement: string = "";
    code: string = "";

    nodes: MapElementInfo[] = [];

    actionMapping: IActionMapping = {
        mouse: {
            dblClick: (tree, node, event) => {
                this.code = node.data.path;
            }
        }
    };

    treeOptions: ITreeOptions = {
        displayField: 'name', allowDrag: false, actionMapping: this.actionMapping,
        getChildren: (node => {
            //return this.parent.loadChildren(node.data);
            return this.mapDistribucionSvc.getElementChildrens(node.data);
        })
    };

    constructor(protected context: Context, private builder: FormBuilder, private mapDistribucionSvc: MapaDistribucionService) {
        this.formGroup = builder.group({
            'esquema': [''],
            'statement': ['']
        });
    }

    //  Methods
    onSelectedEsquema(esquema: EsquemaInfo) {
        this.context.app.showSpinner();
        this.mapDistribucionSvc.getElementByEsquma(esquema.identity)
            .then(result => {
                this.nodes = result;
                this.context.app.hideSpinner();
            }).catch(error => {
                console.log(error);
                this.context.app.hideSpinner();
            });
    }


    //Tree events
    private onTreeUpdated() {

    }

}