﻿//Angular
import { Component, forwardRef, ViewChild, ChangeDetectorRef, Input, Output, EventEmitter } from '@angular/core';
import { Route, Router, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Framework 3
import { BaseComponent, CatalogService, Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor } from 'framework3/editors';
//Features
import { RetencionAsignacionInfo } from '../models/retencion-asignacion';
import { RetencionService } from '../services/retencion.service';
import { RetencionInfo } from '../models/retencion';
import { RetencionesSelectorComponent } from '../controls/retenciones-selector.component';

@Component({
    selector: 'accounting-retencion-asignacion',
    templateUrl: './retencion-asignacion.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RetencionAsignacionEditor)
        }
    ]
})

export class RetencionAsignacionEditor extends CatalogEditor {
    //variables
    editorForm: FormGroup;
    tiposCalculo: { id: number, name: string }[] = [
        { "id": 1, "name": "Base" },
        { "id": 2, "name": "Cascada" }
    ];

    //Inputs
    @Input() data: RetencionAsignacionInfo = new RetencionAsignacionInfo();
    //Outputs
    @Output() dataUpdater: EventEmitter<RetencionAsignacionInfo> = new EventEmitter<RetencionAsignacionInfo>();

    //viewChilds
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild('selectorRetenciones') selectorRetenciones: RetencionesSelectorComponent;

    constructor(protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetectorRef: ChangeDetectorRef) {
        super(context, router, route, changeDetectorRef);
        this.editorForm = formBuilder.group({
            'retenciones': ['', AztecaValidators.selected],
            'tipoCalculo': [0, AztecaValidators.selected],
            'ordenCalculo': [1]
        });
    }

    onTipoCalculoChange() {
        if (this.editorForm.controls.tipoCalculo.value == 1) {
            this.data.tipoCalculo = 'B';
            this.data.ordenCalculo = 1;
        }
        else
            this.data.tipoCalculo = 'C';
        this.data.ordenCalculo = 2;
    }

    show() {
        this.data = new RetencionAsignacionInfo();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => this.editorForm.reset());
        Control.invalidateAll();
    }

    onUpdate() {
        this.dataUpdater.emit(this.data);
        this.dialog.hide();
    }

    onItemChanged(args: RetencionInfo) {
        this.data.retencion = args;
    }

}