﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection, CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { TipoCambioItem } from '../models/divisa';
import { DivisaService } from '../services/divisa.service';
import { DivisasSelectorComponent } from '../controls/divisas-selector.component';

@Component({
    selector: 'inventario-tipos-cambio-editor',
    templateUrl: './tipos-cambio-editor.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TipoCambioEditorComponent)
        },
        {
            provide: CatalogService, useExisting: DivisaService
        }
    ]
})
export class TipoCambioEditorComponent extends CatalogEditor {
    editorForm: FormGroup;
    info: TipoCambioItem = new TipoCambioItem();
    tipoFormato: String = ''

    //Inputs
    @Input() data: TipoCambioItem = new TipoCambioItem();
    @Input() nuevoFlag: Boolean = new Boolean();
    //outputs
    @Output() dataUpdated: EventEmitter<TipoCambioItem> = new EventEmitter<TipoCambioItem>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('selectorDivisas') selectorDivisas: DivisasSelectorComponent;

    constructor(private divisaService: DivisaService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetector: ChangeDetectorRef) {
        super(context, router, route, changeDetector);
        this.editorForm = formBuilder.group({
            'divisa': ['', AztecaValidators.selected],
            'compra': ['', Validators.required],
            'venta': ['', Validators.required],
            'otro': ['', Validators.required]
        });
    }

    show() {
        this.info = new TipoCambioItem();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => {
            this.editorForm.reset();
            this.info = this.data;
            this.data = new TipoCambioItem();
        });
        Control.invalidateAll();
    }

    onSelectedItemChanged(args: any) {
        if (args != null)
            this.tipoFormato = 'n' + args.decimales.toString();
    }

    onUpdate() {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(divisa => {
                this.info.idDivisaDestino = divisa.identity;
                this.info.divisaDestino = divisa.nombre;
                this.info.divisaOrigen = this.selectorDivisas.displayValue;
                this.dataUpdated.emit(this.info);
                this.dialog.hide();
            })
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}