﻿import { Component, ViewChild, Input } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { DivisaService } from '../services/divisa.service';
import { TipoCambioEditorComponent } from '../components/tipos-cambio-editor.component';
import { TipoCambioItem } from '../models/divisa';

@Component({
    selector: 'articulos-tipos-cambio.component',
    templateUrl: './tipos-cambio.component.html'
})
export class TipoCambioComponent extends BaseComponent {
    currentItem: CatalogItem;
    tipCamCollection: TrackableCollection = new TrackableCollection();
    nuevoFlag: Boolean = new Boolean();

    @ViewChild(TipoCambioEditorComponent) tipoCambioEditarDialog: TipoCambioEditorComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    @Input() data: TipoCambioItem[] = [];
    //
    constructor(private context: Context, private divisaService: DivisaService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadTiposCambio() {
        this.context.app.showSpinner();
        this.divisaService.loadTiposCambio(this.context.app.defaultCompany.identity)
            .then(tiposCambio => {
                this.data = tiposCambio;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    ngOnInit() {
        this.loadTiposCambio();
    }

    ngAfterViewChecked() {
        this.tipCamCollection.sourceCollection = this.data;
    }

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SAVE') {
            this.guardarTiposCambio();
        }
        if (button.name == 'ADD') {
            this.tipoCambioEditarDialog.show();
            this.tipoCambioEditarDialog.nuevoFlag = true;
            this.nuevoFlag = true;
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    guardarTiposCambio() {
        this.divisaService.updateTiposCambio(this.data)
            .then(_ => this.loadTiposCambio())
    }

    onEditItem(tipoCambioItem: TipoCambioItem) {
        this.tipoCambioEditarDialog.show();
        this.tipoCambioEditarDialog.data = tipoCambioItem;
        this.tipoCambioEditarDialog.nuevoFlag = false;
        this.nuevoFlag = false;
    }

    actualizarTipoCambio(args: TipoCambioItem) {
        let impuestoExists: boolean = false;
        if (this.nuevoFlag == true) {
            this.data.forEach(tipoCambio => {
                if (tipoCambio.idDivisaOrigen == args.idDivisaOrigen) {
                    this.alerts.showAlert("La divisa ya fue agregada", "información");
                    impuestoExists = true;
                }
            })
            if (!impuestoExists) {
                args.idEmpresa = this.context.app.defaultCompany.identity;
                args.empresa = this.context.app.defaultCompany.nombre;
                this.data.push(args);
                this.tipCamCollection.refresh();
            }
        } else {
            this.data.forEach(tipoCambio => {
                if (tipoCambio.idDivisaOrigen == args.idDivisaOrigen) {
                    if (tipoCambio.state == undefined) {
                        tipoCambio.state = 'CHANGED';
                    }
                    tipoCambio.precioCompra = args.precioCompra;
                    tipoCambio.precioVenta = args.precioVenta;
                    tipoCambio.precioOtro = args.precioOtro;
                }
            })
            this.tipCamCollection.refresh();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}