﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { CategoriaArticulosInfo, CategoriaArticulosItem } from '../models/categoria-articulos';
import { CategoriaArticulosService } from '../services/categoria-articulos.service';

@Component({
    selector: 'accounting-articulo-categoria-fiscal-selector',
    templateUrl: './articulo-categoria-fiscal-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ArticuloCategoriaFiscalComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ArticuloCategoriaFiscalComponent),
        multi: true
    }]
})
export class ArticuloCategoriaFiscalComponent extends CatalogSelector {
    categoriafiscal: any[];
    selectedItem: CategoriaArticulosInfo = new CategoriaArticulosInfo;

    constructor(protected context: Context, private categoriaArticulosService: CategoriaArticulosService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findcategoriafiscal(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.categoriaArticulosService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + '-' + info.nombre;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.categoriaArticulosService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.categoriafiscal = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findcategoriafiscal(keyword: string): Observable<CatalogItem[]> {
        return this.categoriaArticulosService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

