﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
import { DocumentSelector } from 'framework3/search/models/document-selector'; 
//
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';
//

import { NotaRelaciones } from '../models/nota-relaciones'

@Component({
    selector: 'accounting-comprobante-fiscal-selector',
    templateUrl: './comprobante-fiscal-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ComprobanteFiscalSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ComprobanteFiscalSelectorComponent),
        multi: true
    }]

})
export class ComprobanteFiscalSelectorComponent extends DocumentSelector {
    comprobanteFiscal: any[];
    selectedItem: ComprobanteFiscalInfo = new ComprobanteFiscalInfo;
    //selectedItem: NotaRelaciones = new NotaRelaciones;
    @Input() idContribuyente: number = 0;
    //
    //@Output() selectedItemChanged: EventEmitter<NotaRelaciones> = new EventEmitter<NotaRelaciones>();
    @Output() selectedItemChanged: EventEmitter<ComprobanteFiscalInfo> = new EventEmitter<ComprobanteFiscalInfo>();

    constructor(protected context: Context, private comprobanteFiscalService: ComprobanteFiscalService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findComprobantesFiscales(keyword)));
    }

    loadItem(itemID: string) {
        if (itemID != '' && itemID != '0') {

            var sRet: string[] = itemID.split('/');

            this.selectedItem = null;
            /*
            this.selectedItem = new NotaRelaciones;

            this.selectedItem.serie = sRet[0];
            this.selectedItem.folio = +sRet[1];
            this.selectedItem.uuid = sRet[2];
            this.selectedItem.total = +sRet[3];
            */
            
            this.comprobanteFiscalService.getDetail(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.serie + '-' + info.folio;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
            
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let condiciones: SearchCondition = { field: "idContribuyente", operator: "=", valueType: "int", value: this.idContribuyente.toString() };
        request.conditions.push(condiciones);


        this.comprobanteFiscalService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.comprobanteFiscal = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findComprobantesFiscales(keyword: string): Observable<ComprobanteFiscalItem[] > {
        return this.comprobanteFiscalService.searchKeyword(this.context.app.defaultSite.identity, keyword, this.idContribuyente);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}