﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { ContribuyenteInfo, ContribuyenteItem } from '../models/contribuyente';
import { ContribuyenteService } from '../services/contribuyente.service';

@Component({
    selector: 'contabilidad-contribuyente-selector',
    templateUrl: './contribuyente-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ContribuyenteSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ContribuyenteSelectorComponent),
        multi: true
    }]
})
export class ContribuyenteSelectorComponent extends CatalogSelector {
    contribuyentes: any[];
    selectedItem: ContribuyenteInfo = null;

    @Output() itemSelected: EventEmitter<ContribuyenteInfo> = new EventEmitter<ContribuyenteInfo>();

    constructor(protected context: Context, private contribuyenteService: ContribuyenteService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findContribuyente(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.contribuyenteService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.razonSocial;
                    this.itemSelected.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.contribuyenteService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.contribuyentes = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findContribuyente(keyword: string): Observable<CatalogItem[]> {
        return this.contribuyenteService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

