﻿
import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search';
//
import { DivisaInfo, DivisaItem } from '../models/divisa';
import { DivisaService } from '../services/divisa.service';

@Component({
    selector: 'accounting-divisas-selector',
    templateUrl: './divisas-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => DivisasSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => DivisasSelectorComponent),
        multi: true
    }]
})
export class DivisasSelectorComponent extends CatalogSelector {
    divisas: DivisaItem[];
    selectedItem: DivisaInfo = null;
    //Outputs
    @Output() selectedItemChanged: EventEmitter<DivisaInfo> = new EventEmitter<DivisaInfo>();

    constructor(protected context: Context, private service: DivisaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findKeyword(keyword)));
    }

    ngAfterViewInit() {
        if (this.selectedItem == null) {
            this.getDivisaDefault();
        }
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + '-' + info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.getDivisaDefault();
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();
        //Agregar aqui filtros
        //let condicion: SearchCondition = { field: "campo", operator: "=", valueType: "int", value: this.inputMember.toString() };
        //request.conditions.push(condicion);
        //
        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.divisas = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findKeyword(keyword: string): Observable<CatalogItem[]> {
        return this.service.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

    getDivisaDefault() {
        this.service.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.selectedItem = r;
                this.displayValue = r.code + '-' + r.nombre;
                this.selectedItemChanged.emit(this.selectedItem);

            })
            .catch(error => this.handleError(error));

    }
}
