﻿//Angular
import { Component, forwardRef, Input, Output, EventEmitter } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
//Framework 3
import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { CatalogSelector, SearchRequest, SearchCondition } from 'framework3/search';
//Reactive
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import { map } from 'rxjs/operators';
//Features
import { EsquemaInfo, EsquemaItem } from '../models/esquema';
import { EsquemaService } from '../services/esquema.service';

@Component({
    selector: 'accounting-esquema-selector',
    templateUrl: 'esquema-selector.component.html',
    providers: [
        {
            provide: BaseComponent,
            useExisting: forwardRef(() => EsquemaSelectorComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => EsquemaSelectorComponent),
            multi: true
        }
    ]
})

export class EsquemaSelectorComponent extends CatalogSelector {
    //variables
    items: any[];
    selectedItem: EsquemaInfo = new EsquemaInfo();

    //Outputs
    @Output() selectedItemChanged = new EventEmitter<EsquemaInfo>();

    constructor(protected context: Context, private service: EsquemaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findItems(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + '-' + info.nombre;
                    this.selectedItemChanged.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();
        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();

                this.items = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(Error));
    }

    findItems(keyword: string): Observable<CatalogItem[]> {
        return this.service.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }
}