﻿//Angular
import { Component, forwardRef, Input, Output, EventEmitter } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
//Framework 3
import { BaseComponent, Context } from 'framework3/core';
import { DocumentSelector, SearchRequest, SearchCondition } from 'framework3/search';

//
import * as moment from 'moment';
//Reactive
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import { map } from 'rxjs/operators';
//Features
import { PeriodoItem, PeriodoInfo } from '../models/periodo';
import { PeriodoService } from '../services/periodo.service';
import { formatDate } from '@angular/common';

@Component({
    selector: 'accounting-periodos-selector',
    templateUrl: 'periodos-selector.component.html',
    providers: [
        {
            provide: BaseComponent,
            useExisting: forwardRef(() => PeriodosSelectorComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => PeriodosSelectorComponent),
            multi: true
        }
    ]
})

export class PeriodosSelectorComponent extends DocumentSelector {
    //variables
    periodos: any[];
    selectedItem: PeriodoInfo = new PeriodoInfo();

    //Outputs
    @Output() selectedItemChanged = new EventEmitter<PeriodoInfo>();
    //inputs
    @Input() idEjercicioTipo: number = 0;

    constructor(protected context: Context, private periodoService: PeriodoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findPeriodos(keyword)));           
    }

    //loadItem(itemID: number) {
    //    if (itemID > 0) {
    //        this.periodoService.getDetail(itemID.toString())
    //            .then(info => {
    //                this.selectedItem = info;
    //                console.log(JSON.stringify(info));
    //                this.displayValue =   formatDate(info.fechaIni, 'dd/MM/yyyy', 'es') + " - " + formatDate(info.fechaFin, 'dd/MM/yyyy', 'es')
    //                this.selectedItemChanged.emit(info);
    //            })
    //            .catch(error => this.handleError(error));
    //    }
    //    else {
    //        this.selectedItem = null;
    //        this.displayValue = "";
    //    }
    //}

    loadItem(itemID: string) {
        if (itemID != '') {
            this.periodoService.getDetail(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = formatDate(info.fechaIni, 'dd/MM/yyyy', 'es') + " - " + formatDate(info.fechaFin, 'dd/MM/yyyy', 'es')
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        request.conditions.filter(item => item.field == "fechaCreacion").forEach(fecha => {
            fecha.field = "FechaIni";

            //modificar esto desde el endPeriod cuando se actualice el framework
            if (fecha.operator = "<=")
                fecha.value = moment().add(90, 'day').toDate();
        });


        if (this.idEjercicioTipo && this.idEjercicioTipo > 0) {
            let condition: SearchCondition = { field: 'IdEjercicioTipo', operator: '=', valueType: 'int', value: this.idEjercicioTipo };
            request.conditions.push(condition);        
        }

        //let condiciones: SearchCondition = { field: "fechaIni", operator: "=", valueType: "date", value: this.f.toString() };
        //request.conditions.push(condiciones);

        this.periodoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.periodos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findPeriodos(keyword: string): Observable<PeriodoItem[]> {
        return this.periodoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}