﻿//Angular
import { Component, forwardRef, Input, Output, EventEmitter } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
//Framework 3
import { BaseComponent, Context } from 'framework3/core';
import { CatalogSelector, SearchRequest, SearchCondition } from 'framework3/search';
//Reactive
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import { map } from 'rxjs/operators';
//Features
import { RetencionItem, RetencionInfo } from '../models/retencion';
import { RetencionService } from '../services/retencion.service';

@Component({
    selector: 'accounting-retenciones-selector',
    templateUrl: 'retenciones-selector.component.html',
    providers: [
        {
            provide: BaseComponent,
            useExisting: forwardRef(() => RetencionesSelectorComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => RetencionesSelectorComponent),
            multi: true
        }
    ]
})

export class RetencionesSelectorComponent extends CatalogSelector {
    //variables
    retenciones: any[];
    selectedItem: RetencionInfo = new RetencionInfo();

    //Inputs
    @Input() variable: boolean = true;
    @Input() local: boolean = true;

    //Outputs
    @Output() selectedItemChanged = new EventEmitter<RetencionInfo>();

    constructor(protected context: Context, private service: RetencionService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findRetenciones(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + '-' + info.nombre;
                    this.selectedItemChanged.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        let condicionVariable: SearchCondition = { field: "variable", operator: "=", valueType: "boolean", value: this.variable }
        let condicionLocal: SearchCondition = { field: "local", operator: "=", valueType: "boolean", value: this.local }
        request.conditions.push(condicionVariable);
        request.conditions.push(condicionLocal);

        this.context.app.showSpinner();        
        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();

                this.retenciones = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(Error));
    }

    findRetenciones(keyword: string): Observable<RetencionItem[]> {
        return this.service.searchKeywordRetenciones(this.context.app.defaultSite.identity, keyword)
            .pipe(map(retenciones => {
                return retenciones.filter(retencion => retencion.variable == this.variable || retencion.local == this.local);
            }));
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }
}