﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { CategoriaUnidadesMedidaInfo, CategoriaUnidadesMedidaItem } from '../models/categoria-unidades-medida';
import { CategoriaUnidadesMedidaService } from '../services/categoria-unidades-medida.service';

@Component({
    selector: 'accounting-unidades-categoria-fiscal-selector',
    templateUrl: './unidad-categoria-fiscal-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => UnidadMedidaCategoriaFiscalComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => UnidadMedidaCategoriaFiscalComponent),
        multi: true
    }]
})
export class UnidadMedidaCategoriaFiscalComponent extends CatalogSelector {
    categoriafiscal: any[];
    selectedItem: CategoriaUnidadesMedidaInfo = new CategoriaUnidadesMedidaInfo;

    constructor(protected context: Context, private categoriaUnidadesMedidaService: CategoriaUnidadesMedidaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findcategoriafiscal(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.categoriaUnidadesMedidaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + '-' + info.nombre;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.categoriaUnidadesMedidaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.categoriafiscal = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findcategoriafiscal(keyword: string): Observable<CatalogItem[]> {
        return this.categoriaUnidadesMedidaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

