﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
// Components
import { ComprobantesListComponent } from '../components/comprobantes-dialog.component'
//Locales
import { ComprobanteCancelacionInfo, ComprobanteCancelacionItem } from '../models/comprobante-cancelacion';
import { ComprobanteCancelacionDetail } from '../models/comprobante-cancelacion-detail';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal'

import { ComprobanteCancelacionService } from '../services/comprobante-cancelacion.service';
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service'

import * as moment from 'moment';
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';

@Component({
    selector: 'contabilidad-comprobante-cancelacion-editor',
    templateUrl: './comprobantes-cancelaciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ComprobantesCancelacionesEditor)
        },
        {
            provide: DocumentService, useExisting: ComprobanteCancelacionService
        }
    ]
})
export class ComprobantesCancelacionesEditor extends DocumentEditor {

    comprobanteCancelacion = new ComprobanteCancelacionInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('comprobantesList') comprobantesList: ModalDirective;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private ejercicioService: ComprobanteCancelacionService,
        private comprobanteFiscalSvc: ComprobanteFiscalService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.ejercicioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.comprobanteCancelacion = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.ejercicioService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.comprobanteCancelacion = doc;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.ejercicioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.comprobanteCancelacion = doc;
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        
        this.context.app.showSpinner();
        this.ejercicioService.update(this.comprobanteCancelacion)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.comprobanteCancelacion.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.comprobanteCancelacion = updatedDoc;
            })
            .catch(error => this.handleError(error));
        
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.ejercicioService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ejercicioService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.comprobanteCancelacion.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    ///////////// COMPROBANTES /////////////////////////

    onAddingDetail() {
        //this.comprobantesList.config.backdrop = false;
        this.comprobantesList.show();
    }


    onComprobanteSelected(comprobanteItem: ComprobanteFiscalItem) {
        this.comprobanteFiscalSvc.getDetail(comprobanteItem.id)
            .then(item => {
                //this.comprobante = item
                //this.comprobantesList.hide;
            })
            .catch(error => this.handleError(error));
    }



}