﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Locales
import { ComprobanteCancelacionInfo, ComprobanteCancelacionItem } from '../models/comprobante-cancelacion';
import { ComprobanteCancelacionDetail } from '../models/comprobante-cancelacion-detail';
import { ComprobanteCancelacionService } from '../services/comprobante-cancelacion.service';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';

@Component({
    selector: 'contabilidad-comprobantes-cancelaciones-manager',
    templateUrl: './comprobantes-cancelaciones.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ComprobantesCancelacionesManager)
        },
        {
            provide: DocumentService, useExisting: ComprobanteCancelacionService
        }
    ]
})
export class ComprobantesCancelacionesManager extends DocumentManager {

    comprobantesCancelaciones: ComprobanteCancelacionItem[];
    currentComprobante = new ComprobanteFiscalInfo();

    @ViewChild('viewDialog') viewDialog: ModalDirective;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute,
        private comprobanteCancelacionService: ComprobanteCancelacionService, private comprobanteFiscalService: ComprobanteFiscalService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.comprobanteCancelacionService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.comprobantesCancelaciones = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.comprobanteCancelacionService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.comprobanteCancelacionService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.comprobanteCancelacionService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'EVALUAR') {
            this.context.app.showSpinner();
            this.comprobanteCancelacionService.evaluarProceso(this.manager.currentDocument.id)
                .then(lote => {
                    this.context.app.hideSpinner();
                    this.refreshList();
                    this.showAlert("Se ha evaluado la solicitud de cancelación", "success");
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == 'OPEN') {
            this.context.app.showSpinner();
            var idDocument = this.manager.currentDocument.id;
            var idComprobante: string = "";
            this.comprobantesCancelaciones.forEach(function (item) {
                if (item.id == idDocument) {
                    idComprobante = item.idComprobante;
                }
            });
            this.comprobanteFiscalService.getDetail(idComprobante)
                .then(comprobante => {
                    this.currentComprobante = comprobante;
                    this.context.app.hideSpinner();
                    this.viewDialog.config.backdrop = false;
                    this.viewDialog.show();
                })
                .catch(error => this.handleError(error));

        }
    }
}

