﻿import { Component, EventEmitter, Input, Output, ViewChild, forwardRef } from '@angular/core';
import { Context, ProcessComponent, PolicyInfo, ProcesoService, ProcesoInfo } from 'framework3/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';

import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';

@Component({
    selector: 'accounting-comprobantes-loader',
    templateUrl: './comprobantes.loader.html',
})
export class ComprobantesLoaderComponent extends ProcessComponent {
    form: FormGroup;
    sentidos: { id: string, nombre: string }[] = [];
    sentido: string;
    rutaComprobantes: string;
    idLoader: number;

    cargando: boolean = false;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, private procesoSvc: ProcesoService) {
        super(context, router, route);
        this.form = formBuilder.group({
            'sentido': ['', Validators.required],
        });

        this.sentidos =
            [
                { "id": "E", "nombre":"Emitidos" },
                { "id": "R", "nombre": "Recibidos" },
                { "id": "ER", "nombre": "Ambos" }
            ];

    }

    ngOnInit() {
        this.procesoSvc.getDetailByCode(this.context.app.defaultCompany.idBranch, 'ACC_CFDIS')
            .then(proc => {
                this.rutaComprobantes = proc.url;
                this.procesoSvc.getDetailByCode(this.context.app.defaultCompany.idBranch, 'ACC_CFLOADER')
                    .then(procc => {
                        this.idLoader = procc.identity;
                    })
                    .catch(error => this.handleError(error));
            })
            .catch(error => this.handleError(error));

    }

    selectedSentido(item: any) {
        this.sentido = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            if (this.sentido == 'ER')
            {
                this.router.navigate([this.rutaComprobantes + '/' + this.idLoader.toString()]);
            }
                else
            {
                this.router.navigate([this.rutaComprobantes + '/' + this.idLoader.toString() + '/' + this.sentido]);
            }
            
        }
    }


    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.cargando = false;
        this.alerts.showAlert(message, 'danger');
    }


    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

}