﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { ModalDirective, idLocale } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem, FileLink } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { LoteImpresionInfo, LoteImpresionService } from 'framework3/printing';

//Locales
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';
import { ComprobanteFiscalDetail } from '../models/comprobante-fiscal-detail';
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';
import { MenuItem, DownloadPopupComponent } from 'framework3/controls';
import { timer } from 'rxjs/observable/timer';
import { JobService, JobSourceService } from 'framework3/queue';
import { log } from 'util';

@Component({
    selector: 'contabilidad-comprobantes-manager',
    templateUrl: 'comprobantes.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ComprobantesManager)
        },
        {
            provide: DocumentService, useExisting: ComprobanteFiscalService
        }
    ]
})
export class ComprobantesManager extends DocumentManager {
    comprobantes: ComprobanteFiscalItem[];
    currentComprobante = new ComprobanteFiscalInfo();
    comprobantesDownLoad: string[] = [];
    comprobantesExport: string[] = [];
    correo: string = '';
    intervalId: any;
    intervalExportacion:any
    sentido: string = '';
    allowCancel: boolean = false;

    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;
    @ViewChild('viewDialog') viewDialog: ModalDirective;
    @ViewChild('sendMailDialog') sendDialog: ModalDirective;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private comprobanteFiscalService: ComprobanteFiscalService, private loteImpresionService: LoteImpresionService,private jobService:JobService) {
        super(context, router, route);

    }

    onConfigureDocument() {
        var thisRoot = this;
        this.refreshList();
        this.applyPolicies();
        this.process.politicas.forEach(procesoAcceso => {
            if (procesoAcceso.nombre == "allowCancel")
                thisRoot.allowCancel = (procesoAcceso.valor?true:false);
        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.sentido = params['sentido'];
        });
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.sentido != undefined) {
            let condiciones: SearchCondition = { field: "idSentido", operator: "=", valueType: "string", value: this.sentido };
            request.conditions.push(condiciones);
        }

        this.comprobanteFiscalService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.comprobantes = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        
    }

    onViewingItem(document: DocumentItem) {
        this.context.app.showSpinner();
        this.comprobanteFiscalService.getDetail(document.id)
            .then(comprobante => {
                this.currentComprobante = comprobante;
                this.context.app.hideSpinner();
                this.viewDialog.config.backdrop = false;
                this.viewDialog.show();
            })
            .catch(error => this.handleError(error));
    }

    onEditingItem(document: DocumentItem) {
        
    }

    onDeletingItem(document: DocumentItem) {

    }

    onCloningItem(document: DocumentItem) {
        
    }

    onExportingItem(document: DocumentItem) {
        this.comprobanteFiscalService.export(document.id)
            .then(_ => {
                this.refreshList();
                this.showAlert('El proceso de exportación ha terminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {

    }

    onApplyingPreviousStep(document: DocumentItem) {

    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'EXPORTALL') {
            this.context.app.showSpinner();
            var i = 0;
            for (i = 0; i <= this.manager.baseList.documentsView.totalItemCount - 1; i++) {
                this.comprobantesExport[i] = this.manager.baseList.documentsView._view[i].id;
            }

            this.comprobanteFiscalService.getLoteExportacion(this.comprobantesExport)
                .then(loteId => {
                    console.log(loteId);
                    if (loteId > 0) {
                        this.showAlert("Se a generado la tarea de exportacion", "success");
                        this.intervalExportacion = setInterval(_ => this.checaStatusExportacion(loteId), 5000);
                    } else {
                        this.context.app.hideSpinner();
                        this.showAlert("Se a generado un error al encolar la tarea de exportacion", "error");
                    }

                })
                .catch(error => this.handleError(error));
        }

        if (option.name == 'TIMBRAR') {
            this.context.app.showSpinner();
            this.comprobanteFiscalService.timbrar(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    this.showAlert("El comprobante se ha timbrado con exito", "success");
                })
                .catch(error => this.handleError(error));
        }

        if (option.name == 'CANCEL') {
            this.context.app.showSpinner();
            this.comprobanteFiscalService.cancelar(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    this.showAlert("El comprobante se ha cancelado con exito", "success");
                })
                .catch(error => this.handleError(error));
        }

        if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.comprobanteFiscalService.generarPdfTemporal(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + file.uri, '_blank');
                })
                .catch(error => this.handleError(error));
        }

        if (option.name == 'REFRESH') {
            this.context.app.showSpinner();
            this.comprobanteFiscalService.regenerarPdfTemporal(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + file.uri, '_blank');
                })
                .catch(error => this.handleError(error));
        }

        if (option.name == "DOWNLOAD") {
            this.context.app.showSpinner();
            //Generar PDF
            this.comprobanteFiscalService.generarPdfTemporal(this.manager.currentDocument.id)
                .then(pdf => {
                    //Descargar xml
                    this.comprobanteFiscalService.getXmlFile(this.manager.currentDocument.id)
                        .then(xml => {
                            this.context.app.hideSpinner();
                            let links: FileLink[] = [];
                            links.push(pdf);
                            links.push(xml);
                            this.downloadPopup.show(links);
                        })
                        .catch(error => this.handleError(error));

                })
                .catch(error => this.handleError(error));
        }

        if (option.name == "DOWNLOAD-ALL") {

            this.comprobantesDownLoad = [];

            this.context.app.showSpinner();
            var i = 0;
            for (i = 0; i <= this.manager.baseList.documentsView.totalItemCount - 1; i++) {
                this.comprobantesDownLoad[i] = this.manager.baseList.documentsView._view[i].id;
            }

            this.comprobanteFiscalService.getLoteDescarga(this.comprobantesDownLoad)
                .then(lote => {
                    this.intervalId = setInterval(_ => this.checaStatus(lote.id), 5000);
                })
                .catch(error => this.handleError(error));
            
        }


        if (option.name == 'SEND') {
            let comprobante: ComprobanteFiscalItem = this.manager.currentDocument as ComprobanteFiscalItem;
            this.correo = comprobante.correo;
            this.sendDialog.config.backdrop = false;
            this.sendDialog.show();
        }

        if (option.name == "VALIDATE") {
            this.comprobanteFiscalService.validarComprobanteSAT(this.manager.currentDocument.id)
                .then(result => {
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }

    }

    checaStatusExportacion(idLote) {
        this.comprobanteFiscalService.getLoteExportacionStatus(idLote)
            .then(result => {
                if (result != undefined) {
                    if (result.completed) {
                        if (result.errorMessage.length > 0) {
                            clearInterval(this.intervalExportacion);
                            this.context.app.hideSpinner();
                            this.showAlert("Error  al exportar " + result.errorMessage.length, "error");
                        } else {
                            clearInterval(this.intervalExportacion);
                            this.context.app.hideSpinner();
                            this.showAlert("Exportación terminada", "success");
                        }
                    }
                }
            })
            .catch(error => this.handleError(error))
    }

    checaStatus(idLote) {
        this.context.app.hideSpinner();
        this.loteImpresionService.getStatus(idLote)
            .then(result => {
                //alert(result);
                if (result == 'C') {
                    this.loteImpresionService.getFiles(idLote)
                        .then(files => {
                            let links: FileLink[] = [];
                            let PDF: FileLink = new FileLink;
                            let XML: FileLink = new FileLink;
                            PDF.uri = files[0];
                            XML.uri = files[1];
                            PDF.fileName = files[0];
                            XML.fileName = files[1];
                            links.push(PDF);
                            links.push(XML);
                            this.downloadPopup.show(links);
                            clearInterval(this.intervalId);
                        })
                }
                if (result == 'R') {
                    this.showAlert('Empaquetando archivos de comprobantes, el proceso puede tardar varios minutos', 'success');
                }

                if (result == 'E') {
                    clearInterval(this.intervalId);
                    this.showAlert('Error al obtener el paquete de comprobantes', 'success');
                }

            })
            .catch(error => this.handleError(error))

    }

    sendMail() {
        this.context.app.showSpinner();
        this.comprobanteFiscalService.enviar(this.manager.currentDocument.id, this.correo, 'Comprobante fiscal')
            .then(file => {
                this.context.app.hideSpinner();
                this.showAlert("El comprobante se ha enviado con exito", "success");
            })
            .catch(error => this.handleError(error));
    }

}

