﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { EjercicioInfo, EjercicioItem } from '../models/ejercicio';
import { EjercicioDetail } from '../models/ejercicio-detail';
import { EjercicioService } from '../services/ejercicio.service';
import * as moment from 'moment';

@Component({
    selector: 'contabilidad-ejercicios-manager',
    templateUrl: 'ejercicios.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EjerciciosManager)
        },
        {
            provide: DocumentService, useExisting: EjercicioService
        }
    ]
})
export class EjerciciosManager extends DocumentManager {
    ejercicios: EjercicioItem[];

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private ejercicioService: EjercicioService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.ejercicioService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.ejercicios = response.rows;
                this.ejercicios.forEach(ejercicio => {
                    if (ejercicio.fechaApertura != undefined) ejercicio.fechaApertura = moment(ejercicio.fechaApertura).toDate()
                    if (ejercicio.fechaCierre != undefined) ejercicio.fechaCierre = moment(ejercicio.fechaCierre).toDate()
                })
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.ejercicioService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.ejercicioService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ejercicioService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

}