﻿import { Component, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

import { BaseComponent, Context, FileLink } from "framework3/core";
import { AlertsBarComponent } from "framework3/controls"

import { ComprobanteFiscalService } from '../../services/comprobante-fiscal.service';
import { ModalDirective } from 'ngx-bootstrap';
//
import { ComprobanteFiscalInfo } from '../../models/comprobante-fiscal';

@Component({
    selector: "accounting-comprobante-search",
    templateUrl: './comprobante-search.component.html'
})
export class ComprobanteSearch {

    private form: FormGroup;

    serie: string = "";
    folio: string = "";
    consultando: boolean = false;
    links: FileLink[] = [];

    comprobante: ComprobanteFiscalInfo;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild("dlgSearch") dlgSearch: ModalDirective;

    constructor(private context: Context, private formBuilder: FormBuilder, private comprobanteFiscalService: ComprobanteFiscalService) {
        this.form = formBuilder.group({
            'serie': [''],
            'folio': ['']
        });
    }

    onClickSearchComprobante() {
        this.links.length = 0;
        this.serie = "";
        this.folio = "";
        this.dlgSearch.config.backdrop = false;
        this.dlgSearch.show();
    }

    onClickGetComprobante() {
        this.consultando = true;
        this.comprobanteFiscalService.getComprobanteBySerieFolio(this.serie, this.folio, this.context.app.defaultCompany.identity)
            .then(result => {
                this.consultando = false;
                this.openPDF(result.identity);
            }).catch(error => {
                this.handleError(error);
            });
    }

    openPDF(idComprobante: string) {
        this.links.length = 0;
        this.consultando = true;
        //Generar PDF
        this.comprobanteFiscalService.generarPdfTemporal(idComprobante)
            .then(pdf => {
                //Descargar xml
                this.comprobanteFiscalService.getXmlFile(idComprobante)
                    .then(xml => {
                        this.consultando = false;
                        this.links.push(pdf);
                        this.links.push(xml);
                        window.open('/file-server/' + pdf.uri, '_blank');
                    })
                    .catch(error => this.handleError(error));
            })
            .catch(error => this.handleError(error));
    }

    hideListDialog() {
        this.dlgSearch.hide();
    }

    handleError(error: any) {
        this.consultando = false;
        console.log(error);
        this.context.app.hideSpinner();
        var errorMessage = this.context.getExceptionMessage(error);
        this.alerts.showAlert(errorMessage, 'danger');
    }

}