﻿import { Component, ViewChild ,Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
//Locales
import { NotaDetail } from '../../models/nota-detail';
import { NotaConceptoSelectorComponent, ImpuestoDetailInfo } from '../../../accounting';
import { NotaConceptoInfo, NotaConceptoItem } from '../../models/nota-concepto';



@Component({
    selector: 'contabilidad-nota-detail-editor',
    templateUrl: './notas-detail.editor.html'
})
export class NotaDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new NotaDetail();
    @ViewChild('selectorNotaConcepto') selectorNotaConcepto: NotaConceptoSelectorComponent;
    notaConcepto: NotaConceptoInfo = new NotaConceptoInfo();
    //Objeto original que se editará
    @Input() detail: NotaDetail = new NotaDetail();
    
    impuestoCalculado: number; 

    constructor(protected context: Context, private builder: FormBuilder) {
        super(context);

        this.form = builder.group({
            'idNotaConcepto': [0, Validators.required],
            'importe': [0, Validators.required],
            'concepto': [''],
        });
    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////

    onSelectedItemChanged(concepto: NotaConceptoInfo) {
        if (concepto) {
            this.notaConcepto = concepto;
        }
    }


    onCreatingItem() {
        this.detailCopy = this.detail;
        this.form.reset({ idNotaConcepto: 0 });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: NotaDetail ) {
        this.detailCopy = <NotaDetail>this.copyItem(this.detail);
    }

    onUpdatingItem(detail: NotaDetail ) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();

            ////Calculando los impuestos////
            this.notaConcepto.impuestos.forEach(det => {
                var nuevoImpuesto: ImpuestoDetailInfo = new ImpuestoDetailInfo;
                nuevoImpuesto.importe = detail.importe;//detail.importe * det.tasaOCuota;
                nuevoImpuesto.tasaOCuota = det.tasaOCuota;
                nuevoImpuesto.tipoAplicacion = det.tipoAplicacion;
                nuevoImpuesto.tipoFactor = det.impuesto.factor;
                nuevoImpuesto.base = detail.importe / (1 + det.tasaOCuota);
                nuevoImpuesto.impuesto = det.impuesto;
                
                if (det.impuesto.factor == "T")
                {
                    this.impuestoCalculado = detail.importe * det.tasaOCuota;
                }
                else
                {
                    this.impuestoCalculado = det.tasaOCuota;
                }
                this.detailCopy.impuestos.push(nuevoImpuesto);
            });
            ////////////////////////////////

        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
    }
}
