﻿//Frameworks

import { FormBuilder, FormGroup, Validators } from '@angular/forms';
//Bootstrap
//Features
import { Component, Input, Output, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import * as moment from 'moment';
import { CollectionView } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { AztecaValidators } from 'framework3/controls';
import { Context } from 'framework3/core';
import { TracingModule } from 'framework3/tracing';
//
import { PolizaService } from '../../services/poliza.service';
import { AsociacionItem } from '../../models/asociacion';
import { ComprobanteFiscalItem } from '../../models/comprobante-fiscal';

import { ComprobanteFiscalService } from '../../services/comprobante-fiscal.service';


import { ComprobantesListComponent } from '../../components/comprobantes-dialog.component'

import * as _ from 'lodash';

@Component({
    selector: 'accounting-polizas-asociaciones-component',
    templateUrl: './polizas-asociaciones.component.html'
})
export class PolizasAsociacionesComponent {
    editorForm: FormGroup;
    logCollection = new CollectionView();
    currentEntry: AsociacionItem = new AsociacionItem();
    //
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('detailsDialog') detailsDialog: ModalDirective;
    @ViewChild('dlgAsociacionesList') dlgAsociacionesList: ModalDirective;
    @ViewChild('comprobantesList') comprobantesList: ComprobantesListComponent;

    @Input() idPoliza: string;
    @Input() fecha: Date;
    @Input() idCuenta: number;
    @Input() consecutivo: number;
    addEnabled: boolean=true;

    constructor(private formBuilder: FormBuilder, private context: Context, private router: Router, private route: ActivatedRoute,
        private polizaService: PolizaService, private comprobanteFiscalService: ComprobanteFiscalService) {
        this.editorForm = formBuilder.group({
            
        });
    }

    /////////////////////// METODOS ////////////////////////////////////////////////

    loadLog() {
        this.context.app.showSpinner();
        this.polizaService.loadAsociaciones(this.idPoliza)
            .then(log => {

                if (this.idCuenta > 0) {
                    this.logCollection.sourceCollection = log.filter(x=> x.idCuenta==this.idCuenta && x.consecutivo == this.consecutivo);
                }
                else {
                    var result = _(log).groupBy('uuid').transform(function (result, current) {
                        result.push({
                            idComprobante:current[0].idComprobante,
                            uuid: current[0].uuid,
                            serie: current[0].serie,
                            folio: current[0].folio,
                            rfc: current[0].rfc,
                            razonSocial: current[0].razonSocial,
                            subTotal: current[0].subTotal,
                            descuentos: current[0].descuentos,
                            impTrasladados: current[0].impTrasladados,
                            impRetenidos: current[0].impRetenidos,
                            total: current[0].total,
                            //pets: _.map(current, 'pet')
                        });
                    }, []).value();
                    this.logCollection.sourceCollection = result// log;
                }                
                
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    /////////////////////// HANDLERS ///////////////////////////////////////////////
    mainToolbarClick(button: ToolbarButton) {
        if (button.name == 'REFRESH') {
            this.loadLog();
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.comprobantesList.show();
        }
    }

    onRowView(item: AsociacionItem) {
        this.currentEntry = item;
        //this.detailsDialog.config.backdrop = false;
        //this.detailsDialog.show();

        //if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.comprobanteFiscalService.generarPdfTemporal(item.idComprobante)
                .then(file => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + file.uri, '_blank');
                })
                .catch(error => this.handleError(error));
        //}

    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    show(idPoliza:string, fecha:Date, idCuenta:number, consecutivo:number) {
        this.logCollection = new CollectionView();
        this.dlgAsociacionesList.config.backdrop = false;
        this.idPoliza = idPoliza;
        this.fecha = fecha;
        this.idCuenta = idCuenta;
        this.consecutivo = consecutivo;
        this.dlgAsociacionesList.show();
        this.loadLog();
    }

    onComprobanteSelected(comprobanteItem: ComprobanteFiscalItem) {
        let comprobanteItemBak: ComprobanteFiscalItem = this.comprobantesList.comprobantes.find(item => item.id == comprobanteItem.id);
        if (comprobanteItemBak) {
            this.polizaService.addAsociacion(this.idPoliza,this.fecha, this.idCuenta, this.consecutivo, comprobanteItemBak.uuid, 0, 0)
                .then(log => {
                    this.loadLog();
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }

    }

    onMenuSeleccionadosClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Eliminar
            this.polizaService.removeAsociacion(this.idPoliza, this.fecha, this.idCuenta, this.consecutivo, this.logCollection.currentItem.uuid)
                .then(log => {
                    //this.loadLog();
                    this.logCollection.remove(this.logCollection.currentItem);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));   
            
        }
        
    }


}