﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
//Locales
import { PolizaService } from '../../services/poliza.service';
import { PolizaDetail } from '../../models/poliza-detail';
import { PolizaInfo } from '../../models/poliza';
import { PolizaPerfilItem } from '../../models/poliza-perfil';

@Component({
    selector: 'accounting-polizas-by-lote',
    templateUrl: './polizas-by-lote.editor.html'
})
export class PolizasByLoteEditor extends ProcessComponent {
    form: FormGroup;
    menuItems: MenuItem[] = [];

    polizaPerfilCurrent: PolizaPerfilItem = new PolizaPerfilItem();
    idPolizaPerfil: number;

    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    perfiles: PolizaPerfilItem[] = [];   
    poliza = new PolizaInfo();
    cargos: number;
    abonos: number;
    //currentMovimiento = new PolizaDetail();
    rango: boolean = true;
    cargando: boolean = false;
    //
    @ViewChild('viewDialog') viewDialog: ModalDirective;
    @ViewChild(DocumentListComponent) documentList: DocumentListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder,
        private polizaService: PolizaService) {
        super(context, router, route);
        //Crear formulario de parámetros
        this.form = formBuilder.group({
            'fechaInicial': [new Date()],
            'fechaFinal': [new Date()],
            'perfil': [0, Validators.required],
            'abonos': [0],
            'cargos': [0],
            'folio': [0]
        });
    }

    ngOnInit() {
        //this.createMenu();
        this.loadPerfiles();
    }

    ngAfterViewInit() {
        this.fechaInicial = moment().toDate();
        this.fechaFinal = moment().add(1, 'day').toDate();
        //setTimeout(_ => this.refreshList());
    }
        
    /////////////////////// METODOS ////////////////////////////////////

    loadPerfiles() {
        this.polizaService.loadPerfilesByLote(this.context.app.defaultSite.identity, this.context.session.user.identity)
            .then(response => {
                this.perfiles = response;
            })
            .catch(error => this.handleError(error));
    }

    onCargarPoliza() {
        this.context.app.showSpinner();
        setTimeout(_ => this.resetScreen());
        this.fechaInicial.setHours(0);
        this.fechaFinal.setHours(0);
        this.fechaInicial.setMinutes(0);
        this.fechaFinal.setMinutes(0);
        this.fechaInicial.setSeconds(0);
        this.fechaFinal.setSeconds(0);
        this.cargando = true;

        if (this.polizaPerfilCurrent.rango == "D") {
            this.fechaFinal = this.fechaInicial;
            setTimeout(_ => this.getPolizaExists());
        }
        else
            setTimeout(_ => this.loadPoliza());
    }

    onCargarMapaPoliza() {
        this.context.app.showSpinner();
        this.polizaService.loadContabilizacionByMapa(this.context.app.defaultSite.identity, this.idPolizaPerfil, this.fechaInicial)
            .then(response => {
                this.context.app.hideSpinner();
                this.poliza = response;
                this.cargando = false;
                this.cargos = Math.round(this.poliza.cargos * 100) / 100;
                this.abonos = Math.round(this.poliza.abonos * 100) / 100;
                this.alerts.showAlert('Se ha cargado la informacion', 'success');
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error)
            });
    }

    getPolizaExists() {
        this.context.app.showSpinner();
        //Verificamos si existe una poliza de este perfil en esta fecha 
        this.polizaService.getPolizaByPerfil(this.context.app.defaultSite.identity, this.idPolizaPerfil, this.fechaInicial)
            .then(response => {                
                this.poliza = response;
                if (this.poliza.identity.trim() == '')
                    setTimeout(_ => this.loadPoliza());
                else {
                    this.context.app.hideSpinner();
                    this.cargando = false;
                    this.cargos = Math.round(this.poliza.cargos * 100) / 100 ;
                    this.abonos = Math.round(this.poliza.abonos * 100) / 100;
                }
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error)
            });
    }

    loadPoliza()
    {
        this.context.app.showSpinner();
        this.polizaService.contabilizarByLote(this.context.app.defaultSite.identity, this.idPolizaPerfil, this.fechaInicial, this.fechaFinal)
            .then(response => {
                this.context.app.hideSpinner();
                this.poliza = response;
                this.cargando = false;
                this.cargos = Math.round(this.poliza.cargos * 100) / 100;
                this.abonos = Math.round(this.poliza.abonos * 100) / 100;
                this.alerts.showAlert('Se ha cargado la informacion', 'success');
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error)
            });
    }


    savePoliza() {
        this.context.app.showSpinner();
        if (this.poliza.idPolizaOrigen == 0) this.poliza.idPolizaOrigen = 1 //Manual

        this.polizaService.update(this.poliza)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('La información ha sido actualizada con éxito', 'success');                
                this.poliza = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

   

    //createMenu() {
    //    //Agregar opciones estándar del catálogo
    //    let mnuView: MenuItem = { icon: "glyphicon glyphicon-folder-open", text: "Ver", name: "OPEN", smallText: "ver resumen del comprobante" };
    //    let mnuPdf: MenuItem = { icon: "glyphicon glyphicon-download-alt", text: "Descargar PDF", name: "PDF", smallText: "descargar en formato PDF" };
    //    let mnuXml: MenuItem = { icon: "glyphicon glyphicon-download-alt", text: "Descargar XML", name: "XML", smallText: "descargar en formato XML" };

    //    this.menuItems.push(mnuView, mnuPdf, mnuXml);
    //    this.menuItems.forEach(item => {
    //        if (item.enabled == undefined) item.enabled = true;
    //    });

    //}
    

    /////////////////////// HANDLERS //////////////////////////////////

    onMainToolbar(button: ToolbarButton) {
        if (button.name == 'SAVE') {
            this.savePoliza();
            button.enabled = false;
        }
    }

    onPolizaPerfilChanged(idPolizaPerfil:number) {
        this.polizaPerfilCurrent = this.perfiles.find(perfil => perfil.id == idPolizaPerfil);
        this.idPolizaPerfil = idPolizaPerfil;

        if (this.polizaPerfilCurrent.rango == "D") {
            this.rango = false;           
        }

        setTimeout(_=>this.resetScreen());
    }

    fechaIniChange(arg: any) {
        setTimeout(_ => this.resetScreen());
    }

    resetScreen() {
        this.poliza = new PolizaInfo();
        this.cargos = 0;
        this.abonos = 0;
    }


   


    //optionClick(option: MenuItem) {
    //    if (option.name == 'OPEN') {
    //        //this.viewComprobante();
    //    }
    //    if (option.name == 'PDF') {

    //    }
    //    if (option.name == 'XML') {

    //    }
    //}

    onValidateMenu() {

    }

    currentItemChangedHandler() {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.cargando = false;
        this.alerts.showAlert(message, 'danger');        
    }

    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }


}
