﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
//Locales
import { AzComboboxComponent } from 'framework3/controls';
import { AztecaValidators } from 'framework3/controls';

import { PolizaDetail } from '../../models/poliza-detail';
import { CuentaSelectorComponent } from '../../components/cuenta-selector.component';
import { CuentaInfo } from '../../models/cuenta';
import { SegmentoNegocioItem } from '../../models/segmento-negocio';
import { SegmentoNegocioService } from '../../services/segmento-negocio.service';

@Component({
    selector: 'contabilidad-poliza-detail-editor',
    templateUrl: './polizas-detail.editor.html'
})
export class PolizaDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new PolizaDetail();
    //Objeto original que se editará
    @Input() detail: PolizaDetail = new PolizaDetail();
    afectadores: { id: string, nombre: string }[] = [];
    segmentos: SegmentoNegocioItem[] = [];
    
    @ViewChild('cboSegmento') cboSegmento: AzComboboxComponent;
    @ViewChild('selCuenta') selCuenta: CuentaSelectorComponent;

    constructor(protected context: Context, private builder: FormBuilder, private segmentoNegocioService: SegmentoNegocioService) {
        super(context);

        this.form = builder.group({
            'cuenta': [0, AztecaValidators.selected],
            'afectador': ['', Validators.required],
            'importe': [0, Validators.required],
            'segmento': [0],           
        });

        this.afectadores =
            [
                { "id": "C", "nombre": "Cargo" },
                { "id": "A", "nombre": "Abono" }
            ];

       
    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////
    ngOnInit() {
        //this.loadEjercicioActual();
        this.loadSegmentos();
    }

    onCreatingItem() {
        this.detailCopy = this.detail;
        this.form.reset({
            cuenta: 0,
            importe: 0,
            segmento: 0, 
            afectador: '', 
        });
        this.selCuenta.clear();
        this.form.markAsPristine();
    }

    onLoadingItem(detail: PolizaDetail ) {
        this.detailCopy = <PolizaDetail >this.copyItem(this.detail);
    }

    onUpdatingItem(detail: PolizaDetail ) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
    }


    selectedCuenta(cuenta:CuentaInfo) {
        if (cuenta != undefined) {
            this.detailCopy.cuentaText = cuenta.code;
            this.detailCopy.conceptoText = cuenta.nombre;
        }
    }

    selectedAfectador(afectador) {
        if (afectador != undefined) {
            if (afectador == 'C')
                this.detailCopy.afectadorText = "Cargo";
            else
                this.detailCopy.afectadorText = "Abono";
        }
    } 

    selectedSegmento(segmento) {
        if (segmento != undefined) {
            this.detailCopy.segmentoNegocioText = this.cboSegmento.selectedItem.nombre;
        }
    }

    loadSegmentos() {
        this.segmentoNegocioService.loadList(this.context.app.defaultCompany.identity)
            .then(segmentos => this.segmentos = segmentos)
         .catch(error => console.error=error);    
    }

}
