﻿//Frameworks

import { FormBuilder, FormGroup, Validators } from '@angular/forms';
//Bootstrap
//Features
import { Component, Input, Output, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import * as moment from 'moment';
import { CollectionView } from 'wijmo/wijmo';
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { AztecaValidators } from 'framework3/controls';
import { Context } from 'framework3/core';
import { TracingModule } from 'framework3/tracing';
//
import { PolizaService } from '../../services/poliza.service';
import { PolizaLogEntry } from '../../models/poliza-log-entry';


@Component({
    selector: 'accounting-polizas-log-component',
    templateUrl: './polizas-log.component.html'
})
export class PolizasLogComponent {
    editorForm: FormGroup;
    logCollection = new CollectionView();
    currentEntry: PolizaLogEntry = new PolizaLogEntry();
    //
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('detailsDialog') detailsDialog: ModalDirective;
    @ViewChild('polizaLog') polizaLogDialog: ModalDirective;

    @Input() id: string;

    constructor(private formBuilder: FormBuilder,private context: Context, private router: Router, private route: ActivatedRoute,
        private polizaService: PolizaService) {  
        this.editorForm = formBuilder.group({
            'idInsumo': [0],           
        });        
    }

    /////////////////////// METODOS ////////////////////////////////////////////////

    loadLog() {
        this.context.app.showSpinner();
        this.polizaService.loadLog(this.id)
            .then(log => {
                this.logCollection.sourceCollection = log;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    /////////////////////// HANDLERS ///////////////////////////////////////////////
    mainToolbarClick(button: ToolbarButton) {
        if (button.name == 'REFRESH') {
            this.loadLog();
        }
    }

    onRowView(entry: PolizaLogEntry) {
        this.currentEntry = entry;
        this.detailsDialog.config.backdrop = false;
        this.detailsDialog.show();
    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    show(id) {
        this.logCollection = new CollectionView();
        this.polizaLogDialog.config.backdrop = false;                
        this.id = id;
        this.polizaLogDialog.show();
        this.loadLog();
    }

}