﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Locales
import { PeriodoInfo, PeriodoItem } from '../models/periodo';
import { PeriodoDetail } from '../models/periodo-detail';
import { PeriodoService } from '../services/periodo.service';

@Component({
    selector: 'contabilidad-periodos-manager',
    templateUrl: './periodos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PeriodosManager)
        },
        {
            provide: DocumentService, useExisting: PeriodoService
        }
    ]
})
export class PeriodosManager extends DocumentManager {
    @Input() idEjercicio: string = '';
    @Input() fechaEjercicio: Date;
    @Input() fechaIni: Date;
    @Input() fechaFin: Date;
    periodos: PeriodoItem[];

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private periodoService: PeriodoService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEjercicio = String(params['id']);
        });
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        let condDocto: SearchCondition = { field: 'IdEjercicio', operator: '=', value: this.idEjercicio, valueType: 'string' };
        request.conditions.push(condDocto);
        this.periodoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.periodos = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['accounting/periodosEditor/' + this.process.id + '/' + this.idEjercicio + '/0']);
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate(['accounting/periodosEditor/' + this.process.id + '/' + this.idEjercicio + '/' + document.id, { readonly: true }]);
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate(['accounting/periodosEditor/' + this.process.id + '/' + this.idEjercicio + '/' + document.id]);
    }

    onDeletingItem(document: DocumentItem) {
        this.periodoService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.periodoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.periodoService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

}

