﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { MenuItem } from 'framework3/controls';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { PolizaInfo, PolizaItem } from '../models/poliza';
import { PolizaDetail } from '../models/poliza-detail';
import { PolizaService } from '../services/poliza.service';
import { PolizasLogComponent } from './parts/polizas-log.component';
import { PolizasAsociacionesComponent } from './parts/polizas-asociaciones.component';


import { ModalDirective } from 'ngx-bootstrap';


@Component({
    selector: 'contabilidad-polizas-manager',
    templateUrl: 'polizas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PolizasManager)
        },
        {
            provide: DocumentService, useExisting: PolizaService
        }
    ]
})
export class PolizasManager extends DocumentManager {
    polizasTit: PolizaItem[];
    idPolizaCurrent: string;
    @ViewChild('polizasLog') polizaLog: PolizasLogComponent;
    @ViewChild('dlgAsociaciones') dlgAsociaciones: PolizasAsociacionesComponent;
    
    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private polizaService: PolizaService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.polizaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.polizasTit = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.polizaService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.polizaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.polizaService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    menuItemClick(option: MenuItem) {
        this.idPolizaCurrent = this.manager.currentDocument.id;

        if (option.name == 'Exportar') {
            this.polizaService.marcarExportacion(this.manager.currentDocument.id, -1, '-')
                .then(_ => {
                    this.context.app.hideSpinner();                   

                    let polizaItem:PolizaItem =  this.polizasTit.find(item=> item.id == this.manager.currentDocument.id) 
                                        
                    polizaItem.importacionEstatus = 'En Cola';

                    let newList: PolizaItem[] = this.polizasTit.slice();                                    

                    this.polizasTit = newList;
                   
                })
                .catch(error => this.handleError(error));
        }
        
        if (option.name == 'Log') {            
            //this.polizaLog.config.backdrop = false;            
            this.polizaLog.show(this.manager.currentDocument.id);
        }
        if (option.name == 'Asociaciones') {
            let poliza:PolizaItem =<PolizaItem>this.manager.documentList.find(doc=> doc.id == this.manager.currentDocument.id);
            if (poliza) {
                this.dlgAsociaciones.show(this.manager.currentDocument.id, poliza.fechaSys, 0, 0);
            }
        }

    }

}

