﻿import { Component, ViewChild } from "@angular/core";

import { FormsModule, FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { Menu } from "wijmo/wijmo.input";
import { ModalDirective } from "ngx-bootstrap";
import * as moment from 'moment';

import { MenuItem, ToolbarButton, AlertsBarComponent } from "framework3/controls";
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';

import { CancelacionService } from "../services/cancelacion.service";
import { FilterGroup, SearchRequest, SearchCondition } from "framework3/search";

import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';

@Component({
    selector: 'accounting-admon-cancelacion',
    templateUrl: './admon-cancelacion.maintenance.html'
})
export class AdmonCancelacion extends BaseComponent {

    form: FormGroup;
    filterPanelCollapsed: boolean = false;
    showFilterPanel: boolean = true;
    menuItems: MenuItem[] = [];
    filterGroups: FilterGroup[] = [];
    selectedGroup: FilterGroup;

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;
    @ViewChild('dlgConfirmacion') dlgConfirmacion: ModalDirective;
    @ViewChild('viewDialog') viewDialog: ModalDirective;

    request: SearchRequest;
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    currentItems: Array<any> = new Array<any>();
    currentItem: any;
    totalRows: number = 0;
    pageIndex: number = 0;
    respuesta: boolean = false;

    currentComprobante = new ComprobanteFiscalInfo();


    constructor(private context: Context, private router: Router, private route: ActivatedRoute, private location: Location,
        private formBuilder: FormBuilder, private cancelacionSvc: CancelacionService, private comprobanteFiscalService: ComprobanteFiscalService) {
        super();
        this.form = formBuilder.group({
            'fechaInicial': [],
            'fechaFinal': []
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.fechaInicial = moment().subtract(30, 'day').toDate();
            this.fechaFinal = moment().add(1, 'day').toDate();
            this.refreshList();
        });
        this.createMenu();
    }

    createMenu() {
        //Agregar opciones estándar del catálogo
        //<azteca-menu-item [name]="'VIEW_COMPROBANTE'" [icon]="'glyphicon glyphicon-folder-open'" [text]="'Abrir'" [smallText]="'abrir en modo de sólo lectura'"></azteca-menu-item>
        let mnuDownload: MenuItem = { icon: "glyphicon glyphicon-circle-arrow-down", text: "Consultar", name: "DOWNLOAD", smallText: "Consultar pendientes aprobar" };
        let mnuOpen: MenuItem = { icon: "glyphicon glyphicon-circle-arrow-down", text: "Abrir", name: "OPEN", smallText: "abrir en modo de sólo lectura" };
        let mnuAceptar: MenuItem = { icon: "glyphicon glyphicon-ok", text: "Aceptar", name: "ACEPTAR", smallText: "Acteptar solicitud" };
        let mnuCancelar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Rechazar", name: "RECHAZAR", smallText: "Rechazar solicitud" };
        //
        this.menuItems.push(mnuDownload);
        this.menuItems.push(mnuOpen);
        this.menuItems.push(mnuAceptar);
        this.menuItems.push(mnuCancelar);
        //
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }


    /*Permite tomar todo el espacio disponible cuando el panel de filtro no está visible y 75% cuando lo está*/
    get classGridPanel(): string {
        return !this.showFilterPanel || this.filterPanelCollapsed ? 'col-12' : 'col-12 col-lg-9';
    }

    toggleCollapse() {
        this.filterPanelCollapsed = !this.filterPanelCollapsed;
    }

    onMenuItemClicked(menu: Menu, args: any) {
        let item: MenuItem = this.menuItems[menu.selectedIndex];
        this.optionClick(item);
    }

    optionClick(option: MenuItem) {
        if (option.name == "DOWNLOAD") {
            this.downloadPeticionPendiente();
        }
        else if (option.name == "ACEPTAR") {
            this.respuesta = true;
            this.dlgConfirmacion.config.backdrop = false;
            this.dlgConfirmacion.show();
        }
        else if (option.name == "RECHAZAR") {
            this.respuesta = false;
            this.dlgConfirmacion.config.backdrop = false;
            this.dlgConfirmacion.show();
        }
        else if (option.name == 'OPEN') {
            this.context.app.showSpinner();
            this.comprobanteFiscalService.getDetail(this.currentItem.data.idComprobante)
                .then(comprobante => {
                    this.currentComprobante = comprobante;
                    this.context.app.hideSpinner();
                    this.viewDialog.config.backdrop = false;
                    this.viewDialog.show();
                })
                .catch(error => this.handleError(error));

        }
    }

    onParamsToolbar(button: ToolbarButton) {
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onCurrentItemChanged(item: any) {
        this.currentItem = item;
    }

    searchRequested(request: SearchRequest) {
        this.request = request;
        this.refreshList();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    onIsDroppedDown(menu: Menu, args: any) {
        this.onValidateMenu(this.currentItem);
    }

    onValidateMenu(item: any) {
        if (item) {
            let mnuAceptar = this.getMenuItem('ACEPTAR');
            let mnuRechazar = this.getMenuItem('RECHAZAR');
            if (item.estatus != "Pendiente") {
                mnuAceptar.enabled = false
                mnuRechazar.enabled = false;
            }
        }
    }

    getMenuItem(name: string): MenuItem {
        let retItem: MenuItem = null;
        this.menuItems.forEach(item => {
            if (item.name == name) {
                retItem = item;
            }
        });
        return retItem;
    }

    onFilterGroupChanged(group: FilterGroup) {
        this.selectedGroup = group;
    }

    ////////////////////////////////////////

    downloadPeticionPendiente() {
        this.context.app.showSpinner();
        this.cancelacionSvc.downloadPeticionPendiente(this.context.app.defaultCompany.identity).then(result => {
            this.context.app.hideSpinner();
            this.refreshList();
        }).catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }
        let exist = false;
        
        this.request.conditions.forEach(r => {
            if (r.field == "fecha" && r.operator == ">=") {
                exist = true;
                r.value = moment(this.fechaInicial).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
            else if (r.field == "fecha" && r.operator == "<=") {
                exist = true;
                r.value = moment(this.fechaFinal).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
        });
        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fecha", operator: ">=", valueType: "datetime", value: moment(this.fechaInicial).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conFechaFin: SearchCondition = { field: "fecha", operator: "<=", valueType: "datetime", value: moment(this.fechaFinal).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            this.request.conditions.push(conFechaIni);
            this.request.conditions.push(conFechaFin);
        }
        this.cancelacionSvc.search(this.context.app.defaultSite.identity, this.request)
            .then(response => {
                response.rows.forEach(r => {
                    r['fecha'] = moment(r['fecha']).toDate();
                });
                this.currentItems = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = this.request.pageIndex;
                //
                var filterGroup = new FilterGroup();
                filterGroup.tag = "ALL";
                filterGroup.name = "Todos";
                filterGroup.rowCount = this.totalRows;
                this.filterGroups.length = 0;
                this.filterGroups.push(filterGroup);
                //
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onCancelar() {
        this.dlgConfirmacion.hide();
    }

    onAceptar() {
        this.dlgConfirmacion.hide();
        this.context.app.showSpinner();
        this.cancelacionSvc.aceptarRechazar(this.context.app.defaultSite.idBranch, this.respuesta, this.currentItem.id)
            .then(response => {
                this.context.app.hideSpinner();
                this.refreshList();
            }).catch(error => this.handleError(error));
    }

}