﻿//Frameworks
import { Component, ViewChild, forwardRef, SimpleChange, OnChanges, Renderer, ElementRef } from '@angular/core';
import { FormsModule, FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';

import { ModalDirective } from 'ngx-bootstrap'

import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { FileInfo } from 'framework3/general/models/file';

//Features
import { ComprobanteService } from '../services/comprobante.service'
import { ComprobanteInfo } from '../models/comprobante'
import { MenuItem } from "framework3/controls";

import { ComparativoComprobantesSAT } from '../models/comprobante';

@Component({
    selector: 'comprobante-maintenance',
    templateUrl: './comprobante.maintenance.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ComprobanteMaintenance)
        },
        {
            provide: CatalogService, useExisting: ComprobanteService
        }
    ]
})
export class ComprobanteMaintenance extends CatalogManager {

    form: FormGroup;
    formFile: FormGroup;

    companias: Array<any> = [{ id: '01', nombre: 'Embotelladora AGA' }, { id: '02', nombre: 'POLI' }, { id: '03', nombre: 'PET' }, { id: '04', nombre: 'CONCENTRADOS' }, { id: '05', nombre: 'LOMAS' }, { id: '06', nombre: 'VARIOS' }];

    compania: string = "01";
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    tienePoliza: boolean = false;
    noTienePoliza: boolean = false;
    existeSiSatNoPixcua: boolean = false;
    existeNoSatSiPixcua: boolean = false;
    cancSiSatNoPixcua: boolean = false;
    cancNoSatSiPixcua: boolean = false;
    docEmitido: boolean = false;
    docRecibido: boolean = false;

    comprobantes: Array<any> = new Array<any>();
    selectedFile: FileInfo = new FileInfo();
    search: SearchRequest = null;
    cancelado: boolean = false;
    editable: false;

    comparativo: ComparativoComprobantesSAT = null;
    timerInterval: any = null;
    

    @ViewChild('addComprobanteDialog') addComprobanteDialog: ModalDirective;

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private comprobanteService: ComprobanteService, private formBuilder: FormBuilder) {
        super(context, router, route);
        this.form = formBuilder.group({
            'cboCompania': [],
            'fechaInicial': [new Date()],
            'fechaFinal': [new Date()],
            'flbComprobante': ['', Validators.required],
            'chkCancelado': [false],
            'chkTitnePoliza': [false],
            'chkNoTitnePoliza': [false],
            'chkExisteSiSatNoPixcua': [false],
            'chkExisteNoSatSiPixcua': [false],
            'chkCanSiSatNoPixcua': [false],
            'chkCanNoSatSiPixcua': [false],
            'chkDocEmitido': [false],
            'chkDocRecibido':[false]
        });
    }


    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = 100;
        this.manager.allowNew = false;
        this.manager.allowView = true;
        this.manager.allowEdit = false;
        this.manager.allowDelete = false;
        this.manager.allowClone = false;
        this.manager.allowChangeState = false;
        this.manager.allowAudit = false;
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.search = request;
        this.context.app.showSpinner();
        this.comprobanteService.search(this.compania, moment(this.fechaInicial).format(), moment(this.fechaFinal).format(), this.tienePoliza, this.noTienePoliza, this.existeSiSatNoPixcua, this.existeNoSatSiPixcua, this.cancSiSatNoPixcua, this.cancNoSatSiPixcua, this.docEmitido, this.docRecibido, this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();

                response.rows.forEach(r => {
                    r['comFechaFactura'] = moment(r['comFechaFactura']).toDate();
                    r['activo'] = r['activo'] == '1';
                });

                this.comprobantes = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.addComprobanteDialog.config.backdrop = false;
        this.addComprobanteDialog.show();
    }

    onViewingItem(item: CatalogItem) {
    }

    onEditingItem(item: CatalogItem) {
        
    }

    onDeletingItem(item: CatalogItem) {
    }

    onCloningItem(item: CatalogItem) {
    }

    onChangingItemState(item: CatalogItem) {
    }


    onFilterGroupChanged(group: any) {
    }

    hideAddComprobanteDialog() {
        this.addComprobanteDialog.hide();
    }

    addComprobante() {
        this.context.app.showSpinner();
        this.comprobanteService.uploadComprobante(this.cancelado, this.selectedFile)
            .then(info => {
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
        this.addComprobanteDialog.hide();
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'PRINT') {
            this.context.app.showSpinner();
            this.comprobanteService.getComprobanteFiscalV32(this.manager.currentItem.data.comSerie, this.manager.currentItem.data.comFolio, moment(this.manager.currentItem.data.comFechaFactura).format(), this.manager.currentItem.data.uuid)
                .then(report => {
                    window.open("data:application/pdf;base64, " + report);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        else if (item.name == 'DOWNLOAD-EXCEL') {
            this.context.app.showSpinner();
            this.comprobanteService.getComprativoComprobante(this.compania, moment(this.fechaInicial).format(), moment(this.fechaFinal).format(), this.tienePoliza, this.noTienePoliza, this.existeSiSatNoPixcua, this.existeNoSatSiPixcua, this.cancSiSatNoPixcua, this.cancNoSatSiPixcua, this.docEmitido, this.docRecibido, this.context.app.defaultSite.identity, this.search)
                .then(comparativo => {

                    this.comparativo = comparativo;

                    if (this.timerInterval != null)
                        clearInterval(this.timerInterval);

                    this.timerInterval = setInterval(() => {
                        this.verificarImportExcel();
                    }, 3000);

                })
                .catch(error => this.handleError(error));
        }
        else if (item.name == 'DOWNLOAD-EXCEL') {
            this.context.app.showSpinner();
            this.comprobanteService.getComprativoComprobante(this.compania, moment(this.fechaInicial).format(), moment(this.fechaFinal).format(), this.tienePoliza, this.noTienePoliza, this.existeSiSatNoPixcua, this.existeNoSatSiPixcua, this.cancSiSatNoPixcua, this.cancNoSatSiPixcua, this.docEmitido, this.docRecibido, this.context.app.defaultSite.identity, this.search)
                .then(comparativo => {

                    this.comparativo = comparativo;

                    if (this.timerInterval != null)
                        clearInterval(this.timerInterval);

                    this.timerInterval = setInterval(() => {
                        this.verificarImportExcel();
                    }, 3000);

                })
                .catch(error => this.handleError(error));
        }
    }

    verificarImportExcel() {
        this.context.app.showSpinner();
        this.comprobanteService.getProgramacionComparativo(this.comparativo.idProgramacion)
            .then(response => {
                if (response.idStatus == 2) {
                    window.open(this.context.fileServerPath + "/" + response.fileServer);
                    clearInterval(this.timerInterval);
                    this.context.app.hideSpinner();
                }
            })
            .catch(error => this.handleError(error));
    }

    createMenu() {
        
    }

}
