﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
//Locales
import { ImpuestoService } from '../services/impuesto.service'
import { IEPSPreview } from '../models/ieps-preview'

import { AsyncTaskService } from "framework3/queue/services/async-task.service";

@Component({
    selector: 'accounting-ieps-archivo-generador',
    templateUrl: './ieps-archivo-generador.maintenance.html'
})
export class IepsArchivoGenerador extends ProcessComponent {
    form: FormGroup;

    previewCollection: CollectionView = new CollectionView();
    iepsPreviewList: IEPSPreview[] = [];

    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();

    cargando: boolean = false;

    selectedCollection: CollectionView = new CollectionView();
    iepsSelectedList: IEPSPreview[] = [];
    iepsSelectorList: IEPSPreview[] = [];
    seleccionadosCount: number = 0;
    totalIepsSelected: number = 0;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder
        , private impuestoService: ImpuestoService
        , private asyncTaskSvc: AsyncTaskService) {
        super(context, router, route);
        this.form = formBuilder.group({
            'fechaInicial': [new Date()],
            'fechaFinal': [new Date()]


        });
    }


    // TODO: AsynTask load
    llenandoIepsPreview() {
        this.context.app.showSpinner();
        this.impuestoService.asyncPreviewIeps(this.context.app.defaultCompany.identity, this.fechaInicial, this.fechaFinal)
            .then(recs => {
                this.asyncTaskSvc.subscribeTask(recs.idAsyncTask).subscribe(info => {
                    //this.graph.asyncTask = info;
                    //this.graph.asyncTask.resultJson = JSON.parse(this.graph.asyncTask.result);

                    this.context.app.hideSpinner();
                    console.log(info);

                    this.iepsPreviewList = JSON.parse(info.result);
                    //this.previewCollection.sourceCollection = JSON.parse(info.result);
                    /**************Selector**********************/

                    //Ordenandolo
                    var iepsPreviewListSorted: IEPSPreview[];
                    iepsPreviewListSorted = this.iepsPreviewList.slice(0);

                    iepsPreviewListSorted.sort((leftSide, rightSide): number => {
                        if (leftSide.RFC < rightSide.RFC) return -1;
                        if (leftSide.RFC > rightSide.RFC) return 1;
                        return 0;
                    });

                    //Sumarizando para grid superior
                    var rfcActual = ''
                    iepsPreviewListSorted.forEach(y => {
                            
                            if (rfcActual != y.RFC)
                            {

                                rfcActual = y.RFC

                                var grupoRFC = iepsPreviewListSorted.filter(subList => subList.RFC == rfcActual);

                                var item: IEPSPreview = new IEPSPreview;
                                item = grupoRFC[0];

                                var iRfc: IEPSPreview = new IEPSPreview
                                iRfc.RFC = item.RFC;
                                iRfc.RazonSocial = item.RazonSocial;
                                iRfc.CondicionesPago = item.CondicionesPago;
                                iRfc.Estado = '';
                                iRfc.CodigoEstado = '';
                                //sumarizando
                                grupoRFC.forEach(x => {
                                    iRfc.IEPS = iRfc.IEPS + x.IEPS;
                                    iRfc.IVA = iRfc.IVA + x.IVA;
                                    iRfc.IVA_IEPS = iRfc.IVA_IEPS + x.IVA_IEPS;
                                    iRfc.Litros = iRfc.Litros + x.Litros;
                                    iRfc.SubTotal = iRfc.SubTotal + x.SubTotal;
                                });

                                var newRfc = this.iepsSelectorList.slice();
                                newRfc.push(iRfc);

                                this.iepsSelectorList = newRfc;

                            }

                        this.previewCollection.sourceCollection = this.iepsSelectorList;

                        });

                    /********************************************/

                });
            })
            .catch(error => this.handleError(error));
    }

    



    

    generandoIepsFile() {

        var file: FileLink;

        this.impuestoService.archivoIeps('', this.fechaInicial, this.fechaFinal, this.iepsSelectedList)
            .then(ret => {

                let links: FileLink[] = [];
                links.push(ret);
                this.downloadPopup.show(links);

            })
            .catch(error => this.handleError(error));

    }

    seleccionarRFC() {
        var item: IEPSPreview = new IEPSPreview;
        item = this.previewCollection.currentItem

        var grupoRFC = this.iepsPreviewList.filter(subList => subList.RFC == item.RFC);

        grupoRFC.forEach(x => {

                var iRfc: IEPSPreview = new IEPSPreview
                iRfc.RFC = x.RFC;
                iRfc.RazonSocial = x.RazonSocial;
                iRfc.CondicionesPago = x.CondicionesPago;
                iRfc.Estado = x.Estado;
                iRfc.CodigoEstado = x.CodigoEstado;
                iRfc.IEPS = x.IEPS;
                iRfc.IVA = x.IVA;
                iRfc.IVA_IEPS = x.IVA_IEPS;
                iRfc.Litros = x.Litros;
                iRfc.SubTotal = x.SubTotal;

                var newRfc = this.iepsSelectedList.slice();
                newRfc.push(iRfc);

                this.iepsSelectedList = newRfc;

                this.selectedCollection.sourceCollection = newRfc;
                this.selectedCollection.refresh;

                this.calcularSeleccionados();

            });
        };





    
    /* antes de la modificacion
    seleccionarRFC() {
        var item: IEPSPreview = new IEPSPreview;
        item = this.previewCollection.currentItem

        var iRfc: IEPSPreview = new IEPSPreview
        iRfc.RFC = item.RFC;
        iRfc.RazonSocial = item.RazonSocial;
        iRfc.CondicionesPago = item.CondicionesPago;
        iRfc.Estado = item.Estado;
        iRfc.CodigoEstado = item.CodigoEstado;
        //sumarizando
        this.iepsPreviewList.forEach(x => {
            if (x.RFC == item.RFC && x.Estado == item.Estado) {
                iRfc.IEPS = iRfc.IEPS + x.IEPS;
                iRfc.IVA = iRfc.IVA + x.IVA;
                iRfc.IVA_IEPS = iRfc.IVA_IEPS + x.IVA_IEPS;
                iRfc.Litros = iRfc.Litros + x.Litros;
                iRfc.SubTotal = iRfc.SubTotal + x.SubTotal;
            }
        });


        var newRfc = this.iepsSelectedList.slice();
        newRfc.push(iRfc);

        this.iepsSelectedList = newRfc;

        this.selectedCollection.sourceCollection = newRfc;
        this.selectedCollection.refresh;

        this.calcularSeleccionados();

    }
    */

    calcularSeleccionados() {
        //calculando el total de rfcs distintos seleccionados
        var selectedRfcs: string[] = [];
        this.totalIepsSelected = 0;
        this.iepsSelectedList.forEach(y => {
            var result = selectedRfcs.filter(
                function (value) {
                    return (value == y.RFC);
                });
            if (result.length == 0) {
                selectedRfcs.push(y.RFC);
            }
            this.totalIepsSelected = this.totalIepsSelected + y.IEPS;
            
        });
        this.seleccionadosCount = selectedRfcs.length;
    }


    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.cargando = false;
        this.alerts.showAlert(message, 'danger');
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CONSULTAR') {
            this.llenandoIepsPreview();
        }
        if (button.name == 'GENERAR') {
            this.generandoIepsFile();
        }
        if (button.name == 'SELECCIONAR') {
            this.seleccionarRFC();
        }
    }


    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

    onMenuSeleccionadosClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Eliminar
            //this.iepsSelectedList = this.selectedCollection.currentItem;
            this.selectedCollection.remove(this.selectedCollection.currentItem);
        }
        this.calcularSeleccionados();

    }

}
