﻿import { DocumentInfo, DocumentItem } from 'framework3/core';
import { ComprobanteFiscalDetail } from './comprobante-fiscal-detail';
import { DivisaInfo } from './divisa';

export class ComprobanteFiscalInfo extends DocumentInfo<ComprobanteFiscalDetail> {	
	serie: string;
	idCFDSerie: number;
	folio: string;
	fecha: Date;	
	idContribuyente: number;
	idDocumento: string;
	fechaDocumento: Date;
    archivo: string;
    emisor: EmisorInfo;
    receptor: ReceptorInfo;
    impuestos: ImpuestosInfo;
    comprobantesRelacionados: ComprobantesRelacionadosInfo;
    timbreFiscal: TimbreFiscalInfo;
	cadenaOriginal: string;
	cadenaOriginalTimbre: string;
	selloDigital: string;	
    numeroCertificado: string;
    certificado: string;	
	tipoComprobante: number;
    divisa: DivisaInfo;
	tipoCambio: number;
	subTotal: number;
	descuentos: number;
	total: number;
	credito: boolean;
	diasCredito: number;
	idAddenda: number;
	gLNBuyer: string;
	gLNSeller: string;
	gLNStore: string;
	sellerID: string;
	version: number;
	referencia: string;
	observaciones: string;
	cancelado: boolean;
	fechaCancelacion: Date;
	acuseCancelacion: string;
	idPAC: number;
    emitido: boolean;
    contenidoXML: string;
    errorTimbrado: string;
    uuid: string;

    impuestosLocales: ImpuestosLocalesInfo;

    constructor() {
        super();
        this.emisor = new EmisorInfo();
        this.receptor = new ReceptorInfo();
        this.impuestos = new ImpuestosInfo();
        this.comprobantesRelacionados = new ComprobantesRelacionadosInfo();
        this.timbreFiscal = new TimbreFiscalInfo();
        this.divisa = new DivisaInfo();
        this.impuestosLocales = new ImpuestosLocalesInfo();
    }
}

export class ComprobanteFiscalItem implements DocumentItem {
    id: string;    
    fechaCreacion: Date;
    fechaTimbrado?: Date;
    fechaCancelacion?: Date;
    serie?: string;
    folio?: number;
    uuid: string;
    tipoComprobante?: string;
    rfcEmisor?: string;
    rfcReceptor?: string;
    correo?: string;
    idStatus: number;
    status: string;
    idWorkflowStep: number;
    workflowStep: string;
    sentido?: string;
    idComprobante: string;
    selected?: boolean;
}

export class EmisorInfo {
    rfc: string;
    nombre: string;
    regimenFiscal: string;

    constructor() {
        this.rfc = '';
        this.nombre = '';
        this.regimenFiscal = '';
    }
}

export class ReceptorInfo {
    rfc: string;
    nombre: string;
    residenciaFiscal: string;
    numRegIdTrib: string;
    usoCFDI: string;
    domicilio: domicilioInfo;

    constructor() {
        this.rfc = '';
        this.nombre = '';
        this.domicilio = new domicilioInfo();
    }
}

export class ImpuestosInfo {
    totalTraslados: number;
    totalRetenciones: number;

    constructor() {
        this.totalTraslados = 0;
        this.totalRetenciones = 0;
    }
}

export class ComprobantesRelacionadosInfo {
    tipoRelacion: string;
    comprobantes: string[];

    constructor() {
        this.tipoRelacion = '';
        this.comprobantes = [];
    }
}

export class TimbreFiscalInfo {
    version: string;
    fecha: Date;
    uuid: string;
    rfcProvCertif: string;
    selloSAT: string;
    numeroCertificadoSAT: string;
    selloCFD: string;
    leyenda: string;
}

export class ImpuestosLocalesInfo {
    totalTraslados: number;
    totalRetenciones: number;
}

export class domicilioInfo {
    calle: string;
    numeroExterior: string;
    numeroInterior: string;
    colonia: string;
    localidad: string;
    referencia: string;
    municipio: string;
    estado: string;
    pais: string;
    codigoPostal: string;
    codigoEstado: string;
    codigoPais: string;

    constructor() {
        this.calle = '';
        this.numeroExterior= '';
        this.numeroInterior= '';
        this.colonia= '';
        this.localidad= '';
        this.referencia= '';
        this.municipio= '';
        this.estado= '';
        this.pais= '';
        this.codigoPostal= '';
        this.codigoEstado= '';
        this.codigoPais= '';
    }
}