﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import { map } from 'rxjs/operators';
import * as moment from 'moment';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
import { SearchRequest, SearchResult } from 'framework3/search';
//Locales
import { CFDCertificadoInfo, CFDCertificadoItem } from '../models/cfd-certificado'

@Injectable()
export class CFDCertificadoService extends CatalogService<CFDCertificadoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/contabilidad/cfd-certificados");
    }

    create(): Promise<CFDCertificadoInfo> {
        return Promise.resolve(new CFDCertificadoInfo());
    }

    search(idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idBranch', idBranch.toString());
        //
        return this.apiService.post<SearchResult>('search', params, request)
            .pipe(map(result => {
                result.rows.forEach(row => {
                    row.validFrom = moment(row.validFrom).toDate();
                    row.validTo = moment(row.validTo).toDate();
                });
                return result;
            }))
            .toPromise();
    }

    getDetailByID(itemID: number): Promise<CFDCertificadoInfo> {
        let params = new HttpParams()
            .set('itemID', itemID.toString());
        return this.apiService.get<CFDCertificadoInfo>('get-detail-id', params)
            .pipe(map(info => {
                info.validFrom = moment(info.validFrom).toDate();
                info.validTo = moment(info.validTo).toDate();
                return info;
            }))
            .toPromise();
    }

    getDetailByCode(idBranch: number, code: string): Promise<CFDCertificadoInfo> {
        let params = new HttpParams()
            .set('idBranch', idBranch.toString())
            .set('code', code);
        return this.apiService.get<CFDCertificadoInfo>('get-detail-code', params)
            .pipe(map(info => {
                info.validFrom = moment(info.validFrom).toDate();
                info.validTo = moment(info.validTo).toDate();
                return info;
            }))
            .toPromise();
    }

    loadVigencias(idEmpresa: number): Promise<CFDCertificadoItem[]> {
        let params = new HttpParams()
            .set('idEmpresa', idEmpresa.toString());
        //
        return this.apiService.get<CFDCertificadoItem[]>('vigencias', params)
            .pipe(map(vigencias => {
                vigencias.forEach(cert => {
                    cert.validFrom = moment(cert.validFrom).toDate();
                    cert.validTo = moment(cert.validTo).toDate();
                });
                return vigencias;
            }))
            .toPromise();
    }


}

