﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';
import { LoteImpresionInfo } from 'framework3/printing';
//Locales
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';
import { ComprobanteFiscalDetail } from '../models/comprobante-fiscal-detail';
import { AddendaItem } from '../models/addenda';
import { EnvioComprobantesItem, ResetComprobantesItem } from '../../credit/components/envio-comprobantes-fiscales.component'
import { idLocale } from 'ngx-bootstrap';

@Injectable()
export class ComprobanteFiscalService extends DocumentService<ComprobanteFiscalDetail, ComprobanteFiscalInfo> {

    constructor(protected http: HttpClient, protected context: Context) {
        super(http, context, 'api/contabilidad/comprobantes');
    }

    create(): Promise<ComprobanteFiscalInfo> {
        let newDoc = new ComprobanteFiscalInfo();
        return Promise.resolve(newDoc);
    }

    loadListAddendas(idEmpresa: number): Promise<AddendaItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<AddendaItem[]>("load-list-addendas", params)            
            .toPromise();
    }

    searchKeyword(idBranch: number, keyword: string, idContribuyente: number): Observable<ComprobanteFiscalItem[]> {
        let params: HttpParams = new HttpParams()
              .set("idBranch", idBranch.toString())
              .set("keyword", keyword)
              .set("idContribuyente", idContribuyente.toString());
        return this.apiService.get<ComprobanteFiscalItem[]>("search-comprobantes", params);
    }

    generarPdfTemporal(idComprobante: string): Promise<FileLink> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobante);
        return this.apiService.get<FileLink>("pdf-temporal", params)            
            .toPromise();
    }

    regenerarPdfTemporal(idComprobante: string): Promise<FileLink> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobante);
        return this.apiService.get<FileLink>("regenerar-pdf-temporal", params)
            .toPromise();
    }

    generarPdfTemporalMultiple(comprobantes: Array<string>): Promise<string[]> {
        let params: HttpParams = new HttpParams()
        return this.apiService.post<string[]>("pdf-temporal-multiple", params,comprobantes)
            .toPromise();
    }

    generarPdfTemporalArr(idComprobante: string[]): Promise<FileLink> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<FileLink>("pdf-temporal-arr",params, idComprobante)            
            .toPromise();
    }

    getLoteDescarga(idComprobantes: string[]): Promise<LoteImpresionInfo> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<LoteImpresionInfo>("descargar-lote", params, idComprobantes)
            .toPromise();
    }

    getLoteExportacion(idComprobantes: string[]): Promise<any> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<any>("exportar-lote", params, idComprobantes)
            .toPromise();
    }
    getLoteExportacionStatus(idLote:number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("IdLote", idLote.toString());
        return this.apiService.post<any>("exportar-lote-status", params)
            .toPromise();
    }
    

    getXml(idComprobanteFiscal: string): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobanteFiscal);
        return this.apiService.getText("xml", params)            
            .toPromise();
    }

    getXmlFileArr(idComprobanteFiscal: string[]): Promise<FileLink> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<FileLink>("xml-file-arr", params, idComprobanteFiscal)            
            .toPromise();
    }


    getXmlFile(idComprobanteFiscal: string): Promise<FileLink> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobanteFiscal);
        return this.apiService.get<FileLink>("xml-file", params)            
            .toPromise();
    }

    cancelar(idComprobanteFiscal: string): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobanteFiscal);
        return this.apiService.getText("cancelar", params)            
            .toPromise();
    }

    timbrar(idComprobanteFiscal: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobanteFiscal);
        return this.apiService.get<any>("timbrar", params)
            .toPromise();
    }

    enviar(idComprobanteFiscal: string, correo: string, mensaje: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobanteFiscal)
            .set("correo", correo)
            .set("mensaje", mensaje);
        return this.apiService.get<any>("send", params)
            .toPromise();
    }

    getEnvioFacturas(idEmpresa: number, fechaIni: Date, fechaFin: Date): Promise<EnvioComprobantesItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString())
            .set('fechaIni', moment(fechaIni).format())
            .set('fechaFin', moment(fechaFin).format());
        return this.apiService.get<EnvioComprobantesItem[]>("envio-facturas-list", params)            
            .toPromise();
    }

    ComprobantesDestinosList(): Promise<any[]> {
        let params: HttpParams = new HttpParams();
        return this.apiService.get<any[]>("comprobantes-destino-list", params)            
            .toPromise();
    }

    resetEnvioFacturas(idJob: ResetComprobantesItem[]): Promise<any> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<any>('reset-envio-facturas', params, idJob)
            .toPromise();
    }


    validarComprobanteSAT(idComprobante: string): Promise<boolean> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobante);
        return this.apiService.get<boolean>('validar-comprobante-sat', params)
            .toPromise();
    }

    getComprobanteBySerieFolio(serie: string, folio: string, idEmpresa: number): Promise<ComprobanteFiscalInfo> {
        let params: HttpParams = new HttpParams()
            .set("serie", serie)
            .set("folio", folio)
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<ComprobanteFiscalInfo>("get-comprobante-serie-folio", params)
            .toPromise();
    }

    //Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
    protected mapSearchResult(row: any) {
        row.fechaTimbrado = moment(row.fechaTimbrado).toDate();
    }


    //Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: ComprobanteFiscalInfo) {
        document.fecha = moment(document.fecha).toDate();
        document.fechaAfectacion = moment(document.fechaAfectacion).toDate();
        document.timbreFiscal.fecha = moment(document.timbreFiscal.fecha).toDate();
    }

    //Portal de clientes
    portalClientes(idCliente: number, fechaIni: Date, fechaFin: Date): Promise<any[]> {
        let params: HttpParams = new HttpParams()
            .set("idCliente", idCliente.toString())
            .set('fechaIni', moment(fechaIni).format())
            .set('fechaFin', moment(fechaFin).format());
        return this.apiService.get<any[]>("portal-clientes", params)            
            .toPromise();
    }

    getAddenda(idComprobanteFiscal: string):Promise<string> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobanteFiscal);
        return this.apiService.getText("get-addenda", params)
            .toPromise();
    }


    reemplazaAddenda(idComprobanteFiscal: string, addenda: string[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("idComprobanteFiscal", idComprobanteFiscal);
        return this.apiService.post<any>("reemplazar-addenda", params, addenda)
            .toPromise();
    }


}