﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import { SearchResult, SearchRequest } from 'framework3/search';
//Azteca Kernel
import { Context, BaseService } from 'framework3/core';

import { CatalogOptions } from 'framework3/custom';

import { FileInfo } from 'framework3/general/models/file';
import { ComparativoComprobantesSAT } from '../models/comprobante';
import { ComprobanteFiscalItem } from '../models/comprobante-fiscal';

@Injectable()
export class ComprobanteService extends BaseService {

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/contabilidad/comprobante");
    }

    getOptions(): Promise<CatalogOptions> {
        let options = new CatalogOptions();
        options.busquedasServidor = true;
        options.tamanioPagina = 10;
        return Promise.resolve(options);
    }


    uploadComprobante(cancelado: boolean, file: FileInfo): Promise<any> {
        let params = new HttpParams()
            .set('cancelado', cancelado.toString());
        //
        return this.apiService.post<any>('upload-comprobante', params, file)
            .toPromise();
    }

    search(compania: string, fechaIni: string, fechaFin: string, tienePoliza: boolean, noTienePoliza: boolean, existeSiSatNoPixcua:boolean, existeNoSatSiPixcua: boolean, cancSiSatNoPixcua: boolean, cancNoSatSiPixcua: boolean, docEmitido: boolean, docRecibido: boolean, idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('compania', compania)
            .set('fechaIni', fechaIni)
            .set('fechaFin', fechaFin)
            .set('tienePoliza', tienePoliza.toString())
            .set('noTienePoliza', noTienePoliza.toString())
            .set('existeSiSatNoPixcua', existeSiSatNoPixcua.toString())
            .set('existeNoSatSiPixcua', existeNoSatSiPixcua.toString())
            .set('cancSiSatNoPixcua', cancSiSatNoPixcua.toString())
            .set('cancNoSatSiPixcua', cancNoSatSiPixcua.toString())
            .set('docEmitido', docEmitido.toString())
            .set('docRecibido', docRecibido.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<SearchResult>('search', params, request)
            .toPromise();
    }

    getComprobanteFiscalV32(serie: string, folio: number, fecha: string, uuid: string): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('serie', serie)
            .set('folio', folio.toString())
            .set('fecha', fecha)
            .set('uuid', uuid);
        return this.apiService.get<string>('comprobante-fiscal-v32', params)        
            .toPromise();
    }

    getComprativoComprobante(compania: string, fechaIni: string, fechaFin: string, tienePoliza: boolean, noTienePoliza: boolean, existeSiSatNoPixcua: boolean, existeNoSatSiPixcua: boolean, cancSiSatNoPixcua: boolean, cancNoSatSiPixcua: boolean, docEmitido: boolean, docRecibido: boolean, idBranch: number, request: SearchRequest): Promise<ComparativoComprobantesSAT> {
        let params: HttpParams = new HttpParams()
            .set('compania', compania)
            .set('fechaIni', fechaIni)
            .set('fechaFin', fechaFin)
            .set('tienePoliza', tienePoliza.toString())
            .set('noTienePoliza', noTienePoliza.toString())
            .set('existeSiSatNoPixcua', existeSiSatNoPixcua.toString())
            .set('existeNoSatSiPixcua', existeNoSatSiPixcua.toString())
            .set('cancSiSatNoPixcua', cancSiSatNoPixcua.toString())
            .set('cancNoSatSiPixcua', cancNoSatSiPixcua.toString())
            .set('docEmitido', docEmitido.toString())
            .set('docRecibido', docRecibido.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<ComparativoComprobantesSAT>('comparativo-comprobante-fiscal', params, request)
            .toPromise();
    }

    getProgramacionComparativo(idProgramacion:number): Promise<ComparativoComprobantesSAT> {
        var params: HttpParams = new HttpParams()
            .set("idProgramacion", idProgramacion.toString());
        return this.apiService.get<ComparativoComprobantesSAT>('programacion-comparativo', params)
            .toPromise();
    }


}