﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { ContribuyenteInfo, UsoCFDIItem } from '../models/contribuyente'


@Injectable()
export class ContribuyenteService extends CatalogService<ContribuyenteInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/contabilidad/contribuyentes");
    }

    create(): Promise<ContribuyenteInfo> {
        return Promise.resolve(new ContribuyenteInfo());
    }

    consultarRfcInscrito(idContribuyente: number): Promise<boolean> {
        let params: HttpParams = new HttpParams()
            .set('idContribuyente', idContribuyente.toString());
        return this.apiService.get<boolean>('consultar-rfc-inscrito', params)            
            .toPromise();
    }

    inListaNegraEFO(rfc: string, idEmpresa: number): Promise<boolean> {
        let params: HttpParams = new HttpParams()
            .set('RFC', rfc)
            .set('IdEmpresa', idEmpresa.toString());

        return this.apiService.get<boolean>('in-lista-negra-efo', params)
            .toPromise();
    }
    

    loadUsosCFDI(idEmpresa: number): Promise<UsoCFDIItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<UsoCFDIItem[]>("load-usos-cfdi", params)            
            .toPromise();
    }

}

