﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { CuentaBancariaInfo, CuentaBancariaItem } from '../models/cuenta-bancaria'
import { UsuarioItem } from 'framework3/core';


@Injectable()
export class CuentaBancariaService extends CatalogService<CuentaBancariaInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/contabilidad/cuentas-bancarias');
    }

    create(): Promise<CuentaBancariaInfo> {
        return Promise.resolve(new CuentaBancariaInfo());
    }

    loadListByUser(idSucursal: number): Promise<CuentaBancariaItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<CuentaBancariaItem[]>('load-list-by-user', params)
            .toPromise();
    }

    getMembers(idCuentaBancaria: number): Promise<UsuarioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCuentaBancaria', idCuentaBancaria.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)
            .toPromise();
    }

    addMembers(idCuentaBancaria: number, users: UsuarioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCuentaBancaria', idCuentaBancaria.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }

    removeMember(idCuentaBancaria: number, idUser: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCuentaBancaria', idCuentaBancaria.toString())
            .set('idUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }


}

