﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { DivisaInfo, DivisaItem, TipoCambioItem } from '../models/divisa'

@Injectable()
export class DivisaService extends CatalogService<DivisaInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/contabilidad/divisas");
    }

    create(): Promise<DivisaInfo> {
        return Promise.resolve(new DivisaInfo());
    }

    loadList(idEmpresa: number): Promise<DivisaItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<DivisaItem[]>("load-list", params)
            
            .toPromise();
    }

    getDetailByEmpresa(idEmpresa: number): Promise<DivisaInfo> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<DivisaInfo>("divisa-by-empresa", params)            
            .toPromise();
    }

    //Tipos de Cambio
    loadTiposCambio(idEmpresa: number): Promise<TipoCambioItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<TipoCambioItem[]>("load-tipos-cambio", params)            
            .toPromise();
    }

    updateTiposCambio(data: TipoCambioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<any>('update-tipos-cambio', params, data)
            .toPromise();
    }
}

