﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { EmpresaInfo,RegimenFiscalItem } from '../models/empresa'
import { EjercicioItem } from '../models/ejercicio'
import { PeriodoItem } from '../models/periodo'


@Injectable()
export class EmpresaService extends CatalogService<EmpresaInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/contabilidad/empresas");
    }

    create(): Promise<EmpresaInfo> {
        return Promise.resolve(new EmpresaInfo());
    }
    	
    getPeriodoActual(idEmpresa: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            params.set('IdEmpresa', idEmpresa.toString());
        return this.apiService.get<any>('get-periodo-actual', params)            
            .toPromise();
    }

    loadEjercicios(idEmpresa: number, soloAbiertos: boolean): Promise<EjercicioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdEmpresa', idEmpresa.toString())
            .set('SoloAbiertos', soloAbiertos.toString());
        return this.apiService.get<EjercicioItem[]>('load-list-ejercicios', params)            
            .toPromise();
    }

    nuevoEjercicio(idEmpresa: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdEmpresa', idEmpresa.toString());
        return this.apiService.post<any>('nuevo-ejercicio', params, null)
            .toPromise();
    }

    abrirEjercicio(idEjercicio:string):Promise<any>{    
        let params: HttpParams = new HttpParams()
            .set('IdEjercicio', idEjercicio.toString());
        return this.apiService.post<any>('abrir-ejercicio', params, null)
            .toPromise();
    }

    cerrarEjercicio(idEjercicio: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdEjercicio', idEjercicio.toString());
        return this.apiService.post<any>('cerrar-ejercicio', params, null)
            .toPromise();
    }

    cargarPeriodos(idEjercicio: string, soloAbiertos: boolean): Promise<PeriodoItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdEjercicio', idEjercicio.toString())
            .set('SoloAbiertos', soloAbiertos.toString());
        return this.apiService.get<PeriodoItem[]>('cargar-periodos', params)            
            .toPromise();
    }

    abrirPeriodo(idPeriodo: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdPeriodo', idPeriodo.toString());
        return this.apiService.post<any>('abrir-periodo', params, null)
            .toPromise();
    }

    cerrarPeriodo(idPeriodo: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdPeriodo', idPeriodo.toString());
        return this.apiService.post<any>('cerrar-periodo', params, null)
            .toPromise();
    }

    loadRegimenesFiscales(idEmpresa: number): Promise<RegimenFiscalItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<RegimenFiscalItem[]>("load-regimenes-fiscales", params)            
            .toPromise();
    }

}

