﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Kernel
import { Context, CatalogService, FileLink } from 'framework3/core';
//Locales
import { ImpuestoInfo, ImpuestoTipoItem, ImpuestoItem } from '../models/impuesto'
import { IEPSPreview } from '../models/ieps-preview'
import { AsyncTaskInfo } from 'framework3/queue/models/async-task';

@Injectable()
export class ImpuestoService extends CatalogService<ImpuestoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/contabilidad/impuestos");
    }

    create(): Promise<ImpuestoInfo> {
        return Promise.resolve(new ImpuestoInfo());
    }

    loadTipos(idEmpresa: string): Promise<ImpuestoTipoItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa);
        return this.apiService.get<ImpuestoTipoItem[]>("load-tipos", params)            
            .toPromise();
    }
    
    searchKeywordImpuestos(idBranch: number, keyword: string): Observable<ImpuestoItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idBranch', idBranch.toString())
            .set('keyword', keyword);
        return this.apiService.get<ImpuestoItem[]>('search-keyword', params)
            
    }

    previewIeps(idEmpresa: number, fechaIni: Date, fechaFin: Date): Promise<IEPSPreview[]> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString())
            .set("fechaIni", moment(fechaIni).format())
            .set("fechaFin", moment(fechaFin).format());
        return this.apiService.get<IEPSPreview[]>('ieps-preview', params)
            .toPromise();
    }

    archivoIeps(nombreArchivo: string, fechaIni: Date, fechaFin: Date, rfcs: IEPSPreview[]): Promise<FileLink> {
        let params: HttpParams = new HttpParams()
            .set("nombreArchivo", nombreArchivo)
            .set("fechaIni", moment(fechaIni).format())
            .set("fechaFin", moment(fechaFin).format());
        return this.apiService.post<FileLink>("file-ieps", params, rfcs)
            .toPromise();
    }

    asyncPreviewIeps(idEmpresa: number, fechaIni: Date, fechaFin: Date): Promise<AsyncTaskInfo> {
        let params: HttpParams = new HttpParams()
            .set("idEmpresa", idEmpresa.toString())
            .set("fechaIni", moment(fechaIni).format())
            .set("fechaFin", moment(fechaFin).format());
        return this.apiService.get<AsyncTaskInfo>('get-async-ieps-preview', params)
            .toPromise();
    }

}

