﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Locales
import { NotaInfo, NotaItem } from '../models/nota';
import { NotaDetail } from '../models/nota-detail';
import { NotaRelaciones } from '../models/nota-relaciones';

@Injectable()
export class NotaService extends DocumentService<NotaDetail, NotaInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/contabilidad/notas');
    }

    create(): Promise<NotaInfo> {
        let params: HttpParams = new HttpParams();
        return this.apiService.get<NotaInfo>("create", params)            
            .toPromise();
    }

    ListarComprobantes(idNota: string): Promise<NotaRelaciones[]> {
        let params: HttpParams = new HttpParams()
            .set("idNota", idNota);
        return this.apiService.get<NotaRelaciones[]>("listar-comprobantes", params)            
            .toPromise();
    }

    AgregarComprobante(idNota: string, idComprobanteFiscal: string, uuid: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("idNota", idNota)
            .set("idComprobanteFiscal", idComprobanteFiscal)
            .set("uuid", uuid);
        return this.apiService.post<any>("agregar-comprobante", params,'')
            .toPromise();
    }

    EliminarComprobante(idNotaRelacion: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("idNotaRelacion", idNotaRelacion.toString());
        return this.apiService.delete("eliminar-comprobante", params)
            .toPromise();
    }


    updateNota(document: NotaInfo): Promise<NotaInfo> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<NotaInfo>("update-nota", params, document)            
            .toPromise();
    }


	/*
    sampleMethod(param: string): Promise<MovimientoInfo> {
        let params: URLSearchParams = new URLSearchParams();
        params.set('param', param);                
        return this.apiService.getData('sample-method', params)
            .map(response => response.json() as MovimientoInfo)
            .toPromise();
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: NotaInfo) {

    }
	*/



}