﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';

//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Locales
import { PolizaInfo, PolizaItem } from '../models/poliza';
import { PolizaPerfilItem } from '../models/poliza-perfil';
import { PolizaDetail } from '../models/poliza-detail';
import { PolizaLogEntry } from '../models/poliza-log-entry';
import { AsociacionItem } from '../models/asociacion';
import { PolizaTipo } from '../models/poliza-tipo';


@Injectable()
export class PolizaService extends DocumentService<PolizaDetail, PolizaInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/contabilidad/polizas-tit');
    }

    create(): Promise<PolizaInfo> {
        let newDoc = new PolizaInfo();
        return Promise.resolve(newDoc);
    }

	
    marcarExportacion(idPolizaTit: string, status: number, mensaje:string ): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idPolizaTit', idPolizaTit)               
            .set('status', status.toString())
            .set('mensaje', mensaje.toString());        
        return this.apiService.post<any>('marcar-exportacion', params, null)
            .toPromise();
    }

    loadLog(idPolizaTit: string): Promise<PolizaLogEntry[]> {
        let params: HttpParams = new HttpParams()
            .set("idPolizaTit", idPolizaTit);
        return this.apiService.get<PolizaLogEntry[]>("load-log", params)            
            .toPromise();
    }

    loadAsociaciones(idPolizaTit: string): Promise<AsociacionItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idPolizaTit", idPolizaTit);
        return this.apiService.get<AsociacionItem[]>("load-asociaciones", params)            
            .toPromise();
    }

    loadPerfilesByLote(idSucursal: number, idUsuario:number): Promise<PolizaPerfilItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idSucursal", idSucursal.toString())
            .set("idUsuario", idUsuario.toString());
        return this.apiService.get<PolizaPerfilItem[]>("load-perfiles-by-lote", params)            
            .toPromise();
    }

    loadPerfiles(idSucursal: number, idUsuario: number): Promise<PolizaPerfilItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idSucursal", idSucursal.toString())
            .set("idUsuario", idUsuario.toString());
        return this.apiService.get<PolizaPerfilItem[]>("load-perfiles", params)
            .toPromise();
    }

    contabilizarByLote(idSucursal: number, idPerfil: number, fechaIni: Date, fechaFin: Date): Promise<PolizaInfo> {
        let fi = moment(fechaIni).format();
        let ff = moment(fechaFin).format();
        
        let params: HttpParams = new HttpParams()
            .set("IdSucursal", idSucursal.toString())
            .set("IdPerfil", idPerfil.toString())
            .set("FechaIni",fi)
            .set("FechaFin",ff);
        return this.apiService.get<PolizaInfo>("load-contabilizacion-by-lote", params)            
            .toPromise();
    }

    getPolizaByPerfil(idSucursal: number, idPerfil: number, fecha: Date): Promise<PolizaInfo> {
        let f = moment(fecha).format();

        let params: HttpParams = new HttpParams()        
            .set("IdSucursal", idSucursal.toString())
            .set("IdPerfil", idPerfil.toString())
            .set("Fecha", f);        
        return this.apiService.get<PolizaInfo>("get-poliza-by-perfil", params)            
            .toPromise();
    }

    loadContabilizacionByMapa(idSucursal: number, idPerfil: number, fecha: Date): Promise<PolizaInfo> {
        let f = moment(fecha).format();

        let params: HttpParams = new HttpParams()
            .set("IdSucursal", idSucursal.toString())
            .set("IdPerfil", idPerfil.toString())
            .set("FechaIni", f);
        return this.apiService.get<PolizaInfo>("load-contabilizacion-by-mapa", params)
            .toPromise();
    }

    loadPolizaByDocumento(idDocumento: string, idPolizaPerfil: number, idBranch: number): Promise<PolizaInfo>  {
        let params = new HttpParams()
            .set("idDocumento", idDocumento)
            .set("idPolizaPerfil", idPolizaPerfil.toString())
            .set("idSucursal", idBranch.toString());
        return this.apiService.get<PolizaInfo>("get-poliza-by-documento", params).toPromise();
    }

    loadPolizaTipo(): Promise<PolizaTipo[]> {
        let params: HttpParams = new HttpParams()
        return this.apiService.get<PolizaTipo[]>("load-types", params).toPromise();
    }

    addAsociacion(idPoliza: string, fechaSys: Date, idCuenta: number, consecutivo:number, uuid: string, idCFDSerie:number, folio:number) {
        let f = moment(fechaSys).format();

        let params: HttpParams = new HttpParams()
            .set('IdPolizaTit', idPoliza)
            .set("FechaSys", f)
            .set('IdCuenta', idCuenta.toString())
            .set('Consecutivo', consecutivo.toString())
            .set('uuid', uuid)
            .set('IdCFDSerid', idCFDSerie.toString())
            .set('Folio', folio.toString());

        return this.apiService.post<any>('add-asociacion', params, null)
            .toPromise();
    }

    removeAsociacion(idPoliza: string, fechaSys: Date, idCuenta: number, consecutivo: number, uuid: string) {
        let f = moment(fechaSys).format();

        let params: HttpParams = new HttpParams()
            .set('IdPolizaTit', idPoliza)
            .set("FechaSys", f)
            .set('IdCuenta', idCuenta.toString())
            .set('Consecutivo', consecutivo.toString())
            .set('uuid', uuid);            

        return this.apiService.post<any>('remove-asociacion', params, null)
            .toPromise();
    }


	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: PolizaInfo) {

    }
	*/



}