﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
//
import { Context, UsuarioService } from 'framework3/core';
import { AlertsBarComponent } from 'framework3/controls';

@Component({
    selector: 'change-password',
    templateUrl: './change-password.component.html'
})
export class ChangePasswordComponent {
    validControls: boolean;
    form: FormGroup;
    passwordActual: string = '';
    nuevoPassword: string = '';
    confirmacion: string = '';

    @Input() idUsuario: number;

    @Output() passwordChange = new EventEmitter();
    @Output() canceled = new EventEmitter();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    get valid(): boolean {
        return this.validControls && this.nuevoPassword == this.confirmacion;
    }

    constructor(private context: Context, private usuarioService: UsuarioService, private formBuilder: FormBuilder) {
        this.form = formBuilder.group({
            'actual': ['', Validators.required],
            'nueva': ['', Validators.required],
            'confirmacion': ['', Validators.required]
        });

        this.form.statusChanges.subscribe(_ => {
            setTimeout(_ => this.validControls = this.form.valid);
        });
    }

    changePassword() {
        this.context.app.showSpinner();
        this.usuarioService.changePassword(this.idUsuario, this.passwordActual, this.nuevoPassword)
            .then(_ => {
                this.context.app.hideSpinner();
                this.passwordChange.emit();
            })
            .catch(error => this.handleError(error));
    }

    cancel() {
        this.canceled.emit();
    }

    handleError(error: any) {
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}
