﻿import { Component, EventEmitter, Input, Output, ViewChild, OnInit } from '@angular/core';
import { Router } from '@angular/router';

import { Context, Server, UsuarioService, UsuarioInfo, SessionInfo, SucursalService, EmpresaService, ModuloService } from 'framework3/core';

@Component({
    selector: 'login-form',
    templateUrl: './login.component.html'
})
export class LoginComponent {
    //Propiedades
    UserName: string;
    Password: string;

    //Eventos
    @Output() UserAuthenticated = new EventEmitter<SessionInfo>();

    //Campos privados
    error: string = "";

    constructor(private router: Router, private context: Context, private server: Server, private usuarioService: UsuarioService, private sucursalService: SucursalService,
        private empresaService: EmpresaService, private moduloService: ModuloService) { }

    ngOnInit() {
        this.context.app.hideSpinner();
    }

    loginUser() {
        this.context.app.showSpinner();

        this.usuarioService.login(this.UserName, this.Password, "")
            .then(session => {
                this.context.app.hideSpinner();
                if (session != null) {
                    this.context.setSession(session);
                    this.UserAuthenticated.emit(session);
                    if (session.user.empresas.length > 1 || session.user.sucursales.length > 1)
                        this.router.navigate(['/select-company']);
                    else {
                        if (session.user.empresas.length == 1 && session.user.sucursales.length == 1) {
                            this.redirect();
                        }
                        else {
                            this.error = "No tiene configurado acceso a ninguna sucursal";
                        }
                    }
                }
                else {
                    this.error = "Nombre de usuario o contraseña incorrecta";
                }
            })
            .catch(error => this.handleError(error));
    }

    redirect() {
        let idEmpresa: number = this.context.session.user.empresas[0].id;
        let idSucursal: number = this.context.session.user.sucursales[0].id;
        this.context.app.showSpinner();
        //Cargar información de la empresa
        let promesaEmp = this.empresaService.getDetailByID(idEmpresa)
            .then(empresa => {
                this.context.app.setDefaultCompany(empresa);
            })
            .catch(reason => this.handleError(reason));

        //Cargar información de sucursal
        let promesaSuc = this.sucursalService.getDetailByID(idSucursal)
            .then(sucursal => {
                this.context.app.setDefaultSite(sucursal);
            })
            .catch(reason => this.handleError(reason));

        //Cargar módulos raíz
        let promesaRoots = this.moduloService.loadRootModules()
            .then(roots => this.context.app.setRootModules(roots))
            .catch(error => this.handleError(error));

        //Cargar accesos a menú
        let promesaAccesos = this.moduloService.loadMenuModules(idEmpresa, idSucursal, this.context.session.user.identity)
            .then(accesos => this.context.setAccess(accesos))
            .catch(error => this.handleError(error));

        //Cuando ambas promesas terminen, ir a menú principal
        Promise.all<void>([promesaEmp, promesaSuc, promesaRoots, promesaAccesos]).then(() => {
            this.context.app.hideSpinner();
            if (this.context.redirectUrl)
                window.location.href = this.context.redirectUrl;
            else
                this.router.navigate(['/kernel/analytic/dashboard']);
        });
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        console.log(error);
        this.error = this.context.getExceptionMessage(error);
    }
}
