﻿import { Component, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { switchMap } from 'rxjs/operators';
//
import { Context, SessionInfo, UsuarioService, UsuarioInfo, ModuloService, ModuloInfo, ModuloItem, CategoriaModulos, ProcesoItem, ProcesoAcceso, Server } from 'framework3/core';

@Component({
    selector: 'main-form',
    templateUrl: './main.component.html'
})
export class MainComponent {
    categorias: CategoriaModulos[] = [];
    childrenModules: ModuloItem[] = [];

    constructor(private context: Context, private router: Router, private route: ActivatedRoute, private moduloService: ModuloService, private serverSvc: Server) { }

    ngAfterViewInit() {
        setTimeout(_ => {

            this.moduloService.loadCategories()
                .then(categorias => this.categorias = categorias)
                .catch(error => this.handlerError(error));

            this.serverSvc.getParamater('System/Reports', 'ReportViewerUrl')
                .then(value => {
                    this.context.setReportViewerUrl(value);
                });
        });
        /*Checar si se pasó un root a cargar*/
        this.route.params.pipe(switchMap((params: Params) => {
            let rootId: number = 0;
            if (params['rootModule']) rootId = +params['rootModule'];
            if (rootId > 0) {
                this.context.app.currentModule = rootId;
                return Promise.resolve(this.context.access.filter(modulo => modulo.idPadre == rootId));
            }
            else {
                return Promise.resolve(this.context.access.filter(modulo => modulo.idPadre == this.context.app.currentModule));
            }
        }))
        .subscribe(children => {
            this.childrenModules = children
        });
    }

    cargarCategorias() {
        this.moduloService.loadCategories()
            .then(categorias => this.categorias = categorias)
            .catch(error => this.handlerError(error));
    }


    onRootModuleSelected(root: ModuloItem) {
        this.context.app.currentModule = root.id;
        this.childrenModules = this.context.access.filter(modulo => modulo.idPadre == root.id);
    }

    parseParameters(paramsJson: string): { [param: string]: string } {
        if (paramsJson && paramsJson != '') {
            let params = JSON.parse(paramsJson) as { [param: string]: string };
            return params;
        }
        else {
            return {};
        }
    }


    executeProc(proceso: ProcesoAcceso) {
        let params = this.parseParameters(proceso.parametros);
        this.router.navigate([proceso.url, proceso.id, params]);
        return false;
    }

    handlerError(error: any) {
        this.context.app.hideSpinner();
    }

}