﻿import { Component, Input, Output, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Location } from '@angular/common';

import { Context, SessionInfo, UsuarioService, UsuarioInfo, EmpresaItem, EmpresaService, SucursalItem, SucursalService, ModuloService } from 'framework3/core';

@Component({
    selector: 'select-company-form',
    templateUrl: './select-company.component.html'    
})
export class SelectComanyComponent {
    empresas: EmpresaItem[];
    sucursales: SucursalItem[];
    sucursalesEmpresa: SucursalItem[];

    empresaSelected: number;
    sucursalSelected: number;

    constructor(private router: Router, private location: Location, private context: Context,
        private empresaService: EmpresaService, private sucursalService: SucursalService, private moduloService: ModuloService) {
    }

    ngOnInit(): void {
        this.empresas = this.context.session.user.empresas.slice();
        this.sucursales = this.context.session.user.sucursales.slice();
        //Agregar empresa default
        this.empresas.unshift({ id: 0, codigo: '', nombre: '[Seleccione una empresa]', activo: true });
        //
        //Cargar módulos raíz
        this.moduloService.loadRootModules()
            .then(roots => {
                this.context.app.setRootModules(roots);
            })
            .catch(error => this.handleError(error));

    }

    selectEmpresa(idEmpresa: number) {
        this.sucursalesEmpresa = this.sucursales.filter(
            (sucursal, index, sucursales) => sucursal.idEmpresa == idEmpresa
        );
        this.sucursalesEmpresa.unshift({ id: null, idEmpresa: 0, codigo: '', nombre: '[Seleccione una sucursal]', activo: true });
        this.sucursalSelected = null;
    }

    goMainMenu() {
        if (this.empresaSelected > 0 && this.sucursalSelected > 0) {
            this.context.app.showSpinner();

            //Cargar accesos a menú
            this.moduloService.loadMenuModules(this.empresaSelected, this.sucursalSelected, this.context.session.user.identity)
                .then(accesos => {
                    this.context.setAccess(accesos);
                    //Cargar información de la empresa
                    let promesaEmp = this.empresaService.getDetailByID(this.empresaSelected)
                        .then(empresa => {
                            this.context.app.setDefaultCompany(empresa);
                        })
                        .catch(reason => this.handleError(reason));

                    //Cargar información de sucursal
                    let promesaSuc = this.sucursalService.getDetailByID(this.sucursalSelected)
                        .then(sucursal => {
                            this.context.app.setDefaultSite(sucursal);
                        })
                        .catch(reason => this.handleError(reason));

                    //Cuando ambas promesas terminen, ir a menú principal
                    Promise.all<void>([promesaEmp, promesaSuc]).then(() => {
                        this.context.app.hideSpinner();
                        if (this.context.redirectUrl)
                            window.location.href = this.context.redirectUrl;
                        else
                            this.router.navigate(['/kernel/analytic/dashboard']);
                    });
                })
                .catch(error => this.handleError(error));
        }
    }


    handleError(reason: any) {
        this.context.app.hideSpinner();
        console.log(reason);
    }

    goBack() {
        this.location.back();
    }
}