import { Component, ViewContainerRef, Compiler, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { ModalDirective } from 'ngx-bootstrap';

import { Context, UsuarioService, ModuloItem, ProcesoItem } from 'framework3/core';
import { MessageService, MessageItem, MessageInfo } from 'framework3/notification';
//
@Component({
    selector: 'my-app',
    templateUrl: './app.component.html'
})
export class AppComponent {
    newMessagesQuantity: number = 0;
    newMessages: MessageItem[] = [];
    @ViewChild(ModalDirective) changePasswordDialog: ModalDirective;

    get isLoading(): boolean {
        return this.context.app.isLoading;
    }

    get containerClass(): string {
        let css: string = "container-fluid ";
        if (this.isLoading) css += " loading";
        return css;
    }

    get userName(): string {
        if (this.context.session) {
            return this.context.session.user.code;
        }
        else {
            return 'Annimo';
        }
    }

    get userPhoto(): string {
        if (this.context.session) {
            if (this.context.session.user.foto.fileUri != '')
                return '/file-server/' + this.context.session.user.foto.fileUri;
            else
                return '/assets/img/kernel/user.png';
        }
        else {
            return '/assets/img/kernel/user.png';
        }
    }

    get userId(): number {
        return this.context.session ? this.context.session.user.identity : 0;
    }

    get currentCompany(): string {
        return this.context.app ? this.context.app.defaultCompany.nombre : '';
    }

    get currentSite(): string {
        return this.context.app ? this.context.app.defaultSite.nombre : '';
    }

    constructor(private context: Context, private router: Router, private compiler: Compiler,
        private usuarioService: UsuarioService, private messageService: MessageService) {
        this.compiler.clearCache();
    }
    /////////// METODOS //////////////////////////////////////////////

    ngAfterViewInit() {
        setInterval(_ => this.checkNewMessages(), 60000);
    }

    parseParameters(paramsJson: string): { [param: string]: string } {
        if (paramsJson && paramsJson != '') {
            let params = JSON.parse(paramsJson) as { [param: string]: string };
            return params;
        }
        else {
            return {};
        }
    }


    isActive(data: any[]): boolean {
        return this.router.isActive(this.router.createUrlTree(data), true);
    }

    onRootModuleSelected(root: ModuloItem) {
        this.router.navigate(['main', { rootModule: root.id }]);
        return false;
    }

    closeSession() {
        this.usuarioService.logout(this.context.session.user.identity, this.context.session.workStation)
            .then(_ => {
                this.context.clearSession();
                this.router.navigate(['login']);
            })
            .catch(error => this.handleError(error));
        return false;
    }

    showcChangePassword() {
        this.changePasswordDialog.show();
        return false;
    }

    checkNewMessages() {
        if (this.context.session) {
            this.messageService.hasNewMessages(this.context.session.user.identity)
                .then(quantity => {
                    this.newMessagesQuantity = quantity;
                    this.messageService.getNewMessages(this.context.session.user.identity)
                        .then(items => {
                            this.newMessages = items;
                        })
                        .catch(error => this.handleError(error));
                })
                .catch(error => this.handleError(error));
        }
    }

    showInbox() {
        return false;
    }

    showMessage(idMensaje: number) {
        return false;
    }

    /////////// HANDLERS //////////////////////////////////////////////

    onPasswordChanged() {
        this.changePasswordDialog.hide();
        this.usuarioService.logout(this.context.session.user.identity, this.context.session.workStation)
            .then(_ => {
                this.context.clearSession();
                this.router.navigate(['login']);
            })
            .catch(error => this.handleError(error));
    }

    onExecuteProc(proceso: ProcesoItem) {
        //Verificar que el usuario tiene acceso al proceso
        if (this.context.session) {
            let hasAccess: boolean = false;
            this.context.access.forEach(modulo => {
                if (modulo.procesos.find(p => p.id == proceso.id)) {
                    hasAccess = true;
                }
            });

            if (hasAccess) {
                let params = this.parseParameters(proceso.parametros);
                this.router.navigate([proceso.url, proceso.id, params]);
            }
        }
    }

    handleError(error: any) {
        console.error(error);
    }

}
