﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, DownloadPopupComponent } from 'framework3/controls';
//Locales

import { ComprobanteFiscalInfo, ComprobanteFiscalItem, ComprobanteFiscalService } from '../../accounting';
import { PortalContext } from '../services/portal-context';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';


@Component({
    selector: 'clientes-download-invoice',
    templateUrl: './download-invoice.component.html'
})
export class DownloadInvoiceComponent {
    form: FormGroup;
    menuItems: MenuItem[] = [];
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    idDocumento: string;
    pageSize: number = 20;
    maxRows: number = 3000;
    totalRows: number = 0;
    comprobantes: ComprobantesPortalClientesItem[] = [];
    selectedGroup: FilterGroup;
    filterGroups: FilterGroup[] = [];

    currentComprobante = new ComprobanteFiscalInfo();

    //
    @ViewChild('viewDialog') viewDialog: ModalDirective;
    @ViewChild(DocumentListComponent) documentList: DocumentListComponent;
    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder,
        private comprobanteService: ComprobanteFiscalService, private portalContext: PortalContext) {
        //Crear formulario de parámetros
        this.form = formBuilder.group({
            'fechaInicial': [new Date()],
            'fechaFinal': [new Date()]
        });
    }

    ngOnInit() {
        //this.createMenu();
    }

    ngAfterViewInit() {
        this.fechaInicial = moment().subtract(15, 'day').toDate();
        this.fechaFinal = moment().add(1, 'day').toDate();
        setTimeout(_ => this.refreshList());
    }

    /////////////////////// METODOS ////////////////////////////////////

    refreshList() {
        this.context.app.showSpinner();

        this.comprobanteService.portalClientes(this.portalContext.cliente.identity, this.fechaInicial, this.fechaFinal)
            .then(response => {
                this.context.app.hideSpinner();
                //
                response.forEach(x => x.fechaComprobante = moment(x.fechaComprobate).toDate())
                response.forEach(x => x.fechaTimbrado = moment(x.fechaTimbrado).toDate())
                this.comprobantes = response;
                //this.totalRows = response.totalRows;
                //this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    /*createMenu() {
        //Agregar opciones estándar del catálogo
        let mnuView: MenuItem = { icon: "glyphicon glyphicon-folder-open", text: "Ver", name: "OPEN", smallText: "ver resumen del comprobante" };

        this.menuItems.push(mnuView);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });

    }*/

    viewComprobante() {
        this.context.app.showSpinner();
        this.comprobanteService.getDetail(this.documentList.currentDocument.id)
            .then(comprobante => {
                this.currentComprobante = comprobante;
                this.context.app.hideSpinner();
                this.viewDialog.config.backdrop = false;
                this.viewDialog.show();
            })
            .catch(error => this.handleError(error));
    }

    /////////////////////// HANDLERS //////////////////////////////////

    onMainToolbar(button: ToolbarButton) {
        if (button.name == 'DOWNLOAD') {
            this.context.app.showSpinner();
            //Generar PDF
            this.comprobanteService.generarPdfTemporal(this.idDocumento)
                .then(pdf => {
                    this.context.app.hideSpinner();
                    //Descargar xml
                    this.comprobanteService.getXmlFile(this.idDocumento)
                        .then(xml => {
                            let links: FileLink[] = [];
                            links.push(pdf);
                            links.push(xml);
                            this.downloadPopup.show(links);
                        })
                        .catch(error => this.handleError(error));

                })
                .catch(error => this.handleError(error));
        }
    }

    /*optionClick(option: MenuItem) {
        if (option.name == 'OPEN') {
            this.viewComprobante();
        }
    }*/

    onValidateMenu() {

    }

    currentItemChangedHandler(item: any) {
        this.idDocumento = item.id;
    }

    handleError(error: any) {
        console.log(error);
        let message = this.context.getExceptionMessage(error);
    }

}

export class ComprobantesPortalClientesItem {
    id: string;
    fechaComprobante: Date;
    fechaTimbrado: Date;
    serie: string;
    folio: number;
}