﻿import { Component, EventEmitter, Input, Output, ViewChild, OnInit } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router } from '@angular/router';
//
import { Context, UsuarioService, UsuarioInfo, EmpresaService, EmpresaInfo } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
//
import { ClienteService } from '../../sales/services/cliente.service';
import { PortalContext } from '../services/portal-context';

@Component({
    selector: 'clientes-login',
    templateUrl: './login.component.html'
})
export class LoginComponent {
    loginForm: FormGroup;
    correo: string;
    password: string;
    error: string;

    constructor(private router: Router, private context: Context, private formBuilder: FormBuilder,
        private portalContext: PortalContext, private usuarioService: UsuarioService, private empresaService: EmpresaService,
        private clienteService: ClienteService) {

        this.loginForm = formBuilder.group({
            'correo': ['', Validators.email],
            'password': ['', Validators.required]
        });
    }

    ngOnInit() {
        this.context.app.hideSpinner();
    }

    checkLogin() {
        this.context.app.showSpinner();

        this.usuarioService.login(this.correo, this.password, 'web')
            .then(session => {
                this.context.app.hideSpinner();
                if (session != null) {
                    this.context.setSession(session);
                    //Cargar empresas a las que el usuario asociado al cliente puede ingresasr
                    this.clienteService.getDetailByUser(this.context.session.user.identity, this.context.session.user.idCompany)
                        .then(cliente => {
                            this.portalContext.setCliente(cliente);
                            this.router.navigate(['clientes/download']);
                        })
                        .catch(error => this.handleError(error));
                }
                else {
                    this.error = 'Nombre de usuario o contraseña incorrecta';
                }
            })
            .catch(error => this.handleError(error));
    }

    checkReg() {
        this.router.navigate(['clientes/pre-registro']);
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        console.log(error);
        this.error = this.context.getExceptionMessage(error);
    }

}