﻿import { Component, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//
import { Context, Server } from 'framework3/core';
import { AlertsBarComponent } from 'framework3/controls';
import { ClienteInfo } from '../../sales/models/cliente';
import { ClienteService } from '../../sales/services/cliente.service';

@Component({
    selector: 'clientes-registro',
    templateUrl: './registro.component.html'
})
export class RegistroComponent {
    form: FormGroup;
    password: string = '';
    confirmacion: string = '';
    idCliente: number;
    expira: number;
    enlaceCaducado = false;
    registrado = false;
    cliente = new ClienteInfo();

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute,
        private context: Context, private server: Server, private clienteService: ClienteService) {
        this.form = this.formBuilder.group({
            'password': ['', Validators.required],
            'confirmacion': ['', Validators.required]
        });

        this.route.params.forEach((params: Params) => {
            this.idCliente = +params['cliente'];
            this.expira = +params['expira'];
        });
    }

    ngOnInit() {
        this.cargarSystemSession();
    }

    cargarCliente() {
        this.clienteService.getDetailByID(this.idCliente)
            .then(cliente => {
                this.cliente = cliente;
                //Validar si el cliente ya tiene un usuario asignado
                if (this.cliente.usuario.identity > 0) {
                    this.registrado = true;
                    this.alertsBar.showAlert('Cliente ya registrado en el portal', 'warning');
                }
            })
            .catch(error => this.handleError(error));
    }

    validarCaducidadLink() {
        let fechaCaducidad = new Date(this.expira);
        let today = new Date();
        if (today > fechaCaducidad) {
            this.alertsBar.showAlert('El enlace ya ha caducado.', 'warning');
            this.enlaceCaducado = true;
        }
        else {
            setTimeout(_ => this.cargarCliente());
        }
    }

    cargarSystemSession() {
        this.server.getSystemSession()
            .then(session => {
                this.context.setSession(session);
                this.validarCaducidadLink();
            })
            .catch(error => this.handleError(error));
    }

    ConfirmarRegistro() {
        this.clienteService.registrar(this.idCliente, this.password)
            .then(_ => {
                this.registrado = true;
                this.alertsBar.showAlert('El registro se ha completado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }
}