﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { CajeroService} from '../services/cajero.service';
import { CajeroInfo, CajeroItem} from '../models/cajero';

@Component({
    selector: 'credito-cajeros-editor',
    templateUrl: './cajeros.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CajerosEditor)
        },
        {
            provide: CatalogService, useExisting: CajeroService
        }
    ]
})
export class CajerosEditor extends CatalogEditor {
    cajero: CajeroInfo = new CajeroInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private cajeroService: CajeroService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'usuario': ['', Validators.required],
            'empleado': ['', Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.cajeroService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cajero = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.cajeroService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cajero = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.cajeroService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cajero = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.cajeroService.update(this.cajero)
            .then(info => {
                this.context.app.hideSpinner();
                this.cajero = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

}