﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { DepositoTipoService} from '../services/deposito-tipo.service';
import { DepositoTipoInfo, DepositoTipoItem} from '../models/deposito-tipo';

@Component({
    selector: 'credito-deposito-tipo-editor',
    templateUrl: './depositos-tipos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DepositosTiposEditor)
        },
        {
            provide: CatalogService, useExisting: DepositoTipoService
        }
    ]
})
export class DepositosTiposEditor extends CatalogEditor {
    depositoTipo: DepositoTipoInfo = new DepositoTipoInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private depositoTipoService: DepositoTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.depositoTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.depositoTipo = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.depositoTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.depositoTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.depositoTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.depositoTipo = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.depositoTipoService.update(this.depositoTipo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.depositoTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.depositoTipo = info;
            })
            .catch(error => this.handleError(error));        
    }

}