﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, TrackableItem } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { DeudorService } from '../services/deudor.service';
import { DeudorInfo, DeudorItem } from '../models/deudor';
import { GrupoDeudorInfo } from '../models/grupo-deudor';

import { FormaPagoService } from '../services/forma-pago.service';
import { FormaPagoItem } from '../models/forma-pago';

import { GrupoDeudorSelectorComponent } from '../components/grupo-deudores-selector.component';

import { AddendaService } from '../../accounting/services/addenda.service';
import { ComprobanteFiscalService } from '../../accounting/services/comprobante-fiscal.service';
import { DomicilioComprobanteInfo } from '../models/domicilio-comprobante';

import { ComprobanteCorreo } from '../../accounting/models/comprobante-correo';
import { ComprobantesCorreoEditor } from '../../accounting/components/comprobante-correo.editor';

import { PerfilAutoCobroTipoItem, PerfilAutoCobroTipoInfo } from '../models/perfil-auto-cobro-tipo';
import { DeudorPerfilAutoCobro } from '../models/deudor-perfil-auto-cobro';
import { PerfilAutoCobroTipoService } from '../services/perfil-auto-cobro-tipo.service';

import { NotaTipoInfo, NotaTipoItem } from '../models/nota-tipo';
import { NotaTipoService } from '../services/nota-tipo.service';


@Component({
    selector: 'credito-deudores-editor',
    templateUrl: './deudores.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DeudoresEditor)
        },
        {
            provide: CatalogService, useExisting: DeudorService
        }
    ]
})
export class DeudoresEditor extends CatalogEditor {
    deudor: DeudorInfo = new DeudorInfo();
    condicionesPago: { id: string, nombre: string }[] = [];
    formasPago: FormaPagoItem[] = [];
    adendacbo: { id: number, nombre: string }[] = [];
    comprobantecbo: { id: number, nombre: string }[] = [];
    listdomiciliosComprobantes: { value: string, text: string }[] = [];
    listRazonSocial: { value: string, text: string }[] = [];
    metodosPago: { metodo: string }[] = [];
    comprobantesCorreosCollection: TrackableCollection = new TrackableCollection();
    perfilesAutocobranzaCollection: TrackableCollection = new TrackableCollection();
    idPerfil: number;
    tiposPerfiles: PerfilAutoCobroTipoItem[] = [];
    notaTipos: NotaTipoItem[] = [];

    @ViewChild('selGrupoDeudor') selInsumo: GrupoDeudorSelectorComponent;
    @ViewChild(ComprobantesCorreoEditor) comprobanteCorreoDialog: ComprobantesCorreoEditor;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private deudorService: DeudorService, private formaPagoService: FormaPagoService, private addendaService: AddendaService,
        private comprobanteFiscalService: ComprobanteFiscalService, private perfilAutoCobroTipoService: PerfilAutoCobroTipoService,
        private notaTipoSvc: NotaTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'condiciones': ['', Validators.required],
            'domicilio': [''],
            'limiteCredito': [''],
            'diasCredito': [''],
            'idFormaPago': [''],
            'cuentaPago': [''],
            'cuenta': [''],
            'idGrupoDeudor': [0],
            'facturable': [''],
            'rfc': [''],
            'razonSocial': [''],
            'desgloseIEPS': [''],
            'addenda': [''],
            'gln': [''],
            'glnTienda': [''],
            'glnComprador': [''],
            'numProveedor': [''],
            'comprobanteDestino': [''],
            'domicilioFiscal': [''],
            'domicilioSucursal': [''],
            'razonSocialComprobante': [''],
            'metodoPago': [''],
            'idPerfil': [0],
            'idNotaTipo': [0],
            'adeudoPrestamo': [0],
            'administraSaldo':[false],
        });

        this.condicionesPago =
            [
                { "id": 'C', "nombre": "Credito" },
                { "id": 'E', "nombre": "Contado" }
            ];
        this.listdomiciliosComprobantes =
            [
                { "value": "CONTRIBUYENTE", "text": "Domicilio del Contribuyente" },
                { "value": "GRUPO_DEUDOR", "text": "Domicilio del Grupo Deudor" },
                { "value": "DEUDOR", "text": "Domicilio del Cliente" }
            ];
        this.listRazonSocial =
            [
                { "value": "CONTRIBUYENTE", "text": "Razón social del Contribuyente" },
                { "value": "DEUDOR", "text": "Nombre del Deudor" }
            ];
        this.metodosPago =
            [
                {"metodo" : "PUE" },
                {"metodo" : "PPD" }
            ];
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadFormasPago();
        this.loadNotaTipos();
    }

    ngOnInit() {
        this.addendaService.AddendaList(this.context.app.defaultCompany.identity)
            .then(x => this.adendacbo = x)
            .catch(error => this.handleError(error))

        this.comprobanteFiscalService.ComprobantesDestinosList()
            .then(x => this.comprobantecbo = x)
            .catch(error => this.handleError(error))

        this.perfilAutoCobroTipoService.loadList(this.context.app.defaultCompany.identity)
            .then(x => this.tiposPerfiles = x)
            .catch(error => this.handleError(error))

    }

    ngAfterViewChecked() {
        this.comprobantesCorreosCollection.sourceCollection = this.deudor.comprobantesCorreos;
        this.perfilesAutocobranzaCollection.sourceCollection = this.deudor.perfilesAutoCobro;
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.deudorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.deudor = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.deudorService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.deudor = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.deudorService.getDetailByID(id)
            .then(info => {
                console.log(JSON.stringify(info));
                this.context.app.hideSpinner();
                this.deudor = info;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.deudorService.update(this.deudor)
            .then(info => {
                this.context.app.hideSpinner();
                this.deudor = info;
                this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }

    loadFormasPago() {
        this.formaPagoService.loadList(this.context.app.defaultSite.identity)
            .then(formas => this.formasPago = formas)
            .catch(error => this.handleError(error));
    }

    onGrupoDeudorSelected(grupo: GrupoDeudorInfo) {
        this.deudor.contribuyente = grupo.contribuyente;
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }
    //Correos
    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.comprobanteCorreoDialog.show();
        }
    }

    onAgregarComprobanteCorreo(args: ComprobanteCorreo) {
        let comprobanteExists: boolean = false;

        this.deudor.comprobantesCorreos.forEach(comproMail => {
            if (comproMail.idComprobanteTipo == args.idComprobanteTipo && comproMail.state !='DELETED') {
                this.showAlert("El tipo de comprobante ya fue agregado", "danger");
                comprobanteExists = true;
            }
        })
        if (!comprobanteExists) {
            this.deudor.comprobantesCorreos.push(args);
            this.comprobantesCorreosCollection.refresh();
        }
    }

    onRemoveItem(item) {
        this.comprobantesCorreosCollection.remove(item);
        this.comprobantesCorreosCollection.refresh();
    }

    onRemovePerfil(item) {
        this.perfilesAutocobranzaCollection.remove(item);
        this.perfilesAutocobranzaCollection.refresh();
    }

    onAgregarPerfil() {
        let perfilExists: boolean = false;

        this.deudor.perfilesAutoCobro.forEach(compro => {
            if (compro.idPerfilAutoCobro == this.idPerfil && compro.state != 'DELETED') {
                this.showAlert("El perfil ya fue agregado", "danger");
                perfilExists = true;
            }
        })

        if (!perfilExists) {
            this.perfilAutoCobroTipoService.getDetailByID(this.idPerfil)
                .then(perfil => {
                    let newP: DeudorPerfilAutoCobro;
                    newP = new DeudorPerfilAutoCobro();
                    newP.idPerfilAutoCobro = perfil.identity;
                    newP.perfil = perfil;

                    this.deudor.perfilesAutoCobro.push(newP);
                    this.perfilesAutocobranzaCollection.refresh();

                })
                .catch(error => this.handleError(error));
        }
            
    }

    loadNotaTipos() {
        this.notaTipoSvc.loadList(this.context.app.defaultCompany.identity)
            .then(tipos => this.notaTipos = tipos)
            .catch(error => this.handleError(error));
    }


    onTipoPerfilChanged(args: EditorEventArgs) {

    }

}