﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { FormaPagoService} from '../services/forma-pago.service';
import { FormaPagoInfo, FormaPagoItem} from '../models/forma-pago';
import { MetodoPagoItem } from '../../accounting/models/metodo-pago';
import { MetodoPagoService } from '../../accounting/services/metodo-pago.service';

@Component({
    selector: 'credito-formaspago-editor',
    templateUrl: './formas-pago.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FormasPagoEditor)
        },
        {
            provide: CatalogService, useExisting: FormaPagoService
        }
    ]
})
export class FormasPagoEditor extends CatalogEditor {
    formapago: FormaPagoInfo = new FormaPagoInfo();  
    metodosPago: MetodoPagoItem[] = [];  
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private formaPagoService: FormaPagoService, private metodoPagoService: MetodoPagoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idMetodoPago': ['', Validators.required],
            'cuenta': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadMetodosPago();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.formaPagoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.formapago = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.formaPagoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.formapago = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.formaPagoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.formapago = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.formaPagoService.update(this.formapago)
            .then(info => {
                this.context.app.hideSpinner();
                this.formapago = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    loadMetodosPago() {
        this.metodoPagoService.loadList(this.context.app.defaultCompany.identity)
            .then(metodos => this.metodosPago = metodos)
            .catch(error => this.handleError(error));        
    }
}