﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, CollectionView } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, TrackableItem } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { GrupoDeudorService} from '../services/grupo-deudor.service';
import { GrupoDeudorInfo, GrupoDeudorItem} from '../models/grupo-deudor';
import { SerieFiscalItem } from '../../accounting/models/serie-fiscal';
import { SerieFiscalService } from '../../accounting/services/serie-fiscal.service';
import { UsoCFDIItem } from '../../accounting/models/contribuyente';
import { ContribuyenteService } from '../../accounting/services/contribuyente.service';

import { PerfilAutoCobroTipoItem, PerfilAutoCobroTipoInfo } from '../models/perfil-auto-cobro-tipo';
import { DeudorPerfilAutoCobro } from '../models/deudor-perfil-auto-cobro';
import { PerfilAutoCobroTipoService } from '../services/perfil-auto-cobro-tipo.service';
import { DeudorItem } from '../models/deudor';

import { NotaTipoInfo, NotaTipoItem } from '../models/nota-tipo';
import { NotaTipoService } from '../services/nota-tipo.service';


@Component({
    selector: 'credito-grupos-deudores-editor',
    templateUrl: './grupos-deudores.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => GruposDeudoresEditor)
        },
        {
            provide: CatalogService, useExisting: GrupoDeudorService
        }
    ]
})
export class GruposDeudoresEditor extends CatalogEditor {
    grupodeudor: GrupoDeudorInfo = new GrupoDeudorInfo();        
    series: SerieFiscalItem[] = [];
    usosCFDI: UsoCFDIItem[] = [];
    notaTipos: NotaTipoItem[] = [];
    perfilesAutocobranzaCollection: TrackableCollection = new TrackableCollection();
    idPerfil: number;
    tiposPerfiles: PerfilAutoCobroTipoItem[] = [];
    deudores: DeudorItem[] = [];
    deudoresCollection: CollectionView = new CollectionView();

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private grupoDeudorService: GrupoDeudorService, private serieFiscalService: SerieFiscalService, private contribuyenteService: ContribuyenteService,
        private notaTipoSvc: NotaTipoService, private perfilAutoCobroTipoService: PerfilAutoCobroTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'limiteCredito': [''],
            'diasCredito': ['', Validators.required],
            'desglosaIEPS': [''],
            'rfc': [''],
            'razonSocial': [''],
            'domicilio': [''],
            'idCFDSerie': [''],
            'cuenta': [''],
            'idUsoCFDI': [0],
            'idNotaTipo': [0],
            'idPerfil': [0],
            'adeudoPrestamo': [0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadSeries();
        this.loadUsosCFDI();
        this.loadNotaTipos();

    }

    ngOnInit() {
        this.perfilAutoCobroTipoService.loadList(this.context.app.defaultCompany.identity)
            .then(x => this.tiposPerfiles = x)
            .catch(error => this.handleError(error))

    }

    ngAfterViewChecked() {
        this.perfilesAutocobranzaCollection.sourceCollection = this.grupodeudor.perfilesAutoCobro;
    }


    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoDeudorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupodeudor = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.grupoDeudorService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupodeudor = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoDeudorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupodeudor = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.grupoDeudorService.update(this.grupodeudor)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupodeudor = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    loadSeries() {
        this.serieFiscalService.loadList(this.context.app.defaultCompany.identity.toString())
            .then(series => this.series = series)
            .catch(error => this.handleError(error));
    }

    loadDeudores() {
        if (this.deudores.length == 0) {
            this.grupoDeudorService.getDeudoresList(this.grupodeudor.identity)
                .then(deudores => {
                    this.context.app.hideSpinner();
                    this.deudores = deudores;
                    this.deudoresCollection.sourceCollection = deudores;
                })
                .catch(error => this.handleError(error));
        }
    }

    loadUsosCFDI() {
        this.contribuyenteService.loadUsosCFDI(this.context.app.defaultCompany.identity)
            .then(usos => this.usosCFDI = usos)
            .catch(error => this.handleError(error));
    }


    loadNotaTipos() {
        this.notaTipoSvc.loadList(this.context.app.defaultCompany.identity)
            .then(tipos => this.notaTipos = tipos)
            .catch(error => this.handleError(error));
    }

    onSelectTab(args: any) {

        if (args.heading == 'Deudores del grupo') {
            this.loadDeudores();
            Control.invalidateAll();
        }

        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onRemovePerfil(item) {
        this.perfilesAutocobranzaCollection.remove(item);
        this.perfilesAutocobranzaCollection.refresh();
    }

    onTipoPerfilChanged(args: EditorEventArgs) {

    }

    onAgregarPerfil() {
        let perfilExists: boolean = false;

        this.grupodeudor.perfilesAutoCobro.forEach(compro => {
            if (compro.idPerfilAutoCobro == this.idPerfil && compro.state != 'DELETED') {
                this.showAlert("El perfil ya fue agregado", "danger");
                perfilExists = true;
            }
        })

        if (!perfilExists) {
            this.perfilAutoCobroTipoService.getDetailByID(this.idPerfil)
                .then(perfil => {
                    let newP: DeudorPerfilAutoCobro;
                    newP = new DeudorPerfilAutoCobro();
                    newP.idPerfilAutoCobro = perfil.identity;
                    newP.perfil = perfil;

                    this.grupodeudor.perfilesAutoCobro.push(newP);
                    this.perfilesAutocobranzaCollection.refresh();

                })
                .catch(error => this.handleError(error));
        }

    }

}