﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, CollectionView } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { NotaTipoService} from '../services/nota-tipo.service';
import { NotaTipoInfo, NotaTipoItem } from '../models/nota-tipo';
import { CobroTipoItem, CobroTipoInfo } from '../models/cobro-tipo';
import { NotaTipoConcepto } from '../models/nota-tipo-concepto';
import { CobroTipoService } from '../services/cobro-tipo.service';
import { NotaConceptoSelectorComponent } from '../../accounting/components/nota-concepto-selector.component';
import { NotaConceptoInfo } from '../../accounting/models/nota-concepto';

@Component({
    selector: 'credito-nota-tipo-editor',
    templateUrl: './notas-tipos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => NotasTiposEditor)
        },
        {
            provide: CatalogService, useExisting: NotaTipoService
        }
    ]
})
export class NotasTiposEditor extends CatalogEditor {
    notaTipo: NotaTipoInfo = new NotaTipoInfo();
    cobroTipos: CobroTipoItem[] = [];

    //conceptosCollection: CollectionView = new CollectionView();
    conceptosCollection: TrackableCollection = new TrackableCollection();

    @ViewChild('selectorNotaConcepto') selectorNotaConcepto: NotaConceptoSelectorComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private notaTipoService: NotaTipoService, private cobroTipoService: CobroTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idCobroTipo': [0, Validators.required],
            'idNotaConcepto': [0, Validators.required],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    
    loadCobroTipos() {
        this.cobroTipoService.loadList(this.context.app.defaultSite.identity, false)
            .then(tipos => this.cobroTipos = tipos)
            .catch(error => this.handleError(error));
    }
    
	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadCobroTipos();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.notaTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.notaTipo = info;
                this.conceptosCollection.sourceCollection = this.notaTipo.notaTipoConceptos;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.notaTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.notaTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.notaTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.notaTipo = info;         
                this.conceptosCollection.sourceCollection = this.notaTipo.notaTipoConceptos;
                this.conceptosCollection.refresh;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.notaTipoService.update(this.notaTipo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.notaTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.notaTipo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            var concepto: NotaConceptoInfo;
            concepto = this.selectorNotaConcepto.selectedItem;
            
            var rel: NotaTipoConcepto = new NotaTipoConcepto;

            rel.codigoConcepto = concepto.code;
            rel.codigoTipo = this.notaTipo.code;
            rel.idNotaConcepto = concepto.identity;
            rel.idNotaTipo = this.notaTipo.identity;
            rel.nombreConcepto = concepto.nombre;
            rel.nombreTipo = this.notaTipo.nombre;

            var newRel = this.notaTipo.notaTipoConceptos.slice();

            newRel.push(rel);

            this.notaTipo.notaTipoConceptos = newRel;

            this.conceptosCollection.sourceCollection = newRel;
            this.conceptosCollection.refresh;

        }
    }

    onRemoveItem(item) {

        this.conceptosCollection.remove(item);
        this.conceptosCollection.refresh();

    }


}