﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Features
import { PerfilAutoCobroTipoService} from '../services/perfil-auto-cobro-tipo.service';
import { PerfilAutoCobroTipoInfo, PerfilAutoCobroTipoItem} from '../models/perfil-auto-cobro-tipo';

@Component({
    selector: 'credito-perfil-auto-cobro-tipo-editor',
    templateUrl: './perfiles-auto-cobros-tipos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PerfilesAutoCobrosTiposEditor)
        },
        {
            provide: CatalogService, useExisting: PerfilAutoCobroTipoService
        }
    ]
})
export class PerfilesAutoCobrosTiposEditor extends CatalogEditor {
    perfilAutoCobroTipo: PerfilAutoCobroTipoInfo = new PerfilAutoCobroTipoInfo();
    bSuma: boolean;
    bResta: boolean;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private perfilAutoCobroTipoService: PerfilAutoCobroTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'codigoReferencia': [''],
            'cantidad': [0, Validators.required],
            'bSuma': [0],
            'bResta': [0],
            'porcentaje': [0],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.bSuma = false;
        this.bResta = true;
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.perfilAutoCobroTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.perfilAutoCobroTipo = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.perfilAutoCobroTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.perfilAutoCobroTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.perfilAutoCobroTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                if (info.afectador == 'S') {
                    this.bSuma = true;
                    this.bResta = false;
                }
                else {
                    this.bSuma = false;
                    this.bResta = true;
                }

                this.perfilAutoCobroTipo = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        if (this.bSuma) this.perfilAutoCobroTipo.afectador = "S";
        if (this.bResta) this.perfilAutoCobroTipo.afectador = "R";
        this.perfilAutoCobroTipoService.update(this.perfilAutoCobroTipo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.perfilAutoCobroTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.perfilAutoCobroTipo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onChangeSuma(args: any) {
        this.bResta = false;
        this.perfilAutoCobroTipo.afectador = 'S';
        Control.invalidateAll();
    }

    onChangeResta(args: any) {
        this.bSuma = false;
        this.perfilAutoCobroTipo.afectador = 'R';
        Control.invalidateAll();
    }


}