﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, Output, EventEmitter  } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { SucursalService} from '../services/sucursal.service';
import { SucursalInfo, SucursalItem} from '../models/sucursal';

@Component({    
    selector: 'credito-sucursales-manager',
    templateUrl: './sucursales.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SucursalesManager)
        },
        {
            provide: CatalogService, useExisting: SucursalService
        }
	]
})
export class SucursalesManager extends CatalogManager {
    @Input() sucursales: SucursalItem[] = [];
    @Input() idEmpresa: number;
    @Output() refreshListParent: EventEmitter<void> = new EventEmitter<void>();
    //

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private sucursalService: SucursalService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {      
    }

    doSearch(request: SearchRequest) {      
    }

    onAddingItem() {
        this.router.navigate(['/credit/sucursales-credito', this.process.id, this.idEmpresa, 0]);

    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate(['/credit/sucursales-credito', this.process.id, this.idEmpresa, item.id]);
    }

    onDeletingItem(item: CatalogItem) {
        this.sucursalService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.refreshListParent.emit();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate(['/credit/sucursales-credito', this.process.id, this.idEmpresa, 0, { cloneId: item.id }]);
    }

    onChangingItemState(item: CatalogItem) {
        this.sucursalService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.refreshListParent.emit();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
