﻿
import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search';
//
import { CajaInfo, CajaItem } from '../models/caja';
import { CajaService } from '../services/caja.service';

@Component({
    selector: 'credit-cajas-selector',
    templateUrl: './cajas-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => CajasSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => CajasSelectorComponent),
        multi: true
    }]
})
export class CajasSelectorComponent extends CatalogSelector {
    cajas: CajaItem[];
    selectedItem: CajaInfo = null;
    //Outputs
    @Output() selectedItemChanged: EventEmitter<CajaInfo> = new EventEmitter<CajaInfo>();

    constructor(protected context: Context, private service: CajaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findKeyword(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(this.selectedItem);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();
        //Agregar aqui filtros
        //let condicion: SearchCondition = { field: "campo", operator: "=", valueType: "int", value: this.inputMember.toString() };
        //request.conditions.push(condicion);
        //
        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.cajas = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findKeyword(keyword: string): Observable<CatalogItem[]> {
        return this.service.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}
