﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs, DateTime } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
//Locales
import { DeudorService } from '../services/deudor.service'
//import { IEPSPreview } from '../models/ieps-preview'

import { AsyncTaskService } from "framework3/queue/services/async-task.service";
import { DeudorItem } from '../models/deudor';

@Component({
    selector: 'credit-deudores-bloqueados',
    templateUrl: './deudores-bloqueados.component.html'
})
export class DeudoresBloqueadosComponent extends ProcessComponent {
    form: FormGroup;
    fecha: Date;
    collectionView: CollectionView = new CollectionView();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, private deudorService: DeudorService)
    {
        super(context, router, route);
        this.form = formBuilder.group({
            'fecha': [new Date()],          

        });

        this.fecha = new Date();
        this.form.controls['fecha'].valueChanges.subscribe(value => {
            setTimeout(_ => this.collectionView.sourceCollection = [] );
        });
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CONSULTAR') {
            this.loadBloqueados();
        }        
    }

    loadBloqueados() {
        this.deudorService.loadBloqueados(this.context.app.defaultCompany.identity, this.fecha)
            .then(result => {
                this.collectionView.sourceCollection = result;
                //this.totalRows = result.totalRows;
                //this.pageIndex = request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onMenuSeleccionadosClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Eliminar                     
            let deudor: DeudorItem = <DeudorItem>this.collectionView.currentItem
            this.deudorService.desbloquear(deudor.id, this.fecha)
                .then(result => {
                    this.collectionView.remove(this.collectionView.currentItem);
                    this.context.app.hideSpinner();
                    this.alerts.showAlert('El deudor ha sido desbloqueado correctamente', 'success');
                })
                .catch(error => this.handleError(error));
        }        
    }

    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);       
        this.alerts.showAlert(message, 'danger');
    }

}