﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { DeudorInfo, DeudorItem } from '../models/deudor';
import { DeudorService } from '../services/deudor.service';

@Component({
    selector: 'credit-deudores-selector',
    templateUrl: './deudores-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => DeudorSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => DeudorSelectorComponent),
        multi: true
    }]
})
export class DeudorSelectorComponent extends CatalogSelector {
    deudores: DeudorItem[];
    selectedItem: DeudorInfo = null;

    @Output() itemSelected: EventEmitter<DeudorInfo> = new EventEmitter<DeudorInfo>();

    constructor(protected context: Context, private deudorService: DeudorService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findDeudor(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.deudorService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + '-' + info.nombre;
                    this.itemSelected.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.itemSelected.emit(this.selectedItem)
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.deudorService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.deudores = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findDeudor(keyword: string): Observable<CatalogItem[]> {
        return this.deudorService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

