﻿//Framework
import { Component, Input, Output, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import * as moment from 'moment';
//Wijmo
import { Control, CollectionView, DateTime } from 'wijmo/wijmo';
//Feature
import { AztecaValidators, AlertsBarComponent, ToolbarButton } from 'framework3/controls';
import { Context } from 'framework3/core';
import { ComprobanteFiscalService } from '../../accounting/services/comprobante-fiscal.service';
import { ModificarAddendaEditor } from '../controls/modificar-addenda.component';
import { CuentaPorCobrarService } from '../services/cuenta-por-cobrar.service';
import { WjFlexGrid } from 'wijmo/wijmo.angular2.grid';
import { Row } from 'wijmo/wijmo.grid';
import { Confirmacion } from '../parts/confirmacion.component'


@Component({
    selector: 'credit-envio-comprobantes-fiscales',
    templateUrl: './envio-comprobantes-fiscales.component.html'
})
export class EnvioComprobantesFiscalesComponent {
    form: FormGroup;
    comprobantesDestinosCollection: CollectionView = new CollectionView();
    comprobantesList: EnvioComprobantesItem[] = [];

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;
    @ViewChild(ModificarAddendaEditor) modificarAddendaDialog: ModificarAddendaEditor;
    @ViewChild('gridComprobantes') gridComprobantes: WjFlexGrid;
    @ViewChild(Confirmacion) confirma: Confirmacion;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute, private context: Context,
        private comprobanteFiscalService: ComprobanteFiscalService, private cuentaPorCobrarService: CuentaPorCobrarService) {
        this.form = this.formBuilder.group({
            'fechaBusquedaInicial': [],
            'fechaBusquedaFinal': []
        });
    }

    busquedaComprobantes() {
        this.context.app.showSpinner();
        this.comprobantesDestinosCollection.sourceCollection = [];
        this.comprobantesDestinosCollection.refresh();
        let fechaIni = moment(this.form.controls.fechaBusquedaInicial.value).startOf('day').toDate();
        let fechaFin = moment(this.form.controls.fechaBusquedaFinal.value).endOf('day').toDate();
        this.comprobanteFiscalService.getEnvioFacturas(this.context.app.defaultCompany.identity, fechaIni, fechaFin)
            .then(comprobantes => {
                this.context.app.hideSpinner();
                comprobantes.forEach(x => {
                    x.selected = false;
                    x.fechaComprobantes = moment(x.fechaComprobantes).toDate();
                });
                this.comprobantesList = comprobantes;
                this.comprobantesDestinosCollection.sourceCollection = this.comprobantesList;
                this.comprobantesDestinosCollection.collectionChanged.addHandler((collectionChanged) => {
                    this.gridComprobantes.rows.forEach((row: Row) => {
                        if (row.dataItem['completado'] == true) {
                            row.isReadOnly = false;
                        }
                    });
                })
            })
            .catch(error => this.handleError(error))
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SEARCHCOMPROBANTES') {
            this.busquedaComprobantes();
        }
        if (button.name == 'RESENDCOMPROBANTES') {
            try {
                this.context.app.showSpinner();
                let jobs: ResetComprobantesItem[] = [];
                this.comprobantesList.filter(x => x.selected).forEach(x => {
                    let job: ResetComprobantesItem = new ResetComprobantesItem();
                    job.idComprobante = x.id.toString();
                    job.idComprobanteDestino = x.idComprobanteDestino;
                    job.idJob = x.idJob;
                    jobs.push(job);
                })
                if (jobs.length > 0 && jobs != undefined) {
                    this.comprobanteFiscalService.resetEnvioFacturas(jobs)
                        .then(x => {
                            this.busquedaComprobantes();
                            this.alertsBar.showAlert('Se ha reiniciado el proceso de envió correctamente', 'success');
                        })
                        .catch(error => this.handleError(error))
                }
                else {
                    this.alertsBar.showAlert('No se ha seleccionado ningún comprobante', 'info');
                }
            }
            finally {
                this.context.app.hideSpinner();
            }

        }
    }

    modificarAddendaOpenDialog(comprobantes: EnvioComprobantesItem) {
        this.modificarAddendaDialog.data = comprobantes;
        this.modificarAddendaDialog.show();
    }

    mostrarError(error: string) {
        this.confirma.show(1, error);
    }

    handleError(error: any) {
        console.error(error);
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
        this.context.app.hideSpinner();
    }

    getStatusColor(status: string): string {
        if (status == 'Error al enviar')
            return '#ff471a';
        if (status == 'Programado')
            return '#ff9900';
        if (status == 'Enviado')
            return 'green';
    }

    onModificarAddenda(args: EnvioComprobantesItem) {
        this.cuentaPorCobrarService.modificarAddenda(args)
            .then(x => {
                if (x.length == 1) {
                    this.cuentaPorCobrarService.generarAddendaCustom(x[0].idPedidoTit, x[0].fechaPedido, x[0].origen)
                        .then(x => {

                        })
                        .catch(error => this.handleError(error))
                }
            })
            .catch(error => this.handleError(error))
    }

    onReemplazarAddenda(addenda: string) {

        var addd: string[] = [];
        addd[0] = addenda;

        this.comprobanteFiscalService.reemplazaAddenda(this.modificarAddendaDialog.data.id.toString(), addd)
            .then(x => {

            })
            .catch(error => this.handleError(error))
    }

}

export class EnvioComprobantesItem {
    id: number;
    idJob: number;
    fechaComprobantes: Date;
    uuid: string;
    serie: string;
    folio: string;
    razonSocial: string;
    status: string;
    completado: boolean;
    modificaAddenda: boolean;
    ordenCompra: string;
    fechaOrdenCompra: Date;
    numRemision: string;
    fechaRemision: Date;
    fechaEntrega: Date;
    idCuentaPorCobrarTit: string;
    selected?: boolean;
    idComprobanteDestino: number;
}

export class ResetComprobantesItem {
    idComprobante: string;
    idJob: number;
    idComprobanteDestino: number;
}