﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { GrupoDeudorInfo, GrupoDeudorItem } from '../models/grupo-deudor';
import { GrupoDeudorService } from '../services/grupo-deudor.service';

@Component({
    selector: 'credit-grupo-deudores-selector',
    templateUrl: './grupo-deudores-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => GrupoDeudorSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => GrupoDeudorSelectorComponent),
        multi: true
    }]
})
export class GrupoDeudorSelectorComponent extends CatalogSelector {
    gruposDeudores: any[];
    selectedItem: GrupoDeudorInfo = null;

    @Output() itemSelected: EventEmitter<GrupoDeudorInfo> = new EventEmitter<GrupoDeudorInfo>();

    constructor(protected context: Context, private grupoDeudorService: GrupoDeudorService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findInsumo(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.grupoDeudorService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.itemSelected.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.grupoDeudorService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.gruposDeudores = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findInsumo(keyword: string): Observable<CatalogItem[]> {
        return this.grupoDeudorService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

