﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
import { DocumentSelector } from 'framework3/search/models/document-selector';
//
import { AnticipoInfo, AnticipoItem } from '../models/anticipo';
import { AnticipoService } from '../services/anticipo.service';

@Component({
    selector: 'credit-anticipo-selector',
    templateUrl: './anticipos-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => AnticipoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => AnticipoSelectorComponent),
        multi: true
    }]

})
export class AnticipoSelectorComponent extends DocumentSelector {
    anticipos: any[];
    selectedItem: AnticipoInfo = new AnticipoInfo;
    @Input() idContribuyente: number = 0;
    //
    @Output() selectedItemChanged: EventEmitter<AnticipoInfo> = new EventEmitter<AnticipoInfo>();

    constructor(protected context: Context, private anticipoService: AnticipoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findAnticipos(keyword)));
    }

    loadItem(itemID: string) {
        if (itemID != '') {
            this.anticipoService.getDetail(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.identity + '-' + info.fechaSys;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        //let condiciones: SearchCondition = { field: "idContribuyente", operator: "=", valueType: "int", value: this.idContribuyente.toString() };
        //request.conditions.push(condiciones);


        this.anticipoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.anticipos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findAnticipos(keyword: string): Observable<AnticipoItem[]> {
        return this.anticipoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}