﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
import { DocumentSelector } from 'framework3/search/models/document-selector';
//
import { CuentaPorCobrarInfo, CuentaPorCobrarItem } from '../models/cuenta-por-cobrar';
import { CuentaPorCobrarService } from '../services/cuenta-por-cobrar.service';

@Component({
    selector: 'credit-cpc-selector',
    templateUrl: './cpc-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => CuentasPorCobrarSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => CuentasPorCobrarSelectorComponent),
        multi: true
    }]

})
export class CuentasPorCobrarSelectorComponent extends DocumentSelector {
    cuentaPorCobrar: any[];
    selectedItem: CuentaPorCobrarInfo = new CuentaPorCobrarInfo;
    @Input() idDeudor: number = 0;
    @Input() idGrupoDeudor: number = 0;
    //
    @Output() selectedItemChanged: EventEmitter<CuentaPorCobrarInfo> = new EventEmitter<CuentaPorCobrarInfo>();

    constructor(protected context: Context, private cPcService: CuentaPorCobrarService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findCpC(keyword)));
    }

    loadItem(itemID: string) {
        if (itemID != '') {
            this.cPcService.getDetail(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.referenciaAlf + '-' + info.referenciaNum.toString();
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let condiciones: SearchCondition = { field: "idDeudor", operator: "=", valueType: "int", value: this.idDeudor.toString() };
        request.conditions.push(condiciones);


        this.cPcService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.cuentaPorCobrar = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findCpC(keyword: string): Observable<CuentaPorCobrarItem[]> {
        return this.cPcService.searchKeyword(this.context.app.defaultSite.identity, keyword, this.idDeudor, this.idGrupoDeudor);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}