﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
import { DocumentSelector } from 'framework3/search/models/document-selector';
//
import { DepositoInfo, DepositoItem } from '../models/deposito';
import { DepositoService } from '../services/deposito.service';

@Component({
    selector: 'credit-depositos-selector',
    templateUrl: './depositos-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => DepositosSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => DepositosSelectorComponent),
        multi: true
    }]

})
export class DepositosSelectorComponent extends DocumentSelector {
    depositos: any[];
    selectedItem: DepositoInfo = new DepositoInfo;
    @Input() idWorflowStep: number = 0;
    //
    @Output() selectedItemChanged: EventEmitter<DepositoInfo> = new EventEmitter<DepositoInfo>();

    constructor(protected context: Context, private depositoService: DepositoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findAnticipos(keyword)));
    }

    loadItem(itemID: string) {
        if (itemID != '') {
            this.depositoService.getDetail(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.identity + '-' + info.fechaSys;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.idWorflowStep > 0) {
            let condiciones: SearchCondition = { field: "idWorkflowStep", operator: "=", valueType: "int", value: this.idWorflowStep.toString() };
            request.conditions.push(condiciones);
        }
        this.depositoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.depositos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findAnticipos(keyword: string): Observable<DepositoItem[]> {
        return this.depositoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}