﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
import { DocumentSelector } from 'framework3/search/models/document-selector';
//
import { FacturaInfo, FacturaItem } from '../models/factura';
import { FacturaService } from '../services/factura.service';

@Component({
    selector: 'credit-facturas-selector',
    templateUrl: './facturas-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => FacturaSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => FacturaSelectorComponent),
        multi: true
    }]

})
export class FacturaSelectorComponent extends DocumentSelector {
    factura: any[];
    selectedItem: FacturaInfo = new FacturaInfo;
    @Input() idDeudor: number = 0;
    @Input() idGrupoDeudor: number = 0;
    //
    @Output() selectedItemChanged: EventEmitter<FacturaInfo> = new EventEmitter<FacturaInfo>();

    constructor(protected context: Context, private facturaService: FacturaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findFacturas(keyword)));
    }

    loadItem(itemID: string) {
        if (itemID != '') {
            this.facturaService.getDetail(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.comprobante.serie + '-' + info.comprobante.folio;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let condiciones: SearchCondition = { field: "idDeudor", operator: "=", valueType: "int", value: this.idDeudor.toString()};
        request.conditions.push(condiciones);


        this.facturaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.factura = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findFacturas(keyword: string): Observable<FacturaItem[]> {
        return this.facturaService.searchKeyword(this.context.app.defaultSite.identity, keyword, this.idDeudor, this.idGrupoDeudor);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}