﻿//Angular
import { Component, forwardRef, ViewChild, ChangeDetectorRef, Input, Output, EventEmitter } from '@angular/core';
import { Route, Router, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Framework 3
import { BaseComponent, CatalogService, Context } from 'framework3/core';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { EnvioComprobantesItem } from '../components/envio-comprobantes-fiscales.component';
import { ComprobanteFiscalService } from '../../accounting/services/comprobante-fiscal.service';
import { ComprobanteFiscalInfo } from '../../accounting/models/comprobante-fiscal'

import { AceEditorModule } from 'ng2-ace-editor';
import 'brace/theme/chrome';
import 'brace/mode/xml';
import 'brace/worker/xml';

@Component({
    selector: 'modificar-addenda',
    templateUrl: './modificar-addenda.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ModificarAddendaEditor)
        },
        {
            provide: CatalogService, useExisting: ComprobanteFiscalService
        }
    ]
})

export class ModificarAddendaEditor extends CatalogEditor {
    //variables
    editorForm: FormGroup;
    envioComprobantesItem: EnvioComprobantesItem = new EnvioComprobantesItem();
    textAdenda: string = "";

    //Inputs
    @Input() data: EnvioComprobantesItem = new EnvioComprobantesItem();
    @Output() dataUpdater: EventEmitter<string> = new EventEmitter<string>();

    //viewChilds
    @ViewChild('dialog') dialog: ModalDirective;

    constructor(private comprobanteFiscalService: ComprobanteFiscalService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetectorRef: ChangeDetectorRef) {
        super(context, router, route, changeDetectorRef);
        this.editorForm = formBuilder.group({
            'orden': ['', Validators.maxLength(32)],
            'fechaOrden': [''],
            'remision': ['', Validators.maxLength(32)],
            'fechaRemision': [''],
            'fechaEntrega': [''],
            'addenda': ['']
        });
    }

    show() {
        this.dialog.config.backdrop = false;
        this.obtenerAddenda();
        this.dialog.show();
        Control.invalidateAll();
    }

    obtenerAddenda() {
        this.comprobanteFiscalService.getAddenda(this.data.id.toString())
            .then(doc => {
                this.textAdenda = doc.toString();
            })
            .catch(error => this.handleError(error));
    }

    onUpdate() {
        this.dataUpdater.emit(this.textAdenda);
        this.dialog.hide();
    }


}