﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { NotaConceptoInfo, NotaConceptoItem } from '../../accounting/models/nota-concepto';
import { NotaConceptoService } from '../../accounting/services/nota-concepto.service';
//

@Component({
    selector: 'credit-nota-concepto-selector',
    templateUrl: './nota-concepto-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => NotaConceptoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => NotaConceptoSelectorComponent),
        multi: true
    }]

})
export class NotaConceptoSelectorComponent extends CatalogSelector {
    notaConcepto: any[];
    selectedItem: NotaConceptoInfo = new NotaConceptoInfo;
    @Input() idNotaTipo: number = 0;
    //
    @Output() selectedItemChanged: EventEmitter<NotaConceptoInfo> = new EventEmitter<NotaConceptoInfo>();

    constructor(protected context: Context, private notaConceptoService: NotaConceptoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findNotasConceptos(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.notaConceptoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.notaConceptoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.notaConcepto = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findNotasConceptos(keyword: string): Observable<CatalogItem[]> {
        return this.notaConceptoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}