﻿import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
//Modulos Azteca
import { CoreModule } from 'framework3/core';
import { EditorsModule } from 'framework3/editors';
import { SearchModule } from 'framework3/search';
import { GeneralModule } from 'framework3/general';
import { ControlsModule } from 'framework3/controls';
import { SecurityModule } from 'framework3/security';
import { AccountingModule } from '../accounting/accounting.module';
import { RhModule } from '../rh/rh.module';
import { CreditRoutingModule } from './credit.routing.module';

//Bootstrap
import { TabsModule, ModalModule } from 'ngx-bootstrap';
//Wijmo
import { WjCoreModule } from 'wijmo/wijmo.angular2.core';
import { WjGridModule } from 'wijmo/wijmo.angular2.grid';
import { WjGridFilterModule } from 'wijmo/wijmo.angular2.grid.filter';
import { WjInputModule } from 'wijmo/wijmo.angular2.input';
import { WjGridDetailModule } from 'wijmo/wijmo.angular2.grid.detail';

//
import { AceEditorModule } from 'ng2-ace-editor';

/////////////////////////////// Servicios ////////////////////////////
import { CajaService } from './services/caja.service';
import { CajeroService } from './services/cajero.service';
import { GrupoDeudorService } from './services/grupo-deudor.service';
import { DeudorService } from './services/deudor.service';
import { FormaPagoService } from './services/forma-pago.service';
import { CuentaPorCobrarService } from './services/cuenta-por-cobrar.service';
import { CobranzaService } from './services/cobranza.service';
import { ConceptoService } from './services/concepto.service';
import { ReporteService } from './services/reporte.service';
import { AnticipoService } from './services/anticipo.service';
import { EmpresaService } from './services/empresa.service';
import { SucursalService } from './services/sucursal.service';

import { NotaService } from './services/nota.service';
import { FacturaService } from './services/factura.service';
import { NotaTipoService } from './services/nota-tipo.service';
import { CobroTipoService } from './services/cobro-tipo.service';
import { DepositoTipoService } from './services/deposito-tipo.service';
import { DepositoService } from './services/deposito.service';
import { PerfilAutoCobroTipoService } from './services/perfil-auto-cobro-tipo.service';

//////////////////////////// Componentes /////////////////////////////
import { GrupoDeudorSelectorComponent } from './components/grupo-deudores-selector.component';
import { DeudorSelectorComponent } from './components/deudores-selector.component';
import { CajasSelectorComponent } from './components/cajas-selector.component';
import { EnvioComprobantesFiscalesComponent } from './components/envio-comprobantes-fiscales.component';
import { Confirmacion } from './parts/confirmacion.component';

import { DeudoresBloqueadosComponent } from './components/deudores-bloqueados.component';

//////////////////////////// Controls /////////////////////////////
import { AnticipoSelectorComponent } from './controls/anticipos-selector.component';
import { DepositosSelectorComponent } from './controls/depositos-selector.component';

import { FacturaSelectorComponent } from './controls/facturas-selector.component';
import { CuentasPorCobrarSelectorComponent } from './controls/cpc-selector.component';
import { ModificarAddendaEditor } from './controls/modificar-addenda.component';
import { NotaConceptoSelectorComponent } from './controls/nota-concepto-selector.component';

//////////////////////////// Catalogos ////////////////////////////////
import { CajasManager } from './catalogs/cajas.manager';
import { CajasEditor } from './catalogs/cajas.editor';
import { CajerosManager } from './catalogs/cajeros.manager';
import { CajerosEditor } from './catalogs/cajeros.editor';
import { GruposDeudoresManager } from './catalogs/grupos-deudores.manager';
import { GruposDeudoresEditor } from './catalogs/grupos-deudores.editor';
import { DeudoresManager } from './catalogs/deudores.manager';
import { DeudoresEditor } from './catalogs/deudores.editor';
import { FormasPagoManager } from './catalogs/formas-pago.manager';
import { FormasPagoEditor } from './catalogs/formas-pago.editor';
import { EmpresasManager } from './catalogs/empresas.manager';
import { EmpresasEditor } from './catalogs/empresas.editor';
import { SucursalesManager } from './catalogs/sucursales.manager';
import { SucursalesEditor } from './catalogs/sucursales.editor';

import { NotasTiposManager } from './catalogs/notas-tipos.manager';
import { NotasTiposEditor } from './catalogs/notas-tipos.editor';
import { DepositosTiposManager } from './catalogs/depositos-tipos.manager';
import { DepositosTiposEditor } from './catalogs/depositos-tipos.editor';
import { PerfilesAutoCobrosTiposManager } from './catalogs/perfiles-auto-cobros-tipos.manager';
import { PerfilesAutoCobrosTiposEditor } from './catalogs/perfiles-auto-cobros-tipos.editor';

///////////////////////////// Document /////////////////////////////////
import { CuentasPorCobrarEditor } from './documents/cuentas-por-cobrar.editor';
import { CuentasPorCobrarManager } from './documents/cuentas-por-cobrar.manager';
import { CobranzasEditor } from './documents/cobranzas.editor';
import { CobranzasManager } from './documents/cobranzas.manager';
import { AnticiposEditor } from './documents/anticipos.editor';
import { AnticiposManager } from './documents/anticipos.manager';
import { NotasEditor } from './documents/notas.editor';
import { NotasManager } from './documents/notas.manager';
import { DepositosEditor } from './documents/depositos.editor';
import { DepositosManager } from './documents/depositos.manager';


//////////////////////////// Parts /////////////////////////////////////
import { CajaMiembrosComponent } from './parts/caja-miembros.component';
import { CajerosListComponent } from './dialogs/cajeros-list.component';
import { FacturacionComponent } from './documents/parts/facturacion.component';
import { CuentaPorCobrarDetailEditor } from './documents/parts/cuentas-por-cobrar-detail.editor';
import { CobranzaDetailEditor } from './documents/parts/cobranzas-detail.editor';
import { CorteComponent } from './parts/corte.component';
import { CorteHistoricoComponent } from './parts/corte-historico.component';
import { CobranzaComponent } from './parts/cobranza.component';
import { AnticipoDetailEditor } from './documents/parts/anticipos-detail.editor';
import { NotaDetailEditor } from './documents/parts/notas-detail.editor';
import { DepositoDetailEditor } from './documents/parts/depositos-detail.editor';

/////////////////////////////////////////////////////////////////////////////////
let COMPONENTES = [CajasManager, CajasEditor, CajerosManager, CajerosEditor, CajaMiembrosComponent, CajerosListComponent, CorteComponent, GruposDeudoresManager, GruposDeudoresEditor,
    DeudoresManager, DeudoresEditor, GrupoDeudorSelectorComponent, DeudorSelectorComponent, FormasPagoManager, FormasPagoEditor,
    CuentasPorCobrarManager, CuentasPorCobrarEditor, CuentaPorCobrarDetailEditor, CobranzasManager, CobranzasEditor, CobranzaDetailEditor, CobranzaComponent, ModificarAddendaEditor,
    CajasSelectorComponent, FacturacionComponent, CorteHistoricoComponent, NotasEditor, NotasManager, NotaDetailEditor, EnvioComprobantesFiscalesComponent,
    AnticiposEditor, AnticiposManager, AnticipoDetailEditor, EmpresasManager, EmpresasEditor, SucursalesManager, SucursalesEditor, AnticipoSelectorComponent, FacturaSelectorComponent, CuentasPorCobrarSelectorComponent,
    NotasTiposManager, NotasTiposEditor, NotaConceptoSelectorComponent, DepositosTiposManager, DepositosTiposEditor, DepositosManager, DepositosEditor, DepositoDetailEditor, DepositosSelectorComponent, Confirmacion,
    PerfilesAutoCobrosTiposManager, PerfilesAutoCobrosTiposEditor, DeudoresBloqueadosComponent];
    

let SERVICIOS = [CajaService, CajeroService, GrupoDeudorService, DeudorService, FormaPagoService, CuentaPorCobrarService, CobranzaService, ConceptoService, ReporteService, AnticipoService,
    EmpresaService, SucursalService, NotaService, FacturaService, NotaTipoService, CobroTipoService, DepositoTipoService, DepositoService, PerfilAutoCobroTipoService];

@NgModule({
    imports: [
        CommonModule, FormsModule, ReactiveFormsModule, AceEditorModule,
        TabsModule.forRoot(), ModalModule.forRoot(),
        CoreModule, EditorsModule, SearchModule, GeneralModule, ControlsModule, SecurityModule, CreditRoutingModule, AccountingModule, RhModule,
        WjCoreModule, WjInputModule, WjGridModule, WjGridFilterModule, WjGridDetailModule
    ],
    declarations: COMPONENTES,
    providers: SERVICIOS,
    exports: COMPONENTES
})
export class CreditModule {

}
