﻿import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

//////////////////////////// Guards //////////////////////////////////
import { AuthGuard } from 'framework3/core';
import { EditorDeactivateGuard } from 'framework3/editors';

//////////////////////////// Catalogos ////////////////////////////////
import { CajasManager } from './catalogs/cajas.manager';
import { CajasEditor } from './catalogs/cajas.editor';
import { CajerosManager } from './catalogs/cajeros.manager';
import { CajerosEditor } from './catalogs/cajeros.editor';
import { GruposDeudoresManager } from './catalogs/grupos-deudores.manager';
import { GruposDeudoresEditor } from './catalogs/grupos-deudores.editor';
import { DeudoresManager } from './catalogs/deudores.manager';
import { DeudoresEditor } from './catalogs/deudores.editor';
import { FormasPagoManager } from './catalogs/formas-pago.manager';
import { FormasPagoEditor } from './catalogs/formas-pago.editor';
import { EmpresasManager } from './catalogs/empresas.manager';
import { EmpresasEditor } from './catalogs/empresas.editor';
import { SucursalesManager } from './catalogs/sucursales.manager';
import { SucursalesEditor } from './catalogs/sucursales.editor';

import { NotasTiposManager } from './catalogs/notas-tipos.manager';
import { NotasTiposEditor } from './catalogs/notas-tipos.editor';
import { DepositosTiposManager } from './catalogs/depositos-tipos.manager';
import { DepositosTiposEditor } from './catalogs/depositos-tipos.editor';

import { PerfilesAutoCobrosTiposManager } from './catalogs/perfiles-auto-cobros-tipos.manager';
import { PerfilesAutoCobrosTiposEditor } from './catalogs/perfiles-auto-cobros-tipos.editor';

///////////////////////////// Document /////////////////////////////////
import { CuentasPorCobrarEditor } from './documents/cuentas-por-cobrar.editor';
import { CuentasPorCobrarManager } from './documents/cuentas-por-cobrar.manager';
import { CobranzasEditor } from './documents/cobranzas.editor';
import { CobranzasManager } from './documents/cobranzas.manager';
import { AnticiposEditor } from './documents/anticipos.editor';
import { AnticiposManager } from './documents/anticipos.manager';
import { AnticipoDetailEditor } from './documents/parts/anticipos-detail.editor';
import { NotasEditor } from './documents/notas.editor';
import { NotasManager } from './documents/notas.manager';

import { DepositosEditor } from './documents/depositos.editor';
import { DepositosManager } from './documents/depositos.manager';



///////////////////////////// Components /////////////////////////////////
import { FacturacionComponent } from './documents/parts/facturacion.component';
import { EnvioComprobantesFiscalesComponent } from './components/envio-comprobantes-fiscales.component';
import { DeudoresBloqueadosComponent } from './components/deudores-bloqueados.component';

//Parts//
import { CorteComponent } from './parts/corte.component';
import { CajaMiembrosComponent } from './parts/caja-miembros.component';

///////////////////////////// Rutas /////////////////////////////////

const routes: Routes = [
    {
        path: '',
        canActivateChild: [AuthGuard],
        children: [
            { path: 'cajas/:proceso', component: CajasManager },
            { path: 'cajas/:proceso/:id', component: CajasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'cajas/:proceso/:id/members', component: CajaMiembrosComponent },
            { path: 'cajeros/:proceso', component: CajerosManager },
            { path: 'cajeros/:proceso/:id', component: CajerosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'cortes/:proceso', component: CorteComponent, canDeactivate: [EditorDeactivateGuard] },
            { path: 'gruposDeudores/:proceso', component: GruposDeudoresManager },
            { path: 'gruposDeudores/:proceso/:id', component: GruposDeudoresEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'deudores/:proceso', component: DeudoresManager },
            { path: 'deudores/:proceso/:id', component: DeudoresEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'formasPago/:proceso', component: FormasPagoManager },
            { path: 'formasPago/:proceso/:id', component: FormasPagoEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'empresas-credito/:proceso', component: EmpresasManager },
            { path: 'empresas-credito/:proceso/:id', component: EmpresasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'sucursales-credito/:proceso', component: SucursalesManager },
            { path: 'sucursales-credito/:proceso/:idEmpresa/:id', component: SucursalesEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'envio-comprobantes-fiscales/:proceso', component: EnvioComprobantesFiscalesComponent },

            { path: 'notas-tipos/:proceso', component: NotasTiposManager },
            { path: 'notas-tipos/:proceso/:id', component: NotasTiposEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'depositos-tipos/:proceso', component: DepositosTiposManager },
            { path: 'depositos-tipos/:proceso/:id', component: DepositosTiposEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'perfil-auto-cobro-tipo/:proceso', component: PerfilesAutoCobrosTiposManager },
            { path: 'perfil-auto-cobro-tipo/:proceso/:id', component: PerfilesAutoCobrosTiposEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'deudores-bloqueados/:proceso', component: DeudoresBloqueadosComponent},
            

            //DOCUMENTS
            { path: 'cuentasPorCobrar/:proceso', component: CuentasPorCobrarManager },
            { path: 'cuentasPorCobrar/:proceso/:id', component: CuentasPorCobrarEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'facturacion/:proceso', component: FacturacionComponent },
            { path: 'cobranza/:proceso', component: CobranzasManager },
            { path: 'cobranza/:proceso/:id', component: CobranzasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'anticipos/:proceso', component: AnticiposManager },
            { path: 'anticipos/:proceso/:id', component: AnticiposEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'notas/:proceso', component: NotasManager },
            { path: 'notas/:proceso/:id', component: NotasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'depositos/:proceso', component: DepositosManager },
            { path: 'depositos/:proceso/:id', component: DepositosEditor, canDeactivate: [EditorDeactivateGuard] },
        ]
    },
];

@NgModule({
    imports: [
        RouterModule.forChild(routes)
    ],
    exports: [
        RouterModule
    ]
})
export class CreditRoutingModule { }
