﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, AzComboboxComponent } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { AnticipoInfo, AnticipoItem } from '../models/anticipo';
import { AnticipoDetail } from '../models/anticipo-detail';
import { AnticipoService } from '../services/anticipo.service';
import { AnticipoDetailEditor } from './parts/anticipos-detail.editor';

import { GrupoDeudorInfo } from '../models/grupo-deudor';
import { DeudorInfo } from '../models/deudor';

import { FormaPagoService } from '../services/forma-pago.service';
import { FormaPagoItem } from '../models/forma-pago';

import { EmpresaService } from '../services/empresa.service';
import { EmpresaInfo, EmpresaItem } from '../models/empresa';

import { EmpresaService as EmpresaServiceCont } from '../../accounting/services/empresa.service';
import { EmpresaInfo as EmpresaInfoCont } from '../../accounting/models/empresa';

import { CajasSelectorComponent } from '../../credit'

import { ImpuestoService } from '../../accounting/services/impuesto.service';
import { ImpuestoInfo } from '../../accounting/models/impuesto';

import { DivisaService } from '../../accounting/services/divisa.service';
import { DivisaInfo } from '../../accounting/models/divisa';


@Component({
    selector: 'credito-anticipos-editor',
    templateUrl: './anticipos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AnticiposEditor)
        },
        {
            provide: DocumentService, useExisting: AnticipoService
        }
    ]
})
export class AnticiposEditor extends DocumentEditor {
    anticipo = new AnticipoInfo();
    empresa = new EmpresaInfo();
    divisa = new DivisaInfo();
    empresaCont = new EmpresaInfoCont();
    impuesto = new ImpuestoInfo();
    formasPago: FormaPagoItem[] = [];
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: AnticipoDetailEditor;
    @ViewChild('selectorCaja') selectorCaja: CajasSelectorComponent;
    @ViewChild('cboSegmento') cboSegmento: AzComboboxComponent;

    busquedaTipos: { id: string, nombre: string }[] = [];
    tipoBusqueda: string;
    idGrupoDeudor: number;
    idDeudor: number;
    sImporteImpuesto: string;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private anticipoService: AnticipoService,
        private formaPagoService: FormaPagoService,
        private empresaService: EmpresaService,
        private empresaServiceCont: EmpresaServiceCont,
        private impuestoService: ImpuestoService,
        private divisaService: DivisaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'busqueda': ['', Validators.required],
            'idDeudor': [0],
            'idGrupoDeudor':[0],
            'importe': [0, Validators.required],
            'importeImpuesto': [0],
            'baseImpuesto': [0],
            'concepto': ['Anticipo del bien o servicio', Validators.required],
            'idCaja': [0],
            'idFormaPago': [0, Validators.required],
            'idBanco': [0],
            'cuenta': ['', Validators.required]
        });


        this.documentForm.controls.importe.valueChanges.subscribe(value => {
            this.onImporteChanged(value);
        });

        this.busquedaTipos =
            [
                { "id": "G", "nombre": "Grupo Deudor" },
                { "id": "D", "nombre": "Deudor" }
            ];

    }

    selectedBusqueda(item: any) {
        this.tipoBusqueda = item;
    }

    onGrupoDeudorSelected(grupo: GrupoDeudorInfo) {
        this.idGrupoDeudor = grupo.identity;
    }

    onDeudorSelected(deudor: DeudorInfo) {
        this.idDeudor = deudor.identity;
    }


    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.anticipo.concepto = 'Anticipo del bien o servicio';
        this.loadFormasPago();
        this.loadEmpresa();
        this.loadEmpresaCont();
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.anticipoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.anticipo = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.anticipoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.anticipo = doc;
                this.detailsEditor.clientIndexStart = this.anticipo.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.anticipoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.anticipo = doc;
                if (this.anticipo.idDeudor > 0) {
                    this.tipoBusqueda = 'D'
                }
                if (this.anticipo.idGrupoDeudor > 0) {
                    this.tipoBusqueda = 'G'
                }
                this.detailsEditor.clientIndexStart = this.anticipo.details.length;
                if (this.anticipo.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.anticipo.idCorte = this.selectorCaja.selectedItem.idCorteActual;

        this.anticipo.idImpuesto = this.empresa.idImpuestoAnticipos;
        this.anticipo.tasaOCuota = this.empresa.tasaOCuotaAnticipos;
        this.anticipo.tipoFactor = this.impuesto.factor;

        this.anticipoService.update(this.anticipo)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.anticipo.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.anticipo = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.anticipoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.anticipoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.anticipo.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new AnticipoDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: AnticipoDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: AnticipoDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: AnticipoDetail[]) {
        this.anticipo.details = updatedDetails;
    }

    onSavingDetail(detail: AnticipoDetail) {
        this.detailsList.updateItem(detail);
    }

    loadFormasPago() {
        this.formaPagoService.loadList(this.context.app.defaultSite.identity)
            .then(formas => this.formasPago = formas)
            .catch(error => this.handleError(error));
    }

    onImporteChanged(importe: number) {

        if (this.impuesto.factor == 'T')
        {
            this.anticipo.importeImpuesto = (importe * this.empresa.tasaOCuotaAnticipos) / (1 + this.empresa.tasaOCuotaAnticipos);
            this.sImporteImpuesto = this.anticipo.importeImpuesto.toString();
            this.anticipo.baseImpuesto = importe * (1 - this.empresa.tasaOCuotaAnticipos / (1 + this.empresa.tasaOCuotaAnticipos));
        }
        if (this.impuesto.factor == 'M')
        {
            this.anticipo.importeImpuesto = this.empresa.tasaOCuotaAnticipos;
            this.anticipo.baseImpuesto = importe - this.empresa.tasaOCuotaAnticipos;
        }

    }

    loadEmpresa() {
        this.empresaService.getDetailByID(this.context.app.defaultCompany.idCompany)
            .then(doc => {
                this.empresa = doc;
                this.loadImpuesto(this.empresa.idImpuestoAnticipos);
            })
            .catch(error => this.handleError(error));
    }

    loadDivisa(idDivisa: number) {
        this.divisaService.getDetailByID(idDivisa)
            .then(doc => {
                this.divisa = doc;
            })
            .catch(error => this.handleError(error));
    }


    loadEmpresaCont() {
        this.empresaServiceCont.getDetailByID(this.context.app.defaultCompany.idCompany)
            .then(doc => {
                this.empresaCont = doc;
                this.loadDivisa(this.empresaCont.idDivisa);
            })
            .catch(error => this.handleError(error));
    }

    loadImpuesto(idImpuesto: number) {
        this.impuestoService.getDetailByID(idImpuesto)
            .then(doc => {
                this.impuesto = doc;
            })
            .catch(error => this.handleError(error));
    }

}

