﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, CollectionView} from 'wijmo/wijmo';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { CobranzaInfo, CobranzaItem } from '../models/cobranza';
import { CobranzaDetail } from '../models/cobranza-detail';
import { CobranzaService } from '../services/cobranza.service';
import { CajaService } from '../services/caja.service';
import { CajaItem } from '../models/caja';
import { CobroTipoItem} from '../models/cobro-tipo';
import { CobroTipoService } from '../services/cobro-tipo.service';
//Cobranza
import { CobranzaComponent } from '../parts/cobranza.component';
import { CobranzaFormaPagoDetail } from '../models/forma-pago-detail';

import { CobranzaDetailEditor } from './parts/cobranzas-detail.editor';
import { DepositoInfo } from '../models/deposito';
import { DepositoService } from '../services/deposito.service';

import { moment } from 'ngx-bootstrap/chronos/test/chain';
import { forEach } from '@angular/router/src/utils/collection';
import { CuentaPorCobrarService } from '../services/cuenta-por-cobrar.service';
import { ModalDirective } from 'ngx-bootstrap';

@Component({
    selector: 'credito-cobranzas-editor',
    templateUrl: './cobranzas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CobranzasEditor)
        },
        {
            provide: DocumentService, useExisting: CobranzaService
        }
    ]
})
export class CobranzasEditor extends DocumentEditor {
    cobranza = new CobranzaInfo();
    cajas: CajaItem[] = [];
    cobrosTipos: CobroTipoItem[] = [];
    acumulado: number=0;
    cuentasXCobrar: CollectionView = new CollectionView();
    documentType: DocumentTypeInfo;
    argsEditorEvent: EditorEventArgs;
    argsCurrentDocument: WorkflowEventArgs;    
    idDeudorDefault: number;
    idCuentaBeneficiario:number=0
    deposito = new DepositoInfo();

    formato: string = "C2";
    decimales: number = 2;

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: CobranzaDetailEditor;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent
    @ViewChild('dialogDecimales') dialogDecimales: ModalDirective;
    //
    @ViewChild('dlgCobranza') dlgCobranza: CobranzaComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private cobranzaService: CobranzaService,
        private cajaService: CajaService,
        private cobroTipoService: CobroTipoService, private depositoService: DepositoService, private cuentasPorCobrarService: CuentaPorCobrarService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'caja': ['', Validators.required],
            'idCobroTipo': ['', Validators.required],
            'referenciaAlf': [''],
            'referenciaNum': [''],            
            'observaciones': [''],
            //'monto': [''],
            'monto': ['0', _ => this.validateMontos()],
            'acumulado': ['0', _ => this.validateMontos2()]            
        });
    }
    
    validateMontos() {
        console.log(this.cobranza.total + " " + this.acumulado);
        if ( Math.abs(this.cobranza.total - this.acumulado)<=0.001 && this.acumulado > 0)
            return null;
        else
            return 'No coincide el monto deseado con el monto acumulado.';
    }

    validateMontos2() {
        console.log(this.cobranza.total + " " + this.acumulado);
        if (Math.abs(this.cobranza.total - this.acumulado) <= 0.001 && this.acumulado > 0) 
            return null;        
        else
            return 'No coincide el monto deseado con el monto acumulado.';
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.loadCajas();
        this.loadCobrosTipos();
        this.documentType = docType;
        this.deposito = new DepositoInfo();
    }

    onCreatingDocument() {
        this.cobranza.fechaAfectacion = new Date();
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.cobranzaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cobranza = doc;
                this.cuentasXCobrar.sourceCollection = this.cobranza.details;
                this.calcularAcumulado();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.cobranzaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cobranza = doc;
                this.detailsEditor.clientIndexStart = this.cobranza.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.cobranzaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cobranza = doc;
                this.cuentasXCobrar.sourceCollection = this.cobranza.details;
                this.calcularAcumulado();
                this.detailsEditor.clientIndexStart = this.cobranza.details.length;
                if (this.cobranza.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        if (this.cuentasXCobrar.sourceCollection.filter(search => search.state == "MODIFIED" && search.cancelacionPendiente == true && search.importe>0).length == 0) {
            this.context.app.showSpinner();
            this.cobranza.details = this.cuentasXCobrar.sourceCollection;
            if (this.cobranza.idCobroTipo == 0 || this.cobranza.idCobroTipo == undefined)
                this.cobranza.idCobroTipo = 1;

            let CuentasXCobrar: string[] = [];
            for (let cxc of this.cuentasXCobrar.sourceCollection.filter(search => search.state == "MODIFIED")) {
                CuentasXCobrar.push(cxc.idCuentaPorCobrarTit)
            }

            this.cuentasPorCobrarService.verficarComprobanteTimbrado(CuentasXCobrar)
                .then(Timbrado => {
                    if (Timbrado) {
                        this.argsEditorEvent = args;
                        //this.context.app.hideSpinner();
                        this.cobranzaService.update(this.cobranza)
                            .then(updatedDoc => {
                                this.context.app.hideSpinner();
                                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                                this.reset();
                                this.cobranza = updatedDoc;
                                this.editor.document = updatedDoc;
                                this.editor.nextStep();
                            })
                            .catch(error => this.handleError(error));
                    }
                    else {
                        this.context.app.hideSpinner();
                        this.showAlert("Error: No se puede aplicar cobros a comprobantes no timbrados", "danger");
                    }
                })
                .catch(error => this.handleError(error));
        } else
            this.showAlert("La cobranza tiene comprobantes que estan con una cancelación pendiente", "danger");
        
        
    }

    onApplyingNextStep(args: WorkflowEventArgs) {   
        this.argsCurrentDocument = args;
        args.step.activities.forEach(activity => {
            if (activity.tag == 'APPLY') {
                if (this.cobranza.idDeposito.trim() != '') //Si proviene de un deposito, nos traemos la cuenta a la que se deposito
                    this.getCuentaBeneficiario(this.cobranza.idDeposito);
                else {
                    this.dlgCobranza.setCxC(this.cuentasXCobrar.sourceCollection.filter(x => x.importe > 0));
                    this.dlgCobranza.showSinCaja(this.cobranza.total, this.cobranza.idCaja, this.idDeudorDefault);
                }
            }
            else {
                this.context.app.showSpinner();
                this.cobranzaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
                    .then(_ => {
                        this.context.app.hideSpinner();
                        //Volver a cargar documento
                        this.onViewingDocument(args.document.id);
                    })
                    .catch(error => this.handleError(error));
            }
        });
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.cobranzaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.cobranza.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        //this.detailsEditor.detail = new CobranzaDetail();
        //this.detailsEditor.show();
    }

    onEditingDetail(detail: CobranzaDetail) {
        //this.detailsEditor.detail = detail;
        //this.detailsEditor.show();
    }

    onDeletingDetail(detail: CobranzaDetail) {
        //this.detailsList.deleteItem(detail);
    }

    //onListItemsUpdated(updatedDetails: CobranzaDetail[]) {
    //    if (this.detailsEditor.anticipo.identity != '') {
    //        this.anticipo = this.detailsEditor.anticipo;
    //        this.cobranza.total = this.anticipo.importe;
    //        this.cobranza.idAnticipo = this.anticipo.identity;
    //        this.cobranza.fechaAnticipo = this.anticipo.fechaSys;
    //    }
    //    this.cobranza.details = updatedDetails;
    //    this.detailsList.listItems = updatedDetails;        
    //    this.cuentasXCobrar.sourceCollection = updatedDetails;
    //}

    onSavingDetail(detail: CobranzaDetail) {
        //this.detailsList.updateItem(detail);
    }

    onDetailsLoaded(details: CobranzaDetail[]) {
        this.acumulado = 0;
        if (this.detailsEditor.deposito.identity != '') {
            this.deposito = this.detailsEditor.deposito;
            this.cobranza.total = this.deposito.monto;
            this.cobranza.idDeposito= this.deposito.identity;
            this.cobranza.fechaDeposito = this.deposito.fechaSys;
        }

        if (details.length > 0)
           this.idDeudorDefault = details[0].idDeudor;

        let detailsOrder: CobranzaDetail[] = details;
        this.detailsList.listItems = detailsOrder;
        this.cobranza.details = detailsOrder;
        this.cuentasXCobrar.sourceCollection = detailsOrder;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.detailsEditor.detail = new CobranzaDetail();
            this.detailsEditor.show();
        } else if (button.name = 'DECIMALES') {
            this.dialogDecimales.config.backdrop = false;
            this.dialogDecimales.show();
        }
    }

    //onImporteCheck(item: CobranzaDetail)
    //{  
    //    let condicionPago = this.cobranza.details.find(search => search.importe > 0 && search.condicionesPago != item.condicionesPago);
    //    if (condicionPago == undefined) {
    //        let tempCXC: CobranzaDetail[] = this.cuentasXCobrar.sourceCollection.slice();
    //        item.importe = item.saldo;
    //        item.state = "MODIFIED";                                   
    //        this.cuentasXCobrar.sourceCollection = tempCXC;   
    //        //this.cuentasXCobrar.refresh();
    //        setTimeout(_ => this.calcularAcumulado());  
    //    }
    //    else
    //        this.showAlert("No puede mezclar documentos de diferentes condiciones de pago[Credito/Contado]","danger")
    //}

    cellEditEndingHandler(sender: any, e: any) {
        let cxc: any = sender.rows[e.row].dataItem;  
        
        var flex = sender,
            oldVal = flex.getCellData(e.row, e.col),
            newVal = flex.activeEditor.value;               

        
        let condicionPago = this.cuentasXCobrar.sourceCollection.find(search => search.importe > 0 && search.condicionesPago != cxc.condicionesPago);
        if (condicionPago == undefined)
            setTimeout(_ => this.calcularAcumulado());        
        else {
            e.cancel = true;
            this.showAlert("No puede mezclar documentos de diferentes condiciones de pago[Credito/Contado]", "danger");
        }

        if (newVal > cxc.saldo) {
            this.showAlert("No puede abonar un importe mayor al saldo del documento", "danger");
            e.cancel = true;
            cxc.importe = cxc.saldo;
        }  

        
        if (!isNaN(newVal)) // returns true if NaN, otherwise false            
            cxc.saldoFin = cxc.saldo - newVal;
        else  //proviene del keydown ya que activaron el beginEdit
            e.cancel = true;

        cxc.state = "MODIFIED";    

    }

    onCellEditEnded(flex: any, e: any)
    {
        this.cuentasXCobrar.refresh();
        let Total: number = this.cobranza.total;
        this.cobranza.total = Total;
        setTimeout(_ => this.calcularAcumulado());        
    }

    keydown(grid, args, cell) {
        if (args.keyCode == 13) {
            
            let cxc: any = grid.rows[grid.selection.row].dataItem;

            if (this.acumulado == undefined)
                this.acumulado == 0;

            let disponible: number = this.cobranza.total - this.acumulado
            if (disponible < 0 || disponible == undefined || isNaN(disponible))
                disponible = 0;

            if (disponible > cxc.saldo)
                cxc.importe = cxc.saldo;
            else
                cxc.importe = disponible;


            cxc.saldoFin = cxc.saldo - cxc.importe;
            cxc.state = "MODIFIED"; 

            //let total: number = this.cobranza.total+1;
            //this.cobranza.total = total-1;
            
            //this.documentForm.get('monto').markAsPristine();
            //this.documentForm.get('monto').markAsPending();
            this.detailsList.refresh();    

            this.documentForm.get('monto').updateValueAndValidity();            

            setTimeout(_ => this.calcularAcumulado());
        }        
    }       

    loadCajas() {
        this.cajaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cajas => this.cajas = cajas)
            .catch(error => this.handleError(error));
    }

    loadCobrosTipos() {
        this.cobroTipoService.loadList(this.context.app.defaultSite.identity, false)
            .then(cobrosTipos => this.cobrosTipos = cobrosTipos)
            .catch(error => this.handleError(error));
    }

    calcularAcumulado() {
        this.acumulado = this.cobranza.details
            .map(c => c.importe)
            .reduce((sum, current) => sum + current);

        this.documentForm.get('monto').updateValueAndValidity();            
    }

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }

    cobroAceptado(formasPagoCobro: CobranzaFormaPagoDetail) {
        let cobro = { formasPago: formasPagoCobro }
        this.context.app.showSpinner();
        this.cobranzaService.nextStep(this.editor.document.identity, this.argsCurrentDocument.step.idWorkflowStep, cobro)
            .then(_ => {
                this.context.app.hideSpinner();
                //this.refreshList();
                this.dlgCobranza.clear();
                this.onViewingDocument(this.editor.document.identity);

                if (!this.argsEditorEvent.closeEditor && this.cobranza.identity == '') {
                    this.router.navigate(['../', this.editor.document.identity], { relativeTo: this.route });
                }
                else {
                    if (this.argsEditorEvent.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }


            })
            .catch(error => this.handleError(error));
    }

    //itemFormatter = (cell, item) => {
    //    cell.style.color = "red";
    //}

    getStatusColor(vencimiento: Date): string {

        let venc = new Date(vencimiento);
        let today = new Date();
        if (today < venc)
            return '#298A08';
        else
            return 'orangered';
    }

    getStatusColorCancelacion(cancelacionPendinte: boolean): string {

        if (!cancelacionPendinte)
            return '#298A08';
        else
            return 'orangered';
    }

    //itemFormatter = (panel, r, c, cell) => {
    //    if (panel.cellType === CellType.Cell) {
    //        const col: ColumnDefinition = this.columns[c];
    //        if (col.visible === true && col.colorBackground) {
    //            cell.style.backgroundColor = col.colorBackground;
    //        }
    //    }
    //};

    getCuentaBeneficiario(idDeposito: string) {
        this.depositoService.getDetail(idDeposito)
            .then(deposito => {
                this.idCuentaBeneficiario = deposito.idCuentaBancaria;
                this.dlgCobranza.setCxC(this.cuentasXCobrar.sourceCollection.filter(x=>x.importe>0));
                this.dlgCobranza.showSinCaja(this.cobranza.total, this.cobranza.idCaja, this.idDeudorDefault);
            })
            .catch(error => this.handleError(error));        
    }   

    assignFormat() {
        this.formato = 'C' + this.decimales.toString();
        this.dialogDecimales.hide();
        this.calcularAcumulado();
    }
        

}



