﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem, FileLink } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem, DownloadPopupComponent } from 'framework3/controls';

// Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Locales
import { CobranzaInfo, CobranzaItem } from '../models/cobranza';
import { CobranzaDetail } from '../models/cobranza-detail';
import { CobranzaService } from '../services/cobranza.service';
import { DepositoService } from '../services/deposito.service';

import { PolizaInfo, PolizaPerfilListComponent, PolizaPerfilItem, PolizaPerfilService, PolizaService } from '../../accounting';

//TODO.ALEX. Mover a donde se vaya a desplegar el cobro
import { CobranzaComponent } from '../parts/cobranza.component';
import { CobranzaFormaPagoDetail } from '../models/forma-pago-detail';
//

@Component({
    selector: 'credito-cobranzas-manager',
    templateUrl: 'cobranzas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CobranzasManager)
        },
        {
            provide: DocumentService, useExisting: CobranzaService
        }
    ]
})
export class CobranzasManager extends DocumentManager {
    cobranzasTit: CobranzaItem[];
    argsCurrentDocument: WorkflowEventArgs;
    idCuentaBeneficiario: number = 0;
    currentPoliza: PolizaInfo = new PolizaInfo();

    //TODO.ALEX. Mover a donde se vaya a desplegar el cobro
    @ViewChild('dlgCobranza') dlgCobranza: CobranzaComponent;
    @ViewChild('viewDialog') dlgPolizaViewer: ModalDirective;
    //
    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;
    @ViewChild(PolizaPerfilListComponent) dlgPolizaPerfil: PolizaPerfilListComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private cobranzaService: CobranzaService, private depositoService: DepositoService, private polizaSvc: PolizaService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.cobranzaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.cobranzasTit = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.cobranzaService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.argsCurrentDocument = args;

        let cobranza = this.cobranzasTit.find(doc => doc.id == args.document.id)
        console.log(JSON.stringify(args) + " " + JSON.stringify(cobranza));
        this.context.app.showSpinner();
        this.cobranzaService.getDetail(cobranza.id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.dlgCobranza.setCxC(doc.details.filter(x => x.importe > 0))
                if (cobranza.idDeposito.trim() != '')
                    this.getCuentaBeneficiario(cobranza);
                else
                    this.dlgCobranza.showSinCaja(cobranza.total, cobranza.idCaja, cobranza.idDeudorDefault); 
            })
            .catch(error => this.handleError(error));               
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.cobranzaService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

        
    cobroAceptado(formasPagoCobro: CobranzaFormaPagoDetail) {
        let cobro = { formasPago: formasPagoCobro }
        this.context.app.showSpinner();
        this.cobranzaService.nextStep(this.argsCurrentDocument.document.id, this.argsCurrentDocument.step.idWorkflowStep, cobro)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
                this.dlgCobranza.clear();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.cobranzaService.generarPdfFactura(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + file.uri, '_blank');
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "DOWNLOAD") {
            this.context.app.showSpinner();
            //Generar PDF
            this.cobranzaService.generarPdfFactura(this.manager.currentDocument.id)
                .then(pdf => {
                    //Descargar xml
                    this.cobranzaService.getXmlFactura(this.manager.currentDocument.id)
                        .then(xml => {
                            this.context.app.hideSpinner();
                            let links: FileLink[] = [];
                            links.push(pdf);
                            links.push(xml);
                            this.downloadPopup.show(links);
                        })
                        .catch(error => this.handleError(error));

                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "SUSTITUIR") {
            this.context.app.showSpinner();            
            this.cobranzaService.sustituir(this.manager.currentDocument.id)
                .then(_ => {
                    this.context.app.hideSpinner();
                    this.manager.showAlert('El comprobante de pago se ha sustituido con éxito', 'success');
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "CONTABILIZAR") {
            this.dlgPolizaPerfil.show();
        }
    }
    getStatusColor(uuid: string): string {        
        if (uuid != '')
            return '#298A08';
        else
            return 'orangered';
    }

    getCuentaBeneficiario(cobranza: CobranzaItem) {
        this.depositoService.getDetail(cobranza.idDeposito)
            .then(deposito => {
                this.idCuentaBeneficiario = deposito.idCuentaBancaria;
                this.dlgCobranza.showSinCaja(cobranza.total, cobranza.idCaja, cobranza.idDeudorDefault);
            })
            .catch(error => this.handleError(error));
    }

    getTimbradoColor(folio: number, timbrado: boolean): string {
        if(folio==0)
            return ''
        else if (timbrado)
            return '#298A08';
        else
            return 'orangered';
            //let venc = new Date(vencimiento);
            //let today = new Date();
            //if (today < venc)
            //    return '#298A08';
            //else
            //    return 'orangered';        
    }

    onSelectedPolizaPerfil(item: PolizaPerfilItem) {
        this.context.app.showSpinner();
        this.polizaSvc.loadPolizaByDocumento(this.manager.currentDocument.id, item.id, this.context.app.defaultSite.identity)
            .then(result => {
                this.currentPoliza = result;
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.config.backdrop = false;
                this.dlgPolizaViewer.show();
            }).catch(error => this.handleError(error));
    }

    onClickGuardar() {
        this.context.app.showSpinner();
        if (this.currentPoliza.idPolizaOrigen == 0) this.currentPoliza.idPolizaOrigen = 1 //Manual
        this.polizaSvc.update(this.currentPoliza)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.hide();
                this.manager.showAlert('La poliza ha sido registrada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }

}

