﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { CuentaPorCobrarInfo, CuentaPorCobrarItem } from '../models/cuenta-por-cobrar';
import { CuentaPorCobrarDetail } from '../models/cuenta-por-cobrar-detail';
import { CuentaPorCobrarService } from '../services/cuenta-por-cobrar.service';
import { CuentaPorCobrarDetailEditor } from './parts/cuentas-por-cobrar-detail.editor';

@Component({
    selector: 'credito-cuentas-por-cobrar-editor',
    templateUrl: './cuentas-por-cobrar.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CuentasPorCobrarEditor)
        },
        {
            provide: DocumentService, useExisting: CuentaPorCobrarService
        }
    ]
})
export class CuentasPorCobrarEditor extends DocumentEditor {
    cuentaPorCobrar = new CuentaPorCobrarInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: CuentaPorCobrarDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private cuentaPorCobrarService: CuentaPorCobrarService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'sample': ['', Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.cuentaPorCobrarService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cuentaPorCobrar = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.cuentaPorCobrarService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cuentaPorCobrar = doc;
                this.detailsEditor.clientIndexStart = this.cuentaPorCobrar.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.cuentaPorCobrarService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cuentaPorCobrar = doc;
                this.detailsEditor.clientIndexStart = this.cuentaPorCobrar.details.length;
                if (this.cuentaPorCobrar.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.cuentaPorCobrarService.update(this.cuentaPorCobrar)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.cuentaPorCobrar.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.cuentaPorCobrar = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.cuentaPorCobrarService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.cuentaPorCobrarService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.cuentaPorCobrar.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new CuentaPorCobrarDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: CuentaPorCobrarDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: CuentaPorCobrarDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: CuentaPorCobrarDetail[]) {
        this.cuentaPorCobrar.details = updatedDetails;
    }

    onSavingDetail(detail: CuentaPorCobrarDetail) {
        this.detailsList.updateItem(detail);
    }

}

