﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Locales
import { CuentaPorCobrarInfo, CuentaPorCobrarItem } from '../models/cuenta-por-cobrar';
import { CuentaPorCobrarDetail } from '../models/cuenta-por-cobrar-detail';
import { CuentaPorCobrarService } from '../services/cuenta-por-cobrar.service';

@Component({
    selector: 'credito-cuentas-por-cobrar-manager',
    templateUrl: 'cuentas-por-cobrar.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CuentasPorCobrarManager)
        },
        {
            provide: DocumentService, useExisting: CuentaPorCobrarService
        }
    ]
})
export class CuentasPorCobrarManager extends DocumentManager {
    cuentasPorCobrarTit: CuentaPorCobrarItem[];

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private cuentaPorCobrarService: CuentaPorCobrarService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        console.log('Datos de Parametro: ' + this.context.app.defaultSite.identity + ' ');
        console.log('Request: '+JSON.stringify(request));
        this.cuentaPorCobrarService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.cuentasPorCobrarTit = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.cuentaPorCobrarService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.cuentaPorCobrarService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.cuentaPorCobrarService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    
    onMenuItemClick(option: MenuItem) {
        if (option.name == 'CANCEL') {
            this.context.app.showSpinner();
            this.cuentaPorCobrarService.requerirCancelacion(this.manager.currentDocument.id)
                .then(response => {
                    //
                    this.context.app.hideSpinner();
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
    }
    

}

