﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { DepositoInfo, DepositoItem } from '../models/deposito';
import { DepositoDetail } from '../models/deposito-detail';
import { DepositoService } from '../services/deposito.service';

@Component({
    selector: 'credito-depositos-manager',
    templateUrl: './depositos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DepositosManager)
        },
        {
            provide: DocumentService, useExisting: DepositoService
        }
    ]
})
export class DepositosManager extends DocumentManager {
    depositos: DepositoItem[];

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private depositoService: DepositoService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.depositoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.depositos = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.depositoService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        if (args.step.tag == 'APPLY') {
            this.context.app.hideSpinner();
            this.showAlert('El documento solo se aplica desglosando el cobro en el proceso de cobranza', 'warning');
        }
        else {
            this.depositoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
                .then(_ => {
                    this.context.app.hideSpinner();
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();
        if (document.workflowStep == 'APPLY') {
            this.context.app.hideSpinner();
            this.showAlert('El documento en estatus aplicado solo puede ser cancelado en el proceso de cobranza', 'warning');
        }
        else
        this.depositoService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

}

