﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, CollectionView } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, AzComboboxComponent } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { NotaInfo, NotaItem } from '../models/nota';
import { FacturaInfo } from '../models/factura';
import { NotaRelaciones } from '../models/nota-relaciones';
import { CuentaPorCobrarInfo } from '../models/cuenta-por-cobrar';
import { NotaDetail } from '../../accounting/models/nota-detail';
import { NotaService } from '../services/nota.service';
//import { NotaDetailEditor } from '../../accounting/documents/parts/notas-detail.editor';
import { NotaDetailEditor } from '../../credit/documents/parts/notas-detail.editor';
import { DivisaService } from '../../accounting/services/divisa.service';

import { FormaPagoService } from '../../accounting/services/forma-pago.service';
import { FormaPagoItem } from '../../accounting/models/forma-pago';

import { ContribuyenteService } from '../../accounting/services/contribuyente.service';
import { GrupoDeudorInfo,  DeudorInfo, FacturaSelectorComponent, CuentasPorCobrarSelectorComponent, DeudorSelectorComponent } from '../../credit';
import { UsoCFDIItem } from '../../accounting/models/contribuyente';
import { NotaTipoInfo, NotaTipoItem } from '../models/nota-tipo';
import { NotaTipoService } from '../services/nota-tipo.service';

//Cobranza
import { CobranzaDetail } from '../models/cobranza-detail';
import { CobranzaDetailEditor } from './parts/cobranzas-detail.editor';
import { CobranzaInfo, CobranzaItem } from '../models/cobranza';
import { ModalDirective } from 'ngx-bootstrap';



@Component({
    selector: 'credito-notas-editor',
    templateUrl: './notas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => NotasEditor)
        },
        {
            provide: DocumentService, useExisting: NotaService
        }
    ]
})
export class NotasEditor extends DocumentEditor {
    nota = new NotaInfo();
    formasPago: FormaPagoItem[] = [];
    version3p3: boolean;
    version3p2: boolean;
    usosCFDI: UsoCFDIItem[] = [];
    notaTipos: NotaTipoItem[] = [];
    //Cobranza
    acumulado: number = 0;
    sumaDetalle: number = 0;

    formato: string = "C2";
    decimales: number = 2;

    idDeudorDefault: number;
    cobranza = new CobranzaInfo();
    cuentasXCobrar: CollectionView = new CollectionView();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: NotaDetailEditor;
    @ViewChild('cobranzaDetailsEditor') cobranzaDetailsEditor: CobranzaDetailEditor;
    @ViewChild('cboAplica') cboAplica: AzComboboxComponent;
    @ViewChild('dialogDecimales') dialogDecimales: ModalDirective;


    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private notaService: NotaService,
        private formaPagoService: FormaPagoService,
        private divisaService: DivisaService,
        private contribuyenteService: ContribuyenteService,
        private notaTipoSvc: NotaTipoService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'divisa': [0, Validators.required],
            'idFormaPago': [0, Validators.required],
            'observaciones': ['', Validators.required],
            'idUsoCFDI': [0, Validators.required],
            'idNotaTipo': [0, Validators.required],
            'monto': ['0', _ => this.validateMontos()],
            'acumulado': ['0', _ => this.validateMontos2()]            
        });

    }



    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.loadFormasPago();
        this.applyPolicies();
        this.getDivisaDefault();
        this.loadUsosCFDI();
        this.loadNotaTipos();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.notaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nota = doc;
                this.detailsEditor.clientIndexStart = this.nota.details.length;
                //this.calcularAcumulado();
                this.setReadonlyMode();

                this.cuentasXCobrar.sourceCollection = this.nota.relaciones;

            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.notaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nota = doc;
                this.detailsEditor.clientIndexStart = this.nota.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.notaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nota = doc;
                //this.calcularAcumulado();
                this.detailsEditor.clientIndexStart = this.nota.details.length;
                if (this.nota.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();

                this.cuentasXCobrar.sourceCollection = this.nota.relaciones;
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        if (this.cuentasXCobrar.sourceCollection.filter(search => search.state == "MODIFIED" && search.cancelacionPendiente == true && search.importe > 0).length == 0) {
            this.context.app.showSpinner();

            this.cobranza.details.forEach(detail => {
                if (detail.importe > 0) {
                    this.asignarCobranza(detail);
                }
            });


            if (this.cobranzaDetailsEditor.tipoBusqueda == 'D') {
                this.nota.idDeudor = this.cobranzaDetailsEditor.idDeudor
                this.nota.idGrupoDeudor = 0
            }

            if (this.cobranzaDetailsEditor.tipoBusqueda == 'G') {
                this.nota.idGrupoDeudor = this.cobranzaDetailsEditor.idGrupoDeudor
                this.nota.idDeudor = 0
            }


            this.notaService.update(this.nota)
                .then(updatedDoc => {
                    this.notaService.nextStep(updatedDoc.identity, 1, '')
                        .then(n => {
                            this.context.app.hideSpinner();
                            //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                            this.reset();
                            this.router.navigate(['../'], { relativeTo: this.route });
                            /*
                            if (!args.closeEditor && this.nota.identity == '') {
                                this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                            }
                            else {
                                if (args.closeEditor)
                                    this.router.navigate(['../'], { relativeTo: this.route });
                                else
                                    this.showAlert('La información ha sido actualizada con éxito', 'success');
                            }
                            */
                            this.nota = updatedDoc;
                        })
                        .catch(error => this.handleError(error));
                })
                .catch(error => this.handleError(error));
        } else
            this.showAlert("La nota tiene comprobantes que estan con una cancelación pendiente", "danger");
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.notaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.notaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.nota.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new NotaDetail();
        this.detailsEditor.idNotaTipo = this.nota.idNotaTipo;
        this.detailsEditor.show();
    }

    onEditingDetail(detail: NotaDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.idNotaTipo = this.nota.idNotaTipo;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: NotaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onDetailsLoaded(details: NotaDetail[]) {
        
    }

    onListItemsUpdated(updatedDetails: NotaDetail[]) {
        this.nota.notaBase.details = updatedDetails;
    }

    onSavingDetail(detail: NotaDetail) {
        this.detailsList.updateItem(detail);
        //this.sumaDetalle = 345;
        setTimeout(_ => this.calcularMonto());
        setTimeout(_ => this.calcularAcumulado());
    }

    loadFormasPago() {
        this.formaPagoService.loadList(this.context.app.defaultSite.identity)
            .then(formas => this.formasPago = formas)
            .catch(error => this.handleError(error));
    }

    asignarCobranza(cPc: CobranzaDetail) {

        var rel: NotaRelaciones = new NotaRelaciones;

        rel.idCuentaPorCobrar = cPc.idCuentaPorCobrarTit;
        rel.idNotaRelacion = 0;
        rel.saldoInicial = cPc.saldo;
        rel.importe = cPc.importe;
        rel.saldoFinal = cPc.saldo;

        var newRel = this.nota.relaciones.slice();

        newRel.push(rel);


        this.nota.relaciones = newRel;

    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {

            //this.detailsEditor.detail = new CobranzaDetail();
            //this.detailsEditor.show();

            this.cobranzaDetailsEditor.detail = new CobranzaDetail();
            this.cobranzaDetailsEditor.show();

        }
        else if (button.name == 'DECIMALES') {
            this.dialogDecimales.config.backdrop = false;
            this.dialogDecimales.show();
        }
    }
    //Cobranza//Init
    onDetailsCobranzaLoaded(details: CobranzaDetail[]) {
        this.acumulado = 0;
        if (details.length > 0)
            this.idDeudorDefault = details[0].idDeudor;

        let detailsOrder: CobranzaDetail[] = details;
        //this.detailsList.listItems = detailsOrder;
        this.cobranza.details = detailsOrder;
        this.cuentasXCobrar.sourceCollection = detailsOrder;
    }

    getStatusColor(vencimiento: Date): string {

        let venc = new Date(vencimiento);
        let today = new Date();
        if (today < venc)
            return '#298A08';
        else
            return 'orangered';
    }

    keydown(grid, args, cell) {
        if (args.keyCode == 13) {

            let cxc: any = grid.rows[grid.selection.row].dataItem;

            if (this.acumulado == undefined)
                this.acumulado == 0;

            let disponible: number = this.cobranza.total - this.acumulado
            if (disponible < 0 || disponible == undefined || isNaN(disponible))
                disponible = 0;

            if (disponible > cxc.saldo)
                cxc.importe = cxc.saldo;
            else
                cxc.importe = disponible;

            cxc.state = "MODIFIED";

            //let total: number = this.cobranza.total+1;
            //this.cobranza.total = total-1;

            //this.documentForm.get('monto').markAsPristine();
            //this.documentForm.get('monto').markAsPending();
            this.detailsList.refresh();

            this.documentForm.get('monto').updateValueAndValidity();


            setTimeout(_ => this.calcularAcumulado());

        }
    }      

    calcularAcumulado() {
        if (this.cobranza.details.length > 0) {
            this.acumulado = this.cobranza.details
                .map(c => c.importe)
                .reduce((sum, current) => sum + current);
            this.acumulado = Number(this.acumulado.toFixed(this.decimales))
            this.documentForm.get('acumulado').updateValueAndValidity();
        }
    }

    validateMontos() {
        if (this.cobranza.total == this.acumulado && this.acumulado > 0)
            return null;
        else
            return 'No coincide el monto deseado con el monto acumulado.';
    }

    validateMontos2() {
        if (this.cobranza.total == this.acumulado && this.acumulado > 0)
            return null;
        else
            return 'No coincide el monto deseado con el monto acumulado.';
    }


    calcularMonto() {
        if (this.nota.notaBase.details.length > 0) {
            this.sumaDetalle = this.nota.notaBase.details
                .map(c => c.importe)
                .reduce((sum, current) => sum + current);

            this.cobranza.total = this.sumaDetalle;

            this.documentForm.get('monto').updateValueAndValidity();

            
        } 
    }


    cellEditEndingHandler(sender: any, e: any) {
        let cxc: any = sender.rows[e.row].dataItem;

        var flex = sender,
            oldVal = flex.getCellData(e.row, e.col),
            newVal = flex.activeEditor.value;

        let condicionPago = this.cuentasXCobrar.sourceCollection.find(search => search.importe > 0 && search.condicionesPago != cxc.condicionesPago);
        if (condicionPago == undefined)
            setTimeout(_ => this.calcularAcumulado());
        else {
            e.cancel = true;
            this.showAlert("No puede mezclar documentos de diferentes condiciones de pago[Credito/Contado]", "danger");
        }

        if (newVal > cxc.saldo) {
            this.showAlert("No puede abonar un importe mayor al saldo del documento", "danger");
            e.cancel = true;
            cxc.importe = cxc.saldo;
        }
        cxc.state = "MODIFIED";

    }

    onCellEditEnded(flex: any, e: any) {
        this.cuentasXCobrar.refresh();
        let Total: number = this.cobranza.total;
        this.cobranza.total = Total;
        setTimeout(_ => this.calcularAcumulado());
        setTimeout(_ => this.calcularMonto());
    }


    //Cobranza//End
    getDivisaDefault() {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.nota.notaBase.idDivisa = r.identity;
            })
            .catch(error => this.handleError(error));

    }

    loadUsosCFDI() {
        this.contribuyenteService.loadUsosCFDI(this.context.app.defaultCompany.identity)
            .then(usos => this.usosCFDI = usos)
            .catch(error => this.handleError(error));
    }

    loadNotaTipos() {
        this.notaTipoSvc.loadList(this.context.app.defaultCompany.identity)
            .then(tipos => this.notaTipos = tipos)
            .catch(error => this.handleError(error));
    }

    assignFormat() {
        this.formato = 'C' + this.decimales.toString();
        this.dialogDecimales.hide();
        this.calcularAcumulado();
        this.calcularMonto();
        
    }

    getStatusColorCancelacion(cancelacionPendinte: boolean): string {

        if (!cancelacionPendinte)
            return '#298A08';
        else
            return 'orangered';
    }
    


}

