﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators, FormControl } from '@angular/forms';
import * as moment from 'moment';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { AztecaValidators, AlertsBarComponent } from 'framework3/controls';
//Locales
import { CobranzaDetail } from '../../models/cobranza-detail';
import { CobranzaService} from '../../services/cobranza.service';

import { GrupoDeudorInfo } from '../../models/grupo-deudor';
import { DeudorInfo } from '../../models/deudor';
import { DepositoInfo } from '../../models/deposito';
import { ContribuyenteInfo } from '../../../accounting/models/contribuyente';


@Component({
    selector: 'credito-cobranza-detail-editor',
    templateUrl: './cobranzas-detail.editor.html'
})
export class CobranzaDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new CobranzaDetail();
    //Objeto original que se editará
    @Input() detail: CobranzaDetail = new CobranzaDetail();
    @Input() depositosVisible: boolean = false;

    details: CobranzaDetail[] = [];
    @Output() dataLoaded: EventEmitter<CobranzaDetail[]> = new EventEmitter<CobranzaDetail[]>();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent

  
    busquedaTipos: { id: string, nombre: string }[] = [];
    fechasFiltro: { id: string, nombre: string }[] = [];

    tipoBusqueda: string;
    fechaSearch: string = 'F';
    idGrupoDeudor: number;
    idDeudor: number;
    idDeposito: string;
    idContribuyente: number;
    rfc: string;
    fechaIni: Date;
    fechaFin: Date;
    @Output() deposito = new DepositoInfo();

    constructor(protected context: Context, private builder: FormBuilder, private cobranzaService: CobranzaService) {
        super(context);

        this.form = builder.group({
            'busqueda': ['',Validators.required],
            'idGrupoDeudor': [''],
            'idDeudor': [''],
            'idDeposito': [''],
            'idContribuyente': [''],
            'fechaIni': [new Date()],
            'fechaFin': [new Date()],
            'fechaBusqueda': ['', Validators.required],
        });

        this.form.controls['busqueda'].valueChanges.subscribe(value => {
            if (value == 'G') {
                this.form.controls['idGrupoDeudor'].setValidators(AztecaValidators.selected);                
            }
            else {
                this.form.controls['idGrupoDeudor'].setValidators(null);
            }

            if (value == 'D') {
                this.form.controls['idDeudor'].setValidators(AztecaValidators.selected);
            }
            else {
                this.form.controls['idDeudor'].setValidators(null);
            }

            if (value == 'C') {
                this.form.controls['idContribuyente'].setValidators(AztecaValidators.selected);
            }
            else {
                this.form.controls['idContribuyente'].setValidators(null);
            }

            if (value == 'X') {
                this.form.controls['idDeposito'].setValidators(AztecaValidators.selected);
            }
            else {
                this.form.controls['idDeposito'].setValidators(null);
            }

            this.form.updateValueAndValidity();
        });

        this.busquedaTipos =
            [
                { "id": "G", "nombre": "Grupo Deudor" },
                { "id": "D", "nombre": "Deudor" },            
            ];

        if (this.depositosVisible)
            this.busquedaTipos.push({ "id": "X", "nombre": "Deposito" });

        this.fechasFiltro =
            [
                { "id": "F", "nombre": "Factura" },
                { "id": "V", "nombre": "Vencimiento" }

            ];

        this.fechaSearch = "F";
        this.fechaIni = moment().subtract(3, 'months').toDate();
        this.fechaFin = moment().add(2, 'week').toDate();
    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {        
        this.fechaSearch = "F";
        this.deposito = new DepositoInfo();
        this.idDeposito = '';
        this.form.reset({ sample: '' });
        this.form.markAsPristine();
        this.fechaIni = moment().subtract(3, 'months').toDate();
        this.fechaFin = moment().add(2, 'week').toDate();
        this.fechaSearch = "F";
        this.form.get('fechaBusqueda').setValue("F");
        if (this.depositosVisible)
            this.busquedaTipos.push({ "id": "X", "nombre": "Deposito" });
    }

    onLoadingItem(detail: CobranzaDetail ) {
        this.detailCopy = <CobranzaDetail>this.copyItem(this.detail);
        if (this.depositosVisible)
            this.busquedaTipos.push({ "id": "X", "nombre": "Deposito" });
    }

    onUpdatingItem(detail: CobranzaDetail) {
        //if (this.detailCopy._clientID == 0) {
        //    this.detailCopy._clientID = this.createClientID();
        //}
        //else {
        //    this.detailCopy.state = ChangeState.MODIFIED;
        //}
        //this.savingItem.emit(this.detailCopy);
        if (this.deposito.idDeudor > 0) {
            this.idDeudor = this.deposito.idDeudor;        
        }
        if (this.deposito.idGrupoDeudor > 0) {
            this.idGrupoDeudor = this.deposito.idGrupoDeudor;        
        }

        if (this.tipoBusqueda == "G" || this.deposito.idGrupoDeudor>0)
            this.loadCxCbyGrupoDeudor();
        else if (this.tipoBusqueda == "D" || this.deposito.idDeudor>0)
            this.loadCxCbyDeudor();
        else if (this.tipoBusqueda == "C")
            this.loadCxCbyRFC();       
    }

    loadCxCbyGrupoDeudor() {
        this.context.app.showSpinner();
        this.cobranzaService.loadCxCByGrupoDeudor(moment(this.fechaIni).format(), moment(this.fechaFin).format(), this.idGrupoDeudor, this.fechaSearch )
            .then(cxc => {
                this.context.app.hideSpinner();
                //this.details = cxc;
                this.dataLoaded.emit(cxc);
            })
            .catch(error => this.showAlert(error.message,'danger'));
    }
    loadCxCbyDeudor() {
        this.context.app.showSpinner();
        this.cobranzaService.loadCxCByDeudor(moment(this.fechaIni).format(), moment(this.fechaFin).format(), this.idDeudor, this.fechaSearch)
            .then(cxc => {
                this.context.app.hideSpinner();
                //this.details = cxc;
                this.dataLoaded.emit(cxc);
            })
            .catch(error => this.showAlert(error.message,'danger'));
    }
    loadCxCbyRFC() {
        this.context.app.showSpinner();
        this.cobranzaService.loadCxCByRFC(moment(this.fechaIni).format(), moment(this.fechaFin).format(), this.rfc)
            .then(cxc => {
                this.context.app.hideSpinner();
                //this.details = cxc;
                this.dataLoaded.emit(cxc);
            })
            .catch(error => this.showAlert(error.message, 'danger'));
    }

    selectedBusqueda(item: any) {
        this.tipoBusqueda = item;
        this.fechaSearch = "F";
    }

    onGrupoDeudorSelected(grupo: GrupoDeudorInfo) {
        this.idGrupoDeudor = grupo.identity;
    }

    onDeudorSelected(deudor: DeudorInfo) {
        this.idDeudor = deudor.identity;
    }

    onContribuyenteSelected(contribuyente: ContribuyenteInfo) {
        this.rfc= contribuyente.rfc;
    }      

    onDepositoSelected(deposito: DepositoInfo) {
        this.deposito = deposito;
       
    }      

    showAlert(message: string, type: string) {
        this.context.app.hideSpinner();
        this.alerts.showAlert(message, type);
    }
}
