﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location, CommonModule  } from '@angular/common'

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent, ToolbarButton } from 'framework3/controls';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
import { CatalogOptions } from 'framework3/custom';

//Features
//import { SerieFiscalItem } from '../../../accounting/models/serie-fiscal';
//import { ContribuyenteItem } from '../../../accounting/models/contribuyente';

import { CuentaPorCobrarService } from '../../services/cuenta-por-cobrar.service';
import { CuentaPorCobrarItem } from '../../models/cuenta-por-cobrar';
import * as moment from 'moment';
import * as _ from 'lodash';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import * as wjcCore from 'wijmo/wijmo';
//import * as wjcGrid from 'wijmo/wijmo.grid';

@Component({
    selector: 'credit-facturacion-component',
    templateUrl: './facturacion.component.html'
})
export class FacturacionComponent {
    editorForm: FormGroup;
    idGrupoDeudor: number;
    idDeudor: number=0;
    fechaIni: Date;
    fechaFin: Date;
    cxc: CuentaPorCobrarItem[] = [];
    subtotal: number;
    descuentos: number;
    impuestosTrasladados: number;
    impuestosRetenidos: number;
    total: number;
    
    detailMode = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandSingle];
    detailModeMulti = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandMulti];

    series: { serie: string, total: number }[] = [];

    busquedaTipos: { id: string, nombre: string }[] = [];    
    tipoBusqueda: string='D';
    
    private _CacheContribuyentes = {};
    private _CacheCondiciones = {};
    private _CacheDeudores = {};
    private _CacheOrigenes = {};
    private _CacheCXC = {};

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent

    constructor(private formBuilder: FormBuilder, private router: Router, private cuentaPorCobrarService: CuentaPorCobrarService, private context: Context) {
        this.editorForm = formBuilder.group({
            'fechaIni': [0, Validators.required],
            'fechaFin': [0, Validators.required],
            'busqueda':["D"],
            'idGrupoDeudor': [0],
            'idDeudor': [0],            
            'subtotal': [0],
            'descuentos': [0],
            'impuestosTrasladados': [0],
            'impuestosRetenidos': [0],
            'total': [0, Validators.required],
        });
        this.fechaIni = moment().subtract(2, 'week').toDate();
        //this.fechaFin = moment().add(2, 'week').toDate();
        this.busquedaTipos =
            [
                { "id": "G", "nombre": "Grupo Deudor" },
                { "id": "D", "nombre": "Deudor" },
            ];
    }

    onParamsToolbar(button: ToolbarButton) {
        this.context.app.showSpinner();

        if (button.name == 'REFRESH') {
            let idActor: number = this.idDeudor
            if (this.tipoBusqueda == 'G')
                idActor = this.idGrupoDeudor;

            this.cuentaPorCobrarService.loadFacturacionPendiente(this.context.app.defaultSite.identity, this.tipoBusqueda, idActor, this.fechaIni, this.fechaFin)
                .then(cxc => {
                    this.context.app.hideSpinner();
                    this.fillCxC(cxc);
                })
               .catch (error => this.handleError(error));   
        }      
    }

    reset() {
        this.cxc = [];
        this.series = [];
        this._CacheContribuyentes = {};
        this._CacheCondiciones = {};
        this._CacheDeudores = {};
        this._CacheOrigenes = {};
        this._CacheCXC = {};
        this.subtotal = 0;
        this.descuentos = 0;
        this.impuestosTrasladados = 0;
        this.impuestosRetenidos = 0;
        this.total = 0;
    }

    fillCxC(cxc: any) {
        let total: number = 0;
        this.reset();
        this.cxc = cxc;

        var series = _.chain(cxc).map('serie').uniq().value()
               
        series.forEach(serie => {
            total = 0;
            this.cxc.forEach(item => {
                if (item.serie == serie) {
                    total = total + item.total;
                }
            });           

            this.series.push({ 'serie': serie, 'total': total });
        });
    }

    getContribuyentes(serie: any) {
        let total: number = 0;
        let contribuyentes: { serie: string, contribuyente: string, total: number } [] = [];

        var view = this._CacheContribuyentes[serie.serie];
        if (!view) {                       
            view = new wjcCore.CollectionView(this.cxc);
            view.filter = function (item) {
                return item.serie == serie.serie;
            }

            var contribuyentesUniques = _.chain(view.sourceCollection).map('contribuyente').uniq().value()
            contribuyentesUniques.forEach(contribuyente => {                
                total = 0;
                this.cxc.forEach(item => {                    
                    if (item.serie == serie.serie && item.contribuyente==contribuyente)
                        total = total + item.total;
                });                  
                if (total != 0)
                    contribuyentes.push({ 'serie': serie.serie, 'contribuyente': contribuyente, 'total': total});
            });

            this._CacheContribuyentes[serie.serie] = contribuyentes;
            view = contribuyentes;
        }
        return view;
    }

    getCondiciones(contribuyente: any) {
        let total: number = 0;
        let condicionesPago: { serie: string, contribuyente: string, condicionesPago: string; total: number } [] = [];
        let idCache = contribuyente.serie + '|' + contribuyente.contribuyente;
        
        var view = this._CacheCondiciones[idCache];
        if (!view) {
            view = new wjcCore.CollectionView(this.cxc);
            view.filter = function (item) {
                return item.serie == contribuyente.serie && item.contribuyente == contribuyente.contribuyente;
            }

            var condiciones = _.chain(view.sourceCollection).map('condicionesPago').uniq().value()
            condiciones.forEach(condicion => {
                total = 0;
                this.cxc.forEach(item => {
                    if (item.serie == contribuyente.serie && item.contribuyente == contribuyente.contribuyente && item.condicionesPago == condicion)
                        total = total + item.total;
                });
                if(total != 0)
                    condicionesPago.push({ 'serie': contribuyente.serie, 'contribuyente': contribuyente.contribuyente, 'condicionesPago': condicion, 'total': total });
            });

            this._CacheCondiciones[idCache] = condicionesPago;
            view = condicionesPago;
        }
        return view;
    }

    getDeudores(condiciones: any) {
        let total: number = 0;
        let deudores: { serie: string, contribuyente: string, condicionesPago: string; codigoDeudor: string, deudor:string, total: number }[] = [];
        let idCache = condiciones.serie + '|' + condiciones.contribuyente + '|' + condiciones.condicionesPago;

        var view = this._CacheDeudores[idCache];
        if (!view) {
            view = new wjcCore.CollectionView(this.cxc);
            view.filter = function (item) {
                return item.serie == condiciones.serie && item.contribuyente == condiciones.contribuyente && item.condicionesPago == condiciones.condicionesPago;
            }

            var deudoreUniques = _.chain(view.sourceCollection).map('codigoDeudor').uniq().value()
            deudoreUniques.forEach(codigoDeudor => {
                total = 0;
                this.cxc.forEach(item => {
                    if (item.serie == condiciones.serie && item.contribuyente == condiciones.contribuyente && item.condicionesPago == condiciones.condicionesPago && item.codigoDeudor == codigoDeudor)
                        total = total + item.total;
                });
                if (total != 0) {
                    let deudor:string = this.cxc.find(item => item.codigoDeudor == codigoDeudor).deudor;
                    deudores.push({ 'serie': condiciones.serie, 'contribuyente': condiciones.contribuyente, 'condicionesPago': condiciones.condicionesPago, 'codigoDeudor': codigoDeudor, 'deudor': deudor, 'total': total });
                }
            });

            this._CacheDeudores[idCache] = deudores;
            view = deudores;
        }
        return view;
    }
    
    getOrigenes(deudores: any) {
        let total: number = 0;
        let origenes: { serie: string, contribuyente: string, condicionesPago: string; codigoDeudor: string, origen: string, total: number }[] = [];
        let idCache = deudores.serie + '|' + deudores.contribuyente + '|' + deudores.condicionesPago +'|'+ deudores.codigoDeudor;

        var view = this._CacheOrigenes[idCache];
        if (!view) {
            view = new wjcCore.CollectionView(this.cxc);
            view.filter = function (item) {
                return item.serie == deudores.serie && item.contribuyente == deudores.contribuyente && item.condicionesPago == deudores.condicionesPago && item.codigoDeudor==deudores.codigoDeudor ;
            }

            var origenesUniques = _.chain(view.sourceCollection).map('origen').uniq().value()
            origenesUniques.forEach(origen => {
                total = 0;
                this.cxc.forEach(item => {
                    if (item.serie == deudores.serie && item.contribuyente == deudores.contribuyente && item.condicionesPago == deudores.condicionesPago && item.codigoDeudor == deudores.codigoDeudor &&  item.origen== origen )
                        total = total + item.total;
                });
                if (total != 0)
                    origenes.push({'serie': deudores.serie, 'contribuyente': deudores.contribuyente, 'condicionesPago': deudores.condicionesPago, 'codigoDeudor': deudores.codigoDeudor, 'origen':origen, 'total': total });
            });

            this._CacheOrigenes[idCache] = origenes;
            view = origenes;
        }
        return view;
    }


    getCxC(origen: any) {
        let total: number = 0;
        let idCache = origen.serie + '|' + origen.contribuyente + '|' + origen.condicionesPago + '|' + origen.codigoDeudor + ' | '+ origen.origen;

        var view = this._CacheCXC[idCache];
        if (!view) {
            view = new wjcCore.CollectionView(this.cxc);
            view.filter = function (item) {
                return item.serie == origen.serie && item.contribuyente == origen.contribuyente && item.condicionesPago == origen.condicionesPago && item.codigoDeudor == origen.codigoDeudor && item.origen == origen.origen;
            }
            this._CacheCXC[idCache] = view._view;
            view = view._view;
        }        
        return view;
    }

    cellEditEndingHandler(flex: any, e: any) {
        let cxc: any = flex.rows[e.row].dataItem;

        let validarSerie = this.cxc.find(search => search.sel==true && search.serie != cxc.serie);
        if (validarSerie != undefined){
            e.cancel = true;
            this.showAlert("No puede mezclar documentos de diferentes series", "danger");
        }

        let validarContribuyente = this.cxc.find(search => search.sel == true && search.contribuyente != cxc.contribuyente);
        if (validarContribuyente != undefined){
            e.cancel = true;
            this.showAlert("No puede mezclar documentos de diferentes contribuyentes", "danger");
        }

        let validarCondicionesPago = this.cxc.find(search => search.sel == true && search.condicionesPago != cxc.condicionesPago);
        if (validarCondicionesPago != undefined){
            e.cancel = true;
            this.showAlert("No puede mezclar documentos de diferentes condiciones de pago[Credito/Contado]", "danger");
        }        

        this.calcularAcumulado();
    }


    onCellEditEnded(flex: any, row: any) {
        this.calcularAcumulado();
    }

    calcularAcumulado()
    {
        var view = new wjcCore.CollectionView(this.cxc);
        view.filter = function (item) {
            return item.sel == true;
        }

        this.subtotal = view._view
            .map(c => c.subtotal)
            .reduce((sum, current) => sum + current)            
        this.descuentos = view._view
            .map(c => c.descuentos)
            .reduce((sum, current) => sum + current)
        this.impuestosTrasladados = view._view
            .map(c => c.impuestosTrasladados)
            .reduce((sum, current) => sum + current)
        this.impuestosRetenidos = view._view
            .map(c => c.impuestosRetenidos)
            .reduce((sum, current) => sum + current)
        this.total = view._view
            .map(c => c.total)
            .reduce((sum, current) => sum + current)
    }

    onAccept() {
        var view = new wjcCore.CollectionView(this.cxc);
        view.filter = function (item) {
            return item.sel == true;
        }

        let item: CuentaPorCobrarItem = this.cxc.find(item => item.sel == true);

        if (item !=null)
            this.cuentaPorCobrarService.GeneraFactura(item.idCFDSerie, this.context.app.defaultSite.identity, item.idContribuyente, view._view)
                .then(_ => {
                    this.showAlert('Se ha generado la factura', 'success');
                    this.reset();
                })
                .catch(error => this.handleError(error));
    }

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }

    handleError(error: any) {
        console.error(error);
        this.alerts.showAlert(error.tostring(), 'danger');
    }

    onGrupoDeudorSelected(grupoDeudor: any) {

    }
    onDeudorSelected(deudor: any) {

    }

  
}