﻿import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
//Locales
import { NotaDetail } from '../../../accounting/models/nota-detail';
import { ImpuestoDetailInfo } from '../../../accounting';
//import { NotaConceptoSelectorComponent } from '../../controls/nota-concepto-selector.component'
import { NotaConceptoInfo, NotaConceptoItem } from '../../../accounting/models/nota-concepto';
import { NotaConceptoService } from '../../../accounting/services/nota-concepto.service';
import { NotaTipoService } from '../../../credit/services/nota-tipo.service';


@Component({
    selector: 'credito-nota-detail-editor',
    templateUrl: './notas-detail.editor.html'
})
export class NotaDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new NotaDetail();
    //@ViewChild('selectorNotaConcepto') selectorNotaConcepto: NotaConceptoSelectorComponent;
    notaConcepto: NotaConceptoInfo = new NotaConceptoInfo();
    //Objeto original que se editará
    @Input() detail: NotaDetail = new NotaDetail();
    @Input() idNotaTipo: number = 0;
    @Input() formato: string = "n2";
    impuestoCalculado: number; 
    notaConceptos: NotaConceptoItem[] = [];

    constructor(protected context: Context, private builder: FormBuilder,
        private notaTipoSvc: NotaTipoService,
        private notaConceptoSvc: NotaConceptoService) {
        super(context);

        this.form = builder.group({
            'idNotaConcepto': [0, Validators.required],
            'importe': [0, Validators.required],
            'concepto': [''],
        });
    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////

    //onSelectedItemChanged(concepto: NotaConceptoInfo) {
    //    if (concepto) {
    //        this.notaConcepto = concepto;
    //    }
    //}

    onSelectItem(idConcepto: number) {
        this.notaConceptoSvc.getDetailByID(idConcepto)
            .then(conc => {
                this.notaConcepto = conc;
            })
            .catch(error => this.handleError(error));

    }


    onCreatingItem() {
        this.detailCopy = this.detail;
		this.form.reset({ sample: '' });
        this.form.markAsPristine();
        this.loadConceptos();
    }

    onLoadingItem(detail: NotaDetail ) {
        this.detailCopy = <NotaDetail>this.copyItem(this.detail);
        this.loadConceptos();
    }

    onUpdatingItem(detail: NotaDetail ) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();

            ////Calculando los impuestos////
            this.notaConcepto.impuestos.forEach(det => {
                var nuevoImpuesto: ImpuestoDetailInfo = new ImpuestoDetailInfo;
                nuevoImpuesto.importe = detail.importe;//detail.importe * det.tasaOCuota;
                nuevoImpuesto.tasaOCuota = det.tasaOCuota;
                nuevoImpuesto.tipoAplicacion = det.tipoAplicacion;
                nuevoImpuesto.tipoFactor = det.impuesto.factor;
                nuevoImpuesto.base = detail.importe / (1 + det.tasaOCuota);
                nuevoImpuesto.impuesto = det.impuesto;

                if (det.impuesto.factor == "T") {
                    this.impuestoCalculado = detail.importe * det.tasaOCuota;
                }
                else {
                    this.impuestoCalculado = det.tasaOCuota;
                }
                this.detailCopy.impuestos.push(nuevoImpuesto);
            });
            ////////////////////////////////

        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
    }

    loadConceptos() {
        this.notaTipoSvc.loadConceptosList(this.context.app.defaultCompany.identity, this.idNotaTipo)
            .then(conceptos => this.notaConceptos = conceptos)
            .catch(error => this.handleError(error));
    }

}
