﻿import { Component, Input, Output, ViewChild, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
//import { RegExp } from "typescript-dotnet-commonjs/System/Text/RegularExpressions";
//import { RegExp } from "RegularExpressions/Regex"; //

//
//features
import { CollectionView, Control } from 'wijmo/wijmo';
import { ModalDirective } from 'ngx-bootstrap';

import { BaseComponent, Context, CatalogItem, PolicyInfo } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent, AzComboboxComponent } from 'framework3/controls';
import { ClasificacionValor} from 'framework3/general';
//
import { UsuariosListComponent } from 'framework3/security';
//
import { DeudorService } from '../services/deudor.service';
import { DeudorInfo, DeudorItem } from '../models/deudor';
import { CajaService } from '../services/caja.service';
import { CajaInfo, CajaItem } from '../models/caja';
import { FormaPagoItem } from '../models/forma-pago';
import { CobranzaFormaPagoDetail, FormaPagoDetail, FormaPagoCxC } from '../models/forma-pago-detail';
import { FormaPagoService } from '../services/forma-pago.service';
import { FormaPagoService as FormaPagoBaseService } from '../../accounting/services/forma-pago.service';

import { CuentaBancariaItem } from '../../accounting/models/cuenta-bancaria';
import { CuentaBancariaService} from '../../accounting/services/cuenta-bancaria.service';
import { CobranzaInfo } from '../models/cobranza';
import { FlexGrid } from 'wijmo/wijmo.grid';
import { CobranzaDetail } from '../models/cobranza-detail';
import { forEach } from '@angular/router/src/utils/collection';



@Component({
    selector: 'credit-cobranza',
    templateUrl: './cobranza.component.html'
})
export class CobranzaComponent {
    title: string = '';
    cuentasXCobrar: FormaPagoCxC[]=[];
    CxCSelected: FormaPagoCxC[]=[];
    editorForm: FormGroup;    
    formasPago: FormaPagoItem[] = [];
    cajas: CajaItem[] = [];
    cuentasBeneficiario: CuentaBancariaItem[] = [];

    idCaja: number = 0;
    formasPagoCobro: CollectionView = new CollectionView();
    solicitado: number=0;
    recibido: number=0;
    cambio: number=0;
    saldo: number = 0;
    cajaVisible: boolean = true;

    formaPagoDet: FormaPagoDetail = new FormaPagoDetail();

    //codeAccounting: string = "";
    idFormaPago: number=0;    
    //monto: number=0;
    //referencia: string = "";
    //idBanco: number = 0;
    //cuenta: string = "";
    //fecha: Date;
    allReference: boolean = false;
    regExpCuentaOrdenante: string = "[0-9]{11}|[0-9]{18}";
    regExpCuentaBeneficiario: string = "[0-9]{11}|[0-9]{18}";
    @Output() cobroAcceptado: EventEmitter<CobranzaFormaPagoDetail> = new EventEmitter<CobranzaFormaPagoDetail>();
    @Input() idCuentaBeneficiario: number=0;
    cuentaBeneficiarioFija: boolean = false;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('dlgCobranza') dlgCobranza: ModalDirective;
    @ViewChild('cboFormaPago') cboFormaPago: AzComboboxComponent;
    
    constructor(private formBuilder: FormBuilder, private context: Context, private router: Router, private route: ActivatedRoute, private formaPagoService: FormaPagoService,
        private formaPagoBaseService: FormaPagoBaseService, private cajaService: CajaService, private deudorService: DeudorService, private cuentaBancariaService: CuentaBancariaService) {
        this.editorForm = formBuilder.group({   
            'idCaja': ['',Validators.required],
            'totalMonto': [''],
            'totalRecibido': [''],
            'totalCambio': [''],
            'formaPago': [''],
            'monto': [''],
            'referencia': [''],
            'banco': [''],            
            'cuentaOrd': ['', (control) => this.validateCuentaOrdenante(this.regExpCuentaOrdenante, control)],
            'cuentaBenef': [0, Validators.required], //(control) => this.validateCuentaBeneficiario(this.regExpCuentaBeneficiario, control)],
            'rfcOrdenante': ['', Validators.pattern("^([A-ZÑ\x26]{3,4}([0-9]{2})(0[1-9]|1[0-2])(0[1-9]|1[0-9]|2[0-9]|3[0-1])([A-Z]|[0-9]){2}([A]|[0-9]){1})?$")],
            'rfcBeneficiario': ['', Validators.pattern("^([A-ZÑ\x26]{3,4}([0-9]{2})(0[1-9]|1[0-2])(0[1-9]|1[0-9]|2[0-9]|3[0-1])([A-Z]|[0-9]){2}([A]|[0-9]){1})?$")],                                                      
            'fecha': [''],
        });
        this.formaPagoDet = new FormaPagoDetail();
    }

    ngAfterViewInit() {
        //setTimeout(_ => this.loadOptions());
        this.formaPagoDet = new FormaPagoDetail();
    }

    validateCuentaOrdenante(regExpCuentaOrdenante: string, control: FormControl) {
        let pattern =  Validators.pattern(regExpCuentaOrdenante);          
        //let pattern = Validators.pattern(regExpCuentaOrdenante);          

        if (control.value == undefined) return null;

        if (control.value.match(regExpCuentaOrdenante))
        //if (pattern.isMatch(control.value))
             return null;
        else
            return 'Cuenta no válida';
    }

    validateCuentaBeneficiario(regExpCuentaBeneficiario: string, control: FormControl) {        
        
        if (control.value == undefined) return null;

        if (control.value.match(regExpCuentaBeneficiario))
            //if (pattern.isMatch(control.value))
            return null;
        else
            return 'Cuenta no válida';
    }

    setCxC(cxc: CobranzaDetail[]) {
        this.cuentasXCobrar = [];
        for (let det of cxc.slice()) {
            let cuentaPorCobrar: FormaPagoCxC = new FormaPagoCxC();
            let formaPago = this.formasPagoCobro.sourceCollection.find(x => x.cuentasPorCobrar.find(y => y.idCuentaPorCobrarTit == det.idCuentaPorCobrarTit))
            if (formaPago != undefined) {
                let cuentaPorCobrar = formaPago.cuentasPorCobrar.find(y => y.idCuentaPorCobrarTit == det.idCuentaPorCobrarTit)
                if (cuentaPorCobrar != undefined) {
                    if (cuentaPorCobrar.importe != det.importe) {
                        cuentaPorCobrar.idCuentaPorCobrarTit = det.idCuentaPorCobrarTit;
                        cuentaPorCobrar.fechaCuentaPorCobrar = det.fechaCuentaPorCobrar;
                        cuentaPorCobrar.importe = det.importe - cuentaPorCobrar.importe;
                        cuentaPorCobrar.folioFactura = det.folioFactura;
                        cuentaPorCobrar.saldo = det.saldo;
                        cuentaPorCobrar.total = det.total;
                        this.cuentasXCobrar.push(cuentaPorCobrar);
                    }
                } 
            } else {
                cuentaPorCobrar.idCuentaPorCobrarTit = det.idCuentaPorCobrarTit;
                cuentaPorCobrar.fechaCuentaPorCobrar = det.fechaCuentaPorCobrar;
                cuentaPorCobrar.importe = det.importe;
                cuentaPorCobrar.folioFactura = det.folioFactura;
                cuentaPorCobrar.saldo = det.saldo;
                cuentaPorCobrar.total = det.total;
                this.cuentasXCobrar.push(cuentaPorCobrar);
            }      
        }
        Control.invalidateAll();

    }

    showSinCaja(totalSolicitado: number, idCaja: number, idDeudor: number) {
        this.cajaVisible = true;
        if (idCaja > 0) {
            this.cajaVisible = false;
            this.idCaja = idCaja;
        }
        this.show(totalSolicitado, idDeudor);        
    }

    show(totalSolicitado: number, idDeudor: number) {
        if (totalSolicitado != undefined) {
            this.solicitado = totalSolicitado;
            this.saldo = totalSolicitado;
        }

        this.allReference = false;
        if (idDeudor > 0) {
            this.deudorService.getDetailFromCache(idDeudor)
                .then(deudor => {
                    if (deudor.idFormaPago > 0) {
                        this.idFormaPago = deudor.idFormaPago;
                        this.selectedFormaPagoChanged(this.idFormaPago);
                    }
                    //Si el deudor tiene una clasificacion con el codigo COB_FP_REF  es porque maneja que algunos deudores exigen que se les muestre todas las referencias a la forma de pago
                    let clasificacionReferenciasFormaPago: ClasificacionValor = deudor.clasificaciones.find(clas => clas.codigoGrupo == 'COB_FP_REF');
                    if (clasificacionReferenciasFormaPago != null)
                        this.allReference = (clasificacionReferenciasFormaPago.valor=='1')?true:false;

                })
                .catch(error => this.showAlert(error.message, 'danger'))
        }

        this.dlgCobranza.config.backdrop = false
        this.loadFormasPago();
        this.loadCajas();
        this.loadCuentasBeneficiario();
        this.dlgCobranza.show();
    }


    clear() {
        this.formasPagoCobro = new CollectionView();
        this.solicitado = 0;
        this.recibido= 0;
        this.cambio= 0;
    }

    loadFormasPago()
    {
        this.formaPagoService.loadList(this.context.app.defaultSite.identity)
            .then(formasPago => this.formasPago = formasPago)
            .catch(error=> this.showAlert(error.message,'danger'))
    }

    loadCajas() {
        this.cajaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cajas => {
                this.cajas = cajas;
            })
            .catch(error => this.showAlert(error,'danger'));
    }

    loadCuentasBeneficiario() {
        this.cuentaBancariaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cuentasBeneficiario => {
                this.cuentasBeneficiario = cuentasBeneficiario;
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

    selectedFormaPagoChanged(idFormaPago: number) {  
        this.formaPagoDet = new FormaPagoDetail();
        this.formaPagoDet.fecha.setHours(0);
        this.formaPagoDet.fecha.setMinutes(0);
        this.formaPagoDet.fecha.setSeconds(0);
        this.formaPagoDet.fecha.setMilliseconds(0);
        this.formaPagoDet.monto = this.saldo;
        this.formaPagoDet.idFormaPago = idFormaPago;
        this.formaPagoService.getDetailByID(idFormaPago)
            .then(formaPago => {
                this.formaPagoDet.formaPago = formaPago;  
                this.regExpCuentaOrdenante = formaPago.formaPagoBase.regExpCuentaOrdenante; 
                this.regExpCuentaBeneficiario = formaPago.formaPagoBase.regExpCuentaBeneficiario;  
            })
            .catch(error => this.showAlert(error.message, 'danger'))
        if (this.idCuentaBeneficiario > 0) {
            this.formaPagoDet.idCuentaBeneficiario = this.idCuentaBeneficiario
            this.cuentaBeneficiarioFija = true;
        }

        this.validateFormaPago();
    }
    
    addDetail()
    {
        let importeCxC: number = (this.cuentasXCobrar.length>0?this.cuentasXCobrar
            .map(c => c.importe)
            .reduce((sum, current) => sum + current):0);

        let importeCxCselected: number = (this.CxCSelected.length>0?this.CxCSelected
            .map(c => c.importe)
            .reduce((sum, current) => sum + current):0);
        console.log((Math.round(importeCxC * 100) / 100));
        console.log((Math.round(importeCxCselected * 100) / 100));
        //El monto que ingresen debe de ser menor o igual al total de importes de las cuentas por cobrar seleccionadas
        if ((this.formaPagoDet.monto <= (Math.round(importeCxC * 100) / 100) && this.formaPagoDet.monto <= (Math.round(importeCxCselected * 100) / 100))||this.cuentasXCobrar.length==0) {

            let cambioLocal: number = 0;
            let newFormasPago: FormaPagoDetail[] = this.formasPagoCobro.sourceCollection.slice();

            //Vefificamos que no se haya capturado esta misma forma de pago antes. Si asi es, la acumulamos
            let formaPagoExist = newFormasPago.find(search => search.idFormaPago == this.formaPagoDet.idFormaPago && search.referencia == this.formaPagoDet.referencia);



            if (Math.round((this.recibido + this.formaPagoDet.monto) * 100) / 100 > Math.round(this.solicitado * 100) / 100 && this.formaPagoDet.formaPago.formaPagoBase.code != "01") //Solo el efectivo regresa cambio  -- Configurar.  Ver cuales piden fecha y cuales piden banco porque la tarjeta no deberia      
                this.showAlert("Esta forma de pago no admite cambio", "danger");
            else {
                cambioLocal = (this.recibido + this.formaPagoDet.monto) - this.solicitado
                if (cambioLocal < 0)
                    cambioLocal = 0

                //Creamos un temporal del monto para restarlo conforme agregamos cuentas por cobrar
                let tempMonto: number = this.formaPagoDet.monto;
                let tempCxC = this.cuentasXCobrar.slice();
                let tempCxCSelected = this.CxCSelected.slice();
                for (let detail of tempCxCSelected) {
                    //si con el monto se puede pagar la factura en cuestión
                    if ((Math.round(detail.importe * 100) / 100) <= (Math.round(tempMonto * 100) / 100)) {
                        let cuenta = tempCxC.filter(x => x.idCuentaPorCobrarTit == detail.idCuentaPorCobrarTit);
                        tempCxC.splice(tempCxC.indexOf(cuenta[0]), 1);
                        tempMonto -= (Math.round(detail.importe * 100) / 100);
                    }
                    else {
                        let cuenta = tempCxC.filter(x=>x.idCuentaPorCobrarTit==detail.idCuentaPorCobrarTit);
                        detail.importe = tempMonto;
                        cuenta[0].importe -= tempMonto;
                        tempMonto = 0;
                    }
                }

                this.cuentasXCobrar = tempCxC.slice();


                if (formaPagoExist == undefined) {
                    this.formaPagoDet.cambio = cambioLocal;
                    this.formaPagoDet.cuentasPorCobrar = tempCxCSelected.slice().filter(x=>x.importe>0);
                    newFormasPago.push(this.formaPagoDet);
                    this.formaPagoDet = new FormaPagoDetail();
                }
                else {
                    formaPagoExist.monto += this.formaPagoDet.monto;
                    formaPagoExist.cuentasPorCobrar = formaPagoExist.cuentasPorCobrar.concat(tempCxCSelected.slice().filter(x => x.importe > 0));
                }

                this.formasPagoCobro.sourceCollection = newFormasPago;

                this.resetFormaPago();
                this.calculaTotales();
                this.CxCSelected = []; //caso 1824
            }
        }
        else {
            this.showAlert("El monto que se ingrese debe de ser menor o igul al importe de las cuentas por cobrar", "danger");
        }
    }

    resetFormaPago() {
        this.formaPagoDet = new FormaPagoDetail();
        this.validateFormaPago();
    }

    validateFormaPago() {
        if (this.formaPagoDet.formaPago.identity == 0 || this.formaPagoDet.monto == 0)
            return false;
        //else if (this.formaPagoDet.formaPago.formaPagoBase.bancarizado && this.formaPagoDet.idBanco == 0)
        //    return false;
        else if (this.formaPagoDet.formaPago.formaPagoBase.cuentaOrdenante && this.formaPagoDet.cuentaOrdenante.trim() == "")
            return false;
        else if (this.formaPagoDet.formaPago.formaPagoBase.cuentaBeneficiario && (this.formaPagoDet.idCuentaBeneficiario == 0 || this.formaPagoDet.idCuentaBeneficiario == undefined))
            return false;
        else if (this.formaPagoDet.formaPago.formaPagoBase.requiereNumOperacion && this.formaPagoDet.numeroOperacion.trim() == "")
            return false;
        return true;
    }

    validaAceptar() {
        return (Math.round(this.solicitado * 100) / 100 != Math.round((this.recibido - this.cambio)*100)/100);
    }


    calculaTotales() {
        this.recibido = 0;
        this.cambio = 0;
        this.recibido = this.formasPagoCobro.sourceCollection
            .map(c => c.monto)
            .reduce((sum, current) => sum + current);
            
        if (this.recibido > this.solicitado)
            this.cambio = this.recibido - this.solicitado;
        else
            this.saldo = this.solicitado - this.recibido;

    }

    onFormaPagoDelete(item: FormaPagoDetail) {
        //Recorremos las cuentas por cobrar que tiene la forma de pago que se va a eliminar para regresar el importe que se iba a pagar
        let tempCxC: FormaPagoCxC[] = this.cuentasXCobrar.slice();
        for (let detail of item.cuentasPorCobrar) {
            let cXc = tempCxC.filter(x => x.idCuentaPorCobrarTit == detail.idCuentaPorCobrarTit);
            //si existe una cuenta por cobrar con el mismo ID acumulamos el importe
            console.log(cXc.length);
            if (cXc.length > 0) {
                console.log(cXc[0].importe);
                console.log(detail.importe);
                cXc[0].importe += detail.importe;
            }
            else { //Si no existe simplemente le agregamos el detalle
                tempCxC.push(detail);
            }
        }

        this.cuentasXCobrar = tempCxC;

        this.formasPagoCobro.remove(item);
        this.calculaTotales()
    }

    accept() {
        let cobranza = new CobranzaFormaPagoDetail();
        cobranza.idCaja = this.idCaja;
        cobranza.FormaPagoDetail = this.formasPagoCobro.sourceCollection;
        this.cobroAcceptado.emit(cobranza);
        this.dlgCobranza.hide();
    }

    discardChanges() {
        this.dlgCobranza.hide();
    }

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }

    onSelectCxC(s: FlexGrid, e: any) {
        this.CxCSelected = JSON.parse(JSON.stringify(s.selectedItems));
    }
}