﻿//Frameworks

import { FormBuilder, FormGroup, Validators } from '@angular/forms';
//Bootstrap
//Features
import { Component, Input, Output, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import * as moment from 'moment';
import { CollectionView } from 'wijmo/wijmo';
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { AztecaValidators } from 'framework3/controls';
import { Context } from 'framework3/core';
import { TracingModule } from 'framework3/tracing';
//
import { CajaService } from '../services/caja.service';
import { CorteItem } from '../models/caja'
//import { PolizaLogEntry } from '../../models/poliza-log-entry';


@Component({
    selector: 'credit-corte-historico-component',
    templateUrl: './corte-historico.component.html'
})
export class CorteHistoricoComponent {
    editorForm: FormGroup;
    logCollection = new CollectionView();

    currentEntry: CorteItem = new CorteItem();
    //
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('detailsDialog') detailsDialog: ModalDirective;
    @ViewChild('corteHistorico') corteHistoricoDialog: ModalDirective;

    @Input() idCaja: number;

    fechaIni: Date;
    fechaFin: Date;

    constructor(private formBuilder: FormBuilder, private context: Context, private router: Router, private route: ActivatedRoute,
        private cajaService: CajaService) {
        this.editorForm = formBuilder.group({
            'fechaIni': [0],
            'fechaFin': [0],
        });
    }

    /////////////////////// METODOS ////////////////////////////////////////////////

    loadLog() {
        this.context.app.showSpinner();
        this.cajaService.loadCortesHistorico(this.idCaja, this.fechaIni, this.fechaFin)
            .then(log => {
                this.logCollection.sourceCollection = log;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    /////////////////////// HANDLERS ///////////////////////////////////////////////
    mainToolbarClick(button: ToolbarButton) {
        if (button.name == 'REFRESH') {
            this.loadLog();
        }
    }

    onRowView(entry: CorteItem) {
        this.currentEntry = entry;
        //this.detailsDialog.config.backdrop = false;
        //this.detailsDialog.show();
        this.context.app.showSpinner();
        this.cajaService.generarPdfCorte(this.currentEntry.idCorte)
            .then(uri => {
                this.context.app.hideSpinner();
                window.open('/file-server/' + uri.uri, '_blank');
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    show(idCaja:number) {
        this.logCollection = new CollectionView();
        this.corteHistoricoDialog.config.backdrop = false;
        this.idCaja = idCaja;
        this.corteHistoricoDialog.show();
        this.fechaIni = moment().subtract(2, 'week').toDate();
        this.fechaFin = moment().toDate();
        this.loadLog();
    }

}