﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent, ToolbarButton } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';

//Wijmo
import { FlexGrid, HitTestInfo, CellRange, CellType, Column, CellRangeEventArgs } from 'wijmo/wijmo.grid';
import { Menu } from 'wijmo/wijmo.input';
import { CollectionView, EventArgs } from 'wijmo/wijmo';

//Features
import { CorteItem} from '../models/caja';
import { CajaService } from '../services/caja.service';
import { CajerosListComponent } from '../dialogs/cajeros-list.component';
import { CorteHistoricoComponent } from '../parts/corte-historico.component';

@Component({
    selector: 'corte-component',
    templateUrl: './corte.component.html'
})
export class CorteComponent {
    editorForm: FormGroup;
    catalogOptions: CatalogOptions = new CatalogOptions();

    cortes: CollectionView = new CollectionView();

    idCajero: number;
    //
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent
    @ViewChild(CajerosListComponent) cajerosDialog: CajerosListComponent;
    @ViewChild(CorteHistoricoComponent) corteHistoricoDialog: CorteHistoricoComponent;

    @Output() contextMenuOpen: EventEmitter<CorteItem> = new EventEmitter<CorteItem>();
    currentItem: CorteItem = null;

    constructor(private formBuilder: FormBuilder, private router: Router, private context: Context, private cajaService: CajaService) {
        this.editorForm = formBuilder.group({
            
        });
    }

    ///////////// SUBPRODUCTOS///////////////////////
    gridCortesInitialized(sender: FlexGrid, args: any) {
        var self = this;
        //Agregar manejador para menú contextual        
        sender.addEventListener(sender.hostElement, "contextmenu", (e: any) => {
            let ht: HitTestInfo = sender.hitTest(e);
            sender.select(new CellRange(ht.row, ht.col), true);
            self.contextMenuOpen.emit(self.currentItem);


        });
       
        this.cortes.currentChanged.addHandler((args: EventArgs) => {
            let row: any = self.cortes.currentItem;
            if (row != null)
                this.currentItem = { idCorte: row.idCorte, idCaja:row.idCaja, caja:row.caja,  cajero:row.cajero, horaApertura: row.horaApertura, horaCierre:row.horaCierre, activo: row.activo, estatus: row.estatus /*, data: row*/ };
            else
                this.currentItem = null;
            //
            //self.currentItemChanged.emit(self.currentItem);
        }, self);

    }


    mainToolbarHandler(button: ToolbarButton) {
        if (button.name == "REFRESH") {
            this.loadCortes();
        }
    }

    loadCortes() {
        this.cajaService.loadCortes(this.context.app.defaultSite.identity)
            .then(cortes => this.cortes.sourceCollection = cortes)
            .catch(error => this.showAlert(error.message, "danger"))
    }

    onMenuItemClicked(menu: Menu, args: any) {
        //this.currentCorte = this.cortes.currentItem;
        if (menu.selectedIndex == 0) {
            if (this.currentItem.idCorte == null ||this.currentItem.idCorte == undefined ||this.currentItem.idCorte == '00000000-0000-0000-0000-000000000000') {
                //seleccionar el cajero asignado
                this.cajerosDialog.show();
            }
            else
                this.showAlert("La caja ya tiene un corte abierto", "danger")
        }
        if (menu.selectedIndex == 1) {
            //Cerrar
            
            this.cajaService.cerrar(this.currentItem.idCorte)
                .then(cajas => {
                    this.loadCortes();
                    this.showAlert("Se realizo el cierre de corte de caja", "success")
                })
                .catch(error => this.showAlert(error.message, "danger"))
 
        }
        if (menu.selectedIndex == 3) {
            //Historico
            this.showHistoricoCortes(this.currentItem.idCaja);
        }
    } 

    onSelectedCajeroChanged(arg: any) {              
        this.cajaService.abrir(this.currentItem.idCaja, arg.id)
            .then(_ => {
                this.loadCortes();
                this.showAlert("Se realizo la apertura del corte de caja", "success")
            })
            .catch(error => this.showAlert(error.message, "danger"))      
    }


    handleError(error: any) {
        console.error(error);
    }

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }

    showHistoricoCortes(idCaja:number)
    {
        this.corteHistoricoDialog.show(idCaja);

    }

}

