﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, FileLink } from 'framework3/core';
//Locales
import { CajaInfo, CajaItem, CorteItem } from '../models/caja'
import { UsuarioItem } from 'framework3/core';
import * as moment from 'moment';

@Injectable()
export class CajaService extends CatalogService<CajaInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/credito/cajas");
    }

    create(): Promise<CajaInfo> {
        return Promise.resolve(new CajaInfo());
    }
    	
	//lista de cajas

    loadList(idSucursal: number): Promise<CajaItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<CajaItem[]>('load-list', params)
            .toPromise();
    }

    loadListByUser(idSucursal: number): Promise<CajaItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<CajaItem[]>('load-list-by-user', params)            
            .toPromise();
    }

    getMembers(idCaja: number): Promise<UsuarioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCaja', idCaja.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)            
            .toPromise();
    }

    addMembers(idCaja: number, users: UsuarioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCaja', idCaja.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }
    
    removeMember(idCaja: number, idUser: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCaja', idCaja.toString())
            .set('idUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

    //cortes
    loadCortes(idSucursal: number): Promise<CorteItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<CorteItem[]>('cortes', params)            
            .toPromise();
    }

    abrir(idCaja: number, idCajero:number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCaja', idCaja.toString())
            .set('idCajero', idCajero.toString());
        return this.apiService.get<any>('abrir', params)           
            .toPromise();
    }

    cerrar(idCorte: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCorte', idCorte.toString());
        return this.apiService.get<any>('cerrar', params)            
            .toPromise();
    }

    loadCortesHistorico(idCaja: number, fechaIni: Date, fechaFin: Date): Promise<CorteItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdCaja', idCaja.toString())
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format());

        return this.apiService.get<CorteItem[]>('load-cortes-historico', params)            
            .toPromise();
    }

    generarPdfCorte(idCorte: string): Promise<FileLink> {
        let params = new HttpParams()
            .set("idCorte", idCorte);
        return this.apiService.get<FileLink>("pdf-corte", params)
            .toPromise();
    }

}

