﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';

//Locales
import { CobranzaInfo, CobranzaItem } from '../models/cobranza';
import { CobranzaDetail } from '../models/cobranza-detail';

@Injectable()
export class CobranzaService extends DocumentService<CobranzaDetail, CobranzaInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/credito/cobranzas-tit');
    }

    create(): Promise<CobranzaInfo> {
        let newDoc = new CobranzaInfo();
        return Promise.resolve(newDoc);
    }

    //cargar cxc por grupo deudor
    loadCxCByGrupoDeudor(fechaIni:string, fechaFin:string, idGrupoDeudor: number, fechaFiltro): Promise<CobranzaDetail[]> {
        let params: HttpParams = new HttpParams()
            .set('FechaIni', fechaIni.toString())
            .set('FechaFin', fechaFin.toString())
            .set('IdGrupoDeudor', idGrupoDeudor.toString())
            .set('FechaFiltro', fechaFiltro);
        return this.apiService.get<CobranzaDetail[]>('load-cxc-by-grupo-deudor', params)            
            .toPromise();
    }
    //cargar cxc por deudor
    loadCxCByDeudor(fechaIni: string, fechaFin: string, idDeudor: number, fechaFiltro): Promise<CobranzaDetail[]> {
        let params: HttpParams = new HttpParams()
            .set('fechaIni', fechaIni.toString())
            .set('fechaFin', fechaFin.toString())
            .set('idDeudor', idDeudor.toString())
            .set('FechaFiltro', fechaFiltro);
        return this.apiService.get<CobranzaDetail[]>('load-cxc-by-deudor', params)            
            .toPromise();
    }
    //cargar cxc por rfc
    loadCxCByRFC(fechaIni: string, fechaFin: string, rfc: string): Promise<CobranzaDetail[]> {
        let params: HttpParams = new HttpParams()
            .set('fechaIni', fechaIni.toString())
            .set('fechaFin', fechaFin.toString())
            .set('rfc', rfc);
        return this.apiService.get<CobranzaDetail[]>('load-cxc-by-rfc', params)            
            .toPromise();
    }

    generarPdfFactura(idCobranza: string): Promise<FileLink> {
        let params: HttpParams = new HttpParams()
            .set("idCobranza", idCobranza);
        return this.apiService.get<FileLink>("pdf-comprobante", params)            
            .toPromise();
    }

    getXmlFactura(idCobranza: string): Promise<FileLink> {
        let params: HttpParams = new HttpParams()
            .set("idCobranza", idCobranza);
        return this.apiService.get<FileLink>("xml-comprobante", params)            
            .toPromise();
    }

    sustituir(idCobranza: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("IdCobranza", idCobranza);
        //
        return this.apiService.post<any>('sustituir-comprobante-pago', params, null)
            .toPromise();
    }
    
	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: CobranzaInfo) {

    }
	*/



}