﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, WebService } from 'framework3/core';
//Locales
import { CuentaPorCobrarInfo, CuentaPorCobrarItem } from '../models/cuenta-por-cobrar';
import { EnvioComprobantesItem } from '../../credit/components/envio-comprobantes-fiscales.component';
import { CuentaPorCobrarDetail } from '../models/cuenta-por-cobrar-detail';
import * as moment from 'moment';

@Injectable()
export class CuentaPorCobrarService extends DocumentService<CuentaPorCobrarDetail, CuentaPorCobrarInfo> {

    constructor(protected http: HttpClient, protected context: Context) {
        super(http, context, 'api/credito/cuentas-por-cobrar-tit');
    }

    create(): Promise<CuentaPorCobrarInfo> {
        let newDoc = new CuentaPorCobrarInfo();
        return Promise.resolve(newDoc);
    }

    loadFacturacionPendiente(idSucursal: number, tipoActor: string, idActor: number, fechaIni: Date, fechaFin: Date): Promise<CuentaPorCobrarItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdSucursal', idSucursal.toString())
            .set('TipoActor', tipoActor)
            .set('IdActor', idActor.toString())
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format());
        return this.apiService.get<CuentaPorCobrarItem[]>('load-facturacion-pendiente', params)
            .toPromise();
    }

    GeneraFactura(idCFDSerie: number, idSucursal: number, idContribuyente: number, cxc: CuentaPorCobrarItem[]) {
        let codigoMetodoPago: string = '';
        let cuentaBanco: string = '';
        let params: HttpParams = new HttpParams()
            .set('IdCFDSerie', idCFDSerie.toString())
            .set('CodigoMetodoPago', codigoMetodoPago)
            .set('CuentaBanco', cuentaBanco)
            .set('IdSucursal', idSucursal.toString())
            .set('IdContribuyente', idContribuyente.toString());
        return this.apiService.post<CuentaPorCobrarItem>('genera-factura', params, cxc)
            .toPromise();
    }

    searchKeyword(idBranch: number, keyword: string, idDeudor: number, idGrupoDeudor: number): Observable<CuentaPorCobrarItem[]> {
        let params: HttpParams = new HttpParams()
            .set("idBranch", idBranch.toString())
            .set("keyword", keyword)
            .set("idDeudor", idDeudor.toString())
            .set("idGrupoDeudor", idGrupoDeudor.toString());
        return this.apiService.get<CuentaPorCobrarItem[]>("search-cpc", params)
    }

    modificarAddenda(data: EnvioComprobantesItem): Promise<any> {
        let fechaOrdenCompra: Date = moment(data.fechaOrdenCompra).startOf('day').toDate();
        let fechaRemision: Date = moment(data.fechaRemision).startOf('day').toDate();
        let fechaEntrega: Date = moment(data.fechaEntrega).startOf('day').toDate();
        let params: HttpParams = new HttpParams()
            .set("IdCuentaPorCobrar", data.idCuentaPorCobrarTit)
            .set("OrdenCompra", data.ordenCompra)
            .set("FechaOrdenCompra", moment(fechaOrdenCompra).format())
            .set("NumRemision", data.numRemision)
            .set("FechaRemision", moment(fechaRemision).format())
            .set("FechaEntrega", moment(fechaEntrega).format());
        return this.apiService.post<any>('modificar-addenda', params, data)
            .toPromise();
    }

    generarAddendaCustom(idPedidoTit: string, fecha: string, origen: string): Promise<any> {
        let apiCustom = new WebService(this.http, 'api/custom/aga/credito/cuentasPorCobrar', this.context);
        //
        let params: HttpParams = new HttpParams()
            .set("idPedidoTit", idPedidoTit)
            .set("fecha", fecha)
            .set('origen', origen);
        //
        return apiCustom.post<any>('generar-adenda', params, null)
            .toPromise();
    }

    requerirCancelacion(idCuentaPorCobrar: string) : Promise<string> {
        let params: HttpParams = new HttpParams()
            .set("idCuentaPorCobrar", idCuentaPorCobrar);
        return this.apiService.post<string>('requerir-cancelacion', params, null)
            .toPromise();
    }

    verficarComprobanteTimbrado(idCuentasPorCobrarTit: string[]): Promise<any> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<string>('verifica-comprobante-timbrado', params, idCuentasPorCobrarTit)
            .toPromise();
    }

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: CuentaPorCobrarInfo) {

    }
	*/



}