﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
import { ClasificacionItem } from 'framework3/general'
//Locales
import { ReporteInfo } from '../models/reporte'

@Injectable()
export class ReporteService extends CatalogService<ReporteInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/credito/reportes");
    }

    create(): Promise<ReporteInfo> {
        return Promise.resolve(new ReporteInfo());
    }

    printAbonos(fechaIni: Date, fechaFin: Date, tipoAbono: number): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format())
            .set('IdConcepto', tipoAbono.toString());
        return this.apiService.get<string>('print-abonos', params)            
            .toPromise();
    }

    printAntiguedadSaldos(idSucursal: number, fechaIni: Date, fechaFin: Date, rango: number): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('IdSucursal', idSucursal.toString())
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format())
            .set('Lapso', rango.toString());
        return this.apiService.get<string>('print-antiguedad-saldos', params)            
            .toPromise();
    }

    printAntiguedadSaldosDetallado(idSucursal: number, fechaIni: Date, fechaFin: Date, rango: number): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('IdSucursal', idSucursal.toString())
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format())
            .set('Lapso', rango.toString());
        return this.apiService.get<string>('print-antiguedad-saldos-detallado', params)            
            .toPromise();
    }

    printCobranzaImpuestos(fechaIni: Date, fechaFin: Date): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format());
        return this.apiService.get<string>('print-cobranza-impuestos', params)            
            .toPromise();
    }

    printComportamiento(fechaIni: Date, fechaFin: Date, cadenaTipo: number): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format())
            .set('IdTipoCadena', cadenaTipo.toString());
        return this.apiService.get<string>('print-comportamiento', params)            
            .toPromise();
    }

    printControlInventarioEfectoIEPS(fechaIni: Date, fechaFin: Date, aplicaIEPS: number): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format())
            .set('AplicaIEPS', aplicaIEPS.toString());
        return this.apiService.get<string>('print-control-inventario-efecto-ieps', params)            
            .toPromise();
    }

    printCreditosOtogados(fechaIni: Date, fechaFin: Date): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format());
        return this.apiService.get<string>('print-creditos-otorgados', params)            
            .toPromise();
    }

    printEstructuraClientesCadenas(idRegion: number): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('IdRegion', idRegion.toString());
        return this.apiService.get<string>('print-estructura-clientes-cadenas', params)            
            .toPromise();
    }

    loadClasificacionesList(): Promise<ClasificacionItem[]> {
        let params: HttpParams = new HttpParams();
        return this.apiService.get<ClasificacionItem[]>("load-clasificaciones-list", params)            
            .toPromise();
    }

    printEstructuraClasificaciones(idClasificacion: number): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('IdClasificacion', idClasificacion.toString());
        return this.apiService.get<string>('print-estructura-clasificaciones', params)
            
            .toPromise();
    }

    printFacturasCredito(fechaIni: Date, fechaFin: Date, idSucursal: number, ruta: string, cliente: string, cadena: string, activas: number): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('FechaIni', moment(fechaIni).format())
            .set('FechaFin', moment(fechaFin).format())
            .set('IdSucursal', idSucursal.toString())
            .set('Ruta', ruta)
            .set('Cliente', cliente)
            .set('Cadena', cadena)
            .set('Activas', activas.toString());
        return this.apiService.get<string>('print-facturas-credito', params)            
            .toPromise();
    }

    printHistoricoSaldos(fecha: Date): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set('Fecha', moment(fecha).format());
        return this.apiService.get<string>('print-historico-saldos', params)            
            .toPromise();
    }

    printSaldosCredito(): Promise<string> {
        let params: HttpParams = new HttpParams();
        return this.apiService.get<string>('print-saldos-credito', params)            
            .toPromise();
    }
}
