﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo, FileLink } from 'framework3/core';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent } from 'framework3/controls';
//Locales
import { CustomMantenimientoCreditoService } from '../services/mantenimiento-credito.service'
import { ContribuyenteRfcMultiplicado, ContribuyenteRfcMultiplicadoDetalle } from '../models/contribuyente-rfc-multiplicado'
import { AsyncTaskService } from "framework3/queue/services/async-task.service";


@Component({
    selector: 'contribuyentes-multiplicados-unificar',
    templateUrl: './contribuyentes-multiplicados-unificar.component.html'
})
export class contribuyentesUnificador extends ProcessComponent {
    form: FormGroup;
    previewCollection: CollectionView = new CollectionView();
    rfcRepetidos: ContribuyenteRfcMultiplicado[] = [];

    selectedCollection: CollectionView = new CollectionView();
    rfcRepetidosDetalle: ContribuyenteRfcMultiplicadoDetalle[] = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder
        , private mantenimientoCreditoService: CustomMantenimientoCreditoService
        , private asyncTaskSvc: AsyncTaskService
    ) {
        super(context, router, route);
        this.form = formBuilder.group({

        });
    }

    llenadoRfcsRepetidos() {
        this.context.app.showSpinner();
        this.mantenimientoCreditoService.listarClientesRFCRepetido(this.context.app.defaultCompany.identity)
            .then(rec => {
                this.rfcRepetidos = rec;
                this.previewCollection.sourceCollection = this.rfcRepetidos;
                this.selectedCollection.sourceCollection = [];
                this.context.app.hideSpinner();
            }
            )
    }

    llenadoDetalle() {
        this.context.app.showSpinner();
        var item: ContribuyenteRfcMultiplicado = new ContribuyenteRfcMultiplicado;
        item = this.previewCollection.currentItem;

        this.mantenimientoCreditoService.clientesRFCRepetido(item.rfc, this.context.app.defaultCompany.identity)
            .then(rec => {
                this.rfcRepetidosDetalle = rec;
                this.selectedCollection.sourceCollection = this.rfcRepetidosDetalle;
                this.context.app.hideSpinner();
            })



    }

    unificar() {
        var item: ContribuyenteRfcMultiplicadoDetalle = new ContribuyenteRfcMultiplicadoDetalle;
        item = this.selectedCollection.currentItem;
        this.context.app.showSpinner();

        this.mantenimientoCreditoService.unificarContribuyentes(item.idContribuyente, this.context.app.defaultCompany.identity)
            .then(rec => {
                this.context.app.hideSpinner();
                this.previewCollection.sourceCollection = [];
                this.selectedCollection.sourceCollection = [];
            }
             )
    }
    
    asyncUnificar() {
        var item: ContribuyenteRfcMultiplicadoDetalle = new ContribuyenteRfcMultiplicadoDetalle;
        item = this.selectedCollection.currentItem;
        this.context.app.showSpinner();

        this.mantenimientoCreditoService.asyncUnificarContribuyentes(item.idContribuyente, this.context.app.defaultCompany.identity)
            .then(ret => {
                this.asyncTaskSvc.subscribeTask(ret.idAsyncTask).subscribe(info => {
                    this.context.app.hideSpinner();
                    this.previewCollection.sourceCollection = [];
                    this.selectedCollection.sourceCollection = [];
                })

            })
            //.catch(error => this.handleError(error));

    }
    
    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CONSULTAR') {
            this.llenadoRfcsRepetidos();
        }
        if (button.name == 'SELECCIONAR') {
            this.llenadoDetalle();
        }

        if (button.name == 'UNIFICAR') {
            this.asyncUnificar();
            //this.unificar();
            //this.previewCollection.sourceCollection = [];
            //this.selectedCollection.sourceCollection = [];
        }

    }


    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

    /*
    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

*/

}