﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router'
import { FormBuilder, FormGroup, Validators } from '@angular/forms';


//Azteca Kernel
import { Context, PolicyInfo, ChangeState } from 'framework3/core';
import { AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { ProcessComponent } from 'framework3/core';
import { TrackableCollection } from 'framework3/editors';

//Features
import { CobranzaFormatoInfo } from '../models/cobranza-formato';
import { ParameterItem, TranspilerInfo, DatasetService } from '../../../../translator/index';

import { CobranzaFormatoService } from '../services/cobranza-formato.service';

@Component({
    selector: 'custom-fotmato-cobranza',
    templateUrl: './formato-cobranza.editor.html'
})
export class FormatoCobranzaEditor extends ProcessComponent {

    editForm: FormGroup;
    cobranzaFormato: CobranzaFormatoInfo = new CobranzaFormatoInfo();
    cvParametros: TrackableCollection;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, private cobranzaFormatoSvc: CobranzaFormatoService,
        private datasetSvc: DatasetService) {
        super(context, router, route);
        this.editForm = formBuilder.group({
            'idDataset': [AztecaValidators.selected],
            'archivo': [null, Validators.required]
        });
        this.cvParametros = new TrackableCollection();
        this.cvParametros.filter = (item: ParameterItem) => {
            return (!item.hidden && item.state != ChangeState.DELETED);
        }
    }

    onTranspilerSelected(item: TranspilerInfo) {
        if (item) {
            this.cobranzaFormato.transpiler = item;
            this.context.app.showSpinner();
            let promises = [];
            if (item.idDatasetInput > 0) {
                let promiseInput = this.datasetSvc.getDetailByID(item.idDatasetInput)
                    .then(info => {
                        this.cobranzaFormato.transpiler.datasetInput = info

                        var exists = false;
                        info.parameters.forEach(param => {
                            if (param.field == 'IdSucursal') {
                                param.value = this.context.app.defaultSite.idBranch.toString();
                                exists = true;
                            }
                            else if (param.defaultValue) {
                                param.value = param.defaultValue;
                            }
                        });

                        if (!exists) {
                            let param = new ParameterItem();
                            param.label = 'IdSucursal';
                            param.field = 'IdSucursal';
                            param.hidden = true;
                            param.value = this.context.app.defaultSite.idBranch.toString();
                            info.parameters.push(param);
                        }                        

                        this.cvParametros.sourceCollection = info.parameters;
                        this.cvParametros.refresh();
                    })
                    .catch(error => this.handleError(error));
                promises.push(promiseInput);
            }
            if (item.idDatasetOutput > 0) {
                let promisOutput = this.datasetSvc.getDetailByID(item.idDatasetOutput)
                    .then(info => this.cobranzaFormato.transpiler.datasetOutput = info)
                    .catch(error => this.handleError(error));
                promises.push(promisOutput);
            }
            Promise.all<void>(promises).then(() => {
                this.context.app.hideSpinner();
            });
        }
    }

    closeEditor() {
        this.router.navigate(['../'], { relativeTo: this.route });
    }

    onClickGuardar() {
        this.context.app.showSpinner();
        this.cobranzaFormatoSvc.updateCobranzaFormato(this.cobranzaFormato)
            .then(result => {
                this.context.app.hideSpinner();
                this.router.navigate(['../'], { relativeTo: this.route });
            })
            .catch(error => {
                this.handleError(error);
            });
    }

    applyPolicy(policy: PolicyInfo) {
        console.log('Aplicar politica');
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}