﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormGroup, FormBuilder } from '@angular/forms';
import { Location } from '@angular/common';

import * as moment from 'moment';

import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, PolicyInfo, BaseComponent } from 'framework3/core';
import { ProcessComponent } from 'framework3/core';
import { MenuItem, AlertsBarComponent, ToolbarButton } from 'framework3/controls';
import { FilterGroup, SearchRequest, SearchCondition } from 'framework3/search';
import { Menu } from 'wijmo/wijmo.input';


import { CobranzaFormatoService } from '../services/cobranza-formato.service';
import { JobService } from 'framework3/queue';
import { TranspilerService } from '../../../../translator';



@Component({
    selector: 'custom-formato-cobranza',
    templateUrl: './formato-cobranza.manager.html'
})
export class FormatoCobranzaManager extends ProcessComponent { 

    form: FormGroup;
    cobranzas = [];
    item: any;
    errorDisplay: string = "";

    filterPanelCollapsed: boolean = false;
    showFilterPanel: boolean = true;
    menuItems: MenuItem[] = [];
    filterGroups: FilterGroup[] = [];
    selectedGroup: FilterGroup;

    request: SearchRequest;
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    currentItems: Array<any> = new Array<any>();
    currentItem: any;
    totalRows: number = 0;
    pageIndex: number = 0;
    respuesta: boolean = false;
    idJobOrigen: number = 0;

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;
    @ViewChild('dlgError') dlgError: ModalDirective;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private location: Location,
        private formBuilder: FormBuilder, private cobranzaFormatoSvc: CobranzaFormatoService, private jobSvc: JobService, private transpilerSvc: TranspilerService) {
        super(context, router, route);

        this.form = formBuilder.group({
            'fechaInicial': [],
            'fechaFinal': []
        });

        this.route.params.forEach((params: Params) => {
            this.idJobOrigen = +params['origen'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.fechaInicial = moment().subtract(30, 'day').toDate();
            this.fechaFinal = moment().add(1, 'day').toDate();
            this.refreshList();
        });
        this.createMenu();
    }

    createMenu() {
        //Agregar opciones estándar del catálogo
        let mnuDownload: MenuItem = { icon: "glyphicon glyphicon-info-sign", text: "Error", name: "ERROR", smallText: "Ver error" };
        let mnuOpen: MenuItem = { icon: "glyphicon glyphicon-share-alt", text: "Cobranza", name: "OPEN", smallText: "abrir cobranza" };
        //
        this.menuItems.push(mnuDownload);
        this.menuItems.push(mnuOpen);
        //
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    /*Permite tomar todo el espacio disponible cuando el panel de filtro no está visible y 75% cuando lo está*/
    get classGridPanel(): string {
        return !this.showFilterPanel || this.filterPanelCollapsed ? 'col-12' : 'col-12 col-lg-9';
    }

    toggleCollapse() {
        this.filterPanelCollapsed = !this.filterPanelCollapsed;
    }

    onMenuItemClicked(menu: Menu, args: any) {
        let item: MenuItem = this.menuItems[menu.selectedIndex];
        this.optionClick(item);
    }

    optionClick(option: MenuItem) {
        if (option.name == "ERROR") {
            this.context.app.showSpinner();
            this.jobSvc.getDetailByID(this.item.data.id)
                .then(info => {
                    this.errorDisplay = info.errorMessage;
                    this.context.app.hideSpinner();
                    this.dlgError.config.backdrop = false;
                    this.dlgError.show();
                }).catch(e => this.handleError(e));
        }
        else if (option.name == 'OPEN') {
            this.context.app.showSpinner();
            this.transpilerSvc.getLastHist(this.item.data.id)
                .then(info => {
                    this.context.app.hideSpinner();
                    this.router.navigate(['credit/cobranza', this.process.id, info.result]);
                }).catch(e => this.handleError(e));
        }
    }

    onParamsToolbar(button: ToolbarButton) {
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }


    refreshList() {
        this.context.app.showSpinner();
        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }
        let exist = false;

        this.request.conditions.forEach(r => {
            if (r.field == "fecha" && r.operator == ">=") {
                exist = true;
                r.value = moment(this.fechaInicial).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
            else if (r.field == "fecha" && r.operator == "<=") {
                exist = true;
                r.value = moment(this.fechaFinal).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
        });

        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fecha", operator: ">=", valueType: "datetime", value: moment(this.fechaInicial).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conFechaFin: SearchCondition = { field: "fecha", operator: "<=", valueType: "datetime", value: moment(this.fechaFinal).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conJobOrigen: SearchCondition = { field: "idJobOrigen", operator: "=", valueType: "int", value: this.idJobOrigen };
            this.request.conditions.push(conFechaIni);
            this.request.conditions.push(conFechaFin);
            this.request.conditions.push(conJobOrigen);
        }

        this.cobranzaFormatoSvc.searchJobs(this.context.app.defaultSite.identity, this.request)
            .then(response => {
                response.rows.forEach(r => {
                    r['fecha'] = moment(r['fecha']).toDate();
                });
                this.currentItems = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = this.request.pageIndex;
                //
                var filterGroup = new FilterGroup();
                filterGroup.tag = "ALL";
                filterGroup.name = "Todos";
                filterGroup.rowCount = this.totalRows;
                this.filterGroups.length = 0;
                this.filterGroups.push(filterGroup);
                //
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onCurrentItemChanged(item: any) {
        this.item = item;
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    newItemHandler() {
        this.router.navigate(['new'], { relativeTo: this.route });
    }

}