﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

import * as moment from 'moment';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { ProcessComponent } from 'framework3/core';
import { SearchRequest, SearchCondition} from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';

//Features
import { CustomCuentaPorCobrarService } from '../services/cuenta-por-cobrar.service';
import { enable } from 'wijmo/wijmo';


@Component({
    selector: 'pixcua-importar-pedidos-component',
    templateUrl: './importar-pedidos.component.html'
})
export class ImportarPedidosComponent extends ProcessComponent {
    editorForm: FormGroup;
    fechaCobro: Date;
    claveSaldos: string = "";

    request: SearchRequest;
    fechaIni: Date = new Date();
    fechaFin: Date = new Date();
    currentPedidos: Array<any> = new Array<any>();
    currentItem: any;
    totalRows: number = 0;
    pageIndex: number = 0;
    menuItems: MenuItem[] = [];
    titleContentView: string = "";
    fileNameContentView: string = "";
    errorServer: string = "";
    recalcularCXC: boolean = false;

    allowRefacturar: boolean = false;
    allowAddenda: boolean = false;
    allowCancel: boolean = false;
    allowSustitucion: boolean = false;

    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('dlgError') dlgError: ModalDirective;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, private cuentaPorCobrarService: CustomCuentaPorCobrarService) {
        super(context, router, route);
        this.editorForm = formBuilder.group({
            'fechaCobro': [0, Validators.required],
            'claveSaldos': [''],
            'fechaInicial': [''],
            'fechaFinal': [''],
            'errorServer': [''],
            'recalcularCXC':['']
        });
        this.fechaCobro = new Date();
    }

    onAccept() {
        this.context.app.showSpinner();
        this.cuentaPorCobrarService.importarVentas(this.context.app.defaultSite.code, this.context.app.defaultSite.identity, this.fechaCobro)
            .then(_ => {
                this.context.app.hideSpinner();
                this.showAlert("Se han generado las Cuentas por Cobrar.", "success");
            })
            .catch(error=> this.handleError(error))        
    }

    ngAfterViewInit() {
        var thisRoot = this;

        setTimeout(_ => {
            this.refreshList();
        });

        this.process.politicas.forEach(procesoAcceso => {
            if (procesoAcceso.nombre == "allowRefacturar")
                this.allowRefacturar = (procesoAcceso.valor ? true : false);
            if (procesoAcceso.nombre == "allowAddenda")
                this.allowAddenda = (procesoAcceso.valor ? true : false);
            if (procesoAcceso.nombre == "allowCancel")
                this.allowCancel = (procesoAcceso.valor ? true : false);
            if (procesoAcceso.nombre == "allowSustitucion")
                this.allowSustitucion = (procesoAcceso.valor ? true : false);
        });
        this.createMenu();
        this.refreshList();
    }

    createMenu() {
        let mnuImprimir: MenuItem = { icon: "glyphicon glyphicon-print", text: "Imprimir", name: "IMPRIMIR", smallText: "Imprimir recibo de n&oacute;mina" };
        let mnuDescargar: MenuItem = { icon: "glyphicon glyphicon-download-alt", text: "Descargar", name: "DESCARGAR", smallText: "Descargar XML" };
        let mnuTimbrar: MenuItem = { icon: "glyphicon glyphicon-qrcode", text: "Timbrar", name: "TIMBRAR", smallText: "Timbrar comprobante" };
        let mnuRefacturar: MenuItem = { icon: "glyphicon glyphicon-qrcode", text: "Refacturar", name: "REFACTURAR", smallText: "Nuevo comprobante", enabled: this.allowRefacturar };
        let mnuAdenda: MenuItem = { icon: "glyphicon glyphicon-repeat", text: "Adenda", name: "ADENDA", smallText: "Regenerar adenda", enabled: this.allowAddenda};
        let mnuCancelar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Cancelar", name: "CENCELAR", smallText: "Cancelar factura", enabled: this.allowCancel};
        let mnuSustituir: MenuItem = { icon: "glyphicon glyphicon-repeat", text: "Sustituir", name: "SUSTITUIR", smallText: "Sustituir comprobante", enabled: this.allowSustitucion };

        this.menuItems.push(mnuImprimir, mnuDescargar, mnuTimbrar, mnuRefacturar, mnuAdenda, mnuCancelar, mnuSustituir);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    refreshList() {
        this.context.app.showSpinner();
        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }
        let exist = false;
        this.request.conditions.forEach(r => {
            if (r.field == "fecha" && r.operator == ">=") {
                exist = true;
                r.value = moment(this.fechaIni).format("YYYY-MM-DD") + "T00:00:00-00:00" ;
            }
            else if (r.field == "fecha" && r.operator == "<=") {
                exist = true;
                r.value = moment(this.fechaFin).format("YYYY-MM-DD") + "T00:00:00-00:00" ;
            }
        });
        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fecha", operator: ">=", valueType: "datetime", value: moment(this.fechaIni).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conFechaFin: SearchCondition = { field: "fecha", operator: "<=", valueType: "datetime", value: moment(this.fechaFin).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            this.request.conditions.push(conFechaIni);
            this.request.conditions.push(conFechaFin);
        }
        this.cuentaPorCobrarService.searchPedidoCliente(this.context.app.defaultSite.identity, this.request)
            .then(response => {
                response.rows.forEach(r => {
                    r['fecha'] = moment(r['fecha']).toDate();
                });
                this.currentPedidos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = this.request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }


    onCurrentItemChanged(item: any) {
        this.currentItem = item;
    }

    searchRequested(request: SearchRequest) {
        this.request = request;
        this.refreshList();
    }

    optionClick(option: MenuItem) {
        if (option.name == "IMPRIMIR") {
            this.context.app.showSpinner();
            this.cuentaPorCobrarService.printComprobantePdf(this.currentItem.data.id, moment(this.currentItem.data.fecha).format("YYYY-MM-DD"), this.currentItem.data.origen)
                .then(report => {
                    this.context.app.hideSpinner();
                    this.titleContentView = "Ped_" + this.currentItem.data.id + "_" + this.currentItem.data.sucursal + "_" + this.currentItem.data.cliente + "_" + this.currentItem.data.preventa + "_" + this.currentItem.data.reparto + + "_" + this.currentItem.data.reparto + ".pdf" ;
                    this.fileNameContentView = "Ped_" + this.currentItem.data.id + "_" + this.currentItem.data.sucursal + "_" + this.currentItem.data.cliente + "_" + this.currentItem.data.preventa + "_" + this.currentItem.data.reparto + + "_" + this.currentItem.data.reparto + ".pdf" ;
                    this.reportViewer.show("data:application/pdf; base64, " + report);
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "DESCARGAR") {
            this.context.app.showSpinner();
            this.cuentaPorCobrarService.loadComprobanteXml(this.currentItem.data.id, moment(this.currentItem.data.fecha).format("YYYY-MM-DD"), this.currentItem.data.origen)
                .then(response => {
                    this.titleContentView = "Recibo " + this.currentItem.data.numTrabajador;
                    this.fileNameContentView = this.currentItem.data.numTrabajador + ".xml";
                    this.reportViewer.show("data:text/xml, " + response);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "TIMBRAR") {
            this.context.app.showSpinner();
            this.cuentaPorCobrarService.generarCuentaPorCobrar(this.currentItem.data.id, this.context.app.defaultSite.identity, this.currentItem.data.codigoSucursal, this.currentItem.data.origen, false)
                .then(response => {
                    this.context.app.hideSpinner();
                    this.alerts.showAlert("Pedido " + this.currentItem.data.id + " timbrado", 'success');
                })
                .catch(error => {
                    this.handleError(error);
                    this.dlgError.config.backdrop = false;
                    this.dlgError.show();
                });
        }
        else if (option.name == "REFACTURAR") {
            this.context.app.showSpinner();
            this.cuentaPorCobrarService.generarCuentaPorCobrarFecha(this.currentItem.data.id, this.context.app.defaultSite.identity, this.currentItem.data.codigoSucursal, this.currentItem.data.origen, true)
                .then(response => {
                    this.context.app.hideSpinner();
                    this.alerts.showAlert("Pedido " + this.currentItem.data.id + " refacturado", 'success');
                })
                .catch(error => {
                    this.handleError(error);
                    this.dlgError.config.backdrop = false;
                    this.dlgError.show();
                });
        }
        else if (option.name == "CENCELAR") {
            this.context.app.showSpinner();
            this.cuentaPorCobrarService.cancelarCuentaPorCobrar(this.currentItem.data.id, moment(this.currentItem.data.fecha).format("YYYY-MM-DD"), this.currentItem.data.origen)
                .then(response => {
                    this.context.app.hideSpinner();
                    this.alerts.showAlert("Factura " + this.currentItem.data.id + " cancelada", 'success');
                })  
                .catch(error => {
                    this.handleError(error);
                    this.dlgError.config.backdrop = false;
                    this.dlgError.show();
                });
        }
        else if (option.name == "ADENDA") {
            this.context.app.showSpinner();
            this.cuentaPorCobrarService.generarAdenda(this.currentItem.data.id, moment(this.currentItem.data.fecha).format("YYYY-MM-DD"), this.currentItem.data.origen)
                .then(response => {
                    this.context.app.hideSpinner();
                    this.alerts.showAlert("Adenda " + this.currentItem.data.id + " generada", 'success');
                })
                .catch(error => {
                    this.handleError(error);
                    this.dlgError.config.backdrop = false;
                    this.dlgError.show();
                });
        }
        else if (option.name = "SUSTITUIR") {
            this.context.app.showSpinner();
            this.cuentaPorCobrarService.sustituir(this.currentItem.data.id, moment(this.currentItem.data.fecha).format("YYYY-MM-DD"), this.currentItem.data.origen)
                .then(response => {
                    this.context.app.hideSpinner();
                    this.alerts.showAlert("Comprobante sustituido", 'success');
                })
                .catch(error => this.handleError(error));
        }
    }

    onImportSaldos() {
        this.context.app.showSpinner();

        if (this.recalcularCXC==false)            
            this.cuentaPorCobrarService.importarSaldos(this.context.app.defaultCompany.identity)
                .then(_ => {
                    this.context.app.hideSpinner();
                    this.showAlert("Se han generado las Cuentas por Cobrar.", "success");
                })
                    .catch(error => this.handleError(error))
        else
            this.cuentaPorCobrarService.importarSaldosRecalcular(this.context.app.defaultCompany.identity)
                .then(_ => {
                    this.context.app.hideSpinner();
                    this.showAlert("Se han generado las Cuentas por Cobrar.", "success");
                })
                .catch(error => this.handleError(error))

    }

    showAlert(message: string, type: string) {
        this.context.app.hideSpinner();
        this.alerts.showAlert(message, type);
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.errorServer = message;
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo) {
        console.log('Aplicar politica');
    }

    onRemoveBranch() {

    }

    onCloseDialogError() {
        this.dlgError.hide();
    }

}