﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http'; import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, BaseService } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';
//Locales
import * as moment from 'moment';
import { DateTime } from 'wijmo/wijmo';

@Injectable()
export class CustomCuentaPorCobrarService extends BaseService {

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/custom/aga/credito/cuentasPorCobrar");
    }

    importarVentas(codigoSucursal: string, idSucursal: number, fechaCobro: Date): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('codigoSucursal', codigoSucursal)
            .set('idSucursal', idSucursal.toString())
            .set('fechaCobro', moment(fechaCobro).format());
        return this.apiService.get<any>('importar-ventas', params)
            .toPromise();
    }

    importarSaldos(idEmpresa: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get<any>('importar-saldos', params)
            .toPromise();
    }

    importarSaldosRecalcular(idEmpresa: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get<any>('importar-saldos-recalcular', params)
            .toPromise();
    }

    generarCuentaPorCobrar(idPedidoTit: string, idSucursal: number, codigoSucursal: string, origen: string, refacturar: boolean) {
        let fecha = new DateTime;
        fecha = Date.now();

        let params: HttpParams = new HttpParams()
            .set('idPedidoTit', idPedidoTit)
            .set('idSucursal', idSucursal.toString())
            .set('codigoSucursal', codigoSucursal)
            .set('origen', origen)
            .set('refacturar', refacturar.toString());            

        return this.apiService.post<any>('generar-cuenta-por-cobrar', params, null)
            .toPromise();
    }

    generarCuentaPorCobrarFecha(idPedidoTit: string, idSucursal: number, codigoSucursal: string, origen: string, refacturar: boolean) {
        let fecha = new DateTime;
        fecha = Date.now();

        let params: HttpParams = new HttpParams()
            .set('idPedidoTit', idPedidoTit)
            .set('idSucursal', idSucursal.toString())
            .set('codigoSucursal', codigoSucursal)
            .set('origen', origen)
            .set('refacturar', refacturar.toString())
            .set('fecha', moment(fecha).format());

        return this.apiService.post<any>('generar-cuenta-por-cobrar-fecha', params, null)
            .toPromise();
    }

    cancelarCuentaPorCobrar(idPedidoTit: string, fecha: string, origen: string) {
        let params: HttpParams = new HttpParams()
            .set("idPedidoTit", idPedidoTit)
            .set("fecha", fecha)
            .set('origen', origen);
        return this.apiService.post<any>('cancelar-cuenta-por-cobrar', params, null)
            .toPromise();
    }

    searchPedidoCliente(idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params: HttpParams = new HttpParams()
            .set('idBranch', idBranch.toString());
        return this.apiService.post<any>('search-pedidos-cliente', params, request)
            .toPromise();
    }

    printComprobantePdf(idPedidoTit: string, fecha: string, origen: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("idPedidoTit", idPedidoTit)
            .set("fecha", fecha)
            .set('origen', origen);
        return this.apiService.get<any>("generar-impresion-pedido", params)
            .toPromise();
    }

    loadComprobanteXml(idPedidoTit: string, fecha: string, origen: string): Promise<string> {
        let params: HttpParams = new HttpParams()
            .set("idPedidoTit", idPedidoTit)
            .set("fecha", fecha)
            .set('origen', origen);
        return this.apiService.getText("get-comprobante-xml", params)
            .toPromise();
    }

    generarAdenda(idPedidoTit: string, fecha: string, origen: string) {
        let params: HttpParams = new HttpParams()
            .set("idPedidoTit", idPedidoTit)
            .set("fecha", fecha)
            .set('origen', origen);
        return this.apiService.post<any>('generar-adenda', params, null)
            .toPromise();
    }

    sustituir(idPedidoTit: string, fecha: string, origen: string) {
        let params: HttpParams = new HttpParams()
            .set('idPedidoTit', idPedidoTit)
            .set('fechaPedido', fecha)
            .set('origen', origen);
        return this.apiService.post<any>('sustituir', params, null)
            .toPromise();
    }

}