﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http'; import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, BaseService } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';
//Locales
import { ContribuyenteRfcMultiplicado, ContribuyenteRfcMultiplicadoDetalle } from '../models/contribuyente-rfc-multiplicado'
import { AsyncTaskInfo } from 'framework3/queue/models/async-task';

@Injectable()
export class CustomMantenimientoCreditoService extends BaseService {

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/custom/aga/credito/mantenimientoCredito");
    }

    listarClientesRFCRepetido(idEmpresa: number): Promise<ContribuyenteRfcMultiplicado[]> {
        let params: HttpParams = new HttpParams()
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get<ContribuyenteRfcMultiplicado[]>('contribuyentes-repetidos-listar', params)
            .toPromise();
    }

    clientesRFCRepetido(rfc: string, idEmpresa: number): Promise<ContribuyenteRfcMultiplicadoDetalle[]> {
        let params: HttpParams = new HttpParams()
            .set('rfc', rfc)
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get<ContribuyenteRfcMultiplicadoDetalle[]>('contribuyente-repetido', params)
            .toPromise();
    }
    
    unificarContribuyentes(idContribuyente: number, idEmpresa: number) {
        let params: HttpParams = new HttpParams()
            .set('idContribuyente', idContribuyente.toString())
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.post<any>('unificar-contribuyentes', params, null)
            .toPromise();
    }

    asyncUnificarContribuyentes(idContribuyente: number, idEmpresa: number): Promise<AsyncTaskInfo> {
        let params: HttpParams = new HttpParams()
            .set('idContribuyente', idContribuyente.toString())
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get<AsyncTaskInfo>('async-unificar-contribuyentes', params)
            .toPromise();
    }


}