﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

import * as moment from 'moment';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { ProcessComponent } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';

//Features
import { CustomCuentaPorCobrarService } from '../../credit/services/cuenta-por-cobrar.service';
import { PedidoPixcuaService } from '../services/pedido-pixcua.service';



@Component({
    selector: 'pedido-pixcua-component',
    templateUrl: './pedido-pixcua.component.html'
})
export class PedidoPixcuaComponent extends ProcessComponent {

    editorForm: FormGroup;
    

    request: SearchRequest;
    fechaIni: Date = new Date();
    fechaFin: Date = new Date();
    currentPedidos: Array<any> = new Array<any>();
    currentItem: any;
    totalRows: number = 0;
    pageIndex: number = 0;
    menuItems: MenuItem[] = [];
    titleContentView: string = "";
    fileNameContentView: string = "";
    errorServer: string = "";
    recalcularCXC: boolean = false;

    allowRefacturar: boolean = false;
    allowAddenda: boolean = false;
    allowCancel: boolean = false;
    allowSustitucion: boolean = false;

    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('dlgError') dlgError: ModalDirective;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, private cuentaPorCobrarService: CustomCuentaPorCobrarService, private pedidoPixcuaSvc: PedidoPixcuaService) {
        super(context, router, route);
        this.editorForm = formBuilder.group({
            'fechaInicial': [''],
            'fechaFinal': [''],
            'errorServer': ['']
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.refreshList();
        });
        this.createMenu();
    }

    createMenu() {
        let mnuImportar: MenuItem = { icon: "glyphicon glyphicon-share", text: "Importar", name: "IMPORTAR", smallText: "Importar pedido" };
        this.menuItems.push(mnuImportar);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    refreshList() {
        this.context.app.showSpinner();
        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }
        let exist = false;
        this.request.conditions.forEach(r => {
            if (r.field == "fecha" && r.operator == ">=") {
                exist = true;
                r.value = moment(this.fechaIni).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
            else if (r.field == "fecha" && r.operator == "<=") {
                exist = true;
                r.value = moment(this.fechaFin).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
        });
        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fecha", operator: ">=", valueType: "datetime", value: moment(this.fechaIni).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conFechaFin: SearchCondition = { field: "fecha", operator: "<=", valueType: "datetime", value: moment(this.fechaFin).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            this.request.conditions.push(conFechaIni);
            this.request.conditions.push(conFechaFin);
        }
        this.cuentaPorCobrarService.searchPedidoCliente(this.context.app.defaultSite.identity, this.request)
            .then(response => {
                response.rows.forEach(r => {
                    r['fecha'] = moment(r['fecha']).toDate();
                });
                this.currentPedidos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = this.request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }


    onCurrentItemChanged(item: any) {
        this.currentItem = item;
    }

    searchRequested(request: SearchRequest) {
        this.request = request;
        this.refreshList();
    }

    optionClick(option: MenuItem) {
        if (option.name == "IMPORTAR") {
            this.context.app.showSpinner();
            this.pedidoPixcuaSvc.generarPedidoPixcua(this.currentItem.data.id, this.context.app.defaultSite.identity, this.currentItem.data.codigoSucursal, this.currentItem.data.origen, false)
                .then(response => {
                    this.context.app.hideSpinner();
                    this.alerts.showAlert("Pedido " + this.currentItem.data.id + " importado", 'success');
                })
                .catch(error => {
                    this.handleError(error);
                    this.dlgError.config.backdrop = false;
                    this.dlgError.show();
                });
        }
        
    }


    showAlert(message: string, type: string) {
        this.context.app.hideSpinner();
        this.alerts.showAlert(message, type);
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.errorServer = message;
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo) {
        console.log('Aplicar politica');
    }

    onRemoveBranch() {

    }

    onCloseDialogError() {
        this.dlgError.hide();
    }

}