﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { CelulaService} from '../services/celula.service';
import { CelulaInfo, CelulaItem} from '../models/celula';

import { Position, MapComponent } from 'framework3/geolocation';


@Component({
    selector: 'distribucion-celula-editor',
    templateUrl: './celulas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CelulasEditor)
        },
        {
            provide: CatalogService, useExisting: CelulaService
        }
    ]
})
export class CelulasEditor extends CatalogEditor {
    celula: CelulaInfo = new CelulaInfo();        
    position: Position;
    @ViewChild('map') map: MapComponent;

    center: Position;
    zoom: number;
    startPoint: Position;
    cellColor: string = '#1E90FF';
    strokeColor: string = '#1E9000';
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private celulaService: CelulaService) {

        super(context, router, route, changeDetector);
        
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'estven': ['', Validators.required],
            'cuenta': ['']
        });

        this.position = new Position(0, 0);
        this.center = new Position(20.67262, -103.289075);
        this.zoom = 12;
        this.startPoint = new Position(20.67262, -103.289075);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
                
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.celulaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.celula = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.celulaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.celula = info;               
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.celulaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.celula = info;    

                if (this.celula.area && this.celula.area.length>0)
                    this.position = new Position(this.celula.area[0].lat, this.celula.area[0].lng);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.celulaService.update(this.celula)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.celula.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.celula = info;
            })
            .catch(error => this.handleError(error));        
    }

    onTabSelect(args: any) {
        if (args.heading == 'Área geográfica') {            
            this.map.triggerResize(true);       
        }
    }

}