﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { PositionComponent, Position } from 'framework3/geolocation';
//Features
import { CentrosService } from '../services/centros.service';
import { CentrosInfo, CentrosItem } from '../models/centros';
import { AlmacenItem } from '../../inventory/models/almacen';
import { AreaService } from '../../inventory/services/area.service';

@Component({
    selector: 'distribucion-centros-editor',
    templateUrl: './centros.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CentrosEditor)
        },
        {
            provide: CatalogService, useExisting: CentrosService
        }
    ]
})
export class CentrosEditor extends CatalogEditor {
    centros: CentrosInfo = new CentrosInfo();
    ubicacion: Position = new Position(20.677034, -103.346984);
    area: number = 0;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private centrosService: CentrosService, private areaService: AreaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idAreaInventario': ['', AztecaValidators.selected],
            'idAlmacenProducto': ['', AztecaValidators.selected],
            'idAlmacenTarimas': ['', AztecaValidators.selected],
            'ubicacion': ['']
        });
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.centrosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centros = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.centrosService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centros = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.centrosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centros = info;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.centros.latitud = this.ubicacion.lat;
        this.centros.longigud = this.ubicacion.lng;
        this.centrosService.update(this.centros)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.centros.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.centros = info;
            })
            .catch(error => this.handleError(error));
    }

    onItemChanged(args: any) {
        this.area = args.identity;
        this.catalogForm.controls.idAlmacenProducto.setValue(-1);
        this.catalogForm.controls.idAlmacenTarimas.setValue(-1);
    }

}