﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

import { Control } from 'wijmo/wijmo';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';

//Features
import { ClienteService} from '../services/cliente.service';
import { ClienteInfo, ClienteItem } from '../models/cliente';

import { CelulaInfo, CelulaItem } from '../models/celula';
import { CelulaService } from '../services/celula.service';
import { GrupoBeneficiosInfo, GrupoBeneficiosItem } from '../models/grupo-beneficios';
import { GrupoBeneficiosService } from '../services/grupo-beneficios.service';
import { GamaInfo, GamaItem } from '../models/gama';
import { GamaService } from '../services/gama.service';


import { ListaPreciosService } from '../../sales/services/lista-precios.service';
import { ListaPreciosItem } from '../../sales/models/lista-precios';
import { GrupoDescuentoService } from '../../sales/services/grupo-descuento.service';
import { GrupoDescuentoItem } from '../../sales/models/grupo-descuento';

import { FormaPagoService } from '../../credit/services/forma-pago.service';
import { FormaPagoItem } from '../../credit/models/forma-pago';
import { AddendaItem } from '../../accounting/models/addenda';
import { ComprobanteFiscalService } from '../../accounting/services/comprobante-fiscal.service';
import { GrupoDeudorSelectorComponent } from '../../credit/components/grupo-deudores-selector.component';
import { GrupoDeudorInfo } from '../../credit/models/grupo-deudor';


@Component({
    selector: 'distribucion-cliente-editor',
    templateUrl: './clientes.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ClientesEditor)
        },
        {
            provide: CatalogService, useExisting: ClienteService
        }
    ]
})
export class ClientesEditor extends CatalogEditor {
    cliente: ClienteInfo = new ClienteInfo();        
    //
    listasPrecio: ListaPreciosItem[] = [];
    gruposDescuento: GrupoDescuentoItem[] = [];
    formasPago: FormaPagoItem[] = [];
    addendas: AddendaItem[] = [];

    celulas: CelulaItem[] = [];
    gamas: GamaItem[] = [];
    gruposBeneficio: GrupoBeneficiosItem[] = [];

    condicionesPago: { id: string, nombre: string }[] = [];
    clasificadores: Clasificacion[] = [];
    @ViewChild('selGrupoDeudor') selInsumo: GrupoDeudorSelectorComponent;

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private clienteService: ClienteService, private listaPreciosService: ListaPreciosService,
        private grupoDescuentoService: GrupoDescuentoService, private formaPagoService: FormaPagoService, private comprobanteFiscalService: ComprobanteFiscalService,
        private celulaService: CelulaService, private grupoBeneficiosService: GrupoBeneficiosService, private gamaService: GamaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'telefonos': [''],
            'correo': ['', Validators.required],
            'pagina': [''],
            'pagWeb': [''],
            'webUser': [''],
            'webPassword': [''],
            'cuenta': [''],
            'domicilio': [''],

            'idGrupoDeudor': [0],
            'facturable': [''],
            'rfc': [''],
            'razonSocial': [''],
            'idAddenda': [''],
            'idListaPrecio': ['', Validators.required],
            'idGrupoDescuento': ['', Validators.required],
            'idFormaPago': [''],
            'cuentaPago': [''],

            'condiciones': ['', Validators.required],
            'limiteCredito': [''],
            'diasCredito': [''],
            'clasificaciones': [[], AztecaValidators.validClasifications],

            'codigoBarras': [''],
            'atiende': [''],
            'paquetes': [''],
            'celula': [''],
            'gama': [''],
            'grupoBeneficio': [''],           
            'vales': [''],           
            'descVales': [''],           
            'documentos': [''],   
        });
        this.condicionesPago =
            [
                { "id": 'C', "nombre": "Credito" },
                { "id": 'E', "nombre": "Contado" }
            ];
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadListasPrecio();
        this.loadGruposDescuento();
        this.loadFormasPago();
        this.loadClasificadores();
        this.loadAddendas();

        this.loadCelulas();
        this.loadGamas();
        this.loadGruposBeneficio();
    }

    loadClasificadores() {
        this.clienteService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => {
                this.clasificadores = clasificadores
            })
            .catch(error => this.handleError(error));
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.clienteService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cliente = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.clienteService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cliente = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.clienteService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cliente = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.clienteService.update(this.cliente)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.cliente.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.cliente = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadListasPrecio() {
        this.listaPreciosService.loadList(this.context.app.defaultSite.identity)
            .then(listas => this.listasPrecio = listas)
            .catch(error => this.handleError(error));
    }
    loadGruposDescuento() {
        this.grupoDescuentoService.loadList(this.context.app.defaultSite.identity)
            .then(grupos => this.gruposDescuento = grupos)
            .catch(error => this.handleError(error));
    }
    loadFormasPago() {
        this.formaPagoService.loadList(this.context.app.defaultSite.identity)
            .then(formas => this.formasPago = formas)
            .catch(error => this.handleError(error));
    }
    loadAddendas() {
        this.comprobanteFiscalService.loadListAddendas(this.context.app.defaultCompany.identity)
            .then(addendas => this.addendas = addendas)
            .catch(error => {
                this.handleError(error)
            });
    }

    loadCelulas()
    {
        this.celulaService.loadList(this.context.app.defaultSite.identity)
            .then(celulas => this.celulas = celulas)
            .catch(error => {
                this.handleError(error)
            });
    }
    loadGamas()
    {
        this.gamaService.loadList(this.context.app.defaultSite.identity)
            .then(gamas => this.gamas = gamas)
            .catch(error => {
                this.handleError(error)
            });
    }
    loadGruposBeneficio()
    {
        this.grupoBeneficiosService.loadList(this.context.app.defaultSite.identity)
            .then(gruposBeneficio => this.gruposBeneficio = gruposBeneficio)
            .catch(error => {
                this.handleError(error)
            });
    }

    //Administracion de contactos
    onMenuItemClick(option: MenuItem) {
        if (option.name == 'CONTACTS') {
            this.router.navigate(['sales/clientes/contacts', this.process.id, this.cliente.identity])
        }
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }
    onGrupoDeudorSelected(grupo: GrupoDeudorInfo) {
        this.cliente.clienteVentas.deudor.contribuyente = grupo.contribuyente;
    }


}