﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { EstVenInfo } from '../models/est-ven';
import { EstVenService } from '../services/est-ven.service';
import { EstVenTipoItem } from '../models/est-ven-tipo';


@Component({    
    selector: 'distribucion-est-ven-manager',
    templateUrl: './est-ven.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EstVenManager)
        },
        {
            provide: CatalogService, useExisting: EstVenService
        }
    ]
})
export class EstVenManager extends HierarchyManager {
    estVen: EstVenInfo[] = [];
    estVenEdicion: EstVenInfo = new EstVenInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    estVenTipos: EstVenTipoItem[] = [];

    constructor(protected context: Context, protected router: Router, protected route:ActivatedRoute, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private estVenService: EstVenService) {
        super(context, router, route, changeDetector);
       
        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],            
            'nombre': ['', Validators.required],
            'tipo': [''],
            'responsable': [''],
            'auxiliar': [''],
            'cuenta': ['']
        });
    }

	////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.estVenService.loadHierarchy(this.context.app.defaultCompany.identity, 0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.estVen = nodes;
            })
            .catch(error => this.handleError(error));
    }


	//////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadHierarchy();
        this.loadTipos();
        //Hacer llamadas de carga inicial de datos aquí

    }


    onNodesChanged(estven: EstVenInfo[]) {
        this.estVen = estven;        
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.estVenEdicion = new EstVenInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.estVenEdicion = new EstVenInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.estVenEdicion.idBranch = this.context.app.defaultSite.identity;
        this.estVenEdicion.idCompany = this.context.app.defaultCompany.identity;
		//
        this.currentParent = parentNode;        
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {        
        this.estVenEdicion = this.estVenService.copy(node.data);
        this.showEditor();        
    }

    onSavingNode() {        
        //Si es un nuevo nodo
        if (this.estVenEdicion.clientID == 0) {
            this.addNode(this.estVenEdicion, this.currentParent);            
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.estVenEdicion);
        }
		this.estVenEdicion = new EstVenInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<EstVenInfo>) {
        this.context.app.showSpinner();
        this.estVenService.saveHierarchy(changes)
            .then(estven => {
                this.estVen = estven;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));            
    }

    loadTipos() {
        this.estVenService.loadListTipos(this.context.app.defaultSite.identity)
            .then(tipos => this.estVenTipos = tipos)
            .catch(error => this.handleError(error));
    }

}