﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { FormaVentaInfo } from '../models/forma-venta';
import { FormaVentaService } from '../services/forma-venta.service';

@Component({    
    selector: 'distribucion-formas-ventas-manager',
    templateUrl: './formas-ventas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FormasVentasManager)
        },
        {
            provide: CatalogService, useExisting: FormaVentaService
        }
    ]
})
export class FormasVentasManager extends HierarchyManager {
    formasventas: FormaVentaInfo[] = [];
    formaventaEdicion: FormaVentaInfo = new FormaVentaInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    
    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, protected router: Router, protected route: ActivatedRoute,
         private formBuilder: FormBuilder,
        private formaVentaService: FormaVentaService) {
        super(context, router, route, changeDetector);
       
        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required]
        });
    }

	////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.formaVentaService.loadHierarchy(this.context.app.defaultCompany.identity, 0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.formasventas = nodes;
            })
            .catch(error => this.handleError(error));
    }


	//////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadHierarchy();
        //Hacer llamadas de carga inicial de datos aquí

    }


    onNodesChanged(formasventas: FormaVentaInfo[]) {
        this.formasventas = formasventas;        
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.formaventaEdicion = new FormaVentaInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.formaventaEdicion = new FormaVentaInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.formaventaEdicion.idBranch = this.context.app.defaultSite.identity;
        this.formaventaEdicion.idCompany = this.context.app.defaultCompany.identity;
		//
        this.currentParent = parentNode;        
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {        
        this.formaventaEdicion = this.formaVentaService.copy(node.data);
        this.showEditor();        
    }

    onSavingNode() {        
        //Si es un nuevo nodo
        if (this.formaventaEdicion.clientID == 0) {
            this.addNode(this.formaventaEdicion, this.currentParent);            
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.formaventaEdicion);
        }
		this.formaventaEdicion = new FormaVentaInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<FormaVentaInfo>) {
        this.context.app.showSpinner();
        this.formaVentaService.saveHierarchy(changes)
            .then(formasventas => {
                this.formasventas = formasventas;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));            
    }

}