﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { FrecuenciaVisitaService} from '../services/frecuencia-visita.service';
import { FrecuenciaVisitaInfo, FrecuenciaVisitaItem} from '../models/frecuencia-visita';

@Component({
    selector: 'distribucion-frecuencia-visita-editor',
    templateUrl: './frecuencias-visita.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FrecuenciasVisitaEditor)
        },
        {
            provide: CatalogService, useExisting: FrecuenciaVisitaService
        }
    ]
})
export class FrecuenciasVisitaEditor extends CatalogEditor {
    frecuenciaVisita: FrecuenciaVisitaInfo = new FrecuenciaVisitaInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private frecuenciaVisitaService: FrecuenciaVisitaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'lunes': [false],
            'martes': [false],
            'miercoles': [false],
            'jueves': [false],
            'viernes': [false],
            'sabado': [false],
            'domingo': [false],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.frecuenciaVisitaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.frecuenciaVisita = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.frecuenciaVisitaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.frecuenciaVisita = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.frecuenciaVisitaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.frecuenciaVisita = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.frecuenciaVisitaService.update(this.frecuenciaVisita)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.frecuenciaVisita.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.frecuenciaVisita = info;
            })
            .catch(error => this.handleError(error));        
    }

}