﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { FrecuenciaService} from '../services/frecuencia.service';
import { FrecuenciaInfo, FrecuenciaItem} from '../models/frecuencia';
import { FrecuenciaDia } from '../models/frecuencia-dia';
import { forEach } from '@angular/router/src/utils/collection';
import { Position, MapComponent } from 'framework3/geolocation';


@Component({
    selector: 'distribucion-frecuencia-editor',
    templateUrl: './frecuencias.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FrecuenciasEditor)
        },
        {
            provide: CatalogService, useExisting: FrecuenciaService
        }
    ]
})
export class FrecuenciasEditor extends CatalogEditor {
    frecuencia: FrecuenciaInfo = new FrecuenciaInfo();        
    tipos: any = [{ 'Tipo': 'S', 'Nombre': 'Semanal' }, { 'Tipo': 'M', 'Nombre': 'Mensual' }, { 'Tipo': 'P', 'Nombre': 'Personalizado' }];
    dias: FrecuenciaDia[] = [];
    meses: Mes[] =[];
    //
    position: Position;
    @ViewChild('map') map: MapComponent;

    center: Position;
    zoom: number;
    startPoint: Position;
    cellColor: string = '#1E90FF';
    strokeColor: string = '#1E9000';

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private frecuenciaService: FrecuenciaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': [''],
            'nombre': ['',Validators.required],
            'tipo' : ['', Validators.required]
        });

        this.position = new Position(0, 0);
        this.center = new Position(20.67262, -103.289075);
        this.zoom = 12;
        this.startPoint = new Position(20.67262, -103.289075);
    }

	///////////////////////// METODOS ///////////////////////////////////////
    loadMeses() {
        if (this.frecuencia.tipo == 'P') {
            this.createMeses();
            let numeroMes: number = 0;
            let acumDia: number = 0
            let nombreMeses: string[] = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
            let diasMes: number[] = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
            this.dias.forEach(dia => {
                acumDia++;
                this.meses[numeroMes].nombre = nombreMeses[numeroMes];
                this.meses[numeroMes].numeroMes = numeroMes;
                this.meses[numeroMes].dias.push(dia);
                if (acumDia == diasMes[numeroMes]) {
                    acumDia = 0;
                    numeroMes++;
                }
            });
        }
    }

    createMeses() {
        this.meses = [];
        for (let i: number = 0; i < 12; i++) {
            let mes = new Mes();
            this.meses.push(mes);
        }
    }

    createVisitas(tipo: string) {
        let i: number = 0;
        let acum: number;

        switch (tipo) {
            case 'S':
                acum = 7;
                break;
            case 'M':
                acum = 31;
                break;
            case 'P':
                acum = 365;
                this.createMeses();
                break;
        }
        this.dias = [];

        let numeroMes: number = 0;
        let acumDia: number = 0
        let nombreMeses: string[] = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
        let diasMes: number[] = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

        for (i = 0; i < acum; i++) {
            let dia: FrecuenciaDia = new FrecuenciaDia();
            dia.identity = 0;
            dia.dia = i + 1;
            dia.visita = false;
            this.dias.push(dia);

            if (tipo == 'P') {
                acumDia++;
                this.meses[numeroMes].nombre = nombreMeses[numeroMes];
                this.meses[numeroMes].numeroMes = numeroMes;
                this.meses[numeroMes].dias.push(dia);
                if (acumDia == diasMes[numeroMes]) {
                    acumDia = 0;
                    numeroMes++;
                }
            }
        }
    }

    visitar(dia: FrecuenciaDia) {
        if(!this.readonlyMode)
            dia.visita = !dia.visita;
    }

    getTextSemanal(dia: FrecuenciaDia) {
        let nombres: string[] = ['Domingo', 'Lunes', 'Martes', 'Miercoles', 'Jueves', 'Viernes', 'Sabado'];
        return nombres[dia.dia-1];
    }

    getTextPersonalizado(dia: FrecuenciaDia, mes:Mes) {
        let diasMes: number[] = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365];
        return (diasMes[mes.numeroMes] - dia.dia)*-1;
    }

    selected(dia: FrecuenciaDia) {
        if (dia.visita == true)
            return 'bg-primary text-white';
        else
            return '';
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.frecuenciaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.frecuencia = info;        
                this.dias = this.frecuencia.dias.slice();
                this.loadMeses();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.frecuenciaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.frecuencia = info;
                this.dias = this.frecuencia.dias.slice();
                this.loadMeses();
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();
        
        this.frecuenciaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.frecuencia = info;     
                this.dias = this.frecuencia.dias.slice();
                this.loadMeses();
                if (this.frecuencia.area && this.frecuencia.area.length > 0)
                    this.position = new Position(this.frecuencia.area[0].lat, this.frecuencia.area[0].lng);

            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.frecuencia.dias = this.dias.slice();
        this.frecuenciaService.update(this.frecuencia)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.frecuencia.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
                this.frecuencia = info;
                this.dias = this.frecuencia.dias.slice();
                this.loadMeses();
            })
            .catch(error => this.handleError(error));        
    }

    onTabSelect(args: any) {
        if (args.heading == 'Área geográfica') {
            this.map.triggerResize(true);
        }
    }

}

class Mes {
    nombre: string;
    numeroMes: number;
    dias: FrecuenciaDia[] = [];
}