﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, FileLink } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem, DownloadPopupComponent } from 'framework3/controls';
//Features
import { FrecuenciaService} from '../services/frecuencia.service';
import { FrecuenciaInfo, FrecuenciaItem} from '../models/frecuencia';

@Component({    
    selector: 'distribucion-frecuencias-manager',
    templateUrl: './frecuencias.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FrecuenciasManager)
        },
        {
            provide: CatalogService, useExisting: FrecuenciaService
        }
	]
})
export class FrecuenciasManager extends CatalogManager {
    frecuencias: FrecuenciaItem[]=[];    
    //    
    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private frecuenciaService: FrecuenciaService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.frecuenciaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.frecuencias = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.frecuenciaService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.frecuenciaService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'AREA') {
            this.router.navigate(['distribution/frecuencia-load-area/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == "DOWNLOAD") {
            this.context.app.showSpinner();
            //Generar KML
            this.frecuenciaService.generarKmlArea(this.manager.currentItem.id)
                .then(kml => {
                    //Descargar xml
                    this.context.app.hideSpinner();
                    let links: FileLink[] = [];
                    //links.push(pdf);
                    links.push(kml);
                    this.downloadPopup.show(links);
                })
                .catch(error => this.handleError(error));
        }

    }

}
