﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { GamaService} from '../services/gama.service';
import { GamaInfo, GamaItem } from '../models/gama';

import { CuentaSelectorComponent } from '../../accounting/components/cuenta-selector.component';

@Component({
    selector: 'distribucion-gamas-editor',
    templateUrl: './gamas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => GamasEditor)
        },
        {
            provide: CatalogService, useExisting: GamaService
        }
    ]
})
export class GamasEditor extends CatalogEditor {
    gama: GamaInfo = new GamaInfo();        

    //ViewChilds
    @ViewChild('selectorCuenta') selectorCuenta: CuentaSelectorComponent;

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private gamaService: GamaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuenta': [0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.gamaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.gama = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.gamaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.gama = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.gamaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.gama = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.gamaService.update(this.gama)
            .then(info => {
                this.context.app.hideSpinner();
                this.gama = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'ELEMENTS') {
            this.router.navigate(['distribution/gamas/elements', this.process.id, this.gama.identity])
        } else if (option.name == 'PRODUCTOS') {
            this.router.navigate(['distribution/gamas-productos/' + this.process.id + '/' + this.gama.identity]);
        }
    }

}