﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { GrupoBeneficiosService} from '../services/grupo-beneficios.service';
import { GrupoBeneficiosInfo, GrupoBeneficiosItem} from '../models/grupo-beneficios';

@Component({
    selector: 'distribucion-gruposbeneficios-editor',
    templateUrl: './grupos-beneficios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => GruposBeneficiosEditor)
        },
        {
            provide: CatalogService, useExisting: GrupoBeneficiosService
        }
    ]
})
export class GruposBeneficiosEditor extends CatalogEditor {
    grupobeneficios: GrupoBeneficiosInfo = new GrupoBeneficiosInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private grupoBeneficiosService: GrupoBeneficiosService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoBeneficiosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupobeneficios = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.grupoBeneficiosService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupobeneficios = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoBeneficiosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupobeneficios = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.grupoBeneficiosService.update(this.grupobeneficios)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupobeneficios = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

}