﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';
//Features
import { ProductosService } from '../services/productos.service';
import { ArticuloService } from '../../sales/services/articulo.service';
import { ArticuloStockService } from '../../inventory/services/articulo-stock.service';
import { UnidadMedidaService } from '../../inventory/services/unidad-medida.service';
import { UnidadMedidaTipoService } from '../../inventory/services/unidad-medida-tipo.service';
import { UnidadMedidaTipoItem } from '../../inventory/models/unidad-medida-tipo';
import { UnidadMedidaItem } from '../../inventory/models/unidad-medida';
import { ProductosInfo, ProductosItem } from '../models/productos';
//import { ImpuestoAsignacionInfo } from '../../accounting/models/impuesto-asignacion';
import { ImpuestoAsignacionZonaInfo } from '../../sales/models/impuesto-asignacion-zona';
import { ImpuestosAsignacionEditor } from '../../accounting/components/impuesto-asignacion.editor';

@Component({
    selector: 'distribucion-productos-editor',
    templateUrl: './productos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ProductosEditor)
        },
        {
            provide: CatalogService, useExisting: ProductosService
        }
    ]
})
export class ProductosEditor extends CatalogEditor {
    productos: ProductosInfo = new ProductosInfo();
    tiposUnidad: UnidadMedidaTipoItem[] = [];
    unidadesMedida: UnidadMedidaItem[] = [];
    clasificadores: Clasificacion[] = [];
    artImpCollection: TrackableCollection = new TrackableCollection();
    articuloImpuestos: ImpuestoAsignacionZonaInfo[] = [];
    //parametro opcional del url
    idArticulo: number;
    idArticuloStock: number;
    codigo: string;

    //@ViewChilds
    @ViewChild(ImpuestosAsignacionEditor) articuloImpuestosDialog: ImpuestosAsignacionEditor;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef, private articuloStockService: ArticuloStockService,
        private articuloService: ArticuloService, private unidadMedidaService: UnidadMedidaService, private unidadMedidaTipoService: UnidadMedidaTipoService, protected context: Context, private location: Location, private productosService: ProductosService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'articuloStockGeneral': [''],
            'articuloStockControles': [''],
            'tipoUnidad': ['', AztecaValidators.selected],
            'unidadAlmacenamiento': ['', AztecaValidators.selected],
            'unidadVenta': ['', AztecaValidators.selected],
            'unidadCarga': ['', AztecaValidators.selected],
            'unidadContenido': ['', AztecaValidators.selected],
            'clasificaciones': [[], AztecaValidators.validClasifications],
            'comercial': [''],
            'productosInfo': ['']
        });
    }

    ngAfterViewChecked() {
        this.artImpCollection.sourceCollection = this.productos.articulo.impuestos;
    }

    ngOnInit() {
        this.route.queryParams.forEach(params => {
            this.idArticulo = +params['idArticulo'] || 0;
        })

        this.route.queryParams.forEach(params => {
            this.idArticuloStock = +params['idArticuloStock'] || 0;
        })

        this.route.queryParams.forEach(params => {
            this.codigo = params['codigo'];
        })
    }

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadClasificadores();
        this.loadTiposUnidad();
    }

    loadClasificadores() {
        this.articuloStockService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => this.clasificadores = clasificadores)
            .catch(error => this.handleError(error));
    }

    onCreatingItem() {
        this.context.app.showSpinner();
        this.productosService.create()
            .then(producto => {
                this.context.app.hideSpinner();
                this.productos = producto;
                if (this.codigo.length > 0) {
                    this.productos.articulo.articuloStock.code = this.codigo;
                }
            })
            .catch(error => this.handleError(error));

        if (this.idArticulo > 0) {
            this.articuloService.getDetailByID(this.idArticulo)
                .then(articulo => {
                    this.context.app.hideSpinner();
                    this.productos.articulo = articulo;
                    this.loadUnidadesMedida(articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
                })
                .catch(error => this.handleError(error));
        }

        if (this.idArticuloStock > 0) {
            this.articuloStockService.getDetailByID(this.idArticuloStock)
                .then(articuloStock => {
                    this.context.app.hideSpinner();
                    this.productos.articulo.articuloStock = articuloStock;
                    this.loadUnidadesMedida(articuloStock.unidadMedida.idUnidadMedidaTipo);
                })
                .catch(error => this.handleError(error));
        }

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.productosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.productos = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.productosService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.productos = info;
                this.loadUnidadesMedida(info.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.productosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.productos = info;
                this.loadUnidadesMedida(info.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.productosService.update(this.productos)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.productos.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.productos = info;
                this.loadUnidadesMedida(info.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    loadTiposUnidad() {
        this.unidadMedidaTipoService.loadList()
            .then(tipos => this.tiposUnidad = tipos)
            .catch(error => this.handleError(error));
    }

    onTipoUnidadChanged(idTipoUnidad) {
        // si se manejan series quitamos los validadores de las unidades de medida, en caso contrario se aplican validadores y se parcha el valor para reiniciar la validación
        if (this.productos.articulo.articuloStock.manejaSeries) {
            this.catalogForm.controls.unidadAlmacenamiento.clearValidators();
            this.catalogForm.controls.unidadAlmacenamiento.updateValueAndValidity();
            this.catalogForm.controls.unidadVenta.clearValidators();
            this.catalogForm.controls.unidadVenta.updateValueAndValidity();
            this.catalogForm.controls.unidadCarga.clearValidators();
            this.catalogForm.controls.unidadCarga.updateValueAndValidity();
            this.catalogForm.controls.unidadContenido.clearValidators();
            this.catalogForm.controls.unidadContenido.updateValueAndValidity();
        } else {
            this.catalogForm.controls.unidadAlmacenamiento.setValidators(AztecaValidators.selected);
            this.catalogForm.controls.unidadAlmacenamiento.patchValue(-1);
            this.catalogForm.controls.unidadAlmacenamiento.updateValueAndValidity();
            this.catalogForm.controls.unidadVenta.setValidators(AztecaValidators.selected);
            this.catalogForm.controls.unidadVenta.patchValue(-1);
            this.catalogForm.controls.unidadVenta.updateValueAndValidity();
            this.catalogForm.controls.unidadCarga.setValidators(AztecaValidators.selected);
            this.catalogForm.controls.unidadCarga.patchValue(-1);
            this.catalogForm.controls.unidadCarga.updateValueAndValidity();
            this.catalogForm.controls.unidadContenido.setValidators(AztecaValidators.selected);
            this.catalogForm.controls.unidadContenido.patchValue(-1);
            this.catalogForm.controls.unidadContenido.updateValueAndValidity();
        }

        this.loadUnidadesMedida(idTipoUnidad);
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        this.unidadMedidaService.loadList(idTipoUnidad)
            .then(unidades => this.unidadesMedida = unidades)
            .catch(error => this.handleError(error));
    }

    onArticuloStockGeneralValidChange() {
        this.changeDetector.detectChanges();
    }

    onArticuloStockControlesValidChange() {
        this.changeDetector.detectChanges();
    }

    onArticuloImpuestosValidChange() {
        this.changeDetector.detectChanges();
    }

    onProductosInfoValidChange() {
        this.changeDetector.detectChanges();
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.articuloImpuestosDialog.show();
        }
    }

    onAgregarImpuesto(args: ImpuestoAsignacionZonaInfo) {
        let impuestoExists: boolean = false;
        this.productos.articulo.impuestos.forEach(articuloImp => {
            if (articuloImp.idImpuesto == args.idImpuesto) {
                this.showAlert("El impuesto ya fue agregado", "información");
                impuestoExists = true;
            }
        })
        if (!impuestoExists) {
            this.productos.articulo.impuestos.push(args);
            this.artImpCollection.refresh();
        }
    }

    onRemoveItem(item) {
        this.artImpCollection.remove(item);
        this.artImpCollection.refresh();
    }


}