﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { RutasRepartoService } from '../services/rutas-reparto.service';
import { RutasRepartoInfo, RutasRepartoItem, DotacionesInfo } from '../models/rutas-reparto';
import { AgregarDotacionesComponent } from '../components/agregar-dotacion.component';
import { CelulaItem } from '../models/celula';

@Component({
    selector: 'distribucion-rutas-reparto-editor',
    templateUrl: './rutas-reparto.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RutasRepartoEditor)
        },
        {
            provide: CatalogService, useExisting: RutasRepartoService
        }
    ]
})
export class RutasRepartoEditor extends CatalogEditor {
    rutasReparto: RutasRepartoInfo = new RutasRepartoInfo();
    celulas: CelulaItem[] = [];
    tiposRuta: { id: string, nombre: string }[] = [];

    //ViewChilds
    @ViewChild(AgregarDotacionesComponent) agregarDotacionesDialog: AgregarDotacionesComponent;

    dotacionesCollection: TrackableCollection = new TrackableCollection();
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private rutasRepartoService: RutasRepartoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'estVen': ['', AztecaValidators.selected],
            'tipoRuta': ['', Validators.required],
            'clienteGenerico': ['', AztecaValidators.selected],
            'idDispositivoCaptura': ['', AztecaValidators.selected],
            'idAlmacenCarga': ['', AztecaValidators.selected],
            'idVehiculo': ['', AztecaValidators.selected],
            'capacidad': ['', Validators.min(0)],
            'dotacion': [false],
            'idCuenta': ['']
        });

        this.tiposRuta =
            [
                { "id": "P", "nombre": "Reparto Preventa" },
                { "id": "A", "nombre": "Reparto Autoventa" },
                { "id": "C", "nombre": "Autoventa Controlada" }
            ];
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ngAfterViewChecked() {
        this.dotacionesCollection.sourceCollection = this.rutasReparto.dotaciones;
    }

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.rutasRepartoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.rutasReparto = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.rutasRepartoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.rutasReparto = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.rutasRepartoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.rutasReparto = info;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        if (this.rutasReparto.capacidad == null) {
            this.rutasReparto.capacidad = 0;
        }

        this.rutasRepartoService.update(this.rutasReparto)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.rutasReparto.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.rutasReparto = info;
            })
            .catch(error => this.handleError(error));
    }

    onTipoRutaChanged() {

    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'AgregarDotacion') {
            this.agregarDotacionesDialog.show();
        }
    }

    onAgregarDotacion(args: DotacionesInfo) {
        let articuloExists: boolean = false;
        this.rutasReparto.dotaciones.forEach(rutasRep => {
            if (rutasRep.articuloStock.identity == args.articuloStock.identity) {
                this.showAlert("El articulo ya fue agregado", "información");
                articuloExists = true;
            }
        })
        if (!articuloExists) {
            this.rutasReparto.dotaciones.push(args);
            this.dotacionesCollection.refresh();
        }
    }

    onRemoveDotacion(item: DotacionesInfo) {
        var index = this.rutasReparto.dotaciones.findIndex(x => x.articuloStock.identity == item.articuloStock.identity);
        if (index > -1) {
            this.rutasReparto.dotaciones.splice(index, 1);
        }

        this.dotacionesCollection.remove(item);
        this.dotacionesCollection.refresh();
    }

}