﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { PositionComponent, Position, MapComponent } from 'framework3/geolocation';
//Features
import { SucursalService} from '../services/sucursal.service';
import { SucursalInfo, SucursalItem } from '../models/sucursal';

import { EstVenService } from '../services/est-ven.service';
import { EstVenInfo, EstVenItem } from '../models/est-ven';


import { UnidadNegocioService } from '../../sales/services/unidad-negocio.service'
import { UnidadNegocioInfo, UnidadNegocioItem } from '../../sales/models/unidad-negocio'

@Component({
    selector: 'distribucion-sucursal-editor',
    templateUrl: './sucursales.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SucursalesEditor)
        },
        {
            provide: CatalogService, useExisting: SucursalService
        }
    ]
})
export class SucursalesEditor extends CatalogEditor {
    sucursal: SucursalInfo = new SucursalInfo();        
    ubicacion: Position = new Position(20.677034, -103.346984);

    @ViewChild('mapComponent') map: PositionComponent;


    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private sucursalService: SucursalService,
        private unidadNegocioService: UnidadNegocioService, private estVenService: EstVenService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': [''],
            'idCuenta': [''],
            'idUnidadNegocio': [''],
            'idEstVen': [''],
            'idAreaInventario': ['', AztecaValidators.selected],
            'ubicacion': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.sucursalService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.sucursal = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.sucursalService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.sucursal = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.sucursalService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.ubicacion = new Position(info.latitud, info.longitud);
                this.sucursal = info; 
                this.sucursal.identity = info.sucursalBase.identity;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();

        this.sucursal.latitud = this.ubicacion.lat;
        this.sucursal.longitud = this.ubicacion.lng;


        this.sucursalService.update(this.sucursal)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.sucursal.identity == 0) {
                    //this.router.navigate(['../', info.identity], { relativeTo: this.route });
                    this.router.navigate(['../', this.process.id, info.identity]);
                }
                else {					
					if (args.closeEditor)
                    //    this.router.navigate(['../'], { relativeTo: this.route });
                        this.location.back();
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.sucursal = info;
            })
            .catch(error => this.handleError(error));        
    }


    onSelectTab(args: any) {
        if (args.heading == 'geolocalización') {
            this.map.refresh();
        }
    }


    onCancelingEdit() {
        this.location.back();
    }

}