﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, Output, EventEmitter  } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { SucursalService} from '../services/sucursal.service';
import { SucursalInfo, SucursalItem} from '../models/sucursal';

@Component({    
    selector: 'distribucion-sucursales-manager',
    templateUrl: './sucursales.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SucursalesManager)
        },
        {
            provide: CatalogService, useExisting: SucursalService
        }
	]
})
export class SucursalesManager extends CatalogManager {
    @Input() sucursales: SucursalItem[] = [];
    @Input() idEmpresa: number;
    @Output() refreshListParent: EventEmitter<void> = new EventEmitter<void>(); 
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private sucursalService: SucursalService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
  //      this.manager.serverSide = options.busquedasServidor;
  //      this.manager.pageSize = options.tamanioPagina;        
  //      this.refreshList();
		//this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        //this.context.app.showSpinner();

        //this.sucursalService.search(this.context.app.defaultSite.identity, request)
        //    .then(response => {
        //        this.context.app.hideSpinner();
        //        //
        //        this.sucursales = response.rows;
        //        this.totalRows = response.totalRows;
        //        this.pageIndex = request.pageIndex;                                
        //    })
        //    .catch(error => this.handleError(error));
    }

    onAddingItem() {
        //this.router.navigate([0], { relativeTo: this.route });
        this.router.navigate(['/distribution/sucursales-distribucion', this.process.id, this.idEmpresa, 0]);
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        //this.router.navigate([item.id], { relativeTo: this.route });
        this.router.navigate(['/distribution/sucursales-distribucion', this.process.id, this.idEmpresa, item.id]);
    }

    onDeletingItem(item: CatalogItem) {
        this.sucursalService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        //this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
        this.router.navigate(['/distribution/sucursales-distribucion', this.process.id, this.idEmpresa, 0, { cloneId: item.id }]);
    }

    onChangingItemState(item: CatalogItem) {
        this.sucursalService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
