﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Features
import { TripulanteTipoService} from '../services/tripulante-tipo.service';
import { TripulanteTipoInfo, TripulanteTipoItem} from '../models/tripulante-tipo';

@Component({
    selector: 'distribucion-tripulante-tipo-editor',
    templateUrl: './tripulantes-tipos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TripulantesTiposEditor)
        },
        {
            provide: CatalogService, useExisting: TripulanteTipoService
        }
    ]
})
export class TripulantesTiposEditor extends CatalogEditor {
    tripulanteTipo: TripulanteTipoInfo = new TripulanteTipoInfo();
    bPreventa: boolean;
    bReparto: boolean;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private tripulanteTipoService: TripulanteTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'bPreventa': [0],
            'bReparto': [0],
            'idPuesto':[0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.bReparto = true;
        this.bPreventa = false;
        this.tripulanteTipo.tipoTripulacion = 'R';

        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.tripulanteTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tripulanteTipo = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.tripulanteTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tripulanteTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.tripulanteTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                if (info.tipoTripulacion == 'P') {
                    this.bPreventa = true;
                    this.bReparto = false;
                }
                else {
                    this.bPreventa = false;
                    this.bReparto = true;
                }
                this.tripulanteTipo = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.tripulanteTipoService.update(this.tripulanteTipo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.tripulanteTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.tripulanteTipo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onChangePreventa(args: any) {
        this.bReparto = false;
        this.tripulanteTipo.tipoTripulacion = 'P';
        Control.invalidateAll();
    }

    onChangeReparto(args: any) {
        this.bPreventa = false;
        this.tripulanteTipo.tipoTripulacion = 'R';
        Control.invalidateAll();
    }

}