﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Wijmo
import { Control, CollectionView } from 'wijmo/wijmo';
//Features
import { ValeTipoService } from '../services/vale-tipo.service';
import { ValeTipoInfo, ValeTipoItem, CelulaValeTipo, ClienteValeTipo, ProductoValeTipo, EstVenValeTipo, RutaValeTipo, ProrrogaValeTipo } from '../models/vale-tipo';

import { CelulasSelectorComponent } from '../controls/celula-selector.component';
import { CelulaInfo } from '../models/celula';

import { ClienteSelectorComponent } from '../controls/cliente-selector.component';
import { ClienteInfo } from '../models/cliente';

import { ProductosSelectorComponent } from '../controls/producto-selector.component';
import { ProductosInfo } from '../models/productos';

import { EstVenSelectorComponent } from '../controls/est-ven-selector.component';
import { EstVenInfo } from '../models/est-ven';

import { RutasRepartoSelectorComponent } from '../controls/ruta-reparto-selector.component';
import { RutasPreventaSelectorComponent } from '../controls/ruta-preventa-selector.component';
import { RutasRepartoInfo } from '../models/rutas-reparto';
import { RutasPreventaInfo } from '../models/rutas-preventa';

import { TipoConteoService } from '../services/tipo-conteo.service';
import { TipoConteoInfo, TipoConteoItem } from '../models/tipo-conteo';

@Component({
    selector: 'distribucion-vale-tipo-editor',
    templateUrl: './vales-tipos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ValesTiposEditor)
        },
        {
            provide: CatalogService, useExisting: ValeTipoService
        }
    ]
})
export class ValesTiposEditor extends CatalogEditor {
    valeTipo: ValeTipoInfo = new ValeTipoInfo();        
    fechaInicio: Date;
    fechaFin: Date;
    fechaVigencia: Date;
    vigenciaDias: boolean;
    vigenciaRango: boolean;
    vigenciaFecha: boolean;
    rdReparto: boolean;
    rdPreventa: boolean;
    diasProrroga: number;
    autorizacion: boolean;
    orden: number;
    tiposConteo: TipoConteoItem[] = [];


    //celulasCollection: CollectionView = new CollectionView();
    celulasCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorCelula') selectorCelula: CelulasSelectorComponent;

    clientesCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorCliente') selectorCliente: ClienteSelectorComponent;

    productosCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorProducto') selectorProducto: ProductosSelectorComponent;

    estVenCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorEstVen') selectorEstVen: EstVenSelectorComponent;

    rutaCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorRutaReparto') selectorRutaReparto: RutasRepartoSelectorComponent;
    @ViewChild('selectorRutaPreventa') selectorRutaPreventa: RutasPreventaSelectorComponent;

    prorrogasCollection: TrackableCollection = new TrackableCollection();

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private valeTipoService: ValeTipoService, private tipoConteoService: TipoConteoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cajasMinimo': [0],
            'cajasMaximo': [0],
            'respetarCredito': [0],
            'respetarDescuento': [0],
            'capturaRuta': [0],
            'pagosParcial': [0],
            'restriccionDia': [0],
            'restriccionRuta': [0],
            'restriccionCliente': [0],
            'restriccionProducto': [0],
            'restriccionEstVen': [0],
            'lunes': [0],
            'martes': [0],
            'miercoles': [0],
            'jueves': [0],
            'viernes': [0],
            'sabado': [0],
            'domingo': [0],
            'diasVigencia': [0],
            'fechaInicio': [new Date],
            'fechaFin': [new Date],
            'fechaVigencia': [new Date],
            'vigenciaDias':[true],
            'vigenciaRango':[false],
            'vigenciaFecha': [false],
            'idCliente': [0],
            'idCelula': [0],
            'idProducto': [0],
            'idEstVen': [0],
            'idRutaReparto': [0],
            'idRutaPreventa': [0],
            'rdReparto': [0],
            'rdPreventa': [0],
            'diasProrroga': [0],
            'autorizacion': [false],
            'orden': [0],
            'idTipoConteo': [0],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.rdReparto = true;
        this.rdPreventa = false;
        this.selectorRutaReparto.enabled = true;
        this.selectorRutaPreventa.enabled = false;

        this.loadTiposConteo();

        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.valeTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.valeTipo = info;
                this.fechaVigencia = info.fechaVigencia;
                this.fechaInicio = info.fechaInicio;
                this.fechaFin = info.fechaFin;

                this.celulasCollection.sourceCollection = info.celulas;
                this.celulasCollection.refresh;

                this.clientesCollection.sourceCollection = info.clientes;
                this.clientesCollection.refresh;

                this.productosCollection.sourceCollection = info.productos;
                this.productosCollection.refresh;

                this.estVenCollection.sourceCollection = info.estructurasDeVenta;
                this.estVenCollection.refresh;

                this.rutaCollection.sourceCollection = info.rutas;
                this.rutaCollection.refresh;

                this.prorrogasCollection.sourceCollection = info.prorrogas;
                this.prorrogasCollection.refresh

                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.valeTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.valeTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.valeTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.fechaVigencia = info.fechaVigencia;
                this.fechaInicio = info.fechaInicio;
                this.fechaFin = info.fechaFin;
                this.vigenciaRango = (info.tipoVigencia == "P");
                this.vigenciaFecha = (info.tipoVigencia == "F");
                this.vigenciaDias = (info.tipoVigencia == "D");
                this.valeTipo = info;   

                this.celulasCollection.sourceCollection = info.celulas;
                this.celulasCollection.refresh;

                this.clientesCollection.sourceCollection = info.clientes;
                this.clientesCollection.refresh;

                this.productosCollection.sourceCollection = info.productos;
                this.productosCollection.refresh;

                this.estVenCollection.sourceCollection = info.estructurasDeVenta;
                this.estVenCollection.refresh;

                this.rutaCollection.sourceCollection = info.rutas;
                this.rutaCollection.refresh;

                this.prorrogasCollection.sourceCollection = info.prorrogas;
                this.prorrogasCollection.refresh

            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.valeTipo.fechaFin = this.fechaFin;
        this.valeTipo.fechaInicio = this.fechaInicio;
        this.valeTipo.fechaVigencia = this.fechaVigencia;
        if (this.vigenciaDias) this.valeTipo.tipoVigencia = "D";
        if (this.vigenciaFecha) this.valeTipo.tipoVigencia = "F";
        if (this.vigenciaRango) this.valeTipo.tipoVigencia = "P";
        this.valeTipoService.update(this.valeTipo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.valeTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.valeTipo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onChangeRango(args: any) {
        this.vigenciaDias = false;
        this.vigenciaFecha = false;
        Control.invalidateAll();
    }
    onChangeDias(args: any) {
        this.vigenciaRango = false;
        this.vigenciaFecha = false;
        Control.invalidateAll();
    }
    onChangeFecha(args: any) {
        this.vigenciaDias = false;
        this.vigenciaRango = false;
        Control.invalidateAll();
    }

    onChangeReparto(args: any) {
        this.rdPreventa = false;
        this.selectorRutaReparto.enabled = true;
        this.selectorRutaPreventa.enabled = false;
    }

    onChangePreventa(args: any) {
        this.rdReparto = false;
        this.selectorRutaReparto.enabled = false;
        this.selectorRutaPreventa.enabled = true;
    }


    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD-CELULA') {
            var celulaValeTipoNew: CelulaValeTipo = new CelulaValeTipo();
            var celInf: CelulaInfo = new CelulaInfo();

            celulaValeTipoNew.idCelula = this.selectorCelula.selectedItem.identity;
            celulaValeTipoNew.nombre = this.selectorCelula.selectedItem.nombre;
            celulaValeTipoNew.codigo = this.selectorCelula.selectedItem.code;

            var celulaValeTipoSlice = this.valeTipo.celulas.slice();

            celulaValeTipoSlice.push(celulaValeTipoNew);

            this.valeTipo.celulas = celulaValeTipoSlice;

            this.celulasCollection.sourceCollection = celulaValeTipoSlice;
            this.celulasCollection.refresh;
        }
        if (button.name == 'ADD-CLIENTE') {
            var clienteValeTipoNew: ClienteValeTipo = new ClienteValeTipo();
            var cliInf: ClienteInfo = new ClienteInfo();

            clienteValeTipoNew.idCliente = this.selectorCliente.selectedItem.identity;
            clienteValeTipoNew.nombre = this.selectorCliente.selectedItem.clienteVentas.nombre;
            clienteValeTipoNew.codigo = this.selectorCliente.selectedItem.code;

            var clienteValeTipoSlice = this.valeTipo.clientes.slice();

            clienteValeTipoSlice.push(clienteValeTipoNew);

            this.valeTipo.clientes = clienteValeTipoSlice;

            this.clientesCollection.sourceCollection = clienteValeTipoSlice;
            this.clientesCollection.refresh;
        }

        if (button.name == 'ADD-PRODUCTO') {
            var productoValeTipoNew: ProductoValeTipo = new ProductoValeTipo();
            var proInf: ProductosInfo = new ProductosInfo();

            productoValeTipoNew.idProducto = this.selectorProducto.selectedItem.identity;
            productoValeTipoNew.nombre = this.selectorProducto.selectedItem.articulo.nombre;
            productoValeTipoNew.codigo = this.selectorProducto.selectedItem.code;

            var productoValeTipoSlice = this.valeTipo.productos.slice();

            productoValeTipoSlice.push(productoValeTipoNew);

            this.valeTipo.productos = productoValeTipoSlice;

            this.productosCollection.sourceCollection = productoValeTipoSlice;
            this.productosCollection.refresh;
        }

        if (button.name == 'ADD-EST-VEN') {
            var estVenValeTipoNew: EstVenValeTipo = new EstVenValeTipo();
            var estVenInf: EstVenInfo = new EstVenInfo();

            estVenValeTipoNew.idEstVen = this.selectorEstVen.selectedItem.identity;
            estVenValeTipoNew.nombre = this.selectorEstVen.selectedItem.nombre;
            estVenValeTipoNew.codigo = this.selectorEstVen.selectedItem.code;

            var estVenValeTipoSlice = this.valeTipo.estructurasDeVenta.slice();

            estVenValeTipoSlice.push(estVenValeTipoNew);

            this.valeTipo.estructurasDeVenta = estVenValeTipoSlice;

            this.estVenCollection.sourceCollection = estVenValeTipoSlice;
            this.estVenCollection.refresh;
        }

        if (button.name == 'ADD-RUTA') {
            var rutaValeTipoNew: RutaValeTipo = new RutaValeTipo();
            var rutaRepVenInf: RutasRepartoInfo = new RutasRepartoInfo();
            var rutaPreVenInf: RutasPreventaInfo = new RutasPreventaInfo();

            if (this.rdReparto) {
                rutaValeTipoNew.idRuta = this.selectorRutaReparto.selectedItem.identity;
                rutaValeTipoNew.codigo = this.selectorRutaReparto.selectedItem.code;
                rutaValeTipoNew.tipoRuta = 'R';
                rutaValeTipoNew.tipoRutaDesc = 'Reparto';
            }
            else
            {
                rutaValeTipoNew.idRuta = this.selectorRutaPreventa.selectedItem.identity;
                rutaValeTipoNew.codigo = this.selectorRutaPreventa.selectedItem.code;
                rutaValeTipoNew.tipoRuta = 'P';
                rutaValeTipoNew.tipoRutaDesc = 'Preventa';
            }

            var rutaValeTipoSlice = this.valeTipo.rutas.slice();

            rutaValeTipoSlice.push(rutaValeTipoNew);

            this.valeTipo.rutas = rutaValeTipoSlice;

            this.rutaCollection.sourceCollection = rutaValeTipoSlice;
            this.rutaCollection.refresh;
        }

        if (button.name == 'ADD-PRORROGA') {
            var prorrogaValeTipoNew: ProrrogaValeTipo = new ProrrogaValeTipo();

            prorrogaValeTipoNew.autorizacion = this.autorizacion;
            prorrogaValeTipoNew.diasProrroga = this.diasProrroga;
            prorrogaValeTipoNew.orden = this.orden;

            var prorrogaValeTipoSlice = this.valeTipo.prorrogas.slice();

            prorrogaValeTipoSlice.push(prorrogaValeTipoNew);

            this.valeTipo.prorrogas = prorrogaValeTipoSlice;

            this.prorrogasCollection.sourceCollection = prorrogaValeTipoSlice;
            this.prorrogasCollection.refresh;
        }

    }

    onRemoveCelula(item) {
        this.celulasCollection.remove(item);
        this.celulasCollection.refresh();
    }

    onRemoveCliente(item) {
        this.clientesCollection.remove(item);
        this.clientesCollection.refresh();
    }

    onRemoveProducto(item) {
        this.productosCollection.remove(item);
        this.productosCollection.refresh();
    }

    onRemoveEstVen(item) {
        this.estVenCollection.remove(item);
        this.estVenCollection.refresh();
    }

    onRemoveRuta(item) {
        this.rutaCollection.remove(item);
        this.rutaCollection.refresh();
    }

    onRemoveProrroga(item) {
        this.prorrogasCollection.remove(item);
        this.prorrogasCollection.refresh();
    }

    loadTiposConteo() {
        this.tipoConteoService.getTiposConteoCbo(this.context.app.defaultSite.identity)
            .then(tipos => this.tiposConteo = tipos)
            .catch(error => {
                this.handleError(error)
            });
    }

}