﻿//Angular
import { Component, forwardRef, ViewChild, ChangeDetectorRef, Input, Output, EventEmitter } from '@angular/core';
import { Route, Router, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Framework 3
import { BaseComponent, CatalogService, Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor } from 'framework3/editors';
//Features
import { DotacionesInfo } from '../models/rutas-reparto';
import { ArticuloStockInfo } from '../../inventory/models/articulo-stock';
import { ArticuloStockSelectorComponent } from '../../inventory/controls/articulo-stock-selector.component';

@Component({
    selector: 'distribution-agregar-dotacion',
    templateUrl: './agregar-dotacion.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AgregarDotacionesComponent)
        }
    ]
})

export class AgregarDotacionesComponent extends CatalogEditor {
    //variables
    editorForm: FormGroup;

    //Inputs
    @Input() data: DotacionesInfo = new DotacionesInfo();
    //Outputs
    @Output() dataUpdater: EventEmitter<DotacionesInfo> = new EventEmitter<DotacionesInfo>();

    //viewChilds
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild('articuloStockSelector') articuloStockSelectorComponent: ArticuloStockSelectorComponent;

    constructor(protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetectorRef: ChangeDetectorRef) {
        super(context, router, route, changeDetectorRef);
        this.editorForm = formBuilder.group({
            'idArticuloStock': ['', AztecaValidators.selected],
            'cantidad': [0, Validators.min(0)]
        });
    }

    show() {
        this.data = new DotacionesInfo();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => this.editorForm.reset());
        Control.invalidateAll();
    }

    onUpdate() {
        if (this.data.cantidad == null) {
            this.data.cantidad = 0;
        }
        this.dataUpdater.emit(this.data);
        this.dialog.hide();
    }

    onItemChanged(args: ArticuloStockInfo) {
        if (args != null) {
            this.data.articuloStock = args;
        }
    }

}