﻿//Framework
import { Component, Input, Output, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import * as moment from 'moment';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Feature
import { AztecaValidators, AlertsBarComponent, ToolbarButton } from 'framework3/controls';
import { Context } from 'framework3/core';
import { ClienteService } from '../services/cliente.service';

@Component({
    selector: 'distribution-cancelar-facturas',
    templateUrl: './cancelar-facturas.component.html'
})

export class CancelarFacturasComponent {
    form: FormGroup;
    tipos: { id: string, nombre: string }[] = [];

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private clienteService: ClienteService, private router: Router, private route: ActivatedRoute, private context: Context) {
        this.form = this.formBuilder.group({
            'idSucursal': [0, Validators.required],
            'fechaBusqueda': []
        });
    }

    cancelarFacturas() {
        this.context.app.showSpinner();
        let fecha = moment(this.form.controls.fechaBusqueda.value).startOf('day').toDate();
        this.clienteService.cancelarFacturas(fecha, this.context.app.defaultSite.identity)
            .then(r => {
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        console.error(error);
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
        this.context.app.hideSpinner();
    }


    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }


}