﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo } from 'framework3/core';
import { ProductosListComponent } from './dialogs/productos-list';

import { GamaService } from '../services/gama.service';
import { ProductosItem } from '../models/productos';

@Component({
    selector: 'distribucion-gama-productos',
    templateUrl: './gama-productos.component.html'
})
export class GamaProductosComponent extends BaseComponent {
    productos: ProductosItem[];
    idGama: number;
    currentItem: CatalogItem;
    /*get title(): string {
        return 'Miembros del grupo - ' + this.grupo.nombre;
    }*/
    //
    @ViewChild(ProductosListComponent) productosDialog: ProductosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private gamaService: GamaService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idGama = +param['idGama'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadArea());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadArea() {
        this.gamaService.LoadProductosList(this.idGama)
            .then(result => this.productos = result)
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.gamaService.LoadProductosList(this.idGama)
            .then(result => {
                this.productos = result;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.productosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }


    onSelectedProdChanged(newUsers: ProductosItem[]) {
        this.gamaService.addMembers(this.idGama, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser() {
        this.context.app.showSpinner();
        this.gamaService.removeMember(this.idGama, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.productos = this.productos.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}