﻿//Framework
import { Component, Input, Output, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import * as moment from 'moment';
//Wijmo
import { Control, CollectionView } from 'wijmo/wijmo';
//Feature
import { AztecaValidators, AlertsBarComponent, ToolbarButton } from 'framework3/controls';
import { Context } from 'framework3/core';
import { PrinterService } from 'framework3/printing';
import { ClienteService } from '../services/cliente.service';
import { LoteImpresionCfdiItem } from '../models/lote-impresion-cfdi';
import { FacturaImpresionItem } from '../models/factura';
import { ComprobanteFiscalService } from '../../accounting/services/comprobante-fiscal.service';
import { TabsetComponent } from 'ngx-bootstrap';
import { FlexGrid } from 'wijmo/wijmo.grid';

@Component({
    selector: 'distribution-impresion-facturas',
    templateUrl: './impresion-facturas.component.html'
})
export class ImpresionFacturasComponent {
    form: FormGroup;
    tipos: { id: string, nombre: string }[] = [];
    comprobantesCollection: CollectionView = new CollectionView();
    facturasCollection: CollectionView = new CollectionView();
    comprobantesList: LoteImpresionCfdiItem[] = [];
    facturasList: FacturaImpresionItem[] = [];

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;
    @ViewChild('tabDirective') tabDirective: TabsetComponent;

    constructor(private formBuilder: FormBuilder, private clienteService: ClienteService, private router: Router, private route: ActivatedRoute, private context: Context,
        private printerService: PrinterService, private comprobanteFiscalService: ComprobanteFiscalService) {
        this.form = this.formBuilder.group({
            'tipos': ['', Validators.required],
            'fechaComprobantes': [],
            'fechaBusquedaLotes': [],
            'fechaBusquedaFacturas': []
        });

        this.tipos =
            [
                { "id": 'C', "nombre": "Credito" },
                { "id": 'E', "nombre": "Contado" }
            ];
    }

    generarLote() {
        this.context.app.showSpinner();
        let fecha = moment(this.form.controls.fechaComprobantes.value).startOf('day').toDate();
        this.clienteService.impresionLoteCFDI(fecha, this.context.app.defaultSite.identity, '', this.form.controls.tipos.value)
            .then(r => {
                this.context.app.hideSpinner();
                this.alertsBar.showAlert('Se ha generado el lote de impresión con exíto', 'success');
                this.tabDirective.tabs[1].active = true;
            })
            .catch(error => this.handleError(error));
    }

    busquedaComprobantes() {
        this.context.app.showSpinner();
        this.comprobantesCollection.sourceCollection = [];
        this.comprobantesCollection.refresh();
        let fechaBusqueda = moment(this.form.controls.fechaBusquedaLotes.value).startOf('day').toDate();
        this.clienteService.loadLotesImpresionCFDI(this.context.app.defaultSite.identity, fechaBusqueda)
            .then(comprobantes => {
                this.context.app.hideSpinner();
                comprobantes.forEach(x => x.selected = false);
                this.comprobantesList = comprobantes;
                this.comprobantesCollection.sourceCollection = this.comprobantesList;
            })
            .catch(error => this.handleError(error))
    }

    busquedaFacturas() {
        this.context.app.showSpinner();
        this.facturasCollection.sourceCollection = [];
        this.facturasCollection.refresh();
        let fechaIni = moment(this.form.controls.fechaBusquedaFacturas.value).startOf('day').toDate();
        let fechaFin = moment(this.form.controls.fechaBusquedaFacturas.value).endOf('day').toDate();
        this.clienteService.getFacturasPeriodo(this.context.app.defaultSite.identity, fechaIni, fechaFin)
            .then(facturas => {
                this.context.app.hideSpinner();
                facturas.forEach(x => x.selected = false);
                this.facturasList = facturas;
                this.facturasCollection.sourceCollection = this.facturasList;
            })
            .catch(error => this.handleError(error))
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'PRINTLOTES') {
            var lotes: Array<number> = [];
            this.comprobantesList.filter(x => x.selected).forEach(x => {
                lotes.push(x.id);
            })

            this.context.app.showSpinner();
            this.clienteService.getArchivosLoteImpresionMultiple(lotes)
                .then(archivos => {
                    archivos.forEach(archivo => {
                        this.printerService.printPdfFile('/file-server/' + archivo);
                    });
                    this.alertsBar.showAlert('Se ha enviado la impresión con exíto', 'success');
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error))
        }
        if (button.name == 'SEARCHLOTES') {
            this.busquedaComprobantes();
        }
        if (button.name == 'PRINTFACTURAS') {
            //this.facturasList.filter(x => x.selected).forEach(x => {
            //    this.context.app.showSpinner();

            //    this.comprobanteFiscalService.generarPdfTemporal(x.id.toString())
            //        .then(file => {
            //            this.printerService.printPdfFile('/file-server/' + file.uri);
            //            this.context.app.hideSpinner();
            //        })
            //        .catch(error => this.handleError(error));

            //})

            //Caso 1125 solo se afecta a los items que se estan viendo en el grid
            var i = 0;
            var comprobantes: Array<string> = []
            for (i = 0; i <= this.facturasCollection.totalItemCount - 1; i++) {
                if (this.facturasCollection._view[i].selected) {
                    comprobantes.push(this.facturasCollection._view[i].id.toString());
                }
            }

            this.context.app.showSpinner();
            this.comprobanteFiscalService.generarPdfTemporalMultiple(comprobantes)
                .then(file => {
                    //this.printerService.printPdfFile('/file-server/' + file.uri);
                    file.forEach(x => {
                        this.printerService.printPdfFile('/file-server/' + x);
                    })
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        if (button.name == 'INVERTIRSELECCION') {
            //this.facturasList.forEach(x => x.selected = !x.selected);
            //his.facturasCollection.sourceCollection = this.facturasList;

            //Caso 1125 solo se afecta a los items que se estan viendo en el grid
            var i = 0;
            for (i = 0; i <= this.facturasCollection.totalItemCount - 1; i++) {
                this.facturasCollection._view[i].selected = !this.facturasCollection._view[i].selected
            }
            
            Control.invalidateAll();
        }
        if (button.name == 'SEARCHFACTURAS') {
            this.busquedaFacturas();
        }
    }

    selectedCell(lote: LoteImpresionCfdiItem, selected: boolean) {
        this.comprobantesList.forEach(x => {
            if (x.id == lote.id) {
                if (selected == true) {
                    x.selected = true;
                }
                else {
                    x.selected = false;
                }
            }
        })
    }

    selectedCellFacturas(facturas: FacturaImpresionItem, selected: boolean) {
        this.facturasList.forEach(x => {
            if (x.id == facturas.id) {
                if (selected == true) {
                    x.selected = true;
                }
                else {
                    x.selected = false;
                }
            }
        })
    }

    handleError(error: any) {
        console.error(error);
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
        this.context.app.hideSpinner();
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }
}