﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo } from 'framework3/core';
import { CelulasListComponent } from './dialogs/celulas-list';

import { RutasPreventaService } from '../services/rutas-preventa.service';
import { CelulaItem } from '../models/celula';

@Component({
    selector: 'distribucion-rutaspreventa-celulas',
    templateUrl: './rutaspreventa-celulas.component.html'
})
export class RutasPreventaCelulasComponent extends BaseComponent {
    celulas: CelulaItem[];
    idRutaPreventa: number;
    currentItem: CatalogItem;

    @ViewChild(CelulasListComponent) CelulasDialog: CelulasListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private rutasPreventaService: RutasPreventaService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idRutaPreventa = +param['idRutaPreventa'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadList());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadList() {
        this.rutasPreventaService.LoadCelulasList(this.idRutaPreventa)
            .then(result => this.celulas = result)
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.rutasPreventaService.LoadCelulasList(this.idRutaPreventa)
            .then(result => {
                this.celulas = result;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.CelulasDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }


    onSelectedCelulaChanged(celulas: CelulaItem[]) {
        this.rutasPreventaService.addMembers(this.idRutaPreventa, celulas)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Las celulas han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveItem() {
        this.context.app.showSpinner();
        this.rutasPreventaService.removeMember(this.idRutaPreventa, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.celulas = this.celulas.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}