﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { CelulaInfo, CelulaItem } from '../models/celula';
import { CelulaService } from '../services/celula.service';


@Component({
    selector: 'distribution-celula-selector',
    templateUrl: './celula-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => CelulasSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => CelulasSelectorComponent),
        multi: true
    }]

})
export class CelulasSelectorComponent extends CatalogSelector {
    celula: any[];
    selectedItem: CelulaInfo = new CelulaInfo;
    //
    @Output() selectedItemChanged: EventEmitter<CelulaInfo> = new EventEmitter<CelulaInfo>();

    constructor(protected context: Context, private celulaService: CelulaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findCelulas(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.celulaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.celulaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.celula = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findCelulas(keyword: string): Observable<CatalogItem[]> {
        return this.celulaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

