﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { CentrosInfo, CentrosItem } from '../models/centros';
import { CentrosService } from '../services/centros.service';


@Component({
    selector: 'distribution-centro-selector',
    templateUrl: './centro-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => CentrosSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => CentrosSelectorComponent),
        multi: true
    }]

})
export class CentrosSelectorComponent extends CatalogSelector {
    centro: any[];
    selectedItem: CentrosInfo = new CentrosInfo;
    //
    @Output() selectedItemChanged: EventEmitter<CentrosInfo> = new EventEmitter<CentrosInfo>();

    constructor(protected context: Context, private centrosService: CentrosService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findCentros(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.centrosService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.centrosService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.centro = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findCentros(keyword: string): Observable<CatalogItem[]> {
        return this.centrosService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

