﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { ClienteInfo, ClienteItem } from '../models/cliente';
import { ClienteService } from '../services/cliente.service';


@Component({
    selector: 'distribution-cliente-selector',
    templateUrl: './cliente-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ClienteSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ClienteSelectorComponent),
        multi: true
    }]

})
export class ClienteSelectorComponent extends CatalogSelector {
    cliente: ClienteItem[];
    selectedItem: ClienteInfo = new ClienteInfo;
    //
    @Output() selectedItemChanged: EventEmitter<ClienteInfo> = new EventEmitter<ClienteInfo>();
    @Input() allBranchs: boolean = false;

    constructor(protected context: Context, private clienteService: ClienteService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findCliente(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.clienteService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + '-' + info.clienteVentas.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        //Agregar aqui filtros
        if (this.allBranchs == false) {
            let condicion: SearchCondition = { field: "idSucursal", operator: "=", valueType: "int", value: this.context.app.defaultSite.identity };
            request.conditions.push(condicion);
        }

        this.clienteService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.cliente = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findCliente(keyword: string): Observable<CatalogItem[]> {
        return this.clienteService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

