﻿import { Component, Input, Output, ViewChild, ContentChildren, OnInit, EventEmitter, QueryList, forwardRef, Optional, SkipSelf } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SearchCondition, SortOrder, HierarchySelector, ITreeHost } from 'framework3/search/index'; //'../../search/index';
//
//
//
import { EstVenInfo, EstVenItem } from '../models/est-ven';
import { EstVenService } from '../services/est-ven.service';

@Component({
    selector: 'distribution-est-ven-selector',
    templateUrl: './est-ven-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => EstVenSelectorComponent)
    },
    {
        provide: ITreeHost,
        useExisting: forwardRef(() => EstVenSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => EstVenSelectorComponent),
        multi: true
    }]
})
export class EstVenSelectorComponent extends HierarchySelector implements ITreeHost {
    estVenArray: EstVenItem[];
    nodes: EstVenInfo[];
    selectedItem: EstVenInfo = null;
    //Inputs
    @Input() idDocumento: number = 0;
    @Input() version: number = 0;
    @Input() idPadre: number = 0;
    //Outputs
    @Output() selectedItemChanged: EventEmitter<EstVenInfo> = new EventEmitter<EstVenInfo>();

    constructor(protected context: Context, private estVenService: EstVenService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        }).pipe(mergeMap((keyword: string) => this.findKeyword(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.estVenService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + "-" + info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = '';
            this.selectedItemChanged.emit(this.selectedItem);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.estVenService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.estVenArray = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findKeyword(keyword: string): Observable<CatalogItem[]> {
        return this.estVenService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

    onLoadingHierarchy() {
        //Cargar nodos raíz al abrir diálogo de jerarquía
        this.estVenService.loadChildren(this.context.app.defaultCompany.identity, 0)
            .then(rootNodes => this.nodes = rootNodes)
            .catch(error => this.handleError(error));
    }


    loadChildren(parent: EstVenInfo): Promise<EstVenInfo[]> {
        return this.estVenService.loadChildren(this.context.app.defaultCompany.identity, parent.identity);
    }


}