﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { GamaInfo, GamaItem } from '../models/gama';
import { GamaService } from '../services/gama.service';


@Component({
    selector: 'distribution-gama-selector',
    templateUrl: './gama-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => GamasSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => GamasSelectorComponent),
        multi: true
    }]

})
export class GamasSelectorComponent extends CatalogSelector {
    gama: any[];
    selectedItem: GamaInfo = new GamaInfo;
    //
    @Output() selectedItemChanged: EventEmitter<GamaInfo> = new EventEmitter<GamaInfo>();

    constructor(protected context: Context, private gamaService: GamaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findGamas(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.gamaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.gamaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.gama = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findGamas(keyword: string): Observable<CatalogItem[]> {
        return this.gamaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

