﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { RutasPreventaInfo, RutasPreventaItem } from '../models/rutas-preventa';
import { RutasPreventaService } from '../services/rutas-preventa.service';

@Component({
    selector: 'distribution-ruta-preventa-selector',
    templateUrl: './ruta-preventa-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => RutasPreventaSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => RutasPreventaSelectorComponent),
        multi: true
    }]
})
export class RutasPreventaSelectorComponent extends CatalogSelector {
    rutaPreventa: any[];
    selectedItem: RutasPreventaInfo = new RutasPreventaInfo;
    //
    @Output() selectedItemChanged: EventEmitter<RutasPreventaInfo> = new EventEmitter<RutasPreventaInfo>();

    constructor(protected context: Context, private rutasPreventaService: RutasPreventaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findRutasPreventa(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.rutasPreventaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.rutasPreventaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.rutaPreventa = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findRutasPreventa(keyword: string): Observable<CatalogItem[]> {
        return this.rutasPreventaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}
