﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { RutasRepartoInfo, RutasRepartoItem } from '../models/rutas-reparto';
import { RutasRepartoService } from '../services/rutas-reparto.service';


@Component({
    selector: 'distribution-ruta-reparto-selector',
    templateUrl: './ruta-reparto-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => RutasRepartoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => RutasRepartoSelectorComponent),
        multi: true
    }]

})
export class RutasRepartoSelectorComponent extends CatalogSelector {
    rutaReparto: any[];
    selectedItem: RutasRepartoInfo = new RutasRepartoInfo;
    //
    @Output() selectedItemChanged: EventEmitter<RutasRepartoInfo> = new EventEmitter<RutasRepartoInfo>();

    constructor(protected context: Context, private rutasRepartoService: RutasRepartoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findRutasReparto(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.rutasRepartoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.rutasRepartoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.rutaReparto = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findRutasReparto(keyword: string): Observable<CatalogItem[]> {
        return this.rutasRepartoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

