﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { SegmentosInfo, SegmentosItem } from '../models/segmentos';
import { SegmentosService } from '../services/segmentos.service';


@Component({
    selector: 'distribution-segmento-selector',
    templateUrl: './segmento-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => SegmentosSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => SegmentosSelectorComponent),
        multi: true
    }]

})
export class SegmentosSelectorComponent extends CatalogSelector {
    segmento: any[];
    selectedItem: SegmentosInfo = new SegmentosInfo;
    //
    @Output() selectedItemChanged: EventEmitter<SegmentosInfo> = new EventEmitter<SegmentosInfo>();

    constructor(protected context: Context, private segmentosService: SegmentosService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findSegmentos(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.segmentosService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.segmentosService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.segmento = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findSegmentos(keyword: string): Observable<CatalogItem[]> {
        return this.segmentosService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

