﻿import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

//////////////////////////// Guards //////////////////////////////////
import { AuthGuard } from 'framework3/core';
import { EditorDeactivateGuard } from 'framework3/editors';

//////////////////////////// Catalogos ////////////////////////////////
import { GamasManager } from './catalogs/gamas.manager';
import { GamasEditor } from './catalogs/gamas.editor';
import { GruposBeneficiosManager } from '../distribution/catalogs/grupos-beneficios.manager';
import { GruposBeneficiosEditor } from '../distribution/catalogs/grupos-beneficios.editor';
import { EstVenManager } from '../distribution/catalogs/est-ven.manager';
import { CelulasManager } from '../distribution/catalogs/celulas.manager';
import { CelulasEditor } from '../distribution/catalogs/celulas.editor';
import { ClientesManager } from '../distribution/catalogs/clientes.manager';
import { ClientesEditor } from '../distribution/catalogs/clientes.editor';
import { RutasRepartoManager } from './catalogs/rutas-reparto.manager';
import { RutasRepartoEditor } from './catalogs/rutas-reparto.editor';
import { ProductosManager } from './catalogs/productos.manager';
import { ProductosEditor } from './catalogs/productos.editor';
import { CentrosManager } from './catalogs/centros.manager';
import { CentrosEditor } from './catalogs/centros.editor';
import { RutasPreventaManager } from './catalogs/rutas-preventa.manager';
import { RutasPreventaEditor } from './catalogs/rutas-preventa.editor';
import { SegmentosManager } from './catalogs/segmentos.manager';
import { SegmentosEditor } from './catalogs/segmentos.editor';
import { FormasVentasManager } from './catalogs/formas-ventas.manager';
import { TiposConteosEditor } from './catalogs/tipos-conteos.editor';
import { TiposConteosManager } from './catalogs/tipos-conteos.manager';
import { EmpresasEditor } from './catalogs/empresas.editor';
import { EmpresasManager } from './catalogs/empresas.manager';
import { SucursalesEditor } from './catalogs/sucursales.editor';
import { SucursalesManager } from './catalogs/sucursales.manager';
import { ValesTiposEditor } from './catalogs/vales-tipos.editor';
import { ValesTiposManager } from './catalogs/vales-tipos.manager';
import { TiposCapturasEditor } from './catalogs/tipos-capturas.editor';
import { TiposCapturasManager } from './catalogs/tipos-capturas.manager';
import { TripulantesTiposEditor } from './catalogs/tripulantes-tipos.editor';
import { TripulantesTiposManager } from './catalogs/tripulantes-tipos.manager';
import { FrecuenciasVisitaEditor } from './catalogs/frecuencias-visita.editor'
import { FrecuenciasVisitaManager } from './catalogs/frecuencias-visita.manager';

///////////////////////////// Components /////////////////////////////////
import { GamaProductosComponent } from './components/gama-productos.component';
import { RutasRepartoCelulasComponent } from './components/rutasreparto-celulas.component';
import { RutasPreventaCelulasComponent } from './components/rutaspreventa-celulas.component';
import { ImpresionFacturasComponent } from './components/impresion-facturas.component'
import { CancelarFacturasComponent } from './components/cancelar-facturas.component'

///////////////////////////// Maps /////////////////////////////////////
import { TerritoryPlannerComponent } from './maps/territory-planner.component';

///////////////////////////// Mantenimientos /////////////////////////////////////
import { Conteo } from './maintenances/conteo';
import { ReestructuraDistribucion } from './maintenances/Reestructura.maintenance'; 

///////////////////////////// Document /////////////////////////////////
import { PedidosManager } from './documents/pedidos.manager';
import { PedidosEditor } from './documents/pedidos.editor';

import { CargasFisicasManager } from './documents/cargas-fisicas.manager';
import { CargasFisicasEditor } from './documents/cargas-fisicas.editor';

import { ConteosManager } from '../distribution/documents/conteos.manager';
import { ConteosEditor } from '../distribution/documents/conteos.editor';

import { ValesManager } from './documents/vales.manager';
import { ValesEditor } from './documents/vales.editor';

import { ValesPagosManager } from './documents/vales-pagos.manager';
import { ValesPagosEditor } from './documents/vales-pagos.editor';

import { TripulacionesManager } from './documents/tripulaciones.manager';
import { TripulacionesEditor } from './documents/tripulaciones.editor';
import { NuevaTripulacionComponent } from './documents/parts/nueva-trupulacion.component';
import { NuevoPagoValeComponent } from './documents/parts/nuevo-vale-pago.component';
///////////////////////////// Maintenance /////////////////////////////////
import { CargaFisicaMaintenanceComponent } from './maintenances/carga-fisica.maintenance';
import { CobranzaMaintenanceComponent } from './maintenances/cobranza.maintenance';

///////////////////////////// Parts /////////////////////////////////
import { GamasElementosComponent } from './parts/gamas-elementos.component';
import { FrecuenciasManager } from './catalogs/frecuencias.manager';
import { FrecuenciasEditor } from './catalogs/frecuencias.editor';
import { CelulasAreaComponent } from './parts/celulas-area.component';
import { FrecuenciasAreaComponent } from './parts/frecuencias-area.component';

import { ReestructuraEditor } from './maintenances/reestructura-editor';
import { reestructurafile } from './maintenances/reestructura-file';
import { reestructura_editor_detalle } from './maintenances/reestructura-editor-detalle';

const routes: Routes = [
    {
        path: '',
        canActivateChild: [AuthGuard],
        children: [
            { path: 'gamas/:proceso', component: GamasManager },
            { path: 'gamas/:proceso/:id', component: GamasEditor, canDeactivate: [EditorDeactivateGuard] },            
            { path: 'gamas/elements/:proceso/:id', component: GamasElementosComponent },
            { path: 'grupos-beneficio/:proceso', component: GruposBeneficiosManager },
            { path: 'grupos-beneficio/:proceso/:id', component: GruposBeneficiosEditor, canDeactivate: [EditorDeactivateGuard] },           
            { path: 'est-ven/:proceso', component: EstVenManager },
            { path: 'celulas/:proceso', component: CelulasManager },
            { path: 'celulas/:proceso/:id', component: CelulasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'clientes/:proceso', component: ClientesManager },
            { path: 'clientes/:proceso/:id', component: ClientesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'rutas-reparto/:proceso', component: RutasRepartoManager },
            { path: 'rutas-reparto/:proceso/:id', component: RutasRepartoEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'productos/:proceso', component: ProductosManager },
            { path: 'productos/:proceso/:id', component: ProductosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'gamas-productos/:proceso/:idGama', component: GamaProductosComponent },
            { path: 'centros/:proceso', component: CentrosManager },
            { path: 'centros/:proceso/:id', component: CentrosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'rutasreparto-celulas/:proceso/:idRutaReparto', component: RutasRepartoCelulasComponent },
            { path: 'rutas-preventa/:proceso', component: RutasPreventaManager },
            { path: 'rutas-preventa/:proceso/:id', component: RutasPreventaEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'rutaspreventa-celulas/:proceso/:idRutaPreventa', component: RutasPreventaCelulasComponent },
            { path: 'segmentos/:proceso', component: SegmentosManager },
            { path: 'segmentos/:proceso/:id', component: SegmentosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'impresion-facturas/:proceso', component: ImpresionFacturasComponent },
            { path: 'cancelar-facturas/:proceso', component: CancelarFacturasComponent },
            { path: 'formas-ventas/:proceso', component: FormasVentasManager },
            { path: 'tipos-conteos/:proceso', component: TiposConteosManager },
            { path: 'tipos-conteos/:proceso/:id', component: TiposConteosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'empresas-distribucion/:proceso', component: EmpresasManager },
            { path: 'empresas-distribucion/:proceso/:id', component: EmpresasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'sucursales-distribucion/:proceso', component: SucursalesManager },
            { path: 'sucursales-distribucion/:proceso/:idEmpresa/:id', component: SucursalesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'vales-tipos/:proceso', component: ValesTiposManager },
            { path: 'vales-tipos/:proceso/:id', component: ValesTiposEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'tipos-capturas/:proceso', component: TiposCapturasManager },
            { path: 'tipos-capturas/:proceso/:id', component: TiposCapturasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'tripulantes-tipos/:proceso', component: TripulantesTiposManager },
            { path: 'tripulantes-tipos/:proceso/:id', component: TripulantesTiposEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'frecuencias-visita/:proceso', component: FrecuenciasVisitaManager },
            { path: 'frecuencias-visita/:proceso/:id', component: FrecuenciasVisitaEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'frecuencias/:proceso', component: FrecuenciasManager },
            { path: 'frecuencias/:proceso/:id', component: FrecuenciasEditor, canDeactivate: [EditorDeactivateGuard] },
                        
            { path: 'celulas-load-area/:proceso/:idCelula', component: CelulasAreaComponent },
            { path: 'frecuencia-load-area/:proceso/:idFrecuencia', component: FrecuenciasAreaComponent },
            //
            { path: 'territory-planner/:proceso', component: TerritoryPlannerComponent },

            //Documents
            { path: 'pedidos/:proceso', component: PedidosManager },
            { path: 'pedidos/:proceso/:id', component: PedidosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'conteos/:proceso', component: ConteosManager },
            { path: 'conteos/:proceso/:id', component: ConteosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'cargas-fisicas/:proceso', component: CargasFisicasManager },
            { path: 'cargas-fisicas/:proceso/:id', component: CargasFisicasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'vales/:proceso', component: ValesManager },
            { path: 'vales/:proceso/:id', component: ValesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'tripulaciones/:proceso', component: TripulacionesManager },
            { path: 'tripulaciones/:proceso/new', component: NuevaTripulacionComponent },
            { path: 'tripulaciones/:proceso/:id', component: TripulacionesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'vales-pagos/:proceso', component: ValesPagosManager },
            { path: 'vales-pagos/:proceso/new', component: NuevoPagoValeComponent },
            { path: 'vales-pagos/:proceso/:id', component: ValesPagosEditor, canDeactivate: [EditorDeactivateGuard] },

            // MAINTENANCE
            { path: 'conteo/:proceso', component: Conteo },
            { path: 'carga-fisica/:proceso', component: CargaFisicaMaintenanceComponent },
            { path: 'cobranzas/:proceso', component: CobranzaMaintenanceComponent },
            { path: 'Reestructura/:proceso', component: ReestructuraEditor },
            { path: 'Reestructura/:proceso/new', component: reestructurafile },
            { path: 'Reestructura/:proceso/new/:id', component: reestructura_editor_detalle}

        ]
    },
];



@NgModule({
    imports: [
        RouterModule.forChild(routes)
    ],
    exports: [
        RouterModule
    ]
})
export class DistributionRoutingModule { }

