﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { CargaFisicaInfo, CargaFisicaItem } from '../models/carga-fisica';
import { CargaFisicaDetail } from '../models/carga-fisica-detail';
import { CargaFisicaService } from '../services/carga-fisica.service';
import { CargaFisicaDetailEditor } from './parts/cargas-fisicas-detail.editor';

@Component({
    selector: 'distribution-cargas-fisicas-editor',
    templateUrl: './cargas-fisicas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CargasFisicasEditor)
        },
        {
            provide: DocumentService, useExisting: CargaFisicaService
        }
    ]
})
export class CargasFisicasEditor extends DocumentEditor {
    cargaFisica = new CargaFisicaInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: CargaFisicaDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private cargaFisicaService: CargaFisicaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'sample': ['', Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.cargaFisicaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cargaFisica = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.cargaFisicaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cargaFisica = doc;
                this.detailsEditor.clientIndexStart = this.cargaFisica.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.cargaFisicaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.cargaFisica = doc;
                this.detailsEditor.clientIndexStart = this.cargaFisica.details.length;
                if (this.cargaFisica.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.cargaFisicaService.update(this.cargaFisica)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.cargaFisica.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.cargaFisica = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.cargaFisicaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.cargaFisicaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.cargaFisica.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new CargaFisicaDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: CargaFisicaDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: CargaFisicaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: CargaFisicaDetail[]) {
        this.cargaFisica.details = updatedDetails;
    }

    onSavingDetail(detail: CargaFisicaDetail) {
        this.detailsList.updateItem(detail);
    }

}

