﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder } from '@angular/forms';
//Framework
import { Context } from 'framework3/core';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//Distribucion
import { RutasRepartoSelectorComponent } from '../../controls/ruta-reparto-selector.component';
import { RutasPreventaSelectorComponent } from '../../controls/ruta-preventa-selector.component';
import { RutasRepartoInfo } from '../../models/rutas-reparto';
import { RutasPreventaInfo } from '../../models/rutas-preventa';


@Component({
    selector: 'distribution-nueva-tripulacion',
    templateUrl: './nueva-tripulacion.component.html'
})
export class NuevaTripulacionComponent {
    form: FormGroup;
    idRuta: number;
    idVehiculo: number;
    idDispositivo: number;
    tipoTripulacion: string;
    bPreventa: boolean;
    bReparto: boolean;

    @ViewChild('selectorRutaReparto') selectorRutaReparto: RutasRepartoSelectorComponent;
    @ViewChild('selectorRutaPreventa') selectorRutaPreventa: RutasPreventaSelectorComponent;


    //
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute,
        private context: Context, ) {
        this.form = formBuilder.group({
            'bPreventa': [0],
            'bReparto': [0],
            'idRutaReparto': [0],
            'idRutaPreventa': [0]
        })
    }

    /////////// METHODS

    onChangePreventa(args: any) {
        this.bReparto = false;
        this.tipoTripulacion = 'P';
        this.selectorRutaReparto.enabled = false;
        this.selectorRutaPreventa.enabled = true;
    }

    onChangeReparto(args: any) {
        this.bPreventa = false;
        this.tipoTripulacion = 'R';
        this.selectorRutaReparto.enabled = true;
        this.selectorRutaPreventa.enabled = false;
    }


    /////////// HANDLERS

    onToolbarClick(button: ToolbarButton) {
        if (button.name == 'OK') {
            if (this.tipoTripulacion == 'P') {
                this.idRuta = this.selectorRutaPreventa.selectedItem.identity;
                this.idVehiculo = this.selectorRutaPreventa.selectedItem.idVehiculo;
                this.idDispositivo = this.selectorRutaPreventa.selectedItem.idDispositivoCaptura;
            }
            if (this.tipoTripulacion == 'R') {
                this.idRuta = this.selectorRutaReparto.selectedItem.identity;
                this.idVehiculo = this.selectorRutaReparto.selectedItem.idVehiculo;
                this.idDispositivo = this.selectorRutaReparto.selectedItem.idDispositivoCaptura;
            }

            this.router.navigate(['../', '0', { idRuta: this.idRuta, tipoTripulacion: this.tipoTripulacion, idVehiculo: this.idVehiculo, idDispositivo: this.idDispositivo }], { relativeTo: this.route });
        }
        else if (button.name == 'CANCEL') {
            this.router.navigate(['../'], { relativeTo: this.route });
        }
    }

    handleError(error: any) {
        console.log(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }
}