﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder } from '@angular/forms';
//Framework
import { Context } from 'framework3/core';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';

//Distribucion
import { ValePagoService } from '../../services/vale-pago.service';
import { ValePagoPendiente } from '../../models/vale-pago-pendiente';


@Component({
    selector: 'distribution-nuevo-vale-pago',
    templateUrl: './nuevo-vale-pago.component.html'
})

export class NuevoPagoValeComponent {
    form: FormGroup;
    fechaIni: Date = new Date();
    fechaFin: Date = new Date();
    filtro: string = 'Captura';
    valesPendientes: ValePagoPendiente[] = [];
    previewValesPendientes: CollectionView = new CollectionView();

    //
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute,
        private context: Context,
        private valePagoSvc: ValePagoService) {
        this.form = formBuilder.group({
            'fechaIni': [new Date()],
            'fechaFin': [new Date()]
        })
    }

    /////////// HANDLERS
    onToolbarClick(button: ToolbarButton) {
        if (button.name == 'BUSCAR') {
            this.context.app.showSpinner();
            this.valePagoSvc.getPendientes(this.fechaIni, this.fechaFin, this.filtro, this.context.app.defaultSite.idBranch)
                .then(recs => {
                    this.valesPendientes = recs;
                    this.previewValesPendientes.sourceCollection = this.valesPendientes;
                    this.context.app.hideSpinner();
                })
        }
        else if (button.name == 'SELECCIONAR') {
            var item: ValePagoPendiente = new ValePagoPendiente;
            item = this.previewValesPendientes.currentItem;
            //this.router.navigate(['../', '0', { idValePagar: item.idVale, nombreCliente: item.cliente, fechaCaptura: item.fechaCaptura, fechaCarga: item.fechaCarga, fechaVencimiento: item.fechaVencimiento, rutaReparto: item.rutaReparto, totalCantidad: item.totalCantidad, totalMonto: item.total }], { relativeTo: this.route });
            this.router.navigate(['../', '0', item], { relativeTo: this.route });
        }
        else if (button.name == 'CANCEL') {
            this.router.navigate(['../'], { relativeTo: this.route });
        }
    }


    handleError(error: any) {
        console.log(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }


}