﻿import { Component, Input, Output, EventEmitter, ViewChild, OnInit } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';
//Wijmo
import { Globalize, DateTime } from 'wijmo/wijmo';
//
import * as moment from 'moment';
//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//Locales
import { ValeDetail } from '../../models/vale-detail';
import { ProductosInfo } from '../../models/productos';
import { ProductosSelectorComponent } from '../../controls/producto-selector.component';

//Inventario
import { Empaquetado, UnidadMedidaInfo, UnidadMedidaItem, StockInfo, LoteInfo, SerieInfo, UnidadMedidaService, AlmacenService, MetodoValuacion, QuantityInputComponent, QuantityLabelComponent } from '../../../inventory';
//Sales
import { ArticuloService, SucursalService } from '../../../sales';


@Component({
    selector: 'distribucion-vale-detail-editor',
    templateUrl: './vales-detail.editor.html'
})
export class ValeDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new ValeDetail();
    precioUnitario: string = '$0.0';
    private disponible: string = '0';
    unidadesMedida: UnidadMedidaItem[] = [];
    stockConfig: StockInfo;
    selectedUM = new UnidadMedidaItem();

    idZonaVenta: number = 0;
    @Input() idClienteVentas: number = 0;
    @Input() idAlmacenCarga: number = 0;

    //Objeto original que se editará
    @Input() detail: ValeDetail = new ValeDetail();

    @ViewChild(ProductosSelectorComponent) selProducto: ProductosSelectorComponent;
    @ViewChild(QuantityInputComponent) quantity: QuantityInputComponent;
    @ViewChild(QuantityLabelComponent) quantityLbl: QuantityLabelComponent;


    constructor(protected context: Context, private builder: FormBuilder, private articuloService: ArticuloService, private almacenService: AlmacenService,
        private unidadMedidaService: UnidadMedidaService, private sucursalService: SucursalService) {
        super(context);

        this.form = builder.group({
            'producto': [0, AztecaValidators.selected]

        });
    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////

    ngOnInit() {
        setTimeout(_ => {
            this.loadSucursal();
        });
    }
    loadSucursal() {
        this.sucursalService.getDetailByID(this.context.app.defaultSite.identity)
            .then(suc => {
                this.idZonaVenta = suc.idZonaVenta;
            })
            .catch(error => this.handleError(error));
    }


    onCreatingItem() {
        this.detailCopy = this.detail;
        this.form.reset({ producto: 0 });
        this.precioUnitario = "$"
        this.stockConfig = undefined;

        this.form.markAsPristine();
    }

    onLoadingItem(detail: ValeDetail ) {
        this.detailCopy = <ValeDetail>this.copyItem(this.detail);
    }

    onUpdatingItem(detail: ValeDetail) {
        let paquete = new Empaquetado(this.detailCopy.cantidad, this.detailCopy.producto.articulo.unidadMedida.factor, this.detailCopy.botellas);

        if (paquete.toDecimal() > (this.stockConfig.disponible / this.detailCopy.producto.articulo.unidadMedida.factor)) {
            this.showAlert("No es posible capturar una cantidad mayor a la disponible", "warning");
        }
        else {
            if (this.detailCopy._clientID == 0) {
                this.detailCopy._clientID = this.createClientID();
            }
            else {
                this.detailCopy.state = ChangeState.MODIFIED;
            }
            this.savingItem.emit(this.detailCopy);
        }
    }

    //Metodos
    onProductoChanged(producto: ProductosInfo) {
        if (producto) {

            if (1!=1) {
                this.showAlert("No es posible capturar este producto para esta forma de venta [Introducción]", "warning");
                this.detailCopy.idProducto = 0; this.selProducto.clear();
            }
            else {

                this.detailCopy.producto = producto;

                this.loadStock(producto.articulo.articuloStock.identity);

                this.articuloService.calcularPrecioCliente(this.detailCopy.producto.articulo.identity, this.idClienteVentas, this.context.app.defaultCompany.identity, this.idZonaVenta)
                    .then(precio => {
                        this.detailCopy.precioUnitario = precio.precioBase;
                        //this.detailCopy.precioNeto = precio.precioNeto;
                        this.detailCopy.descuentos = precio.descuentos;
                        this.detailCopy.impuestos = precio.impuestos;
                        this.precioUnitario = Globalize.formatNumber(this.detailCopy.precioUnitario, 'c');

                    })
                    .catch(error => this.handleError(error));
            }
        }
        else {
            this.unidadesMedida = [];
            this.stockConfig = new StockInfo();
        }
    }

    loadStock(idArticulo: number) {
        this.almacenService.loadStock(this.idAlmacenCarga, idArticulo)
            .then(stock => {
                this.stockConfig = stock;
                this.loadUnidadesMedida(this.detailCopy.producto.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
        //en caso contrario, cargar lista de unidades de medida del mismo tipo
        if (this.stockConfig.manejaSeries) {
            let unidadArticulo = this.detailCopy.producto.articulo.articuloStock.unidadMedida;
            let newUnidad: UnidadMedidaItem = {
                id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base,
                factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
            };
            let newUnidades = [];
            newUnidades.push(newUnidad);
            this.unidadesMedida = newUnidades;

            this.detailCopy.idUnidadMedida = unidadArticulo.identity;
        }
        else {
            this.unidadMedidaService.loadList(idTipoUnidad)
                .then(unidadesMedida => {
                    this.unidadesMedida = unidadesMedida;
                    this.detailCopy.idUnidadMedida = this.detailCopy.producto.articulo.idUnidadMedida;
                    this.setCurrentUnidad(this.detailCopy.producto.articulo.idUnidadMedida);
                })
                .catch(error => this.handleError(error));
        }
    }

    setCurrentUnidad(idUnidadMedida: number) {
        this.selectedUM = this.unidadesMedida.find(u => u.id == idUnidadMedida);
        if (this.quantity) {
            this.quantity.unidadMedida = this.selectedUM;
        }
        //Actualizar valor de existencia disponible y el costo unitario basado en la unidad de medida seleccionada
        if (this.stockConfig && this.selectedUM) {
            this.disponible = (this.stockConfig.disponible / this.selectedUM.factor).toString();
            this.quantityLbl.Show(+ this.disponible, this.selectedUM);
            this.detailCopy.idUnidadMedida = idUnidadMedida;
            //this.costoUnitario = Globalize.formatNumber((this.stockConfig.costoPromedio * this.selectedUM.factor), 'c');
            //if (this.idUnidadMedida == 0 || this.form.controls['idUnidadMedida'].value != this.idUnidadMedida) {
            ////if (this.detailCopy.producto.articulo.idUnidadMedida == 0 || this.form.controls['idUnidadMedida'].value != this.detailCopy.producto.articulo.idUnidadMedida) {
            //    this.form.controls['idUnidadMedida'].setValue(this.selectedUM.id);
            //}
        }
        //this.validChange.emit(this.valid);
    }

    onCantidadChange(cantidad: number) {
        let paquete = new Empaquetado(cantidad, this.detailCopy.producto.articulo.unidadMedida.factor)
        this.detailCopy.cajas = cantidad;

        if (paquete.toDecimal() > (this.stockConfig.disponible / this.detailCopy.producto.articulo.unidadMedida.factor)) {
            this.showAlert("No es posible capturar una cantidad mayor a la disponible", "warning");
        }
        else {
            this.detailCopy.cantidad = paquete.toDecimal();
            this.detailCopy.importe = this.detailCopy.cantidad * this.detailCopy.precioUnitario;
            //this.cantidadChange.emit(cantidad);
            //this.validChange.emit(this.valid);
        }
    }

    onUnidadesChange(unidades: number) {
        let paquete = new Empaquetado(this.detailCopy.cajas, this.detailCopy.producto.articulo.unidadMedida.factor, unidades)

        this.detailCopy.botellas = unidades;
        if (paquete.toDecimal() > (this.stockConfig.disponible / this.detailCopy.producto.articulo.unidadMedida.factor)) {
            this.showAlert("No es posible capturar una cantidad mayor a la disponible", "warning");
        }
        else {
            this.detailCopy.cantidad = paquete.toDecimal();

            this.detailCopy.importe = this.detailCopy.cantidad * this.detailCopy.precioUnitario;
            //this.cantidadChange.emit(cantidad);
            //this.validChange.emit(this.valid);
        }
    }



}
