﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//
import * as _ from 'lodash';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { PedidoInfo, PedidoItem } from '../models/pedido';
import { PedidoDetail } from '../models/pedido-detail';
import { PedidoService } from '../services/pedido.service';
import { PedidoDetailEditor } from './parts/pedidos-detail.editor';
import { ClienteInfo, ClienteRutasItem } from '../models/cliente';
import { ClienteService } from '../services/cliente.service';
import { RutasPreventaItem } from '../models/rutas-preventa';
import { RutasRepartoItem } from '../models/rutas-reparto';

import { ImportesInfo } from '../../accounting/models/importes';
import { Empaquetado } from '../../inventory';
//import { moment } from 'ngx-bootstrap/chronos/test/chain';
//
import * as moment from 'moment';
import { FormaVentaInfo } from '../models/forma-venta';

@Component({
    selector: 'distribucion-pedidos-editor',
    templateUrl: './pedidos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PedidosEditor)
        },
        {
            provide: DocumentService, useExisting: PedidoService
        }
    ]
})
export class PedidosEditor extends DocumentEditor {
    pedido = new PedidoInfo();
    formaVenta: FormaVentaInfo;

    get importes() {
        return this.importesDesglosados();
    }

    rutasItems: ClienteRutasItem[] = [];
    rutasPreventa: RutasPreventaItem[] = [];
    rutasReparto: RutasRepartoItem[] = [];
    idAlmacenCarga: number;

    //

    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: PedidoDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private pedidoService: PedidoService, private clienteService: ClienteService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idCliente': [0, AztecaValidators.selected],
            'idFormaVenta': [0, AztecaValidators.selected],
            'idRutaPreventa': [0, AztecaValidators.selected],
            'idRutaReparto': [0, AztecaValidators.selected],
            'fechaEntrega': [new Date, (control) => this.validateFecha(control)],
            'observaciones':[''],
        });

        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        this.documentForm.controls['idRutaPreventa'].valueChanges.subscribe(value => {
            setTimeout(_ => this.loadRutasReparto(value));
        });

        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        this.documentForm.controls['idRutaReparto'].valueChanges.subscribe(value => {
            setTimeout(_ => this.onRutaRepartoChanged(value));
        });

    }

    validateFecha(control: FormControl) {
        let FechaHoy: Date;
        FechaHoy = moment(new Date).hour(0).minute(0).second(0).millisecond(0).toDate();

        if (control.value) {
            let Fecha = control.value;
            Fecha = moment(Fecha).hour(0).minute(0).second(0).millisecond(0).toDate();
            if (Fecha.getTime() >= FechaHoy.getTime())
                return null;
            else
                return 'La fecha no debe ser anterior a hoy';
        }
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.pedido.fechaEntrega = moment(new Date()).hour(0).minute(0).second(0).millisecond(0).toDate();
    }

    onCreatingDocument() {
        this.pedido.fechaEntrega = moment(new Date).utc().toDate();
        this.pedido.fechaEntrega.setHours(0);
        this.pedido.fechaEntrega.setMinutes(0);
        this.pedido.fechaEntrega.setSeconds(0);
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.pedidoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.pedido = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.pedidoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.pedido = doc;
                this.detailsEditor.clientIndexStart = this.pedido.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.pedidoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.pedido = doc;

                this.loadRutas(this.pedido.idCliente);

                //this.pedido.details.forEach(det => {
                //    let paquete = new Empaquetado(det.cantidadPTA, det.producto.articulo.unidadMedida.factor);
                //    det.cantidad = paquete.cantidad;
                //    det.unidades = paquete.unidades;
                //});

                this.detailsEditor.clientIndexStart = this.pedido.details.length;
                if (this.pedido.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
                
                setTimeout(_ => this.loadRutasReparto(doc.idRutaPreventa),1000);
                setTimeout(_ => this.onRutaRepartoChanged(doc.idRutaReparto),2000);

            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {

        //esperar wait
        this.validateDetail()
            .then(value => {
                if (value == true) {

                    this.context.app.showSpinner();
                    this.pedidoService.update(this.pedido)
                        .then(updatedDoc => {
                            this.context.app.hideSpinner();
                            //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                            this.reset();
                            if (!args.closeEditor && this.pedido.identity == '') {
                                this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                            }
                            else {
                                if (args.closeEditor)
                                    this.router.navigate(['../'], { relativeTo: this.route });
                                else
                                    this.showAlert('La información ha sido actualizada con éxito', 'success');
                            }
                            this.pedido = updatedDoc;
                            this.pedido.details.forEach(det => {
                                let paquete = new Empaquetado(det.cantidadPTA, det.producto.articulo.unidadMedida.factor);
                                det.cantidad = paquete.cantidad;
                                det.unidades = paquete.unidades;
                            });
                        })
                       .catch(error => this.handleError(error));
                }
            })
            .catch(error => this.handleError(error));

    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.pedidoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.pedidoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.pedido.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        if (this.pedido.idCliente > 0) {
            this.detailsEditor.detail = new PedidoDetail();
            this.detailsEditor.show();
        }
        else
            this.showAlert("Debe seleccionar un cliente para poder agregar productos", "warning");
        
    }

    onEditingDetail(detail: PedidoDetail) {       
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();       
    }

    onDeletingDetail(detail: PedidoDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: PedidoDetail[]) {
        this.pedido.details = updatedDetails;
    }

    onSavingDetail(detail: PedidoDetail) {
        this.detailsList.updateItem(detail);
    }

    ///metodos locales
    importesDesglosados(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        let elementos = this.pedido.details.filter(item => item.state != 'DELETED');//this.combineDetail();

        if(elementos.length>0)
            this.pedido.totalCantidad = elementos.map(item => item.cantidadPTA).reduce((a, b) => a + b);

        elementos.forEach(item => {
            desglose.subtotal += item.precioUnitario * item.cantidadPTA;
            desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidadPTA;
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidadPTA).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidadPTA).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        return desglose;
    }    

    clienteSelectedChanged(cliente: ClienteInfo) {
        if (cliente) {            
            this.pedido.cliente = cliente;
            this.loadRutas(cliente.identity);
            if (this.pedido.identity == '')
                this.pedido.details = [];
        }
    }

    loadRutas(idCliente: number) {
        this.clienteService.getRutas(idCliente)
            .then(rutas => {
                //
                this.rutasItems = rutas;

                //Agrupamos y cargamos Rutas de Preventa
                var result = _(this.rutasItems).groupBy('idRutaPreventa').transform(function (result, current) {
                    result.push({
                        id: current[0].idRutaPreventa,
                        nombre: current[0].rutaPreventa,
                    });
                }, []).value();

                this.rutasPreventa = result;
            })
            .catch(error => this.handleError(error));
    }

    loadRutasReparto(idRutaPreventa: number) {
        var result = _(this.rutasItems.filter(item => item.idRutaPreventa == idRutaPreventa)).groupBy('idRutaReparto').transform(function (result, current) {
            result.push({
                id: current[0].idRutaReparto,
                nombre: current[0].rutaReparto,
                idAlmacenCarga: current[0].idAlmacenCarga,
            });
        }, []).value();

        this.rutasReparto = result;
    }

    onRutaRepartoChanged(idRutaReparto: number) {        
        let rutaRep: RutasRepartoItem = this.rutasReparto.find(rr => rr.id == idRutaReparto)
        if (rutaRep)
            this.idAlmacenCarga = rutaRep.idAlmacenCarga;
    }

    async validateDetail() {
        return <boolean>await this.checkDetails();
    };

    checkDetails() {
        let result: boolean = true;

        //validamos que todos los articulos tengan serie en caso de que se maneje
        if (this.pedido.details.filter(det => det.state != 'DELETED' ).length == 0) {
            this.showAlert("Debe calcular al menos una partida de productos al pedido", 'warning');
            result = false;
        }

        return result;
    }

    formaVentaSelected(formaVenta: FormaVentaInfo) {
        this.formaVenta = formaVenta;
    }

    onFechaEntregachanged(Fecha: Date) {
        //let FechaHoy: Date;
        //FechaHoy = moment(new Date).hour(0).minute(0).second(0).toDate();
        
        //if (Fecha.getTime() >= FechaHoy)
        //    return null;
        //else
        //    this.showAlert("Debe establecer una fecha igual o posterior a hoy", 'warning');

    }



}

