﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { ValePagoInfo, ValePagoItem } from '../models/vale-pago';
import { ValePagoDetail } from '../models/vale-pago-detail';
import { ValePagoPendiente } from '../models/vale-pago-pendiente';
import { ValePagoService } from '../services/vale-pago.service';
import { ValePagoDetailEditor } from './parts/vales-pagos-detail.editor';
import { ValeService } from '../services/vale.service'
import { ValeInfo } from '../models/vale';

import { Empaquetado } from '../../inventory';

@Component({
    selector: 'distribucion-vales-pagos-editor',
    templateUrl: './vales-pagos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ValesPagosEditor)
        },
        {
            provide: DocumentService, useExisting: ValePagoService
        }
    ]
})
export class ValesPagosEditor extends DocumentEditor {
    valePago = new ValePagoInfo();
    vale = new ValeInfo();
    pendienteItem: ValePagoPendiente = new ValePagoPendiente;
    nombreCliente: string = '';
    idValePagar: string;
    fechaCaptura: Date;
    fechaCarga: Date;
    fechaVencimiento: Date;
    rutaReparto: string;
    totalCantidad: number;
    totalMonto: number;

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: ValePagoDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private valePagoService: ValePagoService,
        private valeService: ValeService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'nombreCliente': [''],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {
        this.route.params.forEach(params => {
            if (params['idVale']) {
                this.idValePagar = params['idVale'];
                this.valePago.idVale = params['idVale'];

            }
            if (params['cliente']) {
                this.pendienteItem.cliente = params['cliente'];
            }
            if (params['fechaCaptura']) {
                this.pendienteItem.fechaCaptura = params['fechaCaptura'];
            }
            if (params['fechaCarga']) {
                this.pendienteItem.fechaCarga = params['fechaCarga'];
            }
            if (params['fechaVencimiento']) {
                this.pendienteItem.fechaVencimiento = params['fechaVencimiento'];
            }
            if (params['rutaReparto']) {
                this.pendienteItem.rutaReparto = params['rutaReparto'];
            }

            
        });
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.valePagoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.valePago = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.valePagoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.valePago = doc;
                this.detailsEditor.clientIndexStart = this.valePago.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.valePagoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.valePago = doc;
                this.pendienteItem.cliente = this.valePago.vale.cliente.clienteVentas.nombre;
                this.pendienteItem.fechaCaptura = this.valePago.vale.fechaSys;
                this.pendienteItem.fechaCarga = this.valePago.vale.fechaCarga;
                this.pendienteItem.fechaVencimiento = this.valePago.vale.fechaVencimiento;
                this.pendienteItem.rutaReparto = this.valePago.vale.rutaReparto.nombre;
                this.pendienteItem.total = this.valePago.vale.total;
                this.pendienteItem.totalCantidad = this.valePago.vale.totalCantidad;

                this.valePago.details.forEach(item => {
                    let paqueteVale = new Empaquetado(item.cantidadVale, item.producto.articulo.unidadMedida.factor);
                    item.cajasVale = paqueteVale.paquetes;
                    item.botellasVale = paqueteVale.unidades;

                    let paqueteSaldo = new Empaquetado(item.cantidadSaldo, item.producto.articulo.unidadMedida.factor);
                    item.cajasSaldo = paqueteSaldo.paquetes;
                    item.botellasSaldo = paqueteSaldo.unidades;

                    let paqueteREC = new Empaquetado(item.cantidadREC, item.producto.articulo.unidadMedida.factor);
                    item.cajasREC = paqueteREC.paquetes;
                    item.botellasREC = paqueteREC.unidades;

                    let paqueteVTA = new Empaquetado(item.cantidadVTA, item.producto.articulo.unidadMedida.factor);
                    item.cajasVTA = paqueteVTA.paquetes;
                    item.botellasVTA = paqueteVTA.unidades;

                });

                this.detailsEditor.clientIndexStart = this.valePago.details.length;
                if (this.valePago.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();

        this.valePago.details.forEach(item => {

            let paqueteREC = new Empaquetado(item.cajasREC, item.producto.articulo.unidadMedida.factor, item.botellasREC);
            item.cantidadREC = paqueteREC.toDecimal();

            let paqueteVTA = new Empaquetado(item.cajasVTA, item.producto.articulo.unidadMedida.factor, item.botellasVTA);
            item.cantidadVTA = paqueteVTA.toDecimal();

        })


        this.valePagoService.update(this.valePago)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.valePago.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.valePago = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.valePagoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.valePagoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.valePago.identity);
            }
            if (args.heading == 'Detalle' && this.valePago.identity == '') {
                this.getVale(this.idValePagar);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new ValePagoDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: ValePagoDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onCellUpdated(ref: any) {

        var saldoValida: number;

        let item: ValePagoDetail = <ValePagoDetail>this.detailsList.currentItem;
        
        let paqueteREC = new Empaquetado(item.cajasREC, item.producto.articulo.unidadMedida.factor, item.botellasREC);
        item.cantidadREC = paqueteREC.toDecimal();

        let paqueteVTA = new Empaquetado(item.cajasVTA, item.producto.articulo.unidadMedida.factor, item.botellasVTA);
        item.cantidadVTA = paqueteVTA.toDecimal();

        let paqueteSaldo = new Empaquetado(item.cajasSaldo, item.producto.articulo.unidadMedida.factor, item.botellasSaldo);
        item.cantidadSaldo = paqueteSaldo.toDecimal();

        if ((item.cantidadSaldo - (item.cantidadREC + item.cantidadVTA)) < 0) {
            this.showAlert('No se puede pagar ni rechazar una cantidad mayor al saldo', 'error');
            ref.cancel = true;
        }

    }

    onDeletingDetail(detail: ValePagoDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: ValePagoDetail[]) {
        this.valePago.details = updatedDetails;
    }

    onSavingDetail(detail: ValePagoDetail) {
        this.detailsList.updateItem(detail);
    }

    /*****Metodos privados***********/

    getVale(idVale: string) {
        this.valeService.getDetail(idVale)
            .then(ret => {
                this.vale = ret;
                this.valePago.idVale = this.vale.identity;
                this.valePago.fechaVale = this.vale.fechaSys;
                var detalles: ValePagoDetail[] = [];
                this.vale.details.forEach(item => {
                    var detNew: ValePagoDetail = new ValePagoDetail;
                    detNew.idProducto = item.idProducto;
                    detNew.idLote = item.idLote;
                    detNew.numSerie = item.numSerie;

                    detNew.producto = item.producto;

                    detNew.fechaCaducidad = item.fechaCaducidad;
                    detNew.cantidadVTA = item.cantidad;
                    detNew.cantidadREC = 0;

                    let paqueteVale = new Empaquetado(item.cantidad, item.producto.articulo.unidadMedida.factor);
                    detNew.cajasVale = paqueteVale.paquetes;
                    detNew.botellasVale = paqueteVale.unidades;

                    let paqueteSaldo = new Empaquetado(item.saldo, item.producto.articulo.unidadMedida.factor);
                    detNew.cajasSaldo = paqueteSaldo.paquetes;
                    detNew.botellasSaldo = paqueteSaldo.unidades;

                    detalles.push(detNew);
                });
                this.valePago.details = detalles;
            })
            .catch(error => this.handleError(error));
    }

    

}

