﻿import { Component, OnInit, ViewChild } from '@angular/core';

import { ToolbarButton, AlertsBarComponent } from 'framework3/controls/index';

//Wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import { Menu } from 'wijmo/wijmo.input';

//Bootstra
import { ModalDirective } from 'ngx-bootstrap';

//moment
import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { CargaFisicaService } from '../services/carga-fisica.service';
import { CargaFisicaInfo } from '../models/carga-fisica';
import { CargaFisicaDetail, CargaFisicaProducto } from '../models/carga-fisica-detail';
import { TipoConteoService } from '../services/tipo-conteo.service';

import { MenuItem } from 'framework3/controls';
import { ContenedorItem } from '../models/productos';
import { lang } from 'moment';

@Component({
    selector: 'distribution-carga-fisica',
    templateUrl: 'carga-fisica.maintenance.html'
})
export class CargaFisicaMaintenanceComponent extends BaseComponent implements OnInit {

    cvRutas: wjcCore.CollectionView;
    cvDocuments: wjcCore.CollectionView;
    cvContenedor: wjcCore.CollectionView;

    currentRuta: any;
    currentDocument: any;
    documents: CargaFisicaDetail[] = [];
    tiposConteoMap: wjcGrid.DataMap;

    fechaInicial: Date;
    fechaFinal: Date;
    porCargar: boolean = true;

    menuItems: MenuItem[] = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('dlgContenedor') dlgContenedor: ModalDirective;

    constructor(private context: Context,
        private cargaFisciaSvc: CargaFisicaService, private tipoConteoSvc: TipoConteoService) {
        super();
        this.cvRutas = new wjcCore.CollectionView();
        this.cvDocuments = new wjcCore.CollectionView();
        this.cvContenedor = new wjcCore.CollectionView();

        this.fechaInicial = new Date();
        this.fechaFinal = new Date();
        this.porCargar = true;
    }

    ngOnInit(): void {
        setTimeout(_ => {
            this.context.app.showSpinner();
            this.tipoConteoSvc.getTiposConteoCbo(this.context.app.defaultSite.identity)
                .then(info => {
                    this.tiposConteoMap = new wjcGrid.DataMap(info, "id", "nombre");
                })
                .catch(error => this.handleError(error));
            this.onClickActualizarRuta(this.porCargar);
        });
        this.createMenu();
    }

    createMenu() {
        let mnuCargar: MenuItem = { icon: "glyphicon glyphicon-ok", text: "Cargar", name: "CARGAR", smallText: "Cargar pedido" };
        let mnuCancelar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Cancelar", name: "CENCELAR", smallText: "Cancelar carga" };
        this.menuItems.push(mnuCargar, mnuCancelar);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    onCambioPorCargar(args: boolean) {
        this.onClickActualizarRuta(args);
    }
    //////////////////////////////////////////////////////////////////
    // Rutas

    initializedRutas(sender: wjcGrid.FlexGrid, args: any) {
        var self = this;
        this.cvRutas.currentChanged.addHandler((args: wjcCore.EventArgs) => {
            self.currentRuta = self.cvRutas.currentItem;
            if (self.currentRuta) {
                self.loadDocumets(self.currentRuta.idRutaReparto, self.porCargar);
            }
            else {
                self.loadDocumets(null, self.porCargar);
            }
        }, self);
    }

    onClickActualizarRuta(porCargar: boolean) {
        if (porCargar == null) {
            porCargar = this.porCargar;
        }
        this.context.app.showSpinner();
        this.cargaFisciaSvc.getRutasReparto(this.context.app.defaultSite.identity, moment(this.fechaInicial).format(), moment(this.fechaFinal).format(), porCargar)
            .then(info => {
                this.cvRutas.sourceCollection = info;
                this.cvRutas.refresh();
                this.context.app.hideSpinner();
            })
            .catch(error => {
                this.handleError(error);
            });
    }

    onClickCargarRuta() {
        if (this.currentRuta) {
            this.documents = this.copyDetails(<CargaFisicaDetail[]>this.cvDocuments.sourceCollection);
            var documentsDet = [];
            this.documents.forEach(item => {
                documentsDet.push(...item.documentsDet);
            });
            this.loadContenedores(documentsDet);
        }
    }


    onClickCancelarCargarRuta() {
        if (this.currentRuta) {
            this.documents = this.copyDetails(<CargaFisicaDetail[]>this.cvDocuments.sourceCollection);
            this.context.app.showSpinner();
            this.canelarDocumentos();
        }
    }

   
    //////////////////////////////////////////////////////////////////
    // Documents

    initializedDocuments(sender: wjcGrid.FlexGrid, args: any) {
        var self = this;
        this.cvDocuments.currentChanged.addHandler((args: wjcCore.EventArgs) => {
            self.currentDocument = self.cvDocuments.currentItem;
        }, self);
    }

    loadDocumets(idRutaReparto: number, porCargar: boolean) {
        this.context.app.showSpinner();
        if (idRutaReparto) {
            this.cargaFisciaSvc.getCargaFisicaDocumentos(idRutaReparto, moment(this.fechaInicial).format(), moment(this.fechaFinal).format(), porCargar)
                .then(info => {
                    this.cvDocuments.sourceCollection = info;
                    this.cvDocuments.refresh();
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.cvDocuments.sourceCollection = [];
        }
    }

    contextMenuPedidoClick(menu: Menu) {
        let option = this.menuItems[menu.selectedIndex];
        if (this.currentDocument) {
            if (option.name == 'CARGAR') {
                this.documents = [];
                this.loadContenedores(this.currentDocument.documentsDet);
                /*
                
                */
            }
            if (option.name == 'CENCELAR') {
                this.context.app.showSpinner();
                this.cargaFisciaSvc.cancelarAplicarCargaDocumento(this.currentDocument.idDocumento, this.currentDocument.origen)
                    .then(info => {
                        this.context.app.hideSpinner();
                        this.alerts.showAlert("Carga de documento " + this.currentDocument.idDocumento + " cancelada", 'success');
                    })
                    .catch(error => this.handleError(error));
            }
        }
    }

    cargarDocumentos() {
        if (this.documents.length > 0) {
            let cargaFisicaDetail: CargaFisicaDetail = this.documents.shift();
            this.cargaFisciaSvc.aplicarCargaDocumento(cargaFisicaDetail.idDocumento, cargaFisicaDetail.origen)
                .then(info => {
                    this.cargarDocumentos();
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.context.app.hideSpinner();
            this.alerts.showAlert("Documentos cargados", 'success');
        }
    }

    canelarDocumentos() {
        if (this.documents.length > 0) {
            let cargaFisicaDetail: CargaFisicaDetail = this.documents.shift();
            this.cargaFisciaSvc.cancelarAplicarCargaDocumento(cargaFisicaDetail.idDocumento, cargaFisicaDetail.origen)
                .then(info => {
                    this.canelarDocumentos();
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.context.app.hideSpinner();
            this.alerts.showAlert("Cancelación documentos cargados", 'success');
        }
    }

    //////////////////////////////////////////////////////////////////
    // Contenedores

    loadContenedores(documentsDet: any[]) {
        let contendeores: ContenedorItem[] = [];
        // Por cada detalle, identificamos los distintos tipos de contenedores
        this.currentDocument.documentsDet.forEach(item => {
            item.producto.contenedores.forEach(itemContenedor => {
                var exists = false;
                contendeores.forEach(detail => {
                    exists = !exists ? detail.idContenedor == itemContenedor.idContenedor : exists;
                });
                if (!exists) {
                    contendeores.push(itemContenedor);
                }
            })
        });

        let cargaProductos: CargaFisicaProducto[] = [];
        contendeores.forEach(contenedor => {
            let cargaProducto = new CargaFisicaProducto();
            cargaProducto.idProducto = contenedor.contenedor.idProducto;
            cargaProducto.producto = contenedor.contenedor.producto;
            cargaProducto.state = ChangeState.ADDED;
            cargaProductos.push(cargaProducto);
        });

        this.cvContenedor.sourceCollection = cargaProductos;
        this.dlgContenedor.config.backdrop = false;
        this.dlgContenedor.show();
    }

    onCantidadChange(item: any, value: any) {
        item.cantidad = value;
    }

    onCloseDialogContenedor() {
        this.dlgContenedor.hide();
    }

    onAceptarDialogContenedor() {
        //Generamos la carga de contenedores
        let cargaFisica = new CargaFisicaInfo();
        cargaFisica.idRutaReparto = this.currentRuta.idRutaReparto;
        cargaFisica.productos = <CargaFisicaProducto[]>this.cvContenedor.sourceCollection;
        this.dlgContenedor.hide();

        this.context.app.showSpinner();
        this.cargaFisciaSvc.cargarDocumento(cargaFisica)
            .then(info => {
                if (this.documents != null && this.documents.length > 0) {
                    this.cargarDocumentos();
                }
                else if (this.currentDocument != null) {
                    this.cargaFisciaSvc.aplicarCargaDocumento(this.currentDocument.idDocumento, this.currentDocument.origen)
                        .then(info => {
                            this.context.app.hideSpinner();
                            this.alerts.showAlert("Documento " + this.currentDocument.idDocumento + " cargado", 'success');
                        })
                        .catch(error => this.handleError(error));
                }
            })
            .catch(error => this.handleError(error)); 
    }

    //////////////////////////////////////////////////////////////////

    applyPolicy(policy: PolicyInfo): void {
        console.log("Method not implemented.");
    }

    private handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }


    copyDetails(details: CargaFisicaDetail[]): CargaFisicaDetail[] {
        return JSON.parse(JSON.stringify(details));
    }

    copyDetail(detail: CargaFisicaDetail): CargaFisicaDetail {
        return JSON.parse(JSON.stringify(detail));
    }

}