﻿import { Component, OnInit, ViewChild } from '@angular/core';

import { ToolbarButton, AlertsBarComponent } from 'framework3/controls/index';

//moment
import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo, TrackableItem, ChangeState } from 'framework3/core';
import { TrackableCollection } from 'framework3/editors';

//Wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import { Menu } from 'wijmo/wijmo.input';

import { CobranzaService as DistribucionCobranzaService } from '../services/cobranza.service';
import { CobranzaInfo as DistribucionCobranzaInfo, CobranzaItem as DistribucionCobranzaItem, CobranzaDocumento } from '../models/cobranza';
import { CobranzaDetail as DistribucionCobranzaDetail, CobranzaDocumentsDet } from '../models/cobranza-detail';

@Component({
    selector: 'distribution-cobranza',
    templateUrl: './cobranza.maintenance.html'
})
export class CobranzaMaintenanceComponent extends BaseComponent implements OnInit {

    cvRutas: wjcCore.CollectionView;
    cvTiposPagos: TrackableCollection;
    cvDocumentos: TrackableCollection;

    cobranza: DistribucionCobranzaInfo;
    currentRuta: any;
    currentTipoPago: any;
    fechaCobranza: Date;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    //////////////////////////////////////////////////////////////////
    get importe() {
        var importe: number = 0;
        if (this.cobranza) {
            this.cobranza.documentos.forEach(documento => {
                if (documento && documento.importe) {
                    importe += documento.importe;
                }
            });
        }
        return importe;
    }

    get monto() {
        var monto: number = 0;
        if (this.cobranza) {
            this.cobranza.details.forEach(detail => {
                if (detail && detail.monto) {
                    monto += detail.monto;
                }
            });
        }
        return monto;
    }

    get total() {
        return this.importe - this.monto;
    }

    constructor(private context: Context, private cobranzaSvc: DistribucionCobranzaService) {
        super();
        this.cvRutas = new wjcCore.CollectionView();
        this.cvTiposPagos = new TrackableCollection();
        this.cvDocumentos = new TrackableCollection();
        this.cvDocumentos.filter = (item: CobranzaDocumento) => {
            return (!item.state || item.state != ChangeState.DELETED) && item.idTipoPago == 0;
        }
    }

    ngOnInit(): void {
        setTimeout(() => {
            this.onClickActualizarRuta();
        });
    }

    onClickAplicarCobranza() {
        let info = this.cloneInfo<DistribucionCobranzaInfo>(this.cobranza);
        this.context.app.showSpinner();
        this.cobranzaSvc.enqueueCobranza(info)
            .then(info => {
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onClickCancelarCobranza() {

    }

    //////////////////////////////////////////////////////////////////
    // Rutas

    initializedRutas(sender: wjcGrid.FlexGrid, args: any) {
        var self = this;
        this.cvRutas.currentChanged.addHandler((args: wjcCore.EventArgs) => {
            self.currentRuta = self.cvRutas.currentItem;
            if (self.currentRuta) {
                self.loadCobranza(self.currentRuta.idRutaReparto);
            }
            else {
                self.clearCobranza();
            }
        }, self);
    }

    onClickActualizarRuta() {
        this.context.app.showSpinner();
        this.cobranzaSvc.getRutasReparto(this.context.app.defaultSite.identity, moment(this.fechaCobranza).format())
            .then(info => {
                this.cvRutas.sourceCollection = info;
                this.cvRutas.refresh();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////////////////////////////////////////////////////
    // Documentos

    initializeGridTiposPagos(flex: wjcGrid.FlexGrid) {
        flex.columnFooters.rows.push(new wjcGrid.GroupRow());
        flex.bottomLeftCells.setCellData(0, 0, 'Σ');
        // Handled current item
        var self = this;
        this.cvTiposPagos.currentChanged.addHandler((args: wjcCore.EventArgs) => {
            self.currentTipoPago = self.cvTiposPagos.currentItem;
        }, self);
    }

    initializeGridDocumentos(flex: wjcGrid.FlexGrid) {
        flex.columnFooters.rows.push(new wjcGrid.GroupRow());
        flex.bottomLeftCells.setCellData(0, 0, 'Σ');
    }

    loadCobranza(idRutaReparto: number) {
        this.context.app.showSpinner();
        this.cobranzaSvc.getCobranzaDocumento(idRutaReparto, moment(this.fechaCobranza).format())
            .then(info => {
                this.cobranza = info;
                this.cobranza.details.forEach(item => {

                    item._clientID = this.createClientID();
                    let documents: CobranzaDocumentsDet[] = [];
                    item.documentsDet.forEach(det => {
                        det._clientID = this.createClientID();
                        documents.push(this.cloneInfo<CobranzaDocumentsDet>(det));
                    });
                    item.cvDocumentsDet = new TrackableCollection();
                    item.cvDocumentsDet.sourceCollection = documents;

                });
                this.cvDocumentos.sourceCollection = this.cobranza.documentos;
                this.cvTiposPagos.sourceCollection = this.cobranza.details;
                this.cvDocumentos.refresh();
                this.cvTiposPagos.refresh();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    clearCobranza() {
        this.context.app.showSpinner();
        this.cvTiposPagos.sourceCollection = [];
        this.cvTiposPagos.refresh();
        this.cobranza = null;
        this.context.app.hideSpinner();
    }

    onRowDeleteDocumento(grdDocumentos: wjcGrid.FlexGrid ,  item: CobranzaDocumentsDet) {
        this.context.app.showSpinner();
        // Remover el documento del tipo de pago
        let cv = <TrackableCollection>grdDocumentos.itemsSource;
        cv.remove(item);
        if (this.currentTipoPago) {
            this.cvTiposPagos.editItem(this.currentTipoPago);
            this.currentTipoPago.importe -= item.importe;
            this.cvTiposPagos.commitEdit();
        }

        //
        this.cobranza.documentos.forEach(itemDocumento => {
            if (itemDocumento.idDocumento == item.idDocumento) {
                itemDocumento.state = ChangeState.MODIFIED;
                itemDocumento.idTipoPago = 0;
            }
        });
        this.cvDocumentos.refresh();
        this.context.app.hideSpinner();
    }

    onRowAddDocumento(item: CobranzaDocumento) {
        this.cvDocumentos.editItem(item);
        item.idTipoPago = item.tipoPago.identity;
        this.cvDocumentos.commitEdit();

        this.cobranza.details.forEach(itemDet => {
            // Se identifica el tipo de pago marcado
            if (itemDet.idTipoPago == item.idTipoPago) {
                var documentDet: CobranzaDocumentsDet = null;

                // Buscar detalles del tipo de pago el documento
                itemDet.documentsDet.forEach(itemDocumentDet => {
                    if (item._clientID == itemDocumentDet._clientID) {
                        documentDet = itemDocumentDet;
                    }
                });

                if (documentDet != null) {
                    // Si existe el documento de tipo de pago, se marca como agregado o modificado
                    itemDet.cvDocumentsDet.editItem(documentDet);
                    documentDet.state = documentDet.id == 0 ? ChangeState.ADDED : ChangeState.MODIFIED;
                    itemDet.cvDocumentsDet.commitEdit();
                }
                else {
                    // Si no existe, crea el documento del detalle 
                    documentDet = <CobranzaDocumentsDet> itemDet.cvDocumentsDet.addNew();
                    documentDet.idDocumento = item.idDocumento;
                    documentDet.fechaDocumento = item.fechaDocumento;
                    documentDet.importe = item.importe;
                }

                // Actualizamos el importe del tipo de pago
                this.cvTiposPagos.editItem(itemDet);
                itemDet.importe += documentDet.importe;
                this.cvTiposPagos.commitEdit();
                itemDet.cvDocumentsDet.refresh();
            }
        });
    }


    onSelectTipoPago(item: CobranzaDocumento, value: number) {
        if (value != null) {
            item.formaVenta.tiposPagos.forEach(itemTipoPago => {
                if (itemTipoPago.idTipoPago == value) {
                    item.tipoPago = itemTipoPago.tipoPago;
                }
            });
        }
        else {
            item.tipoPago = null;
        }
    }

    //////////////////////////////////////////////////////////////////

    applyPolicy(policy: PolicyInfo): void {
        console.log("Method not implemented.");
    }

    private handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element, this.replacer));
    }

    replacer(key, value) {
        if (key == "cvDocumentsDet")
            return null;
        else
            return value;
    }

    createClientID(): number {
        return parseInt((Math.random() * 1000).toString()) * 1000 + parseInt(moment().format("HHmmss"))
    }

}