﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Location, CommonModule } from '@angular/common'
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent, ContentViewerComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
//Inventario
import { Empaquetado } from '../../inventory';
//Locales
import { RutasRepartoService } from '../services/rutas-reparto.service'
import { ConteoService } from '../services/conteo.service'
import { RutaConteoItem } from '../models/ruta-conteo-item'
import { ConteoDocumento } from '../models/conteo-documento'
import { ConteoDetail } from '../models/conteo-detail'
import { ConteoInfo } from '../models/conteo'

import * as _ from 'lodash';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import * as wjcCore from 'wijmo/wijmo';


@Component({
    selector: 'distribucion-conteo',
    templateUrl: './conteo.html'
})

export class Conteo extends ProcessComponent {
    form: FormGroup;
    previewCollectionRutas: CollectionView = new CollectionView();
    currentRutaStatus: number = 0;
    previewCollectionPlasticos: CollectionView = new CollectionView();
    previewCollectionOtros: CollectionView = new CollectionView();
    idStatusCierre: number = 0;
    rutasPendientes: RutaConteoItem[] = [];
    documentosConteo: ConteoDocumento[] = [];
    detailConteo: ConteoDetail[] = [];
    idRutaReparto: number = 0;

    titleContentView: string = "";
    fileNameContentView: string = "";


    fecha: Date = new Date();

    conteos: { codigo: string, nombre: string, total: number }[] = [];

    private _cacheTiposConteo = {};

    detailMode = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandSingle];
    detailModeMulti = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandMulti];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder
        , private rutaService: RutasRepartoService
        , private conteoService: ConteoService) {
        super(context, router, route);
        this.form = formBuilder.group({
            'fecha': [new Date()]
        });
    }

    llenadoRutasPendientes() {
        this.context.app.showSpinner();
        this.rutaService.rutasPendientesConteo(this.context.app.defaultSite.idBranch, this.fecha)
            .then(recs => {
                this.rutasPendientes = recs;
                if (recs.length > 0) this.idStatusCierre = recs[0].idStatusCierre;
                this.previewCollectionRutas.sourceCollection = this.rutasPendientes;
                this.context.app.hideSpinner();
            })
    }

    llenarDocumentosConteo() {

        this._cacheTiposConteo = {}; 
        //this.detailConteo = [];

        var item: RutaConteoItem = new RutaConteoItem;
        item = this.previewCollectionRutas.currentItem;
        this.idRutaReparto = item.idRutaReparto;

        this.context.app.showSpinner();
        this.conteoService.loadDocumentosConteo(item.idRutaReparto, this.fecha, this.context.app.defaultCompany.idCompany)
            .then(recs => {
                this.documentosConteo = recs;
                this.conteoService.toConteoDetail(this.documentosConteo)
                    .then(recs => {
                        this.detailConteo = recs;
                        this.llenarConteos(this.detailConteo);
                        this.context.app.hideSpinner();
                    })
            })
    }



    llenarConteos(det: any) {
        let total: number = 0;
        let nombre: string = '';
        let conteosLocal: { codigo: string, nombre: string, total: number } [] = [];
        var cont = _.chain(det).map('tipoConteo.code', ).uniq().value()
        cont.forEach(conteo => {
            total = 0;
            nombre = '';
            this.detailConteo.forEach(item => {
                if (item.tipoConteo.code == conteo) {
                    total = total + item.cantidad;
                    nombre = item.tipoConteo.nombre;
                }
            });
            conteosLocal.push({ 'codigo': conteo, 'nombre': nombre, 'total': total })
        });
        this.conteos = conteosLocal;
    }

    conteosRepasar() {
        this.conteos.forEach(itemConteos => {
            this.getDet(itemConteos);
        })
    }


    imprimir() {
        this.conteosRepasar(); // para en caso de que no se haya desplegado la ruta mediante un click en el grid.

        var detailConteoToProc: ConteoDetail[] = [];
        this.context.app.showSpinner();
        this.conteos.forEach(itemConteos => {
            this._cacheTiposConteo[itemConteos.codigo].forEach(item => {
                var detNew: ConteoDetail = new ConteoDetail;
                detNew.cantidad = item.cantidad;
                detNew.codigo = item.codigo;
                detNew.faltante = item.faltanteShow;
                detNew.idProducto = item.idProducto;
                detNew.idTipoConteo = item.idTipoConteo;
                detNew.nombre = item.nombre;

                let paquete = new Empaquetado(item.cajas, item.factor, item.botellas);
                detNew.conteo = paquete.toDecimal();

                detailConteoToProc.push(detNew);
            });
        })


        var conteo: ConteoInfo = new ConteoInfo();

        conteo.fechaAfectacion = this.fecha;
        conteo.idRutaReparto = this.idRutaReparto;

        conteo.idSucursal = this.context.app.defaultSite.idBranch;

        conteo.details = detailConteoToProc;

        var item: RutaConteoItem = new RutaConteoItem;
        item = this.previewCollectionRutas.currentItem;

        this.conteoService.toPdf(conteo)
            .then(pdf => {
                this.context.app.hideSpinner();
                this.titleContentView = "Conteo_" + item.codigo + ".pdf";
                this.fileNameContentView = "Conteo_" + item.codigo + ".pdf";
                this.reportViewer.show("data:application/pdf; base64, " + pdf);

            })

    }

    procesar() {

        this.conteosRepasar(); // para en caso de que no se haya desplegado la ruta mediante un click en el grid.

        var detailConteoToProc: ConteoDetail[] = [];
        this.context.app.showSpinner();
        this.conteos.forEach(itemConteos => {
            this._cacheTiposConteo[itemConteos.codigo].forEach(item => {
                var detNew: ConteoDetail = new ConteoDetail;
                detNew.cantidad = item.cantidad;
                detNew.codigo = item.codigo;
                detNew.faltante = item.faltanteShow;
                detNew.idProducto = item.idProducto;
                detNew.idTipoConteo = item.idTipoConteo;
                detNew.nombre = item.nombre;

                let paquete = new Empaquetado(item.cajas, item.factor, item.botellas);
                detNew.conteo = paquete.toDecimal();

                detailConteoToProc.push(detNew);
            });
        })

        var conteo: ConteoInfo = new ConteoInfo();

        conteo.fechaAfectacion = this.fecha;
        conteo.idRutaReparto = this.idRutaReparto;

        conteo.idSucursal = this.context.app.defaultSite.idBranch;

        conteo.details = detailConteoToProc;
        conteo.documentosConteo = this.documentosConteo;

/*----------------------------SIN ENCOLAR Y ENCOLADO-----------------------------------*/
        /*
        this.conteoService.update(conteo)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                conteo = updatedDoc;
                this.aplicar(conteo.identity);
                this.alerts.showAlert('La información ha sido actualizada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
        */
        
        this.conteoService.enqueueConteo(conteo)
            .then(updatedDoc => {
                this.llenadoRutasPendientes();
                this.context.app.hideSpinner();
                conteo = updatedDoc;
                this.alerts.showAlert('La información ha sido actualizada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
        
/*-------------------------------------------------------------------------------------*/
    }

    cerrar() {
        this.conteoService.cerrar(this.fecha, this.context.app.defaultSite.idBranch)
            .then(ret => {
                this.llenadoRutasPendientes();
                this.alerts.showAlert('El cierre se ha realizado con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }

    cancelarConteo(idConteo: string) {
        this.context.app.showSpinner();
        this.conteoService.previousStep(idConteo,null)
            .then(_ => {
                this.llenadoRutasPendientes();
                this.context.app.hideSpinner();
                this.alerts.showAlert('El conteo ha sido cancelado.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    cancelarCierre() {
        this.conteoService.cancelarCierre(this.fecha, this.context.app.defaultSite.idBranch)
            .then(ret => {
                this.llenadoRutasPendientes();
                this.alerts.showAlert('El cierre de conteo se ha cancelado', 'success');
            })
            .catch(error => this.handleError(error));

    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CONSULTAR') {
            this.llenadoRutasPendientes();
        }
        if (button.name == 'PROCESAR') {
            this.procesar();
        }
        if (button.name == 'CERRAR') {
            this.cerrar();
        }
        if (button.name == 'CANCELAR-CONTEO') {
            var item: RutaConteoItem = new RutaConteoItem;
            item = this.previewCollectionRutas.currentItem;
            this.cancelarConteo(item.idConteo);
        }
        if (button.name == 'CANCELAR-CIERRE') {
            this.cancelarCierre();
        }
        if (button.name == 'CONSULTAR-RUTA') {
            this.getDetalleRuta();
        }
        if (button.name == 'IMPRIMIR') {
            this.imprimir();
        }


    }

    aplicar(idConteo: string) {
        this.conteoService.nextStep(idConteo, 1, null)
            .then(_ => {
                this.llenadoRutasPendientes();
                //Volver a cargar documento
                //this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));

    }

    getDetalleRuta() {
        var item: RutaConteoItem = new RutaConteoItem;
        item = this.previewCollectionRutas.currentItem;
        this.currentRutaStatus = item.idStatus;
        if (item.idConteo.trim() == '') {
            this.llenarDocumentosConteo();
        }
        else {
            this._cacheTiposConteo = {};
            this.conteoService.getDetail(item.idConteo)
                .then(ret => {
                    var conteo: ConteoInfo = new ConteoInfo();
                    conteo = ret;
                    this.idRutaReparto = conteo.idRutaReparto;
                    this.documentosConteo = conteo.documentosConteo;
                    this.detailConteo = conteo.details;
                    this.previewCollectionPlasticos.sourceCollection = this.detailConteo.filter(search => search.tipoConteo.code == 'PLASTICO');
                    this.previewCollectionOtros.sourceCollection = this.detailConteo.filter(search => search.tipoConteo.code == 'TARIMA');
                    this.llenarConteos(this.detailConteo);

                })
                .catch(error => this.handleError(error));
        }
    }

    onSelectionChanged(args: any) {
        this.conteos = [];
        
    }

    onChangeFecha(args: any) {
        this.rutasPendientes = [];
        this.previewCollectionRutas.sourceCollection = this.rutasPendientes;
    }

    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {
        this._cacheTiposConteo
    }

    getDet(prods: any) {
        let det: { idTipoConteo: number, idProducto: number, cantidad: number, cantidadBase: number, conteo: number, faltante: number, faltanteShow: number, codigo: string, nombre: string, codigoTipoConteo: string, cajas: number, botellas: number, cajasVta: number, botellasVta: number, factor: number, fraccionable: boolean }[] = [];

        var view = this._cacheTiposConteo[prods.codigo];

        if (!view) {
            view = new wjcCore.CollectionView(this.detailConteo);
            view.filter = function (item) {
                return item.tipoConteo.code == prods.codigo;
            }

            this.detailConteo.forEach(item => {
                if (item.tipoConteo.code == prods.codigo) {
                    let paquete = new Empaquetado(0, item.producto.articulo.unidadMedida.factor, item.cantidadBase);
                    let paqueteVta = new Empaquetado(0, item.producto.articulo.unidadMedida.factor, item.cantidadVtaBase);
                    det.push({ 'idTipoConteo': item.idTipoConteo, 'idProducto': item.idProducto, 'cantidad': item.cantidad, 'cantidadBase': item.cantidadBase, 'conteo': item.conteo, 'faltante': item.faltante, 'faltanteShow': item.faltante, 'codigo': item.codigo, 'nombre': item.nombre, 'codigoTipoConteo': item.tipoConteo.code, 'cajas': paquete.paquetes, 'botellas': paquete.unidades, 'cajasVta': paqueteVta.paquetes, 'botellasVta': paqueteVta.unidades, 'factor': item.producto.articulo.unidadMedida.factor, 'fraccionable': item.producto.articulo.unidadMedida.fraccionable });
                }
            });

            this._cacheTiposConteo[prods.codigo] = det;
            view = det;
        }
        return view;
    }

    onCellEditEnded(flex: any, e: any) {
        let det: any = flex.rows[e.row].dataItem;

        let paquete = new Empaquetado(det.cajas, det.factor, det.botellas);
        det.conteo = paquete.toDecimal() * det.factor;

        det.faltante = det.cantidadBase - det.conteo
        let paqueteFaltante = new Empaquetado(0, det.factor, det.faltante);
        det.faltanteShow = paqueteFaltante.toDecimal();

        /// para que se muestren como cajas nuevamente///
        let paqueteView = new Empaquetado(0, det.factor, det.conteo);
        det.cajas = paqueteView.paquetes;
        det.botellas = paqueteView.unidades;
        
    }

    getStatusColor(status: string): string {
        if (status == 'Pendiente')
            return '#ff471a';
        if (status == 'Procesada')
            return 'green';
    }


    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }


}