﻿import { Component, ViewChild, Input, Output, EventEmitter, OnInit } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';

import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent, } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { ProcessComponent } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { TrackableCollection } from 'framework3/editors';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
//Wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import { Menu } from 'wijmo/wijmo.input';
import { forwardRef } from '@angular/core';
import { CollectionView, DateTime } from 'wijmo/wijmo';
//Features
import { DatasetInfo, ParameterItem, TranspilerInfo, DatasetService } from '../../translator/index';
import { ReestructuraTranspilerFormato } from '../models/reestructura-transpiler-formato';
import { ReestructuraService } from '../services/reestructura.service';
import { getTituloReestructura } from '../models/get-reestructura-titulo';
import { getReestructuraDetalle } from '../models/get-reestructura-detalle';
import { error } from '@angular/compiler/src/util';



@Component({
    selector: 'distribution-reestructura-distribucion',
    templateUrl: './reestructura-editor-detalle.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => reestructura_editor_detalle)
        }
    ]
})
export class reestructura_editor_detalle implements OnInit {

    identity: number;
    fechaInicio: DateTime;
    fechaFinal: DateTime;
    infoReestructura: getTituloReestructura[] = [];
    restructTitulo: getTituloReestructura[] = [];
    restructDetalle: getReestructuraDetalle[] = [];
    previewCollection: CollectionView = new CollectionView();
    selectedCollection: CollectionView = new CollectionView();
    form: FormGroup;
    res: ReestructuraTranspilerFormato = new ReestructuraTranspilerFormato();
    detalle: getReestructuraDetalle = new getReestructuraDetalle();
    tit: getTituloReestructura = new getTituloReestructura();
    cvParametros: TrackableCollection;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('viewDialog') viewDialog: ModalDirective;

    constructor
        (
        private formBuilder: FormBuilder,
        private restruct: ReestructuraService,
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private datasetSvc: DatasetService
        ) {
        this.form = formBuilder.group({
            'Arch': '',
            'IdReestructura': 0,
            'IdCliente': 0,
            'IdReestructuraTit': 0,
            'Referencia': '',
            'CodigoSucursal': '',
            'CodigoZona': '',
            'CodigoRuta': '',
            'CodigoFrecuencia': '',
            'IdSucursalAnt': 0,
            'IdZonaAnt': 0,
            'IdFrecuenciaAnt': 0,
            'IdRutaAnt': 0,
            'Error': false,
            'ErrorText': '',
            'Nombre': '',
            'Destino': '',
            'Estatus': 'B',
            'UserCreate': '',
            'CreateDate': '',
            'LastUpdate': '',
            'PCUpdate': '',
            'idDataset': [AztecaValidators.selected],
            'IdClienteDestino': 0,
            'CodigoClientePixcua': '',
            'fechaIni': '',
            'fechaFin': ''
        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            if (params['id']) {
                this.identity = +params['id'];
                if (this.identity > 0) {
                    //this.GetDetalleReestructura(this.identity);
                    this.restruct.getReestructuraDetalle(this.identity)
                        .then(result => {
                            this.restructDetalle = result;
                            this.previewCollection.sourceCollection = this.restructDetalle;
                        })
                }
            }
        });
    }   
    
    GetDetalleReestructura(id:number) {
        this.restruct.getReestructuraDetalle(id)
            .then(result => {
                this.restructDetalle = result;
                this.previewCollection.sourceCollection = this.restructDetalle;
            })
    }
    
    //Obtiene la información del titulo de la reestructura
    GetTituloReestructura() {
        this.context.app.showSpinner();
        this.restruct.getReestructuraTitulo(this.fechaInicio, this.fechaFinal)
            .then(result => {
                this.context.app.hideSpinner();
                this.restructTitulo = result;
                this.previewCollection.sourceCollection = this.restructTitulo;
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Error obtener información.", "danger")
            })
    }

    //Actualiza información de la reestructura.
    ActualizarReestructura() {
        this.context.app.showSpinner();
        this.restruct.ActualizaReestructura(this.restructDetalle)
            .then(result => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Se actualizó la información de reestructura.", "success")
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Error al actualizar información de reestructura, valida los datos.", "danger")
            })
    }
    //Muestra mensaje de error.
    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert("Error en reestructura maintenance.ts", "danger")
    }

    // Con base a la fila seleccionada se carga el detalle
    onSelectionChanged(args: any) {
        var currentItem: any = new getTituloReestructura;
        currentItem = this.previewCollection.currentItem;
        var tit: getTituloReestructura;
        tit = <getTituloReestructura>currentItem;
        this.selectedCollection.sourceCollection = tit.reestructuraDetalle;
    }
}



