﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormGroup, FormBuilder } from '@angular/forms';
import { Location } from '@angular/common';

import * as moment from 'moment';

//Azteca Kernel
import { Context, PolicyInfo, BaseComponent, ReporteInfo, ParametroInfo, ReportService} from 'framework3/core';
import { ProcessComponent } from 'framework3/core';
import { MenuItem, AlertsBarComponent, ToolbarButton, ContentViewerComponent } from 'framework3/controls';
import { FilterGroup, SearchRequest, SearchCondition } from 'framework3/search';
import { Menu } from 'wijmo/wijmo.input';
import { ReestructuraEditorService } from '../services/reestructura-editor-service';
import { FlexGrid, CellRange } from 'wijmo/wijmo.grid';
import { HitTestInfo } from 'wijmo/wijmo.chart';
import { Output } from '@angular/core';
import { optimizeGroupPlayer } from '@angular/animations/browser/src/render/shared';


@Component({
    selector: 'distribution-reestructura',
    templateUrl: './reestructura-editor.html'
})
export class ReestructuraEditor extends BaseComponent {

    form: FormGroup;
    filterPanelCollapsed: boolean = false;
    showFilterPanel: boolean = true;
    menuItems: MenuItem[] = [];
    filterGroups: FilterGroup[] = [];
    selectedGroup: FilterGroup;
    request: SearchRequest;
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    currentItems: Array<any> = new Array<any>();
    currentItem: any;
    totalRows: number = 0;
    pageIndex: number = 0;
    respuesta: boolean = false;
    idJobOrigen: number = 0;
    titleContentView: string = "";
    fileNameContentView: string = "";
    message: string = "";
    idRes: number = 0;
    

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private location: Location, private reportSvc: ReportService,
        private formBuilder: FormBuilder, private reestructura_editor_svc: ReestructuraEditorService) {
        super();
        this.form = formBuilder.group({
            'fechaInicial': [],
            'fechaFinal': []
        });

        this.route.params.forEach((params: Params) => {
            this.idJobOrigen = +params['origen'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.fechaInicial = moment().subtract(30, 'day').toDate();
            this.fechaFinal = moment().add(1, 'day').toDate();
            this.refreshList();
        });
        this.createMenu();
    }

    createMenu() {
        //Agregar opciones estándar del catálogo
        let mnuDownload: MenuItem = { icon: "glyphicon glyphicon-circle-arrow-down", text: "Consultar", name: "DOWNLOAD", smallText: "Consultar pendientes aprobar" };
        let mnuOpen: MenuItem = { icon: "glyphicon glyphicon-circle-arrow-down", text: "Abrir", name: "OPEN", smallText: "abrir en modo de sólo lectura" };
        //
        this.menuItems.push(mnuDownload);
        this.menuItems.push(mnuOpen);
        //
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    /*Permite tomar todo el espacio disponible cuando el panel de filtro no está visible y 75% cuando lo está*/
    get classGridPanel(): string {
        return !this.showFilterPanel || this.filterPanelCollapsed ? 'col-12' : 'col-12 col-lg-9';
    }

    toggleCollapse() {
        this.filterPanelCollapsed = !this.filterPanelCollapsed;
    }

    onMenuItemClicked(menu: Menu, args: any) {
        let item: MenuItem = this.menuItems[menu.selectedIndex];
        this.optionClick(item);
    }

  
    onParamsToolbar(button: ToolbarButton) {
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    refreshList() {
        this.context.app.showSpinner();
        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }
        let exist = false;

        this.request.conditions.forEach(r => {
            if (r.field == "fecha" && r.operator == ">=") {
                exist = true;
                r.value = moment(this.fechaInicial).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
            else if (r.field == "fecha" && r.operator == "<=") {
                exist = true;
                r.value = moment(this.fechaFinal).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
        });

        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fecha", operator: ">=", valueType: "datetime", value: moment(this.fechaInicial).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conFechaFin: SearchCondition = { field: "fecha", operator: "<=", valueType: "datetime", value: moment(this.fechaFinal).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            //let conJobOrigen: SearchCondition = { field: "idJobOrigen", operator: "=", valueType: "int", value: 40 };

            this.request.conditions.push(conFechaIni);
            this.request.conditions.push(conFechaFin);
            //this.request.conditions.push(conJobOrigen);   
        }

        this.reestructura_editor_svc.searchJobs(this.context.app.defaultSite.identity, this.request)
            .then(response => {
                response.rows.forEach(r => {
                    r['fecha'] = moment(r['fecha']).toDate();
                });
                this.currentItems = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = this.request.pageIndex;
                //
                var filterGroup = new FilterGroup();
                filterGroup.tag = "ALL";
                filterGroup.name = "Todos";
                filterGroup.rowCount = this.totalRows;
                this.filterGroups.length = 0;
                this.filterGroups.push(filterGroup);
                //
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }
    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    newItemHandler() {
        this.router.navigate(['new'], { relativeTo: this.route });
    }
    
    onClickSel(item: any)
    {
        //this.router.navigate(['new/5'], { relativeTo: this.route });
        this.currentItem = item;
        if (item.data.idReestructura > 0) {
            this.router.navigate(['new/' + item.data.idReestructura, { readonly: true }], { relativeTo: this.route });
            
        } else
        {
            this.alertsBar.showAlert("No contiene detalle de reestructura", 'danger');
        }
    }



    optionClick(option: MenuItem) {
        if (option.name == "informe") {
            this.context.app.showSpinner();
            let informe = new ReporteInfo();
            let parametroReestruc = new ParametroInfo();
            parametroReestruc.name = "IdReestructura";
            parametroReestruc.values.push(this.idRes.toString());
            informe.parametros.push(parametroReestruc);
            informe.path = "/Onili/Distribucion/InfoPostReestructura";
            informe.format = "excel";
            this.reportSvc.loadReporte(informe)
                .then(info => {
                    this.context.app.hideSpinner();
                    this.currentItem
                    this.titleContentView = "Reestructura Informe "+this.idRes.toString();
                    this.fileNameContentView = this.idRes.toString() + ".xls";
                    this.reportViewer.show("data:application/vnd.ms-excel;base64, " + info.content);
                })
                .catch(error => {
                    this.context.app.hideSpinner();
                    this.handleError(error);
                });
        }
    }

    onMenuItemClick(option: MenuItem)
    {
        alert("mensaje"+this.idRes)
        option = this.currentItem;
        if (option.name == "informe") {
            alert("informe")
        }
        else {
            alert("else")
        }
    }

    onCurrentItemChanged(item: any) {
        this.idRes = item.data.idReestructura
    }
}