﻿import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';

import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent, } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { ProcessComponent } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { TrackableCollection } from 'framework3/editors';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
//Wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import { Menu } from 'wijmo/wijmo.input';
import { forwardRef } from '@angular/core';
import { CollectionView, DateTime } from 'wijmo/wijmo';
//Features
import { DatasetInfo, ParameterItem, TranspilerInfo, DatasetService } from '../../translator/index';
import { ReestructuraTranspilerFormato } from '../models/reestructura-transpiler-formato';
import { ReestructuraService } from '../services/reestructura.service';
import { getTituloReestructura } from '../models/get-reestructura-titulo';
import { getReestructuraDetalle } from '../models/get-reestructura-detalle';
import { error } from '@angular/compiler/src/util';


@Component({
    selector: 'distribution-reestructura-distribucion',
    templateUrl: './reestructura-file.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => reestructurafile)
        }
    ]
})
export class reestructurafile {
    previewCollection: CollectionView = new CollectionView();
    selectedCollection: CollectionView = new CollectionView();
    form: FormGroup;
    res: ReestructuraTranspilerFormato = new ReestructuraTranspilerFormato();
    cvParametros: TrackableCollection;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('viewDialog') viewDialog: ModalDirective;
    constructor
        (
        private formBuilder: FormBuilder,
        private restruct: ReestructuraService,
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private datasetSvc: DatasetService
        ) {
        this.form = formBuilder.group({
            'Arch': '',
            'IdReestructura': 0,
            'IdCliente': 0,
            'IdReestructuraTit': 0,
            'Referencia': '',
            'CodigoSucursal': '',
            'CodigoZona': '',
            'CodigoRuta': '',
            'CodigoFrecuencia': '',
            'IdSucursalAnt': 0,
            'IdZonaAnt': 0,
            'IdFrecuenciaAnt': 0,
            'IdRutaAnt': 0,
            'Error': false,
            'ErrorText': '',
            'Nombre': '',
            'Destino': '',
            'Estatus': 'B',
            'UserCreate': '',
            'CreateDate': '',
            'LastUpdate': '',
            'PCUpdate': '',
            'idDataset': [AztecaValidators.selected],
            'fechaLiquidacion': [''],
            'IdClienteDestino': 0,
            'CodigoClientePixcua': '',
            'fechaIni': '',
            'fechaFin': ''
        });
    }
    //ngOnInit(): void {
    //    setTimeout(() => {
    //        this.onClickGuardar();
    //    });
    //}

    //Encola transpiler
    onClickGuardar() {
        if (this.res.file == null) {
            this.alerts.showAlert("Asegúrate de seleccionar un archivo Csv", "danger");
            throw "Seleccionar archivo.";
        }
        this.context.app.showSpinner();
        this.res.transpiler.datasetInput.parameters.forEach(param => {
            if (param.field.includes('FileContent')) {
                param.value = this.res.file.content;
            }
            else if (param.field.includes('Delimiter')) {
                param.value = ",";
            } else if (param.field.includes('FileName')) {
                param.value = this.res.file.originalFileName;
            }
        })

      

        this.restruct.encolarTranspiler(this.res)
            .then(result => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Se ha encolado con éxito", "success")
                this.delay()    
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Error al encolar, revisar excel Nota: Archivo CSV tiene que contener la palabra Pixcua", "danger")
            })
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert("Error en reestructura maintenance.ts", "danger")
    }

    delay()
    {
        setTimeout(() => {
            this.router.navigate(['/distribution/Reestructura/278'], { relativeTo: this.route });
        },
            500);
    }

    onTranspilerSelected(item: TranspilerInfo) {
        if (item) {
            this.res.transpiler = item;
            this.context.app.showSpinner();
            let promises = [];
            if (item.idDatasetInput > 0) {
                let promiseInput = this.datasetSvc.getDetailByID(item.idDatasetInput)
                    .then(info => {
                        this.res.transpiler.datasetInput = info
                        this.cvParametros.sourceCollection = info.parameters;
                        this.cvParametros.refresh();
                    })
                    .catch(error => "Error 1");
                promises.push(promiseInput);
            }
            if (item.idDatasetOutput > 0) {
                let promisOutput = this.datasetSvc.getDetailByID(item.idDatasetOutput)
                    .then(info => this.res.transpiler.datasetOutput = info)
                    .catch(error => "Error 2");
                promises.push(promisOutput);
            }
            Promise.all<void>(promises).then(() => {
                this.context.app.hideSpinner();
            });
        }
    }
}



