﻿import { Component, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//
import { Context } from 'framework3/core';
import { MapComponent, Position } from 'framework3/geolocation';
import { AlertsBarComponent } from 'framework3/controls';

//
import { TerritoryPlannerService } from '../services/territory-planner.service';
import { ClienteItem } from '../models/cliente';
import { CelulaInfo } from '../models/celula';
import { FrecuenciaInfo } from '../models/frecuencia';
import { FrecuencyOptions } from '../models/frecuency-options';
import { TerritoryOptions } from '../models/territory-options';
import { ClusteringResult } from '../models/clustering-result';
import { ClusterInfo } from '../models/culster';

@Component({
    selector: 'distribution-territory-planner',
    templateUrl: './territory-planner.component.html'
})
export class TerritoryPlannerComponent {
    center: Position;
    zoom: number;
    startPoint: Position;
    cellColor: string = '#1E90FF';
    strokeColor: string = '#1E9000';
    celulas: CelulaInfo[];
    clusters: ClusterInfo[] = [];
    noise: ClienteItem[] = [];
    selectedCell: CelulaInfo = new CelulaInfo();
    maxClientesFrecuencia: number = 65;
    form: FormGroup;
    //
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;


    constructor(private context: Context, private territoryService: TerritoryPlannerService, private formBuilder: FormBuilder) {
        this.center = new Position(20.67262, -103.289075);
        this.zoom = 12;
        this.startPoint = new Position(20.67262, -103.289075);
        //
        this.form = formBuilder.group({
            'maxClientesFrecuencia': [0]
        });
    }


    ngOnInit() {

    }

    onPolygonClick(cell: CelulaInfo, e: any) {
        this.selectedCell = cell;
    }

    onClusterClick(cluster: ClusterInfo) {

    }

    generarClusters() {
        let options = new TerritoryOptions();
        options.startPoint = this.startPoint;
        //
        options.limitZone.push(new Position(20.726575291034344, -103.34976196289062));
        options.limitZone.push(new Position(20.728822994637675, -103.23234558105469));
        options.limitZone.push(new Position(20.587795414459123, -103.23062896728516));
        options.limitZone.push(new Position(20.588759600194518, -103.34186553955078));
        //
        let misa = new FrecuencyOptions();
        misa.Color = '#1E90FF';
        misa.CustomerLimit = this.maxClientesFrecuencia;
        options.frecuencies.push(misa);
        //
        let luju = new FrecuencyOptions();
        luju.Color = '#2ECCFA';
        luju.CustomerLimit = this.maxClientesFrecuencia;
        options.frecuencies.push(luju);
        //
        let mavi = new FrecuencyOptions();
        mavi.Color = '#CECEF6';
        mavi.CustomerLimit = this.maxClientesFrecuencia;
        options.frecuencies.push(mavi);
        //
        this.context.app.showSpinner();

        this.territoryService.getClusters(options)
            .then(result => {
                this.context.app.hideSpinner();
                this.clusters = result.clusters;
                this.noise = result.noise;
            })
            .catch(error => this.handleError(error));
    }

    generarCelulas() {
        let options = new TerritoryOptions();
        options.startPoint = this.startPoint;
        //
        options.limitZone.push(new Position(20.726575291034344, -103.34976196289062));
        options.limitZone.push(new Position(20.728822994637675, -103.23234558105469));
        options.limitZone.push(new Position(20.587795414459123, -103.23062896728516));
        options.limitZone.push(new Position(20.588759600194518, -103.34186553955078));
        //
        let misa = new FrecuencyOptions();
        misa.Color = '#1E90FF';
        misa.CustomerLimit = this.maxClientesFrecuencia;
        options.frecuencies.push(misa);
        //
        let luju = new FrecuencyOptions();
        luju.Color = '#2ECCFA';
        luju.CustomerLimit = this.maxClientesFrecuencia;
        options.frecuencies.push(luju);
        //
        let mavi = new FrecuencyOptions();
        mavi.Color = '#CECEF6';
        mavi.CustomerLimit = this.maxClientesFrecuencia;
        options.frecuencies.push(mavi);
        //
        this.context.app.showSpinner();

        
        this.territoryService.calculateZones(options)
            .then(zones => {
                this.celulas = zones;
                this.selectedCell = this.celulas[0];
                this.context.app.hideSpinner();
                this.alerts.showAlert('Proceso de generación terminado con éxito.', 'success');
            })
            .catch(error => {
                this.handleError(error)
            });
    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'error');
    }

}