﻿import { Context, CatalogInfo, CatalogItem } from 'framework3/core';
import { TrackableItem } from 'framework3/core';
import { TipoConteoInfo } from '../models/tipo-conteo'

export class ValeTipoInfo extends CatalogInfo {
	nombre: string;
	tipoVigencia: string;
	prorrogasPermitidas: number;
	diasVigencia: number;
	respetarCredito: boolean;
	respetarDescuento: boolean;
	capturaRuta: boolean;
	pagosParcial: boolean;
	cajasMinimo: number;
	cajasMaximo: number;
	fechaInicio: Date;
	fechaFin: Date;
	fechaVigencia: Date;
	restriccionDia: boolean;
	lunes: boolean;
	martes: boolean;
	miercoles: boolean;
	jueves: boolean;
	viernes: boolean;
	sabado: boolean;
	domingo: boolean;
	restriccionRuta: boolean;
	restriccionCliente: boolean;
	restriccionProducto: boolean;
    restriccionEstVen: boolean;
    idTipoConteo: number;
    tipoConteo: TipoConteoInfo;

    prorrogas: ProrrogaValeTipo[];
    celulas: CelulaValeTipo[];
    clientes: ClienteValeTipo[];
    estructurasDeVenta: EstVenValeTipo[];
    productos: ProductoValeTipo[];
    rutas: RutaValeTipo[];

    constructor() {
        super();
        this.tipoConteo = new TipoConteoInfo();
        this.prorrogas = [];
        this.celulas = [];
        this.clientes = [];
        this.estructurasDeVenta = [];
        this.productos = [];
        this.rutas = [];
    }
}

export class ValeTipoItem implements CatalogItem {    
    id: number;
    codigo: string;
    nombre: string;
    activo: boolean;
    estatus?: string;
	selected?:boolean = false;
}

export class ProrrogaValeTipo extends TrackableItem {
    idValeProrroga: number;
    diasProrroga: number;
    autorizacion: boolean;
    orden: number;

    constructor() {
        super();
        this.idValeProrroga = 0;
        this.diasProrroga = 0;
        this.autorizacion = false;
        this.orden = 0;
    }

}

export class CelulaValeTipo extends TrackableItem {
    idCelula: number;
    nombre: string;
    codigo: string;

    constructor() {
        super();
        this.nombre = '';
        this.codigo = '';
    }
}

export class ClienteValeTipo extends TrackableItem {
    idCliente: number;
    nombre: string;
    codigo: string;

    constructor() {
        super();
        this.nombre = '';
        this.codigo = '';
    }
}

export class EstVenValeTipo extends TrackableItem {
    idEstVen: number;
    nombre: string;
    codigo: string;

    constructor() {
        super();
        this.nombre = '';
        this.codigo = '';
    }
}

export class ProductoValeTipo extends TrackableItem {
    idProducto: number;
    nombre: string;
    codigo: string;
    cantMinima: number;
    cantMaxima: number;

    constructor() {
        super();
        this.cantMinima = 0;
        this.cantMaxima = 0;
        this.nombre = '';
        this.codigo = '';
    }
}

export class RutaValeTipo extends TrackableItem {
    idRuta: number;
    codigo: string;
    tipoRuta: string;
    tipoRutaDesc: string;

    constructor() {
        super();
        this.codigo = '';
        this.tipoRuta = '';
        this.tipoRutaDesc = '';
    }
}


