﻿import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

import * as moment from 'moment';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { ProcessComponent } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { TrackableCollection } from 'framework3/editors';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';

//Wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import { Menu } from 'wijmo/wijmo.input';

//Features
import { CelulaArea } from '../models/celula-area';
import { DatasetInfo, ParameterItem, TranspilerInfo, DatasetService } from '../../translator/index';

import { CelulaService } from '../services/celula.service';

@Component({
    selector: 'celulas-load-area',
    templateUrl: './celulas-area.component.html'
})
export class CelulasAreaComponent extends ProcessComponent {

    editForm: FormGroup;
    celulaArea: CelulaArea = new CelulaArea();
    cvParametros: TrackableCollection;
    idCelula: number = 0;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, private location: Location, private celulaService: CelulaService,
        private datasetSvc: DatasetService) {
        super(context, router, route);
        this.editForm = formBuilder.group({
            'idDataset': [AztecaValidators.selected],
            archivo: [null, Validators.required]
        });
        this.cvParametros = new TrackableCollection();
        this.cvParametros.filter = (item: ParameterItem) => {
            return (!item.hidden && item.state != ChangeState.DELETED);
        }

        this.route.params.forEach(params => {
            this.idCelula = params['idCelula'];
        });

    }

    //onCreatingDocument() {
    //    //Cargar parámetros que se proporcionan cuando es un nuevo documento
    //    //this.route.params.forEach(params => {
    //    //    this.recepcion.idCentroRecepcion = +params['idCentroRecepcion'];
    //    //    this.recepcion.idOrdenTit = params['idOrden'];

    //    //});
     
    //}

    onTranspilerSelected(item: TranspilerInfo) {
        if (item) {
            this.celulaArea.transpiler = item;
            this.context.app.showSpinner();
            let promises = [];
            if (item.idDatasetInput > 0) {
                let promiseInput = this.datasetSvc.getDetailByID(item.idDatasetInput)
                    .then(info => {
                        this.celulaArea.transpiler.datasetInput = info
                        this.cvParametros.sourceCollection = info.parameters;
                        this.cvParametros.refresh();

                        let param:ParameterItem = info.parameters.find(x => x.label == "IdCelula") ;
                        param.value = this.idCelula.toString();

                    })
                    .catch(error => this.handleError(error));
                promises.push(promiseInput);
            }
            if (item.idDatasetOutput > 0) {
                let promisOutput = this.datasetSvc.getDetailByID(item.idDatasetOutput)
                    .then(info => this.celulaArea.transpiler.datasetOutput = info)
                    .catch(error => this.handleError(error));
                promises.push(promisOutput);
            }
            Promise.all<void>(promises).then(() => {
                this.context.app.hideSpinner();
            });
        }
    }

    closeEditor() {
        //this.router.navigate(['../'], { relativeTo: this.route });
        this.location.back();
    }

    onClickGuardar() {
        this.context.app.showSpinner();
        this.celulaService.updateAreaFromFile(this.celulaArea)
            .then(result => {
                this.context.app.hideSpinner();
                //this.router.navigate(['../'], { relativeTo: this.route });
                this.location.back();
            })
            .catch(error => {
                this.handleError(error);
            });
    }

    applyPolicy(policy: PolicyInfo) {
        console.log('Aplicar politica');
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}