﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem, PolicyInfo, UsuarioInfo, UsuarioItem, SucursalService, SucursalInfo } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search/index';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls/index';
//
import { UsuariosListComponent } from 'framework3/security/components/dialogs/usuarios-list.component';

@Component({
    selector: 'gamas-elementos',
    templateUrl: './gamas-elementos.component.html'
})
export class GamasElementosComponent extends BaseComponent {
    title: string = '';
    articulos: UsuarioItem[];
    idGama: number;
    gama: SucursalInfo;
    currentItem: CatalogItem;
    //
    @ViewChild(UsuariosListComponent) articulosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private sucursalService: SucursalService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idGama = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadGama());

    }

    ///////////////// METODOS ///////////////////////////////

    loadGama() {
        this.sucursalService.getDetailByID(this.idGama)
            .then(info => {
                this.gama = info;
                this.title = 'Miembros de la gama - ' + this.gama.nombre;
            })
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.sucursalService.getMembers(this.idGama)
            .then(usuarios => {
                this.articulos = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ///////////////// HANDLERS ///////////////////////////////
    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.articulosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedUsersChanged(newArticulos: UsuarioItem[]) {
        this.sucursalService.addMembers(this.idGama, newArticulos)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los articulos han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveArticulo() {
        this.context.app.showSpinner();
        this.sucursalService.removeMember(this.idGama, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.articulos = this.articulos.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    onCancel() {
        this.location.back();
    }
}