﻿import { Component, Input, Output, EventEmitter, forwardRef, SimpleChange, ViewChild } from '@angular/core';
import { ControlValueAccessor, Validators, NG_VALUE_ACCESSOR, FormGroup, FormBuilder } from '@angular/forms';
//
import { Context, CatalogItem, BaseComponent, PolicyInfo } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { BaseControl } from 'framework3/controls/models/base-control';
//Features
import { ProductosInfo } from '../models/productos';

@Component({
    selector: 'productos-info',
    templateUrl: './productos-info.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ProductosInfoComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => ProductosInfoComponent), multi: true
        }
    ],
    styles: [
        '.ng-valid-component { border-left: 1px solid #ccc!important;}'
    ]
})

export class ProductosInfoComponent extends BaseControl implements ControlValueAccessor {
    private form: FormGroup;
    //Indica si la información de la forma es válida
    get valid(): boolean {
        return this.form.valid;
    }

    //Inputs
    @Input() producto: ProductosInfo = new ProductosInfo();

    //Outputs
    @Output() productoChange = new EventEmitter<ProductosInfo>();
    @Output() validChange = new EventEmitter<boolean>();
    //Campos privados para enlazar a UI

    //
    constructor(private formBuilder: FormBuilder) {
        super();

        this.form = formBuilder.group({
            'espacioUtilizado': [0, Validators.required],
            'contenido': [0, Validators.required],
            'aplicaCambios': [false],
            'aplicaIntroduccion': [false],
            'aplicaBonificacion': [false]
        });
    }

    ngOnChanges(changes: SimpleChange) {

    }

    ngAfterViewInit() {
        this.form.statusChanges.subscribe(_ => {
            this.validChange.emit(this.form.valid);
        });
    }


    //////////////////////////////// METODOS ////////////////////////////////////////

    notifiyChange() {
        let newProducto: ProductosInfo = JSON.parse(JSON.stringify(this.producto));
        newProducto.espacioUtilizado = this.producto.espacioUtilizado;
        newProducto.contenido = this.producto.contenido;
        newProducto.aplicaCambios = this.producto.aplicaCambios;
        newProducto.aplicaIntroduccion = this.producto.aplicaIntroduccion;
        newProducto.aplicaBonificacion = this.producto.aplicaBonificacion;
        //
        this.productoChange.emit(newProducto);
        this.propagateChange(newProducto);
    }

    //////////////////////////////// HANDLERS ////////////////////////////////////////

    onAplicaCambiosChange(args: any) {
        this.notifiyChange();
    }
    onAplicaIntroduccionChange(args: any) {
        this.notifiyChange();
    }
    onAplicaBonificacionChange(args: any) {
        this.notifiyChange();
    }

    //////////////////////////////// CONTROL VALUE ACCESOR ////////////////////////////////////////

    writeValue(value: any) {

    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() {
    }


}