﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import { map } from 'rxjs/operators';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Locales
import { CargaFisicaInfo, CargaFisicaItem } from '../models/carga-fisica';
import { CargaFisicaDetail } from '../models/carga-fisica-detail';

@Injectable()
export class CargaFisicaService extends DocumentService<CargaFisicaDetail, CargaFisicaInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/distribution/cargas-fisicas');
    }

    create(): Promise<CargaFisicaInfo> {
        let newDoc = new CargaFisicaInfo();
        return Promise.resolve(newDoc);
    }

    getRutasReparto(idSucursal: number, fechaIni: string, fechaFin: string, porCargar: boolean): Promise<any> {
        let params = new HttpParams()
            .set("idSucursal", idSucursal.toString())
            .set("fechaIni", fechaIni)
            .set("fechaFin", fechaFin)
            .set("porCargar", porCargar.toString());
        return this.apiService.get<any>('get-rutas-reparto', params).toPromise();
    }

    getCargaFisicaDocumentos(idRutaReparto: number, fechaIni: string, fechaFin: string, porCargar: boolean): Promise<CargaFisicaDetail[]> {
        let params = new HttpParams()
            .set("idRutaReparto", idRutaReparto.toString())
            .set("fechaIni", fechaIni)
            .set("fechaFin", fechaFin)
            .set("porCargar", porCargar.toString());
        return this.apiService.get<CargaFisicaDetail[]>('get-carga-fisica-doctos', params)
            .pipe(map(info => {
                info.forEach(item => {
                    item.fechaDocumento = moment(item.fechaDocumento).toDate();
                    item.fechaApartado = moment(item.fechaApartado).toDate();
                    item.fechaCarga = moment(item.fechaCarga).toDate();
                });
                return info;
            }))
            .toPromise();
    }

    cargarDocumento(cargaFisica: CargaFisicaInfo): Promise<CargaFisicaInfo> {
        let params = new HttpParams();
        return this.apiService.post<CargaFisicaInfo>('cargar-documentos', params, cargaFisica).toPromise();
    }

    aplicarCargaDocumento(idDocumento: string, origen: string): Promise<CargaFisicaInfo> {
        let params = new HttpParams()
            .set("idDocumento", idDocumento)
            .set("origen", origen);
        return this.apiService.post<CargaFisicaInfo>('aplicar-carga-documento', params, null).toPromise();
    }

    cancelarAplicarCargaDocumento(idDocumento: string, origen: string): Promise<CargaFisicaInfo> {
        let params = new HttpParams()
            .set("idDocumento", idDocumento)
            .set("origen", origen);
        return this.apiService.post<CargaFisicaInfo>('cancelar-aplicar-carga-documento', params, null).toPromise();
    }
}