﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
import { map } from 'rxjs/operators';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { ClienteInfo,ClienteRutasItem } from '../models/cliente'
import { LoteImpresionCfdiItem } from '../models/lote-impresion-cfdi'
import { FacturaImpresionItem } from '../models/factura'

@Injectable()
export class ClienteService extends CatalogService<ClienteInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/distribucion/clientes');
    }

    create(): Promise<ClienteInfo> {
        return Promise.resolve(new ClienteInfo());
    }

    cancelarFacturas(fecha:Date, idSucursal:number): Promise<any> {
        let fechaLoc = moment(fecha).format();
        let params = new HttpParams()
            .append("fecha", fechaLoc)
            .append("idSucursal", idSucursal.toString());
        return this.apiService.get<any>("cancelar-facturas", params)
            .toPromise();
    }


    impresionLoteCFDI(fechaComprobante: Date, idSucursal: number, rfc: string, condicionesPago: string): Promise<any> {
        let fecha = moment(fechaComprobante).format();
        let params = new HttpParams()
            .append("fechaComprobantes", fecha)
            .append("idSucursal", idSucursal.toString())
            .append("rfc", rfc.toString())
            .append("condicionesPago", condicionesPago.toString());
        return this.apiService.get<any>("impresion-lote-cfdi", params)
            .toPromise();
    }
    
    loadLotesImpresionCFDI(idSucursal: number, fechaBusqueda: Date): Promise<LoteImpresionCfdiItem[]> {
        let fecha = moment(fechaBusqueda).format();
        let params = new HttpParams()
            .append("fechaLotes", fecha)
            .append("idSucursal", idSucursal.toString());
        return this.apiService.get<LoteImpresionCfdiItem[]>("lotes-cfdi", params)
            .pipe(map(response => {
                response.forEach(lote => {
                    lote.fechaComprobantes = moment(lote.fechaComprobantes).toDate();
                    lote.fechaLote = moment(lote.fechaLote).toDate();
                });
                return response;
            }))
            .toPromise();
    }

    getArchivosLoteImpresion(idLote: number): Promise<string[]> {
        let params = new HttpParams()
            .append("idLote", idLote.toString());
        return this.apiService.get<string[]>("archivos-lote-impresion", params)
            .toPromise();
    }

    getArchivosLoteImpresionMultiple(lotes: Array<number>): Promise<string[]> {
        let params = new HttpParams()
        return this.apiService.post<string[]>("archivos-lote-impresion-multiple", params,lotes)
            .toPromise();
    }

    getFacturasPeriodo(idSucursal: number, fechaIni: Date, fechaFin: Date): Promise<FacturaImpresionItem[]> {
        let params = new HttpParams()
            .append("idSucursal", idSucursal.toString())
            .append("fechaIni", moment(fechaIni).format())
            .append("fechaFin", moment(fechaFin).format());
        return this.apiService.get<FacturaImpresionItem[]>("facturas-impresion", params)
            .toPromise();
    }

    getRutas(idCliente: number): Promise<ClienteRutasItem[]> {
        let params = new HttpParams()
            .append("idCliente", idCliente.toString());
        return this.apiService.get<ClienteRutasItem[]>("clientes-rutas", params)
            .toPromise();
    }

}