﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import { map } from 'rxjs/operators';
import * as moment from 'moment';

//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Locales
import { CobranzaInfo as DistribucionCobranzaInfo, CobranzaItem as DistribucionCobranzaItem} from '../models/cobranza';
import { CobranzaDetail as DistribucionCobranzaDetail } from '../models/cobranza-detail';

@Injectable()
export class CobranzaService extends DocumentService<DistribucionCobranzaDetail, DistribucionCobranzaInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/distribucion/cobranzas');
    }

    create(): Promise<DistribucionCobranzaInfo> {
        let newDoc = new DistribucionCobranzaInfo();
        return Promise.resolve(newDoc);
    }

    getRutasReparto(idSucursal: number, fecha: string): Promise<any> {
        let params = new HttpParams()
            .set("idSucursal", idSucursal.toString())
            .set("fecha", fecha);
        return this.apiService.get<any>('get-rutas-reparto', params).toPromise();
    }

    getCobranzaDocumento(idRutaReparto: number, fecha: string): Promise<DistribucionCobranzaInfo>  {
        let params = new HttpParams()
            .set("idRutaReparto", idRutaReparto.toString())
            .set("fecha", fecha);
        return this.apiService.get<DistribucionCobranzaInfo>('get-cobranza-docto', params)
            .pipe(map(info => {

                info.documentos.forEach(documento => {
                    documento.fechaDocumento = moment(documento.fechaDocumento).toDate();
                });

                info.details.forEach(item => {
                    item.documentsDet.forEach(itemDoc => {
                        itemDoc.fechaDocumento = moment(itemDoc.fechaDocumento).toDate();
                    });
                });

                return info;
            }))
            .toPromise();
    }

    enqueueCobranza(cobranza: DistribucionCobranzaInfo): Promise<DistribucionCobranzaInfo> {
        let params = new HttpParams();
        return this.apiService.post<DistribucionCobranzaInfo>('enqueue-cobranza', params, cobranza).toPromise();
    }

}