﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';
//Locales
import { ConteoInfo, ConteoItem } from '../models/conteo';
import { ConteoDetail } from '../models/conteo-detail';
import { ConteoDocumento } from '../models/conteo-documento';

@Injectable()
export class ConteoService extends DocumentService<ConteoDetail, ConteoInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/distribucion/conteos');
    }

    create(): Promise<ConteoInfo> {
        let newDoc = new ConteoInfo();
        return Promise.resolve(newDoc);
    }


    loadDocumentosConteo(idRutaReparto: number, fecha: Date, idEmpresa: number): Promise<ConteoDocumento[]> {
        let params: HttpParams = new HttpParams()
            .set("idRutaReparto", idRutaReparto.toString())
            .set("fecha", moment(fecha).format())
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<ConteoDocumento[]>("load-documentos-conteo", params)
            .toPromise();

    }

    toConteoDetail(documentosConteo: ConteoDocumento[]): Promise<ConteoDetail[]> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<ConteoDetail[]>("to-conteo-detail", params, documentosConteo)
            .toPromise();
    }

    cerrar(fecha: Date, idSucursal: number): Promise<boolean> {
        let params = new HttpParams()
            .append("fecha", moment(fecha).format())
            .append("idSucursal", idSucursal.toString())
        return this.apiService.post<boolean>("cerrar", params)
            .toPromise();
    }

    cancelarCierre(fecha: Date, idSucursal: number): Promise<boolean> {
        let params = new HttpParams()
            .append("fecha", moment(fecha).format())
            .append("idSucursal", idSucursal.toString())
        return this.apiService.post<boolean>("cancelar-cierre", params)
            .toPromise();
    }

    enqueueConteo(conteo: ConteoInfo): Promise<ConteoInfo> {
        let params = new HttpParams();
        return this.apiService.post<ConteoInfo>('enqueue-conteo', params, conteo).toPromise();
    }

    isRutaProcesada(idRuta: number, fechaCobro: Date): Promise<boolean> {
        let params = new HttpParams()
            .append("idRutaReparto", idRuta.toString())
            .append("fechaCobro", moment(fechaCobro).format())
        return this.apiService.get<boolean>("ruta-procesada", params)
            .toPromise();
    }

    toPdf(conteo: ConteoInfo): Promise<any> {
        let params = new HttpParams();
        return this.apiService.post<FileLink>("to-pdf", params, conteo).toPromise();
    }


	/*
    sampleMethod(param1: number, param2: string): Promise<ConteoInfo> {
        let params = new HttpParams()
			.set('param1', param1.toString())
			.set('param2', param2);
		//
        return this.apiService.get<ConteoInfo>('sample-method', params)
            .toPromise();
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: ConteoInfo) {

    }
	*/



}