﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Locales
import { Empaquetado } from '../../inventory';
import { PedidoInfo, PedidoItem } from '../models/pedido';
import { PedidoDetail } from '../models/pedido-detail';

@Injectable()
export class PedidoService extends DocumentService<PedidoDetail, PedidoInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/distribucion/pedidos');
    }

    create(): Promise<PedidoInfo> {
        let newDoc = new PedidoInfo();
        return Promise.resolve(newDoc);
    }

    protected mapDocument(document: PedidoInfo) {
        document.details.filter(detail => detail.cantidadPTA > 0).map(detail => <PedidoDetail>detail).forEach(detail => {
            detail.fechaCaducidad = moment(detail.fechaCaducidad).toDate();
            //Si la unidad de medida es un paquete, separar unidades
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidadPTA, detail.unidadMedida.factor);
                detail.cantidad = cantidad.paquetes;
                detail.unidades = cantidad.unidades;
            }
            else {
                detail.unidades = 0;
            }
        });
    }
	/*
    sampleMethod(param1: number, param2: string): Promise<PedidoInfo> {
        let params = new HttpParams()
			.set('param1', param1.toString())
			.set('param2', param2);
		//
        return this.apiService.get<PedidoInfo>('sample-method', params)
            .toPromise();
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: PedidoInfo) {

    }
	*/



}