﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { RutasRepartoInfo } from '../models/rutas-reparto';
import { CelulaItem } from '../models/celula';
import { RutaConteoItem } from '../models/ruta-conteo-item';

@Injectable()
export class RutasRepartoService extends CatalogService<RutasRepartoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/distribucion/rutas-reparto');
    }

    create(): Promise<RutasRepartoInfo> {
        return Promise.resolve(new RutasRepartoInfo());
    }

	 ///////////////////////////// Celulas ////////////////////////////////////////////////

    LoadCelulasList(idRutaReparto: number): Promise<CelulaItem[]> {
        let params = new HttpParams()
            .set('idRutaReparto', idRutaReparto.toString());
        return this.apiService.get<CelulaItem[]>("load-celulas-list", params)
            .toPromise();
    }

    addMembers(idRutaReparto: number, celulas: CelulaItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idRutaReparto', idRutaReparto.toString());
        return this.apiService.post<any>('add-members', params, celulas)
            .toPromise();
    }

    removeMember(idRutaReparto: number, idCelula: number): Promise<any> {
        let params = new HttpParams()
            .set('idRutaReparto', idRutaReparto.toString())
            .set('idCelula', idCelula.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

    rutasPendientesConteo(idSucursal: number, fecha: Date): Promise<RutaConteoItem[]> {
        let params = new HttpParams()
            .set('idSucursal', idSucursal.toString())
            .set('fecha', moment(fecha).format());
        return this.apiService.get<RutaConteoItem[]>("rutas-conteo-pendientes", params)
            .toPromise();


    }
}

