﻿import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import 'rxjs/add/operator/toPromise';
import { Context, WebService, BaseService } from 'framework3/core';
//
import { TerritoryOptions } from '../models/territory-options';
import { FrecuencyOptions } from '../models/frecuency-options';
import { ClienteItem } from '../models/cliente';
import { CelulaInfo } from '../models/celula';
import { ClusteringResult } from '../models/clustering-result';

@Injectable()
export class TerritoryPlannerService extends BaseService {

    constructor(protected http: HttpClient, protected context: Context) {
        super(http, context, 'api/distribucion/territories');
    }

    getCustomers(path: google.maps.LatLng[]): Promise<ClienteItem[]> {
        //Crear un array que se pueda transformar a json
        let jsPath: { lat: number, lng: number }[] = [];
        path.forEach(point => {
            jsPath.push({ lat: point.lat(), lng: point.lng() });
        });
        //                
        let params = new HttpParams()
        return this.apiService.post<ClienteItem[]>('customers', params, jsPath)
            .toPromise();
    }

    calculateZones(options: TerritoryOptions): Promise<CelulaInfo[]> {
        let params = new HttpParams()
        return this.apiService.post<CelulaInfo[]>('calculate-zones', params, options)
            .toPromise();
    }

    getClusters(options: TerritoryOptions): Promise<ClusteringResult> {
        return this.apiService.post<ClusteringResult>('clusters', null, options)
            .toPromise();
    }

}