﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, ContentViewerComponent } from 'framework3/controls';
//Locales

import { ComprobanteFiscalInfo, ComprobanteFiscalItem, ComprobanteFiscalService } from '../../accounting';
import { PortalContext } from '../services/portal-context';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
import { ReciboAdamService } from '../../rh/services/recibo-adam.service';
import { NominaAdamInfo, ConsultaAsistenciaInfo, AsistenciaInfo } from '../../rh/models/recibo-nomina-adam';
import { ReciboNominaService } from '../../rh/services/recibo-nomina.service';
import { ChecadorInfo, ChecadorService, AdamNominaInfo } from '../../rh/index';
import { CatalogManager } from 'framework3/editors';
import { EmpleadoInfo } from '../index';
import { HttpParams } from '@angular/common/http';
import { ConsultaAsistenciaEmpleadoModelInfo } from '../../accounting/models/ConsultaAsistenciaEmpleadoModel';
import { Console } from '@angular/core/src/console';

@Component({
    selector: 'clientes-download-invoice',
    templateUrl: './download-invoice.component.html'
})
export class DownloadInvoiceComponent extends CatalogManager {
    form: FormGroup;
    menuItems: MenuItem[] = [];
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    idDocumento: string = '';
    request: SearchRequest;
    pageSize: number = 20;
    pageIndex: number = 0;
    maxRows: number = 3000;
    totalRows: number = 0;
    currentRecibos: Array<any> = new Array<any>();
    filterRecibos: Array<any> = new Array<any>();
    selectedGroup: FilterGroup;
    filterGroups: FilterGroup[] = [];
    fileNameContentView: string = "";
    titleContentView: string = "";
    asistencias: NominaAdamInfo[] = [];
    a: AsistenciaInfo[] = [];

    Consult: ConsultaAsistenciaEmpleadoModelInfo;

    currentComprobante = new ComprobanteFiscalInfo();

    //
    @ViewChild('viewDialog') viewDialog: ModalDirective;
    @ViewChild(DocumentListComponent) documentList: DocumentListComponent;
    @ViewChild(ContentViewerComponent) contentViewerComponent: ContentViewerComponent;


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder,
        private comprobanteService: ComprobanteFiscalService, private portalContext: PortalContext, private reciboAdamService: ReciboAdamService, private reciboNominaSvc: ReciboNominaService) {
        super(context, router, route);
        //Crear formulario de parámetros
        //, private check:ChecadorService
        this.form = formBuilder.group({
            'fechaInicial': [new Date()],
            'fechaFinal': [new Date()]
        });
    }
    ngOnInit() {
    }
    ngAfterViewInit() {
        this.fechaInicial = moment().subtract(15, 'day').toDate();
        this.fechaFinal = moment().add(1, 'day').toDate();
        setTimeout(_ => this.refreshList());
    }

    /////////////////////// METODOS ////////////////////////////////////

    refreshList()
    {
        this.context.app.showSpinner();
        this.filterRecibos = new Array<any>();

        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }

        let exist = false;
        this.request.conditions.forEach(r => {
            if (r.field == "fechaCreacion" && r.operator == ">=") {
                exist = true;
                r.value = moment(this.fechaInicial).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
            else if (r.field == "fechaCreacion" && r.operator == "<=") {
                exist = true;
                r.value = moment(this.fechaFinal).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
        });

        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fechaCreacion", operator: ">=", valueType: "datetime", value: moment(this.fechaInicial).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conFechaFin: SearchCondition = { field: "fechaCreacion", operator: "<=", valueType: "datetime", value: moment(this.fechaFinal).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let sitio: SearchCondition = { field: "sitio", operator: "=", valueType: "string", value: "sitioemp" };
            this.request.conditions.push(conFechaIni);
            this.request.conditions.push(conFechaFin);
            this.request.conditions.push(sitio);
        }

        this.reciboAdamService.search(this.portalContext.empleado.idBranch, this.request)
            .then(response =>
            {

                response.rows.forEach(r => {
                    if (r.numTrabajador === this.portalContext.empleado.code) {
                        this.filterRecibos.push(r)
                    }
                });

                response.rows.forEach(r => {
                    r['fechaCreacion'] = moment(r['fechaCreacion']).toDate();
                });

                this.currentRecibos = this.filterRecibos;
                this.totalRows = this.currentRecibos.length;
                //this.pageIndex = this.request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }
    //Consulta asistencias de los empleados en un rango de fechas
    dataLoad()
    {
        this.context.app.showSpinner();
        let Codigo = this.portalContext.empleado.code;
        this.reciboAdamService.consultarAsistencias(Codigo,this.fechaInicial,this.fechaFinal)
            .then(info => {
                this.context.app.hideSpinner();
                this.a = info;
                console.log(JSON.stringify(info));
            })
            .catch(error => this.handleError(error));
    }
    //Metodo para ver comprobante.
    viewComprobante() {
        this.context.app.showSpinner();
        this.comprobanteService.getDetail(this.documentList.currentDocument.id)
            .then(comprobante => {
                this.currentComprobante = comprobante;
                this.context.app.hideSpinner();
                this.viewDialog.config.backdrop = false;
                this.viewDialog.show();
            })
            .catch(error => this.handleError(error));
    }
    /////////////////////// HANDLERS //////////////////////////////////
    onMainToolbar(button: ToolbarButton) {
        if (button.name == 'DOWNLOAD' && this.idDocumento != '')
        {
            this.context.app.showSpinner();
            //Generar PDF
            this.reciboNominaSvc.printReciboPdf(this.idDocumento)
                .then(response => {
                    this.titleContentView = "Recibo " + this.portalContext.empleado.code;
                    this.fileNameContentView = this.portalContext.empleado.code + ".pdf";
                    this.contentViewerComponent.show("data:application/pdf; base64, " + response);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    onValidateMenu() {

    }

    currentItemChangedHandler(item: any) {
        if (item != null) {
            this.currentRecibos.forEach(itemRecibo => {
                if (itemRecibo.id === item.id) {
                    this.idDocumento = itemRecibo.idReciboNomina;
                }
            });
        }
        else {
            this.idDocumento = '';
        }
    }

    handleError(error: any) {
        console.log(error);
        let message = this.context.getExceptionMessage(error);
        this.context.app.hideSpinner();
    }

}