﻿import { Component, EventEmitter, Input, Output, ViewChild, OnInit } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import * as moment from 'moment';
//
import { Context, UsuarioInfo } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
//

import { EmpleadoService as EmpleadoSvc } from '../services/empleado.service'

import { PortalContext } from '../services/portal-context';
import { VALID } from '@angular/forms/src/model';

@Component({
    selector: 'empleados-login',
    templateUrl: './login.component.html'
})
export class LoginComponent {
    loginForm: FormGroup;

    error: string = "";
    codigo: string = "";
    contrasena: string = "";
    fechaNac: Date = new Date();

    constructor(private router: Router, private context: Context, private formBuilder: FormBuilder,
        private portalContext: PortalContext, private empleadosvc: EmpleadoSvc) {

        this.loginForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'fechaNac': [''],
            'contrasena': [''] 
        });
    }

    ngOnInit() {
        
    }

    checkLogin() {
        this.context.app.showSpinner();
        this.empleadosvc.loginEmpleado(this.codigo, moment(this.fechaNac).format("YYYY-MM-DD"), this.contrasena)
            .then(empleado => {
                this.context.app.hideSpinner();
                if (empleado.session != null) {
                    this.context.setSession(empleado.session);
                    this.portalContext.setEmpleado(empleado.empleado);
                    this.router.navigate(['empleados/download']);
                }
                else {
                    this.error = "Nombre de usuario o contraseña incorrecta, acuda a RH para modificarla";
                }
            })
            .catch(error => this.handleError(error));
        
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        console.log(error);
        this.error = this.context.getExceptionMessage(error);
    }
    
}