﻿import { Component, ViewChild, Input } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { AlmacenEditarStock } from '../components/almacen-editar-stock.component';

import { AlmacenService } from '../services/almacen.service';

import { StockInfo } from '../models/stock';

@Component({
    selector: 'inventario-articulos-stock',
    templateUrl: './almacen-articulos-stock.html'
})
export class AlmacenArticulosStock extends BaseComponent {
    articulosStock: StockInfo[] = [];
    idAlmacen: number;
    currentItem: CatalogItem;

    @ViewChild(AlmacenEditarStock) articulosStockDialog: AlmacenEditarStock;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    @Input() data: StockInfo = new StockInfo();
    //
    constructor(private context: Context, private almacenService: AlmacenService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idAlmacen = +param['idAlmacen'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadArticulosStock());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadArticulosStock() {
        this.context.app.showSpinner();
        this.almacenService.loadStockAlmacen(this.idAlmacen)
            .then(articulosStock => {
                this.articulosStock = articulosStock;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.articulosStockDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.loadArticulosStock();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onEditItem(Stock: StockInfo) {
        this.articulosStockDialog.show();
        this.articulosStockDialog.data = Stock;
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    actualizarStock(arg: StockInfo) {
        this.almacenService.updateStock(this.idAlmacen, arg)
            .then(r => {
                this.loadArticulosStock()
            })
            .catch(error => this.handleError(error));
    }

}