﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';
//Features
import { AlmacenService} from '../services/almacen.service';
import { AlmacenInfo, AlmacenItem} from '../models/almacen';

@Component({
    selector: 'inventario-almacenes-editor',
    templateUrl: './almacenes.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AlmacenesEditor)
        },
        {
            provide: CatalogService, useExisting: AlmacenService
        }
    ]
})
export class AlmacenesEditor extends CatalogEditor {
    idArea: number = 0;
    almacen: AlmacenInfo = new AlmacenInfo();        
    clasificadores: Clasificacion[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private almacenService: AlmacenService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'Negativos': [''],
            'InventarioFisico': [''],
            'RestriccionUsuario': [''],
            'RestriccionArticulos': [''],
            'clasificaciones': [[], AztecaValidators.validClasifications]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        //Cargar catálogos
        this.loadClasificadores();
    }

    ///////////// METODOS /////////////////////////////////
    loadClasificadores() {
        this.almacenService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => this.clasificadores = clasificadores)
            .catch(error => this.handleError(error));
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idArea = +params['idArea'];
        });
    }

    onCreatingItem() {
        this.almacen.idArea = this.idArea;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.almacenService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.almacen = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.almacenService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.almacen = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.almacenService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.almacen = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.almacenService.update(this.almacen)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.almacen.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.almacen = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'USERS') {
            this.router.navigate(['inventory/almacenes-usuarios/' + this.process.id + '/' + this.almacen.identity.toString()]);
        }
        else if (item.name == 'ALLOWARTICLES') {
            this.router.navigate(['inventory/almacenes-clasificaciones-articulos/' + this.process.id + '/' + this.almacen.identity.toString()]);
        }
        else if (item.name == 'LOCATIONS') {
            this.router.navigate(['inventory/ubicaciones/' + this.process.id + '/' + this.almacen.identity.toString()]);
        }
        else if (item.name == 'EXISTENCES') {
            this.router.navigate(['inventory/almacenes-existencias/' + this.process.id + '/' + this.almacen.identity.toString()]);
        }
        else if (item.name == 'STOCK') {
            this.router.navigate(['inventory/almacenes-stock/' + this.process.id + '/' + this.almacen.identity.toString()]);
        }
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }
}