﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { ToolbarButton, AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { AlmacenService} from '../services/almacen.service';
import { AlmacenInfo, AlmacenItem} from '../models/almacen';

@Component({    
    selector: 'inventario-almacenes-manager',
    templateUrl: './almacenes.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AlmacenesManager)
        },
        {
            provide: CatalogService, useExisting: AlmacenService
        }
	]
})
export class AlmacenesManager extends CatalogManager {
    almacenes: AlmacenItem[] = [];
    idArea: number;
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private almacenService: AlmacenService, private location: Location) {
        super(context, router, route);
        this.route.params.subscribe((params: Params) => {
            this.idArea = +params['idArea'];
        });

    }

	///////////////////////// METODOS ///////////////////////////////////////

    loadAlmacenes() {
    }

	///////////////////////// HANDLERS ///////////////////////////////////////

    ngOnInit() {
        this.loadAlmacenes();
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();
        let condition: SearchCondition = { field: 'idArea', operator: '=', valueType: 'int', value: this.idArea };
        request.conditions.push(condition);

        this.almacenService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.almacenes = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.almacenService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.almacenService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem, selectedItem: CatalogItem) {
        if (item.name == 'USERS') {
            this.router.navigate(['inventory/almacenes-usuarios/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == 'ALLOWARTICLES') {
            this.router.navigate(['inventory/almacenes-clasificaciones-articulos/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == 'LOCATIONS') {
            this.router.navigate(['inventory/ubicaciones/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == 'EXISTENCES') {
            this.router.navigate(['inventory/almacenes-existencias/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == 'STOCK') {
            this.router.navigate(['inventory/almacenes-stock/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == 'PROVISION') {
            this.router.navigate(['inventory/almacenes-provisiones/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }


}
