﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { AreaService} from '../services/area.service';
import { AreaInfo, AreaItem} from '../models/area';

@Component({
    selector: 'inventario-areas-editor',
    templateUrl: './areas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AreasEditor)
        },
        {
            provide: CatalogService, useExisting: AreaService
        }
    ]
})
export class AreasEditor extends CatalogEditor {   
    area: AreaInfo = new AreaInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private areaService: AreaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        this.context.app.showSpinner();

        this.areaService.LoadTurnos(0)
            .then(info => {
                this.context.app.hideSpinner();
                this.area.turnos = info;
            })
            .catch(error => this.handleError(error));
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.areaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.area = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.areaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.area = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.areaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.area = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        //guardar la información del area
        this.areaService.update(this.area)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.area.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.area = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'WAREHOUSE') {
            this.router.navigate(['inventory/almacenes/' + this.process.id + '/' + this.area.identity.toString()]);
        }
        else if (item.name == 'USERS') {
            this.router.navigate(['inventory/areas-usuarios/' + this.process.id + '/' + this.area.identity.toString()]);
        }
        /*else if (item.name == 'SHIFT') {

        }*/
    }

}