﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { AreaService} from '../services/area.service';
import { AreaInfo, AreaItem} from '../models/area';

@Component({    
    selector: 'inventario-areas-manager',
    templateUrl: './areas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AreasManager)
        },
        {
            provide: CatalogService, useExisting: AreaService
        }
	]
})
export class AreasManager extends CatalogManager {
    areas: AreaItem[]=[];    
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private areaService: AreaService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.areaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.areas = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.areaService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.areaService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'WAREHOUSE') {
            this.router.navigate(['inventory/almacenes/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == 'USERS') {
            this.router.navigate(['inventory/areas-usuarios/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
    }

}
