﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';

import { AlmacenService } from '../services/almacen.service';

import { Existencias } from '../models/almacen';

@Component({
    selector: 'inventario-articulo-existencias',
    templateUrl: './articulo-existencias.html'
})
export class ArticuloExistencias extends BaseComponent {
    articuloExistencias: Existencias[] = [];
    idArticulo: number;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private almacenService: AlmacenService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idArticulo = +param['idArticulo'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadExistencias());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadExistencias() {
        this.context.app.showSpinner();
        this.almacenService.loadExistenciasbyArt(this.idArticulo, this.context.app.defaultSite.identity)
            .then(almacenExistencias => {
                this.articuloExistencias = almacenExistencias;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'REFRESH') {
            this.loadExistencias();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}