﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { ArticuloStockService } from '../services/articulo-stock.service';
import { ArticuloStockInfo, ArticuloStockItem } from '../models/articulo-stock';
import { MenuItem } from "framework3/controls";

@Component({    
    selector: 'inventory-articulos-stock-manager',
    templateUrl: './articulos-stock.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticulosStockManager)
        },
        {
            provide: CatalogService, useExisting: ArticuloStockService
        }
	]
})
export class ArticulosStockManager extends CatalogManager {
    articulosstock: ArticuloStockItem[];    
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private articuloStockService: ArticuloStockService) {
        super(context, router, route);
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.articuloStockService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.articulosstock = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.articuloStockService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.articuloStockService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'CONFIG') {
            this.router.navigate(['inventory/articulos-stock-configuracion/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == 'PICS') {
            this.router.navigate(['inventory/articulos-fotografias/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
        else if (item.name == 'STOCK') {
            this.router.navigate(['inventory/articulos-existencias/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
    }

}
