﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';

//Features
import { ConceptoService} from '../services/concepto.service';
import { ConceptoInfo, ConceptoItem} from '../models/concepto';

@Component({
    selector: 'inventario-concepto-editor',
    templateUrl: './conceptos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ConceptosEditor)
        },
        {
            provide: CatalogService, useExisting: ConceptoService
        }
    ]
})
export class ConceptosEditor extends CatalogEditor {
    concepto: ConceptoInfo = new ConceptoInfo();
    clasificadores: Clasificacion[] = [];
    tipos = [{id: '', nombre: '[Seleccione tipo]'}, { id: 'E', nombre: 'Entrada' }, { id: 'S', nombre: 'Salida' }, { id: 'T', nombre: 'Traspaso' }];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private conceptoService: ConceptoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'tipo': ['', Validators.required],
            'diasVigencia': [0],
            'requiereApartado': [false],
            'requiereTransporte': [false],
            'consumirReserva':[false],
            'diasCancelacion': [0],
            'clasificaciones': [[], AztecaValidators.validClasifications]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    loadClasificadores() {
        this.conceptoService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => this.clasificadores = clasificadores)
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadClasificadores();
        this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.conceptoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.concepto = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.conceptoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.concepto = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.conceptoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.concepto = info;
                if (info.automatico) this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.conceptoService.update(this.concepto)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.concepto.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.concepto = info;
            })
            .catch(error => this.handleError(error));        
    }

    onTabSelect(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['users'], { relativeTo: this.route });
        }
    }


}