﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { UbicacionInfo } from '../models/ubicacion';
import { TiposUbicacionItem } from '../models/tipo-ubicacion';
import { UbicacionService } from '../services/ubicacion.service';
import { TipoUbicacionService } from '../services/tipo-ubicacion.service';

@Component({
    selector: 'inventario-ubicaciones-manager',
    templateUrl: './ubicaciones.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => UbicacionesManager)
        },
        {
            provide: CatalogService, useExisting: TipoUbicacionService
        }
    ]
})
export class UbicacionesManager extends HierarchyManager {
    ubicaciones: UbicacionInfo[] = [];
    ubicacionEdicion: UbicacionInfo = new UbicacionInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    idAlmacen: number = 0
    tiposUbicacion: TiposUbicacionItem[] = [];

    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private ubicacionService: UbicacionService, private tipoUbicacionService: TipoUbicacionService,
        protected router: Router, protected route: ActivatedRoute) {
        super(context, router, route, changeDetector);

        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'tipoUbicacion': [''],
            'idTipoUbicacion': [0, AztecaValidators.selected]
        });
    }

    ////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.ubicacionService.loadJerarquia(this.context.app.defaultSite.identity, this.idAlmacen)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.ubicaciones = nodes;
            })
            .catch(error => this.handleError(error));
    }


    //////////////////// HANDLERS //////////////////////////////////////

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idAlmacen = +param['idAlmacen'];
        });
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadHierarchy();
        this.loadTiposUbicaciones();
        //Hacer llamadas de carga inicial de datos aquí

    }

    loadTiposUbicaciones() {
        this.tipoUbicacionService.loadList()
            .then(tipos => this.tiposUbicacion = tipos)
            .catch(error => this.handleError(error));
    }

    onNodesChanged(ubicaciones: UbicacionInfo[]) {
        this.ubicaciones = ubicaciones;
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.ubicacionEdicion = new UbicacionInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.ubicacionEdicion = new UbicacionInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.ubicacionEdicion.idBranch = this.context.app.defaultSite.identity;
        this.ubicacionEdicion.idCompany = this.context.app.defaultCompany.identity;
        this.ubicacionEdicion.idImpresora = 0;
        this.ubicacionEdicion.idCuenta = 0;
        this.ubicacionEdicion.idAlmacen = this.idAlmacen;
        //
        this.currentParent = parentNode;
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {
        this.ubicacionEdicion = this.ubicacionService.copy(node.data);
        this.showEditor();
    }

    onSavingNode() {
        //Si es un nuevo nodo
        if (this.ubicacionEdicion.clientID == 0) {
            this.addNode(this.ubicacionEdicion, this.currentParent);
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.ubicacionEdicion);
        }
        this.ubicacionEdicion = new UbicacionInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<UbicacionInfo>) {
        this.context.app.showSpinner();
        this.ubicacionService.saveHierarchy(changes)
            .then(ubicaciones => {
                this.ubicaciones = ubicaciones;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}