﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { UnidadMedidaTipoService } from '../services/unidad-medida-tipo.service';
import { UnidadMedidaTipoInfo } from '../models/unidad-medida-tipo';
import { UnidadMedidaItem } from '../models/unidad-medida';
import { UnidadMedidaService } from '../services/unidad-medida.service';

@Component({
    selector: 'inventario-unidades-medida-tipo-editor',
    templateUrl: './unidades-medida-tipo.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => UnidadesMedidaTipoEditor)
        },
        {
            provide: CatalogService, useExisting: UnidadMedidaTipoService
        }
    ]
})
export class UnidadesMedidaTipoEditor extends CatalogEditor {
    unidadMedidaTipo: UnidadMedidaTipoInfo = new UnidadMedidaTipoInfo();        
    unidadesMedida: UnidadMedidaItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private unidadMedidaTipoService: UnidadMedidaTipoService,
        private unidadMedidaService: UnidadMedidaService) {

        super(context,router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required]            
        });
    }

    ///////// METODOS //////////////////////

    loadUnidadesMedida() {
        this.unidadMedidaService.loadList(this.unidadMedidaTipo.identity)
            .then(unidadesMedida => this.unidadesMedida = unidadesMedida)
            .catch(error => this.handleError(error));
    }


    ////////// HANDLERS /////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.unidadMedidaTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.unidadMedidaTipo = info;                
                this.setReadonlyMode();
                this.loadUnidadesMedida();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.unidadMedidaTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.unidadMedidaTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.unidadMedidaTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.unidadMedidaTipo = info;
                this.loadUnidadesMedida();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.unidadMedidaTipoService.update(this.unidadMedidaTipo)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.unidadMedidaTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.unidadMedidaTipo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onUnidadesUpdated() {
        this.loadUnidadesMedida();
    }

}