﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { UnidadMedidaService } from '../services/unidad-medida.service';
import { UnidadMedidaInfo } from '../models/unidad-medida';

@Component({
    selector: 'inventario-unidades-medida-editor',
    templateUrl: './unidades-medida.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => UnidadesMedidaEditor)
        },
        {
            provide: CatalogService, useExisting: UnidadMedidaService
        }
    ]
})
export class UnidadesMedidaEditor extends CatalogEditor {
    idTipoUnidad: number = 0;
    unidadMedida: UnidadMedidaInfo = new UnidadMedidaInfo();
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private unidadMedidaService: UnidadMedidaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'abreviatura': ['', Validators.required],            
            'factor': [1],
            'categoriaFiscal': [0],
            'base': [false],
            'fraccionable': [false],
            'paquete': [false]
        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idTipoUnidad = +params['idTipo'];
        });
    }


    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCancelingEdit() {        
        this.location.back();
    }

    onCreatingItem() {
        this.unidadMedida.idUnidadMedidaTipo = this.idTipoUnidad
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.unidadMedidaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.unidadMedida = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.unidadMedidaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.unidadMedida = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.unidadMedidaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.unidadMedida = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        //Si la unidad es fraccionable o es la unidad base, desmarcar bandera de paquete
        if (this.unidadMedida.fraccionable || this.unidadMedida.base) {
            this.unidadMedida.paquete = false;
        }
        //
        this.unidadMedidaService.update(this.unidadMedida)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.unidadMedida.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                    this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                else {
                    if (args.closeEditor)
                        this.location.back();
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.unidadMedida = info;
            })
            .catch(error => this.handleError(error));        
    }

}