﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, Output, EventEmitter } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { UnidadMedidaService} from '../services/unidad-medida.service';
import { UnidadMedidaInfo, UnidadMedidaItem} from '../models/unidad-medida';

@Component({    
    selector: 'inventario-unidades-medida-manager',
    templateUrl: './unidades-medida.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => UnidadesMedidaManager)
        },
        {
            provide: CatalogService, useExisting: UnidadMedidaService
        }
	]
})
export class UnidadesMedidaManager extends CatalogManager {    
    @Input() idTipoUnidad: number = 0;
    @Input() unidadesMedida: UnidadMedidaItem[] = [];
    @Output() listUpdated: EventEmitter<void> = new EventEmitter<void>();
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private unidadMedidaService: UnidadMedidaService) {
        super(context, router, route);
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
    }

    doSearch(request: SearchRequest) {

    }

    onAddingItem() {
        this.router.navigate(['inventory/unidades-medida/' + this.process.id + '/' + this.idTipoUnidad + '/0']);
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate(['inventory/unidades-medida/' + this.process.id + '/' + item.data.idUnidadMedidaTipo + '/' + item.id, { readonly: true }]);
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate(['inventory/unidades-medida/' + this.process.id + '/' + item.data.idUnidadMedidaTipo + '/' + item.id]);
    }

    onDeletingItem(item: CatalogItem) {
        this.unidadMedidaService.delete(item.id)
            .then(success => {
                this.listUpdated.emit();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate(['inventory/unidades-medida/' + this.process.id + '/' + this.idTipoUnidad + '/'+ 0, { cloneId: item.id }]);
    }

    onChangingItemState(item: CatalogItem) {
        this.unidadMedidaService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.listUpdated.emit();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
