﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection, CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { StockInfo } from '../models/stock';
import { TiposInventarioItem } from '../models/tipo-inventario';
import { AlmacenService } from '../services/almacen.service';
import { ArticuloStockService } from '../services/articulo-stock.service';
import { ArticuloStockSelectorComponent } from '../../inventory';

@Component({
    selector: 'inventario-almacen-editar-stock',
    templateUrl: './almacen-editar-stock.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AlmacenEditarStock)
        },
        {
            provide: CatalogService, useExisting: AlmacenService
        }
    ]
})
export class AlmacenEditarStock extends CatalogEditor {
    editorForm: FormGroup;
    idAlmacen: number;
    articulosStock: StockInfo = new StockInfo();
    tiposInventariocbo: TiposInventarioItem[] = [];
    tiposValuacioncbo: { id: string, name: string }[] = [
        { "id": "N", "name": "Ninguno" },
        { "id": "F", "name": "PEPS" },
        { "id": "L", "name": "UEPS" }
    ];
    //Inputs
    @Input() data: StockInfo = new StockInfo();
    //outputs
    @Output() dataUpdated: EventEmitter<StockInfo> = new EventEmitter<StockInfo>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('selectorArticulo') selectorArticulo: ArticuloStockSelectorComponent;

    constructor(private almacenService: AlmacenService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetector: ChangeDetectorRef, private articulosStockService: ArticuloStockService) {
        super(context, router, route, changeDetector);
        this.editorForm = formBuilder.group({
            'articulo': ['', AztecaValidators.selected],
            'stockMinimo': [''],
            'stockMaximo': [''],
            'puntoReorden': [''],
            'reserva': [0],
            'ubicaciones': [''],
            'tiposInventario': [0, AztecaValidators.selected],
            'controlSerie': [''],
            'controlLotes': [''],
            'controlCaducidad': [''],
            'tiposValuacion': [0, AztecaValidators.selected]
        });

        this.editorForm.controls['controlLotes'].valueChanges.subscribe(value => {
            if (value == true) {
                this.editorForm.controls['tiposValuacion'].setValidators(Validators.required);
            }
            else {
                this.editorForm.controls['tiposValuacion'].setValidators(Validators.nullValidator);
                this.editorForm.controls['tiposValuacion'].setValue('');
            }
            this.editorForm.updateValueAndValidity();
        });

    }

    show() {
        this.articulosStock = new StockInfo();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => {
            this.editorForm.reset();
            this.articulosStock = this.data;
            this.data = new StockInfo();
        });
        Control.invalidateAll();
    }

    ngOnInit() {
        this.loadTiposInventario();
        this.route.params.forEach(param => {
            this.idAlmacen = +param['idAlmacen'];
        });
    }

    loadTiposInventario() {
        this.articulosStockService.loadList()
            .then(tipos => this.tiposInventariocbo = tipos)
            .catch(error => this.handleError(error));
    }

    onUpdate() {
        this.dataUpdated.emit(this.articulosStock);
        this.dialog.hide();
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}