﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
//import { UsuariosListComponent } from 'framework3/security/components/dialogs/usuarios-list.component';
import { AlmacenListComponent }from '../dialogs/almacenes-list.component'

import { AlmacenService } from '../services/almacen.service';

import { AlmacenProvisionItem } from '../models/almacen';

@Component({
    selector: 'inventario-almacen-provisiones',
    templateUrl: './almacen-provision.component.html'
})
export class AlmacenProvisionesComponent extends BaseComponent {
    almacenes: AlmacenProvisionItem[];
    /*grupo: GrupoInfo = new GrupoInfo();*/
    idAlmacen: number;
    currentItem: CatalogItem;

    almacenesCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(AlmacenListComponent) almacenesDialog: AlmacenListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private almacenService: AlmacenService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idAlmacen = +param['idAlmacen'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadAlmacen());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////
    /*loadGrupo() {
        this.grupoService.getDetailByID(1)
            .then(grupo => this.grupo = grupo)
            .catch(error => this.handleError(error));
    }*/

    loadAlmacen() {
        this.almacenService.LoadAlmacenesProvision(this.idAlmacen)
            .then(usuarios => this.almacenes = usuarios)
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.almacenService.LoadAlmacenesProvision(this.idAlmacen)
            .then(almacenes => {
                this.almacenes = almacenes;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ////////////saveUserList() {
    ////////////    this.context.app.showSpinner();
    ////////////    this.almacenService.updateMembers(this.idAlmacen, this.usuarios)
    ////////////        .then(_ => {
    ////////////            this.context.app.hideSpinner();
    ////////////            this.alerts.showAlert('Los cambios han sido agregados con éxito.', 'success');
    ////////////            this.refreshList();
    ////////////        })
    ////////////        .catch(error => this.handleError(error));
    ////////////}

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SAVE') {

            //this.saveUserList();
        }
        if (button.name == 'ADD') {
            this.almacenesDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }


    onSelectedUsersChanged(newAlmacenes: AlmacenProvisionItem[]) {
        this.almacenService.addAlmacenesProvision(this.idAlmacen, newAlmacenes)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los almacenes han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser(item) {
        this.context.app.showSpinner();
        let thisitem = item;
        this.almacenService.removeAlmacenProvision(this.idAlmacen, item.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.almacenes = this.almacenes.filter(item => item.id != thisitem.id);
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}