﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { UsuariosListComponent } from 'framework3/security/components/dialogs/usuarios-list.component';

import { AlmacenService } from '../services/almacen.service';

import { AlmacenUsuarioItem } from '../models/almacen';

@Component({
    selector: 'inventario-area-usuarios',
    templateUrl: './almacen-usuarios.component.html'
})
export class AlmacenUsuariosComponent extends BaseComponent {
    usuarios: AlmacenUsuarioItem[];
    /*grupo: GrupoInfo = new GrupoInfo();*/
    idAlmacen: number;
    currentItem: CatalogItem;

    usuariosCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private almacenService: AlmacenService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idAlmacen = +param['idAlmacen'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadAlmacen());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////
    /*loadGrupo() {
        this.grupoService.getDetailByID(1)
            .then(grupo => this.grupo = grupo)
            .catch(error => this.handleError(error));
    }*/

    loadAlmacen() {
        this.almacenService.LoadAccessUsersList(this.idAlmacen)
            .then(usuarios => this.usuarios = usuarios)
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.almacenService.LoadAccessUsersList(this.idAlmacen)
            .then(usuarios => {
                this.usuarios = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    saveUserList() {
        this.context.app.showSpinner();
        this.almacenService.updateMembers(this.idAlmacen, this.usuarios)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los cambios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SAVE') {
            this.saveUserList();
        }
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }


    onSelectedUsersChanged(newUsers: AlmacenUsuarioItem[]) {
        this.almacenService.addMembers(this.idAlmacen, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser(item) {
        this.context.app.showSpinner();
        let thisitem = item;
        this.almacenService.removeMember(this.idAlmacen, item.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != thisitem.id);
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}