﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { AreaInfo, AreaItem } from '../models/area';
import { AreaService } from '../services/area.service';

@Component({
    selector: 'inventory-area-selector',
    templateUrl: './area-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => AreasSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => AreasSelectorComponent),
        multi: true
    }]

})
export class AreasSelectorComponent extends CatalogSelector {
    area: any[];
    selectedItem: AreaInfo = new AreaInfo;
    //
    @Output() selectedItemChanged: EventEmitter<AreaInfo> = new EventEmitter<AreaInfo>();

    constructor(protected context: Context, private areaService: AreaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findAreas(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.areaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.areaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.area = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findAreas(keyword: string): Observable<CatalogItem[]> {
        return this.areaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

