﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService} from 'framework3/core';
import { UsuariosListComponent } from 'framework3/security/components/dialogs/usuarios-list.component';

import { AreaService } from '../services/area.service';

@Component({
    selector: 'inventario-area-usuarios',
    templateUrl: './area-usuarios.component.html'
})
export class AreasUsuariosComponent extends BaseComponent {
    usuarios: UsuarioItem[];
    /*grupo: GrupoInfo = new GrupoInfo();*/
    idArea: number;
    currentItem: CatalogItem;
    /*get title(): string {
        return 'Miembros del grupo - ' + this.grupo.nombre;
    }*/
    //
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private areaService: AreaService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idArea = +param['idArea'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadArea());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////
    /*loadGrupo() {
        this.grupoService.getDetailByID(1)
            .then(grupo => this.grupo = grupo)
            .catch(error => this.handleError(error));
    }*/

    loadArea() {
        this.areaService.LoadAccessUsersList(this.idArea)
            .then(usuarios => this.usuarios = usuarios)
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.areaService.LoadAccessUsersList(this.idArea)
            .then(usuarios => {
                this.usuarios = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }


    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.areaService.addMembers(this.idArea, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser() {
        this.context.app.showSpinner();
        this.areaService.removeMember(this.idArea, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}