﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import { ProcessComponent, Context } from 'framework3/core';
import { TrackableCollection } from 'framework3/editors';
import { PolicyInfo } from 'framework3/core';
import { Menu } from 'wijmo/wijmo.input';

import { AztecaValidators, ToolbarButton } from 'framework3/controls';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Features
import { ArticulosFotografia } from '../models/articulo-stock';
import { ArticuloStockService } from '../services/articulo-stock.service';

@Component({
    selector: 'inventario-articulos-fotografias',
    templateUrl: './articulos-fotografias.component.html'
})


export class ArticulosFotografias extends ProcessComponent {

    @ViewChild('dialog') dialog: ModalDirective;

    fotografiaItem: ArticulosFotografia = new ArticulosFotografia;
    fotografias: ArticulosFotografia[] = [];
    fotografiasCollection: TrackableCollection = new TrackableCollection();
    fileSelected: string = '';
    idArticulo: number = 0;

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private articuloStockService: ArticuloStockService, private location: Location) {
        super(context, router, route);
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idArticulo = params['idArticulo'];
        });
    }

    applyPolicy(policy: PolicyInfo) {

    }

    ngAfterViewInit() {
        this.llenandoGrid(this.idArticulo);

    }

    llenandoGrid(idArticulo: number) {
        this.articuloStockService.loadFotografias(idArticulo)
            .then(docs => {
                this.fotografias = docs
                this.fotografiasCollection.sourceCollection = this.fotografias;
            })
            .catch(error => this.handleError(error));
    }


    agregarFotografia(idArticulo: number, articulosFotografia: ArticulosFotografia) {
        this.articuloStockService.agregarFotografia(idArticulo, articulosFotografia)
            .then(r => {
                this.llenandoGrid(this.idArticulo);
            })
            .catch(error => this.handleError(error));
    }


    show() {
        this.dialog.config.backdrop = false;
        this.dialog.config.ignoreBackdropClick = true;
        this.dialog.config.animated = true;
        this.dialog.show();
    }

    handleError(error: any) {
        console.log = error;
    }

    onUpdate() {
        this.fotografiaItem.archivo = this.fileSelected;
        this.agregarFotografia(this.idArticulo, this.fotografiaItem);
        this.dialog.hide();
    }


    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.onUpdate();
            this.fileSelected = '';
            this.fotografiaItem.descripcion = '';
            //this.show();
        }
        if (button.name == 'DEL' && this.fotografiasCollection.currentItem != null) {
            this.articuloStockService.eliminarFotografia(this.idArticulo, this.fotografiasCollection.currentItem.idArchivo)
                .then(r => {
                    this.llenandoGrid(this.idArticulo);
                })
                .catch(error => this.handleError(error));
        }
        if (button.name == 'BACK') {
            this.location.back();
        }
    }
}
