﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { ClasificacionesArticulosComponent } from './clasificaciones-articulos-list.component';

import { AlmacenService } from '../services/almacen.service';

import { ClasificacionesArticulosItem } from '../models/almacen';

@Component({
    selector: 'inventario-articulos-permitidos',
    templateUrl: './articulos-permitidos.component.html'
})
export class ArticulosPermitidosComponent extends BaseComponent {
    articulosPermitidos: ClasificacionesArticulosItem[] = [];
    idAlmacen: number;
    currentItem: CatalogItem;

    usuariosCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(ClasificacionesArticulosComponent) articulosPermitidosDialog: ClasificacionesArticulosComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private almacenService: AlmacenService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idAlmacen = +param['idAlmacen'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadArticulosPermitidos());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadArticulosPermitidos() {
        this.almacenService.LoadClasificacionesPermitidas(this.idAlmacen)
            .then(articulosPermitidos => {
                this.articulosPermitidos = articulosPermitidos;
            })
            .catch(error => this.handleError(error));
    }

    actualizarClasificaciones(data: ClasificacionesArticulosItem[]) {
        this.context.app.showSpinner();
        this.almacenService.updateClasificacionesArticulos(this.idAlmacen, data)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los cambios han sido agregados con éxito.', 'success');
                this.loadArticulosPermitidos();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.articulosPermitidosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.loadArticulosPermitidos();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onRemoveClasificacion(item) {
        this.context.app.showSpinner();
        let thisitem = item;
        this.almacenService.removeClasificacionesArticulos(this.idAlmacen, item.idClasificacion)
            .then(_ => {
                this.context.app.hideSpinner();
                this.articulosPermitidos = this.articulosPermitidos.filter(item => item.idClasificacion != thisitem.idClasificacion);
                this.loadArticulosPermitidos();

            })
            .catch(error => this.handleError(error));
    }



    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}