﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection } from 'framework3/editors';
//Features
import { ClasificacionesArticulosItem } from '../models/almacen';
import { AlmacenService } from '../services/almacen.service';
import { PolicyInfo } from 'framework3/core';
import { Control } from 'wijmo/wijmo';

@Component({
    selector: 'inventario-clasificaciones-articulos',
    templateUrl: './clasificaciones-articulos-list.component.html'
})
export class ClasificacionesArticulosComponent extends BaseComponent {
    editorForm: FormGroup;
    clasificacionesArticulosItem: ClasificacionesArticulosItem[] = [];
    //Inputs
    @Input() data: ClasificacionesArticulosItem[] = [];
    //outputs
    @Output() dataUpdated: EventEmitter<ClasificacionesArticulosItem[]> = new EventEmitter<ClasificacionesArticulosItem[]>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private almacenService: AlmacenService, private context: Context, private formBuilder: FormBuilder) {
        super();
        this.editorForm = formBuilder.group({
            /*'parametro': ['', Validators.required],
            'xmlPath': ['', Validators.required]*/
        });
    }

    show() {
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => this.editorForm.reset());
        Control.invalidateAll();
    }

    ngOnInit() {
        this.almacenService.LoadClasificacionesArticulos()
            .then(clasificacionesArticulosItem => {
                this.clasificacionesArticulosItem = clasificacionesArticulosItem;
                this.data = clasificacionesArticulosItem;
            })
            .catch(error => this.handleError(error));
    }

    onUpdate() {
        this.dataUpdated.emit(this.data);
        this.dialog.hide();
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}