﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//
import { BaseComponent, Context, CatalogItem, PolicyInfo, UsuarioItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { UsuariosListComponent } from 'framework3/security';
//
import { ConceptoInfo } from '../models/concepto';
import { ConceptoService } from '../services/concepto.service';

@Component({
    selector: 'inventory-concepto-usuarios',
    templateUrl: './concepto-usuarios.component.html'
})
export class ConceptoUsuariosComponent extends BaseComponent {
    usuarios: UsuarioItem[]=[];
    idConcepto: number;
    concepto = new ConceptoInfo();
    currentItem: CatalogItem;
    get title(): string {
        return 'Accesos al concepto - ' + this.concepto.nombre;
    }
    //
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private conceptoService: ConceptoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idConcepto = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadConcepto());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////
    loadConcepto() {
        this.conceptoService.getDetailByID(this.idConcepto)
            .then(concepto => this.concepto = concepto)
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.conceptoService.getUsers(this.idConcepto)
            .then(usuarios => {
                this.usuarios = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.conceptoService.addUsers(this.idConcepto, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser() {
        this.context.app.showSpinner();
        this.conceptoService.removeUser(this.idConcepto, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}