﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
import * as moment from 'moment';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { AreaService } from '../services/area.service';
import { AreaItem, TurnoItem } from '../models/area';

@Component({
    selector: 'inventario-corte',
    templateUrl: './corte.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CortesComponent)
        },
        {
            provide: CatalogService, useExisting: AreaService
        }
    ]
})
export class CortesComponent extends CatalogEditor {
    form: FormGroup;
    areascbo: AreaItem[] = [];
    turnoscbo: TurnoItem[] = [];
    corteActivo: string = 'No se a seleccionado area';
    corteActivoBool: boolean = false;
    ajustarInventario: boolean = false;
    fechaApertura: Date;
    idCorte: string;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetector: ChangeDetectorRef, private areaService: AreaService) {
        super(context, router, route, changeDetector);
        this.form = formBuilder.group({
            'area': ['', AztecaValidators.selected],
            'turno': [''],
            'corteActivo': [''],
            'fechaApertura': [''],
            'ajustarInventario': ['']
        });
    }

    ngOnInit() {
        this.loadAreas();
    }

    loadAreas() {
        this.areaService.loadListByUser(this.context.app.defaultSite.identity, this.context.session.user.identity)
            .then(x => {
                this.areascbo = x;
            })
            .catch(error => this.handleError(error));
    }

    loadCortes(idArea: number) {
        this.areaService.getDetailByID(idArea)
            .then(x => {
                if (x.idCorteActual == '00000000-0000-0000-0000-000000000000') {
                    this.corteActivo = 'No existe corte actual';
                    this.corteActivoBool = false;
                }
                else {
                    this.areaService.LoadCorteActual(idArea)
                        .then(x => {
                            let fecha: string = moment(x.fechaApertura).format();
                            this.fechaApertura = moment(fecha).toDate();
                            this.idCorte = x.idCorte;
                            this.form.controls.turno.patchValue(x.idTurno);
                            this.corteActivoBool = true;
                            this.corteActivo = 'Información del corte actual';
                        })
                        .catch(error => this.handleError(error))
                }

            })
            .catch(error => this.handleError(error));
    }

    loadTurnos(idArea: number) {
        this.areaService.LoadTurnos(idArea)
            .then(x => {
                if (x) {
                    this.ajustarInventario = false;
                    this.turnoscbo = x.filter(turno => turno.selected == true);
                }
            })
            .catch(error => this.handleError(error));
    }

    onItemChanged(args: number) {
        if (args != null) {
            this.form.controls.turno.patchValue('');
            this.loadTurnos(args);
            this.loadCortes(args);
        }
    }

    onAbrirCorte() {
        this.areaService.AbrirCorte(this.form.controls.area.value, this.form.controls.turno.value, this.context.session.user.identity)
            .then(x => {
                this.onItemChanged(this.form.controls.area.value)
                this.alerts.showAlert('El corte se a abierto con éxito', 'success');
            })
            .catch(error => this.handleError(error))
    }

    onCerrarCorte() {
        this.areaService.CerrarCorte(this.idCorte, this.ajustarInventario, this.context.session.user.identity)
            .then(x => {
                this.onItemChanged(this.form.controls.area.value)
                this.alerts.showAlert('El corte se a cerrado con éxito', 'success');
            })
            .catch(error => this.handleError(error))
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}