﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { AlmacenInfo, AlmacenItem} from '../models/almacen';
import { AlmacenService } from '../services/almacen.service';

@Component({
    selector: 'inventory-almacenes-selector',
    templateUrl: './almacenes-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => AlmacenesSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => AlmacenesSelectorComponent),
        multi: true
    }]
})
export class AlmacenesSelectorComponent extends CatalogSelector {
    almacenes: any[];
    selectedItem: AlmacenInfo = new AlmacenInfo;
    @Input() area: number = 0;

    constructor(protected context: Context, private almacenService: AlmacenService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findAlmacenes(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.almacenService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        if (this.area != 0) {
            let condicionArea: SearchCondition = { field: "idArea", operator: "=", valueType: "integer", value: this.area }
            request.conditions.push(condicionArea);
        }
        
        this.context.app.showSpinner();
        this.almacenService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.almacenes = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findAlmacenes(keyword: string): Observable<CatalogItem[]> {
        return this.almacenService.searchKeywordAlmacenes(this.context.app.defaultSite.identity, keyword)
            .map(almacenes => {
                if (this.area > 0) {
                    return almacenes.filter(almacen => almacen.idArea === this.area);
                } else {
                    return almacenes
                }
            
        });
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

