﻿import { Component, Input, Output, EventEmitter, SimpleChange } from '@angular/core';
//Wijmo
import { CollectionView, NotifyCollectionChangedEventArgs, NotifyCollectionChangedAction } from 'wijmo/wijmo';
//Framework3
import { Context, ChangeState } from 'framework3/core';
//Inventario
import { LoteInfo } from '../models/lote';
import { StockInfo } from '../models/stock';
import { ArticuloStockInfo } from '../models/articulo-stock';
import { UnidadMedidaInfo } from '../models/unidad-medida';
import { ArticuloStockService } from '../services/articulo-stock.service';
import { AlmacenService } from '../services/almacen.service';

@Component({
    selector: 'inventory-articulo-lotes',
    templateUrl: './articulo-lotes.component.html'
})
export class ArticuloLotesComponent {
    private lotes: LoteInfo[] = [];
    private collectionView = new CollectionView();
    /////////// INPUTS
    @Input() idAlmacen: number = 0;
    @Input() idArticuloStock: number = 0;
    @Input() unidadMedida: UnidadMedidaInfo;
    @Input() stockConfig: StockInfo;
    @Input() height: number = 250;

    /////////// OUTPUTS
    @Output() lotesChange = new EventEmitter<LoteInfo[]>();
    @Output() error = new EventEmitter<any>();

    constructor(private context: Context, private articuloService: ArticuloStockService) {
        this.collectionView.collectionChanged.addHandler((sender: any, args: NotifyCollectionChangedEventArgs) => {
            if (args.action == NotifyCollectionChangedAction.Change && args.item != null) {
                this.lotesChange.emit(this.lotes);
            }
        });
    }

    ngOnChanges(changes: SimpleChange) {
        if (changes['idAlmacen'] || changes['idArticuloStock'] || changes['unidadMedida']) {
            if (this.idAlmacen > 0 && this.unidadMedida && this.idArticuloStock > 0) {
                this.loadLotes();
            }
        }
    }

    /////////// METHODS

    loadLotes() {
        this.context.app.showSpinner();
        this.articuloService.loadExistenciasLote(this.idArticuloStock, this.idAlmacen)
            .then(lotes => {
                this.lotes = lotes;
                this.collectionView.sourceCollection = this.lotes;
                this.lotesChange.emit(this.lotes);
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    count(): number {
        return this.lotes.length;
    }

    quantityFormat(): string {
        if (this.unidadMedida && !this.unidadMedida.paquete && this.unidadMedida.fraccionable) {
            return 'n3';
        }
        else {
            return null;
        }
    }

    /////////// HANDLERS

    handleError(error: any) {
        console.log(error);
        this.error.emit(error);
        this.context.app.hideSpinner();
    }
}
