﻿import { Component, Input, Output, EventEmitter, ElementRef, ViewChild } from '@angular/core';

//Wijmo
import { CollectionView } from 'wijmo/wijmo';
//Framework3
import { Context } from 'framework3/core';
//Inventario
//Models
import { SerieInfo } from '../models/serie';
//Services
import { ArticuloStockService } from '../services/articulo-stock.service';

@Component({
    selector: 'inventario-articulo-series',
    templateUrl: './articulo-series.component.html'
})
export class ArticuloSeriesComponent {
    private series: SerieInfo[] = [];
    private serie: string = '';
    private seriesCollection = new CollectionView();
    //
    @ViewChild('txtSerie') serieInput: ElementRef;

    /////////// INPUTS
    @Input() idArticuloStock: number;
    @Input() idAlmacen: number;

    /////////// OUTPUS
    @Output() seriesChange = new EventEmitter<SerieInfo[]>();

    constructor(private context: Context, private articuloService: ArticuloStockService) {

    }

    /////////// METHODS
    clear() {
        this.series = [];
        this.seriesCollection.sourceCollection = this.series;
        this.serieInput.nativeElement.focus();
        this.seriesChange.emit(this.series);
    }

    addSerie() {
        //Verificar que la serie tenga contenido
        if (this.serie && this.serie != '') {
            this.context.app.showSpinner();
            this.articuloService.getSerieInfo(this.idArticuloStock, this.idAlmacen, this.serie)
                .then(info => {                    
                    if (info) {
                        this.updateSeries(info);
                    }
                    else {
                        let newSerie = new SerieInfo();
                        //Crear un nuevo item para la serie que se agregó
                        newSerie.numeroSerie = this.serie;
                        newSerie.idArticuloStock = this.idArticuloStock;
                        newSerie.estatus = 'N';
                        this.updateSeries(newSerie);
                    }
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    updateSeries(newSerie: SerieInfo) {
        //Agregar a la colección
        let newSeries = this.series.slice();
        newSeries.push(newSerie);
        //Limpiar control para una posible nueva captura y establecer foco en el textbox
        this.serie = '';
        this.serieInput.nativeElement.focus();
        this.series = newSeries;
        //Actualizar grid y emitir evento de que las series han cambiado
        this.seriesCollection.sourceCollection = newSeries;
        this.seriesChange.emit(this.series);
    }

    count(): number {
        return this.series.length;
    }

    /////////// HANDLERS

    onAddSerie() {
        this.addSerie();
    }

    onSerieKeyup(args: any) {
        //Si se presiona ENTER, ejecutar adición de elemento
        if (args.keyCode == 13) {
            this.addSerie();
        }
    }

    onRowDelete(row: SerieInfo) {
        this.seriesCollection.remove(row);
        this.seriesChange.emit(this.series);
    }

    handleError(error: any) {
        console.log(error);
        this.context.app.hideSpinner();
    }
}
