﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { ArticuloStockInfo, ArticuloStockItem } from '../models/articulo-stock';
import { ArticuloStockService } from '../services/articulo-stock.service';

@Component({
    selector: 'inventory-articulo-stock-selector',
    templateUrl: './articulo-stock-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ArticuloStockSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ArticuloStockSelectorComponent),
        multi: true
    }]
})
export class ArticuloStockSelectorComponent extends CatalogSelector {
    articulosstock: any[];
    selectedItem: ArticuloStockInfo = new ArticuloStockInfo;
    //
    @Output() selectedItemChanged = new EventEmitter<ArticuloStockInfo>();
    //
    constructor(protected context: Context, private articuloStockService: ArticuloStockService) {
        super(context);
        
        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findArticulosStock(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.articuloStockService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + ' - ' + info.nombre;
                    this.selectedItemChanged.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = '';
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.articuloStockService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.articulosstock = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findArticulosStock(keyword: string): Observable<CatalogItem[]> {
        return this.articuloStockService.searchKeyword2(this.context.session.user.identity, this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

