﻿import { Component, Input, Output, EventEmitter, SimpleChange } from '@angular/core';
import { ControlValueAccessor, FormControlName, NG_VALUE_ACCESSOR } from '@angular/forms';
//Framework3
import { BaseControl } from 'framework3/controls';
//Inventario
import { UnidadMedidaInfo, UnidadMedidaItem } from '../models/unidad-medida';

@Component({
    selector: 'inventory-quantity-input',
    templateUrl: './quantity-input.component.html'
})
export class QuantityInputComponent extends BaseControl implements ControlValueAccessor {

    /////////// INPUTS
    @Input() unidadMedida: UnidadMedidaInfo | UnidadMedidaItem;
    @Input() cantidad: number = 0;
    @Input() unidades: number = 0;
    @Input() enabledCantidad: boolean = true;

    //Formato del campo cantidad, cuando la UM sea piezas el formato sera entero, en caso contrario se permitiran decimales
    get quantityFormat(): string {
        if (this.unidadMedida && this.unidadMedida.fraccionable) {
            return null;
        }
        else {
            return 'n0';
        }
    }
    get quantityClass(): string {
        if (this.unidadMedida && this.unidadMedida.paquete) {
            return 'col-12 col-lg-6';
        }
        else {
            return 'col-12';
        }
    }
    //Obtener el valor minimo dependiendo de si la unidad es fraccionable o no
    get minQuantity(): number {
        if (this.unidadMedida && this.unidadMedida.fraccionable) {
            return 0.0001;;
        }
        else if (this.unidadMedida && this.unidadMedida.paquete) {
            return 0;
        }
        else {
            return 1;
        }
    }
    //Obtener la cantidad absoluta que servira para validar el registro (cantidad > 0)
    get total(): number {
        if (this.unidadMedida) {
            return (this.cantidad * this.unidadMedida.factor) + this.unidades;
        }
        else {
            return 0;
        }
    }


    get totalDecimal(): number {
        if (this.unidadMedida) {
            return this.cantidad + (this.unidades/ this.unidadMedida.factor );
        }
        else {
            return 0;
        }
    }
    /////////// OUTPUS
    @Output() cantidadChange = new EventEmitter<number>();
    @Output() unidadesChange = new EventEmitter<number>();

    constructor() {
        super();
    }

    ngOnChanges(changes: SimpleChange) {
        if (this.unidadMedida && changes['unidadMedida']) {
            if (!this.unidadMedida.paquete) {
                this.unidades = 0;
                this.unidadesChange.emit(0);
                this.propagateChange(0);
            }
            else {
                this.propagateChange(this.total);
            }
        }
    }

    /////////// METHODS

    /////////// HANDLERS
    onCantidadChange(value: number) {
        if (value != this.cantidad) {
            this.cantidad = value;
            this.cantidadChange.emit(value);
            this.propagateChange(this.total);
        }
    }

    onUnidadesChange(value: number) {
        if (this.unidades != value) {
            this.unidades = value;
            this.unidadesChange.emit(value);
            this.propagateChange(this.total);
        }
    }

    //Implementación de control value accesor
    writeValue(value: any) {
    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() { }


}
