﻿import { Component, Input, Output, ViewChild, EventEmitter, SimpleChange } from '@angular/core';
import { ControlValueAccessor, FormControlName, NG_VALUE_ACCESSOR } from '@angular/forms';
//Framework3
import { BaseControl, AzLabelComponent } from 'framework3/controls';
//Inventario
import { UnidadMedidaInfo, UnidadMedidaItem } from '../models/unidad-medida';
import { Empaquetado } from '../../inventory';

@Component({
    selector: 'inventory-quantity-label',
    templateUrl: './quantity-label.component.html'
})

export class QuantityLabelComponent extends BaseControl {


    @ViewChild('lblCantidad') lblCantidad: AzLabelComponent;
    @ViewChild('lblUnidades') lblUnidades: AzLabelComponent;


    private cantidad: number;
    private unidadMedida: UnidadMedidaInfo | UnidadMedidaItem;

    //Formato del campo cantidad, cuando la UM sea piezas el formato sera entero, en caso contrario se permitiran decimales
    get quantityFormat(): string {
        if (this.unidadMedida && this.unidadMedida.fraccionable) {
            return null;
        }
        else {
            return 'n0';
        }
    }
    get quantityClass(): string {
        if (this.unidadMedida && this.unidadMedida.paquete) {
            return 'col-12 col-lg-6';
        }
        else {
            return 'col-12';
        }
    }

    Show(cantidad: number, unidadMedida: UnidadMedidaInfo | UnidadMedidaItem) {
        this.cantidad = cantidad;
        this.unidadMedida = unidadMedida;

        setTimeout(_ => this.setInfo(), 1000);

    }

    setInfo() {
        let paquete = new Empaquetado(this.cantidad, this.unidadMedida.factor);
        if (this.lblCantidad)
            this.lblCantidad.text = "";

        if (this.lblUnidades)
            this.lblUnidades.text = "";

        if (this.unidadMedida && this.unidadMedida.fraccionable) {
            this.lblCantidad.text = paquete.cantidad.toString();
        }
        else {
            this.lblCantidad.text = paquete.paquetes.toFixed(0).toString();
        }

        if (this.lblUnidades)
            this.lblUnidades.text = paquete.unidades.toFixed(0).toString();

    }

}