﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { UbicacionInfo, UbicacionItem } from '../models/ubicacion';
import { UbicacionService } from '../services/ubicacion.service';

@Component({
    selector: 'inventory-ubicaciones-selector',
    templateUrl: './ubicaciones-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => UbicacionesSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => UbicacionesSelectorComponent),
        multi: true
    }]
})
export class UbicacionesSelectorComponent extends CatalogSelector {
    ubicaciones: any[];
    selectedItem: UbicacionInfo = new UbicacionInfo;

    @Input() idAlmacen: number = 0;
    @Output() selectedItemChanged = new EventEmitter<UbicacionInfo>();

    constructor(protected context: Context, private ubicacionesService: UbicacionService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findUbicaciones(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.ubicacionesService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.namedPath;
                    this.selectedItemChanged.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.ubicacionesService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.ubicaciones = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findUbicaciones(keyword: string): Observable<CatalogItem[]> {
        return this.ubicacionesService.searchByAlmacen(this.idAlmacen, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

