﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Framework3
import { Context } from 'framework3/core';
//Inventario
import { UnidadMedidaInfo, UnidadMedidaItem } from '../../models/unidad-medida';
import { Empaquetado } from '../../models/empaquetado';

@Component({
    selector: 'inventario-detail-quantity',
    templateUrl: './detail-quantity.editor.html'
})
export class DetailQuantityEditor {    
    /////////// INPUTS
    @Input() cantidad: number = 0;
    @Input() unidades: number = 0;
    @Input() unidadMedida: UnidadMedidaInfo | UnidadMedidaItem;

    /////////// OUTPUS
    @Output() updateQuantity = new EventEmitter<Empaquetado | number>();

    @ViewChild(ModalDirective) dialog: ModalDirective;

    constructor(private context: Context) {}

    /////////// METHODS

    show() {
        this.dialog.config.backdrop = false;
        this.dialog.show();
    }

    /////////// HANDLERS

    onCantidadChange(cantidad: number) {
        this.cantidad = cantidad;
    }

    onUnidadesChange(unidades: number) {
        this.unidades = unidades;
    }

    onUpdate() {
        let result: any;
        if (this.unidadMedida.paquete) {
            result = new Empaquetado(this.cantidad, this.unidadMedida.factor, this.unidades);
        }
        else {
            result = this.cantidad;
        }
        this.updateQuantity.emit(result);
        this.dialog.hide();
    }
}
