﻿import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//Locales
import { EntradaDetail } from '../../models/entrada-detail';
import { ArticuloStockInfo } from '../../models/articulo-stock';
import { StockInfo } from '../../models/stock';
import { UnidadMedidaItem } from '../../models/unidad-medida';
import { UbicacionItem, UbicacionInfo } from '../../models/ubicacion';
//
import { ArticuloStockService } from '../../services/articulo-stock.service';
import { UnidadMedidaService } from '../../services/unidad-medida.service';
import { AlmacenService } from '../../services/almacen.service';

@Component({
    selector: 'inventario-entrada-detail-editor',
    templateUrl: './entrada-detail.editor.html'
})
export class EntradaDetailEditor extends BasicEditor {
    unidadesMedida: UnidadMedidaItem[] = [];
    stockConfig: StockInfo;
    validInput: boolean = false; 
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new EntradaDetail();
    //Objeto original que se editará
    @Input() detail: EntradaDetail = new EntradaDetail();

    @Input() idAlmacen: number = 0;

    constructor(protected context: Context, private builder: FormBuilder, private unidadService: UnidadMedidaService, private almacenService: AlmacenService) {
        super(context);

        this.form = builder.group({
            'idArticulo': [0, AztecaValidators.selected]
        });
    }

    /////////////////// METODOS ///////////////////////////////

    loadUnidadesMedida(idTipoUnidad: number) {
        //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
        //en caso contrario, cargar lista de unidades de medida del mismo tipo
        if (this.stockConfig.manejaSeries) {            
            let unidadArticulo = this.detailCopy.articuloStock.unidadMedida;
            let newUnidad: UnidadMedidaItem = {
                id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base,
                factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
            };
            let newUnidades = [];
            newUnidades.push(newUnidad);
            this.unidadesMedida = newUnidades;
            this.detailCopy.idUnidadMedida = unidadArticulo.identity;
        }
        else {
            this.unidadService.loadList(idTipoUnidad)
                .then(unidadesMedida => {
                    this.unidadesMedida = unidadesMedida;
                    this.detailCopy.idUnidadMedida = this.detailCopy.articuloStock.idUnidadMedida;
                })
                .catch(error => this.handleError(error));
        }
    }

    loadStock(idArticulo: number) {
        this.almacenService.loadStock(this.idAlmacen, idArticulo)
            .then(stock => {
                this.stockConfig = stock;
                this.loadUnidadesMedida(this.detailCopy.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {        
        this.detailCopy = this.detail;
        this.form.reset({ idArticulo: 0 });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: EntradaDetail ) {
        this.detailCopy = <EntradaDetail>this.copyItem(this.detail);
        this.form.updateValueAndValidity();
    }

    onUpdatingItem(detail: EntradaDetail) {
        this.unidadService.getDetailFromCache(this.detailCopy.idUnidadMedida)
            .then(unidad => {
                this.detailCopy.unidadMedida = unidad;
                if (this.detailCopy._clientID == 0) {
                    this.detailCopy._clientID = this.createClientID();
                }
                else {
                    this.detailCopy.state = ChangeState.MODIFIED;
                }
                this.savingItem.emit(this.detailCopy);
                this.detailCopy = new EntradaDetail();
            })
            .catch(error => this.handleError(error));
    }

    onArticuloChanged(articulo: ArticuloStockInfo) {
        if (articulo) {
            this.detailCopy.articuloStock = articulo;
            this.loadStock(articulo.identity);
        }
        else {            
            this.unidadesMedida = [];
            this.stockConfig = new StockInfo();
        }
    }

    onInputValidChange(valid: boolean) {
        this.validInput = valid;        
    }

}
