﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder } from '@angular/forms';
//Framework
import { Context } from 'framework3/core';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//Inventario
import { AreaItem } from '../../models/area';
import { AlmacenItem } from '../../models/almacen';
import { AreaService } from '../../services/area.service';
import { AlmacenService } from '../../services/almacen.service';

@Component({
    selector: 'inventory-nueva-salida',
    templateUrl: './nueva-salida.component.html'
})
export class NuevaSalidaComponent {
    valid = false;
    form: FormGroup;
    idArea: number;
    idAlmacen: number;
    areas: AreaItem[] = [];
    almacenes: AlmacenItem[] = [];

    //
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute,
        private context: Context, private areaService: AreaService, private almacenService: AlmacenService) {
        this.form = formBuilder.group({
            'idArea': [0],
            'idAlmacen': [0, AztecaValidators.selected]
        })
        //
        this.form.statusChanges.subscribe(_ => setTimeout(_ => this.valid = this.form.valid));
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadAreas());
    }

    /////////// METHODS

    private loadAreas() {
        this.context.app.showSpinner();
        this.areaService.loadListByUser(this.context.app.defaultSite.identity, this.context.session.user.identity)
            .then(areas => {
                this.areas = areas;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));

    }

    private loadAlmacenes(idArea: number) {
        this.idAlmacen = 0;
        this.context.app.showSpinner();
        this.almacenService.loadListByUser(idArea, this.context.session.user.identity)
            .then(almacenes => {
                this.almacenes = almacenes.filter(almacen => almacen.salida);
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }


    /////////// HANDLERS

    onToolbarClick(button: ToolbarButton) {
        if (button.name == 'OK') {
            this.router.navigate(['../', '0', { idAlmacen: this.idAlmacen }], { relativeTo: this.route });
        }
        else if (button.name == 'CANCEL') {
            this.router.navigate(['../'], { relativeTo: this.route });
        }
    }

    handleError(error: any) {
        console.log(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }
}
