﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { TransferenciaInfo, TransferenciaItem } from '../models/transferencia';
import { TransferenciaDetail } from '../models/transferencia-detail';
import { TransferenciaService } from '../services/transferencia.service';
import { TrensferenciaAceptarComponent } from '../parts/transferencia-aceptar.component';


@Component({
    selector: 'inventario-transferencias-tit-manager',
    templateUrl: './transferencias-tit.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TransferenciasTitManager)
        },
        {
            provide: DocumentService, useExisting: TransferenciaService
        }
    ]
})
export class TransferenciasTitManager extends DocumentManager {
    transferenciasTit: TransferenciaItem[];
    transferencia: TransferenciaInfo;

    argsCurrentDocument: WorkflowEventArgs;    

    @ViewChild('dlgTransferenciaAceptar') dlgTransferenciaAceptar: TrensferenciaAceptarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private transferenciaService: TransferenciaService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.transferenciaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.transferenciasTit = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['new'], { relativeTo: this.route });
        //this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.transferenciaService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {

        this.argsCurrentDocument = args;
        args.step.activities.forEach(activity => {
            if (activity.tag == 'ACEPTAR') {
                this.transferenciaService.getDetail(args.document.id)
                    .then(trans => {
                        this.dlgTransferenciaAceptar.showAceptar(trans.idSucursalDestino);
                    })
                    .catch(error => this.handleError(error));
            }
            else {
                this.context.app.showSpinner();

                this.transferenciaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
                    .then(_ => {
                        this.context.app.hideSpinner();
                        this.refreshList();
                    })
                    .catch(error => this.handleError(error));
            }
        });

    }


    onTransferenciaAceptada(idAlmacen) {
        this.context.app.showSpinner();

        let almacenD = { idAlmacenDestino: idAlmacen }

        this.transferenciaService.nextStep(this.argsCurrentDocument.document.id, this.argsCurrentDocument.step.idWorkflowStep, almacenD)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));

    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.transferenciaService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

}

