﻿import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
//Modulos Azteca
import { CoreModule } from 'framework3/core';
import { EditorsModule } from 'framework3/editors';
import { SearchModule } from 'framework3/search';
import { GeneralModule } from 'framework3/general';
import { ControlsModule } from 'framework3/controls';
import { SecurityModule } from 'framework3/security';
import { InventoryRoutingModule } from './inventory.routing.module';

//Bootstrap
import { TabsModule, ModalModule, CarouselModule } from 'ngx-bootstrap';
//Wijmo
import { WjCoreModule } from 'wijmo/wijmo.angular2.core';
import { WjGridModule } from 'wijmo/wijmo.angular2.grid';
import { WjGridFilterModule } from 'wijmo/wijmo.angular2.grid.filter';
import { WjInputModule } from 'wijmo/wijmo.angular2.input';
//Modulos Locales
import { AccountingModule } from '../accounting/accounting.module';

/////////////////////////////// Servicios ////////////////////////////
import { UnidadMedidaService } from './services/unidad-medida.service';
import { UnidadMedidaTipoService } from './services/unidad-medida-tipo.service';
import { AreaService } from './services/area.service';
import { AlmacenService } from './services/almacen.service';
import { ArticuloStockService } from './services/articulo-stock.service';
import { UbicacionService } from './services/ubicacion.service';
import { TipoUbicacionService } from './services/tipo-ubicacion.service';
import { ConceptoService } from './services/concepto.service';
import { EntradaService } from './services/entrada.service';
import { SalidaService } from './services/salida.service';
import { TraspasoService } from './services/traspaso.service';
import { TransferenciaService } from './services/transferencia.service';
import { MetaAlmacenService } from './services/meta-almacen.service';

//////////////////////////// Componentes /////////////////////////////
import { AlmacenEditarStock } from './components/almacen-editar-stock.component';
import { ArticulosEditarStock } from './components/articulos-editar-stock.component';

import { AreasUsuariosComponent } from './components/area-usuarios.component';
import { AreasSelectorComponent } from './components/area-selector.component';
import { AlmacenUsuariosComponent } from './components/almacen-usuarios.component';

import { ArticulosPermitidosComponent } from './components/articulos-permitidos.component';
import { ClasificacionesArticulosComponent } from './components/clasificaciones-articulos-list.component';

import { ArticulosFotografias } from './components/articulos-fotografias.component';

import { ConceptoUsuariosComponent } from './components/concepto-usuarios.component';

import { CortesComponent } from './components/corte.component';

//////////////////////////// Catalogos ////////////////////////////////
import { UnidadesMedidaManager } from './catalogs/unidades-medida.manager';
import { UnidadesMedidaEditor } from './catalogs/unidades-medida.editor';
import { UnidadesMedidaTipoManager } from './catalogs/unidades-medida-tipo.manager';
import { UnidadesMedidaTipoEditor } from './catalogs/unidades-medida-tipo.editor';

import { AreasManager } from './catalogs/areas.manager';
import { AreasEditor } from './catalogs/areas.editor';

import { AlmacenesManager } from './catalogs/almacenes.manager';
import { AlmacenesEditor } from './catalogs/almacenes.editor';

import { ArticulosStockManager } from './catalogs/articulos-stock.manager';
import { ArticulosStockEditor } from './catalogs/articulos-stock.editor';

import { AlmacenExistencias } from './catalogs/almacen-existencias';
import { ArticuloExistencias } from './catalogs/articulo-existencias';

import { AlmacenArticulosStock } from './catalogs/almacen-articulos-stock';
import { ArticulosStockConfiguracion } from './catalogs/articulos-stock-configuracion';

import { UbicacionesManager } from './catalogs/ubicaciones.manager';
import { TiposUbicacionManager } from './catalogs/tipos-ubicacion.manager';
import { TiposUbicacionEditor } from './catalogs/tipos-ubicacion.editor';

import { ConceptosManager } from './catalogs/conceptos.manager';
import { ConceptosEditor } from './catalogs/conceptos.editor';

import { MetaAlmacenesManager } from './catalogs/meta-almacenes.manager';
import { MetaAlmacenesEditor } from './catalogs/meta-almacenes.editor';



///////////////////////////// Controles /////////////////////////////////
import { ArticuloStockSelectorComponent } from './controls/articulo-stock-selector.component';
import { UbicacionesSelectorComponent } from './controls/ubicaciones-selector.component';
import { ArticuloStockAutocompleteCellComponent } from './controls/articulo-stock-autocomplete-cell.component';
import { AlmacenesSelectorComponent } from './controls/almacenes-selector.component';
import { QuantityInputComponent } from './controls/quantity-input.component';
import { QuantityLabelComponent } from './controls/quantity-label.component';
import { ArticuloStockEntradaComponent } from './controls/articulo-stock-entrada.component';
import { ArticuloStockSalidaComponent } from './controls/articulo-stock-salida.component';
import { ArticuloLotesComponent } from './controls/articulo-lotes.component';
import { ArticuloSeriesComponent } from './controls/articulo-series.component';
import { AlmacenProvisionesComponent } from './components/almacen-provision.component'

///////////////////////////// Documents /////////////////////////////////
import { EntradasEditor } from './documents/entradas.editor';
import { EntradasManager } from './documents/entradas.manager';
import { SalidasEditor } from './documents/salidas.editor';
import { SalidasManager } from './documents/salidas.manager';
import { TraspasosTitEditor } from './documents/traspasos-tit.editor';
import { TraspasosTitManager } from './documents/traspasos-tit.manager';
import { TransferenciasTitEditor } from './documents/transferencias-tit.editor';
import { TransferenciasTitManager } from './documents/transferencias-tit.manager';

//////////////////////////// Parts /////////////////////////////////////
import { EntradaDetailEditor } from './documents/parts/entrada-detail.editor';
import { SalidaDetailEditor } from './documents/parts/salida-detail.editor';
import { NuevaEntradaComponent } from './documents/parts/nueva-entrada.component';
import { NuevaSalidaComponent } from './documents/parts/nueva-salida.component';
import { DetailQuantityEditor } from './documents/parts/detail-quantity.editor';
import { ArticuloStockGeneralComponent } from './parts/articulos-stock-general.component';
import { ArticuloStockControlesComponent } from './parts/articulos-stock-controles.component';
import { TraspasoDetailEditor } from './documents/parts/traspasos-tit-detail.editor';
import { TransferenciaDetailEditor } from './documents/parts/transferencias-tit-detail.editor';
import { NuevaTransferenciaComponent } from './documents/parts/nueva-transferencia.component';
import { TrensferenciaAceptarComponent } from './parts/transferencia-aceptar.component';

////////////////////////////////Mantenimientos/////////////////////////
import { ArticuloCostoEstandar } from './maintenances/articulos-costo-estandar.maintenance';

import { AlmacenListComponent } from './dialogs/almacenes-list.component'


/////////////////////////////////////////////////////////////////////////////////
let COMPONENTES = [UnidadesMedidaManager, UnidadesMedidaEditor, UnidadesMedidaTipoManager, UnidadesMedidaTipoEditor, AreasManager, AreasEditor,
    AreasUsuariosComponent, AlmacenUsuariosComponent, AlmacenesManager, AlmacenesEditor, ArticulosStockManager, UbicacionesManager, TiposUbicacionManager, TiposUbicacionEditor,
    ArticulosStockEditor, ArticuloStockSelectorComponent, UbicacionesSelectorComponent, AlmacenesSelectorComponent, ArticulosPermitidosComponent, ClasificacionesArticulosComponent, ArticulosFotografias,
    AlmacenEditarStock, ArticulosEditarStock, AlmacenExistencias, ArticuloExistencias, AlmacenArticulosStock, ArticulosStockConfiguracion, EntradasEditor, EntradasManager, EntradaDetailEditor,
    ArticuloStockAutocompleteCellComponent, ConceptosManager, ConceptosEditor, NuevaEntradaComponent, ConceptoUsuariosComponent, QuantityInputComponent, QuantityLabelComponent, ArticuloStockEntradaComponent,
    ArticuloStockSalidaComponent, SalidasEditor, SalidasManager, NuevaSalidaComponent, SalidaDetailEditor, ArticuloLotesComponent, DetailQuantityEditor, ArticuloSeriesComponent,
    ArticuloStockGeneralComponent, ArticuloStockControlesComponent, AreasSelectorComponent, CortesComponent,
    TraspasosTitManager, TraspasosTitEditor, TraspasoDetailEditor, TransferenciasTitManager, TransferenciasTitEditor, TransferenciaDetailEditor, NuevaTransferenciaComponent, TrensferenciaAceptarComponent,
    MetaAlmacenesManager, MetaAlmacenesEditor, ArticuloCostoEstandar, AlmacenListComponent, AlmacenProvisionesComponent];

let SERVICIOS = [UnidadMedidaService, UnidadMedidaTipoService, AreaService, ArticuloStockService, AlmacenService, EntradaService, UbicacionService, TipoUbicacionService,
    ConceptoService, SalidaService, TraspasoService, TransferenciaService, MetaAlmacenService];

@NgModule({
    imports: [
        CommonModule, FormsModule, ReactiveFormsModule,
        TabsModule.forRoot(), ModalModule.forRoot(), CarouselModule.forRoot(),
        CoreModule, EditorsModule, SearchModule, GeneralModule, SecurityModule, ControlsModule, InventoryRoutingModule, AccountingModule,
        WjCoreModule, WjInputModule, WjGridModule, WjGridFilterModule
    ],
    declarations: COMPONENTES,
    providers: SERVICIOS,
    exports: COMPONENTES
})
export class InventoryModule {

}
