﻿import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

//////////////////////////// Guards //////////////////////////////////
import { AuthGuard } from 'framework3/core';
import { EditorDeactivateGuard } from 'framework3/editors';

//////////////////////////// Catalogos ////////////////////////////////
import { UnidadesMedidaManager } from './catalogs/unidades-medida.manager';
import { UnidadesMedidaEditor } from './catalogs/unidades-medida.editor';
import { UnidadesMedidaTipoManager } from './catalogs/unidades-medida-tipo.manager';
import { UnidadesMedidaTipoEditor } from './catalogs/unidades-medida-tipo.editor';

import { AreasManager } from './catalogs/areas.manager';
import { AreasEditor } from './catalogs/areas.editor';

import { AlmacenesManager } from './catalogs/almacenes.manager';
import { AlmacenesEditor } from './catalogs/almacenes.editor';

import { ArticulosStockManager } from './catalogs/articulos-stock.manager';
import { ArticulosStockEditor } from './catalogs/articulos-stock.editor';

import { AlmacenExistencias } from './catalogs/almacen-existencias';
import { ArticuloExistencias } from './catalogs/articulo-existencias';

import { UbicacionesManager } from './catalogs/ubicaciones.manager';

import { AlmacenArticulosStock } from './catalogs/almacen-articulos-stock';
import { ArticulosStockConfiguracion } from './catalogs/articulos-stock-configuracion';

import { TiposUbicacionManager } from './catalogs/tipos-ubicacion.manager';
import { TiposUbicacionEditor } from './catalogs/tipos-ubicacion.editor';

import { ConceptosManager } from './catalogs/conceptos.manager';
import { ConceptosEditor } from './catalogs/conceptos.editor';

import { MetaAlmacenesManager } from './catalogs/meta-almacenes.manager';
import { MetaAlmacenesEditor } from './catalogs/meta-almacenes.editor';

//////////////////////////// Componentes ////////////////////////////////
import { AreasUsuariosComponent } from './components/area-usuarios.component';
import { AlmacenUsuariosComponent } from './components/almacen-usuarios.component';

import { ArticulosPermitidosComponent } from './components/articulos-permitidos.component';
import { ClasificacionesArticulosComponent } from './components/clasificaciones-articulos-list.component';

import { ArticulosFotografias } from './components/articulos-fotografias.component';

import { ConceptoUsuariosComponent } from './components/concepto-usuarios.component';
import { CortesComponent } from './components/corte.component';
///////////////////////////Mantenimientos///////////////////////////////
import { ArticuloCostoEstandar } from './maintenances/articulos-costo-estandar.maintenance';
///////////////////////////// Document /////////////////////////////////
import { EntradasEditor } from './documents/entradas.editor';
import { EntradasManager } from './documents/entradas.manager';
import { SalidasEditor } from './documents/salidas.editor';
import { SalidasManager } from './documents/salidas.manager';
import { TraspasosTitEditor } from './documents/traspasos-tit.editor'
import { TraspasosTitManager } from './documents/traspasos-tit.manager'
import { TransferenciasTitEditor } from './documents/transferencias-tit.editor';
import { TransferenciasTitManager } from './documents/transferencias-tit.manager';

import { NuevaEntradaComponent } from './documents/parts/nueva-entrada.component';
import { NuevaSalidaComponent } from './documents/parts/nueva-salida.component';
import { NuevaTransferenciaComponent } from './documents/parts/nueva-transferencia.component';

import { AlmacenProvisionesComponent } from './components/almacen-provision.component';

///////////////////////////// Rutas /////////////////////////////////

const routes: Routes = [
    {
        path: '',
        canActivateChild: [AuthGuard],
        children: [
            { path: 'unidades-medida-tipo/:proceso', component: UnidadesMedidaTipoManager },
            { path: 'unidades-medida-tipo/:proceso/:id', component: UnidadesMedidaTipoEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'unidades-medida/:proceso/:idTipo/:id', component: UnidadesMedidaEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'articulos-stock/:proceso', component: ArticulosStockManager },
            { path: 'articulos-stock/:proceso/:id', component: ArticulosStockEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'areas/:proceso', component: AreasManager },
            { path: 'areas/:proceso/:id', component: AreasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'areas-usuarios/:proceso/:idArea', component: AreasUsuariosComponent },
            { path: 'almacenes/:proceso/:idArea', component: AlmacenesManager },
            { path: 'almacenes/:proceso/:idArea/:id', component: AlmacenesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'almacenes-usuarios/:proceso/:idAlmacen', component: AlmacenUsuariosComponent },
            { path: 'almacenes-clasificaciones-articulos/:proceso/:idAlmacen', component: ArticulosPermitidosComponent },
            { path: 'almacenes-existencias/:proceso/:idAlmacen', component: AlmacenExistencias },
            { path: 'articulos-existencias/:proceso/:idArticulo', component: ArticuloExistencias },
            { path: 'almacenes-stock/:proceso/:idAlmacen', component: AlmacenArticulosStock },
            { path: 'articulos-stock-configuracion/:proceso/:idArticulo', component: ArticulosStockConfiguracion },

            { path: 'almacenes-provisiones/:proceso/:idAlmacen', component: AlmacenProvisionesComponent },
             

            { path: 'ubicaciones/:proceso/:idAlmacen', component: UbicacionesManager },
            { path: 'articulos-fotografias/:proceso/:idArticulo', component: ArticulosFotografias },
            { path: 'ubicaciones-tipo/:proceso', component: TiposUbicacionManager },
            { path: 'ubicaciones-tipo/:proceso/:id', component: TiposUbicacionEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'conceptos/:proceso', component: ConceptosManager },
            { path: 'conceptos/:proceso/:id', component: ConceptosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'conceptos/:proceso/:id/users', component: ConceptoUsuariosComponent },
            { path: 'cortes/:proceso', component: CortesComponent },
            { path: 'meta-almacenes/:proceso', component: MetaAlmacenesManager },
            { path: 'meta-almacenes/:proceso/:id', component: MetaAlmacenesEditor, canDeactivate: [EditorDeactivateGuard] },

            //DOCUMENTS
            { path: 'entradas/:proceso', component: EntradasManager },
            { path: 'entradas/:proceso/new', component: NuevaEntradaComponent },
            { path: 'entradas/:proceso/:id', component: EntradasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'salidas/:proceso', component: SalidasManager },
            { path: 'salidas/:proceso/new', component: NuevaSalidaComponent },
            { path: 'salidas/:proceso/:id', component: SalidasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'traspasos/:proceso', component: TraspasosTitManager },
            { path: 'traspasos/:proceso/:id', component: TraspasosTitEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'transferencias/:proceso', component: TransferenciasTitManager },
            { path: 'transferencias/:proceso/new', component: NuevaTransferenciaComponent },
            { path: 'transferencias/:proceso/:id', component: TransferenciasTitEditor, canDeactivate: [EditorDeactivateGuard] },

            //MANTENIMIENTOS
            { path: 'articulos-costo-estandar/:proceso', component: ArticuloCostoEstandar }
        ]
    },
];

@NgModule({
    imports: [
        RouterModule.forChild(routes)
    ],
    exports: [
        RouterModule
    ]
})
export class InventoryRoutingModule { }

