﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo } from 'framework3/core';
import { MenuItem, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection } from 'framework3/editors';
//Inventario
import { ArticuloCostoEstandar as ArticuloCostoEstandarInf } from '../models/articulo-costo-estandar';
import { ArticuloStockService } from '../services/articulo-stock.service';



@Component({
    selector: 'inventory-articulos-costo-estandar',
    templateUrl: './articulos-costo-estandar.maintenance.html'
})
export class ArticuloCostoEstandar extends ProcessComponent {
    form: FormGroup;

    articulosCostoEstandar: ArticuloCostoEstandarInf[] = [];
    articulosCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder
        , private articuloService: ArticuloStockService) {
        super(context, router, route);
        this.form = formBuilder.group({
            'fechaInicial': [new Date()],
            'fechaFinal': [new Date()]


        });
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CONSULTAR') {

            this.llenaListado();

        }
        if (button.name == 'GUARDAR') {
            this.guardar();
        }
    }

    llenaListado() {
        this.articuloService.loadCostoEstandarList(this.context.app.defaultSite.identity)
            .then(ret => {
                this.articulosCollection.sourceCollection = ret;
                this.articulosCostoEstandar = ret;
            })
            .catch(error => this.handleError(error));
    }

    guardar() {
        this.articuloService.costosEstandarUpdate(this.articulosCostoEstandar).then(_ => {
            this.alerts.showAlert("Información guardada correctamente",'success');
        })
        .catch(error => this.handleError(error));
    }


    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}