﻿import { Component, Input, Output, EventEmitter, forwardRef, SimpleChange } from '@angular/core';
import { ControlValueAccessor, Validators, NG_VALUE_ACCESSOR, FormGroup, FormBuilder } from '@angular/forms';
//
import { Context, CatalogItem, BaseComponent, PolicyInfo } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { BaseControl } from 'framework3/controls/models/base-control';
import { ArticuloStockInfo, ArticuloStockItem } from '../models/articulo-stock';
import { MetodoValuacion } from '../models/metodo-valuacion';

@Component({
    selector: 'articulos-stock-controles',
    templateUrl: './articulos-stock-controles.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticuloStockControlesComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => ArticuloStockControlesComponent), multi: true
        }
    ],
    styles: [
        '.ng-valid-component { border-left: 1px solid #ccc!important;}'
    ]
})

export class ArticuloStockControlesComponent extends BaseControl implements ControlValueAccessor {
    private form: FormGroup;
    metodosValuacion = MetodoValuacion.list;
    //Indica si la información de la forma es válida
    get valid(): boolean {
        return this.form.valid;
    }
    //Inputs
    @Input() articuloStock: ArticuloStockInfo = new ArticuloStockInfo();

    //Outputs
    @Output() articuloStockChange = new EventEmitter<ArticuloStockInfo>();
    @Output() validChange = new EventEmitter<boolean>();
    //Campos privados para enlazar a UI
    private manejaSeries: boolean = false;
    private manejaLotes: boolean = false;
    private manejaCaducidad: boolean = false;
    private metodoValuacion: string = '';
    //
    constructor(private formBuilder: FormBuilder) {
        super();

        this.form = formBuilder.group({
            'manejaSeries': [false],
            'manejaLotes': [false],
            'manejaCaducidad': [false],
            'metodoValuacion': ['']
        });
        //Hacer requerido el método de valuación sólo cuando se manejen lotes
        this.form.controls['manejaLotes'].valueChanges.subscribe(value => {
            if (value == true) {
                this.form.controls['metodoValuacion'].setValidators(Validators.required);
            }
            else {
                this.form.controls['metodoValuacion'].setValue('');
                this.form.controls['metodoValuacion'].setValidators(null);
            }
            this.form.controls['metodoValuacion'].updateValueAndValidity();
        });
    }

    ngOnChanges(changes: SimpleChange) {
        if (changes['persona']) {
            this.updateArticuloStockControles();
        }
    }

    ngAfterViewInit() {
        this.form.statusChanges.subscribe(_ => {
            this.validChange.emit(this.form.valid);
        });
    }


    //////////////////////////////// METODOS ////////////////////////////////////////

    updateArticuloStockControles() {
        this.metodoValuacion = this.articuloStock.metodoValuacion;
    }

    notifiyChange() {
        let newArticuloStock: ArticuloStockInfo = JSON.parse(JSON.stringify(this.articuloStock));
        newArticuloStock.manejaSeries = this.articuloStock.manejaSeries;
        newArticuloStock.manejaLotes = this.articuloStock.manejaLotes;
        newArticuloStock.manejaCaducidad = this.articuloStock.manejaCaducidad;
        newArticuloStock.metodoValuacion = this.articuloStock.metodoValuacion;
        //
        this.articuloStockChange.emit(newArticuloStock);
        this.propagateChange(newArticuloStock);
    }

    //////////////////////////////// HANDLERS ////////////////////////////////////////

    onSerieCheckChange(args: any) {
        this.notifiyChange();
    }
    onLotesCheckChange(args: any) {
        if (args.target.checked == false) {
            this.articuloStock.metodoValuacion = 'N';
        }
        this.notifiyChange();
    }
    onCaducidadCheckChange(args: any) {
        this.notifiyChange();
    }
    onMetodoValuacionChange(args: any) {
        this.articuloStock.metodoValuacion = this.form.controls.metodoValuacion.value;
        this.notifiyChange();
    }

    //////////////////////////////// CONTROL VALUE ACCESOR ////////////////////////////////////////

    writeValue(value: any) {
        //this.articuloStock = value;
        this.updateArticuloStockControles();
    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() {
    }


}