﻿import { Component, Input, Output, EventEmitter, forwardRef, SimpleChange } from '@angular/core';
import { ControlValueAccessor, Validators, NG_VALUE_ACCESSOR, FormGroup, FormBuilder } from '@angular/forms';
//
import { Context, CatalogItem, BaseComponent, PolicyInfo } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { BaseControl } from 'framework3/controls/models/base-control';
import { ArticuloStockInfo, ArticuloStockItem } from '../models/articulo-stock';

@Component({
    selector: 'articulos-stock-general',
    templateUrl: './articulos-stock-general.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticuloStockGeneralComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => ArticuloStockGeneralComponent), multi: true
        }
    ],
    styles: [
        '.ng-valid-component { border-left: 1px solid #ccc!important;}'
    ]
})

export class ArticuloStockGeneralComponent extends BaseControl implements ControlValueAccessor {
    private form: FormGroup;
    //Indica si la información de la forma es válida
    get valid(): boolean {
        return this.form.valid;
    }
    //Inputs
    @Input() articuloStock: ArticuloStockInfo = new ArticuloStockInfo();

    //Outputs
    @Output() articuloStockChange = new EventEmitter<ArticuloStockInfo>();
    @Output() validChange = new EventEmitter<boolean>();
    //
    constructor(private formBuilder: FormBuilder) {
        super();

        this.form = formBuilder.group({
            'idCategoriaFiscal': [0, AztecaValidators.selected],
            'gtin': [''],
            'descripcion': ['']
        });

    }

    ngOnChanges(changes: SimpleChange) {
        
    }

    ngAfterViewInit() {
        this.form.statusChanges.subscribe(_ => {
            this.validChange.emit(this.form.valid);
        });
    }


    //////////////////////////////// METODOS ////////////////////////////////////////

    notifiyChange() {
        let newArticuloStock: ArticuloStockInfo = JSON.parse(JSON.stringify(this.articuloStock));
        newArticuloStock.idCategoriaFiscal = this.articuloStock.idCategoriaFiscal;
        newArticuloStock.gtin = this.articuloStock.gtin;
        newArticuloStock.descripcion = this.articuloStock.descripcion;
        //
        this.articuloStockChange.emit(newArticuloStock);
        this.propagateChange(newArticuloStock);
    }

    //////////////////////////////// HANDLERS ////////////////////////////////////////
    onCategoriaFiscalChanged(args: any) {
        this.notifiyChange();
    }

    onGtinChanged(args: any) {
        this.notifiyChange();
    }

    onDescripcionChanged(args: any) {
        this.notifiyChange();
    }

    //////////////////////////////// CONTROL VALUE ACCESOR ////////////////////////////////////////

    writeValue(value: any) {
        //this.articuloStock = value;
    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() {
    }


}