﻿import { Component, Input, Output, ViewChild, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';

//
//features
import { CollectionView } from 'wijmo/wijmo';
import { ModalDirective } from 'ngx-bootstrap';

import { BaseComponent, Context, CatalogItem, PolicyInfo } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent, AzComboboxComponent } from 'framework3/controls';

//Inventario
import { AreaItem } from '../models/area';
import { AlmacenItem } from '../models/almacen';
import { AreaService } from '../services/area.service';
import { AlmacenService } from '../services/almacen.service';

@Component({
    selector: 'inventory-transferencia-aceptar',
    templateUrl: './transferencia-aceptar.component.html'
})
export class TrensferenciaAceptarComponent {
    editorForm: FormGroup;    
    areas: AreaItem[] = [];
    almacenes: AlmacenItem[] = [];
    idAlmacen: number;

    @Output() transferenciaAceptada: EventEmitter<number> = new EventEmitter<number>();
    //@Input() idSucursalDestino: number = 0;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('dlgTransferenciaAceptar') dlgTransferenciaAceptar: ModalDirective;

    constructor(private formBuilder: FormBuilder, private context: Context, private router: Router, private route: ActivatedRoute, private areaService: AreaService, private almacenService: AlmacenService) {
        this.editorForm = formBuilder.group({
            'idArea': [0, Validators.required], 
            'idAlmacen': [0, Validators.required] 
        });

    }

    private loadAreas(idSucursalDestino:number) {
        this.context.app.showSpinner();
        this.areaService.loadListByUser(idSucursalDestino, this.context.session.user.identity)
            .then(areas => {
                this.areas = areas;
                this.context.app.hideSpinner();
            })
            .catch(error => this.showAlert(error.message, 'danger'));

    }

    private loadAlmacenes(idArea: number) {
        this.idAlmacen = 0;
        this.context.app.showSpinner();
        this.almacenService.loadListByUser(idArea, this.context.session.user.identity)
            .then(almacenes => {
                this.almacenes = almacenes.filter(almacen => almacen.salida);
                this.context.app.hideSpinner();
            })
            .catch(error => this.showAlert(error.message, 'danger'));
    }

    showAceptar(idSucursalDestino: number) {
        this.dlgTransferenciaAceptar.config.backdrop = false
        this.loadAreas(idSucursalDestino);
        this.dlgTransferenciaAceptar.show();
    }

    accept() {
        this.transferenciaAceptada.emit(this.idAlmacen);
        this.dlgTransferenciaAceptar.hide();
    }


    discardChanges() {
        this.dlgTransferenciaAceptar.hide();
    }


    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }


}
