﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';
import { CatalogInfo, CatalogItem } from 'framework3/core';
//Locales
import { AlmacenInfo, AlmacenItem, AlmacenUsuarioInfo, AlmacenUsuarioItem, ClasificacionesArticulosItem, Existencias, AlmacenProvisionItem } from '../models/almacen'
import { StockInfo } from '../models/stock';

@Injectable()
export class AlmacenService extends CatalogService<AlmacenInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/inventario/almacenes");
    }

    create(): Promise<AlmacenInfo> {
        return this.apiService.get<AlmacenInfo>('create', null)
            .toPromise();
    }

    LoadAccessUsersList(idAlmacen: number): Promise<AlmacenUsuarioItem[]> {
        let params = new HttpParams()
            .set("idAlmacen", idAlmacen.toString());
        return this.apiService.get<AlmacenUsuarioItem[]>("load-access-users-list", params)
            .toPromise();
    }

    addMembers(idAlmacen: number, usuarios: AlmacenUsuarioItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idAlmacen', idAlmacen.toString());
        return this.apiService.post<AlmacenUsuarioItem[]>('add-members', params, usuarios)
            .toPromise();
    }

    updateMembers(idAlmacen: number, usuarios: AlmacenUsuarioItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idAlmacen', idAlmacen.toString());
        return this.apiService.post<AlmacenUsuarioItem[]>('update-members', params, usuarios)
            .toPromise();
    }

    removeMember(idAlmacen: number, idUsuario: number): Promise<any> {
        let params = new HttpParams()
            .set('idAlmacen', idAlmacen.toString())
            .set('idUsuario', idUsuario.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

    loadList(idArea: number): Promise<AlmacenItem[]> {
        let params = new HttpParams()
            .append("idArea", idArea.toString());
        return this.apiService.get<AlmacenItem[]>("load-list", params)
            .toPromise();
    }

    loadListByUser(idArea: number, idUsuario: number): Promise<AlmacenItem[]> {
        let params = new HttpParams()
            .append("idArea", idArea.toString())
            .append("idUsuario", idUsuario.toString());
        return this.apiService.get<AlmacenItem[]>("load-list-by-user", params)
            .toPromise();
    }

    //////ArticulosPermitidos////////

    LoadClasificacionesArticulos(): Promise<ClasificacionesArticulosItem[]> {
        let params = new HttpParams()
        return this.apiService.get<ClasificacionesArticulosItem[]>("load-clasificaciones-articulos", params)
            .toPromise();
    }

    LoadClasificacionesPermitidas(idAlmacen: number): Promise<ClasificacionesArticulosItem[]> {
        let params = new HttpParams()
            .append("idAlmacen", idAlmacen.toString());
        return this.apiService.get<ClasificacionesArticulosItem[]>("load-clasificaciones-permitidas", params)
            .toPromise();
    }

    updateClasificacionesArticulos(idAlmacen: number, data: ClasificacionesArticulosItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idAlmacen', idAlmacen.toString());
        return this.apiService.post<any>('update-clasificaciones-articulos', params, data)
            .toPromise();
    }

    removeClasificacionesArticulos(idAlmacen: number, idClasificacion: number): Promise<any> {
        let params = new HttpParams()
            .set('idAlmacen', idAlmacen.toString())
            .set('idClasificacion', idClasificacion.toString());
        return this.apiService.delete<any>('remove-clasificaciones-articulos', params)
            .toPromise();
    }

    //////Existencias////////

    loadExistencias(idAlmacen: number): Promise<Existencias[]> {
        let params = new HttpParams()
            .append("idAlmacen", idAlmacen.toString());
        return this.apiService.get<Existencias[]>("load-existencias", params)
            .toPromise();
    }

    loadExistenciasbyArt(idArticuloStock: number, idSucursal: number): Promise<Existencias[]> {
        let params = new HttpParams()
            .append("idArticuloStock", idArticuloStock.toString())
            .append("idSucursal", idSucursal.toString());
        return this.apiService.get<Existencias[]>("load-existencias-by-article", params)
            .toPromise();
    }

    ////// STOCK ////////

    loadStock(idAlmacen: number, idArticuloStock: number): Promise<StockInfo> {
        let params = new HttpParams()
            .append("idAlmacen", idAlmacen.toString())
            .append("idArticuloStock", idArticuloStock.toString());
        return this.apiService.get<StockInfo>("stock", params)
            .toPromise();
    }

    loadStockAlmacen(idAlmacen: number): Promise<StockInfo[]> {
        let params = new HttpParams()
            .append("idAlmacen", idAlmacen.toString());
        return this.apiService.get<StockInfo[]>("stock-almacen", params)
            .toPromise();
    }

    loadStockArticulo(idArticulo: number, idSucursal: number): Promise<StockInfo[]> {
        let params = new HttpParams()
            .append("idArticulo", idArticulo.toString())
            .append("idSucursal", idSucursal.toString());
        return this.apiService.get<StockInfo[]>("stock-articulo", params)
            .toPromise();
    }

    updateStockByArticulo(idArticulo: number, data: StockInfo): Promise<any> {
        let params = new HttpParams()
            .set('idArticulo', idArticulo.toString());
        return this.apiService.post<any>('update-stock-by-article', params, data)
            .toPromise();
    }

    updateStock(idAlmacen: number, data: StockInfo): Promise<any> {
        let params = new HttpParams()
            .set('idAlmacen', idAlmacen.toString());
        return this.apiService.post<any>('update-stock', params, data)
            .toPromise();
    }


    //Metodos del selector customizados
    searchKeywordAlmacenes(idBranch: number, keyword: string): Observable<AlmacenItem[]> {
        let params = new HttpParams()
            .append('idBranch', idBranch.toString())
            .append('keyword', keyword);
        return this.apiService.get<AlmacenItem[]>('search-keyword', params);
    }



    //almacen de provision
    LoadAlmacenesProvision(idAlmacen: number): Promise<AlmacenProvisionItem[]> {
        let params = new HttpParams()
            .set("idAlmacen", idAlmacen.toString());
        return this.apiService.get<AlmacenUsuarioItem[]>("load-almacenes-provisiones", params)
            .toPromise();
    }

    addAlmacenesProvision(idAlmacen: number, almacenes: AlmacenProvisionItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idAlmacen', idAlmacen.toString());
        return this.apiService.post<AlmacenUsuarioItem[]>('add-almacen-provision', params, almacenes)
            .toPromise();
    }

    //updateMembers(idAlmacen: number, usuarios: AlmacenUsuarioItem[]): Promise<any> {
    //    let params = new HttpParams()
    //        .set('idAlmacen', idAlmacen.toString());
    //    return this.apiService.post<AlmacenUsuarioItem[]>('update-members', params, usuarios)
    //        .toPromise();
    //}

    removeAlmacenProvision(idAlmacen: number, idAlmacenProvision: number): Promise<any> {
        let params = new HttpParams()
            .set('idAlmacen', idAlmacen.toString())
            .set('idAlmacenProvision', idAlmacenProvision.toString());
        return this.apiService.delete<any>('remove-almacen-provision', params)
            .toPromise();
    }


}