﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { AreaInfo, AreaItem, AreaUsuarioInfo, AreaUsuarioItem, TurnoItem } from '../models/area'
import { AlmacenItem} from '../models/almacen'

@Injectable()
export class AreaService extends CatalogService<AreaInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/inventario/areas");
    }

    create(): Promise<AreaInfo> {
        return Promise.resolve(new AreaInfo());
    }

    loadList(idSucursal: number): Promise<AreaItem[]> {
        let params = new HttpParams()
            .append("idSucursal", idSucursal.toString());
        return this.apiService.get<AreaItem[]>("load-list", params)
            .toPromise();
    }

    loadListByUser(idSucursal: number, idUsuario:number ): Promise<AreaItem[]> {
        let params = new HttpParams()
            .append("idSucursal", idSucursal.toString())
            .append("idUsuario", idUsuario.toString());
        return this.apiService.get<AreaItem[]>("load-list-by-user", params)
            .toPromise();
    }

    loadAlmacenes(idArea: number): Promise<AlmacenItem[]> {
        let params = new HttpParams()
            .append("idArea", idArea.toString());
        return this.apiService.get<AlmacenItem[]>("load-almacenes", params)
            .toPromise();
    }

    loadAlmacenesByUser(idArea: number,idUsuario:number): Promise<AlmacenItem[]> {
        let params = new HttpParams()
            .append("idArea", idArea.toString())
            .append("idUsuario", idUsuario.toString());
        return this.apiService.get<AlmacenItem[]>("load-almacenes-by-user", params)
            .toPromise();
    }

    LoadAccessUsersList(idArea: number): Promise<AreaUsuarioItem[]> {
        let params = new HttpParams()
            .append("idArea", idArea.toString());
        return this.apiService.get<AreaUsuarioItem[]>("load-access-users-list", params)
            .toPromise();
    }

    ///////////////////////////// Turnos ////////////////////////////////////////////////

    getMembers(idGrupo: number): Promise<AreaUsuarioItem[]> {
        let params = new HttpParams()
            .set('idGrupo', idGrupo.toString());
        return this.apiService.get<AreaUsuarioItem[]>('get-members', params)
            .toPromise();
    }

    addMembers(idArea: number, usuarios: AreaUsuarioItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idArea', idArea.toString());
        return this.apiService.post<any>('add-members', params, usuarios)
            .toPromise();
    }

    removeMember(idArea: number, idUsuario: number): Promise<any> {
        let params = new HttpParams()
            .set('idArea', idArea.toString())
            .set('idUsuario', idUsuario.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

    LoadTurnos(idArea: number): Promise<TurnoItem[]> {
        let params = new HttpParams()
            .set('idArea', idArea.toString());
        return this.apiService.get<TurnoItem[]>('load-turnos', params)
            .toPromise();
    }

    /*SaveTurnos(idArea: number, usuarios: AreaUsuarioItem[]): Promise<any> {
        let params: URLSearchParams = new URLSearchParams();
        params.set('idArea', idArea.toString());
        return this.apiService.postData('add-members', params, usuarios)
            .toPromise();
    }*/

    ///////////////////////////// Turnos ////////////////////////////////////////////////

    LoadCorte(idCorte: string): Promise<any[]> {
        let params = new HttpParams()
            .set('idCorte', idCorte);
        return this.apiService.get<any[]>('load-corte', params)
            .toPromise();
    }

    LoadCorteActual(idArea: number): Promise<any> {
        let params = new HttpParams()
            .set('idArea', idArea.toString());
        return this.apiService.get<any>('load-corte-actual', params)
            .toPromise();
    }

    AbrirCorte(idArea: number, idTurno: number, idUsuario: number): Promise<any> {
        let params = new HttpParams()
            .set('idArea', idArea.toString())
            .set('idTurno', idTurno.toString())
            .set('idUsuario', idUsuario.toString());
        return this.apiService.get<any>('abrir-corte', params)
            .toPromise();
    }

    CerrarCorte(idCorte: string, ajustarInventario: boolean, idUsuario: number): Promise<any> {
        let params = new HttpParams()
            .set('idCorte', idCorte)
            .set('ajustarInventario', ajustarInventario.toString())
            .set('idUsuario', idUsuario.toString());
        return this.apiService.delete<any>('cerrar-corte', params)
            .toPromise();
    }

}