﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
import { map } from 'rxjs/operators';
//Azteca Kernel
import { Context, CatalogInfo, CatalogItem, CatalogService } from 'framework3/core';
//Locales
import { ArticuloStockInfo, ArticulosFotografia } from '../models/articulo-stock';
import { LoteInfo } from '../models/lote';
import { SerieInfo } from '../models/serie';
import { TiposInventarioItem } from '../models/tipo-inventario';
import { ArticuloCostoEstandar } from '../models/articulo-costo-estandar';

@Injectable()
export class ArticuloStockService extends CatalogService<ArticuloStockInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/inventory/articulos-stock");
    }

    create(): Promise<ArticuloStockInfo> {
        return this.apiService.get<ArticuloStockInfo>('create', null)
            .toPromise();
    }

    loadList(): Promise<TiposInventarioItem[]> {
        let params = new HttpParams()
        return this.apiService.get<TiposInventarioItem[]>("load-list", params)
            .toPromise();
    }

    loadFotografias(idArticuloStock: number): Promise<ArticulosFotografia[]> {
        let params = new HttpParams()
            .set("idArticuloStock", idArticuloStock.toString());
        return this.apiService.get<ArticulosFotografia[]>("load-fotografias", params)
            .toPromise();
    }

    loadCostoEstandarList(idSucursal: number): Promise<ArticuloCostoEstandar[]> {
        let params = new HttpParams()
            .set("idSucursal", idSucursal.toString());
        return this.apiService.get<ArticuloCostoEstandar[]>("load-costo-estandar-list", params)
            .toPromise();
    }

    costosEstandarUpdate(articulosCostoEstandar: ArticuloCostoEstandar[]): Promise<any> {
        let params = new HttpParams();
        return this.apiService.post<any>("update-costo-estandar", params, articulosCostoEstandar)
            .toPromise();
    }

    agregarFotografia(idArticuloStock: number, articulosFotografia: ArticulosFotografia): Promise<any> {
        let params = new HttpParams()
            .set("idArticuloStock", idArticuloStock.toString());
        return this.apiService.post<any>("agregar-fotografia", params, articulosFotografia)
            .toPromise();

    }

    eliminarFotografia(idArticuloStock: number, idArchivo: string): Promise<any> {
        let params = new HttpParams()
            .set("idArticuloStock", idArticuloStock.toString())
            .set("idArchivo", idArchivo);
        return this.apiService.delete<any>("eliminar-fotografia", params)
            .toPromise();
    }

    loadExistenciasLote(idArticuloStock: number, idAlmacen: number): Promise<LoteInfo[]> {
        let params = new HttpParams()
            .set('idArticuloStock', idArticuloStock.toString())
            .set('idAlmacen', idAlmacen.toString());
        return this.apiService.get<LoteInfo[]>('existencia-lotes', params)
            .pipe(map(response => {
                response.forEach(lote => {
                    lote.fechaCaducidad = moment(lote.fechaCaducidad).toDate();
                    lote.fechaIngreso = moment(lote.fechaIngreso).toDate();
                    lote.fecha = moment(lote.fecha).toDate();
                });
                return response;
            }))
            .toPromise();
    }

    loadExistenciasSerie(idArticuloStock: number, idAlmacen: number): Promise<SerieInfo[]> {
        let params = new HttpParams()
            .set('idArticuloStock', idArticuloStock.toString())
            .set('idAlmacen', idAlmacen.toString());
        return this.apiService.get<SerieInfo[]>('existencia-serie', params)
            .pipe(map(response => {
                response.forEach(serie => {
                    serie.fechaCaducidad = moment(serie.fechaCaducidad).toDate();
                    serie.fechaIngreso = moment(serie.fechaIngreso).toDate();                    
                });
                return response;
            }))
            .toPromise();
    }

    getSerieInfo(idArticuloStock: number, idAlmacen: number, numeroSerie: string): Promise<SerieInfo> {
        let params = new HttpParams()
            .set('idArticuloStock', idArticuloStock.toString())
            .set('idAlmacen', idAlmacen.toString())
            .set('numeroSerie', numeroSerie);
        return this.apiService.get<SerieInfo>('serie', params)
            .pipe(map(response => {
                if (response) {
                    response.fechaCaducidad = moment(response.fechaCaducidad).toDate();
                    response.fechaIngreso = moment(response.fechaIngreso).toDate();
                }
                return response;
            }))
            .toPromise();
    }

    searchKeyword2(idUsuario: number, idBranch: number, keyword: string): Observable<CatalogItem[]> {
        let params = new HttpParams()
            .set("idBranch", idBranch.toString())
            .set("tipo", idUsuario.toString())
            .set("keyWord", keyword);
        return this.apiService.get<CatalogItem[]>("search-articulo", params);

    }

}