﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, UsuarioItem } from 'framework3/core';
//Locales
import { ConceptoInfo, ConceptoItem } from '../models/concepto'

@Injectable()
export class ConceptoService extends CatalogService<ConceptoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/inventario/conceptos');
    }

    create(): Promise<ConceptoInfo> {
        return this.apiService.get<ConceptoInfo>('create', null)
            .toPromise();
    }

    loadList(idEmpresa: number, tipo: string): Promise<ConceptoItem[]> {
        let params = new HttpParams()
            .set('idEmpresa', idEmpresa.toString())        
            .set('tipo', tipo);
        return this.apiService.get<ConceptoItem[]>('load-list', params)
            .toPromise();
    }

	
    loadListByUser(idEmpresa: number, idUsuario: number, tipo: string): Promise<ConceptoItem[]> {
        let params = new HttpParams()
            .set('idEmpresa', idEmpresa.toString())
            .set('idUsuario', idUsuario.toString())
            .set('tipo', tipo);
        return this.apiService.get<ConceptoItem[]>('load-list-by-user', params)
            .toPromise();
    }

    getUsers(idConcepto: number): Promise<UsuarioItem[]> {
        let params = new HttpParams()
            .set('idConcepto', idConcepto.toString());
        return this.apiService.get<UsuarioItem[]>('users', params)
            .toPromise();
    }

    addUsers(idConcepto: number, users: UsuarioItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idConcepto', idConcepto.toString());
        return this.apiService.post<any>('add-users', params, users)
            .toPromise();
    }

    removeUser(idConcepto: number, idUsuario: number): Promise<any> {
        let params = new HttpParams()
            .set('idConcepto', idConcepto.toString())
            .set('idUsuario', idUsuario.toString());
        return this.apiService.get<any>('remove-user', params)            
            .toPromise();
    }

}

