﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Locales
import { SalidaInfo, SalidaItem } from '../models/salida';
import { SalidaDetail } from '../models/salida-detail';
import { Empaquetado } from '../models/empaquetado';

@Injectable()
export class SalidaService extends DocumentService<SalidaDetail, SalidaInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/inventario/salidas');
    }

    create(): Promise<SalidaInfo> {
        let newDoc = new SalidaInfo();
        return Promise.resolve(newDoc);
    }

    protected mapSearchResult(row: any) {
        row.horaAplicado = moment(row.horaAplicado).toDate();
    }


    protected mapDocument(document: SalidaInfo) {
        document.details.forEach(detail => {
            detail.fechaCaducidad = moment(detail.fechaCaducidad).toDate();
            //Determinar si no existe fecha de caducidad
            if (!detail.stock.manejaCaducidad) {
                detail.fechaCaducidad = null;
            }
            //Si la unidad de medida es un paquete, separar unidades
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor);
                detail.cantidad = cantidad.paquetes;
                detail.unidades = cantidad.unidades;
            }
            else {
                detail.unidades = 0;
            }
        });
    }

    update(info: SalidaInfo): Promise<SalidaInfo> {
        //Si no tiene asignado la empresa y sucursal, asignar la actual
        if (info.idSucursal == 0) info.idSucursal = this.context.app.defaultSite.identity;
        //Actualizar campo de cantidad basado en las propiedades auxiliares
        info.details.forEach(detail => {
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor, detail.unidades);
                detail.cantidad = cantidad.toDecimal();
            }
        });
        //
        let Promise = super.update(info);
        //Actualizar detalle del paquete retornado (actualizado)
        Promise.then(salida => {
            salida.details.forEach(detail => {
                if (detail.articuloStock.unidadMedida.paquete && detail.articuloStock.unidadMedida.factor > 1) {
                    let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor, detail.unidades);
                    detail.cantidad = cantidad.paquetes;
                    detail.unidades = cantidad.unidades;
                }
                if (!detail.stock.manejaCaducidad) {
                    detail.fechaCaducidad = null;
                }
            });
        });

        return Promise;
    }


}