﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Locales
import { TraspasoInfo, TraspasoItem } from '../models/traspaso';
import { TraspasoDetail } from '../models/traspaso-detail';
import { Empaquetado } from '../models/empaquetado';

@Injectable()
export class TraspasoService extends DocumentService<TraspasoDetail, TraspasoInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/inventario/traspasos-tit');
    }

    create(): Promise<TraspasoInfo> {
        let newDoc = new TraspasoInfo();
        return Promise.resolve(newDoc);
    }

    update(info: TraspasoInfo): Promise<TraspasoInfo> {
        //Si no tiene asignado la empresa y sucursal, asignar la actual
        if (info.idSucursal == 0) info.idSucursal = this.context.app.defaultSite.identity;
        //Actualizar campo de cantidad basado en las propiedades auxiliares
        info.details.forEach(detail => {
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor, detail.unidades);
                detail.cantidad = cantidad.toDecimal();
                detail.cantidadEnviada = cantidad.toDecimal();
            }
        });
        //
        let Promise = super.update(info);
        //Actualizar detalle del paquete retornado (actualizado)
        Promise.then(salida => {
            salida.details.forEach(detail => {
                if (detail.articuloStock.unidadMedida.paquete && detail.articuloStock.unidadMedida.factor > 1) {
                    let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor, detail.unidades);
                    detail.cantidad = cantidad.paquetes;
                    detail.unidades = cantidad.unidades;
                }
                if (!detail.stock.manejaCaducidad) {
                    detail.fechaCaducidad = null;
                }
            });
        });

        return Promise;
    }

    protected mapDocument(document: TraspasoInfo) {
        document.details.forEach(detail => {
            detail.fechaCaducidad = moment(detail.fechaCaducidad).toDate();
            //Determinar si no existe fecha de caducidad
            if (!detail.stock.manejaCaducidad) {
                detail.fechaCaducidad = null;
            }
            //Si la unidad de medida es un paquete, separar unidades
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidadEnviada, detail.unidadMedida.factor);
                detail.cantidad = cantidad.paquetes;
                detail.unidades = cantidad.unidades;
            }
            else {
                detail.unidades = 0;
            }
        });
    }

	/*
    sampleMethod(param1: number, param2: string): Promise<TraspasoInfo> {
        let params = new HttpParams()
			.set('param1', param1.toString())
			.set('param2', param2);
		//
        return this.apiService.get<TraspasoInfo>('sample-method', params)
            .toPromise();
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: TraspasoInfo) {

    }
	*/



}