﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';

//Locales
import { AplicacionService } from '../services/aplicacion.service';
import { AplicacionInfo } from '../models/aplicacion';

@Component({
    selector: 'maintenance-aplicacion-configuraciones-qr',
    templateUrl: './aplicacion-configuraciones-qr.component.html'
})
export class AplicacionConfiguracionesQr extends ProcessComponent {
    form: FormGroup;
    idAplicacion: number = 0;
    aplicacion: AplicacionInfo = new AplicacionInfo;

    configuracionesCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;


    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, private aplicacionSvc: AplicacionService) {
        super(context, router, route);
        this.route.params.subscribe((params: Params) => {
            this.idAplicacion = params['id'];
        });


        this.form = formBuilder.group({


        });
    }


    ngOnInit() {
        this.aplicacionSvc.getDetailByID(this.idAplicacion)
            .then(item => {
                this.aplicacion = item;

                this.configuracionesCollection.sourceCollection = this.aplicacion.configuraciones;
            })
            .catch(error => this.handleError(error));

    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'GENERAR-QR') {
            this.context.app.showSpinner()
            this.aplicacionSvc.getFileConfiguracionesQr(this.aplicacion.package, this.aplicacion.configuraciones)
                .then(result => {
                    var sResult: FileLink;
                    sResult = result;

                    var content = window.location.protocol + "//" + window.location.hostname + ":" + window.location.port + "/file-server/" + result.uri;
                    window.open(content);
                    this.context.app.hideSpinner();
                })
                .catch(e => this.handleError(e));

        }
        if (button.name == 'INVERTIR') {
            this.aplicacion.configuraciones.forEach(item => {
                item.enviarQr = !item.enviarQr;
            });
            this.configuracionesCollection.sourceCollection = this.aplicacion.configuraciones;
            this.configuracionesCollection.refresh;
            Control.invalidateAll();
        }


    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}