﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { FileInfo } from 'framework3/general';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ComplementoService} from '../services/complemento.service';
import { ComplementoInfo, ComplementoItem } from '../models/complemento';
import { TipoLicenciaItem } from '../models/tipo-licencia';
import { TipoLicenciaService } from '../services/tipo-licencia.service';
import { SistemaService } from '../services/sistema.service';
import { SistemaInfo } from '../models/sistema';
import { PropietarioItem } from '../models/propietario';
import { PropietarioService } from '../services/propietario.service';

import { CFDValiationResult } from '../../purchasing/models/cfd-validation-result';
import { ComprobanteFiscalService } from '../../purchasing/services/comprobante-fiscal.service';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../../purchasing/models/comprobante-fiscal';
import { ProveedorInfo } from '../../purchasing/models/proveedor';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

import * as moment from 'moment';


@Component({
    selector: 'mantenimiento-complemento-editor',
    templateUrl: './complementos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ComplementosEditor)
        },
        {
            provide: CatalogService, useExisting: ComplementoService
        }
    ]
})
export class ComplementosEditor extends CatalogEditor {
    complemento: ComplementoInfo = new ComplementoInfo();    
    tiposLicencias: TipoLicenciaItem[] = [];
    propietarios: PropietarioItem[] = [];
    idSistema: number = 0;
    idSoftware: number = 0;
    sistema: SistemaInfo = new SistemaInfo;


    origenesCF: { id: string, nombre: string }[] = [];
    idProveedor: number = 0;
    origenCF: string = "L";

    selectedFile: FileInfo;


    @ViewChild('cargaComprobanteFiscalDialog') origenCFDialog: ModalDirective;
    @ViewChild('comprobantesList') comprobantesListDialog: ModalDirective;
    @ViewChild('uploadDialog') uploadDialog: ModalDirective;


    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private complementoService: ComplementoService,
        private tipoLicenciaService: TipoLicenciaService, private sistemaService: SistemaService, private propietarioService: PropietarioService,
        private comprobanteFiscalService: ComprobanteFiscalService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'descripcion': ['', Validators.required],
            'noVersion': ['', Validators.required],
            'idTipoLicencia': [0, Validators.required],
            'idPropietario': [0, Validators.required],
            'idSubMarca': [0, Validators.required],
            'adquisicion': [new Date, (control) => this.validateFecha(control)],
            'activacion': [new Date, (control) => this.validateFecha(control)],
            'vencimiento': [new Date, (control) => this.validateFecha(control)],
            'numSerie': ['', Validators.required],
            'clabe': ['', Validators.required],
            'cantActivacion': [0, Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

    loadTiposLicencias() {
        this.tipoLicenciaService.loadList(this.context.app.defaultSite.identity)
            .then(tipos => this.tiposLicencias = tipos)
            .catch(error => this.handleError(error));
    }

    loadSistema() {
        this.sistemaService.getDetailByID(this.idSistema)
            .then(sis => {
                this.sistema = sis;
            })
            .catch(error => this.handleError(error));
    }

    loadPropietarios() {
        this.propietarioService.loadList(this.context.app.defaultSite.identity)
            .then(tipos => this.propietarios = tipos)
            .catch(error => this.handleError(error));
    }


    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idSoftware = +params['idSoftware'];
            this.idSistema = +params['idSistema'];
        });

        this.origenesCF =
            [
                { "id": 'L', "nombre": "Ruta Local" },
                { "id": 'A', "nombre": "Almacen digital" },
            ];


    }

    validateFecha(control: FormControl) {
        let FechaHoy: Date;
        FechaHoy = moment(new Date).hour(0).minute(0).second(0).toDate();

        if (control.value) {
            let Fecha = control.value;
            Fecha = moment(Fecha).hour(0).minute(0).second(0).toDate();
            /*
            if (Fecha.getTime() >= FechaHoy.getTime())
                return null;
            else
                return 'La fecha no debe ser anterior a hoy';
            */
        }
    }



	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadTiposLicencias();
        this.loadPropietarios();
        this.loadSistema();

    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.complementoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.complemento = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.complementoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.complemento = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.complementoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.complemento = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();

        this.complemento.licencia.idSistema = this.idSistema;
        this.complemento.idSoftware = this.idSoftware;

        this.complementoService.update(this.complemento)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.complemento.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.complemento = info;
            })
            .catch(error => this.handleError(error));        
    }


    onOrigenCFSelected() {

        if (this.origenCF == "A" && this.idProveedor == 0)
            this.showAlert("Debe seleccionar un proveedor ", 'danger');
        else {
            this.origenCFDialog.hide();

            // Ver que va a cargar
            //this.comprobantesListDialog.config.backdrop = false;
            if (this.origenCF == "A")
                this.comprobantesListDialog.show();
            else {
                this.uploadDialog.config.backdrop = false;
                this.uploadDialog.show();
            }
        }

    }

    onFileChanged(file: FileInfo) {
        this.selectedFile = file;
    }

    onValidateFile() {
        this.uploadDialog.hide();
        this.validateFile();
    }

    validateFile() {
        this.context.app.showSpinner();
        this.comprobanteFiscalService.validate(this.context.app.defaultCompany.identity, this.selectedFile)
            .then(result => {
                this.context.app.hideSpinner();
                this.cathComprobantes(result);
                //this.validatedFiles = result;
                //this.comprobantesCollection.sourceCollection = result;
            })
            .catch(error => this.handleError(error));
    }

    cathComprobantes(ValidCFDcomprobantes: CFDValiationResult[]) {
        let promises: any[] = [];

        ValidCFDcomprobantes.forEach(validCFD => {
            if (validCFD.valid) {
                this.context.app.showSpinner();
                let savePromise = this.comprobanteFiscalService.save(0, this.context.app.defaultCompany.identity, this.context.app.defaultSite.identity, validCFD.comprobante.comprobanteBase.contenidoXML)
                    .then(comprobante => {
                        this.context.app.hideSpinner();
                        this.idProveedor = comprobante.idProveedor;
                        this.onAsignar(comprobante);
                    })
                    .catch(error => this.handleError(error));
            }
            else
                this.showAlert("Algunos comprobantes no se pudieron validar correctamente:[" + validCFD.validationMessage + "]", "danger")
        });
    }


    onProveedorChanged(proveedor: ProveedorInfo) {
        this.idProveedor = proveedor.identity;
    }

    onComprobanteSelected(comprobante: ComprobanteFiscalItem) {
        this.comprobanteFiscalService.getDetail(comprobante.id)
            .then(comp => {
                this.onAsignar(comp);
            })
            .catch(error => this.handleError(error));
    }

    onRemoveComprobante(item) {
        /*
        this.previewCollectionDetallesComprobantes.remove(item);
        this.previewCollectionDetallesComprobantes.refresh();
        */
    }


    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SELECCIONAR') {

            this.origenCFDialog.config.backdrop = false;
            this.idProveedor = 0;
            this.origenCFDialog.show();

        }
    }

    onAsignar(info: ComprobanteFiscalInfo) {

        this.complemento.licencia.idComprobante = info.identity;
        this.complemento.licencia.fechaComprobante = info.fechaSys;

    }



}