﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition, ConditionValueType } from 'framework3/search';
//Features
import { ComplementoService} from '../services/complemento.service';
import { ComplementoInfo, ComplementoItem} from '../models/complemento';

@Component({    
    selector: 'mantenimiento-complementos-manager',
    templateUrl: './complementos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ComplementosManager)
        },
        {
            provide: CatalogService, useExisting: ComplementoService
        }
	]
})
export class ComplementosManager extends CatalogManager {
    complementos: ComplementoItem[] = [];    
    idSistema: number = 0;
    idSoftware: number = 0;
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private complementoService: ComplementoService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////
    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idSistema = +params['idSistema'];
            this.idSoftware = +params['idSoftware'];
        });
    }

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let grupoCond: SearchCondition = { field: 'idSoftware', operator: '=', value: this.idSoftware.toString(), valueType: ConditionValueType.INT };
        request.conditions.push(grupoCond);


        this.complementoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.complementos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0, { idSistema: this.idSistema, idSoftware: this.idSoftware }], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id, { idSistema: this.idSistema, idSoftware: this.idSoftware }], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.complementoService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.complementoService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
