﻿import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';

import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent, AzCheckboxComponent, } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { ProcessComponent } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { TrackableCollection, CatalogEditor, CatalogManager } from 'framework3/editors';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
//Wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import { Menu } from 'wijmo/wijmo.input';
import { forwardRef } from '@angular/core';
import { CollectionView, DateTime } from 'wijmo/wijmo';
//Features
import { DatasetInfo, ParameterItem, TranspilerInfo, DatasetService } from '../../translator/index';
import { error } from '@angular/compiler/src/util';
import { ConfigDispositivosService } from '../services/configdispositivos.service';
import { getDispositivosConfiguracionModelos } from '../models/DispositivosConfiguracionModelo';

@Component({
    selector: 'maintenance-configuraciones-dispositivos',
    templateUrl: './configuraciones-dispositivos.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ConfiguracionesDispositivos)
        },
        {
            provide: CatalogService, useExisting: ConfigDispositivosService
        }
    ]
})


export class ConfiguracionesDispositivos extends CatalogManager{
    chbxPreventa:   boolean = false;
    chbxReparto:    boolean = false;
    chbxAutoventa: boolean = false;
    dispositivo: string;

    fechaFinal: DateTime;
    dispositivosconfig: getDispositivosConfiguracionModelos[] = [];
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    previewCollection: CollectionView = new CollectionView();

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private dispositivosService: ConfigDispositivosService) {
        super(context, router, route);
    }


    OnInit(request: SearchRequest) {
        this.context.app.showSpinner();
        this.dispositivosService.getClasificacionesDispositivos(this.dispositivo)
            .then(result => {
                this.context.app.hideSpinner();
                this.dispositivosconfig = result;
                //this.previewCollection.sourceCollection = this.dispositivosconfig;  
                this.pageIndex = request.pageIndex;
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Error obtener información.", "danger")
            })
    }



    selectionChangePreventa(input: HTMLInputElement) {
        if (input.checked === true) {
            this.dispositivo = "preventa"
            this.chbxReparto = false;
            this.chbxAutoventa = false;
        }
    }

    selectionChangeReparto(input: HTMLInputElement) {
        if (input.checked === true) {
            this.dispositivo = "reparto"
            this.chbxPreventa = false;
            this.chbxAutoventa = false;
        }
    }

    selectionChangeAutoventa(input: HTMLInputElement) {
        if (input.checked === true) {
            this.dispositivo = "autoventa"
            this.chbxReparto = false;
            this.chbxPreventa = false;
        }
    }



    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;
        this.refreshList();
        this.applyPolicies();
    }

    //CARGAR INFORMACION DE CATALOGO CONFIGURACIONES
    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.dispositivosService.getClasificacionesDispositivos(this.dispositivo)
            .then(result => {
                this.context.app.hideSpinner();
                this.dispositivosconfig = result;
                this.previewCollection.sourceCollection = this.dispositivosconfig;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Error obtener información.", "danger")
            })

    }



    //ENVIAR NOTIFICACION
    enviarnotificacion() {
        this.context.app.showSpinner();
        this.dispositivosService.sendNotificaciones(this.dispositivo)
            .then(result => {
                this.context.app.hideSpinner();
                
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Error al enviar notificaciones.", "danger")
            })
    }



}