﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ConfiguracionService} from '../services/configuracion.service';
import { ConfiguracionInfo, ConfiguracionItem} from '../models/configuracion';

@Component({
    selector: 'mantenimiento-configuracion-editor',
    templateUrl: './configuraciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ConfiguracionesEditor)
        },
        {
            provide: CatalogService, useExisting: ConfiguracionService
        }
    ]
})
export class ConfiguracionesEditor extends CatalogEditor {
    configuracion: ConfiguracionInfo = new ConfiguracionInfo();   
    tipos: { id: string, nombre: string }[] = [];
    dataTypes: { id: string, nombre: string }[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private configuracionService: ConfiguracionService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'keyConfiguracion': ['', Validators.required],
            'defaultValue': ['', Validators.required],
            'descripcion': ['', Validators.required],
            'tipo': ['', Validators.required],
            'dataType': ['', Validators.required]
        });

        this.tipos =
            [
                { "id": "D", "nombre": "Dispositivo" },
                { "id": "A", "nombre": "Aplicación" }
            ];

        this.dataTypes =
            [
            { "id": "string", "nombre": "string" },
            { "id": "int", "nombre": "int" },
            { "id": "double", "nombre": "double" },
            { "id": "long", "nombre": "long" },
            { "id": "float", "nombre": "float" },
            { "id": "boolean", "nombre": "boolean" }
            ];
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.configuracionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.configuracion = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.configuracionService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.configuracion = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.configuracionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.configuracion = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.configuracionService.update(this.configuracion)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.configuracion.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.configuracion = info;
            })
            .catch(error => this.handleError(error));        
    }

}