﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { DespieceInfo } from '../models/despiece';
import { DespieceService } from '../services/despiece.service';

import { RefaccionInfo, RefaccionItem, RefaccionSelectorComponent } from '../../maintenance';
import { EquipoGrupoService } from '../services/equipo-grupo.service';
import { GrupoEquiposInfo } from '../models/grupo-equipos';


@Component({    
    selector: 'mantenimiento-despiece-manager',
    templateUrl: './despiece.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DespieceManager)
        },
        {
            provide: CatalogService, useExisting: DespieceService
        }
    ]
})
export class DespieceManager extends HierarchyManager {
    despiece: DespieceInfo[] = [];
    despieceEdicion: DespieceInfo = new DespieceInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    idEquipoGrupo: number = 0;
    grupoEquipo: GrupoEquiposInfo = new GrupoEquiposInfo();

    @ViewChild('selectorRefaccion') selectorRefaccion: RefaccionSelectorComponent;


    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private despieceService: DespieceService, protected router: Router, protected route: ActivatedRoute, private equipoGrupoService: EquipoGrupoService) {
        super(context, router, route, changeDetector);
       
        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'descripcion': ['', Validators.required],
            'idRefaccion':[0]
        });
    }

    ////////////////////// METODOS //////////////////////////////////////////////

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipoGrupo = +params['idEquipoGrupo'];
        });
    }


    loadHierarchy() {
        this.context.app.showSpinner();
        //this.despieceService.loadHierarchy(this.context.app.defaultCompany.identity)
        this.despieceService.LoadPiezas(this.idEquipoGrupo, this.context.app.defaultCompany.identity)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.despiece = nodes;
            })
            .catch(error => this.handleError(error));
    }

    loadEquipoGrupo() {
        this.equipoGrupoService.getDetailByID(this.idEquipoGrupo)
            .then(e => {
                this.grupoEquipo = e;
            })
            .catch(error => this.handleError(error));
    }


	//////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadHierarchy();
        this.despieceEdicion.idEquipoGrupo = this.idEquipoGrupo;
        //Hacer llamadas de carga inicial de datos aquí
        this.loadEquipoGrupo();
    }


    onNodesChanged(despiece: DespieceInfo[]) {
        this.despiece = despiece;        
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.despieceEdicion = new DespieceInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.despieceEdicion = new DespieceInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.despieceEdicion.idBranch = this.context.app.defaultSite.identity;
        this.despieceEdicion.idCompany = this.context.app.defaultCompany.identity;
        this.despieceEdicion.idEquipoGrupo = this.idEquipoGrupo;
		//
        this.currentParent = parentNode;        
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {        
        this.despieceEdicion = this.despieceService.copy(node.data);
        this.showEditor();        
    }

    onSavingNode() {        
        //Si es un nuevo nodo
        if (this.despieceEdicion.clientID == 0) {
            this.addNode(this.despieceEdicion, this.currentParent);            
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.despieceEdicion);
        }
		this.despieceEdicion = new DespieceInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<DespieceInfo>) {
        this.context.app.showSpinner();
        this.despieceService.saveHierarchy(changes)
            .then(despiece => {
                this.despiece = despiece;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));            
    }

}