﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Features
import { DispositivoGrupoService} from '../services/dispositivo-grupo.service';
import { DispositivoGrupoInfo, DispositivoGrupoItem} from '../models/dispositivo-grupo';

@Component({    
    selector: 'mantenimiento-dispositivos-grupos-manager',
    templateUrl: './dispositivos-grupos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DispositivosGruposManager)
        },
        {
            provide: CatalogService, useExisting: DispositivoGrupoService
        }
	]
})
export class DispositivosGruposManager extends CatalogManager {
    dispositivosGrupos: DispositivoGrupoItem[]=[];    
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private dispositivoGrupoService: DispositivoGrupoService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////
    onMenuItemClick(item: MenuItem) {

        if (item.name == 'DISPOSITIVOS') {

            this.dispositivoGrupoService.getDetailByID(this.manager.currentItem.id)
                .then(det => {
                    this.router.navigate(['maintenance/dispositivos', this.process.id, det.identity, det.identity, det.idEquipoGrupo]);
                })
                .catch(error => this.handleError(error));


        }


    }
	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.dispositivoGrupoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.dispositivosGrupos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.dispositivoGrupoService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.dispositivoGrupoService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
