﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { GrupoEquiposIndustrialesService} from '../services/grupo-equipos-industriales.service';
import { GrupoEquiposIndustrialesInfo, GrupoEquiposIndustrialesItem } from '../models/grupo-equipos-industriales';

import { MarcaItem, MarcaSelectorComponent, MarcaInfo } from '../../maintenance';

@Component({
    selector: 'mantenimiento-equipos-industriales-grupos-editor',
    templateUrl: './equipos-industriales-grupos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EquiposIndustrialesGruposEditor)
        },
        {
            provide: CatalogService, useExisting: GrupoEquiposIndustrialesService
        }
    ]
})
export class EquiposIndustrialesGruposEditor extends CatalogEditor {
    grupoEquiposIndustriales: GrupoEquiposIndustrialesInfo = new GrupoEquiposIndustrialesInfo();      
    @ViewChild('selectorMarca') selectorMarca: MarcaSelectorComponent;
    marca: MarcaInfo = new MarcaInfo();
    idServicioTipo: number = 2;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private grupoEquiposIndustrialesService: GrupoEquiposIndustrialesService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'marca': [''],
            'idSubMarca': [0, AztecaValidators.selected],
            'modelo': ['', Validators.required],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoEquiposIndustrialesService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoEquiposIndustriales = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.grupoEquiposIndustrialesService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoEquiposIndustriales = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoEquiposIndustrialesService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoEquiposIndustriales = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.grupoEquiposIndustrialesService.update(this.grupoEquiposIndustriales)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoEquiposIndustriales = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'EQUIPOSINDUSTRIALES') {
            this.router.navigate(['maintenance/equipos-industriales', this.process.id, this.grupoEquiposIndustriales.identity, this.grupoEquiposIndustriales.identity, this.grupoEquiposIndustriales.idEquipoGrupo]);
        }
        if (item.name == 'DESPIECE') {
            this.router.navigate(['maintenance/despiece', this.process.id, this.grupoEquiposIndustriales.idEquipoGrupo]);
        }

    }


    onSelectedItemChanged(marca: MarcaInfo) {
        if (marca) {
            this.marca = marca;
        }
    }


}