﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { AztecaValidators, MenuItem } from 'framework3/controls';
//Features
import { GrupoEquiposIndustrialesService} from '../services/grupo-equipos-industriales.service';
import { GrupoEquiposIndustrialesInfo, GrupoEquiposIndustrialesItem} from '../models/grupo-equipos-industriales';

@Component({    
    selector: 'mantenimiento-equipos-industriales-grupos-manager',
    templateUrl: './equipos-industriales-grupos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EquiposIndustrialesGruposManager)
        },
        {
            provide: CatalogService, useExisting: GrupoEquiposIndustrialesService
        }
	]
})
export class EquiposIndustrialesGruposManager extends CatalogManager {
    equiposIndustrialesGrupos: GrupoEquiposIndustrialesItem[]=[];    
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private grupoEquiposIndustrialesService: GrupoEquiposIndustrialesService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.grupoEquiposIndustrialesService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.equiposIndustrialesGrupos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.grupoEquiposIndustrialesService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.grupoEquiposIndustrialesService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem) {
        
        if (item.name == 'EQUIPOSINDUSTRIALES') {

            this.grupoEquiposIndustrialesService.getDetailByID(this.manager.currentItem.id)
                .then(det => {
                    this.router.navigate(['maintenance/equipos-industriales', this.process.id, det.identity, det.identity, det.idEquipoGrupo]);

                })
                .catch(error => this.handleError(error));


        }
        
        if (item.name == 'DESPIECE') {
            this.grupoEquiposIndustrialesService.getDetailByID(this.manager.currentItem.id)
                .then(det => {
                    this.router.navigate(['maintenance/despiece', this.process.id, det.idEquipoGrupo]);
            })
            .catch(error => this.handleError(error));
            
        }

    }


}
