﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { EquipoIndustrialService} from '../services/equipo-industrial.service';
import { EquipoIndustrialInfo, EquipoIndustrialItem } from '../models/equipo-industrial';

import { GrupoEquiposIndustrialesService } from '../services/grupo-equipos-industriales.service';
import { GrupoEquiposIndustrialesInfo } from '../models/grupo-equipos-industriales';

import { LocalizacionInfo, LocalizacionItem, LocalizacionSelectorComponent } from '../../maintenance';

@Component({
    selector: 'mantenimiento-equipos-industriales-editor',
    templateUrl: './equipos-industriales.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EquiposIndustrialesEditor)
        },
        {
            provide: CatalogService, useExisting: EquipoIndustrialService
        }
    ]
})
export class EquiposIndustrialesEditor extends CatalogEditor {
    equipoIndustrial: EquipoIndustrialInfo = new EquipoIndustrialInfo();        
    idEquipoGrupo: number = 0;
    idEquipoIndustrialGrupo: number = 0;
    equipoGrupo: GrupoEquiposIndustrialesInfo = new GrupoEquiposIndustrialesInfo;
    @ViewChild('selectorLocalizacion') selectorMarca: LocalizacionSelectorComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private equipoIndustrialService: EquipoIndustrialService, private grupoEquipoIndustrialService: GrupoEquiposIndustrialesService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'numEconomico': [''],
            'idLocalizacion': [0],
            'perfil': ['QUEDA PENDIENTE HASTA COMPLETAR MODULO DE MANTENIMIENTO']
        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipoGrupo = params['idEquipoGrupo'];
            this.idEquipoIndustrialGrupo = params['idEquipoIndustrialGrupo'];
        });
    }


	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadEquipoGrupo();
    }

    onCreatingItem() {
        this.equipoIndustrial.equipo.idEquipoGrupo = this.idEquipoGrupo;
    }

    loadEquipoGrupo() {
        this.grupoEquipoIndustrialService.getDetailByID(this.idEquipoIndustrialGrupo)
            .then(info => {
                this.equipoGrupo = info;
            })
            .catch(error => this.handleError(error));
    }


    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.equipoIndustrialService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoIndustrial = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.equipoIndustrialService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoIndustrial = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.equipoIndustrialService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoIndustrial = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.equipoIndustrialService.update(this.equipoIndustrial)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.equipoIndustrial.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.equipoIndustrial = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'DOCUMENTOS') {
            this.router.navigate(['maintenance/documentos', this.equipoIndustrial.idEquipo, this.process.id]);
        }
        if (item.name == 'FOTOGRAFIAS') {
            this.router.navigate(['maintenance/fotografias', this.equipoIndustrial.idEquipo, this.process.id]);
        }

    }


}