﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition, ConditionValueType } from 'framework3/search';
//Features
import { EquipoIndustrialService} from '../services/equipo-industrial.service';
import { EquipoIndustrialInfo, EquipoIndustrialItem} from '../models/equipo-industrial';

@Component({    
    selector: 'mantenimiento-equipos-industriales-manager',
    templateUrl: './equipos-industriales.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EquiposIndustrialesManager)
        },
        {
            provide: CatalogService, useExisting: EquipoIndustrialService
        }
	]
})
export class EquiposIndustrialesManager extends CatalogManager {
    equiposIndustriales: EquipoIndustrialItem[] = [];
    idEquipoGrupo: number = 0;
    idEquipoIndustrialGrupo: number = 0;
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private equipoIndustrialService: EquipoIndustrialService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    ngOnInit() {
        this.route.params.forEach((params: Params) => {

            this.idEquipoGrupo = params['idEquipoGrupo'];
            this.idEquipoIndustrialGrupo = params['idEquipoIndustrialGrupo'];


        });
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let grupoCond: SearchCondition = { field: 'IdEquipoIndustrialGrupo', operator: '=', value: this.idEquipoIndustrialGrupo.toString(), valueType: ConditionValueType.INT };
        request.conditions.push(grupoCond);


        this.equipoIndustrialService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.equiposIndustriales = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.equipoIndustrialService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.equipoIndustrialService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
