﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { EquipoInformaticaGrupoService} from '../services/equipo-informatica-grupo.service';
import { EquipoInformaticaGrupoInfo, EquipoInformaticaGrupoItem} from '../models/equipo-informatica-grupo';

import { MarcaItem, MarcaSelectorComponent, MarcaInfo, EquipoInformaticaTipoItem } from '../../maintenance';

import { EquipoInformaticaTipoService } from '../services/equipo-informatica-tipo.service';


@Component({
    selector: 'mantenimiento-equipo-informatica-editor',
    templateUrl: './equipos-informatica-grupos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EquiposInformaticaGruposEditor)
        },
        {
            provide: CatalogService, useExisting: EquipoInformaticaGrupoService
        }
    ]
})
export class EquiposInformaticaGruposEditor extends CatalogEditor {
    equipoInformaticaGrupo: EquipoInformaticaGrupoInfo = new EquipoInformaticaGrupoInfo();        
    @ViewChild('selectorMarca') selectorMarca: MarcaSelectorComponent;
    marca: MarcaInfo = new MarcaInfo();
    idEquipoInformaticaTipo: number = 0;
    tiposEquipos: EquipoInformaticaTipoItem[] = [];
    idServicioTipo: number = 4;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private equipoInformaticaGrupoService: EquipoInformaticaGrupoService,
        private tipoEquipoService: EquipoInformaticaTipoService
    ) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'marca': [''],
            'idSubMarca': [0, AztecaValidators.selected],
            'modelo': [''],
            'idEquipoInformaticaTipo': [0, Validators.required],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadTiposEquipos();
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.equipoInformaticaGrupoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoInformaticaGrupo = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.equipoInformaticaGrupoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoInformaticaGrupo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.equipoInformaticaGrupoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoInformaticaGrupo = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.equipoInformaticaGrupoService.update(this.equipoInformaticaGrupo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.equipoInformaticaGrupo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.equipoInformaticaGrupo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'EQUIPOSINFORMATICA') {
            this.router.navigate(['maintenance/equipos-informatica', this.process.id, this.equipoInformaticaGrupo.identity, this.equipoInformaticaGrupo.identity, this.equipoInformaticaGrupo.idEquipoGrupo]);
        }
        if (item.name == 'DESPIECE') {
            this.router.navigate(['maintenance/despiece', this.process.id, this.equipoInformaticaGrupo.idEquipoGrupo]);
        }

    }


    onSelectedItemChanged(marca: MarcaInfo) {
        if (marca) {
            this.marca = marca;
        }
    }


    loadTiposEquipos() {
        this.tipoEquipoService.LoadList()
            .then(tipos => this.tiposEquipos = tipos)
            .catch(error => this.handleError(error));
    }


}