﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';
//Features
import { EquipoInformaticaService} from '../services/equipo-informatica.service';
import { EquipoInformaticaInfo, EquipoInformaticaItem} from '../models/equipo-informatica';

import { EquipoInformaticaGrupoService } from '../services/equipo-informatica-grupo.service';
import { EquipoInformaticaGrupoInfo, EquipoInformaticaGrupoItem } from '../models/equipo-informatica-grupo';


@Component({
    selector: 'mantenimiento-equipo-informatica-editor',
    templateUrl: './equipos-informatica.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EquiposInformaticaEditor)
        },
        {
            provide: CatalogService, useExisting: EquipoInformaticaService
        }
    ]
})
export class EquiposInformaticaEditor extends CatalogEditor {
    equipoInformatica: EquipoInformaticaInfo = new EquipoInformaticaInfo();        
    idEquipoGrupo: number = 0;
    idEquipoInformaticaGrupo: number = 0;
    equipoInformaticaGrupo: EquipoInformaticaGrupoInfo = new EquipoInformaticaGrupoInfo();
    clasificadores: Clasificacion[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private equipoInformaticaService: EquipoInformaticaService, private equipoInformaticaGrupoService: EquipoInformaticaGrupoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'procesador': [''],
            'velocidadProcesador': [0],
            'memoriaRAM': [0],
            'capacidadHD': [0],
            'direccionIP': [''],
            'numEconomico': ['', Validators.required],
            'idLocalizacion': [0, Validators.required],
            'perfil': ['QUEDA PENDIENTE HASTA COMPLETAR MODULO DE MANTENIMIENTO'],
            'clasificaciones': [[], AztecaValidators.validClasifications],
            'fechaCompra': ['']
        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipoGrupo = params['idEquipoGrupo'];
            this.idEquipoInformaticaGrupo = params['idEquipoInformaticaGrupo'];
        });
    }

    loadClasificadores() {
        this.equipoInformaticaService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => this.clasificadores = clasificadores)
            .catch(error => this.handleError(error));
    }



	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////


    loadEquipoInformaticaGrupo() {
        this.equipoInformaticaGrupoService.getDetailByID(this.idEquipoInformaticaGrupo)
            .then(info => {
                this.equipoInformaticaGrupo = info;
            })
            .catch(error => this.handleError(error));
    }


    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadEquipoInformaticaGrupo();
        this.loadClasificadores();
		this.applyPolicies();
    }

    onCreatingItem() {
        this.equipoInformatica.equipo.idEquipoGrupo = this.idEquipoGrupo;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.equipoInformaticaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoInformatica = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.equipoInformaticaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoInformatica = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.equipoInformaticaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.equipoInformatica = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.equipoInformaticaService.update(this.equipoInformatica)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.equipoInformatica.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.equipoInformatica = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'DOCUMENTOS') {
            this.router.navigate(['maintenance/documentos', this.equipoInformatica.idEquipo, this.process.id]);
        }
        if (item.name == 'FOTOGRAFIAS') {
            this.router.navigate(['maintenance/fotografias', this.equipoInformatica.idEquipo, this.process.id]);
        }

    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }



}