﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition, ConditionValueType } from 'framework3/search';
//Features
import { EquipoInformaticaService} from '../services/equipo-informatica.service';
import { EquipoInformaticaInfo, EquipoInformaticaItem} from '../models/equipo-informatica';

@Component({    
    selector: 'mantenimiento-equipos-informatica-manager',
    templateUrl: './equipos-informatica.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EquiposInformaticaManager)
        },
        {
            provide: CatalogService, useExisting: EquipoInformaticaService
        }
	]
})
export class EquiposInformaticaManager extends CatalogManager {
    equiposInformatica: EquipoInformaticaItem[] = []; 
    idEquipoInformaticaGrupo: number = 0;
    idEquipoGrupo: number = 0;
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private equipoInformaticaService: EquipoInformaticaService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipoGrupo = params['idEquipoGrupo'];
            this.idEquipoInformaticaGrupo = params['idEquipoInformaticaGrupo'];

        });
    }


    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let grupoCond: SearchCondition = { field: 'IdEquipoInformaticaGrupo', operator: '=', value: this.idEquipoInformaticaGrupo.toString(), valueType: ConditionValueType.INT };
        request.conditions.push(grupoCond);


        this.equipoInformaticaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.equiposInformatica = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.equipoInformaticaService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.equipoInformaticaService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
