﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features

import { MarcaItem, MarcaSelectorComponent, MarcaInfo } from '../../maintenance';
import { LlantaModeloService} from '../services/llanta-modelo.service';
import { LlantaModeloInfo, LlantaModeloItem} from '../models/llanta-modelo';

@Component({
    selector: 'mantenimiento-llanta-modelo-editor',
    templateUrl: './llantas-modelos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => LlantasModelosEditor)
        },
        {
            provide: CatalogService, useExisting: LlantaModeloService
        }
    ]
})
export class LlantasModelosEditor extends CatalogEditor {
    llantaModelo: LlantaModeloInfo = new LlantaModeloInfo(); 
    @ViewChild('selectorMarca') selectorMarca: MarcaSelectorComponent;
    marca: MarcaInfo = new MarcaInfo();
    idServicioTipo: number = 1;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private llantaModeloService: LlantaModeloService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'marca': [''],
            'idSubMarca': [0, AztecaValidators.selected],
            'tipo': ['', Validators.required],
            'ancho': [0, Validators.required],
            'alto': [0, Validators.required],
            'diametroRin': [0, Validators.required],
            'indiceCarga': [0, Validators.required],
            'limiteCarga': [0, Validators.required],
            'presionMax': [0, Validators.required],
            'profundidadMin': [0, Validators.required],
            'profundidadMax': [0, Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.llantaModeloService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.llantaModelo = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.llantaModeloService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.llantaModelo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.llantaModeloService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.llantaModelo = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.llantaModeloService.update(this.llantaModelo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.llantaModelo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.llantaModelo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'LLANTAS') {
            this.router.navigate(['maintenance/llantas', this.process.id, this.llantaModelo.identity, this.llantaModelo.identity, this.llantaModelo.idEquipoGrupo, this.llantaModelo.grupoEquipos.idMarca]);
        }

    }


    onSelectedItemChanged(marca: MarcaInfo) {
        if (marca) {
            this.marca = marca;
        }
    }


}