﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Features
import { LlantaModeloService} from '../services/llanta-modelo.service';
import { LlantaModeloInfo, LlantaModeloItem} from '../models/llanta-modelo';

@Component({    
    selector: 'mantenimiento-llantas-modelos-manager',
    templateUrl: './llantas-modelos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => LlantasModelosManager)
        },
        {
            provide: CatalogService, useExisting: LlantaModeloService
        }
	]
})
export class LlantasModelosManager extends CatalogManager {
    llantasModelos: LlantaModeloItem[]=[];    
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private llantaModeloService: LlantaModeloService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.llantaModeloService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.llantasModelos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.llantaModeloService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.llantaModeloService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    getModeloLlanta(idModeloLlanta: number):LlantaModeloInfo {

        var llantaModeloRet: LlantaModeloInfo;

        this.llantaModeloService.getDetailByID(idModeloLlanta)
            .then(inf => {
                llantaModeloRet = inf;
            })
            .catch(error => this.handleError(error));

        return llantaModeloRet;
    }

    onMenuItemClick(item: MenuItem) {

        if (item.name == 'LLANTAS') {

            this.llantaModeloService.getDetailByID(this.manager.currentItem.id)
                .then(inf => {
                    this.router.navigate(['maintenance/llantas', this.process.id, this.manager.currentItem.id, this.manager.currentItem.id, inf.idEquipoGrupo, inf.grupoEquipos.idMarca]);
                })
                .catch(error => this.handleError(error));

        }

    }


}
