﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition, ConditionValueType } from 'framework3/search';
//Features
import { LlantaService} from '../services/llanta.service';
import { LlantaInfo, LlantaItem } from '../models/llanta';
import { } from '../services/equipo.service'

@Component({    
    selector: 'mantenimiento-llantas-manager',
    templateUrl: './llantas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => LlantasManager)
        },
        {
            provide: CatalogService, useExisting: LlantaService
        }
	]
})
export class LlantasManager extends CatalogManager {
    llantas: LlantaItem[] = [];    
    idEquipoGrupo: number = 0;
    idMarca: number = 0;
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private llantaService: LlantaService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipoGrupo = +params['idEquipoGrupo'];
            this.idMarca = +params['idMarca'];
        });
    }


	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let grupoCond: SearchCondition = { field: 'idEquipoGrupo', operator: '=', value: this.idEquipoGrupo.toString(), valueType: ConditionValueType.INT };
        request.conditions.push(grupoCond);


        this.llantaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.llantas = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0, { idMarca: this.idMarca}], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id, { idMarca: this.idMarca }], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.llantaService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.llantaService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
