﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { LocalizacionInfo } from '../models/localizacion';
import { LocalizacionService } from '../services/localizacion.service';
import { LocalizacionTipoItem } from '../models/localizacion-tipo';
import { LocalizacionTipoService } from '../services/localizacion-tipo.service';

@Component({    
    selector: 'mantenimiento-localizaciones-manager',
    templateUrl: './localizaciones.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => LocalizacionesManager)
        },
        {
            provide: CatalogService, useExisting: LocalizacionService
        }
    ]
})
export class LocalizacionesManager extends HierarchyManager {
    localizaciones: LocalizacionInfo[] = [];
    localizacionEdicion: LocalizacionInfo = new LocalizacionInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    localizacionesTipos: LocalizacionTipoItem[] = [];

    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private localizacionService: LocalizacionService, protected router: Router, protected route: ActivatedRoute, private localizacionTipoService: LocalizacionTipoService) {
        super(context,router, route, changeDetector);
       
        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idLocalizacionTipo':[0]
        });
    }

	////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.localizacionService.loadHierarchy(this.context.app.defaultCompany.identity,0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.localizaciones = nodes;
            })
            .catch(error => this.handleError(error));
    }

    loadTipos() {
        this.localizacionTipoService.loadList(this.context.app.defaultSite.identity)
            .then(tipos => this.localizacionesTipos = tipos)
            .catch(error => this.handleError(error));
    }


	//////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadHierarchy();
        //Hacer llamadas de carga inicial de datos aquí
        this.loadTipos();
    }


    onNodesChanged(localizaciones: LocalizacionInfo[]) {
        this.localizaciones = localizaciones;        
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.localizacionEdicion = new LocalizacionInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.localizacionEdicion = new LocalizacionInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.localizacionEdicion.idBranch = this.context.app.defaultSite.identity;
        this.localizacionEdicion.idCompany = this.context.app.defaultCompany.identity;
		//
        this.currentParent = parentNode;        
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {        
        this.localizacionEdicion = this.localizacionService.copy(node.data);
        this.showEditor();        
    }

    onSavingNode() {        
        //Si es un nuevo nodo
        if (this.localizacionEdicion.clientID == 0) {
            this.addNode(this.localizacionEdicion, this.currentParent);            
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.localizacionEdicion);
        }
		this.localizacionEdicion = new LocalizacionInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<LocalizacionInfo>) {
        this.context.app.showSpinner();
        this.localizacionService.saveHierarchy(changes)
            .then(localizaciones => {
                this.localizaciones = localizaciones;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));            
    }

}