﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@Angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { MarcaService } from '../services/marcas.service';
import { MarcaInfo, MarcaItem } from '../models/marcas';

@Component({
    selector: 'inventory-marcas-manager',
    templateUrl: './marcas.manager.html',
    providers: [
        {
            provide: BaseComponent,useExisting:forwardRef(()=>MarcasManager)
        },
        {
            provide: CatalogService, useExisting: MarcaService
        }
    ]
})

export class MarcasManager extends CatalogManager {
    marcas: MarcaItem[];
    //

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private marcaService: MarcaService) {
        super(context, router, route);
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.marcaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.marcas = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.marcaService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, { cloneId: item.id }], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.marcaService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}