﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { PaqueteriaInfo } from '../models/paqueteria';
import { PaqueteriaService } from '../services/paqueteria.service';

import { SoftwareItem } from '../models/software'
import { SoftwareService } from '../services/software.service'

@Component({    
    selector: 'mantenimiento-paqueterias-manager',
    templateUrl: './paqueterias.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PaqueteriasManager)
        },
        {
            provide: CatalogService, useExisting: PaqueteriaService
        }
    ]
})
export class PaqueteriasManager extends HierarchyManager {
    paqueterias: PaqueteriaInfo[] = [];
    paqueteriaEdicion: PaqueteriaInfo = new PaqueteriaInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    softwares: SoftwareItem[] = [];
    
    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private paqueteriaService: PaqueteriaService, protected router: Router, protected route: ActivatedRoute, private softwareService: SoftwareService) {
        super(context, router, route, changeDetector);
       
        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idSoftware': [0, Validators.required],
            'descripcion': ['', Validators.required]
        });
    }

	////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.paqueteriaService.loadHierarchy(this.context.app.defaultCompany.identity,0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.paqueterias = nodes;
            })
            .catch(error => this.handleError(error));
    }

    loadSoftwares() {
        this.softwareService.loadList(this.context.app.defaultSite.identity)
            .then(sw => this.softwares = sw)
            .catch(error => this.handleError(error));
    }
	//////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadHierarchy();
        //Hacer llamadas de carga inicial de datos aquí
        this.loadSoftwares();

    }


    onNodesChanged(paqueterias: PaqueteriaInfo[]) {
        this.paqueterias = paqueterias;        
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.paqueteriaEdicion = new PaqueteriaInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.paqueteriaEdicion = new PaqueteriaInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.paqueteriaEdicion.idBranch = this.context.app.defaultSite.identity;
        this.paqueteriaEdicion.idCompany = this.context.app.defaultCompany.identity;
		//
        this.currentParent = parentNode;        
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {        
        this.paqueteriaEdicion = this.paqueteriaService.copy(node.data);
        this.showEditor();        
    }

    onSavingNode() {        
        //Si es un nuevo nodo
        if (this.paqueteriaEdicion.clientID == 0) {
            this.addNode(this.paqueteriaEdicion, this.currentParent);            
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.paqueteriaEdicion);
        }
		this.paqueteriaEdicion = new PaqueteriaInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<PaqueteriaInfo>) {
        this.context.app.showSpinner();
        this.paqueteriaService.saveHierarchy(changes)
            .then(paqueterias => {
                this.paqueterias = paqueterias;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));            
    }

}