﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { PoliticaService} from '../services/politica.service';
import { PoliticaInfo, PoliticaItem} from '../models/politica';

@Component({
    selector: 'mantenimiento-politica-editor',
    templateUrl: './politicas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PoliticasEditor)
        },
        {
            provide: CatalogService, useExisting: PoliticaService
        }
    ]
})
export class PoliticasEditor extends CatalogEditor {
    politica: PoliticaInfo = new PoliticaInfo();  
    tipos: { id: string, nombre: string }[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private politicaService: PoliticaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'keyPolitica': ['', Validators.required],
            'defaultValue': [0],
            'descripcion': ['', Validators.required],
            'tipo': ['', Validators.required]

        });

        this.tipos =
            [
                { "id": "D", "nombre": "Dispositivo" },
                { "id": "A", "nombre": "Aplicación" }
            ];

    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.politicaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.politica = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.politicaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.politica = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.politicaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.politica = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.politicaService.update(this.politica)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.politica.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.politica = info;
            })
            .catch(error => this.handleError(error));        
    }

}