﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { RecursoService} from '../services/recurso.service';
import { RecursoInfo, RecursoItem} from '../models/recurso';

import { EmpleadoItem, EmpleadoSelectorComponent, EmpleadoInfo } from '../../rh';
import { LocalizacionInfo, LocalizacionItem, LocalizacionSelectorComponent } from '../../maintenance';

import { TipoServicioService } from '../services/tipos-servicio.service';
import { TipoServicioItem } from '../models/tipos-servicio';


@Component({
    selector: 'mantenimiento-recurso-editor',
    templateUrl: './recursos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RecursosEditor)
        },
        {
            provide: CatalogService, useExisting: RecursoService
        }
    ]
})
export class RecursosEditor extends CatalogEditor {
    recurso: RecursoInfo = new RecursoInfo();        
    @ViewChild('selectorEmpleado') selectorEmpleado: EmpleadoSelectorComponent;
    @ViewChild('selectorLocalizacion') selectorLocalizacion: LocalizacionSelectorComponent;
    empleado: EmpleadoInfo = new EmpleadoInfo();
    tiposServicio: TipoServicioItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private recursoService: RecursoService, private tipoService: TipoServicioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': [''],
            'empleado': [0, Validators.required],
            'idLocalizacion': [0, Validators.required],
            'idServicioTipo': [0, AztecaValidators.selected],
            'costoHora': [0, Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadTiposServicio();
    }

    loadTiposServicio() {
        this.tipoService.loadList()
            .then(tipos => this.tiposServicio = tipos)
            .catch(error => this.handleError(error));
    }


    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.recursoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.recurso = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.recursoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.recurso = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.recursoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.recurso = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.recursoService.update(this.recurso)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.recurso.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.recurso = info;
            })
            .catch(error => this.handleError(error));        
    }

    onSelectedItemChanged(empleado: EmpleadoInfo) {
        if (empleado) {
                this.empleado = empleado;
            }
            
        }
    
}