﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import { Control, CollectionView } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { ToolbarButton } from 'framework3/controls';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { RefaccionService } from '../services/refaccion.service';
import { RefaccionInfo, RefaccionItem} from '../models/refaccion';
import { RefaccionArticulo } from '../models/refaccion-articulo';
import { TipoServicioService } from '../services/tipos-servicio.service';
import { TipoServicioItem } from '../models/tipos-servicio';

import { ArticuloStockItem, ArticuloStockSelectorComponent } from '../../inventory';
import { MarcaItem, MarcaSelectorComponent } from '../../maintenance';

@Component({
    selector: 'mantenimiento-refacciones-editor',
    templateUrl: './refacciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RefaccionesEditor)
        },
        {
            provide: CatalogService, useExisting: RefaccionService
        }
    ]
})
export class RefaccionesEditor extends CatalogEditor {
    refaccion: RefaccionInfo = new RefaccionInfo();        
    tiposServicio: TipoServicioItem[] = [];

    articulos: RefaccionArticulo[] = [];

    currentArticulo: ArticuloStockItem = new ArticuloStockItem();
    articulosCollection: TrackableCollection = new TrackableCollection();


    @ViewChild('selectorArticulo') selectorArticulo: ArticuloStockSelectorComponent;
    @ViewChild('selectorMarca') selectorMarca: MarcaSelectorComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private refaccionService: RefaccionService, private tipoService: TipoServicioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idServicioTipo': [0, AztecaValidators.selected],
            'articulo': [''],
            'marca':[''],
            'exigirDevolucion': [0],
            'numParte': [''],
            'idRefaccionReparada': [0],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadTiposServicio();
    }

    loadTiposServicio() {
        this.tipoService.loadList()
            .then(tipos => this.tiposServicio = tipos)
            .catch(error => this.handleError(error));
    }


    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.refaccionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.refaccion = info;

                this.articulosCollection.sourceCollection = info.articulos;
                this.articulosCollection.refresh;

                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.refaccionService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.refaccion = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.refaccionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.refaccion = info;  

                this.articulosCollection.sourceCollection = info.articulos;
                this.articulosCollection.refresh;

            })
            .catch(error => this.handleError(error));

        
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.refaccionService.update(this.refaccion)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.refaccion.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.refaccion = info;
            })
            .catch(error => this.handleError(error));        
    }


    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {

            var articuloNew: RefaccionArticulo = new RefaccionArticulo();

            articuloNew.idArticuloStock = this.selectorArticulo.selectedItem.identity;
            articuloNew.idMarca = this.selectorMarca.selectedItem.identity;
            articuloNew.predeterminado = false;

            articuloNew.articulo = this.selectorArticulo.selectedItem.nombre;
            articuloNew.codigo = this.selectorArticulo.selectedItem.code;
            articuloNew.marca = this.selectorMarca.selectedItem.nombre;
            if (this.selectorArticulo.selectedItem.active) {
                articuloNew.estatus = 'Activo';
            }
            else {
                articuloNew.estatus = 'Inactivo';
            }

            var articuloSlice = this.refaccion.articulos.slice();

            articuloSlice.push(articuloNew);

            this.refaccion.articulos = articuloSlice;

            this.articulosCollection.sourceCollection = articuloSlice;
            this.articulosCollection.refresh;


        }
    }

    onRemoveArticulo(item) {
        this.articulosCollection.remove(item);
        this.articulosCollection.refresh();
    }


    onMenuArticulosClicked(menu: Menu, args: any) {
    }

    selectArticulosTab() {
        //refresca los controles de wijmo
        Control.invalidateAll();
    }


    onArticuloDelete(row: RefaccionArticulo) {
        
    }


}