﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { GrupoRefrigeradoresService} from '../services/grupo-refrigeradores.service';
import { GrupoRefrigeradoresInfo, GrupoRefrigeradoresItem } from '../models/grupo-refrigeradores';

import { MarcaItem, MarcaSelectorComponent, MarcaInfo } from '../../maintenance';

@Component({
    selector: 'mantenimiento-refrigeradoresgrupos-editor',
    templateUrl: './refrigeradores-grupos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RefrigeradoresGruposEditor)
        },
        {
            provide: CatalogService, useExisting: GrupoRefrigeradoresService
        }
    ]
})
export class RefrigeradoresGruposEditor extends CatalogEditor {
    grupoRefrigeradores: GrupoRefrigeradoresInfo = new GrupoRefrigeradoresInfo();        
    @ViewChild('selectorMarca') selectorMarca: MarcaSelectorComponent;
    marca: MarcaInfo = new MarcaInfo();
    idServicioTipo: number = 3;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private grupoRefrigeradoresService: GrupoRefrigeradoresService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'marca': [''],
            'idSubMarca': [0, AztecaValidators.selected],
            'modelo': ['', Validators.required],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoRefrigeradoresService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoRefrigeradores = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.grupoRefrigeradoresService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoRefrigeradores = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoRefrigeradoresService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoRefrigeradores = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.grupoRefrigeradoresService.update(this.grupoRefrigeradores)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.grupoRefrigeradores.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.grupoRefrigeradores = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'REFRIGERADORES') {
            this.router.navigate(['maintenance/refrigeradores', this.process.id, this.grupoRefrigeradores.identity, this.grupoRefrigeradores.identity, this.grupoRefrigeradores.idEquipoGrupo]);
        }
        if (item.name == 'DESPIECE') {
            this.router.navigate(['maintenance/despiece', this.process.id, this.grupoRefrigeradores.idEquipoGrupo]);
        }

    }


    onSelectedItemChanged(marca: MarcaInfo) {
        if (marca) {
            this.marca = marca;
        }
    }


}