﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { RefrigeradorService} from '../services/refrigerador.service';
import { RefrigeradorInfo, RefrigeradorItem } from '../models/refrigerador';

import { GrupoRefrigeradoresService } from '../services/grupo-refrigeradores.service';
import { GrupoRefrigeradoresInfo, GrupoRefrigeradoresItem } from '../models/grupo-refrigeradores';

import { LocalizacionInfo, LocalizacionItem, LocalizacionSelectorComponent } from '../../maintenance';

@Component({
    selector: 'mantenimiento-refrigeradores-editor',
    templateUrl: './refrigeradores.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RefrigeradoresEditor)
        },
        {
            provide: CatalogService, useExisting: RefrigeradorService
        }
    ]
})
export class RefrigeradoresEditor extends CatalogEditor {
    refrigerador: RefrigeradorInfo = new RefrigeradorInfo();
    idGrupoRefrigerador: number = 0;
    idEquipoGrupo: number = 0;
    refrigeradorGrupo: GrupoRefrigeradoresInfo = new GrupoRefrigeradoresInfo();

    @ViewChild('selectorLocalizacion') selectorMarca: LocalizacionSelectorComponent;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private refrigeradorService: RefrigeradorService, private grupoRefrigeradorServie: GrupoRefrigeradoresService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'capacidad': ['', Validators.required],
            'logoEtiqueta': ['', Validators.required],
            'puertas': [0, Validators.required],
            'modeloBalastro': ['', Validators.required],
            'fechaCompra': ['', Validators.required],
            'compresorSerie': ['', Validators.required],
            'compresorCapacidad': ['', Validators.required],
            'microSerie': ['', Validators.required],
            'numeroParrillas': [0, Validators.required],
            'numEconomico': ['', Validators.required],
            'idLocalizacion': [0],
            'numSerie': ['', Validators.required]
        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idGrupoRefrigerador = params['idGrupoRefrigeradores']
            this.idEquipoGrupo = params['idEquipoGrupo']
        });
    }


    loadRefrigeradorGrupo() {
        this.grupoRefrigeradorServie.getDetailByID(this.idGrupoRefrigerador)
            .then(info => {
                this.refrigeradorGrupo = info;
            })
            .catch(error => this.handleError(error));
    }


	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadRefrigeradorGrupo();
    }

    onCreatingItem() {
        this.refrigerador.equipo.idEquipoGrupo = this.idEquipoGrupo;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.refrigeradorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.refrigerador = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.refrigeradorService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.refrigerador = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.refrigeradorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.refrigerador = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.refrigeradorService.update(this.refrigerador)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.refrigerador.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.refrigerador = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'DOCUMENTOS') {
            this.router.navigate(['maintenance/documentos', this.refrigerador.idEquipo, this.process.id]);
        }
        if (item.name == 'FOTOGRAFIAS') {
            this.router.navigate(['maintenance/fotografias', this.refrigerador.idEquipo, this.process.id]);
        }
    }

}