﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { MenuItem } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { TipoServicioService} from '../services/tipos-servicio.service';
import { TipoServicioInfo, TipoServicioItem} from '../models/tipos-servicio';

@Component({
    selector: 'mantenimiento-tipo-servicio-editor',
    templateUrl: './servicios-tipos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ServiciosTiposEditor)
        },
        {
            provide: CatalogService, useExisting: TipoServicioService
        }
    ]
})
export class ServiciosTiposEditor extends CatalogEditor {
    tipoServicio: TipoServicioInfo = new TipoServicioInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private tipoServicioService: TipoServicioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoServicioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoServicio = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.tipoServicioService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoServicio = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoServicioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoServicio = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.tipoServicioService.update(this.tipoServicio)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.tipoServicio.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.tipoServicio = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'USERS') {
            this.router.navigate(['maintenance/servicio-tipo-usuarios/' + this.process.id + '/' + this.tipoServicio.identity.toString()]);
        }
    }


}