﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { MenuItem } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Features
import { TipoServicioService} from '../services/tipos-servicio.service';
import {  TipoServicioInfo, TipoServicioItem} from '../models/tipos-servicio';

@Component({    
    selector: 'mantenimiento-servicios-tipos-manager',
    templateUrl: './servicios-tipos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ServiciosTiposManager)
        },
        {
            provide: CatalogService, useExisting: TipoServicioService
        }
	]
})
export class ServiciosTiposManager extends CatalogManager {
    serviciosTipos: TipoServicioItem[]=[];    
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private tipoServicioService: TipoServicioService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.tipoServicioService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.serviciosTipos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.tipoServicioService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.tipoServicioService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'USERS') {
            this.router.navigate(['maintenance/servicio-tipo-usuarios/' + this.process.id + '/' + this.manager.currentItem.id.toString()]);
        }
    }


}
