﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ServicioService} from '../services/servicio.service';
import { ServicioInfo, ServicioItem } from '../models/servicio';
import { TipoOrdenService } from '../services/tipo-orden.service';
import { TipoOrdenItem } from '../models/tipo-orden';


@Component({
    selector: 'mantenimiento-servicio-editor',
    templateUrl: './servicios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ServiciosEditor)
        },
        {
            provide: CatalogService, useExisting: ServicioService
        }
    ]
})
export class ServiciosEditor extends CatalogEditor {
    servicio: ServicioInfo = new ServicioInfo();
    tiposOrden: TipoOrdenItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private servicioService: ServicioService, private tipoOrdenSvc: TipoOrdenService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'descripcion': ['', Validators.required],
            'tiempoEstimado': [0, Validators.required],
            'idTipoOrden': [0, AztecaValidators.selected],
            'recursoNecesario': [0],
            'frecuencia': [0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadTiposOrden();
		this.applyPolicies();
    }

    loadTiposOrden() {
        this.tipoOrdenSvc.loadList()
            .then(tipos => this.tiposOrden = tipos)
            .catch(error => this.handleError(error));
    }


    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.servicioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.servicioService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.servicioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.servicioService.update(this.servicio)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.servicio.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.servicio = info;
            })
            .catch(error => this.handleError(error));        
    }

}