﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { SoftwareContactoService} from '../services/software-contacto.service';
import { SoftwareContactoInfo, SoftwareContactoItem } from '../models/software-contacto';
import { SoftwareItem } from '../models/software'
import { SoftwareService } from '../services/software.service'

@Component({
    selector: 'mantenimiento-software-contacto-editor',
    templateUrl: './softwares-contactos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SoftwaresContactosEditor)
        },
        {
            provide: CatalogService, useExisting: SoftwareContactoService
        }
    ]
})
export class SoftwaresContactosEditor extends CatalogEditor {
    softwareContacto: SoftwareContactoInfo = new SoftwareContactoInfo();
    softwares: SoftwareItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private softwareContactoService: SoftwareContactoService, private softwareService: SoftwareService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idSoftware': [0, Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    loadSoftwares() {
        this.softwareService.loadList(this.context.app.defaultSite.identity)
            .then(sw => this.softwares = sw)
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadSoftwares();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.softwareContactoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.softwareContacto = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.softwareContactoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.softwareContacto = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.softwareContactoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.softwareContacto = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.softwareContactoService.update(this.softwareContacto)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.softwareContacto.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.softwareContacto = info;
            })
            .catch(error => this.handleError(error));        
    }

}