﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition, ConditionValueType } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Features
import { SoftwareService} from '../services/software.service';
import { SoftwareInfo, SoftwareItem} from '../models/software';

@Component({    
    selector: 'mantenimiento-softwares-manager',
    templateUrl: './softwares.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => SoftwaresManager)
        },
        {
            provide: CatalogService, useExisting: SoftwareService
        }
	]
})
export class SoftwaresManager extends CatalogManager {
    softwares: SoftwareItem[] = [];
    idSistema: number = 0;
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private softwareService: SoftwareService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////
    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idSistema = +params['idSistema'];
        });
    }

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let grupoCond: SearchCondition = { field: 'idSistema', operator: '=', value: this.idSistema.toString(), valueType: ConditionValueType.INT };
        request.conditions.push(grupoCond);


        this.softwareService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.softwares = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0, { idSistema: this.idSistema }], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id, { idSistema: this.idSistema }], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.softwareService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.softwareService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }


    onMenuItemClick(item: MenuItem) {

        if (item.name == 'COMPLEMENTOS') {

            this.router.navigate(['maintenance/complementos', this.process.id, this.idSistema ,this.manager.currentItem.id]);

        }

    }


}
