﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { TallerService} from '../services/taller.service';
import { TallerInfo, TallerItem } from '../models/taller';
import { AreaService } from '../../inventory/services/area.service';
import { AreaItem } from '../../inventory/models/area';


import { TipoServicioService } from '../services/tipos-servicio.service';
import { TipoServicioItem } from '../models/tipos-servicio';


import { AlmacenesSelectorComponent, AlmacenItem, AlmacenService } from '../../inventory';

@Component({
    selector: 'mantenimiento-taller-editor',
    templateUrl: './talleres.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TalleresEditor)
        },
        {
            provide: CatalogService, useExisting: TallerService
        }
    ]
})
export class TalleresEditor extends CatalogEditor {
    taller: TallerInfo = new TallerInfo(); 
    tiposServicios: TipoServicioItem[] = [];
    idServicioTipo: number = 0;
    areas: AreaItem[] = [];
    idArea: number = 0;
    idAreaRefacciones: number = 0;
    idAreaRefaccionesUsadas: number = 0;


    @ViewChild('selectorAlmacen') selectorAlmacen: AlmacenesSelectorComponent;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private tallerService: TallerService, private tipoService: TipoServicioService, private areaService: AreaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idArea': [-1],
            'idAreaRefacciones': [-1],
            'idAreaRefaccionesUsadas': [-1],
            'idAlmacen': [0, Validators.required],
            'idAlmacenRefaccion': [0, Validators.required],
            'idAlmacenRefaccionUsada': [0, Validators.required],
            'idLocalizacion': [0],
            'idTipoServicio': [0, Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

    private loadAreas() {
        this.context.app.showSpinner();
        this.areaService.loadListByUser(this.context.app.defaultSite.identity, this.context.session.user.identity)
            .then(areas => {
                this.areas = areas;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));

    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadTiposServicio();
        this.loadAreas();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.tallerService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.idArea = info.almacen.idArea;
                this.idAreaRefacciones = info.almacenRefaccion.idArea;
                this.idAreaRefaccionesUsadas = info.almacenRefaccionUsada.idArea;
                this.taller = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.tallerService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.taller = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.tallerService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.idArea = info.almacen.idArea;
                this.idAreaRefacciones = info.almacenRefaccion.idArea;
                this.idAreaRefaccionesUsadas = info.almacenRefaccionUsada.idArea;
                this.taller = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.tallerService.update(this.taller)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.taller.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.taller = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadTiposServicio() {
        this.tipoService.loadList()
            .then(tipos => this.tiposServicios = tipos)
            .catch(error => this.handleError(error));
    }

}