﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { DocumentoTipoService} from '../services/documento-tipo.service';
import { DocumentoTipoInfo, DocumentoTipoItem } from '../models/documento-tipo';
import { TipoServicioService } from '../services/tipos-servicio.service';
import { TipoServicioItem } from '../models/tipos-servicio';


@Component({
    selector: 'mantenimiento-tipos-documento-editor',
    templateUrl: './tipos-documento.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TiposDocumentoEditor)
        },
        {
            provide: CatalogService, useExisting: DocumentoTipoService
        }
    ]
})
export class TiposDocumentoEditor extends CatalogEditor {
    documentoTipo: DocumentoTipoInfo = new DocumentoTipoInfo();        
    tiposServicio: TipoServicioItem[] = [];

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private documentoTipoService: DocumentoTipoService, private tipoService: TipoServicioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'mime':[''],
            'sample_field': [''],
            'idServicioTipo': [0, AztecaValidators.selected]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadTiposServicio();
    }


    loadTiposServicio() {
        this.tipoService.loadList()
            .then(tipos => this.tiposServicio = tipos)
            .catch(error => this.handleError(error));
    }


    onCreatingItem() {
        this.loadTiposServicio();
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.documentoTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.documentoTipo = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.documentoTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.documentoTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.documentoTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.documentoTipo = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.documentoTipoService.update(this.documentoTipo)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.documentoTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.documentoTipo = info;
            })
            .catch(error => this.handleError(error));        
    }

}