﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { TipoLicenciaService} from '../services/tipo-licencia.service';
import { TipoLicenciaInfo, TipoLicenciaItem} from '../models/tipo-licencia';

@Component({
    selector: 'mantenimiento-tipo-licencia-editor',
    templateUrl: './tipos-licencias.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TiposLicenciasEditor)
        },
        {
            provide: CatalogService, useExisting: TipoLicenciaService
        }
    ]
})
export class TiposLicenciasEditor extends CatalogEditor {
    tipoLicencia: TipoLicenciaInfo = new TipoLicenciaInfo();        
    tiposDestinos: { id: string, nombre: string, descripcion: string }[] = [];

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private tipoLicenciaService: TipoLicenciaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'descripcion': ['', Validators.required],
            'tipoDestino': ['', Validators.required],
            'temporal': [0],
            'sample_field': ['']
        });

        this.tiposDestinos =
            [
            { "id": "U", "nombre": "Usuario: Se destina a un usuario especifico", "descripcion": "Se destina a un usuario especifico" },
            { "id": "E", "nombre": "Equipo: Se destina a un equipo", "descripcion": "Se destina a un equipo" },
            { "id": "S", "nombre": "Software: Se desinta a un software como para la activación de un complemento dentro del mismo", "descripcion": "Se desinta a un software como para la activación de un complemento dentro del mismo" },
            { "id": "C", "nombre": "Complemento: Se destina a un complemento del Software", "descripcion": "Se destina a un complemento del Software" }
            ];


    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoLicenciaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoLicencia = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.tipoLicenciaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoLicencia = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoLicenciaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoLicencia = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.tipoLicenciaService.update(this.tipoLicencia)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.tipoLicencia.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.tipoLicencia = info;
            })
            .catch(error => this.handleError(error));        
    }

}