﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { TipoMotorService} from '../services/tipo-motor.service';
import { TipoMotorInfo, TipoMotorItem } from '../models/tipo-motor';
import { TipoCombustibleItem } from '../models/tipo-combustible';
import { TipoCombustibleService } from '../services/tipo-combustible.service';
@Component({
    selector: 'mantenimiento-tiposmotor-editor',
    templateUrl: './tipos-motor.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TiposMotorEditor)
        },
        {
            provide: CatalogService, useExisting: TipoMotorService
        }
    ]
})
export class TiposMotorEditor extends CatalogEditor {
    tipomotor: TipoMotorInfo = new TipoMotorInfo();
    combustibles: TipoCombustibleItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private tipoMotorService: TipoMotorService, private tipoCombustibleService: TipoCombustibleService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'cilindraje': [0, Validators.required],
            'centimetroscubicos': [0, Validators.required],
            'potencia': [0, Validators.required],
            'idTipoCombustible': [0, Validators.required]
        });
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadCombustibles();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoMotorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipomotor = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.tipoMotorService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipomotor = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoMotorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipomotor = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.tipoMotorService.update(this.tipomotor)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.tipomotor.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.tipomotor = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadCombustibles() {
        this.tipoCombustibleService.loadList()
            .then(tipos => this.combustibles = tipos)
            .catch(error => this.handleError(error));
    }


}