﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { GrupoVehiculosService} from '../services/grupo-vehiculos.service';
import { GrupoVehiculosInfo, GrupoVehiculosItem} from '../models/grupo-vehiculos';

import { MarcaItem, MarcaSelectorComponent, MarcaInfo, TipoMotorItem, TipoTransmisionItem } from '../../maintenance';
import { TipoMotorService } from '../services/tipo-motor.service';
import { TipoTransmisionService } from '../services/tipo-transmision.service';



@Component({
    selector: 'mantenimiento-vehiculosgrupos-editor',
    templateUrl: './vehiculos-grupos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => VehiculosGruposEditor)
        },
        {
            provide: CatalogService, useExisting: GrupoVehiculosService
        }
    ]
})
export class VehiculosGruposEditor extends CatalogEditor {
    grupoVehiculos: GrupoVehiculosInfo = new GrupoVehiculosInfo();       
    @ViewChild('selectorMarca') selectorMarca: MarcaSelectorComponent;
    marca: MarcaInfo = new MarcaInfo();
    tiposMotor: TipoMotorItem[] = [];
    tiposTransmision: TipoTransmisionItem[] = [];
    idServicioTipo: number = 1;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private grupoVehiculosService: GrupoVehiculosService,
        private tipoMotorServ: TipoMotorService, private tipoTransmisionServ: TipoTransmisionService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': [''],
            'marca': [''],
            'idSubMarca': [0, AztecaValidators.selected],
            'anio': [0, Validators.required],
            'version': ['', Validators.required],
            'idTipoMotor': [0, Validators.required],
            'idTipoTransmision': [0, Validators.required],
        });
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadTiposMotor();
        this.loadTiposTransmision();
    }

    loadTiposMotor() {
        this.tipoMotorServ.loadList()
            .then(tipos => this.tiposMotor = tipos)
            .catch(error => this.handleError(error));
    }

    loadTiposTransmision() {
        this.tipoTransmisionServ.loadList(this.context.session.user.idBranch)
            .then(tipos => this.tiposTransmision = tipos)
            .catch(error => this.handleError(error));
    }


    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoVehiculosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoVehiculos = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.grupoVehiculosService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoVehiculos = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoVehiculosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupoVehiculos = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.grupoVehiculosService.update(this.grupoVehiculos)
            .then(info => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.grupoVehiculos.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.grupoVehiculos = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'VEHICULOS') {
            this.router.navigate(['maintenance/vehiculos', this.process.id, this.grupoVehiculos.identity, this.grupoVehiculos.identity, this.grupoVehiculos.idEquipoGrupo]);
        }
        if (item.name == 'DESPIECE') {
            this.router.navigate(['maintenance/despiece', this.process.id, this.grupoVehiculos.idEquipoGrupo]);
        }


    }

    onSelectedItemChanged(marca: MarcaInfo) {
        if (marca) {
            this.marca = marca;
        }
    }


}