﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition, ConditionValueType } from 'framework3/search';
//Features
import { VehiculoService} from '../services/vehiculo.service';
import { VehiculoInfo, VehiculoItem} from '../models/vehiculo';

@Component({    
    selector: 'mantenimiento-vehiculos-manager',
    templateUrl: './vehiculos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => VehiculosManager)
        },
        {
            provide: CatalogService, useExisting: VehiculoService
        }
	]
})
export class VehiculosManager extends CatalogManager {
    vehiculos: VehiculoItem[] = [];
    idEquipoGrupo: number = 0;
    idVehiculoGrupo: number = 0;
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private vehiculoService: VehiculoService) {
        super(context, router, route);
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipoGrupo = +params['idEquipoGrupo'];
            this.idVehiculoGrupo = +params['idVehiculoGrupo'];

        });
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let grupoCond: SearchCondition = { field: 'IdEquipoGrupo', operator: '=', value: this.idEquipoGrupo.toString(), valueType: ConditionValueType.INT };
        request.conditions.push(grupoCond);

        this.vehiculoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.vehiculos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.vehiculoService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.vehiculoService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}
