﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import { ProcessComponent, Context } from 'framework3/core';
import { TrackableCollection } from 'framework3/editors';
import { PolicyInfo } from 'framework3/core';
import { Menu } from 'wijmo/wijmo.input';
import { FileInfo } from 'framework3/general';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Features
import { Documento } from '../models/documento';
import { LicenciaService } from '../services/licencia.service';

@Component({
    selector: 'mantenimiento-documentos-licencias',
    templateUrl: './documentos-licencias.component.html'
})


export class DocumentosLicencias extends ProcessComponent {

    @ViewChild('dialog') dialog: ModalDirective;

    documentoItem: Documento = new Documento;
    documentos: Documento[] = [];
    documentosCollection: TrackableCollection = new TrackableCollection();
    fileSelected: string = '';
    idLicencia: number = 0;
    selectedFile: FileInfo;

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private licenciaService: LicenciaService) {
        super(context, router, route);
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idLicencia = params['idLicencia'];
        });
    }

    applyPolicy(policy: PolicyInfo) {

    }

    ngAfterViewInit() {

        this.llenandoGrid(this.idLicencia);
    }

    llenandoGrid(idLicencia: number) {
        this.licenciaService.LoadDocumentos(idLicencia)
            .then(docs => {
                this.documentos = docs
                this.documentosCollection.sourceCollection = this.documentos;
            })
            .catch(error => this.handleError(error));
    }

    agregarDocumento(idLicencia: number, idTipoDocumento: number, fileInfo: FileInfo) {
        this.licenciaService.AgregarDocumento(idLicencia, idTipoDocumento, fileInfo)
            .then(r => {
                this.llenandoGrid(this.idLicencia);
            })
            .catch(error => this.handleError(error));
    }

    eliminarDocumento(idLicencia: number, idTipoDocumento: number, idArchivo: string) {
        this.licenciaService.EliminarDocumento(idLicencia, idTipoDocumento, idArchivo)
            .then(r => {
                this.llenandoGrid(this.idLicencia);
            })
            .catch(error => this.handleError(error));
    }


    show() {
        this.dialog.config.backdrop = false;
        this.dialog.show();
    }

    onMenuDocumentosClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Agregar
            this.show();

        }
        if (menu.selectedIndex == 2) {
            //Eliminar
            this.documentoItem = this.documentosCollection.currentItem;
            this.eliminarDocumento(this.idLicencia, this.documentoItem.idTipoDocumento, this.documentoItem.gDocumento);
        }
    }

    onUpdate() {
        this.documentoItem = this.documentosCollection.currentItem;
        this.agregarDocumento(this.idLicencia, this.documentoItem.idTipoDocumento, this.selectedFile);
        this.dialog.hide();
    }


    handleError(error: any) {
        console.log = error;
    }

    onFileChanged(file: FileInfo) {
        this.selectedFile = file;
        this.fileSelected = file.filePath;
    }

}

