﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import { ProcessComponent, Context } from 'framework3/core';
import { TrackableCollection } from 'framework3/editors';
import { PolicyInfo } from 'framework3/core';
import { Menu } from 'wijmo/wijmo.input';
import { FileInfo } from 'framework3/general';


//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Features
import { Documento } from '../models/documento';
import { EquipoService } from '../services/equipo.service';

@Component({
    selector: 'mantenimiento-documentos',
    templateUrl: './documentos.component.html'
})


export class Documentos extends ProcessComponent  {

    @ViewChild('dialog') dialog: ModalDirective;

    documentoItem: Documento = new Documento;
    documentos: Documento[] = [];
    documentosCollection: TrackableCollection = new TrackableCollection();
    fileSelected: string = '';
    idEquipo: number = 0;
    selectedFile: FileInfo;
    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private equipoService: EquipoService) {
        super(context, router, route);
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipo = params['idEquipo'];
        });
    }

    applyPolicy(policy: PolicyInfo) {

    }

    ngAfterViewInit() {

        this.llenandoGrid(this.idEquipo);
    }

    llenandoGrid(idEquipo: number) {
        this.equipoService.LoadDocumentos(idEquipo)
            .then(docs => {
                this.documentos = docs
                this.documentosCollection.sourceCollection = this.documentos;
            })
            .catch(error => this.handleError(error));
    }

    agregarDocumento(idEquipo: number, idTipoDocumento: number, fileInfo: FileInfo) {
        this.equipoService.AgregarDocumento(idEquipo, idTipoDocumento, fileInfo)
            .then(r=>{
                this.llenandoGrid(this.idEquipo);
            })
            .catch(error => this.handleError(error));
    }

    eliminarDocumento(idEquipo: number, idTipoDocumento: number, idArchivo: string) {
        this.equipoService.EliminarDocumento(idEquipo, idTipoDocumento, idArchivo)
            .then(r => {
                this.llenandoGrid(this.idEquipo);
            })
            .catch(error => this.handleError(error));
    }


    show() {
        this.dialog.config.backdrop = false;
        this.dialog.show();
    }

    onMenuDocumentosClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Agregar
            this.show();
            
        }
        if (menu.selectedIndex == 2) {
            //Eliminar
            this.documentoItem = this.documentosCollection.currentItem;
            this.eliminarDocumento(this.idEquipo, this.documentoItem.idTipoDocumento, this.documentoItem.gDocumento);
        }
    }

    onUpdate() {
        this.documentoItem = this.documentosCollection.currentItem;
        this.agregarDocumento(this.idEquipo, this.documentoItem.idTipoDocumento, this.selectedFile);
        this.dialog.hide();
    }


    handleError(error: any) {
        console.log = error;
    }

    onFileChanged(file: FileInfo) {
        this.selectedFile = file;
        this.fileSelected = file.filePath;
    }

}

