﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
//
import { EquipoInformaticaInfo, EquipoInformaticaItem } from '../models/equipo-informatica';
import { EquipoInformaticaService } from '../services/equipo-informatica.service';


@Component({
    selector: 'maintenance-informatica-computo-selector',
    templateUrl: './equipo-informatica-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => EquipoInformaticaSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => EquipoInformaticaSelectorComponent),
        multi: true
    }]

})
export class EquipoInformaticaSelectorComponent extends CatalogSelector {
    equipo: any[];
    selectedItem: EquipoInformaticaInfo = new EquipoInformaticaInfo;
    //
    @Output() selectedItemChanged: EventEmitter<EquipoInformaticaInfo> = new EventEmitter<EquipoInformaticaInfo>();

    constructor(protected context: Context, private equipoInformaticaService: EquipoInformaticaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findEquipos(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.equipoInformaticaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.equipo.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.equipoInformaticaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.equipo = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findEquipos(keyword: string): Observable<CatalogItem[]> {
        return this.equipoInformaticaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

