﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
//
import { EquipoInfo, EquipoItem } from '../models/equipo';
import { EquipoService } from '../services/equipo.service';


@Component({
    selector: 'maintenance-equipo-selector',
    templateUrl: './equipo-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => EquipoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => EquipoSelectorComponent),
        multi: true
    }]

})
export class EquipoSelectorComponent extends CatalogSelector {
    equipo: any[];
    selectedItem: EquipoInfo = new EquipoInfo;
    @Input() idTipoServicio: number = 0;
    //
    @Output() selectedItemChanged: EventEmitter<EquipoInfo> = new EventEmitter<EquipoInfo>();

    constructor(protected context: Context, private equipoService: EquipoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findEquipos(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.equipoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let condiciones: SearchCondition = { field: "idServicioTipo", operator: "=", valueType: "int", value: this.idTipoServicio.toString() };
        request.conditions.push(condiciones);


        this.equipoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.equipo = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findEquipos(keyword: string): Observable<CatalogItem[]> {
        return this.equipoService.searchKeyword2(this.context.app.defaultSite.identity, keyword, this.idTipoServicio);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

