﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import { ProcessComponent, Context } from 'framework3/core';
import { TrackableCollection } from 'framework3/editors';
import { PolicyInfo } from 'framework3/core';
import { Menu } from 'wijmo/wijmo.input';

import { AztecaValidators, ToolbarButton } from 'framework3/controls';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';


//Features
import { FotografiaEquipo  } from '../models/fotografia-equipo';
import { EquipoService } from '../services/equipo.service';

@Component({
    selector: 'mantenimiento-fotografias',
    templateUrl:'./fotografias.component.html'
})


export class Fotografias extends ProcessComponent {

    @ViewChild('dialog') dialog: ModalDirective;

    fotografiaItem: FotografiaEquipo = new FotografiaEquipo;
    fotografias: FotografiaEquipo[] = [];
    fotografiasCollection: TrackableCollection = new TrackableCollection();
    fileSelected: string = '';
    idEquipo: number = 0;

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private equipoService: EquipoService) {
        super(context, router, route);
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idEquipo = params['idEquipo'];
        });

    }

    applyPolicy(policy: PolicyInfo) {

    }

    ngAfterViewInit() {
        this.llenandoGrid(this.idEquipo);

    }

    llenandoGrid(idEquipo: number) {
        this.equipoService.LoadFotografias(idEquipo)
            .then(docs => {
                this.fotografias = docs
                this.fotografiasCollection.sourceCollection = this.fotografias;
            })
            .catch(error => this.handleError(error));
    }


    agregarFotografia(idEquipo: number, fotografiaEquipo: FotografiaEquipo) {
        this.equipoService.AgregarFotografia(idEquipo, fotografiaEquipo)
            .then(r => {
                this.llenandoGrid(this.idEquipo);
            })
            .catch(error => this.handleError(error));
    }


    show() {
        this.dialog.config.backdrop = false;
        this.dialog.show();
    }

    handleError(error: any) {
        console.log = error;
    }

    onUpdate() {
        this.fotografiaItem.archivo = this.fileSelected;

        this.agregarFotografia(this.idEquipo, this.fotografiaItem);
        this.dialog.hide();
    }


    onToolbarButtonClick(button: ToolbarButton) {

        if (button.name == 'ADD') {
            this.fileSelected = '';
            this.fotografiaItem.descripcion = '';
            this.show();
        }

        if (button.name == 'DEL') {
            this.equipoService.EliminarFotografia(this.idEquipo, this.fotografiasCollection.currentItem.idArchivo);
        }


        
    }
}
