﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//

import { LlantaInfo, LlantaItem } from '../models/llanta';
import { LlantaService } from '../services/llanta.service';

@Component({
    selector: 'maintenance-llanta-selector',
    templateUrl: './llanta-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => LlantasSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => LlantasSelectorComponent),
        multi: true
    }]

})
export class LlantasSelectorComponent extends CatalogSelector {
    llanta: any[];
    selectedItem: LlantaInfo = new LlantaInfo;
    //
    @Output() selectedItemChanged: EventEmitter<LlantaInfo> = new EventEmitter<LlantaInfo>();
    @Input() sinVehiculo: boolean = false;

    constructor(protected context: Context, private llantaService: LlantaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findLlantas(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.llantaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.pieza.equipo.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.sinVehiculo) {
            let condicion: SearchCondition = { field: "idVehiculo", operator: "=", valueType: "integer", value: 0 }
            request.conditions.push(condicion);
        }

        this.llantaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.llanta = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findLlantas(keyword: string): Observable<CatalogItem[]> {
        return this.llantaService.searchKeywordLlantas(this.context.app.defaultSite.identity, keyword, this.sinVehiculo);
       
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

