﻿
import { Component, Input, Output, ViewChild, ContentChildren, OnInit, EventEmitter, QueryList, forwardRef, Optional, SkipSelf } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, HierarchySelector, ITreeHost } from 'framework3/search/index';
//
import { LocalizacionInfo, LocalizacionItem } from '../models/localizacion';
import { LocalizacionService } from '../services/localizacion.service';

@Component({
    selector: 'mantenimiento-localizacion-selector',
    templateUrl:'./localizacion-selector.component.html',
    providers: [
        {
        provide: BaseComponent,
        useExisting: forwardRef(() => LocalizacionSelectorComponent)
        },
        {
            provide: ITreeHost,
            useExisting: forwardRef(() => LocalizacionSelectorComponent)
        },
        {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => LocalizacionSelectorComponent),
        multi: true
        }
    ]
})
export class LocalizacionSelectorComponent extends HierarchySelector implements ITreeHost {
    modulos: LocalizacionItem[];
    nodes: LocalizacionInfo[];
    selectedItem: LocalizacionInfo = null;

    constructor(protected context: Context, private moduloService: LocalizacionService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findKeyword(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.moduloService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.namedPath;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.moduloService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.modulos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findKeyword(keyword: string): Observable<CatalogItem[]> {
        return this.moduloService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

    onLoadingHierarchy() {
        //Cargar nodos raíz al abrir diálogo de jerarquía
        this.moduloService.loadChildren(this.context.app.defaultCompany.identity, 0)
            .then(rootNodes => this.nodes = rootNodes)
            .catch(error => this.handleError(error));
    }


    loadChildren(parent: LocalizacionInfo): Promise<LocalizacionInfo[]> {
        return this.moduloService.loadChildren(this.context.app.defaultCompany.identity, parent.identity);
    }


}

