﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
//
import { MarcaInfo, MarcaItem } from '../models/marcas';
import { MarcaService } from '../services/marcas.service';

@Component({
    selector: 'maintenance-marca-selector',
    templateUrl: './marca-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(()=>MarcaSelectorComponent)
    },
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => MarcaSelectorComponent),
            multi: true
    }]

})
export class MarcaSelectorComponent extends CatalogSelector {
    marca: any[];
    selectedItem: MarcaInfo = new MarcaInfo;
    //
    @Output() selectedItemChanged: EventEmitter<MarcaInfo> = new EventEmitter<MarcaInfo>();
    @Input() idServicioTipo: number = 0;

    constructor(protected context: Context, private marcaService: MarcaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findMarcas(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.marcaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.idServicioTipo != 0) {
            let condition: SearchCondition = { field: 'IdServicioTipo', operator: '=', valueType: 'number', value: this.idServicioTipo };
            request.conditions.push(condition);
        }

        this.marcaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.marca = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findMarcas(keyword: string): Observable<CatalogItem[]> {
        return this.marcaService.searchKeyword2(this.context.app.defaultSite.identity, keyword, this.idServicioTipo);

    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}