﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
//
import { RecursoInfo, RecursoItem } from '../models/recurso';
import { RecursoService } from '../services/recurso.service';

@Component({
    selector: 'mantenimiento-recurso-selector',
    templateUrl: './recurso-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => RecursoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => RecursoSelectorComponent),
        multi: true
    }]

})
export class RecursoSelectorComponent extends CatalogSelector {
    recurso: any[];
    selectedItem: RecursoInfo = new RecursoInfo;
    @Input() idTipoServicio: number = 0;
    //
    @Output() selectedItemChanged: EventEmitter<RecursoInfo> = new EventEmitter<RecursoInfo>();

    constructor(protected context: Context, private recursoService: RecursoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findRecursos(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.recursoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let condiciones: SearchCondition = { field: "idServicioTipo", operator: "=", valueType: "int", value: this.idTipoServicio.toString() };
        request.conditions.push(condiciones);


        this.recursoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.recurso = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findRecursos(keyword: string): Observable<RecursoItem[]> {
        return this.recursoService.searchKeyword2(this.context.app.defaultSite.identity, keyword, this.idTipoServicio);
    }


    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}
