﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition, ConditionValueType } from 'framework3/search/index'; //'../../search/index';
//
import { RefaccionInfo, RefaccionItem } from '../models/refaccion';
import { RefaccionService } from '../services/refaccion.service';

@Component({
    selector: 'mantenimiento-refaccion-selector',
    templateUrl: './refaccion-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => RefaccionSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => RefaccionSelectorComponent),
        multi: true
    }]

})
export class RefaccionSelectorComponent extends CatalogSelector {
    refaccion: any[];
    selectedItem: RefaccionInfo = new RefaccionInfo;
    //
    @Output() selectedItemChanged: EventEmitter<RefaccionInfo> = new EventEmitter<RefaccionInfo>();
    @Input() idServicioTipo: number = 0;

    constructor(protected context: Context, private refaccionService: RefaccionService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findRefacciones(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.refaccionService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.idServicioTipo != 0) {
            let condition: SearchCondition = { field: 'IdServicioTipo', operator: '=', valueType: 'number', value: this.idServicioTipo };
            request.conditions.push(condition);
        }

        this.refaccionService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.refaccion = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findRefacciones(keyword: string): Observable<CatalogItem[]> {
        return this.refaccionService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}