﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { VehiculoInfo, VehiculoItem } from '../models/vehiculo';
import { VehiculoService } from '../services/vehiculo.service';


@Component({
    selector: 'maintenance-vehiculo-selector',
    templateUrl: './vehiculo-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => VehiculosSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => VehiculosSelectorComponent),
        multi: true
    }]

})
export class VehiculosSelectorComponent extends CatalogSelector {
    vehiculo: any[];
    selectedItem: VehiculoInfo = new VehiculoInfo;
    //
    @Output() selectedItemChanged: EventEmitter<VehiculoInfo> = new EventEmitter<VehiculoInfo>();

    constructor(protected context: Context, private vehiculoService: VehiculoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findVehiculos(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.vehiculoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.equipo.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.vehiculoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.vehiculo = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findVehiculos(keyword: string): Observable<CatalogItem[]> {
        return this.vehiculoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

