﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { AplicacionInfo, AplicacionItem } from '../models/aplicacion';
import { AplicacionService } from '../services/aplicacion.service';


@Component({
    selector: 'maintenance-aplicacion-selector',
    templateUrl: './aplicacion-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => AplicacionSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => AplicacionSelectorComponent),
        multi: true
    }]

})
export class AplicacionSelectorComponent extends CatalogSelector {
    aplicaciones: any[];
    selectedItem: AplicacionInfo = new AplicacionInfo;
    //
    @Output() selectedItemChanged: EventEmitter<AplicacionInfo> = new EventEmitter<AplicacionInfo>();

    constructor(protected context: Context, private aplicacionService: AplicacionService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findAplicaciones(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.aplicacionService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.aplicacionService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.aplicaciones = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findAplicaciones(keyword: string): Observable<CatalogItem[]> {
        return this.aplicacionService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

