﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { ConfiguracionInfo, ConfiguracionItem } from '../models/configuracion';
import { ConfiguracionService } from '../services/configuracion.service';


@Component({
    selector: 'maintenance-configuracion-selector',
    templateUrl: './configuracion-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ConfiguracionSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ConfiguracionSelectorComponent),
        multi: true
    }]

})
export class ConfiguracionSelectorComponent extends CatalogSelector {
    configuraciones: any[];
    selectedItem: ConfiguracionInfo = new ConfiguracionInfo;
    //
    @Output() selectedItemChanged: EventEmitter<ConfiguracionInfo> = new EventEmitter<ConfiguracionInfo>();
    @Input() tipo: string = '';

    constructor(protected context: Context, private configuracionService: ConfiguracionService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findConfiguraciones(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.configuracionService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.tipo != '') {
            let condition: SearchCondition = { field: 'Tipo', operator: '=', valueType: 'string', value: this.tipo };
            request.conditions.push(condition);
        }


        this.configuracionService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.configuraciones = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findConfiguraciones(keyword: string): Observable<CatalogItem[]> {
        return this.configuracionService.searchKeyword2(this.context.app.defaultSite.identity, keyword, this.tipo);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

