﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { DispositivoInfo, DispositivoItem } from '../models/dispositivo';
import { DispositivoService } from '../services/dispositivo.service';


@Component({
    selector: 'maintenance-dispositivo-selector',
    templateUrl: './dispositivo-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => DispositivoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => DispositivoSelectorComponent),
        multi: true
    }]

})
export class DispositivoSelectorComponent extends CatalogSelector {
    dispositivos: any[];
    selectedItem: DispositivoInfo = new DispositivoInfo;
    //
    @Output() selectedItemChanged: EventEmitter<DispositivoInfo> = new EventEmitter<DispositivoInfo>();

    constructor(protected context: Context, private dispositivoService: DispositivoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findDispositivos(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.dispositivoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.equipo.grupo.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.dispositivoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.dispositivos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findDispositivos(keyword: string): Observable<CatalogItem[]> {
        return this.dispositivoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

