﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { OrdenServicioInfo, OrdenServicioItem } from '../models/orden-servicio';
import { OrdenServicioDetail } from '../models/orden-servicio-detail';
import { OrdenServicioService } from '../services/orden-servicio.service';
import { OrdenServicioDetailEditor } from './parts/ordenes-servicios-detail.editor';
import { ServicioService } from '../services/servicio.service';
import { ServicioItem } from '../models/servicio';

import { OrdenInfo } from '../models/orden';
import { OrdenService } from '../services/orden.service';

@Component({
    selector: 'mantenimiento-ordenes-servicios-editor',
    templateUrl: './ordenes-servicios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => OrdenesServiciosEditor)
        },
        {
            provide: DocumentService, useExisting: OrdenServicioService
        }
    ]
})
export class OrdenesServiciosEditor extends DocumentEditor {
    ordenServicio = new OrdenServicioInfo();
    idOrden: string = '';
    orden = new OrdenInfo();
    servicios: ServicioItem[] = [];
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: OrdenServicioDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private ordenServicioService: OrdenServicioService,
        private servicioService: ServicioService,
        private ordenService: OrdenService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idServicio': [0, AztecaValidators.selected],
            'diagnostico': [''],
            'observaciones': [''],
            'horaIni': [0],
            'horaFin': [0],
            'idResponsable': [0, AztecaValidators.selected],

        });
    }

    ngOnInit() {
        this.route.params.forEach((params: Params) => {
            this.idOrden = params['idOrden'];
        });
        
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.loadServicios();
        this.loadOrden();
        this.ordenServicio.idOrden = this.idOrden;
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.ordenServicioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ordenServicio = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.ordenServicioService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ordenServicio = doc;
                this.detailsEditor.clientIndexStart = this.ordenServicio.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.ordenServicioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.ordenServicio = doc;
                this.detailsEditor.clientIndexStart = this.ordenServicio.details.length;
                if (this.ordenServicio.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.ordenServicioService.update(this.ordenServicio)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.ordenServicio.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.ordenServicio = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.ordenServicioService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ordenServicioService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.ordenServicio.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new OrdenServicioDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: OrdenServicioDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: OrdenServicioDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: OrdenServicioDetail[]) {
        this.ordenServicio.details = updatedDetails;
    }

    onSavingDetail(detail: OrdenServicioDetail) {
        this.detailsList.updateItem(detail);
    }

    loadServicios() {
        this.servicioService.LoadList(this.idOrden)
            .then(tipos => this.servicios = tipos)
            .catch(error => this.handleError(error));
    }

    loadOrden() {
        this.ordenService.getDetail(this.idOrden)
            .then(ord => this.orden = ord)
            .catch(error => this.handleError(error));
    }
}

