﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { OrdenServicioInfo, OrdenServicioItem } from '../models/orden-servicio';
import { OrdenServicioDetail } from '../models/orden-servicio-detail';
import { OrdenServicioService } from '../services/orden-servicio.service';

@Component({
    selector: 'mantenimiento-ordenes-servicios-manager',
    templateUrl: 'ordenes-servicios.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => OrdenesServiciosManager)
        },
        {
            provide: DocumentService, useExisting: OrdenServicioService
        }
    ]
})
export class OrdenesServiciosManager extends DocumentManager {
    ordenesServicios: OrdenServicioItem[];
    @Input() idOrden: string = '';

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private ordenServicioService: OrdenServicioService) {
        super(context, router, route);
    }



    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.ordenServicioService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.ordenesServicios = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['/maintenance/ordenes-servicios/', this.process.id, this.idOrden, 0]);
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        //this.router.navigate([document.id], { relativeTo: this.route });
        this.router.navigate(['/maintenance/ordenes-servicios/', this.process.id, this.idOrden, document.id]);
    }

    onDeletingItem(document: DocumentItem) {
        this.ordenServicioService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.ordenServicioService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ordenServicioService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

}

