﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, CollectionView } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { OrdenInfo, OrdenItem } from '../models/orden';
import { OrdenDetail } from '../models/orden-detail';
import { OrdenService } from '../services/orden.service';
import { OrdenDetailEditor } from './parts/ordenes-detail.editor';

import { RecursoItem, RecursoInfo, RecursoSelectorComponent, EquipoSelectorComponent, EquipoInfo } from '../../maintenance';

import { TipoServicioService } from '../services/tipos-servicio.service';
import { TipoServicioItem } from '../models/tipos-servicio';

import { TipoOrdenService } from '../services/tipo-orden.service';
import { TipoOrdenItem } from '../models/tipo-orden';

import { TallerService } from '../services/taller.service';
import { TallerItem } from '../models/taller';


@Component({
    selector: 'mantenimiento-ordenes-editor',
    templateUrl: './ordenes.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => OrdenesEditor)
        },
        {
            provide: DocumentService, useExisting: OrdenService
        }
    ]
})
export class OrdenesEditor extends DocumentEditor {
    orden = new OrdenInfo();
    tiposServicios: TipoServicioItem[] = [];
    idServicioTipo: number = 0;
    tiposOrdenes: TipoOrdenItem[] = [];
    idTipoOrden: number = 0;
    talleres: TallerItem[] = [];
    idTaller: number = 0;

    recursos: RecursoItem[] = [];
    currentRecurso: RecursoItem = new RecursoItem();
    recursosCollection: CollectionView = new CollectionView();


    @ViewChild('selectorRecurso') selectorRecurso: RecursoSelectorComponent;
    @ViewChild('cboTipoServicio') cboTupoServicio: object;
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: OrdenDetailEditor;
    @ViewChild('selectorEquipo') selectorEquipo: EquipoSelectorComponent;
    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private ordenService: OrdenService,
        private tipoService: TipoServicioService,
        private tipoOrdService: TipoOrdenService,
        private tallerService: TallerService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idServicioTipo': [0, Validators.required],
            'idTipoOrden': [0, Validators.required],
            'idTaller': [0, AztecaValidators.selected],
            'idEquipo': [0, Validators.required],
            'idSupervisor': [0],
            'fechaProgramada': [''],
            'tiempoEstimado': [0],
            'prioridad': [0],
            'descripcion': ['', Validators.required],
            'recurso': [0],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {
        this.applyPolicies();
        this.loadTiposServicio();
        this.loadTiposOrden();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.ordenService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.orden = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.ordenService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.orden = doc;
                this.detailsEditor.clientIndexStart = this.orden.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.ordenService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.orden = doc;
                this.loadTalleres(this.context.app.defaultSite.idBranch, this.orden.equipo.idServicioTipo);
                this.listRecursos();
                this.detailsEditor.clientIndexStart = this.orden.details.length;
                if (this.orden.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.ordenService.update(this.orden)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.orden.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.orden = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.ordenService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ordenService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.orden.identity);
            }
            if (args.heading == 'Servicios') {

            }

            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    loadTiposServicio() {
        this.tipoService.loadList()
            .then(tipos => this.tiposServicios = tipos)
            .catch(error => this.handleError(error));
    }


    loadTiposOrden() {
        this.tipoOrdService.loadList()
            .then(tipos => this.tiposOrdenes = tipos)
            .catch(error => this.handleError(error));
    }


    loadTalleres(idSucursal: number, idServicioTipo: number) {
        this.tallerService.loadList(idSucursal, idServicioTipo)
            .then(tipos => this.talleres = tipos)
            .catch(error => this.handleError(error));
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {

        this.detailsEditor.detail = new OrdenDetail();
        this.detailsEditor.show();

    }

    onEditingDetail(detail: OrdenDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: OrdenDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: OrdenDetail[]) {
        this.orden.details = updatedDetails;
    }

    onSavingDetail(detail: OrdenDetail) {
        this.detailsList.updateItem(detail);
    }

    onSelectedItemChanged(id: number) {
        /*this.loadTiposOrden();*/
        this.loadTalleres(this.context.app.defaultSite.idBranch, id);
    }

    listRecursos() {
        this.ordenService.ListRecursos(this.orden.identity)
            .then(recs => {
                this.recursos = recs
                this.recursosCollection.sourceCollection = this.recursos;
            })
            .catch(error => this.handleError(error));
    }

    asignarRecurso(idOrden: string, idRecurso: number) {
        this.ordenService.AsignarRecurso(idOrden, idRecurso, '')
            .then(r => {
                this.listRecursos();
            })
            .catch(error => this.handleError(error));
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.asignarRecurso(this.orden.identity, this.selectorRecurso.selectedItem.identity);
        }
    }

    onMenuRecursosClicked(menu: Menu, args: any) {
        if (menu.selectedIndex == 0) {
            //Eliminar
            this.recursos = this.recursosCollection.currentItem;
            this.ordenService.DesasignarRecurso(this.orden.identity, this.recursosCollection.currentItem.id)
                .then(r => {
                    this.recursosCollection.remove(this.recursos);
                })
                .catch(error => this.handleError(error));
        }

    }

}