﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { RequisicionServicioInfo, RequisicionServicioItem } from '../models/requisicion-servicio';
import { RequisicionServicioDetail } from '../models/requisicion-servicio-detail';
import { RequisicionServicioService } from '../services/requisicion-servicio.service';
import { RequisicionServicioDetailEditor } from './parts/requisiciones-servicios-detail.editor';

import { TipoServicioService } from '../services/tipos-servicio.service';
import { TipoServicioItem } from '../models/tipos-servicio';

import { EquipoSelectorComponent, EquipoInfo } from '../../maintenance';

@Component({
    selector: 'mantenimiento-requisiciones-servicios-editor',
    templateUrl: './requisiciones-servicios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RequisicionesServiciosEditor)
        },
        {
            provide: DocumentService, useExisting: RequisicionServicioService
        }
    ]
})
export class RequisicionesServiciosEditor extends DocumentEditor {
    requisicionServicio = new RequisicionServicioInfo();
    tiposServicios: TipoServicioItem[] = [];
    idServicioTipo: number = 0;
    equipo: EquipoInfo = new EquipoInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: RequisicionServicioDetailEditor;
    @ViewChild('selectorEquipo') selectorEquipo: EquipoSelectorComponent;
    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private requisicionServicioService: RequisicionServicioService,
        private tipoService: TipoServicioService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idServicioTipo': [0, Validators.required],
            'equipo': ['', Validators.required],
            'prioridad': [0, Validators.required],
            'descripcion': ['', Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.loadTiposServicio();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.requisicionServicioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.requisicionServicio = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.requisicionServicioService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.requisicionServicio = doc;
                this.detailsEditor.clientIndexStart = this.requisicionServicio.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.requisicionServicioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.requisicionServicio = doc;
                this.idServicioTipo = doc.equipo.idServicioTipo;
                this.detailsEditor.clientIndexStart = this.requisicionServicio.details.length;
                if (this.requisicionServicio.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.requisicionServicioService.update(this.requisicionServicio)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.requisicionServicio.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.requisicionServicio = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.requisicionServicioService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.requisicionServicioService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.requisicionServicio.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    loadTiposServicio() {
        this.tipoService.loadList()
            .then(tipos => this.tiposServicios = tipos)
            .catch(error => this.handleError(error));
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new RequisicionServicioDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: RequisicionServicioDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: RequisicionServicioDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: RequisicionServicioDetail[]) {
        this.requisicionServicio.details = updatedDetails;
    }

    onSavingDetail(detail: RequisicionServicioDetail) {
        this.detailsList.updateItem(detail);
    }

    onSelectedItemChanged(equipo: EquipoInfo) {
        if (equipo) {
            this.equipo = equipo;
        }
    }


}

