﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { ServicioRequisicionInfo, ServicioRequisicionItem } from '../models/servicio-requisicion';
import { ServicioRequisicionDetail } from '../models/servicio-requisicion-detail';
import { ServicioRequisicionService } from '../services/servicio-requisicion.service';
import { ServicioRequisicionDetailEditor } from './parts/servicios-requisiciones-tit-detail.editor';

@Component({
    selector: 'mantenimiento-servicios-requisiciones-tit-editor',
    templateUrl: './servicios-requisiciones-tit.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ServiciosRequisicionesTitEditor)
        },
        {
            provide: DocumentService, useExisting: ServicioRequisicionService
        }
    ]
})
export class ServiciosRequisicionesTitEditor extends DocumentEditor {
    servicioRequisicion = new ServicioRequisicionInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: ServicioRequisicionDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private servicioRequisicionService: ServicioRequisicionService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'sample': ['', Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.servicioRequisicionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.servicioRequisicion = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.servicioRequisicionService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.servicioRequisicion = doc;
                this.detailsEditor.clientIndexStart = this.servicioRequisicion.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.servicioRequisicionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.servicioRequisicion = doc;
                this.detailsEditor.clientIndexStart = this.servicioRequisicion.details.length;
                if (this.servicioRequisicion.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.servicioRequisicionService.update(this.servicioRequisicion)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.servicioRequisicion.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.servicioRequisicion = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.servicioRequisicionService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.servicioRequisicionService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.servicioRequisicion.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new ServicioRequisicionDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: ServicioRequisicionDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: ServicioRequisicionDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: ServicioRequisicionDetail[]) {
        this.servicioRequisicion.details = updatedDetails;
    }

    onSavingDetail(detail: ServicioRequisicionDetail) {
        this.detailsList.updateItem(detail);
    }

}

