﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, CatalogItem } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';
//Locales
import { EquipoInfo, EquipoItem } from '../models/equipo'
import { Documento } from '../models/documento'
import { FotografiaEquipo } from '../models/fotografia-equipo'
import { FileInfo } from 'framework3/general';

@Injectable()
export class EquipoService extends CatalogService<EquipoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/mantenimiento/equipos");
    }

    create(): Promise<EquipoInfo> {
        return Promise.resolve(new EquipoInfo());
    }

    LoadDocumentos(idEquipo: number): Promise<Documento[]> {
        let params = new HttpParams()
            .set("idEquipo", idEquipo.toString());
        return this.apiService.get<Documento[]>("load-documentos", params)
            .toPromise();
    }

    AgregarDocumento(idEquipo: number, idTipoDocumento: number, fileInfo: FileInfo): Promise<any> {
        let params = new HttpParams()
            .set("idEquipo", idEquipo.toString())
            .set("idTipoDocumento", idTipoDocumento.toString());
         return this.apiService.post<any>("agregar-documento", params, fileInfo)
            .toPromise();
    }

    EliminarDocumento(idEquipo: number, idTipoDocumento: number, idArchivo: string): Promise<any> {
        let params = new HttpParams()
            .set("idEquipo", idEquipo.toString())
            .set("idTipoDocumento", idTipoDocumento.toString())
            .set("idArchivo", idArchivo);
        return this.apiService.delete<any>("eliminar-documento", params)
            .toPromise();

    }

    LoadFotografias(idEquipo: number): Promise<FotografiaEquipo[]> {
        let params = new HttpParams()
            .set("idEquipo", idEquipo.toString());
        return this.apiService.get<FotografiaEquipo[]>("load-fotografias", params)
            .toPromise();
    }

    AgregarFotografia(idEquipo: number, fotografiaEquipo: FotografiaEquipo): Promise<any> {
        let params = new HttpParams()
            .set("idEquipo", idEquipo.toString());
        return this.apiService.post<any>("agregar-fotografia", params, fotografiaEquipo)
            .toPromise();

    }

    EliminarFotografia(idEquipo: number, idArchivo: string): Promise<any> {
        let params = new HttpParams()
            .set("idEquipo", idEquipo.toString())
            .set("idArchivo", idArchivo);
        return this.apiService.delete<any>("eliminar-fotografia", params)
            .toPromise();
    }

    searchKeyword2(idBranch: number, keyword: string, idServicioTipo: number): Observable<CatalogItem[]> {
        let params = new HttpParams()
            .set("idBranch", idBranch.toString())
            .set("keyword", keyword)
            .set("idServicioTipo", idServicioTipo.toString());
        return this.apiService.get<CatalogItem[]>("search-equipo", params)
           
    }
}

