﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { TipoServicioInfo, TipoServicioItem, TipoServicioUsuarioItem } from '../models/tipos-servicio'

@Injectable()
export class TipoServicioService extends CatalogService<TipoServicioInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/mantenimiento/servicios-tipos');
    }

    create(): Promise<TipoServicioInfo> {
        return Promise.resolve(new TipoServicioInfo());
    }

    loadList(): Promise<TipoServicioItem[]> {
        let params = new HttpParams()
        return this.apiService.get<TipoServicioItem[]>("load-list", params)
            .toPromise();
    }


    LoadAccessUsersList(idTipoServicio: number): Promise<TipoServicioUsuarioItem[]> {
        let params = new HttpParams()
            .append("idTipoServicio", idTipoServicio.toString());
        return this.apiService.get<TipoServicioUsuarioItem[]>("load-access-users-list", params)
            .toPromise();
    }


    addMembers(idTipoServicio: number, usuarios: TipoServicioUsuarioItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idTipoServicio', idTipoServicio.toString());
        return this.apiService.post<any>('add-members', params, usuarios)
            .toPromise();
    }

    removeMember(idTipoServicio: number, idUsuario: number): Promise<any> {
        let params = new HttpParams()
            .set('idTipoServicio', idTipoServicio.toString())
            .set('idUsuario', idUsuario.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }


}
