﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

//wijmo
import { Control } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellRange, CellType, Column, CellRangeEventArgs } from 'wijmo/wijmo.grid';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, ChangeState } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ExplosionService } from '../services/explosion.service';
import { ProductoService } from '../services/producto.service';
import { ExplosionInfo, ExplosionItem} from '../models/explosion';
import { ProductoInsumo } from '../models/producto-insumo';

@Component({
    selector: 'produccion-explosiones-editor',
    templateUrl: './explosiones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ExplosionesEditor)
        },
        {
            provide: CatalogService, useExisting: ExplosionService
        }
    ]
})
export class ExplosionesEditor extends CatalogEditor {
    explosion: ExplosionInfo = new ExplosionInfo();
    insumosCollection: TrackableCollection = new TrackableCollection();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private explosionService: ExplosionService, private productoService: ProductoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'fecha': ['', Validators.required],
            'idProducto': ['', Validators.required],
            'cantidad': ['', Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.explosionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.explosion = info;   
                this.insumosCollection.sourceCollection = this.explosion.insumos;             
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.explosionService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.explosion = info;
                this.insumosCollection.sourceCollection = this.explosion.insumos; 
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.explosionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.explosion = info;      
                this.insumosCollection.sourceCollection = this.explosion.insumos;           
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.explosionService.update(this.explosion)
            .then(info => {
                this.context.app.hideSpinner();
                this.explosion = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }
    onExplosionarClick() {
        this.productoService.explosionar(this.explosion.idProducto, this.explosion.cantidad)
            .then(insumos => this.fillInsumos(insumos))
            .catch(error => this.handleError(error));
    }

    fillInsumos(insumos: ProductoInsumo[]) {
        let insumosTemp: ProductoInsumo[] = insumos.slice();

        insumosTemp.forEach(s => {
            s.state = ChangeState.MODIFIED;
        });

        this.explosion.insumos = insumosTemp;
        this.insumosCollection.sourceCollection = insumosTemp;
        
    }
}