﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
import { Control, CollectionView } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, AzComboboxComponent } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { InsumoService } from '../services/insumo.service';
import { InsumoInfo, InsumoItem } from '../models/insumo';
import { InsumoArticulo } from '../models/insumo-articulo';

import { UnidadMedidaService, UnidadMedidaTipoService, UnidadMedidaTipoItem, UnidadMedidaItem, ArticuloStockItem, ArticuloStockSelectorComponent } from '../../inventory';

@Component({
    selector: 'produccion-insumos-editor',
    templateUrl: './insumos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => InsumosEditor)
        },
        {
            provide: CatalogService, useExisting: InsumoService
        }
    ]
})
export class InsumosEditor extends CatalogEditor {
    insumo: InsumoInfo = new InsumoInfo();   
    unidadesMedidaTipo: UnidadMedidaTipoItem[] = [];     
    unidadesMedida: UnidadMedidaItem[] = [];
    currentArticulo: ArticuloStockItem = new ArticuloStockItem();     
    articulosCollection: CollectionView = new CollectionView();
    //@ViewChild('dialog') articulosDialog: ModalDirective;
    @ViewChild('cboUnidadMedidaTipo') cboUnidadMedidaTipo: AzComboboxComponent;
    @ViewChild('selectorArticulo') selectorArticulo: ArticuloStockSelectorComponent;
    
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private insumoService: InsumoService, private unidadMedidaTipoService: UnidadMedidaTipoService, private unidadMedidaService: UnidadMedidaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'descripcion': [''],
            'idUnidadMedidaTipo': [''],
            'idUnidadMedida': ['', Validators.required],
            'articulo': ['']
        });
    }

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadUnidadesMedidaTipo();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.insumoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.insumo = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.insumoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.insumo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.insumoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                //this.cboUnidadMedidaTipo. = this.unidadesMedidaTipo.find(item => item[id = info.idUnidadMedidaTipo]);
                this.insumo = info;                
                this.articulosCollection.sourceCollection = info.articulos;
                this.selectUnidadTipo( this.insumo.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.insumoService.update(this.insumo)
            .then(info => {
                this.context.app.hideSpinner();
                this.insumo = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    loadUnidadesMedidaTipo() {
        this.unidadMedidaTipoService.loadList()
            .then(unidades => this.unidadesMedidaTipo = unidades)
            .catch(error => this.handleError(error));
    }

    selectUnidadTipo(idUnidadMedidaTipo) {
        if (idUnidadMedidaTipo != undefined) {
            this.unidadMedidaService.loadList(idUnidadMedidaTipo)
                .then(unidades => this.unidadesMedida = unidades)
                .catch(error => this.handleError(error));
        }
    }

    selectArticulosTab() {
        //refresca los controles de wijmo
        Control.invalidateAll();
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            let articulo = new InsumoArticulo();
            articulo.idArticuloStock = this.selectorArticulo.selectedItem.identity;
            articulo.codigo = this.selectorArticulo.selectedItem.code;
            articulo.articulo = this.selectorArticulo.selectedItem.nombre;

            let newArticulo: InsumoArticulo[] = this.insumo.articulos.slice();
            newArticulo.push(articulo);
            this.insumo.articulos= newArticulo;
            this.articulosCollection.sourceCollection = newArticulo;

            this.selectorArticulo.clear();
        }
    }

    onArticuloDelete(row: InsumoArticulo) {
        this.articulosCollection.remove(row);
    }

    

}