﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { InsumoInfo, InsumoItem } from '../models/insumo';
import { InsumoService } from '../services/insumo.service';

@Component({
    selector: 'production-insumo-selector',
    templateUrl: './insumo-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => InsumoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => InsumoSelectorComponent),
        multi: true
    }]
})
export class InsumoSelectorComponent extends CatalogSelector {
    insumos: any[];
    selectedItem: InsumoInfo = null;

    @Output() itemSelected: EventEmitter<InsumoInfo> = new EventEmitter<InsumoInfo>();

    constructor(protected context: Context, private insumoService: InsumoService) {
        super(context);               

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findInsumo(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.insumoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre; 
                    this.itemSelected.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.insumoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.insumos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findInsumo(keyword: string): Observable<CatalogItem[]> {
        return this.insumoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

