﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';

//Features
import { TrackableCollection } from 'framework3/editors';
import { ExplosionService } from '../services/explosion.service';
import { ArticuloStockInfo, ArticuloStockItem } from '../../inventory/models/articulo-stock';

@Component({
    selector: 'insumos-conversion-component',
    templateUrl: './insumos-conversion.component.html'
})
export class InsumosConversionComponent {

    conversionCollection: TrackableCollection = new TrackableCollection();
    
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent
    constructor(private formBuilder: FormBuilder, private router: Router, private explosionService: ExplosionService, private context: Context) {
      
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadInsumos());
    }

    loadInsumos() {       
        this.context.app.showSpinner();
        this.explosionService.loadInsumosConversion(this.context.app.defaultSite.identity)
            .then(conversion => {
                this.conversionCollection.sourceCollection = conversion;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onUpdate() {
        this.context.app.showSpinner();
        this.explosionService.saveInsumosConversion(this.conversionCollection.sourceCollection)
            .then(_ => {
                this.alerts.showAlert("Los cambios han sido guardados con exito", 'ok');
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onCancel() {

    }

    onSelectArticuloConversion(item: ArticuloStockItem, cell: any) {
        if (item) {
            cell.item.articuloConversion.nombre = item.nombre;
        }
        else {
            cell.item.articuloConversion.nombre = '';
        }
    }
    

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
        this.alerts.showAlert(error.toString(), 'danger');
    }

}