﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
import { CatalogOptions } from 'framework3/custom';

//Features
import { ExplosionService } from '../services/explosion.service';


@Component({
    selector: 'production-explosion-component',
    templateUrl: './producto-explosion.component.html'
})
export class ProductoExplosionComponent {
    editorForm: FormGroup;
    fechaIni: Date;
    fechaFin: Date;   

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent

    constructor(private formBuilder: FormBuilder, private router: Router, private explosionService: ExplosionService, private context: Context) {
        this.editorForm = formBuilder.group({
            'fechaIni': [0, Validators.required],
            'fechaFin': [0, Validators.required]
        });
    }

    onAccept() {
        this.explosionService.printComparativo(this.context.app.defaultSite.identity, this.fechaIni, this.fechaFin)        
            .then(report => this.printReport(report));
    }

    printReport(report: any) {
        this.reportViewer.show("data:application/pdf;base64, " + report);
    }

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }

    handleError(error: any) {
        console.error(error);
        this.alerts.showAlert(error.tostring(), 'danger');
    }

}