﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
//Features

import { ProductoInfo} from '../models/producto';
import { ProductoService } from '../services/producto.service';
import { ArticuloStockService, ArticuloStockInfo, ArticuloStockSelectorComponent } from '../../inventory';

@Component({
    selector: 'production-producto-nuevo-component',
    templateUrl: './producto-nuevo.component.html'
})
export class ProductoNuevoComponent {
    editorForm: FormGroup;
    catalogOptions: CatalogOptions = new CatalogOptions();
    codigo: string;
    idArticuloStock: number;
    existeArticulo: boolean = false;
    existeProducto: boolean = false;        

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent
    @ViewChild('confirmLinkDialog') dialogLink: ModalDirective;

    modalidades: {id: string, nombre: string}[] = [  { id: "1", nombre: 'Nuevo en inventario' },
                    { id: "2", nombre: 'Enlazar a existente en inventario' }] ;
   

    constructor(private formBuilder: FormBuilder, private router: Router, private productoService: ProductoService, private context: Context, private articuloStockService: ArticuloStockService) {
         this.editorForm = formBuilder.group({
            'idArticuloStock': [0, this.validateArticulo],
            'modalidad': [0, Validators.required],
            'codigo': ['', (control) => this.validateCode(this.catalogOptions.codigoAutogenerado, control)]
        });
    }

    ngAfterViewInit() {
        setTimeout(_=> this.loadOptions());
    }

    loadOptions() {   
        this.productoService.getOptions()
            .then(options => this.catalogOptions = options)
            .catch(error => this.handleError(error));
    }

    selectModalidad(arg) {
        let i: number;
    }

    validateArticulo(autogenerado: boolean, control: FormControl) {
        if (autogenerado || control.value != 0)
            return null;
        else
            return 'Articulo no válido';
    }

    validateCode(autogenerado: boolean, control: FormControl) {
        if (autogenerado || control.value != '')
            return null;
        else
            return 'Código no válido';
    }

    onCodigoSet() {        
        let producto: ProductoInfo;
        //let articulo: ArticuloStockInfo;
        this.existeProducto = true;
        this.existeArticulo = true;

        let promiseProd = this.productoService.getDetailByCode(this.context.app.defaultSite.identity, this.codigo)
            .then(prod => producto= prod)
            .catch(error => this.existeProducto=false);    

        let promiseArt = this.articuloStockService.getDetailByCode(this.context.app.defaultSite.identity, this.codigo)
            .then(art => this.idArticuloStock = art.identity)
            .catch(error => this.existeArticulo = false);


        Promise.all([promiseProd, promiseArt])
            .then(result => {                                
                this.validContinue();
            })
            .catch(error => {        
                this.validContinue();
            });             
    }

    validContinue() {
        if (this.existeProducto) {
            this.showAlert("Ya existe un producto con este código", 'danger');
        }
        else if (this.existeArticulo) {
            this.dialogLink.config.backdrop = false;
            this.dialogLink.show();
        }
        else
            this.goEditor();
    }

    linkArticulo() {
        this.goEditor();
    }

    goEditor() {
        this.router.navigate(['production/productos/0', { codigo: this.codigo, idArticuloStock: this.idArticuloStock }]);
    }

    handleError(error:any) {
        console.error(error);
    }

    showAlert(message: string,type:string) {
        this.alerts.showAlert(message, type);
    }
}