﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { ProductoInfo, ProductoItem } from '../models/producto';
import { ProductoService } from '../services/producto.service';

@Component({
    selector: 'production-producto-selector',
    templateUrl: './producto-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ProductoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ProductoSelectorComponent),
        multi: true
    }]
})
export class ProductoSelectorComponent extends CatalogSelector {
    productos: any[];
    selectedItem: ProductoInfo = null;

    @Output() itemSelected: EventEmitter<ProductoInfo> = new EventEmitter<ProductoInfo>();

    constructor(protected context: Context, private productoService: ProductoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findProducto(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.productoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre; 
                    this.itemSelected.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.productoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.productos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findProducto(keyword: string): Observable<CatalogItem[]> {
        return this.productoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

