﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
//Features
import { InsumoInfo } from '../models/insumo';
import { InsumoSelectorComponent } from '../components/insumo-selector.component';
import { InsumoArticulo} from '../models/insumo-articulo';
import { ProductoInsumo } from '../models/producto-insumo';
import { UnidadMedidaItem, UnidadMedidaService, ArticuloStockItem } from '../../inventory';

@Component({
    selector: 'production-producto-insumo-editor',
    templateUrl:'./producto-insumo.editor.html'
})
export class ProductoInsumoEditor {
    editorForm: FormGroup;
    //
    @Input() data: ProductoInsumo;
    //
    @Output() dataUpdated: EventEmitter<ProductoInsumo> = new EventEmitter<ProductoInsumo>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild('selInsumo') selInsumo: InsumoSelectorComponent ;
    //
    articulos: InsumoArticulo[] = [];
    unidadesMedida: UnidadMedidaItem[] = [];
    

    constructor(private formBuilder: FormBuilder, private unidadMedidaService: UnidadMedidaService) {

        this.editorForm = formBuilder.group({
            'idInsumo':[0, Validators.required],
            'idArticuloStock': [0, Validators.required],
            'idUnidadMedida':[0, Validators.required],
            'cantidad': [0],
            'porcentajeMerma':[0]
        });        
    }

    show() {
        this.dialog.config.backdrop = false;
        this.dialog.show();
        if (this.data.idInsumo == 0) {
            setTimeout(_ => this.editorForm.reset());
        }
        else {
            this.selInsumo.loadItem(this.data.idInsumo);
        }
        
    }

    onUpdate() {
        this.dataUpdated.emit(this.data);
        this.dialog.hide();
    }

    onInsumoSelected(insumo: InsumoInfo) {
        
        this.articulos = insumo.articulos;
        this.data.insumo = insumo.nombre;
        this.data.codigo = insumo.code;

        this.unidadMedidaService.loadList(insumo.idUnidadMedidaTipo)
            .then(unidades => this.unidadesMedida = unidades)
            .catch(error => this.handleError(error));
    }

    //loadUnidadesMedida(unidades) {
    //    this.unidadesMedida = unidades;
    //    this.selectUnidadMedida(this.data.idUnidadMedida);   
    //}

    selectArticulo(idArticulo) {
        this.data.articulo = this.articulos.find(item => item.idArticuloStock==idArticulo).articulo;
    }

    selectUnidadMedida(idUnidadMedida) {
        this.data.unidadMedida = this.unidadesMedida.find(item => item.id == idUnidadMedida).nombre;
    }

    handleError(error) {
        console.error(error)
    }

}
