﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
//Features
import { ProductoInfo } from '../models/producto';
import { ProductoSubproducto } from '../models/producto-subproducto';

import { UnidadMedidaItem, UnidadMedidaService } from '../../inventory';

@Component({
    selector: 'production-producto-subproducto-editor',
    templateUrl: './producto-subproducto.editor.html'
})
export class ProductoSubproductoEditor {
    editorForm: FormGroup;
    //
    @Input() data: ProductoSubproducto;
    //
    @Output() dataUpdated: EventEmitter<ProductoSubproducto> = new EventEmitter<ProductoSubproducto>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    //
    unidadesMedida: UnidadMedidaItem[] = [];


    constructor(private formBuilder: FormBuilder, private unidadMedidaService: UnidadMedidaService) {

        this.editorForm = formBuilder.group({
            'idProducto': [0, Validators.required],
            'idUnidadMedida': [0, Validators.required],
            'cantidad': [0],
            'porcentajeMerma': [0]
        });
    }

    show() {
        this.dialog.config.backdrop = false;
        this.dialog.show();
        if (this.data.idProducto == 0) {
            setTimeout(_ => this.editorForm.reset());
        }
        else {

        }
        
    }

    onUpdate() {
        this.dataUpdated.emit(this.data);
        this.dialog.hide();
    }

    onProductoSelected(producto: ProductoInfo) {        
        this.data.codigo = producto.code;
        this.data.producto = producto.nombre;

        this.unidadMedidaService.loadList(producto.articulo.unidadMedida.idUnidadMedidaTipo)
            .then(unidades => this.unidadesMedida = unidades)
            .catch(error => this.handleError(error));
    }
    
    selectUnidadMedida(idUnidadMedida) {
        this.data.unidadMedida = this.unidadesMedida.find(item => item.id == idUnidadMedida).nombre;
    }

    handleError(error) {
        console.error(error)
    }

}
