﻿import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

import { FormsModule, ReactiveFormsModule } from '@angular/forms';
//Modulos Azteca
import { CoreModule } from 'framework3/core';
import { EditorsModule } from 'framework3/editors';
import { SearchModule } from 'framework3/search';
import { GeneralModule } from 'framework3/general';
import { ControlsModule } from 'framework3/controls';
import { SecurityModule } from 'framework3/security';
import { ProductionRoutingModule } from './production.routing.module';
//Azteca Kernel
import { InventoryModule } from '../inventory/inventory.module';
//Bootstrap
import { TabsModule, ModalModule } from 'ngx-bootstrap';
//Wijmo
import { WjCoreModule } from 'wijmo/wijmo.angular2.core';
import { WjGridModule } from 'wijmo/wijmo.angular2.grid';
import { WjGridFilterModule } from 'wijmo/wijmo.angular2.grid.filter';
import { WjInputModule } from 'wijmo/wijmo.angular2.input';

/////////////////////////////// Servicios ////////////////////////////
import { InsumoService } from './services/insumo.service';
import { ProductoService } from './services/producto.service';
import { ExplosionService } from './services/explosion.service';


//////////////////////////// Catalogos ////////////////////////////////
import { InsumosManager } from './catalogs/insumos.manager';
import { InsumosEditor } from './catalogs/insumos.editor';
import { ProductosManager } from './catalogs/productos.manager';
import { ProductosEditor } from './catalogs/productos.editor';
import { ExplosionesManager } from './catalogs/explosiones.manager';
import { ExplosionesEditor } from './catalogs/explosiones.editor';


//////////////////////////// Componentes /////////////////////////////
import { InsumoSelectorComponent } from './components/insumo-selector.component';
import { ProductoSelectorComponent } from './components/producto-selector.component';
import { ProductoNuevoComponent } from './components/producto-nuevo.component';
import { ProductoExplosionComponent } from './components/producto-explosion.component';
import { InsumosConversionComponent } from './components/insumos-conversion.component';

///////////////////////////// Document /////////////////////////////////

//////////////////////////// Parts /////////////////////////////////////
import { ProductoInsumoEditor } from './parts/producto-insumo.editor';
import { ProductoSubproductoEditor } from './parts/producto-subproducto.editor';


/////////////////////////////////////////////////////////////////////////////////
let COMPONENTES = [InsumosManager, InsumosEditor, ProductosManager, ProductosEditor, ExplosionesManager, ExplosionesEditor, InsumoSelectorComponent, ProductoSelectorComponent, ProductoInsumoEditor, ProductoSubproductoEditor, ProductoNuevoComponent, ProductoExplosionComponent, InsumosConversionComponent];
let SERVICIOS = [InsumoService, ProductoService, ExplosionService];

@NgModule({
    imports: [
        CommonModule, FormsModule, ReactiveFormsModule,
        TabsModule.forRoot(), ModalModule.forRoot(),
        CoreModule, EditorsModule, SearchModule, GeneralModule, ControlsModule, SecurityModule, InventoryModule, ProductionRoutingModule,
        WjCoreModule, WjInputModule, WjGridModule, WjGridFilterModule
    ],
    declarations: COMPONENTES,
    providers: SERVICIOS,
    exports: COMPONENTES
})
export class ProductionModule {

}
