﻿import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

//////////////////////////// Guards //////////////////////////////////
import { AuthGuard } from 'framework3/core';
import { EditorDeactivateGuard } from 'framework3/editors';

//////////////////////////// Catalogos ////////////////////////////////
import { InsumosManager } from './catalogs/insumos.manager';
import { InsumosEditor } from './catalogs/insumos.editor';
import { ProductosManager } from './catalogs/productos.manager';
import { ProductosEditor } from './catalogs/productos.editor';
import { ExplosionesManager } from './catalogs/explosiones.manager';
import { ExplosionesEditor } from './catalogs/explosiones.editor';
import { ProductoNuevoComponent } from './components/producto-nuevo.component';
import { ProductoExplosionComponent } from './components/producto-explosion.component';
import { InsumosConversionComponent } from './components/insumos-conversion.component';

///////////////////////////// Document /////////////////////////////////

///////////////////////////// Rutas /////////////////////////////////

const routes: Routes = [
    {
        path: '',
        canActivateChild: [AuthGuard],
        children: [
            { path: 'insumos/:proceso', component: InsumosManager },
            { path: 'insumos/:proceso/:id', component: InsumosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'productos/:proceso', component: ProductosManager },
            { path: 'productos/:proceso/nuevo', component: ProductoNuevoComponent },
            { path: 'productos/:proceso/:id', component: ProductosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'explosiones/:proceso', component: ExplosionesManager },
            { path: 'explosiones/:proceso/:id', component: ExplosionesEditor },
            { path: 'explosionesComparativo/:proceso', component: ProductoExplosionComponent },
            { path: 'conversiones/:proceso', component: InsumosConversionComponent },
        ]
    },
];



@NgModule({
    imports: [
        RouterModule.forChild(routes)
    ],
    exports: [
        RouterModule
    ]
})
export class ProductionRoutingModule { }

