﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';

//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { ExplosionInfo } from '../models/explosion'
import { InsumoConversion } from '../models/insumo-conversion'

@Injectable()
export class ExplosionService extends CatalogService<ExplosionInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/produccion/explosiones");
    }

    create(): Promise<ExplosionInfo> {
        return Promise.resolve(new ExplosionInfo());
    }
    
    printComparativo(idSucursal:number, fechaIni:Date, fechaFin: Date): Promise<string> {
        let params = new HttpParams()
            .set('IdSucursal', idSucursal.toString())
            .set('FechaIni', moment(fechaIni).format() )
            .set('FechaFin', moment(fechaFin).format() );
        return this.apiService.get<string>('printComparativo', params)
            .toPromise();
    }

    loadInsumosConversion(idSucursal: number): Promise<InsumoConversion[]> {
        let params = new HttpParams()
            .set('IdSucursal', idSucursal.toString());
        return this.apiService.get<InsumoConversion[]>('load-insumos-conversion', params)
            .toPromise();
    }

    saveInsumosConversion(conversiones: InsumoConversion[]): Promise<boolean> {
        let params = new HttpParams()
        // params.set('idBranch', idBranch.toString());
        return this.apiService.post<boolean>('save-insumos-conversion', params, conversiones)
            .toPromise();
    }

}

