﻿import { Component, EventEmitter, Input, Output, ViewChild, OnInit } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router } from '@angular/router';
//
import { Context, UsuarioService, UsuarioInfo, EmpresaService, EmpresaInfo, SucursalService } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
//
import { ProveedorService } from '../../purchasing/services/proveedor.service';
import { ProveedorInfo } from '../../purchasing/models/proveedor';
import { PortalContext } from '../services/portal-context';

@Component({
    selector: 'proveedores-login',
    templateUrl: './login.component.html'
})
export class LoginComponent {
    loginForm: FormGroup;
    correo: string;
    password: string;
    error: string;

    constructor(private router: Router, private context: Context, private formBuilder: FormBuilder,
        private portalContext: PortalContext, private usuarioService: UsuarioService, private empresaService: EmpresaService, private sucursalService: SucursalService,
        private proveedorService: ProveedorService) {

        this.loginForm = formBuilder.group({
            'correo': ['', Validators.email],
            'password': ['', Validators.required]
        });
    }

    ngOnInit() {
        this.context.app.hideSpinner();
    }

    checkLogin() {
        this.context.app.showSpinner();

        this.usuarioService.login(this.correo, this.password, 'web')
            .then(session => {
                this.context.app.hideSpinner();
                if (session != null) {
                    this.context.setSession(session);
                    //Cargar empresas a las que el usuario asociado al proveedor puede ingresasr
                    this.proveedorService.loadEmpresas(session.user.identity)
                        .then(empresas => {
                            this.portalContext.setCompanies(empresas);
                            /* se solicita en el caso 877 que no pida seleccionar sucursal
                            if (empresas.length > 1 || session.user.sucursales.length > 1) {
                                this.router.navigate(['/select-company']);
                            }
                            else if (empresas.length == 1) {
                                this.context.app.hideSpinner();
                                this.redirect(empresas[0].id);
                            }
                            */
                            if (empresas.length >= 1) {
                                this.context.app.hideSpinner();
                                this.redirect(empresas[0].id);
                            }
                            else {
                                this.error = 'Proveedor no asignado a alguna empresa.';
                            }
                        })
                        .catch(error => {
                            this.handleError(error);
                        });
                }
                else {
                    this.error = 'Nombre de usuario o contraseña incorrecta';
                }
            })
            .catch(error => this.handleError(error));        
    }

    redirect(idEmpresa: number) {
        this.context.app.showSpinner();

        this.empresaService.getDetailByID(idEmpresa)
            .then(empresa => {
                this.context.app.setDefaultCompany(empresa);
                //Cargar información del proveedor
                this.proveedorService.getDetailByUser(this.context.session.user.identity, idEmpresa)
                    .then(proveedor => {
                        this.portalContext.setProveedor(proveedor);
                        //
                        this.sucursalService.getDetailByID(proveedor.idBranch)
                            .then(sucursal => {
                                this.context.app.setDefaultSite(sucursal);

                                this.router.navigate(['proveedores/upload']);
                            })
                            .catch(reason => this.handleError(reason));
                        
                    })
                    .catch(error => this.handleError(error));
            })
            .catch(reason => this.handleError(reason));
    }

    checkReg() {
        this.router.navigate(['proveedores/pre-registro']);
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        console.log(error);
        this.error = this.context.getExceptionMessage(error);
    }

}