﻿import { Component, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//
import { Context, Server } from 'framework3/core';
import { AlertsBarComponent } from 'framework3/controls';
import { ProveedorInfo, PreRegistro } from '../../purchasing/models/proveedor';
import { ProveedorService } from '../../purchasing/services/proveedor.service';

@Component({
    selector: 'proveedores-pre-registro',
    templateUrl: './pre-registro.component.html'
})
export class PreRegistroComponent {
    form: FormGroup;
    password: string = '';
    confirmacion: string = '';
    registrado = false;
    proveedor = new ProveedorInfo();
    pRegistro = new PreRegistro();

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute,
        private context: Context, private server: Server, private proveedorService: ProveedorService) {
        this.form = this.formBuilder.group({
            'password': ['', Validators.required],
            'confirmacion': ['', Validators.required],
            'nombreComercial': ['', Validators.required],
            'correo': ['', Validators.required],
            'rfc': ['', Validators.required],
            'razonSocial': ['', Validators.required]
        });

    }

    ngOnInit() {
        this.cargarSystemSession();
    }



    cargarSystemSession() {
        this.server.getSystemSession()
            .then(session => {
                this.context.setSession(session);
            })
            .catch(error => this.handleError(error));
    }

    PreRegistrar() {
        this.pRegistro.password = this.password;
        this.proveedorService.preRegistrar(this.pRegistro)
            .then(_ => {
                this.registrado = true;
                this.alertsBar.showAlert('El registro se ha completado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }
}