﻿import { Component, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//
import { Context, Server } from 'framework3/core';
import { AlertsBarComponent } from 'framework3/controls';
import { ProveedorService } from '../../purchasing/services/proveedor.service';
import { ProveedorInfo } from '../../purchasing/models/proveedor';

@Component({
    selector: 'proveedores-registro',
    templateUrl: './registro.component.html'
})
export class RegistroComponent {
    form: FormGroup;
    password: string = '';
    confirmacion: string = '';
    idProveedor: number;
    expira: number;
    enlaceCaducado = false;
    registrado = false;
    proveedor = new ProveedorInfo();

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute,
        private context: Context, private server: Server, private proveedorService: ProveedorService) {
        this.form = this.formBuilder.group({
            'password': ['', Validators.required],
            'confirmacion': ['', Validators.required]
        });        

        this.route.params.forEach((params: Params) => {
            this.idProveedor = +params['proveedor'];
            this.expira = +params['expira'];
        });        
    }

    ngOnInit() {        
        this.cargarSystemSession();
    }

    cargarProveedor() {
        this.proveedorService.getDetailByID(this.idProveedor)
            .then(proveedor => {
                this.proveedor = proveedor;
                //Validar si el proveedor ya tiene un usuario asignado
                if (this.proveedor.usuario.identity > 0) {
                    this.registrado = true;
                    this.alertsBar.showAlert('Proveedor ya registrado en el portal', 'warning');
                    setTimeout(_ => this.router.navigate(['../login'], {relativeTo: this.route}), 1000);
                }
                else {
                    this.validarCaducidadLink();
                }
            })
            .catch(error => this.handleError(error));
    }

    validarCaducidadLink() {
        let fechaCaducidad = new Date(this.expira);
        let today = new Date();
        if (today > fechaCaducidad) {
            this.alertsBar.showAlert('El enlace ya ha caducado.', 'warning');
            this.enlaceCaducado = true;
        }
        else {
            setTimeout(_ => this.cargarProveedor());
        }
    }

    cargarSystemSession() {
        this.server.getSystemSession()
            .then(session => {
                this.context.setSession(session);
                this.cargarProveedor();
            })
            .catch(error => this.handleError(error));
    }

    confirmarRegistro() {
        this.proveedorService.registrar(this.idProveedor, this.password)
            .then(_ => {
                this.registrado = true;
                this.alertsBar.showAlert('El registro se ha completado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();        
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }
}