﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context,DocumentService } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
import { DocumentManagerComponent } from 'framework3/editors';

//Locales
import { ComprobanteFiscalService } from '../../purchasing/services/comprobante-fiscal.service';
import { ProveedorService } from '../../purchasing/services/proveedor.service';
import { ProveedorInfo } from '../../purchasing/models/proveedor';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../../purchasing/models/comprobante-fiscal';
import { PortalContext } from '../services/portal-context';

import { DomSanitizer, SafeResourceUrl } from '@angular/platform-browser';

// OC
import { OrdenService } from '../../purchasing/services/orden.service';
import { OrdenInfo, OrdenItem} from '../../purchasing/models/orden';
//import { OrdenesManager } from '../../purchasing/documents/ordenes.manager';
import { OrdenesListComponent } from '../../purchasing/dialogs/ordenes-list.component';


@Component({
    selector: 'proveedores-upload-invoice',
    templateUrl: './upload-invoice.component.html'
    ,providers: [
        //{
        //    provide: BaseComponent, useExisting: forwardRef(() => OrdenesManager)
        //},
        {
            provide: DocumentService, useExisting: OrdenService
        }
    ]
})
export class UploadInvoiceComponent {
    form: FormGroup;
    menuItems: MenuItem[] = [];
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();
    pageSize: number = 20;
    comprobantes: ComprobanteFiscalItem[] = [];
    selectedGroup: FilterGroup;
    filterGroups: FilterGroup[] = [];

    ordenes: OrdenItem[];

    currentComprobante = new ComprobanteFiscalInfo();

    _url: string = "";

    get url(): string {
        return this._url;
    }

    set url(value) {
        this._url = value;
    }
    urlSafe: SafeResourceUrl;

    //
    @ViewChild('viewDialog') viewDialog: ModalDirective;
    @ViewChild(DocumentListComponent) documentList: DocumentListComponent;

    @ViewChild('ordenesManager') ordenesManager: DocumentManagerComponent;
    @ViewChild('ordenesList') ordenesList: OrdenesListComponent;


    constructor(private context: Context, private router: Router, private route: ActivatedRoute, private formBuilder: FormBuilder,
        private portalContext: PortalContext, private comprobanteService: ComprobanteFiscalService, private ordenService: OrdenService,
        public sanitizer: DomSanitizer) {
        
        //Crear formulario de parámetros
        this.form = formBuilder.group({
            'fechaInicial': [new Date()],
            'fechaFinal': [new Date()]
        });
    }

    ngOnInit() {
        this.createMenu();
    }

    ngAfterViewInit() {
        this.fechaInicial = moment().subtract(15, 'day').toDate();
        this.fechaFinal = moment().add(1, 'day').toDate();
        setTimeout(_ => {
            this.refreshList();
            this.url = "http://www.embotelladoraaga.com.mx/pixcua/servicios/Privado/Facturas/Menu.aspx?UserName=" + this.portalContext.proveedor.contribuyente.rfc + "&Password=123";
            this.urlSafe = this.sanitizer.bypassSecurityTrustResourceUrl(this.url);
        });
    }

    /////////////////////// METODOS ////////////////////////////////////

    refreshList() {
        this.context.app.showSpinner();

        this.comprobanteService.loadListByProveedor(this.context.app.defaultCompany.identity, this.portalContext.proveedor.identity, this.fechaInicial, this.fechaFinal)
            .then(comprobantes => {
                this.context.app.hideSpinner();
                //
                this.comprobantes = comprobantes;
               

            })
            .catch(error => this.handleError(error));
    }

    createMenu() {
        //Agregar opciones estándar del catálogo
        let mnuView: MenuItem = { icon: "glyphicon glyphicon-folder-open", text: "Ver", name: "OPEN", smallText: "ver resumen del comprobante" };
        let mnuLink: MenuItem = { icon: "glyphicon glyphicon-indent-left", text: "Asociar orden de compra", name: "LINK", smallText: "asociar comprobante con orden de compra" };

        this.menuItems.push(mnuView);
        this.menuItems.push(mnuLink);
        
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });

    }

    viewComprobante() {
        this.context.app.showSpinner();
        this.comprobanteService.getDetail(this.documentList.currentDocument.id)
            .then(comprobante => {
                this.currentComprobante = comprobante;
                this.context.app.hideSpinner();
                this.viewDialog.config.backdrop = false;
                this.viewDialog.show();
            })
            .catch(error => this.handleError(error));
    }

    onOrdenSelected(orden: OrdenItem) {
        //let item = this.manager.documentList.find(item => item.id == ingreso.id);
        //this.ingresosDialog.hide();
        this.context.app.showSpinner();
        if (orden != null) {
            this.comprobanteService.marcarDocumentoRelacionado(this.documentList.currentDocument.id, orden.id)
            .then( _ => {                
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
        }
    }


    //Ordenes de compra
    onConfigureDocument() {
        //this.refreshList();
        //this.applyPolicies();        
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
       
        //Si se especificó status, agregar filtro
        //if (this.idWorkflowStep && this.idWorkflowStep != 0) {
            let conditionWF: SearchCondition = { field: 'IdWorkflowStep', operator: '>', valueType: 'int', value: 0 };
            request.conditions.push(conditionWF);
        //}

        // agregar filtro proveedor
        let conditionProv: SearchCondition = { field: 'IdProveedor', operator: '=', valueType: 'int', value: this.portalContext.proveedor.identity };
        request.conditions.push(conditionProv);
               
        //Si se especificó que se solicitan solo los pendientes de saldo, agregar filtro        
        let conditionSaldo: SearchCondition = { field: 'Saldo', operator: '>', valueType: 'decimal', value: 0 };
        request.conditions.push(conditionSaldo);
                
        this.ordenService.search(0, request)
            .then(result => {
                this.ordenes = result.rows;
                //this.totalRows = result.totalRows; se habilita cuando se agregue el extends a DocumentListDialog
                //this.pageIndex = request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    /////////////////////// HANDLERS //////////////////////////////////

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.ordenService.generarPdfOrden(this.ordenesManager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + file.uri, '_blank');
                })
                .catch(error => this.handleError(error));
        }       
    }

    onMainToolbar(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.router.navigate(['./validacion'], { relativeTo: this.route });
        }
    }

    optionClick(option: MenuItem) {
        if (option.name == 'OPEN') {
            this.viewComprobante();
        }
        if (option.name == 'LINK') {
            this.ordenesList.show();
        }
    }

    onValidateMenu() {

    }

    currentItemChangedHandler() {

    }

    handleError(error: any) {        
        console.log(error);
        let message = this.context.getExceptionMessage(error);

    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

}