﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import * as wijmo from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, TrackableItem  } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';

//Components
import { ImpuestoAsignacionInfo } from '../../accounting/models/impuesto-asignacion';
import { ImpuestosAsignacionEditor } from '../../accounting/components/impuesto-asignacion.editor';

//Features
import { ArticuloService} from '../services/articulo.service';
import { ArticuloInfo, ArticuloItem } from '../models/articulo';

import { UnidadMedidaService } from '../../inventory/services/unidad-medida.service';
import { UnidadMedidaInfo, UnidadMedidaItem } from '../../inventory/models/unidad-medida';
import { UnidadMedidaTipoService } from '../../inventory/services/unidad-medida-tipo.service';
import { UnidadMedidaTipoItem } from '../../inventory/models/unidad-medida-tipo';
import { FormControl } from '@angular/forms/src/model';
import { ProductoProveedorComponent } from '../components/producto-proveedor.component';
import { ArticuloImpuestoComponent } from '../components/articulo-impuesto.component';



@Component({
    selector: 'compras-articulo-editor',
    templateUrl: './articulos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticulosEditor)
        },
        {
            provide: CatalogService, useExisting: ArticuloService
        }
    ]
})
export class ArticulosEditor extends CatalogEditor {

    articulo: ArticuloInfo = new ArticuloInfo();        
    tiposUnidad: UnidadMedidaTipoItem[] = [];
    unidadesMedida: UnidadMedidaItem[] = [];
    svcImpCollection: TrackableCollection = new TrackableCollection();
    proceso: number = 0;

    tabControl: string = "";

    formCtrlMetodoValuacion: FormControl;

    @ViewChild(ImpuestosAsignacionEditor) servicioImpuestosDialog: ImpuestosAsignacionEditor;
    @ViewChild(ProductoProveedorComponent) proveedorProducto: ProductoProveedorComponent;
    @ViewChild(ArticuloImpuestoComponent) articuloImpuestos: ArticuloImpuestoComponent;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private articuloService: ArticuloService, private unidadMedidaService: UnidadMedidaService, private tipoUnidadService: UnidadMedidaTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idCuenta': [''],
            'idFlujo': [''],
            'gtin': [''],
            'idTipoUnidad': [0, Validators.required],            
            'idUnidadMedida': [0, Validators.required],
            'manejaSeries': [''],
            'manejaLotes': [''],
            'manejaCaducidad': [''],
            'metodoValuacion': [''],
            'descripcion': [''],
            'impuestos': [[]]
        });

        this.formCtrlMetodoValuacion = <FormControl>this.catalogForm.controls['metodoValuacion'];


    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.proceso = +param['proceso'];
        });

        this.formCtrlMetodoValuacion.valueChanges.subscribe((value) => {
            this.articulo.articuloStock.metodoValuacion = value;
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

    loadTiposUnidad() {
        this.tipoUnidadService.loadList()
            .then(tipos => this.tiposUnidad = tipos)
            .catch(error => this.handleError(error));
    }

    onTipoUnidadChanged(idTipoUnidad) {
        this.loadUnidadesMedida(idTipoUnidad);
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        this.unidadMedidaService.loadList(idTipoUnidad)
            .then(unidades => this.unidadesMedida = unidades)
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadTiposUnidad();
    }

    onCreatingItem() {
        this.formCtrlMetodoValuacion.setValue(this.articulo.articuloStock.metodoValuacion);
        this.articulo.impuestos = [];
        this.svcImpCollection.sourceCollection = this.articulo.impuestos;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.articuloService.getDetailByID(id)
            .then(info => {
                this.articulo = info;  
                this.loadUnidadesMedida(this.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
                this.loadTiposUnidad();
                this.formCtrlMetodoValuacion.setValue(this.articulo.articuloStock.metodoValuacion);
                this.context.app.hideSpinner();
                this.setReadonlyMode();
                this.proveedorProducto.setReadOnly();
                this.articuloImpuestos.readOnly = true;
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.articuloService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.articulo = info;
                this.loadTiposUnidad();
                this.formCtrlMetodoValuacion.setValue(this.articulo.articuloStock.metodoValuacion);
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.articuloService.getDetailByID(id)
            .then(info => {
                this.articulo = info;
                this.loadUnidadesMedida(this.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
                this.loadTiposUnidad();
                this.formCtrlMetodoValuacion.setValue(this.articulo.articuloStock.metodoValuacion);
                this.context.app.hideSpinner();
                this.controlSeries();
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();  
        this.articuloService.update(this.articulo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.articulo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.articulo = info;
            })
            .catch(error => this.handleError(error));        
    }

    onAgregarProveedor() {
        this.router.navigate(['purchasing/proveedor-articulo', this.proceso, 0, 0, this.articulo.identity, 0, 1, 0, 1])
    }

    refreshControls(control: string) {
        if (this.tabControl != control) {
            this.tabControl = control;
            wijmo.Control.invalidateAll();
        }
    }

    /*------------------------------------------ IMPUESTOS ----------------------------------*/
    ngAfterViewChecked() {
        this.svcImpCollection.sourceCollection = this.articulo.impuestos;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.servicioImpuestosDialog.show();
        }
    }

    onAgregarImpuesto(args: ImpuestoAsignacionInfo) {
        let impuestoExists: boolean = false;
        this.articulo.impuestos.forEach(servicioImp => {
            if (servicioImp.idImpuesto == args.idImpuesto && servicioImp.state != 'DELETED') {
                this.showAlert("El impuesto ya fue agregado", "información");
                impuestoExists = true;
            }
        })
        if (!impuestoExists) {
            this.articulo.impuestos.push(args);
            this.svcImpCollection.refresh();
        }
    }

    onRemoveItem(item) {
        this.articulo.impuestos.forEach(x => {
            if (x.idImpuesto == item.idImpuesto)
                x.state = 'DELETED'
        })

        this.svcImpCollection.remove(item);
        this.svcImpCollection.refresh();
    }

    controlSeries() {
        if (this.articulo.articuloStock.manejaSeries) {
            this.catalogForm.controls['idTipoUnidad'].clearValidators();
            this.catalogForm.controls['idTipoUnidad'].updateValueAndValidity();
            //this.catalogForm.controls['idUnidadMedidaCompra'].clearValidators();
            //this.catalogForm.controls['idUnidadMedidaCompra'].updateValueAndValidity();
            this.catalogForm.controls['idUnidadMedida'].clearValidators();
            this.catalogForm.controls['idUnidadMedida'].updateValueAndValidity();
        }
        else {
            this.catalogForm.controls['idTipoUnidad'].validator = Validators.required;
            this.catalogForm.controls['idTipoUnidad'].updateValueAndValidity();
            //this.catalogForm.controls['idUnidadMedidaCompra'].validator = Validators.required;
            //this.catalogForm.controls['idUnidadMedidaCompra'].updateValueAndValidity();
            this.catalogForm.controls['idUnidadMedida'].validator = Validators.required;
            this.catalogForm.controls['idUnidadMedida'].updateValueAndValidity();
        }
    }

    manejaSeriesChecked() {
        this.controlSeries();    
    }
}