﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { CategoriasPresupuestalesService } from '../services/categorias-presupuestales.service';
import { CategoriasPresupuestalesInfo, Articulo, Servicio } from '../models/categorias-presupuestales';
import { ServicioListComponent } from '../dialogs/servicio-list.component';
import { ArticuloListComponent } from "../dialogs/articulo-list.component";
import { Control } from 'wijmo/wijmo';
import { ArticuloItem } from '../models/articulo';
import { ServicioItem } from '../models/servicio';

@Component({
    selector: 'compras-categorias-presupuestales-editor',
    templateUrl: './categorias-presupuestales.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CategoriasPresupuestalesEditor)
        },
        {
            provide: CatalogService, useExisting: CategoriasPresupuestalesService
        }
    ]
})
export class CategoriasPresupuestalesEditor extends CatalogEditor {
    categoriasPresupuestales: CategoriasPresupuestalesInfo = new CategoriasPresupuestalesInfo();
    Editing: boolean = true;

    articulosCollection: TrackableCollection = new TrackableCollection();
    serviciosCollection: TrackableCollection = new TrackableCollection();

    @ViewChild(forwardRef(() => ArticuloListComponent)) articuloDialog: ArticuloListComponent;
    @ViewChild(forwardRef(() => ServicioListComponent)) servicioDialog: ServicioListComponent;


    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private categoriasPresupuestalesService: CategoriasPresupuestalesService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idCuenta': ['', Validators.required],
        });
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();
        this.Editing = false;
        this.categoriasPresupuestalesService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.categoriasPresupuestales = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.categoriasPresupuestalesService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.categoriasPresupuestales = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();
        this.Editing = true;
        this.categoriasPresupuestalesService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.categoriasPresupuestales = info;

                this.articulosCollection.sourceCollection = this.categoriasPresupuestales.articulos;
                this.articulosCollection.refresh();

                this.serviciosCollection.sourceCollection = this.categoriasPresupuestales.servicios;
                this.serviciosCollection.refresh();

            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.categoriasPresupuestalesService.update(this.categoriasPresupuestales)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.categoriasPresupuestales.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }

                this.categoriasPresupuestales = info;

                this.articulosCollection.sourceCollection = this.categoriasPresupuestales.articulos;
                this.articulosCollection.refresh();

                this.serviciosCollection.sourceCollection = this.categoriasPresupuestales.servicios;
                this.serviciosCollection.refresh();

            })
            .catch(error => this.handleError(error));
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(args: ToolbarButton) {
        if (args.name == 'ADD-ARTICULO')
            this.onAgregarArticulo();
        else if (args.name == 'ADD-SERVICIO')
            this.onAgregarServicio();
    }

    onAgregarArticulo() {
        this.articuloDialog.show();
    }

    onAgregarServicio() {
        this.servicioDialog.show();
    }

    onSelectedArticuloChanged(articulos: ArticuloItem[]) {
        for (var articulo of articulos) {
            if (this.categoriasPresupuestales.articulos.filter(X => X.id == articulo.id).length == 0) {
                var articulosSlice = this.categoriasPresupuestales.articulos.slice();
                var articuloNew = new Articulo();
                articuloNew.id = articulo.id;
                articuloNew.codigo = articulo.codigo;
                articuloNew.nombre = articulo.nombre;
                articuloNew.activo = true;
                articulosSlice.push(articuloNew);

                this.categoriasPresupuestales.articulos = articulosSlice;
                this.articulosCollection.sourceCollection = articulosSlice;
                this.articulosCollection.refresh();
            }
        }
    }

    onSelectedServicioChanged(servicios: ServicioItem[]) {
        for (var servicio of servicios) {
            if (this.categoriasPresupuestales.servicios.filter(X => X.id == servicio.id).length == 0) {
                var serviciosSlice = this.categoriasPresupuestales.servicios.slice();
                var servicioNew = new Servicio();
                servicioNew.id = servicio.id;
                servicioNew.codigo = servicio.codigo;
                servicioNew.nombre = servicio.nombre;
                servicioNew.activo = true;
                serviciosSlice.push(servicioNew);

                this.categoriasPresupuestales.servicios = serviciosSlice;
                this.serviciosCollection.sourceCollection = serviciosSlice;
                this.serviciosCollection.refresh();
            }
        }
    }

    onRemoveArticulo(item) {
        this.articulosCollection.remove(item);
        this.articulosCollection.refresh();
    }

    onRemoveServicio(item) {
        this.serviciosCollection.remove(item);
        this.serviciosCollection.refresh();
    }

}