﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';

import { AreaItem } from '../../inventory/models/area';
import { AreaService } from '../../inventory/services/area.service';
import { AlmacenItem } from '../../inventory/models/almacen';

//Features
import { CentroRecepcionService} from '../services/centro-recepcion.service';
import { CentroRecepcionInfo, CentroRecepcionItem} from '../models/centro-recepcion';

@Component({
    selector: 'compras-centro-recepcion-editor',
    templateUrl: './centros-recepcion.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CentrosRecepcionEditor)
        },
        {
            provide: CatalogService, useExisting: CentroRecepcionService
        }
    ]
})
export class CentrosRecepcionEditor extends CatalogEditor {
    centroRecepcion: CentroRecepcionInfo = new CentroRecepcionInfo();        

    //idArea: number;
    areas: AreaItem[] = [];
    almacenes: AlmacenItem[] = [];

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private centroRecepcionService: CentroRecepcionService, private areaService: AreaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'area': ['', Validators.required],
            'almacen': ['', Validators.required],
            'idCuenta': [0],

            'domicilio': [''],
            'telefono1': [''],
            'telefono2': [''],
            'fax': [''],
            'correo': [''],
        });

        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        this.catalogForm.controls['area'].valueChanges.subscribe(value => {
            setTimeout(_ => this.selectArea(value));
        });
                
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadAreas();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.centroRecepcionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centroRecepcion = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.centroRecepcionService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centroRecepcion = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.centroRecepcionService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.centroRecepcion = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.centroRecepcionService.update(this.centroRecepcion)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.centroRecepcion.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.centroRecepcion = info;
            })
            .catch(error => this.handleError(error));        
    }

    onSelectTab() {
        //refresca los controles de wijmo
        Control.invalidateAll();
    }

    loadAreas() {
        this.areaService.loadList(this.context.app.defaultSite.identity)
            .then(areas => this.areas = areas)
            .catch(error => this.handleError(error));
    }

    selectArea(idArea) {
        this.areaService.loadAlmacenes(idArea)
            .then(almacenes => this.almacenes = almacenes)
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['members'], { relativeTo: this.route });
        }
    }

}