﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { DepartamentoService} from '../services/departamento.service';
import { DepartamentoInfo, DepartamentoItem} from '../models/departamento';
import { CajaService } from '../../treasury/services/caja.service';
import { CajaInfo, CajaItem } from '../../treasury/models/caja';


@Component({
    selector: 'compras-departamento-editor',
    templateUrl: './departamentos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DepartamentosEditor)
        },
        {
            provide: CatalogService, useExisting: DepartamentoService
        }
    ]
})
export class DepartamentosEditor extends CatalogEditor {
    departamento: DepartamentoInfo = new DepartamentoInfo();        
    tiposPresupuesto: { id: string, nombre: string }[] = [];
    cajas: CajaItem[] = [];

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private departamentoService: DepartamentoService, private cajaService: CajaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'tipoPresupuesto': ['', Validators.required],
            'idCaja': [0],
            'idCuenta': [0],
            'idCentroCosto': [0],
        });

        this.tiposPresupuesto =
            [
                { 'id': 'N', 'nombre': 'No Controlado' },
                //{ 'id': 'D', 'nombre': 'Nivel Departamento' },
                { 'id': 'C', 'nombre': 'Nivel Categoria' }
            ];            
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadCajas();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.departamentoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.departamento = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.departamentoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.departamento = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.departamentoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.departamento = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.departamentoService.update(this.departamento)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.departamento.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.departamento = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadCajas() {
        this.cajaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cajas => {
                this.cajas = cajas;
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['members'], { relativeTo: this.route });
        }
        else if (option.name == 'C.RECEP') {
            this.router.navigate(['centros-recepcion'], { relativeTo: this.route });
        }
        else if (option.name == 'C.COSTO') {
            this.router.navigate(['centros-costo'], { relativeTo: this.route });
        }
        else if (option.name == 'CAT.PRES') {
                this.router.navigate(['categorias-presupuestales'], { relativeTo: this.route });
            
        }
    }

}