﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { GastoTipoService} from '../services/gasto-tipo.service';
import { GastoTipoInfo, GastoTipoItem } from '../models/gasto-tipo';

import { OperacionTipoItem } from '../models/operacion-tipo'
import { OperacionTipoService } from '../services/operacion-tipo.service'

@Component({
    selector: 'compras-gasto-tipo-editor',
    templateUrl: './gastos-tipos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => GastosTiposEditor)
        },
        {
            provide: CatalogService, useExisting: GastoTipoService
        }
    ]
})
export class GastosTiposEditor extends CatalogEditor {
    gastoTipo: GastoTipoInfo = new GastoTipoInfo();        

    operacionesTipo: OperacionTipoItem[] = []; 


    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private gastoTipoService: GastoTipoService,
        private operacionTipoService: OperacionTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idOperacionTipo': [0, AztecaValidators.selected],
            'idCuenta': [0],
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadOperacionesTipo();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.gastoTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.gastoTipo = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.gastoTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.gastoTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.gastoTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.gastoTipo = info;                
            })
            .catch(error => this.handleError(error));
    }

    loadOperacionesTipo() {
        this.operacionTipoService.loadListByUser(this.context.app.defaultSite.identity)
            .then(operaciones => {
                this.operacionesTipo = operaciones;
            })
            .catch(error => this.handleError(error));
    }


    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.gastoTipoService.update(this.gastoTipo)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.gastoTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.gastoTipo = info;
            })
            .catch(error => this.handleError(error));        
    }

}