﻿import { CatalogEditor, EditorEventArgs } from "framework3/editors";
import { CatalogService, BaseComponent, Context, CatalogItem } from "framework3/core";
import { ProveedorTipoService } from "../services";
import { forwardRef, Component, ChangeDetectorRef } from "@angular/core";
import { Router, ActivatedRoute } from "@angular/router";
import { FormBuilder, Validators } from "@angular/forms";
import { CatalogOptions } from "framework3/custom";
import { ProveedorTipoInfo } from "../models/proveedor-tipo";
import { Location } from '@angular/common';

@Component({
    selector: 'proveedores-tipos-editor',
    templateUrl: './proveedor-tipo.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ProveedorTipoEditor)
        },
        {
            provide: CatalogService, useExisting: ProveedorTipoService  
        }
    ],
})
export class ProveedorTipoEditor extends CatalogEditor {
    proveedorTipo: ProveedorTipoInfo = new ProveedorTipoInfo();
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private proveedorTipoService: ProveedorTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idCuenta': ['']
        });
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.proveedorTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.proveedorTipo = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));

    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.proveedorTipoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.proveedorTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.proveedorTipoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.proveedorTipo = info;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.proveedorTipoService.update(this.proveedorTipo)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.proveedorTipo.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.proveedorTipo = info;
            })
            .catch(error => this.handleError(error));
    }

}