﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import * as wijmo from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ProveedorService} from '../services/proveedor.service';
import { ProveedorInfo, ProveedorItem } from '../models/proveedor';
import { ArticuloService } from '../services/articulo.service';
import { ProveedorTipoService } from '../services';
import { ProveedorTipoInfo } from '../models/proveedor-tipo';
import { ProveedorProductoComponent } from '../components/proveedor-producto.component';

@Component({
    selector: 'compras-proveedor-editor',
    templateUrl: './proveedores.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ProveedoresEditor)
        },
        {
            provide: CatalogService, useExisting: ProveedorService
        }
    ]
})
export class ProveedoresEditor extends CatalogEditor {
    proveedor: ProveedorInfo = new ProveedorInfo();
    tiposProveedor: ProveedorTipoInfo[] = [];
    proceso: number;
    idProveedor: number = 0;
    allowComprobante: boolean =false;
    allowEditDiasCredito: boolean = false;
    selectSiNo: { id: boolean, nombre: string }[] = [];


    @ViewChild('proveedorArticulos') proveedorArticulo: ProveedorProductoComponent;
    @ViewChild('proveedorServicios') proveedorServicio: ProveedorProductoComponent;
    
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private proveedorService: ProveedorService, private articuloSvc: ArticuloService, private proveedorTipoSvc: ProveedorTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            //Generales
            'tipo': [0],
            'telefono': [''],
            'celular': [''],
            'correo': ['', AztecaValidators.email],
            'paginaWeb': [''],
            'idCuenta':[0],
            //
            'domicilio': [''],
            //Pagos
            'limiteCredito': [''],
            'diasCredito': [''],
            'idBanco': [0],
            'cuentaBancaria': [''],
            'leyenda': [''],
            'comprobanteFiscal': [true],
            //Datos Fiscales
            'rfc': ['',Validators.required],
            'razonSocial': ['',Validators.required],
            'domicilioFiscal': [''],
            'parametro': ['']
        });

        this.selectSiNo =
            [
                { "id": true, "nombre": "Si" },
                { "id": false, "nombre": "No" }
            ];

    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.proceso = +param['proceso'];
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    applyPolicy(policy: PolicyInfo): void {      
        //si estan con el usuario, entonces las habilita. No se comprueba el campo Acceso.
        if (policy.nombre == 'allowComprobante')
            this.allowComprobante = true;//(policy.valor=='false'?false:true);                   
        else if (policy.nombre == 'allowEditDiasCredito')
            this.allowEditDiasCredito = true;//(policy.valor == 'false' ? false : true);                   
    }

    onCreatingItem() {
        this.loadProveedoresTipos();
        this.proveedor.comprobanteFiscal = true;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();
        this.proveedorService.getDetailByID(id)
            .then(info => {
                this.loadProveedoresTipos();
                this.context.app.hideSpinner();
                this.proveedor = info;                
                this.setReadonlyMode();
                this.proveedorArticulo.setReadOnly();
                this.proveedorServicio.setReadOnly();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.proveedorService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.proveedor = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();
        this.proveedorService.getDetailByID(id)
            .then(info => {
                this.loadProveedoresTipos();
                this.context.app.hideSpinner();
                this.proveedor = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {

        this.context.app.showSpinner();
        this.proveedorService.update(this.proveedor)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.proveedor.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.proveedor = info;
            })
            .catch(error => this.handleError(error));

    }


    onAgregarArticulo() {
        this.router.navigate(['purchasing/proveedor-articulo', this.proceso, 0, this.proveedor.identity, 0, 0, 0, 1, 1])
    }
    onAgregarServicio() {
        this.router.navigate(['purchasing/proveedor-servicio', this.proceso, 0, this.proveedor.identity, 0, 0, 0, 1, 1])        
    }

    refreshControls() {
        wijmo.Control.invalidateAll();
    }

    loadProveedoresTipos() {
        this.proveedorTipoSvc.loadList(this.context.app.defaultSite.identity)
            .then(info => {
                this.context.app.hideSpinner();
                this.tiposProveedor = info;
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'ARTICULOS') {
            this.onAgregarArticulo();
        }
        else if (item.name == 'SERVICIOS') {
            this.onAgregarServicio();
        }
        else if (item.name == 'CONTACTOS') {
            this.router.navigate(['purchasing/proveedores/contacts',this.process.id, this.proveedor.identity]);
        }
       
    }
}