﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';

//Features
import { ProveedorService} from '../services/proveedor.service';
import { ProveedorInfo, ProveedorItem} from '../models/proveedor';

@Component({    
    selector: 'compras-proveedores-manager',
    templateUrl: './proveedores.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ProveedoresManager)
        },
        {
            provide: CatalogService, useExisting: ProveedorService
        }
	]
})
export class ProveedoresManager extends CatalogManager {
    proveedores: ProveedorItem[] = [];    
    proveedor: CatalogItem;
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private proveedorService: ProveedorService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

    enviarRegistro() {
        this.proveedorService.enviarRegistro(this.manager.currentItem.id)
            .then(_ => {
                this.showAlert('Se ha enviado un correo electrónico con la invitación de registro', 'success');
            })
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.proveedorService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.proveedores = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.proveedorService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.proveedorService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onItemChange(item: CatalogItem) {
        this.proveedor = item;
    }

    onAgregarArticulo() {
        this.router.navigate(['purchasing/proveedor-articulo', this.process.id, 0, this.proveedor.id, 0, 0, 0, 1, 1])
    }
    onAgregarServicio() {
        this.router.navigate(['purchasing/proveedor-servicio', this.process.id, 0, this.proveedor.id, 0, 0, 0, 1, 1])
    }
    onAgregarContacto() {
        this.router.navigate(['purchasing/proveedores/contacts', this.process.id, this.proveedor.id]);
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'ARTICULOS') {
            this.onAgregarArticulo();
        }
        else if (option.name == 'SERVICIOS') {
            this.onAgregarServicio();
        }
        else if (option.name == 'CONTACTOS') {
            this.onAgregarContacto();
        }
        else if (option.name == 'REGISTRO') {
            this.enviarRegistro();
        }
    }

}
