﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, TrackableItem } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ServicioService} from '../services/servicio.service';
import { ServicioInfo, ServicioItem } from '../models/servicio';
import { ImpuestoAsignacionInfo } from '../../accounting/models/impuesto-asignacion';
import { ImpuestosAsignacionEditor } from '../../accounting/components/impuesto-asignacion.editor';
import { ProductoProveedorComponent } from '../components/producto-proveedor.component';
import { ArticuloImpuestoComponent } from '../components/articulo-impuesto.component';


@Component({
    selector: 'azteca.business.compras-servicio-editor',
    templateUrl: './servicios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ServiciosEditor)
        },
        {
            provide: CatalogService, useExisting: ServicioService
        }
    ]
})
export class ServiciosEditor extends CatalogEditor {
    servicio: ServicioInfo = new ServicioInfo();
    proceso: number = 0;
    svcImpCollection: TrackableCollection = new TrackableCollection();
    @ViewChild(ImpuestosAsignacionEditor) servicioImpuestosDialog: ImpuestosAsignacionEditor;
    @ViewChild(ProductoProveedorComponent) proveedorProducto: ProductoProveedorComponent;
    @ViewChild(ArticuloImpuestoComponent) articuloImpuestos: ArticuloImpuestoComponent;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private servicioService: ServicioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            //general
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'libre':[false],
            'idCuenta': [''],
            'impuestos': [null]
        });
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.proceso = +param['proceso'];
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    ngAfterViewChecked() {
        this.svcImpCollection.sourceCollection = this.servicio.impuestos;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.servicioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;                
                this.setReadonlyMode();
                this.proveedorProducto.setReadOnly();
                this.articuloImpuestos.readOnly = true;
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.servicioService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.servicioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.servicio = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.servicioService.update(this.servicio)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.servicio.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.servicio = info;
            })
            .catch(error => this.handleError(error));        
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }    


    /*------------------------------------------ PROVEEDORES ----------------------------------*/
    onAgregarProveedor() {
        this.router.navigate(['purchasing/proveedor-servicio', this.proceso, 0, 0, this.servicio.identity, 0, 1, 0, 1])
                                          
    }


}