﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import * as wijmo from 'wijmo/wijmo';
import * as wjcCore from 'wijmo/wijmo';
import { FlexGrid, HitTestInfo, CellRange, CellType, Column, CellRangeEventArgs } from 'wijmo/wijmo.grid';
import { FlexGridFilter, FilterType, ColumnFilter, Operator } from 'wijmo/wijmo.grid.filter';

//Azteca Kernel
import { Context, BaseComponent, CatalogService, UsuarioItem, CatalogItem} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, CatalogListComponent } from 'framework3/search';
//Features
import { ZonaCompraService} from '../services/zona-compra.service';
import { ZonaCompraInfo, ZonaCompraItem } from '../models/zona-compra';
import { ProveedorService } from '../services/proveedor.service';
import { UsuariosListComponent } from 'framework3/security/components/dialogs/usuarios-list.component';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
@Component({
    selector: 'compras-zona-compra-editor',
    templateUrl: './zonas-compra.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ZonasCompraEditor)
        },
        {
            provide: CatalogService, useExisting: ZonaCompraService
        }
    ],
})
export class ZonasCompraEditor extends CatalogEditor {
    zonaCompra: ZonaCompraInfo = new ZonaCompraInfo();        
    usuarios: UsuarioItem[];
    currentItem: CatalogItem;
    // Proveedores
    requestProveedor: SearchRequest;
    proveedores = [];
    pageSizeProveedor: number = 20;
    serverSideProveedor: boolean = false;
    totalRowsProveedor: number = 0;
    pageIndexProveedor: number = 0;

    agregarUsuarios: boolean = false;

    restricciones: { id: number, nombre: string }[] = [];


    @ViewChild(CatalogListComponent) catalogList: CatalogListComponent;
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild('dialog') dialog: ModalDirective;


    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private zonaCompraService: ZonaCompraService, private proveedorSvc: ProveedorService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idCuenta': [''],
            'restriccionCosto':[0]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        this.agregarUsuarios = false;
        this.loadRestriccionesCostos();
    }

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();
        this.loadRestriccionesCostos();
        this.zonaCompraService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.zonaCompra = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
       
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.zonaCompraService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.zonaCompra = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.agregarUsuarios = true;
        this.context.app.showSpinner();
        this.loadRestriccionesCostos();
        this.zonaCompraService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.zonaCompra = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.zonaCompraService.update(this.zonaCompra)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.zonaCompra.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.zonaCompra = info;
            })
            .catch(error => this.handleError(error));        
    }


    searchArticuloRequested(request: SearchRequest) {
        this.requestProveedor = request;
        this.refreshListProveedores();
    }


    refreshListProveedores() {
        if (this.zonaCompra.identity > 0) {
            this.context.app.showSpinner();
            if (this.requestProveedor == null) {
                this.requestProveedor = new SearchRequest();
                this.requestProveedor.pageIndex = this.pageIndexProveedor;
                this.requestProveedor.pageSize = this.pageSizeProveedor;
                this.requestProveedor.paged = this.serverSideProveedor;
            }
            this.proveedorSvc.searchProveedorByZona(this.zonaCompra.identity, this.context.app.defaultSite.identity, this.requestProveedor)
                .then(response => {
                    this.proveedores = response.rows;
                    this.totalRowsProveedor = response.totalRows;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    refreshListAccess() {
        if (this.zonaCompra.identity > 0) {
            this.context.app.showSpinner();
            this.zonaCompraService.loadAccessUsersList(this.zonaCompra.identity)
                .then(usuarios => {
                    this.usuarios = usuarios;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        else if (button.name == 'ADDPROVIDER') {
            this.show();
        }
    }

    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.context.app.showSpinner();
        this.zonaCompraService.addUser(this.zonaCompra.identity, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                //this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshListAccess();
            })
            .catch(error => this.handleError(error));

        console.log(JSON.stringify(this.usuarios));
    }

    onRemoveUser() {
        this.context.app.showSpinner()
        console.log(this.currentItem.id);
        this.zonaCompraService.removeUser(this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error))
    }

    show() {
        this.dialog.config.backdrop = "static";
        this.dialog.config.ignoreBackdropClick = true;
        this.dialog.config.animated = true;
        
        this.dialog.show();
    }

    loadRestriccionesCostos() {
        this.zonaCompraService.loadRestriccionesCostos()
            .then(info => {
                this.restricciones = info;
            })
            .catch(error => this.handleError(error));
    }
}