﻿//Frameworks
import { Input, Output, EventEmitter, Component, ViewChild, forwardRef, Renderer } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { FormBuilder, FormGroup, ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { Alert, AztecaValidators, AlertsBarComponent, MenuItem, BaseControl } from 'framework3/controls';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap'

// Wijmo
import * as wjcCore from 'wijmo/wijmo';
import { FlexGrid, CellRangeEventArgs, DataMap } from 'wijmo/wijmo.grid';

// Features
import { ImpuestoService } from '../../accounting/services/impuesto.service';
import { ImpuestoInfo } from '../../accounting/models/impuesto';
import { ArticuloImpuesto } from '../models/articulo';
import { ArticuloService } from '../services/articulo.service';
import { ImpuestoAsignacionInfo } from '../../accounting/models/impuesto-asignacion';
import { ImpuestosAsignacionEditor } from '../../accounting/components/impuesto-asignacion.editor';

@Component({
    selector: 'purchasing-articulo-impuesto',
    templateUrl: './articulo-impuesto.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticuloImpuestoComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => ArticuloImpuestoComponent), multi: true
        }
    ]
})
export class ArticuloImpuestoComponent extends BaseControl implements ControlValueAccessor {

    _Impuestos: Array<ImpuestoAsignacionInfo> = new Array<ImpuestoAsignacionInfo>();

    @Input() impuestos = [];
    @Output() impuestosChange: EventEmitter<Array<ImpuestoAsignacionInfo>> = new EventEmitter<Array<ImpuestoAsignacionInfo>>();

    dataArticuloImpuesto: wjcCore.CollectionView;
    tiposCalculo = [{ "tipo": "B", "nombre": "Base" }, { "tipo": "C", "nombre": "Cascada" }];
    tipoCalculoMap: DataMap;

    readOnly: boolean = false;

    @ViewChild(ImpuestosAsignacionEditor) servicioImpuestosDialog: ImpuestosAsignacionEditor;

    constructor(private renderer: Renderer) {
        super();
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.tipoCalculoMap = new DataMap(this.tiposCalculo, 'tipo', 'nombre');
            this.dataArticuloImpuesto = new wjcCore.CollectionView(this._Impuestos);
        });
    }

    // Handlers
    updateImpuestos() {
        this._Impuestos.length = 0;
        this.impuestos.forEach(item => {
            let impuesto = new ImpuestoAsignacionInfo();
            impuesto.idImpuesto = item.idImpuesto;
            impuesto.impuesto = item.impuesto;
            impuesto.base = item.base;
            impuesto.tasaOCuota = item.tasaOCuota;
            impuesto.tipoCalculo = item.tipoCalculo;
            impuesto.ordenCalculo = item.ordenCalculo;
            impuesto.tipoAplicacion = item.tipoAplicacion;
            impuesto.state = ChangeState.UNCHANGED;
            this._Impuestos.push(impuesto);
        });

        if(this.dataArticuloImpuesto!=undefined)
            this.dataArticuloImpuesto.refresh();
    }

    notifyChange() {
        let impuestos = new Array<ImpuestoAsignacionInfo>();

        this.impuestos.forEach(item => {
            if (item.state == ChangeState.UNCHANGED) {
                let impuesto = new ImpuestoAsignacionInfo();
                impuesto.idImpuesto = item.idImpuesto;
                impuesto.impuesto = item.impuesto;
                impuesto.base = item.base;
                impuesto.tasaOCuota = item.tasaOCuota;
                impuesto.tipoCalculo = item.tipoCalculo;
                impuesto.ordenCalculo = item.ordenCalculo;
                impuesto.tipoAplicacion = item.tipoAplicacion;
                impuesto.state = ChangeState.UNCHANGED;
                impuestos.push(impuesto);
            }
        });

        this.dataArticuloImpuesto.itemsAdded.forEach(item => {
            let impuesto = new ImpuestoAsignacionInfo();
            impuesto.idImpuesto = item.idImpuesto;
            impuesto.impuesto = item.impuesto;
            impuesto.base = item.base;
            impuesto.tasaOCuota = item.tasaOCuota;
            impuesto.tipoCalculo = item.tipoCalculo;
            impuesto.ordenCalculo = item.ordenCalculo;
            impuesto.tipoAplicacion = item.tipoAplicacion;
            impuesto.state = ChangeState.ADDED;
            impuestos.push(impuesto);
        });

        this.dataArticuloImpuesto.itemsEdited.forEach(item => {
            let impuesto = new ImpuestoAsignacionInfo();
            impuesto.idImpuesto = item.idImpuesto;
            impuesto.impuesto = item.impuesto;
            impuesto.base = item.base;
            impuesto.tasaOCuota = item.tasaOCuota;
            impuesto.tipoCalculo = item.tipoCalculo;
            impuesto.ordenCalculo = item.ordenCalculo;
            impuesto.tipoAplicacion = item.tipoAplicacion;
            impuesto.state = ChangeState.MODIFIED;
            impuestos.push(impuesto);
        });

        this.dataArticuloImpuesto.itemsRemoved.forEach(item => {
            let impuesto = new ImpuestoAsignacionInfo();
            impuesto.idImpuesto = item.idImpuesto;
            impuesto.impuesto = item.impuesto;
            impuesto.base = item.base;
            impuesto.tasaOCuota = item.tasaOCuota;
            impuesto.tipoCalculo = item.tipoCalculo;
            impuesto.ordenCalculo = item.ordenCalculo;
            impuesto.tipoAplicacion = item.tipoAplicacion;
            impuesto.state = ChangeState.DELETED;
            impuestos.push(impuesto);
        });

        this.impuestosChange.emit(impuestos);
        this.propagateChange(impuestos);
    }

    // Control Events
    onAgregar() {
        this.servicioImpuestosDialog.show();
    }

    onCellUpdated(args: CellRangeEventArgs) {
        switch (args.col) {
            case 4:
            case 5:
                if (this.dataArticuloImpuesto.currentItem != null) {
                    this.dataArticuloImpuesto.itemsEdited.push(this.dataArticuloImpuesto.currentItem);
                }
                this.notifyChange();
                break;
        }
    }

    onRowDeleteImpuesto(impuesto: ImpuestoAsignacionInfo) {
        this.dataArticuloImpuesto.remove(impuesto);
        this.dataArticuloImpuesto.itemsRemoved.push(impuesto);
        this.notifyChange();
    }

    onAgregarImpuesto(args: ImpuestoAsignacionInfo) {
        let impuestoExists: boolean = false;
        this._Impuestos.forEach(servicioImp => {
            if (servicioImp.idImpuesto == args.idImpuesto && servicioImp.state != 'DELETED') {
                impuestoExists = true;
            }
        })
        if (!impuestoExists) {
            this._Impuestos.push(args);
            this.dataArticuloImpuesto.itemsAdded.push(args);
            this.dataArticuloImpuesto.refresh();
            this.notifyChange();
        }
    }

    // Base control implementations
    applyPolicy(policy: PolicyInfo): void {
    }

    //Implementación de control value accesor
    writeValue(value: any) {
        this.impuestos = value ? value : [];
        this.updateImpuestos();
    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() { }

}