﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { ArticuloInfo, ArticuloItem } from '../models/articulo';
import { ArticuloService } from '../services/articulo.service';

@Component({
    selector: 'purchasing-articulo-selector',
    templateUrl: './articulo-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ArticuloSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ArticuloSelectorComponent),
        multi: true
    }]
})
export class ArticuloSelectorComponent extends CatalogSelector {

    articulos: any[];
    selectedItem: ArticuloInfo = new ArticuloInfo();


    @Output() articuloUpdated = new EventEmitter();
    @Input() articuloSelector: boolean = true;

    constructor(protected context: Context, private articuloService: ArticuloService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findArticulo(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.articuloService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + "-" + info.nombre;
                    this.articuloUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.articuloUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.articuloService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.articulos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findArticulo(keyword: string): Observable<CatalogItem[]> {
        return this.articuloService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

