﻿import { Component, Input, ViewChild, Output, EventEmitter, forwardRef } from "@angular/core";
import { BaseComponent, PolicyInfo, Context } from "framework3/core";
import { Router, ActivatedRoute } from "@angular/router";
import { SearchRequest } from "framework3/search";
import { AlertsBarComponent, MenuItem } from "framework3/controls";
//
import { ArticuloInfo } from "../models/articulo";
import { CategoriasPresupuestalesService } from "../services/categorias-presupuestales.service";
import { ArticuloListComponent } from "../dialogs/articulo-list.component";
import { transition } from "@angular/animations";
import { TrackableCollection } from "framework3/editors";


@Component({
    selector: 'categorias-presupuestales-articulos',
    templateUrl: './categorias-presupuestales-articulos.component.html'
})

export class CategoriasPresupuestalesArticulosComponent extends BaseComponent{
    
    articulos = [];
    articulosCollection: TrackableCollection = new TrackableCollection();
    articulosEliminados = [];
    enableAdd: boolean = true;

    @Input() pageSizeArticulo: number = 20;
    @Input() totalRowsArticulo: number = 0;
    @Input() pageIndexArticulo: number = 0;
    @Input() serverSideArticulo: boolean = false;
    @Input() Editing: boolean = true;
    @Input() set idCategoriaPresupuestal(IdCategoriaPresupuestal: number) {
        this._IdCategoriaPresupuestal = IdCategoriaPresupuestal;
        this.refreshListArticulios();
    }

    @Output() articulosSelected: EventEmitter<any[]> = new EventEmitter<any[]>();
    @Output() articulosDeleted: EventEmitter<any[]> = new EventEmitter<any[]>();
    private _IdCategoriaPresupuestal: number = 0;

    // Articulo
    requestArticulo: SearchRequest;
    currentItemArticulo: any;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(forwardRef(() =>ArticuloListComponent)) ArticuloDialog: ArticuloListComponent;

    constructor(private context: Context,  protected router: Router, protected route: ActivatedRoute,
        private categoriaPresupuestalsvc: CategoriasPresupuestalesService) {
        super();
    }

    applyPolicy(policy: PolicyInfo): void {

    }

    refreshListArticulios() {
        if (this._IdCategoriaPresupuestal > 0) {
            this.enableAdd = false;
            this.context.app.showSpinner();
            if (this.requestArticulo == null) {
                this.requestArticulo = new SearchRequest();
                this.requestArticulo.pageIndex = this.pageIndexArticulo;
                this.requestArticulo.pageSize = this.pageSizeArticulo;
                this.requestArticulo.paged = this.serverSideArticulo;
            }
            /*this.categoriaPresupuestalsvc.searchArticuloCategoriaPresupuestal(this._IdCategoriaPresupuestal, this.context.app.defaultSite.identity)
                .then(response => {
                    this.articulos = response.rows;
                    this.articulosCollection.sourceCollection = this.articulos;
                    this.totalRowsArticulo = response.totalRows;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));*/
        }
        else {
            this.enableAdd = true;
        }
    }

    onCurrentItemArticuloChanged(args: any) {
        this.currentItemArticulo = args;
    }

    searchArticuloRequested(request: SearchRequest) {
        this.requestArticulo = request;
        this.refreshListArticulios();
    }

    optionArticuloClick(option: MenuItem) {
        if (option.name == "DELETE") {
            let newArticulos=this.articulos.slice()
            newArticulos.splice(newArticulos.findIndex(K => K.id == this.currentItemArticulo.id), 1);
            this.articulosEliminados.push(this.currentItemArticulo);
            this.articulos = newArticulos;
            this.articulosDeleted.emit(this.articulosEliminados);
            this.articulosSelected.emit(this.articulos);
        }
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }


    onSelectedArticuloChanged(args: any) {
        if (args.length > 0) {
            let newArticulos =this.articulos.slice()
            for (var i = 0; i < args.length; i++) {
                if (newArticulos.findIndex(K => K.id == args[i].id)==-1) {
                    newArticulos.push(args[i]);
                }
            }
            this.articulos = newArticulos.slice();
            this.articulosSelected.emit(this.articulos);
        }
    }

    onAgregarArticulo() {
        this.ArticuloDialog.show();
    }
}