﻿import { Component, Input, ViewChild, Output, EventEmitter, forwardRef } from "@angular/core";
import { BaseComponent, PolicyInfo, Context } from "framework3/core";

import { ServicioListComponent } from '../dialogs/servicio-list.component'
import { Router, ActivatedRoute } from "@angular/router";
import { SearchRequest } from "framework3/search";
import { AlertsBarComponent, MenuItem } from "framework3/controls";
import { ServicioItem, ServicioInfo } from '../models/servicio';
import { CategoriasPresupuestalesService, } from "../services/categorias-presupuestales.service";

@Component({
    selector: 'categorias-presupuestales-servicios',
    templateUrl: './categorias-presupuestales-servicios.component.html'
})

export class CategoriasPresupuestalesServiciosComponent extends BaseComponent{
    
    servicios = [];
    serviciosEliminados = [];
    enableAdd: boolean = true;

    @Input() pageSizeServicio: number = 20;
    @Input() totalRowsArticulo: number = 0;
    @Input() pageIndexServicio: number = 0;
    @Input() serverSideServicio: boolean = false;
    @Input() Editing: boolean = true;
    @Input() set idCategoriaPresupuestal(IdCategoriaPresupuestal: number) {
        this._IdCategoriaPresupuestal = IdCategoriaPresupuestal;
        this.refreshListServicios();
    }

    @Output() servicioSelected: EventEmitter<any[]> = new EventEmitter<any[]>();
    @Output() servicioDeleted: EventEmitter<any[]> = new EventEmitter<any[]>();
    private _IdCategoriaPresupuestal: number = 0;

    // Articulo
    requestServicio: SearchRequest;
    currentItemServicio: any;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(forwardRef(() =>ServicioListComponent)) ServicioDialog: ServicioListComponent;

    constructor(private context: Context,  protected router: Router, protected route: ActivatedRoute,
        private categoriaPresupuestalsvc: CategoriasPresupuestalesService) {
        super();
    }

    applyPolicy(policy: PolicyInfo): void {

    }

    refreshListServicios() {
        if (this._IdCategoriaPresupuestal > 0) {
            this.enableAdd = false;
            this.context.app.showSpinner();
            if (this.requestServicio == null) {
                this.requestServicio = new SearchRequest();
                this.requestServicio.pageIndex = this.pageIndexServicio;
                this.requestServicio.pageSize = this.pageSizeServicio;
                this.requestServicio.paged = this.serverSideServicio;
            }
            this.categoriaPresupuestalsvc.searchServicioCategoriaPresupuestal(this._IdCategoriaPresupuestal, this.context.app.defaultSite.identity, this.requestServicio)
                .then(response => {
                    this.servicios = response.rows;
                    this.totalRowsArticulo = response.totalRows;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.enableAdd = true;
        }
    }

    onCurrentItemServicioChanged(args: any) {
        this.currentItemServicio = args;
    }

    searchServicioRequested(request: SearchRequest) {
        this.requestServicio = request;
        this.refreshListServicios();
    }

    optionServicioClick(option: MenuItem) {
        if (option.name == "DELETE") {
            let newServicios = this.servicios.slice();
            newServicios.splice(newServicios.findIndex(K => K.id == this.currentItemServicio.id),1);
            this.serviciosEliminados.push(this.currentItemServicio);
            this.servicios = newServicios;
            this.servicioDeleted.emit(this.serviciosEliminados);
            this.servicioSelected.emit(this.servicios);            
        }
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }


    onSelectedServicioChanged(args: any) {
        if (args.length > 0) {
            let newServicios = this.servicios.slice()
            for (var i = 0; i < args.length; i++) {
                if (newServicios.findIndex(K => K.id == args[i].id) == -1) {
                    newServicios.push(args[i]);
                }
            }
            this.servicios = newServicios.slice();
            this.servicioSelected.emit(this.servicios);
        }
    }

    onAgregarServicio() {
        this.ServicioDialog.show();
    }
}