﻿import { Component, EventEmitter, Input, Output, ViewChild, forwardRef } from '@angular/core';
//
//import { Context, SucursalItem, SucursalService } from '../../../core';
import { Context } from 'framework3/core';
import { DocumentListDialog, SearchRequest, SearchCondition } from 'framework3/search';

import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service'
import { ComprobanteFiscalItem } from '../models/comprobante-fiscal'
//

@Component({
    selector: 'purchasing-comprobantes-list',
    templateUrl: './comprobantes-dialog.component.html'
})

//cambiar el extends
export class ComprobantesListComponent extends DocumentListDialog {
    comprobantes: ComprobanteFiscalItem[];
    //Inputs    
    @Input() idProveedor: number = 0;
    @Input() idComprobanteTipo: number = 0;
    @Input() Activos: boolean = false;
    @Input() sinReferencia: boolean = false;

    pageIndex: number = 0;
    pageSize: number = 15;
    startPeriod: number = 15;

    constructor(protected context: Context, private comprobanteFiscalService: ComprobanteFiscalService) {
        super(context);
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        //Si se especificó status, agregar filtro
        if (this.idProveedor != 0) {
            let condition: SearchCondition = { field: 'IdProveedor', operator: '=', valueType: 'int', value: this.idProveedor };
            request.conditions.push(condition);
        }
        //Si se especificó el tipo de comprobante, agregar filtro
        if (this.idComprobanteTipo != 0) {
            let condition: SearchCondition = { field: 'IdComprobanteTipo', operator: '=', valueType: 'int', value: this.idComprobanteTipo };
            request.conditions.push(condition);
        }

        //Si se especificó status, agregar filtro
        if (this.Activos) {
            let condition: SearchCondition = { field: 'Cancelado', operator: '=', valueType: 'bool', value: 0 };
            request.conditions.push(condition);
        }

        //Si solo se quieren los que no tienen referencia
        if (this.sinReferencia) {
            let condition: SearchCondition = { field: 'IdRecepcionTit', operator: '=', valueType: 'string', value: '' };
            request.conditions.push(condition);
        }

        this.comprobanteFiscalService.search(this.context.app.defaultSite.identity, request)
            .then(result => {
                this.comprobantes = result.rows;
                //this.totalRows = result.totalRows; //se habilita cuando se agregue el extends a DocumentListDialog
                this.pageIndex = request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedItemsChanged(selecteItems: ComprobanteFiscalItem[]) {
        this.selectedItemsChanged.emit(selecteItems);
    }

    onSelectedItemChanged(item: ComprobanteFiscalItem) {
        if (item != null) {
            this.selectedItemChanged.emit(item);
        }
    }

    //se borra cuando se agregue el extends a DocumentListDialog
    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
    }


}