﻿//Frameworks
import { Input, Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { Alert, AztecaValidators, AlertsBarComponent, MenuItem } from 'framework3/controls';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap'
//Features
import { ArticuloService } from '../services/articulo.service';
import { ArticuloInfo, ArticuloItem } from '../models/articulo';
import { ProveedorInfo } from '../models/proveedor';
import { ProveedorService } from '../services/proveedor.service';
import { ZonaCompraService } from '../services';

@Component({
    selector: 'purchasing-producto-proveedor',
    templateUrl: './producto-proveedor.component.html'
})
export class ProductoProveedorComponent extends BaseComponent {

    form: FormGroup;
    proceso: number = 0;
    //
    proveedores = [];

    // Proveedores by articulo
    private _IdArticulo: number;
    private _IdServicio: number;

    @Input() set idArticulo(idArticulo: number) {
        this._IdArticulo = idArticulo;
        this.refreshListByArticulio();
    }

    @Input() set idServicio(idServicio: number) {
        this._IdServicio = idServicio;
        this.refreshListByServicio();
    }

    @Input() searchElement: boolean = true;
    @Input() searchByArticulo: boolean = true;
    @Input() searchByServicio: boolean = true;
    @Input() showTitle: boolean = true;

    //
    menuItemsProveedor: MenuItem[] = [];
    requestProveedor: SearchRequest;
    currentProveedor: any;
    pageSizeProveedor: number = 20;
    totalRowsProveedor: number = 0;
    pageIndexProveedor: number = 0;
    serverSideProveedor: boolean = false;;

    //
    @ViewChild('dlgConfirmacion') dlgConfirmacion: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    confirmacionText: string = "";
    
    constructor(private context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        private articuloSvc: ArticuloService, private proveedorSvc: ProveedorService, private zonaCompraSvc: ZonaCompraService) {
        super();
        this.form = formBuilder.group({
            'idArticulo': ['']
        });
    }

    applyPolicy(policy: PolicyInfo): void {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.proceso = +param['proceso'];
        });
    }

    ngAfterViewInit() {
        var thisRoot = this;
        setTimeout(_ => {
            if (this.searchByArticulo) {
                this.refreshListByArticulio();
            }
            else {
                this.refreshListByServicio();
            }
        });
        this.createMenuProveedor();
    }

    createMenuProveedor() {
        let mnuEditar: MenuItem = { icon: "glyphicon glyphicon-pencil", text: "Editar", name: "EDIT", smallText: "Abrir en modo edición" };
        let mnuEliminar: MenuItem = { icon: "glyphicon glyphicon-trash", text: "Eliminar", name: "DELETE", smallText: "Eliminar permanentemente" };
        this.menuItemsProveedor.push(mnuEditar, mnuEliminar);
        this.menuItemsProveedor.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    ////////////////////////// Articulo //////////////////////////

    onSelectedArticulo(args: ArticuloInfo) {
        this._IdArticulo = args.identity;
        this.refreshListByArticulio();
    }

    onCurrentItemProveedorChanged(item: any) {
        this.currentProveedor = item;
    }

    searchProveedorRequested(request: SearchRequest) {
        this.requestProveedor = request;
    }

    optionProveedorClick(option: MenuItem) {
        if (option.name == "EDIT") {
            if (this.searchByArticulo) {
                this.context.app.showSpinner();
                this.zonaCompraSvc.hasAccess(this.context.session.user.identity, this.currentProveedor.data.idzonaCompra)
                    .then(access => {
                        this.context.app.hideSpinner();
                        if (access)
                            this.router.navigate(['purchasing/proveedor-articulo', this.proceso, this.currentProveedor.data.idProveedorArticulo, 0, 0, 0, 0, 0, 0]);
                        else
                            this.alerts.showAlert('No tienes acceso a esa zona de compra', 'danger');
                    }).catch(error => this.handleError(error));
               
            }
            else
                this.router.navigate(['purchasing/proveedor-servicio', this.proceso, this.currentProveedor.data.idProveedorServicio, 0, 0, 0, 0, 0, 0]);
        }
        else if (option.name == "DELETE") {
            this.confirmacionText = "¿Está seguro que desea elmininar el proveedor " + this.currentProveedor.data.nombre + "?";            
            this.dlgConfirmacion.config.backdrop = false;
            this.dlgConfirmacion.show();
        }
    }

    refreshListByArticulio() {
        if (this._IdArticulo > 0) {
            this.context.app.showSpinner();
            if (this.requestProveedor == null) {
                this.requestProveedor = new SearchRequest();
                this.requestProveedor.pageIndex = this.pageIndexProveedor;
                this.requestProveedor.pageSize = this.pageSizeProveedor;
                this.requestProveedor.paged = this.serverSideProveedor ;
            }
            this.proveedorSvc.searchProveedorByArticulo(this._IdArticulo, this.context.app.defaultSite.identity, this.requestProveedor)
                .then(response => {
                    this.proveedores = response.rows;
                    this.totalRowsProveedor = response.totalRows;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    refreshListByServicio() {
        if (this._IdServicio > 0) {
            this.context.app.showSpinner();
            if (this.requestProveedor == null) {
                this.requestProveedor = new SearchRequest();
                this.requestProveedor.pageIndex = this.pageIndexProveedor;
                this.requestProveedor.pageSize = this.pageSizeProveedor;
                this.requestProveedor.paged = this.serverSideProveedor;
            }
            this.proveedorSvc.searchProveedorByServicio(this._IdServicio, this.context.app.defaultSite.identity, this.requestProveedor)
                .then(response => {
                    this.proveedores = response.rows;
                    this.totalRowsProveedor = response.totalRows;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    onAceptarConfirmacion() {
        this.context.app.showSpinner();

        if (this.searchByArticulo)
            this.proveedorSvc.deleteProveedorArticulo(this.currentProveedor.data.idProveedorArticulo)
                .then(result => {
                    this.context.app.hideSpinner();
                    this.refreshListByArticulio();                    
                })
                .catch(error => this.handleError(error));
        else
            this.proveedorSvc.deleteProveedorServicio(this.currentProveedor.data.idProveedorServicio)
                .then(result => {
                    this.context.app.hideSpinner();
                    this.refreshListByServicio();
                })
                .catch(error => this.handleError(error));

        this.dlgConfirmacion.hide();
    }

    setReadOnly() {
        this.menuItemsProveedor = [];
    }

};


