﻿import { Input, Component, ViewChild, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Validators } from '@angular/forms';
//
import { BaseComponent, Context, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { ToolbarButton, ToolbarButtonDirective, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//
import { ProveedorArticulo } from '../models/proveedor';
import { ArticuloService } from '../services/articulo.service';
import { ArticuloInfo, ArticuloItem } from '../models/articulo';

import { UnidadMedidaService } from '../../inventory/services/unidad-medida.service';
import { UnidadMedidaInfo, UnidadMedidaItem } from '../../inventory/models/unidad-medida';
import { UnidadMedidaTipoService } from '../../inventory/services/unidad-medida-tipo.service';
import { UnidadMedidaTipoItem } from '../../inventory/models/unidad-medida-tipo';

import { DivisasSelectorComponent } from '../../accounting/controls/divisas-selector.component';

import { ArticuloImpuestoComponent } from './articulo-impuesto.component';
import { ProveedorService } from '../services/proveedor.service';


@Component({
    selector: 'purchasing-proveedor-articulo',
    templateUrl: './proveedor-articulo.component.html'
})
export class ProveedorArtioculoComponent extends BaseComponent {
    
    form: FormGroup;
    //
    proveedorArticulo: ProveedorArticulo = new ProveedorArticulo();
    //articulo: ArticuloInfo = new ArticuloInfo();
    tiposUnidad: UnidadMedidaTipoItem[] = [];
    unidadesMedida: UnidadMedidaItem[] = [];

    //
    @Input() editProveedor: boolean = true;
    @Input() editArticulo: boolean = true;
    @Input() editZona: boolean = true;
    @Input() standAlone: boolean = true;

    //
    @Input() idProveedorArticulo: number = 0;
    @Input() idProveedor: number = 0;
    @Input() idArticulo: number = 0;
    @Input() idZona: number = 0;

    //
    @Output() savedCost: EventEmitter<ProveedorArticulo> = new EventEmitter<ProveedorArticulo>();
    @Output() cancelCost: EventEmitter<Boolean> = new EventEmitter<Boolean>();
    //
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //@ViewChild('ctrlArticuloImpuestos') ctrlArticuloImpuestos: ArticuloImpuestoComponent;
    @ViewChild('selDivisa') selDivisa: DivisasSelectorComponent;

    constructor(private context: Context, private formBuilder: FormBuilder, private route: ActivatedRoute, private location: Location,
        private proveedorSvc: ProveedorService, private articuloSvc: ArticuloService, private unidadMedidaService: UnidadMedidaService, private tipoUnidadService: UnidadMedidaTipoService) {
        super();
        this.form = formBuilder.group({
            'idProveedor': [''],
            'idZonaCompra': [''],
            'idArticulo': [],
            'costo': [''],
            'diasCredito': [''],
            'diasEntrega': [''],
            'porcAnticipo': [''],
            'idDivisa': [''],
            'idTipoUnidad': [''],
            'idUnidadMedida': [''],
            'porcDescuento1': [''],
            'porcDescuento2': [''],
            'porcDescuento3': [''],
            'porcDescuento4': [''],
            'porcDescuento5': [''],
            'predeterminado': ['']
        });   
    }

    ngOnInit() {
        if (this.route.params) {
            this.route.params.forEach(param => {
                this.idProveedorArticulo = +param['id'];
                this.idProveedor = +param['idProveedor'];
                this.idArticulo = +param['idArticulo'];
                this.idZona = +param['idZona'];
                this.editProveedor = +param['editProveedor'] == 1;
                this.editArticulo = +param['editArticulo'] == 1;
                this.editZona = +param['editZona'] == 1;
            });
        }
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.loadTiposUnidad();
            this.loadInitialData();
        });
    }

    loadInitialData() {
        
        if (this.idProveedorArticulo > 0) {
            this.context.app.showSpinner();
            this.proveedorSvc.getDetailProveedorArticulo(this.idProveedorArticulo)
                .then(result => {
                    this.context.app.hideSpinner();
                    this.proveedorArticulo = result;
                    this.selDivisa.loadItem(this.proveedorArticulo.idDivisa);
                    this.loadUnidadesMedida(this.proveedorArticulo.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.proveedorArticulo.idProveedor = this.idProveedor;
            this.proveedorArticulo.idArticulo = this.idArticulo;
            this.proveedorArticulo.idZonaCompra = this.idZona;
                       
        }
    }

    applyPolicy(policy: PolicyInfo): void {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    loadTiposUnidad() {
        this.tipoUnidadService.loadList()
            .then(tipos => this.tiposUnidad = tipos)
            .catch(error => this.handleError(error));
    }

    articuloChanged(articulo: ArticuloInfo) {
        if (articulo) {
            this.proveedorArticulo.articulo = articulo;
            this.loadUnidadesMedida(articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
        }
    }

    onTipoUnidadChanged(idTipoUnidad) {
        this.loadUnidadesMedida(idTipoUnidad);
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        this.unidadMedidaService.loadList(idTipoUnidad)
            .then(unidades => {
                this.unidadesMedida = unidades;
            })
            .catch(error => this.handleError(error));
    }

    mainToolbarHandler(button: ToolbarButtonDirective) {
        if (button.name == "SAVE") {
            if (this.proveedorArticulo.costo > 0) {
                this.context.app.showSpinner();    
                this.proveedorArticulo.idDivisa = this.selDivisa.selectedItem.identity;
                // Guardar proveedor articulo
                this.proveedorSvc.updateProveedorArticulo(this.proveedorArticulo)
                    .then(result => {
                        this.context.app.hideSpinner();
                        if (result) {
                            if (this.standAlone)
                                this.location.back();
                            else
                                this.savedCost.emit(this.proveedorArticulo);
                        }
                    })
                    .catch(error => {
                        this.context.app.hideSpinner();
                        this.alerts.showAlert(error.error.message, 'danger');
                    });
            }
            else
                this.alerts.showAlert("El costo no puede ser igual o menor a 0", 'danger');
        }
        else if (button.name == "CLOSE") {  
            if(this.standAlone)
                this.location.back();
            else
                this.cancelCost.emit(true);
                
        }
    }
        

}