﻿//Frameworks
import { Input, Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { Alert, AztecaValidators, AlertsBarComponent, MenuItem } from 'framework3/controls';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap'
//Features
import { ArticuloService } from '../services/articulo.service';
import { ArticuloInfo, ArticuloItem } from '../models/articulo';
import { ProveedorInfo } from '../models/proveedor';
import { ProveedorService } from '../services/proveedor.service';

import { ServicioService } from '../services/servicio.service';
import { ServicioInfo, ServicioItem } from '../models/servicio';
import { ZonaCompraService } from '../services';

@Component({
    selector: 'purchasing-proveedor-producto',
    templateUrl: './proveedor-producto.component.html'
})
export class ProveedorProductoComponent extends BaseComponent {

    form: FormGroup;
    proceso: number = 0;
    //
    articulos = [];
    servicios = [];

    // Articulo
    requestArticulo: SearchRequest;      requestServicio: SearchRequest;
    currentItemArticulo: any;            currentItemServicio: any;
    menuItemsArticulo: MenuItem[] = [];  menuItemsServicio: MenuItem[] = [];

    @Input() showTitle: boolean = true;
    @Input() pageSizeArticulo: number = 20;
    @Input() totalRowsArticulo: number = 0;
    @Input() pageIndexArticulo: number = 0;
    @Input() serverSideArticulo: boolean = false;
    //
    @ViewChild('dlgConfirmacion') dlgConfirmacion: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    //
    private _IdProveedor: number = 0;

    //
    confirmacionText: string = "";
    onAceptarConfimracion: any;


    @Input() set idProveedor(idProveedor: number) {
        this._IdProveedor = idProveedor;
        this.refreshListArticulios();
        this.refreshListServicios();
    }
    @Input() selectProveedor: boolean = true;
    @Input() showArticulo: boolean = true;
    @Input() showServicio: boolean = true;

    constructor(private context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        private articuloSvc: ArticuloService, private proveedorSvc: ProveedorService, private servicioSvc: ServicioService, private zonaCompraSvc:ZonaCompraService) {
        super();
        this.form = formBuilder.group({
            'idProveedor': ['']
        });
    }

    applyPolicy(policy: PolicyInfo): void {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.proceso = +param['proceso'];
        });
    }

    ngAfterViewInit() {
        var thisRoot = this;
        setTimeout(_ => {
            if(this.showArticulo) this.refreshListArticulios();
            if(this.showServicio) this.refreshListServicios();
        });
        if (this.showArticulo) this.createMenuArticulo();
        if (this.showServicio) this.createMenuServicio();
    }

    createMenuArticulo() {        
        let mnuEditar: MenuItem = { icon: "glyphicon glyphicon-pencil", text: "Editar", name: "EDIT", smallText: "Abrir en modo edición" };
        let mnuEliminar: MenuItem = { icon: "glyphicon glyphicon-trash", text: "Eliminar", name: "DELETE", smallText: "Eliminar permanentemente" };
        this.menuItemsArticulo.push(mnuEditar, mnuEliminar);
        this.menuItemsArticulo.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    createMenuServicio() {
        let mnuEditar: MenuItem = { icon: "glyphicon glyphicon-pencil", text: "Editar", name: "EDIT", smallText: "Abrir en modo edición" };
        let mnuEliminar: MenuItem = { icon: "glyphicon glyphicon-trash", text: "Eliminar", name: "DELETE", smallText: "Eliminar permanentemente" };
        this.menuItemsServicio.push(mnuEditar, mnuEliminar);
        this.menuItemsServicio.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }
    /////////////////////////////

    onSelectedProveedor(args: ProveedorInfo) {
        this._IdProveedor = args.identity;
        if (this.showArticulo) this.refreshListArticulios();
        if (this.showServicio) this.refreshListServicios();
    }


    onCurrentItemArticuloChanged(args: any) {
        this.currentItemArticulo = args;
    }

    onCurrentItemServicioChanged(args: any) {
        this.currentItemServicio = args;
    }

    optionArticuloClick(option: MenuItem) {
        if (option.name == "EDIT") {
            this.context.app.showSpinner();
            this.zonaCompraSvc.hasAccess(this.context.session.user.identity, this.currentItemArticulo.data.idzonaCompra)
                .then(access => {
                    this.context.app.hideSpinner();
                    if (access)
                        this.router.navigate(['purchasing/proveedor-articulo', this.proceso, this.currentItemArticulo.data.idProveedorArticulo, 0, 0, 0, 0, 0, 0]);
                    else
                        this.alerts.showAlert('No tienes acceso a esa zona de compra', 'danger');
                }).catch(error => this.handleError(error));
            
        }
        else if (option.name == "DELETE") {
            this.confirmacionText = "¿Está seguro que desea elmininar el artículo " + this.currentItemArticulo.data.nombre + "?";
            this.onAceptarConfimracion = () => {
                this.context.app.showSpinner();
                this.proveedorSvc.deleteProveedorArticulo(this.currentItemArticulo.data.idProveedorArticulo)
                    .then(result => {
                        this.context.app.hideSpinner();
                        this.refreshListArticulios();
                    })
                    .catch(error => this.handleError(error));
            };
            this.dlgConfirmacion.config.backdrop = false;
            this.dlgConfirmacion.show();
        }
    }

    optionServicioClick(option: MenuItem) {
        if (option.name == "EDIT") {
            this.router.navigate(['purchasing/proveedor-servicio', this.proceso, this.currentItemServicio.data.idProveedorServicio, 0, 0, 0, 0, 0, 0]);
        }
        else if (option.name == "DELETE") {
            this.confirmacionText = "¿Está seguro que desea elmininar el servicio " + this.currentItemServicio.data.nombre + "?";
            this.onAceptarConfimracion = () => {
                this.context.app.showSpinner();
                this.proveedorSvc.deleteProveedorServicio(this.currentItemServicio.data.idProveedorServicio)
                    .then(result => {
                        this.context.app.hideSpinner();
                        this.refreshListServicios();
                    })
                    .catch(error => this.handleError(error));
            };
            this.dlgConfirmacion.config.backdrop = false;
            this.dlgConfirmacion.show();
        }
    }

    searchArticuloRequested(request: SearchRequest) {
        this.requestArticulo = request;
        this.refreshListArticulios();
    }

    searchServicioRequested(request: SearchRequest) {
        this.requestServicio = request;
        this.refreshListServicios();
    }

    refreshListArticulios() {
        if (this._IdProveedor > 0) {
            this.context.app.showSpinner();
            if (this.requestArticulo == null) {
                this.requestArticulo = new SearchRequest();
                this.requestArticulo.pageIndex = this.pageIndexArticulo;
                this.requestArticulo.pageSize = this.pageSizeArticulo;
                this.requestArticulo.paged = this.serverSideArticulo;
            }
            this.articuloSvc.searchArticuloProveedor(this._IdProveedor, this.context.app.defaultSite.identity, this.requestArticulo)
                .then(response => {
                    this.articulos = response.rows;
                    this.totalRowsArticulo = response.totalRows;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    refreshListServicios() {
        if (this._IdProveedor > 0) {
            this.context.app.showSpinner();
            if (this.requestServicio == null) {
                this.requestServicio = new SearchRequest();
                this.requestServicio.pageIndex = this.pageIndexArticulo;
                this.requestServicio.pageSize = this.pageSizeArticulo;
                this.requestServicio.paged = this.serverSideArticulo;
            }
            this.servicioSvc.searchServicioProveedor(this._IdProveedor, this.context.app.defaultSite.identity, this.requestServicio)
                .then(response => {
                    this.servicios = response.rows;
                    this.totalRowsArticulo = response.totalRows;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }
    onCancelarConfirmacion() {
        this.dlgConfirmacion.hide();
    }

    setReadOnly() {
        this.menuItemsArticulo = [];
        this.menuItemsServicio = [];
    }
}
