﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { ProveedorInfo, ProveedorItem } from '../models/proveedor';
import { ProveedorService } from '../services/proveedor.service';

@Component({
    selector: 'purchasing-proveedor-selector',
    templateUrl: './proveedor-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ProveedorSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ProveedorSelectorComponent),
        multi: true
    }]
})
export class ProveedorSelectorComponent extends CatalogSelector {
    proveedores: any[];
    selectedItem:ProveedorInfo = new ProveedorInfo;


    @Output() proveedorUpdated = new EventEmitter();
    @Input() proveedorSelector: boolean = true;


    constructor(protected context: Context, private proveedorService: ProveedorService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findProveedor(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.proveedorService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + "-" + info.nombre;
                    this.proveedorUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.proveedorUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.proveedorService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.proveedores = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findProveedor(keyword: string): Observable<CatalogItem[]> {
        return this.proveedorService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.clear();
        this.markUnselected();
    }

}

