﻿import { Input, EventEmitter, Component, ViewChild, Output } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Validators } from '@angular/forms';
//
import { BaseComponent, Context, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { ToolbarButton, ToolbarButtonDirective, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//
import { ProveedorServicio} from '../models/proveedor';
import { ServicioService } from '../services/servicio.service';
import { ServicioInfo, ServicioItem } from '../models/servicio';

import { ProveedorService } from '../services/proveedor.service';
import { DivisaInfo } from '../../accounting';


@Component({
    selector: 'purchasing-proveedor-servicio',
    templateUrl: './proveedor-servicio.component.html'
})
export class ProveedorServicioComponent extends BaseComponent {

    form: FormGroup;
    //
    proveedorServicio: ProveedorServicio = new ProveedorServicio();
    servicio: ServicioInfo = new ServicioInfo();
    //tiposUnidad: UnidadMedidaTipoItem[] = [];
    //unidadesMedida: UnidadMedidaItem[] = [];

    //
    @Input() showTitle: boolean = true;
    @Input() editProveedor: boolean = true;
    @Input() editServicio: boolean = true;
    @Input() editZona: boolean = true;
    @Input() standAlone: boolean = true;
    //
    @Input() idProveedorServicio: number = 0;
    @Input() idProveedor: number = 0;
    @Input() idServicio: number = 0;
    @Input() idZona: number = 0;

    //
    @Output() savedCost: EventEmitter<ProveedorServicio> = new EventEmitter<ProveedorServicio>();
    @Output() cancelCost: EventEmitter<Boolean> = new EventEmitter<Boolean>();
    //
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //@ViewChild('ctrlArticuloImpuestos') ctrlArticuloImpuestos: ArticuloImpuestoComponent;

    constructor(private context: Context, private formBuilder: FormBuilder, private route: ActivatedRoute, private location: Location,
        private proveedorSvc: ProveedorService, private servicioSvc: ServicioService) {
        super();
        this.form = formBuilder.group({
            'idProveedor': [''],
            'idZonaCompra': [''],
            'idServicio': [],
            'costo': [''],
            'diasCredito': [''],
            'diasEntrega': [''],
            'porcAnticipo': [''],
            'idDivisa': [''],
            'porcDescuento1': [''],
            'porcDescuento2': [''],
            'porcDescuento3': [''],
            'porcDescuento4': [''],
            'porcDescuento5': [''],
            'predeterminado': ['']
        });
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idProveedorServicio = +param['id'];
            this.idProveedor = +param['idProveedor'];
            this.idServicio = +param['idServicio'];
            this.idZona = +param['idZona'];
            this.editProveedor = +param['editProveedor'] == 1;
            this.editServicio = +param['editServicio'] == 1;
            this.editZona = +param['editZona'] == 1;
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadInitialData());
    }

    loadInitialData() {
        //this.loadTiposUnidad();
        if (this.idProveedorServicio > 0) {
            this.context.app.showSpinner();
            this.proveedorSvc.getDetailProveedorServicio(this.idProveedorServicio)
                .then(result => {
                    this.context.app.hideSpinner();
                    this.proveedorServicio = result;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.proveedorServicio.idProveedor = this.idProveedor;
            this.proveedorServicio.idServicio = this.idServicio;
            this.proveedorServicio.idZonaCompra = this.idZona;
        }
    }

    applyPolicy(policy: PolicyInfo): void {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    //loadTiposUnidad() {
    //    this.tipoUnidadService.loadList()
    //        .then(tipos => this.tiposUnidad = tipos)
    //        .catch(error => this.handleError(error));
    //}

    //onTipoUnidadChanged(idTipoUnidad) {
    //    this.loadUnidadesMedida(idTipoUnidad);
    //}

    //loadUnidadesMedida(idTipoUnidad: number) {
    //    this.unidadMedidaService.loadList(idTipoUnidad)
    //        .then(unidades => this.unidadesMedida = unidades)
    //        .catch(error => this.handleError(error));
    //}

    mainToolbarHandler(button: ToolbarButtonDirective) {
        if (button.name == "SAVE") {
            this.context.app.showSpinner();
            // Guardar impuestos
            //this.articulo.articulosImpuestos = this.ctrlArticuloImpuestos.getItems();
            //this.articuloSvc.updateArticuloImpuestos(this.articulo.articulosImpuestos)
            //    .then(resultImpuestos => {
            //        if (resultImpuestos) {

            //            // Guardar proveedor articulo
            if (this.proveedorServicio.porcAnticipo >= 0 && this.proveedorServicio.porcAnticipo <= 100) {
                this.proveedorSvc.updateProveedorServicio(this.proveedorServicio)
                    .then(result => {
                        this.context.app.hideSpinner();
                        if (result) {
                            if (this.standAlone)
                                this.location.back();
                            else
                                this.savedCost.emit(this.proveedorServicio);
                        }
                    })
                    .catch(error => {
                        console.log(error);
                        this.context.app.hideSpinner();
                        this.alerts.showAlert(error.error.message, 'danger');
                    });
            }
            else
                this.handleError("El anticipo debe de ser entre 0 - 100");
        //            }
        //        })
        //        .catch(error => this.handleError(error));
        }
        else if (button.name == "CLOSE") {
            if (this.standAlone)
                this.location.back();
            else
                this.cancelCost.emit(true);
        }
    }
    onDivisaSelected(divisa:DivisaInfo) {
        this.proveedorServicio.idDivisa = divisa.identity;
    }

}