﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
//
import { ZonaCompraInfo, ZonaCompraItem } from '../models/zona-compra';
import { ZonaCompraService } from '../services/zona-compra.service';

@Component({
    selector: 'purchasing-zona-compra-selector',
    templateUrl: './zona-compra-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ZonaCompraSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ZonaCompraSelectorComponent),
        multi: true
    }]
})
export class ZonaCompraSelectorComponent extends CatalogSelector {
    zonaCompras: any[];
    selectedItem: ZonaCompraInfo = new ZonaCompraInfo;


    @Output() zonaCompraUpdated = new EventEmitter();
    @Input() zonaCompraSelector: boolean = true;

    constructor(protected context: Context, private zonaCompraService: ZonaCompraService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findZonaCompra(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.zonaCompraService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + "-" + info.nombre;
                    this.zonaCompraUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.zonaCompraUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();
        this.zonaCompraService.search2(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.zonaCompras = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findZonaCompra(keyword: string): Observable<CatalogItem[]> {
        return this.zonaCompraService.searchKeyword2(this.context.session.user.identity, this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

