﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index';
//

import { CentroCostoService, CategoriasPresupuestalesService } from '../services';
import { CategoriasPresupuestalesInfo } from '..';
@Component({
    selector: 'purchasing-categorias-presupuestales-selector',
    templateUrl: './categorias-presupuestales-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => CategoriaPresupuestalSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => CategoriaPresupuestalSelectorComponent),
        multi: true
    }]
})
export class CategoriaPresupuestalSelectorComponent extends CatalogSelector {


    categoriasPresupuestales: any[];
    selectedItem: CategoriasPresupuestalesInfo = null;

    @Output() itemSelected: EventEmitter<CategoriasPresupuestalesInfo> = new EventEmitter<CategoriasPresupuestalesInfo>();
    @Input() enabled: boolean = true;

    constructor(protected context: Context, private service: CategoriasPresupuestalesService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findGrupo(keyword)));
    }

    loadItem(itemID: number): void {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.itemSelected.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    findGrupo(keyword: string): Observable<CatalogItem[]> {
        return this.service.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.categoriasPresupuestales = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }
}