﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index';
//
import {   DepartamentoService } from '../services/departamento.service';
import { DepartamentoInfo } from '../models/departamento';
@Component({
    selector: 'purchasing-departamentos-selector',
    templateUrl: './departamentos-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => DepartamentosSelectorComponent )
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => DepartamentosSelectorComponent ),
        multi: true
    }]
})
export class DepartamentosSelectorComponent extends CatalogSelector {


    departamentos: any[];
    selectedItem: DepartamentoInfo = null;

    @Output() selectedItemChanged: EventEmitter<DepartamentoInfo> = new EventEmitter<DepartamentoInfo>();
    @Input() enabled: boolean = true;

    constructor(protected context: Context, private service: DepartamentoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findGrupo(keyword)));
    }

    loadItem(itemID: number): void {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    findGrupo(keyword: string): Observable<CatalogItem[]> {
        return this.service.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.departamentos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }
}