﻿import { Component, EventEmitter, Input, Output, ViewChild, forwardRef } from '@angular/core';
//
//import { Context, SucursalItem, SucursalService } from '../../../core';
import { Context } from 'framework3/core';
import { DocumentListDialog, SearchRequest, SearchCondition } from 'framework3/search';

import { OrdenService } from '../services/orden.service'
import { OrdenItem } from '../models/orden'
//

@Component({
    selector: 'purchasing-ordenes-list',
    templateUrl: './ordenes-list.component.html'
})

export class OrdenesListComponent extends DocumentListDialog {
    ordenes: OrdenItem[];
    //Inputs    
    @Input() idProveedor: number = 0;
    @Input() idCentroRecepcion: number = 0;
    @Input() idWorkflowStep: number = 0;
    @Input() saldoPendiente: boolean = false;
    @Input() startPeriod: number;

    constructor(protected context: Context, private ordenService: OrdenService) {
        super(context);
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        ////agregar filtro de empresa
        //let condEmpresa: SearchCondition = { field: 'idSucursal', operator: '=', valueType: 'int', value: this.context.app.defaultSite.identity.toString() };
        //request.conditions.push(condEmpresa);

        //Si se especificó status, agregar filtro
        if (this.idWorkflowStep && this.idWorkflowStep != 0) {
            let condition: SearchCondition = { field: 'IdWorkflowStep', operator: '=', valueType: 'int', value: this.idWorkflowStep };
            request.conditions.push(condition);
        }
        //Si se especificó status, agregar filtro
        if (this.idProveedor && this.idProveedor != 0) {
            let condition: SearchCondition = { field: 'IdProveedor', operator: '=', valueType: 'int', value: this.idProveedor };
            request.conditions.push(condition);
        }
        //Si se especificó centro de recepcion, agregar filtro
        if (this.idCentroRecepcion && this.idCentroRecepcion != 0) {
            let condition: SearchCondition = { field: 'IdCentroRecepcion', operator: '=', valueType: 'int', value: this.idCentroRecepcion };
            request.conditions.push(condition);
        }
        //Si se especificó que se solicitan solo los pendientes de saldo, agregar filtro
        if (this.saldoPendiente && this.saldoPendiente) {
            let condition: SearchCondition = { field: 'Saldo', operator: '>', valueType: 'decimal', value: 0 };
            request.conditions.push(condition);
        }


        this.ordenService.search(this.context.app.defaultSite.identity, request)
            .then(result => {
                this.ordenes = result.rows;
                //this.totalRows = result.totalRows; se habilita cuando se agregue el extends a DocumentListDialog
                this.pageIndex = request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedItemsChanged(selecteItems: OrdenItem[]) {
        this.selectedItemsChanged.emit(selecteItems);
    }

    onSelectedItemChanged(item: OrdenItem) {
        if (item != null) {
            this.selectedItemChanged.emit(item);
        }
    }

    //se borra cuando se agregue el extends a DocumentListDialog
    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
    }


}