﻿import { Component, EventEmitter, Input, Output, ViewChild, forwardRef } from '@angular/core';
//
import { FlexGrid, HitTestInfo, CellRange, CellType, Column, CellRangeEventArgs } from 'wijmo/wijmo.grid';
import { FlexGridFilter, FilterType, ColumnFilter, Operator } from 'wijmo/wijmo.grid.filter';
import { Menu } from 'wijmo/wijmo.input';
import * as wijmo from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';

import { CollectionView, EventArgs, DataType } from 'wijmo/wijmo';
import { ModalDirective } from 'ngx-bootstrap'

//
import { AlertsBarComponent } from 'framework3/controls';
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest } from 'framework3/search';
import { ProveedorService } from '../services/proveedor.service';
import { ProveedorServicioItem, ProveedorServicio } from '../models/proveedor'
//
import { ProveedorServicioComponent } from '../components/proveedor-servicio.component'
//

@Component({
    selector: 'purchasing-proveedores-servicio-list',
    templateUrl: './proveedores-servicio-list.component.html'
})
export class ProveedoresServicioListComponent extends BaseComponent {

    proveedores: ProveedorServicioItem[];
    idServicio: number;

    currentItem: ProveedorServicioItem = null;
    @Output() selectedItem: EventEmitter<ProveedorServicioItem> = new EventEmitter<ProveedorServicioItem>();

    collectionView: CollectionView = new CollectionView();

    //
    @ViewChild('listDialog') listDialog: ModalDirective;
    @ViewChild('grid') grid: FlexGrid;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('cambioPrecioDialog') costoDialog: ModalDirective;
    @ViewChild('proveedorCostoComponent') proveedorCostoComponent: ProveedorServicioComponent;


    constructor(protected context: Context, private proveedorService: ProveedorService) {
        super();
    }

    applyPolicy() {

    }

    ngAfterViewInit() {
        // this.setFilterType();
        this.currentItem = new ProveedorServicioItem();
        this.triggerSearch();
    }
    //private setFilterType() {
    //    if (this._serverSide) {
    //        this.filter.defaultFilterType = FilterType.Condition;
    //        this._currentFilter = this.filter.filterDefinition;
    //    }
    //    else {
    //        this.filter.defaultFilterType = FilterType.Both;
    //        this._currentFilter = this._emptyFilter;
    //    }
    //    this.setDefaultFilterOperator();
    //}

    initializeGrid(sender: wjcGrid.FlexGrid) {
        var self = this;

        //Manejo de doble click
        sender.addEventListener(sender.hostElement, "dblclick", (e: any) => {
            let ht: HitTestInfo = sender.hitTest(e);
            if (ht.cellType == CellType.Cell) {
                this.selectedItem.emit(this.currentItem);
            }
        });

        this.collectionView.currentChanged.addHandler((args: EventArgs) => {
            this.currentItem = self.collectionView.currentItem;
        }, self);

        //Establecer el operador "comienza con" para todas cadenas
        //this.setDefaultFilterOperator();
        //        
    }

    onSearchRequested(request: SearchRequest) {
        if (this.idServicio != 0 && this.idServicio != undefined) {
            this.proveedorService.searchProveedorByServicio(this.idServicio, this.context.app.defaultSite.identity, request)
                .then(result => {
                    this.collectionView.sourceCollection = <ProveedorServicioItem[]>result.rows;
                    //this.totalRows = result.totalRows;
                    //this.pageIndex = request.pageIndex;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    triggerSearch() {
        let request: SearchRequest = new SearchRequest();
        request.pageIndex = 0;
        request.pageSize = 20;//this.pageSize;
        request.paged = false;//this.serverSide ? true : false;
        //
        this.onSearchRequested(request);//this.searchRequested.emit(request);
    }

    //onSelectedItemsChanged(selectedItems: any[]) {
    //    this.selectedItemsChanged.emit(selectedItems);
    //}

    //onSelectedItemChanged(item: any) {
    //    this.selectedItemChanged.emit(item);
    //}

    //setDefaultFilterOperator() {
    //    for (let i: number = 0; i < this.grid.columns.length; i++) {
    //        let col: Column = this.grid.columns[i];
    //        let colFilter: ColumnFilter = this.filter.getColumnFilter(col);
    //        if (colFilter != null) {
    //            if (col.dataType == undefined || col.dataType == DataType.String) {
    //                colFilter.conditionFilter.condition1.operator = Operator.BW;
    //            }
    //        }
    //    }
    //}

    show() {
        this.currentItem = null;
        this.listDialog.config.backdrop = false;
        this.triggerSearch();
        this.listDialog.show();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    isValid(): boolean {
        return this.selectedItem != null;
    }

    onChangeCost(proveedor: any) {
        this.costoDialog.config.backdrop = false;
        this.proveedorCostoComponent.idServicio = this.idServicio;
        this.proveedorCostoComponent.idProveedor = this.currentItem.idProveedor;
        this.proveedorCostoComponent.idProveedorServicio = this.currentItem.idProveedorServicio;
        this.proveedorCostoComponent.standAlone = false;
        this.proveedorCostoComponent.loadInitialData();
        this.costoDialog.show();
    }

    onSavedCost(item: ProveedorServicio) {
        this.currentItem = <ProveedorServicioItem>item;
        this.selectedItem.emit(this.currentItem);
        this.costoDialog.hide();
        this.listDialog.hide();
        this.grid.refresh();
    }

    onCancelCost(item: boolean) {
        this.costoDialog.hide();
    }

    private onConfirmClick() {
        this.selectedItem.emit(this.currentItem);
        this.listDialog.hide();
    }

    private onCancelClick() {
        this.listDialog.hide();
    }

    refreshControls() {
        wijmo.Control.invalidateAll();
    }

}