﻿import { Component, EventEmitter, Input, Output, ViewChild, forwardRef } from '@angular/core';
//
//import { Context, SucursalItem, SucursalService } from '../../../core';
import { Context, BaseComponent } from 'framework3/core';
import { DocumentListDialog, SearchRequest, SearchCondition } from 'framework3/search';


import { RecepcionItem } from '../models/recepcion';
import { RecepcionService } from '../services';
//

@Component({
    selector: 'purchasing-recepciones-list',
    templateUrl: './recepciones-list.component.html'
})

export class RecepcionesListComponent extends DocumentListDialog {
    recepciones: RecepcionItem[];

    @Input() idWorkflowStep: number = 0;
    @Input() idBeneficiario: number = 0;
    @Input() idDivisa: number = 0;
    @Input() idProveedor: number = 0;
    constructor(protected context: Context, private recepcionService: RecepcionService) {
        super(context);
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        ////agregar filtro de empresa
        //let condEmpresa: SearchCondition = { field: 'idSucursal', operator: '=', valueType: 'int', value: this.context.app.defaultSite.identity.toString() };
        //request.conditions.push(condEmpresa);

        //Si se especificó status, agregar filtro
        if (this.idWorkflowStep && this.idWorkflowStep != 0) {
            let condition: SearchCondition = { field: 'IdWorkflowStep', operator: '=', valueType: 'int', value: this.idWorkflowStep };
            request.conditions.push(condition);
        }

        //Si se especificó Beneficiario, agregar filtro
        if (this.idBeneficiario != 0) {
            let condition: SearchCondition = { field: 'IdBeneficiario', operator: '=', valueType: 'int', value: this.idBeneficiario };
            request.conditions.push(condition);
        }

        //Si se especificó divisa, agregar filtro
        if (this.idDivisa != 0) {
            let condition: SearchCondition = { field: 'IdDivisa', operator: '=', valueType: 'int', value: this.idDivisa };
            request.conditions.push(condition);
        }

        //Si se especificó Beneficiario, agregar filtro
        if (this.idProveedor != 0) {
            let condition: SearchCondition = { field: 'IdProveedor', operator: '=', valueType: 'int', value: this.idBeneficiario };
            request.conditions.push(condition);
        }

        this.recepcionService.search(this.context.app.defaultSite.identity, request)
            .then(result => {
                this.recepciones = result.rows;
                //this.totalRows = result.totalRows; se habilita cuando se agregue el extends a DocumentListDialog
                this.pageIndex = request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedItemsChanged(selecteItems: RecepcionItem[]) {
        let elements = [];
        selecteItems.forEach(item => {
            this.recepciones.forEach(itemCuentaPorPagar => {
                if (itemCuentaPorPagar.id == item.id) {
                    elements.push(itemCuentaPorPagar);
                }
            });
        });
        this.selectedItemsChanged.emit(elements);
    }

    onSelectedItemChanged(item: RecepcionItem) {
        if (item != null) {
            this.recepciones.forEach(itemCuentaPorPagar => {
                if (itemCuentaPorPagar.id == item.id) {
                    this.selectedItemChanged.emit(itemCuentaPorPagar);
                }
            });
        }
    }

    //se borra cuando se agregue el extends a DocumentListDialog
    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
    }


}