﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { AdelantoInfo, AdelantoItem } from '../models/adelanto';
import { AdelantoDetail } from '../models/adelanto-detail';
import { AdelantoService } from '../services/adelanto.service';
import { OrdenService } from '../services/orden.service';
import { OrdenesListComponent } from '../dialogs/ordenes-list.component';
import { OrdenItem } from '../models/orden';

@Component({
    selector: 'compras-adelantos-manager',
    templateUrl: './adelantos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => AdelantosManager)
        },
        {
            provide: DocumentService, useExisting: AdelantoService
        }
    ]
})
export class AdelantosManager extends DocumentManager {
    adelantos: AdelantoItem[];
    idProveedor: number = 0;
    //
    @ViewChild('ordenesList') ordenesList: OrdenesListComponent;
    @ViewChild('proveedorDialog') proveedorDialog: ModalDirective;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private adelantoService: AdelantoService, private ordenService: OrdenService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.adelantoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.adelantos = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.idProveedor = 0;
        this.proveedorDialog.config.backdrop = false;
        this.proveedorDialog.show();   
        //this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        //this.router.navigate([document.id], { relativeTo: this.route });
        this.router.navigate([document.id, '0'], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.adelantoService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.adelantoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.adelantoService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }
       
    /////////////////////////////
    onProveedorSelected() {       
            this.proveedorDialog.hide();
            //
            //this.ordenesList.config.backdrop = false;
            this.ordenesList.show();       
    }

    onOrdenSelected(orden: OrdenItem) {
        //let item = this.manager.documentList.find(item => item.id == ingreso.id);
        //this.ingresosDialog.hide();

        //let OrdenFull = <OrdenItem>this.manager.documentList.find(item => item.id == orden.id)
        this.ordenService.getDetail(orden.id)
            .then(ordenFull => {
                if (ordenFull.idAnticipo && ordenFull.idAnticipo.trim() != '')
                    this.showAlert('La orden seleccionada ya tiene relacionado un adelanto previo con folio[' + ordenFull.idAnticipo + '].', 'warning');
                else
                    this.router.navigate(['0', orden.id], { relativeTo: this.route });

            })
            .catch(error => this.handleError(error));
    }

    cerrarProveedor() {
        this.proveedorDialog.hide();
    }

}

