﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { CostoPresupuestoInfo, CostoPresupuestoItem } from '../models/costo-presupuesto';
import { CostoPresupuestoDetail } from '../models/costo-presupuesto-detail';
import { CostoPresupuestoService } from '../services/costo-presupuesto.service';
import { CostoPresupuestoDetailEditor } from './parts/costos-presupuestos-detail.editor';

import { DivisaService } from '../../accounting/services/divisa.service';
import { DivisaInfo } from '../../accounting/models/divisa';
import { EmpresaService } from '../../accounting/services/empresa.service';
import { EmpresaInfo } from '../../accounting/models/empresa';

import { BeneficiarioSelectorComponent } from '../../treasury/controls/beneficiario-selector.component'

import { CuentasPorPagarListComponent } from '../../treasury/dialogs/cuenta-por-pagar-list.component';
import { CuentaPorPagarItem } from '../../treasury/models/cuenta-por-pagar'

import { ServicioItem, ServicioInfo } from '../models/servicio';
import { ServicioService } from '../services/servicio.service'

import { ServicioListComponent } from '../dialogs/servicio-list.component';

import {  ChangeState } from 'framework3/editors';

@Component({
    selector: 'compras-costos-presupuestos-editor',
    templateUrl: './costos-presupuestos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CostosPresupuestosEditor)
        },
        {
            provide: DocumentService, useExisting: CostoPresupuestoService
        }
    ]
})
export class CostosPresupuestosEditor extends DocumentEditor {
    costoPresupuesto = new CostoPresupuestoInfo();
    idDivisaDefault: number = 0;
    tipoCambio: string;
    currentDetailCopy = new CostoPresupuestoDetail();
    idServicio: number = 0;

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: CostoPresupuestoDetailEditor;
    @ViewChild('selectorBeneficiario') selectorBeneficiario: BeneficiarioSelectorComponent;
    @ViewChild('dlgCuentasPorPagar') dlgCuentasPorPagar: CuentasPorPagarListComponent;
    @ViewChild('dlgServicio') dlgServicio: ServicioListComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private costoPresupuestoService: CostoPresupuestoService,
        private divisaService: DivisaService,
        private empresaService: EmpresaService,
        private servicioService: ServicioService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idBeneficiario': [0, Validators.required],
            'idDivisa': [0, Validators.required],
            'idServicio': [0],
            'observaciones': ['', Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {
        this.applyPolicies();
        this.getDivisaDefault(false);
    }

    onCreatingDocument() {
        this.route.params.forEach(params => {
            if (params['idPresupuesto']) {
                this.costoPresupuesto.idPresupuesto = params['idPresupuesto'];
            }
            if (params['idDepartamento']) {
                this.costoPresupuesto.idDepartamento = params['idDepartamento'];
            }
            if (params['idBeneficiario']) {
                this.costoPresupuesto.idBeneficiario = params['idBeneficiario'];
            }

        });
        this.getDivisaDefault(true);
        this.loadNewDetalle(this.costoPresupuesto.idPresupuesto);

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.costoPresupuestoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.costoPresupuesto = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.costoPresupuestoService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.costoPresupuesto = doc;
                this.detailsEditor.clientIndexStart = this.costoPresupuesto.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.costoPresupuestoService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.costoPresupuesto = doc;
                this.detailsEditor.clientIndexStart = this.costoPresupuesto.details.length;
                if (this.costoPresupuesto.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();

        this.costoPresupuesto.tipoBeneficiario = this.selectorBeneficiario.selectedItem.tipoBeneficiario;

        this.costoPresupuestoService.update(this.costoPresupuesto)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.costoPresupuesto.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.costoPresupuesto = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.costoPresupuestoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.costoPresupuestoService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.costoPresupuesto.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onMenuItemClick(args: any) {
        if (args.name == 'SERVICIO') {
            var detail: CostoPresupuestoDetail;
            detail = new CostoPresupuestoDetail();

            detail = <CostoPresupuestoDetail>this.detailsList.currentItem;

            this.currentDetailCopy = detail;

            this.dlgServicio.show()
        }
    }

    onAddingDetail() {
    }

    onEditingDetail(detail: CostoPresupuestoDetail) {

        this.currentDetailCopy = detail;
        this.dlgCuentasPorPagar.show();


    }

    onDeletingDetail(detail: CostoPresupuestoDetail) {
        detail.idCuentaPorPagar = '';
        detail.importe = 0;
        detail.state = ChangeState.MODIFIED;
        Control.invalidateAll();
    }

    onListItemsUpdated(updatedDetails: CostoPresupuestoDetail[]) {
        this.costoPresupuesto.details = updatedDetails;
    }

    onSavingDetail(detail: CostoPresupuestoDetail) {
        this.detailsList.updateItem(detail);
    }

    getDivisaDefault(creating: boolean) {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.idDivisaDefault = r.identity;
                if (creating) { this.costoPresupuesto.idDivisa = r.identity; }

            })
            .catch(error => this.handleError(error));

    }


    onDivisaSelected(args: any) {
        if (this.costoPresupuesto.idDivisa == this.idDivisaDefault) {
            this.costoPresupuesto.tipoCambio = 1;
            this.tipoCambio = this.costoPresupuesto.tipoCambio.toString();
        }
        else {
            this.divisaService.loadTiposCambio(this.context.app.defaultCompany.identity)
                .then(tipos => {
                    let tCambio = tipos.find(search => search.idDivisaOrigen == this.costoPresupuesto.idDivisa && search.idDivisaDestino == this.idDivisaDefault);
                    if (tCambio == undefined) {
                        this.showAlert("No está definido el tipo de cambio para la divisa seleccionada", "danger");
                        this.tipoCambio = null;
                        this.costoPresupuesto.idDivisa = null;
                        this.costoPresupuesto.tipoCambio = 0;
                    }
                    else {
                        this.empresaService.getDetailFromCache(this.context.app.defaultCompany.identity)
                            .then(emp => {
                                if (emp.tipoCambio == 'O') {
                                    this.costoPresupuesto.tipoCambio = tCambio.precioOtro;
                                    this.tipoCambio = tCambio.precioOtro.toString();
                                }
                                else if (emp.tipoCambio == 'V') {
                                    this.costoPresupuesto.tipoCambio = tCambio.precioVenta;
                                    this.tipoCambio = tCambio.precioVenta.toString();
                                }
                                else if (emp.tipoCambio == 'C') {
                                    this.costoPresupuesto.tipoCambio = tCambio.precioCompra;
                                    this.tipoCambio = tCambio.precioCompra.toString();
                                }
                            })
                    }
                });
        }
    }

    loadNewDetalle(idPresupuesto: string) {
        this.costoPresupuestoService.loadDetalleNew(idPresupuesto)
            .then(det => {
                this.costoPresupuesto.details = det;
            })
            .catch(error => this.handleError(error));
    }

    tieneCPP(): boolean {

        var detail: CostoPresupuestoDetail;
        detail = new CostoPresupuestoDetail();

        var ret: boolean;

        if (this.detailsList.currentItem != undefined) {
            detail = <CostoPresupuestoDetail>this.detailsList.currentItem;

            ret = (detail.idCuentaPorPagar.trim() != "")
        }
        else {
            ret = false;
        }
        return ret;
    }

    onServicioSelected(args: ServicioItem) {

        this.servicioService.getDetailByID(args.id)
            .then(doc => {
                this.currentDetailCopy.idServicio = args.id;
                this.currentDetailCopy.nombreServicio = doc.nombre;

                this.currentDetailCopy.state = ChangeState.MODIFIED;

                //Actualizar controles de wijmo
                Control.invalidateAll();

            })
            .catch(error => this.handleError(error));




    }

    onCuentaPorPagarSelected(args: CuentaPorPagarItem) {


        this.currentDetailCopy.idCuentaPorPagar = args.id;
        this.currentDetailCopy.fechaCuentaPorPagar = args.fechaSys;
        this.currentDetailCopy.importe = args.total;

        this.currentDetailCopy.state = ChangeState.MODIFIED;

        //Actualizar controles de wijmo
        Control.invalidateAll();



    }

}

