﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { CostoPresupuestoInfo, CostoPresupuestoItem } from '../models/costo-presupuesto';
import { CostoPresupuestoDetail } from '../models/costo-presupuesto-detail';
import { CostoPresupuestoService } from '../services/costo-presupuesto.service';

@Component({
    selector: 'compras-costos-presupuestos-manager',
    templateUrl: './costos-presupuestos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CostosPresupuestosManager)
        },
        {
            provide: DocumentService, useExisting: CostoPresupuestoService
        }
    ]
})
export class CostosPresupuestosManager extends DocumentManager {
    costosPresupuestos: CostoPresupuestoItem[];

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private costoPresupuestoService: CostoPresupuestoService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.costoPresupuestoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.costosPresupuestos = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        //this.router.navigate(['0'], { relativeTo: this.route });
        this.router.navigate(['new'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.costoPresupuestoService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.costoPresupuestoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.costoPresupuestoService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

}

