﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
//Locales
import { GastoInfo, GastoItem } from '../models/gasto';
import { GastoDetail } from '../models/gasto-detail';
import { GastoComprobarItem } from '../models/gasto-comprobar-item';
import { GastoComprobanteItem } from '../models/gasto-comprobante-item';
import { GastoService } from '../services/gasto.service';
import { GastoDetailEditor } from './parts/gastos-detail.editor';

import { CFDValiationResult } from '../models/cfd-validation-result'
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';

import { ProveedorInfo } from '../models/proveedor';

import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';


@Component({
    selector: 'purchasing-gasto-comprobar',
    templateUrl: './gasto-comprobar.component.html'
})
export class GastoComprobar extends ProcessComponent {
    form: FormGroup;
    gasto = new GastoInfo();
    idGasto: string;

    origenesCF: { id: string, nombre: string }[] = [];
    idProveedor: number = 0;
    origenCF: string = "L";

    selectedFile: FileInfo;


    previewCollectionDetallesGasto: CollectionView = new CollectionView();
    previewCollectionDetallesGastoCats: CollectionView = new CollectionView();
    previewCollectionDetallesComprobantes: TrackableCollection = new TrackableCollection();
    //previewCollectionDetallesComprobantes: CollectionView = new CollectionView();

    detailMode = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandSingle];
    detailModeMulti = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandMulti];


    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('cargaComprobanteFiscalDialog') origenCFDialog: ModalDirective;
    @ViewChild('comprobantesList') comprobantesListDialog: ModalDirective;
    @ViewChild('uploadDialog') uploadDialog: ModalDirective;


    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder,
        private gastoService: GastoService,
        private comprobanteFiscalService: ComprobanteFiscalService
    )
    {
        super(context, router, route);
        this.route.params.subscribe((params: Params) => {
            this.idGasto = params['idGasto'];
        });


        this.form = formBuilder.group({


        });
    }

    ngOnInit() {
        this.gastoService.getDetail(this.idGasto)
            .then(gto => {
                this.gasto = gto;

                this.previewCollectionDetallesGasto.sourceCollection = this.gasto.details;
                this.previewCollectionDetallesComprobantes.sourceCollection = this.gasto.comprobantes;
            })
            .catch(error => this.handleError(error));

        this.origenesCF =
            [
                { "id": 'L', "nombre": "Ruta Local" },
                { "id": 'A', "nombre": "Almacen digital" },
            ];

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SELECCIONAR') {

            this.origenCFDialog.config.backdrop = false;
            this.idProveedor = 0;
            this.origenCFDialog.show();

        }
        if (button.name == 'GUARDAR') {
            this.gastoService.updateComprobantes(this.gasto)
                .then(ret => {
                    this.alerts.showAlert('Informacion guardada con éxito', 'success');
                })
                .catch(error => this.handleError(error));
        }
    }

    onAsignar(info: ComprobanteFiscalInfo) {

        if (this.previewCollectionDetallesGastoCats.currentItem == null) {
            this.alerts.showAlert("Seleccione primero un servicio de la categría presupuestal", 'success');
        }
        else {
            var item: GastoComprobarItem = new GastoComprobarItem;
            item = <GastoComprobarItem>this.previewCollectionDetallesGastoCats.currentItem;

            var comprobante: GastoComprobanteItem = new GastoComprobanteItem();

            comprobante.idCategoriaPresupuestal = item.idCategoriaPresupuestal;
            comprobante.idComprobante = info.identity;
            comprobante.idFlujo = 0;
            comprobante.referenciaAlf = info.comprobanteBase.serie;
            comprobante.referenciaNum = +info.comprobanteBase.folio;
            comprobante.total = info.comprobanteBase.total;
            comprobante.idServicio = item.idServicio;
            comprobante.nombreServicio = item.nombreServicio;
            comprobante.impuestosRetenidos = info.comprobanteBase.impuestos.totalRetenciones;
            comprobante.impuestosTrasladados = info.comprobanteBase.impuestos.totalTraslados;
            comprobante.subTotal = info.comprobanteBase.subTotal;
            comprobante.descuentos = info.comprobanteBase.descuentos;

            var newCompGto = this.gasto.comprobantes.slice();
            newCompGto.push(comprobante);
            this.gasto.comprobantes = newCompGto;

            this.previewCollectionDetallesComprobantes.sourceCollection = this.gasto.comprobantes;
            this.previewCollectionDetallesComprobantes.refresh;
        }

    }

    getDet(toComp: GastoComprobarItem[]) {
        this.previewCollectionDetallesGastoCats.sourceCollection = toComp;
        return this.previewCollectionDetallesGastoCats;
    }

    onOrigenCFSelected() {

        if (this.origenCF == "A" && this.idProveedor == 0)
            this.alerts.showAlert("Debe seleccionar un proveedor ", 'danger');
        else {
            this.origenCFDialog.hide();

            // Ver que va a cargar
            //this.comprobantesListDialog.config.backdrop = false;
            if (this.origenCF == "A")
                this.comprobantesListDialog.show();
            else {
                this.uploadDialog.config.backdrop = false;
                this.uploadDialog.show();
            }
        }

    }

    onFileChanged(file: FileInfo) {
        this.selectedFile = file;
    }

    onValidateFile() {
        this.uploadDialog.hide();
        this.validateFile();
    }

    validateFile() {
        this.context.app.showSpinner();
        this.comprobanteFiscalService.validate(this.context.app.defaultCompany.identity, this.selectedFile)
            .then(result => {
                this.context.app.hideSpinner();
                this.cathComprobantes(result);
                //this.validatedFiles = result;
                //this.comprobantesCollection.sourceCollection = result;
            })
            .catch(error => this.handleError(error));
    }

    cathComprobantes(ValidCFDcomprobantes: CFDValiationResult[]) {
        let promises: any[] = [];

        ValidCFDcomprobantes.forEach(validCFD => {
            if (validCFD.valid) {
                this.context.app.showSpinner();
                let savePromise = this.comprobanteFiscalService.save(0, this.context.app.defaultCompany.identity, this.context.app.defaultSite.identity, validCFD.comprobante.comprobanteBase.contenidoXML)
                    .then(comprobante => {
                        this.context.app.hideSpinner();
                        this.idProveedor = comprobante.idProveedor;
                        this.onAsignar(comprobante);
                    })
                    .catch(error => this.handleError(error));
            }
            else
                this.alerts.showAlert("Algunos comprobantes no se pudieron validar correctamente:[" + validCFD.validationMessage + "]", "danger")
        });
    }            

    onProveedorChanged(proveedor: ProveedorInfo) {
        this.idProveedor = proveedor.identity;
    }

    onComprobanteSelected(comprobante: ComprobanteFiscalItem) {
        this.comprobanteFiscalService.getDetail(comprobante.id)
            .then(comp => {
                this.onAsignar(comp);
            })
            .catch(error => this.handleError(error));
    }

    onRemoveComprobante(item) {
        this.previewCollectionDetallesComprobantes.remove(item);
        this.previewCollectionDetallesComprobantes.refresh();
    }


}