﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { GrupoRequisicionesInfo, GrupoRequisicionesItem } from '../models/grupo-requisiciones';
import { GrupoRequisicionesDetail } from '../models/grupo-requisiciones-detail';
import { GrupoRequisicionesService } from '../services/grupo-requisiciones.service';
import { GrupoRequisicionesDetailEditor } from './parts/grupo-requisiciones-detail.editor';
import { RequisicionItem } from '../models/requisicion';

@Component({
    selector: 'compras-grupo-requisiciones-editor',
    templateUrl: './grupo-requisiciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => GrupoRequisicionesEditor)
        },
        {
            provide: DocumentService, useExisting: GrupoRequisicionesService
        }
    ]
})
export class GrupoRequisicionesEditor extends DocumentEditor {
    grupoRequisiciones = new GrupoRequisicionesInfo();
    @Input() requisiciones: RequisicionItem[] = [];

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: GrupoRequisicionesDetailEditor;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private grupoRequisicionesService: GrupoRequisicionesService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'sample': ['', Validators.required],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.grupoRequisicionesService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.grupoRequisiciones = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.grupoRequisicionesService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.grupoRequisiciones = doc;
                this.detailsEditor.clientIndexStart = this.grupoRequisiciones.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.grupoRequisicionesService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.grupoRequisiciones = doc;
                this.detailsEditor.clientIndexStart = this.grupoRequisiciones.details.length;
                if (this.grupoRequisiciones.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.grupoRequisicionesService.update(this.grupoRequisiciones)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.grupoRequisiciones.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.grupoRequisiciones = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.grupoRequisicionesService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.grupoRequisicionesService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.grupoRequisiciones.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new GrupoRequisicionesDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: GrupoRequisicionesDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: GrupoRequisicionesDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: GrupoRequisicionesDetail[]) {
        this.grupoRequisiciones.details = updatedDetails;
    }

    onSavingDetail(detail: GrupoRequisicionesDetail) {
        this.detailsList.updateItem(detail);
    }

    loadRequisiciones() {
        let request: SearchRequest = new SearchRequest();
        request.paged = false;
        //Agregar condición de empresa actual
        let condicion: SearchCondition = { field: "IdEmpresa", operator: "=", valueType: "int", value: this.grupoRequisiciones.identity };
        request.conditions.push(condicion);
        //
        this.context.app.showSpinner();
        this.grupoRequisicionesService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.requisiciones = response.rows;
            })
            .catch(error => this.handleError(error));
    }

}

