﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { ImportesInfo } from '../../accounting/models/importes';
import { NotaInfo, NotaItem } from '../models/nota';
import { NotaDetail } from '../models/nota-detail';
import { NotaService } from '../services/nota.service';
import { NotaDetailEditor } from './parts/notas-detail.editor';

import { ProveedorInfo } from '../models/proveedor';
import { ProveedorSelectorComponent } from '../components/proveedor-selector.component';

import { CFDValiationResult } from '../models/cfd-validation-result'
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';

import { DivisaService } from '../../accounting/services/divisa.service';
import { DivisaInfo } from '../../accounting/models/divisa';
import { EmpresaService } from '../../accounting/services/empresa.service';
import { EmpresaInfo } from '../../accounting/models/empresa';


@Component({
    selector: 'compras-notas-editor',
    templateUrl: './notas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => NotasEditor)
        },
        {
            provide: DocumentService, useExisting: NotaService
        }
    ]
})
export class NotasEditor extends DocumentEditor {
    nota = new NotaInfo();

    origenesCF: { id: string, nombre: string }[] = [];
    origenCF: string = "L";
    idProveedor: number;
    nombreProveedor: string= "";
    selectedFile: FileInfo;

    idDivisaDefault: number = 0;
    tipoCambio: string;


    get importesC() {
        return this.importesDesglosados('C');
    }

    get importesN() {
        return this.importesDesglosados('N');
    }

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: NotaDetailEditor;

    @ViewChild('cargaComprobanteFiscalDialog') origenCFDialog: ModalDirective;
    @ViewChild('comprobantesList') comprobantesListDialog: ModalDirective;
    @ViewChild('uploadDialog') uploadDialog: ModalDirective;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private notaService: NotaService,
        private divisaService: DivisaService,
        private empresaService: EmpresaService,
        private comprobanteFiscalService: ComprobanteFiscalService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idProveedor': [0, Validators.required],
            'idDivisa': [0, Validators.required],
            'nombreProveedor':[''],
        });
    }
    /*
    ngOnInit() {

        this.origenesCF =
            [
                { "id": 'L', "nombre": "Ruta Local" },
                { "id": 'A', "nombre": "Almacen digital" },
            ];
    }
    */
    onConfigureDocument(docType: DocumentTypeInfo) {    
        this.applyPolicies();
        this.getDivisaDefault(true);
        this.origenesCF =
            [
                { "id": 'L', "nombre": "Ruta Local" },
                { "id": 'A', "nombre": "Almacen digital" },
            ];

    }

    onCreatingDocument() {
        this.getDivisaDefault(true);
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.notaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nota = doc;
                this.getDivisaDefault(false);
                //this.divisaAsignar();
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.notaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nota = doc;
                this.detailsEditor.clientIndexStart = this.nota.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.notaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.nombreProveedor = doc.proveedor.nombre;
                this.nota = doc;
                //this.divisaAsignar();
                this.detailsEditor.clientIndexStart = this.nota.details.length;
                if (this.nota.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }
    
    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.notaService.update(this.nota)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.nota.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.nota = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.notaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.notaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.nota.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        //this.detailsEditor.detail = new NotaDetail();
        //this.detailsEditor.show();
    }

    onEditingDetail(detail: NotaDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: NotaDetail) {
        //this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: NotaDetail[]) {
        this.nota.details = updatedDetails;
    }

    onSavingDetail(detail: NotaDetail) {
        this.detailsList.updateItem(detail);
        let suma: number = 0;
        this.nota.details.forEach(i => {
            suma = suma + i.importe;
            this.nota.total = suma;
        });

    }

    onProveedorChanged(proveedor: ProveedorInfo) {
        this.idProveedor = proveedor.identity;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SELECCIONAR') {

            this.origenCFDialog.config.backdrop = false;
            this.idProveedor = 0;
            this.origenCFDialog.show();

        }
    }

    onOrigenCFSelected() {

        if (this.origenCF == "A" && this.idProveedor == 0)
            this.showAlert("Debe seleccionar un proveedor ", 'danger');
        else {
            this.origenCFDialog.hide();

            // Ver que va a cargar
            //this.comprobantesListDialog.config.backdrop = false;
            if (this.origenCF == "A")
                this.comprobantesListDialog.show();
            else {
                this.uploadDialog.config.backdrop = false;
                this.uploadDialog.show();
            }
        }

    }

    onComprobanteSelected(comprobante: ComprobanteFiscalItem) {
        this.comprobanteFiscalService.getDetail(comprobante.id)
            .then(comp => {
                this.onAsignar(comp);
            })
            .catch(error => this.handleError(error));
    }

    onAsignar(info: ComprobanteFiscalInfo) {

        this.nombreProveedor = info.proveedor.nombre;
        this.nota.idProveedor = info.proveedor.identity;
        this.nota.idDivisa = info.comprobanteBase.divisa.identity;
        this.nota.idComprobante = info.identity;
        this.nota.fechaComprobante = info.fechaSys;
        this.nota.impuestosRetenidos = info.comprobanteBase.impuestos.totalRetenciones;
        this.nota.impuestosTrasladados = info.comprobanteBase.impuestos.totalTraslados;
        this.nota.comprobante = info;

        //this.divisaAsignar();

        this.getDetailNew(info.proveedor.identity);

    }

    getDetailNew(idProveedor: number) {
        this.notaService.loadDetalleNew(idProveedor)
            .then(detNew => {
                this.nota.details = detNew;
            })
            .catch(error => this.handleError(error));
    }

    

    importesDesglosados(origen: string): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        if (origen == 'C') {
            desglose.impuestosTrasladados = this.nota.impuestosTrasladados;
            desglose.impuestosRetenidos = this.nota.impuestosRetenidos;
            desglose.subtotal = this.nota.comprobante.comprobanteBase.subTotal;

            desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;
        }
        if (origen == 'N') {
            this.nota.impuestosTrasladados = this.nota.comprobante.comprobanteBase.impuestos.totalTraslados;
            this.nota.impuestosRetenidos = this.nota.comprobante.comprobanteBase.impuestos.totalRetenciones;

            desglose.impuestosTrasladados = this.nota.impuestosTrasladados;
            desglose.impuestosRetenidos = this.nota.impuestosRetenidos;
            desglose.total = this.nota.total;

            desglose.subtotal = desglose.total + desglose.descuentos - desglose.impuestosTrasladados + desglose.impuestosRetenidos;
        }

        return desglose;
    }    

    divisaAsignar(args: any) {
        if (this.nota.idDivisa == this.idDivisaDefault) {
            this.nota.tipoCambio = 1;
            this.tipoCambio = this.nota.tipoCambio.toString();
        }
        else {
            this.divisaService.loadTiposCambio(this.context.app.defaultCompany.identity)
                .then(tipos => {
                    let tCambio = tipos.find(search => search.idDivisaOrigen == this.nota.idDivisa && search.idDivisaDestino == this.idDivisaDefault);
                    if (tCambio == undefined) {
                        this.showAlert("No está definido el tipo de cambio para la divisa seleccionada", "danger");
                        this.tipoCambio = null;
                        this.nota.idDivisa = null;
                        this.nota.tipoCambio = 0;
                    }
                    else {
                        this.empresaService.getDetailFromCache(this.context.app.defaultCompany.identity)
                            .then(emp => {
                                if (emp.tipoCambio == 'O') {
                                    this.nota.tipoCambio = tCambio.precioOtro;
                                    this.tipoCambio = tCambio.precioOtro.toString();
                                }
                                else if (emp.tipoCambio == 'V') {
                                    this.nota.tipoCambio = tCambio.precioVenta;
                                    this.tipoCambio = tCambio.precioVenta.toString();
                                }
                                else if (emp.tipoCambio == 'C') {
                                    this.nota.tipoCambio = tCambio.precioCompra;
                                    this.tipoCambio = tCambio.precioCompra.toString();
                                }
                            })
                    }
                });
        }

    }

    getDivisaDefault(creating: boolean) {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.idDivisaDefault = r.identity;
                if (creating) { this.nota.idDivisa = r.identity; }
            })
            .catch(error => this.handleError(error));

    }

    onFileChanged(file: FileInfo) {
        this.selectedFile = file;
    }

    onValidateFile() {
        this.uploadDialog.hide();
        this.validateFile();
    }

    validateFile() {
        this.context.app.showSpinner();
        this.comprobanteFiscalService.validate(this.context.app.defaultCompany.identity, this.selectedFile)
            .then(result => {
                this.context.app.hideSpinner();
                this.cathComprobantes(result);
                //this.validatedFiles = result;
                //this.comprobantesCollection.sourceCollection = result;
            })
            .catch(error => this.handleError(error));
    }

    cathComprobantes(ValidCFDcomprobantes: CFDValiationResult[]) {
        let promises: any[] = [];

        ValidCFDcomprobantes.forEach(validCFD => {
            if (validCFD.valid) {
                this.context.app.showSpinner();
                let savePromise = this.comprobanteFiscalService.save(0, this.context.app.defaultCompany.identity, this.context.app.defaultSite.identity, validCFD.comprobante.comprobanteBase.contenidoXML)
                    .then(comprobante => {
                        this.context.app.hideSpinner();
                        this.idProveedor = comprobante.proveedor.identity;
                        this.onAsignar(comprobante);
                    })
                    .catch(error => this.handleError(error));
            }
            else
                this.showAlert("Algunos comprobantes no se pudieron validar correctamente:[" + validCFD.validationMessage + "]", "danger")
        });
    }        



}

