﻿//Frameworks
import { Input, Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { TrackableCollection } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { Alert, AztecaValidators, AlertsBarComponent, MenuItem } from 'framework3/controls';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap'
//Features
import { CentralService } from '../services/central.service';
import { OrdenService } from '../services/orden.service';
import { SecurityContext } from '../models/security-context';
import { CentralItem } from '../models/central';
import { DepartamentoItem } from '../models/departamento';
import { OrdenRequisicionDetail } from '../models/orden-detail';
import { ProveedoresArticuloListComponent } from '../dialogs/proveedores-articulo-list.component';
import { ProveedoresServicioListComponent } from '../dialogs/proveedores-servicio-list.component';
//
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import { FlexGrid, HitTestInfo, CellType, GroupRow } from 'wijmo/wijmo.grid';
import * as moment from 'moment';
//
import * as _ from 'lodash';
import { groupBy } from 'rxjs/operators';
import { identity } from 'rxjs';

@Component({
    selector: 'purchasing-ordenes-wizard',
    templateUrl: './ordenes-wizard.editor.html'
})
export class OrdenesWizardEditorComponent extends BaseComponent {
    currentStep: number = 1;
    form: FormGroup;
    idCentralCompras: number;
    idDepartamento: number;
    fechaIni: Date;
    fechaFin: Date;
    securityContext: SecurityContext[] = [];
    centrales: CentralItem[] = [];
    departamentos: DepartamentoItem[] = [];

    requisiciones: OrdenRequisicionDetail[] = [];
    //collectionView = new TrackableCollection();
    data: wjcCore.CollectionView;
    dataPreview: wjcCore.CollectionView;

    grid: FlexGrid;
    groupItems: OrdenRequisicionDetail[];
           
    //
    //@ViewChild('dlgConfirmacion') dlgConfirmacion: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('proveedoresArticulosDialog') proveedoresArticulosDialog: ProveedoresArticuloListComponent;
    @ViewChild('proveedoresServiciosDialog') proveedoresServiciosDialog: ProveedoresServicioListComponent;
    @ViewChild('gridPreview') gridPreview: FlexGrid;

    constructor(private context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,  private location: Location,
        private centralService: CentralService, private ordenService: OrdenService) {
        super();
        this.form = formBuilder.group({
            'idCentralCompras': [0],
            'idDepartamento': [0],
            'fechaIni': [new Date],
            'fechaFin': [new Date],
        });

        this.fechaFin = moment().toDate();
        this.fechaIni = moment().add(-30, 'day').toDate();

    }

    ngOnInit() {
        //this.route.params.forEach(param => {
        //    this.proceso = +param['proceso'];
        //});
        this.loadCentrales();
    }
    
    applyPolicy(policy: PolicyInfo): void {

    }

    //handleError(error: any) {
    //    this.context.app.hideSpinner();
    //    let message = this.getErrorMessage(error);
    //    this.alerts.showAlert(message, 'danger');
    //}
    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }


    initializeGrid(flex: wjcGrid.FlexGrid) {
        this.grid = flex;
        // start collapsed
        if(flex)
            flex.collapseGroupsToLevel(0);

        //Manejo de doble click
        //flex.addEventListener(flex.hostElement, "dblclick", (e: wjcGrid.FormatItemEventArgs) => {
        //    //let ht: HitTestInfo = flex.hitTest(e);
        //    //if (ht.cellType == CellType.Cell) {
        //    //    if (this.enabled && this.allowEdit) {
        //    //        this.editingItem.emit(this.currentItem);
        //    //    }
        //    //}

        //    // cells and column footer panels only
        //    if (e.panel == flex.cells) {
        //        let item: OrdenRequisicionDetail = flex.rows[e.row].dataItem
        //        item.articuloServicio="editado"
        //    }

        //});


        //// custom cell calculation
        //flex.formatItem.addHandler((s: wjcGrid.FlexGrid, e: wjcGrid.FormatItemEventArgs) => {
        //    // cells and column footer panels only
        //    if (e.panel == s.cells) {
        //        // get row, column, and data item (or group description)
        //        let r = s.rows[e.row],
        //            c = s.columns[e.col],
        //            item = s.rows[e.row].dataItem,
        //            group = r instanceof wjcGrid.GroupRow ? item : null,
        //            negative = false; // assume value is not negative

                
        //    //    // calculate profit
        //    //    //if (c.binding == 'profit') {
        //    //    //    let profit = group
        //    //    //        ? group.getAggregate('Sum', 'cantidad') 
        //    //    //        : item.cantidad;

        //    //    //    e.cell.textContent = wjcCore.Globalize.format(profit, c.format);
        //    //    //    negative = profit < 0;
        //    //    //}
        //        let texto:string=""
        //        if (group && c.binding == 'proveedor') {
        //            let profit = group
        //                ? item.proveedor
        //                : item.proveedor;

        //            if (group.items.length > 0)
        //                texto = group.items[0].proveedor;
                                    
        //            e.cell.textContent = texto;
        //    //    //    negative = profit < 0;
        //        }
                               
        //    //    if (c.binding == 'costoUnitario') {
        //    //        let profit = group
        //    //            ? group.getAggregate('Sum', 'importe')
        //    //            : item.costoUnitario;

        //    //        e.cell.textContent = wjcCore.Globalize.format(profit, c.format);
        //    //        negative = profit < 0;
        //    //    }

        //    //    if (c.header == '_') {
        //    //        item.group = true;
        //    //    }

        //    //    // update 'negative' class on cell
        //    //    wjcCore.toggleClass(e.cell, 'negative', negative);
        //    }
        //});


    }

    loadCentrales() {        
        this.centralService.loadSecurityContext(this.context.app.defaultCompany.identity)
            .then(items => {
                this.securityContext = items;
                var result = _(this.securityContext).groupBy('idCentral').transform(function (result, current) {
                    result.push({
                        id: current[0].idCentral,
                        nombre: current[0].central,
                    });
                }, []).value();

                this.centrales = result;
            })
            .catch(error => this.handleError(error));
    }

    loadDepartamentos(idCentral: number) {       
        var result = _(this.securityContext.filter(item => item.idCentral == idCentral)).groupBy('idDepartamento').transform(function (result, current) {
            result.push({
                id: current[0].idDepartamento,
                nombre: current[0].departamento,
                tipoPresupuesto: current[0].tipoPresupuesto,
                capturaDetallada: current[0].capturaDetallada
            });
        }, []).value();

        this.departamentos = result;
        if (this.departamentos.length > 0)
            this.idDepartamento = this.departamentos[0].id;
    }

    onClickRequisiciones() {
        this.ordenService.requisicionesPendientes(this.idCentralCompras, this.fechaIni, this.fechaFin)
            .then(items => {
                this.requisiciones = items;
                this.requisiciones.forEach(req => {
                    req.selected = false;
                    let prefijo: string = req.idArticulo > 0 ? "Art:[" : "Serv:[ " 
                    let um: string = req.idArticulo > 0 ? "  {UM:" + req.unidadMedida.abreviatura + "}" : "" 
                    req.key = prefijo + req.codigo + " - " + req.articuloServicio + um + " ]"
                    req.group = false;
                })

                //this.collectionView.sourceCollection = this.requisiciones; 

                //Agrupar solo por estas columnas
                this.data = new wjcCore.CollectionView(this.requisiciones, {
                    sortDescriptions: ['key'],
                    groupDescriptions: ['key']
                });

                setTimeout(_ => {
                    if (this.grid)
                        this.grid.collapseGroupsToLevel(0);
                });
                
            })
            .catch(error => this.handleError(error));        
    }
    
    agregarProveedor(group: any){
        if (group.items.length > 0) {
            this.groupItems = group.items;
            let item: OrdenRequisicionDetail = group.items[0];

            if (item.idArticulo > 0) {
                this.proveedoresArticulosDialog.idArticulo = item.idArticulo;
                this.proveedoresArticulosDialog.idUnidadMedida = item.idUnidadMedida;
                this.proveedoresArticulosDialog.show();
            }
            else if (item.idServicio > 0) {
                this.proveedoresServiciosDialog.idServicio = item.idServicio;
                this.proveedoresServiciosDialog.show();
            }
        }
    }

    proveedorArticuloSelected(proveedor:any) {
        this.groupItems.forEach(item => {
            item.idProveedor = proveedor.id;
            item.proveedor = proveedor.nombre;
            item.costoUnitario = proveedor.costo;
            item.idDivisa = proveedor.idDivisa;
            item.importe = item.cantidad * item.costoUnitario;
        });
        this.grid.refresh();
    }

    proveedorServicioSelected(proveedor: any) {
        this.groupItems.forEach(item => {
            item.idProveedor = proveedor.id;
            item.proveedor = proveedor.nombre;
            item.costoUnitario = proveedor.costo;
            item.idDivisa = proveedor.idDivisa;
            item.importe = item.cantidad * item.costoUnitario;
        });
        this.grid.refresh();
    }
    
    onCellEditEnded(flex: any, e: any, cell: any) {
        
    }

    showPreview() {
        //Agrupar solo por estas columnas
        this.dataPreview = new wjcCore.CollectionView(this.requisiciones.filter(reqs => reqs.idProveedor > 0 ), {
            sortDescriptions: ['proveedor','centroRecepcion'],
            groupDescriptions: ['proveedor','centroRecepcion']
        });

        setTimeout(_ => {
            if (this.gridPreview)
                this.gridPreview.collapseGroupsToLevel(1);
        });
    }

    onClickPrev() {
        this.currentStep = this.currentStep - 1;
        //if (this.currentStep == 1)
        //    this.collectionView.filter = null;

    }

    onClickNext() {
        this.currentStep = this.currentStep + 1;

        if (this.currentStep == 2)
            this.showPreview()
    }

    onClickGenerar() {
        this.context.app.showSpinner();
        this.ordenService.generarDesdeRequisiciones(this.idCentralCompras, this.idDepartamento, this.context.app.defaultSite.identity, this.context.app.defaultCompany.identity, <OrdenRequisicionDetail[]> this.dataPreview.sourceCollection)
            .then( _ => {
                this.context.app.hideSpinner();
                this.location.back();
            })
            .catch(error => {
                this.handleError(error);
            });     
    }

    onClickCancel() {
        this.location.back();
    }

}