﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { OrdenInfo, OrdenItem } from '../models/orden';
import { OrdenDetail } from '../models/orden-detail';
import { OrdenService } from '../services/orden.service';

import { OrdenDetailArticuloEditor } from './parts/ordenes-detail-articulos.editor';
import { OrdenDetailServicioEditor } from './parts/ordenes-detail-servicios.editor';


import { CentralItem } from '../models/central';
import { CentralService } from '../services/central.service';
import { DepartamentoItem } from '../models/departamento';
import { DepartamentoService } from '../services/departamento.service';
import { CentroRecepcionItem } from '../models/centro-recepcion';
import { CentroRecepcionService } from '../services/centro-recepcion.service';
import { ProveedorInfo } from '../models/proveedor';
import { ProveedorSelectorComponent } from '../components/proveedor-selector.component';

import { SecurityContext } from '../models/security-context';
import { ImportesInfo } from '../../accounting/models/importes';
import { DivisaService } from '../../accounting/services/divisa.service';
import { ContribuyenteService } from '../../accounting/services/contribuyente.service';

import * as _ from 'lodash';


//import { CompraArticuloDetail } from '../models/compra-articulo-detail'
//import { Servicio} from '../models/servicio'

import { moment } from 'ngx-bootstrap/chronos/test/chain';
import { DivisaInfo } from '../../accounting';
import { OrdenDetailArticulo } from '../models/orden-detail-articulo';
import { OrdenDetailServicio } from '../models/orden-detail-servicio';

@Component({
    selector: 'compras-ordenes-editor',
    templateUrl: './ordenes.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => OrdenesEditor)
        },
        {
            provide: DocumentService, useExisting: OrdenService
        }
    ]
})
export class OrdenesEditor extends DocumentEditor {
    orden = new OrdenInfo();
    securityContext: SecurityContext[] = [];
    centrales: CentralItem[] = [];
    departamentos: DepartamentoItem[] = [];
    centrosRecepcion: CentralItem[] = [];
    articulos: OrdenDetail[] = [];
    servicios: OrdenDetail[] = [];

    idDivisaDefault: number;
    tipoCambio: string;

    //
    @ViewChild('articulosList') articulosList: EditableListComponent;
    @ViewChild('serviciosList') serviciosList: EditableListComponent;
    //    
    @ViewChild('detailsEditorArticulos') articulosEditor: OrdenDetailArticuloEditor;
    @ViewChild('detailsEditorServicios') serviciosEditor: OrdenDetailServicioEditor;
    //
    @ViewChild('selProveedor') selProveedor: ProveedorSelectorComponent;
       
    get importes() {
        return this.importesDesglosados();
    }

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private ordenService: OrdenService,
        private centralService: CentralService,
        private departamentoService: DepartamentoService,
        private centroRecepcionService: CentroRecepcionService,
        private divisaService: DivisaService,
        private contribuyenteService: ContribuyenteService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idCentral': [0, AztecaValidators.selected],
            'idDepartamento': [0, AztecaValidators.selected],
            'idCentroRecepcion': [0, AztecaValidators.selected],
            'idProveedor': [0, AztecaValidators.selected],
            'fechaEntrega': [new Date],
            'idDivisa': [0, AztecaValidators.selected],
        });

        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        //this.documentForm.controls['idCentral'].valueChanges.subscribe(value => {
        //    setTimeout(_ => this.loadCentrosRecepcion(value));
        //});

        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        //this.documentForm.controls['idCentroRecepcion'].valueChanges.subscribe(value => {
        //    setTimeout(_ =>this.loadDepartamentos(value));
        //});

    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.loadCentrales();
    }

    onCreatingDocument() {
        this.getDivisaDefault(false);
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.ordenService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.orden = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.ordenService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.orden = doc;
                //
                this.articulos = this.orden.details.map(d => <OrdenDetailArticulo>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.servicios = this.orden.details.map(d => <OrdenDetailServicio>d).filter(d => d.idServicio && d.idServicio > 0);
                //
                //this.detailsEditor.clientIndexStart = this.articulos.length;
                this.articulosEditor.clientIndexStart = this.articulos.length;
                this.serviciosEditor.clientIndexStart = this.servicios.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.ordenService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.loadCentrosRecepcion(doc.idCentralCompras);
                this.loadDepartamentos(doc.idCentroRecepcion);                
                this.orden = doc;

                //separar el detalle en arreglo de articulos y servicios
                this.separarDetail();
                               
                
                if (this.orden.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();

                //setTimeout(_ => this.refreshValidity());

            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.orden.details = this.combineDetail();

        this.ordenService.update(this.orden)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.orden.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.orden = updatedDoc;
                this.separarDetail();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.ordenService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ordenService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    //////////////////////////// Metodos locales //////////////////////              
    //Separar articulos y servicios
    separarDetail() {
        //
        this.articulos = this.orden.details.map(d => <OrdenDetailArticulo>d).filter(d => d.idArticulo && d.idArticulo > 0);
        this.servicios = this.orden.details.map(d => <OrdenDetailServicio>d).filter(d => d.idServicio && d.idServicio > 0);
        //
        //this.detailsEditor.clientIndexStart = this.articulos.length + this.servicios.length;
        this.articulosEditor.clientIndexStart = this.articulos.length;
        this.serviciosEditor.clientIndexStart = this.servicios.length;
    }

    combineDetail(): OrdenDetail[] {
        let newDetail: OrdenDetail[] = [];
        this.servicios.forEach(s => newDetail.push(s));
        this.articulos.forEach(a => newDetail.push(a));
        return newDetail;
    }

    loadCentrales() {
         //this.centralService.loadListByUser(this.context.app.defaultSite.identity)
        //    .then(items => this.centrales = items)
        //    .catch(error => this.handleError(error));
        this.centralService.loadSecurityContext(this.context.app.defaultCompany.identity)
            .then(items => {
                this.securityContext = items;
                var result = _(this.securityContext).groupBy('idCentral').transform(function (result, current) {
                    result.push({
                        id: current[0].idCentral,
                        nombre: current[0].central,
                        
                    });
                }, []).value();

                this.centrales = result;
            })
            .catch(error => this.handleError(error));       
    }

    loadCentrosRecepcion(idCentral: number) {
        //this.centroRecepcionService.loadListByDepartamentoAndUser(idDepartamento, this.context.app.defaultSite.identity)
        //    .then(items => this.centrosRecepcion = items)
        //    .catch(error => this.handleError(error));
        var result = _(this.securityContext.filter(item => item.idCentral == idCentral)).groupBy('idCentroRecepcion').transform(function (result, current) {
            result.push({
                id: current[0].idCentroRecepcion,
                nombre: current[0].centroRecepcion,
            });
        }, []).value();

        this.centrosRecepcion = result;
        if (this.centrosRecepcion.length > 0 && this.orden.identity=='')
            this.orden.idCentroRecepcion = this.centrosRecepcion[0].id;

    }

    loadDepartamentos(idCentroRecepcion: number) {
        //this.departamentoService.loadListByCentralUser(idCentral, this.context.app.defaultSite.identity)
        //    .then(items => this.departamentos = items)
        //    .catch(error => this.handleError(error));
        var result = _(this.securityContext.filter(item => item.idCentral == this.orden.idCentralCompras && item.idCentroRecepcion== idCentroRecepcion)).groupBy('idDepartamento').transform(function (result, current) {
            result.push({
                id: current[0].idDepartamento,
                nombre: current[0].departamento,
                tipoPresupuesto: current[0].tipoPresupuesto,
                capturaDetallada: current[0].capturaDetallada
            });
        }, []).value();

        this.departamentos = result; 
        if (this.departamentos.length > 0 && this.orden.identity == '')
            this.orden.idDepartamento = this.departamentos[0].id ;
    }

    importesDesglosados(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        let elementos = this.combineDetail().filter(item => item.state != 'DELETED');

        elementos.forEach(item => {
            desglose.subtotal += item.costoUnitario * item.cantidad;
            desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidad;
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        return desglose;
    }    

    //Metodos generales
    getDivisaDefault(creating: boolean) {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(r => {
                this.orden.idDivisa = r.identity;
                this.idDivisaDefault = r.identity;
            })
            .catch(error => this.handleError(error));
    }

    ////////////////////////// Manejo de eventos /////////////
    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.orden.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    onProveedorChanged(proveedor: ProveedorInfo) {
        if (proveedor && proveedor.identity > 0) {
            let hoy: Date = new Date();
            this.orden.fechaEntrega = hoy; // moment(hoy).add(proveedor.);        
            
            if (this.orden.identity == "") {

                this.contribuyenteService.inListaNegraEFO(proveedor.contribuyente.rfc, proveedor.idCompany)
                    .then(result => {
                        if (result == true) {
                            this.showAlert("El contribuyente esta registrado como EFO", "danger");
                            this.selProveedor.clear();
                        }
                    })
                    .catch(error => this.handleError(error));

                this.articulos = [];
                this.servicios = [];
            }
        }
    }

    onDivisaChanged(divisa: DivisaInfo) {
        this.orden.divisa = divisa;
        this.orden.details = [];
        //this.recepcionSinOC..sourceCollection = [];
        if (this.orden.idDivisa == this.idDivisaDefault) {
            this.orden.tipoCambio = 1;            
        }
        else {
            if (this.orden.identity == '') //solo para documentos nuevos
            {
                this.divisaService.loadTiposCambio(this.context.app.defaultSite.identity)
                    .then(tipos => {
                        let tCambio = tipos.find(search => search.idDivisaOrigen == this.orden.idDivisa && search.idDivisaDestino == divisa.identity);
                        if (tCambio == undefined) {
                            this.showAlert("No está definido el tipo de cambio para la divisa seleccionada", "danger");
                            this.tipoCambio = null;
                            this.orden.idDivisa = null;
                            this.orden.tipoCambio = 0;
                        }
                        else {
                            this.orden.tipoCambio = tCambio.precioOtro;                            
                        }
                    });
            }
        }
        //
        this.tipoCambio = this.orden.tipoCambio.toString();
    }
    ///////////// ARTICULOS /////////////////////////

    onAddingArticulo() {       
        if (this.orden.idProveedor ==  undefined || this.orden.idProveedor == 0){
            this.showAlert("No se ha seleccionado un proveedor", "warning");
        }
            else {
            this.articulosEditor.detail = new OrdenDetailArticulo();
            this.articulosEditor.show();
        }
    }

    onEditingArticulo(detail: OrdenDetailArticulo) {        
        this.articulosEditor.detail = detail;
        this.articulosEditor.onLoadingItem(detail);
        this.articulosEditor.show();
        //this.quantityEditor.unidadMedida = detail.articulo.articuloStock.unidadMedida;
        //this.quantityEditor.cantidad = detail.cantidad;
        //this.quantityEditor.unidades = detail.unidades;
        //this.quantityEditor.show();
    }

    onDeletingArticulo(detail: OrdenDetailArticulo) {
        this.articulosList.deleteItem(detail);
    }

    onArticulosUpdated(updatedItems: OrdenDetailArticulo[]) {
        this.articulos = updatedItems;
    }

    onSavingArticulo(detail: OrdenDetailArticulo) {
        this.articulosList.updateItem(detail);
    }

    onSavingArticulos(details: OrdenDetailArticulo[]) {
        this.articulosList.updateItems(details);
    }

    //onUpdateQuantity(value: any) {
    //    let detail = <OrdenDetailArticulo>this.articulosList.currentItem;
    //    if (detail.unidadMedida.paquete) {
    //        let cantidad = <Empaquetado>value;
    //        detail.cantidad = cantidad.paquetes;
    //        detail.unidades = cantidad.unidades;
    //    }
    //    else {
    //        detail.cantidad = <number>value;
    //    }
    //    this.articulosList.updateItem(detail);
    //}

     
    ///////////// SERVICIOS /////////////////////////

    onAddingServicio() {        
        if (this.orden.idProveedor == undefined || this.orden.idProveedor == 0) {
            this.showAlert("No se ha seleccionado un proveedor", "warning");
        }
        else {
            this.serviciosEditor.detail = new OrdenDetailServicio();
            this.serviciosEditor.show();
        }
    }   

    onEditingServicio(detail: OrdenDetailServicio) {        
        this.serviciosEditor.detail = detail;
        this.serviciosEditor.show();
    }

    onDeletingServicio(detail: OrdenDetailServicio) {
        this.serviciosList.deleteItem(detail);
    }

    onServiciosUpdated(updatedServices: OrdenDetailServicio[]) {
        this.servicios = updatedServices;
    }

    onSavingServicio(detail: OrdenDetailServicio) {
        this.serviciosList.updateItem(detail);
    }

    onSavingServicios(details: OrdenDetailServicio[]) {
        this.serviciosList.updateItems(details);
    }

     ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    //onAddingDetail() {
    //    this.detailsEditor.detail = new OrdenDetail();
    //    this.detailsEditor.show();
    //}

    //onEditingDetail(detail: OrdenDetail) {
    //    this.detailsEditor.detail = detail;
    //    this.detailsEditor.show();
    //}

    //onDeletingDetail(detail: OrdenDetail) {
    //    this.detailsList.deleteItem(detail);
    //}

    //onListItemsUpdated(updatedDetails: OrdenDetail[]) {
    //    this.orden.details = updatedDetails;
    //}

    //onSavingDetail(detail: OrdenDetail) {
    //    detail.importe = detail.costoUnitario * detail.cantidad;
    //    if (detail.idArticulo > 0)
    //        this.articulosList.updateItem(detail);
    //    else
    //        this.serviciosList.updateItem(detail);
    //}


}

