﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
// Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem, PolicyInfo } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { MenuItem } from 'framework3/controls';
import { SearchRequest } from 'framework3/search';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
//Locales
import { OrdenInfo, OrdenItem } from '../models/orden';
import { OrdenDetail } from '../models/orden-detail';
import { OrdenService } from '../services/orden.service';

import { ReporteInfo, ParametroInfo, ReportService } from 'framework3/core';


@Component({
    selector: 'compras-ordenes-manager',
    templateUrl: './ordenes.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => OrdenesManager)
        },
        {
            provide: DocumentService, useExisting: OrdenService
        }
    ]
})
export class OrdenesManager extends DocumentManager {
    ordenesTit: OrdenItem[];

    opcionesNew: { id: string, nombre: string }[] = [];
    opcionNew: string = "N";
    @ViewChild('tipoDialog') dlgTipo: ModalDirective;

    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;
    titleContentView: string = "";
    fileNameContentView: string = "";


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private ordenService: OrdenService, private reportSvc: ReportService) {
        super(context, router, route);

        this.opcionesNew =
            [               
                { "id": 'N', "nombre": "Normal" },
                { "id": 'W', "nombre": "Asistente para requisiciones" }
            ];

    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    //applyPolicies() {
    //    let mnuAsistente: MenuItem = { icon: "glyphicon glyphicon-duplicate", text: "Asistente para generación de ordenes", name: "mnuAsistente", smallText: "generación de ordenes a partir de requisiciones" };
    //    this.manager.menuItems.push(mnuAsistente);
    //}

    applyPolicy(policy: PolicyInfo): void {
        if (policy.valor == 'ASISTENTE_OC') {
            let mnuAsistente: MenuItem = { icon: "glyphicon glyphicon-duplicate", text: "Asistente para generación de ordenes", name: "ASISTENTE_OC", smallText: "generación de ordenes a partir de requisiciones", enabled: true };
            this.manager.menuItems.push(mnuAsistente);
        }
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.ordenService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.ordenesTit = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {        
        // Dialogo para seleccionar si se quiere usar el asistente de oc o el editor de oc
        //this.dlgTipo.config.backdrop = false;
        //this.dlgTipo.show();

        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.ordenService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.ordenService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ordenService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PDF') {
            if (this.manager.currentDocument.idStatus > 1) {
                this.context.app.showSpinner();
                this.ordenService.generarPdfOrden(this.manager.currentDocument.id)
                    .then(file => {
                        this.context.app.hideSpinner();
                        window.open('/file-server/' + file.uri, '_blank');
                    })
                    .catch(error => this.handleError(error));
            }
            else {
                this.showAlert('No puede imprimir una orden en el estatus actual.', 'success');
            }
        }
        if (option.name == 'ASISTENTE_OC') {
            this.router.navigate(['0/wizard'], { relativeTo: this.route });
        }
        //else if (option.name == 'PDF') {

        //    this.context.app.showSpinner();
        //    let item = <OrdenItem> this.manager.currentDocument;
        //    let informe = new ReporteInfo();
        //    let parametro = new ParametroInfo();
        //    parametro.name = "IdOrdenTit";
        //    parametro.values.push(item.id);
        //    informe.parametros.push(parametro);
        //    informe.path = "/Onili/Compras/OrdenRPT";
        //    informe.format = "pdf";
        //    this.reportSvc.loadReporte(informe)
        //        .then(info => {
        //            this.context.app.hideSpinner();
        //            let item = <OrdenItem>this.manager.currentDocument;
        //            this.titleContentView = "Orden " + item.id;
        //            this.fileNameContentView = item.id + ".pdf";
        //            this.reportViewer.show("data:application/pdf; base64, " + info.content);
        //        })
        //        .catch(error => this.handleError(error));

        //}


    }

    onTipoAddAceptar() {
        if (this.opcionNew == "W")
            this.router.navigate(['0/wizard'], { relativeTo: this.route });
        else
            this.router.navigate(['0'], { relativeTo: this.route });
    }

}

