﻿import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
//Locales
import { NotaDetail } from '../../models/nota-detail';

@Component({
    selector: 'compras-nota-detail-editor',
    templateUrl: './notas-detail.editor.html'
})
export class NotaDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new NotaDetail();
    //Objeto original que se editará
    @Input() detail: NotaDetail = new NotaDetail();
    total: number = 0;


    constructor(protected context: Context, private builder: FormBuilder) {
        super(context);

        this.form = builder.group({
            'total': [0],
        });
    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
		this.form.reset({ sample: '' });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: NotaDetail ) {
        this.detailCopy = <NotaDetail>this.copyItem(this.detail);
    }

    onUpdatingItem(detail: NotaDetail ) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
    }

    onCellEditEnded(flex: any, e: any) {

        let sum: number = 0;

        flex.rows.forEach(item => {
            sum = sum + item.dataItem.importe;
        });

        this.detailCopy.importe = sum;

    }


}
