﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder } from '@angular/forms';
//Framework
import { Context } from 'framework3/core';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';

//Compras
import { CostoPresupuestoService } from '../../services/costo-presupuesto.service';
import { PresupuestosService } from '../../services/presupuestos.service';
import { PresupuestosInfo, PresupuestosItem } from '../../models/presupuestos';
import { DepartamentoInfo } from '../../models/departamento';


@Component({
    selector: 'compras-nuevo-costos-presupuestos',
    templateUrl: './nuevo-costos-presupuestos.component.html'
})

export class NuevoCostosPresupuestosComponent {
    form: FormGroup;
    idDepartamento: number;
    idPresupuestoSelected: string;
    presupuestos: PresupuestosItem[] = [];
    departamento: DepartamentoInfo;
    idBeneficiario: number;


    //
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute,
        private context: Context,
        private costoPresupuestoSvc: CostoPresupuestoService,
        private presupuestoService: PresupuestosService) {
        this.form = formBuilder.group({
            'idDepartamento': [0],
            'idPresupuestoSelected': [''],
            'idBeneficiario': [0]
        })
    }

    /////////// HANDLERS
    onToolbarClick(button: ToolbarButton) {
        if (button.name == 'ACEPTAR') {

            this.router.navigate(['../', '0', { idPresupuesto: this.idPresupuestoSelected, idDepartamento: this.idDepartamento, idBeneficiario: this.idBeneficiario}], { relativeTo: this.route });

        }
        else if (button.name == 'CANCEL') {
            this.router.navigate(['../'], { relativeTo: this.route });
        }
    }

    ngOnInit() {

    }


    loadPresupuestos(depa: DepartamentoInfo) {
        this.presupuestoService.loadPresupuestos(depa.identity)
            .then(ret => {
                this.presupuestos = ret;
            })
            .catch(error => this.handleError(error));

    }


    handleError(error: any) {
        console.log(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }



}

