﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder } from '@angular/forms';
//Framework
import { Context } from 'framework3/core';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';

//Compras
import { GastoService } from '../../services/gasto.service';
import { PresupuestosService } from '../../services/presupuestos.service';
import { GastoTipoService } from '../../services/gasto-tipo.service';
import { PresupuestosInfo, PresupuestosItem } from '../../models/presupuestos';
import { DepartamentoInfo, DepartamentoItem } from '../../models/departamento';
import { DepartamentoService } from '../../services/departamento.service'
import { GastoTipoItem } from '../../models/gasto-tipo';


@Component({
    selector: 'compras-nuevo-gasto',
    templateUrl: './nuevo-gasto.component.html'
})

export class NuevoGastoComponent {
    form: FormGroup;
    idDepartamento: number;
    idPresupuestoSelected: string;
    idGastoTipo: number;
    presupuestos: PresupuestosItem[] = [];
    gastosTipos: GastoTipoItem[] = [];
    departamento: DepartamentoInfo;
    departamentos: DepartamentoItem[] = [];



    //
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute,
        private context: Context,
        private gastoSvc: GastoService,
        private presupuestoService: PresupuestosService,
        private gastoTipoService: GastoTipoService,
        private departamentoService: DepartamentoService) {
        this.form = formBuilder.group({
            'idDepartamento': [0],
            'idGastoTipo': [0],
            'idPresupuestoSelected': ['']
        })
    }

    /////////// HANDLERS
    onToolbarClick(button: ToolbarButton) {
        if (button.name == 'ACEPTAR') {

            this.router.navigate(['../', '0', { idPresupuesto: this.idPresupuestoSelected, idDepartamento: this.idDepartamento, idGastoTipo: this.idGastoTipo }], { relativeTo: this.route });

        }
        else if (button.name == 'CANCEL') {
            this.router.navigate(['../'], { relativeTo: this.route });
        }
    }

    ngOnInit() {
        this.loadGastosTipos();
        this.loadDepartamentos();
    }


    loadPresupuestos(idDepartamento) {
        this.presupuestoService.loadPresupuestos(idDepartamento)
            .then(ret => {
                this.presupuestos = ret.filter(item => item.workFlowStepTag == 'OPEN');
            })
            .catch(error => this.handleError(error));

    }

    loadGastosTipos() {
        this.gastoTipoService.loadListByUser(this.context.app.defaultSite.idBranch)
            .then(ret => {
                this.gastosTipos = ret;
            })
            .catch(error => this.handleError(error));
    }

    loadDepartamentos() {
        this.departamentoService.loadListByUser(this.context.app.defaultSite.identity)
            .then(items => this.departamentos = items)
            .catch(error => this.handleError(error));
    }


    handleError(error: any) {
        console.log(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }



}

