﻿
import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators, FormControl } from '@angular/forms';
//Wijmo
import { Control, Globalize, CollectionView } from 'wijmo/wijmo';
import { FlexGrid, CellRangeEventArgs, FormatItemEventArgs, CellType } from 'wijmo/wijmo.grid';

//Kernel
import { Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';

//Inventario
import { ArticuloStockInfo, UnidadMedidaInfo, UnidadMedidaItem, StockInfo, LoteInfo, SerieInfo, UnidadMedidaService, AlmacenService, MetodoValuacion } from '../../../inventory';
import { ImpuestoDetailInfo, TipoAplicacion, TipoFactor } from '../../../accounting';


//Locales
import { OrdenDetailArticulo } from '../../models/orden-detail-articulo';
import { DepartamentoItem, DepartamentoInfo } from '../../models/departamento';
import { PresupuestosService } from '../../services/presupuestos.service';
import { PresupuestosItem, PresupuestosInfo } from '../../models/presupuestos';
import { CategoriasPresupuestalesItem } from '../../models/categorias-presupuestales';
import { ArticuloInfo, ArticuloItem } from '../../models/articulo';
import { ArticuloService } from '../../services/articulo.service';
import { ProveedorService } from '../../services/proveedor.service';
import { QuantityInputComponent} from '../../../inventory/controls/quantity-input.component';

@Component({
    selector: 'compras-orden-detail-articulos-editor',
    templateUrl: './ordenes-detail-articulos.editor.html'
})
export class OrdenDetailArticuloEditor extends BasicEditor {
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new OrdenDetailArticulo();

    trasladosCollection = new CollectionView();
    retencionesCollection = new CollectionView();

    //Objeto original que se editará
    @Input() detail: OrdenDetailArticulo = new OrdenDetailArticulo();
    @Input() departamentos: DepartamentoItem[] = [];
    @Input() idProveedor: number = 0;
    costoUnitario: number = 0;
    departamento: DepartamentoItem;
    presupuestos: PresupuestosItem[] = [];
    categoriasPresupuestales: CategoriasPresupuestalesItem[]=[];
    unidadesMedida: { id: number, nombre: string }[] = []; //UnidadMedidaItem[] = [];
    unidadesMedidaItems: UnidadMedidaItem[] = [];

    @ViewChild('quantity') quantity: QuantityInputComponent;

    private selectedUM: UnidadMedidaItem;
    validInput: boolean = false; 

    get costo() {
        return Globalize.formatNumber(this.costoUnitario,"c");
    }

    constructor(protected context: Context, private builder: FormBuilder, private unidadMedidaService: UnidadMedidaService,
        private presupuestoService: PresupuestosService,
        private articuloService: ArticuloService,
        private proveedorService: ProveedorService) {

        super(context);

        this.form = builder.group({
            'idDepartamento': [0, AztecaValidators.selected],
            'idArticulo': [0, AztecaValidators.selected],
            'idUnidadMedida': [0, AztecaValidators.selected],            
            //'cantidad': [0, Validators.required],
            'idPresupuesto': [''],
            'idCategoriaPresupuestal': [0], //, (control) => this.validateCategoria(control)],
            'observaciones': [''],
            'costo': [0],
        });

        this.trasladosCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.TRASLADO;
        };

        this.retencionesCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.RETENCION;
        };


        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        this.form.controls['idUnidadMedida'].valueChanges.subscribe(value => {
            setTimeout(_ => {
                this.setCurrentUnidad(value);
                this.onUnidadMedidaChanged(value);
            });
        });

        //Si se manejan series, la unidad de medida no será requerida
        this.form.controls['idDepartamento'].valueChanges.subscribe(value => {
            this.form.controls['idPresupuesto'].setValidators(null);
            this.form.controls['idCategoriaPresupuestal'].setValidators(null);

            if (this.departamento && this.departamento.tipoPresupuesto != "N") {
                this.form.controls['idPresupuesto'].setValidators(Validators.required);
                if (this.departamento && this.departamento.tipoPresupuesto == "C")
                    this.form.controls['idCategoriaPresupuestal'].setValidators(AztecaValidators.selected);
            }
            this.form.controls['idPresupuesto'].updateValueAndValidity();
            this.form.controls['idCategoriaPresupuestal'].updateValueAndValidity();
        });
    }

    /////////////////// METODOS ///////////////////////////////
    //validatePresupuesto( control: FormControl) {
    //    if (this.departamento && this.departamento.tipoPresupuesto != "N" && control.value == "")
    //        return 'Debe establecerse un presupuesto';
    //    else
    //        return null;
    //}
    //validateCategoria( control: FormControl) {
    //    if (this.departamento && this.departamento.tipoPresupuesto == "C" && control.value == "")
    //        return 'Debe establecer una categoria presupuestal';
    //    else
    //        return null;
    //}
    /////////////////// HANDLERS //////////////////////////////
    
    onCreatingItem() {
        this.detailCopy = this.detail;
        //this.departamento = new DepartamentoItem();
        this.form.reset({ idArticulo: 0, idServicio: 0, idUnidadMedida: 0, Cantidad: 0, idDepartamento: 0, idPresupuesto: '', idCategoriaPresupuestal: 0, costo: 0 });        
        this.form.markAsPristine();

        if (this.departamentos.length > 0) {
            this.detailCopy.idDepartamento = this.departamentos[0].id;
            this.onDepartamentoChanged(this.departamentos[0].id);
        }
    }

    onLoadingItem(detail: OrdenDetailArticulo) {
        this.detailCopy = <OrdenDetailArticulo>this.copyItem(this.detail);
        //this.validators();
    }

    onUpdatingItem(detail: OrdenDetailArticulo) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }

        this.savingItem.emit(this.detailCopy);
    }


    onDepartamentoChanged(idDepartamento: number) {
        if (idDepartamento > 0) {
            this.departamento = this.departamentos.find(item => item.id == idDepartamento);
            this.detailCopy.departamento = this.departamento.nombre;
            if (this.departamento.tipoPresupuesto != 'N') {
                this.loadPresupuestos(idDepartamento);
            }           
        }
    }

    onPresupuestoChanged(idPresupuesto: string) {
        this.loadCategoriasPresupuestales(idPresupuesto);
    }

    onArticuloChanged(articulo: ArticuloInfo) {
        if (articulo) {
            this.detailCopy.articulo = articulo;
            this.loadUnidadesMedida(this.detailCopy.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
            this.loadCategoriasPresupuestales(this.detailCopy.idPresupuesto);
            //this.articuloService.calcularCostoProveedor(this.detailCopy.idArticulo,articulo.idUnidadMedida, this.idProveedor, this.context.app.defaultCompany.identity)
            //    .then(costo => {
            //        this.detailCopy.costoUnitario = costo.costoBase;
            //        this.detailCopy.costoNeto = costo.costoNeto;
            //        this.detailCopy.descuentos = costo.descuentos;
            //        this.detailCopy.impuestos = costo.impuestos;
            //        this.costoUnitario = this.detailCopy.costoUnitario;
            //        this.detailCopy.importe = this.detailCopy.costoUnitario * this.detailCopy.cantidad;
            //        //
            //        this.trasladosCollection.sourceCollection = costo.impuestos;
            //        this.retencionesCollection.sourceCollection = costo.impuestos;                    
            //    })
            //    .catch(error => this.handleError(error));

        }
        else {
            this.detailCopy.articulo = new ArticuloInfo();
        }
        this.trasladosCollection.sourceCollection = null;
        this.retencionesCollection.sourceCollection = null;
    }

    cantidadChanged(monto: number) {
        this.detailCopy.importe = this.detailCopy.costoUnitario * monto;
    }

    onCantidadChange(cantidad: number) {
        this.detailCopy.cantidad = cantidad;
        this.detailCopy.importe = this.detailCopy.costoUnitario * this.quantity.totalDecimal;        
    }

    onUnidadesChange(unidades: number) {
        this.detailCopy.unidades = unidades;
        this.detailCopy.importe = this.detailCopy.costoUnitario * this.quantity.totalDecimal;//this.detailCopy.cantidad;
    }

    onUnidadMedidaChanged(idUnidadMedida: number) {
        if (this.detailCopy.idArticulo > 0 && idUnidadMedida > 0) {
            this.articuloService.calcularCostoProveedor(this.detailCopy.idArticulo, idUnidadMedida, this.idProveedor, this.context.app.defaultCompany.identity)
                .then(costo => {
                    this.detailCopy.costoUnitario = costo.costoBase;
                    this.detailCopy.costoNeto = costo.costoNeto;
                    this.detailCopy.descuentos = costo.descuentos;
                    this.detailCopy.impuestos = costo.impuestos;
                    this.costoUnitario = this.detailCopy.costoUnitario;
                    this.detailCopy.importe = this.detailCopy.costoUnitario * this.detailCopy.cantidad;
                    //
                    this.trasladosCollection.sourceCollection = costo.impuestos;
                    this.retencionesCollection.sourceCollection = costo.impuestos;
                })
                .catch(error => this.handleError(error));
        }
    }

    onInputValidChange(valid: boolean) {
        this.validInput = valid;
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
        //en caso contrario, cargar lista de unidades de medida del mismo tipo
        if (this.detailCopy.articulo.articuloStock.manejaSeries) {
            let unidadArticulo = this.detailCopy.articulo.articuloStock.unidadMedida;
            let newUnidad: UnidadMedidaItem = {
                id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base, nombre:unidadArticulo.nombre,
                factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
            };
            let newUnidades = [];
            newUnidades.push(newUnidad);
            this.unidadesMedida = newUnidades;

            this.detailCopy.idUnidadMedida = unidadArticulo.identity;
        }
        else {
            this.unidadesMedida = [];
            this.unidadMedidaService.loadList(idTipoUnidad)
                .then(unidadesMedidaItems => {
                    this.unidadesMedidaItems = unidadesMedidaItems;
                    //this.detailCopy.idUnidadMedida = this.detailCopy.articulo.idUnidadMedida;

                    let requestProveedor = new SearchRequest();
                    requestProveedor.pageIndex = 0;
                    requestProveedor.pageSize = 0;
                    requestProveedor.paged = false;
                    this.proveedorService.searchProveedorByArticulo(this.detail.articulo.identity, this.context.app.defaultSite.identity, requestProveedor)                        
                        .then(unidadesMedida => {                            
                            unidadesMedida.rows.forEach(item => {
                                let um = this.unidadesMedida.find(x => x.id == item.idUnidadMedida)
                                if (um == null || um == undefined)
                                    this.unidadesMedida.push({ id: item.idUnidadMedida, nombre: item.unidadMedida });
                            });
                            this.detailCopy.idUnidadMedida = this.detailCopy.articulo.articuloStock.idUnidadMedida;
                        })
                        .catch(error => this.handleError(error));

                })
                .catch(error => this.handleError(error));            
        }

    }

    setCurrentUnidad(idUnidadMedida: number) {
        this.selectedUM = this.unidadesMedidaItems.find(u => u.id == idUnidadMedida);       
    }


    loadPresupuestos(idDepartamento: number) {
        if (this.departamento.tipoPresupuesto != 'N') {
            this.presupuestoService.loadPresupuestos(idDepartamento)
                .then(info => {                                        
                    if (this.detail.id == 0) //nuevo
                        this.presupuestos = info.filter(opens => opens.workFlowStepTag == 'OPEN'); //solo los presupuesto abiertos 
                    else
                        this.presupuestos = info; //mostrar todos los presupuestos
                })
                .catch(error => this.handleError(error));
        }
    }

    loadCategoriasPresupuestales(idPresupuesto: string) {
        if (this.departamento && this.departamento.tipoPresupuesto =='C' && idPresupuesto != "" && this.detailCopy.idArticulo != 0) {
            this.presupuestoService.loadCategoriasConArticulo(idPresupuesto, this.detailCopy.idArticulo)
                .then(info => {
                    this.categoriasPresupuestales = info;
                })
                .catch(error => this.handleError(error));
        }
    }

    //validators() {
    //    if (this.detailCopy.articuloServicio == "A") {
    //        this.form.controls.idArticulo.setValidators(AztecaValidators.selected);
    //        this.form.controls.idUnidadMedida.setValidators(AztecaValidators.selected);
    //        this.form.controls.idServicio.setValidators(null);
    //    }
    //    else {
    //        this.form.controls.idServicio.setValidators(AztecaValidators.selected);
    //        this.form.controls.idArticulo.setValidators(null);
    //        this.form.controls.idUnidadMedida.setValidators(null);
    //    }
    //    this.form.controls.idArticulo.updateValueAndValidity();
    //    this.form.controls.idUnidadMedida.updateValueAndValidity();
    //    this.form.controls.idServicio.updateValueAndValidity();
    //}

    onEditCell(grid: FlexGrid, args: CellRangeEventArgs) {
        if (!grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cancel = true;
        }
    }

    onFormatCell(grid: FlexGrid, args: FormatItemEventArgs) {
        if (args.panel.cellType == CellType.Cell && !grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cell.style.backgroundColor = '#C0C0C0';
        }
    }
}
