﻿
import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators, FormControl } from '@angular/forms';
//Wijmo
import { Control, Globalize, CollectionView } from 'wijmo/wijmo';
import { FlexGrid, CellRangeEventArgs, FormatItemEventArgs, CellType } from 'wijmo/wijmo.grid';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';

//Inventario
//import { ArticuloStockInfo, UnidadMedidaInfo, UnidadMedidaItem, StockInfo, LoteInfo, SerieInfo, UnidadMedidaService, AlmacenService, MetodoValuacion } from '../../../inventory';
import { ImpuestoDetailInfo, TipoAplicacion, TipoFactor } from '../../../accounting';


//Locales
import { OrdenDetailServicio } from '../../models/orden-detail-servicio';
import { DepartamentoItem } from '../../models/departamento';
import { PresupuestosService } from '../../services/presupuestos.service';
import { PresupuestosItem, PresupuestosInfo } from '../../models/presupuestos';
import { CategoriasPresupuestalesItem } from '../../models/categorias-presupuestales';
import { ServicioInfo, ServicioItem } from '../../models/servicio';
import { ServicioService } from '../../services/servicio.service';


@Component({
    selector: 'compras-orden-detail-servicios-editor',
    templateUrl: './ordenes-detail-servicios.editor.html'
})
export class OrdenDetailServicioEditor extends BasicEditor {
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new OrdenDetailServicio();

    trasladosCollection = new CollectionView();
    retencionesCollection = new CollectionView();

    //Objeto original que se editará
    @Input() detail: OrdenDetailServicio = new OrdenDetailServicio();
    @Input() departamentos: DepartamentoItem[] = [];
    @Input() idProveedor: number = 0;
    costoUnitario: string = '$0.0';
    departamento: DepartamentoItem;
    presupuestos: PresupuestosItem[] = [];
    presupuesto: PresupuestosItem = new PresupuestosItem();
    categoriasPresupuestales: CategoriasPresupuestalesItem[] = [];
    //unidadesMedida: UnidadMedidaItem[] = [];

    constructor(protected context: Context, private builder: FormBuilder,
        private presupuestoService: PresupuestosService,
        private servicioService: ServicioService) {

        super(context);

        this.form = builder.group({
            'idDepartamento': [0, AztecaValidators.selected],
            'idServicio': [0, AztecaValidators.selected],
            'cantidad': [0, Validators.required],
            'idPresupuesto': [''], // (control) => this.validatePresupuesto(this.departamento, control)],
            'idCategoriaPresupuestal': [0],// (control) => this.validateCategoria(this.departamento, control)],
            'observaciones': [''],
            'costo': [0],
        });

        this.trasladosCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.TRASLADO;
        };

        this.retencionesCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.RETENCION;
        };

        //Si se manejan series, la unidad de medida no será requerida
        this.form.controls['idDepartamento'].valueChanges.subscribe(value => {
            this.form.controls['idPresupuesto'].setValidators(null);
            this.form.controls['idCategoriaPresupuestal'].setValidators(null);

            if (this.departamento && this.departamento.tipoPresupuesto != "N") {
                this.form.controls['idPresupuesto'].setValidators(Validators.required);
                if (this.departamento && this.departamento.tipoPresupuesto == "C")
                    this.form.controls['idCategoriaPresupuestal'].setValidators(AztecaValidators.selected);
            }
            this.form.controls['idPresupuesto'].updateValueAndValidity();
            this.form.controls['idCategoriaPresupuestal'].updateValueAndValidity();
        });
    }

    ///////////////////// METODOS ///////////////////////////////
    //validatePresupuesto(departamento: DepartamentoItem, control: FormControl) {
    //    if (departamento && departamento.tipoPresupuesto != "N" && control.value == "")
    //        return 'Debe establecerse un presupuesto';
    //    else
    //        return null;
    //}
    //validateCategoria(departamento: DepartamentoItem, control: FormControl) {
    //    if (departamento && departamento.tipoPresupuesto == "C" && control.value == "")
    //        return 'Debe establecer una categoria presupuestal';
    //    else
    //        return null;
    //}
    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
        this.form.reset({ idArticulo: 0, idServicio: 0, idUnidadMedida: 0, Cantidad: 0, idDepartamento: 0, idPresupuesto: '', idCategoriaPresupuestal: 0, costo: 0 });
        //this.departamento = new DepartamentoItem();
        this.form.markAsPristine();

        if (this.departamentos.length > 0) {
            this.detailCopy.idDepartamento = this.departamentos[0].id;
            this.onDepartamentoChanged(this.departamentos[0].id);
        }
    }

    onLoadingItem(detail: OrdenDetailServicio) {
        this.detailCopy = <OrdenDetailServicio>this.copyItem(this.detail);        
    }

    onUpdatingItem(detail: OrdenDetailServicio) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }

        this.savingItem.emit(this.detailCopy);
    }


    onDepartamentoChanged(idDepartamento: number) {
        if (idDepartamento > 0) {
            this.departamento = this.departamentos.find(item => item.id == idDepartamento);
            this.detailCopy.departamento = this.departamento.nombre;
            if (this.departamento.tipoPresupuesto != 'N') {
                this.loadPresupuestos(idDepartamento);
            }
        }
    }

    onPresupuestoChanged(idPresupuesto: string) {
        this.loadCategoriasPresupuestales(idPresupuesto);
    }

    onServicioChanged(servicio: ServicioInfo) {
        if (servicio) {
            this.detailCopy.servicio = servicio;
            ////Hacer requerido el campo de descripción cuando el servicio sea de concepto libre
            //if (this.detailCopy.articuloServicio=="S") {
            //    this.form.controls.idServicio.setValidators(Validators.required);
            //}
            //else {
            //    this.form.controls.idServicio.setValidators(null);
            //}
            //this.form.controls.descripcion.updateValueAndValidity();
            this.loadCategoriasPresupuestales(this.detailCopy.idPresupuesto);
            //Cargar información de precios del servicio
            this.servicioService.calcularCostoProveedor(this.detailCopy.idServicio, this.idProveedor, this.context.app.defaultCompany.identity, 0)
                .then(costo => {
                    this.detailCopy.costoUnitario = costo.costoBase;
                    this.detailCopy.costoNeto = costo.costoNeto;
                    this.detailCopy.impuestos = costo.impuestos;
                    this.detailCopy.descuentos = costo.descuentos;
                    this.detailCopy.importe = this.detailCopy.costoUnitario * this.detailCopy.cantidad;
                    //
                    this.trasladosCollection.sourceCollection = costo.impuestos;
                    this.retencionesCollection.sourceCollection = costo.impuestos;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.detailCopy.servicio = new ServicioInfo();
        }
        this.trasladosCollection.sourceCollection = null;
        this.retencionesCollection.sourceCollection = null;
    }

    loadPresupuestos(idDepartamento: number) {
        if (this.departamento && this.departamento.tipoPresupuesto != 'N') {
            this.presupuestoService.loadPresupuestos(idDepartamento)
                .then(info => {
                    if (this.detail.id == 0) //nuevo
                        this.presupuestos = info.filter(opens => opens.workFlowStepTag == 'OPEN'); //solo los presupuesto abiertos 
                    else
                        this.presupuestos = info; //mostrar todos los presupuestos
                })
                .catch(error => this.handleError(error));
        }
    }

    loadCategoriasPresupuestales(idPresupuesto: string) {
        if (this.departamento && this.departamento.tipoPresupuesto == 'C' && idPresupuesto != "" && this.detailCopy.idServicio != 0) {
            this.presupuestoService.loadCategoriasConServicio(idPresupuesto, this.detailCopy.idServicio)
                .then(info => {
                    this.categoriasPresupuestales = info;
                })
                .catch(error => this.handleError(error));
        }
    }

    cantidadChanged(monto: number) {
        this.detailCopy.importe = this.detailCopy.costoUnitario * monto;
    }
    
    onEditCell(grid: FlexGrid, args: CellRangeEventArgs) {
        if (!grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cancel = true;
        }
    }

    onFormatCell(grid: FlexGrid, args: FormatItemEventArgs) {
        if (args.panel.cellType == CellType.Cell && !grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cell.style.backgroundColor = '#C0C0C0';
        }
    }
}
