﻿
import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';
//Wijmo
import { Control, Globalize, CollectionView } from 'wijmo/wijmo';
//import { FlexGrid, CellRangeEventArgs, FormatItemEventArgs, CellType } from 'wijmo/wijmo.grid';

////Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';

////Inventario
import { ArticuloStockInfo, UnidadMedidaInfo, UnidadMedidaItem, StockInfo, LoteInfo, SerieInfo, UnidadMedidaService, AlmacenService, MetodoValuacion } from '../../../inventory';
import { ImpuestoDetailInfo, TipoAplicacion, TipoFactor } from '../../../accounting';


////Locales
import { OrdenDetail } from '../../models/orden-detail';
import { DepartamentoItem } from '../../models/departamento';
//import { DepartamentoInfo } from '../../models/departamento';
import { PresupuestosService } from '../../services/presupuestos.service';
import { PresupuestosItem, PresupuestosInfo } from '../../models/presupuestos';
import { ArticuloInfo, ArticuloItem } from '../../models/articulo';
import { ArticuloService } from '../../services/articulo.service';
//import { ServicioInfo, ServicioItem } from '../../models/servicio';
import { ServicioService } from '../../services/servicio.service';
//import { AztecaValidators } from 'framework3/controls';

@Component({
    selector: 'compras-orden-detail-editor',
    templateUrl: './ordenes-detail.editor.html'
})
export class OrdenDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new OrdenDetail();

    trasladosCollection = new CollectionView();
    retencionesCollection = new CollectionView();

    //Objeto original que se editará
    @Input() detail: OrdenDetail = new OrdenDetail();
    @Input() departamentos: DepartamentoItem[] = [];
    @Input() idProveedor: number = 0;
    costoUnitario: string = '$0.0';
    departamento: DepartamentoItem = new DepartamentoItem();
    presupuestos: PresupuestosItem[]=[];
    presupuesto: PresupuestosItem = new PresupuestosItem();
    
    unidadesMedida: UnidadMedidaItem[] = [];

    constructor(protected context: Context, private builder: FormBuilder, private unidadMedidaService: UnidadMedidaService,
        private presupuestoService: PresupuestosService,
        private articuloService: ArticuloService,
        private servicioService: ServicioService) {

        super(context);

        this.form = builder.group({
            'idDepartamento': [0, Validators.required],
            'idArticulo': [0],
            'idUnidadMedida': [0],
            'idServicio': [0],   
            'cantidad': [0, Validators.required],
            'idPresupuesto': [0],
            'idCategoriaPresupuestal': [0],
            'observaciones': [''],
            'costo': [0],
        });

        this.trasladosCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.TRASLADO;
        };

        this.retencionesCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.RETENCION;
        };

    }

    ///////////////////// METODOS ///////////////////////////////


    ///////////////////// HANDLERS //////////////////////////////

    //onCreatingItem() {
    //    this.detailCopy = this.detail;
    //    this.form.reset({ idArticulo: 0, idServicio: 0, idUnidadMedida: 0, Cantidad: 0, idPresupuesto: 0, idCategoriaPresupuestal:0, costo:0  });
    //    this.form.markAsPristine();
    //}

    //onLoadingItem(detail: OrdenDetail ) {
    //    this.detailCopy = <OrdenDetail>this.copyItem(this.detail);
    //    this.validators();
    //}

    //onUpdatingItem(detail: OrdenDetail ) {
    //    if (this.detailCopy._clientID == 0) {
    //        this.detailCopy._clientID = this.createClientID();
    //    }
    //    else {
    //        this.detailCopy.state = ChangeState.MODIFIED;
    //    }

    //    this.savingItem.emit(this.detailCopy);
    //}
    

    //onDepartamentoChanged(idDepartamento: number) {
    //   this.departamento = this.departamentos.find(item => item.id == idDepartamento);        
    //}

    //onServicioChanged(servicio: ServicioInfo) {
    //    if (servicio) {
    //        this.detailCopy.servicio = servicio;
    //        ////Hacer requerido el campo de descripción cuando el servicio sea de concepto libre
    //        //if (this.detailCopy.articuloServicio=="S") {
    //        //    this.form.controls.idServicio.setValidators(Validators.required);
    //        //}
    //        //else {
    //        //    this.form.controls.idServicio.setValidators(null);
    //        //}
    //        //this.form.controls.descripcion.updateValueAndValidity();
    //        //Cargar información de precios del servicio
    //        this.servicioService.calcularCostoProveedor(this.detailCopy.idServicio, this.idProveedor, this.context.app.defaultCompany.identity, 0)
    //            .then(costo => {
    //                this.detailCopy.costoUnitario = costo.costoBase;
    //                this.detailCopy.costoNeto = costo.costoNeto;
    //                this.detailCopy.impuestos = costo.impuestos;
    //                this.detailCopy.descuentos = costo.descuentos;
    //                //
    //                this.trasladosCollection.sourceCollection = costo.impuestos;
    //                this.retencionesCollection.sourceCollection = costo.impuestos;
    //            })
    //            .catch(error => this.handleError(error));
    //    }
    //    else {
    //        this.detailCopy.servicio = new ServicioInfo();
    //    }
    //    this.trasladosCollection.sourceCollection = null;
    //    this.retencionesCollection.sourceCollection = null;
    //}

    //onArticuloChanged(articulo: ArticuloInfo) {
    //    if (articulo) {
    //        this.detailCopy.articulo = articulo;
    //        this.loadUnidadesMedida(this.detailCopy.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
    //        this.articuloService.calcularCostoProveedor(this.detailCopy.idArticulo, this.idProveedor, this.context.app.defaultCompany.identity)
    //            .then(costo => {
    //                this.detailCopy.costoUnitario = costo.costoBase;
    //                this.detailCopy.costoNeto = costo.costoNeto;
    //                this.detailCopy.descuentos = costo.descuentos;
    //                this.detailCopy.impuestos = costo.impuestos;
    //                this.costoUnitario = Globalize.formatNumber(this.detailCopy.costoUnitario, 'c');
    //                //
    //                this.trasladosCollection.sourceCollection = costo.impuestos;
    //                this.retencionesCollection.sourceCollection = costo.impuestos;
    //            })
    //            .catch(error => this.handleError(error));
    //    }        
    //    else {
    //        this.detailCopy.servicio = new ServicioInfo();
    //    }
    //    this.trasladosCollection.sourceCollection = null;
    //    this.retencionesCollection.sourceCollection = null;
    //}

    //loadUnidadesMedida(idTipoUnidad: number) {
    //    //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
    //    //en caso contrario, cargar lista de unidades de medida del mismo tipo
    //    if (this.detailCopy.articulo.articuloStock.manejaSeries) {
    //        let unidadArticulo = this.detailCopy.articulo.articuloStock.unidadMedida;
    //        let newUnidad: UnidadMedidaItem = {
    //            id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base,
    //            factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
    //        };
    //        let newUnidades = [];
    //        newUnidades.push(newUnidad);
    //        this.unidadesMedida = newUnidades;

    //        this.detailCopy.idUnidadMedida = unidadArticulo.identity;
    //    }
    //    else {
    //        this.unidadMedidaService.loadList(idTipoUnidad)
    //            .then(unidadesMedida => {
    //                this.unidadesMedida = unidadesMedida;
    //                this.detailCopy.idUnidadMedida = this.detailCopy.articulo.idUnidadMedida;
    //            })
    //            .catch(error => this.handleError(error));
    //    }
    //}

   

    //validators(){
    //    if (this.detailCopy.articuloServicio == "A") {
    //        this.form.controls.idArticulo.setValidators(AztecaValidators.selected);
    //        this.form.controls.idUnidadMedida.setValidators(AztecaValidators.selected);
    //        this.form.controls.idServicio.setValidators(null);
    //    }
    //    else {
    //        this.form.controls.idServicio.setValidators(AztecaValidators.selected);
    //        this.form.controls.idArticulo.setValidators(null);
    //        this.form.controls.idUnidadMedida.setValidators(null);
    //    }
    //    this.form.controls.idArticulo.updateValueAndValidity();
    //    this.form.controls.idUnidadMedida.updateValueAndValidity();
    //    this.form.controls.idServicio.updateValueAndValidity();
    //}

    //onEditCell(grid: FlexGrid, args: CellRangeEventArgs) {
    //    if (!grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
    //        args.cancel = true;
    //    }
    //}

    //onFormatCell(grid: FlexGrid, args: FormatItemEventArgs) {
    //    if (args.panel.cellType == CellType.Cell && !grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
    //        args.cell.style.backgroundColor = '#C0C0C0';
    //    }
    //}
}
