﻿import { Component, Input, Output, EventEmitter, ChangeDetectorRef } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';
//Wijmo
import { Globalize } from 'wijmo/wijmo';
//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//Inventario
import { ArticuloStockInfo, UnidadMedidaInfo, UnidadMedidaItem, StockInfo, LoteInfo, SerieInfo, UnidadMedidaService, AlmacenService, MetodoValuacion } from '../../../inventory';
//Locales
import { RecepcionDetail } from '../../models/recepcion-detail';
import { RecepcionArticuloDetail } from '../../models/recepcion-articulo-detail';
import { setTimeout } from 'timers';

@Component({
    selector: 'compras-recepcion-articulos-detail-editor',
    templateUrl: './recepcion-articulos-detail.editor.html'
})
export class RecepcionArticulosDetailEditor extends BasicEditor {
    unidadesMedida: UnidadMedidaItem[] = [];
    stockConfig: StockInfo;
    validInput: boolean = false;    

    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new RecepcionArticuloDetail();
    //Objeto original que se editará
    @Input() detail: RecepcionArticuloDetail = new RecepcionArticuloDetail();
    @Input() idAlmacen: number = 0;    
    @Input() enabledCantidad:boolean = true;
    //

    get articuloCode() {
        return this.detailCopy.articulo ? this.detailCopy.articulo.code + '-' + this.detailCopy.articulo.nombre : '';
    }

    constructor(protected context: Context, private builder: FormBuilder, private unidadMedidaService: UnidadMedidaService, private almacenService: AlmacenService, private cdRef: ChangeDetectorRef) {
        super(context);
        //
        this.form = builder.group({
        });
    }

    /////////////////// METODOS ///////////////////////////////
    loadStock(idArticulo: number) {
        if (idArticulo && idArticulo > 0) {
            this.almacenService.loadStock(this.idAlmacen, idArticulo)
                .then(stock => {
                    this.stockConfig = stock;
                    this.loadUnidadesMedida(this.detailCopy.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
                    //this.form.updateValueAndValidity();
                })
                .catch(error => this.handleError(error));
        }
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
        //en caso contrario, cargar lista de unidades de medida del mismo tipo
        if (this.stockConfig.manejaSeries) {
            let unidadArticulo = this.detailCopy.articulo.articuloStock.unidadMedida;
            let newUnidad: UnidadMedidaItem = {
                id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base,
                factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
            };
            let newUnidades = [];
            newUnidades.push(newUnidad);
            this.unidadesMedida = newUnidades;
            this.detailCopy.idUnidadMedida = unidadArticulo.identity;
        }
        else {
            this.unidadMedidaService.loadList(idTipoUnidad)
                .then(unidadesMedida => {
                    this.unidadesMedida = unidadesMedida;
                    this.detailCopy.idUnidadMedida = this.detailCopy.articulo.articuloStock.idUnidadMedida;
                })
                .catch(error => this.handleError(error));
        }
    }

    
    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
        this.cdRef.detectChanges();
        this.form.reset({ articulo: '' });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: RecepcionArticuloDetail) {
        this.detailCopy = <RecepcionArticuloDetail>this.copyItem(this.detail);
        //this.onCreatingItem();
        this.loadStock(this.detailCopy.articulo.articuloStock.identity)        
        //setTimeout(_ => this.loadStock(this.detailCopy.articulo.articuloStock.identity), 1000);
    }

    ngAfterViewInit() {
        //setTimeout(_ => this.loadStock(this.detailCopy.articulo.articuloStock.identity),1000);            
    }

    onUpdatingItem(detail: RecepcionArticuloDetail) {
        this.unidadMedidaService.getDetailFromCache(this.detailCopy.idUnidadMedida)
            .then(unidad => {
                this.detailCopy.unidadMedida = unidad;
                if (this.detailCopy._clientID == 0) {
                    this.detailCopy._clientID = this.createClientID();
                }
                else {
                    this.detailCopy.state = ChangeState.MODIFIED;
                }
                this.savingItem.emit(this.detailCopy);
                this.detailCopy = new RecepcionArticuloDetail();
            })
            .catch(error => this.handleError(error));
    }

    onInputValidChange(valid: boolean) {
        this.validInput = valid;
    }

    //onLotesChange(lotes: LoteInfo[]) {
    //    this.lotes = lotes;
    //}

    //onSeriesChange(series: SerieInfo[]) {
    //    this.series = series;
    //}

    onCantidadCambio(value: number) {
        this.detailCopy.importe = this.detailCopy.costoUnitario * this.detailCopy.cantidad;
    }

    onUnidadesCambio(value: number) {
        this.detailCopy.importe = this.detailCopy.costoUnitario * this.detailCopy.cantidad;
    }


    onChangeFechaCaducidad(value: any) {
        if (this.detailCopy.articulo.articuloStock.manejaCaducidad)
            this.detailCopy.fechaCaducidad = value;
        
        this.cdRef.detectChanges();
    }

}
