﻿import { Component, Input, ViewChild, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators, FormControl } from '@angular/forms';
//wijmo
import { Control, Globalize, CollectionView } from 'wijmo/wijmo';
import { FlexGrid, CellRangeEventArgs, FormatItemEventArgs, CellType } from 'wijmo/wijmo.grid';

//Kernel
import { Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { RecepcionSinOCDetail } from '../../models/recepcion-sin-oc-detail';
import { ArticuloInfo } from '../../models/articulo'
import { DepartamentoItem, DepartamentoInfo } from '../../models/departamento';
import { ArticuloService} from '../../services/articulo.service'

import { RecepcionSinOCDetailArticulo } from '../../models/recepcion-sin-oc-detail-articulo';

//otros modulos
import { StockInfo } from '../../../inventory/models/stock';
import { UnidadMedidaItem } from '../../../inventory/models/unidad-medida';
import { ArticuloStockEntradaComponent } from '../../../inventory/controls/articulo-stock-entrada.component';
import { UnidadMedidaService } from '../../../inventory/services/unidad-medida.service';
import { AlmacenService } from '../../../inventory/services/almacen.service';
import { PresupuestosService } from '../../services/presupuestos.service';
import { PresupuestosItem, PresupuestosInfo } from '../../models/presupuestos';
import { ImpuestoDetailInfo, TipoAplicacion, TipoFactor } from '../../../accounting';
import { CategoriasPresupuestalesItem } from '../../models/categorias-presupuestales';
import { setTimeout } from 'timers';
import { ProveedorService } from '../../services/proveedor.service';

@Component({
    selector: 'compras-recepcion-sin-oc-articulos-editor',
    templateUrl: './recepciones-sin-oc-detail-articulos.editor.html'
})

export class RecepcionSinOCArticulosEditor extends BasicEditor {
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new RecepcionSinOCDetailArticulo();

    stockConfig: StockInfo;
    unidadesMedida: { id: number, nombre: string }[] = []; //UnidadMedidaItem[] = [];
    trasladosCollection = new CollectionView();
    retencionesCollection = new CollectionView();

    //Objeto original que se editará
    @Input() detail: RecepcionSinOCDetailArticulo = new RecepcionSinOCDetailArticulo();
    @Input() idAlmacen: number = 0; 
    @Input() idProveedor: number = 0; 
    @Input() departamentos: DepartamentoItem[] = [];
        
    @ViewChild('articuloEntrada') articuloEntrada: ArticuloStockEntradaComponent;

    departamento: DepartamentoItem;
    presupuestos: PresupuestosItem[] = [];
    presupuesto: PresupuestosItem = new PresupuestosItem();
    categoriasPresupuestales: CategoriasPresupuestalesItem[] = [];
    costoUnitario: number = 0;

    get costo() {
        return Globalize.formatNumber(this.costoUnitario, "c");
    }

    constructor(protected context: Context, private builder: FormBuilder, private articuloService: ArticuloService,
        private unidadMedidaService: UnidadMedidaService, private almacenService: AlmacenService,
        private presupuestosService: PresupuestosService, private proveedorService: ProveedorService) {
        super(context);

        this.form = builder.group({            
            'idDepartamento': [0, AztecaValidators.selected],
            'idArticulo': [0, AztecaValidators.selected],
            'idPresupuesto': [''],
            'idCategoriaPresupuestal': [0], //, (control) => this.validateCategoria(control)],
        });

        this.trasladosCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.TRASLADO;
        };

        this.retencionesCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.RETENCION;
        };

        //Si se manejan presupuesto
        this.form.controls['idDepartamento'].valueChanges.subscribe(value => {
            this.form.controls['idPresupuesto'].setValidators(null);
            this.form.controls['idCategoriaPresupuestal'].setValidators(null);

            if (this.departamento && this.departamento.tipoPresupuesto != "N") {
                this.form.controls['idPresupuesto'].setValidators(Validators.required);
                if (this.departamento && this.departamento.tipoPresupuesto == "C")
                    this.form.controls['idCategoriaPresupuestal'].setValidators(AztecaValidators.selected);
            }
            this.form.controls['idPresupuesto'].updateValueAndValidity();
            this.form.controls['idCategoriaPresupuestal'].updateValueAndValidity();
        });
    }

    /////////////////// METODOS ///////////////////////////////
    //validatePresupuesto(departamento: DepartamentoItem, control: FormControl) {
    //    if (departamento && departamento.tipoPresupuesto != "N" && control.value == "")
    //        return 'Debe establecerse un presupuesto';
    //    else
    //        return null;
    //}
    //validateCategoria(departamento: DepartamentoItem, control: FormControl) {
    //    if (departamento && departamento.tipoPresupuesto == "C" && control.value == "")
    //        return 'Debe establecer una categoria presupuestal';
    //    else
    //        return null;
    //}

    onArticuloChanged(articulo: ArticuloInfo) {
        
        if (articulo)
        {
            if (this.detailCopy.idArticulo > 0) {
                this.costoUnitario = 0;
                this.detailCopy.articulo = articulo;
                this.loadStock(this.detailCopy.articulo.articuloStock.identity);
                this.loadCategoriasPresupuestales(this.detailCopy.idPresupuesto);
                //this.articuloService.calcularCostoProveedor(this.detailCopy.articulo.identity, articulo.idUnidadMedida, this.idProveedor, this.context.app.defaultCompany.identity)
                //    .then(costo => {
                //        this.detailCopy.costoUnitario = costo.costoBase;
                //        this.detailCopy.costoNeto = costo.costoNeto;
                //        this.detailCopy.descuentos = costo.descuentos;
                //        this.detailCopy.impuestos = costo.impuestos;
                //        this.detailCopy.importe = costo.costoBase * this.detailCopy.cantidad;
                //        this.costoUnitario = this.detailCopy.costoUnitario;
                //        //this.costoUnitario = Globalize.formatNumber(this.detailCopy.costoUnitario, 'c');
                //        //
                //        this.trasladosCollection.sourceCollection = costo.impuestos;
                //        this.retencionesCollection.sourceCollection = costo.impuestos;
                //    })
                //    .catch(error => this.handleError(error));
            }
        }
        //else {
        //    let impuestosEmpty: ImpuestoDetailInfo[] = [];
        //    this.trasladosCollection.sourceCollection = impuestosEmpty;
        //    this.retencionesCollection.sourceCollection = impuestosEmpty;
        //}
    }

    loadStock(idArticulo: number) {
        this.almacenService.loadStock(this.idAlmacen, idArticulo)
            .then(stock => {
                this.stockConfig = stock;                
                this.loadUnidadesMedida(this.detailCopy.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
        //en caso contrario, cargar lista de unidades de medida del mismo tipo
        if (this.stockConfig.manejaSeries) {
            let unidadArticulo = this.detailCopy.articulo.articuloStock.unidadMedida;
            let newUnidad: UnidadMedidaItem = {
                id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base,
                factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
            };
            let newUnidades = [];
            newUnidades.push(newUnidad);
            this.unidadesMedida = newUnidades;
            this.detailCopy.idUnidadMedida = unidadArticulo.identity;
            this.detailCopy.unidadMedida.nombre = unidadArticulo.nombre;
        }
        else {            
            this.unidadesMedida = [];
            let requestProveedor = new SearchRequest();
            requestProveedor.pageIndex = 0;
            requestProveedor.pageSize = 0;
            requestProveedor.paged = false;
            this.proveedorService.searchProveedorByArticulo(this.detailCopy.articulo.identity, this.context.app.defaultSite.identity, requestProveedor)
                //this.unidadMedidaService.loadList(idTipoUnidad)
                .then(unidadesMedida => {
                    //this.unidadesMedida = unidadesMedida;
                    unidadesMedida.rows.forEach(item => {
                        let um = this.unidadesMedida.find(x => x.id == item.idUnidadMedida)
                        if (um == null || um == undefined)
                            this.unidadesMedida.push({ id: item.idUnidadMedida, nombre: item.unidadMedida });
                    });
                    this.detailCopy.idUnidadMedida = this.detailCopy.articulo.articuloStock.idUnidadMedida;
                    this.onUnidadMedidaChange(this.detailCopy.articulo.articuloStock.idUnidadMedida);
                })
                .catch(error => this.handleError(error));
        }
    }
    
    onUnidadMedidaChange(idUnidadMedida: number) {
        this.articuloService.calcularCostoProveedor(this.detailCopy.idArticulo, idUnidadMedida, this.idProveedor, this.context.app.defaultCompany.identity)
            .then(costo => {
                this.detailCopy.costoUnitario = costo.costoBase;
                this.detailCopy.costoNeto = costo.costoNeto;
                this.detailCopy.descuentos = costo.descuentos;
                this.detailCopy.impuestos = costo.impuestos;
                this.costoUnitario = this.detailCopy.costoUnitario;
                this.detailCopy.importe = this.detailCopy.costoUnitario * this.detailCopy.cantidad;
                //
                this.trasladosCollection.sourceCollection = costo.impuestos;
                this.retencionesCollection.sourceCollection = costo.impuestos;
            })
            .catch(error => this.handleError(error));
    }

    onCantidadChanged(cantidad: number) {
        this.detailCopy.importe = this.detailCopy.costoUnitario * cantidad;
    }
    onCostoUnitarioChanged(costo: number) {
        this.detailCopy.importe = this.detailCopy.importe//this.detailCopy.cantidad * costo;
    }

    onDepartamentoChanged(idDepartamento: number) {
        if (idDepartamento > 0) {
            this.departamento = this.departamentos.find(item => item.id == idDepartamento);
            this.detailCopy.departamento = this.departamento.nombre;
            if (this.departamento.tipoPresupuesto != 'N') {
                this.loadPresupuestos(idDepartamento);
            }
        }
    }

    onPresupuestoChanged(idPresupuesto: string) {
        this.loadCategoriasPresupuestales(idPresupuesto);
    }

    loadPresupuestos(idDepartamento: number) {
        if (this.departamento.tipoPresupuesto != 'N') {
            this.presupuestosService.loadPresupuestos(idDepartamento)
                .then(info => {
                    //Si es nuevo, solo que despliegue los presupuestos abiertos
                    if (this.detailCopy.id == 0) {
                        this.presupuestos = info.filter(prestos=>prestos.workFlowStepTag=='OPEN')
                    }
                    else
                        this.presupuestos = info;
                })
                .catch(error => this.handleError(error));
        }
    }

    loadCategoriasPresupuestales(idPresupuesto: string) {
        if (this.departamento && this.departamento.tipoPresupuesto == 'C' && idPresupuesto != "" && this.detailCopy.idArticulo != 0) {
            this.presupuestosService.loadCategoriasConArticulo(idPresupuesto, this.detailCopy.idArticulo)
                .then(info => {
                    this.categoriasPresupuestales = info;
                })
                .catch(error => this.handleError(error));
        }
    }
    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
        this.form.reset({ idArticulo: 0, costoUnitario: 0, idDepartamento: 0 });
        this.costoUnitario = 0; 
        this.detailCopy.costoUnitario = 0;
        this.trasladosCollection = new CollectionView();
        this.retencionesCollection = new CollectionView();
        this.form.markAsPristine();

        if (this.departamentos.length > 0) {
            this.detailCopy.idDepartamento = this.departamentos[0].id;
            this.onDepartamentoChanged(this.departamentos[0].id);
        }
    }

    onLoadingItem(detail: RecepcionSinOCDetailArticulo) {    
        
        this.detailCopy = <RecepcionSinOCDetailArticulo>this.copyItem(this.detail);

        setTimeout(_ => {

            if (this.departamentos.length > 0) {
                this.onDepartamentoChanged(detail.idDepartamento);
            }

            this.loadStock(this.detailCopy.articulo.articuloStock.identity);            
            this.costoUnitario = this.detailCopy.costoUnitario;
            this.trasladosCollection.sourceCollection = this.detailCopy.impuestos;
            this.retencionesCollection.sourceCollection = this.detailCopy.impuestos;

        }, 1*1000);

    }

    onUpdatingItem(detail: RecepcionSinOCDetail) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
    }
    
    onEditCell(grid: FlexGrid, args: CellRangeEventArgs) {
        if (!grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cancel = true;
        }       
    }

    onEditedCell(grid: FlexGrid, args: CellRangeEventArgs) {
        if (grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {            
            grid.rows[args.row].dataItem.importe = grid.rows[args.row].dataItem.base * grid.rows[args.row].dataItem.tasaOCuota;
        }
    }

    onFormatCell(grid: FlexGrid, args: FormatItemEventArgs) {
        if (args.panel.cellType == CellType.Cell && !grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cell.style.backgroundColor = '#C0C0C0';
        }
    }



}