﻿import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators, FormControl } from '@angular/forms';
//wijmo
import { Control, Globalize, CollectionView } from 'wijmo/wijmo';
import { FlexGrid, CellRangeEventArgs, FormatItemEventArgs, CellType } from 'wijmo/wijmo.grid';

//Kernel
import { Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { BasicEditor, ChangeState } from 'framework3/editors';

//Locales
import { RecepcionSinOCDetailServicio } from '../../models/recepcion-sin-oc-detail-servicio';
import { ServicioService } from '../../services/servicio.service';
import { ServicioInfo } from '../../models/servicio';
import { DepartamentoItem } from '../../models/departamento';
import { PresupuestosService } from '../../services/presupuestos.service';
import { PresupuestosItem, PresupuestosInfo } from '../../models/presupuestos';
import { ImpuestoDetailInfo, TipoAplicacion, TipoFactor } from '../../../accounting';
import { CategoriasPresupuestalesItem } from '../../models/categorias-presupuestales';

////otros modulos
//import { StockInfo } from '../../../inventory/models/stock';
//import { UnidadMedidaItem } from '../../../inventory/models/unidad-medida';
//import { UnidadMedidaService } from '../../../inventory/services/unidad-medida.service';
//import { AlmacenService } from '../../../inventory/services/almacen.service';

@Component({
    selector: 'compras-recepcion-sin-oc-servicios-editor',
    templateUrl: './recepciones-sin-oc-detail-servicios.editor.html'
})

export class RecepcionSinOCServiciosEditor extends BasicEditor {
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new RecepcionSinOCDetailServicio();

    //stockConfig: StockInfo;
    //unidadesMedida: UnidadMedidaItem[] = [];
    trasladosCollection = new CollectionView();
    retencionesCollection = new CollectionView();

    //Objeto original que se editará
    @Input() detail: RecepcionSinOCDetailServicio = new RecepcionSinOCDetailServicio();
    @Input() idAlmacen: number = 0;
    @Input() idProveedor: number = 0;
    @Input() departamentos: DepartamentoItem[] = [];

    departamento: DepartamentoItem = new DepartamentoItem();
    presupuestos: PresupuestosItem[] = [];
    presupuesto: PresupuestosItem = new PresupuestosItem();
    categoriasPresupuestales: CategoriasPresupuestalesItem[] = [];

    constructor(protected context: Context, private builder: FormBuilder, private servicioService: ServicioService, private presupuestosService: PresupuestosService) {
        super(context);

        this.form = builder.group({
            'idDepartamento': [0, AztecaValidators.selected],
            'idServicio': [0, AztecaValidators.selected],
            'costoUnitario': [0, Validators.required],
            'cantidad': [0, Validators.required],
            'idPresupuesto': [''], // (control) => this.validatePresupuesto(this.departamento, control)],
            'idCategoriaPresupuestal': [0],// (control) => this.validateCategoria(this.departamento, control)],
        });

        this.filtrarImpuestos();

        //Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        this.form.controls['cantidad'].valueChanges.subscribe(value => {
            setTimeout(_ => this.onCantidadChange(value));
        });

        //Si se manejan series, la unidad de medida no será requerida
        this.form.controls['idDepartamento'].valueChanges.subscribe(value => {
            this.form.controls['idPresupuesto'].setValidators(null);
            this.form.controls['idCategoriaPresupuestal'].setValidators(null);

            if (this.departamento && this.departamento.tipoPresupuesto != "N") {
                this.form.controls['idPresupuesto'].setValidators(Validators.required);
                if (this.departamento && this.departamento.tipoPresupuesto == "C")
                    this.form.controls['idCategoriaPresupuestal'].setValidators(AztecaValidators.selected);
            }
            this.form.controls['idPresupuesto'].updateValueAndValidity();
            this.form.controls['idCategoriaPresupuestal'].updateValueAndValidity();
        });
    }

    /////////////////// METODOS ///////////////////////////////
    //validatePresupuesto(departamento: DepartamentoItem, control: FormControl) {
    //    if (departamento && departamento.tipoPresupuesto != "N" && control.value == "")
    //        return 'Debe establecerse un presupuesto';
    //    else
    //        return null;
    //}
    //validateCategoria(departamento: DepartamentoItem, control: FormControl) {
    //    if (departamento && departamento.tipoPresupuesto == "C" && control.value == "")
    //        return 'Debe establecer una categoria presupuestal';
    //    else
    //        return null;
    //}

    onServicioChanged(servicio: ServicioInfo) {
        if (servicio && servicio.identity>0) {
            this.detailCopy.servicio = servicio;
            //this.loadStock(this.detailCopy.articulo.articuloStock.identity);
            this.loadPresupuestos(this.departamento.id);
            this.servicioService.calcularCostoProveedor(servicio.identity, this.idProveedor, this.context.app.defaultCompany.identity, 0)
                .then(costo => {
                    this.detailCopy.costoUnitario = costo.costoBase;
                    this.detailCopy.costoNeto = costo.costoNeto;
                    this.detailCopy.descuentos = costo.descuentos;
                    this.detailCopy.impuestos = costo.impuestos;
                    this.detailCopy.importe = costo.costoBase * this.detailCopy.cantidad;
                    //this.costoUnitario = Globalize.formatNumber(this.detailCopy.costoUnitario, 'c');
                    //
                    this.trasladosCollection.sourceCollection = costo.impuestos;
                    this.retencionesCollection.sourceCollection = costo.impuestos;
                    this.filtrarImpuestos();
                })
                .catch(error => this.handleError(error));
        }
        this.trasladosCollection.sourceCollection = null;
        this.retencionesCollection.sourceCollection = null;
    }

    onCantidadChange(cantidad:number) {
        this.detailCopy.importe = this.detailCopy.costoUnitario * cantidad;
    }

    onCostoChange(costo: number) {
        if (this.detailCopy.idServicio > 0) {
            this.servicioService.calcularCostoProveedor(this.detailCopy.idServicio, this.idProveedor, this.context.app.defaultCompany.identity, costo)
                .then(costo => {
                    this.detailCopy.costoUnitario = costo.costoBase;
                    this.detailCopy.costoNeto = costo.costoNeto;
                    this.detailCopy.descuentos = costo.descuentos;
                    this.detailCopy.impuestos = costo.impuestos;
                    this.detailCopy.importe = costo.costoBase * this.detailCopy.cantidad;
                    //this.costoUnitario = Globalize.formatNumber(this.detailCopy.costoUnitario, 'c');
                    //
                    this.trasladosCollection.sourceCollection = costo.impuestos;
                    this.retencionesCollection.sourceCollection = costo.impuestos;
                    this.filtrarImpuestos();
                })
                .catch(error => this.handleError(error));
        }
    }

    onDepartamentoChanged(idDepartamento: number) {
        if (idDepartamento > 0) {
            this.departamento = this.departamentos.find(item => item.id == idDepartamento);
            this.detailCopy.departamento = this.departamento.nombre;
            if (this.departamento.tipoPresupuesto != 'N') {
                this.loadPresupuestos(idDepartamento);
            }
        }
    }

    onPresupuestoChanged(idPresupuesto: string) {
        this.loadCategoriasPresupuestales(idPresupuesto);
    }

    loadPresupuestos(idDepartamento: number) {
        if (this.departamento.tipoPresupuesto != 'N') {
            this.presupuestosService.loadPresupuestos(idDepartamento)
                .then(info => {
                    //Si es nuevo, solo que despliegue los presupuestos abiertos
                    if (this.detailCopy.id == 0) {
                        this.presupuestos = info.filter(prestos => prestos.workFlowStepTag == 'OPEN')
                    }
                    else
                        this.presupuestos = info;
                })
                .catch(error => this.handleError(error));
        }
    }

    loadCategoriasPresupuestales(idPresupuesto: string) {
        if (this.departamento.tipoPresupuesto == 'C' && idPresupuesto != "" && this.detailCopy.idServicio != 0) {
            this.presupuestosService.loadCategoriasConServicio(idPresupuesto, this.detailCopy.idServicio)
                .then(info => {
                    this.categoriasPresupuestales = info;
                })
                .catch(error => this.handleError(error));
        }
    }

    filtrarImpuestos() {
        this.trasladosCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.TRASLADO;
        };

        this.retencionesCollection.filter = (item: ImpuestoDetailInfo) => {
            return item.tipoAplicacion == TipoAplicacion.RETENCION;
        };
    }
    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
        this.form.reset({ idServicio: 0, costoUnitario: 0, idDepartamento: 0  });
        this.trasladosCollection = new CollectionView();
        this.retencionesCollection = new CollectionView();
        this.form.markAsPristine();

        if (this.departamentos.length > 0) {
            this.detailCopy.idDepartamento = this.departamentos[0].id;
            this.onDepartamentoChanged(this.departamentos[0].id);
        }
    }

    onLoadingItem(detail: RecepcionSinOCDetailServicio) {
        this.detailCopy = <RecepcionSinOCDetailServicio>this.copyItem(this.detail);

        if (this.departamentos.length > 0) {
            this.onDepartamentoChanged(detail.idDepartamento);
        }
                        
        this.trasladosCollection.sourceCollection = this.detailCopy.impuestos;
        this.retencionesCollection.sourceCollection = this.detailCopy.impuestos;
        this.filtrarImpuestos();
    }

    onUpdatingItem(detail: RecepcionSinOCDetailServicio) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
    }

    onEditCell(grid: FlexGrid, args: CellRangeEventArgs) {
        if (!grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cancel = true;
        }
    }

    onEditedCell(grid: FlexGrid, args: CellRangeEventArgs) {
        if (grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            grid.rows[args.row].dataItem.importe = grid.rows[args.row].dataItem.base * grid.rows[args.row].dataItem.tasaOCuota;
        }
    }

    onFormatCell(grid: FlexGrid, args: FormatItemEventArgs) {
        if (args.panel.cellType == CellType.Cell && !grid.rows[args.row].dataItem.impuesto.rango && grid.columns[args.col].binding == 'tasaOCuota') {
            args.cell.style.backgroundColor = '#C0C0C0';
        }
    }


}