﻿import { Component, Input, Output, EventEmitter,ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';
//Wijmo
import { Control, Globalize } from 'wijmo/wijmo';
//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState, EditableListComponent } from 'framework3/editors';
//Locales
import { RecepcionSinOCDetail } from '../../models/recepcion-sin-oc-detail';
import { RecepcionSinOCDetailArticulo } from '../../models/recepcion-sin-oc-detail-articulo';
//import { OrdenDetailServicio } from '../../models/orden-detail-servicio';
import { DepartamentoItem } from '../../models/departamento';

import { RecepcionSinOCArticulosEditor } from './recepciones-sin-oc-detail-articulos.editor';
import { RecepcionSinOCServiciosEditor } from './recepciones-sin-oc-detail-servicios.editor';
import { OrdenDetail } from '../../models/orden-detail';

//modulos del sistema
import { ImportesInfo } from '../../../accounting/models/importes';
import { Empaquetado } from '../../../inventory/models/empaquetado';
import { RecepcionSinOCDetailServicio } from '../../models/recepcion-sin-oc-detail-servicio';

@Component({
    selector: 'compras-recepcion-sin-oc-detail-editor',
    templateUrl: './recepciones-sin-oc-detail.editor.html'
})
export class RecepcionSinOCDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new RecepcionSinOCDetail();
    //Objeto original que se editará
    @Input() detail: RecepcionSinOCDetail = new RecepcionSinOCDetail();    
    @Input() idAlmacen: number = 0;
    @Input() departamentos: DepartamentoItem[] = [];
    
    //
    idProveedor: number = 0;
    totalCF: number = 0;
    totalRecepcion: number = 0;
    montoDiferencia: string = '$0.00';

    //
    @ViewChild('articulosList') articulosList: EditableListComponent;
    @ViewChild('serviciosList') serviciosList: EditableListComponent;
    //
    @ViewChild('articulosEditor') articulosEditor: RecepcionSinOCArticulosEditor;
    @ViewChild('serviciosEditor') serviciosEditor: RecepcionSinOCServiciosEditor;

    constructor(protected context: Context, private builder: FormBuilder) {
        super(context);

        this.form = builder.group({
            'montoDeDiferencia': [0, (control) => this.validDiferencia()]
            
        });
    }

    get importesComprobante() {
        let desglose: ImportesInfo = new ImportesInfo();

        if (this.detail.comprobante) {
            desglose.total = this.detail.comprobante.total;
            desglose.subtotal = this.detail.comprobante.subtotal;
            desglose.descuentos = this.detail.comprobante.descuentos;
            desglose.impuestosTrasladados = this.detail.comprobante.impuestosTrasladados;
            desglose.impuestosRetenidos = this.detail.comprobante.impuestosRetenidos;
            this.totalCF = desglose.total;
            //this.visualValidacion();
        }

        return desglose;
    }

    get importesRecepcion() {
        return this.calcularImportes();
    }
    /////////////////// METODOS ///////////////////////////////
    validDiferencia() {
        let resultado: string = 'No correspone el total detallado contra el total del comprobante fiscal';

        if (Math.abs(this.totalCF - this.totalRecepcion) < .50 && this.totalCF + this.totalRecepcion>0)//|| !this.recepcion.proveedor.comprobanteFiscal)
            resultado = null;
                
        return resultado;
    }

    calcularImportes(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        let elementos = this.combineDetail().filter(item => item.state != 'DELETED');

        elementos.forEach(item => {
            desglose.subtotal += item.costoUnitario * item.cantidad;
            desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidad;

            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        this.totalRecepcion = desglose.total;
        //this.visualValidacion();
        this.montoDiferencia = Globalize.formatNumber(this.totalRecepcion - this.totalCF, 'c');
        return desglose;
    }

    //Separar articulos y servicios
    separarDetail() {
    //    //
    //    this.articulos = this.recepcion.details.map(d => <RecepcionArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
    //    this.servicios = this.recepcion.details.map(d => <RecepcionServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
    //    //
    //    //this.detailsEditor.clientIndexStart = this.articulos.length + this.servicios.length;
    //    this.articulosEditor.clientIndexStart = this.articulos.length;
    //    this.serviciosEditor.clientIndexStart = this.servicios.length;
    }

    combineDetail(): OrdenDetail[] {
        let defaultDate: Date = new Date(1901, 1, 1);
        let newDetail: OrdenDetail[] = [];
        
        this.detailCopy.servicios.filter(f => f.state != "DELETED").forEach(s => newDetail.push(s));
        this.detailCopy.articulos.filter(f => f.state != "DELETED").forEach(a => {
            if (!a.articulo.articuloStock.manejaCaducidad)
                a.fechaCaducidad = defaultDate;
            newDetail.push(a);
        });
        return newDetail;
    }

    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
        this.idProveedor = this.detailCopy.comprobante.idProveedor
		this.form.reset({ sample: '' });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: RecepcionSinOCDetail ) {
        this.detailCopy = <RecepcionSinOCDetail>this.copyItem(this.detail);
        this.idProveedor = this.detailCopy.comprobante.idProveedor
        //this.articulos = this.detailCopy.articulos;
        //this.servicios = this.detailCopy.servicios;
        //separar el detalle en arreglo de articulos y servicios
        this.separarDetail();       
    }

    onUpdatingItem(detail: RecepcionSinOCDetail ) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }

        //this.detail.orden.details = this.combineDetail();

        this.savingItem.emit(this.detailCopy);
    }


    //Handlers Articulos / servicios

    ///////////// ARTICULOS /////////////////////////
    onAddingArticulo() {
        this.articulosEditor.detail = new RecepcionSinOCDetailArticulo();
        this.articulosEditor.show();
    }

    onEditingArticulo(detail: RecepcionSinOCDetailArticulo) {
        this.articulosEditor.detail = detail;
        //this.articulosEditor.onLoadingItem(detail);
        this.articulosEditor.show();
        //this.quantityEditor.unidadMedida = detail.unidadMedida;
        //this.quantityEditor.cantidad = detail.cantidad;
        //this.quantityEditor.unidades = detail.unidades;
        //this.quantityEditor.show();
    }

    onDeletingArticulo(detail: RecepcionSinOCDetailArticulo) {
        this.articulosList.deleteItem(detail);
    }

    onArticulosUpdated(updatedItems: RecepcionSinOCDetailArticulo[]) {
        this.detailCopy.articulos = updatedItems;
    }

    onSavingArticulo(detail: RecepcionSinOCDetailArticulo) {
        this.articulosList.updateItem(detail);
    }

    onSavingArticulos(details: RecepcionSinOCDetailArticulo[]) {
        this.articulosList.updateItems(details);
    }

    onUpdateQuantity(value: any) {
        let detail = <RecepcionSinOCDetailArticulo>this.articulosList.currentItem;
        if (detail.unidadMedida.paquete) {
            let cantidad = <Empaquetado>value;
            detail.cantidad = cantidad.paquetes;
            detail.unidades = cantidad.unidades;
        }
        else {
            detail.cantidad = <number>value;
        }
        this.articulosList.updateItem(detail);
    }

    ///////////// SERVICIOS /////////////////////////

    onAddingServicio() {
        this.serviciosEditor.detail = new RecepcionSinOCDetailServicio();
        this.serviciosEditor.show();
    }

    onEditingServicio(detail: RecepcionSinOCDetailServicio) {
        this.serviciosEditor.detail = detail;
        //this.serviciosEditor.onLoadingItem(detail);
        this.serviciosEditor.show();
    }

    onDeletingServicio(detail: RecepcionSinOCDetailServicio) {
        this.serviciosList.deleteItem(detail);
    }

    onServiciosUpdated(updatedServices: RecepcionSinOCDetailServicio[]) {
        this.detailCopy.servicios = updatedServices;
    }

    onSavingServicio(detail: RecepcionSinOCDetailServicio) {
        this.serviciosList.updateItem(detail);
    }

    onSavingServicios(details: RecepcionSinOCDetailServicio[]) {
        this.serviciosList.updateItems(details);
    }




    onTabSelected(args: any) {
        if (args.heading) {     
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


}
