﻿import { BasicEditor } from "framework3/editors";
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { CollectionView, Control, Globalize, EventArgs } from 'wijmo/wijmo';

//
import { Context } from 'framework3/core';
import { AlertsBarComponent } from 'framework3/controls'
//
import { PresupuestosService } from '../services/presupuestos.service';
import { PresupuestosInfo } from '../models/presupuestos';
import { PresupuestosDetail, CategoriaDetail } from '../models/presupuestos-detail';
import { FlexGrid  } from "wijmo/wijmo.grid";

@Component({
    selector: 'compras-presupuestos-cambio-editor',
    templateUrl: './presupuestos-cambio.editor.html'    
})

export class PresupuestosCambioEditor extends BasicEditor {
    presupuestos :PresupuestosInfo
    idPresupuesto: string = "";
    collectionView: CollectionView= new CollectionView();
    grid: FlexGrid;
    detailCurrent: PresupuestosDetail;
    idCategoriaSelected: number = 0;
    conceptoEdit: string = ""
    montoEdit: number = 0

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('detailsList') detailsList: FlexGrid;// EditableListComponent;
    @ViewChild('conceptoDialog') dialogConceptos: ModalDirective;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private presupuestosService: PresupuestosService) {

        super(context);

        this.form = formBuilder.group({
            '': [0, Validators.required],            

        });
        this.presupuestos = new PresupuestosInfo();

    }

    ngOnInit() {
        if (this.route.params) {
            this.route.params.forEach(param => {
                this.idPresupuesto = param['id'];                         
            });


        }
    }

    ngAfterViewInit() {
        setTimeout(_ => this.onLoadingDocument(this.idPresupuesto));
    }

    agregarConcepto(detail: PresupuestosDetail) {
        this.idCategoriaSelected = detail.categoriaPresupuestal.identity;
        this.detailCurrent = detail;
        //this.dialogArticulos.idCategoriaPresupuestal = detail.categoriaPresupuestal.identity;
        this.dialogConceptos.config.backdrop = false;
        this.dialogConceptos.show();
        Control.invalidateAll();
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.presupuestosService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.presupuestos = doc;
                this.collectionView.sourceCollection = this.presupuestos.details;

                //this.data = new CollectionView(this.presupuestos.details, {
                //    sortDescriptions: ['key'],
                //    groupDescriptions: ['key']
                //});

                //setTimeout(_ => {
                //    if (this.grid)
                //        this.grid.collapseGroupsToLevel(0);
                //});

                this.recalcularImportes();
                //this.detailsEditor.clientIndexStart = this.presupuestos.details.length;
                //if (this.presupuestos.status != DocumentStatus.DRAFT)
                //    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCellEditEnded(flex: any, e: any, cell: any) {
        this.recalcularDetalle();
    }
        
    recalcularDetalle() {
        let monto: number = 0;
        if (this.detailCurrent.conceptos.filter(search => search.state != 'DELETED').length > 0)
            monto += this.detailCurrent.conceptos.filter(search => search.state != 'DELETED').map(c => c.montoPresupuesto).reduce((sum, current) => sum + current);
        if (this.detailCurrent.articulos.filter(search => search.state != 'DELETED').length > 0)
            monto += this.detailCurrent.articulos.filter(search => search.state != 'DELETED').map(c => c.montoPresupuesto).reduce((sum, current) => sum + current);
        if (this.detailCurrent.servicios.filter(search => search.state != 'DELETED').length > 0)
            monto += this.detailCurrent.servicios.filter(search => search.state != 'DELETED').map(c => c.montoPresupuesto).reduce((sum, current) => sum + current);

        this.detailCurrent.montoPresupuesto = monto

        this.recalcularImportes();
        this.detailCurrent.state = "MODIFIED";

        this.detailsList.refreshCells(true);
        Control.invalidateAll();
    }

    recalcularImportes() {
        this.presupuestos.montoPresupuesto = this.presupuestos.details.map(c => c.montoPresupuesto).reduce((sum, current) => sum + current); //this.categoriasPresupuestales.map(c => c.montoPresupuesto).reduce((sum, current) => sum + current);
        this.presupuestos.montoUtilizado = this.presupuestos.details.map(c => c.montoUtilizado).reduce((sum, current) => sum + current);
        this.presupuestos.montoDisponible = this.presupuestos.details.map(c => c.montoDisponible).reduce((sum, current) => sum + current);
        this.presupuestos.montoApartado = this.presupuestos.details.map(c => c.montoApartado).reduce((sum, current) => sum + current);
    } 

    initialized(flex: FlexGrid) {
        var self = this;
        this.grid = flex;

        this.collectionView.currentChanged.addHandler((args: EventArgs) => {
            this.detailsList.refresh();
            this.detailsList.collapseGroupsToLevel(1);

            this.detailCurrent = self.collectionView.currentItem;

            let curRow = this.detailsList.rows[this.detailsList.selection.row];
            curRow.isCollapsed = false;
            curRow.isCollapsed = true;

        }, self);

        // start collapsed
        if (flex)
            flex.collapseGroupsToLevel(0);


    }

    onConceptoSelected(concepto: string, monto: number) {
        let bakItems: CategoriaDetail[] = this.detailCurrent.conceptos.slice();

        //items.forEach(articulo => {
        let searchItem = this.detailCurrent.conceptos.find(concep => concep.descripcion == concepto);
        if (searchItem == undefined) {
            let newCatDetail = new CategoriaDetail();
            newCatDetail._clientID = bakItems.length + 1;
            newCatDetail.idArticulo = 0;
            newCatDetail.descripcion = concepto;
            newCatDetail.montoPresupuesto = monto;
            newCatDetail.idServicio = 0;
            //newCatDetail.idPresupuestoDet = 0;
            bakItems.push(newCatDetail);
        }
        //});

        this.detailCurrent.conceptos = bakItems;
        this.detailCurrent.state = "MODIFIED";

        this.conceptoEdit = "";
        this.montoEdit = 0;
        this.recalcularDetalle();
    }

    //////////////////////////// Atributos de lectura ////////////////////////////////
    get montoPresupuesto() {
        if (this.presupuestos.montoPresupuesto == undefined)
            this.presupuestos.montoPresupuesto = 0;
        return Globalize.formatNumber(this.presupuestos.montoPresupuesto, 'c');
    }

    onConfirmClick() {//(args: EditorEventArgs) {
        //this.loadDetails();
        this.context.app.showSpinner();

        this.presupuestosService.saveChange(this.presupuestos)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                //this.reset();
                //if (!args.closeEditor && this.presupuestos.identity == '') {
                //    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                //}
                //else {
                //    if (args.closeEditor)
                this.router.navigate(['../../'], { relativeTo: this.route });
                //    else
                //        this.showAlert('La información ha sido actualizada con éxito', 'success');
                //}
                //this.presupuestos = updatedDoc;
            })
            .catch(error => {
                this.handleError(error)
            });
    }

    onCancelClick() {
        this.location.back();
    }

    handleError(error) {        
        this.context.app.hideSpinner();
        var message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}