﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';
import '../../../css/styles.css';
import * as wjcCore from 'wijmo/wijmo';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, AzTextboxComponent } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { PresupuestosInfo, PresupuestosItem } from '../models/presupuestos';
import { PresupuestosDetail, CategoriaDetail } from '../models/presupuestos-detail';
import { PresupuestosService } from '../services/presupuestos.service';
import { PresupuestosDetailEditor } from './parts/presupuestos-detail.editor';
import { PeriodoInfo } from '../../accounting/models/periodo';

import { CategoriasPresupuestalesService } from '../services/categorias-presupuestales.service';
import { ModalDirective } from 'ngx-bootstrap';
import * as _ from 'lodash';
import { Globalize, EventArgs  } from 'wijmo/wijmo';


//import { detachEmbeddedView } from '@angular/core/src/view';
//import { now } from 'moment';
//import { getDate } from 'ngx-bootstrap/chronos/utils/date-getters';

import { DepartamentoService } from '../services/departamento.service';
import { CategoriaPresupuestalArticulosListComponent } from '../dialogs/categorias-presupuestal-articulos-list.component';
import { CategoriaPresupuestalServiciosListComponent } from '../dialogs/categorias-presupuestal-servicios-list.component';
import { ArticuloItem } from '../models/articulo';
import { ServicioItem } from '../models/servicio';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//import { PresupuestoDetail } from '../../ models/presupuesto-detail';
import { SecurityContext } from '../models/security-context';
import { DepartamentoItem } from '../models/departamento';
import { CentralService } from '../services/central.service';
import { EmpresaInfo } from '../models/empresa';
import { EmpresaService} from '../services/empresa.service'

@Component({
    selector: 'azteca.compras-presupuestos-editor',
    templateUrl: './presupuestos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PresupuestosEditor)
        },
        {
            provide: DocumentService, useExisting: PresupuestosService
        }
    ]
})
export class PresupuestosEditor extends DocumentEditor {
    presupuestos = new PresupuestosInfo();
    //categoriasPresupuestales: PresupuestoCategoriaPresupuestalItem[] = [];    
    detailCurrent: PresupuestosDetail;
    collectionView = new TrackableCollection();
    
    conceptoEdit:string=""
    montoEdit: number = 0

    articulos = [];
    pageSizeArticulo: number = 20;
    totalRowsArticulo: number = 0;
    pageIndexArticulo: number = 0;
    private IdCategoriaSelected: number;
    public disabled: boolean = true;
    private idEjercicioTipo: number = 0;

    securityContext: SecurityContext[] = [];    
    departamentos: DepartamentoItem[] = [];

    detailMode = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandSingle];
    detailModeMulti = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandMulti];

    //
    
    @ViewChild('detailsList') detailsList: FlexGrid;// EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: PresupuestosDetailEditor;
    //@ViewChild('dialogArticulo') dialogArticulo: ModalDirective;
    @ViewChild('dialogArticulos') dialogArticulos: CategoriaPresupuestalArticulosListComponent;
    @ViewChild('dialogServicios') dialogServicios: CategoriaPresupuestalServiciosListComponent;
    @ViewChild('conceptoDialog') dialogConceptos: ModalDirective;
    
    @ViewChild('txtConcepto') txtConcepto: AzTextboxComponent;// EditableListComponent;
    @ViewChild('conceptosList') conceptosList: FlexGrid;// EditableListComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private presupuestosService: PresupuestosService,
        protected categoriaPresupuestalsvc: CategoriasPresupuestalesService,
        private departamentoService: DepartamentoService,
        private centralService: CentralService,
        private empresaService: EmpresaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'periodo': ['', Validators.required],
            'departamento': ['', Validators.required],
            'presupuesto': [''],
            'apartado': [''],
            'utilizado': [''],
            'disponible': [''],
            'montoPresupuesto':[0]
        });


        this.documentForm.controls['montoPresupuesto'].valueChanges.subscribe(value => {
            this.documentForm.controls['montoPresupuesto'].setValidators(null);            

            if (this.presupuestos.departamento && this.presupuestos.departamento.tipoPresupuesto == "D") {
                this.documentForm.controls['montoPresupuesto'].setValidators(Validators.required);
            }
            this.documentForm.controls['montoPresupuesto'].updateValueAndValidity();            
        });
    }

    
    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.loadDepartamentos();


        this.empresaService.getDetailFromCache(this.context.app.defaultCompany.identity)
            .then(info => {
                this.idEjercicioTipo = info.idEjercicioTipo;
            })
            .catch(error => this.handleError(error));
        
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.presupuestosService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.presupuestos = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.presupuestosService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.presupuestos = doc;
                //this.detailsEditor.clientIndexStart = this.presupuestos.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.presupuestosService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.presupuestos = doc;
                this.collectionView.sourceCollection = this.presupuestos.details;                
                this.recalcularImportes();
                //this.detailsEditor.clientIndexStart = this.presupuestos.details.length;
                if (this.presupuestos.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        //this.loadDetails();
        this.context.app.showSpinner();
        
        this.presupuestosService.update(this.presupuestos)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.presupuestos.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });                   
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else 
                        this.showAlert('La información ha sido actualizada con éxito', 'success');                      
                    
                }
                this.presupuestos = updatedDoc;

                this.collectionView.sourceCollection = this.presupuestos.details;
                //Si se dio guardar hay q actualizar el document detail ya actualizado con el id
                if (!args.closeEditor && this.detailCurrent != undefined) {
                    let detail: PresupuestosDetail = this.presupuestos.details.find(detail => detail.idCategoriaPresupuestal == this.detailCurrent.idCategoriaPresupuestal)
                    this.detailCurrent = detail;
                    this.detailCurrent.state = "MODIFIED";
                    this.getConceptoCollection(this.detailCurrent);
                    if (this.detailsList)
                        this.detailsList.refresh();
                    if (this.conceptosList)
                        this.conceptosList.refresh();

                }

            })
            .catch(error => this.handleError(error));
        
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.presupuestosService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.presupuestosService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }
       
    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    //onAddingDetail() {
    //    this.detailsEditor.detail = new PresupuestosDetail();
    //    this.detailsEditor.show();
    //}

    //onEditingDetail(detail: PresupuestosDetail) {
    //    this.detailsEditor.detail = detail;
    //    this.detailsEditor.show();
    //}

    //onDeletingDetail(detail: PresupuestosDetail) {
    //    this.detailsList.deleteItem(detail);
    //}

    //onListItemsUpdated(updatedDetails: PresupuestosDetail[]) {
    //    this.presupuestos.details = updatedDetails;
    //}

    //onSavingDetail(detail: PresupuestosDetail) {
    //    this.detailsList.updateItem(detail);
    //}


    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.presupuestos.identity);
            }
            else if (args.heading == 'Detalle') {
                //this.loadCategoriasPresupuestales();
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    //metodos y eventos del componente
    onPeriodoChanged(periodo: PeriodoInfo) {
        this.presupuestos.idPeriodo = periodo.identity;
        this.presupuestos.fechaPeriodo = periodo.fechaSys;
    }

    onDepartamentoItemChanged(idDepto: number)
    {
        
        this.departamentoService.getDetailFromCache(idDepto)
            .then(info => {
                this.presupuestos.departamento = info;//

                if (info.tipoPresupuesto == 'N') {
                    this.documentForm.reset();
                    this.showAlert('Este departamento no controla presupuesto', 'success');                      
                }
                else {
                    if (this.presupuestos.identity == "") {
                        this.presupuestos.details = [];
                        this.loadCategoriasPresupuestales();
                    }
                }
            })
            .catch(error => this.handleError(error)); 
    }

    agregarConcepto(detail: PresupuestosDetail) {
        this.IdCategoriaSelected = detail.categoriaPresupuestal.identity;
        this.detailCurrent = detail;
        //this.dialogArticulos.idCategoriaPresupuestal = detail.categoriaPresupuestal.identity;
        this.dialogConceptos.config.backdrop = false;
        this.dialogConceptos.show();
        Control.invalidateAll();      
    }

    agregarArticulos(detail: PresupuestosDetail) {
        this.IdCategoriaSelected = detail.categoriaPresupuestal.identity;
        this.detailCurrent = detail;
        this.dialogArticulos.idCategoriaPresupuestal = detail.categoriaPresupuestal.identity;
        this.dialogArticulos.show();
        Control.invalidateAll();        
    }

    agregarServicios(detail: PresupuestosDetail) {
        this.IdCategoriaSelected = detail.categoriaPresupuestal.identity;
        this.detailCurrent = detail;
        this.dialogServicios.idCategoriaPresupuestal = detail.categoriaPresupuestal.identity;
        this.dialogServicios.show();
        Control.invalidateAll();
    }

    onArticulosSelected(items: ArticuloItem[]) {
        let bakItems: CategoriaDetail[] = this.detailCurrent.articulos.slice();

        items.forEach(articulo => {
            let searchArt = this.detailCurrent.articulos.find(art => art.idArticulo == articulo.id);
            if (searchArt == undefined) {
                let newCatDetail = new CategoriaDetail();
                newCatDetail._clientID = bakItems.length + 1;
                newCatDetail.idArticulo = articulo.id;
                newCatDetail.descripcion = articulo.nombre;
                newCatDetail.idServicio = 0;
                //newCatDetail.idPresupuestoDet = 0;
                bakItems.push(newCatDetail);
            }
        });
                
        this.detailCurrent.articulos = bakItems;                
        this.detailCurrent.state = "MODIFIED";
    }    

    onConceptoSelected(concepto:string, monto:number) {
        let bakItems: CategoriaDetail[] = this.detailCurrent.conceptos.slice();
                
        //items.forEach(articulo => {
            let searchItem = this.detailCurrent.conceptos.find(concep => concep.descripcion == concepto);
            if (searchItem == undefined) {
                let newCatDetail = new CategoriaDetail();
                newCatDetail._clientID = bakItems.length + 1;
                newCatDetail.idArticulo = 0;
                newCatDetail.descripcion = concepto;
                newCatDetail.montoPresupuesto = monto;
                newCatDetail.idServicio = 0;
                //newCatDetail.idPresupuestoDet = 0;
                bakItems.push(newCatDetail);
            }
        //});

        

        this.detailCurrent.conceptos = bakItems;
        this.detailCurrent.state = "MODIFIED";

        this.conceptoEdit = "";
        this.montoEdit = 0;
               

        this.conceptosList.focus();        
        this.detailsList.collapseGroupsToLevel(0);
        

        //if (this.detailsList.selectedRows.length > 0) {
        //    this.detailsList.autoSizeRow(this.detailsList.selectedRows[0].index, true);

        //    this.conceptosList.refresh();
        //    this.conceptosList.scrollIntoView(this.detailCurrent.conceptos.length - 1, 0); //this.conceptosList.scrollIntoView(this.conceptosList.rows.length - 1, 0);
        //    this.conceptosList.select(this.detailCurrent.conceptos.length - 1, 0);
        //}

        //if (this.conceptosList.rows.length > 0) {
        //    this.conceptosList.scrollIntoView(this.conceptosList.rows.length - 1, -1)
        //    let rc = this.conceptosList.cells.getCellBoundingRect(this.conceptosList.rows.length - 1, 0, true);
        //    this.conceptosList.scrollPosition = new wjcCore.Point(this.conceptosList.scrollPosition.x, rc.bottom);
        //}        
                
        this.detailsList.refresh();
        this.recalcularDetalle();

        this.txtConcepto.textbox.nativeElement.focus();        
        
    }

    conceptosLoaded(a: any, b: any, c: any) {
        if (this.detailsList.selectedRows.length > 0) {
            this.detailsList.autoSizeRow(this.detailsList.selectedRows[0].index, true);
        }

        if (this.detailCurrent.conceptos.length > 0) {   
            if (this.conceptosList != undefined) {
                this.conceptosList.refresh();
                this.conceptosList.scrollIntoView(this.conceptosList.rows.length - 1, 0); //this.conceptosList.scrollIntoView(this.detailCurrent.conceptos.length - 1, 0);
                this.conceptosList.select(this.conceptosList.rows.length - 1, 0);
            }
        }
    }

    onServiciosSelected(items: ServicioItem[]) {
        let bakItems: CategoriaDetail[] = this.detailCurrent.servicios.slice();

        items.forEach(servicio => {
            let searchArt = this.detailCurrent.servicios.find(serv => serv.idArticulo == servicio.id);
            if (searchArt == undefined) {
                let newCatDetail = new CategoriaDetail();
                newCatDetail._clientID = bakItems.length + 1;
                newCatDetail.idServicio = servicio.id;
                newCatDetail.descripcion = servicio.nombre;
                newCatDetail.idArticulo = 0;
                //newCatDetail.idPresupuestoDet = 0;

                bakItems.push(newCatDetail);
            }
        });

        this.detailCurrent.servicios = bakItems;
        this.detailCurrent.state = "MODIFIED";
    }

    //////////////////////////// Atributos de lectura ////////////////////////////////
    get montoPresupuesto() {
        if (this.presupuestos.montoPresupuesto==undefined)
            this.presupuestos.montoPresupuesto = 0;
        return Globalize.formatNumber(this.presupuestos.montoPresupuesto , 'c');
    }

    get montoApartado() {
        if (this.presupuestos.montoApartado == undefined)
            this.presupuestos.montoApartado = 0;
        return Globalize.formatNumber(this.presupuestos.montoApartado, 'c');
    }

    get montoUtilizado() {
        if (this.presupuestos.montoUtilizado == undefined)
            this.presupuestos.montoUtilizado = 0;
        return Globalize.formatNumber(this.presupuestos.montoUtilizado, 'c');
    }

    get montoDisponible() {
        if (this.presupuestos.montoDisponible == undefined)
            this.presupuestos.montoDisponible = 0;
        return Globalize.formatNumber(this.presupuestos.montoDisponible, 'c');
    }


    /////////////////////////////////////////////////////////////////////////////////////

    loadDepartamentos() {
        this.departamentoService.loadListByUser(this.context.app.defaultSite.identity)
            .then(result => {                  
                //Muestra a los que el usuario tiene permisos y que son presupuestables
                this.departamentos = result.filter(presupuestables => presupuestables.tipoPresupuesto != 'N')  ;//
            })
            .catch(error => this.handleError(error));  

        //this.centralService.loadSecurityContext(this.context.app.defaultCompany.identity)
        //    .then(items => {
        //        this.securityContext = items;
        //        var result = _(this.securityContext).groupBy('idDepartamento').transform(function (result, current) {
        //            result.push({
        //                id: current[0].idDepartamento,
        //                nombre: current[0].departamento,

        //            });
        //        }, []).value();

        //        this.departamentos = result;//
        //    })
        //    .catch(error => this.handleError(error));  

        //this.departamentos = result;
        if (this.departamentos.length > 0 && this.presupuestos.identity == '')
            this.presupuestos.idDepartamento = this.departamentos[0].id;
    }

    loadCategoriasPresupuestales() {
        //if (this.presupuestos.idDepartamento > 0 && this.categoriasPresupuestales.length==0) {
        //this.presupuestosService.getCategoriasPresupuestales(this.presupuestos.idDepartamento, this.presupuestos.identity)
        this.departamentoService.getCategoriasPresupuestales(this.presupuestos.idDepartamento)
            .then(info => {
                let details: PresupuestosDetail[]=[];
                info.forEach(categoriaDepto => {                        
                    let detail = new PresupuestosDetail();
                    detail.idCategoriaPresupuestal = categoriaDepto.id;
                    detail.categoriaPresupuestal.identity = categoriaDepto.id;
                    detail.categoriaPresupuestal.nombre = categoriaDepto.nombre;
                    detail.montoPresupuesto = 0; detail.montoUtilizado = 0; detail.montoApartado = 0; detail.montoDisponible = 0;
                    details.push(detail);                        
                });                    
                this.presupuestos.details = details;
                this.collectionView.sourceCollection = this.presupuestos.details;
            })
            .catch(error => this.handleError(error));        
    }
    
    initialized(sender: FlexGrid, args: any) {
        var self = this;

        this.collectionView.currentChanged.addHandler((args: EventArgs) => {
            this.detailsList.refresh();
            this.detailsList.collapseGroupsToLevel(0);

            if (self.collectionView.currentItem != null) {
                this.detailCurrent = self.collectionView.currentItem;               
            }

        }, self);

    }

    onCellEditEnded(flex: any, e: any, cell: any) {
        this.recalcularDetalle();
    }
        
    recalcularDetalle() {
        let monto: number = 0;
        if (this.detailCurrent) {
            if (this.detailCurrent.conceptos.filter(search => search.state != 'DELETED').length > 0)
                monto += this.detailCurrent.conceptos.filter(search => search.state != 'DELETED').map(c => c.montoPresupuesto).reduce((sum, current) => sum + current);
            if (this.detailCurrent.articulos.filter(search => search.state != 'DELETED').length > 0)
                monto += this.detailCurrent.articulos.filter(search => search.state != 'DELETED').map(c => c.montoPresupuesto).reduce((sum, current) => sum + current);
            if (this.detailCurrent.servicios.filter(search => search.state != 'DELETED').length > 0)
                monto += this.detailCurrent.servicios.filter(search => search.state != 'DELETED').map(c => c.montoPresupuesto).reduce((sum, current) => sum + current);

            this.detailCurrent.montoPresupuesto = monto

            this.recalcularImportes();
            this.detailCurrent.state = "MODIFIED";

            this.detailsList.refreshCells(true);
            Control.invalidateAll();
        }
    }

    recalcularImportes() {                
        this.presupuestos.montoPresupuesto= this.presupuestos.details.map(c => c.montoPresupuesto).reduce((sum, current) => sum + current); //this.categoriasPresupuestales.map(c => c.montoPresupuesto).reduce((sum, current) => sum + current);
        this.presupuestos.montoUtilizado  = this.presupuestos.details.map(c => c.montoUtilizado).reduce((sum, current) => sum + current);
        this.presupuestos.montoDisponible = this.presupuestos.details.map(c => c.montoDisponible).reduce((sum, current) => sum + current);
        this.presupuestos.montoApartado = this.presupuestos.details.map(c => c.montoApartado).reduce((sum, current) => sum + current);
    } 


    getConceptoCollection(item:PresupuestosDetail) {
        return item.conceptos;
    }

}

