﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//
import * as _ from 'lodash';
//
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control,Globalize } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { FileInfo } from 'framework3/general';

//Locales
import { RecepcionSinOCInfo, RecepcionSinOCItem } from '../models/recepcion-sin-oc';
import { RecepcionSinOCDetail } from '../models/recepcion-sin-oc-detail';
import { RecepcionSinOCService } from '../services/recepcion-sin-oc.service';
import { OrdenDetail } from '../models/orden-detail';
import { OperacionTipoItem } from '../models/operacion-tipo'
import { OperacionTipoService } from '../services/operacion-tipo.service'

import { RecepcionSinOCDetailEditor } from './parts/recepciones-sin-oc-detail.editor';

import { CFDValiationResult} from '../models/cfd-validation-result'
import { CentralService } from '../services/central.service';
import { CentroRecepcionService } from '../services/centro-recepcion.service';
import { CentralItem } from '../models/central';
import { DepartamentoItem } from '../models/departamento';
import { SecurityContext } from '../models/security-context';
import { ProveedorInfo } from '../models/proveedor';
import { ProveedorService } from '../services/proveedor.service';

import { ImportesInfo } from '../../accounting/models/importes';
import { DivisaService } from '../../accounting/services/divisa.service';

import { BeneficiarioInfo } from '../../treasury/models/beneficiario';

import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';
import { ComprobanteFiscalInfo,ComprobanteFiscalItem } from '../models/comprobante-fiscal';
import { DivisaItem } from '../../accounting';
//import { ComprobanteInfo } from '../../accounting';
//import { SecureContext } from 'tls';
import { ProveedorSelectorComponent } from '../components/proveedor-selector.component';
import { ContribuyenteService } from '../../accounting/services/contribuyente.service';


@Component({
    selector: 'compras-recepciones-sin-oc-editor',
    templateUrl: './recepciones-sin-oc.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RecepcionesSinOCEditor)
        },
        {
            provide: DocumentService, useExisting: RecepcionSinOCService
        }
    ]
})
export class RecepcionesSinOCEditor extends DocumentEditor {
    recepcionSinOC = new RecepcionSinOCInfo();
    securityContext: SecurityContext[] = [];
    centrales: CentralItem[] = [];
    departamentos: DepartamentoItem[] = [];
    centrosRecepcion: CentralItem[] = [];

    origenesCF: { id: string, nombre: string }[] = [];
    idProveedor: number = 0;  
    idAlmacen: number = 0;
    selectedFile: FileInfo;

    operacionesTipo:OperacionTipoItem[] = []; 

    //
    @ViewChild(AlertsBarComponent) alertsBarComprobante: AlertsBarComponent;
    @ViewChild('cargaComprobanteFiscalDialog') origenCFDialog: ModalDirective;
    @ViewChild('comprobantesList') comprobantesListDialog: ModalDirective;
    @ViewChild('uploadDialog') uploadDialog: ModalDirective;
    @ViewChild('selProveedor') selProveedor: ProveedorSelectorComponent;

    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: RecepcionSinOCDetailEditor;


    origenCF: string = "L";
    //
    totalComprobantes: number = 0;
    totalRecepciones: number = 0;
    montoDiferencia: string = '$0.00';
    idDivisaDefault: number;
    tipoCambio: string;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private recepcionSinOCService: RecepcionSinOCService,
        private centralService: CentralService,
        private comprobanteFiscalService: ComprobanteFiscalService,
        private centroRecepcionService: CentroRecepcionService,
        private operacionTipoService: OperacionTipoService,
        private divisaService: DivisaService,
        private contribuyenteService: ContribuyenteService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idCentral': [0, AztecaValidators.selected],
            'idDepartamento': [0, AztecaValidators.selected],
            'idCentroRecepcion': [0, AztecaValidators.selected],
            'idBeneficiario': [0, AztecaValidators.selected],
            'idOperacionTipo': [0, AztecaValidators.selected],
            'idDivisa': [0, AztecaValidators.selected],
            'idProveedor': [0],
            'origenCF': [''],           
            'montoDeDiferencia': [0, (control) => this.validDiferencia()]
        });

        ////Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        //this.documentForm.controls['idCentral'].valueChanges.subscribe(value => {
        //    setTimeout(_ => this.loadCentrosRecepcion(value));
        //});

        ////Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        //this.documentForm.controls['idCentroRecepcion'].valueChanges.subscribe(value => {
        //    setTimeout(_ => this.loadDepartamentos(value));
        //});

    }
    

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
        this.loadCentrales();
        this.loadOperacionesTipo();
        this.getDivisaDefault(false);

        this.origenesCF =
            [
                { "id": 'L', "nombre": "Ruta Local" },
                { "id": 'A', "nombre": "Almacen digital" },
            ];
    }
        
    onCreatingDocument() {
        
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.recepcionSinOCService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.recepcionSinOC = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.recepcionSinOCService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.recepcionSinOC = doc;
                this.detailsEditor.clientIndexStart = this.recepcionSinOC.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.recepcionSinOCService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
               
                this.recepcionSinOC = doc;
                this.loadCentrosRecepcion(doc.idCentralCompras);
                this.loadDepartamentos(doc.idCentroRecepcion);

                this.detailsEditor.clientIndexStart = this.recepcionSinOC.details.length;
                if (this.recepcionSinOC.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();

                this.documentForm.controls['montoDeDiferencia'].markAsPristine();
                this.documentForm.markAsPristine();
                this.calcularImportes();
                this.tipoCambio = doc.tipoCambio.toString();
               
                this.documentForm.controls['montoDeDiferencia'].updateValueAndValidity();

              
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.recepcionSinOCService.update(this.recepcionSinOC)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.recepcionSinOC.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.recepcionSinOC = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.recepcionSinOCService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.recepcionSinOCService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.recepcionSinOC.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.origenCFDialog.config.backdrop = false;
        this.idProveedor = 0;
        this.origenCFDialog.show();
    }

    onEditingDetail(detail: RecepcionSinOCDetail) {        
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: RecepcionSinOCDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: RecepcionSinOCDetail[]) {
        this.recepcionSinOC.details = updatedDetails;
    }

    onSavingDetail(detail: RecepcionSinOCDetail) {
        this.detailsList.updateItem(detail);
    }

    /////////////////////// HANDLERS //////////////////////////////////

    onFileChanged(file: FileInfo) {
        this.selectedFile = file;
    }

    onValidateFile() {
        this.uploadDialog.hide();
        this.validateFile();
    }

    onOrigenCFSelected() {

        if (this.origenCF == "A" && this.idProveedor == 0)
            this.alertsBarComprobante.showAlert("Debe seleccionar un proveedor ", 'danger');
        else {
            this.origenCFDialog.hide();

            // Ver que va a cargar
            //this.comprobantesListDialog.config.backdrop = false;
            if (this.origenCF == "A")
                this.comprobantesListDialog.show();
            else {
                this.uploadDialog.config.backdrop = false;
                this.uploadDialog.show();
            }
        }

    }

    onProveedorChanged(proveedor: ProveedorInfo) {
        if (proveedor) {
            this.idProveedor = proveedor.identity;
            this.contribuyenteService.inListaNegraEFO(proveedor.contribuyente.rfc, proveedor.idCompany)
                .then(result => {
                    if (result == true) {
                        this.showAlert("El contribuyente esta registrado como EFO", "danger");
                        this.selProveedor.clear();
                    }
                })
                .catch(error => this.handleError(error));
        }
    }
    
    onBeneficiarioChanged(beneficiario: BeneficiarioInfo) {
        
    }

    onComprobanteSelected(comprobante: ComprobanteFiscalItem) {       
        this.comprobanteFiscalService.getDetail(comprobante.id)
            .then(comp => {
                this.addDetail(this.parseComprobanteItem(comp));
            })
            .catch(error => this.handleError(error));
    }

    onDivisaChanged(divisa: DivisaItem) {
        if (this.recepcionSinOC.identity == "") {
            this.recepcionSinOC.details = [];
            //this.recepcionSinOC..sourceCollection = [];
            if (this.recepcionSinOC.idDivisa == this.idDivisaDefault) {
                this.recepcionSinOC.tipoCambio = 1;
                this.tipoCambio = this.recepcionSinOC.tipoCambio.toString();
            }
            else {
                this.divisaService.loadTiposCambio(this.context.app.defaultSite.identity)
                    .then(tipos => {
                        let tCambio = tipos.find(search => search.idDivisaOrigen == this.recepcionSinOC.idDivisa && search.idDivisaDestino == divisa.id);
                        if (tCambio == undefined) {
                            this.showAlert("No está definido el tipo de cambio para la divisa seleccionada", "danger");
                            this.tipoCambio = null;
                            this.recepcionSinOC.idDivisa = null;
                            this.recepcionSinOC.tipoCambio = 0;
                        }
                        else {
                            this.recepcionSinOC.tipoCambio = tCambio.precioOtro;
                            this.tipoCambio = tCambio.precioOtro.toString();
                        }
                    });
            }
        }
    }

    /////////////////////// METODOS DE CARGA INICIAL///////////////////////////////////   
    loadCentrales() {
        //this.centralService.loadListByUser(this.context.app.defaultSite.identity)
        //    .then(items => this.centrales = items)
        //    .catch(error => this.handleError(error));
        this.centralService.loadSecurityContext(this.context.app.defaultCompany.identity)
            .then(items => {
                this.securityContext = items;
                var result = _(this.securityContext).groupBy('idCentral').transform(function (result, current) {
                    result.push({
                        id: current[0].idCentral,
                        nombre: current[0].central,

                    });
                }, []).value();

                this.centrales = result;
            })
            .catch(error => this.handleError(error));
    }

    loadCentrosRecepcion(idCentral: number) {
        //this.centroRecepcionService.loadListByDepartamentoAndUser(idDepartamento, this.context.app.defaultSite.identity)
        //    .then(items => this.centrosRecepcion = items)
        //    .catch(error => this.handleError(error));
        var result = _(this.securityContext.filter(item => item.idCentral == idCentral)).groupBy('idCentroRecepcion').transform(function (result, current) {
            result.push({
                id: current[0].idCentroRecepcion,
                nombre: current[0].centroRecepcion,
            });
        }, []).value();

        this.centrosRecepcion = result;
        if (this.centrosRecepcion.length > 0 && this.recepcionSinOC.identity == '')
            this.recepcionSinOC.idCentroRecepcion = this.centrosRecepcion[0].id;
    }

    loadDepartamentos(idCentroRecepcion: number) {
        if (idCentroRecepcion) {
            let item: SecurityContext = this.securityContext.find(item => item.idCentroRecepcion == idCentroRecepcion);
            if (item)
                this.idAlmacen = item.idAlmacen;

            //this.departamentoService.loadListByCentralUser(idCentral, this.context.app.defaultSite.identity)
            //    .then(items => this.departamentos = items)
            //    .catch(error => this.handleError(error));

            var result = _(this.securityContext.filter(item => item.idCentral == this.recepcionSinOC.idCentralCompras && item.idCentroRecepcion == idCentroRecepcion)).groupBy('idDepartamento').transform(function (result, current) {
                result.push({
                    id: current[0].idDepartamento,
                    nombre: current[0].departamento,
                    tipoPresupuesto: current[0].tipoPresupuesto,
                    capturaDetallada: current[0].capturaDetallada
                });
            }, []).value();

            this.centroRecepcionService.getDetailByID(idCentroRecepcion)
                .then(centro => {
                    this.idAlmacen = centro.idAlmacen;
                })
                .catch(error => this.handleError(error));

            this.departamentos = result;
            if (this.departamentos.length > 0 && this.recepcionSinOC.identity == '')
                this.recepcionSinOC.idDepartamento = this.departamentos[0].id;
        }
    }

    loadOperacionesTipo() {
        this.operacionTipoService.loadListByUser(this.context.app.defaultSite.identity)
            .then(operaciones => {
                this.operacionesTipo = operaciones;
            })
            .catch(error => this.handleError(error));
    }
    /////////////////////// METODOS DE CARGA DE COMPROBANTES///////////////////////////////////   
        
    validateFile() {
        this.context.app.showSpinner();
        this.comprobanteFiscalService.validate(this.context.app.defaultCompany.identity, this.selectedFile)
            .then(result => {
                this.context.app.hideSpinner();
                this.cathComprobantes(result);                 
                //this.validatedFiles = result;
                //this.comprobantesCollection.sourceCollection = result;
            })
            .catch(error => this.handleError(error));
    }
    
    cathComprobantes(ValidCFDcomprobantes: CFDValiationResult[]) {
        let promises: any[] = [];

        ValidCFDcomprobantes.forEach(validCFD => {
            if (validCFD.valid) {                                                         
                this.context.app.showSpinner();    
                let savePromise = this.comprobanteFiscalService.save(0, this.context.app.defaultCompany.identity, this.context.app.defaultSite.identity, validCFD.comprobante.comprobanteBase.contenidoXML)
                    .then(comprobante => {                        
                        this.context.app.hideSpinner();
                        this.idProveedor = comprobante.proveedor.identity;
                        validCFD.comprobante.proveedor = comprobante.proveedor;
                        this.addDetail( this.parseComprobanteItem(comprobante));
                    })
                    .catch(error => this.handleError(error));
            }
            else
                this.showAlert("Algunos comprobantes no se pudieron validar correctamente:[" + validCFD.validationMessage + "]", "danger")
        });
    }            

    parseComprobanteItem(info: ComprobanteFiscalInfo) {
        let item = new ComprobanteFiscalItem();
        item.id = info.identity;
        item.proveedor= info.proveedor.nombre;
        item.rfcEmisor = info.comprobanteBase.emisor.rfc;
        item.nombreEmisor= info.comprobanteBase.emisor.nombre;
        item.uuid= info.comprobanteBase.timbreFiscal.uuid;
        item.serie= info.comprobanteBase.serie;
        item.folio= info.comprobanteBase.folio;
        item.fechaComprobante= info.fechaSys;
        item.subtotal=info.comprobanteBase.subTotal;
        item.descuentos=info.comprobanteBase.descuentos;
        item.impuestosTrasladados=info.comprobanteBase.impuestos.totalTraslados;
        item.impuestosRetenidos = info.comprobanteBase.impuestos.totalRetenciones;
        item.total = info.comprobanteBase.total;
        item.idProveedor = info.proveedor.identity;
        return item;
    }

    addDetail(comprobante: ComprobanteFiscalItem) {

        let exists: RecepcionSinOCDetail = this.recepcionSinOC.details.find(det => det.idComprobante == comprobante.id);

        if (exists == null) {

            let detalle = new RecepcionSinOCDetail();

            detalle.idComprobante = comprobante.id;
            detalle.comprobante = comprobante;

            let newDetails: RecepcionSinOCDetail[] = this.recepcionSinOC.details.slice();
            if (detalle._clientID == 0) {
                detalle._clientID = this.recepcionSinOC.details.length + 1;
                newDetails.push(detalle)
            }
            else {
                let index: number = newDetails.findIndex(item => item._clientID == detalle._clientID);
                newDetails[index] = detalle;
            }

            this.recepcionSinOC.details = newDetails;
        }
        else {
            this.showAlert("El comprobantes ya ha sido agregado a una partida", "warning");
        }

    }


    //////////////////////////CALCULO DE TOTALES ////////////////////////////////////

    get importesComprobantes() {
        let desglose: ImportesInfo = new ImportesInfo();

        this.recepcionSinOC.details.filter(det => det.state != "DELETED") .forEach(detail => {
            desglose.total += detail.comprobante.total;
            desglose.subtotal += detail.comprobante.subtotal;
            desglose.descuentos += detail.comprobante.descuentos;
            desglose.impuestosTrasladados += detail.comprobante.impuestosTrasladados;
            desglose.impuestosRetenidos += detail.comprobante.impuestosRetenidos;
        });
        this.totalComprobantes = desglose.total;
        //this.visualValidacion();

        return desglose;
    }

    get importesRecepciones() {
        return this.calcularImportes();
    }
    
    validDiferencia() {
        let resultado: string = 'No correspone el total detallado contra el total del comprobante fiscal';

        if (Math.abs(this.totalComprobantes - this.totalRecepciones) < .05 && this.totalComprobantes + this.totalRecepciones > 0)//|| !this.recepcion.proveedor.comprobanteFiscal)
            resultado = null;

        return resultado;
    }

    calcularImportes(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        let elementos = this.combineDetail().filter(item => item.state != 'DELETED');

        elementos.forEach(item => {
            desglose.subtotal += item.costoUnitario * item.cantidad;
            desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidad;

            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        this.totalRecepciones = desglose.total;
        //this.visualValidacion();
        this.montoDiferencia = Globalize.formatNumber(this.totalComprobantes - this.totalRecepciones, 'c');
        return desglose;
    }

    combineDetail(): OrdenDetail[] {
        let defaultDate: Date = new Date(1901, 1, 1);
        let newDetail: OrdenDetail[] = [];

        this.recepcionSinOC.details.filter(item => item.state != 'DELETED').forEach(detail => {
            detail.servicios.filter(f => f.state != "DELETED").forEach(s => newDetail.push(s));
            detail.articulos.filter(f => f.state != "DELETED").forEach(a => {
                if (!a.articulo.articuloStock.manejaCaducidad)
                    a.fechaCaducidad = defaultDate;
                newDetail.push(a);
            });
        });
                
        return newDetail;
    }

    //Metodos generales
    getDivisaDefault(creating: boolean) {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
        .then(r => {
            this.recepcionSinOC.idDivisa = r.identity;
            this.idDivisaDefault = r.identity;
        })
        .catch(error => this.handleError(error));
        }

}

