﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, Globalize } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
//import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs, BasicEditor } from 'framework3/editors';
//import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { StockInfo, AlmacenItem, AlmacenInfo, AlmacenService, DetailQuantityEditor, Empaquetado } from '../../inventory';
//
import { RecepcionInfo, RecepcionItem } from '../models/recepcion';
import { RecepcionDetail } from '../models/recepcion-detail';
import { RecepcionArticuloDetail } from '../models/recepcion-articulo-detail';
import { RecepcionServicioDetail } from '../models/recepcion-servicio-detail';
import { RecepcionService } from '../services/recepcion.service';
//import { RecepcionDetailEditor } from './parts/recepciones-detail.editor';
import { RecepcionArticulosDetailEditor } from './parts/recepcion-articulos-detail.editor';
import { RecepcionServiciosDetailEditor } from './parts/recepcion-servicios-detail.editor';

import { ImportesInfo } from '../../accounting/models/importes';
import { CentroRecepcionService } from '../services/centro-recepcion.service';
import { CentroRecepcionInfo } from '../models/centro-recepcion';
import { OrdenService } from '../services/orden.service';
import { OrdenInfo, OrdenItem } from '../models/orden';
import { OrdenDetail } from '../models/orden-detail';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service'
import { ComprobantesListComponent } from '../components/comprobantes-dialog.component'
import { OrdenDetailArticulo } from '../models/orden-detail-articulo';
import { OrdenDetailServicio } from '../models/orden-detail-servicio';
//import { state } from '@angular/animations';
//import { getDateOffset } from 'ngx-bootstrap/chronos/units/offset';


@Component({
    selector: 'compras-recepciones-editor',
    templateUrl: './recepciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RecepcionesEditor)
        },
        {
            provide: DocumentService, useExisting: RecepcionService
        }
    ]
})
export class RecepcionesEditor extends DocumentEditor {
    recepcion = new RecepcionInfo();
    orden = new OrdenInfo();
    comprobante: ComprobanteFiscalInfo;

    centroRecepcion = new CentroRecepcionInfo();
    idAlmacen: number = 0;
    //
    articulos: RecepcionArticuloDetail[] = [];
    servicios: RecepcionServicioDetail[] = [];//
    //
    @ViewChild('articulosList') articulosList: EditableListComponent;
    @ViewChild('serviciosList') serviciosList: EditableListComponent;
    //
    @ViewChild('articulosEditor') articulosEditor: RecepcionArticulosDetailEditor;
    @ViewChild('serviciosEditor') serviciosEditor: RecepcionServiciosDetailEditor;
    //
    @ViewChild('comprobantesList') comprobantesList: ComprobantesListComponent;
    @ViewChild(DetailQuantityEditor) quantityEditor: DetailQuantityEditor;
    editarCantidades: boolean = true;

    totalOC: number = 0;
    totalCF: number = 0;

    get proveedorText() {
        return this.recepcion.proveedor.code + '-' + this.recepcion.proveedor.nombre;
    }

    get importesRecepcion() {        
        return this.importesDesglosados();
    }

    get importesComprobante() {
        let desglose: ImportesInfo = new ImportesInfo();

        if (this.comprobante) {
            desglose.total = this.comprobante.comprobanteBase.total;
            desglose.subtotal = this.comprobante.comprobanteBase.subTotal;
            desglose.descuentos = this.comprobante.comprobanteBase.descuentos;
            desglose.impuestosTrasladados = this.comprobante.comprobanteBase.impuestos.totalTraslados;
            desglose.impuestosRetenidos = this.comprobante.comprobanteBase.impuestos.totalRetenciones;
            this.totalCF = desglose.total;  
            this.visualValidacion();
        }
        
        return desglose;
    }

    colorBotonDiferencia: string = "red";
    montoDiferencia: string = '$0.00';

    visualValidacion() {
        this.montoDiferencia=Globalize.formatNumber(this.totalOC - this.totalCF, 'c');

        if (this.comprobante && this.totalOC == this.totalCF) {
            if (this.colorBotonDiferencia != "green")
                this.colorBotonDiferencia = "green";
        }
        else {
            if (this.colorBotonDiferencia != 'red')
                this.colorBotonDiferencia = "red";
        }
    }
       
    validDiferencia() {
        let resultado: string = 'No correspone el total recibido contra el total fiscal';

        //si se asigno el comprobante o el proveedor no lo requiere
        if (this.comprobante || !this.recepcion.proveedor.comprobanteFiscal)
            if (Math.abs(this.totalOC - this.totalCF) < .05 || !this.recepcion.proveedor.comprobanteFiscal)
                resultado = null;

        return resultado;
    }

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private recepcionService: RecepcionService,
        private ordenService: OrdenService,
        private centroRecepcionService: CentroRecepcionService,
        private comprobanteFiscalService: ComprobanteFiscalService,
        private cdRef: ChangeDetectorRef) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idCentroRecepcion': [0, Validators.required],
            'montoDeDiferencia': ['', (control) => this.validDiferencia()]
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {
        this.applyPolicies();
    }

    onCreatingDocument() {
        this.articulos = [];
        this.servicios = [];
        //Cargar parámetros que se proporcionan cuando es un nuevo documento
        this.route.params.forEach(params => {
            this.recepcion.idCentroRecepcion = +params['idCentroRecepcion'];
            this.recepcion.idOrdenTit = params['idOrden'];

        });

        setTimeout(_ => this.cargarInfomacionEntidades());
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.recepcionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.recepcion = doc;
                this.setReadonlyMode();
                this.articulos = this.recepcion.details.map(d => <RecepcionArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.servicios = this.recepcion.details.map(d => <RecepcionServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.recepcionService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.recepcion = doc;
                //
                this.articulos = this.recepcion.details.map(d => <RecepcionArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.servicios = this.recepcion.details.map(d => <RecepcionServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
                //
                //this.detailsEditor.clientIndexStart = this.articulos.length;
                this.articulosEditor.clientIndexStart = this.articulos.length;
                this.serviciosEditor.clientIndexStart = this.servicios.length;

            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.recepcionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.recepcion = doc;

                //separar el detalle en arreglo de articulos y servicios
                this.separarDetail();

                if (this.recepcion.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();

                this.loadComprobante(this.recepcion.idComprobante);

                setTimeout(_ => this.refreshValidity());

            })
            .catch(error => this.handleError(error));
    }
    
    onSavingDocument(args: EditorEventArgs) {
        //this.argsEditorEvent = args;
        
        this.recepcion.details = this.combineDetail();
        this.recepcion.idOperacionTipo = 1;  //quemado que es la recepcion de OC

        //esperar wait
        this.validateDetail()
            .then(value => {
                if (value == true) {

                    this.context.app.showSpinner();
                    this.recepcionService.update(this.recepcion)
                        .then(updatedDoc => {
                            this.context.app.hideSpinner();
                            //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                            this.reset();
                            this.recepcion = updatedDoc;
                            //separar el detalle en arreglo de articulos y servicios
                            this.separarDetail();

                            if (!args.closeEditor && this.recepcion.identity == '') {
                                this.router.navigate(['../../../', updatedDoc.identity, updatedDoc.idCentroRecepcion, updatedDoc.idOrdenTit], { relativeTo: this.route });
                            }
                            else {
                                if (args.closeEditor)
                                    this.router.navigate( ['../../../'], { relativeTo: this.route });
                                    //this.location.back;
                                else
                                    this.showAlert('La información ha sido actualizada con éxito', 'success');
                            }
                            this.recepcion = updatedDoc;
                        })
                        .catch(error => this.handleError(error));
                }
            })
            .catch (error => this.handleError(error));

    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.recepcionService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.recepcionService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onCancelingEdit() {
        this.location.back();
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.recepcion.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    //onAddingDetail() {
    //    this.detailsEditor.detail = new RecepcionDetail();
    //    this.detailsEditor.show();
    //}

    //onEditingDetail(detail: RecepcionDetail) {
    //    this.detailsEditor.detail = detail;
    //    this.detailsEditor.show();
    //}

    //onDeletingDetail(detail: RecepcionDetail) {
    //    this.detailsList.deleteItem(detail);
    //}

    //onListItemsUpdated(updatedDetails: RecepcionDetail[]) {
    //    this.recepcion.details = updatedDetails;
    //}

    //onSavingDetail(detail: RecepcionDetail) {
    //    this.detailsList.updateItem(detail);
    //}



    ///////////// ARTICULOS /////////////////////////
    onAddingArticulo() {
        this.articulosEditor.detail = new RecepcionArticuloDetail();
        this.articulosEditor.show();
    }

    onEditingArticulo(detail: RecepcionArticuloDetail) {
        this.articulosEditor.detail = detail;
        this.articulosEditor.onLoadingItem(detail);
        this.articulosEditor.show();
        //this.quantityEditor.unidadMedida = detail.unidadMedida;
        //this.quantityEditor.cantidad = detail.cantidad;
        //this.quantityEditor.unidades = detail.unidades;
        //this.quantityEditor.show();
    }

    onDeletingArticulo(detail: RecepcionArticuloDetail) {
        if(this.orden.idAnticipo.trim() == '') //Si tiene anticipo, se debe de recibir la orden completa
            this.articulosList.deleteItem(detail);
        else
            this.showAlert("La orden tiene aplicado un anticipo y requiere que se reciba completa", 'warning');
    }

    onArticulosUpdated(updatedItems: RecepcionArticuloDetail[]) {
        this.articulos = updatedItems;
        this.importesDesglosados()
    }

    onSavingArticulo(detail: RecepcionArticuloDetail) {
        this.articulosList.updateItem(detail);
        this.importesDesglosados()
    }

    onSavingArticulos(details: RecepcionArticuloDetail[]) {
        this.articulosList.updateItems(details);
        this.importesDesglosados()
    }

    onUpdateQuantity(value: any) {
        let detail = <RecepcionArticuloDetail>this.articulosList.currentItem;
        if (detail.unidadMedida.paquete) {
            let cantidad = <Empaquetado>value;
            detail.cantidad = cantidad.paquetes;
            detail.unidades = cantidad.unidades;
        }
        else {
            detail.cantidad = <number>value;
        }
        this.articulosList.updateItem(detail);
    }

    ///////////// SERVICIOS /////////////////////////

    onAddingServicio() {
        this.serviciosEditor.detail = new RecepcionServicioDetail();
        this.serviciosEditor.show();
    }

    onEditingServicio(detail: RecepcionServicioDetail) {
        this.serviciosEditor.detail = detail;
        this.serviciosEditor.show();
    }

    onDeletingServicio(detail: RecepcionDetail) {
        if (this.orden.idAnticipo.trim() == '') //Si tiene anticipo, se debe de recibir la orden completa
            this.serviciosList.deleteItem(detail);
        else
            this.showAlert("La orden tiene aplicado un anticipo y requiere que se reciba completa", 'warning');
    }

    onServiciosUpdated(updatedServices: RecepcionServicioDetail[]) {
        this.servicios = updatedServices;
        this.importesDesglosados();
    }

    onSavingServicio(detail: RecepcionServicioDetail) {
        this.serviciosList.updateItem(detail);
        this.importesDesglosados()
    }

    onSavingServicios(details: RecepcionServicioDetail[]) {
        this.serviciosList.updateItems(details);
        this.importesDesglosados()
    }


    //
    ///metodos locales

    cargarInfomacionEntidades() {
        //this.proveedorService.getDetailByID(this.recepcion.idProveedor)
        //    .then(proveedor => {
        //        this.recepcion.proveedor = proveedor;
        //    })
        //    .catch(error => this.handleError(error));

        this.ordenService.getDetail(this.recepcion.idOrdenTit)
            .then(orden => {
                this.orden = orden;
                this.recepcion.proveedor = orden.proveedor;
                this.recepcion.centroRecepcion = this.centroRecepcion.nombre;
                this.recepcion.fechaOrden = orden.fechaSys;
                this.recepcion.idDivisa = orden.idDivisa;
                this.recepcion.tipoCambio = orden.tipoCambio;
                this.recepcion.idProveedor = orden.idProveedor;

                let articulos: RecepcionArticuloDetail[] = [];
                let servicios: RecepcionServicioDetail[] = [];//

                //Si la orden tiene un anticipo, no permite editar las cantidades para que se reciba completa la orden
                if (this.orden.idAnticipo.trim() != "")
                    this.editarCantidades = false;

                //Cargamos los articulos con saldo
                this.orden.details.map(d => <OrdenDetailArticulo>d).filter(d => d.idArticulo && d.idArticulo > 0 && d.saldo > 0)
                    .forEach(item => {
                        //Vemos a ver si maneja series
                        if (item.articulo.articuloStock.manejaSeries) {
                            for (let count = 1; count <= item.saldo; count++) {
                                let newArt: RecepcionArticuloDetail = this.parseArticulo(item);
                                newArt.unidadMedida = item.unidadMedida;
                                newArt._clientID = articulos.length + 1;
                                newArt.cantidad = 1;
                                articulos.push(newArt);
                            }
                        }
                        else {
                            let newArt: RecepcionArticuloDetail = this.parseArticulo(item);
                            newArt._clientID = articulos.length + 1;
                            articulos.push(newArt);
                        }
                    });

                // cargamos los servicios pendientes de la orden
                this.orden.details.map(d => <OrdenDetailServicio>d).filter(d => d.idServicio && d.idServicio > 0 && d.saldo > 0)
                    .forEach(item => {
                        let newSvc: RecepcionServicioDetail = this.parseServicio(item);
                        newSvc._clientID = servicios.length + 1;
                        servicios.push(newSvc);
                    });

                //
                this.articulos = articulos;
                this.servicios = servicios;
                this.articulosList.updateItems(articulos);
                this.serviciosList.updateItems(servicios);
                //

                this.centroRecepcionService.getDetailByID(this.recepcion.idCentroRecepcion)
                    .then(centro => {
                        this.centroRecepcion = centro
                        this.recepcion.centroRecepcion = centro.nombre;
                        this.idAlmacen = centro.idAlmacen;
                    })
                    .catch(error => this.handleError(error));

                //Verificar si la OC tiene relacionado una Factura
                this.ordenService.getComprobanteRelacionado(orden.identity)
                    .then(comprobante => {                        
                        if (comprobante) {
                            //this.loadComprobante(comprobanteItem.id)
                            this.comprobante = comprobante;
                            this.recepcion.idComprobante = comprobante.identity;
                            this.recepcion.fechaComprobante = comprobante.fechaSys;
                        }
                    })
                    .catch(error => this.handleError(error));
                

                setTimeout(_ => this.refreshValidity());
            })
            .catch(error => this.handleError(error));

    }

    parseArticulo(item: OrdenDetailArticulo): RecepcionArticuloDetail {
        let detail: RecepcionArticuloDetail = new RecepcionArticuloDetail();
        detail._clientID = this.articulos.length + 1;
        let cantidad = item.saldo;
        if (item.articulo.articuloStock.manejaSeries) {
            cantidad = 1;
        }

        detail.state = 'ADDED'; detail.idOrdenDetRef = item.id;        
        detail.idArticulo = item.idArticulo; detail.articulo = item.articulo, detail.cantidad = cantidad; detail.unidades = 0; detail.idUnidadMedida = item.idUnidadMedida; detail.unidadMedida = item.unidadMedida;
        detail.costoUnitario = item.costoUnitario; detail.costoNeto = item.costoNeto; detail.importe = cantidad * item.costoUnitario, detail.descuentos = item.descuentos; detail.impuestos = item.impuestos; detail.idDepartamento = item.idDepartamento;

        detail.numSerie = ""; detail.fechaCaducidad = new Date(); detail.idLote = 0;
        return detail;
    }
    parseServicio(item: OrdenDetailServicio): RecepcionServicioDetail {
        let detail: RecepcionServicioDetail = new RecepcionServicioDetail();
        detail._clientID = this.servicios.length + 1;
        detail.state = 'ADDED'; detail.idOrdenDetRef = item.id;
        detail.idServicio = item.idServicio; detail.servicio = item.servicio, detail.cantidad = item.saldo; detail.idUnidadMedida = item.idUnidadMedida; detail.unidadMedida = item.unidadMedida;
        detail.costoUnitario = item.costoUnitario; detail.costoNeto = item.costoNeto; detail.importe = item.importe, detail.descuentos = item.descuentos; detail.impuestos = item.impuestos; detail.idDepartamento = item.idDepartamento;
        return detail;
    }

    private refreshValidity() {
        this.editor.valid = this.documentForm.valid;
    }

    //Separar articulos y servicios
    separarDetail() {
        //
        this.articulos = this.recepcion.details.map(d => <RecepcionArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
        this.servicios = this.recepcion.details.map(d => <RecepcionServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
        //
        //this.detailsEditor.clientIndexStart = this.articulos.length + this.servicios.length;
        this.articulosEditor.clientIndexStart = this.articulos.length;
        this.serviciosEditor.clientIndexStart = this.servicios.length;
    }

    combineDetail(): RecepcionDetail[] {
        let defaultDate: Date = new Date(1901, 1, 1);
        let newDetail: RecepcionDetail[] = [];
        this.servicios.forEach(s => newDetail.push(s));
        this.articulos.forEach(a => {
            if (!a.articulo.articuloStock.manejaCaducidad)
                a.fechaCaducidad= defaultDate;
            newDetail.push(a);
        });
        return newDetail;
    }

    importesDesglosados(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        let elementos = this.combineDetail().filter(item => item.state != 'DELETED');

        elementos.forEach(item => {
            desglose.subtotal += item.costoUnitario * item.cantidad;
            desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidad;

            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        this.totalOC = desglose.total;
        this.visualValidacion();
        return desglose;
    }

    addAsociar() {        
        this.comprobantesList.show();
    }

    onComprobanteSelected(comprobanteItem: ComprobanteFiscalItem) {        
        this.loadComprobante(comprobanteItem.id)
    }

    loadComprobante(idComprobante: string) {
        this.comprobanteFiscalService.getDetail(idComprobante)
            .then(item => {
                this.comprobante = item;
                this.recepcion.idComprobante = item.identity;
                this.recepcion.fechaComprobante = item.fechaSys;
                //this.recepcion.uuid = item.uuid;

                //this.documentForm.get("montoDeDiferencia").valid;                
            })
            .catch(error => this.handleError(error));
    }

    async validateDetail() {
        return <boolean>await this.checkDetails();        
    };

    checkDetails() {
        let result: boolean = true;

        //validamos que todos los articulos tengan serie en caso de que se maneje
        if (this.recepcion.details.filter(detail => detail.idArticulo > 0).map(detail => <RecepcionArticuloDetail>detail).filter(item => item.articulo.articuloStock.manejaSeries && item.numSerie.trim()=='' && item.state != 'DELETED').length > 0){
            this.showAlert("Existen detalles de articulos configurados con manejo de Serie que no se han establecido", 'warning');
            result= false;
        }
        
        return result;
    }

}

