﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, AzComboboxComponent, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { RequisicionInfo, RequisicionItem } from '../models/requisicion';
import { TipoRequisicionItem } from '../models/tipo-requisicion';
import { RequisicionDetail } from '../models/requisicion-detail';
import { RequisicionService } from '../services/requisicion.service';
import { RequisicionDetailEditor } from './parts/requisiciones-detail.editor';
import { DepartamentoItem } from '../models/departamento';
import { DepartamentoService } from '../services/departamento.service';
import { TipoRequisicionService } from '../services/tipo-requisicion.service';
import { CentroRecepcionItem } from '../models/centro-recepcion';
import { CentroRecepcionService } from '../services/centro-recepcion.service';
import { ArticuloInfo, ArticuloItem } from '../models/articulo';
import { ArticuloService } from '../services/articulo.service';
import { ProveedorService } from '../services/proveedor.service';
import { PresupuestosService } from '../services/presupuestos.service';
import { PresupuestosItem, PresupuestosInfo } from '../models/presupuestos';
import { CentralItem } from '../models/central'
import { CentralService } from '../services/central.service'
import { SecurityContext } from '../models/security-context'
import * as _ from 'lodash';

import { CategoriasPresupuestalesItem } from '../models/categorias-presupuestales';

//Inventario
import { ArticuloStockInfo, UnidadMedidaItem, StockInfo, LoteInfo, SerieInfo, UnidadMedidaService, AlmacenService, MetodoValuacion } from '../../inventory';
import { ServicioInfo } from '../models/servicio';
import { CentroCostoItem } from '../models/centro-costo';
import { CentroCostoService } from '../services';
import { ArticuloSelectorComponent } from '../components/articulo-selector.component';
//import { ComboBox } from 'wijmo/wijmo.input';


@Component({
    selector: 'compras-requisiciones-editor',
    templateUrl: './requisiciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RequisicionesEditor)
        },
        {
            provide: DocumentService, useExisting: RequisicionService
        }
    ]
})
export class RequisicionesEditor extends DocumentEditor {
    requisicion = new RequisicionInfo();
    idGrupoRequisicion: string= "";
    requisicionesTipo: TipoRequisicionItem[]=[];

    securityContext: SecurityContext[] = [];
    centrales: CentralItem[] = [];
    centrosRecepcion: CentroRecepcionItem[] = [];
    departamento: DepartamentoItem;
    departamentos: DepartamentoItem[] = [];    
    unidadesMedida: { id: number, nombre: string }[]=[]; //UnidadMedidaItem[] = [];
    presupuestos: PresupuestosItem[] = [];
    presupuesto: PresupuestosInfo = new PresupuestosInfo();
    centrosCostos: CentroCostoItem[] = [];


    //presupuesto: PresupuestosItem = new PresupuestosItem();
    categoriasPresupuestales: CategoriasPresupuestalesItem[] = [];


    articuloServicio: string = "A";
    articuloServicioItems: { id: string, nombre: string } [] = [
        { "id": 'A', "nombre": "Articulo" },
        { "id": 'S', "nombre": "Servicio" }
    ];

    //    
    articulo: ArticuloInfo = new ArticuloInfo();    
    //

    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: RequisicionDetailEditor;
    
    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private requisicionService: RequisicionService,
        private departamentoService: DepartamentoService,
        private centroRecepcionService: CentroRecepcionService,
        private unidadMedidaService: UnidadMedidaService,
        private articuloService: ArticuloService,
        private presupuestoService: PresupuestosService,
        private centralService: CentralService,
        private tipoRequisicionService: TipoRequisicionService,
        private proveedorService: ProveedorService,
        private centroCostosSvc: CentroCostoService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idTipoRequisicion': [0, AztecaValidators.selected],
            'idCentral': [0, AztecaValidators.selected],
            'idDepartamento': [0, AztecaValidators.selected],
            'idCentroRecepcion': [0, AztecaValidators.selected],
            'articuloServicio': ['A', Validators.required],
            'idArticulo': [0],
            'idServicio': [0],
            'cantidad': [0,Validators.required],
            'idUnidadMedida': [0],
            'observaciones': [''],            
            'idPresupuesto': [''],
            'idCategoriaPresupuestal': [0],
            'centroCosto': [0],
        });

        //////////Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        //////////this.documentForm.controls['idCentral'].valueChanges.subscribe(value => {
        //////////    setTimeout(_ => this.loadCentrosRecepcion(value));
        //////////});
        
        //////////Suscribirse a cambios en central ya que esta determina varios comportamientos en los campos y validaciones
        ////////this.documentForm.controls['idCentroRecepcion'].valueChanges.subscribe(value => {
        ////////    setTimeout(_ => this.loadDepartamentos(value));
        ////////});

        this.documentForm.controls['idDepartamento'].valueChanges.subscribe(value => {
            //this.onDepartamentoChanged(value)
            //setTimeout(_ => {
                this.documentForm.controls['idPresupuesto'].setValidators(null);
                this.documentForm.controls['idCategoriaPresupuestal'].setValidators(null);

                if (this.departamento && this.departamento.tipoPresupuesto != "N") {
                    this.documentForm.controls['idPresupuesto'].setValidators(AztecaValidators.selected);
                    if (this.departamento && this.departamento.tipoPresupuesto == "C")
                        this.documentForm.controls['idCategoriaPresupuestal'].setValidators(AztecaValidators.selected);
                }
                this.documentForm.controls['idPresupuesto'].updateValueAndValidity();
                this.documentForm.controls['idCategoriaPresupuestal'].updateValueAndValidity();
            //});
        });

        //this.documentForm.controls['idPresupuesto'].valueChanges.subscribe(value => {
            //this.onPresupuestoChanged(value);
            //setTimeout(_ => { 
            //   this.documentForm.controls['idPresupuesto'].updateValueAndValidity();
            //    this.documentForm.controls['idCategoriaPresupuestal'].updateValueAndValidity();
            //});
        //});

        //Hacer requerido el método de valuación sólo cuando se manejen lotes
        this.documentForm.controls['articuloServicio'].valueChanges.subscribe(value => {
            if (value == 'A') {
                this.documentForm.controls['idArticulo'].setValidators(AztecaValidators.selected);
                this.documentForm.controls['idUnidadMedida'].setValidators(AztecaValidators.selected);
                this.documentForm.controls['idServicio'].setValue('');
                this.documentForm.controls['idServicio'].setValidators(null);
            }
            else
            {
                this.documentForm.controls['idServicio'].setValidators(AztecaValidators.selected);
                this.documentForm.controls['idArticulo'].setValue('');
                this.documentForm.controls['idArticulo'].setValidators(null);
                this.documentForm.controls['idUnidadMedida'].setValue('');
                this.documentForm.controls['idUnidadMedida'].setValidators(null);
            }

            this.documentForm.controls['idArticulo'].updateValueAndValidity();
            this.documentForm.controls['idServicio'].updateValueAndValidity();
            if (value == "A")
                this.documentForm.controls['idUnidadMedida'].updateValueAndValidity();

            console.log("Log UM: " + this.documentForm.controls['idUnidadMedida'].value)
        });



        //////this.documentForm.controls['idArticulo'].valueChanges.subscribe(value => {
            //////if (this.articulo) {
            //////    setTimeout(_ => this.loadUnidadesMedida(this.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo));

                //////////////this.documentForm.controls['idPresupuesto'].setValidators(null);
                //////////////this.documentForm.controls['idCategoriaPresupuestal'].setValidators(null);

                //////////////if (this.departamento && this.departamento.tipoPresupuesto != "N") {
                //////////////    this.documentForm.controls['idPresupuesto'].setValidators(AztecaValidators.selected);
                //////////////    if (this.departamento && this.departamento.tipoPresupuesto == "C")
                //////////////        this.documentForm.controls['idCategoriaPresupuestal'].setValidators(AztecaValidators.selected);
                //////////////}
                //////////////this.documentForm.controls['idPresupuesto'].updateValueAndValidity();
                //////////////this.documentForm.controls['idCategoriaPresupuestal'].updateValueAndValidity();
            //}
        //////});

        ////////this.documentForm.controls['idServicio'].valueChanges.subscribe(value => {
                            
        ////////    this.documentForm.controls['idPresupuesto'].setValidators(null);
        ////////    this.documentForm.controls['idCategoriaPresupuestal'].setValidators(null);

        ////////    if (this.departamento && this.departamento.tipoPresupuesto != "N") {
        ////////        this.documentForm.controls['idPresupuesto'].setValidators(AztecaValidators.selected);
        ////////        if (this.departamento && this.departamento.tipoPresupuesto == "C")
        ////////            this.documentForm.controls['idCategoriaPresupuestal'].setValidators(AztecaValidators.selected);
        ////////    }
        ////////    this.documentForm.controls['idPresupuesto'].updateValueAndValidity();
        ////////    this.documentForm.controls['idCategoriaPresupuestal'].updateValueAndValidity();
            
        ////////});

    }

    onConfigureDocument(docType: DocumentTypeInfo) {
        this.route.params.forEach((params: Params) => {
            this.idGrupoRequisicion = params['idGrupoRequisicion'];
        });

        this.applyPolicies();
        this.loadCentrales();     
        this.loadTiposRequisiciones();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.requisicionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.requisicion = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.requisicionService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.requisicion = doc;
                this.detailsEditor.clientIndexStart = this.requisicion.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.requisicionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();  
                this.articuloServicio = doc.idArticulo > 0 ? 'A' : 'S';                      
                this.requisicion = doc; 
                
                //setTimeout(_ => {
                    this.loadCentrosRecepcion(doc.idCentral);
                    //setTimeout(_ => {
                        this.loadDepartamentos(doc.idCentroRecepcion);
                        //setTimeout(_ => {
                            this.departamento = this.departamentos.find(item => item.id == doc.idDepartamento);

                            if (this.departamento.tipoPresupuesto != 'N') {
                                this.loadPresupuestos(doc.idDepartamento);
                                //setTimeout(_ => {
                                    console.log("onLoadingDocument - cargando categorias");
                                    this.loadCategoriasPresupuestales(doc.idPresupuesto);
                                //});                                
                                this.documentForm.controls['idPresupuesto'].updateValueAndValidity();
                                this.documentForm.controls['idCategoriaPresupuestal'].updateValueAndValidity();
                                //console.log("Presupuesto: " + doc.idPresupuesto + " - " + this.documentForm.controls['idPresupuesto'].value);
                                //console.log("Categoria Presupuestal: " + doc.idCategoriaPresupuestal + " - " + this.documentForm.controls['idCategoriaPresupuestal'].value);
                            }
                        //});
                    //});
                //});                

                //if (doc.idArticulo > 0 && this.articulo) {
                //    this.selArticulo.loadItem(doc.idArticulo);
                //    setTimeout(_ => {
                //        this.articulo = this.selArticulo.selectedItem;
                //        this.loadUnidadesMedida(this.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
                //        setTimeout(_ => {
                //            this.documentForm.controls['idUnidadMedida'].updateValueAndValidity();
                //        });

                //        if (this.cboUM)
                //            this.cboUM.selectedValue = this.requisicion.idUnidadMedida;
                //    });
                //}
                this.detailsEditor.clientIndexStart = this.requisicion.details.length;                

                if (this.requisicion.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.requisicion.idGrupoRequisiciones = this.idGrupoRequisicion;
        if (this.articuloServicio == 'A') this.requisicion.idServicio = 0;
        if (this.articuloServicio == 'S') this.requisicion.idArticulo = 0;

        this.requisicionService.update(this.requisicion)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.requisicion.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.requisicion = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.requisicionService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.requisicionService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.requisicion.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    onCancelingEdit() {
        this.location.back();
    }

    ///////////// DETALLE DEL DOCUMENTO /////////////////////////
    onAddingDetail() {
        this.detailsEditor.detail = new RequisicionDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: RequisicionDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: RequisicionDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: RequisicionDetail[]) {
        this.requisicion.details = updatedDetails;
    }

    onSavingDetail(detail: RequisicionDetail) {
        this.detailsList.updateItem(detail);
    }

    loadTiposRequisiciones() {
        this.tipoRequisicionService.loadListByUser(this.context.app.defaultSite.identity)
            .then(items => {                
                this.requisicionesTipo = items;
            })
            .catch(error => this.handleError(error));
    }

    loadCentrales() {
        //this.centralService.loadListByUser(this.context.app.defaultSite.identity)
        //    .then(items => this.centrales = items)
        //    .catch(error => this.handleError(error));
        this.centralService.loadSecurityContext(this.context.app.defaultCompany.identity)
            .then(items => {
                this.securityContext = items;
                var result = _(this.securityContext).groupBy('idCentral').transform(function (result, current) {
                    result.push({
                        id: current[0].idCentral,
                        nombre: current[0].central,

                    });
                }, []).value();

                this.centrales = result;
            })
            .catch(error => this.handleError(error));
    }

    loadCentrosRecepcion(idCentral: number) {
        //this.centroRecepcionService.loadListByDepartamentoAndUser(idDepartamento, this.context.app.defaultSite.identity)
        //    .then(items => this.centrosRecepcion = items)
        //    .catch(error => this.handleError(error));
        var result = _(this.securityContext.filter(item => item.idCentral == idCentral)).groupBy('idCentroRecepcion').transform(function (result, current) {
            result.push({
                id: current[0].idCentroRecepcion,
                nombre: current[0].centroRecepcion,
            });
        }, []).value();

        this.centrosRecepcion = result;
        if (this.centrosRecepcion.length > 0 && this.requisicion.identity == '')
            this.requisicion.idCentroRecepcion = this.centrosRecepcion[0].id;

    }

    loadDepartamentos(idCentroRecepcion: number) {
        //this.departamentoService.loadListByCentralUser(idCentral, this.context.app.defaultSite.identity)
        //    .then(items => this.departamentos = items)
        //    .catch(error => this.handleError(error));
        var result = _(this.securityContext.filter(item => item.idCentral == this.requisicion.idCentral && item.idCentroRecepcion == idCentroRecepcion)).groupBy('idDepartamento').transform(function (result, current) {
            result.push({
                id: current[0].idDepartamento,
                nombre: current[0].departamento,
                tipoPresupuesto: current[0].tipoPresupuesto,
                capturaDetallada: current[0].capturaDetallada
            });
        }, []).value();

        this.departamentos = result;
        //this.requisicion.idDepartamento = 0;
        //this.requisicion.idPresupuesto = '';
        //this.requisicion.idCategoriaPresupuestal = 0;
        if (this.departamentos.length > 0) {
            if (this.requisicion.identity == '')
                this.requisicion.idDepartamento = this.departamentos[0].id;
            else if(this.departamentos.length==1)
                this.onDepartamentoChanged(this.requisicion.idDepartamento);
        }
        
        
    }

    //Handlers
    onArticuloChanged(articulo: ArticuloInfo) {
        if (articulo) {
            if (articulo.active == false && this.requisicion.identity == "") {
                this.showAlert("El articulo seleccionado esta inactivo.", 'warning');
                this.requisicion.idArticulo = 0;
            }
            else {
                this.articulo = articulo;
                //setTimeout(_ => {
                    this.documentForm.controls['idPresupuesto'].updateValueAndValidity();
                    this.loadUnidadesMedida(this.articulo.articuloStock.unidadMedida.idUnidadMedidaTipo);
                    //setTimeout(_ => {
                        this.loadCategoriasPresupuestales(this.requisicion.idPresupuesto);
                        this.loadCentrosCosto(articulo.identity);
                    //});
                //})
            }
        }
        else
            this.unidadesMedida = [];
    }

    onServicioChanged(servicio: ServicioInfo) {
        if (servicio.active == false && this.requisicion.identity == "") {
            this.showAlert("El servicio seleccionado esta inactivo.", 'warning');
            this.requisicion.idServicio = 0;
        }
        else {
            //setTimeout(_ => {
                this.loadCategoriasPresupuestales(this.requisicion.idPresupuesto);
                this.loadCentrosCosto(servicio.identity);
            //});   
        }
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        if (idTipoUnidad > 0) {
            //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
            //en caso contrario, cargar lista de unidades de medida del mismo tipo
            if (this.articulo.articuloStock.manejaSeries) {
                let unidadArticulo = this.articulo.articuloStock.unidadMedida;
                let newUnidad: UnidadMedidaItem = {
                    id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base, nombre: unidadArticulo.nombre,
                    factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
                };
                let newUnidades = [];
                newUnidades.push(newUnidad);
                this.unidadesMedida = newUnidades;
                this.requisicion.idUnidadMedida = unidadArticulo.identity;    
                console.log("Maneja series");
            }
            else {
                this.unidadesMedida = [];
                let requestProveedor = new SearchRequest();
                requestProveedor.pageIndex = 0;
                requestProveedor.pageSize = 0;
                requestProveedor.paged = false;
                this.proveedorService.searchProveedorByArticulo(this.articulo.identity, this.context.app.defaultSite.identity, requestProveedor)
                    //this.unidadMedidaService.loadList(idTipoUnidad)
                    .then(unidadesMedida => {
                        let UMs: { id: number, nombre: string }[] = [];
                        //this.unidadesMedida = unidadesMedida;
                        unidadesMedida.rows.forEach(item => {
                            let um = UMs.find(x => x.id == item.idUnidadMedida)
                            if (um == null || um == undefined) { 
                                UMs.push({ id: item.idUnidadMedida, nombre: item.unidadMedida });
                            }
                        });              
                        this.unidadesMedida = UMs;

                        if(this.requisicion.identity=="")
                            this.requisicion.idUnidadMedida = this.articulo.articuloStock.idUnidadMedida;                        

                    })
                    .catch(error => this.handleError(error));
            }
           
        }
    }

    onDepartamentoChanged(idDepartamento: number) {
        if (idDepartamento > 0) {
            this.categoriasPresupuestales = [];   
            if(this.requisicion.identity=="") this.requisicion.idPresupuesto = '';
            this.departamento = this.departamentos.find(item => item.id == idDepartamento);            
            if (this.departamento && this.departamento.tipoPresupuesto != 'N') {
                this.loadPresupuestos(idDepartamento);
            }
        }
    }

    onPresupuestoChanged(idPresupuesto: string) {        
        //setTimeout(_ => {
            console.log("onPresupuestoChanged");
            this.loadCategoriasPresupuestales(idPresupuesto);
        //});           
    }


    loadPresupuestos(idDepartamento: number) {
        this.presupuestos = [];
        this.categoriasPresupuestales = [];
        if (this.departamento.tipoPresupuesto != 'N') {
            this.presupuestoService.loadPresupuestos(idDepartamento)
                .then(info => {
                    if (this.requisicion.identity == '' && this.requisicion.status==DocumentStatus.DRAFT) //nuevo
                        this.presupuestos = info.filter(opens => opens.workFlowStepTag == 'OPEN'); //solo los presupuesto abiertos 
                    else
                        this.presupuestos = info; //mostrar todos los presupuestos

                    if (this.presupuestos.length > 0) {
                        if (this.requisicion.identity == '')
                            this.requisicion.idPresupuesto = this.presupuestos[0].id;
                        else
                            //setTimeout(_ => {
                                console.log("onLoadPresupuestos");
                                this.loadCategoriasPresupuestales(this.requisicion.idPresupuesto);
                            //});                            
                    }
                })
                .catch(error => this.handleError(error));        }
    }

  

    loadCategoriasPresupuestales(idPresupuesto: string) {
        if (idPresupuesto != null && idPresupuesto.trim() != '') {
            if (this.departamento && this.departamento.tipoPresupuesto == 'C' && idPresupuesto != "" && this.requisicion.idArticulo != undefined && this.requisicion.idArticulo != 0) {
                this.presupuestoService.loadCategoriasConArticulo(idPresupuesto, this.requisicion.idArticulo)
                    .then(info => {
                        this.categoriasPresupuestales = info;
                    })
                    .catch(error => this.handleError(error));
            }

            if (this.departamento && this.departamento.tipoPresupuesto == 'C' && idPresupuesto != "" && this.requisicion.idServicio != undefined && this.requisicion.idServicio != 0) {
                this.presupuestoService.loadCategoriasConServicio(idPresupuesto, this.requisicion.idServicio)
                    .then(info => {
                        this.categoriasPresupuestales = info;
                    })
                    .catch(error => this.handleError(error));
            }
        }
        else
            this.categoriasPresupuestales = [];
    }

    loadCentrosCosto(id:number) {
        this.centroCostosSvc.loadList(this.context.app.defaultSite.identity, this.articuloServicio, id)
            .then(info => {
                this.centrosCostos = info;
            }).catch(error => this.handleError(error));
    }


}

