﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, Output, EventEmitter } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FlexGrid, CellRangeEventArgs, FormatItemEventArgs, CellType } from 'wijmo/wijmo.grid';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { RequisicionInfo, RequisicionItem } from '../models/requisicion';
import { RequisicionDetail } from '../models/requisicion-detail';
import { RequisicionService } from '../services/requisicion.service';
import { Color } from 'wijmo/wijmo';

@Component({
    selector: 'compras-requisiciones-manager',
    templateUrl: './requisiciones.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RequisicionesManager)
        },
        {
            provide: DocumentService, useExisting: RequisicionService
        }
    ]
})
export class RequisicionesManager extends DocumentManager {
    @Input() requisiciones: RequisicionItem[];
    @Input() idGrupoRequisiciones: string;

    @Input() showFilterPanel: boolean = true;
    @Input() standAlone: boolean = true;

    @Output() refreshListParent: EventEmitter<void> = new EventEmitter<void>(); 
    
    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private requisicionService: RequisicionService) {
        super(context, router, route);

       

    }

    onConfigureDocument() {
        if (this.standAlone) {
            this.refreshList();
            this.applyPolicies();
        }

        //Cell
        this.manager.baseList.grid.formatItem.addHandler(function (grid: FlexGrid, args: FormatItemEventArgs) {
            if (args.panel.cellType == CellType.Cell && grid.columns[args.col].binding == 'workflowStep' &&
                grid.rows[args.row].dataItem.diasSinOrden > 2) {
                args.cell.style.backgroundColor = '#dc3545';
            }
            else if (args.panel.cellType == CellType.Cell && grid.columns[args.col].binding == 'workflowStep' && 
                grid.rows[args.row].dataItem.diasSinAutorizar > 2) {
                args.cell.style.backgroundColor = '#dca535';
            }
            else
                args.cell.style.backgroundColor = '';
        });

    }

    onSearchRequested(request: SearchRequest) {
        if (this.standAlone) {
            this.context.app.showSpinner();

            this.requisicionService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.requisiciones = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
        }
    }

    onAddingItem() {
        if (this.standAlone) 
            this.router.navigate(['0'], { relativeTo: this.route });
        else
        this.router.navigate(['purchasing/requisiciones', this.process.id, this.idGrupoRequisiciones, 0]);
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        if (this.standAlone) 
            this.router.navigate([document.id], { relativeTo: this.route });
        else
            this.router.navigate(['purchasing/requisiciones', this.process.id, this.idGrupoRequisiciones, document.id]);
    }

    onDeletingItem(document: DocumentItem) {
        this.requisicionService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
                this.refreshListParent.emit();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.requisicionService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
                this.refreshListParent.emit();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.requisicionService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
                this.refreshListParent.emit();
            })
            .catch(error => this.handleError(error));
    }
    
}

