﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection, CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { ProveedorService } from '../services/proveedor.service'
import { ProveedorInfo, PreRegistro } from '../models/proveedor'

@Component({
    selector: 'purchasing-proveedores-registrar-editor',
    templateUrl: './proveedores-registrar-editor.maintenance.html'
})
export class ProveedoresRegistrarEditor extends CatalogEditor {
    editorForm: FormGroup;
    info: PreRegistro = new PreRegistro();
    pRegistro: PreRegistro = new PreRegistro();
    idProveedor: number;
    proveedor: ProveedorInfo = new ProveedorInfo();


    //Inputs
    @Input() data: PreRegistro = new PreRegistro();
    //outputs
    @Output() dataUpdated: EventEmitter<PreRegistro> = new EventEmitter<PreRegistro>();


    @ViewChild('dialog') dialog: ModalDirective;

    constructor(private proveedorService: ProveedorService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute, protected changeDetector: ChangeDetectorRef) {
        super(context, router, route, changeDetector);
        this.editorForm = formBuilder.group({
            'idProveedor': [0]
        });
    }


    show(inf: PreRegistro) {
        this.pRegistro = inf;
        this.info = new PreRegistro();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => {
            this.editorForm.reset();
            this.info = this.data;
            this.data = new PreRegistro();
        });
        Control.invalidateAll();
    }


    onSelectedItemChanged(args: any) {
        if (args != null) {
            this.proveedor = args;
            this.idProveedor = args.identity;
        }
    }

    onConsultarCliente() {

        this.router.navigate(['purchasing/proveedores', this.process.id, this.idProveedor, { readonly: true }])

    }

    onUpdate(args: PreRegistro) {

        this.proveedorService.registrarDePreregistro(this.idProveedor, this.pRegistro.password, this.pRegistro.idPreRegistro, this.pRegistro.correo)
            .then(_ => {
                this.dataUpdated.emit(this.pRegistro);
                this.dialog.hide();
            }
            )
            .catch(error => this.handleError(error))

    }


}