﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
//Locales
import { ProveedorService } from '../services/proveedor.service'
import { PreRegistro } from '../models/proveedor'
import { ProveedoresRegistrarEditor } from './proveedores-registrar-editor.maintenance'
import { Confirmacion } from './parts/confirmacion.component'

@Component({
    selector: 'purchasing-proveedores-registrar',
    templateUrl: './proveedores-registrar.maintenance.html'
})

export class ProveedoresRegistrar extends ProcessComponent {
    from: FormGroup;
    pRegistro: PreRegistro[] = [];

    @ViewChild(ProveedoresRegistrarEditor) proveedoresRegistrarEditor: ProveedoresRegistrarEditor;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(Confirmacion) confirma: Confirmacion;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, private proveedorService: ProveedorService) {
        super(context, router, route);
    }


    ngAfterViewInit() {
        setTimeout(_ => this.listarCandidatos());
    }

    onRefrescar() {
        setTimeout(_ => this.listarCandidatos());
    }


    listarCandidatos() {
        this.proveedorService.preRegistros()
            .then(candidatos => {
                this.pRegistro = candidatos;
            })
            .catch(error => this.handleError(error));
    }

    onEditItem(item: PreRegistro) {
        this.proveedoresRegistrarEditor.show(item);
    }

    onDataUpdated(args: PreRegistro) {
        setTimeout(_ => this.listarCandidatos());
    }

    onRemoveItem(args: PreRegistro) {
        this.confirma.show(args.idPreRegistro, "¿Esta seguro de eliminar el elemento seleccionado?");
    }

    onConfirmar(idPreRegistro: number) {
        if (idPreRegistro > 0) {
            this.proveedorService.preRegistroEliminar(idPreRegistro)
                .then(_ => {
                    this.listarCandidatos();
                })
                .catch(error => this.handleError(error));

        }

    }

    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }


}
