﻿import { Context, CatalogInfo, CatalogItem, UsuarioInfo } from 'framework3/core';
import { DomicilioInfo, IParametrizable, ParametroValor } from 'framework3/general';
//
import { ContribuyenteInfo } from '../../accounting';
import { BeneficiarioInfo } from '../../treasury';
import { ArticuloInfo } from './articulo';


export class ProveedorInfo extends CatalogInfo implements IParametrizable {
	idTipoProveedor: number;
    nombre: string;
    domicilio: DomicilioInfo;
	telefono: string;
	fax: string;
    celular: string;
	correo: string;
	paginaWeb: string;
	diasAnticipo: number;
	porcentajeAnticipo: number;	
	idCuenta: number;
	idContribuyente: number;
    idBeneficiario: number;
    comprobanteFiscal: boolean;
    usuario: UsuarioInfo;
    contribuyente: ContribuyenteInfo;
    beneficiario: BeneficiarioInfo;
    parametros: ParametroValor[];

    constructor() {
        super();
        this.nombre = '';
        this.correo = '';
        //
        this.domicilio = new DomicilioInfo();
        this.contribuyente = new ContribuyenteInfo();
        this.beneficiario = new BeneficiarioInfo();
        this.usuario = new UsuarioInfo();
    }
}

export class ProveedorItem implements CatalogItem {    
    id: number;
    codigo: string;
    nombre: string;
    rfc: string;
    correo: string;
    telefono: string;
    domicilio: string;
    poblacion: string;
    cp: string;
    idBeneficiario: number;
    idContribuyente: number;
    activo: boolean;
    estatus?: string;
	selected?:boolean = false;
}

export class ProveedorArticulo {

    idProveedorArticulo: number;
    idProveedor: number =0;
    idZonaCompra: number;
    idArticulo: number;
    idDivisa: number;
    costo: number;
    diasCredito: number;
    diasEntrega: number;
    porcAnticipo: number;
    idUnidadMedida: number;
    porcDescuento1: number;
    porcDescuento2: number;
    porcDescuento3: number;
    porcDescuento4: number;
    porcDescuento5: number;
    predeterminado: number;
    userCreate: number;
    userUpdate: number;
    lastUpdate: Date;
    createDate: Date;

    articulo: ArticuloInfo;
    constructor() {
        this.articulo = new ArticuloInfo();
    }
}

export class ProveedorServicio {

    idProveedorServicio: number;
    idProveedor: number = 0;
    idZonaCompra: number;
    idServicio: number;
    idDivisa: number;
    costo: number;
    diasCredito: number;
    diasEntrega: number;
    porcAnticipo: number;    
    porcDescuento1: number;
    porcDescuento2: number;
    porcDescuento3: number;
    porcDescuento4: number;
    porcDescuento5: number;
    predeterminado: number;
    userCreate: number;
    userUpdate: number;
    lastUpdate: Date;
    createDate: Date;

    constructor() {
        this.porcAnticipo = 0;
    }
}

export class PreRegistro {
    idPreRegistro: number;
    rfc: string;
    nombreComercial: string;
    razonSocial: string;
    correo: string;
    password: string;
}

export class ProveedorArticuloItem extends ProveedorArticulo
{
    codigoProv: string = "";
    proveedor: string = "";
    telefono: string = "";
    codigoZona: string = "";
    zona: string = "";
}

export class ProveedorServicioItem extends ProveedorServicio {
    codigoProv: string = "";
    proveedor: string = "";
    telefono: string = "";
    codigoZona: string = "";
    zona: string = "";
}