﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem, PolicyInfo } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { DepartamentosListComponent } from '../dialogs/departamentos-list.component';
//
import { CentralService } from '../services/central.service';
import { CentralInfo, CentralItem } from '../models/central';
import { DepartamentoItem } from '../models/departamento';

@Component({
    selector: 'purchasing-central-departamentos',
    templateUrl: './central-departamentos.component.html'
})
export class CentralDepartamentosComponent extends BaseComponent {
    title: string = '';
    departamentos: DepartamentoItem[];
    idCentral: number;
    central: CentralInfo;
    currentItem: CatalogItem;

    //
    @ViewChild(DepartamentosListComponent) departamentosDialog: DepartamentosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private centralService: CentralService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idCentral = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadCentral());

    }

    ///////////////// METODOS ///////////////////////////////

    loadCentral() {
        this.centralService.getDetailByID(this.idCentral)
            .then(info => {
                this.central = info;
                this.title = 'Departamentos de la central - ' + this.central.nombre;
            })
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.centralService.getDepartamentos(this.idCentral)
            .then(departamentos => {
                this.departamentos = departamentos;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ///////////////// HANDLERS ///////////////////////////////
    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.departamentosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedItemsChanged(newDepto: DepartamentoItem[]) {
        this.centralService.addDepartamentos(this.idCentral, newDepto)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los departamentos han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveDepartamento() {
        this.context.app.showSpinner();
        this.centralService.removeDepartamento(this.idCentral, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.departamentos = this.departamentos.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}