﻿import { Component, ViewChild } from "@angular/core";
import { BaseComponent, UsuarioItem, CatalogItem, Context, PolicyInfo } from "framework3/core";
import { UsuariosListComponent } from "framework3/security";
import { AlertsBarComponent, ToolbarButton } from "framework3/controls";
import { CentroCostoInfo } from "../models/centro-costo";
import { CentroCostoService } from "../services/centro-costo.service";
import { Router, ActivatedRoute } from "@angular/router";
import { Location } from '@angular/common';

@Component({
    selector: 'purchasing-centros-costos-miembros',
    templateUrl: './centros-costos-miembros.component.html'
})
export class CentrosCostoMiembrosComponent extends BaseComponent {
    title: string = '';
    usuarios: UsuarioItem[];
    idCentroCosto: number;
    centroCosto: CentroCostoInfo;
    currentItem: CatalogItem;

    //
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private centroCostoSvc: CentroCostoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idCentroCosto = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadCentral());

    }

    ///////////////// METODOS ///////////////////////////////

    loadCentral() {
        this.centroCostoSvc.getDetailByID(this.idCentroCosto)
            .then(info => {
                this.centroCosto = info;
                this.title = 'Miembros de la central - ' + this.centroCosto.nombre;
            })
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.centroCostoSvc.getMembers(this.idCentroCosto)
            .then(usuarios => {
                this.usuarios = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ///////////////// HANDLERS ///////////////////////////////
    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.centroCostoSvc.addMembers(this.idCentroCosto, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser() {
        this.context.app.showSpinner();
        this.centroCostoSvc.removeMember(this.idCentroCosto, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}