﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem, PolicyInfo, UsuarioItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { UsuariosListComponent } from 'framework3/security';
//
import { CentroRecepcionService } from '../services/centro-recepcion.service';
import { CentroRecepcionInfo, CentroRecepcionItem } from '../models/centro-recepcion';


@Component({
    selector: 'purchasing-centros-recepcion-miembros',
    templateUrl: './centros-recepcion-miembros.component.html'
})
export class CentrosRecepcionMiembrosComponent extends BaseComponent {
    title: string = '';
    usuarios: UsuarioItem[];
    idCentroRecepcion: number;
    centroRecepcion: CentroRecepcionInfo;
    currentItem: CatalogItem;

    //
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private centroRecepcionService: CentroRecepcionService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idCentroRecepcion = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadCentral());

    }

    ///////////////// METODOS ///////////////////////////////

    loadCentral() {
        this.centroRecepcionService.getDetailByID(this.idCentroRecepcion)
            .then(info => {
                this.centroRecepcion = info;
                this.title = 'Miembros de la central - ' + this.centroRecepcion.nombre;
            })
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.centroRecepcionService.getMembers(this.idCentroRecepcion)
            .then(usuarios => {
                this.usuarios = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ///////////////// HANDLERS ///////////////////////////////
    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.centroRecepcionService.addMembers(this.idCentroRecepcion, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser() {
        this.context.app.showSpinner();
        this.centroRecepcionService.removeMember(this.idCentroRecepcion, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}