﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem, PolicyInfo } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { CentroRecepcionItem } from '../models/centro-recepcion';
import { CentrosRecepcionListComponent } from '../dialogs/centros-recepcion-list.component';
//
import { DepartamentoService } from '../services/departamento.service';
import { DepartamentoInfo, DepartamentoItem } from '../models/departamento';


@Component({
    selector: 'purchasing-departamentos-centros-recepcion',
    templateUrl: './departamentos-centros-recepcion.component.html'
})
export class DepartamentosCentrosRecepcionComponent extends BaseComponent {
    title: string = '';
    centrosRecepcion: CentroRecepcionItem[];
    idDepartamento: number;
    departamento: DepartamentoInfo;
    currentItem: CatalogItem;

    //
    @ViewChild(CentrosRecepcionListComponent) centrosRecepcionDialog: CentrosRecepcionListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private departamentoService: DepartamentoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idDepartamento = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadDepartamento());

    }

    ///////////////// METODOS ///////////////////////////////

    loadDepartamento() {
        this.departamentoService.getDetailByID(this.idDepartamento)
            .then(info => {
                this.departamento = info;
                this.title = 'Centros de Recepción del departamento - ' + this.departamento.nombre;
            })
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.departamentoService.getCentrosRecepcion(this.idDepartamento)
            .then(items => {
                this.centrosRecepcion = items;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ///////////////// HANDLERS ///////////////////////////////
    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.centrosRecepcionDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedItemsChanged(newItems: CentroRecepcionItem[]) {
        this.departamentoService.addCentrosRecepcion(this.idDepartamento, newItems)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los centros de recepcion han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveItem() {
        this.context.app.showSpinner();
        this.departamentoService.removeCentroRecepcion(this.idDepartamento, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.centrosRecepcion = this.centrosRecepcion.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}