﻿import { Component, Input, ViewChild } from "@angular/core";
import { CatalogService, BaseComponent, Context, PolicyInfo } from "framework3/core";
import { PersonaService, PersonaInfo } from "framework3/general";
import { AlertsBarComponent } from "framework3/controls";
import { Router, ActivatedRoute } from "@angular/router";
import { Location } from '@angular/common';
import { ProveedorService } from "../services/proveedor.service";

@Component({
    selector: 'proveedores-clientes-contactos',
    templateUrl: './proveedores-contactos.component.html',
    providers: [
        {
            provide: CatalogService, useExisting: PersonaService
        }
    ]
})

export class ProveedoresContactosComponent extends BaseComponent {
    title: string = '';
    idProveedor: number;
    proceso: number;
    @Input() personas: PersonaInfo[] = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private context: Context, private proveedorService: ProveedorService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.proceso = +param['proceso'];
            this.idProveedor = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadContactos());
    }

    applyPolicy(policy: PolicyInfo) {

    }

    loadContactos() {
        this.context.app.showSpinner();
        this.proveedorService.loadContactos(this.idProveedor)
            .then(contactos => {
                this.personas = contactos;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onSearchRequested(item: any) {
        this.loadContactos();
    }

    onAddPersona() {
        //this.router.navigate(['contacts'], { relativeTo: this.route });
        this.router.navigate(['purchasing/proveedores/contacts', this.proceso, this.idProveedor, 0, 0])
    }

    onEditPersona(item: any) {
        //this.router.navigate(['contacts'], { relativeTo: this.route });
        this.router.navigate(['purchasing/proveedores/contacts', this.proceso, this.idProveedor, 0, item.data.identity])
    }

    onRemovePersonas(item: any) {
        this.context.app.showSpinner();
        this.proveedorService.removeContacto(this.idProveedor, item.data.identity)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('El contacto han sido eliminado con éxito.', 'success');
                this.loadContactos();
            })
            .catch(error => this.handleError(error));
    }

    onCancel() {
        this.location.back();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}