﻿import { Component, ViewChild, Output, EventEmitter, ChangeDetectorRef } from "@angular/core";
import { CatalogService, BaseComponent, Context, PolicyInfo } from "framework3/core";
import { PersonaService, PersonaInfo } from "framework3/general";
import { FormGroup, FormBuilder } from "@angular/forms";
import { AlertsBarComponent } from "framework3/controls";
import { ModalDirective } from "ngx-bootstrap";
import { ActivatedRoute } from "@angular/router";
import { Location } from '@angular/common';
import { ProveedorService } from "../services";

@Component({
    selector: 'purchasing-proveedores-contactos-editor',
    templateUrl: './proveedores-contactos.editor.html',
    providers: [
        {
            provide: CatalogService, useExisting: PersonaService
        }
    ]
})
export class ProveedoresContactosEditor extends BaseComponent {
    editorForm: FormGroup;
    title: string = '';
    idProveedor: number;
    idContacto: number;
    idPersona: number;
    persona: PersonaInfo = new PersonaInfo;

    //@Input() personas: PersonaItem[] = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    //
    @Output() dataUpdated: EventEmitter<number> = new EventEmitter<number>();
    //
    @ViewChild('dialog') dialog: ModalDirective;


    constructor(private context: Context, private route: ActivatedRoute, private formBuilder: FormBuilder, private location: Location, protected changeDetector: ChangeDetectorRef, private personaService: PersonaService, private proveedorService: ProveedorService) {
        super();
        this.editorForm = formBuilder.group({
            'persona': [''],
        });
    }


    ngOnInit() {
        this.route.params.forEach(param => {
            this.idProveedor = +param['id'];
            this.idContacto = +param['idContacto'];
            this.idPersona = +param['idPersona'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadContacto());
    }

    loadContacto() {
        if (this.idPersona == 0)
            this.persona = new PersonaInfo;
        else
            this.loadPersona();
    }

    loadPersona() {
        this.personaService.getDetailByID(this.idPersona)
            .then(persona => this.persona = persona)
            .catch(error => this.handleError(error));
    }

    onUpdate() {
        this.proveedorService.addContacto(this.idProveedor, this.persona)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('El contacto han sido agregado con éxito.', 'success');
                this.location.back();
            })
            .catch(error => this.handleError(error));
    }

    onCancel() {
        this.location.back();
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onPersonValidChange() {
        this.changeDetector.detectChanges();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}