﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//Locales
import { CFDValiationResult } from '../models/cfd-validation-result';
import { ComprobanteFiscalService } from '../services/comprobante-fiscal.service';

@Component({
    selector: 'purchasing-validacion-cfd',
    templateUrl: './validacion-cfd.component.html'
})
export class ValidacionCFDComponent extends ProcessComponent {
    selectedFile: FileInfo;
    comprobantesCollection = new CollectionView();
    menuItems: MenuItem[] = [];
    validatedFiles: CFDValiationResult[] = [];
    //
    @ViewChild('uploadDialog') uploadDialog: ModalDirective;
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute,
        private comprobanteService: ComprobanteFiscalService) {
        super(context, router, route);
    }


    /////////////////////// METODOS ///////////////////////////////////

    validateFile() {
        this.context.app.showSpinner();
        this.comprobanteService.validate(this.context.app.defaultCompany.identity, this.selectedFile)
            .then(result => {
                this.context.app.hideSpinner();
                this.validatedFiles = result;
                this.comprobantesCollection.sourceCollection = result;
            })
            .catch(error => this.handleError(error));
    }

    createMenu() {
        //Agregar opciones estándar del catálogo
        //let mnuView: MenuItem = { icon: "glyphicon glyphicon-folder-open", text: "Ver", name: "OPEN", smallText: "ver resumen del comprobante" };

        //this.menuItems.push(mnuView, mnuPdf, mnuXml);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });

    }

    save() {
        let promises: any[] = [];
        this.validatedFiles.filter(file => file.valid && !file.saved).forEach(file => {
            this.context.app.showSpinner();
            let savePromise = this.comprobanteService.save(0, this.context.app.defaultCompany.identity, this.context.app.defaultSite.identity, file.comprobante.comprobanteBase.contenidoXML)
                .then(comprobante => {
                    file.validationMessage = 'Guardado';
                    file.saved = true;
                })
                .catch(error => this.handleError(error));
            promises.push(savePromise);
        });
        Promise.all(promises)
            .then(_ => {
                this.comprobantesCollection.refresh();
                this.context.app.hideSpinner();
                this.alertsBar.showAlert('El proceso terminó con éxito.', 'success');
            });
    }


    /////////////////////// HANDLERS //////////////////////////////////

    onToolbarClick(button: ToolbarButton) {
        if (button.name == 'FILE') {
            this.uploadDialog.config.backdrop = false;
            this.uploadDialog.show();
        }
        else if (button.name == 'SAVE') {
            this.save();
        }
        else if (button.name == 'CLOSE') {
            this.router.navigate(['../'], { relativeTo: this.route });
        }
    }

    initialized(sender: FlexGrid, args: any) {
        var self = this;
        //Agregar manejador para menú contextual        
        sender.addEventListener(sender.hostElement, "contextmenu", (e: any) => {
            let ht: HitTestInfo = sender.hitTest(e);
            sender.select(new CellRange(ht.row, ht.col), true);
        });
    }


    onFileChanged(file: FileInfo) {
        this.selectedFile = file;
    }

    onValidateFile() {
        this.uploadDialog.hide();
        this.validateFile();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        console.log(error);
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

}