﻿import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

//////////////////////////// Guards //////////////////////////////////
import { AuthGuard } from 'framework3/core';
import { EditorDeactivateGuard } from 'framework3/editors';

/////////////////////////////  Componentes //////////////////////////////
import * as fromCatalogs from './catalogs';
import * as fromDocuments from './documents';
import * as fromComponents from './components';
import * as fromParts from './parts';

//////////////////////////// Mantenimientos ////////////////////////////////
import { ProveedoresRegistrar } from './maintenances/proveedores-registrar.maintenance';
import { DepartamentosCentrosRecepcionComponent, ProveedoresContactosEditor } from './parts';
import { ProveedoresContactosComponent } from './parts/proveedores-contactos.component';

///////////////////////////// Rutas /////////////////////////////////
const routes: Routes = [
    {
        path: '',
        canActivateChild: [AuthGuard],
        children: [
            // CATALOGS
            { path: 'empresas-compras/:proceso', component: fromCatalogs.EmpresasManager },
            { path: 'empresas-compras/:proceso/:id', component: fromCatalogs.EmpresasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'centrales/:proceso', component: fromCatalogs.CentralesManager },
            { path: 'centrales/:proceso/:id', component: fromCatalogs.CentralesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'centrales/:proceso/:id/members', component: fromParts.CentralMiembrosComponent },
            { path: 'centrales/:proceso/:id/departamentos', component: fromParts.CentralDepartamentosComponent },
            { path: 'departamentos/:proceso', component: fromCatalogs.DepartamentosManager },
            { path: 'departamentos/:proceso/:id', component: fromCatalogs.DepartamentosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'departamentos/:proceso/:id/members', component: fromParts.DepartamentosMiembrosComponent },
            { path: 'departamentos/:proceso/:id/centros-recepcion', component: fromParts.DepartamentosCentrosRecepcionComponent },
            { path: 'departamentos/:proceso/:id/categorias-presupuestales', component: fromParts.DepartamentosCategoriasPresupuestalesComponent},
            //{ path: 'departamentos/:proceso/:id/centros-costo', component: DepartamentosCentrosRecepcionComponent },
            { path: 'centros-recepcion/:proceso', component: fromCatalogs.CentrosRecepcionManager },
            { path: 'centros-recepcion/:proceso/:id', component: fromCatalogs.CentrosRecepcionEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'centros-recepcion/:proceso/:id/members', component: fromParts.CentrosRecepcionMiembrosComponent },
            { path: 'centros-costo/:proceso', component: fromCatalogs.CentrosCostoManager },
            { path: 'centros-costo/:proceso/:id', component: fromCatalogs.CentrosCostoEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'centros-costo/:proceso/:id/members', component: fromParts.CentrosCostoMiembrosComponent },
            { path: 'proveedores/:proceso', component: fromCatalogs.ProveedoresManager },
            { path: 'proveedores/:proceso/:id', component: fromCatalogs.ProveedoresEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'zonas-compra/:proceso', component: fromCatalogs.ZonasCompraManager },
            { path: 'zonas-compra/:proceso/:id', component: fromCatalogs.ZonasCompraEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'articulos/:proceso', component: fromCatalogs.ArticulosManager },
            { path: 'articulos/:proceso/:id', component: fromCatalogs.ArticulosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'proveedor-articulo/:proceso/:id/:idProveedor/:idArticulo/:idZona/:editProveedor/:editArticulo/:editZona', component: fromComponents.ProveedorArtioculoComponent, canDeactivate: [EditorDeactivateGuard] },
            { path: 'proveedor-producto/:proceso', component: fromComponents.ProveedorProductoComponent },
            { path: 'producto-proveedor/:proceso', component: fromComponents.ProductoProveedorComponent },

            { path: 'servicios-compras/:proceso', component: fromCatalogs.ServiciosManager },
            { path: 'servicios-compras/:proceso/:id', component: fromCatalogs.ServiciosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'proveedor-servicio/:proceso/:id/:idProveedor/:idServicio/:idZona/:editProveedor/:editServicio/:editZona', component: fromComponents.ProveedorServicioComponent, canDeactivate: [EditorDeactivateGuard] },

            { path: 'categorias-presupuestales/:proceso', component: fromCatalogs.CategoriasPresupuestalesManager },
            { path: 'categorias-presupuestales/:proceso/:id', component: fromCatalogs.CategoriasPresupuestalesEditor, canDeactivate: [EditorDeactivateGuard]},
            { path: 'presupuestos/:proceso', component: fromDocuments.PresupuestosManager },
            { path: 'presupuestos/:proceso/:id', component: fromDocuments.PresupuestosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'presupuestos/:proceso/:id/change', component: fromDocuments.PresupuestosCambioEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'proveedores-tipos/:proceso/:id', component: fromCatalogs.ProveedorTipoEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'proveedores-tipos/:proceso', component: fromCatalogs.ProveedoresTiposManager},
            { path: 'tipos-operaciones/:proceso', component: fromCatalogs.OperacionesTiposManager },
            { path: 'tipos-operaciones/:proceso/:id', component: fromCatalogs.OperacionesTiposEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'tipos-operaciones/:proceso/:id/members', component: fromParts.OperacionesTiposMiembrosComponent },
            { path: 'sucursales-compras/:proceso/:idEmpresa/:id', component: fromCatalogs.SucursalesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'tipos-requisiciones/:proceso', component: fromCatalogs.TiposRequisicionManager},
            { path: 'tipos-requisiciones/:proceso/:id', component: fromCatalogs.TiposRequisicionEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'gastos-tipos/:proceso', component: fromCatalogs.GastosTiposManager },
            { path: 'gastos-tipos/:proceso/:id', component: fromCatalogs.GastosTiposEditor, canDeactivate: [EditorDeactivateGuard] },

            //DOCUMENTS
            { path: 'almacen-digital/:proceso', component: fromDocuments.AlmacenDigitalComponent },
            //PARTS
            { path: 'almacen-digital/:proceso/upload', component: fromParts.ValidacionCFDComponent },

            { path: 'grupo-requisiciones/:proceso', component: fromDocuments.GrupoRequisicionesManager },
            { path: 'grupo-requisiciones/:proceso/:id', component: fromDocuments.GrupoRequisicionesEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'requisiciones/:proceso', component: fromDocuments.RequisicionesManager },
            { path: 'requisiciones/:proceso/:id', component: fromDocuments.RequisicionesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'requisiciones/:proceso/:idGrupoRequisiciones/:id', component: fromDocuments.RequisicionesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'ordenes-compra/:proceso', component: fromDocuments.OrdenesManager },
            { path: 'ordenes-compra/:proceso/:id', component: fromDocuments.OrdenesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'ordenes-compra/:proceso/:id/wizard', component: fromDocuments.OrdenesWizardEditorComponent, canDeactivate: [EditorDeactivateGuard] },

            { path: 'recepciones/:proceso', component: fromDocuments.RecepcionesManager },
            { path: 'recepciones/:proceso/:id/:idCentroRecepcion/:idOrden', component: fromDocuments.RecepcionesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'recepciones-sin-oc/:proceso', component: fromDocuments.RecepcionesSinOCManager },
            { path: 'recepciones-sin-oc/:proceso/:id', component: fromDocuments.RecepcionesSinOCEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'gastos/:proceso', component: fromDocuments.GastosManager },
            { path: 'gastos/:proceso/new', component: fromDocuments.NuevoGastoComponent },
            { path: 'gastos/:proceso/:id', component: fromDocuments.GastosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'gasto-comprobar/:proceso/:idGasto', component: fromDocuments.GastoComprobar },
            { path: 'adelantos/:proceso', component: fromDocuments.AdelantosManager },
            { path: 'adelantos/:proceso/:id/:idOrden', component: fromDocuments.AdelantosEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'proveedores-registrar/:proceso', component: ProveedoresRegistrar },

            { path: 'notas/:proceso', component: fromDocuments.NotasManager },
            { path: 'notas/:proceso/:id', component: fromDocuments.NotasEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'costos-presupuestos/:proceso', component: fromDocuments.CostosPresupuestosManager },
            { path: 'costos-presupuestos/:proceso/new', component: fromDocuments.NuevoCostosPresupuestosComponent },
            { path: 'costos-presupuestos/:proceso/:id', component: fromDocuments.CostosPresupuestosEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'proveedores/contacts/:proceso/:id', component: ProveedoresContactosComponent },
            { path: 'proveedores/contacts/:proceso/:id/:idContacto/:idPersona', component: ProveedoresContactosEditor },

        ]
    },
];


@NgModule({
    imports: [
        RouterModule.forChild(routes)
    ],
    exports: [
        RouterModule
    ]
})
export class PurchasingRoutingModule { }

