﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';
//Locales
import { CostoInfo } from '../models/costo'
import { ArticuloInfo } from '../models/articulo'
import { ArticuloImpuesto } from '../index';

@Injectable()
export class ArticuloService extends CatalogService<ArticuloInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/compras/articulos');
    }

    create(): Promise<ArticuloInfo> {
        return Promise.resolve(new ArticuloInfo());
    }
    

    searchArticuloProveedor(idProveedor: number, idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params: HttpParams = new HttpParams()
            .set('idProveedor', idProveedor.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<any>('search-articulo-proveedor', params, request)
            .toPromise();
    }

    calcularCostoProveedor(idArticulo: number,idUnidadMedida:number, idProveedor: number, idEmpresa: number): Promise<CostoInfo> {
        let params = new HttpParams()
            .set("idArticulo", idArticulo.toString())
            .set("idUnidadMedida", idUnidadMedida.toString())
            .set("idProveedor", idProveedor.toString())
            .set("idEmpresa", idEmpresa.toString());
        return this.apiService.get<CostoInfo>('costo-proveedor', params)
            .toPromise();
    }
}

