﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { CentralInfo, CentralItem } from '../models/central'
import { UsuarioItem } from 'framework3/core';
import { DepartamentoItem } from '../models/departamento'
import { CentroRecepcionItem } from '../models/centro-recepcion'
import { SecurityContext } from '../models/security-context'


@Injectable()
export class CentralService extends CatalogService<CentralInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/compras/centrales');
    }

    create(): Promise<CentralInfo> {
        return Promise.resolve(new CentralInfo());
    }

    loadSecurityContext(idEmpresa: number): Promise<SecurityContext[]> {
        let params: HttpParams = new HttpParams()            
            .set('IdEmpresa', idEmpresa.toString());
        return this.apiService.get<SecurityContext[]>('load-security-context', params)
            .toPromise();
    }

    loadListByUser(idSucursal: number): Promise<CentralItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<CentralItem[]>('load-list', params)
            .toPromise();
    }

    getMembers(idCentral: number): Promise<UsuarioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)
            .toPromise();
    }

    addMembers(idCentral: number, users: UsuarioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }

    removeMember(idCentral: number, idUser: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString())
            .set('idUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

    //Centros recepcion
    getCentrosRecepcion(idCentral: number): Promise<CentroRecepcionItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString());
        return this.apiService.get<CentroRecepcionItem[]>('get-centros-recepcion', params)
            .toPromise();
    }

    addCentrosRecepcion(idCentral: number, centrosRecepcion: CentroRecepcionItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString());
        return this.apiService.post<any>('add-centro-recepcion', params, centrosRecepcion)
            .toPromise();
    }

    removeCentroRecepcion(idCentral: number, idCentroRecepcion: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString())
            .set('idCentroRecepcion', idCentroRecepcion.toString());
        return this.apiService.delete<any>('remove-centro-recepcion', params)
            .toPromise();
    }

    //Departamentos
    getDepartamentos(idCentral: number): Promise<DepartamentoItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString());
        return this.apiService.get<DepartamentoItem[]>('get-central-departamentos', params)
            .toPromise();
    }    

    addDepartamentos(idCentral: number, departamentos: DepartamentoItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString());
        return this.apiService.post<any>('add-departamento', params, departamentos)
            .toPromise();
    }

    removeDepartamento(idCentral: number, idDepartamento: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString())
            .set('idDepartamento', idDepartamento.toString());
        return this.apiService.delete<any>('remove-departamento', params)
            .toPromise();
    }


}

