﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, UsuarioItem } from 'framework3/core';
//Locales
import { CentroCostoInfo } from '../models/centro-costo'

@Injectable()
export class CentroCostoService extends CatalogService<CentroCostoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/compras/centros-costo');
    }

    create(): Promise<CentroCostoInfo> {
        return Promise.resolve(new CentroCostoInfo());
    }

    getMembers(idCentroCosto: number): Promise<UsuarioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCentroCosto', idCentroCosto.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)
            .toPromise();
    }

    addMembers(idCentroCosto: number, users: UsuarioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentroCosto', idCentroCosto.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }

    removeMember(idCentroCosto: number, idUser: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentroCosto', idCentroCosto.toString())
            .set('idUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

    loadList(idSucursal: number, type: string, id: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdSucursal', idSucursal.toString())
            .set('type', type)
            .set('id', id.toString());
        return this.apiService.get<any[]>('load-list', params)
            .toPromise();
    }

	/*
	//Sample method
    sampleMethod(param1: number, param2: string): Promise<CentroCostoInfo> {
        let params = new HttpParams()
			.set('param1', param1.toString())
			.set('param2', param2);
		//
        return this.apiService.get<CentroCostoInfo>('sample-method', params)
            .toPromise();
    }
	*/
}

