﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { CentroRecepcionInfo, CentroRecepcionItem } from '../models/centro-recepcion'
import { UsuarioItem } from 'framework3/core';

@Injectable()
export class CentroRecepcionService extends CatalogService<CentroRecepcionInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/compras/centros-recepcion');
    }

    create(): Promise<CentroRecepcionInfo> {
        return Promise.resolve(new CentroRecepcionInfo());
    }

    loadListByDepartamentoAndUser(idDepartamento, idSucursal: number): Promise<CentroRecepcionItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString())
            .set('IdSucursal', idSucursal.toString());
        return this.apiService.get<CentroRecepcionItem[]>('load-list-by-departamento', params)
            .toPromise();
    }

    getMembers(idCentroRecepcion: number): Promise<UsuarioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCentroRecepcion', idCentroRecepcion.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)
            .toPromise();
    }

    addMembers(idCentroRecepcion: number, users: UsuarioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentroRecepcion', idCentroRecepcion.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }

    removeMember(idCentroRecepcion: number, idUser: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCentroRecepcion', idCentroRecepcion.toString())
            .set('idUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

}

