﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
import { map } from 'rxjs/operators';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
import { FileInfo } from 'framework3/general';
//Locales
import { ComprobanteFiscalDetail } from '../../accounting';
import { ComprobanteFiscalInfo, ComprobanteFiscalItem } from '../models/comprobante-fiscal';
import { CFDValiationResult } from '../models/cfd-validation-result';

@Injectable()
export class ComprobanteFiscalService extends DocumentService<ComprobanteFiscalDetail, ComprobanteFiscalInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/compras/comprobantes');
    }

    create(): Promise<ComprobanteFiscalInfo> {
        let newDoc = new ComprobanteFiscalInfo();
        return Promise.resolve(newDoc);
    }

    validate(idEmpresa: number, file: FileInfo): Promise<CFDValiationResult[]> {
        let params = new HttpParams()
            .set('idEmpresa', idEmpresa.toString());                
        return this.apiService.post<CFDValiationResult[]>('validate', params, file)
            .pipe(map(result => {
                result.forEach(row => {
                    row.fecha = moment(row.fecha).toDate();
                });
                return result;
            }))
            .toPromise();
    }

    save(idProveedor: number, idEmpresa: number, idSucursal: number, xml: string): Promise<ComprobanteFiscalInfo> {
        let params = new HttpParams()
            .set('idProveedor', idProveedor.toString())
            .set('idEmpresa', idEmpresa.toString())
            .set('idSucursal', idSucursal.toString());
        //
        return this.apiService.post<ComprobanteFiscalInfo>('save', params, JSON.stringify(xml))
            .toPromise();
    }

    loadListByProveedor(idEmpresa: number, idProveedor: number, fechaInicial: Date, fechaFinal: Date): Promise<ComprobanteFiscalItem[]> {
        let params = new HttpParams()
            .set('idProveedor', idProveedor.toString())
            .set('idEmpresa', idEmpresa.toString())
            .set('fechaInicial', moment(fechaInicial).format())
            .set('fechaFinal', moment(fechaFinal).format());
        //
        return this.apiService.get<ComprobanteFiscalItem[]>('list-proveedor', params)
            .pipe(map(response => {
                response.forEach(c => { 
                    c.fechaComprobante = moment(c.fechaComprobante).toDate();
                    c.fechaCreacion = moment(c.fechaCreacion).toDate();
                });
                return response;
            }))
            .toPromise();
    }
		
	protected mapSearchResult(row: any) {
        let comprobante = <ComprobanteFiscalItem>row;
        comprobante.fechaComprobante = moment(comprobante.fechaComprobante).toDate();
    }

    marcarDocumentoRelacionado(idComprobante: string, idDocumentoRel: string): Promise<any> {
        let params = new HttpParams()
            .set('IdComprobante', idComprobante.toString())
            .set('IdDocumentoRel', idDocumentoRel.toString());
        return this.apiService.get<any>('marcar-documento-rel', params)
            .toPromise();
    }

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: ComprobanteFiscalInfo) {
    }
	*/

}