﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { DepartamentoInfo, DepartamentoItem } from '../models/departamento'
import { UsuarioItem } from 'framework3/core'
import { CentroRecepcionItem } from '../models/centro-recepcion'
import { CategoriasPresupuestalesItem } from '../models/categorias-presupuestales'

@Injectable()
export class DepartamentoService extends CatalogService<DepartamentoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/compras/departamentos');
    }

    create(): Promise<DepartamentoInfo> {
        return Promise.resolve(new DepartamentoInfo());
    }

    // ACCESOS
    loadListByUser(idSucursal: number): Promise<DepartamentoItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdSucursal', idSucursal.toString());
        return this.apiService.get<DepartamentoItem[]>('load-list', params)
            .toPromise();
    }

    loadListByCentralUser(idCentral: number, idSucursal:number): Promise<DepartamentoItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCentral', idCentral.toString())
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<UsuarioItem[]>('load-list-by-central-user', params)
            .toPromise();
    }

    getMembers(idDepartamento: number): Promise<UsuarioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)
            .toPromise();
    }

    addMembers(idDepartamento: number, users: UsuarioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }

    removeMember(idDepartamento: number, idUser: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString())
            .set('IdUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

    //Centros de  Recepcion
    getCentrosRecepcion(idDepartamento: number): Promise<CentroRecepcionItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString());
        return this.apiService.get<CentroRecepcionItem[]>('load-centros-recepcion', params)
            .toPromise();
    }

    addCentrosRecepcion(idDepartamento: number, centrosRecepcion: CentroRecepcionItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString());
        return this.apiService.post<any>('add-centros-recepcion', params, centrosRecepcion)
            .toPromise();
    }

    removeCentroRecepcion(idDepartamento: number, idCentroRecepcion: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString())
            .set('IdCentroRecepcion', idCentroRecepcion.toString());
        return this.apiService.delete<any>('remove-centro-recepcion', params)
            .toPromise();
    }

    //Categorias Presupuestales
    getCategoriasPresupuestales(idDepartamento: number): Promise<CategoriasPresupuestalesItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString());
        return this.apiService.get<CategoriasPresupuestalesItem[]>('load-categorias-presupuestales', params)
            .toPromise();
    }

    addCategoriaPresupuestal(idDepartamento: number, categoriasPresupuestales: CategoriasPresupuestalesItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString());
        return this.apiService.post<any>('add-categoria-presupuestal', params, categoriasPresupuestales)
            .toPromise();
    }

    removeCategoriaPresupuestal(idDepartamento: number, idCategoriaPresupuestal: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdDepartamento', idDepartamento.toString())
            .set('IdCategoriaPresupuestal', idCategoriaPresupuestal.toString());
        return this.apiService.delete<any>('remove-categoria-presupuestal', params)
            .toPromise();
    }


}

