﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';
//
import * as moment from 'moment';
//Locales
import { OrdenInfo, OrdenItem } from '../models/orden';
import { OrdenDetail, OrdenRequisicionDetail } from '../models/orden-detail';
import { ComprobanteFiscalInfo } from '../models/comprobante-fiscal';
import { Empaquetado } from '../../inventory';
import { OrdenDetailArticulo } from '../models/orden-detail-articulo';

@Injectable()
export class OrdenService extends DocumentService<OrdenDetail, OrdenInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/compras/ordenes');
    }

    create(): Promise<OrdenInfo> {
        let newDoc = new OrdenInfo();
        return Promise.resolve(newDoc);
    }

    update(info: OrdenInfo): Promise<OrdenInfo> {
        //Si no tiene asignado la empresa y sucursal, asignar la actual
        if (info.idSucursal == 0) info.idSucursal = this.context.app.defaultSite.identity;
        //Actualizar campo de cantidad basado en las propiedades auxiliares (solo para articulos)
        info.details.filter(detail => detail.idArticulo > 0).map(detail => <OrdenDetailArticulo>detail).forEach(detail => {
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor, detail.unidades);
                detail.cantidad = cantidad.toDecimal();
            }
        });
        
        let params = new HttpParams();
        return this.apiService.post<OrdenInfo>('save', params, info)
            .toPromise();
    }

    protected mapDocument(document: OrdenInfo) {
        document.details.filter(detail => detail.idArticulo > 0).map(detail => <OrdenDetailArticulo>detail).forEach(detail => {            
            //Si la unidad de medida es un paquete, separar unidades
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor);
                detail.cantidad = cantidad.paquetes;
                detail.unidades = cantidad.unidades;
            }
            else {
                detail.unidades = 0;
            }
        });
    }


    generarPdfOrden(idOrden: string): Promise<FileLink> {
        let params = new HttpParams()
            .set("idOrden", idOrden);
        return this.apiService.get<FileLink>("pdf-orden", params)
            .toPromise();
    }

    requisicionesPendientes(idCentral: number, fechaIni: Date, fechaFin: Date): Promise<OrdenRequisicionDetail[]> {
        let fi = moment(fechaIni).format();
        let ff = moment(fechaFin).format();

        let params = new HttpParams()
            .set("IdCentral", idCentral.toString()) 
            .set("FechaIni", fi)
            .set("FechaFin", ff);

        return this.apiService.get<OrdenRequisicionDetail[]>("requisiciones-pendientes", params)
            .toPromise();
    }

    generarDesdeRequisiciones(idCentralCompras: number, idDepartamento: number, idSucursal:number, idEmpresa:number, requisiciones: OrdenRequisicionDetail[]): Promise<boolean> {
        let params = new HttpParams()
            .set("IdCentral", idCentralCompras.toString())
            .set("IdDepartamento", idDepartamento.toString())
            .set("IdSucursal", idSucursal.toString())
            .set("IdEmpresa", idEmpresa.toString());

        return this.apiService.post<boolean>('generar-desde-requisiciones', params, requisiciones) 
            .toPromise();
    }

    getComprobanteRelacionado(idOrden: string): Promise<ComprobanteFiscalInfo> {
        let params = new HttpParams()
            .set("idOrden", idOrden);
        return this.apiService.get<ComprobanteFiscalInfo>("get-comprobante-relacionado", params)
            .toPromise();
    }

}