﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';
//Locales
import { PresupuestosInfo, PresupuestosItem } from '../models/presupuestos';
import { PresupuestosDetail } from '../models/presupuestos-detail';
import { CategoriasPresupuestalesItem } from '../models/categorias-presupuestales';


@Injectable()
export class PresupuestosService extends DocumentService<PresupuestosDetail, PresupuestosInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/compras/presupuestos');
    }

    create(): Promise<PresupuestosInfo> {
        let newDoc = new PresupuestosInfo();
        return Promise.resolve(newDoc);
    }


    loadPresupuestos(idDepartamento: number): Promise<PresupuestosItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idDepartamento', idDepartamento.toString());
        return this.apiService.get<PresupuestosItem[]>('load-presupuestos', params)
            .toPromise();
    }

    unlock(idPresupuesto: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdPresupuesto', idPresupuesto);
        return this.apiService.post<any>('unlock', params)
            .toPromise();
    }

    lock(idPresupuesto: string): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('IdPresupuesto', idPresupuesto);
        return this.apiService.post<any>('lock', params)
            .toPromise();
    }

    
    loadCategoriasConArticulo(idPresupuesto: string, idArticulo: number): Promise<CategoriasPresupuestalesItem[] > {
        let params: HttpParams = new HttpParams()
            .set('IdPresupuesto', idPresupuesto)
            .set('IdArticulo', idArticulo.toString());
        return this.apiService.get<CategoriasPresupuestalesItem[]>('load-categorias-presupuestales-con-articulo', params)
            .toPromise();
    }

    loadCategoriasConServicio(idPresupuesto: string, idServicio: number): Promise<CategoriasPresupuestalesItem[]> {
        let params: HttpParams = new HttpParams()
            .set('IdPresupuesto', idPresupuesto)
            .set('IdServicio', idServicio.toString());
        return this.apiService.get<CategoriasPresupuestalesItem[]>('load-categorias-presupuestales-con-servicio', params)
            .toPromise();
    }

    saveChange(presupuesto: PresupuestosInfo): Promise<void> {
        let params: HttpParams = new HttpParams();
            
        return this.apiService.post<any>('change', params, presupuesto)
            .toPromise();
    }
    generarPdfImpresion(idPresupuesto: string): Promise<FileLink> {
        let params = new HttpParams()
            .set("idPresupuesto", idPresupuesto);
        return this.apiService.get<FileLink>("pdf-presupuesto", params)
            .toPromise();
    }

}