﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, EmpresaItem } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';
//Locales
import { ProveedorInfo, ProveedorServicio, ProveedorArticulo, PreRegistro } from '../models/proveedor'
import { PersonaInfo } from 'framework3/general';


@Injectable()
export class ProveedorService extends CatalogService<ProveedorInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/compras/proveedores');
    }

    create(): Promise<ProveedorInfo> {
        return Promise.resolve(new ProveedorInfo());
    }

    enviarRegistro(idProveedor: number): Promise<any> {
        let params = new HttpParams()
            .set('idProveedor', idProveedor.toString());
        return this.apiService.get<any>('enviar-registro', params)
            .toPromise();
    }

    registrar(idProveedor: number, password: string): Promise<any> {
        let params = new HttpParams()
            .set('idProveedor', idProveedor.toString())
            .set('password', password);
        return this.apiService.get<any>('registrar', params)
            .toPromise();
    }

    getDetailByUser(idUsuario: number, idEmpresa: number): Promise<ProveedorInfo> {
        let params = new HttpParams()
            .set('idUsuario', idUsuario.toString())
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get<ProveedorInfo>('detail-by-user', params)
            .toPromise();
    }

    loadEmpresas(idUsuario: number): Promise<EmpresaItem[]> {
        let params = new HttpParams()
            .set('idUsuario', idUsuario.toString());
        return this.apiService.get<EmpresaItem[]>('empresas', params)
            .toPromise();
    }

    //Administrar articulos
    updateProveedorArticulo(proveedorArticulo: ProveedorArticulo): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('update-proveedor-articulo', params, proveedorArticulo)
            .toPromise();
    }

    getDetailProveedorArticulo(idProveedorArticulo: number): Promise<ProveedorArticulo> {
        let params = new HttpParams()
            .set('idProveedorArticulo', idProveedorArticulo.toString());
        return this.apiService.get<ProveedorArticulo>("get-detail-proveedor-articulo", params).toPromise();
    }

    deleteProveedorArticulo(idProveedorArticulo: number): Promise<boolean> {
        let params = new HttpParams()
            .set('idProveedorArticulo', idProveedorArticulo.toString());
        return this.apiService.post<boolean>("delete-proveedor-articulo", params).toPromise();
    }

    searchProveedorByArticulo(idArticulo: number, idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params: HttpParams = new HttpParams()
            .set('idArticulo', idArticulo.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<any>('search-proveedor-articulo', params, request)
            .toPromise();
    }

    searchProveedorArticuloByZona(idZonaCompra: number, idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idZonaCompra', idZonaCompra.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<SearchResult>('search-proveedor-articulo-by-zona', params, request).toPromise();
    }

    searchProveedorByZona(idZonaCompra: number, idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idZonaCompra', idZonaCompra.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<SearchResult>('search-proveedor-by-zona', params, request).toPromise();
    }

    //Administrar servicios
    updateProveedorServicio(proveedorServicio: ProveedorServicio): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('update-proveedor-servicio', params, proveedorServicio) //falta
            .toPromise();
    }

    getDetailProveedorServicio(idProveedorServicio: number): Promise<ProveedorServicio> {
        let params = new HttpParams()
            .set('idProveedorServicio', idProveedorServicio.toString());
        return this.apiService.get<ProveedorServicio>("get-detail-proveedor-servicio", params).toPromise(); //falta
    }

    deleteProveedorServicio(idProveedorServicio: number): Promise<boolean> {
        let params = new HttpParams()
            .set('idProveedorServicio', idProveedorServicio.toString());
        return this.apiService.post<boolean>("delete-proveedor-servicio", params).toPromise();
    }

    searchProveedorByServicio(idServicio: number, idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params: HttpParams = new HttpParams()
            .set('idServicio', idServicio.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<any>('search-proveedor-servicio', params, request)
            .toPromise();
    }

    searchProveedorServicioByZona(idZonaCompra: number, idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idZonaCompra', idZonaCompra.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<SearchResult>('search-proveedor-servicio-by-zona', params, request).toPromise();
    }

    //Pre registro
    registrarDePreregistro(idProveedor: number, password: string, idPreRegistro: number, correo: string): Promise<any> {
        let params = new HttpParams()
            .set('idProveedor', idProveedor.toString())
            .set('password', password)
            .set('idPreRegistro', idPreRegistro.toString())
            .set('correo', correo);
        return this.apiService.get<any>('registrar-de-preregistro', params)
            .toPromise();
    }

    preRegistroEliminar(idPreRegistro: number): Promise<any> {
        let params = new HttpParams()
            .set('idPreRegistro', idPreRegistro.toString());
        return this.apiService.post<any>('pre-registro-delete', params)
            .toPromise();

    }

    preRegistrar(preReg: PreRegistro): Promise<any> {
        let params = new HttpParams()
        return this.apiService.post<any>("pre-registrar", params, preReg)
            .toPromise();
    }


    preRegistros(): Promise<PreRegistro[]> {
        let params = new HttpParams()
        return this.apiService.get<PreRegistro[]>('pre-registros', params)
            .toPromise();
    }

    loadContactos(idProveedor: number): Promise<PersonaInfo[]> {
        let params = new HttpParams()
            .append("idProveedor", idProveedor.toString());
        return this.apiService.get<PersonaInfo[]>("load-contactos", params)
            .toPromise();
    }

    addContacto(idProveedor: number, contacto: PersonaInfo): Promise<any> {
        let params = new HttpParams()
            .set('idProveedor', idProveedor.toString());
        return this.apiService.post<any>('add-contacto', params, contacto)
            .toPromise();
    }

    removeContacto(idProveedor: number, idPersona: number): Promise<any> {
        let params = new HttpParams()
            .set('idProveedor', idProveedor.toString())
            .set('idPersona', idPersona.toString());
        return this.apiService.delete<any>('remove-contacto', params)
            .toPromise();
    }

}

