﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';
import * as moment from 'moment';

//Inventario
import { Empaquetado } from '../../inventory';
import { RecepcionArticuloDetail } from '../models/recepcion-articulo-detail';

//Locales
import { RecepcionInfo, RecepcionItem } from '../models/recepcion';
import { RecepcionDetail } from '../models/recepcion-detail';

@Injectable()
export class RecepcionService extends DocumentService<RecepcionDetail, RecepcionInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/compras/recepciones');
    }

    create(): Promise<RecepcionInfo> {
        let newDoc = new RecepcionInfo();
        return Promise.resolve(newDoc);
    }

    update(info: RecepcionInfo): Promise<RecepcionInfo> {
        //Si no tiene asignado la empresa y sucursal, asignar la actual
        if (info.idSucursal == 0) info.idSucursal = this.context.app.defaultSite.identity;
        //Actualizar campo de cantidad basado en las propiedades auxiliares (solo para articulos)
        info.details.filter(detail => detail.idArticulo > 0).map(detail => <RecepcionArticuloDetail>detail).forEach(detail => {
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor, detail.unidades);
                detail.cantidad = cantidad.toDecimal();
            }
        });
        //
        let params = new HttpParams();
        return this.apiService.post<RecepcionInfo>('save', params, info)
            .toPromise();
    }

    protected mapDocument(document: RecepcionInfo) {
        document.details.filter(detail => detail.idArticulo > 0).map(detail => <RecepcionArticuloDetail>detail).forEach(detail => {
            detail.fechaCaducidad = moment(detail.fechaCaducidad).toDate();
            //Si la unidad de medida es un paquete, separar unidades
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor);
                detail.cantidad = cantidad.paquetes;
                detail.unidades = cantidad.unidades;
            }
            else {
                detail.unidades = 0;
            }
        });
    }

    generarPdfRecepcion(idRecepcion: string): Promise<FileLink> {
        let params = new HttpParams()
            .set("idRecepcion", idRecepcion);
        return this.apiService.get<FileLink>("pdf-recepcion", params)
            .toPromise();
    }


	/*
    sampleMethod(param1: number, param2: string): Promise<RecepcionInfo> {
        let params = new HttpParams()
			.set('param1', param1.toString())
			.set('param2', param2);
		//
        return this.apiService.get<RecepcionInfo>('sample-method', params)
            .toPromise();
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: RecepcionInfo) {

    }
	*/



}