﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, UsuarioItem, CatalogItem } from 'framework3/core';
//Locales
import { ZonaCompraInfo, ZonaCompraItem } from '../models/zona-compra'
import { Http } from '@angular/http';
import { SearchResult, SearchRequest } from 'framework3/search';

@Injectable()
export class ZonaCompraService extends CatalogService<ZonaCompraInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/compras/zonas-compra');
    }

    create(): Promise<ZonaCompraInfo> {
        return Promise.resolve(new ZonaCompraInfo());
    }

    loadListByUser(idSucursal: number): Promise<ZonaCompraItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<ZonaCompraItem[]>('load-list', params)
            .toPromise();
    }

    addUser(idZonaCompra: number,usuarios:UsuarioItem[]): Promise<any> {
        let params = new HttpParams()            
            .set('IdZonaCompra', idZonaCompra.toString());
        return this.apiService.post<any>('add-user', params,usuarios)
            .toPromise();
    }

    loadAccessUsersList(idZonaCompra: number): Promise<UsuarioItem[]> {
        let params = new HttpParams()
            .append("IdZonaCompra", idZonaCompra.toString());
        return this.apiService.get<UsuarioItem[]>("load-access-users-list", params)
            .toPromise();
    }

    removeUser(idZonaCompraAcceso: number): Promise<any> {
        let params = new HttpParams()
            .append("IdZonaCompraAcceso", idZonaCompraAcceso.toString());
        return this.apiService.post<any>("remove-user", params)
            .toPromise();
    }

    searchKeyword2(idUsuario: number, idSucursal: number, keyword: string): Observable<CatalogItem[]>{
        let params = new HttpParams()
            .append("idSucursal", idSucursal.toString())
            .append("idUsuario", idUsuario.toString())
            .append("keyword", keyword);
        return this.apiService.get<CatalogItem[]>("search-zona-compra", params);
    }

    loadRestriccionesCostos(): Promise<any[]> {
        let params = new HttpParams();
        return this.apiService.get<any[]>("load-resticciones-costos", params)
            .toPromise();
    }

    search2(idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idBranch', idBranch.toString());
        //
        return this.apiService.post<SearchResult>('search2', params, request)
            .toPromise();
    }

    hasAccess(idUsuario: number, idZonaCompra: number): Promise<boolean> {
        let params = new HttpParams()
            .set('idUsuario', idUsuario.toString())
            .set('idZonaCompra', idZonaCompra.toString());
        return this.apiService.get<boolean>('has-access', params).toPromise();
    }
}

