﻿//Frameworks
import { Component, OnInit, ViewChild, ContentChildren, Input, forwardRef, QueryList, ChangeDetectorRef, enableProdMode } from '@angular/core';
import { DecimalPipe } from '@angular/common';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
import { Control, CollectionView } from 'wijmo/wijmo';
import { MenuItemDirective } from 'framework3/controls/directives/menu-item.directive';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

// wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import { WjGridModule } from 'wijmo/wijmo.angular2.grid';
import { WjInputModule } from 'wijmo/wijmo.angular2.input';
import { Menu } from 'wijmo/wijmo.input';
import * as wjcGridDetail from 'wijmo/wijmo.grid.detail';

import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, AzComboboxComponent } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';

import { EmpleadoInfo } from '../models/empleado';
import { PersonaInfo } from 'framework3/general/models/persona';
//Features
import { CedulaFiniquitoService } from '../services/cedulafiniquito.service';
import { CedulaFiniquitoInfo, CedulaFiniquitoItem, CedulaConcepto, RescisionLaboral, TarifaPagoProvicional, SubsidioEmpleo, SeguroIMSS, ConceptoFiniquito, ComisionPeriodo, AcumuladoDet, AcumuladoBajaTit } from '../models/cedulafiniquito';

@Component({
    selector: 'rh-cedulasfiniquitos-editor',
    templateUrl: './cedulasfiniquitos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CedulasFiniquitosEditor)
        },
        {
            provide: CatalogService, useExisting: CedulaFiniquitoService
        }
    ]
})
export class CedulasFiniquitosEditor extends CatalogEditor {

    cedulaFiniquito: CedulaFiniquitoInfo = new CedulaFiniquitoInfo();
    rescisionesLaborales: Array<RescisionLaboral> = new Array<RescisionLaboral>();
    conceptosCedulaNegosiacion: wjcCore.CollectionView;
    conceptosApoyo: wjcCore.CollectionView;
    conceptosIsr: wjcCore.CollectionView;
    conceptosPrevio: wjcCore.CollectionView;
    calcularComision: boolean = false;
    calcular: boolean = true;
    timerInterval: any = null;
    idConceptoParent: number = 0;

    lastCedulaConcepto: CedulaConcepto;
    newCedulaConcepto: CedulaConcepto = new CedulaConcepto();
    menuItemsConcepto: MenuItem[] = [];

    tileDowmload: string = "";
    nameDownload: string = "";
    fechaAdeudo: Date = new Date();
    maxDate = new Date();

    detailMode = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandSingle];
    detailModeMulti = wjcGridDetail.DetailVisibilityMode[wjcGridDetail.DetailVisibilityMode.ExpandMulti];

    @ContentChildren(MenuItemDirective) menuItems: QueryList<MenuItemDirective>;
    @ViewChild('dlgConfigirmacionCalculo') dlgConfigirmacionCalculo: ModalDirective;
    @ViewChild('dlgConcepto') dlgConcepto: ModalDirective;
    @ViewChild('dlgAdeudo') dlgAdeudo: ModalDirective;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private cedulaFiniquitoService: CedulaFiniquitoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'empleado': [''],
            'nombre': [''],
            'apPaterno': [''],
            'apMaterno': [''],
            'departamento': [''],
            'bodega': [''],
            'sindicalizado': [''],
            'fechaAntiguedad': [''],
            'ultimoDiaLaborado': [''],
            'fechaBaja': [''],
            'fechaUltimoAguinaldo': [''],
            'diasAguinaldoContrato': [''],
            'disfrutePrevioAguinaldo': [''],
            'primaVacacionalPendiente': [''],
            'pirmaVacExentaPagada': [''],
            'porcentajePrimaVacacional': [''],
            'diasVacacionesPendiente': [''],
            'diasVacacionesGeneradas': [''],
            'totalDiasVacaciones': [''],
            'porcentajePrimaAntiguedad': [''],
            'porcentajeVeinteDiasAnio': [''],
            'sueldoDiario': [''],
            'sueldoPromedio': [''],
            'sueldoDiarioIntegrado': [''],
            'ausentismo': [''],
            'sueldoDias': [''],
            'sueldoImporte': [''],
            'comisiones': [''],
            'incentivos': [''],
            'horasExtraImporte': [''],
            'primaVacacionalPendienteEntrega': [''],
            'primaDominicalNumDomingos': [''],
            'primaDominicalImporte': [''],
            'repartoUtilidades': [''],
            'diasNegociadoIndemnizacion': [''],
            'otros': [''],
            'salarioMinimoZonaNegosiacion': [''],
            'salarioMinimoZonaApoyo': [''],
            'idRescicionLaboral': [''],
            'comisionespagado': [''],
            'infonavit': [''],
            'fonacot': [''],
            'adeudo': [''],
            'importeNetoFiniquito': [''],
            'newConcepto': [''],
            'newCantidad': [''],
            'newImporteGravado': [''],
            'newImporteExento': [''],
            'newImporteTotal': [''],
            'primaVacacionalExento': ['']
        });
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ngAfterViewInit() {
        super.ngAfterViewInit();
        this.createMenu();
        
    }

    createMenu() {
        let mnuAgergar: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Ageregar Adeudo", name: "AGREGAR_ADEUDO", smallText: "Agregar nuevo adeudo" };
        let mnuEliminar: MenuItem = { icon: "glyphicon glyphicon-minus", text: "Eliminar Adeudo", name: "ELIMINAR_ADEUDO", smallText: "Eliminar adeudo" };

        let mnuAgergarPercepcion: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Ageregar Percepción", name: "AGREGAR_PERCEPCION", smallText: "Agregar nuevo adeudo" };
        let mnuEliminarPercepcion: MenuItem = { icon: "glyphicon glyphicon-minus", text: "Eliminar Percepción", name: "ELIMINAR_PERCEPCION", smallText: "Eliminar adeudo" };

        this.menuItemsConcepto.push(mnuAgergar);
        this.menuItemsConcepto.push(mnuEliminar);
        this.menuItemsConcepto.push(mnuAgergarPercepcion);
        this.menuItemsConcepto.push(mnuEliminarPercepcion);


        this.menuItemsConcepto.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    onMenuItemClicked(menu: Menu, args: any) {
        let item: MenuItem = this.menuItemsConcepto[menu.selectedIndex];
        if (item.name == "AGREGAR_ADEUDO") {
            this.idConceptoParent = 149;
            this.lastCedulaConcepto = this.conceptosPrevio.currentItem;
            this.conceptosPrevio.addNew();
            this.newCedulaConcepto.concepto = "";
            this.newCedulaConcepto.importeTotal = 0;
            this.dlgConcepto.config.backdrop = false;
            this.dlgConcepto.config.ignoreBackdropClick = true;
            this.dlgConcepto.show();
        }
        else if (item.name == "ELIMINAR_ADEUDO") {
            if (this.conceptosPrevio.currentItem.idParent == 149) {
                this.conceptosPrevio.currentItem.importeTotal = 0;
                this.cedulaFiniquito.eliminar.push(this.conceptosPrevio.currentItem);
                this.conceptosPrevio.remove(this.conceptosPrevio.currentItem);
                this.cedulaPrevioFiniquito();
            }
        }
        else if (item.name == "AGREGAR_PERCEPCION") {
            this.idConceptoParent = 159;
            this.lastCedulaConcepto = this.conceptosPrevio.currentItem;
            this.conceptosPrevio.addNew();
            this.newCedulaConcepto.concepto = "";
            this.newCedulaConcepto.importeTotal = 0;
            this.dlgConcepto.config.backdrop = false;
            this.dlgConcepto.config.ignoreBackdropClick = true;
            this.dlgConcepto.show();
        }
        else if (item.name == "ELIMINAR_PERCEPCION") {
            if (this.conceptosPrevio.currentItem.idParent == 159) {
                this.conceptosPrevio.currentItem.importeTotal = 0;
                this.cedulaFiniquito.eliminar.push(this.conceptosPrevio.currentItem);
                this.conceptosPrevio.remove(this.conceptosPrevio.currentItem);
                this.cedulaPrevioFiniquito();
            }
        }
    }

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;

        this.cedulaFiniquitoService.loadRescisionesLaborales()
            .then(info => {
                this.context.app.hideSpinner();
                this.rescisionesLaborales = info;
            })
            .catch(error => this.handleError(error));
    }

    onCreatingItem() {

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.cedulaFiniquitoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cedulaFiniquito = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.cedulaFiniquitoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cedulaFiniquito = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.cedulaFiniquitoService.getDetailByID(id)
            .then(info => {

                this.calcular = false;
                this.context.app.hideSpinner();
                this.cedulaFiniquito = info;

                this.conceptosCedulaNegosiacion = new wjcCore.CollectionView(this.cedulaFiniquito.conceptosNegociacion);
                this.conceptosCedulaNegosiacion.refresh();
                var cvn = this.conceptosCedulaNegosiacion;
                cvn.beginUpdate();
                cvn.groupDescriptions.clear();
                var groupDesc = new wjcCore.PropertyGroupDescription("clasificacionConcepto.descripcion");
                cvn.groupDescriptions.push(groupDesc);
                cvn.refresh();
                cvn.endUpdate();
                cvn.refresh();

                this.conceptosApoyo = new wjcCore.CollectionView(this.cedulaFiniquito.conceptosApoyo);
                this.conceptosApoyo.refresh();
                var cva = this.conceptosApoyo;
                cva.beginUpdate();
                cva.groupDescriptions.clear();
                var groupDescApoyo = new wjcCore.PropertyGroupDescription("clasificacionConcepto.descripcion");
                cva.groupDescriptions.push(groupDescApoyo);
                cva.refresh();
                cva.endUpdate();
                cva.refresh();

                this.conceptosIsr = new wjcCore.CollectionView(this.cedulaFiniquito.conceptosIsr);
                this.conceptosIsr.refresh();
                var cvIsr = this.conceptosIsr;
                cvIsr.beginUpdate();
                cvIsr.groupDescriptions.clear();

                var groupDescIsr = new wjcCore.PropertyGroupDescription("clasificacionConcepto.descripcion");
                cvIsr.groupDescriptions.push(groupDescIsr);
                cvIsr.refresh();
                cvIsr.endUpdate();
                cvIsr.refresh();

                this.conceptosPrevio = new wjcCore.CollectionView(this.cedulaFiniquito.conceptosPrevio);
                this.conceptosPrevio.refresh();
                var cvPrevio = this.conceptosPrevio;
                cvPrevio.beginUpdate();
                cvPrevio.groupDescriptions.clear();

                var groupDescPrevio = new wjcCore.PropertyGroupDescription("clasificacionConcepto.descripcion");
                cvPrevio.groupDescriptions.push(groupDescPrevio);
                cvPrevio.refresh();
                cvPrevio.endUpdate();
                cvPrevio.refresh();

                this.calcular = true;

                // Desencadenador
                this.cedulaIsrPartesProporcionales();
                this.cedulaDeduccionesIsr(5);
                this.cedulaDeduccionesIsr(10);
                this.calcularSeguros(4);
                this.calcularSeguros(9);

            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.cedulaPrevioFiniquito();
        this.context.app.showSpinner();
        this.cedulaFiniquitoService.update(this.cedulaFiniquito)
            .then(info => {
                this.context.app.hideSpinner();
                this.cedulaFiniquito = info;
                this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onEmpleadoSelected(args: any) {
        if (this.cedulaFiniquito.identity == 0)  {
            this.dlgConfigirmacionCalculo.config.backdrop = false;
            this.dlgConfigirmacionCalculo.config.ignoreBackdropClick = true;
            this.dlgConfigirmacionCalculo.show();
        }
    }

    loadIngresoInformacion(idEmpleado: number, comision: boolean) {
        if (this.cedulaFiniquito.identity == 0) {
            this.context.app.showSpinner();
            this.cedulaFiniquitoService.loadIngresoInformacion(idEmpleado, comision).then(info => {
                this.cedulaFiniquito.empleado = info.empleado;
                this.cedulaFiniquito.clase = info.clase;
                this.cedulaFiniquito.departamento = info.departamento;
                this.cedulaFiniquito.bodega = info.bodega;
                this.cedulaFiniquito.puesto = info.puesto;
                this.cedulaFiniquito.diasAguinaldoContrato = info.diasAguinaldoContrato;
                this.cedulaFiniquito.aguinaldoSindicato = info.aguinaldoSindicato;
                this.cedulaFiniquito.sindicalizado = info.sindicalizado;

                if (this.cedulaFiniquito.sindicalizado) {
                    this.cedulaFiniquito.diasAguinaldoContrato = info.aguinaldoSindicato;
                }

                this.cedulaFiniquito.fechaAntiguedad = moment(info.fechaAntiguedad).toDate();
                this.cedulaFiniquito.fechaUltimoAguinaldo = moment(info.fechaUltimoAguinaldo).toDate();
                this.cedulaFiniquito.ultimoDiaLaborado = moment(info.ultimoDiaLaborado).toDate();
                this.cedulaFiniquito.fechaBaja = moment(info.fechaBaja).toDate();
                this.cedulaFiniquito.disfrutePrevioAguinaldo = info.disfrutePrevioAguinaldo;
                this.cedulaFiniquito.pirmaVacExentaPagada = info.pirmaVacExentaPagada;
                this.cedulaFiniquito.primaVacacionalPendiente = info.primaVacacionalPendiente;
                this.cedulaFiniquito.porcentajePrimaVacacional = info.porcentajePrimaVacacional;
                this.cedulaFiniquito.diasVacacionesPendiente = info.diasVacacionesPendiente;
                this.cedulaFiniquito.diasVacacionesGeneradas = info.diasVacacionesGeneradas;
                this.cedulaFiniquito.porcentajePrimaAntiguedad = info.porcentajePrimaAntiguedad;
                this.cedulaFiniquito.porcentajeVeinteDiasAnio = info.porcentajeVeinteDiasAnio;
                this.cedulaFiniquito.sueldoDiario = info.sueldoDiario;
                this.cedulaFiniquito.sueldoPromedio = info.sueldoPromedio;
                this.cedulaFiniquito.sueldoDiarioIntegrado = info.sueldoDiarioIntegrado;
                this.cedulaFiniquito.ausentismo = info.ausentismo;
                this.cedulaFiniquito.sueldoDias = info.sueldoDias;
                this.cedulaFiniquito.sueldoImporte = info.sueldoImporte;
                this.cedulaFiniquito.comisiones = info.comisiones;
                this.cedulaFiniquito.incentivos = info.incentivos;
                this.cedulaFiniquito.horasExtraImporte = info.horasExtraImporte;
                this.cedulaFiniquito.primaVacacionalPendienteEntrega = info.primaVacacionalPendienteEntrega;
                this.cedulaFiniquito.primaDominicalNumDomingos = info.primaDominicalNumDomingos;
                this.cedulaFiniquito.primaDominicalImporte = info.primaDominicalImporte;
                this.cedulaFiniquito.repartoUtilidades = info.repartoUtilidades;
                this.cedulaFiniquito.diasNegociadoIndemnizacion = info.diasNegociadoIndemnizacion;
                this.cedulaFiniquito.indemnnizacionAdicional = info.indemnnizacionAdicional;
                this.cedulaFiniquito.otros = info.otros;
                this.cedulaFiniquito.salarioMinimoZona = info.salarioMinimoZona;
                this.cedulaFiniquito.uma = info.uma;
                this.cedulaFiniquito.conceptosNegociacion = info.conceptosNegociacion;
                this.cedulaFiniquito.conceptosApoyo = info.conceptosApoyo;
                this.cedulaFiniquito.conceptosIsr = info.conceptosIsr;
                this.cedulaFiniquito.idRescicionLaboral = info.idRescicionLaboral;
                this.cedulaFiniquito.seguros = info.seguros;
                this.cedulaFiniquito.conceptosPrevio = info.conceptosPrevio;
                this.cedulaFiniquito.tarifasPagosProvicionales = info.tarifasPagosProvicionales;
                this.cedulaFiniquito.subsidiosEmpleo = info.subsidiosEmpleo;
                this.cedulaFiniquito.idTaskComision = info.idTaskComision;
                this.cedulaFiniquito.infonavit = info.infonavit;
                this.cedulaFiniquito.fonacot = info.fonacot;
                this.cedulaFiniquito.autoriza = info.autoriza;
                this.cedulaFiniquito.infonavitOrFonacot = info.infonavitOrFonacot;
                this.cedulaFiniquito.periodoPago = info.periodoPago;
                this.cedulaFiniquito.adeudo = info.adeudo;
                this.cedulaFiniquito.variables = info.variables;
                this.cedulaFiniquito.primaVacacionalExento = info.primaVacacionalExento;

                this.conceptosCedulaNegosiacion = new wjcCore.CollectionView(this.cedulaFiniquito.conceptosNegociacion);
                this.conceptosCedulaNegosiacion.refresh();
                var cvn = this.conceptosCedulaNegosiacion;
                cvn.beginUpdate();
                cvn.groupDescriptions.clear();
                var groupDesc = new wjcCore.PropertyGroupDescription("clasificacionConcepto.descripcion");
                cvn.groupDescriptions.push(groupDesc);
                cvn.refresh();
                cvn.endUpdate();
                cvn.refresh();

                this.conceptosApoyo = new wjcCore.CollectionView(this.cedulaFiniquito.conceptosApoyo);
                this.conceptosApoyo.refresh();
                var cva = this.conceptosApoyo;
                cva.beginUpdate();
                cva.groupDescriptions.clear();
                var groupDescApoyo = new wjcCore.PropertyGroupDescription("clasificacionConcepto.descripcion");
                cva.groupDescriptions.push(groupDescApoyo);
                cva.refresh();
                cva.endUpdate();
                cva.refresh();

                this.conceptosIsr = new wjcCore.CollectionView(this.cedulaFiniquito.conceptosIsr);
                this.conceptosIsr.refresh();
                var cvIsr = this.conceptosIsr;
                cvIsr.beginUpdate();
                cvIsr.groupDescriptions.clear();
                var groupDescIsr = new wjcCore.PropertyGroupDescription("clasificacionConcepto.descripcion");
                cvIsr.groupDescriptions.push(groupDescIsr);
                cvIsr.refresh();
                cvIsr.endUpdate();
                cvIsr.refresh();

                this.conceptosPrevio = new wjcCore.CollectionView(this.cedulaFiniquito.conceptosPrevio);
                this.conceptosPrevio.refresh();
                var cvPrevio = this.conceptosPrevio;
                cvPrevio.beginUpdate();
                cvPrevio.groupDescriptions.clear();
                var groupDescPrevio = new wjcCore.PropertyGroupDescription("clasificacionConcepto.descripcion");
                cvPrevio.groupDescriptions.push(groupDescPrevio);
                cvPrevio.refresh();
                cvPrevio.endUpdate();
                cvPrevio.refresh();

                if (this.timerInterval != null)
                    clearInterval(this.timerInterval);

                if (this.cedulaFiniquito.idTaskComision > 0) {
                    this.timerInterval = setInterval(() => {
                        this.verificarTaskComision();
                    }, 3000);
                }
                else {
                    this.context.app.hideSpinner();
                }

            })
            .catch(error => this.handleError(error));
        }

    }

    onCambioSindicalizado(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.sindicalizado = args;
        if (this.cedulaFiniquito.sindicalizado) {
            this.cedulaFiniquito.diasAguinaldoContrato = this.cedulaFiniquito.clase.aguinaldoSindicato;
        }
        else {
            this.cedulaFiniquito.diasAguinaldoContrato = this.cedulaFiniquito.clase.aguinaldo;
        }
    }

    onSeleccionRescision(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.idRescicionLaboral = args;
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(3);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);

        this.cedulaConceptoTotalPagosSeparacion();
        this.cedulaIsrInjustificado();
        this.cedulaConceptoTotalPagosSeparacionInjust();
        this.cedulaIsrJustificado();

        this.cedulaIsrPercepcionesAdeudadas();
        this.cedulaPrevioFiniquito();

    }

    onCambioFechaAntiguedad(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.fechaAntiguedad = args;
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(3);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
    }

    onCambioUltimoDiaLaborado(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.ultimoDiaLaborado = args;
        this.cedulaConceptoAguinaldo(2);
        this.cedulaConceptoAguinaldo(7);
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
    }

    onCambioFechaBaja(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.fechaBaja = args;
        this.cedulaConceptoAguinaldo(2);
        this.cedulaConceptoAguinaldo(7);
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
    }

    onCambioFechaUltimoAguinaldo(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.fechaUltimoAguinaldo = args;
        this.cedulaConceptoAguinaldo(2);
        this.cedulaConceptoAguinaldo(7);
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
    }

    onCambioDiasAguinaldoContrato(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.diasAguinaldoContrato = args;
        this.cedulaConceptoAguinaldo(2);
        this.cedulaConceptoAguinaldo(7);
    }

    onCambioDisfrutePrevioAguinaldo(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.disfrutePrevioAguinaldo = args as boolean;
        this.cedulaConceptoAguinaldo(2);
        this.cedulaConceptoAguinaldo(7);
    }

    onCambioPrimaVacacionalPendiente(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.primaVacacionalPendiente = args;
        this.cedulaConceptoPrimaVacacional(2);
        this.cedulaConceptoPrimaVacacional(7);
    }

    onCambioPirmaVacExentaPagada(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.pirmaVacExentaPagada = args;
        this.cedulaConceptoPrimaVacacional(2);
        this.cedulaConceptoPrimaVacacional(7);
    }

    onCambioPorcentajePrimaVacacional(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.porcentajePrimaVacacional = args;
        this.cedulaConceptoPrimaVacacional(2);
        this.cedulaConceptoPrimaVacacional(7);
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
    }

    onCambioVacacionesPendiente(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.diasVacacionesPendiente = args;
        this.cedulaFiniquito.totalDiasVacaciones = this.cedulaFiniquito.diasVacacionesGeneradas + args;
    }

    onCambioVacacionesGeneradas(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.diasVacacionesGeneradas = args;
        this.cedulaFiniquito.totalDiasVacaciones = this.cedulaFiniquito.diasVacacionesPendiente + args;
        this.cedulaConceptoPrimaVacacional(2);
        this.cedulaConceptoPrimaVacacional(7);
    }

    onCambioTotalDiasVacaciones(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.totalDiasVacaciones = args;
        this.cedulaConceptoVacaciones(2);
        this.cedulaConceptoVacaciones(7);
        this.cedulaConceptoPrimaVacacional(2);
        this.cedulaConceptoPrimaVacacional(7);
    }

    onCambioPorcentajePrimaAntiguedad(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.porcentajePrimaAntiguedad = args;
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(3);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);

        this.cedulaConceptoTotalPagosSeparacion();
        this.cedulaIsrInjustificado();
        this.cedulaConceptoTotalPagosSeparacionInjust();
        this.cedulaIsrJustificado();
        this.cedulaIsrPercepcionesAdeudadas();

        this.cedulaPrevioFiniquito();
    }

    onCambioPorcentajeVeinteDiasAnio(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.porcentajeVeinteDiasAnio = args;
        this.cedulaConceptoIndeminzacionLeyDiasAnio(3);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
        this.cedulaDeduccionesIsr(5);
        this.cedulaDeduccionesIsr(10);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
    }

    onCambioSueldoDiario(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.sueldoDiario = args;
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
        this.cedulaIsrSueldoMensualOrdinario();
    }

    onCambioSueldoPromedio(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.sueldoPromedio = args;
        this.cedulaConceptoAguinaldo(2);
        this.cedulaConceptoAguinaldo(7);
        this.cedulaConceptoVacaciones(2);
        this.cedulaConceptoVacaciones(7);
        this.cedulaConceptoPrimaVacacional(2);
        this.cedulaConceptoPrimaVacacional(7);
        this.cedulaConceptoAdeudoSueldo(1);
        this.cedulaConceptoAdeudoSueldo(6);
        this.cedulaConceptoHorasExtra(1);
        this.cedulaConceptoHorasExtra(6);
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(3);
        this.cedulaConceptoIndeminzacionLeyDiasAnio(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
        this.cedulaConceptoIndeminzacionLeyDias(3);
        this.cedulaConceptoIndeminzacionLeyDias(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
        this.cedulaIsrSueldoMensualOrdinario();
    }

    onCambioSueldoDiarioIntegrado(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.sueldoDiarioIntegrado = args;
        this.cedulaInfonavitFonacot();
        this.cedulaIsrSueldoMensualOrdinario();
    }

    onCambioAusentismo(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.ausentismo = args;
        this.cedulaConceptoAguinaldo(2);
        this.cedulaConceptoAguinaldo(7);
    }

    onCambioSueldoDias(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.sueldoDias = args;
        // Calculos que depende del concepto
        this.cedulaInfonavitFonacot();

        this.cedulaConceptoAdeudoSueldo(1);
        this.cedulaConceptoAdeudoSueldo(6);
        this.cedulaConceptoHorasExtra(1);
        this.cedulaConceptoHorasExtra(6);

        this.cedulaIsrPartesProporcionales();
        this.cedulaIsrInjustificado();
        this.cedulaIsrJustificado();
        this.cedulaIsrPercepcionesAdeudadas();
    }

    onCambioSueldoImporte(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.sueldoImporte = args;
        this.cedulaConceptoAdeudoSueldo(1);
        this.cedulaConceptoAdeudoSueldo(6);
        this.cedulaConceptoHorasExtra(1);
        this.cedulaConceptoHorasExtra(6);
    }

    onCambioComisiones(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.comisiones = args;
        this.cedulaConceptoComisiones(1);
        this.cedulaConceptoComisiones(6);
    }

    onCambioIncentivos(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.incentivos = args;
        this.cedulaConceptoIncentivos(1);
        this.cedulaConceptoIncentivos(6);
    }

    onCambioHorasExtraImporte(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.horasExtraImporte = args;
        this.cedulaConceptoHorasExtra(1);
        this.cedulaConceptoHorasExtra(6);
    }

    onCambioPrimaVacacionalPendienteEntrega(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.primaVacacionalPendienteEntrega = args;
        this.cedulaConceptoPrimaVacacional(2);
        this.cedulaConceptoPrimaVacacional(7);
    }

    onCambioNumDomingos(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.primaDominicalNumDomingos = args;
        this.cedulaConceptoPrimaDominical(2);
        this.cedulaConceptoPrimaDominical(7);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
    }

    onCambioPrimaDominicalImporte(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.primaDominicalImporte = args;
        this.cedulaConceptoPrimaDominical(2);
        this.cedulaConceptoPrimaDominical(7);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
    }

    onCambioRepartoUtilidades(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.repartoUtilidades = args;
        this.cedulaConceptoRepartoUtilidades(1);
        this.cedulaConceptoRepartoUtilidades(6);
    }

    onCambioDiasNegociadosIndemnizacion(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.diasNegociadoIndemnizacion = args;
        this.cedulaConceptoIndeminzacionLeyDias(3);
        this.cedulaConceptoIndeminzacionLeyDias(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
    }

    onCambioIndemnizacionAdicional(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.indemnnizacionAdicional = args;
        this.cedulaConceptoIndeminzacionLeyDias(3);
        this.cedulaConceptoIndeminzacionLeyDias(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
    }

    onCambioOtros(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.otros = args;
        this.cedulaConceptoIndeminzacionLeyDias(3);
        this.cedulaConceptoIndeminzacionLeyDias(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
        this.cedulaConceptoOtros(1);
        this.cedulaConceptoOtros(6);
    }

    onCambioSalarioMinimoZona(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.salarioMinimoZona = args;
        this.onCambioIndemnizacionAdicional(this.cedulaFiniquito.indemnnizacionAdicional);
        this.cedulaConceptoHorasExtra(1);
        this.cedulaConceptoHorasExtra(6);
        this.cedulaConceptoRepartoUtilidades(1);
        this.cedulaConceptoRepartoUtilidades(6);
        this.cedulaConceptoOtros(1);
        this.cedulaConceptoOtros(6);
        this.cedulaConceptoAguinaldo(2);
        this.cedulaConceptoAguinaldo(7);
        this.cedulaConceptoPrimaDominical(2);
        this.cedulaConceptoPrimaDominical(7);
        this.cedulaConceptoPrimaVacacional(2);
        this.cedulaConceptoPrimaVacacional(7);
        this.cedulaConceptoPrimaAntiguedad(3);
        this.cedulaConceptoPrimaAntiguedad(8);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(3);
        this.cedulaConceptoIndeminzacionAdicionalAcordada(8);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
        this.cedulaIsrPartesProporcionales();
        this.cedulaConceptoTotalPagosSeparacion();
        this.cedulaIsrInjustificado();
        this.cedulaConceptoTotalPagosSeparacionInjust();
        this.cedulaIsrJustificado();
        this.calcularSeguros(4);
        this.calcularSeguros(9);
    }

    onCambioInfonavit(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.infonavit = args;
        this.calcularImporteAdeudo(4);
        this.calcularImporteAdeudo(9);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
        this.cedulaPrevioFiniquito();
    }

    onCambioFonacot(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.fonacot = args;
        this.calcularImporteAdeudo(4);
        this.calcularImporteAdeudo(9);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
        this.cedulaPrevioFiniquito();
    }

    onCambioAdeudo(args: any) {
        if (!this.calcular)
            return;
        this.cedulaFiniquito.adeudo = args;
        this.calcularImporteAdeudo(4);
        this.calcularImporteAdeudo(9);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
        this.cedulaPrevioFiniquito();
    }

    consultarCalculo(calcular: boolean) {
        this.loadIngresoInformacion(this.cedulaFiniquito.idEmpleado, calcular);
        this.dlgConfigirmacionCalculo.hide();
    }

    onConceptoCancelar() {
        this.dlgConcepto.hide();
        this.conceptosPrevio.cancelEdit();
        this.conceptosPrevio.cancelNew();
    }

    onConceptoAgregar() {
        this.dlgConcepto.hide();
        this.conceptosPrevio.currentItem.concepto                   = this.newCedulaConcepto.concepto;
        this.conceptosPrevio.currentItem.idClasificacionConcepto    = this.lastCedulaConcepto.idClasificacionConcepto;
        this.conceptosPrevio.currentItem.cantidad                   = this.newCedulaConcepto.cantidad;
        this.conceptosPrevio.currentItem.importeGravado             = this.newCedulaConcepto.importeGravado;
        this.conceptosPrevio.currentItem.importeExento              = this.newCedulaConcepto.importeExento;
        this.conceptosPrevio.currentItem.importeTotal               = this.newCedulaConcepto.importeTotal;
        this.conceptosPrevio.currentItem.idParent                   = this.idConceptoParent; // Conxepto de adedos.
        this.conceptosPrevio.currentItem.acumula                    = this.conceptosPrevio.currentItem.acumula;
        this.cedulaFiniquito.conceptosPrevio.forEach(element => {
            if (element.idConcepto == this.idConceptoParent) {
                this.conceptosPrevio.currentItem.clasificacionConcepto = element.clasificacionConcepto;
            }
        });
        this.conceptosPrevio.commitEdit();
        this.conceptosPrevio.commitNew();
        this.conceptosPrevio.refresh();
        this.cedulaPrevioFiniquito();
    }


    /**
    * Calcula el concepto de adeudo sueldo
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) negosiación 
    */
    cedulaConceptoAdeudoSueldo(idClasificacion: number) {
        var concepto: CedulaConcepto = this.getConcepto(idClasificacion, "Adeudo de Sueldo");
        if (idClasificacion == 1) {
            if (this.cedulaFiniquito.sueldoImporte > 0) {
                concepto.cantidad = (this.cedulaFiniquito.sueldoImporte / this.cedulaFiniquito.sueldoPromedio);
            }
            else {
                concepto.cantidad = this.cedulaFiniquito.sueldoDias;
            }

            if (this.cedulaFiniquito.sueldoImporte > 0) {
                concepto.importeTotal = this.cedulaFiniquito.sueldoImporte;
            }
            else {
                concepto.importeTotal = concepto.cantidad * this.cedulaFiniquito.sueldoDiario;
            }
            concepto.importeGravado = concepto.importeTotal;
        }
        else if (idClasificacion == 6) {
            concepto.cantidad = (this.cedulaFiniquito.sueldoDias);
            concepto.importeTotal = concepto.cantidad * this.cedulaFiniquito.sueldoDiario;
            concepto.importeGravado = concepto.importeTotal;
        }

        // Calculos que depende del concepto
        this.cedulaIsrPartesProporcionales();
        this.cedulaIsrInjustificado();
        this.cedulaIsrJustificado();
        this.cedulaIsrPercepcionesAdeudadas();
    }

    /**
    * Calcula las concepto de comisiones
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoComisiones(idClasificacion: number) {
        var concepto: CedulaConcepto = this.getConcepto(idClasificacion, "Comisiones");
        concepto.importeTotal = (this.cedulaFiniquito.comisiones);
        concepto.importeGravado = concepto.importeTotal;

        // Calculos que depende del concepto
        this.cedulaIsrPartesProporcionales();
        this.cedulaIsrInjustificado();
        this.cedulaIsrJustificado();
        this.cedulaIsrPercepcionesAdeudadas();
    }

    /**
    * Calcula las concepto de incentivos
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoIncentivos(idClasificacion: number) {
        var concepto: CedulaConcepto = this.getConcepto(idClasificacion, "Incentivos");
        concepto.importeTotal = this.cedulaFiniquito.incentivos;
        concepto.importeGravado = concepto.importeTotal;

        // Calculos que depende del concepto
        this.cedulaIsrPartesProporcionales();
        this.cedulaIsrInjustificado();
        this.cedulaIsrJustificado();
        this.cedulaIsrPercepcionesAdeudadas();
    }

    /**
    * Calcula las concepto de Horas Extra
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoHorasExtra(idClasificacion) {
        var concepto = this.getConcepto(idClasificacion, "Horas Extra (Importe)");
        var conceptoAdeudo = this.getConcepto(idClasificacion, "Adeudo de Sueldo");
        concepto.importeTotal = this.cedulaFiniquito.horasExtraImporte;
        if (concepto.importeTotal > 0) {
            concepto.importeExento = this.cedulaFiniquito.salarioMinimoZona * 5 * (conceptoAdeudo.cantidad == 0 ? 0 : ((conceptoAdeudo.cantidad / 7) < 1 ? 1 : Math.round(conceptoAdeudo.cantidad / 7)));
        }
        else {
            concepto.importeExento = 0;
        }
        concepto.importeGravado = concepto.importeTotal - concepto.importeExento;
        // Calculos que depende del concepto
        this.cedulaIsrPartesProporcionales();
        this.cedulaIsrInjustificado();
        this.cedulaIsrJustificado();
        this.cedulaIsrPercepcionesAdeudadas();
    }

    /**
    * Calcula las concepto de Reparto de Utilidades
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoRepartoUtilidades(idClasificacion: number) {
        var concepto: CedulaConcepto = this.getConcepto(idClasificacion, "Reparto Utilidades");
        concepto.importeTotal = this.cedulaFiniquito.repartoUtilidades;
        if (concepto.importeTotal > 0) {
            if (concepto.importeTotal < (this.cedulaFiniquito.salarioMinimoZona * 15)) {
                concepto.importeExento = concepto.importeTotal;
            }
            else {
                concepto.importeExento = (this.cedulaFiniquito.salarioMinimoZona * 15);
            }
        }
        else {
            concepto.importeExento = 0;
        }
        concepto.importeGravado = concepto.importeTotal - concepto.importeExento;

        // Calculos que depende del concepto
        this.cedulaIsrPartesProporcionales();
        this.cedulaIsrInjustificado();
        this.cedulaIsrJustificado();
        this.cedulaIsrPercepcionesAdeudadas();
    }

    /**
    * Calculo del concepto Otros (En otros todo es gravado)
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (1) negociacion o (6) apoyo
    */
    cedulaConceptoOtros(idClasificacion: number) {
        var concepto: CedulaConcepto = this.getConcepto(idClasificacion, "Otros");
        concepto.importeTotal = this.cedulaFiniquito.otros;
        concepto.importeGravado = this.cedulaFiniquito.otros;

        // Calculos que depende del concepto
        this.cedulaIsrPartesProporcionales();
        this.cedulaIsrInjustificado();
        this.cedulaIsrJustificado();
        this.cedulaIsrPercepcionesAdeudadas();
    }

    /**
    * Calcula el concepto de aguninaldo
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) negosiación 
    */
    cedulaConceptoAguinaldo(idClasificacion: number) {
        var concepto: CedulaConcepto = this.getConcepto(idClasificacion, "Aguinaldo");
        if (idClasificacion == 2) {
            if (this.cedulaFiniquito.disfrutePrevioAguinaldo) {
                concepto.cantidad = 0;
            }
            else {
                var dias = (this.getDaysBetweenTwoDates(this.cedulaFiniquito.ultimoDiaLaborado, this.cedulaFiniquito.fechaUltimoAguinaldo) + 1) + (this.cedulaFiniquito.ausentismo) * -1;
                dias = dias / 365;
                dias = dias * this.cedulaFiniquito.diasAguinaldoContrato;
                concepto.cantidad = (dias);
            }
        }
        else if (idClasificacion == 7) {
            if (this.cedulaFiniquito.disfrutePrevioAguinaldo) {
                concepto.cantidad = 0;
            }
            else {
                concepto.cantidad = (((this.getDaysBetweenTwoDates(this.cedulaFiniquito.ultimoDiaLaborado, this.cedulaFiniquito.fechaUltimoAguinaldo) - (this.cedulaFiniquito.ausentismo)) / 365) * this.cedulaFiniquito.diasAguinaldoContrato);
            }
        }

        concepto.importeTotal = (concepto.cantidad * this.cedulaFiniquito.sueldoPromedio);

        if (this.cedulaFiniquito.uma * 30 < concepto.importeTotal) {
            concepto.importeGravado = (concepto.importeTotal - (this.cedulaFiniquito.uma * 30));
        }
        else {
            concepto.importeGravado = 0;
        }

        if (concepto.importeTotal < (this.cedulaFiniquito.uma * 30)) {
            concepto.importeExento = concepto.importeTotal;
        }
        else {
            concepto.importeExento = (this.cedulaFiniquito.uma * 30);
        }

        this.cedulaIsrPartesProporcionales();
    }

    /**
    * Calcula el concepto de vacaciones
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) negosiación 
    */
    cedulaConceptoVacaciones(idClasificacion: number) {
        var concepto = this.getConcepto(idClasificacion, "Vacaciones");
        concepto.cantidad = this.cedulaFiniquito.totalDiasVacaciones;
        concepto.importeTotal = (concepto.cantidad) * this.cedulaFiniquito.sueldoPromedio;
        concepto.importeGravado = (concepto.importeTotal);

        this.cedulaIsrPartesProporcionales();
    }

    /**
    * Calcula el concepto de Prima Dominical
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) negosiación 
    */
    cedulaConceptoPrimaDominical(idClasificacion: number) {
        var concepto = this.getConcepto(idClasificacion, "Prima Dominical");
        concepto.cantidad = this.cedulaFiniquito.primaDominicalNumDomingos;
        concepto.importeTotal = this.cedulaFiniquito.primaDominicalImporte;
        if (concepto.cantidad == 0) {
            concepto.importeExento = 0;
        }
        else {
            concepto.importeExento = concepto.importeTotal <= this.cedulaFiniquito.uma * concepto.cantidad ? concepto.importeTotal : this.cedulaFiniquito.uma * concepto.cantidad;
        }
        concepto.importeGravado = concepto.importeTotal - concepto.importeExento;
    }

    /**
    * Calcula el concepto de Prima Vacacional
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (2) negosiacion o (7) apoyo previo finiquito
    */
    cedulaConceptoPrimaVacacional(idClasificacion: number) {
        var concepto: CedulaConcepto = this.getConcepto(idClasificacion, "Prima Vacacional");
        var conceptoVacaciones: CedulaConcepto = this.getConcepto(idClasificacion, "Vacaciones");
        let importeExento = ((this.cedulaFiniquito.uma * 15) - this.cedulaFiniquito.pirmaVacExentaPagada) < 0 ? 0 : (this.cedulaFiniquito.uma * 15) - this.cedulaFiniquito.pirmaVacExentaPagada;
        if (idClasificacion == 2) {
            if (this.cedulaFiniquito.primaVacacionalPendiente) {
                concepto.cantidad = conceptoVacaciones.cantidad * (this.cedulaFiniquito.porcentajePrimaVacacional / 100);
            }
            else {
                concepto.cantidad = this.cedulaFiniquito.diasVacacionesGeneradas * (this.cedulaFiniquito.porcentajePrimaVacacional / 100);
            }
        }
        else if (idClasificacion == 7) {
            concepto.cantidad = (conceptoVacaciones.cantidad) * 0.5;
        }
        concepto.importeTotal = concepto.cantidad * this.cedulaFiniquito.sueldoPromedio;
        if (importeExento >= concepto.importeTotal) {
            concepto.importeExento = concepto.importeTotal;
        }
        else {
            concepto.importeExento = importeExento;
        }
        concepto.importeGravado = concepto.importeTotal - concepto.importeExento;
        this.cedulaIsrPartesProporcionales();
    }

    /**
    * Calcula el concepto de Prima Antiguedad
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (3) negosiacion o (8) negosiación 
    */
    cedulaConceptoPrimaAntiguedad(idClasificacion: number) {
        var concepto    = this.getConcepto(idClasificacion, "Prima Antigüedad");
        var topeSueldo = 0;
        var sueldoMinimoZona = this.cedulaFiniquito.salarioMinimoZona;

        if (this.cedulaFiniquito.sueldoPromedio > sueldoMinimoZona * 2) {
            topeSueldo = sueldoMinimoZona * 2;
        }
        else if (this.cedulaFiniquito.sueldoPromedio < sueldoMinimoZona * 2 && this.cedulaFiniquito.sueldoPromedio > sueldoMinimoZona) {
            topeSueldo = this.cedulaFiniquito.sueldoPromedio;
        }
        else {
            topeSueldo = sueldoMinimoZona;
        }

        if (idClasificacion == 3) {
            if (this.cedulaFiniquito.idRescicionLaboral == 2 || this.cedulaFiniquito.idRescicionLaboral == 3 || this.getDaysBetweenTwoDates(this.cedulaFiniquito.ultimoDiaLaborado, this.cedulaFiniquito.fechaAntiguedad) >= 5465) {
                concepto.cantidad = (this.getDaysBetweenTwoDates(this.cedulaFiniquito.ultimoDiaLaborado, this.cedulaFiniquito.fechaAntiguedad) + 1) / 365;
                concepto.cantidad = concepto.cantidad > Math.round(concepto.cantidad) ? concepto.cantidad : Math.round(concepto.cantidad);
            }
            else {
                concepto.cantidad = 0;
            }
        }
        else if (idClasificacion == 8) {
            var cantidadCalculo = ((this.getDaysBetweenTwoDates(this.cedulaFiniquito.ultimoDiaLaborado, this.cedulaFiniquito.fechaAntiguedad) + 1) / 365);
            concepto.cantidad = (cantidadCalculo > Math.round(cantidadCalculo) ? cantidadCalculo : Math.round(cantidadCalculo)) * (this.cedulaFiniquito.porcentajePrimaAntiguedad / 100);
        }

        concepto.importeTotal = concepto.cantidad * topeSueldo * 12;

        // Calculos que depende del concepto
        this.cedulaIsrPartesProporcionales();
        this.cedulaGravadoVsExento(idClasificacion);
        this.cedulaConceptoTotalPagosSeparacion();
        this.cedulaIsrInjustificado();
        this.cedulaConceptoTotalPagosSeparacionInjust();
        this.cedulaIsrJustificado();
    }


    /**
    * Calcula el concepto de Indemnizacion Ley(20d x Año)
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (3) negosiacion o (8) negosiación 
    */
    cedulaConceptoIndeminzacionLeyDiasAnio(idClasificacion: number) {
        var concepto = this.getConcepto(idClasificacion, "Indemnización Ley (20d x Año)");
        if (idClasificacion == 3) {
            if (this.cedulaFiniquito.idRescicionLaboral == 3) {
                concepto.cantidad = ((this.getDaysBetweenTwoDates(this.cedulaFiniquito.ultimoDiaLaborado, this.cedulaFiniquito.fechaAntiguedad) + 1) / 365) * 20 * (this.cedulaFiniquito.porcentajeVeinteDiasAnio / 100);
            }
            else {
                concepto.cantidad = 0;
            }
            concepto.importeTotal = concepto.cantidad * this.cedulaFiniquito.sueldoPromedio;
        }
        else {
            concepto.cantidad = ((this.getDaysBetweenTwoDates(this.cedulaFiniquito.ultimoDiaLaborado, this.cedulaFiniquito.fechaAntiguedad) + 1) / 365) * 20 * (this.cedulaFiniquito.porcentajeVeinteDiasAnio / 100);
            concepto.importeTotal = concepto.cantidad * this.cedulaFiniquito.sueldoPromedio;
        }
        // Calculos que depende del concepto
        this.cedulaGravadoVsExento(idClasificacion);
        this.cedulaIsrPartesProporcionales();
        this.cedulaConceptoTotalPagosSeparacion();
        this.cedulaIsrInjustificado();
        this.cedulaConceptoTotalPagosSeparacionInjust();
        this.cedulaIsrJustificado();
    }

    /**
    * Calcula el concepto de Indemnizacion Ley(Días)
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (3) negosiacion o (8) negosiación 
    */
    cedulaConceptoIndeminzacionLeyDias(idClasificacion: number) {
        var concepto = this.getConcepto(idClasificacion, "Indemnización Ley (Días)");
        concepto.cantidad = this.cedulaFiniquito.diasNegociadoIndemnizacion;
        concepto.importeTotal = concepto.cantidad * this.cedulaFiniquito.sueldoPromedio;

        // Calculos que depende del concepto
        this.cedulaGravadoVsExento(idClasificacion);
        this.cedulaIsrPartesProporcionales();
        this.cedulaConceptoTotalPagosSeparacion();
        this.cedulaIsrInjustificado();
        this.cedulaConceptoTotalPagosSeparacionInjust();
        this.cedulaIsrJustificado();
    }

    /**
    * Calcula el concepto de Indemnizacion Adicional Acordada
    *
    * @param {number} idClasificacion - La clasificación determina si son los conceptos de (3) negosiacion o (8) negosiación 
    */
    cedulaConceptoIndeminzacionAdicionalAcordada(idClasificacion: number) {
        var concepto = this.getConcepto(idClasificacion, "Indemnización Adicional Acordada");
        var conceptoIndemnizacionLeyDias = this.getConcepto(idClasificacion, "Indemnización Ley (Días)");

        concepto.importeTotal = this.cedulaFiniquito.indemnnizacionAdicional;
        concepto.cantidad = concepto.importeTotal / this.cedulaFiniquito.sueldoDiario;

        // Calculos que depende del concepto
        this.cedulaGravadoVsExento(idClasificacion);
        this.cedulaConceptoTotalPagosSeparacion();
        this.cedulaIsrPartesProporcionales();
        this.cedulaIsrInjustificado();
        this.cedulaConceptoTotalPagosSeparacionInjust();
        this.cedulaIsrJustificado();
    }

    cedulaGravadoVsExento(idClasificacion: number) {

        var antiguedad = (this.getDaysBetweenTwoDates(this.cedulaFiniquito.ultimoDiaLaborado, this.cedulaFiniquito.fechaAntiguedad) + 1) / 365.0;
        //antiguedad = parseInt(antiguedad.toFixed(0));

        var totalExento = this.cedulaFiniquito.uma * 90 * Math.round(antiguedad);

        var conceptoPrimaAntiguedad = this.getConcepto(idClasificacion, "Prima Antigüedad");
        var conceptoIndemnizacion = this.getConcepto(idClasificacion, "Indemnización Ley (20d x Año)");
        var conceptoIndemnizacionDias = this.getConcepto(idClasificacion, "Indemnización Ley (Días)");
        var conceptoIndemnizacionAdicional = this.getConcepto(idClasificacion, "Indemnización Adicional Acordada");

        // Prima de antigüedad
        if (conceptoPrimaAntiguedad.importeTotal > totalExento) {
            conceptoPrimaAntiguedad.importeGravado = conceptoPrimaAntiguedad.importeTotal - totalExento;
            conceptoPrimaAntiguedad.importeExento = totalExento;
            totalExento = 0;
        }
        else if (conceptoPrimaAntiguedad.importeTotal <= totalExento) {
            conceptoPrimaAntiguedad.importeGravado = 0;
            conceptoPrimaAntiguedad.importeExento = conceptoPrimaAntiguedad.importeTotal;
            totalExento = totalExento - conceptoPrimaAntiguedad.importeTotal;
        }

        // Indemnización Ley (20d x Año)
        if (conceptoIndemnizacion.importeTotal > totalExento) {
            conceptoIndemnizacion.importeGravado = conceptoIndemnizacion.importeTotal - totalExento;
            conceptoIndemnizacion.importeExento = totalExento;
            totalExento = 0;
        }
        else if (conceptoIndemnizacion.importeTotal <= totalExento) {
            conceptoIndemnizacion.importeGravado = 0;
            conceptoIndemnizacion.importeExento = conceptoIndemnizacion.importeTotal;
            totalExento = totalExento - conceptoIndemnizacion.importeTotal;
        }

        // Indemnización Ley (Días)
        if (this.cedulaFiniquito.idRescicionLaboral != 4) {
            if (conceptoIndemnizacionDias.importeTotal > totalExento) {
                conceptoIndemnizacionDias.importeGravado = conceptoIndemnizacionDias.importeTotal - totalExento;
                conceptoIndemnizacionDias.importeExento = totalExento;
                totalExento = 0;
            }
            else if (conceptoIndemnizacionDias.importeTotal <= totalExento) {
                conceptoIndemnizacionDias.importeGravado = 0;
                conceptoIndemnizacionDias.importeExento = conceptoIndemnizacionDias.importeTotal;
                totalExento = totalExento - conceptoIndemnizacionDias.importeTotal;
            }
        }

        // Indemnización Adicional Acordada
        if (conceptoIndemnizacionAdicional.importeTotal > totalExento) {
            conceptoIndemnizacionAdicional.importeGravado = conceptoIndemnizacionAdicional.importeTotal - totalExento;
            conceptoIndemnizacionAdicional.importeExento = totalExento;
            totalExento = 0;
        }
        else if (conceptoIndemnizacionAdicional.importeTotal <= totalExento) {
            conceptoIndemnizacionAdicional.importeGravado = 0;
            conceptoIndemnizacionAdicional.importeExento = conceptoIndemnizacionAdicional.importeTotal;
            totalExento = totalExento - conceptoIndemnizacionAdicional.importeTotal;
        }

    }

    /**
     * Actualiza el ISR
     * @param idClasificacion
     */
    cedulaDeduccionesIsr(idClasificacion) {
        var conceptoIsrPartesProporcionales = this.getConcepto(idClasificacion, "ISR a Retener Partes Proporcionales");
        var conceptoIsrPagosSeparacion      = this.getConcepto(idClasificacion, "ISR a Retener Pagos Separación");

        conceptoIsrPartesProporcionales.importeTotal = this.getConcepto(14, "Impuesto a Cargo").importeTotal;

        if (idClasificacion == 5) {
            conceptoIsrPagosSeparacion.importeTotal = this.getConcepto(3, "Total Pagos Separación").importeGravado * (this.getConcepto(11, 'Tasa Efectiva Impuesto Separación').importeTotal / 100);
        }
        else {
            conceptoIsrPagosSeparacion.importeTotal = this.getConcepto(8, "Total Pagos Separación Injust").importeGravado * (this.getConcepto(11, 'Tasa Efectiva Impuesto Separación').importeTotal / 100);
        }
    }

    calcularSeguros(idClasificacion: number) {
        var sdi: number = this.cedulaFiniquito.sueldoDiarioIntegrado;
        var smdf: number = this.cedulaFiniquito.salarioMinimoZona;
        var total: number = 0;

        for (let seguro of this.cedulaFiniquito.seguros) {
            seguro.dias = this.cedulaFiniquito.sueldoDias;
            if (seguro.seguro == "Prov. TRAB ENF Y MAT E3") {
                seguro.calculo = ((sdi - (smdf * 3)) * seguro.dias) * (seguro.porcentaje / 100);
            }
            else {
                seguro.calculo = ((sdi * seguro.dias) * (seguro.porcentaje / 100));
            }
            total += seguro.calculo;
        }
        var conceptoImssRetener: CedulaConcepto = this.getConcepto(idClasificacion, "IMSS a Retener");
        conceptoImssRetener.importeTotal = total;

        var conceptoImporteNeto = this.getConcepto(idClasificacion, "Importe Neto Finiquito");
        if (idClasificacion == 4) {
            conceptoImporteNeto.importeTotal = (this.getImporteTotal(1) + this.getImporteTotal(2) + this.getConcepto(3, "Total Pagos Separación").importeTotal) - this.getImporteTotal(5) - this.getConcepto(4, 'IMSS a Retener').importeTotal - this.getConcepto(4, 'Importe Adeudos').importeTotal;
        }
        else if (idClasificacion == 9) {
            conceptoImporteNeto.importeTotal = (this.getImporteTotal(6) + this.getImporteTotal(7) + this.getConcepto(8, "Total Pagos Separación Injust").importeTotal) - this.getImporteTotal(10) - this.getConcepto(9, 'IMSS a Retener').importeTotal - this.getConcepto(9, 'Importe Adeudos').importeTotal;
        }
    }

    /**
    * ISR Mensual Ordinario - Sueldo Mensual Ordinario
    */
    calcularImporteAdeudo(idClasificacion: number) {
        var concepto =  this.getConcepto(idClasificacion, "Importe Adeudos");
        concepto.importeTotal  = this.cedulaFiniquito.adeudo + this.cedulaFiniquito.infonavit + this.cedulaFiniquito.fonacot;
    }

    /**
     * Calcula el concepto de Total Pagos Separación
     */
    cedulaConceptoTotalPagosSeparacion() {
        var concepto = this.getConcepto(3, "Total Pagos Separación");
        concepto.importeTotal = 0;

        var conceptoPrimaAntiguedad = this.getConcepto(3, "Prima Antigüedad");
        var conceptoIndemnizacion = this.getConcepto(3, "Indemnización Ley (20d x Año)");
        var conceptoIddemnizacionLey = this.getConcepto(3, "Indemnización Ley (Días)");
        var conceptoIddemnizacionAdicional = this.getConcepto(3, "Indemnización Adicional Acordada");

        concepto.importeTotal = conceptoPrimaAntiguedad.importeTotal + conceptoIndemnizacion.importeTotal + conceptoIddemnizacionLey.importeTotal + conceptoIddemnizacionAdicional.importeTotal;
        concepto.importeGravado = conceptoPrimaAntiguedad.importeGravado + conceptoIndemnizacion.importeGravado + conceptoIddemnizacionLey.importeGravado + conceptoIddemnizacionAdicional.importeGravado;
        concepto.importeExento = conceptoPrimaAntiguedad.importeExento + conceptoIndemnizacion.importeExento + conceptoIddemnizacionLey.importeExento + conceptoIddemnizacionAdicional.importeExento;
    }

    /**
     * Calcula el concepto de Total Pagos Separación (Injust, Negocia)
     */
    cedulaConceptoTotalPagosSeparacionInjust() {
        var conceptoInjust  = this.getConcepto(8, "Total Pagos Separación Injust");
        var conceptoNegocia = this.getConcepto(8, "Total Pagos Separación Negocia");

        conceptoInjust.importeTotal = 0;
        conceptoNegocia.importeTotal = 0;

        var conceptoPrimaAntiguedad         = this.getConcepto(8, "Prima Antigüedad");
        var conceptoIndemnizacion           = this.getConcepto(8, "Indemnización Ley (20d x Año)");
        var conceptoIddemnizacionLey        = this.getConcepto(8, "Indemnización Ley (Días)");
        var conceptoIddemnizacionAdicional  = this.getConcepto(8, "Indemnización Adicional Acordada");

        conceptoInjust.importeTotal = conceptoPrimaAntiguedad.importeTotal + conceptoIndemnizacion.importeTotal + conceptoIddemnizacionLey.importeTotal + conceptoIddemnizacionAdicional.importeTotal;
        conceptoNegocia.importeTotal = conceptoPrimaAntiguedad.importeTotal + conceptoIddemnizacionAdicional.importeTotal;

        conceptoInjust.importeGravado = conceptoPrimaAntiguedad.importeGravado + conceptoIndemnizacion.importeGravado + conceptoIddemnizacionLey.importeGravado + conceptoIddemnizacionAdicional.importeGravado;
        conceptoNegocia.importeGravado = conceptoPrimaAntiguedad.importeGravado + conceptoIddemnizacionAdicional.importeGravado;

        conceptoInjust.importeExento = conceptoPrimaAntiguedad.importeExento + conceptoIndemnizacion.importeExento + conceptoIddemnizacionLey.importeExento + conceptoIddemnizacionAdicional.importeExento;
        conceptoNegocia.importeExento = conceptoPrimaAntiguedad.importeExento + conceptoIddemnizacionAdicional.importeExento;
    }

    /**
    * ISR Mensual Ordinario - Sueldo Mensual Ordinario
    */
    cedulaIsrSueldoMensualOrdinario() {
        var conceptoSueldoMensualOrdinario  = this.getConcepto(11, "Sueldo Mensual Ordinario");
        var conceptoImporteExento           = this.getConcepto(11, "ImporteExento");
        var conceptoBaseImpuesto            = this.getConcepto(11, "Base Impuesto");
        var conceptoLimiteInferior          = this.getConcepto(11, "Limite Inferior");
        var conceptoExcente                 = this.getConcepto(11, "Excente L.I.");
        var conceptoPorcentajeExc           = this.getConcepto(11, "% s/ Exc L.I.");
        var conceptoImptoSinExc             = this.getConcepto(11, "Impto s/ Exc L.I");
        var conceptoCuotaFija               = this.getConcepto(11, "Cuota Fija");
        var conceptoImpuestoBruto           = this.getConcepto(11, "Impuesto Bruto");
        var conceptoImpuestoNeto            = this.getConcepto(11, "Impuesto Neto");
        var conceptoTasaEfectiva            = this.getConcepto(11, "Tasa Efectiva Impuesto Separación");

        conceptoSueldoMensualOrdinario.importeTotal = this.cedulaFiniquito.sueldoDiario * 30;
        conceptoBaseImpuesto.importeTotal = conceptoSueldoMensualOrdinario.importeTotal - conceptoImporteExento.importeTotal;

        var self = this;

        if (this.cedulaFiniquito.empleado.idCompania != undefined) {

            let infoTarifa: TarifaPagoProvicional = this.getTarifaPagoProvicional(conceptoBaseImpuesto.importeTotal);

            conceptoLimiteInferior.importeTotal = infoTarifa != null ? infoTarifa.limiteInferior : 0;
            conceptoExcente.importeTotal        = conceptoBaseImpuesto.importeTotal - conceptoLimiteInferior.importeTotal;
            conceptoPorcentajeExc.importeTotal  = infoTarifa != null ? infoTarifa.porcentaje : 0;
            conceptoImptoSinExc.importeTotal    = conceptoExcente.importeTotal * (conceptoPorcentajeExc.importeTotal / 100);
            conceptoCuotaFija.importeTotal      = infoTarifa != null ? infoTarifa.cuotaFija : 0;
            conceptoImpuestoBruto.importeTotal  = conceptoImptoSinExc.importeTotal + conceptoCuotaFija.importeTotal
            conceptoImpuestoNeto.importeTotal   = conceptoImpuestoBruto.importeTotal;
            conceptoTasaEfectiva.importeTotal   = (conceptoImpuestoNeto.importeTotal / conceptoBaseImpuesto.importeTotal) * 100;

        }

        // Desencadenador
        this.cedulaIsrPartesProporcionales();
        this.cedulaDeduccionesIsr(5);
        this.cedulaDeduccionesIsr(10);
        this.calcularSeguros(4);
        this.calcularSeguros(9);

        if (this.conceptosIsr != undefined )
            this.conceptosIsr.refresh();
    }


    cedulaIsrInjustificado() {
        var conceptoBaseImpuesto            = this.getConcepto(12, "Base Impuesto");
        var conceptoLimiteInferior          = this.getConcepto(12, "Limite Inferior");
        var conceptoExcente                 = this.getConcepto(12, "Excente L.I.");
        var conceptoPorcentajeExc           = this.getConcepto(12, "% s/ Exc L.I.");
        var conceptoImptoSinExc             = this.getConcepto(12, "Impto s/ Exc L.I");
        var conceptoCuotaFija               = this.getConcepto(12, "Cuota Fija");
        var conceptoImpuestoBruto           = this.getConcepto(12, "Impuesto Bruto");
        var conceptoSubsidio                = this.getConcepto(12, "Subsidio");
        var conceptoImpuestoNeto            = this.getConcepto(12, "Impuesto Neto");

        var conceptoTotalPagoSeparacion = this.getConcepto(3, "Total Pagos Separación");
        conceptoBaseImpuesto.importeTotal = conceptoTotalPagoSeparacion.importeGravado + this.getImporteGravado(1);

        if (this.cedulaFiniquito.empleado.idCompania != undefined) {

            let infoTarifa: TarifaPagoProvicional = this.getTarifaPagoProvicional(conceptoBaseImpuesto.importeTotal);
            conceptoLimiteInferior.importeTotal = infoTarifa != null ? infoTarifa.limiteInferior : 0;
            conceptoExcente.importeTotal        = conceptoBaseImpuesto.importeTotal - conceptoLimiteInferior.importeTotal;
            conceptoPorcentajeExc.importeTotal  = infoTarifa != null ? infoTarifa.porcentaje : 0;
            conceptoImptoSinExc.importeTotal    = conceptoExcente.importeTotal * (conceptoPorcentajeExc.importeTotal / 100);
            conceptoCuotaFija.importeTotal      = infoTarifa != null ? infoTarifa.cuotaFija : 0;
            conceptoImpuestoBruto.importeTotal  = conceptoImptoSinExc.importeTotal + conceptoCuotaFija.importeTotal
            conceptoImpuestoNeto.importeTotal   = conceptoImpuestoBruto.importeTotal;

            // Se solicita descartar el subsidio.
            //let infoSubsidio: SubsidioEmpleo = this.getSubsidioEmpleo(conceptoBaseImpuesto.importeTotal);
            //conceptoSubsidio.importeTotal = infoSubsidio != null ? infoSubsidio.subsidio : 0;
            conceptoSubsidio.importeTotal = 0;

            if (conceptoBaseImpuesto.importeTotal == 0) {
                conceptoImpuestoNeto.importeTotal = 0;
            }
            else {
                conceptoImpuestoNeto.importeTotal = conceptoImpuestoBruto.importeTotal;
            }
        }

        this.cedulaDeduccionesIsr(5);
        this.cedulaDeduccionesIsr(10);
        this.calcularSeguros(4);
        this.calcularSeguros(9);

        if (this.conceptosIsr != undefined)
            this.conceptosIsr.refresh();
    }

    cedulaIsrJustificado() {
        var conceptoBaseImpuesto        = this.getConcepto(13, "Base Impuesto");
        var conceptoLimiteInferior      = this.getConcepto(13, "Limite Inferior");
        var conceptoExcente             = this.getConcepto(13, "Excente L.I.");
        var conceptoPorcentajeExc       = this.getConcepto(13, "% s/ Exc L.I.");
        var conceptoImptoSinExc         = this.getConcepto(13, "Impto s/ Exc L.I");
        var conceptoCuotaFija           = this.getConcepto(13, "Cuota Fija");
        var conceptoImpuestoBruto       = this.getConcepto(13, "Impuesto Bruto");
        var conceptoSubsidio            = this.getConcepto(13, "Subsidio");
        var conceptoImpuestoNeto        = this.getConcepto(13, "Impuesto Neto");

        var totalPercepcionAdeudada: number = this.getImporteGravado(6);
        var totalPagosSeparacionNegocia: number = this.getConcepto(8, 'Total Pagos Separación Negocia').importeGravado;

        conceptoBaseImpuesto.importeTotal = totalPercepcionAdeudada + totalPagosSeparacionNegocia;

        if (this.cedulaFiniquito.empleado.idCompania != undefined) {

            let infoTarifa: TarifaPagoProvicional = this.getTarifaPagoProvicional(conceptoBaseImpuesto.importeTotal);

            conceptoLimiteInferior.importeTotal = infoTarifa != null ? infoTarifa.limiteInferior : 0;
            conceptoExcente.importeTotal        = conceptoBaseImpuesto.importeTotal - conceptoLimiteInferior.importeTotal;
            conceptoPorcentajeExc.importeTotal  = infoTarifa != null ? infoTarifa.porcentaje : 0;
            conceptoImptoSinExc.importeTotal    = conceptoExcente.importeTotal * (conceptoPorcentajeExc.importeTotal / 100);
            conceptoCuotaFija.importeTotal      = infoTarifa != null ? infoTarifa.cuotaFija : 0;
            conceptoImpuestoBruto.importeTotal  = conceptoImptoSinExc.importeTotal + conceptoCuotaFija.importeTotal
            conceptoImpuestoNeto.importeTotal   = conceptoImpuestoBruto.importeTotal;

            // Solicitan descartar subsidio
            //let infoSubsidio: SubsidioEmpleo = this.getSubsidioEmpleo(conceptoBaseImpuesto.importeTotal);
            //conceptoSubsidio.importeTotal = infoSubsidio != null ? infoSubsidio.subsidio : 0;
            conceptoSubsidio.importeTotal = 0;

            if (conceptoBaseImpuesto.importeTotal == 0) {
                conceptoImpuestoNeto.importeTotal = 0;
            }
            else {
                conceptoImpuestoNeto.importeTotal = conceptoImpuestoBruto.importeTotal;
            }

            if (this.conceptosIsr != undefined)
                this.conceptosIsr.refresh();
        }
    }

    // Calculo ISR
    cedulaIsrPartesProporcionales() {
        var conceptoAdeudoSueldo            = this.getConcepto(14, "Adeudo de sueldo");
        var conceptoComisiones              = this.getConcepto(14, "Comisiones");
        var conceptoIncentivos              = this.getConcepto(14, "Incentivos");
        var conceptoHorasExtra              = this.getConcepto(14, "Horas Extra");
        var conceptoOtros                   = this.getConcepto(14, "Otros");
        var conceptoTotalPartesProporcional = this.getConcepto(14, "Total Partes Proporcionales");
        var conceptoBaseGravable            = this.getConcepto(14, "Base Gravable");
        var conceptoLimiteInferior          = this.getConcepto(14, "Limite Inferior");
        var conceptoDiferencia              = this.getConcepto(14, "Diferencia");
        var conceptoTarifaAplicable         = this.getConcepto(14, "% Tarifa Aplicable");
        var conceptoCuotaFija               = this.getConcepto(14, "Cuota Fija");
        var conceptoImpuestoMarginal        = this.getConcepto(14, "Impuesto Marginal");
        var conceptoImpuesto                = this.getConcepto(14, "Impuesto");
        var conceptoSubsidioEmpleo          = this.getConcepto(14, "Subsidio Empleo");
        var conceptoImpuestoCargo           = this.getConcepto(14, "Impuesto a Cargo");

        conceptoAdeudoSueldo.importeTotal   = this.getConcepto(1, "Adeudo de Sueldo").importeGravado;
        conceptoComisiones.importeTotal     = this.getConcepto(1, "Comisiones").importeGravado;
        conceptoIncentivos.importeTotal     = this.getConcepto(1, "Incentivos").importeGravado;
        conceptoHorasExtra.importeTotal     = this.getConcepto(1, "Horas Extra (Importe)").importeGravado;
        conceptoOtros.importeTotal          = this.getConcepto(1, "Otros").importeGravado;

        conceptoTotalPartesProporcional.importeTotal = 0;

        conceptoTotalPartesProporcional.importeTotal += this.getConcepto(2, "Aguinaldo").importeGravado;
        conceptoTotalPartesProporcional.importeTotal += this.getConcepto(2, "Vacaciones").importeGravado;
        conceptoTotalPartesProporcional.importeTotal += this.getConcepto(2, "Prima Dominical").importeGravado;
        conceptoTotalPartesProporcional.importeTotal += this.getConcepto(2, "Prima Vacacional").importeGravado;

        // Acumulando las percepciones
        var conceptoAcumuladorPercepcion = new CedulaConcepto();
        conceptoAcumuladorPercepcion.idConcepto = 159;
        conceptoAcumuladorPercepcion.acumula = "1,1,1,1";
        this.getImporteAcumulado(conceptoAcumuladorPercepcion);

        conceptoBaseGravable.importeTotal = conceptoAdeudoSueldo.importeTotal +
                                            conceptoComisiones.importeTotal +
                                            conceptoIncentivos.importeTotal +
                                            conceptoHorasExtra.importeTotal +
                                            conceptoOtros.importeTotal + 
                                            conceptoTotalPartesProporcional.importeTotal +
                                            conceptoAcumuladorPercepcion.importeTotal;

        // ((Total Gravable / 365) * 30.4) * (Sueld Mensual Ordinario * 30)
        //var importeMensual = ((conceptoBaseGravable.importeTotal / 365) * 30.4);
        //var percepcionMensulizada = (importeMensual + this.getConcepto(11, "Sueldo Mensual Ordinario").importeTotal);
        // Consultamos la tarifa 
        //var tarifa = this.getTarifaPagoProvicional(percepcionMensulizada);

        //var factor = ((((percepcionMensulizada - tarifa.limiteInferior) * (tarifa.porcentaje / 100)) + tarifa.cuotaFija) - this.getConcepto(11, "Impuesto Neto").importeTotal) / importeMensual;
        //conceptoImpuesto.importeTotal = factor * conceptoBaseGravable.importeTotal;

        var tarifa = this.getTarifaPagoProvicional(conceptoBaseGravable.importeTotal);
        conceptoLimiteInferior.importeTotal     = tarifa.limiteInferior;
        conceptoDiferencia.importeTotal         = conceptoBaseGravable.importeTotal - conceptoLimiteInferior.importeTotal;
        conceptoTarifaAplicable.importeTotal    = tarifa.porcentaje;
        conceptoCuotaFija.importeTotal          = tarifa.cuotaFija;
        conceptoImpuestoMarginal.importeTotal   = conceptoDiferencia.importeTotal * (tarifa.porcentaje / 100.0);
        conceptoImpuesto.importeTotal           = conceptoCuotaFija.importeTotal + conceptoImpuestoMarginal.importeTotal;


        if (this.cedulaFiniquito.sueldoDias > 0) {
            //Solicitud de modificación cedula finiquitos URGENTE. Solicita que en la cedula de finiquitos de Onili, ya no se utilice el concepto de Subsidio para el Empleo
            //conceptoSubsidioEmpleo.importeTotal = this.getSubsidioEmpleoByDia(this.cedulaFiniquito.sueldoDias, conceptoBaseGravable.importeTotal).subsidio;
            conceptoSubsidioEmpleo.importeTotal = 0;
        }
        else {
            conceptoSubsidioEmpleo.importeTotal = 0;
        }

        conceptoImpuestoCargo.importeTotal = conceptoImpuesto.importeTotal - conceptoSubsidioEmpleo.importeTotal;

        if (this.conceptosIsr != undefined)
            this.conceptosIsr.refresh();
    }

    cedulaIsrPercepcionesAdeudadas() {
        var conceptoBaseImpuesto    = this.getConcepto(15, "Base Impuesto");
        var conceptoLimiteInferior  = this.getConcepto(15, "Limite Inferior");
        var conceptoExcente         = this.getConcepto(15, "Excente L.I.");
        var conceptoPorcentajeExc   = this.getConcepto(15, "% s/ Exc L.I.");
        var conceptoImptoSinExc     = this.getConcepto(15, "Impto s/ Exc L.I");
        var conceptoCuotaFija       = this.getConcepto(15, "Cuota Fija");
        var conceptoImpuestoBruto   = this.getConcepto(15, "Impuesto Bruto");
        var conceptoSubsidio        = this.getConcepto(15, "Subsidio");
        var conceptoImpuestoNeto    = this.getConcepto(15, "Impuesto Neto Perc Adeu");

        conceptoBaseImpuesto.importeTotal = this.getImporteGravado(1);

        var self = this;

        if (this.cedulaFiniquito.empleado.idCompania != undefined) {

            let infoTarifa: TarifaPagoProvicional = this.getTarifaPagoProvicional(conceptoBaseImpuesto.importeTotal);

            conceptoLimiteInferior.importeTotal = infoTarifa != null ? infoTarifa.limiteInferior : 0;
            conceptoExcente.importeTotal        = conceptoBaseImpuesto.importeTotal - conceptoLimiteInferior.importeTotal;
            conceptoPorcentajeExc.importeTotal  = infoTarifa != null ? infoTarifa.porcentaje : 0;
            conceptoImptoSinExc.importeTotal    = conceptoExcente.importeTotal * (conceptoPorcentajeExc.importeTotal / 100);
            conceptoCuotaFija.importeTotal      = infoTarifa != null ? infoTarifa.cuotaFija : 0;
            conceptoImpuestoBruto.importeTotal  = conceptoImptoSinExc.importeTotal + conceptoCuotaFija.importeTotal

            // Solicitan descartar el subsidio del empleo
            //let infoSubsidio: SubsidioEmpleo    = this.getSubsidioEmpleo(conceptoBaseImpuesto.importeTotal);
            //conceptoSubsidio.importeTotal       = infoSubsidio != null ? infoSubsidio.subsidio : 0;
            conceptoSubsidio.importeTotal = 0;

            if (conceptoBaseImpuesto.importeTotal == 0) {
                conceptoImpuestoNeto.importeTotal = 0;
            }
            else {
                conceptoImpuestoNeto.importeTotal = conceptoImpuestoBruto.importeTotal - conceptoSubsidio.importeTotal;
            }
        }

        this.cedulaDeduccionesIsr(5);
        this.cedulaDeduccionesIsr(10);
        this.calcularSeguros(4);
        this.calcularSeguros(9);
        if (this.conceptosIsr != undefined)
            this.conceptosIsr.refresh();
    }

    cedulaPrevioFiniquito() {
        this.getConcepto(16, "Adeudo de Sueldo").importeTotal = this.getConcepto(1, "Adeudo de Sueldo").importeTotal;
            
        this.getConcepto(16, "Comisiones"                       ).importeTotal = this.getConcepto(1, "Comisiones"                       ).importeTotal;
        this.getConcepto(16, "Incentivos"                       ).importeTotal = this.getConcepto(1, "Incentivos"                       ).importeTotal;
        this.getConcepto(16, "Horas Extra (Importe)"            ).importeTotal = this.getConcepto(1, "Horas Extra (Importe)"            ).importeTotal;
        this.getConcepto(16, "Reparto Utilidades"               ).importeTotal = this.getConcepto(1, "Reparto Utilidades"               ).importeTotal;
        this.getConcepto(16, "Otros"                            ).importeTotal = this.getConcepto(1, "Otros"                            ).importeTotal;
        
        this.getConcepto(17, "Aguinaldo"                        ).importeTotal = this.getConcepto(2, "Aguinaldo"                        ).importeTotal;
        this.getConcepto(17, "Vacaciones"                       ).importeTotal = this.getConcepto(2, "Vacaciones"                       ).importeTotal;
        this.getConcepto(17, "Prima Dominical"                  ).importeTotal = this.getConcepto(2, "Prima Dominical"                  ).importeTotal;
        this.getConcepto(17, "Prima Dominical"                  ).importeGravado = this.getConcepto(2, "Prima Dominical"                ).importeGravado;
        this.getConcepto(17, "Prima Dominical"                  ).importeExento = this.getConcepto(2, "Prima Dominical"                 ).importeExento;

        this.getConcepto(17, "Prima Vacacional"                 ).importeTotal = this.getConcepto(2, "Prima Vacacional").importeTotal;
        this.getConcepto(17, "Prima Vacacional"                 ).importeGravado = this.getConcepto(2, "Prima Vacacional").importeGravado;
        this.getConcepto(17, "Prima Vacacional"                 ).importeExento = this.getConcepto(2, "Prima Vacacional").importeExento;


        if (this.cedulaFiniquito.idRescicionLaboral == 2 || this.cedulaFiniquito.idRescicionLaboral == 3 || this.cedulaFiniquito.idRescicionLaboral == 4) {
            this.getConcepto(18, "Indemnización Adicional Acordada").importeTotal = this.getConcepto(8, "Indemnización Adicional Acordada").importeTotal;
            this.getConcepto(18, "Indemnización Adicional Acordada").importeGravado = this.getConcepto(8, "Indemnización Adicional Acordada").importeGravado;
            this.getConcepto(18, "Indemnización Adicional Acordada").importeExento = this.getConcepto(8, "Indemnización Adicional Acordada").importeExento;
            this.getConcepto(18, "Prima Antigüedad").importeTotal = this.getConcepto(8, "Prima Antigüedad").importeTotal;
            this.getConcepto(18, "Prima Antigüedad").importeGravado = this.getConcepto(8, "Prima Antigüedad").importeGravado;
            this.getConcepto(18, "Prima Antigüedad").importeExento = this.getConcepto(8, "Prima Antigüedad").importeExento;
        }
        else {
            if (this.cedulaFiniquito.idRescicionLaboral == 4) {
                this.getConcepto(18, "Indemnización Adicional Acordada").importeTotal = this.getConcepto(8, "Indemnización Adicional Acordada").importeTotal;
                this.getConcepto(18, "Indemnización Adicional Acordada").importeGravado = this.getConcepto(8, "Indemnización Adicional Acordada").importeGravado;
                this.getConcepto(18, "Indemnización Adicional Acordada").importeExento = this.getConcepto(8, "Indemnización Adicional Acordada").importeExento;
            }
            else {
                this.getConcepto(18, "Indemnización Adicional Acordada").importeTotal = 0;
                this.getConcepto(18, "Indemnización Adicional Acordada").importeGravado = 0;
                this.getConcepto(18, "Indemnización Adicional Acordada").importeExento = 0;
            }
            this.getConcepto(18, "Prima Antigüedad").importeTotal = 0;
            this.getConcepto(18, "Prima Antigüedad").importeGravado = 0;
            this.getConcepto(18, "Prima Antigüedad").importeExento = 0;
        }

        if (this.cedulaFiniquito.idRescicionLaboral == 3) {

            this.getConcepto(18, "Indemnización Ley (Días)").importeTotal = this.getConcepto(8, "Indemnización Ley (Días)").importeTotal;
            this.getConcepto(18, "Indemnización Ley (Días)").importeGravado = this.getConcepto(8, "Indemnización Ley (Días)").importeGravado;
            this.getConcepto(18, "Indemnización Ley (Días)").importeExento = this.getConcepto(8, "Indemnización Ley (Días)").importeExento;

            this.getConcepto(18, "Indemnización Ley (20d x Año)").importeTotal = this.getConcepto(8, "Indemnización Ley (20d x Año)").importeTotal;
            this.getConcepto(18, "Indemnización Ley (20d x Año)").importeGravado = this.getConcepto(8, "Indemnización Ley (20d x Año)").importeGravado;
            this.getConcepto(18, "Indemnización Ley (20d x Año)").importeExento = this.getConcepto(8, "Indemnización Ley (20d x Año)").importeExento;
        }
        else {
            this.getConcepto(18, "Indemnización Ley (Días)").importeTotal = 0;
            this.getConcepto(18, "Indemnización Ley (Días)").importeGravado = 0;
            this.getConcepto(18, "Indemnización Ley (Días)").importeExento = 0;

            this.getConcepto(18, "Indemnización Ley (20d x Año)").importeTotal = 0;
            this.getConcepto(18, "Indemnización Ley (20d x Año)").importeGravado = 0;
            this.getConcepto(18, "Indemnización Ley (20d x Año)").importeExento = 0;
        }

        var conceptoTotalPagosSeparacion = this.getConcepto(18, "Total Pagos Separación");
        conceptoTotalPagosSeparacion.importeTotal = 0;
        conceptoTotalPagosSeparacion.importeTotal = this.getImporteTotal(18);

        //ISR a Retener Percepción Adeudada
        //this.getConcepto(20, "ISR a Retener Percepción Adeudada").importeTotal = this.getConcepto(5, "ISR a Retener Percepción Adeudada").importeTotal;


        //ISR a Retener Partes Proporcionales
        this.getConcepto(20, "ISR a Retener Partes Proporcionales").importeTotal = this.getConcepto(5, "ISR a Retener Partes Proporcionales").importeTotal;
        //ISR a Retener Pagos Separación
        if (this.cedulaFiniquito.idRescicionLaboral == 2 || this.cedulaFiniquito.idRescicionLaboral == 3 || this.cedulaFiniquito.idRescicionLaboral == 4) {
            this.getConcepto(20, "ISR a Retener Pagos Separación").importeTotal = this.getConcepto(10, "ISR a Retener Pagos Separación").importeTotal;
        }
        else {
            if (conceptoTotalPagosSeparacion.importeTotal > 0) {
                this.getConcepto(20, "ISR a Retener Pagos Separación").importeTotal = this.getConcepto(5, "ISR a Retener Pagos Separación").importeTotal;
            }
            else {
                this.getConcepto(20, "ISR a Retener Pagos Separación").importeTotal = 0;
            }
        }

        this.getConcepto(19, "IMSS a Retener").importeTotal = this.getConcepto(4, 'IMSS a Retener').importeTotal;
        this.getConcepto(19, "Importe Adeudos").importeTotal = this.cedulaFiniquito.adeudo;
        this.getConcepto(19, "Infonavit").importeTotal = this.cedulaFiniquito.infonavit;
        this.getConcepto(19, "Fonacot").importeTotal = this.cedulaFiniquito.fonacot;

        var conceptoAcumulador = new CedulaConcepto();
        conceptoAcumulador.idConcepto = 149;
        conceptoAcumulador.acumula = "1,1,1,1";
        this.getImporteAcumulado(conceptoAcumulador);

        var conceptoAcumuladorPercepcion = new CedulaConcepto();
        conceptoAcumuladorPercepcion.idConcepto = 159;
        conceptoAcumuladorPercepcion.acumula = "1,1,1,1";
        this.getImporteAcumulado(conceptoAcumuladorPercepcion);

        // Importe Neto Finiquito
        // (1) Renuncia Voluntaria
        // (4) Retiro Negociado
        var totalPercepciones   = this.getImporteTotal(16) + this.getImporteTotal(17) + this.getConcepto(18, 'Total Pagos Separación').importeTotal + conceptoAcumuladorPercepcion.importeTotal;
        var totalDeducciones    = this.getImporteTotal(20) + this.getConcepto(19, "IMSS a Retener").importeTotal + this.cedulaFiniquito.adeudo + this.cedulaFiniquito.infonavit + this.cedulaFiniquito.fonacot + conceptoAcumulador.importeTotal;
        this.getConcepto(19, "Importe Neto Finiquito").importeTotal = parseFloat((totalPercepciones - totalDeducciones).toFixed(2));
        this.cedulaFiniquito.importeNetoFiniquito = parseFloat((totalPercepciones - totalDeducciones).toFixed(2));
    }

    cedulaInfonavitFonacot() {
        if (this.cedulaFiniquito.variables) {
            if (this.cedulaFiniquito.variables.length > 0) {
                let infonavit: number = 0.0;
                let fonacot: number = 0.0;
                this.cedulaFiniquito.variables.forEach(element => {
                    if (element.secuencia == 3) {
                        let porcentaje: number = (element.variableTrabajador / 10000.00);
                        let factor: number = porcentaje * this.cedulaFiniquito.sueldoDiarioIntegrado;
                        infonavit += factor * this.cedulaFiniquito.sueldoDias;
                    }
                    else if (element.secuencia == 11 && this.cedulaFiniquito.periodoPago > 0) {
                        let importe: number = (element.variableTrabajador / this.cedulaFiniquito.periodoPago);
                        infonavit += importe * this.cedulaFiniquito.sueldoDias;
                    }
                    else if (element.secuencia == 5 && this.cedulaFiniquito.periodoPago > 0) {
                        fonacot += (element.variableTrabajador / this.cedulaFiniquito.periodoPago) * this.cedulaFiniquito.sueldoDias;
                    }
                });
                this.cedulaFiniquito.infonavit = infonavit;
                this.cedulaFiniquito.fonacot = fonacot;
            }
        }
    }

    onTabSelectedPrevios(flex: wjcGrid.FlexGrid) {
        this.cedulaPrevioFiniquito();
        if (flex.collectionView != null) {
            flex.collectionView.refresh();
        }
        Control.invalidateAll();
    }

    onCambioComision(event) {
        var total = 0;
        this.cedulaFiniquito.comisionPeriodos.forEach(element => {
            if (element.pagado) {
                total += element.importe;
            }
        });
        this.cedulaFiniquito.comisiones = total;
    }

    /*************************************************************************/

    getConcepto(idClasificacion: number, concepto: string): CedulaConcepto {
        if (idClasificacion >= 1 && idClasificacion <= 5) {
            for (let conceptoCedula of this.cedulaFiniquito.conceptosNegociacion) {
                if (conceptoCedula.idClasificacionConcepto == idClasificacion && conceptoCedula.concepto == concepto) {
                    return conceptoCedula;
                }
            }
        }
        else if (idClasificacion >= 6 && idClasificacion <= 10) {
            for (let conceptoCedula of this.cedulaFiniquito.conceptosApoyo) {
                if (conceptoCedula.idClasificacionConcepto == idClasificacion && conceptoCedula.concepto == concepto) {
                    return conceptoCedula;
                }
            }
        }
        else if (idClasificacion >= 16 && idClasificacion <= 20) {
            for (let conceptoCedula of this.cedulaFiniquito.conceptosPrevio) {
                if (conceptoCedula.idClasificacionConcepto == idClasificacion && conceptoCedula.concepto == concepto) {
                    return conceptoCedula;
                }
            }
        }
        else {
            for (let conceptoCedula of this.cedulaFiniquito.conceptosIsr) {
                if (conceptoCedula.idClasificacionConcepto == idClasificacion && conceptoCedula.concepto == concepto) {
                    return conceptoCedula;
                }
            }
        }
        return new CedulaConcepto();
    }

    getImporteCantidad(idClasificacion: number): number {
        return this.getImporte(idClasificacion, "C");
    }

    getImporteTotal(idClasificacion: number): number {
        return this.getImporte(idClasificacion, "T");
    }

    getImporteGravado(idClasificacion: number): number {
        return this.getImporte(idClasificacion, "G");
    }

    getImporteExento(idClasificacion: number): number {
        return this.getImporte(idClasificacion, "E");
    }

    getImporte(idClasificacion: number, tipo: string): number {
        var cantidad: number = 0;
        var total: number = 0;
        var gravado: number = 0;
        var exento: number = 0;

        if (idClasificacion >= 1 && idClasificacion <= 5) {
            for (let conceptoCedula of this.cedulaFiniquito.conceptosNegociacion) {
                if (conceptoCedula.idClasificacionConcepto == idClasificacion && conceptoCedula.idParent == 0) {
                    cantidad += conceptoCedula.cantidad;
                    total += conceptoCedula.importeTotal;
                    gravado += conceptoCedula.importeGravado;
                    exento += conceptoCedula.importeExento;
                }
            }
        }
        else if (idClasificacion >= 6 && idClasificacion <= 10) {
            for (let conceptoCedula of this.cedulaFiniquito.conceptosApoyo) {
                if (conceptoCedula.idClasificacionConcepto == idClasificacion && conceptoCedula.idParent == 0) {
                    cantidad += conceptoCedula.cantidad;
                    total += conceptoCedula.importeTotal;
                    gravado += conceptoCedula.importeGravado;
                    exento += conceptoCedula.importeExento;
                }
            }
        }
        else if (idClasificacion >= 16 && idClasificacion <= 20) {
            for (let conceptoCedula of this.cedulaFiniquito.conceptosPrevio) {
                if (conceptoCedula.idClasificacionConcepto == idClasificacion && conceptoCedula.idParent == 0) {
                    cantidad += conceptoCedula.cantidad;
                    total += conceptoCedula.importeTotal;
                    gravado += conceptoCedula.importeGravado;
                    exento += conceptoCedula.importeExento;
                }
            }
        }
        else {
            for (let conceptoCedula of this.cedulaFiniquito.conceptosIsr) {
                if (conceptoCedula.idClasificacionConcepto == idClasificacion && conceptoCedula.idParent == 0) {
                    cantidad += conceptoCedula.cantidad;
                    total += conceptoCedula.importeTotal;
                    gravado += conceptoCedula.importeGravado;
                    exento += conceptoCedula.importeExento;
                }
            }
        }
        if (tipo == "C") {
            return cantidad;
        }
        else if (tipo == "T") {
            return total;
        }
        else if (tipo == "G") {
            return gravado;
        }
        else if (tipo == "E") {
            return exento;
        }
        return 0;
    }


    getImporteAcumulado(concepto: CedulaConcepto) {
        if (concepto != null) {
            var acumCantidad: number = 0;
            var acumTotal: number = 0;
            var acumGravado: number = 0;
            var acumExento: number = 0;

            this.cedulaFiniquito.conceptosPrevio.forEach(element => {
                if (element.idParent == concepto.idConcepto) {
                    acumCantidad    += element.cantidad;
                    acumTotal       += element.importeTotal;
                    acumGravado     += element.importeGravado;
                    acumExento      += element.importeExento;
                }
            });

            var reglas: string[] = concepto.acumula.split(",");

            if (reglas[0] == "1") {
                concepto.cantidad += acumCantidad;
            }

            if (reglas[1] == "1") {
                concepto.importeTotal += acumTotal;
            }

            if (reglas[2] == "1") {
                concepto.importeGravado += acumGravado;
            }

            if (reglas[3] == "1") {
                concepto.importeExento += acumExento;
            }

            return concepto;
        }
    }

    onTabSelectedFlexGrid(flex: wjcGrid.FlexGrid) {
        if (flex.collectionView != null) {
            flex.collectionView.refresh();
        }
        Control.invalidateAll();
    }


    getDaysBetweenTwoDates(dateIni: Date, dateFin: Date) {
        try {
            var diasDif = new Date(dateIni.toDateString()).getTime() - new Date(dateFin.toDateString()).getTime();
            return Math.round(diasDif / (1000 * 60 * 60 * 24));
        }
        catch (err)
        {

        }
        return 0;
    }

    onRowDblclicked(s: wjcGrid.FlexGrid, e: any) {
        if (s.selectedRows[0].dataItem.concepto === 'Importe Adeudos') {
            this.context.app.showSpinner();
            this.cedulaFiniquitoService.generarCedulaFiniquitoAdeudo(this.cedulaFiniquito.idEmpleado)
                .then(report => {
                    this.tileDowmload = "Contpaqi - Adeudos";
                    this.nameDownload = "adeudo_" + this.cedulaFiniquito.empleado.code;
                    this.reportViewer.show("data:application/pdf; base64, " + report);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    printKardexAsistencia() {
        if (this.cedulaFiniquito.idEmpleado > 0) {
            this.context.app.showSpinner();
            this.cedulaFiniquitoService.generarKardexAsistencia(this.cedulaFiniquito.empleado.code, moment(this.cedulaFiniquito.fechaAntiguedad).format(), moment(this.cedulaFiniquito.ultimoDiaLaborado).format())
                .then(report => {
                    this.tileDowmload = "Kardex de asistencia";
                    this.nameDownload = "kardex_" + this.cedulaFiniquito.empleado.code;
                    this.reportViewer.show("data:application/pdf; base64, " + report);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    getSubsidioEmpleo(importe: number) {
        for (let subsidio of this.cedulaFiniquito.subsidiosEmpleo) {
            if (subsidio.limiteInferior <= importe && (subsidio.limiteSuperior >= importe || subsidio.limiteSuperior == 0) && subsidio.limiteInferior <= importe) {
                return subsidio;
            }
        }
        return new SubsidioEmpleo();
    }

    getSubsidioEmpleoByDia(dias: number, importe: number) {
        if (dias > 0) {
            for (let subsidio of this.cedulaFiniquito.subsidiosEmpleo) {
                if ((subsidio.subsidioDiario.limiteInferior * dias) <= importe && ((subsidio.subsidioDiario.limiteSuperior * dias) >= importe || (subsidio.subsidioDiario.limiteSuperior * dias) == 0) && (subsidio.subsidioDiario.limiteInferior * dias) <= importe) {
                    return subsidio;
                }
            }
        }
        return new SubsidioEmpleo();
    }


    getTarifaPagoProvicional(importe: number) {
        for (let tarifa of this.cedulaFiniquito.tarifasPagosProvicionales) {
            if ((tarifa.limiteInferior <= importe) && (tarifa.limiteSuperior >= importe || tarifa.limiteSuperior == 0) && tarifa.limiteInferior <= importe) {
                return tarifa;
            }
        }
        return new TarifaPagoProvicional();
    }

    verificarTaskComision() {
        let self = this;
        this.context.app.showSpinner();
        this.cedulaFiniquitoService.getTaskComision(this.cedulaFiniquito.idTaskComision)
            .then(response => {
                if (response.completo) {
                    self.cedulaFiniquito.comisionPeriodos = [];
                    if (!(response.result === '')) {
                        var result = JSON.parse( response.result );
                        result.forEach(element => {
                            var comision: ComisionPeriodo = new ComisionPeriodo();
                            comision.idCedulaFiniquito  = self.cedulaFiniquito.identity;
                            comision.anio               = element.Anio;
                            comision.periodo            = element.Periodo;
                            comision.fechaInicio        = moment(element.FechaInicio).toDate();
                            comision.fechaTermino       = moment(element.FechaTermino).toDate();
                            comision.fechaPago          = moment(element.FechaPago).toDate();
                            comision.importe            = parseFloat(element.Importe);
                            self.cedulaFiniquito.comisionPeriodos.push(comision);
                        });
                    }
                    clearInterval(this.timerInterval);
                    this.context.app.hideSpinner();
                }
            })
            .catch(error => this.handleError(error));
    }


    //Administracion de contactos
    onMenuItemClick(option: MenuItem) {
        if (option.name == 'ADEUDO') {
            if (this.cedulaFiniquito.idEmpleado > 0) {
                this.dlgAdeudo.config.backdrop = false;
                this.dlgAdeudo.show();
            }
            else {
                this.showAlert("Seleccion el empleado", 'danger');
            }
        }
    }

    onConsultarAdeudo() {
        this.dlgAdeudo.hide();
        this.context.app.showSpinner();
        this.cedulaFiniquitoService.loadAdeudosEmpleado(this.cedulaFiniquito.empleado.compania.code, this.cedulaFiniquito.empleado.code, moment(this.cedulaFiniquito.fechaBaja).format())
            .then(info => {
                this.onCambioAdeudo(info);
                this.context.app.hideSpinner();
            })
            .catch(e => this.handleError(e));
    }

    calcularPercepciones(acumulado: AcumuladoBajaTit): string {
        let percepciones: number = acumulado.acumuladoDet
            .map(c => c.percepciones)
            .reduce((sum, current) => sum + current)

        return "Total percepciones $" + percepciones.toFixed(2);
    }

    calcularDeducciones(acumulado: AcumuladoBajaTit): string {
        let deducciones: number = acumulado.acumuladoDet
            .map(c => c.deducciones)
            .reduce((sum, current) => sum + current)
        return "Total deducciones $" + deducciones.toFixed(2);
    }

    calcularTotales(acumulado: AcumuladoBajaTit): string {
        
        let percepciones: number = acumulado.acumuladoDet
            .map(c=>c.percepciones)
            .reduce((sum, current) => sum + current)

        let deducciones: number = acumulado.acumuladoDet
            .map(c => c.deducciones)
            .reduce((sum, current) => sum + current)

        return "Neto a pagar $" + (percepciones - deducciones).toFixed(2);
    }

}