﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';

import { MenuItem } from 'framework3/controls';
//Features
import { CedulaFiniquitoService} from '../services/cedulafiniquito.service';
import { CedulaFiniquitoInfo, CedulaFiniquitoItem} from '../models/cedulafiniquito';

@Component({    
    selector: 'rh-cedulasfiniquitos-manager',
    templateUrl: './cedulasfiniquitos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CedulasFiniquitosManager)
        },
        {
            provide: CatalogService, useExisting: CedulaFiniquitoService
        }
	]
})
export class CedulasFiniquitosManager extends CatalogManager {
    cedulasFiniquitos: CedulaFiniquitoItem[] = [];

    //
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private cedulaFiniquitoService: CedulaFiniquitoService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.cedulaFiniquitoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.cedulasFiniquitos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.cedulaFiniquitoService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.cedulaFiniquitoService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }


    onMenuItemClick(item: MenuItem) {
        if (item.name == 'PRINT') {
            this.context.app.showSpinner();
            this.cedulaFiniquitoService.generarCedulaFiniquito(this.manager.currentItem.id)
                .then(report => {
                    this.reportViewer.show("data:application/pdf; base64, " + report);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

}
