﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import { WjInputModule } from 'wijmo/wijmo.angular2.input';

//Azteca Kernel
import { Context, BaseComponent, CatalogService, ChangeState, UsuarioItem, CatalogItem} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { MenuItem, ToolbarButton } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';

//Wijmo
import { Control, NotifyCollectionChangedEventArgs, NotifyCollectionChangedAction } from 'wijmo/wijmo';
import { FlexGrid, CellRangeEventArgs, DataMap } from 'wijmo/wijmo.grid';
import * as wjcCore from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';

//Controls
import { EmpleadoListComponent } from '../controls/empleado-list.component';

//Features
import { ChecadorService} from '../services/checador.service';
import { FirmwareService} from '../services/firmware.service';
import { ChecadorInfo, ChecadorItem, ChecadorUsuarioIten } from '../models/checador';
import { FirmwareItem } from '../models/firmware';
import { EmpleadoItem } from '../models/empleado';
import { UsuariosListComponent } from 'framework3/security';

@Component({
    selector: 'rh-checador-editor',
    templateUrl: './checadores.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ChecadoresEditor)
        },
        {
            provide: CatalogService, useExisting: ChecadorService
        }
    ]
})
export class ChecadoresEditor extends CatalogEditor {
    checador: ChecadorInfo = new ChecadorInfo();
    firmwares: FirmwareItem[] = [];
    usuarios: UsuarioItem[];
    currentItem: CatalogItem;
    //
    cvUsuarios: wjcCore.CollectionView = new wjcCore.CollectionView();
    itemsDeleted: any[] = []; 
    tipoUsuarioMap: DataMap;

    mnuEmpleadoItems: MenuItem[] = [];
    //
    @ViewChild(EmpleadoListComponent) selEmpleado: EmpleadoListComponent;
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;

    tiposConexion = [
        { 'value': 'E', 'name': 'Ethernet' },
        { 'value': 'P', 'name': 'Puerto Serial' },
        { 'value': 'U', 'name': 'USB / Archivo' }
    ];

    tipoUsuario = [
        { 'value': 1, 'name': 'Grabador' },
        { 'value': 2, 'name': 'Administrador' },
        { 'value': 3, 'name': 'Supervisor' },
        { 'value': 0, 'name': 'Normal' }
    ];

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private checadorService: ChecadorService, private firmwareSvc: FirmwareService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idBranch': [''],
            'ordenDescarga': [''],
            'sincronizarReloj': [''],
            'licencia': [''],
            'tipoConexion': ['', Validators.required],
            'direccionIP': [''],
            'puerto': [0],
            'password': [''],
            'idFirmware': [0],
            'enlaceRemoto': [false],
            'direccionEnlace': [''],
            'puertoEnlace': [0]
        });
        this.cvUsuarios = new wjcCore.CollectionView();
        this.cvUsuarios.collectionChanged.addHandler((sender: any, args: NotifyCollectionChangedEventArgs) => {
            if (args.action == NotifyCollectionChangedAction.Change && args.item != null) {
                args.item.state = ChangeState.MODIFIED;
            }
        });

        this.tipoUsuarioMap = new DataMap(this.tipoUsuario, 'value', 'name');
    }

    ngAfterViewInit() {
        super.ngAfterViewInit();
        setTimeout(_ => {
            this.createMenuDetail();
        });
    }


    createMenuDetail() {
        let mnuAsignar: MenuItem = { icon: "glyphicon glyphicon-pencil", text: "Asignar empleado", name: "ASIGNAR", smallText: "asignar empleado" };

        this.mnuEmpleadoItems.push(mnuAsignar);
        this.mnuEmpleadoItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

    cargarFirmwares() {
        this.firmwareSvc.loadCbo(this.context.app.defaultSite.identity)
            .then(firmwares => {
                this.firmwares = firmwares;
                this.firmwares.unshift({ id: 0, codigo: '', nombre: '[NINGUNO]', activo: true });
            })
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.cargarFirmwares();
    }

    onCreatingItem() {
        this.cvUsuarios.sourceCollection = this.cloneInfo<ChecadorUsuarioIten>(this.checador.usuarios);
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.checadorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.checador = info;  
                this.cvUsuarios.sourceCollection = this.cloneInfo<ChecadorUsuarioIten>(this.checador.usuarios);
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.checadorService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.checador = info;
                this.cvUsuarios.sourceCollection = this.cloneInfo<ChecadorUsuarioIten>(this.checador.usuarios);
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.checadorService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.checador = info;
                this.cvUsuarios.sourceCollection = this.cloneInfo<ChecadorUsuarioIten>(this.checador.usuarios);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        let items: any[] = [];
        this.checador.usuarios = items.concat(this.cvUsuarios.items, this.itemsDeleted);
        this.checadorService.update(this.checador)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.checador.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.checador = info;
                this.cvUsuarios.sourceCollection = this.cloneInfo<ChecadorUsuarioIten>(this.checador.usuarios);
            })
            .catch(error => this.handleError(error));        
    }

    onTabSelected(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onClickTestConnection() {
        this.context.app.showSpinner();
        this.checadorService.testConnection(this.checador)
            .then(result => {
                if (result) {
                    this.showAlert('Conexión exitosa', 'success');
                }
                else {
                    this.showAlert('Conexión exitosa', 'danger');
                }
                this.context.app.hideSpinner();
            }).
            catch(error => this.handleError(error));
    }

    onAgregarEmpleado() {
        this.selEmpleado.multiSelect = true;
        this.selEmpleado.show();
    }

    onSelectedEmleado(item: any) {
        var currentItem = this.cvUsuarios.currentItem;
        if (currentItem) {
            this.cvUsuarios.editItem(currentItem);
            currentItem.idEmpleado = item.id;
            currentItem.empleado = item.data;
            currentItem.codigo = item.data.codigo;
            currentItem.state = ChangeState.MODIFIED;
            this.cvUsuarios.commitEdit();
        }
    }

    onSelectedEmleados(items: EmpleadoItem[]) {
        var elements = this.cvUsuarios.sourceCollection;
        items.forEach(item => {
            var empleado = null;
            elements.forEach(element => {
                if (item.id == element.id) {
                    empleado = item;
                }
            });
            if (empleado == null) {
                let element = <any> this.cvUsuarios.addNew();
                element.idEmpleado = item.id;
                element.empleado = item;
                element.codigo = item.codigo;
                element.activo = true;
                element.afectaIncidencia = true;
                element.tipo = 0;
                element.state = ChangeState.MODIFIED;
                this.cvUsuarios.commitNew();
            }
        });
    }

    onRowDeleteUsuario(item: ChecadorUsuarioIten) {
        item.state = ChangeState.DELETED;
        this.itemsDeleted.push(item);
        this.cvUsuarios.remove(item);
    }

    onClickMnuEmpleado(menu: Menu) {
        let option = this.mnuEmpleadoItems[menu.selectedIndex];
        if (option.name == "ASIGNAR") {
            this.selEmpleado.multiSelect = false;
            this.selEmpleado.show();
        }
    }


    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element));
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
    }

    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.context.app.showSpinner();
        this.checadorService.addUser(this.checador.identity, this.context.session.user.identity, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                //this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshListAccess();
            })
            .catch(error => this.handleError(error));

        console.log(JSON.stringify(this.usuarios));
    }

    onRemoveUser() {
        this.context.app.showSpinner()
        this.checadorService.removeUser(this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error))
    }

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    refreshListAccess() {
        if (this.checador.identity > 0) {
            this.context.app.showSpinner();
            this.checadorService.loadAccessUsersList(this.checador.identity)
                .then(usuarios => {
                    this.usuarios = usuarios;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }
}