﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Features
import { ChecadorService} from '../services/checador.service';
import { ChecadorInfo, ChecadorItem} from '../models/checador';

@Component({    
    selector: 'rh-checadores-manager',
    templateUrl: './checadores.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ChecadoresManager)
        },
        {
            provide: CatalogService, useExisting: ChecadorService
        }
	]
})
export class ChecadoresManager extends CatalogManager {
    checadores: ChecadorItem[]=[];    
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private checadorService: ChecadorService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.checadorService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.checadores = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.checadorService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.checadorService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['rh/checador-usuarios', this.process.id, this.manager.currentItem.id]);
        }
        else if (option.name == "DOWNLOAD") {
            this.context.app.showSpinner();
            this.checadorService.downloadTransacciones(this.manager.currentItem.id)
                .then(result => {
                    this.showAlert('El dispositivo se ha descargado exitosamente.', 'success');
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

}
