﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

//Wijmo
import { Control } from 'wijmo/wijmo';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';

//Features
import { ClaseNominaService} from '../services/clase-nomina.service';
import { ClaseNominaInfo, ClaseNominaItem} from '../models/clase-nomina';

@Component({
    selector: 'rh-clase-nomina-editor',
    templateUrl: './clases-nominas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ClasesNominasEditor)
        },
        {
            provide: CatalogService, useExisting: ClaseNominaService
        }
    ]
})
export class ClasesNominasEditor extends CatalogEditor {
    claseNomina: ClaseNominaInfo = new ClaseNominaInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private claseNominaService: ClaseNominaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'claseNomina': [''],
            'idCompania': [0],
            'descripcion': [0],
            'parametro': []
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.claseNominaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.claseNomina = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.claseNominaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.claseNomina = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.claseNominaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.claseNomina = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.claseNominaService.update(this.claseNomina)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.claseNomina.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.claseNomina = info;
            })
            .catch(error => this.handleError(error));        
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

   

}