﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
import { FormGroup } from '@angular/forms';

//ngx bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Wijmo
import { Control } from 'wijmo/wijmo';

//moment
import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, EditableListComponent } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';

//Features
import { CompaniaService} from '../services/compania.service';
import { CompaniaInfo, CompaniaItem, CompaniaParametroItem} from '../models/compania';
import { CollectionView } from 'wijmo/wijmo';

@Component({
    selector: 'azteca.rh-compania-editor',
    templateUrl: './companias.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CompaniasEditor)
        },
        {
            provide: CatalogService, useExisting: CompaniaService
        }
    ]
})
export class CompaniasEditor extends CatalogEditor {

    compania: CompaniaInfo = new CompaniaInfo();
    newParameter: CompaniaParametroItem = new CompaniaParametroItem();

    @ViewChild('dlgParametro') dlgParameter: ModalDirective;
    @ViewChild('editListParametros') editListParametros: EditableListComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private companiaService: CompaniaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'empresaAdam': ['', Validators.required],
            'registroPatronal': [''],
            'parametro': [[], AztecaValidators.validParameters]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.companiaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.compania = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.companiaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.compania = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.companiaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.compania = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.companiaService.update(this.compania)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.compania.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.compania = info;
            })
            .catch(error => this.handleError(error));        
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }
}