﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//moment
import * as moment from 'moment';

//Wijmo
import { Control } from 'wijmo/wijmo';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, EditableListComponent } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ConceptoAdamService } from '../services/concepto-adam.service';
import { ConceptoAdamInfo, ConceptoAdamItem, ConceptoExpresionItem } from '../models/concepto-adam';


@Component({
    selector: 'rh-concepto-adam-editor',
    templateUrl: './conceptos-adam.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ConceptosAdamEditor)
        },
        {
            provide: CatalogService, useExisting: ConceptoAdamService
        }
    ]
})
export class ConceptosAdamEditor extends CatalogEditor {

    formExpresion: FormGroup;

    conceptoAdam: ConceptoAdamInfo = new ConceptoAdamInfo();
    expresion: ConceptoExpresionItem = new ConceptoExpresionItem();

    @ViewChild('dlgExpresion') dlgExpresion: ModalDirective;
    @ViewChild('editListExpresiones') editListExpresion: EditableListComponent;

    //
    tipoPerDedSelected: string = '';

    tiposTransaccion: Array<any> = [
        { id: 1, nombre: 'Percepción', value: 'Percepciones' },
        { id: 2, nombre: 'Deducción', value: 'Deducciones' },
        { id: 3, nombre: 'Otros Pagos', value: 'Otros Pagos' },
        { id: 4, nombre: 'Incapacidad', value: 'Incapacidades' }
    ];

    tiposGrupo: Array<any> = [
        { tipo: "I", nombre: "Importe" },
        { tipo: "G", nombre: "Gravado" },
        { tipo: "E", nombre: "Exento" },
        { tipo: "O", nombre: "Otro" }
    ];

    tipoExpresion: Array<any> = [
        { tipo: "C", nombre: "Codigo" },
        { tipo: "S", nombre: "SQL Sentencia" }
    ];

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private conceptoAdamService: ConceptoAdamService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'concepto': [''],
            'descripcion': ['', Validators.required],
            'abreviatura': [''],
            'tipoTransaccion': ['', AztecaValidators.selected],
            'tipoPerDed': [''],
            'compania': [''],
            'indemnizacion': [''],
            'deudorInterno': [''],
            'acumulable': ['']
        });

        this.formExpresion = formBuilder.group({
            'expresionCodigo': ['', Validators.required],
            'expresionNombre': ['', Validators.required],
            'expresionOrden': ['', Validators.required],
            'expresionTipo': ['', Validators.required],
            'expresionGrupo': ['', Validators.required],
            'expresionCode': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.conceptoAdamService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.conceptoAdam = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.conceptoAdamService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.conceptoAdam = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.conceptoAdamService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.conceptoAdam = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.conceptoAdamService.update(this.conceptoAdam)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.conceptoAdam.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.conceptoAdam = info;
            })
            .catch(error => this.handleError(error));        
    }

    onSelectedTipoTransaccion(args: any) {
        this.tiposTransaccion.forEach(item => {
            if (item.id == args ) {
                this.tipoPerDedSelected = item.value;
            }
        })
    }


    onTabSelected(args: any) {
        if (args.heading) {
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    ///////////////////////// Expresiones ///////////////////////////////////////
    onAddingItemExpresion() {
        this.expresion = new ConceptoExpresionItem();
        this.expresion._clientID = this.createClientID();
        this.dlgExpresion.config.backdrop = false;
        this.dlgExpresion.show();
    }

    onEditingItemExpresion(item: ConceptoExpresionItem) {
        this.expresion = this.cloneInfo<ConceptoExpresionItem>(item);
        this.dlgExpresion.config.backdrop = false;
        this.dlgExpresion.show();
    }

    onDeletingItemExpresion(item: ConceptoExpresionItem) {
        this.editListExpresion.deleteItem(item);
    }

    onListItemUpdatedExpresion(expresiones: ConceptoExpresionItem[]) {
        this.conceptoAdam.expresiones = expresiones;
    }

    onUpdateExpresion() {
        let item = this.cloneInfo<ConceptoExpresionItem>(this.expresion);
        this.editListExpresion.updateItem(item);
        this.dlgExpresion.hide();
    }

    createClientID(): number {
        return parseInt((Math.random() * 1000).toString()) * 1000 + parseInt(moment().format("HHmmss"))
    }

    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element));
    }

}