﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ConceptosFichasService} from '../services/conceptos-fichas.service';
import { ConceptosFichasInfo, ConceptosFichasItem} from '../models/concepto-ficha';
import { CuentaInfo } from '../../accounting';

@Component({
    selector: 'azteca.business.rh-conceptos-fichas-editor',
    templateUrl: './conceptos-fichas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ConceptosFichasEditor)
        },
        {
            provide: CatalogService, useExisting: ConceptosFichasService
        }
    ]
})
export class ConceptosFichasEditor extends CatalogEditor {
    conceptosFichas: ConceptosFichasInfo = new ConceptosFichasInfo(); 
    afectador: any[] = [{ 'Id': 'C', 'Nombre': 'Cargo' }, { 'Id': 'A', 'Nombre': 'Abono' }];
    mascara: string;
    mascaraCuenta: string = '';
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private conceptosFichasService: ConceptosFichasService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuenta': ['', Validators.required],
            'afectador': ['', Validators.required],
            'isEmpleado': [false],
            'negativo':[false]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        this.mascara = 'XXX-00-00000';
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.conceptosFichasService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.conceptosFichas = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.conceptosFichasService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.conceptosFichas = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.conceptosFichasService.getDetailByID(id)
            .then(info => {
                this.onIsEmpleadoChange(info.isEmpleado);
                setTimeout(() => {
                    this.context.app.hideSpinner();
                    this.conceptosFichas = info;
                    this.mascaraCuenta = this.conceptosFichas.mascaraCuenta;
                },500)
                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        if (!this.mascaraCuenta.includes('_')) {
            this.context.app.showSpinner();
            this.conceptosFichas.mascaraCuenta = this.mascaraCuenta;
            this.conceptosFichasService.update(this.conceptosFichas)
                .then(info => {
                    this.context.app.hideSpinner();
                    this.reset();
                    //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                    if (!args.closeEditor && this.conceptosFichas.identity == 0) {
                        this.router.navigate(['../', info.identity], { relativeTo: this.route });
                    }
                    else {
                        if (args.closeEditor)
                            this.router.navigate(['../'], { relativeTo: this.route });
                        else
                            this.showAlert('La información ha sido actualizada con éxito', 'success');
                    }
                    this.conceptosFichas = info;
                })
                .catch(error => this.handleError(error));
        }
        else
            this.showAlert('La mascara de la cuenta no esta completa', 'danger');
    }
    onIsEmpleadoChange(value: boolean) {
        if (value)
            this.mascara = '000-00-XXXXX';
        else
            this.mascara = 'XXX-00-00000';
    }

}