﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { DepartamentoInfo, DepartamentoItem } from '../models/departamento';
import { DepartamentoService } from '../services/departamento.service';


@Component({
    selector: 'rh-departamentos-manager',
    templateUrl: './departamentos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DepartamentosManager)
        },
        {
            provide: CatalogService, useExisting: DepartamentoService
        }
    ]
})
export class DepartamentosManager extends HierarchyManager {
    departamentos: DepartamentoInfo[] = [];
    departamentoEdicion: DepartamentoInfo = new DepartamentoInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    

    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private departamentoService: DepartamentoService, protected router: Router, protected route: ActivatedRoute) {
        super(context, router, route, changeDetector);

        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuenta': ['']
        });
    }

    ////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.departamentoService.loadHierarchy(this.context.app.defaultCompany.identity, 0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.departamentos = nodes;
            })
            .catch(error => this.handleError(error));
    }


    //////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadHierarchy();
        //Hacer llamadas de carga inicial de datos aquí
    }


    onNodesChanged(departamentos: DepartamentoInfo[]) {
        this.departamentos = departamentos;
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.departamentoEdicion = new DepartamentoInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.departamentoEdicion = new DepartamentoInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.departamentoEdicion.idBranch = this.context.app.defaultSite.identity;
        this.departamentoEdicion.idCompany = this.context.app.defaultCompany.identity;
        //
        this.currentParent = parentNode;
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {
        this.departamentoEdicion = this.departamentoService.copy(node.data);
        this.showEditor();
    }

    onSavingNode() {
        //Si es un nuevo nodo
        if (this.departamentoEdicion.clientID == 0) {
            this.addNode(this.departamentoEdicion, this.currentParent);
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.departamentoEdicion);
        }
        this.departamentoEdicion = new DepartamentoInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<DepartamentoInfo>) {
        this.context.app.showSpinner();
        this.departamentoService.saveHierarchy(changes)
            .then(departamentos => {
                this.departamentos = departamentos;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

}