﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
import { Control, CollectionView } from 'wijmo/wijmo';

//moment
import * as moment from 'moment';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, EditableListComponent } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';
import { MenuItem } from 'framework3/controls';
//Features
import { EmpleadoService } from '../services/empleado.service';
import { EmpleadoInfo, DocumentoEmpleadoItem } from '../models/empleado';
import { TipoRegimenItem } from '../../accounting/models/tipo-regimen';
import { TipoRegimenService } from '../../accounting/services/tipo-regimen.service';
import { TipoContratoItem } from '../../accounting/models/tipo-contrato';
import { TipoContratoService } from '../../accounting/services/tipo-contrato.service';

import { DocumentoTipoListComponent } from '../controls/documentos-tipo-list.component';
import { FormGroup } from '@angular/forms/src/model';
import { DocumentoTipoInfo, DocumentoTipoItem } from '../models/documento-tipo';
import { TurnoSelectorComponent}from '../components/turno-selector.component'

@Component({
    selector: 'rh-empleados-editor',
    templateUrl: './empleados.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EmpleadosEditor)
        },
        {
            provide: CatalogService, useExisting: EmpleadoService
        }
    ]
})
export class EmpleadosEditor extends CatalogEditor implements OnInit {

    empleado: EmpleadoInfo = new EmpleadoInfo();
    tiposRegimen: TipoRegimenItem[] = [];
    tiposContrato: TipoContratoItem[] = [];
    formCambioTurno: FormGroup;
    formPasswd: FormGroup;
    idTurnoCambio: number;
    cambioTurnoFechaIni: Date;
    userContrasena: string = "";

    @ViewChild('dlgCambioTurno') dlgCambioTurno: ModalDirective;
    @ViewChild('dlgPasswd') dlgPasswd: ModalDirective;
    @ViewChild('selTurno') selTurno: TurnoSelectorComponent;
    
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location,
        private empleadoService: EmpleadoService,
        private tipoRegimenSvc: TipoRegimenService,
        private tipoContratoSvc: TipoContratoService
    ) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'apPaterno': ['', Validators.required],
            'apMaterno': ['', Validators.required],
            'foto': [null],
            'sexo': ['', Validators.required],
            'cuenta': [''],
            'fechaNacimiento': [''],
            'fechaIngreso': [''],
            'correo': [''],
            'imss': [''],
            'rfc': ['', Validators.required],
            'curp': [''],
            'titulo': [''],
            'calle': [''],
            'cruce1': [''],
            'cruce2': [''],
            'numExt': [''],
            'numInt': [''],
            'colonia': [''],
            'poblacion': [''],
            'departamento': [''],
            'turno': [''],
            'plaza': [''],
            'cp': [''],
            'tipoRegimen': [0],
            'claseNomina': [0],
            'tipoContrato': [0],
            'sindicalizado':[false],
            'parametro': ['']
        });

        this.formCambioTurno = this.formBuilder.group({
            'idTurnoCambio': ['', AztecaValidators.selected],
            'cambioTurnoFechaIni': [new Date, (control) => this.validateFecha(control)],
        });

        this.formPasswd = this.formBuilder.group({
            'userContrasena': ['', Validators.required],
        });

    }

    ngOnInit(): void {
        setTimeout(() => {
            this.loadTiposRegimen();
            this.loadTiposContrato();
        })
    }

	///////////////////////// METODOS ///////////////////////////////////////

    loadTiposRegimen() {
        this.tipoRegimenSvc.loadCbo()
            .then(info => {
                this.tiposRegimen = info;
            })
            .catch(e => this.handleError(e));
    }

    loadTiposContrato() {
        this.tipoContratoSvc.loadCbo()
            .then(info => {
                this.tiposContrato = info;
            })
            .catch(e => this.handleError(e));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.empleadoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empleado = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.empleadoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empleado = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.empleadoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empleado = info;
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.empleadoService.update(this.empleado)
            .then(info => {
                this.context.app.hideSpinner();
                this.empleado = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'DOCUEMNTS') {
            this.router.navigate(['rh/documento-empleado', this.process.id, this.empleado.identity]) 
        }
        else if (option.name == "UPDATE_PASSWD") {
            this.userContrasena = "";
            this.dlgPasswd.config.backdrop = false;
            this.dlgPasswd.show();
        }
    }

    onClickCambioTurno() {
        this.dlgCambioTurno.config.backdrop = false;
        this.dlgCambioTurno.show();
    }

    onClickCambioTurnoAceptar() {
        this.context.app.showSpinner();
        this.empleadoService.programarCambioTurno(this.empleado.identity, this.empleado.idTurno,this.idTurnoCambio,this.cambioTurnoFechaIni)
            .then( _=> {
                this.context.app.hideSpinner();     
                this.empleado.idTurno = this.idTurnoCambio;
                this.selTurno.loadItem(this.idTurnoCambio);
                this.dlgCambioTurno.hide();
                this.showAlert('La información del cambio de turno se actualizo con éxito', 'success');
            })
            .catch(error => this.handleError(error));    
    }

    onClickPasswdAceptar() {
        this.context.app.showSpinner();
        this.empleadoService.cambioPassword(this.empleado.identity, this.userContrasena)
            .then(_ => {
                this.context.app.hideSpinner();
                this.dlgPasswd.hide();
                this.showAlert('La información del cambio de password se actualizo con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }


    validateFecha(control: FormControl) {
        let FechaHoy: Date;
        FechaHoy = moment(new Date).hour(0).minute(0).second(0).millisecond(0).toDate();

        if (control.value) {
            let Fecha = control.value;
            Fecha = moment(Fecha).hour(0).minute(0).second(0).millisecond(0).toDate();
            if (Fecha.getTime() >= FechaHoy.getTime())
                return null;
            else
                return 'La fecha no debe ser anterior a hoy';
        }
    }
}