﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { AsyncTaskService } from "framework3/queue/services/async-task.service";

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Features
import { EmpleadoService} from '../services/empleado.service';
import { EmpleadoInfo, EmpleadoItem} from '../models/empleado';
import { MenuItem } from 'framework3/controls';

@Component({    
    selector: 'rh-empleados-manager',
    templateUrl: './empleados.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EmpleadosManager)
        },
        {
            provide: CatalogService, useExisting: EmpleadoService
        }
	]
})
export class EmpleadosManager extends CatalogManager {
    empleados: EmpleadoItem[] = [];
    password: string;
    //    
    @ViewChild('dlgPasswd') dlgPasswd: ModalDirective;

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private empleadoService: EmpleadoService, private asyncTaskSvc: AsyncTaskService) {
        super(context, router, route);

    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.empleadoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.empleados = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.empleadoService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.empleadoService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'DOCUEMNTS') {
            this.router.navigate(['rh/documento-empleado', this.process.id, this.manager.currentItem.id])
        }
        if (option.name == 'IMPORTAR-ADAM') {
            this.context.app.showSpinner();
            this.empleadoService.asyncImportarEmpleadosAdam(this.context.app.defaultSite.idBranch)
                .then(r => {
                    this.asyncTaskSvc.subscribeTask(r.idAsyncTask).subscribe(info => {
                        this.context.app.hideSpinner();
                        console.log(info);
                        this.showAlert('Se completo la tarea de importación', 'success');
                    });
                    this.showAlert('Se genero una tarea asincrona para la solicitud. Se notificara cuando se termine', 'warning');
                    
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "UPDATE_PASSWD") {
            this.password = "";
            this.dlgPasswd.config.backdrop = false;
            this.dlgPasswd.show();
        }
    }

    onClickPasswdAceptar() {
        this.context.app.showSpinner();
        this.empleadoService.cambioPassword(this.manager.currentItem.id, this.password)
            .then(_ => {
                this.context.app.hideSpinner();
                this.dlgPasswd.hide();
                this.showAlert('La información del cambio de password se actualizo con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }


}
