﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
import { Control, CollectionView } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, AzComboboxComponent } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { EmpresaAdamService} from '../services/empresaadam.service';
import { EmpresaAdamInfo, EmpresaAdamItem} from '../models/empresaadam';

@Component({
    selector: 'rh-empresasadam-editor',
    templateUrl: './empresasadam.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EmpresasAdamEditor)
        },
        {
            provide: CatalogService, useExisting: EmpresaAdamService
        }
    ]
})
export class EmpresasAdamEditor extends CatalogEditor {
    empresaAdam: EmpresaAdamInfo = new EmpresaAdamInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private empresaAdamService: EmpresaAdamService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cadenaConexion': ['', Validators.required],
            'banco': ['', Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaAdamService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresaAdam = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.empresaAdamService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresaAdam = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.empresaAdamService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresaAdam = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.empresaAdamService.update(this.empresaAdam)
            .then(info => {
                this.context.app.hideSpinner();
                this.empresaAdam = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

}