﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, EnsambladoService, EnsambladoItem } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';

//Features
import { FirmwareService} from '../services/firmware.service';
import { FirmwareInfo, FirmwareItem} from '../models/firmware';

@Component({
    selector: 'rh-firmware-editor',
    templateUrl: './firmwares.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FirmwaresEditor)
        },
        {
            provide: CatalogService, useExisting: FirmwareService
        }
    ]
})
export class FirmwaresEditor extends CatalogEditor {
    firmware: FirmwareInfo = new FirmwareInfo();
    ensamblados: EnsambladoItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private firmwareService: FirmwareService, private ensambladoService: EnsambladoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idEnsamblado': [''],
            'clase': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////
    cargarEnsamblados() {
        this.ensambladoService.loadList()
            .then(ensamblados => {
                this.ensamblados = ensamblados;
                this.ensamblados.unshift({ id: 0, codigo: '', nombre: '[NINGUNO]', activo: true });
            })
            .catch(error => this.handleError(error));
    }

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.cargarEnsamblados();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.firmwareService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.firmware = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.firmwareService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.firmware = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.firmwareService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.firmware = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.firmwareService.update(this.firmware)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.firmware.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.firmware = info;
            })
            .catch(error => this.handleError(error));        
    }

}