﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { HorarioService} from '../services/horario.service';
import { HorarioInfo, HorarioItem} from '../models/horario';

@Component({
    selector: 'rh-horario-editor',
    templateUrl: './horarios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => HorariosEditor)
        },
        {
            provide: CatalogService, useExisting: HorarioService
        }
    ]
})
export class HorariosEditor extends CatalogEditor {
    horario: HorarioInfo = new HorarioInfo();

    tipoHorario: FormControl;
    evaluarHorarioNormal: FormControl;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private horarioService: HorarioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'tipoHorario': [''],
            'horaEntrada': [''],
            'evaluarHorarioNormal': [''],
            'horaSalida': [''],
            'salidaComer': [''],
            'horaComida': [''],
            'duracionComida': [''],
            'toleranciaComida': [''],
            'toleranciaRet': [''],
            'toleranciaFalta': [''],
            'tolAntesEnt': [''],
            'tolAntesSal': [''],
            'tolDespuesSal': [''],
            'inicioHrExtra': [''],
            'diasHorario': [''],
            'jornada': ['']
        });
        this.horario.horaEntrada = new Date();
        this.tipoHorario = <FormControl>this.catalogForm.controls['tipoHorario'];
        this.evaluarHorarioNormal = <FormControl>this.catalogForm.controls['evaluarHorarioNormal'];
        this.tipoHorario.setValue('N');
        this.evaluarHorarioNormal.setValue('SALIDA_FIJA');
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.horarioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.horario = info;   
                this.tipoHorario.setValue(this.horario.tipoHorario);
                this.evaluarHorarioNormal.setValue(this.horario.tipoEvaluacion);
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.horarioService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.horario = info;
                this.tipoHorario.setValue(this.horario.tipoHorario);
                this.evaluarHorarioNormal.setValue(this.horario.tipoEvaluacion);
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.horarioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.horario = info;
                this.tipoHorario.setValue(this.horario.tipoHorario);
                this.evaluarHorarioNormal.setValue(this.horario.tipoEvaluacion);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.horario.tipoHorario = this.tipoHorario.value;
        this.horario.tipoEvaluacion = this.evaluarHorarioNormal.value;
        this.context.app.showSpinner();
        this.horarioService.update(this.horario)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.horario.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.horario = info;
            })
            .catch(error => this.handleError(error));        
    }

}