﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { IncidenciaService } from '../services/incidencia.service';
import { KardexAsistenciaService } from '../services/kardex-asistencia.service';
import { IncidenciaInfo, IncidenciaItem} from '../models/incidencia';

@Component({
    selector: 'rh-incidencia-editor',
    templateUrl: './incidencias.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => IncidenciasEditor)
        },
        {
            provide: CatalogService, useExisting: IncidenciaService
        }
    ]
})
export class IncidenciasEditor extends CatalogEditor {

    incidencia: IncidenciaInfo = new IncidenciaInfo();
    kardexEstatus = [];
    
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private incidenciaService: IncidenciaService, private kardexAsistenciaSvc: KardexAsistenciaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'descripcion': [''],
            'estatus': ['']
        });
    }

    ngOnInit(): void {
        setTimeout(_ => {
            this.kardexAsistenciaSvc.getKardexEstatus()
                .then(info => {
                    this.kardexEstatus = info;
                })
                .catch(error => this.handleError(error));
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.incidenciaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.incidencia = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.incidenciaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.incidencia = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.incidenciaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.incidencia = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.incidenciaService.update(this.incidencia)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.incidencia.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.incidencia = info;
            })
            .catch(error => this.handleError(error));        
    }

}