﻿import { Component, Input, ViewChild, forwardRef } from '@angular/core';
import { ControlValueAccessor, FormGroup, FormBuilder, Validators, NG_VALUE_ACCESSOR } from '@angular/forms';

//ngx bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//moment
import * as moment from 'moment';

//Azteca Kernel
import { BaseComponent, PolicyInfo, Context } from 'framework3/core';
import { BaseControl } from 'framework3/controls';
import { EditableListComponent } from 'framework3/editors';

//Features
import { ParametroItem } from '../../models/parametro';


@Component({
    selector: 'rh-parametros-editor',
    templateUrl: 'parametros-editor.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ParametrosEditorComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => ParametrosEditorComponent), multi: true
        }
    ],
    styles: [
        '.ng-valid-component { border-left: 1px solid #ccc!important;}'
    ]
})
export class ParametrosEditorComponent extends BaseControl implements ControlValueAccessor {

    form: FormGroup;
    newParameter: ParametroItem = new ParametroItem();

    get valid(): boolean {
        return this.form.valid;
    }

    @Input() parametros: ParametroItem[] = [];

    @ViewChild('dlgParametro') dlgParameter: ModalDirective;
    @ViewChild('editListParametros') editListParametros: EditableListComponent;

    constructor(private formBuilder: FormBuilder) {
        super();

        this.form = formBuilder.group({
            'parameterPath': ['', Validators.required],
            'parameterKey': ['', Validators.required],
            'parameterValue': ['', Validators.required]
        });
    }


    onAddingItem() {
        this.newParameter = new ParametroItem();
        this.dlgParameter.config.backdrop = false;
        this.dlgParameter.show();
    }

    onDeletingParameter(item: ParametroItem) {
        this.editListParametros.deleteItem(item);
    }

    onListItemUpdateParameter(items: ParametroItem[]) {
        this.parametros = items;
        this.propagateChange(this.parametros);
    }
    

    onUpdatedParameter() {
        this.newParameter._clientID = this.createClientID();
        this.editListParametros.updateItem(this.cloneInfo<ParametroItem>(this.newParameter));
        this.dlgParameter.hide();
    }

    //////////////////////////////// CONTROL VALUE ACCESOR ////////////////////////////////////////
    writeValue(value: any) {
        this.parametros = value;
    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() {
    }

    createClientID(): number {
        return parseInt((Math.random() * 1000).toString()) * 1000 + parseInt(moment().format("HHmmss"))
    }

    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element));
    }
}