﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';

//Wijmo
import { Control } from 'wijmo/wijmo';
import { WjCalendar } from 'wijmo/wijmo.angular2.input';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//moment
import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, EditableListComponent } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { PeriodicidadPagoService} from '../services/periodicidad-pago.service';
import { PeriodicidadPagoInfo, PeriodicidadPagoItem, PeriodoDiaItem} from '../models/periodicidad-pago';
import { PeriodoItem } from '../models/periodicidad-pago';


@Component({
    selector: 'rh-periodicidad-pago-editor',
    templateUrl: './periodicidad-pago.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PeriodicidadPagoEditor)
        },
        {
            provide: CatalogService, useExisting: PeriodicidadPagoService
        }
    ]
})
export class PeriodicidadPagoEditor extends CatalogEditor {

    formPeriodo: FormGroup;

    periodicidadPago: PeriodicidadPagoInfo = new PeriodicidadPagoInfo();
    periodo: PeriodoItem = new PeriodoItem();
    periodoDia: PeriodoDiaItem = new PeriodoDiaItem();
    fechaSelected: Date = new Date();


    @ViewChild('dlgPeriodo') dlgPeriodoEditor: ModalDirective;
    @ViewChild('calendarFechaIncidencia') calendarFechaIncidencia: WjCalendar;
    @ViewChild('editListPeriodos') editListPeriodos: EditableListComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private periodicidadPagoService: PeriodicidadPagoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'dias': [0, Validators.required],
            'descripcion': ['']
        });

        this.formPeriodo = formBuilder.group({
            'periodoCodigo': ['', Validators.required],
            'periodoNombre': ['', Validators.required],
            'periodoDiaFeriado': [''],
            'periodoDiaFormaPago': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.periodicidadPagoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.periodicidadPago = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.periodicidadPagoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.periodicidadPago = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.periodicidadPagoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.periodicidadPago = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.periodicidadPagoService.update(this.periodicidadPago)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.periodicidadPago.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.periodicidadPago = info;
            })
            .catch(error => this.handleError(error));        
    }


    onTabSelected(args: any) {
        if (args.heading) {
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////////////////// CUSTOM ///////////////////////////////////////

    onAddingItemPeriodo() {
        this.periodo = new PeriodoItem();
        this.periodo._clientID = this.createClientID();
        this.dlgPeriodoEditor.config.backdrop = false;
        this.dlgPeriodoEditor.show();
    }

    onListItemUpdatePeriodo(items: PeriodoItem[]) {
        this.periodicidadPago.periodos = items;
    }

    onEditingItemPeriodo(item: PeriodoItem) {
        this.periodo = this.cloneInfo<PeriodoItem>(this.editListPeriodos.currentItem);
        this.dlgPeriodoEditor.config.backdrop = false;
        this.dlgPeriodoEditor.show();
    }

    onDeletingPeriodo(item: any) {
        this.editListPeriodos.deleteItem(item);
    }

    onUpdatedPeriodo() {
        this.editListPeriodos.updateItem(this.periodo);
        this.dlgPeriodoEditor.hide();
    }

    onDayClicked(event: any) {
        let date = this.calendarFechaIncidencia.value;
        let dia = this.periodo.periodoDias.find(p => moment(p.fechaIncidencia).format("YYYY-MM-DD") == moment(date).format("YYYY-MM-DD"));
        if (dia) {
            this.periodoDia = dia;
        }
        else {
            this.periodoDia = new PeriodoDiaItem();
            this.periodoDia.fechaIncidencia = date;
            this.periodoDia.formaPago = 1;
            this.periodo.periodoDias.push(this.periodoDia);
        }
    }

    createClientID(): number {
        return parseInt((Math.random() * 1000).toString()) * 1000 + parseInt(moment().format("HHmmss"))
    }

    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element), this.replacer);
    }

    replacer(key, value) {
        if (key == "fechaIniPago" || key == "fechaFinPago" || key == "fechaIniIncidencia" || key == "fechaFinIncidencia" || key == "fechaIncidencia")
            return moment(value).toDate();
        else
            return value;
    }
}