﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

//Wijmo
import { Control } from 'wijmo/wijmo';


//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, EditableListComponent } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { TipoNominaService} from '../services/tipo-nomina.service';
import { TipoNominaInfo, TipoNominaItem, TipoNominaConceptoItem } from '../models/tipo-nomina';
import { ConceptoAdamInfo, ConceptoAdamItem } from '../models/concepto-adam';
import { ConceptosListComponent } from '../controls/conceptos-list.component';

@Component({
    selector: 'rh-tipo-nomina-editor',
    templateUrl: './tipos-nominas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TiposNominasEditor)
        },
        {
            provide: CatalogService, useExisting: TipoNominaService
        }
    ]
})
export class TiposNominasEditor extends CatalogEditor {
    tipoNomina: TipoNominaInfo = new TipoNominaInfo();        

    @ViewChild(ConceptosListComponent) dlgConceptos: ConceptosListComponent;
    @ViewChild('editListConceptos') editListConceptos: EditableListComponent;

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private tipoNominaService: TipoNominaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idCompania': [0, AztecaValidators.selected],
            'idPeriodicidadPago': [0, AztecaValidators.selected],
            'descripcion': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoNominaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoNomina = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.tipoNominaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoNomina = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoNominaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoNomina = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.tipoNominaService.update(this.tipoNomina)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.tipoNomina.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.tipoNomina = info;
            })
            .catch(error => this.handleError(error));        
    }

    // Editing items
    onAddingConcepto() {
        this.dlgConceptos.show();
    }

    onListItemUpdatedConcepto(conceptos: TipoNominaConceptoItem[]) {
        this.tipoNomina.conceptos = conceptos;
    }

    onDeletingConcepto(concepto: TipoNominaConceptoItem) {
        this.editListConceptos.deleteItem(concepto);
    }

    // Conceptos
    onSelectedItemsConcepto(items: ConceptoAdamItem[]) {
        let conceptos: TipoNominaConceptoItem[] = [];
        items.forEach(item => {
            let concepto = this.createConcepto(item);
            conceptos.push(concepto);
        });
        this.editListConceptos.updateItems(conceptos);
    }

    onSelectedItemConcepto(item: ConceptoAdamItem) {
        let concepto = this.createConcepto(item);
        this.editListConceptos.updateItem(concepto);
    }

    createConcepto(item: ConceptoAdamItem): TipoNominaConceptoItem {
        var tipoNominaConcepto: TipoNominaConceptoItem = new TipoNominaConceptoItem();
        tipoNominaConcepto._clientID = item.id;
        tipoNominaConcepto.idConcepto = item.id;
        tipoNominaConcepto.codigo = item.codigo;
        tipoNominaConcepto.concepto = item.descripcion;
        return tipoNominaConcepto;
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

}