﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { TipoPerDedService} from '../services/tipo-per-ded.service';
import { TipoPerDedInfo, TipoPerDedItem} from '../models/tipo-per-ded';

@Component({
    selector: 'rh-tipo-per-ded-editor',
    templateUrl: './tipos-per-ded.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TiposPerDedEditor)
        },
        {
            provide: CatalogService, useExisting: TipoPerDedService
        }
    ]
})
export class TiposPerDedEditor extends CatalogEditor {
    tipoPerDed: TipoPerDedInfo = new TipoPerDedInfo();        
    //
    tiposTransaccion: Array<any> = [{ id: 1, nombre: 'Percepción' }, { id: 2, nombre: 'Deducción' }, { id: 3, nombre: 'Otros Pagos' }, { id: 4, nombre: 'Incapacidad' }];

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private tipoPerDedService: TipoPerDedService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'descripcion': [''],
            'tipoTransaccion': ['', Validators.required]
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoPerDedService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoPerDed = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.tipoPerDedService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoPerDed = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.tipoPerDedService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.tipoPerDed = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.tipoPerDedService.update(this.tipoPerDed)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.tipoPerDed.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.tipoPerDed = info;
            })
            .catch(error => this.handleError(error));        
    }

}