﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
import { Control, CollectionView } from 'wijmo/wijmo';

//Wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import { Menu } from 'wijmo/wijmo.input';

//moment
import * as moment from 'moment';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';

//Azteca Kernel
import { Context, BaseComponent, CatalogService, ChangeState, CatalogItem } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, AzComboboxComponent, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { TurnoService } from '../services/turno.service';
import { TurnoInfo, TurnoItem, TurnoHorarioInfo } from '../models/turno';
import { forEach } from '@angular/router/src/utils/collection';
import { take } from 'rxjs-compat/operator/take';
import { JornadaService } from '../../accounting/services/jornada.service';

import { HorarioService } from '../services/horario.service';
import { HorarioItem } from '../models/horario';
import { HorarioListComponent } from '../controls/horario-list.component';
import { FormGroup } from '@angular/forms/src/model';
import { EmpleadoItem } from '../models/empleado';
import { EmpleadoListComponent } from '../controls/empleado-list.component';

@Component({
    selector: 'rh-turno-editor',
    templateUrl: './turnos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => TurnosEditor)
        },
        {
            provide: CatalogService, useExisting: TurnoService
        }
    ]
})
export class TurnosEditor extends CatalogEditor implements OnInit {

    formNumero: FormGroup;

    turno: TurnoInfo = new TurnoInfo();
    horario: number = 0;

    empleados: EmpleadoItem[] = [];
    currentItem: CatalogItem;

    cvCiclo: TrackableCollection;
    menuCicloItems: MenuItem[] = [];
    horarioCiclo: boolean = false;

    titleNumero: string = "";
    numero: number = 0;

    turnoTipos = [
        { 'name': 'Semanal', 'value': 'S' },
        { 'name': 'Mensual', 'value': 'M' },
        { 'name': 'Programado', 'value': 'P' }
    ];

    jornadas = [];

    diasCubreTurno: boolean = false;
    evaluarAccion: FormControl;

    @ViewChild('dlgHorario') dlgHorario: HorarioListComponent;
    @ViewChild('dlgNumero') dlgNumero: ModalDirective;
    @ViewChild(EmpleadoListComponent) selEmpleado: EmpleadoListComponent;

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private turnoService: TurnoService, private jornadaSvc: JornadaService, private horarioSvc: HorarioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'tipo': ['', Validators.required],
            'ciclo': [''],
            'retardos': [''],
            'periodoDias': [''],
            'accion': [''],
            'nocturno': [''],
            'cuenta': [''],
            'diasCubreTurno': [''],
            'evaluarAccion': [''],
            'diasAccion': [''],
            'horario': [''],
            'idJornada': [0],
            'dom': [''],
            'lun': [''],
            'mar': [''],
            'mie': [''],
            'jue': [''],
            'vie': [''],
            'sab': ['']
        });

        this.formNumero = formBuilder.group({
            'numero': [0, Validators.required]
        });

        this.evaluarAccion = <FormControl>this.catalogForm.controls['evaluarAccion'];
        this.cvCiclo = new TrackableCollection();
    }

    ngOnInit() {
        setTimeout(() => {
            this.context.app.showSpinner();
            this.jornadaSvc.getJornadaCbo(this.context.app.defaultSite.identity)
                .then(info => {
                    this.jornadas = info;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
            this.createMenuDetail();
        });

        this.evaluarAccion.valueChanges.subscribe(value => {
            this.turno.accion = value;
        });
    }

    createMenuDetail() {
        let mnuHorarioEliminar: MenuItem = { icon: "glyphicon glyphicon-trash", text: "Quitar Horario", name: "REMOVER_HORARIO", smallText: "Quitar horario" };
        let mnuHorarioCiclo: MenuItem = { icon: "glyphicon glyphicon-time", text: "Horario Ciclo", name: "HORARIO_CICLO", smallText: "Horario ciclo" };
        let mnuDiasCiclo: MenuItem = { icon: "glyphicon glyphicon-pencil", text: "Días Ciclo", name: "DIAS_CICLO", smallText: "Días ciclo" };

        this.menuCicloItems.push(mnuHorarioEliminar, mnuHorarioCiclo, mnuDiasCiclo);
        this.menuCicloItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    contextMnuHorarioClick(menu: Menu) {
        let option = this.menuCicloItems[menu.selectedIndex];
        if (option.name == "REMOVER_HORARIO") {
            this.context.app.showSpinner();
            let item = <TurnoHorarioInfo>this.cvCiclo.currentItem;
            if (item) {
                this.cvCiclo.editItem(item);
                item.idHorario = 0;
                item.horario = null;
                item.state = ChangeState.MODIFIED;
                this.cvCiclo.commitEdit();
            }
            this.context.app.hideSpinner();
        }
        else if (option.name == "HORARIO_CICLO") {
            this.horarioCiclo = true;
            this.dlgHorario.show();
        }
        else if (option.name == "DIAS_CICLO") {
            if (this.turno.tipo == "P") {
                this.titleNumero = "Num. Días";
                this.numero = 1;
                this.dlgNumero.config.backdrop = false;
                this.dlgNumero.show();
            }
            else {
                this.showAlert('Opción para turnos programados', 'warning');
            }
        }
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        this.cvCiclo.sourceCollection = this.turno.horarios;
        this.cvCiclo.refresh();
        this.evaluarAccion.setValue("N");
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.turnoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.turno = info;
                this.cvCiclo.sourceCollection = this.turno.horarios;
                this.cvCiclo.refresh();
                this.evaluarAccion.setValue(this.turno.accion);
                this.diasCubreTurno = this.turno.tipoPeriodo == "T";
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.turnoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.turno = info;
                this.turno.horarios = [];
                this.cvCiclo.sourceCollection = this.turno.horarios;
                this.cvCiclo.refresh();
                this.evaluarAccion.setValue(this.turno.accion);
                this.diasCubreTurno = this.turno.tipoPeriodo == "T";
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();
        this.turnoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.turno = info;
                this.cvCiclo.sourceCollection = this.turno.horarios;
                this.cvCiclo.refresh();
                this.evaluarAccion.setValue(this.turno.accion);
                this.diasCubreTurno = this.turno.tipoPeriodo == "T";
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.turno.accion = this.evaluarAccion.value;
        this.turno.tipoPeriodo = this.diasCubreTurno ? "T" : "F";
        this.context.app.showSpinner();
        this.turnoService.update(this.turno)
            .then(info => {
                this.context.app.hideSpinner();
                this.turno = info;
                this.cvCiclo.sourceCollection = this.turno.horarios;
                this.cvCiclo.refresh();
                this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }

    updateCiclo(tipo: string, ciclo: number) {

        this.cvCiclo.items.forEach(item => {
            this.cvCiclo.editItem(item);
            item.state = ChangeState.DELETED;
            this.cvCiclo.commitEdit();
        });

        switch (tipo) {
            case 'S':
                if (this.cvCiclo.itemCount != ciclo * 7) {
                    for (var indexCiclo = 1; indexCiclo <= ciclo; indexCiclo++) {
                        for (var index = 1; index <= 7; index++) {
                            let item = <TurnoHorarioInfo>this.cvCiclo.addNew();
                            item._clientID = this.createClientID();
                            item.dia = index;
                            item.ciclo = indexCiclo;
                            item.state = ChangeState.ADDED;
                            this.cvCiclo.commitNew();
                        }
                    }
                }
                break;
            case 'M':
                if (this.cvCiclo.itemCount != ciclo * 31) {
                    for (var indexCiclo = 1; indexCiclo <= ciclo; indexCiclo++) {
                        for (var index = 1; index <= 31; index++) {
                            let item = <TurnoHorarioInfo>this.cvCiclo.addNew();
                            item._clientID = this.createClientID();
                            item.dia = index;
                            item.ciclo = indexCiclo;
                            item.state = ChangeState.ADDED;
                            this.cvCiclo.commitNew();
                        }
                    }
                }
                break;
            case 'P':
                for (var indexCiclo = 1; indexCiclo <= ciclo; indexCiclo++) {
                    let item = <TurnoHorarioInfo>this.cvCiclo.addNew();
                    item._clientID = this.createClientID();
                    item.dia = 1;
                    item.ciclo = indexCiclo;
                    item.state = ChangeState.ADDED;
                    this.cvCiclo.commitNew();
                }
                break;
        }
    }

    initializedGrdCiclo(sender: wjcGrid.FlexGrid, args: any) {
        var self = this;
        //Manejo de doble click
        sender.addEventListener(sender.hostElement, "dblclick", (e: any) => {
            let ht: wjcGrid.HitTestInfo = sender.hitTest(e);
            if (ht.cellType == wjcGrid.CellType.Cell) {
                this.dlgHorario.show();
            }
        });
    }

    onTabSelectedFlexGrid(flex: wjcGrid.FlexGrid) {
        if (flex.collectionView != null) {
            flex.collectionView.refresh();
        }
        Control.invalidateAll();
    }


    onSelectedHorario(horarioItem: HorarioItem) {
        let horarioByCiclo = this.horarioCiclo;
        this.horarioCiclo = false;
        this.context.app.showSpinner();
        this.horarioSvc.getDetailByID(horarioItem.id)
            .then(info => {
                let item = <TurnoHorarioInfo>this.cvCiclo.currentItem;
                if (item) {
                    if (horarioByCiclo) {
                        this.cvCiclo.items.forEach(itemCiclo => {
                            if (item.ciclo == itemCiclo.ciclo) {
                                this.cvCiclo.editItem(itemCiclo);
                                itemCiclo.idHorario = info.identity;
                                itemCiclo.horario = info;
                                itemCiclo.state = ChangeState.MODIFIED;
                                this.cvCiclo.commitEdit();
                            }
                        });
                    }
                    else {
                        this.cvCiclo.editItem(item);
                        item.idHorario = info.identity;
                        item.horario = info;
                        item.state = ChangeState.MODIFIED;
                        this.cvCiclo.commitEdit();

                    }
                    this.context.app.hideSpinner();
                }
            })
            .catch(error => this.handleError(error));
    }

    onClickCiclos() {
        this.numero = this.turno.ciclo;
        this.titleNumero = "Num. Ciclos";
        this.dlgNumero.config.backdrop = false;
        this.dlgNumero.show();
    }

    onUpdatedNumro() {
        this.context.app.showSpinner();
        if (this.titleNumero == "Num. Ciclos") {
            this.turno.ciclo = this.numero;
            this.updateCiclo(this.turno.tipo, this.turno.ciclo);
        }
        else {
            let item = <TurnoHorarioInfo>this.cvCiclo.currentItem;
            if (item) {
                this.cvCiclo.items.forEach(itemCiclo => {
                    if (item.ciclo == itemCiclo.ciclo) {
                        this.cvCiclo.editItem(itemCiclo);
                        itemCiclo.state = ChangeState.DELETED;
                        this.cvCiclo.commitEdit();
                    }
                });

                this.cvCiclo.beginUpdate();
                for (var indexDia = 1; indexDia <= this.numero; indexDia++) {
                    let itemCiclo = <TurnoHorarioInfo>this.cvCiclo.addNew();
                    itemCiclo._clientID = this.createClientID();
                    itemCiclo.dia = indexDia;
                    itemCiclo.ciclo = item.ciclo;
                    itemCiclo.state = ChangeState.ADDED;
                }
                this.cvCiclo.endUpdate();
            }
        }
        this.context.app.hideSpinner();
        this.dlgNumero.hide();
    }

    createClientID(): number {
        return parseInt((Math.random() * 1000).toString()) * 1000 + parseInt(moment().format("HHmmss"))
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.selEmpleado.show();
        }
    }

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    refreshListEmployes() {
        if (this.turno.identity > 0) {
            this.context.app.showSpinner();
            this.turnoService.getEmpleadosByTurno(this.turno.identity, this.context.app.defaultSite.identity)
                .then(empleados => {
                    this.empleados = empleados;
                    this.context.app.hideSpinner();
                    Control.invalidateAll();
                })
                .catch(error => this.handleError(error));
        }
    }

    onSelectedEmleados(items: EmpleadoItem[]) {
        this.context.app.showSpinner();
        this.turnoService.addTurnoEmpleado(this.turno.identity, items)
            .then(info => this.refreshListEmployes())
            .catch(error => this.handleError(error));
    }

    onRemoveEmploye() {
        this.context.app.showSpinner();
        this.turnoService.removeTurnoEmpleado(this.currentItem.id)
            .then(info => this.refreshListEmployes())
            .catch(error => this.handleError(error));
    }
}