﻿import { Component, Input, Output, EventEmitter, forwardRef, SimpleChange } from '@angular/core';
import { ControlValueAccessor, Validators, NG_VALUE_ACCESSOR, FormGroup, FormBuilder } from '@angular/forms';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
//
import { BaseControl } from 'framework3/controls/index';
import { BaseComponent } from 'framework3/core';
import { Context } from 'framework3/core';
//
import { AdamNominaInfo } from '../models/adamnomina'
import { EmpresaAdamService } from '../services/empresaadam.service'
import { AdamService } from '../services/adam.service'

@Component({
    selector: 'rh-adam-nomina',
    templateUrl: './adam-nomina.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => AdamNominaComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => AdamNominaComponent),
        multi: true
    }]
})
export class AdamNominaComponent extends BaseControl implements ControlValueAccessor {

    private form: FormGroup;

    private empresaAdamSvc: EmpresaAdamService;
    private adamSvc: AdamService;

    get valid(): boolean {
        return this.form.valid;
    }

    // Inputs
    @Input() adamNomina: AdamNominaInfo = new AdamNominaInfo();
    @Input() idSucursal: number;

    // Outputs
    @Output() adamNominaUpdated = new EventEmitter<AdamNominaInfo>();
    @Output() validChange = new EventEmitter<boolean>();
    @Output() handleError = new EventEmitter<any>();

    // Compania
    @Input() companiaRequired: boolean = false;
    @Input() companiaVisible: boolean = true;
    @Input() companiaEnabled: boolean = true;

    // Clase Nomina
    @Input() claseNominaRequired: boolean = false;
    @Input() claseNominaVisible: boolean = true;
    @Input() claseNominaEnabled: boolean = true;

    // Tipo Nomina
    @Input() tipoNominaRequired: boolean = false;
    @Input() tipoNominaVisible: boolean = true;
    @Input() tipoNominaEnabled: boolean = true;

    // Anio
    @Input() anioRequired: boolean = false;
    @Input() anioVisible: boolean = true;
    @Input() anioEnabled: boolean = true;

    // Periodo
    @Input() periodoRequired: boolean = false;
    @Input() periodoVisible: boolean = true;
    @Input() periodoEnabled: boolean = true;

    // Internal values
    nominaInfo: AdamNominaInfo = new AdamNominaInfo(); 

    empresasAdam: Array<any>    = new Array<any>();
    companiasAdam: Array<any>   = new Array<any>();
    clasesNomina: Array<any>    = new Array<any>();
    tiposNomina: Array<any>     = new Array<any>();
    anios: Array<any>           = new Array<any>();
    periodos: Array<any>        = new Array<any>();

    context: Context; 

    constructor(http: HttpClient, context: Context, private formBuilder: FormBuilder) {
        super();
        this.context = context;
        this.form = formBuilder.group({
            'cboEmpresaAdam': [''],
            'cboCompaniaAdam': [''],
            'cboClaseNomina': [''],
            'cboTipoNomina': [''],
            'cboAnio': [''],
            'cboPeriodo': ['']
        });
        this.empresaAdamSvc = new EmpresaAdamService(http, context);
        this.adamSvc = new AdamService(http, context);
    }

    onChangeEmpresaAdam(args: any) {
        this.nominaInfo.idEmpresaAdam = args;
        this.loadCompaniasAdam();
        this.loadClaseNomina();
        this.notifiyChange();
    }

    onChangeCompania(args: any) {
        this.nominaInfo.compania = args;
        this.loadAnios();
        this.loadPeriodos();
        this.notifiyChange();
    }

    onChangeClaseNomina(args: any) {
        this.nominaInfo.claseNomina = args;
        this.loadTiposNomina();
        this.notifiyChange();
    }

    onChangeTipoNomina(args: any) {
        this.nominaInfo.tipoNomina = args;
        this.loadAnios();
        this.loadPeriodos();
        this.notifiyChange();
    }

    onChangeAnio(args: any) {
        this.nominaInfo.anio = args;
        this.loadPeriodos();
        this.notifiyChange();
    }

    onChangePeriodo(args: any) {
        this.nominaInfo.periodo = args;
        this.notifiyChange();
    }

    ngOnChanges(changes: SimpleChange) {
        if (changes['companiaRequired']) {
            this.form.controls['cboCompaniaAdam'].setValidators(this.companiaRequired ? Validators.required : null);
        }

        if (changes['claseNominaRequired']) {
            this.form.controls['cboClaseNomina'].setValidators(this.companiaRequired ? Validators.required : null);
        }

        if (changes['tipoNominaRequired']) {
            this.form.controls['cboTipoNomina'].setValidators(this.companiaRequired ? Validators.required : null);
        }

        if (changes['anioRequired']) {
            this.form.controls['cboAnio'].setValidators(this.companiaRequired ? Validators.required : null);
        }

        if (changes['periodoRequired']) {
            this.form.controls['cboPeriodo'].setValidators(this.companiaRequired ? Validators.required : null);
        }
    }

    ngAfterViewInit() {
        this.form.statusChanges.subscribe(_ => {
            this.validChange.emit(this.form.valid);
        });
        this.loadEmpresasAdam();
    }

    //////////////////////////////// HANDLERS ////////////////////////////////////////
    loadEmpresasAdam() {
        if (this.idSucursal > 0 && this.empresaAdamSvc != null) {
            this.context.app.showSpinner();
            this.empresaAdamSvc.loadEmpresasAdamCBO(this.idSucursal).then(info => {
                this.empresasAdam = info;
                this.context.app.hideSpinner();
            }).catch(error => {
                this.context.app.hideSpinner();
                this.handleError.emit(error)
            });
        }
    }

    loadCompaniasAdam() {
        if (this.nominaInfo.idEmpresaAdam > 0 &&
            this.adamSvc != null)
        {
            this.context.app.showSpinner();
            this.adamSvc.loadCompaniasAdam(this.nominaInfo.idEmpresaAdam).then(info => {
                this.companiasAdam = info;
                this.context.app.hideSpinner();
            }).catch(error => {
                this.context.app.hideSpinner();
                this.handleError.emit(error)
            });
        }
        else {
            this.companiasAdam.length = 0;
        }
    }

    loadClaseNomina() {
        if (this.nominaInfo.idEmpresaAdam > 0 &&
            this.adamSvc != null)
        {
            this.context.app.showSpinner();
            this.adamSvc.loadClaseNomina(this.nominaInfo.idEmpresaAdam).then(info => {
                this.clasesNomina = info;
                this.context.app.hideSpinner();
            }).catch(error => {
                this.context.app.hideSpinner();
                this.handleError.emit(error)
            });
        }
        else {
            this.clasesNomina.length = 0;
        }
    }

    loadTiposNomina() {
        if (this.nominaInfo.idEmpresaAdam > 0 &&
            this.nominaInfo.claseNomina != undefined &&
            this.nominaInfo.claseNomina != '' &&
            this.adamSvc != null)
        {
            this.context.app.showSpinner();
            this.adamSvc.loadTipoNomina(this.nominaInfo.idEmpresaAdam, this.nominaInfo.claseNomina).then(info => {
                this.tiposNomina = info;
                this.context.app.hideSpinner();
            }).catch(error => {
                this.context.app.hideSpinner();
                this.handleError.emit(error)
            });
        }
        else {
            this.tiposNomina.length = 0;
        }
    }

    loadAnios() {
        if (this.nominaInfo.idEmpresaAdam > 0 &&
            this.nominaInfo.compania != undefined &&
            this.nominaInfo.compania != '' &&
            this.nominaInfo.tipoNomina != undefined &&
            this.nominaInfo.tipoNomina != '' &&
            this.adamSvc != null )
        {
            this.context.app.showSpinner();
            this.adamSvc.loadAnio(this.nominaInfo.idEmpresaAdam, this.nominaInfo.compania, this.nominaInfo.tipoNomina).then(info => {
                this.anios = info;
                this.context.app.hideSpinner();
            }).catch(error => {
                this.context.app.hideSpinner();
                this.handleError.emit(error)
            });
        }
        else {
            this.anios.length = 0;
        }
    }

    loadPeriodos() {
        if (this.nominaInfo.idEmpresaAdam > 0 &&
            this.nominaInfo.compania != undefined &&
            this.nominaInfo.compania != '' &&
            this.nominaInfo.tipoNomina != undefined &&
            this.nominaInfo.tipoNomina != '' &&
            this.nominaInfo.anio > 0 &&
            this.adamSvc != null)
        {
            this.context.app.showSpinner();
            this.adamSvc.loadPeriodo(this.nominaInfo.idEmpresaAdam, this.nominaInfo.compania, this.nominaInfo.tipoNomina, this.nominaInfo.anio).then(info => {
                this.periodos = info;
                this.context.app.hideSpinner();
            }).catch(error => {
                this.context.app.hideSpinner();
                this.handleError.emit(error)
            });
        }
        else {
            this.periodos.length = 0;
        }
    }

    updateNominaAdam() {
        this.nominaInfo.idEmpresaAdam   = this.adamNomina.idEmpresaAdam;
        this.nominaInfo.compania        = this.adamNomina.compania;
        this.nominaInfo.claseNomina     = this.adamNomina.claseNomina;
        this.nominaInfo.tipoNomina      = this.adamNomina.tipoNomina;
        this.nominaInfo.anio            = this.adamNomina.anio;
        this.nominaInfo.periodo         = this.adamNomina.periodo;
    }

    notifiyChange() {
        let newAdamNomina = new AdamNominaInfo();

        newAdamNomina.idEmpresaAdam     = this.nominaInfo.idEmpresaAdam;
        newAdamNomina.compania          = this.nominaInfo.compania;
        newAdamNomina.claseNomina       = this.nominaInfo.claseNomina;
        newAdamNomina.tipoNomina        = this.nominaInfo.tipoNomina;
        newAdamNomina.anio              = this.nominaInfo.anio;
        newAdamNomina.periodo           = this.nominaInfo.periodo;

        this.adamNominaUpdated.emit(newAdamNomina);
        this.propagateChange(newAdamNomina);
    }

    //////////////////////////////// CONTROL VALUE ACCESOR ////////////////////////////////////////

    writeValue(value: any) {
        this.adamNomina = value;
    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() {
    }

}

