﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search';
//
import { ClaseNominaInfo, ClaseNominaItem } from '../models/clase-nomina';
import { ClaseNominaService } from '../services/clase-nomina.service';


@Component({
    selector: 'rh-clase-nomina-selector',
    templateUrl: './clase-nomina-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ClaseNominaSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ClaseNominaSelectorComponent),
        multi: true
    }]
})
export class ClaseNominaSelectorComponent extends CatalogSelector {
    items: any[];
    selectedItem: ClaseNominaInfo = new ClaseNominaInfo();

    @Output() claseNominaUpdated = new EventEmitter();

    constructor(protected context: Context, private service: ClaseNominaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findPlaza(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + ' ' + info.nombre; //+ ", " + info.municipio.estado.nombre;
                    this.claseNominaUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.claseNominaUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.items = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findPlaza(keyword: string): Observable<CatalogItem[]> {
        return this.service.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

