﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { CompaniaInfo, CompaniaItem } from '../models/compania';
import { CompaniaService } from '../services/compania.service';

@Component({
    selector: 'rh-compania-selector',
    templateUrl: './compania-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => CompaniaSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => CompaniaSelectorComponent),
        multi: true
    }]
})
export class CompaniaSelectorComponent extends CatalogSelector {
    compania: any[];
    selectedItem: CompaniaInfo = new CompaniaInfo;

    
    @Output() companiaUpdated = new EventEmitter();
    @Input() companiaSelector: boolean = false;

    constructor(protected context: Context, private companiaService: CompaniaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findCompania(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.companiaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + ' ' +  info.nombre;
                    this.companiaUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.companiaUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.companiaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.compania = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findCompania(keyword: string): Observable<CatalogItem[]> {
        return this.companiaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

