﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search';
//
import { ConceptosFichasInfo } from '../models/concepto-ficha';
import { ConceptosFichasService } from '../services/conceptos-fichas.service';

@Component({
    selector: 'rh-conceptos-fichas-selector',
    templateUrl: './conceptos-fichas-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => ConceptoFichasSelector)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => ConceptoFichasSelector),
        multi: true
    }]
})
export class ConceptoFichasSelector extends CatalogSelector {
    conceptos: any[];
    selectedItem: ConceptosFichasInfo = new ConceptosFichasInfo;


    @Output() conceptoUpdated = new EventEmitter();

    constructor(protected context: Context, private conceptoService: ConceptosFichasService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findConcepto(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.conceptoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + " " + info.nombre; //+ ", " + info.municipio.estado.nombre;
                    this.conceptoUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.conceptoUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.conceptoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.conceptos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findConcepto(keyword: string): Observable<CatalogItem[]> {
        return this.conceptoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

