﻿import { Component, Input, Output, ViewChild, ContentChildren, OnInit, EventEmitter, QueryList, forwardRef, Optional, SkipSelf } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SearchCondition, SortOrder, HierarchySelector, ITreeHost } from 'framework3/search/index'; //'../../search/index';
//
//
//
import { DepartamentoInfo, DepartamentoItem } from '../models/departamento';
import { DepartamentoService } from '../services/departamento.service';

@Component({
    selector: 'rh-departamento-selector',
    templateUrl: './departamento-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => DepartamentoSelectorComponent)
    },
    {
        provide: ITreeHost,
        useExisting: forwardRef(() => DepartamentoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => DepartamentoSelectorComponent),
        multi: true
    }]
})
export class DepartamentoSelectorComponent extends HierarchySelector implements ITreeHost {
    departamentos: any[];//DepartamentoItem[];
    nodes: DepartamentoInfo[];
    selectedItem: DepartamentoInfo = null;
    //Inputs
    @Input() idDocumento: number = 0;
    @Input() version: number = 0;
    @Input() idPadre: number = 0;
    @Input() withAccess: boolean = false;
    //Outputs
    @Output() selectedItemChanged: EventEmitter<DepartamentoInfo> = new EventEmitter<DepartamentoInfo>();

    constructor(protected context: Context, private departamentoService: DepartamentoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        }).pipe(mergeMap((keyword: string) => this.findKeyword(keyword)));
    }

    ngOnInit() {
       //Se inhabilita mientras se encuentra la manera de filtrar el arbol o inhabilitar los nodos sin acceso
       //this.showTreeButton = !this.withAccess;
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.departamentoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + "-" + info.nombre;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = '';
            this.selectedItemChanged.emit(this.selectedItem);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.withAccess) {
            let condition: SearchCondition = { field: 'Acceso', operator: '<>', valueType: 'number', value: '0' };
            request.conditions.push(condition);
        }


        this.departamentoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.departamentos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => {
                this.handleError(error)
            });
    }

    findKeyword(keyword: string): Observable<CatalogItem[]> {
        if (this.withAccess)
            return this.departamentoService.searchKeywordWithAccess(this.context.app.defaultSite.identity, keyword);
        else
            return this.departamentoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

    onLoadingHierarchy() {
        //Cargar nodos raíz al abrir diálogo de jerarquía
        if (this.withAccess) {
            this.departamentoService.nodesWithAccess(this.context.app.defaultCompany.identity)
                .then(rootNodes => this.nodes = rootNodes)
                .catch(error => this.handleError(error));
        }
        else {
            this.departamentoService.loadChildren(this.context.app.defaultCompany.identity, 0)
                .then(rootNodes => this.nodes = rootNodes)
                .catch(error => this.handleError(error));

        }
        
    }


    loadChildren(parent: DepartamentoInfo): Promise<DepartamentoInfo[]> {
        return this.departamentoService.loadChildren(this.context.app.defaultCompany.identity, parent.identity);
    }


}