﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search';
//
import { DocumentoTipoInfo, DocumentoTipoItem } from '../models/documento-tipo';
import { DocumentoTipoService } from '../services/documento-tipo.service';


@Component({
    selector: 'rh-documento-tipo-selector',
    templateUrl: './documento-tipo-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => DocumentoTipoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => DocumentoTipoSelectorComponent),
        multi: true
    }]
})
export class DocumentoTipoSelectorComponent extends CatalogSelector {
    items: any[];
    selectedItem: DocumentoTipoInfo = new DocumentoTipoInfo();

    @Output() documentoTipoUpdated = new EventEmitter();
    @Input() proceso: string = '';
    constructor(protected context: Context, private service: DocumentoTipoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findPlaza(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + ' ' + info.nombre; //+ ", " + info.municipio.estado.nombre;
                    this.documentoTipoUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.documentoTipoUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();
        let condition: SearchCondition = { field: 'Proceso', operator: '=', valueType: 'string', value: this.proceso };
        request.conditions.push(condition);

        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.items = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findPlaza(keyword: string): Observable<CatalogItem[]> {
        return this.service.searchKeyword2(this.proceso, this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

