﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search';
//
import { EmpleadoInfo, EmpleadoItem } from '../models/empleado';
import { EmpleadoService } from '../services/empleado.service';

@Component({
    selector: 'rh-empleado-selector',
    templateUrl: './empleado-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => EmpleadoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => EmpleadoSelectorComponent),
        multi: true
    }]
})
export class EmpleadoSelectorComponent extends CatalogSelector {
    empleado: any[];
    selectedItem: EmpleadoInfo = new EmpleadoInfo;

    
    @Output() empleadoUpdated = new EventEmitter();
    @Input() soloSucursal: boolean = true;

    constructor(protected context: Context, private empleadoService: EmpleadoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findEmpleado(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.empleadoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + " " + info.persona.nombre + " " + info.persona.apellidoPaterno + " " + info.persona.apellidoMaterno; //+ ", " + info.municipio.estado.nombre;
                    this.empleadoUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.empleadoUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.soloSucursal) {
            this.empleadoService.search(this.context.app.defaultSite.identity, request)
                .then(response => {
                    this.context.app.hideSpinner();
                    //
                    this.empleado = response.rows;
                    this.totalRows = response.totalRows;
                    this.pageIndex = request.pageIndex;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.empleadoService.getEmpleadosEmpresa(this.context.app.defaultSite.identity)
                .then(response => {
                    this.context.app.hideSpinner();
                    //
                    this.empleado = response;
                    this.totalRows = response.length;
                    this.pageIndex = request.pageIndex;
                })
                .catch(error => this.handleError(error));

        }
    }

    findEmpleado(keyword: string): Observable<CatalogItem[]> {
        if (this.soloSucursal) {
            return this.empleadoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
        }
        else {
            return this.empleadoService.searchKeyword2(this.context.app.defaultSite.identity, keyword);
        }
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

