﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { EmpresaAdamInfo, EmpresaAdamItem } from '../models/empresaadam';
import { EmpresaAdamService } from '../services/empresaadam.service';

@Component({
    selector: 'rh-empresa-adam-selector',
    templateUrl: './empresa-adam-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => EmpresaAdamSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => EmpresaAdamSelectorComponent),
        multi: true
    }]
})
export class EmpresaAdamSelectorComponent extends CatalogSelector {
    empresaAdam: any[];
    selectedItem: EmpresaAdamInfo = new EmpresaAdamInfo;

    @Output() empresaAdamUpdated = new EventEmitter();

    constructor(protected context: Context, private empresaAdamService: EmpresaAdamService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findEmpresaAdam(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.empresaAdamService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + " " + info.nombre; //+ ", " + info.municipio.estado.nombre;
                    this.empresaAdamUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.empresaAdamUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.empresaAdamService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.empresaAdam = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findEmpresaAdam(keyword: string): Observable<CatalogItem[]> {
        return this.empresaAdamService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

