﻿import { Component, Input, Output, ViewChild, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
//import { RegExp } from "typescript-dotnet-commonjs/System/Text/RegularExpressions";
//import { RegExp } from "RegularExpressions/Regex"; //

//
import { Context } from 'framework3/core';
//features
import { ModalDirective } from 'ngx-bootstrap';
import { AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { TiposDescuentosInfo, DescuentosEmpleado } from '../models/tipo-descuento';
import { FichasTitService } from '../services/fichas-tit.service';
import { TipoDescuentoSelector } from './tipo-descuento-selector.component';
import { EmpresaService } from '../services/empresa.service';
import { EmpresaInfo } from '../models/empresa';
import { error } from '@angular/compiler/src/util';
import { TiposDescuentosService } from '../services/tipos-descuentos.service';
@Component({
    selector: 'rh-ficha',
    templateUrl: './ficha.component.html'
})
export class FichaComponent {
    tipoDescuento: TiposDescuentosInfo = undefined;
    empresa: EmpresaInfo = new EmpresaInfo();
    descuentosEmpleado: DescuentosEmpleado[];
    editorForm: FormGroup;
    idFichatit: string = '';
    tipoNomina: string=''
    claseNomina: number = 0;
    idEmpleado: number = 0;
    monto: number ;
    porcentaje: number ;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('dlgFicha') dlgFicha: ModalDirective;
    @Output() descuentoSelected: EventEmitter<any> = new EventEmitter<any>();

    constructor(private formBuilder: FormBuilder, private context: Context, private router: Router, private route: ActivatedRoute, private fichasTitService: FichasTitService, private empresaService: EmpresaService, private tiposDescuentoService: TiposDescuentosService) {
        this.editorForm = formBuilder.group({
            'descuento': [0, AztecaValidators.selected],
            'monto': [1, Validators.required],
            'porcentaje':[1, Validators.required]
        });
        this.empresaService.getDetailByID(this.context.app.defaultCompany.identity).then(
            info => this.empresa = info)
            .catch(error => console.log(JSON.stringify));
    }

    show() {
        this.context.app.showSpinner();
        if (this.idFichatit.length!=0) {
            this.fichasTitService.getDetail(this.idFichatit).then(info => {
                this.claseNomina = Number(info.claseNomina.trim());
                this.tipoNomina = info.tipoNomina;
                this.idEmpleado = info.idEmpleado;
                this.tiposDescuentoService.infoDescuentosEmpleadoADAM(this.idEmpleado, this.context.app.defaultSite.identity).then(descuentos => {
                    this.context.app.hideSpinner();
                    this.descuentosEmpleado = descuentos;
                    this.dlgFicha.config.backdrop = false;
                    this.dlgFicha.show();
                }).catch(error => {
                    this.context.app.hideSpinner();
                    this.showAlert(error.message,'danger');
                })
            }).catch(error => {
                this.context.app.hideSpinner();
                this.showAlert(error.message, 'danger');
            });
        }
    }

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }

    accept() {
        if (this.tipoDescuento.tipo == 'M' && this.tipoNomina.startsWith('S') && this.monto > this.empresa.montoFicha) {
            this.showAlert('El monto a descontar no debe de ser de mas de ' + this.empresa.montoFicha.toString(), 'danger');
            return;
        } else if (this.tipoDescuento.tipo == 'P' && this, this.porcentaje > this.empresa.porcentajeFicha) {
            this.showAlert('El porcentaje a descontar no debe de ser de mas de ' + this.empresa.porcentajeFicha.toString(), 'danger');
            return;
        }
        this.dlgFicha.hide();
        let object: { 'tipoDescuento': TiposDescuentosInfo, 'monto': number, 'porcentaje': number } = { 'tipoDescuento': null, 'monto': 1, 'porcentaje': 1 };
        object.monto = this.monto;
        object.porcentaje = this.porcentaje;
        object.tipoDescuento = this.tipoDescuento;
        this.descuentoSelected.emit(object);
    }

    discardChanges() {
        this.dlgFicha.hide();
    }

    onTipoDescuentoSelected(args: TiposDescuentosInfo) {
        console.log(JSON.stringify(args))
        this.tipoDescuento = args;
    }
}