﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { HorarioInfo, HorarioItem } from '../models/horario';
import { HorarioService } from '../services/horario.service';

@Component({
    selector: 'rh-horario-selector',
    templateUrl: './horario-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => HorarioSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => HorarioSelectorComponent),
        multi: true
    }]
})
export class HorarioSelectorComponent extends CatalogSelector {

    horarios: any[];
    selectedItem: HorarioInfo = new HorarioInfo();

    @Output() horarioUpdated = new EventEmitter();

    constructor(protected context: Context, private horarioSvc: HorarioService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
        .pipe(mergeMap((keyword: string) => this.findCompania(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.horarioSvc.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + ' ' + info.nombre;
                    this.horarioUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.horarioUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();
        //
        this.horarioSvc.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.horarios = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findCompania(keyword: string): Observable<CatalogItem[]> {
        return this.horarioSvc.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

