﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search';
//
import { PlazaInfo, PlazaItem } from '../models/plaza';
import { PlazaService } from '../services/plaza.service';

@Component({
    selector: 'rh-plaza-selector',
    templateUrl: './plaza-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => PlazaSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => PlazaSelectorComponent),
        multi: true
    }]
})
export class PlazaSelectorComponent extends CatalogSelector {
    plazas: any[];
    selectedItem: PlazaInfo = new PlazaInfo;

    @Output() plazaUpdated = new EventEmitter();

    constructor(protected context: Context, private plazaService: PlazaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findPlaza(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.plazaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + ' ' + info.nombre; //+ ", " + info.municipio.estado.nombre;
                    this.plazaUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.plazaUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.plazaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.plazas = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findPlaza(keyword: string): Observable<CatalogItem[]> {
        return this.plazaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

