﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search';
//
import { TiposDescuentosService } from '../services/tipos-descuentos.service';
import { TiposDescuentosInfo } from '../models/tipo-descuento';

@Component({
    selector: 'rh-tipo-descuento-selector',
    templateUrl: './tipo-descuento-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => TipoDescuentoSelector)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => TipoDescuentoSelector),
        multi: true
    }]
})
export class TipoDescuentoSelector extends CatalogSelector {
    tiposDescuentos: any[];
    selectedItem: TiposDescuentosInfo = new TiposDescuentosInfo;
    @Input() claseNomina: number = 0;

    @Output() tipoDescuentoUpdated = new EventEmitter();

    constructor(protected context: Context, private tipoDescuentoService: TiposDescuentosService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findTipo(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.tipoDescuentoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + " " + info.nombre; 
                    this.tipoDescuentoUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.tipoDescuentoUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.claseNomina != 0 && this.claseNomina!=2 && this.claseNomina!=5 && this.claseNomina!=7) {
            let condition: SearchCondition = { field: 'Tipo', operator: '=', valueType: 'string', value: 'M' };
            request.conditions.push(condition);
        }

        this.tipoDescuentoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.tiposDescuentos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findTipo(keyword: string): Observable<CatalogItem[]> {
        let tipo: string = '';
        if (this.claseNomina != 0 && this.claseNomina != 2 && this.claseNomina != 5 && this.claseNomina != 7) {
           tipo='M'
        }
        return this.tipoDescuentoService.searchKeyword2(tipo, this.context.app.defaultSite.identity, keyword);

        //return this.tipoDescuentoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

