﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search';
//
import { TipoEventoService } from '../services/tipo-evento.service';
import { TipoEventoInfo } from '../models/tipo-evento';

@Component({
    selector: 'tipo-evento-selector',
    templateUrl: './tipo-evento-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => TipoEventoSelector)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => TipoEventoSelector),
        multi: true
    }]
})
export class TipoEventoSelector extends CatalogSelector {
    tiposEventos: any[];
    selectedItem: TipoEventoInfo = new TipoEventoInfo;

    @Output() tipoIncidenciaUpdated = new EventEmitter();

    constructor(protected context: Context, private TipoEventoService: TipoEventoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findTipo(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.TipoEventoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + " " + info.nombre;
                    this.tipoIncidenciaUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.tipoIncidenciaUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();


        this.TipoEventoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.tiposEventos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findTipo(keyword: string): Observable<CatalogItem[]> {
        let tipo: string = '';

        return this.TipoEventoService.searchKeyword(this.context.app.defaultSite.identity, keyword);

    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

