﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search';
//
import { TipoIncidenciaService } from '../services/tipo-incidencia.service';
import { TipoIncidenciaInfo } from '../models/tipo-incidencia';

@Component({
    selector: 'tipo-incidencia-selector',
    templateUrl: './tipo-incidencia-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => TipoIncidenciaSelector)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => TipoIncidenciaSelector),
        multi: true
    }]
})
export class TipoIncidenciaSelector extends CatalogSelector {
    tiposIncidencia: any[];
    selectedItem: TipoIncidenciaInfo = new TipoIncidenciaInfo;
    
    @Output() tipoIncidenciaUpdated = new EventEmitter();

    constructor(protected context: Context, private tipoIncidenciaService: TipoIncidenciaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findTipo(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.tipoIncidenciaService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + " " + info.nombre;
                    this.tipoIncidenciaUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.tipoIncidenciaUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

       
        this.tipoIncidenciaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.tiposIncidencia = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findTipo(keyword: string): Observable<CatalogItem[]> {
        let tipo: string = '';
        
        return this.tipoIncidenciaService.searchKeyword(this.context.app.defaultSite.identity, keyword);

    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

