﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { TipoPerDedInfo, TipoPerDedItem } from '../models/tipo-per-ded';
import { TipoPerDedService } from '../services/tipo-per-ded.service';

@Component({
    selector: 'rh-tipo-per-ded-selector',
    templateUrl: './tipos-per-ded-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => TiposPerDedSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => TiposPerDedSelectorComponent),
        multi: true
    }]
})
export class TiposPerDedSelectorComponent extends CatalogSelector {
    tipoPerDed: any[];
    selectedItem: TipoPerDedInfo = new TipoPerDedInfo;

    
    @Output() tipoPerDedUpdated = new EventEmitter();
    @Input() tipoPerDedSelector: string = '';

    constructor(protected context: Context, private tipoPerDedService: TipoPerDedService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        }).pipe(mergeMap((keyword: string) => this.findTipoPerDed(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.tipoPerDedService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + ' ' + info.descripcion;
                    this.tipoPerDedUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.tipoPerDedUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        if (this.tipoPerDedSelector != '') {
            var exist = false;
            request.conditions.forEach(r => {
                if (!exist && r.field == "tipoTransaccion") {
                    exist = true;
                    r.value = this.tipoPerDedSelector;
                }
            });

            if (!exist) {
                let condition = { field: "tipoTransaccion", operator: "=", valueType: "string", value: this.tipoPerDedSelector };
                request.conditions.push(condition);
            }
        }

        this.tipoPerDedService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.tipoPerDed = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findTipoPerDed(keyword: string): Observable<CatalogItem[]> {
        return this.tipoPerDedService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

