﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index'; //'../../search/index';
//
import { TurnoInfo, TurnoItem } from '../models/turno';
import { TurnoService } from '../services/turno.service';

@Component({
    selector: 'rh-turno-selector',
    templateUrl: './turno-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => TurnoSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => TurnoSelectorComponent),
        multi: true
    }]
})
export class TurnoSelectorComponent extends CatalogSelector {
    turno: any[];
    selectedItem: TurnoInfo = new TurnoInfo;


    @Output() empleadoUpdated = new EventEmitter();

    constructor(protected context: Context, private turnoService: TurnoService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findTurno(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.turnoService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.code + ' ' + info.nombre;
                    this.empleadoUpdated.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.empleadoUpdated.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.turnoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.turno = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findTurno(keyword: string): Observable<CatalogItem[]> {
        return this.turnoService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

