﻿import { Component, EventEmitter, Input, Output, ViewChild, forwardRef } from '@angular/core';
//
import { Context } from 'framework3/core';
import { CatalogListDialog, SearchRequest } from 'framework3/search';

import { ChecadorService } from '../services/checador.service';
import { ChecadorItem } from '../models/checador';

//

@Component({
    selector: 'rh-checador-list',
    templateUrl: './checadores-list.component.html'
})
export class ChecadorListComponent extends CatalogListDialog {
    items: ChecadorItem[];

    @Input() excludedItems: ChecadorItem[] = [];
    @Input() titleDialog: string = 'Checadores';
    @Input() validarPermisos: boolean = true;

    constructor(protected context: Context, private service: ChecadorService) {
        super(context);
    }

    onSearchRequested(request: SearchRequest) {

        if (this.validarPermisos == true)
            this.service.searchChecador(this.context.session.user.identity, this.context.app.defaultSite.identity, request)
                .then(result => {
                    if (this.excludedItems.length > 0) {
                        result.rows = result.rows.filter(fitem => !this.excludedItems.find(sitem => sitem.id == fitem.id));
                    }

                    this.items = result.rows;
                    this.totalRows = result.totalRows;
                    this.pageIndex = request.pageIndex;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
            
         else
            this.service.search(this.context.app.defaultSite.identity, request)
                .then(result => {
                    if (this.excludedItems.length > 0) {
                        result.rows = result.rows.filter(fitem => !this.excludedItems.find(sitem => sitem.id == fitem.id));
                    }

                    this.items = result.rows;
                    this.totalRows = result.totalRows;
                    this.pageIndex = request.pageIndex;
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));

    }

    onSelectedItemsChanged(selectedItem: ChecadorItem[]) {
        this.selectedItemsChanged.emit(selectedItem);
    }

    onSelectedItemChanged(item: ChecadorItem) {
        this.selectedItemChanged.emit(item);
    }

}