﻿import { Component, EventEmitter, Input, Output, ViewChild, forwardRef } from '@angular/core';
//
import { Context } from 'framework3/core';
import { CatalogListDialog, SearchRequest, SearchCondition } from 'framework3/search';

import { DocumentoTipoService } from '../services/documento-tipo.service';
import { DocumentoTipoItem } from '../models/documento-tipo';
//

@Component({
    selector: 'rh-documentos-tipo-list',
    templateUrl: './documentos-tipo-list.component.html'
})
export class DocumentoTipoListComponent extends CatalogListDialog {
    items: DocumentoTipoItem[];

    @Input() titleDialog: string = 'RH - Tipo de Documentos';
    @Input() proceso: string = '';

    constructor(protected context: Context, private service: DocumentoTipoService) {
        super(context);
    }

    onSearchRequested(request: SearchRequest) {

        let condition: SearchCondition = { field: 'Proceso', operator: '=', valueType: 'string', value: this.proceso };
        request.conditions.push(condition);

        this.service.search(this.context.app.defaultSite.identity, request)
            .then(result => {
                this.items = result.rows;
                this.totalRows = result.totalRows;
                this.pageIndex = request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedItemsChanged(selectedItem: DocumentoTipoItem[]) {
        this.selectedItemsChanged.emit(selectedItem);
    }

    onSelectedItemChanged(item: DocumentoTipoItem) {
        this.selectedItemChanged.emit(item);
    }

}