﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo, ReporteInfo, ParametroInfo, ReportService } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, ContentViewerComponent } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { FichasTitInfo, FichasTitItem } from '../models/fichas-tit';
import { EmpleadoInfo} from '../models/empleado';
import { FichasTitDetail } from '../models/fichas-tit-detail';
import { FichasTitService } from '../services/fichas-tit.service';
import { FichasTitDetailEditor } from './parts/fichas-tit-detail.editor';
import { TiposDescuentosInfo } from '../models/tipo-descuento';
import { AdamNominaInfo } from '../models/adamnomina';
import { detachEmbeddedView } from '@angular/core/src/view';
import { FichaComponent } from '../components/ficha.component';
import { PolizaPerfilListComponent, PolizaInfo, PolizaPerfilItem, PolizaService } from '../../accounting';
import { ModalDirective } from 'ngx-bootstrap';

@Component({
    selector: 'rh-fichas-tit-editor',
    templateUrl: './fichas-tit.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => FichasTitEditor)
        },
        {
            provide: DocumentService, useExisting: FichasTitService
        }
    ]
})
export class FichasTitEditor extends DocumentEditor {
    fichasTit = new FichasTitInfo();
    currentPoliza: PolizaInfo = new PolizaInfo();
    adamNomina: AdamNominaInfo = new AdamNominaInfo();
    acumulado: number = 0;
    total: number = 0;
    argsCurrentDocument: WorkflowEventArgs;    
    titleContentView: string = "";
    fileNameContentView: string = "";
    message = "";
    tipos: any[] = [{ 'value': 'N', 'text': 'Descuento Por Nomina' }, { 'value': 'C', 'text': 'Fecha de Pago' }];
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: FichasTitDetailEditor;
    @ViewChild('dlgFicha') dlgFicha: FichaComponent;

    @ViewChild(PolizaPerfilListComponent) dlgPolizaPerfil: PolizaPerfilListComponent;
    @ViewChild('viewDialog') dlgPolizaViewer: ModalDirective;
    @ViewChild('viewType') dlgFichaCargo: ModalDirective;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private fichasTitService: FichasTitService,
        private polizaSvc: PolizaService,
        private reportSvc: ReportService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idEmpleado': [0, AztecaValidators.selected],
            'importe': [0, Validators.required],
            'total': [0],
            'cargo': [0, Validators.required],
            'descripcion': ['', Validators.required],
            'tipo': ['',Validators.required],
            'fechaPago':[''],
            'acumulado': [0]
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();
        this.fichasTitService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.fichasTit = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.fichasTitService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.fichasTit = doc;
                this.detailsEditor.clientIndexStart = this.fichasTit.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.fichasTitService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.fichasTit = doc;
                this.detailsEditor.clientIndexStart = this.fichasTit.details.length;
                if (this.fichasTit.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
                this.acumulado = this.fichasTit.details.map(c => c.importe)
                    .reduce((sum, current) => sum + current);
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        if (this.fichasTit.totalFicha >= this.fichasTit.totalCargo) {
            if (this.total == this.acumulado) {
                this.context.app.showSpinner();
                this.fichasTitService.update(this.fichasTit)
                    .then(updatedDoc => {
                        this.context.app.hideSpinner();
                        //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                        this.reset();
                        if (!args.closeEditor && this.fichasTit.identity == '') {
                            this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                        }
                        else {
                            if (args.closeEditor)
                                this.router.navigate(['../'], { relativeTo: this.route });
                            else
                                this.showAlert('La información ha sido actualizada con éxito', 'success');
                        }
                        this.fichasTit = updatedDoc;
                        this.acumulado = this.fichasTit.details.map(c => c.importe)
                            .reduce((sum, current) => sum + current);
                    })
                    .catch(error => this.handleError(error));
            }
            else {
                this.showAlert('Existe una diferencia entre el total y el caumulado', 'danger');
            }
        } else {
            this.showAlert('El total de la ficha debe de ser mayor o igual al total del cargo', 'danger');
        }
        
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.argsCurrentDocument = args;
        args.step.activities.forEach(activity => {
            if (activity.tag == 'APPLY') {
                if (this.fichasTit.tipo == 'C') {
                    this.message = 'Esta Ficha no se Descuenta Via Nomina Fecha de Pago Compromiso ' + this.fichasTit.fechaPago.toDateString();
                    this.dlgFichaCargo.config.backdrop = false;
                    this.dlgFichaCargo.show();
                } else {
                    this.dlgFicha.idFichatit = args.document.id;
                    this.dlgFicha.show();
                }
            }
            else if (activity.tag == 'AUTHORIZE') {
                this.context.app.showSpinner();
                this.fichasTitService.getCuentaEmpleado(args.document.id).then(
                    idCuenta => {
                        this.context.app.hideSpinner();
                        if (idCuenta > 0) {
                            this.fichasTitService.armarCuentas(args.document.id).then(
                                a => {
                                    this.dlgPolizaPerfil.show();
                                }
                            ).catch(error => {
                                this.context.app.hideSpinner();
                                this.handleError(error);
                            }); 
                        }
                        else
                            this.showAlert('El empleado no tiene cuenta asignada', 'danger');
                    }).catch(error => {
                        this.context.app.hideSpinner();
                        this.handleError(error);
                    });                
            }
        });
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.fichasTitService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.fichasTit.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    onTotalChanged(args: any) {
        this.total = args;
        this.documentForm.updateValueAndValidity();
    }

    onEmpleadoSelected(args: EmpleadoInfo) {
        if (args != undefined && args != null) {
            this.context.app.showSpinner();
            this.fichasTitService.loadInfoEmpleadoAdam(this.context.app.defaultSite.identity, args.code)
                .then(info => {
                    this.context.app.hideSpinner();
                    if (info[0] != undefined) {
                        this.fichasTit.claseNomina = info[0].clase_Nomina;
                        this.fichasTit.compania = info[0].compania;
                        this.fichasTit.ejercicio = info[0].ejercicio;
                        this.fichasTit.periodoDescontar = info[0].periodo;
                        this.fichasTit.tipoNomina = info[0].tipo_Nomina;
                    }
                })
                .catch(error => this.handleError(error));
        }
    }
    onMenuItemClick(option: MenuItem) {
        if (option.name == 'DOCUEMNTS') {
            this.router.navigate(['rh/documento-empleado', this.process.id, this.fichasTit.idEmpleado], { queryParams: { process: this.process.codigo } });
        }
        else if (option.name == 'PDF') {
            if (this.fichasTit.identity.length>0) {
                this.context.app.showSpinner();
                let informe = new ReporteInfo();
                let parametroFicha = new ParametroInfo();
                let parametroSucursal = new ParametroInfo();
                parametroFicha.name = "IdFichaTit";
                parametroFicha.values.push(this.fichasTit.identity);
                parametroSucursal.name = "IdSucursal"
                parametroSucursal.values.push(this.context.app.defaultSite.identity.toString());
                informe.parametros.push(parametroFicha);
                informe.parametros.push(parametroSucursal);
                informe.path = "/Onili/RH/FichaCargo";
                informe.format = "pdf";
                this.reportSvc.loadReporte(informe)
                    .then(info => {
                        this.context.app.hideSpinner();
                        this.titleContentView = "Ficha " + this.fichasTit.identity;
                        this.fileNameContentView = this.fichasTit.identity + ".pdf";
                        this.reportViewer.show("data:application/pdf; base64, " + info.content);
                    })
                    .catch(error => {
                        this.context.app.hideSpinner();
                        this.handleError(error);
                    });
            }

        }
    }

    validateMontos() {
        if (this.acumulado == this.total)
            return null;
        else
            return 'No coincide el total de la ficha con los conceptos capturados';
    }

    validateMontos2() {
        if (this.acumulado == this.total)
            return null;
        else
            return 'No coincide el total de la ficha con los conceptos capturados';
    }

    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        if (this.fichasTit.details.filter(detail => detail.state != 'DELETED').length > 0) {
            this.showAlert('No se puede agregar mas de un concepto a la ficha', 'danger');
        } else {
            this.detailsEditor.detail = new FichasTitDetail();
            this.detailsEditor.show();
        }
    }

    onEditingDetail(detail: FichasTitDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: FichasTitDetail) {
        this.acumulado = this.fichasTit.details.filter(detail => detail.state != 'DELETED')
                                                    .map(c => c.importe)
                                                        .reduce((sum, current) => sum + current);
        this.total = this.fichasTit.totalCargo;
        this.detailsList.deleteItem(detail);
        this.documentForm.updateValueAndValidity();
    }

    onListItemsUpdated(updatedDetails: FichasTitDetail[]) {
        this.fichasTit.details = updatedDetails;
        this.total = this.fichasTit.totalCargo;
        this.acumulado = this.fichasTit.details.filter(detail => detail.state != 'DELETED')
            .map(c => c.importe)
            .reduce((sum, current) => sum + current);
        this.documentForm.updateValueAndValidity();
    }

    onSavingDetail(detail: FichasTitDetail) {
        if (this.fichasTit.details.length == 0)
            this.detailsList.updateItem(detail);
        else
            this.fichasTit.details.forEach(det => {
                if (det.concepto.identity == detail.concepto.identity && det.state != 'DELETED' && detail.state != 'MODIFIED')
                    this.showAlert("El concepto ya fue agregado", "información");
                else
                    this.detailsList.updateItem(detail);
            });
    }

    onTipoDescuentoSelected(args: TiposDescuentosInfo) {
        
    }

    onDescuentoSelected(args: any) {
        this.context.app.showSpinner();
        this.fichasTitService.nextStep(this.editor.document.identity, this.argsCurrentDocument.step.idWorkflowStep, args)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(this.editor.document.identity);
            })
            .catch(error => this.handleError(error));
    }

    onSelectedPolizaPerfil(item: PolizaPerfilItem) {
        this.context.app.showSpinner();
        this.polizaSvc.loadPolizaByDocumento(this.editor.document.identity, item.id, this.context.app.defaultSite.identity)
            .then(result => {
                this.currentPoliza = result;
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.config.backdrop = false;
                this.dlgPolizaViewer.show();
            }).catch(error => this.handleError(error));
    }

    onClickGuardar() {
        this.dlgPolizaViewer.hide();
        this.context.app.showSpinner();
        if (this.currentPoliza.idPolizaOrigen == 0) this.currentPoliza.idPolizaOrigen = 1 //Manual

        this.fichasTitService.nextStep(this.argsCurrentDocument.document.id, this.argsCurrentDocument.step.idWorkflowStep, this.currentPoliza)
            .then(_ => {
                this.dlgPolizaViewer.hide();
                //Volver a cargar documento
                this.onViewingDocument(this.editor.document.identity);
            })
            .catch(error => this.handleError(error));
    }

    onAceptFicha() {
        this.context.app.showSpinner();
        this.fichasTitService.nextStep(this.argsCurrentDocument.document.id, this.argsCurrentDocument.step.idWorkflowStep, null)
            .then(_ => {
                this.dlgFichaCargo.hide();
                //Volver a cargar documento
                this.onViewingDocument(this.editor.document.identity);
            })
            .catch(error => {
                this.dlgFichaCargo.hide();
                this.handleError(error)
            });
    }

}

