﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Locales
import { NominaInfo, NominaItem } from '../models/nomina';
import { NominaDetail } from '../models/nomina-detail';
import { NominaService } from '../services/nomina.service';

@Component({
    selector: 'rh-nominas-manager',
    templateUrl: './nominas.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => NominasManager)
        },
        {
            provide: DocumentService, useExisting: NominaService
        }
    ]
})
export class NominasManager extends DocumentManager {
    nominas: NominaItem[];

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private nominaService: NominaService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.nominaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.nominas = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.nominaService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.nominaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.nominaService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'CALCULAR') {
            this.context.app.showSpinner();
            this.nominaService.calcularNomina(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    this.showAlert("Se ha creado la tarea de cálculo", "success");
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == 'TIMBRAR') {
            this.context.app.showSpinner();
            this.nominaService.timbrarNomina(this.manager.currentDocument.id)
                .then(file => {
                    this.context.app.hideSpinner();
                    this.showAlert("Se ha creado la tarea de timbrado", "success");
                })
                .catch(error => this.handleError(error));
        }
    }

}

