﻿import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
//Locales
import { FichasTitDetail } from '../../models/fichas-tit-detail';
import { AztecaValidators } from 'framework3/controls';
import { ConceptosFichasInfo } from '../../models/concepto-ficha';

@Component({
    selector: 'rh-fichas-tit-detail-editor',
    templateUrl: './fichas-tit-detail.editor.html'
})
export class FichasTitDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new FichasTitDetail();
    //Objeto original que se editará
    @Input() detail: FichasTitDetail = new FichasTitDetail();

    constructor(protected context: Context, private builder: FormBuilder) {
        super(context);

        this.form = builder.group({
            'concepto': ['', AztecaValidators.selected],
            'importe':[0, Validators.required]
        });
    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
        this.form.reset({ concepto: '' });
        this.form.reset({ importe: 0 });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: FichasTitDetail ) {
        this.detailCopy = <FichasTitDetail>this.copyItem(this.detail);
    }

    onUpdatingItem(detail: FichasTitDetail ) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
    }

    onConceptoUpdate(args: ConceptosFichasInfo) {
        this.detailCopy.concepto = args;
    }
}
