﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, Globalize } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';

//moment
import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs, TrackableCollection } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { ReciboNominaInfo, ReciboNominaItem } from '../models/recibo-nomina';
import { ReciboNominaDetail } from '../models/recibo-nomina-detail';
import { ReciboNominaService } from '../services/recibo-nomina.service';
import { ReciboNominaDetailEditor } from './parts/recibos-nominas-detail.editor';
import { TiposPerDedListComponent } from '../controls/tipos-per-ded-list.component';
import { TipoPerDedItem } from '../models/tipo-per-ded';
import { EmpleadoInfo, EmpleadoItem } from '../models/empleado';
import { EmpleadoService } from '../services/empleado.service';
import { CompaniaService } from '../services/compania.service';
import { EmpresaService } from '../../accounting/services/empresa.service';
import { TipoRegimenService } from '../../accounting/services/tipo-regimen.service';
import { RegimenFiscalItem } from '../../accounting/models/empresa';
import { TipoRegimenItem } from '../../accounting/models/tipo-regimen';

import { TipoContratoItem } from '../../accounting/models/tipo-contrato';
import { TipoContratoService } from '../../accounting/services/tipo-contrato.service';
import { JornadaItem } from '../../accounting/models/jornada';
import { JornadaService } from '../../accounting/services/jornada.service';
import { RiesgoPuestoItem } from '../../accounting/models/riesgo-puesto';
import { RiesgoPuestoService } from '../../accounting/services/riesgo-puesto.service';
import { PeriodoPagoItem } from '../../accounting/models/periodo-pago';
import { PeriodoPagoService } from '../../accounting/services/periodo-pago.service';

@Component({
    selector: 'rh-recibos-nominas-editor',
    templateUrl: './recibos-nominas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => RecibosNominasEditor)
        },
        {
            provide: DocumentService, useExisting: ReciboNominaService
        }
    ]
})
export class RecibosNominasEditor extends DocumentEditor implements OnInit {
    reciboNomina = new ReciboNominaInfo();
    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: ReciboNominaDetailEditor;
    @ViewChild(TiposPerDedListComponent) tipoPerDedList: TiposPerDedListComponent;

    menuDetailItems: MenuItem[] = [];
    empledao: EmpleadoInfo;
    regimenesFiscales: RegimenFiscalItem[] = [];
    tiposRegimen: TipoRegimenItem[] = [];
    tiposContrato: TipoContratoItem[] = [];
    jornadas: JornadaItem[] = [];
    riesgosPuesto: RiesgoPuestoItem[] = [];
    periodosPago: PeriodoPagoItem[] = [];

    tiposNominas = [
        { 'tipo': 'O', 'nombre': 'Ordinaria' },
        { 'tipo': 'E', 'nombre': 'Extraordinaria' }
    ];

    get subtotal(): string {
        var total = 0;
        this.reciboNomina.details.forEach(item => {
            if (item.idPercepcion > 0) {
                total += item.gravado + item.exento;
            }
            else if (item.idOtroPago > 0) {
                total += item.importe;
            }
        });
        return Globalize.formatNumber(total, 'c');
    }

    get descuentos(): string {
        var total = 0;
        this.reciboNomina.details.forEach(item => {
            if (item.idDeducccion > 0) {
                total += item.importe;
            }
        });
        return Globalize.formatNumber(total, 'c');
    }

    get total() {
        return Globalize.formatNumber((Globalize.parseFloat(this.subtotal, 'c') - Globalize.parseFloat(this.descuentos, 'c')), 'c');
    }

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private reciboNominaService: ReciboNominaService,
        private empleadoSvc: EmpleadoService,
        private empresaSvc: EmpresaService,
        private companiaSvc: CompaniaService,
        private tipoRegimenSvc: TipoRegimenService,
        private tipoContratoSvc: TipoContratoService,
        private jornadaSvc: JornadaService,
        private riesgoPuestoSvc: RiesgoPuestoService,
        private periodoPagoSvc: PeriodoPagoService
    ) {
        super(context, router, route, documentTypeService);
        this.documentForm = formBuilder.group({
            'idEmpleado': [null, AztecaValidators.selected],
            'tipoNomina': ['', Validators.required],
            'numDiasPago': [0, Validators.required],
            'fechaPago': [''],
            'fechaInicialPago': [''],
            'fechaFinalPago': [''],
            'subtotal': [''],
            'descuentos': [''],
            'total': [''],
            'rfc': ['', Validators.required],
            'nombre': ['', Validators.required],
            'regimenFiscal': ['', Validators.required],
            'curp': [''],
            'registroPatronal': ['', Validators.required],
            'fechaInicioRelLaboral': [''],
            'sindicalizado': [''],
            'antiguedad': [''],
            'tipoContrato': [''],
            'tipoJornada': [''],
            'tipoRegimen': [''],
            'numEmpleado': ['', Validators.required],
            'departamento': [''],
            'puesto': [''],
            'riesgoPuesto': [''],
            'periodicidadPago': [''],
            'banco': [''],
            'cuentaBancaria': [''],
            'claveEntFed': [''],
            'salarioBaseCotApor': [''],
            'salarioDiarioIntegrado': ['']
        });
    }

    ngOnInit(): void {
        this.createMenuDetail();
        setTimeout(() => {
            this.loadRegimenesFiscales();
            this.loadTiposRegimen();
            this.loadJornadas();
            this.loadRiesgosPuesto();
            this.loadPeriodosPago();
            this.loadTiposContrato();
        });
    }

    createMenuDetail() {

        let mnuPercepcion: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Percepcion", name: "PERCEPCION", smallText: "Percepcion" };
        let mnuDeduccion: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Deduccion", name: "DEDUCCION", smallText: "Deduccion" };
        let mnuOtroPago: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Otro pago", name: "OTRO_PAGO", smallText: "Otro pago" };
        let mnuHoraExtra: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Hora extra", name: "HORAS_EXTRA", smallText: "Hora extra" };
        let mnuHoraIncapacidad: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Incapacidad", name: "INCAPCIDAD", smallText: "Incapacidad" };
        this.menuDetailItems.push(mnuPercepcion, mnuDeduccion, mnuOtroPago, mnuHoraExtra, mnuHoraIncapacidad);

        this.menuDetailItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {
        this.context.app.showSpinner();
        this.reciboNominaService.create()
            .then(doc => {
                this.reciboNomina = doc;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.reciboNominaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.reciboNomina = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.reciboNominaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.reciboNomina = doc;
                this.detailsEditor.clientIndexStart = this.reciboNomina.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.reciboNominaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.reciboNomina = doc;
                this.detailsEditor.clientIndexStart = this.reciboNomina.details.length;
                if (this.reciboNomina.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.reciboNominaService.update(this.reciboNomina)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.reciboNomina.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.reciboNomina = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.reciboNominaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.reciboNominaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.reciboNomina.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    loadRegimenesFiscales() {
        this.empresaSvc.loadRegimenesFiscales(this.context.app.defaultCompany.identity)
            .then(regimenes => {
                this.regimenesFiscales = regimenes
                let item: RegimenFiscalItem = new RegimenFiscalItem();
                item.id = 0;
                item.nombre = "[NINGUNO]";
                item.activo = true;
                item.codigo = '';
                this.regimenesFiscales.unshift(item);
            })
            .catch(error => this.handleError(error));
    }

    loadTiposRegimen() {
        this.tipoRegimenSvc.loadCbo()
            .then(info => {
                this.tiposRegimen = info;
            })
            .catch(e => this.handleError(e));
    }

    loadJornadas() {
        this.jornadaSvc.getJornadaCbo(this.context.app.defaultSite.identity)
            .then(info => {
                this.jornadas = info;
            })
            .catch(e => this.handleError(e));
    }

    loadRiesgosPuesto() {
        this.riesgoPuestoSvc.getRiesgoPuestoCbo(this.context.app.defaultSite.identity)
            .then(info => {
                this.riesgosPuesto = info;
            })
            .catch(e => this.handleError(e));
    }

    loadPeriodosPago() {
        this.periodoPagoSvc.loadCbo()
            .then(info => {
                this.periodosPago = info;
            })
            .catch(e => this.handleError(e));
    }

    loadTiposContrato() {
        this.tipoContratoSvc.loadCbo()
            .then(info => {
                this.tiposContrato = info;
            })
            .catch(e => this.handleError(e));
    }

    ///////////// GENERAL EDITOR /////////////////////////

    onEmpleadoUpdated(info: EmpleadoInfo) {
        this.empledao = info;
        // Emisor
        if (this.empledao.idCompania > 0) {
            this.reciboNomina.emisor.registroPatronal = this.empledao.compania.registroPatronal;
            this.reciboNomina.emisor.rfc = this.empledao.compania.rfcEmisor;
            this.reciboNomina.emisor.nombre = this.empledao.compania.razonSocial;
            this.reciboNomina.emisor.regimenFiscal = this.empledao.compania.regimenFiscal;
        }
        // Receptor
        this.reciboNomina.receptor.fechaInicioRelLaboral = this.empledao.fechaIngreso;
        this.reciboNomina.receptor.numEmpleado = this.empledao.code;
        this.reciboNomina.receptor.tipoRegimen = this.empledao.idTipoRegimen.toString();
        this.reciboNomina.receptor.departamento = this.empledao.departamento;
        this.reciboNomina.receptor.puesto = this.empledao.puesto;
        this.reciboNomina.receptor.riesgoPuesto = this.empledao.riesgoPuesto;
        this.reciboNomina.receptor.periodicidadPago = this.empledao.periodicidadPago;
        this.reciboNomina.receptor.claveEntFed = this.empledao.claveEntFed;
        this.reciboNomina.receptor.tipoContrato = this.empledao.claveTipoContrato;
    }

    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onDeletingDetail(detail: ReciboNominaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: ReciboNominaDetail[]) {
        this.reciboNomina.details = updatedDetails;
    }

    onSavingDetail(detail: ReciboNominaDetail) {
        this.detailsList.updateItem(detail);
    }

    onMenuDetailItemClicked(menu: Menu, args: any) {
        let option = this.menuDetailItems[menu.selectedIndex];
        if (option.name == 'PERCEPCION') {
            this.tipoPerDedList.tipoPerDedSelector = "Percepciones";
            this.tipoPerDedList.titleDialog = "Percepciones";
        }
        else if (option.name == 'DEDUCCION') {
            this.tipoPerDedList.tipoPerDedSelector = "Deducciones";
            this.tipoPerDedList.titleDialog = "Deducciones";
        }
        else if (option.name == 'OTRO_PAGO') {
            this.tipoPerDedList.tipoPerDedSelector = "Otros Pagos";
            this.tipoPerDedList.titleDialog = "Otros Pagos";
        }
        else if (option.name == 'INCAPCIDAD') {
            this.tipoPerDedList.tipoPerDedSelector = "Incapacidades";
        }

        if (option.name != 'HORAS_EXTRA') {
            this.tipoPerDedList.show();
        }
    }

    onSelectedTiposPerDedChanged(args: TipoPerDedItem[]) {
        args.forEach(item => {
            let detail: ReciboNominaDetail = new ReciboNominaDetail();
            detail._clientID = this.createClientID();
            detail.tipo = item.codigo;
            detail.concepto = item.descripcion;
            if (item.tipoTransaccion == "Percepciones") {
                detail.idPercepcion = item.id;
            }
            else if (item.tipoTransaccion == "Deducciones") {
                detail.idDeducccion = item.id;
            }
            else if (item.tipoTransaccion == "Otros Pagos") {
                detail.idOtroPago = item.id;
            }
            else if (item.tipoTransaccion == "Incapacidades") {
                detail.idIncapacidad = item.id;
            }
            detail.gravado = 0;
            detail.exento = 0;
            detail.importe = 0;
            this.detailsList.updateItem(detail);
        });
    }

    createClientID(): number {
        return parseInt(moment().format("HHmmss"));
    }
}

