﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, ProcessComponent } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { CatalogEditor, EditorEventArgs, TrackableCollection, ChangeState } from 'framework3/editors';
//Features
import { PolizasNominaService } from '../services/polizas-nomina.service';
import { AdamNominaInfo } from '../models/adamnomina';
import { AdamNominaComponent } from '../components/adam-nomina.component';
import { stat } from 'fs';

@Component({
    selector: 'polizas-nomina',
    templateUrl: './polizas-nomina.html'
})

export class PolizasNomina {
    editorForm: FormGroup;
    adamNomina: AdamNominaInfo = new AdamNominaInfo();
    numeroLiquidacion: number = 0;
    liquidacion: boolean = false;
    cuentasCollection: TrackableCollection = new TrackableCollection();        
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private polizasNominaService: PolizasNominaService, private context: Context) {
        this.editorForm = formBuilder.group({
            'adamNomina': [''],
            'numeroLiquidacion': [0],
            'liquidacion': [false]
        });
    }

    onInterfasClick() {
        this.context.app.showSpinner();
        var tipoPoliza: string;
        if (this.liquidacion) {
            tipoPoliza = 'F';
        }
        else {
            tipoPoliza = 'N';
        }
        this.polizasNominaService.importarPolizaAdam(this.numeroLiquidacion, tipoPoliza, this.adamNomina)
            .then(status => {
                this.context.app.hideSpinner();
                if (status)
                    this.alerts.showAlert('Se importo con exito', 'success');
                else
                    this.alerts.showAlert('Hubo un problema al importar', 'danger');
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error);
            });
    }

    onListarCuentasClick() {
        this.context.app.showSpinner();
        this.polizasNominaService.loadCuentasFaltantes(this.adamNomina.idEmpresaAdam, this.adamNomina.compania, this.adamNomina.claseNomina, this.adamNomina.tipoNomina, this.adamNomina.anio, this.adamNomina.periodo)
            .then(lista => {
                this.context.app.hideSpinner();
                this.cuentasCollection.sourceCollection = lista;
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error)
            });
    }

    handleError(error) {
        console.log(error);
        this.alerts.showAlert(error.error.message, 'danger');
    }
}