﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'
import { ProcessComponent } from 'framework3/core';

import * as moment from 'moment';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';

//Features
import { FiniquitoAutorizacionService } from '../services/finiquito-autorizacion.service'

@Component({
    selector: 'autorizacion-finiquito',
    templateUrl: './autorizacion-finiquito.maintenance.html'
})
export class AutorizacionFiniquitoMaintenance extends ProcessComponent {
    editorForm: FormGroup;

    request: SearchRequest;
    currentItems: Array<any> = new Array<any>();

    currentItem: any;
    totalRows: number = 0;
    pageIndex: number = 0;
    menuItems: MenuItem[] = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(public context: Context, private formBuilder: FormBuilder, router: Router, route: ActivatedRoute, private finiquitoAutorizacionService: FiniquitoAutorizacionService) {
        super(context, router, route);
        this.editorForm = formBuilder.group({
            '':[]
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.refreshList();
        });
        this.createMenu();
        this.refreshList();
    }

    createMenu() {
        let mnuEditar: MenuItem = { icon: "glyphicon glyphicon-repeat", text: "Editar", name: "EDITAR", smallText: "Editar autorizaci&oacute;n" };
        let mnuEliminar: MenuItem = { icon: "glyphicon glyphicon-remove-alt", text: "Eliminar", name: "ELIMINAR", smallText: "Eliminar autorizaci&oacute;n" };
        this.menuItems.push(mnuEditar, mnuEliminar);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    refreshList() {
        this.context.app.showSpinner();

        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }
        this.finiquitoAutorizacionService.searchFiniquitAutorizacion(this.context.app.defaultSite.identity, this.request)
            .then(response => {
                this.currentItems = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = this.request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }


    onCurrentItemChanged(item: any) {
        this.currentItem = item;
    }

    searchRequested(request: SearchRequest) {
        this.request = request;
        this.refreshList();
    }

    optionClick(option: MenuItem) {
        if (option.name == "EDITAR") {
            this.router.navigate(['rh/finiquito-autorizacion/autorizacion', this.process.id, this.currentItem.data.idFiniquitoAutorizacion]);
        }
        else if (option.name == "ELIMINAR") {
            this.finiquitoAutorizacionService.delete(this.currentItem.data.idFiniquitoAutorizacion)
                .then(response => {
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
    }

    showAlert(message: string, type: string) {
        this.context.app.hideSpinner();
        this.alerts.showAlert(message, type);
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onAgregar() {
        this.router.navigate(['rh/finiquito-autorizacion/autorizacion', this.process.id, 0])    
    }

}