﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { Menu } from "wijmo/wijmo.input";
import { FormGroup, FormBuilder, Validators } from '@angular/forms';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';

//wijmo
import { FlexGrid, CellRangeEventArgs, DataMap } from 'wijmo/wijmo.grid';
import * as wjcCore from 'wijmo/wijmo';

//frmework
import { AlertsBarComponent, MenuItem } from "framework3/controls";
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { Context, BaseComponent, ProcessComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';

//Controls
import { EmpleadoListComponent } from '../controls/empleado-list.component';
import { ChecadorListComponent } from '../controls/checadores-list.component';

//Features
import { ChecadorUsuarioIten,ChecadorItem, ChecadorInfo } from '../models/checador';
import { ChecadorService } from '../services/checador.service';


@Component({
    selector: 'rh-checador-usaurio',
    templateUrl: './checador-usuario.maintenance.html'
})
export class ChecadorUsuarioMaintenance extends ProcessComponent {

    formUsuario: FormGroup;
    formPasswd: FormGroup;
    labelChecador = "Administrador de empleados en checador";
    identity: number;
    usuario: ChecadorUsuarioIten = new ChecadorUsuarioIten();
    userContrasena: string = "";

    cvUsuarios: wjcCore.CollectionView = new wjcCore.CollectionView();
    tipoUsuarioMap: DataMap;
    items: ChecadorUsuarioIten[] = [];
    widthClass: string[] = [];
    mnuEmpleadoItems: MenuItem[] = [];

    checadoresExclude: ChecadorItem[];

    //
    @ViewChild(EmpleadoListComponent) selEmpleado: EmpleadoListComponent;
    @ViewChild(ChecadorListComponent) checadoresList: ChecadorListComponent;

    tipoUsuario = [
        { 'value': 1, 'name': 'Grabador' },
        { 'value': 2, 'name': 'Administrador' },
        { 'value': 3, 'name': 'Supervisor' },
        { 'value': 0, 'name': 'Normal' }
    ];

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;
    @ViewChild('dlgUsuario') dlgUsuario: ModalDirective;
    @ViewChild('dlgPasswd') dlgPasswd: ModalDirective;
    @ViewChild('dlgConfirmacion') dlgConfirmacion: ModalDirective;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private checadorSvc: ChecadorService) {
        super(context, router, route);
        this.tipoUsuarioMap = new DataMap(this.tipoUsuario, 'value', 'name');
        this.formUsuario = this.formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'tipoUsuario': [0],
            'contrasena': ['']
        });

        this.formPasswd = this.formBuilder.group({
            'userContrasena': ['', Validators.required],
        });
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.route.params.forEach((params: Params) => {
                this.identity = +params['id'];
            });
            this.onLoadingItem(this.identity);
            this.createMenuDetail();
        });
    }

    createMenuDetail() {
        let mnuAsignar: MenuItem = { icon: "glyphicon glyphicon-user", text: "Asignar empleado", name: "ALLOCATE", smallText: "asignar empleado" };
        let mnuImportar: MenuItem = { icon: "glyphicon glyphicon-download-alt", text: "Importar usuario seleccionado", name: "IMPORT_USR", smallText: "importar usuario" };
        let mnuNuevo: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Importar usuarios nuevos", name: "IMPORT_NEW", smallText: "importar nuevos" };
        let mnuImportHuella: MenuItem = { icon: "glyphicon glyphicon-download", text: "Importar huellas", name: "IMPORT_FINGER", smallText: "importar huellas" };
        let mnuExportHuella: MenuItem = { icon: "glyphicon glyphicon-upload", text: "Exportar huellas", name: "EXPORT_FINGER", smallText: "importar huellas" };
        let mnuAgregar: MenuItem = { icon: "glyphicon glyphicon-plus", text: "Agregar usuario", name: "ADD_USER", smallText: "agregar usuario" };
        let mnuEliminar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Eliminar usuario", name: "DELETE_USER", smallText: "eliminar usuario" };
        let mnuPasswdEliminar: MenuItem = { icon: "glyphicon glyphicon-remove-circle", text: "Eliminar password", name: "DELETE_PASSWD", smallText: "Eliminar password" };
        let mnuPasswdUpdate: MenuItem = { icon: "glyphicon glyphicon-refresh", text: "Establecer password", name: "UPDATE_PASSWD", smallText: "Establecer password" };

        this.mnuEmpleadoItems.push(mnuAsignar, mnuImportar, mnuNuevo, mnuImportHuella, mnuExportHuella, mnuAgregar, mnuEliminar, mnuPasswdEliminar, mnuPasswdUpdate);
        this.mnuEmpleadoItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    onLoadingItem(id: number) {
        if (id && id > 0) {
            this.context.app.showSpinner();
            this.checadorSvc.importUsers(id)
                .then(items => {                    
                    this.items = items;

                    //Si el empleado no tiene id es porque no esta en base de datos, asi que lo marcamos como añadido para cuando se quiere guardar
                    this.items.filter(item => item.id == 0).forEach(item => {
                        item.state = ChangeState.ADDED;
                    });
                    
                    this.cvUsuarios.sourceCollection = this.items;
                    this.getDetail();
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    getDetail() {
        this.checadorSvc.getDetailByID(this.identity)
            .then(info => {
                this.labelChecador = "Administrador de empleados en checador: " +  info.nombre;
            })
            .catch(error => this.handleError(error));
    }
    ///////////////////////// METODOS ///////////////////////////////////////

    onClickActualizar() {
        this.onLoadingItem(this.identity);
    }

    onClickGuardar() {
        this.dlgConfirmacion.config.backdrop = false;
        this.dlgConfirmacion.show();
    }

    onConfirm(sincronizarChecador: boolean) {
        this.context.app.showSpinner();

        let itemsUpdated: ChecadorUsuarioIten[] = [];

        itemsUpdated = this.items.filter(item => item.state != ChangeState.UNCHANGED);

        this.checadorSvc.exportUsers(this.identity, itemsUpdated, sincronizarChecador)
            .then(items => {
                this.context.app.hideSpinner();
                this.items = items;
                this.cvUsuarios.sourceCollection = this.items;
                this.cvUsuarios.refresh();
                this.dlgConfirmacion.hide();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedEmleado(item: any) {
        var currentItem = this.cvUsuarios.currentItem;
        if (currentItem) {
            this.cvUsuarios.editItem(currentItem);
            currentItem.idEmpleado = item.id;
            currentItem.empleado = item.data;
            currentItem.state = ChangeState.MODIFIED;
            this.cvUsuarios.commitEdit();
        }
    }

    onSelectedChecador(item: any) {
        var currentItem = this.cvUsuarios.currentItem;
        if (currentItem) {
            this.exportFingers(item.id);
        }
    }


    onCellUpdated(args: CellRangeEventArgs) {
        var currentItem = this.cvUsuarios.currentItem;
        currentItem.state = ChangeState.MODIFIED;
        this.cvUsuarios.commitEdit();
    }

    onClickMnuEmpleado(menu: Menu) {
        let option = this.mnuEmpleadoItems[menu.selectedIndex];
        if (option.name == "ALLOCATE") {
            this.selEmpleado.multiSelect = false;
            this.selEmpleado.show();
        }
        else if (option.name == "IMPORT_USR" || option.name == "IMPORT_FINGER") {
            this.context.app.showSpinner();
            let usuarioItem = <ChecadorUsuarioIten>this.cvUsuarios.currentItem;
            this.checadorSvc.importUser(this.identity, usuarioItem)
                .then(result => {
                    this.context.app.hideSpinner();
                    this.cvUsuarios.editItem(usuarioItem);
                    usuarioItem.id = result.id;
                    this.cvUsuarios.commitEdit();
                    if (option.name == "IMPORT_USR") {
                        this.alertsBar.showAlert('Usuario importado', 'success ');
                    }
                    else {
                        this.alertsBar.showAlert('Huellas de usuario importadas', 'success ');
                    }
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "IMPORT_NEW") {
            this.context.app.showSpinner();
            let items = <ChecadorUsuarioIten[]>this.cvUsuarios.sourceCollection;
            this.checadorSvc.importNewUser(this.identity, items)
                .then(result => {
                    this.context.app.hideSpinner();
                    this.cvUsuarios.sourceCollection = result;
                    this.cvUsuarios.refresh();
                    this.alertsBar.showAlert('Usuarios nuevos importados', 'success ');
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "EXPORT_FINGER") {
            let checadoresTemp: ChecadorItem[]=[];
            let checador = new ChecadorItem();
            checador.id = this.identity;
            checadoresTemp.push(checador);

            this.checadoresExclude = checadoresTemp;
            this.checadoresList.show();
        }
        else if (option.name == "ADD_USER") {
            this.usuario.nombre = "";
            this.usuario.codigo = "";
            this.usuario.tipo = 0;
            this.usuario.passwd = "";
            this.dlgUsuario.config.backdrop = false;
            this.dlgUsuario.show();
        }
        else if (option.name == "DELETE_USER") {
            this.context.app.showSpinner();
            let usuarioItem = <ChecadorUsuarioIten>this.cvUsuarios.currentItem;
            this.checadorSvc.deleteUser(this.identity, usuarioItem)
                .then(result => {
                    this.context.app.hideSpinner();
                    this.alertsBar.showAlert('Usuario eliminado', 'success ');
                    this.cvUsuarios.remove(usuarioItem);
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "DELETE_PASSWD") {
            this.context.app.showSpinner();
            let item: ChecadorUsuarioIten = <ChecadorUsuarioIten>this.cvUsuarios.currentItem
            this.cvUsuarios.editItem(item);
            item.passwd = "";
            this.cvUsuarios.commitEdit();
            this.alertsBar.showAlert('Password eliminado. Guarde cambios', 'success ');
        }
        else if (option.name == "UPDATE_PASSWD") {
            this.userContrasena = "";
            this.dlgPasswd.config.backdrop = false;
            this.dlgPasswd.show();
        }
    }

    exportFingers(idChecador:number) {
        this.context.app.showSpinner();
        let usuarioItem = <ChecadorUsuarioIten>this.cvUsuarios.currentItem;
        this.checadorSvc.exportFingers(idChecador, usuarioItem)
            .then(result => {
                this.context.app.hideSpinner();
                this.alertsBar.showAlert('Huellas de usuario exportadas', 'success ');
            })
            .catch(error => this.handleError(error));

    }

    onClickUsuarioAceptar() {
        this.dlgUsuario.hide();
        let newUser = <ChecadorUsuarioIten>this.cvUsuarios.addNew();
        newUser.codigo = this.usuario.codigo;
        newUser.nombre = this.usuario.nombre;
        newUser.passwd = this.usuario.passwd;
        newUser.tipo = this.usuario.tipo;
        newUser.afectaIncidencia = this.usuario.afectaIncidencia;
        newUser.idEmpleado = 0;
        newUser.activo = true;
        newUser.state = ChangeState.ADDED;
        this.cvUsuarios.commitNew();
    }

    onClickPasswdAceptar() {
        let item: ChecadorUsuarioIten = <ChecadorUsuarioIten>this.cvUsuarios.currentItem
        this.cvUsuarios.editItem(item);
        item.passwd = this.userContrasena;
        this.cvUsuarios.commitEdit();
        this.dlgPasswd.hide();
        this.alertsBar.showAlert('Password establecido. Guarde cambios', 'success ');
    }

    ///////////////////////// HANDLERS ///////////////////////////////////////
    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
    }

    // Util

    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element));
    }

    onClickKardexSelect()
    {        
        let usuarios: ChecadorUsuarioIten[] = this.cvUsuarios.sourceCollection;
        usuarios.filter(usrKdx => usrKdx.afectaIncidencia == false).forEach(user => {
            user.afectaIncidencia = true;
            user.state = ChangeState.MODIFIED;
        });
        this.cvUsuarios.refresh();
    }


}