﻿import { Component, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { TrackableCollection } from 'framework3/editors';
import { ToolbarButton } from 'framework3/controls';

//Locales
import { Acceso } from '../models/departamento';
import { DepartamentoService } from '../services/departamento.service';
import { DepartamentoSelectorComponent } from '../components/departamento-selector.component';

// moment
import * as moment from 'moment';

import { Context, BaseComponent, ProcessComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState, UsuarioInfo } from 'framework3/core';
import { AlertsBarComponent, AztecaValidators, MenuItem } from 'framework3/controls';

// Wijmo
import * as wjcCore from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';

@Component({
    selector: 'rh-departamentos-accesos',
    templateUrl: './departamentos-accesos.maintenance.html'
})
export class DepartamentosAccesosMaintenance extends ProcessComponent {
    form: FormGroup;
    usuario: UsuarioInfo;
    accesos: Acceso[] = [];

    accesosCollection: TrackableCollection = new TrackableCollection();
    @ViewChild('selectorDepartamento') selectorDepartamento: DepartamentoSelectorComponent;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;


    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        formBuilder: FormBuilder,
        private departamentoService: DepartamentoService
    ) {
        super(context, router, route);
        this.form = formBuilder.group({
            //        'idUsuario': [0, AztecaValidators.selected],           
        });


    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }


    onUsuarioSelected(usuario: UsuarioInfo) {
        this.usuario = usuario;
        this.departamentoService.loadAccesos(usuario.identity)
            .then(ret => {
                this.accesos = ret;
                this.accesosCollection.sourceCollection = this.accesos;
                this.accesosCollection.refresh;
            })
            .catch(error => this.handleError(error));


    }

    onClickAgregar() {

        var coincidencias = this.accesos.filter(subList => subList.departamento.path == this.selectorDepartamento.selectedItem.path);
        if (coincidencias.length == 0) {
            var accesoNew: Acceso = new Acceso();

            accesoNew.idUsuario = this.usuario.identity;
            accesoNew.idDepartamento = this.selectorDepartamento.selectedItem.identity;
            accesoNew.departamento = this.selectorDepartamento.selectedItem;

            var accesosSlice = this.accesos.slice();

            accesosSlice.push(accesoNew);

            this.accesos = accesosSlice;

            this.accesosCollection.sourceCollection = this.accesos;
            this.accesosCollection.refresh;
        }



    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'GUARDAR') {
            this.departamentoService.updateAccesos(this.usuario.identity, this.accesos)
                .then( ret =>{
                    this.alerts.showAlert('La información fué guardada con éxito', 'sucess');
                })
                .catch(error => this.handleError(error));
        }
        if (button.name == 'AGREGAR') {
            this.onClickAgregar();
        }
    }

    onRemoveAcceso(item) {
        this.accesosCollection.remove(item);
        this.accesosCollection.refresh();
    }



    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}
