﻿import { ProcessComponent, PolicyInfo, Context } from "framework3/core";
import { ChecadorService } from "../services/checador.service";
import { ActivatedRoute, Router } from "@angular/router";
import { FormBuilder, FormGroup } from "@angular/forms";
import { AlertsBarComponent, MenuItem } from "framework3/controls";
import { ViewChild, Component } from "@angular/core";
import { SearchRequest, SearchCondition } from "framework3/search";
import moment = require("moment");

@Component({
    selector: 'eventos-usuario',
    templateUrl: './eventos-usuario.maintenance.html'
})
export class EventosUsuarioMaintenance extends ProcessComponent {
    editorForm: FormGroup;


    request: SearchRequest;
    currentItems: Array<any> = new Array<any>();
    fechaIni: Date = new Date();
    fechaFin: Date = new Date();

    currentItem: any;
    totalRows: number = 0;
    pageIndex: number = 0;
    menuItems: MenuItem[] = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(public context: Context, private formBuilder: FormBuilder, router: Router, route: ActivatedRoute, private checadorService: ChecadorService) {
        super(context, router, route);
        this.editorForm = formBuilder.group({
            'fechaIni': [0],
            'fechaFin': [0]
        });
    }

    ngAfterViewInit() {
        this.fechaIni = moment().subtract(30, 'day').toDate();
        this.fechaFin = moment().add(1, 'day').toDate();
        setTimeout(_ => {
            this.refreshList();
        });
        this.createMenu();
        
    }

    createMenu() {
        let mnuEditar: MenuItem = { icon: "glyphicon glyphicon-repeat", text: "Editar", name: "EDITAR", smallText: "Editar autorizaci&oacute;n" };
        let mnuDeactivate: MenuItem = { icon: "glyphicon glyphicon-remove-alt", text: "Desactivar", name: "STATE", smallText: "Cambia el estatus del evento" };
        this.menuItems.push(mnuEditar, mnuDeactivate);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    refreshList() {
        this.context.app.showSpinner();
        this.checadorService.searchChecadoresEventos(this.context.app.defaultSite.identity,this.fechaIni,this.fechaFin)
            .then(response => {
                this.currentItems = response;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onCurrentItemChanged(item: any) {
        this.currentItem = item;
        //Cambiar texto de opción para activar o desactivar registros
        let mnuState: MenuItem = this.menuItems.filter(item => item.name == "STATE")[0];
        if (item.activo) {
            mnuState.text = "Desactivar";
        }
        else {
            mnuState.text = "Activar";
        }
    }

    searchRequested(request: SearchRequest) {
        this.request = request;
        this.refreshList();
    }

    optionClick(option: MenuItem) {
        if (!this.currentItem.data.aplicado) {
            if (option.name == "EDITAR") {
                this.router.navigate(['rh/eventos-usuario/eventos', this.process.id, this.currentItem.data.idEventoUsuario]);
            }
            else if (option.name == "STATE") {
                this.context.app.showSpinner();
                this.checadorService.statusEvento(this.currentItem.data.idEventoUsuario, !this.currentItem.data.activo, this.context.session.user.identity)
                    .then(result => {
                        this.refreshList();
                        this.showAlert('Se desactivo con exito', 'success');
                    })
                    .catch(error => this.handleError(error));            }
        } else {
            this.showAlert('El evento ya fue aplicado, no se puede modificar', 'danger');
        }
    }

    showAlert(message: string, type: string) {
        this.context.app.hideSpinner();
        this.alerts.showAlert(message, type);
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
    }

    onAgregar() {
        this.router.navigate(['rh/eventos-usuario/eventos', this.process.id, 0])
    }

    applyPolicy(policy: PolicyInfo) {

    }
}