﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { Menu } from "wijmo/wijmo.input";
import { FormGroup, FormBuilder, Validators } from '@angular/forms';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';

//wijmo
import { FlexGrid, CellRangeEventArgs, DataMap } from 'wijmo/wijmo.grid';
import * as wjcCore from 'wijmo/wijmo';

//frmework
import { AlertsBarComponent, MenuItem } from "framework3/controls";
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';
import { Context, BaseComponent, ProcessComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState, EmpresaItem, EmpresaService, SucursalItem, SucursalService } from 'framework3/core';
import { SearchCondition, ConditionValueType } from "framework3/search";


//Controls
//import { EmpleadoListComponent } from '../../controls/empleado-list.component';
//import { ChecadorListComponent } from '../../controls/checadores-list.component';

//Features
//import { ChecadorUsuarioIten, ChecadorItem, ChecadorInfo } from '../models/checador';
import { ChecadorService } from '../services/checador.service';
import { EmpleadoService } from '../services/empleado.service';
import { SearchRequest } from 'framework3/search';
import { ChecadorItem, ChecadorUsuarioIten } from '../models/checador';
import { EmpleadoItem } from '../models/empleado';


@Component({
    selector: 'rh-exportar-huellas',
    templateUrl: './exportar-huellas-checadores.maintenance.html'
})
export class ExportarHuellasChecadoresMaintenance extends ProcessComponent {

    formEmpresa: FormGroup;
    cvChecadores: wjcCore.CollectionView = new wjcCore.CollectionView();
    cvEmpleados: wjcCore.CollectionView = new wjcCore.CollectionView();

    widthClass: string[] = [];
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;
    @ViewChild('dlgEmpresa') dlgEmpresa: ModalDirective;

    empresas: EmpresaItem[];
    sucursales: SucursalItem[];
    sucursalesEmpresa: SucursalItem[];

    empresaSelected: number;
    sucursalSelected: number;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private checadorSvc: ChecadorService,
        private empleadoService: EmpleadoService) {

        super(context, router, route);
        //this.tipoUsuarioMap = new DataMap(this.tipoUsuario, 'value', 'name');
        //this.formUsuario = this.formBuilder.group({
        //    'codigo': ['', Validators.required],
        //    'nombre': ['', Validators.required],
        //    'tipoUsuario': [0],
        //    'contrasena': ['']
        //});

    }


    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);

        this.empresas = this.context.session.user.empresas.slice();
        this.sucursales = this.context.session.user.sucursales.slice();
        //Agregar empresa default
        this.empresas.unshift({ id: 0, codigo: '', nombre: '[Seleccione una empresa]', activo: true });
        //

    }

    ngAfterViewInit() {
        setTimeout(_ => {
            //this.route.params.forEach((params: Params) => {
            //    this.identity = +params['id'];
            //});
            this.loadingChecks();
            //this.createMenuDetail();
        });
    }

    loadingChecks() {        
        this.context.app.showSpinner();
        let request: SearchRequest = new SearchRequest();
        request.pageIndex = 0;
        request.pageSize = 0;
        request.paged = false;
        request.maxRows = 20;

        this.checadorSvc.searchChecador(this.context.session.user.identity, this.context.app.defaultSite.identity, request)
                .then(items => {
                    this.cvChecadores.sourceCollection = items.rows;                    
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));        
    }

    selectEmpresa(idEmpresa: number) {
        this.sucursalesEmpresa = this.sucursales.filter(
            (sucursal, index, sucursales) => sucursal.idEmpresa == idEmpresa
        );
        this.sucursalesEmpresa.unshift({ id: null, idEmpresa: 0, codigo: '', nombre: '[Seleccione una sucursal]', activo: true });
        this.sucursalSelected = null;
    }

    onClickActualizar() {
        this.dlgEmpresa.config.backdrop = false;
        this.dlgEmpresa.show();
    }

    cargarEmpleados() {
        this.dlgEmpresa.hide();
        this.context.app.showSpinner();
        let request: SearchRequest = new SearchRequest();
        request.pageIndex = 0;
        request.pageSize = 0;
        request.paged = false;
        request.maxRows = 20;

        let condicionSucursal: SearchCondition = { field: 'idSucursal', operator: '=', valueType: ConditionValueType.INT, value: this.sucursalSelected };
        request.conditions.push(condicionSucursal);

        this.empleadoService.search(this.sucursalSelected, request)
            .then(items => {
                this.cvEmpleados.sourceCollection = items.rows;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));

    }

    onClickExportar() {
        this.context.app.showSpinner();

        let usuariosUpdated: ChecadorUsuarioIten[] = [];

        let empleados: EmpleadoItem[] = this.cvEmpleados.sourceCollection;
        empleados.filter(item => item.selected == true).forEach(empleado => {
            let checkUsr = new ChecadorUsuarioIten();
            checkUsr.codigo= empleado.codigo;
            checkUsr.nombre= empleado.nombre;
            checkUsr.passwd= "";
            checkUsr.tipo= 0;
            checkUsr.afectaIncidencia=false;
            checkUsr.idEmpleado=empleado.id;
            checkUsr.activo=true;
            usuariosUpdated.push(checkUsr);
        });

        let checadores = this.cvChecadores.sourceCollection.filter(item => item.selected == true);
               
        this.checadorSvc.exportEmployeesToChecks(usuariosUpdated, checadores)
            .then(items => {
                this.context.app.hideSpinner();

             })
            .catch(error => this.handleError(error));


    }

    onClickEmpleadosSelect() {
        let usuarios: EmpleadoItem[] = this.cvEmpleados.sourceCollection;
        usuarios.filter(usrKdx => usrKdx.selected == false).forEach(user => {
            user.selected = true;
            //user.state = ChangeState.MODIFIED;
        });
        this.cvEmpleados.refresh();
    }

    ///////////////////////// HANDLERS ///////////////////////////////////////
    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
    }

    // Util

    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element));
    }

}