﻿import { Component, ViewChild, Input } from "@angular/core";

import { FormsModule, FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { Menu } from "wijmo/wijmo.input";
import { ModalDirective } from "ngx-bootstrap";
import * as moment from 'moment';

import { MenuItem, ToolbarButton, AlertsBarComponent, AztecaValidators } from "framework3/controls";
import { Context, BaseComponent, ProcessComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { FilterGroup, SearchRequest, SearchCondition } from "framework3/search";

import { KardexAsistenciaService } from "../services/kardex-asistencia.service";
import { IncidenciaEmpleadoInfo } from '../models/incidencia-empleado';
import { Control, CollectionView } from "wijmo/wijmo";
import { KardexAsistenciaInfo } from "../models/kardex-asistencia";


@Component({
    selector: 'rh-incidencias-empleado',
    templateUrl: './incidencias-empleado.maintenance.html'
})
export class IncidenciaEmpleadoMaintenance extends ProcessComponent {

    form: FormGroup;

    filterPanelCollapsed: boolean = false;
    showFilterPanel: boolean = true;
    menuItems: MenuItem[] = [];
    filterGroups: FilterGroup[] = [];
    selectedGroup: FilterGroup;

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    request: SearchRequest;
    fechaTransaccion: Date = new Date();

    currentItems: Array<any> = new Array<any>();
    currentItem: any;
    totalRows: number = 0;
    pageIndex: number = 0;
    respuesta: boolean = false;
    @Input() enabled: boolean = true;
    @Input() allowNew: boolean = true;


    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder, private kardexAsistenciaSvc: KardexAsistenciaService) {
        super(context, router, route);
        this.form = formBuilder.group({
            'fechaTransaccion': []
        });

        
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.fechaTransaccion = moment().subtract(1, 'day').toDate();
            this.refreshList();
        });
        this.createMenu();
    }

    createMenu() {
        let mnuDelete: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Eliminar", name: "DELETE", smallText: "incidencia empleado" };
        //
        this.menuItems.push(mnuDelete);
        //
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }


    /*Permite tomar todo el espacio disponible cuando el panel de filtro no está visible y 75% cuando lo está*/
    get classGridPanel(): string {
        return !this.showFilterPanel || this.filterPanelCollapsed ? 'col-12' : 'col-12 col-lg-9';
    }

    toggleCollapse() {
        this.filterPanelCollapsed = !this.filterPanelCollapsed;
    }

    onMenuItemClicked(menu: Menu, args: any) {
        let item: MenuItem = this.menuItems[menu.selectedIndex];
        this.optionClick(item);
    }

    optionClick(option: MenuItem) {
        if (option.name == "DELETE") {
            this.context.app.showSpinner();
            this.kardexAsistenciaSvc.deleteIncidenciaEmpleado(this.currentItem.id)
                .then(result => {
                    this.refreshList();
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
    }

    onParamsToolbar(button: ToolbarButton) {
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onCurrentItemChanged(item: any) {
        this.currentItem = item;
    }

    searchRequested(request: SearchRequest) {
        this.request = request;
        this.refreshList();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    onIsDroppedDown(menu: Menu, args: any) {
        this.onValidateMenu(this.currentItem);
    }

    onValidateMenu(item: any) {
        if (item) {
            let mnuError = this.getMenuItem('ERROR');
            let mnuDetail = this.getMenuItem('DETAIL');
            if (item.estatus != "Error") {
                mnuError.enabled = false;
            }
            if (item.estatus != "Error") {
                mnuDetail.enabled = false;
            }
        }
    }

    getMenuItem(name: string): MenuItem {
        let retItem: MenuItem = null;
        this.menuItems.forEach(item => {
            if (item.name == name) {
                retItem = item;
            }
        });
        return retItem;
    }

    onFilterGroupChanged(group: FilterGroup) {
        this.selectedGroup = group;
    }

    ////////////////////////////////////////

    refreshList() {
        this.context.app.showSpinner();
        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }
        let exist = false;

        this.request.conditions.forEach(r => {
            if (r.field == "fecha") {
                exist = true;
                r.value = this.fechaTransaccion;
            }
        });
        if (!exist) {
            let conFecha: SearchCondition = { field: "fecha", operator: "=", valueType: "datetime", value: this.fechaTransaccion };
            this.request.conditions.push(conFecha);
        }
        this.kardexAsistenciaSvc.searchIncidenciaEmpleado(this.context.app.defaultSite.identity, this.request)
            .then(response => {
                response.rows.forEach(r => {
                    r['fecha'] = moment(r['fecha']).toDate();
                });
                this.currentItems = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = this.request.pageIndex;
                //
                var filterGroup = new FilterGroup();
                filterGroup.tag = "ALL";
                filterGroup.name = "Todos";
                filterGroup.rowCount = this.totalRows;
                this.filterGroups.length = 0;
                this.filterGroups.push(filterGroup);
                //
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }


    newItemHandler() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }
}