﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router'
import { FormBuilder, FormGroup, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common'

import * as moment from 'moment';

//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { Alert, AztecaValidators, AlertsBarComponent } from 'framework3/controls';
import { CatalogOptions } from 'framework3/custom';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { MenuItem } from "framework3/controls";
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';


//Features
import { FiniquitoAutorizacionInfo } from '../../models/finiquito-autorizacion'
import { FiniquitoAutorizacionService } from '../../services/finiquito-autorizacion.service'

@Component({
    selector: 'autorizacion-finiquito-editor',
    templateUrl: './autorizacion-finiquito.editor.html'
})
export class AutorizacionFiniquitoEditor extends BaseComponent {
    editorForm: FormGroup;

    idFiniquitoAutorizacion: number;
    currentItem: FiniquitoAutorizacionInfo = new FiniquitoAutorizacionInfo();
    autoriza: string = "";

    clasesNominas: Array<any> = [];
    personasAutorizan: Array<any> = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private route: ActivatedRoute, private location: Location, private finiquitoAutorizacionService: FiniquitoAutorizacionService, private context: Context) {
        super();
        this.editorForm = formBuilder.group({
            'idUsuarioElabora': [],
            'idClaseNomina': [],
            'personaAutoriza': [],
            'autoriza': []
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.loadClasesNominas();
        });
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idFiniquitoAutorizacion = +param['id'];
        });
    }
    
    showAlert(message: string, type: string) {
        this.context.app.hideSpinner();
        this.alerts.showAlert(message, type);
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo) {

    }

    loadClasesNominas() {
        this.context.app.showSpinner();
        this.finiquitoAutorizacionService.loadClasesNominas().then(info => {
            this.clasesNominas = info;
            this.context.app.hideSpinner();

            this.loadPersonasAutorizan();
        }).catch(error => {
            this.context.app.hideSpinner();
            this.handleError(error)
        });
    }

    loadPersonasAutorizan() {
        this.context.app.showSpinner();
        this.finiquitoAutorizacionService.loadPersonasAutorizan().then(info => {
            this.personasAutorizan = info;
            this.context.app.hideSpinner();
            //
            if (this.idFiniquitoAutorizacion > 0) {
                this.loadFiniquitoAutorizacion();
            }
            else {
                this.currentItem = new FiniquitoAutorizacionInfo();
            }
        }).catch(error => {
            this.context.app.hideSpinner();
            this.handleError(error)
        });
    }

    loadFiniquitoAutorizacion() {
        this.context.app.showSpinner();
        this.finiquitoAutorizacionService.getDetail(this.idFiniquitoAutorizacion).then(info => {
            this.currentItem = info;
            this.context.app.hideSpinner();
        }).catch(error => {
            this.context.app.hideSpinner();
            this.handleError(error)
        });
    }


    onSeleccionAutoriza(args: any) {
        this.currentItem.autoriza = args;
    }

    onCancel() {
        this.location.back();
    }

    onUpdate() {
        if (!(this.currentItem.idUsuarioElabora > 0)) {
            this.alerts.showAlert("Seleccione el usuario", 'danger');
        }
        else if (!(this.currentItem.idClaseNomina > 0)) {
            this.alerts.showAlert("Seleccione la clase de nómina", 'danger');
        }
        else if (this.currentItem.autoriza == '') {
            this.alerts.showAlert("Indique el nombre de la presona que autoriza", 'danger');
        }
        else {
            this.context.app.showSpinner();
            this.finiquitoAutorizacionService.update(this.currentItem).then(info => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Información actualizada", 'success');
                this.location.back();
            }).catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error)
            });
        }
    }

}