﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';

import { ProcessComponent, Context, PolicyInfo } from 'framework3/core';
import { AlertsBarComponent } from 'framework3/controls';

import { EvaluacionKardexService } from '../../services/evaluacion-kardex.service';
import { EvaluacionKardexInfo } from '../../models/evaluacion-kardex';

@Component({
    selector: 'rh-evaluacion-kardex-det',
    templateUrl: './evaluacion-kardex-det.html'
})
export class EvaluacionKardexDet extends ProcessComponent {

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    widthClass: string[] = [];

    idEvaluacionKardex: number = 0;
    info: EvaluacionKardexInfo = new EvaluacionKardexInfo();

    constructor(protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private evaluacionKardexSvc: EvaluacionKardexService) {
        super(context, router, route);
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
        this.route.params.forEach((params: Params) => {
            this.idEvaluacionKardex = +params['id'];
            if (this.idEvaluacionKardex > 0) {
                this.context.app.showSpinner();
                this.evaluacionKardexSvc.getDetail(this.idEvaluacionKardex)
                    .then(result => {
                        this.info = result;
                        this.context.app.hideSpinner();
                    }).catch(e => this.handleError(e));
            }
        });
    }

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }


    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    ///////////////////////////////////////////////

    closeEditor() {
        this.location.back();
    }
}
