﻿import { Component, EventEmitter, Input, Output, ViewChild, ContentChildren, QueryList, OnInit, ChangeDetectionStrategy } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { NgForm, FormGroup, FormBuilder, FormControl, Validators } from '@angular/forms';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Menu } from 'wijmo/wijmo.input';
//
import { Context, ProcessComponent, PolicyInfo } from 'framework3/core';
import { Alert, AlertsBarComponent } from 'framework3/controls';

//
import { EvaluacionKardexInfo } from '../../models/evaluacion-kardex';
import { EvaluacionKardexService } from '../../services/evaluacion-kardex.service';

@Component({
    selector: 'rh-evaluacion-kardex-editor',
    templateUrl: './evaluacion-kardex.editor.html'
})
export class EvaluacionKardexEditorComponent extends ProcessComponent {
    
    catalogForm: FormGroup;

    identity: number;
    readonlyMode: boolean = false;
    dirty: boolean = false;
    valid: boolean = false;

    widthClass: string[] = [];
    canceled: boolean = false;
    info: EvaluacionKardexInfo = new EvaluacionKardexInfo();

    @Input() title: string = "Proceso de Transacciones";
    @Output() savedItem: EventEmitter<EvaluacionKardexInfo> = new EventEmitter<EvaluacionKardexInfo>();
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute, 
        private location: Location, 
        private formBuilder: FormBuilder,
        private evaluacionKardexSvc: EvaluacionKardexService) {
        super(context, router, route);

        this.catalogForm = formBuilder.group({
            'fechaIni': [null, Validators.required],
            'fechaFin': [null, Validators.required],
            'idEmpleado': [0],
            'idChecador': [0],
            'idDepartamento': [0],
            'idHorario': [0],
            'idTurno': [0],
            'regenerarPeriodo': [false],
            'regenerarChecada': [false],
            'regenerarKardex': [false]
        });
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
    }

    ngAfterViewInit() {
        if (!this.catalogForm) return;
        this.catalogForm.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }

    private changeState() {
        this.valid = this.catalogForm.valid;
        this.dirty = this.catalogForm.dirty;
    }


    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }


    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    ///////////////////////////////////////////////

    closeEditor() {
        this.location.back();
    }

    saveHandler() {
        this.info.idBranch = this.context.app.defaultSite.identity;
        this.context.app.showSpinner();
        this.evaluacionKardexSvc.update(this.info)
            .then(result => {
                this.context.app.hideSpinner();
                this.location.back();
            })
            .catch(error => this.handleError(error));
    }
}