﻿import { BaseComponent, PolicyInfo, Context } from "framework3/core";
import { FormGroup, FormBuilder, Validators } from "@angular/forms";
import { AlertsBarComponent, AztecaValidators } from "framework3/controls";
import { ViewChild, Component } from "@angular/core";
import { ActivatedRoute } from "@angular/router";
import { Location } from '@angular/common'
import { ChecadorService } from "../../services/checador.service";
import { EventoUsuarioInfo } from "../../models/evento-usuario";
import { EmpleadoInfo } from "../../models/empleado";

@Component({
    selector: 'eventos-usuario-editor',
    templateUrl: './eventos-usuario.editor.html'
})
export class EventosUsuarioEditor extends BaseComponent {
    editorForm: FormGroup;
    idEventoUsuario: number = 0;
    eventoUsuario: EventoUsuarioInfo = new EventoUsuarioInfo();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private route: ActivatedRoute, private location: Location, private checadorService: ChecadorService, private context: Context) {
        super();
        this.editorForm = formBuilder.group({
            'idEmpleado': [0, AztecaValidators.selected],
            'fecha':[0,Validators.required],
            'idChecador': [0, AztecaValidators.selected],
            'restringir': [0],
            'idTipoEvento': [0, AztecaValidators.selected],
            'mensaje': [0, Validators.required]
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.loadEvento();
        });
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idEventoUsuario = +param['id'];
        });
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onEmpleadoSelected(args: EmpleadoInfo) {
        if(args!=undefined && args!=null)
            this.eventoUsuario.codigo = args.code;
    }
    onCancel() {
        this.location.back();
    }

    onUpdate() {
        this.context.app.showSpinner();
        this.checadorService.saveEventoUsuario(this.eventoUsuario).then(
            info => {
                this.context.app.hideSpinner();
                this.alerts.showAlert("Información actualizada", 'success');
                this.location.back();
            }
        ).catch(
            error => {
                this.handleError(error);
            });
    }

    loadEvento() {
        if (this.idEventoUsuario > 0) {
            this.context.app.showSpinner();
            this.checadorService.getEventoUsuario(this.idEventoUsuario).then(
                info => {
                    console.log(JSON.stringify(info));
                    this.eventoUsuario = info[0];
                    console.log(JSON.stringify(this.eventoUsuario));
                    this.context.app.hideSpinner();
                }
            ).catch(
                error => {
                    this.handleError(error);
                });
        }
    }
}