﻿import { Component, EventEmitter, Input, Output, ViewChild, ContentChildren, QueryList, OnInit, ChangeDetectionStrategy } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { NgForm, FormGroup, FormBuilder, FormControl, Validators } from '@angular/forms';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Menu } from 'wijmo/wijmo.input';
import * as wjcCore from 'wijmo/wijmo';
//
import { Context, ProcessComponent, PolicyInfo } from 'framework3/core';
import { Alert, AlertsBarComponent, AztecaValidators } from 'framework3/controls';

//
import { IncidenciaEmpleadoInfo } from '../../models/incidencia-empleado';
import { EmpleadoItem } from '../../models/empleado';
import { KardexAsistenciaService } from '../../services/kardex-asistencia.service';
import { EmpleadoListComponent } from '../../controls/empleado-list.component';
import { IncidenciaInfo } from "../../models/incidencia";
import * as moment from 'moment';

@Component({
    selector: 'rh-incidencias-empleado-editor',
    templateUrl: './incidencias-empleado.editor.html'
})
export class IncidenciaEmpleadoEditorComponent extends ProcessComponent {

    catalogForm: FormGroup;
    catalogFormEmpleado: FormGroup;

    identity: number;
    readonlyMode: boolean = false;
    dirty: boolean = false;
    valid: boolean = false;

    validEmpleado: boolean;
    fechaIni: Date;
    fechaFin: Date;
    idIncidencia: number = 0;
    idEmpleado: number = 0;
    incidenciasByEmpleado: wjcCore.CollectionView = new wjcCore.CollectionView();
    incidencia: IncidenciaInfo;
    folioEmpleado: string = '';
    observacionEmpleado: string = '';

    widthClass: string[] = [];
    canceled: boolean = false;
    info: IncidenciaEmpleadoInfo = new IncidenciaEmpleadoInfo();

    cvEmpleados: wjcCore.CollectionView = new wjcCore.CollectionView();

    @Input() title: string = "Captura de incidencias";
    @Output() savedItem: EventEmitter<IncidenciaEmpleadoInfo> = new EventEmitter<IncidenciaEmpleadoInfo>();
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(EmpleadoListComponent) selEmpleado: EmpleadoListComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private kardexAsistenciaSvc: KardexAsistenciaService) {
        super(context, router, route);

        this.catalogForm = formBuilder.group({
            'fecha': [null, Validators.required],
            'incidencia': [0, AztecaValidators.selected],
            'folio': [''],
            'observacion': ['']
        });

        this.catalogFormEmpleado = formBuilder.group({
            'fechaIni': [null, Validators.required],
            'fechaFin': [null, Validators.required],
            'empleado':[0,AztecaValidators.selected],
            'incidenciaEmpleado': [0],
            'folioEmpleado': [''],
            'observacionEmpleado':['']
        });
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
        this.cvEmpleados.sourceCollection = this.info.empleados;
    }

    ngAfterViewInit() {
        if (!this.catalogForm) return;
        this.catalogForm.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });

        this.catalogFormEmpleado.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });

        setTimeout(_ => {
            this.fechaIni = moment().subtract(10, 'day').toDate();
        });

    }

    private changeState() {
        this.valid = this.catalogForm.valid;
        this.dirty = this.catalogForm.dirty;
        this.validEmpleado = this.catalogFormEmpleado.valid;
    }


    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }


    private handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    ///////////////////////////////////////////////

    closeEditor() {
        this.location.back();
    }

    saveHandler() {
        this.context.app.showSpinner();
        this.kardexAsistenciaSvc.updateIncidenciaEmpleado(this.info)
            .then(result => {
                this.context.app.hideSpinner();
                this.location.back();
            })
            .catch(error => this.handleError(error));
    }

    onAgregarEmpleado() {
        this.selEmpleado.show();
    }

    onSelectedEmleado(items: EmpleadoItem[]) {
        var elements = this.cvEmpleados.sourceCollection;
        items.forEach(item => {
            var empleado = null;
            elements.forEach(element => {
                if (item.id == element.id) {
                    empleado = item;
                }
            });
            if (empleado == null) {
                let element = this.cvEmpleados.addNew();
                element.id = item.id;
                element.codigo = item.codigo;
                element.nombre = item.nombre;
                element.activo = item.activo;
                this.cvEmpleados.commitNew();
            }
        });       
    }

    onRowDeleteImpuesto(item: EmpleadoItem) {
        this.cvEmpleados.beginUpdate();
        this.cvEmpleados.remove(item);
        this.cvEmpleados.commitEdit();
    }

    ////////////////////////////////////////////Empleado///////////////////////////////

    refreshEmpleado() {
        this.context.app.showSpinner();
        this.kardexAsistenciaSvc.getIncidenciasByEmpleado(this.idEmpleado, this.fechaIni, this.fechaFin)
            .then(info => {
                this.context.app.hideSpinner();
                this.incidenciasByEmpleado.sourceCollection = info;
                this.incidenciasByEmpleado.refresh();
            })
            .catch(error => this.handleError(error));
    }

    invertirSeleccion() {
        var i = 0;
        for (i = 0; i <= this.incidenciasByEmpleado.totalItemCount - 1; i++) {
            this.incidenciasByEmpleado._view[i].seleccion = !this.incidenciasByEmpleado._view[i].seleccion
        }

        wjcCore.Control.invalidateAll();
    }

    onIncidenciaUpdated(info: IncidenciaInfo) {
        this.incidencia = info;
    }

    asign() {
        var i = 0;
        if (this.idIncidencia != 0) {
            for (i = 0; i <= this.incidenciasByEmpleado.totalItemCount - 1; i++) {
                if (this.incidenciasByEmpleado._view[i].seleccion) {
                    if (this.incidencia != undefined) {
                        this.incidenciasByEmpleado._view[i].nombreIncidencia = this.incidencia.nombre;
                        this.incidenciasByEmpleado._view[i].idIncidencia = this.incidencia.identity;
                        this.incidenciasByEmpleado._view[i].folio = this.folioEmpleado;
                        this.incidenciasByEmpleado._view[i].observaciones = this.observacionEmpleado;
                    }
                    this.incidenciasByEmpleado._view[i].seleccion = !this.incidenciasByEmpleado._view[i].seleccion
                }

            }
            wjcCore.Control.invalidateAll();
        }
    }

    saveIncidenciasEmpleado() {
        var i = 0;
        var incidencias: any[] = [];
        for (i = 0; i <= this.incidenciasByEmpleado.totalItemCount - 1; i++) {
            if (this.incidenciasByEmpleado._view[i].idIncidencia>0) {
                incidencias.push(this.incidenciasByEmpleado._view[i]);
            }
        }

        if (incidencias.length > 0) {
            this.context.app.showSpinner();
            this.kardexAsistenciaSvc.updateIncidenciasByEmpleado(incidencias)
                .then(result => {
                    this.context.app.hideSpinner();
                    this.alerts.showAlert('Se guardaron con éxito las incidencias', 'success');
                })
                .catch(error => this.handleError(error));
        }
    }
}