﻿import { Component, ViewChild } from "@angular/core";
import { ProcessComponent, Context, PolicyInfo } from "framework3/core";
import { AlertsBarComponent, AztecaValidators } from "framework3/controls";
import { InterfazAdamInfo } from "../../models/interfaz-adam";
import { Router, ActivatedRoute, Params } from "@angular/router";
import { Location } from '@angular/common';
import { FormBuilder, Validators, FormGroup } from "@angular/forms";
import { InterfazAdamService } from "../../services/interfaz-adam.service";

@Component({
    selector: 'rh-interfaz-adam-det',
    templateUrl: './interfaz-adam-det.html'
})
export class InterfazAdamDetail extends ProcessComponent {

    catalogForm: FormGroup;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    widthClass: string[] = [];

    idInterfazAdam: number = 0;
    info: InterfazAdamInfo = new InterfazAdamInfo();

    constructor(protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private interfazAdamsvc: InterfazAdamService) {
        super(context, router, route);
        
        this.catalogForm = formBuilder.group({
            'fechaIni': [null, Validators.required],
            'fechaFin': [null, Validators.required],
            'idDepartamento': [0, AztecaValidators.selected],
            'idEmpresaAdam': [0, AztecaValidators.selected],

        });

    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
        this.route.params.forEach((params: Params) => {
            this.idInterfazAdam = +params['id'];
            if (this.idInterfazAdam > 0) {
                this.context.app.showSpinner();
                this.interfazAdamsvc.getDetail(this.idInterfazAdam)
                    .then(result => {
                        this.info = result;
                        this.context.app.hideSpinner();
                    }).catch(e => this.handleError(e));
            }
        });
        
    }

    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }


    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    ///////////////////////////////////////////////

    closeEditor() {
        this.location.back();
    }
}
