﻿import { Component, Input, Output, ViewChild, EventEmitter } from "@angular/core";
import { ProcessComponent, Context, PolicyInfo } from "framework3/core";
import { FormGroup, FormBuilder, Validators } from "@angular/forms";
import { AlertsBarComponent, AztecaValidators } from "framework3/controls";
import { ActivatedRoute, Router } from "@angular/router";
import { Location } from '@angular/common';
import { InterfazAdamInfo } from "../../models/interfaz-adam";
import { InterfazAdamService } from "../../services/interfaz-adam.service";
@Component({
    selector: 'rh-interfaz-adam-editor',
    templateUrl: './interfaz-adam.editor.html'
})
export class InterfazAdamEditor extends ProcessComponent {

    catalogForm: FormGroup;

    identity: number;
    readonlyMode: boolean = false;
    dirty: boolean = false;
    valid: boolean = false;

    widthClass: string[] = [];
    canceled: boolean = false;
    info: InterfazAdamInfo = new InterfazAdamInfo();

    @Input() title: string = "Interfaz de Incidencias";
    @Output() savedItem: EventEmitter<InterfazAdamInfo> = new EventEmitter<InterfazAdamInfo>();
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private interfazAdamSvc: InterfazAdamService) {
        super(context, router, route);

        this.catalogForm = formBuilder.group({
            'fechaIni': [null, Validators.required],
            'fechaFin': [null, Validators.required],
            'idDepartamento': [0, AztecaValidators.selected],
            'idEmpresaAdam': [0, AztecaValidators.selected],

        });
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
    }

    ngAfterViewInit() {
        if (!this.catalogForm) return;
        this.catalogForm.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }

    private changeState() {
        this.valid = this.catalogForm.valid;
        this.dirty = this.catalogForm.dirty;
    }


    showAlert(message: string, type: string) {
        this.alerts.showAlert(message, type);
    }


    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    ///////////////////////////////////////////////

    closeEditor() {
        this.location.back();
    }

    onCargar() {
        this.context.app.showSpinner();
        this.interfazAdamSvc.getEmpleadosInterfaz(this.info.fechaIni, this.info.fechaFin, this.info.idDepartamento, this.info.idEmpresaAdam, this.context.app.defaultSite.identity, this.context.app.defaultCompany.identity)
            .then(info => {
                this.context.app.hideSpinner();
                console.log(JSON.stringify(info));
                this.info.details = info;
            })
            .catch(error => {
                console.log(error);
                this.context.app.hideSpinner();
                this.alerts.showAlert('Error al consultar ' + error, 'danger');
            })
    }

    saveHandler() {
        this.info.idSucursal = this.context.app.defaultSite.identity;
        console.log(this.info);
        this.context.app.showSpinner();
        this.interfazAdamSvc.update(this.info)
            .then(result => {
                this.context.app.hideSpinner();
                this.location.back();
            })
            .catch(error => this.handleError(error));
    }
}