﻿import { Component, ViewChild } from '@angular/core';
import { FormsModule, FormGroup, FormBuilder, Validators, FormControl } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
// Framework
import { Context, BaseComponent, ProcessComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState, ReporteInfo, ParametroInfo, ReportService } from 'framework3/core';
import { Alert, AlertsBarComponent, AztecaValidators, MenuItem, ContentViewerComponent } from 'framework3/controls';
import { TrackableCollection } from 'framework3/editors';
// Bootstrap
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';
// moment
import * as moment from 'moment';
// Wijmo
import * as wjcCore from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
// Features
import { PeriodoTrabajoService } from '../services/periodo-trabajo.service';
import { HorarioService } from '../services/horario.service';
import { TurnoService } from '../services/turno.service';

import { TurnoListComponent } from '../controls/turno-list.component';
import { HorarioListComponent } from '../controls/horario-list.component';
import { PeriodoTrabajoInfo } from '../index';
import { TurnoInfo } from '../models/turno';
import { HorarioInfo } from '../models/horario';
import { EmpleadoInfo } from '../models/empleado';

@Component({
    selector: 'rh-periodo-trabajo-empleado',
    templateUrl: './periodo-trabajo-empleado.maintenance.html'
})
export class PeriodoTrabajoEmpleadoMaintenance extends ProcessComponent {

    form: FormGroup;
    formFecha: FormGroup;

    tipoHorario: FormControl;

    idEmpleado: number = 0;
    fechaIni: Date = new Date();
    fechaFin: Date = new Date();
    fecha: Date = new Date();
    valid: boolean = false;
    validFecha: boolean = false;
    idTurno: number = 0;
    idHorario: number = 0;

    widthClass: string[] = [];
    menuIncidenciaItems: MenuItem[] = [];

    titleContentView: string = "";
    fileNameContentView: string = "";

    @ViewChild('lstDlgTurno') dlgLstTurno: TurnoListComponent;
    @ViewChild('lstDlgHorario') dlgLstHorario: HorarioListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    cvPeriodoTrabajo: wjcCore.CollectionView = new wjcCore.CollectionView();
    cvPeriodoTrabajoFecha: wjcCore.CollectionView = new wjcCore.CollectionView();
    turnoInfo: TurnoInfo;
    horarioInfo: HorarioInfo;
    empleadoInfo: EmpleadoInfo;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private peridoTrabajoSvc: PeriodoTrabajoService,
        private turnoSvc: TurnoService,
        private horarioSvc: HorarioService,
        private reportSvc: ReportService
    ) {
        super(context, router, route);
        this.form = formBuilder.group({
            'idEmpleado': [null, AztecaValidators.selected],
            'fechaIni': [null, Validators.required],
            'fechaFin': [null, Validators.required]
        });

        this.formFecha = formBuilder.group({
            'fecha': [null, Validators.required],
            'idTurno': [0],
            'idHorario': [0],
            'tipoHorario': ['']
        });

        this.tipoHorario = <FormControl>this.formFecha.controls['tipoHorario'];
        this.tipoHorario.setValue('T');
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
    }

    ngAfterViewInit() {
        if (!this.form) return;
        setTimeout(_ => {
            this.fechaIni = moment().subtract(8, 'day').toDate();
            this.fechaFin = moment().subtract(1, 'day').toDate();
            this.fecha = moment().add(1, 'day').toDate();
            this.createMenuDetail();
        });

        this.form.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });

        this.formFecha.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }

    private changeState() {
        this.valid = this.form.valid;
        this.validFecha = this.formFecha.valid;
    }

    closeEditor() {
        this.location.back();
    }

    createMenuDetail() {
        let mnuTurno: MenuItem = { icon: "glyphicon glyphicon-calendar", text: "Asignar Turno", name: "TURNO", smallText: "asignar turno" };
        let mnuHorario: MenuItem = { icon: "glyphicon glyphicon-time", text: "Asignar Horario", name: "HORARIO", smallText: "asignar horario" };
        let mnuRemove: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Quitar Turno y/o Horario", name: "REMOVE", smallText: "asignar folio" };

        this.menuIncidenciaItems.push(mnuTurno, mnuHorario, mnuRemove);
        this.menuIncidenciaItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    ////////////////////////////////////////////////////////
    // Evnents handler

    onClickPeriodoActualizar() {
        if (this.valid) {
            this.context.app.showSpinner();
            this.peridoTrabajoSvc.getPeriodoTrabajo(this.idEmpleado, this.fechaIni, this.fechaFin)
                .then(info => {
                    this.cvPeriodoTrabajo.sourceCollection = info;
                    this.cvPeriodoTrabajo.refresh();
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.cvPeriodoTrabajo.sourceCollection = [];
            this.cvPeriodoTrabajo.refresh();
        }
    }

    onClickMnuPeriodo(menu: Menu) {
        let option = this.menuIncidenciaItems[menu.selectedIndex];
        if (option.name == "TURNO") {
            this.dlgLstTurno.show();
        }
        else if (option.name == "HORARIO") {
            this.dlgLstHorario.show();
        }
        else if (option.name == "REMOVE") {
            let periodoTrabajo = <PeriodoTrabajoInfo>this.cvPeriodoTrabajo.currentItem;
            if (periodoTrabajo) {
                this.cvPeriodoTrabajo.editItem(periodoTrabajo);
                periodoTrabajo.idTurno = 0;
                periodoTrabajo.idHorario = 0;
                periodoTrabajo.turno = null;
                periodoTrabajo.horario = null;
                periodoTrabajo.entrada = null;
                periodoTrabajo.salida = null;
                periodoTrabajo.state = ChangeState.DELETED;
                this.cvPeriodoTrabajo.commitEdit();
            }
        }
    }

    saveHandler() {
        this.context.app.showSpinner();
        this.peridoTrabajoSvc.updatePeriodoTurnoHorario(this.cvPeriodoTrabajo.items)
            .then(result => {
                this.onClickPeriodoActualizar();
            })
            .catch(error => this.handleError(error));
    }


    onSelectedTurno(catalogItem: CatalogItem) {
        this.context.app.showSpinner();
        this.turnoSvc.getDetailByID(catalogItem.id)
            .then(result => {
                let periodoTrabajo = <PeriodoTrabajoInfo> this.cvPeriodoTrabajo.currentItem;
                this.cvPeriodoTrabajo.editItem(periodoTrabajo);
                periodoTrabajo.turno = result;
                periodoTrabajo.idTurno = result.identity;
                periodoTrabajo.horario = null;
                periodoTrabajo.idHorario = 0;
                periodoTrabajo.manual = true;
                periodoTrabajo.state = ChangeState.MODIFIED;
                this.cvPeriodoTrabajo.commitEdit();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedHorario(catalogItem: CatalogItem) {
        this.context.app.showSpinner();
        this.horarioSvc.getDetailByID(catalogItem.id)
            .then(result => {
                let periodoTrabajo = <PeriodoTrabajoInfo>this.cvPeriodoTrabajo.currentItem;
                this.cvPeriodoTrabajo.editItem(periodoTrabajo);
                periodoTrabajo.horario = result;
                periodoTrabajo.idHorario = result.identity;
                periodoTrabajo.turno = null;
                periodoTrabajo.idTurno = 0;
                periodoTrabajo.manual = true;
                periodoTrabajo.state = ChangeState.MODIFIED;
                this.cvPeriodoTrabajo.commitEdit();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    printRPT() {
        this.context.app.showSpinner();
        let informe = new ReporteInfo();
        let parametroIdEmpleado = new ParametroInfo();
        let parametroFechaIni = new ParametroInfo();
        let parametroFechaFin = new ParametroInfo();
        let parametroNombre = new ParametroInfo();
        parametroIdEmpleado.name = "IdEmpleado";
        parametroIdEmpleado.values.push(this.idEmpleado.toString());
        parametroFechaIni.name = "FechaIni"
        parametroFechaIni.values.push(this.fechaIni.toLocaleDateString());
        parametroFechaFin.name = "FechaFin"
        parametroFechaFin.values.push(this.fechaFin.toLocaleDateString());
        parametroNombre.name = "Nombre"
        parametroNombre.values.push(this.empleadoInfo.persona.apellidoPaterno + " " + this.empleadoInfo.persona.apellidoMaterno + " " + this.empleadoInfo.persona.nombre );

        informe.parametros.push(parametroIdEmpleado);
        informe.parametros.push(parametroFechaIni);
        informe.parametros.push(parametroFechaFin);
        informe.parametros.push(parametroNombre);
        informe.path = "/Onili/RH/ReporteMovimientosPerTrabajo";
        informe.format = "pdf";
        this.reportSvc.loadReporte(informe)
            .then(info => {
                this.context.app.hideSpinner();
                this.titleContentView = "Periodo de Trabajo"
                this.fileNameContentView = "periodo_trabajo.pdf";
                this.reportViewer.show("data:application/pdf; base64, " + info.content);
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error);
            });
    }

    onEmpleadoUpdate(empleado: EmpleadoInfo) {
        this.empleadoInfo = empleado;
    }

    ////////////////////////////////////////////////////////

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    private handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    ///////////////////Por Fecha/////////////////
    refresh() {
        this.context.app.showSpinner();
        this.peridoTrabajoSvc.getPeriodoTrabajoFecha(this.context.app.defaultSite.identity, this.fecha)
            .then(info => {
                this.cvPeriodoTrabajoFecha.sourceCollection = info;
                this.cvPeriodoTrabajoFecha.refresh();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    invertirSeleccion() {
        var i = 0;
        for (i = 0; i <= this.cvPeriodoTrabajoFecha.totalItemCount - 1; i++) {
            this.cvPeriodoTrabajoFecha._view[i].seleccion = !this.cvPeriodoTrabajoFecha._view[i].seleccion
        }

        wjcCore.Control.invalidateAll();
    }

    onHorarioUpdate(info: HorarioInfo) {
        this.horarioInfo = info;
    }

    onTurnoUpdated(info: TurnoInfo) {
        this.turnoInfo = info;
    }

    asign() {
        console.log(this.tipoHorario);
        if (this.cvPeriodoTrabajoFecha.itemCount > 0) {
            var i = 0;
            for (i = 0; i <= this.cvPeriodoTrabajoFecha.totalItemCount - 1; i++) {
                if (this.cvPeriodoTrabajoFecha._view[i].seleccion) {
                    if (this.tipoHorario.value == 'T') {
                        if (this.turnoInfo != undefined) {
                            this.cvPeriodoTrabajoFecha._view[i].idTurnoTit = this.turnoInfo.identity;
                            this.cvPeriodoTrabajoFecha._view[i].turno = this.turnoInfo.nombre;

                            this.cvPeriodoTrabajoFecha._view[i].idHorario = 0;
                            this.cvPeriodoTrabajoFecha._view[i].horario = '';
                        }
                    } else if (this.tipoHorario.value == 'H') {
                        if (this.horarioInfo != undefined) {
                            this.cvPeriodoTrabajoFecha._view[i].idHorario = this.horarioInfo.identity;
                            this.cvPeriodoTrabajoFecha._view[i].horario = this.horarioInfo.nombre;

                            this.cvPeriodoTrabajoFecha._view[i].idTurnoTit = 0;
                            this.cvPeriodoTrabajoFecha._view[i].turno = '';
                        }
                    }

                    this.cvPeriodoTrabajoFecha._view[i].seleccion = false;
                }
            }
            wjcCore.Control.invalidateAll();
        }
    }

    saveFechas() {
        let items: any[] = [];
        var i = 0;

        this.context.app.showSpinner();
        for (i = 0; i <= this.cvPeriodoTrabajoFecha.totalItemCount - 1; i++) {
            if (this.cvPeriodoTrabajoFecha._view[i].idTurnoTit != 0 || this.cvPeriodoTrabajoFecha._view[i].idHorario != 0) {
                items.push(this.cvPeriodoTrabajoFecha._view[i]);
            }
        }

        this.peridoTrabajoSvc.updatePeriodosFecha(this.fecha, items)
            .then(info => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Se agregaron con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }

    printFechaRPT() {
        this.context.app.showSpinner();
        let informe = new ReporteInfo();
        let parametroFecha = new ParametroInfo();
        let parametroSucursal = new ParametroInfo();
        parametroFecha.name = "Fecha"
        parametroFecha.values.push(this.fecha.toLocaleDateString());
        parametroSucursal.name = "IdSucursal"
        parametroSucursal.values.push(this.context.app.defaultSite.identity.toString());

        informe.parametros.push(parametroFecha);
        informe.parametros.push(parametroSucursal);
        
        informe.path = "/Onili/RH/ReporteMovimientosPeriodoTrabajoPorFecha";
        informe.format = "pdf";
        this.reportSvc.loadReporte(informe)
            .then(info => {
                this.context.app.hideSpinner();
                this.titleContentView = "Periodo de Trabajo"
                this.fileNameContentView = "periodo_trabajo.pdf";
                this.reportViewer.show("data:application/pdf; base64, " + info.content);
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error);
            });
    }

    /////////////////////////////////////////////
}