﻿import { Component, ViewChild } from "@angular/core";
import { ProcessComponent, PolicyInfo, Context } from "framework3/core";
import { FormGroup, FormBuilder, Validators } from "@angular/forms";
import { AlertsBarComponent, AztecaValidators } from "framework3/controls";
import { ActivatedRoute, Router } from "@angular/router";
import { Location } from '@angular/common';
import { ChecadorService } from "../services/checador.service";
import { MotivoService } from "../services/motivo.service";
import moment = require("moment");
import { CollectionView, Control } from "wijmo/wijmo";

@Component({
    selector: 'rh-planchar-checada',
    templateUrl: './planchar-checadas.maintenance.html'
})
export class PlancharChecadasMaintenance extends ProcessComponent {
    form: FormGroup;
    fecha: Date = new Date();
    valid: boolean = false;
    widthClass: string[] = [];
    idMotivo: number = 0;

    motivos: any[] = [];

    periodosEmpleados: CollectionView = new CollectionView();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private checadorSvc: ChecadorService,
        private motivoSvc: MotivoService
    ) {
        super(context, router, route);
        this.form = formBuilder.group({
            'idMotivo': [null, AztecaValidators.selected],
            'fecha': [null, Validators.required]
        });
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
        setTimeout(() => {
            this.motivoSvc.loadCbo(this.context.app.defaultSite.identity)
                .then(info => {
                    this.motivos = info;
                    this.motivos.unshift({ id: 0, codigo: '', nombre: '[Seleccione un motivo]', Descripcion: '' });
                })
                .catch(error => this.handleError(error));
        });
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    ngAfterViewInit() {
        if (!this.form) return;
        setTimeout(_ => {
            this.fecha = moment().subtract(1, 'day').toDate();
        });

        this.form.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }


    private changeState() {
        this.valid = this.form.valid;
    }

    closeEditor() {
        this.location.back();
    }

    private handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    refresh() {
        this.context.app.showSpinner();
        this.periodosEmpleados.sourceCollection = [];
        this.periodosEmpleados.refresh();
        this.checadorSvc.getPeriodoTrabaoEmpleados(this.context.app.defaultCompany.identity,this.context.app.defaultSite.identity, this.fecha)
            .then(info => {
                console.log(info);
                this.periodosEmpleados.sourceCollection = info;
                this.periodosEmpleados.refresh();
                this.context.app.hideSpinner();

            })
            .catch(error => {
                this.handleError(error);
            });
    }

    saveHandler() {
        this.context.app.showSpinner();
        let items: any[] = [];
        for (let i = 0; i <= this.periodosEmpleados.totalItemCount - 1; i++) {
            if (this.periodosEmpleados._view[i].sel || this.periodosEmpleados._view[i].selFin || this.periodosEmpleados._view[i].selIni) {
                items.push(this.periodosEmpleados._view[i]);
            }
        }
        this.checadorSvc.plancharChecadas(items, this.idMotivo, this.context.session.user.identity, this.context.session.user.code)
            .then(info => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Se plancharon las checadas y se reprocesarán los empleados', 'success');
                this.periodosEmpleados.sourceCollection = [];
                Control.invalidateAll();
            })
            .catch(error => this.handleError(error));
        
    }
    
}