﻿import { Component, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';

// moment
import * as moment from 'moment';

import { Context, BaseComponent, ProcessComponent, CatalogItem, CatalogService, PolicyInfo, ChangeState } from 'framework3/core';
import { AlertsBarComponent, AztecaValidators, MenuItem } from 'framework3/controls';

// Wijmo
import * as wjcCore from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';

import { ClaseNominaService } from '../services/clase-nomina.service';


import { TransaccionItem } from '../models/transaccion';
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';
import { RangoVacacionalInfo } from '../models/rango-vacacional';

@Component({
    selector: 'rh-rangos-vacacionales',
    templateUrl: './rangos-vacacionales.maintenance.html'
})
export class RangosVacacionalesMaintenance extends ProcessComponent {

    form: FormGroup;
    formRango: FormGroup;

    idClaseNomina: number = null;
    valid: boolean = false;
    widthClass: string[] = [];

    cvRangos: wjcCore.CollectionView = new wjcCore.CollectionView();
    eliminadas: RangoVacacionalInfo[] = [];

    motivos: any[] = [];
    menuRangoItems: MenuItem[] = [];
    
    // 
    currentRango: RangoVacacionalInfo = new RangoVacacionalInfo();
  

    @ViewChild('#alertsGral') alerts: AlertsBarComponent;
    @ViewChild('alertsEdit') alertsEdit: AlertsBarComponent;
    @ViewChild('dlgRango') dlgRango: ModalDirective;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        formBuilder: FormBuilder,
        private claseNominaService: ClaseNominaService
    ) {
        super(context, router, route);
        this.form = formBuilder.group({
    //        'idClaseNomina': [0, AztecaValidators.selected],           
        });

        this.formRango = formBuilder.group({
            'anios': [null, Validators.required],
            'dias': [null, Validators.required],
            'diasAcumulado': [null, Validators.required],
            'diasVacaciones': [null, Validators.required],
            'diasDisfrute': [null, Validators.required],
            'diasVacacionesSin': [null, Validators.required],
            'diasDisfruteSin': [null, Validators.required],
        });
        
        this.cvRangos.sourceCollection = [];
    }

    ngOnInit() {
        //Establecer el ancho establecido para la aplicación
        this.widthClass.push(this.context.app.defaultWidth);
        //setTimeout(() => {
        //    this.motivoSvc.loadCbo(this.context.app.defaultSite.identity)
        //        .then(info => {
        //            this.motivos = info;
        //            this.motivos.unshift({ id: 0, codigo: '', nombre: '[Seleccione un motivo]', Descripcion: '' });
        //        })
        //        .catch(error => this.handleError(error));
        //});
        if (this.route.params) {
            this.route.params.forEach(param => {         
                this.idClaseNomina = +param['idClaseNomina'];
            });
        }
        
        setTimeout(_ => {
            this.onClickActualizar();
        });

    }

    ngAfterViewInit() {
        if (!this.form) return;
        setTimeout(_ => {
               this.createMenuDetail();
        });

        this.form.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }

    private changeState() {
        this.valid = this.form.valid;
    }

    createMenuDetail() {
        let mnuEditar: MenuItem = { icon: "glyphicon glyphicon-pencil", text: "Editar", name: "EDIT", smallText: "editar rango vacacional" };
        let mnuQuitar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Eliminar", name: "QUITAR", smallText: "eliminar rango vacacional" };

        this.menuRangoItems.push(mnuEditar);
        this.menuRangoItems.push(mnuQuitar);
        this.menuRangoItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    ////////////////////////////////////////////////////////

    onClickGuardar() {
        this.context.app.showSpinner();
        let items = this.cvRangos.items;
        let rangos = items.concat(this.eliminadas);
        this.claseNominaService.saveRangosVacacionales(rangos)
            .then(result => {
                this.context.app.hideSpinner();
                this.onClickActualizar();
                this.alerts.showAlert("La información se guardo correctamente", "ok");                
            })
            .catch(error => this.handleError(error));
    }

    onSelectedClaseChanged(item: any) {
        this.onClickActualizar();
    }

    onClickCerrar() {
        this.location.back();
    }
       
    onClickActualizar() {
        this.context.app.showSpinner();
        this.claseNominaService.getRangosVacacionales(this.idClaseNomina)
            .then(result => {
                this.eliminadas = [];
                this.cvRangos.sourceCollection = result;
                this.cvRangos.refresh();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onClickAgregar() {
               //
        this.currentRango = new RangoVacacionalInfo();
        this.currentRango._clientID = 0;
        //

        this.dlgRango.config.backdrop = false;
        this.dlgRango.show();

    }



    onClickMenu(menu: Menu) {
        let option = this.menuRangoItems[menu.selectedIndex];
        if (option.name == "QUITAR") {
            let item = <RangoVacacionalInfo>this.cvRangos.currentItem;
            if (item) {
                item.state = ChangeState.DELETED
                this.cvRangos.remove(item);
                this.eliminadas.push(item);
            }
        }
        if (option.name == "EDIT") {
            //Editar
            this.currentRango = this.copyRango(this.cvRangos.currentItem);         

            this.dlgRango.config.backdrop = false;
            this.dlgRango.show();
        }
    }

    //onClickRangoAceptar() {        
    //    if (this.validarNew()==true) {
    //        this.context.app.showSpinner();
    //        let rangoVacac = <RangoVacacionalInfo>this.cvTransacciones.addNew();
    //        rangoVacac.idClaseNomina = this.idClaseNomina;
    //        rangoVacac.anios = this.anios;
    //        rangoVacac.dias = this.dias;
    //        rangoVacac.diasAcumulado = this.diasAcumulado;
    //        rangoVacac.diasVacaciones = this.diasVacaciones;
    //        rangoVacac.diasDisfrute = this.diasDisfrute;
    //        rangoVacac.state = ChangeState.ADDED;
    //        this.cvTransacciones.commitNew();
    //        this.dlgRango.hide();
    //        this.context.app.hideSpinner();
    //    }
    //}

    onClickRangoAceptar() {
        
        let newRangos: RangoVacacionalInfo[] = this.cvRangos.sourceCollection.slice();
        this.currentRango.state = 'MODIFIED';
        this.currentRango.idClaseNomina = this.idClaseNomina;
        if (this.currentRango._clientID == 0) {
            this.currentRango._clientID = this.cvRangos.sourceCollection.length + 1;
            newRangos.push(this.currentRango);
        }
        else {
            let index: number = newRangos.findIndex(item => item._clientID == this.currentRango._clientID);
            newRangos[index] = this.currentRango;
        }
        //this.producto.insumos = newInsumos;
        this.cvRangos.sourceCollection = newRangos;
        this.dlgRango.hide();
    }


    copyRango(original: RangoVacacionalInfo): RangoVacacionalInfo {
        return JSON.parse(JSON.stringify(original));
    }

    ////////////////////////////////////////////////////////
    validarNew(): boolean
    {
        let existAnio = this.cvRangos.items.find(rango => rango.anios == this.currentRango.anios)

        if (existAnio != null || existAnio != undefined) {
            this.alertsEdit.showAlert("Un rango para esa cantidad de años ya existe en los elementos de esta clase de nómina", 'warning');
            return false;
        }

        if (this.currentRango.dias > this.currentRango.diasAcumulado) {
            this.alertsEdit.showAlert("No puede ser mayor los dias de inicio a los dias acumulados", 'warning');
            return false;
        }

        return true;
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

    private handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}