﻿//Frameworks
import { Component, ViewChild, forwardRef, SimpleChange, OnChanges, Renderer, ElementRef } from '@angular/core';
import { FormsModule, FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';

import { ModalDirective } from 'ngx-bootstrap'
import { Menu } from 'wijmo/wijmo.input';
import { Control, CollectionView } from 'wijmo/wijmo';

//moment
import * as moment from 'moment';

//Wijmo
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';

//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { FileInfo } from 'framework3/general/models/file';

import { ToolbarButton, AlertsBarComponent } from 'framework3/controls/index';

import { NominaAdamInfo, SustitucionInfo } from '../models/recibo-nomina-adam';

//Features
import { MenuItem } from "framework3/controls";
import { ReciboAdamService } from '../services/recibo-adam.service';
import { ReciboNominaService } from '../services/recibo-nomina.service';
import { ProgramacionReciboAdamService } from '../services/programacion-recibo-adam.service'

import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';

@Component({
    selector: 'recibos-adam-maintenance',
    templateUrl: './recibos-adam.maintenance.html'
})
export class ReciboAdamMaintenance extends BaseComponent {

    editForm: FormGroup;
    titleDialog: string = "Recibo de n&oacute;mina"; 

    // Recibos individuales
    currentRecibos: Array<any> = new Array<any>();
    currentItem: any;
    fechaInicial: Date = new Date();
    fechaFinal: Date = new Date();

    // Programacion de nomina
    currentProgramacion: Array<any> = new Array<any>();
    currentItemProg: any;
    fechaInicialProg: Date = new Date();
    fechaFinalProg: Date = new Date();

    // Sustitucion de nomina
    collectionSustitucion: wjcCore.CollectionView;
    fechaInicialSustitucion: Date = new Date();
    fechaFinalSustitucion: Date = new Date();


    errorProgramacion: Array<any> = new Array<any>();

    errorDisplay: string = "";
    idProgramacion: string = "";
    timerInterval: any = null;
    titleContentView: string = "";
    fileNameContentView: string = "";
    errorDialog: boolean = false;
    errorServer: string = "";

    menuItems: MenuItem[] = [];
    menuItemsProg: MenuItem[] = [];

    request: SearchRequest;
    requestProg: SearchRequest;
    requestSustitucion: SearchRequest;

    totalRows: number = 0;
    pageIndex: number = 0;

    totalRowsProg: number = 0;
    pageIndexProg: number = 0;

    totalRowsSustitucion: number = 0;
    pageIndexSustitucion: number = 0;

    adamNomina: NominaAdamInfo = new NominaAdamInfo();
    presupuesto: any;

    toRefresh: any;

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;
    @ViewChild('dlgComprobante') dlgComprobante: ModalDirective;
    @ViewChild('dlgTipoComprobante') dlgTipoComprobante: ModalDirective;
    @ViewChild('dlgErrorComprobante') dlgErrorComprobante: ModalDirective;

    constructor(private context: Context, private router: Router, private route: ActivatedRoute, private location: Location,
        private formBuilder: FormBuilder, private reciboAdamService: ReciboAdamService, private reciboNominaSvc: ReciboNominaService, private programacionReciboSvc: ProgramacionReciboAdamService) {
        super();
        this.editForm = formBuilder.group({
            'fechaInicial': [''],
            'fechaFinal':[''],
            'adamNomina': [''],
            'presupuesto': [''],
            'concepto': [''],
            'empleado': [''],
            'fechaInicialProg': [''],
            'fechaFinalProg': [''],
            'fechaLiquidacion': [''],
            'numLiquidacion': [''],
            'uuid': [''],
            'proCancelar': [''],
            'proGenerar': [''],
            'proTimbrar': [''],
            'proEnviar': [''],
            'errorDisplay': [''],
            'fechaInicialSustitucion': [''],
            'fechaFinalSustitucion': ['']
        });
        this.adamNomina.idSucursal = this.context.app.defaultSite.identity;
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.refreshList();
            this.refreshListProg();
        });
        this.createMenu();
        this.createMenuProg();
        this.refreshList();
        this.refreshListProg();
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.errorDialog = true;
        this.errorServer = message;
        this.alerts.showAlert(message, 'danger');
    }

    onCurrentItemChanged(item: any) {
        this.currentItem = item;
    }

    onCurrentItemProgChanged(item: any) {
        this.currentItemProg = item;
    }

    refreshList() {
        this.context.app.showSpinner();

        if (this.request == null) {
            this.request = new SearchRequest();
            this.request.pageIndex = 0;
            this.request.pageSize = 20;
            this.request.paged = false;
        }

        let exist = false;
        this.request.conditions.forEach(r => {
            if (r.field == "fechaCreacion" && r.operator == ">=") {
                exist = true;
                r.value = this.fechaInicial;
            }
            else if (r.field == "fechaCreacion" && r.operator == "<=") {
                exist = true;
                r.value = this.fechaFinal;
            }
        });

        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fechaCreacion", operator: ">=", valueType: "datetime", value: this.fechaInicial };
            let conFechaFin: SearchCondition = { field: "fechaCreacion", operator: "<=", valueType: "datetime", value: this.fechaFinal };
            this.request.conditions.push(conFechaIni);
            this.request.conditions.push(conFechaFin);
        }

        this.reciboAdamService.search(this.context.app.defaultSite.identity, this.request)
            .then(response => {

                response.rows.forEach(r => {
                    r['fechaCreacion'] = moment(r['fechaCreacion']).toDate();
                });

                this.currentRecibos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = this.request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }


    refreshListSustucion() {
        this.context.app.showSpinner();

        if (this.requestSustitucion == null) {
            this.requestSustitucion = new SearchRequest();
            this.requestSustitucion.pageIndex = 0;
            this.requestSustitucion.pageSize = 20;
            this.requestSustitucion.paged = false;
        }

        let exist = false;
        this.requestSustitucion.conditions.forEach(r => {
            if (r.field == "fechaCreacion" && r.operator == ">=") {
                exist = true;
                r.value = moment(this.fechaInicialSustitucion).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
            else if (r.field == "fechaCreacion" && r.operator == "<=") {
                exist = true;
                r.value = moment(this.fechaFinalSustitucion).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
        });

        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fechaCreacion", operator: ">=", valueType: "datetime", value: moment(this.fechaInicialSustitucion).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conFechaFin: SearchCondition = { field: "fechaCreacion", operator: "<=", valueType: "datetime", value: moment(this.fechaFinalSustitucion).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            this.requestSustitucion.conditions.push(conFechaIni);
            this.requestSustitucion.conditions.push(conFechaFin);
        }

        this.reciboAdamService.search(this.context.app.defaultSite.identity, this.requestSustitucion)
            .then(response => {
                response.rows.forEach(r => {
                    r['fechaCreacion'] = moment(r['fechaCreacion']).toDate();
                    r['selected'] = false;
                });

                this.collectionSustitucion = new wjcCore.CollectionView(response.rows);
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }


    searchRequested(request: SearchRequest) {
        this.request = request;
        this.refreshList();
    }

    createMenu() {
        let mnuImprimir: MenuItem = { icon: "glyphicon glyphicon-print", text: "Imprimir", name: "IMPRIMIR", smallText: "Imprimir recibo de n&oacute;mina" };
        let mnuDescargar: MenuItem = { icon: "glyphicon glyphicon-download-alt", text: "Descargar", name: "DESCARGAR", smallText: "Descargar XML" };
        let mnuEnviar: MenuItem = { icon: "glyphicon glyphicon-envelope", text: "Enviar correo", name: "ENVIAR", smallText: "Enviar correo recibo" };
        let mnuTimbrar: MenuItem = { icon: "glyphicon glyphicon-qrcode", text: "Timbrar", name: "TIMBRAR", smallText: "Timbrar recibo" };
        let mnuSustituir: MenuItem = { icon: "glyphicon glyphicon-refresh", text: "Sustituir", name: "SUSTITUIR", smallText: "Sustituir recibo" };
        let mnuRemplazar: MenuItem = { icon: "glyphicon glyphicon-sort", text: "Remplazar", name: "REMPLAZAR", smallText: "Remplazar recibo" };
        let mnuCancelar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Cancelar", name: "CENCELAR", smallText: "Cancelar recibo" };
        let mnuError: MenuItem = { icon: "glyphicon glyphicon-warning-sign", text: "Error", name: "ERROR", smallText: "Ver error" };
        this.menuItems.push(mnuImprimir, mnuDescargar, mnuEnviar, mnuTimbrar, mnuSustituir, mnuRemplazar, mnuCancelar, mnuError);
        this.menuItems.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    createMenuProg() {
        let mnuCancelar: MenuItem = { icon: "glyphicon glyphicon-remove", text: "Cancelar", name: "CANCELAR", smallText: "Cancelar Programaci&oacute;n" };
        let mnuComparativo: MenuItem = { icon: "glyphicon glyphicon-search", text: "Comparativo", name: "COMPARATIVO", smallText: "Comparativo Adam" };
        let mnuDescargar: MenuItem = { icon: "glyphicon glyphicon-download-alt", text: "Descargar", name: "DESCARGAR", smallText: "Descargar recibos" };
        let mnuError: MenuItem = { icon: "glyphicon glyphicon-warning-sign", text: "Error", name: "ERROR", smallText: "Ver error" };

        this.menuItemsProg.push(mnuCancelar, mnuComparativo, mnuDescargar, mnuError);
        this.menuItemsProg.forEach(item => {
            if (item.enabled == undefined) item.enabled = true;
        });
    }

    optionClick(option: MenuItem) {
        if (option.name == "IMPRIMIR") {
            this.context.app.showSpinner();
            this.reciboNominaSvc.printReciboPdf(this.currentItem.data.idReciboNomina)
                .then(report => {
                    this.context.app.hideSpinner();
                    this.titleContentView = "Recibo " + this.currentItem.data.numTrabajador;
                    this.fileNameContentView = this.currentItem.data.numTrabajador + ".pdf";
                    this.reportViewer.show("data:application/pdf; base64, " + report);
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "DESCARGAR") {
            this.context.app.showSpinner();
            this.reciboNominaSvc.loadReciboXml(this.currentItem.data.idReciboNomina)
                .then(response => {
                    this.titleContentView = "Recibo " + this.currentItem.data.numTrabajador;
                    this.fileNameContentView = this.currentItem.data.numTrabajador + ".xml";
                    this.reportViewer.show("data:text/xml, " + response);
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "ENVIAR") {
            this.context.app.showSpinner();
            this.reciboAdamService.enviarRecibo(this.currentItem.data.id, this.context.app.defaultSite.identity)
                .then(report => {
                    this.context.app.hideSpinner();
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "TIMBRAR") {
            this.context.app.showSpinner();
            this.reciboAdamService.timbrarRecibo(this.currentItem.data.id)
                .then(response => {
                    this.context.app.hideSpinner();
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "ERROR") {
            this.context.app.showSpinner();
            this.reciboNominaSvc.getErrorTimbrado(this.currentItem.data.idReciboNomina).then(result => {
                this.errorDisplay = result;
                this.dlgErrorComprobante.config.backdrop = false;
                this.dlgErrorComprobante.show();
                this.context.app.hideSpinner();
            }).catch(error => this.handleError(error));

        }
        else if (option.name == "CENCELAR") {
            this.context.app.showSpinner();
            this.reciboAdamService.cancelarRecibo(this.currentItem.data.id)
                .then(response => {
                    this.context.app.hideSpinner();
                    this.refreshList();
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "SUSTITUIR") {
            this.context.app.showSpinner();
            this.adamNomina.concepto = "Sustitución de los CFDI previos";
            this.adamNomina.empleado = this.currentItem.data.numTrabajador;
            this.adamNomina.noLiquidacion = this.currentItem.data.numeroLiquidacion;
            this.adamNomina.tipo = this.currentItem.data.numeroLiquidacion > 0 ? 'F' : 'R';
            this.adamNomina.idSucursal = this.context.app.defaultSite.identity;
            this.adamNomina.nomina.claseNomina = this.currentItem.data.claseNomina;
            this.adamNomina.nomina.tipoNomina = this.currentItem.data.tipoNomina;
            this.adamNomina.nomina.anio = this.currentItem.data.anio;
            this.adamNomina.nomina.periodo = this.currentItem.data.periodo;
            this.adamNomina.nomina.compania = this.currentItem.data.compania;
            //
            let sustitucion = new SustitucionInfo();
            sustitucion.idReciboNominaAdam = this.currentItem.data.id;
            sustitucion.empleado = this.adamNomina.empleado;
            //
            this.adamNomina.sustituciones = new Array<SustitucionInfo>();
            this.adamNomina.sustituciones.push(sustitucion);
            //
            this.generarComprobante();
        }
        else if (option.name == "REMPLAZAR") {
            this.adamNomina.concepto = "Sustitución de los CFDI previos";
            this.adamNomina.empleado = this.currentItem.data.numTrabajador;
            this.adamNomina.noLiquidacion = this.currentItem.data.numeroLiquidacion;
            this.adamNomina.tipo = this.currentItem.data.numeroLiquidacion > 0 ? 'F' : 'R';
            this.adamNomina.idSucursal = this.context.app.defaultSite.identity;
            this.adamNomina.nomina.claseNomina = this.currentItem.data.claseNomina;
            this.adamNomina.nomina.tipoNomina = this.currentItem.data.tipoNomina;
            this.adamNomina.nomina.anio = this.currentItem.data.anio;
            this.adamNomina.nomina.periodo = this.currentItem.data.periodo;
            this.adamNomina.nomina.compania = this.currentItem.data.compania;
            //
            let sustitucion = new SustitucionInfo();
            sustitucion.idReciboNominaAdam = this.currentItem.data.id;
            sustitucion.empleado = this.adamNomina.empleado;
            //
            this.adamNomina.sustituciones = new Array<SustitucionInfo>();
            this.adamNomina.sustituciones.push(sustitucion);

            if (this.adamNomina.tipo == 'F') {
                this.onGenerarFiniquito(true);
            }
            else {
                this.onGenerarRecibo(true);
            }
        }
    }

    generarComprobante() {
        if (this.adamNomina.concepto.length == 0) {
            this.errorServer = "Concepto requerido";
            this.errorDialog = true;
        }
        else if ((this.adamNomina.tipo == "R" || this.adamNomina.tipo == "F") && (this.adamNomina.empleado == undefined || this.adamNomina.empleado.length == 0)) {
            this.errorServer = "Num. Empleado requerido";
            this.errorDialog = true;
        }
        else if (this.adamNomina.tipo == "F" && (this.adamNomina.noLiquidacion == undefined || !(this.adamNomina.noLiquidacion > 0))) {
            this.errorServer = "Num. liquidación requerido";
            this.errorDialog = true;
        }
        else if (this.adamNomina.nomina.compania == undefined || this.adamNomina.nomina.compania == null || this.adamNomina.nomina.compania.length == 0) {
            this.errorServer = "Seleccione la compañía";
            this.errorDialog = true;
        }
        else if (this.adamNomina.nomina.claseNomina == undefined || this.adamNomina.nomina.claseNomina == null || this.adamNomina.nomina.claseNomina.length == 0) {
            this.errorServer = "Seleccione la clase de nómina";
            this.errorDialog = true;
        }
        else if (this.adamNomina.nomina.tipoNomina == undefined || this.adamNomina.nomina.tipoNomina == null || this.adamNomina.nomina.tipoNomina.length == 0) {
            this.errorServer = "Seleccione la compañía";
            this.errorDialog = true;
        }
        else if (this.adamNomina.tipo != "F" && (this.adamNomina.nomina.anio == undefined || this.adamNomina.nomina.anio == null || !(this.adamNomina.nomina.anio > 0))) {
            this.errorServer = "Seleccione el año";
            this.errorDialog = true;
        }
        else if (this.adamNomina.tipo != "F" && (this.adamNomina.nomina.periodo == undefined || this.adamNomina.nomina.periodo == null || !(this.adamNomina.nomina.periodo > 0))) {
            this.errorServer = "Seleccione el periodo";
            this.errorDialog = true;
        }
        else if (this.adamNomina.tipo == "F" && (this.adamNomina.noLiquidacion == undefined || this.adamNomina.noLiquidacion == null || !(this.adamNomina.noLiquidacion > 0))) {
            this.errorServer = "Ingrese el número de liquidación";
            this.errorDialog = true;
        }
        else if ((this.adamNomina.tipo == "R" || this.adamNomina.tipo == "F") && (this.adamNomina.empleado == undefined || this.adamNomina.empleado == null || this.adamNomina.empleado.length == 0)) {
            this.errorServer = "Ingrese el número de empleado";
            this.errorDialog = true;
        }
        else {
            if (this.adamNomina.tipo == "R" || this.adamNomina.tipo == "F") {
                this.context.app.showSpinner();
                this.reciboAdamService.generarRecibo(this.context.app.defaultSite.identity, this.adamNomina)
                    .then(response => {
                        this.context.app.hideSpinner();
                        this.dlgComprobante.hide();
                        this.refreshList();
                        this.refreshListProg();
                    })
                    .catch(error => this.handleError(error));
            }
            else {
                this.context.app.showSpinner();
                this.programacionReciboSvc.generarProgramacion(this.context.app.defaultSite.identity, this.adamNomina)
                    .then(response => {
                        this.context.app.hideSpinner();
                        this.dlgComprobante.hide();
                        this.refreshList();
                        this.refreshListProg();
                    })
                    .catch(error => this.handleError(error));
            }
        }
    }

    cancelarComprobante() {
        this.dlgComprobante.hide();
    }

    // Eventos cliente
    agregarComprobante() {
        this.dlgTipoComprobante.config.backdrop = false;
        this.dlgTipoComprobante.show();
    }

    onPresupuestoSelected(args: any) {
        this.dlgTipoComprobante.config.backdrop = false;
        this.dlgTipoComprobante.show();
        this.dlgTipoComprobante.hide();
    }

    onCancelar() {
        this.dlgTipoComprobante.config.backdrop = false;
        this.dlgTipoComprobante.hide();
        this.dlgTipoComprobante.hide();
    }

    onGenerarFiniquito(sustitucion: boolean) {
        this.adamNomina.tipo = "F";
        if (!sustitucion) {
            this.adamNomina.sustituciones = null;
        }
        this.titleDialog = "Recibo Finiquito";
        this.errorDialog = false;
        this.errorServer = "";
        this.dlgComprobante.config.backdrop = false;
        this.dlgComprobante.show();
        this.dlgTipoComprobante.hide();
    }

    onGenerarRecibo(sustitucion: boolean) {
        this.adamNomina.tipo = "R";
        if (!sustitucion) {
            this.adamNomina.sustituciones = null;
        }
        this.titleDialog = "Recibo N&oacute;mina";
        this.errorDialog = false;
        this.errorServer = "";
        this.dlgComprobante.config.backdrop = false;
        this.dlgComprobante.show();
        this.dlgTipoComprobante.hide();
    }

    // Programación de la nómina

    searchRequestedProg(request: SearchRequest) {
        this.requestProg = request;
        this.refreshListProg();
    }

    refreshListProg() {
        this.context.app.showSpinner();

        if (this.requestProg == null) {
            this.requestProg = new SearchRequest();
            this.requestProg.pageIndex = 0;
            this.requestProg.pageSize = 20;
            this.requestProg.paged = false;
        }

        let exist = false;
        this.requestProg.conditions.forEach(r => {
            if (r.field == "fechaSys" && r.operator == ">=") {
                exist = true;
                r.value = moment(this.fechaInicialProg).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
            else if (r.field == "fechaSys" && r.operator == "<=") {
                exist = true;
                r.value = moment(this.fechaFinalProg).format("YYYY-MM-DD") + "T00:00:00-00:00";
            }
        });

        if (!exist) {
            let conFechaIni: SearchCondition = { field: "fechaSys", operator: ">=", valueType: "datetime", value: moment(this.fechaInicialProg).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            let conFechaFin: SearchCondition = { field: "fechaSys", operator: "<=", valueType: "datetime", value: moment(this.fechaFinalProg).format("YYYY-MM-DD") + "T00:00:00-00:00" };
            this.requestProg.conditions.push(conFechaIni);
            this.requestProg.conditions.push(conFechaFin);
        }

        this.programacionReciboSvc.search(this.context.app.defaultSite.identity, this.requestProg)
            .then(response => {

                response.rows.forEach(r => {
                    r['fechaSys'] = moment(r['fechaSys']).toDate();
                });

                this.currentProgramacion = response.rows;
                this.totalRowsProg = response.totalRows;
                this.pageIndexProg = this.requestProg.pageIndex;

                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    optionClickProg(option: MenuItem) {
        if (option.name == "CANCELAR") {
            this.context.app.showSpinner();
            this.programacionReciboSvc.cancelarProgramacion(this.currentItemProg.data.id)
                .then(report => {
                    this.refreshListProg()
                    this.context.app.hideSpinner();
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "COMPARATIVO") {
            this.context.app.showSpinner();
            this.programacionReciboSvc.generarComparativoPDF(this.currentItemProg.data.id)
                .then(report => {
                    this.context.app.hideSpinner();
                    this.titleContentView = "Comparativo";
                    this.fileNameContentView = this.currentItemProg.data.id + ".pdf";
                    this.reportViewer.show("data:application/pdf; base64, " + report);
                })
                .catch(error => this.handleError(error));

        }
        else if (option.name == "DESCARGAR") {
            this.context.app.showSpinner();
            this.programacionReciboSvc.generarDescargaComprobante(this.currentItemProg.data.id)
                .then(result => {
                    this.idProgramacion = result;
                    this.timerInterval = setInterval(() => {
                        this.verificarTaskDownload();
                    }, 60000);
                })
                .catch(error => {
                    this.handleError(error);
                });
        }
        else if (option.name == "ERROR") {
            this.errorDisplay = this.currentItemProg.data.errorTimbrado;
            this.dlgErrorComprobante.config.backdrop = false;
            this.dlgErrorComprobante.show();
        }
    }

    optionClickSustitucion(option: MenuItem) {
        if (option.name == "UNDEFINED") {

        }
    }

    // Eventos cliente
    agregarProgramacion() {
        this.titleDialog = "Programaci&oacute;n de N&oacute;mina";
        this.adamNomina.sustituciones = null;
        this.adamNomina.tipo = "P";
        this.errorDialog = false;
        this.errorServer = "";
        this.dlgComprobante.config.backdrop = false;
        this.dlgComprobante.show();
    }

    agregarProgramacionSustitucion() {
        this.agregarProgramacion();
        this.adamNomina.sustituciones = new Array<SustitucionInfo>();
        this.collectionSustitucion.items.forEach(item => {
            if (item.selected) {
                let sustitucion = new SustitucionInfo();
                sustitucion.empleado = item.numTrabajador;
                sustitucion.idReciboNominaAdam = item.id;
                this.adamNomina.sustituciones.push(sustitucion);
            }
        });
    }

    onProcesoCancelar(args: any) {
        if (args) {
            this.adamNomina.proGenerar = false;
            this.adamNomina.proTimbrar = false;
            this.adamNomina.proEnviar = false;
        }
    }

    // Eventos html
    onCloseDialogComprobante() {
        this.dlgComprobante.hide();
    }

    onCloseDialogErrorComprobante() {
        this.dlgErrorComprobante.hide();
    }

    // Task
    verificarTaskDownload() {
        let self = this;
        this.programacionReciboSvc.getJobDescargaComprobante(this.idProgramacion)
            .then(response => {
                if (response.completed) {
                    clearInterval(this.timerInterval);
                    
                    this.programacionReciboSvc.getReciboToDownloadPath(this.idProgramacion)
                        .then(result => {
                            this.alerts.showAlert("Descarga completa, verifica su carpeta de descarga", 'danger');
                            this.context.app.hideSpinner();
                            window.open(result.toString())
                        })
                        .catch(error => this.handleError(error));

                }
                else if (response.errorCount > 0) {
                    clearInterval(this.timerInterval);
                    this.context.app.hideSpinner();
                    this.errorDisplay = response.errorMessage;
                    this.dlgErrorComprobante.config.backdrop = false;
                    this.dlgErrorComprobante.show();
                }
            })
            .catch(error => {
                clearInterval(this.timerInterval);
                this.handleError(error)
            });
    }

    // Sustitucion

    onClickSeleccionar() {
        if (this.collectionSustitucion != null) {
            this.collectionSustitucion.items.forEach(item => {
                item.selected = true;
            });
            this.collectionSustitucion.commitEdit();
            if (this.toRefresh) {
                clearTimeout(this.toRefresh);
            }
            var self = this;
            this.toRefresh = setTimeout(function () {
                self.collectionSustitucion.refresh();
            }, 500);
        }
    }

    onClickInvertir() {
        if (this.collectionSustitucion != null) {
            this.collectionSustitucion.items.forEach(item => {
                item.selected = !item.selected;
            });
            this.collectionSustitucion.commitEdit();
            if (this.toRefresh) {
                clearTimeout(this.toRefresh);
            }
            var self = this;
            this.toRefresh = setTimeout(function () {
                self.collectionSustitucion.refresh();
            }, 500);
        }
    }

}