﻿import { Context, CatalogInfo, CatalogItem, TrackableItem } from 'framework3/core';
import { ContribuyenteInfo } from '../../accounting/models/contribuyente';
import { PersonaInfo } from 'framework3/general/models/persona';
import { FileInfo } from 'framework3/general/models/file';
import { CompaniaInfo } from './compania';
import { ParametroItem } from './parametro';
import { IParametrizable } from 'framework3/general/models/parametrizable';
import { ParametroValor } from 'framework3/general/models/parametro-valor';
import { DocumentoTipoInfo } from './documento-tipo';

export class EmpleadoInfo extends CatalogInfo implements IParametrizable {
	sexo: string;
    fechaNacimiento: Date;
	titulo: string;
	fechaIngreso: Date;
	idPlaza: number;
	imss: string;
	curp: string;
	rfc: string;
	sueldo: number;
	hrsExt: boolean;
	idTurno: number;
    foto: FileInfo;
	idPersona: number;
    idBeneficiario: number;
    idCompania: number;
    idCuenta: number;
    idTipoRegimen?: number;
    idClaseNomina?: number;
    idTipoContrato?: number;
    sindicalizado: boolean;
    persona: PersonaInfo;
    contribuyente: ContribuyenteInfo;
    compania?: CompaniaInfo;
    departamento?: string;
    puesto?: string;
    riesgoPuesto?: string;
    periodicidadPago?: string;
    claveEntFed?: string;
    claveTipoContrato?: string;
    parametros: ParametroValor[];
    documentos: DocumentoEmpleadoItem[];

    constructor() {
        super();
        this.persona = new PersonaInfo();
        this.contribuyente = new ContribuyenteInfo();
        this.parametros = [];
        this.documentos = [];
    }
}

export class EmpleadoItem implements CatalogItem {    
    id: number;
    codigo: string;
    nombre: string;
    departamento?: string;
    activo: boolean;
    estatus?: string;
	selected?:boolean = false;
}

export class DocumentoEmpleadoItem extends TrackableItem {

    idDocumentoTipo: number;
    idArchivo: string;
    descripcion: string;
    proceso: string;
    documentoTipo: DocumentoTipoInfo;
    archivo: FileInfo;

    constructor() {
        super();
    }

}