﻿import { DocumentInfo, DocumentItem, TrackableItem } from 'framework3/core';
import { ReciboNominaDetail } from './recibo-nomina-detail';

export class ReciboNominaInfo extends DocumentInfo<ReciboNominaDetail> {
	fechaAfectacion: Date;
	idComprobante: string;
	fechaComprobante: Date;
    tipoNomina: string;
	idEmpleado: number;
	percepcionGravado: number;
    percepcionExento: number;
    otrosPagos: number;
	deduccion: number;
	subTotal: number;
    descuento: number;
    total: number;
    numDiasPago: number;
	fechaPago: Date;
	fechaInicialPago: Date;
    fechaFinalPago: Date;
    idBanco: number;
    lugarExpedicion: string;
	idCuentaPorPagar: string;
    fechaCuentaPorPagar: Date;
    idDivisa: number;
    tipoCambio: number;
    idDocumento: string;
    fechaDocumento: string;
    idCFDSerie: number;


    emisor: EmisorInfo = new EmisorInfo();
    receptor: ReceptorInfo = new ReceptorInfo();
    relaciones: RelacionInfo[] = [];
    subContrataciones: SubContratacionInfo[] = [];

    constructor() {
        super();
        this.fechaPago = new Date();
        this.fechaInicialPago = new Date();
        this.fechaFinalPago = new Date();
    }
}

export class ReciboNominaItem extends DocumentItem {

    fechaPago?: Date;
    idComprobante?: string;
    numTrabajador?: string;
}

export class EmisorInfo {

    rfc: string;
    nombre: string;
    regimenFiscal: string;
    curp: string;
    registroPatronal: string;
    origenRecurso: string;
    montoRecursoPropio: number;
    rfcPatronOrigen: string;

}

export class ReceptorInfo {

    fechaInicioRelLaboral: Date;
    antiguedad: string;
    tipoContrato: string;
    sindicalizado: boolean;
    tipoJornada: string;
    tipoRegimen: string;
    numEmpleado: string;
    departamento: string;
    puesto: string;
    riesgoPuesto: string;
    periodicidadPago: string;
    banco: string;
    cuentaBancaria: string;
    salarioBaseCotApor: number;
    salarioDiarioIntegrado: number;
    claveEntFed: string

}

export class RelacionInfo extends TrackableItem {

    idComprobante: string;
    fechaComprobante: Date;
    uuid: string;
    tipoRelacion: string;

}

export class SubContratacionInfo extends TrackableItem {

    rfcLabora: string;
    porcentajeTiempo: number;

}