﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';

import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';

//moment
import * as moment from 'moment';

import { Context, ProcessComponent, PolicyInfo } from "framework3/core";
import { AztecaValidators, ContentViewerComponent, AlertsBarComponent, MenuItem } from 'framework3/controls';
import { EditableListComponent } from 'framework3/editors';

import { EmpleadoService } from '../services/empleado.service';
import { DocumentoEmpleadoItem } from '../models/empleado';
import { DocumentoTipoListComponent } from '../controls/documentos-tipo-list.component';
import { DocumentoTipoInfo, DocumentoTipoItem } from '../models/documento-tipo';


@Component({
    selector: 'rh-documento-empleado',
    templateUrl: './documento-empleado.component.html'
})
export class DocumentoEmpleadoComponent extends ProcessComponent {

    formDocumento: FormGroup;

    idEmpleado: number = 0;
    proceso: string = '';
    titleContentView: string = "";
    fileNameContentView: string = "";

    documentos: DocumentoEmpleadoItem[] = [];
    currentDocumento: DocumentoEmpleadoItem = new DocumentoEmpleadoItem();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('dlgDocumento') dlgDocumento: ModalDirective;
    @ViewChild('dlgDocmentoTipo') dlgDocmentoTipo: DocumentoTipoListComponent;
    @ViewChild('editListDocumentos') editListDocumentos: EditableListComponent;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    constructor(
        context: Context,
        router: Router,
        route: ActivatedRoute,
        builder: FormBuilder,
        private location: Location,
        private empleadoService: EmpleadoService
    ) {
        super(context, router, route);
        this.formDocumento = builder.group({
            'idDocumentoTipo': [null, AztecaValidators.selected],
            'archivo': [null, AztecaValidators.validFile],
            'descripcion': ['']
        });
    }

    ngOnInit(): void {
        setTimeout(() => {
            this.route.params.forEach(param => {

                this.idEmpleado = +param['id'];
            });
            this.route.queryParams.forEach(param => {
                this.proceso = param['process'] || '';
            })
            console.log(this.proceso);
            this.loadDocumentos();

        })
    }

    loadDocumentos() {
        setTimeout(() => {
            this.empleadoService.getDocumentos(this.idEmpleado)
                .then(items => {
                    this.documentos = items.filter(x=>x.proceso==this.proceso);
                })
                .catch(error => this.handleError(error));
        });
    }

    onAddingItemDocumento() {
        this.dlgDocmentoTipo.show();
    }

    onListItemUpdateDocumento(items: DocumentoEmpleadoItem[]) {
        this.documentos = items;
    }

    onDeletingDocumento(item: DocumentoEmpleadoItem) {
        this.context.app.showSpinner();
        this.empleadoService.deleteDocumento(item.id)
            .then(result => {
                this.context.app.hideSpinner();
                this.editListDocumentos.deleteItem(item);
            })
            .catch(error => this.handleError(error));
    }

    onSelectedItemDocumentoTipo(item: DocumentoTipoItem) {
        if (this.documentos.filter(X => X.idDocumentoTipo == item.id && X.state != 'DELETED').length == 0) {
            this.currentDocumento = new DocumentoEmpleadoItem();
            this.currentDocumento._clientID = this.createClientID();
            this.currentDocumento.idDocumentoTipo = item.id;
            this.dlgDocumento.config.backdrop = false;
            this.dlgDocumento.show();
        } else {
            this.alerts.showAlert('Ya existe un documento de ese tipo', 'danger');
        }
    }

    onUpdatedDocumento() {
        this.context.app.showSpinner();
        this.empleadoService.updateDocumento(this.idEmpleado, this.currentDocumento)
            .then(result => {
                this.context.app.hideSpinner();
                this.dlgDocumento.hide();
                this.loadDocumentos();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedDocumentoTipo(item: DocumentoTipoInfo) {
        this.currentDocumento.documentoTipo = item;
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PREVIEW') {
            let documento = <DocumentoEmpleadoItem>this.editListDocumentos.currentItem;
            if (documento) {
                this.context.app.showSpinner();
                this.empleadoService.getDocumento(documento.idArchivo)
                    .then(item => {
                        this.context.app.hideSpinner();
                        this.titleContentView = "Documento " + item.originalFileName;
                        this.fileNameContentView = item.id + "." + item.extension;
                        this.reportViewer.show('/file-server/' + item.fileUri);
                    })
                    .catch(e => this.handleError(e));
            }
        }
    }

    onCancel() {
        this.location.back();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    createClientID(): number {
        return parseInt((Math.random() * 1000).toString()) * 1000 + parseInt(moment().format("HHmmss"))
    }

    cloneInfo<T>(element: any): T {
        return <T>JSON.parse(JSON.stringify(element));
    }

    // Implementation Process
    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

}