﻿import { Component, ViewChild } from "@angular/core";
import { ProcessComponent, PolicyInfo, Context, ReportService, ReporteInfo, ParametroInfo } from "framework3/core";
import { Router, ActivatedRoute } from "@angular/router";
import { FormBuilder, FormGroup, FormControl } from "@angular/forms";
import { AztecaValidators, ContentViewerComponent, AlertsBarComponent } from "framework3/controls";
import { Location } from '@angular/common';

@Component({
    selector: 'rh-empleados-report',
    templateUrl: './empleados-report.component.html'
})
export class EmpleadosReportComponent extends ProcessComponent {

    catalogForm: FormGroup;
   
    title: string = 'Reporte de Empleados'

    idDepartamento: number = 0;
    idTurno: number = 0;
    filtroDepartamento: boolean = false;
    filtroTurno: boolean = false;
    incluirInactivos: boolean = false;
    valid: boolean = true;

    titleContentView: string = "";
    fileNameContentView: string = "";

    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;


    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private reportSvc: ReportService) {
        super(context, router, route);

        this.catalogForm = formBuilder.group({
            'filtroDepartamento': [0],
            'filtroTurno': [0],
            'departamento': [0],
            'turno': [0],
            incluirInactivos: [0]
        });
    }

    ngAfterViewInit() {
        this.catalogForm.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }

    changeState() {
        this.valid = this.catalogForm.valid;
    }

    onDepartamentoChecekd(args: any) {
        console.log(args);
        if (args) 
            this.catalogForm.controls['departamento'].setValidators(AztecaValidators.selected);
        else
            this.catalogForm.controls['departamento'].setValidators(null);

        this.catalogForm.controls['departamento'].updateValueAndValidity();
    }

    onTurnoChecekd(args: any) {
        console.log(args);
        if (args)
            this.catalogForm.controls['turno'].setValidators(AztecaValidators.selected);
        else
            this.catalogForm.controls['turno'].setValidators(null);

        this.catalogForm.controls['turno'].updateValueAndValidity();
    }

    print() {
        this.context.app.showSpinner();
        let informe = new ReporteInfo();

        let parametroIdSucursal = new ParametroInfo();
        let parametroIdDepartamento = new ParametroInfo();
        let parametroIdTurno = new ParametroInfo();
        let parametroIncluirBajas = new ParametroInfo();

        parametroIdSucursal.name = "IdSucursal";
        parametroIdSucursal.values.push(this.context.app.defaultSite.identity.toString());

        parametroIncluirBajas.name = 'IncluirBajas'
        parametroIncluirBajas.values.push(this.incluirInactivos ? 'true' : 'false');


        parametroIdDepartamento.name = "IdDepartamento"
        if (this.filtroDepartamento)
            parametroIdDepartamento.values.push(this.idDepartamento.toString());
        else
            parametroIdDepartamento.values.push('0');

        parametroIdTurno.name = "IdTurno"
        if (this.filtroTurno)
            parametroIdTurno.values.push(this.idTurno.toString());
        else
            parametroIdTurno.values.push('0');


        informe.parametros.push(parametroIdSucursal);
        informe.parametros.push(parametroIdDepartamento);
        informe.parametros.push(parametroIncluirBajas);
        informe.parametros.push(parametroIdTurno);


        informe.path = "/Onili/RH/EmpleadosRPT";
        informe.format = "pdf";
        this.reportSvc.loadReporte(informe)
            .then(info => {
                this.context.app.hideSpinner();
                this.titleContentView = 'Incidencias'
                this.fileNameContentView = "Incidencias.pdf";
                this.reportViewer.show("data:application/pdf; base64, " + info.content);
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error);
            });
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }
}