﻿import { Component, ViewChild } from "@angular/core";
import { ProcessComponent, PolicyInfo, Context, ReportService, SucursalItem, EmpresaItem, ReporteInfo, ParametroInfo } from "framework3/core";
import { FormGroup, FormBuilder, Validators, FormControl } from "@angular/forms";
import { Router, ActivatedRoute } from "@angular/router";
import { AztecaValidators, ContentViewerComponent, AlertsBarComponent } from "framework3/controls";
import { Location } from '@angular/common';
import moment = require("moment");
import { IncidenciaListComponent } from '../controls/incidencia-list.component';
import { IncidenciaItem } from "../models/incidencia";
import * as wjcCore from 'wijmo/wijmo';

@Component({
    selector: 'rh-incidencias-empleado-report',
    templateUrl: './incidencias-empleado-report.component.html'
})
export class IncidenciasEmpleadoReportComponent extends ProcessComponent {
    

    catalogForm: FormGroup;
    tipoReporte: FormControl;
    tipoFiltro: FormControl;

    title: string = 'Reporte Incidencias de Empleado'

    fechaIni: Date;
    fechaFin: Date;
    idEmpresaAdam: number = 0;
    idDepartamento: number = 0;
    idEmpleado: number = 0;
    valid: boolean = false;
    todasIncidencias: boolean = false;
    incluirInactivos: boolean = false;

    titleContentView: string = "";
    fileNameContentView: string = "";

    @ViewChild('lstDlgIncidencia') lstDlgIncidencia: IncidenciaListComponent;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;
    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    cvIncidencias: wjcCore.CollectionView = new wjcCore.CollectionView();
    incidencias: IncidenciaItem[] = [];
    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private reportSvc: ReportService) {
        super(context, router, route);

        this.catalogForm = formBuilder.group({
            'empresaAdam': [0, AztecaValidators.selected],
            'tipoReporte': [''],
            'tipoFiltro': [''],
            'departamento': [0, AztecaValidators.selected],
            'empleado': [0],
            'fechaIni': [null, Validators.required],
            'fechaFin': [null, Validators.required],
            'todasIncidencias': [false],
            'incluirInactivos': [false]
        });

        this.tipoReporte = <FormControl>this.catalogForm.controls['tipoReporte'];
        this.tipoReporte.setValue('I');

        this.tipoFiltro = <FormControl>this.catalogForm.controls['tipoFiltro'];
        this.tipoFiltro.setValue('D');

        this.catalogForm.controls['tipoFiltro'].valueChanges.subscribe(value => {
            if (value == 'D') {
                this.catalogForm.controls['departamento'].setValidators(AztecaValidators.selected);
                this.catalogForm.controls['empleado'].setValidators(null);
            } else if (value == 'E') {
                this.catalogForm.controls['departamento'].setValidators(null);
                this.catalogForm.controls['empleado'].setValidators(AztecaValidators.selected);
            }
            this.catalogForm.controls['departamento'].updateValueAndValidity();
            this.catalogForm.controls['empleado'].updateValueAndValidity();
        })
    }

    ngAfterViewInit() {
        setTimeout(_ => {
            this.fechaIni = moment().subtract(8, 'day').toDate();
            this.fechaFin = moment().subtract(1, 'day').toDate();
        });

        this.catalogForm.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });

        this.catalogForm.statusChanges.subscribe((value) => {
            setTimeout(_ => this.changeState())
        });
    }

    changeState() {
        this.valid = this.catalogForm.valid;
    }

    onAgregarIncidencias() {
        this.lstDlgIncidencia.show();
    }

    onSelectedIncidencia(item: IncidenciaItem) {
        if (this.incidencias.filter(X => X.id == item.id).length == 0) {
            this.incidencias.push(item);
        }
        this.cvIncidencias.sourceCollection = this.incidencias;
        this.cvIncidencias.refresh();
        wjcCore.Control.invalidateAll();
    }

    onSelectedIncidencias(items: IncidenciaItem[]) {
        items.forEach(item => {
            if (this.incidencias.filter(X => X.id == item.id).length == 0) {
                this.incidencias.push(item);
            }
        });
        this.cvIncidencias.sourceCollection = this.incidencias;
        this.cvIncidencias.refresh();
        wjcCore.Control.invalidateAll();
    }

    onRowDeleteIncidencia(item: IncidenciaItem) {
        this.incidencias.splice(this.incidencias.indexOf(item), 1);
        this.cvIncidencias.sourceCollection = this.incidencias;
        this.cvIncidencias.refresh();
    }

    print() {
        this.context.app.showSpinner();
        let informe = new ReporteInfo();

        let parametroFechaIni = new ParametroInfo();
        let parametroFechaFin = new ParametroInfo();
        let parametroIdSucursal = new ParametroInfo();
        let parametroIdEmpleado = new ParametroInfo();
        let parametroIdEmpresaAdam = new ParametroInfo();
        let parametroIdEmpresa = new ParametroInfo();
        let parametroIdDepartamento = new ParametroInfo();
        let parametroIdIncidencia = new ParametroInfo();
        let parametroIncluirInactivos = new ParametroInfo();
        let parametroTipoReporte = new ParametroInfo();
        let parametroFiltroIncidencias = new ParametroInfo();

        parametroFechaIni.name = "FechaIni";
        parametroFechaIni.values.push(this.fechaIni.toDateString());

        parametroFechaFin.name = "FechaFin"
        parametroFechaFin.values.push(this.fechaFin.toDateString());

        parametroIdSucursal.name = "IdSucursal";
        parametroIdSucursal.values.push(this.context.app.defaultSite.identity.toString());

        parametroIdEmpleado.name = "IdEmpleado"
        if(this.tipoFiltro.value=='E')
            parametroIdEmpleado.values.push(this.idEmpleado.toString());
        else
            parametroIdEmpleado.values.push('0');

        parametroIdEmpresaAdam.name = "IdEmpresaAdam";
        parametroIdEmpresaAdam.values.push(this.idEmpresaAdam.toString());

        parametroIdDepartamento.name = "IdDepartamento"
        if (this.tipoFiltro.value == 'D')
            parametroIdDepartamento.values.push(this.idDepartamento.toString());
        else
            parametroIdDepartamento.values.push('0');

        parametroIdEmpresa.name = "IdEmpresa";
        parametroIdEmpresa.values.push(this.context.app.defaultCompany.identity.toString());

        parametroIdIncidencia.name = "IdIncidencia";
        parametroIdIncidencia.values.push('0');

        parametroIncluirInactivos.name = "IncluirInactivos";
        parametroIncluirInactivos.values.push(this.incluirInactivos?'true':'false');

        parametroTipoReporte.name = "TipoReporte";
        if(this.tipoReporte.value=='I' && this.tipoFiltro.value=='D')
            parametroTipoReporte.values.push('1');
        else if (this.tipoReporte.value == 'I' && this.tipoFiltro.value == 'E')
            parametroTipoReporte.values.push('2');
        else if (this.tipoReporte.value == 'S' && this.tipoFiltro.value == 'D')
            parametroTipoReporte.values.push('3');
        else if (this.tipoReporte.value == 'S' && this.tipoFiltro.value == 'E')
            parametroTipoReporte.values.push('4');

        parametroFiltroIncidencias.name = "FiltroIncidencias"
        let incidencias: string = '';
        this.incidencias.forEach(item => {
            incidencias = incidencias + ','+item.id.toString();
        });
        incidencias = incidencias.slice(1, incidencias.length);
        parametroFiltroIncidencias.values.push(incidencias)

        informe.parametros.push(parametroFechaIni);
        informe.parametros.push(parametroFechaFin);
        informe.parametros.push(parametroIdSucursal);
        informe.parametros.push(parametroIdEmpleado);
        informe.parametros.push(parametroIdEmpresaAdam);
        informe.parametros.push(parametroIdDepartamento);
        informe.parametros.push(parametroIdEmpresa);
        informe.parametros.push(parametroIdIncidencia);
        informe.parametros.push(parametroIncluirInactivos);
        informe.parametros.push(parametroTipoReporte);
        informe.parametros.push(parametroFiltroIncidencias);

        informe.path = "/Onili/RH/IncidenciasEmpleado";
        informe.format = "pdf";
        this.reportSvc.loadReporte(informe)
            .then(info => {
                this.context.app.hideSpinner();
                this.titleContentView = 'Incidencias'
                this.fileNameContentView = "Incidencias.pdf";
                this.reportViewer.show("data:application/pdf; base64, " + info.content);
            })
            .catch(error => {
                this.context.app.hideSpinner();
                this.handleError(error);
            });
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
        throw new Error("Method not implemented.");
    }

}