﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';

import { AlertsBarComponent, AztecaValidators, ContentViewerComponent } from "framework3/controls";
import { Context, ProcessComponent, PolicyInfo } from 'framework3/core';

import { KardexAsistenciaService } from '../services/kardex-asistencia.service';

@Component({
    selector: 'rh-tarjeta-asistencia-report',
    templateUrl: './tarjeta-asistencia-report.component.html'
})
export class TarjetaAsistenciaReportComponent extends ProcessComponent {

    catalogForm: FormGroup;

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    title:string  = "Reporte Tarjeta de Asistencias";
    idEmpleado: number = 0;
    idDepartamento: number = 0;
    fechaIni: Date = new Date();
    fechaFin: Date = new Date();
    activo: boolean = true;
    pie: string = "";

    titleContentView: string = "Tarjeta de Asistencias";
    fileNameContentView: string = "TarjetaAsistencia.pdf";

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        private kardexAsistenciaSvc: KardexAsistenciaService) {
        super(context, router, route);

        this.catalogForm = formBuilder.group({
            'idEmpleado': [null, AztecaValidators.selected],
            'idDepartamento': [null, AztecaValidators.selected],
            'fechaIni': [null],
            'fechaFin': [null],
            'activo': [false],
            'pie': ['']
        });
    }

    ///////////////////////// METHODS ///////////////////////////////////////
    printTarjetaAsistencia() {
        this.context.app.showSpinner();
        this.kardexAsistenciaSvc.printKardexAsistencia(this.idEmpleado, this.idDepartamento, this.fechaIni, this.fechaFin, (!this.activo), this.pie)
            .then(result => {
                this.context.app.hideSpinner();
                this.reportViewer.show("data:application/pdf; base64, " + result);
            })
            .catch(error => this.handleError(error));
    }

    ///////////////////////// HANDLERS ///////////////////////////////////////
    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }

    applyPolicy(policy: PolicyInfo): void {
    }
}