﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { CedulaFiniquitoInfo, RescisionLaboral, TarifaPagoProvicional, SubsidioEmpleo, AsyncTaskInfo } from '../models/cedulafiniquito'

@Injectable()
export class CedulaFiniquitoService extends CatalogService<CedulaFiniquitoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/rh/cedulasfiniquitos");
    }

    create(): Promise<CedulaFiniquitoInfo> {
        return Promise.resolve(new CedulaFiniquitoInfo());
    }

    loadIngresoInformacion(idEmpleado: number, calcular: boolean): Promise<CedulaFiniquitoInfo> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString())
            .set('calcualr', calcular.toString());
        return this.apiService.get<CedulaFiniquitoInfo>('load-ingreso-informacion', params)
            .toPromise();
    }

    loadRescisionesLaborales(): Promise<Array<RescisionLaboral>> {
        let params = new HttpParams()
        return this.apiService.get<Array<RescisionLaboral>>('load-rescisiones', params)
            .toPromise();
    }

    getTarifaPagoProvicional(idCompania: number, importe: number): Promise<TarifaPagoProvicional> {
        let params = new HttpParams()
            .set('idCompania', idCompania.toString())
            .set('importe', importe.toString());
        return this.apiService.get<TarifaPagoProvicional>('get-tarifa-provicional', params)
            .toPromise();
    }

    getSubsidioEmpleo(idCompania: number, importe: number): Promise<SubsidioEmpleo> {
        let params = new HttpParams()
            .set('idCompania', idCompania.toString())
            .set('importe', importe.toString());
        return this.apiService.get<SubsidioEmpleo>('get-subsidio-empleo', params)
            .toPromise();
    }

    generarCedulaFiniquito(idCedulaFiniquito: number): Promise<string> {
        let params = new HttpParams()
            .set('idCedulaFiniquito', idCedulaFiniquito.toString());
        return this.apiService.get<string>('generar-cedula-finiquito', params)
            .toPromise();
    }

    generarCedulaFiniquitoAdeudo(idEmpleado: number): Promise<string> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString());
        return this.apiService.get<string>('load-cedula-finiquito-adeudo', params)
            .toPromise();
    }

    generarKardexAsistencia(empleado: string, fechaIni: string, fechaFin: string): Promise<string> {
        let params = new HttpParams()
            .set("empleado", empleado)
            .set("fechaIni", fechaIni)
            .set("fechaFin", fechaFin);

        return this.apiService.get<string>('load-kardex-asistencia', params)
            .toPromise();
    }

    getTaskComision(idTask): Promise<AsyncTaskInfo> {
        let params = new HttpParams()
            .set('idTask', idTask.toString());
        return this.apiService.get<AsyncTaskInfo>('get-task-comision', params)
            .toPromise();
    }

    updateCedula(info: CedulaFiniquitoInfo): Promise<CedulaFiniquitoInfo>{
        let params = new HttpParams();
        return this.apiService.post<CedulaFiniquitoInfo>('update-cedula', params, info)
            .toPromise();
    }

    loadAdeudosEmpleado(compania: string, numEmpleado: string, fechaBaja: string): Promise<number> {
        let params = new HttpParams()
            .set("compania", compania)
            .set("numEmpleado", numEmpleado)
            .set("fechaBaja", fechaBaja);
        return this.apiService.get<number>('get-adeudo-empleado', params)
            .toPromise();
    }
}

