﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import { map } from 'rxjs/operators';

import * as moment from 'moment';

//Azteca Kernel
import { Context, CatalogService, UsuarioItem } from 'framework3/core';
//Locales
import { ChecadorInfo, ChecadorItem, ChecadoresUsuariosCollections } from '../models/checador'
import { ChecadorUsuarioIten } from '../models/checador';
import { TransaccionItem } from '../models/transaccion';
import { SearchRequest, SearchResult } from 'framework3/search';
import { EventoUsuarioInfo } from '../models/evento-usuario';
import { request } from 'https';
import { stat } from 'fs';

@Injectable()
export class ChecadorService extends CatalogService<ChecadorInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/rh/checadores');
    }

    create(): Promise<ChecadorInfo> {
        return Promise.resolve(new ChecadorInfo());
    }

    testConnection(checador: ChecadorInfo): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('test-connection', params, checador)
            .toPromise();
    }

    importUsers(idChecador: number): Promise<ChecadorUsuarioIten[]> {
        let params = new HttpParams()
            .set('idChecador', idChecador.toString());
        return this.apiService.get<ChecadorUsuarioIten[]>('import-users', params)
            .toPromise();
    }

    exportUsers(idChecador: number, usuarios: ChecadorUsuarioIten[], sincronizarChecador:boolean): Promise<ChecadorUsuarioIten[]> {
        let params = new HttpParams()
            .set('idChecador', idChecador.toString())
            .set('sincronizarChecador', sincronizarChecador.toString());            
        return this.apiService.post<ChecadorUsuarioIten[]>('export-users', params, usuarios)
            .toPromise();
    }

    exportEmployeesToChecks(usuarios: ChecadorUsuarioIten[], checadores:ChecadorItem[]): Promise<ChecadorUsuarioIten[]> {
        let checadoresUsuariosCollections = new ChecadoresUsuariosCollections();
        checadoresUsuariosCollections.checadores = checadores;
        checadoresUsuariosCollections.checadoresUsuariosItem = usuarios;

        let params = new HttpParams();            
        return this.apiService.post<ChecadorUsuarioIten[]>('export-employees-to-checks', params, checadoresUsuariosCollections)
            .toPromise();
    }

    downloadTransacciones(idChecador: number): Promise<boolean> {
        let params = new HttpParams()
            .set('idChecador', idChecador.toString());
        return this.apiService.post<boolean>('download-transacciones', params)
            .toPromise();
    }

    importUser(idChecador: number, item: ChecadorUsuarioIten): Promise<ChecadorUsuarioIten> {
        let params = new HttpParams()
            .set('idChecador', idChecador.toString());
        return this.apiService.post<ChecadorUsuarioIten>('import-user', params, item)
            .toPromise();
    }

    importNewUser(idChecador: number, items: ChecadorUsuarioIten[]): Promise<ChecadorUsuarioIten[]> {
        let params = new HttpParams()
            .set('idChecador', idChecador.toString());
        return this.apiService.post<ChecadorUsuarioIten[]>('import-new-user', params, items)
            .toPromise();
    }

    exportFingers(idChecador: number, item: ChecadorUsuarioIten): Promise<boolean> {
        let params = new HttpParams()
            .set('idChecador', idChecador.toString());
        return this.apiService.post<boolean>('export-fingers', params, item)
            .toPromise();
    }

    deleteUser(idChecador: number, item: ChecadorUsuarioIten): Promise<boolean> {
        let params = new HttpParams()
            .set('idChecador', idChecador.toString());
        return this.apiService.post<boolean>('delete-user', params, item)
            .toPromise();
    }

    getTransaccionesEmpleado(fechaIni: Date, fechaFin: Date, idEmpleado: number): Promise<TransaccionItem[]> {
        let params = new HttpParams()
            .set('fechaIni', fechaIni.toLocaleDateString('en-US'))
            .set('fechaFin', fechaFin.toLocaleDateString('en-US'))
            .set('idEmpleado', idEmpleado.toString());
        return this.apiService.get<TransaccionItem[]>('get-transacciones-empleado', params)
            .pipe(map(result => {
                result.forEach(item => {
                    item.fecha = moment(item.fecha).toDate();
                });
                return result;
            })).toPromise();
    }

    updateTransaccionesEmpleado(transacciones: TransaccionItem[]): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('update-transacciones-empleado', params, transacciones).toPromise();
    }

    procesarTransaccionesEmpleado(transacciones: TransaccionItem[]): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('procesar-transacciones-empleado', params, transacciones).toPromise();
    }

    searchChecadoresEventos(idBranch: number, fechaIni:Date, fechaFin : Date): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idBranch', idBranch.toString())
            .set('FechaIni', fechaIni.toLocaleString('en-US'))
            .set('FechaFin', fechaFin.toLocaleString('en-US'));
        return this.apiService.post<any>('search-checadores-eventos', params, request)
            .toPromise();
    }

    saveEventoUsuario(eventoUsuario: EventoUsuarioInfo): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('save-evento-usuario', params, eventoUsuario)
            .toPromise();
    }

    getEventoUsuario( idEventoUsuario: number): Promise<EventoUsuarioInfo> {
        let params = new HttpParams()
            .set('idEventoUsuario', idEventoUsuario.toString());
        return this.apiService.get<EventoUsuarioInfo>('get-evento-usuario', params)
           .toPromise();
    }

    statusEvento(idEventoUsuario: number, status: boolean, idUser: number): Promise<boolean> {
        let params = new HttpParams()
            .set("idEventoUsuario", idEventoUsuario.toString())
            .set("status", status ? 'true' : 'false ');
        return this.apiService.post<boolean>('evento-status', params)
            .toPromise();
    }

    addUser(idChecador: number, userUpdate: number, usuarios: UsuarioItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idChecador', idChecador.toString())
            .set('userUpdate', userUpdate.toString());
        return this.apiService.post<any>('add-user', params, usuarios)
            .toPromise();
    }

    loadAccessUsersList(idZonaCompra: number): Promise<UsuarioItem[]> {
        let params = new HttpParams()
            .append("idChecador", idZonaCompra.toString());
        return this.apiService.get<UsuarioItem[]>("load-access-users-list", params)
            .toPromise();
    }

    removeUser(idChecadorAcceso: number): Promise<any> {
        let params = new HttpParams()
            .append("idChecadorAcceso", idChecadorAcceso.toString());
        return this.apiService.post<any>("remove-user", params)
            .toPromise();
    }

    getPeriodoTrabaoEmpleados(idEmpresa: number, idSucursal: number, fecha: Date): Promise<any[]> {
        let params = new HttpParams()
            .set('idEmpresa', idEmpresa.toString())
            .set('idSucursal', idSucursal.toString())
            .set('fecha', fecha.toDateString());
        return this.apiService.get<any[]>('get-periodos-trabajo-empleado', params)
            .toPromise();
    }

    plancharChecadas(empleados: any[], idMotivo: number, idUsuario: number, usuario: string) {
        let params = new HttpParams()
            .set('idMotivo', idMotivo.toString())
            .set('idUsuario', idUsuario.toString())
            .set('Usuario', usuario);
        return this.apiService.post<boolean>('planchar-checadas', params, empleados)
            .toPromise();
    }

    searchChecador(idUSuario: number, idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params: HttpParams = new HttpParams()
            .set('idUsuario', idUSuario.toString())
            .set('idBranch', idBranch.toString());
        return this.apiService.post<any>('search-admin-checadores', params, request)
            .toPromise();
    }
    
}

