﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, HierarchyInfo, HierarchyService } from 'framework3/core';
//Locales
import { DepartamentoInfo, DepartamentoItem, Acceso } from '../models/departamento';

@Injectable()
export class DepartamentoService extends HierarchyService<DepartamentoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/rh/departamento");
    }

    create(): Promise<DepartamentoInfo> {
        return Promise.resolve(new DepartamentoInfo());
    }

    loadAccesos(idUsuario: number): Promise<Acceso[]> {
        let params = new HttpParams()
            .set("idUsuario", idUsuario.toString());

        return this.apiService.get<Acceso[]>("load-accesos", params).toPromise();
        
    }


    updateAccesos(idUsuario: number, accesos: Acceso[]): Promise<boolean> {
        let params = new HttpParams()
            .set("idUsuario", idUsuario.toString());

        return this.apiService.post<boolean>("update-accesos", params, accesos).toPromise();

    }

    searchKeywordWithAccess(idBranch: number, keyword: string): Observable<DepartamentoItem[]> {
        let params = new HttpParams()
            .set("idBranch", idBranch.toString())
            .set("keyword", keyword);
        return this.apiService.get<DepartamentoItem[]>("search-keyword-with-access", params)

    }

    nodesWithAccess(idBranch: number): Promise<DepartamentoInfo[]> {
        let params = new HttpParams()
            .set("idBranch", idBranch.toString());
        return this.apiService.get<DepartamentoInfo[]>("nodes-with-access", params).toPromise();

    }

}
