﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, CatalogItem } from 'framework3/core';
import { FileInfo } from 'framework3/general/models/file';
//Locales
import { EmpleadoInfo, DocumentoEmpleadoItem, EmpleadoItem } from '../models/empleado';
import { AsyncTaskInfo } from 'framework3/queue/models/async-task';


@Injectable()
export class EmpleadoService extends CatalogService<EmpleadoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/rh/empleados");
    }

    create(): Promise<EmpleadoInfo> {
        return Promise.resolve(new EmpleadoInfo());
    }

    loadByCodeAndCompany(codigo: string, idEmpresa: number): Promise<any> {
        let params = new HttpParams()
            .set('codigo', codigo)
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get<any>('load-employe-code-company', params)
            .toPromise();
    }

    getDocumentos(idEmpleado: number): Promise<DocumentoEmpleadoItem[]> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString());
        return this.apiService.get<DocumentoEmpleadoItem[]>('load-documentos', params)
            .toPromise();
    }

    getDocumento(idArchivo: string): Promise<FileInfo> {
        let params = new HttpParams()
            .set('idArchivo', idArchivo);
        return this.apiService.get<FileInfo>('get-documento', params)
            .toPromise();
    }

    programarCambioTurno(idEmpleado: number, idTurnoActual:number, idTurnoNuevo:number, fecha:Date): Promise<void> {
        let params = new HttpParams()
            .set('IdEmpleado', idEmpleado.toString())
            .set('IdTurnoActual', idTurnoActual.toString())
            .set('IdTurnoNuevo', idTurnoNuevo.toString())
            .set('Fecha', fecha.toLocaleDateString('en-US'));
        return this.apiService.post<void>('programar-cambio-turno', params)
            .toPromise();
    }

    cambioPassword(idEmpleado: number, password: string) {
        let params = new HttpParams()
            .set('IdEmpleado', idEmpleado.toString())
            .set('password', password);
        return this.apiService.post<void>('establecer-password', params)
            .toPromise();
    }

    deleteDocumento(idDocumentoEmpleado: number): Promise<boolean> {
        let params = new HttpParams()
            .set('idDocumentoEmpleado', idDocumentoEmpleado.toString());
        return this.apiService.delete<boolean>('delete-documento-empleado', params)
            .toPromise();
    }

    updateDocumento(idEmpleado: number, docuemnto: DocumentoEmpleadoItem): Promise<DocumentoEmpleadoItem> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString());
        return this.apiService.post<DocumentoEmpleadoItem>('update-documento-empleado', params, docuemnto)
            .toPromise();
    }

    asyncImportarEmpleadosAdam(idSucursal: number): Promise<AsyncTaskInfo> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<AsyncTaskInfo>('importar-empleados-adam', params)
            .toPromise();

    }

    getEmpleadosEmpresa(idSucursal: number): Promise<EmpleadoItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<EmpleadoItem[]>('get-empleados-empresa', params)
            .toPromise();

    }

    searchKeyword2(idBranch: number, keyword: string): Observable<CatalogItem[]> {
        let params = new HttpParams()
            .set("idBranch", idBranch.toString())
            .set("keyword", keyword);
        return this.apiService.get<CatalogItem[]>("search-empleados-empresa", params)

    }


}