﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import { map } from 'rxjs/operators';

//Azteca Kernel
import { Context, BaseService } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';

//Locales
import * as moment from 'moment';

// Features
import { EvaluacionKardexInfo } from '../models/evaluacion-kardex'

@Injectable()
export class EvaluacionKardexService extends BaseService {


    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/rh/evaluacion-kardex');
    }

    getDetail(identity: number) {
        let params: HttpParams = new HttpParams()
            .set('identity', identity.toString());
        return this.apiService.get<EvaluacionKardexInfo>('get-detail', params)
            .pipe(map(result => {
                result.details.forEach(item => {
                    if (item.entrada)
                        item.entrada = moment(item.entrada).toDate();
                    if (item.salida)
                        item.salida = moment(item.salida).toDate();
                    if (item.comidaInicio)
                        item.comidaInicio = moment(item.comidaInicio).toDate();
                    if (item.comidaFin)
                        item.comidaFin = moment(item.comidaFin).toDate();
                });
                return result;
            })
            ).toPromise();
    }

    search(idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params: HttpParams = new HttpParams()
            .set('idBranch', idBranch.toString());
        return this.apiService.post<SearchResult>('search', params, request)
            .pipe(map(result => {
                result.rows.forEach(item => {
                    item.fechaIni = moment(item.fechaIni).toDate();
                    item.fechaFin = moment(item.fechaFin).toDate();
                });
                return result;
            })
        ).toPromise();
    }

    update(info: EvaluacionKardexInfo): Promise<EvaluacionKardexInfo> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<EvaluacionKardexInfo>('update', params, info).toPromise();
    }
}