﻿import { Injectable } from "@angular/core";
import { BaseService, Context } from "framework3/core";
import { HttpClient, HttpParams } from "@angular/common/http";
import { SearchRequest, SearchResult } from "framework3/search";
import { map } from "rxjs/operators";
import moment = require("moment");
import { InterfazAdamInfo } from "../models/interfaz-adam";

@Injectable()
export class InterfazAdamService extends BaseService {

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/rh/interfaz-adam');
    }

    search(idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params: HttpParams = new HttpParams()
            .set('idBranch', idBranch.toString());
        return this.apiService.post<SearchResult>('search', params, request)
            .pipe(map(result => {
                result.rows.forEach(item => {
                    item.fechaIni = moment(item.fechaIni).toDate();
                    item.fechaFin = moment(item.fechaFin).toDate();
                });
                return result;
            })
            ).toPromise();
    }

    getEmpleadosInterfaz(fechaIni: Date, fechaFin: Date, idDepartamento: number, idEmpresaAdam: number, idSucursal: number, idEmpresa: number):Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('fechaIni', fechaIni.toDateString())
            .set('fechaFin', fechaFin.toDateString())
            .set('idDepartamento', idDepartamento.toString())
            .set('idEmpresaAdam', idEmpresaAdam.toString())
            .set('idSucursal', idSucursal.toString())
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get('get-employes', params).toPromise();
    }

    update(info: InterfazAdamInfo): Promise<InterfazAdamInfo> {
        let params: HttpParams= new HttpParams();
        return this.apiService.post<InterfazAdamInfo>('update', params, info).toPromise();
    }

    getDetail(idInterfazAdam: number): Promise<InterfazAdamInfo> {
        let params: HttpParams = new HttpParams()
            .set('idInterfazAdam', idInterfazAdam.toString());
        return this.apiService.get<InterfazAdamInfo>('get-detail', params).toPromise();
    }
}