﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import { map } from 'rxjs/operators';

//Azteca Kernel
import { Context, BaseService } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';

//Locales
import * as moment from 'moment';

// Features
import { IncidenciaEmpleadoInfo } from '../models/incidencia-empleado';
import { KardexAsistenciaInfo } from '../models/kardex-asistencia';
import { Http } from '@angular/http';

@Injectable()
export class KardexAsistenciaService extends BaseService {


    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/rh/kadex-asistencia');
    }

    searchIncidenciaEmpleado(idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params: HttpParams = new HttpParams()
            .set('idBranch', idBranch.toString());
        return this.apiService.post<SearchResult>('search-incidencia-empleado', params, request)
            .pipe(map(result => {
                result.rows.forEach(item => {
                    item.fecha = moment(item.fecha).toDate();
                });
                return result;
            })
            ).toPromise();
    }

    updateIncidenciaEmpleado(info: IncidenciaEmpleadoInfo): Promise<IncidenciaEmpleadoInfo> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<IncidenciaEmpleadoInfo>('update-incidencia-empleado', params, info).toPromise();
    }

    deleteIncidenciaEmpleado(identity: number): Promise<boolean> {
        let params: HttpParams = new HttpParams()
            .set('identity', identity.toString());
        return this.apiService.post<boolean>('delete-incidencia-empleado', params).toPromise();
    }

    printKardexAsistencia(idEmpleado: number, idDepartamento: number, fechaIni: Date, fechaFin: Date, activo: boolean, pie: string): Promise<string> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString())
            .set('idDepartamento', idDepartamento.toString())
            .set('fechaIni', fechaIni.toLocaleDateString('en-US'))
            .set('fechaFin', fechaFin.toLocaleDateString('en-US'))
            .set('activo', activo.toString())
            .set('pie', pie);
        return this.apiService.get<string>('print-tarjeta-asistencias', params)
            .toPromise();
    }

    printTransacciones(idEmpleado: number, idDepartamento: number, fechaIni: Date, fechaFin: Date, activo: boolean, pie: string): Promise<string> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString())
            .set('idDepartamento', idDepartamento.toString())
            .set('fechaIni', fechaIni.toLocaleDateString('en-US'))
            .set('fechaFin', fechaFin.toLocaleDateString('en-US'))
            .set('activo', activo.toString())
            .set('pie', pie);
        return this.apiService.get<string>('print-transacciones', params)
            .toPromise();
    }

    getKardexAsistencia(idEmpleado: number, fechaIni: Date, fechaFin: Date): Promise<KardexAsistenciaInfo[]> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString())
            .set('fechaIni', fechaIni.toLocaleDateString('en-US'))
            .set('fechaFin', fechaFin.toLocaleDateString('en-US'));
        return this.apiService.get<KardexAsistenciaInfo[]>('get-kardex-asistencia', params)
            .pipe(map(result => {
                result.forEach(item => {
                    item.fecha = moment(item.fecha).toDate();
                });
                return result;
            })).toPromise();
    }

    updateKardexAsistencia(incidencias: KardexAsistenciaInfo[]):Promise<boolean>  {
        let params = new HttpParams();
        return this.apiService.post<boolean>('update-kardex-asistencia', params, incidencias).toPromise();
    }

    getKardexEstatus(): Promise<any[]> {
        return this.apiService.get<any[]>('get-kardex-estatus').toPromise();
    }

    getIncidenciasFecha(idSucursal: number, fecha: Date):Promise<any[]> {
        let param = new HttpParams()
            .set('idSucursal', idSucursal.toString())
            .set('fecha', fecha.toLocaleDateString('en-US'));
        return this.apiService.get<any[]>('get-incidencias-fecha', param).toPromise();
    }

    getIncidenciasByEmpleado(idEmpleado: number, fechaIni: Date, fechaFin: Date): Promise<any[]> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString())
            .set('fechaIni', fechaIni.toDateString())
            .set('fechaFin', fechaFin.toDateString());
        return this.apiService.get<any[]>('get-incidencias-by-empleado', params).toPromise();
    }

    updateIncidenciasByEmpleado(incidencias: any[]): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('update-incidencias-by-empleado', params, incidencias).toPromise();
    }
}