﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import { map } from 'rxjs/operators';
import * as moment from 'moment';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { PeriodicidadPagoInfo } from '../models/periodicidad-pago'

@Injectable()
export class PeriodicidadPagoService extends CatalogService<PeriodicidadPagoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/rh/periodicidad-pago');
    }

    create(): Promise<PeriodicidadPagoInfo> {
        return Promise.resolve(new PeriodicidadPagoInfo());
    }

    getDetailByID(itemID: number): Promise<PeriodicidadPagoInfo> {
        let params = new HttpParams()
            .set('itemID', itemID.toString());
        return this.apiService.get<PeriodicidadPagoInfo>('get-detail-id', params)
            .pipe(map(info => {
                info.periodos.forEach(periodo => {
                    periodo.fechaIniPago = moment(periodo.fechaIniPago).toDate();
                    periodo.fechaFinPago = moment(periodo.fechaFinPago).toDate();
                    periodo.fechaIniIncidencia = moment(periodo.fechaIniIncidencia).toDate();
                    periodo.fechaFinIncidencia = moment(periodo.fechaFinIncidencia).toDate();

                    periodo.periodoDias.forEach(periodoDias => {
                        periodoDias.fechaIncidencia = moment(periodoDias.fechaIncidencia).toDate();
                    });
                });
                return info;
            }))
            .toPromise();
    }
    
    getDetailByCode(idBranch: number, code: string): Promise<PeriodicidadPagoInfo> {
        let params = new HttpParams()
            .set('idBranch', idBranch.toString())
            .set('code', code);
        return this.apiService.get<PeriodicidadPagoInfo>('get-detail-code', params)
            .pipe(map(info => {
                info.periodos.forEach(periodo => {
                    periodo.fechaIniPago = moment(periodo.fechaIniPago).toDate();
                    periodo.fechaFinPago = moment(periodo.fechaFinPago).toDate();
                    periodo.fechaIniIncidencia = moment(periodo.fechaIniIncidencia).toDate();
                    periodo.fechaFinIncidencia = moment(periodo.fechaFinIncidencia).toDate();

                    periodo.periodoDias.forEach(periodoDias => {
                        periodoDias.fechaIncidencia = moment(periodoDias.fechaIncidencia).toDate();
                    });
                });
                return info;
            }))
            .toPromise();
    }
   
}

