﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import 'rxjs/add/operator/toPromise';
import { map } from 'rxjs/operators';

//Locales
import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseService } from 'framework3/core';
import { PeriodoTrabajoInfo } from '../models/periodo-trabajo';

@Injectable()
export class PeriodoTrabajoService extends BaseService {

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/rh/periodos-trabajo');
    }

    getPeriodoTrabajo(idEmpleado: number, fechaIni: Date, fechaFin: Date): Promise<PeriodoTrabajoInfo[]> {
        let params = new HttpParams()
            .set('idEmpleado', idEmpleado.toString())
            .set('fechaIni', fechaIni.toLocaleDateString('en-US'))
            .set('fechaFin', fechaFin.toLocaleDateString('en-US'));
        return this.apiService.get<PeriodoTrabajoInfo[]>('get-periodo-trabajo', params)
            .pipe(map(result => {
                result.forEach(item => {
                    item.fecha = moment(item.fecha).toDate();
                    if (item.entrada)
                        item.entrada = moment(item.entrada).toDate();
                    if (item.salida)
                        item.salida = moment(item.salida).toDate();
                });
                return result;
            })).toPromise();
    }

    updatePeriodoTurnoHorario(periodos: PeriodoTrabajoInfo[]): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('update-periodo-turno-horario', params, periodos).toPromise();
    }

    getPeriodoTrabajoFecha(idSucursal: number, fecha: Date): Promise<any[]> {
        let params = new HttpParams()
            .set('idSucursal', idSucursal.toString())
            .set('fecha', fecha.toDateString());
        return this.apiService.get<any[]>('get-periodo-trabajo-fecha', params).toPromise();
    }

    updatePeriodosFecha(fecha:Date,empleados: any[]): Promise<any> {
        let params = new HttpParams()
            .set('fecha', fecha.toDateString());
        return this.apiService.post<any>('update-periodo-turno-horario-fecha', params, empleados).toPromise();
    }
}