﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import 'rxjs/add/operator/toPromise';
import { map } from 'rxjs/operators';

//Locales
import * as moment from 'moment';

//Azteca Kernel
import { Context, BaseService } from 'framework3/core';
import { PeriodoVacacionalInfo } from '../models/periodo-vacacional';

@Injectable()
export class PeriodoVacacionalService extends BaseService {

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/rh/periodos-vacacionales');
    }

    obtenerByFechas(idEmpleado: number, fechaIni: Date, fechaFin: Date): Promise<PeriodoVacacionalInfo[]> {
        let params = new HttpParams()
            .set('IdEmpleado', idEmpleado.toString())
            .set('fechaIni', fechaIni.toLocaleDateString('en-US'))
            .set('fechaFin', fechaFin.toLocaleDateString('en-US'));
            
        return this.apiService.get<PeriodoVacacionalInfo[]>('obtener-generar-by-fechas', params)
            .pipe(map(result => {
                result.forEach(per => {
                    per.fechaMax = moment(per.fechaMax).toDate();
                    per.fechaMin = moment(per.fechaMin).toDate();
                });
                //result.fechaIni = moment(result.fechaMin).toDate();
                //result.fechaFin = moment(result.fechaFin).toDate();
                return result;
            })).toPromise();
    }


    /*
    updatePeriodoTurnoHorario(periodos: PeriodoVacacionalInfo[]): Promise<boolean> {
        let params = new HttpParams();
        return this.apiService.post<boolean>('update-periodo-turno-horario', params, periodos).toPromise();
    }
    */

}