﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, BaseService } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';

import { JobSourceInfo } from 'framework3/queue';
//Locales
import { EmpresaAdamInfo } from '../models/empresaadam'
import { NominaAdamInfo } from '../models/recibo-nomina-adam';

@Injectable()
export class ProgramacionReciboAdamService extends BaseService {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/rh/programaciones-recibos-adam');
    }

    search(idBranch: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idSucursal', idBranch.toString());
        return this.apiService.post<SearchResult>('search', params, request).toPromise();
    }

    generarProgramacion(idSucursal: number, adamNomina: NominaAdamInfo): Promise<boolean> {
        let params = new HttpParams()
            .set("idSucursal", idSucursal.toString());
        return this.apiService.post<boolean>('generar-programacion', params, adamNomina).toPromise();
    }

    generarComparativoPDF(idProgramacion: string): Promise<boolean> {
        let params = new HttpParams()
            .set("idProgramacion", idProgramacion);
        return this.apiService.get<boolean>('generar-comparativo-recibo', params).toPromise();
    }

    generarDescargaComprobante(idProgramacion: string): Promise<string> {
        let params = new HttpParams()
            .set("idProgramacion", idProgramacion);
        return this.apiService.post<string>('generar-descarga-comprobante', params).toPromise();
    }

    getJobDescargaComprobante(idProgramacion: string): Promise<any> {
        let params = new HttpParams()
            .set("idProgramacion", idProgramacion);
        return this.apiService.get<any>('get-task-descarga-comprobante', params).toPromise();
    }

    cancelarProgramacion(idProgramacion: string): Promise<boolean> {
        let params = new HttpParams()
            .set("idProgramacion", idProgramacion);
        return this.apiService.post<boolean>('cancelar-progrmacion', params, null).toPromise();
    }

    getReciboToDownload(idProgramacion: string): Promise<any> {
        let params = new HttpParams()
            .set("idProgramacion", idProgramacion);
        return this.apiService.getText('recibos-to-download', params).toPromise();
    }

    getReciboToDownloadPath(idProgramacion: string): Promise<any> {
        let params = new HttpParams()
            .set("idProgramacion", idProgramacion);
        return this.apiService.getText('recibos-to-download-path', params).toPromise();
    }

}