﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, BaseService } from 'framework3/core';
import { SearchResult, SearchRequest } from 'framework3/search';
//Locales
import { EmpresaAdamInfo } from '../models/empresaadam'
import { NominaAdamInfo } from '../models/recibo-nomina-adam';
import { map } from 'rxjs/operators';

import * as moment from 'moment';

@Injectable()
export class ReciboAdamService extends BaseService {

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/rh/recibos-nominas-adam");
    }

    search(idBranch:number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idBranch', idBranch.toString());
        return this.apiService.post<SearchResult>('search-recibos', params, request)
            .toPromise();
    }

    generarRecibo(idSucursal: number, adamNomina: NominaAdamInfo): Promise<boolean> {
        let params = new HttpParams()
            .set("idSucursal", idSucursal.toString());
        return this.apiService.post<boolean>('generar-recibo', params, adamNomina)
            .toPromise();
    }

    timbrarRecibo(idReciboNominaAdam: string): Promise<boolean> {
        let params = new HttpParams()
            .set("idReciboNominaAdam", idReciboNominaAdam);
        return this.apiService.post<boolean>('timbrar-recibo', params, null)
            .toPromise();
    }

    enviarRecibo(idReciboNominaAdam: string, idBranch: number): Promise<boolean> {
        let params = new HttpParams()
            .set("idReciboNominaAdam", idReciboNominaAdam)
            .set("idBranch", idBranch.toString());
        return this.apiService.get<boolean>('enviar-recibo', params)
            .toPromise();
    }

    cancelarRecibo(idReciboNominaAdam: string): Promise<boolean> {
        let params = new HttpParams()
            .set("idReciboNominaAdam", idReciboNominaAdam);
        return this.apiService.post<boolean>('cancelar-recibo', params, null)
            .toPromise();
    }
    consultarAsistencias(Codigo: string, fechaInicial: Date, fechaFinal: Date): Promise<any> {
        let params = new HttpParams()
            .set("Codigo", Codigo)
            .set("fechaInicial",moment(fechaInicial).format("YYYY-MM-DD"))
            .set("fechaFinal",moment(fechaFinal).format("YYYY-MM-DD"));
        return this.apiService.get<any>('consultar-asistencias', params).pipe(map(info => {
            info.forEach(item => {
                item.fecha = moment(item.fecha).toDate();
            });
            return info;
        }))
            .toPromise();
    }

}

