﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Locales
import { ReciboNominaInfo, ReciboNominaItem } from '../models/recibo-nomina';
import { ReciboNominaDetail } from '../models/recibo-nomina-detail';

@Injectable()
export class ReciboNominaService extends DocumentService<ReciboNominaDetail, ReciboNominaInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/rh/recibos-nominas');
    }

    create(): Promise<ReciboNominaInfo> {
        let params = new HttpParams();
        return this.apiService.get<ReciboNominaInfo>('create', params).toPromise();
    }

    printReciboPdf(idReciboNomina:string): Promise<any> {
        let params = new HttpParams()
            .set("idReciboNomina", idReciboNomina);
        return this.apiService.get<any>("print-recibo-pdf", params)
            .toPromise();
    }

    loadReciboXml(idReciboNomina: string): Promise<string> {
        let params = new HttpParams()
            .set("idReciboNomina", idReciboNomina.toString());
        return this.apiService.getText("get-xml-recibo", params)
            .toPromise();
    }

    enviarRecibo(idReciboNomina: string, idBranch: number): Promise<boolean> {
        let params = new HttpParams()
            .set("idReciboNomina", idReciboNomina)
            .set("idBranch", idBranch.toString());
        return this.apiService.get<boolean>('enviar-recibo', params)
            .toPromise();
    }

    getErrorTimbrado(idReciboNomina: string): Promise<string> {
        let params = new HttpParams()
            .set("idReciboNomina", idReciboNomina)
        return this.apiService.getText('get-error-timbrado', params)
            .toPromise();
    }

}