﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, ToolbarButton, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs, TrackableCollection, TrackableItem } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';
//Features
import { UnidadMedidaTipoService } from '../../inventory/services/unidad-medida-tipo.service';
import { ArticuloService } from '../services/articulo.service';
import { ArticuloInfo, ArticuloImpuestoItem } from '../models/articulo';
import { ImpuestoAsignacionInfo } from '../../accounting/models/impuesto-asignacion';
import { ArticuloStockInfo } from '../../inventory/models/articulo-stock';
import { EstComSelectorComponent } from '../controls/est-com-selector.component';
import { CuentaSelectorComponent } from '../../accounting/components/cuenta-selector.component';
import { FlujoSelectorComponent } from '../../accounting/components/flujo-selector.component';
import { MarcasSelectorComponent } from '../controls/marcas-selector.component';
import { ArticuloCategoriaFiscalComponent } from '../../accounting/controls/articulo-categoria-fiscal-selector.component';
import { UnidadMedidaTipoItem } from '../../inventory/models/unidad-medida-tipo';
import { UnidadMedidaService } from '../../inventory/services/unidad-medida.service';
import { UnidadMedidaItem } from '../../inventory/models/unidad-medida';
import { MetodoValuacion } from '../../inventory/models/metodo-valuacion';
import { ArticuloStockService } from '../../inventory/services/articulo-stock.service';
import { ImpuestosAsignacionEditor } from '../../accounting/components/impuesto-asignacion.editor';
import { ImpuestoAsignacionZonaInfo } from '../models/impuesto-asignacion-zona';
import { ZonaVentaItem } from '../models/zona-venta'
import { ZonaVentaService } from '../services/zona-venta.service'

@Component({
    selector: 'ventas-articulos-ventas-editor',
    templateUrl: './articulos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticulosEditor)
        },
        {
            provide: CatalogService, useExisting: ArticuloService
        }
    ]
})
export class ArticulosEditor extends CatalogEditor {
    idArticulo: number;
    idArticuloStock: number;
    idZonaVenta: number;
    codigoNuevo: string;
    articulosVentas: ArticuloInfo = new ArticuloInfo();
    tiposUnidad: UnidadMedidaTipoItem[] = [];
    unidadesMedida: UnidadMedidaItem[] = [];
    metodosValuacion = MetodoValuacion.list;
    clasificadores: Clasificacion[] = [];
    articuloImpuestos: ImpuestoAsignacionInfo[] = [];
    artImpCollection: TrackableCollection = new TrackableCollection();
    zonasVenta: ZonaVentaItem[] = [];

    //ViewChilds
    @ViewChild('selectorEstCom') selectorEstCom: EstComSelectorComponent;
    @ViewChild('selectorCuenta') selectorCuenta: CuentaSelectorComponent;
    @ViewChild('selectorCatFlujo') selectorCatFlujo: FlujoSelectorComponent;
    @ViewChild('selectorMarca') selectorMarca: MarcasSelectorComponent;
    @ViewChild('selectorCategoriaFiscal') selectorCategoriaFiscal: ArticuloCategoriaFiscalComponent;
    @ViewChild(ImpuestosAsignacionEditor) articuloImpuestosDialog: ImpuestosAsignacionEditor;

    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private articulosService: ArticuloService, private tipoUnidadService: UnidadMedidaTipoService,
        private unidadMedidaService: UnidadMedidaService, private articuloStockService: ArticuloStockService, private zonaVentaService: ZonaVentaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            //general
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuenta': [''],
            'gtin': [''],
            'tipoUnidad': ['', AztecaValidators.selected],
            'unidadAlmacenamiento': ['', AztecaValidators.selected],
            'categoriaFiscal': ['', AztecaValidators.selected],
            'descripcion': [''],

            //controls
            'manejaSeries': [false],
            'manejaLotes': [false],
            'manejaCaducidad': [false],
            'metodoValuacion': [''],

            //clasificaciones
            'clasificaciones': [[], AztecaValidators.validClasifications],

            //Comercial
            'impuestoIncluido': [false],
            'estCom': [''],
            'catFlujo': [''],
            'marcas': [''],
            'modelo': [''],
            'unidadVenta': ['', AztecaValidators.selected],
            //
            'idZonaVenta':[0]
        });
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();

        this.loadZonasVenta();
        this.loadClasificadores();
        this.loadTiposUnidad();
    }

    ngAfterViewChecked() {         
        this.artImpCollection.sourceCollection = this.articulosVentas.impuestos.filter(item => item.idZonaVenta== this.idZonaVenta);
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idArticulo = +param['id'];
        })

        this.route.queryParams.forEach(params => {
            this.idArticuloStock = +params['idArticuloStock'] || 0;
        })

        this.route.queryParams.forEach(params => {
            this.codigoNuevo = params['codigoNuevo'];
        })
    }

    loadClasificadores() {
        this.articuloStockService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => this.clasificadores = clasificadores)
            .catch(error => this.handleError(error));
    }

    loadTiposUnidad() {
        this.tipoUnidadService.loadList()
            .then(tipos => this.tiposUnidad = tipos)
            .catch(error => this.handleError(error));
    }

    onTipoUnidadChanged(idTipoUnidad) {
        this.loadUnidadesMedida(idTipoUnidad);
    }

    loadZonasVenta() {
        this.zonaVentaService.loadList(this.context.app.defaultCompany.identity.toString())
            .then(zonasVenta => this.zonasVenta = zonasVenta)
            .catch(error => this.handleError(error));
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        this.unidadMedidaService.loadList(idTipoUnidad)
            .then(unidades => this.unidadesMedida = unidades)
            .catch(error => this.handleError(error));
    }

    onSerieCheckChange() {
        if (this.catalogForm.controls.manejaSeries.value) {
            this.catalogForm.controls.unidadAlmacenamiento.clearValidators();
            this.catalogForm.controls.unidadAlmacenamiento.updateValueAndValidity();
            this.catalogForm.controls.tipoUnidad.clearValidators();
            this.catalogForm.controls.tipoUnidad.updateValueAndValidity();
            this.catalogForm.controls.unidadVenta.clearValidators();
            this.catalogForm.controls.unidadVenta.updateValueAndValidity();
        } else {
            this.catalogForm.controls.unidadAlmacenamiento.setValidators(AztecaValidators.selected);
            this.catalogForm.controls.unidadAlmacenamiento.updateValueAndValidity();
            this.catalogForm.controls.tipoUnidad.setValidators(AztecaValidators.selected);
            this.catalogForm.controls.tipoUnidad.updateValueAndValidity();
            this.catalogForm.controls.unidadVenta.setValidators(AztecaValidators.selected);
            this.catalogForm.controls.unidadVenta.updateValueAndValidity();
        }
    }

    onCreatingItem() {
        this.context.app.showSpinner();
        this.articulosService.create()
            .then(articulo => {
                this.context.app.hideSpinner();
                this.articulosVentas = articulo;
                if (this.idArticuloStock > 0) {
                    this.articuloStockService.getDetailByID(this.idArticuloStock)
                        .then(articuloStock => {
                            this.context.app.hideSpinner();
                            this.articulosVentas.articuloStock = articuloStock;
                            this.loadUnidadesMedida(articuloStock.unidadMedida.idUnidadMedidaTipo);
                        })
                        .catch(error => this.handleError(error));
                }
                if (this.codigoNuevo) 
                    this.catalogForm.controls.codigo.patchValue(this.codigoNuevo);
            })
            .catch(error => this.handleError(error));

    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();
        this.articulosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.articulosVentas = info;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        this.articulosService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.articulosVentas = info;
                this.loadUnidadesMedida(info.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();
        this.articulosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.articulosVentas = info;
                this.artImpCollection.sourceCollection = this.articulosVentas.impuestos.filter(item => item.idZonaVenta == this.idZonaVenta);
                this.loadUnidadesMedida(info.articuloStock.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.articulosService.update(this.articulosVentas)
            .then(info => {
                this.context.app.hideSpinner();
                this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.articulosVentas.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.articulosVentas = info;
            })
            .catch(error => this.handleError(error));
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            if (this.idZonaVenta > 0 && this.idZonaVenta != undefined)
                this.articuloImpuestosDialog.show();
            else
                this.showAlert("Debe seleccionar una zona de venta para incluir impuestos al articulo", "warning");
        }
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'LP') {
            this.router.navigate(['sales/articulo-listas-precio/' + this.process.id + '/' + this.articulosVentas.identity.toString()]);
        } else if (item.name == 'GD') {
            this.router.navigate(['sales/articulo-grupo-descuento/' + this.process.id + '/' + this.articulosVentas.identity.toString()]);
        }
    }

    onAgregarImpuesto(args: ImpuestoAsignacionZonaInfo) {
        let impuestoExists: boolean = false;
        this.articulosVentas.impuestos.forEach(articuloImp => {
            if (articuloImp.idImpuesto == args.idImpuesto && articuloImp.state != 'DELETED') {
                this.showAlert("El impuesto ya fue agregado", "warning");
                impuestoExists = true;
            }
        })
        if (!impuestoExists) {
            args.idZonaVenta = this.idZonaVenta; //se asigna la zona de venta seleccionada
            this.articulosVentas.impuestos.push(args);
            this.artImpCollection.refresh();
        }
    }

    onRemoveItem(item) {
        this.articulosVentas.impuestos.forEach(x => {
            if (x.idImpuesto == item.idImpuesto)
                x.state = 'DELETED'
        })

        this.artImpCollection.remove(item);
        this.artImpCollection.refresh();
    }

    onZonaVentaChanged(idZonaVenta:number) {
        this.artImpCollection.sourceCollection = this.articulosVentas.impuestos.filter(item => item.idZonaVenta == idZonaVenta);       
    }

}