﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Features
import { ArticuloService } from '../services/articulo.service';
import { ArticuloInfo, ArticuloItem } from '../models/articulo';
import { ArticuloNuevoComponent } from '../components/nuevo-articulo.component';

@Component({
    selector: 'ventas-articulos-manager',
    templateUrl: './articulos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticulosManager)
        },
        {
            provide: CatalogService, useExisting: ArticuloService
        }
    ]
})
export class ArticulosManager extends CatalogManager {
    articulos: ArticuloItem[] = [];
    options: CatalogOptions = new CatalogOptions();
    currentItem: CatalogItem;
    //    

    @ViewChild(ArticuloNuevoComponent) articulosDialog: ArticuloNuevoComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private articulosService: ArticuloService) {
        super(context, router, route);
    }

    ///////////////////////// METODOS ///////////////////////////////////////

    ///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.articulosDialog.autogenerado = options.codigoAutogenerado;
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;
        this.refreshList();
        this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.articulosService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.articulos = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.articulosDialog.show();
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.articulosService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, { cloneId: item.id }], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.articulosService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == 'LP') {
            this.router.navigate(['sales/articulo-listas-precio/' + this.process.id + '/' + this.currentItem.id.toString()]);
        } else if (item.name == 'GD') {
            this.router.navigate(['sales/articulo-grupo-descuento/' + this.process.id + '/' + this.currentItem.id.toString()]);
        }
    }

}
