﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';

import { Control } from 'wijmo/wijmo';

//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem} from 'framework3/controls';

import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { Clasificacion } from 'framework3/general';
//Features
import { ClienteService} from '../services/cliente.service';
import { ClienteInfo, ClienteItem } from '../models/cliente';
import { ListaPreciosService } from '../services/lista-precios.service';
import { ListaPreciosItem } from '../models/lista-precios';
import { GrupoDescuentoService } from '../services/grupo-descuento.service';
import { GrupoDescuentoItem } from '../models/grupo-descuento';

import { FormaPagoService } from '../../credit/services/forma-pago.service';
import { FormaPagoItem } from '../../credit/models/forma-pago';
import { AddendaItem } from '../../accounting/models/addenda';
import { ComprobanteFiscalService } from '../../accounting/services/comprobante-fiscal.service';
import { GrupoDeudorSelectorComponent } from '../../credit/components/grupo-deudores-selector.component';
import { GrupoDeudorInfo } from '../../credit/models/grupo-deudor';
import { DeudorService } from '../../credit/services/deudor.service';

import { ContribuyenteService } from '../../accounting/services/contribuyente.service';
import { UsoCFDIItem } from '../../accounting/models/contribuyente';


@Component({
    selector: 'ventas-clientes-editor',
    templateUrl: './clientes.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ClientesEditor)
        },
        {
            provide: CatalogService, useExisting: ClienteService
        }
    ]
})
export class ClientesEditor extends CatalogEditor {
    cliente: ClienteInfo = new ClienteInfo();  
    listasPrecio: ListaPreciosItem[] = [];
    gruposDescuento: GrupoDescuentoItem[] = [];
    formasPago: FormaPagoItem[] = [];
    addendas: AddendaItem[] = [];
    condicionesPago: { id: string, nombre: string }[] = [];
    clasificadores: Clasificacion[] = [];
    usosCFDI: UsoCFDIItem[] = [];
    @ViewChild('selGrupoDeudor') selInsumo: GrupoDeudorSelectorComponent;

    @ViewChild('domicilioFiscal') domicilioFiscal: GrupoDeudorSelectorComponent;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private clienteService: ClienteService, private listaPreciosService: ListaPreciosService,
        private grupoDescuentoService: GrupoDescuentoService, private formaPagoService: FormaPagoService, private comprobanteFiscalService: ComprobanteFiscalService,
        private contribuyenteService: ContribuyenteService, private  deudorService : DeudorService ) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'telefonos': [''],
            'correo': [''],
            'pagina': [''],
            'pagWeb': [''],
            'webUser': [''],
            'webPassword': [''],
            'cuenta': [''],
            'domicilio': [''],

            'idGrupoDeudor': [0],
            'facturable': [''],
            'rfc': [''],
            'razonSocial': [''],
            'domicilioFiscal': [''],
            'idUsoCFDI': [0],

            'idAddenda':[''],
            'idListaPrecio': [0, AztecaValidators.selected],
            'idGrupoDescuento': [0],
            'idFormaPago': [''],
            'cuentaPago': [''],

            'condiciones': ['', Validators.required],
            'limiteCredito': [''],
            'diasCredito': [''],
            'clasificaciones': [[], AztecaValidators.validClasifications],

            'desgloseIEPS': [''],
            'facturacionCategorizada':[0]

        });

        this.catalogForm.controls['facturable'].valueChanges.subscribe(value => {            
            if (value) {                
               this.catalogForm.controls['rfc'].setValidators(Validators.required);                
            }
            else
               this.catalogForm.controls['rfc'].clearValidators();                

            this.catalogForm.controls['rfc'].updateValueAndValidity();
        });

        this.condicionesPago =
            [
                { "id": '', "nombre": "[Seleccione la condición de pago]" },
                { "id": 'C', "nombre": "Credito" },
                { "id": 'E', "nombre": "Contado" }
            ];
        this.cliente.deudor.condicionesPago = '';
    }


 
	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadListasPrecio();
        this.loadGruposDescuento();
        this.loadFormasPago();
        this.loadClasificadores();
        this.loadUsosCFDI();
        this.loadAddendas();
    }

    loadClasificadores() {
        this.deudorService.loadClassifiers(this.context.app.defaultSite.identity)
            .then(clasificadores => {
                this.clasificadores = clasificadores
            })
            .catch(error => this.handleError(error));
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.clienteService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cliente = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.clienteService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cliente = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.domicilioFiscal.clear;
        this.clienteService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.cliente = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.clienteService.update(this.cliente)
            .then(info => {
                this.context.app.hideSpinner();
                this.cliente = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    loadListasPrecio() {
        this.listaPreciosService.loadList(this.context.app.defaultSite.identity)
            .then(listas => {
                let lista = new ListaPreciosItem;
                lista.id = 0;
                lista.codigo = "";
                lista.nombre = "[Sin Asignar]";
                listas.unshift(lista);
                this.listasPrecio = listas;
            })
            .catch(error => this.handleError(error));
    }

    loadGruposDescuento() {
        this.grupoDescuentoService.loadList(this.context.app.defaultSite.identity)
            .then(grupos => {
                let grupo = new GrupoDescuentoItem;
                grupo.id = 0;
                grupo.codigo = "";
                grupo.nombre = "[Sin Asignar]";
                grupos.unshift(grupo);
                this.gruposDescuento = grupos;
            })
            .catch(error => this.handleError(error));
    }
    loadFormasPago() {
        this.formaPagoService.loadList(this.context.app.defaultSite.identity)
            .then(formas => this.formasPago= formas)
            .catch(error => this.handleError(error));
    }
    loadAddendas() {
        this.comprobanteFiscalService.loadListAddendas(this.context.app.defaultCompany.identity)
            .then(addendas => this.addendas = addendas)
            .catch(error => {
                this.handleError(error)
            });
    }

    loadUsosCFDI() {
        this.contribuyenteService.loadUsosCFDI(this.context.app.defaultCompany.identity)
            .then(usos => this.usosCFDI = usos)
            .catch(error => this.handleError(error));
    }

    //Administracion de contactos
    onMenuItemClick(option: MenuItem) {
        if (option.name == 'CONTACTS') {          
            this.router.navigate(['sales/clientes/contacts', this.process.id, this.cliente.identity])            
        }
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onGrupoDeudorSelected(grupo: GrupoDeudorInfo) {
        this.cliente.deudor.contribuyente = grupo.contribuyente;
    }


}