﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Features
import { ClienteService} from '../services/cliente.service';
import { ClienteInfo, ClienteItem} from '../models/cliente';

@Component({    
    selector: 'ventas-clientes-manager',
    templateUrl: './clientes.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ClientesManager)
        },
        {
            provide: CatalogService, useExisting: ClienteService
        }
	]
})
export class ClientesManager extends CatalogManager {
    clientes: ClienteItem[]=[];    
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private clienteService: ClienteService) {
        super(context,router, route );
    }

	///////////////////////// METODOS ///////////////////////////////////////

    enviarRegistro() {
        this.clienteService.enviarRegistro(this.manager.currentItem.id)
            .then(_ => {
                this.showAlert('Se ha enviado un correo electrónico con la invitaicón de registro', 'success');
            })
            .catch(error => this.handleError(error));
    }


	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.clienteService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.clientes = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.clienteService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.clienteService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'REGISTRO') {
            this.enviarRegistro();
        }
    }



}
