﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { FormsModule, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Tree
import { TreeNode } from 'angular-tree-component';
//
import { Context, BaseComponent, CatalogService, HierarchyChanges } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { HierarchyManager } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//
import { EstructuraComercialInfo } from '../models/estructura-comercial';
import { EstructuraComercialService } from '../services/estructura-comercial.service';

@Component({    
    selector: 'ventas-est-com-manager',
    templateUrl: './est-com.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EstComManager)
        },
        {
            provide: CatalogService, useExisting: EstructuraComercialService
        }
    ]
})
export class EstComManager extends HierarchyManager {
    estCom: EstructuraComercialInfo[] = [];
    estructuraComercialEdicion: EstructuraComercialInfo = new EstructuraComercialInfo();
    currentNode: TreeNode;
    currentParent: TreeNode;
    
    constructor(protected context: Context, protected changeDetector: ChangeDetectorRef, private formBuilder: FormBuilder, private estructuraComercialService: EstructuraComercialService, protected router: Router, protected route: ActivatedRoute) {
        super(context, router, route, changeDetector);
       
        this.nodeForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required]
        });
    }

	////////////////////// METODOS //////////////////////////////////////////////

    loadHierarchy() {
        this.context.app.showSpinner();
        this.estructuraComercialService.loadHierarchy(this.context.app.defaultCompany.identity, 0)
            .then(nodes => {
                this.context.app.hideSpinner();
                this.estCom = nodes;
            })
            .catch(error => this.handleError(error));
    }


	//////////////////// HANDLERS //////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;        
        this.loadHierarchy();
        //Hacer llamadas de carga inicial de datos aquí

    }


    onNodesChanged(estCom: EstructuraComercialInfo[]) {
        this.estCom = estCom;        
    }

    onCurrentNodeChanged(node: TreeNode) {
        this.currentNode = node;
    }

    onCreatingNode(parentNode: TreeNode) {
        this.estructuraComercialEdicion = new EstructuraComercialInfo(); //Crear nueva instancia para evitar que reset modifique la información del último nodo en edición
        this.nodeForm.reset();
        this.estructuraComercialEdicion = new EstructuraComercialInfo(); //Se vuelve a crear instancia para evitar valores nulos creados por reset
        //Inicializar valores del nodo
        this.estructuraComercialEdicion.idBranch = this.context.app.defaultSite.identity;
        this.estructuraComercialEdicion.idCompany = this.context.app.defaultCompany.identity;
		//
        this.currentParent = parentNode;        
        this.showEditor();
    }

    onEditingNode(node: TreeNode) {        
        this.estructuraComercialEdicion = this.estructuraComercialService.copy(node.data);
        this.showEditor();        
    }

    onSavingNode() {        
        //Si es un nuevo nodo
        if (this.estructuraComercialEdicion.clientID == 0) {
            this.addNode(this.estructuraComercialEdicion, this.currentParent);            
        }
        //Si se estaba editando un nodo existente
        else {
            this.updateNode(this.estructuraComercialEdicion);
        }
		this.estructuraComercialEdicion = new EstructuraComercialInfo();
        this.hideEditor();
    }

    onDeletingNode(node: TreeNode) {
        this.deleteNode(node);
    }

    onSavingHierarchy(changes: HierarchyChanges<EstructuraComercialInfo>) {
        this.context.app.showSpinner();
        this.estructuraComercialService.saveHierarchy(changes)
            .then(estCom => {
                this.estCom = estCom;
                this.reset();
                this.context.app.hideSpinner();
                this.showAlert('Los cambios han sido guardados con éxito.', 'success');
            })
            .catch(error => this.handleError(error));            
    }

}