﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { GrupoDescuentoService} from '../services/grupo-descuento.service';
import { GrupoDescuentoInfo, GrupoDescuentoItem} from '../models/grupo-descuento';

@Component({
    selector: 'ventas-gruposdescuentos-editor',
    templateUrl: './grupos-descuentos.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => GruposDescuentosEditor)
        },
        {
            provide: CatalogService, useExisting: GrupoDescuentoService
        }
    ]
})
export class GruposDescuentosEditor extends CatalogEditor {
    grupodescuento: GrupoDescuentoInfo = new GrupoDescuentoInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private grupoDescuentoService: GrupoDescuentoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoDescuentoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupodescuento = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.grupoDescuentoService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupodescuento = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.grupoDescuentoService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupodescuento = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.grupoDescuentoService.update(this.grupodescuento)
            .then(info => {
                this.context.app.hideSpinner();
                this.grupodescuento = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'ARTICULOS') {
            this.router.navigate(['articulos'], { relativeTo: this.route });
        }
        if (option.name == 'SERVICIOS') {
            this.router.navigate(['servicios'], { relativeTo: this.route });
        }
        if (option.name == 'CLIENTES') {
            this.router.navigate(['clientes'], { relativeTo: this.route });
        }
    }
}