﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ListaPreciosService} from '../services/lista-precios.service';
import { ListaPreciosInfo, ListaPreciosItem} from '../models/lista-precios';

@Component({
    selector: 'ventas-listasprecios-editor',
    templateUrl: './listas-precios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ListasPreciosEditor)
        },
        {
            provide: CatalogService, useExisting: ListaPreciosService
        }
    ]
})
export class ListasPreciosEditor extends CatalogEditor {
    listaprecios: ListaPreciosInfo = new ListaPreciosInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private listaPreciosService: ListaPreciosService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required]            
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.listaPreciosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.listaprecios = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.listaPreciosService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.listaprecios = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.listaPreciosService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.listaprecios = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.listaPreciosService.update(this.listaprecios)
            .then(info => {
                this.context.app.hideSpinner();
                this.listaprecios = info;
				this.reset();
                if (args.closeEditor)
                    this.location.back();
                else
                    this.showAlert('La información ha sido actualizada con éxito', 'success');                
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'ARTICULOS') {
            this.router.navigate(['articulos'], { relativeTo: this.route });
        }
        if (option.name == 'SERVICIOS') {
            this.router.navigate(['servicios'], { relativeTo: this.route });
        }
        if (option.name == 'CLIENTES') {
            this.router.navigate(['clientes'], { relativeTo: this.route });
        }

    }

}