﻿//Frameworks
import { Component, ViewChild, forwardRef } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Features
import { ListaPreciosService} from '../services/lista-precios.service';
import { ListaPreciosInfo, ListaPreciosItem} from '../models/lista-precios';

@Component({    
    selector: 'ventas-listas-precios-manager',
    templateUrl: './listas-precios.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ListasPreciosManager)
        },
        {
            provide: CatalogService, useExisting: ListaPreciosService
        }
	]
})
export class ListasPreciosManager extends CatalogManager {
    listasPrecios: ListaPreciosItem[]=[];    
    //    

    constructor(protected router: Router, protected route: ActivatedRoute, protected context: Context, private listaPreciosService: ListaPreciosService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.listaPreciosService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.listasPrecios = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate([0], { relativeTo: this.route });
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate([item.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate([item.id], { relativeTo: this.route });
    }

    onDeletingItem(item: CatalogItem) {
        this.listaPreciosService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate([0, {cloneId: item.id}], { relativeTo: this.route });
    }

    onChangingItemState(item: CatalogItem) {
        this.listaPreciosService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'ARTICULOS') {
            this.router.navigate([this.manager.currentItem.id, 'articulos'], { relativeTo: this.route });
        }
        else if (option.name == 'SERVICIOS') {
            this.router.navigate([this.manager.currentItem.id, 'servicios'], { relativeTo: this.route });
        }
        else if (option.name == 'CLIENTES') {
            this.router.navigate([this.manager.currentItem.id, 'clientes'], { relativeTo: this.route });
        }
    }

}
