﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { UnidadNegocioService} from '../services/unidad-negocio.service';
import { UnidadNegocioInfo, UnidadNegocioItem} from '../models/unidad-negocio';

import { Control, CollectionView } from 'wijmo/wijmo';


import { SerieFiscalItem } from '../../accounting/models/serie-fiscal';
import { SerieFiscalService } from '../../accounting/services/serie-fiscal.service';
import { DivisaItem } from '../../accounting/models/divisa';
import { DivisaService } from '../../accounting/services/divisa.service';
import { AreaItem } from '../../inventory/models/area';
import { AreaService } from '../../inventory/services/area.service';
import { AlmacenItem } from '../../inventory/models/almacen';

@Component({
    selector: 'ventas-unidades-negocio-editor',
    templateUrl: './unidades-negocio.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => UnidadesNegocioEditor)
        },
        {
            provide: CatalogService, useExisting: UnidadNegocioService
        }
    ]
})
export class UnidadesNegocioEditor extends CatalogEditor {
    unidadNegocio: UnidadNegocioInfo = new UnidadNegocioInfo();      
    series: SerieFiscalItem[] = [];
      
    areas: AreaItem[] = [];
    almacenes: AlmacenItem[] = [];
    modalidadesAfectacion: { id: boolean, nombre: string }[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private unidadNegocioService: UnidadNegocioService,
        private serieFiscalService: SerieFiscalService, private divisaService: DivisaService,
        private areaService: AreaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuenta': [''],
            'idCFDSerie': [''],
            'idCFDSerieDev': [''],            
            'gln':[''],
            'descontarExistencias': [''],
            'modalidadAfectacion': [''],
            'area': [],
            'almacen': [],
            'editarAlmacen': [],
            'remitente': [],
            'enviarFacturas': [],
            'operador': [],
            'notificarFacturas': [],
            'notificarFoliosTerminar': [],
            'porcentajeFolios': []
        });

        this.modalidadesAfectacion =
            [
                { "id": true, "nombre": "En Linea" },
                { "id": false, "nombre": "Por Lote" }
            ];
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadSeries();       
        this.loadAreas();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.unidadNegocioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.unidadNegocio = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.unidadNegocioService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.unidadNegocio = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.unidadNegocioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.selectArea(info.idArea);
                this.unidadNegocio = info;     
                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.unidadNegocioService.update(this.unidadNegocio)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.unidadNegocio.identity == 0) {
                    this.router.navigate(['ventas/unidadesNegocio', this.process.id, info.identity]);
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.unidadNegocio = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadSeries() {
        this.serieFiscalService.loadList(this.context.app.defaultCompany.identity.toString())
            .then(series => this.series = series)
            .catch(error => this.handleError(error));
    }
      
        
    loadAreas() {
        this.areaService.loadList(this.context.app.defaultCompany.identity)
            .then(areas => this.areas = areas)
            .catch(error => this.handleError(error));
    }

    selectTab() {
        //refresca los controles de wijmo
        Control.invalidateAll();
    }

    selectArea(idArea) {
        this.areaService.loadAlmacenes(idArea)
            .then(almacenes => this.almacenes = almacenes)
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['members'], { relativeTo: this.route });
        }
    }

    onCancelingEdit() {
        this.location.back();
    }

}