﻿//Frameworks
import { Component, ViewChild, forwardRef, Input, Output, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogItem, CatalogService } from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { CatalogManager } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
//Features
import { UnidadNegocioService} from '../services/unidad-negocio.service';
import { UnidadNegocioInfo, UnidadNegocioItem} from '../models/unidad-negocio';

@Component({    
    selector: 'ventas-unidades-negocio-manager',
    templateUrl: './unidades-negocio.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => UnidadesNegocioManager)
        },
        {
            provide: CatalogService, useExisting: UnidadNegocioService
        }
	]
})
export class UnidadesNegocioManager extends CatalogManager {
    @Input() unidadesNegocio: UnidadNegocioItem[] = [];    
    @Input() idSucursal: number;
    @Output() refreshListParent: EventEmitter<void> = new EventEmitter<void>(); 
    //    

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private unidadNegocioService: UnidadNegocioService) {
        super(context, router, route);
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////

    onConfigureCatalog(options: CatalogOptions) {
        this.manager.serverSide = options.busquedasServidor;
        this.manager.pageSize = options.tamanioPagina;        
        this.refreshList();
		this.applyPolicies();
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.unidadNegocioService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.unidadesNegocio = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;                                
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['sales/unidades-negocio', this.idSucursal, this.process.id, 0]);
    }

    onViewingItem(item: CatalogItem) {
        this.router.navigate(['sales/unidades-negocio', this.idSucursal, this.process.id, item.id, { readonly: true }]);
    }

    onEditingItem(item: CatalogItem) {
        this.router.navigate(['sales/unidades-negocio', this.idSucursal, this.process.id,  item.id]);
    }

    onDeletingItem(item: CatalogItem) {
        this.unidadNegocioService.delete(item.id)
            .then(success => {
                this.refreshList();
                this.refreshListParent.emit();
                this.showAlert('El registro ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));        
    }

    onCloningItem(item: CatalogItem) {
        this.router.navigate(['sales/unidades-negocio', this.idSucursal, this.process.id, item.id, { cloneId: item.id }]);
    }

    onChangingItemState(item: CatalogItem) {
        this.unidadNegocioService.changeStatus(item.id, !item.activo)
            .then(success => {
                this.refreshList();
                this.refreshListParent.emit();   
                this.showAlert('El estatus del registro ha sido actualizado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['sales/unidades-negocio', this.idSucursal, this.process.id, this.manager.currentItem.id, 'members']);
        }
    }

}
