﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { ZonaVentaService} from '../services/zona-venta.service';
import { ZonaVentaInfo, ZonaVentaItem} from '../models/zona-venta';

@Component({
    selector: 'ventas-zonasventa-editor',
    templateUrl: './zonas-venta.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ZonasVentaEditor)
        },
        {
            provide: CatalogService, useExisting: ZonaVentaService
        }
    ]
})
export class ZonasVentaEditor extends CatalogEditor {
    zonaVenta: ZonaVentaInfo = new ZonaVentaInfo();        
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private zonaVentaService: ZonaVentaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.zonaVentaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.zonaVenta = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.zonaVentaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.zonaVenta = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.zonaVentaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.zonaVenta = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.zonaVentaService.update(this.zonaVenta)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.zonaVenta.identity == 0) {
                    this.router.navigate(['ventas/zonasVenta', this.process.id, info.identity]);
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.zonaVenta = info;
            })
            .catch(error => this.handleError(error));        
    }

}