﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection, CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { ArticuloGrupoDescuentoItem } from '../models/articulo-grupo-descuento';
import { ArticuloService } from '../services/articulo.service';

@Component({
    selector: 'ventas-articulo-grupo-descuento-editor',
    templateUrl: './articulo-grupo-descuento-editor.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticuloGrupoDescuentoEditorComponent)
        },
        {
            provide: CatalogService, useExisting: ArticuloService
        }
    ]
})
export class ArticuloGrupoDescuentoEditorComponent extends CatalogEditor {
    editorForm: FormGroup;
    info: ArticuloGrupoDescuentoItem = new ArticuloGrupoDescuentoItem();

    //Inputs
    @Input() data: ArticuloGrupoDescuentoItem = new ArticuloGrupoDescuentoItem();
    //outputs
    @Output() dataUpdated: EventEmitter<ArticuloGrupoDescuentoItem> = new EventEmitter<ArticuloGrupoDescuentoItem>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private articuloService: ArticuloService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetector: ChangeDetectorRef) {
        super(context, router, route, changeDetector);
        this.editorForm = formBuilder.group({
            'porcentajeDescuento1': [0],
            'porcentajeDescuento2': [0],
            'porcentajeDescuento3': [0],
            'porcentajeDescuento4': [0],
            'porcentajeDescuento5': [0]
        });
    }

    show() {
        this.info = new ArticuloGrupoDescuentoItem();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => {
            this.editorForm.reset();
            this.info = this.data;
            this.data = new ArticuloGrupoDescuentoItem();
        });
        Control.invalidateAll();
    }

    onUpdate() {
        this.dataUpdated.emit(this.info);
        this.dialog.hide();
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}