﻿//Angular
import { Component, ViewChild, Input } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';

//Framework3
import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';

//Features
import { ArticuloGrupoDescuentoItem } from '../models/articulo-grupo-descuento';
import { ArticuloService } from '../services/articulo.service';
import { ArticuloGrupoDescuentoEditorComponent } from './articulo-grupo-descuento-editor.component';

@Component({
    selector: 'ventas-articulo-grupo-descuento.component',
    templateUrl: './articulo-grupo-descuento.component.html'
})
export class ArticuloGrupoDescuentoComponent extends BaseComponent {
    title: string = 'Grupo de Descuento del artículo';
    idArticulo: number;
    articuloGrupoDescuento: ArticuloGrupoDescuentoItem[] = [];
    artGruDesCollection: TrackableCollection = new TrackableCollection();
    currentItem: CatalogItem;

    @ViewChild(ArticuloGrupoDescuentoEditorComponent) articuloGrupoDescuentoEditorComponent: ArticuloGrupoDescuentoEditorComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private context: Context, private articuloService: ArticuloService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idArticulo = +param['idArticulo'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadArticuloGrupoDescuento());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadArticuloGrupoDescuento() {
        this.context.app.showSpinner();
        this.articuloService.loadArticuloGrupoDescuento(this.context.app.defaultCompany.identity, this.idArticulo)
            .then(artGruDes => {
                this.articuloGrupoDescuento = artGruDes;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    ngAfterViewChecked() {
        this.artGruDesCollection.sourceCollection = this.articuloGrupoDescuento;
    }

    onEditItem(item: ArticuloGrupoDescuentoItem) {
        this.articuloGrupoDescuentoEditorComponent.show();
        this.articuloGrupoDescuentoEditorComponent.data = item;
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SAVE') {
            this.actualizarArticuloGrupoDescuento();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    actualizarArticuloGrupoDescuento() {
        this.articuloService.updateArticuloGrupoDescuento(this.articuloGrupoDescuento, this.idArticulo)
            .then(r => {
                this.loadArticuloGrupoDescuento()
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    onDataUpdated(args: ArticuloGrupoDescuentoItem) {
        this.articuloGrupoDescuento.forEach(artListPre => {
            if (artListPre.id == args.id) {
                artListPre.state = 'CHANGED'
                artListPre.porcentajeDescuento1 = args.porcentajeDescuento1;
                artListPre.porcentajeDescuento2 = args.porcentajeDescuento2;
                artListPre.porcentajeDescuento3 = args.porcentajeDescuento3;
                artListPre.porcentajeDescuento4 = args.porcentajeDescuento4;
                artListPre.porcentajeDescuento5 = args.porcentajeDescuento5;
            }
        })
        this.artGruDesCollection.refresh();
    }
}