﻿//Angular
import { Component, forwardRef, ViewChild, ChangeDetectorRef, Input, Output, EventEmitter } from '@angular/core';
import { Route, Router, ActivatedRoute } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Framework 3
import { BaseComponent, CatalogService, Context } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor } from 'framework3/editors';
//Features
import { ArticuloService } from '../services/articulo.service';
import { ArticuloImpuestoItem } from '../models/articulo';
import { ImpuestoService } from '../../accounting/services/impuesto.service';
import { ImpuestoInfo } from '../../accounting/models/impuesto';
import { ImpuestosSelectorComponent } from '../../accounting/controls/impuestos-selector.component';

@Component({
    selector: 'sales-articulo-impuestos',
    templateUrl: './articulo-impuestos.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticulosImpuestosComponent)
        },
        {
            provide: CatalogService, useExisting: ArticuloService
        }
    ]
})

export class ArticulosImpuestosComponent extends CatalogEditor {
    //variables
    //articuloImpuestoItem: ArticuloImpuestoItem = new ArticuloImpuestoItem();
    editorForm: FormGroup;
    impuestoInfo: ImpuestoInfo = new ImpuestoInfo();
    tiposFactor: { id: number, name: string }[] = [
        { "id": 1, "name": "Tasa" },
        { "id": 2, "name": "Cuota" }
    ];
    tiposCalculo: { id: number, name: string }[] = [
        { "id": 1, "name": "Base" },
        { "id": 2, "name": "Cascada" }
    ];

    //Inputs
    @Input() data: ArticuloImpuestoItem = new ArticuloImpuestoItem();
    //Outputs
    @Output() dataUpdater: EventEmitter<ArticuloImpuestoItem> = new EventEmitter<ArticuloImpuestoItem>();

    //viewChilds
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild('selectorImpuestos') selectorImpuestos: ImpuestosSelectorComponent;

    constructor(private articulosService: ArticuloService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetectorRef: ChangeDetectorRef) {
        super(context, router, route, changeDetectorRef);
        this.editorForm = formBuilder.group({
            'impuestos': ['', AztecaValidators.selected],
            'tipoFactor': [0, AztecaValidators.selected],
            'valorTasa': [1],
            'valorCuota': [1],
            'tipoCalculo': [0, AztecaValidators.selected],
            'ordenCalculo': [1]
        });
    }

    onFactorChange() {
        //resetear el valor del campo tasa o cuota y si el campo rango es falso se asigna el valor fijo
        if (this.editorForm.controls.tipoFactor.value == 1) {
            this.data.cuota = 0;
            if (!this.impuestoInfo.rango)
                this.data.tasa = this.impuestoInfo.valorFijo;
        }
        if (this.editorForm.controls.tipoFactor.value == 2) {
            this.data.tasa = 0;
            if (!this.impuestoInfo.rango)
                this.data.cuota = this.impuestoInfo.valorFijo;
        }


    }

    onTipoCalculoChange() {
        if (this.editorForm.controls.tipoCalculo.value == 1) {
            this.data.tipoCalculo = 'B';
            this.data.ordenCalculo = 1;
        }
        else
            this.data.tipoCalculo = 'C';
        this.data.ordenCalculo = 2;
    }

    show() {
        this.data = new ArticuloImpuestoItem();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => this.editorForm.reset());
        Control.invalidateAll();
    }

    onUpdate() {
        this.dataUpdater.emit(this.data);
        this.dialog.hide();
    }

    onItemChanged(args: ImpuestoInfo) {
        this.data.impuesto = args.nombre;
        this.data.codigo = args.code;

        this.impuestoInfo = args;

        if (args.factor == 'T')
            this.editorForm.controls.tipoFactor.patchValue(1);
        if (args.factor == 'C')
            this.editorForm.controls.tipoFactor.patchValue(2);

        this.onFactorChange();
    }

}