﻿//Angular
import { Component, ViewChild, Input } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';

//Framework3
import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';

//Features
import { ArticuloListaPreciosItem } from '../models/articulo-lista-precios';
import { ArticuloService } from '../services/articulo.service';
import { ArticuloListaPreciosEditorComponent } from './articulo-lista-precios-editor.component';

@Component({
    selector: 'ventas-articulo-lista-precios.component',
    templateUrl: './articulo-lista-precios.component.html'
})
export class ArticuloListaPreciosComponent extends BaseComponent {
    title: string = 'Lista de precios del artículo';
    idArticulo: number;
    articuloListaPrecios: ArticuloListaPreciosItem[] = [];
    artListPreCollection: TrackableCollection = new TrackableCollection();
    currentItem: CatalogItem;

    @ViewChild(ArticuloListaPreciosEditorComponent) articuloListaPreciosEditorComponent: ArticuloListaPreciosEditorComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private context: Context, private articuloService: ArticuloService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idArticulo = +param['idArticulo'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadArticuloListaPrecios());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadArticuloListaPrecios() {
        this.context.app.showSpinner();
        this.articuloService.loadArticuloListaPrecios(this.context.app.defaultCompany.identity, this.idArticulo)
            .then(artListPre => {
                this.articuloListaPrecios = artListPre;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    ngAfterViewChecked() {
        this.artListPreCollection.sourceCollection = this.articuloListaPrecios;
    }

    onEditItem(item: ArticuloListaPreciosItem) {
        this.articuloListaPreciosEditorComponent.show();
        this.articuloListaPreciosEditorComponent.data = item;
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SAVE') {
            this.actualizarArticuloListaPrecios();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    actualizarArticuloListaPrecios() {
        this.articuloService.updateArticuloListaPrecios(this.articuloListaPrecios, this.idArticulo)
            .then(r => {
                this.loadArticuloListaPrecios()
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    onDataUpdated(args: ArticuloListaPreciosItem) {
        this.articuloListaPrecios.forEach(artListPre => {
            if (artListPre.id == args.id) {
                artListPre.state = 'CHANGED'
                artListPre.idDivisa = args.idDivisa;
                artListPre.divisa = args.divisa;
                artListPre.precio = args.precio;
            }
        })
        this.artListPreCollection.refresh();
    }
}