﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection, CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { ServicioGrupoDescuentoItem } from '../models/servicio-grupo-descuento';
import { ServicioService } from '../services/servicio.service';

@Component({
    selector: 'ventas-servicio-grupo-descuento-editor',
    templateUrl: './servicio-grupo-descuento-editor.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ServicioGrupoDescuentoEditorComponent)
        },
        {
            provide: CatalogService, useExisting: ServicioService
        }
    ]
})
export class ServicioGrupoDescuentoEditorComponent extends CatalogEditor {
    editorForm: FormGroup;
    info: ServicioGrupoDescuentoItem = new ServicioGrupoDescuentoItem();
    formato: String = ''

    //Inputs
    @Input() data: ServicioGrupoDescuentoItem = new ServicioGrupoDescuentoItem();
    //outputs
    @Output() dataUpdated: EventEmitter<ServicioGrupoDescuentoItem> = new EventEmitter<ServicioGrupoDescuentoItem>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private servicioService: ServicioService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetector: ChangeDetectorRef) {
        super(context, router, route, changeDetector);
        this.editorForm = formBuilder.group({
            'porcentajeDescuento1': [0],
            'porcentajeDescuento2': [0],
            'porcentajeDescuento3': [0],
            'porcentajeDescuento4': [0],
            'porcentajeDescuento5': [0]
        });
    }

    show() {
        this.info = new ServicioGrupoDescuentoItem();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => {
            this.editorForm.reset();
            this.info = this.data;
            this.data = new ServicioGrupoDescuentoItem();
        });
        Control.invalidateAll();
    }

    onSelectedItemChanged(args: any) {
        if (args != null)
            this.formato = 'n' + args.decimales.toString();
    }

    onUpdate() {
        this.dataUpdated.emit(this.info);
        this.dialog.hide();
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}