﻿//Angular
import { Component, ViewChild, Input } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';

//Framework3
import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';

//Features
import { ServicioGrupoDescuentoItem } from '../models/servicio-grupo-descuento';
import { ServicioService } from '../services/servicio.service';
import { ServicioGrupoDescuentoEditorComponent } from './servicio-grupo-descuento-editor.component';

@Component({
    selector: 'ventas-servicio-grupo-descuento.component',
    templateUrl: './servicio-grupo-descuento.component.html'
})
export class ServicioGrupoDescuentoComponent extends BaseComponent {
    title: string = 'Grupos de Descuento del servicio';
    idServicio: number;
    servicioGrupoDescuento: ServicioGrupoDescuentoItem[] = [];
    serGruDesCollection: TrackableCollection = new TrackableCollection();
    currentItem: CatalogItem;

    @ViewChild(ServicioGrupoDescuentoEditorComponent) servicioGrupoDescuentoEditorComponent: ServicioGrupoDescuentoEditorComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private context: Context, private servicioService: ServicioService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idServicio = +param['idServicio'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadServicioGrupoDescuento());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadServicioGrupoDescuento() {
        this.context.app.showSpinner();
        this.servicioService.loadServicioGrupoDescuento(this.context.app.defaultCompany.identity, this.idServicio)
            .then(serGruDesc => {
                this.servicioGrupoDescuento = serGruDesc;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    ngAfterViewChecked() {
        this.serGruDesCollection.sourceCollection = this.servicioGrupoDescuento;
    }

    onEditItem(item: ServicioGrupoDescuentoItem) {
        this.servicioGrupoDescuentoEditorComponent.show();
        this.servicioGrupoDescuentoEditorComponent.data = item;
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SAVE') {
            this.actualizarServicioGrupoDescuento();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    actualizarServicioGrupoDescuento() {
        this.servicioService.updateServicioGrupoDescuento(this.servicioGrupoDescuento, this.idServicio)
            .then(r => {
                this.loadServicioGrupoDescuento()
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    onDataUpdated(args: ServicioGrupoDescuentoItem) {
        this.servicioGrupoDescuento.forEach(serGruDesc => {
            if (serGruDesc.id == args.id) {
                serGruDesc.state = 'CHANGED'
                serGruDesc.porcentajeDescuento1 = args.porcentajeDescuento1;
                serGruDesc.porcentajeDescuento2 = args.porcentajeDescuento2;
                serGruDesc.porcentajeDescuento3 = args.porcentajeDescuento3;
                serGruDesc.porcentajeDescuento4 = args.porcentajeDescuento4;
                serGruDesc.porcentajeDescuento5 = args.porcentajeDescuento5;
            }
        })
        this.serGruDesCollection.refresh();
    }
}