﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection, CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { ServicioListaPreciosItem } from '../models/servicio-lista-precios';
import { ServicioService } from '../services/servicio.service';
import { DivisasSelectorComponent } from '../../accounting/controls/divisas-selector.component';

@Component({
    selector: 'ventas-servicio-lista-precios-editor',
    templateUrl: './servicio-lista-precios-editor.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ServicioListaPreciosEditorComponent)
        },
        {
            provide: CatalogService, useExisting: ServicioService
        }
    ]
})
export class ServicioListaPreciosEditorComponent extends CatalogEditor {
    editorForm: FormGroup;
    info: ServicioListaPreciosItem = new ServicioListaPreciosItem();
    formato: String = ''

    //Inputs
    @Input() data: ServicioListaPreciosItem = new ServicioListaPreciosItem();
    //outputs
    @Output() dataUpdated: EventEmitter<ServicioListaPreciosItem> = new EventEmitter<ServicioListaPreciosItem>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild('selectorDivisas') selectorDivisas: DivisasSelectorComponent;

    constructor(private servicioService: ServicioService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute,
        protected changeDetector: ChangeDetectorRef) {
        super(context, router, route, changeDetector);
        this.editorForm = formBuilder.group({
            'divisa': ['', AztecaValidators.selected],
            'precio': ['', Validators.required]
        });
    }

    show() {
        this.info = new ServicioListaPreciosItem();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => {
            this.editorForm.reset();
            this.info = this.data;
            this.data = new ServicioListaPreciosItem();
        });
        Control.invalidateAll();
    }

    onSelectedItemChanged(args: any) {
        if (args != null)
            this.formato = 'n' + args.decimales.toString();
    }

    onUpdate() {
        this.info.divisa = this.selectorDivisas.displayValue;
        this.dataUpdated.emit(this.info);
        this.dialog.hide();
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}