﻿//Angular
import { Component, ViewChild, Input } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';

//Framework3
import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { PolicyInfo, UsuarioItem, GrupoInfo, GrupoService } from 'framework3/core';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';

//Features
import { ServicioListaPreciosItem } from '../models/servicio-lista-precios';
import { ServicioService } from '../services/servicio.service';
import { ServicioListaPreciosEditorComponent } from './servicio-lista-precios-editor.component';

@Component({
    selector: 'ventas-servicio-lista-precios.component',
    templateUrl: './servicio-lista-precios.component.html'
})
export class ServicioListaPreciosComponent extends BaseComponent {
    title: string = 'Lista de precios del servicio';
    idServicio: number;
    servicioListaPrecios: ServicioListaPreciosItem[] = [];
    serListPreCollection: TrackableCollection = new TrackableCollection();
    currentItem: CatalogItem;

    @ViewChild(ServicioListaPreciosEditorComponent) servicioListaPreciosEditorComponent: ServicioListaPreciosEditorComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private context: Context, private servicioService: ServicioService, private grupoService: GrupoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idServicio = +param['idServicio'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadServicioListaPrecios());
    }

    //////////////////// METODOS /////////////////////////////////////////////////////////

    loadServicioListaPrecios() {
        this.context.app.showSpinner();
        this.servicioService.loadServicioListaPrecios(this.context.app.defaultCompany.identity, this.idServicio)
            .then(serListPre => {
                this.servicioListaPrecios = serListPre;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    //////////////////// HANDLERS /////////////////////////////////////////////////////////

    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    ngAfterViewChecked() {
        this.serListPreCollection.sourceCollection = this.servicioListaPrecios;
    }

    onEditItem(item: ServicioListaPreciosItem) {
        this.servicioListaPreciosEditorComponent.show();
        this.servicioListaPreciosEditorComponent.data = item;
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'SAVE') {
            this.actualizarServicioListaPrecios();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    actualizarServicioListaPrecios() {
        this.servicioService.updateServicioListaPrecios(this.servicioListaPrecios, this.idServicio)
            .then(r => {
                this.loadServicioListaPrecios()
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

    onDataUpdated(args: ServicioListaPreciosItem) {
        this.servicioListaPrecios.forEach(serListPre => {
            if (serListPre.id == args.id) {
                serListPre.state = 'CHANGED'
                serListPre.idDivisa = args.idDivisa;
                serListPre.divisa = args.divisa;
                serListPre.precio = args.precio;
            }
        })
        this.serListPreCollection.refresh();
    }
}