﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index';
//
import { EstructuraComercialInfo, EstructuraComercialItem } from '../models/estructura-comercial';
import { EstructuraComercialService } from '../services/estructura-comercial.service';

@Component({
    selector: 'sales-est-com-selector',
    templateUrl: './est-com-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => EstComSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => EstComSelectorComponent),
        multi: true
    }]
})
export class EstComSelectorComponent extends CatalogSelector {
    estCom: any[];
    selectedItem: EstructuraComercialInfo = new EstructuraComercialInfo;

    @Output() selectedItemChanged = new EventEmitter<EstructuraComercialInfo>();

    constructor(protected context: Context, private estructuraComercialService: EstructuraComercialService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findEstCom(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.estructuraComercialService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.namedPath;
                    this.selectedItemChanged.emit(info);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.estructuraComercialService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.estCom = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findEstCom(keyword: string): Observable<CatalogItem[]> {
        return this.estructuraComercialService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

