﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector } from 'framework3/search/index';
//
import { MarcasInfo, MarcasItem} from '../models/marcas';
import { MarcasService } from '../services/marcas.service';

@Component({
    selector: 'sales-marcas-selector',
    templateUrl: './marcas-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => MarcasSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => MarcasSelectorComponent),
        multi: true
    }]
})
export class MarcasSelectorComponent extends CatalogSelector {
    marcas: any[];
    selectedItem: MarcasInfo = new MarcasInfo;

    constructor(protected context: Context, private marcasService: MarcasService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
             observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findMarcas(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.marcasService.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        this.marcasService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.marcas = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findMarcas(keyword: string): Observable<CatalogItem[]> {
        return this.marcasService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}

