﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index'; //'../../search/index';
import { DocumentSelector } from 'framework3/search/models/document-selector';
//
import { VentaInfo, VentaItem } from '../models/venta';
import { VentaService } from '../services/venta.service';

@Component({
    selector: 'sales-ventas-selector',
    templateUrl: './ventas-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => VentasSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => VentasSelectorComponent),
        multi: true
    }]

})
export class VentasSelectorComponent extends DocumentSelector {
    venta: any[];
    selectedItem: VentaInfo = new VentaInfo;

    //
    @Output() selectedItemChanged: EventEmitter<VentaInfo> = new EventEmitter<VentaInfo>();

    constructor(protected context: Context, private ventaService: VentaService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findVentas(keyword)));
    }

    loadItem(itemID: string) {
        if (itemID != '') {
            this.ventaService.getDetail(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.identity;
                    this.selectedItemChanged.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
            this.selectedItemChanged.emit(null);
        }
    }

    doSearch(request: SearchRequest) {
        this.context.app.showSpinner();

        let condicion: SearchCondition = { field: "IdStatus", operator: ">", valueType: "int", value: "1" };
        request.conditions.push(condicion);


        this.ventaService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.venta = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findVentas(keyword: string): Observable<VentaItem[]> {
        return this.ventaService.searchKeyword(this.context.app.defaultSite.identity, keyword);
    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}