﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Inventario
import { Empaquetado } from '../../inventory';
//Locales
import { DevolucionInfo, DevolucionItem } from '../models/devolucion';
import { DevolucionDetail } from '../models/devolucion-detail';
import { DevolucionService } from '../services/devolucion.service';
import { DevolucionDetailEditor } from './parts/devoluciones-detail.editor';

import { DevolucionArticuloDetail } from '../models/devolucion-articulo-detail';
import { DevolucionServicioDetail } from '../models/devolucion-servicio-detail';

import { VentaArticuloDetail } from '../models/venta-articulo-detail';
import { VentaServicioDetail } from '../models/venta-servicio-detail';

import { VentaInfo, VentaItem } from '../models/venta';
import { ImpuestoDetailInfo, DivisaService, DivisaInfo } from '../../accounting';

import { ClienteService } from '../services/cliente.service';
import { ClienteInfo } from '../models/cliente';
import { UnidadNegocioService } from '../services/unidad-negocio.service';
import { UnidadNegocioInfo } from '../models/unidad-negocio';
import { AlmacenItem, AlmacenInfo, AlmacenService } from '../../inventory';

import { VentasSelectorComponent } from '../controls/ventas-selector.component';


@Component({
    selector: 'ventas-devoluciones-editor',
    templateUrl: './devoluciones.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => DevolucionesEditor)
        },
        {
            provide: DocumentService, useExisting: DevolucionService
        }
    ]
})
export class DevolucionesEditor extends DocumentEditor {
    devolucion = new DevolucionInfo();
    venta = new VentaInfo();

    articulos: DevolucionArticuloDetail[] = [];
    servicios: DevolucionServicioDetail[] = [];

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: DevolucionDetailEditor;
    @ViewChild('selectorVentas') selectorVentas: VentasSelectorComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private devolucionService: DevolucionService,
        private almacenService: AlmacenService,
        private divisaService: DivisaService,
        private unidadNegocioService: UnidadNegocioService,
        private clienteService: ClienteService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idVenta': [''],
        });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.devolucionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.devolucion = doc;
                this.setReadonlyMode();
                this.articulos = this.devolucion.details.map(d => <DevolucionArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.servicios = this.devolucion.details.map(d => <DevolucionServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.devolucionService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.devolucion = doc;
                //
                this.articulos = this.devolucion.details.map(d => <DevolucionArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.servicios = this.devolucion.details.map(d => <DevolucionServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
                //

                this.detailsEditor.clientIndexStart = this.devolucion.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.devolucionService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.devolucion = doc;
                //                this.selectorVentas.selectedID = this.devolucion.idVenta;
                this.selectorVentas.displayValue = this.devolucion.idVenta;
                this.selectorVentas.enabled = false;
                //
                this.articulos = this.devolucion.details.map(d => <DevolucionArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.articulos.forEach(art => {
                    let paquete = new Empaquetado(art.cantidadRef, art.unidadMedida.factor);
                    art.cantidadRef = paquete.cantidad;
                    art.unidadesRef = paquete.unidades;
                });
                this.servicios = this.devolucion.details.map(d => <DevolucionServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
                //

                this.detailsEditor.clientIndexStart = this.devolucion.details.length;
                if (this.devolucion.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    combineDetail(): DevolucionDetail[] {
        let newDetail: DevolucionDetail[] = [];
        this.servicios.forEach(s => newDetail.push(s));
        this.articulos.forEach(a => newDetail.push(a));
        return newDetail;
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.devolucion.details = this.combineDetail();

        this.devolucionService.update(this.devolucion)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.devolucion.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.devolucion = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.devolucionService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.devolucionService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.devolucion.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new DevolucionDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: DevolucionDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: DevolucionDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: DevolucionDetail[]) {
        this.devolucion.details = updatedDetails;
    }

    onSavingDetail(detail: DevolucionDetail) {
        this.detailsList.updateItem(detail);
    }

    onSelectedItemChanged(venta: VentaInfo) {

        this.venta = venta;

        this.devolucion.idVenta = venta.identity;
        this.devolucion.fechaVenta = venta.fechaSys;
        this.devolucion.idAlmacen = venta.idAlmacen;
        this.devolucion.idCliente = venta.idCliente;
        this.devolucion.idDivisa = venta.idDivisa;
        this.devolucion.idUnidadNegocio = venta.idUnidadNegocio;
        //this.devolucion.idReferencia = 
        this.devolucion.idSucursal = this.context.app.defaultSite.idBranch;
        this.devolucion.ordenCompra = venta.ordenCompra;
        this.devolucion.tipoCambio = venta.tipoCambio;

        this.cargarInfAdicional();

        venta.details.filter(d => d.idArticulo > 0).map(d => <VentaArticuloDetail>d).forEach(det => {

            var newDetArt: DevolucionArticuloDetail = new DevolucionArticuloDetail;

            newDetArt.articulo = det.articulo;
            newDetArt.unidadMedida = det.unidadMedida;
            newDetArt.stock = det.stock;

            newDetArt.codigo = det.codigo;
            let paquete = new Empaquetado(det.cantidadDevolucion, det.unidadMedida.factor);
            newDetArt.cantidad = det.cantidad - paquete.cantidad; 
            newDetArt.cantidadRef = det.cantidad - paquete.cantidad; 
            newDetArt.idUnidadMedida = det.idUnidadMedida;
            newDetArt.unidades = det.unidades - paquete.unidades;
            newDetArt.unidadesRef = det.unidades - paquete.unidades;
            newDetArt.descripcion = det.descripcion;
            newDetArt.flete = 0;
            newDetArt.idArticulo = det.idArticulo;
            newDetArt.idServicio = det.idServicio;
            newDetArt.descuentos.descuento1 = det.descuentos.descuento1;
            newDetArt.descuentos.descuento2 = det.descuentos.descuento2;
            newDetArt.descuentos.descuento3 = det.descuentos.descuento3;
            newDetArt.descuentos.descuento4 = det.descuentos.descuento4;
            newDetArt.descuentos.descuento5 = det.descuentos.descuento5;
            newDetArt.descuentos.porcentajeDescuento1 = det.descuentos.porcentajeDescuento1;
            newDetArt.descuentos.porcentajeDescuento2 = det.descuentos.porcentajeDescuento2;
            newDetArt.descuentos.porcentajeDescuento3 = det.descuentos.porcentajeDescuento3;
            newDetArt.descuentos.porcentajeDescuento4 = det.descuentos.porcentajeDescuento4;
            newDetArt.descuentos.porcentajeDescuento5 = det.descuentos.porcentajeDescuento5;
            newDetArt.precioUnitario = det.precioUnitario;

            //Llenando los impuestos
            det.impuestos.forEach(imp => {
                var newImp: ImpuestoDetailInfo = new ImpuestoDetailInfo;
                newImp.base = imp.base;
                newImp.importe = imp.importe;
                newImp.impuesto = imp.impuesto;
                newImp.tasaOCuota = imp.tasaOCuota;
                newImp.tipoAplicacion = imp.tipoAplicacion;
                newImp.tipoFactor = imp.tipoFactor;
                newDetArt.impuestos.push(newImp);
            });

            this.devolucion.details.push(newDetArt);
        });

        venta.details.filter(d => d.idServicio > 0).map(d => <VentaArticuloDetail>d).forEach(det => {

            var newDetSer: DevolucionServicioDetail = new DevolucionServicioDetail;

            newDetSer.codigo = det.codigo;
            newDetSer.cantidad = det.cantidad - det.cantidadDevolucion;
            newDetSer.cantidadRef = det.cantidad - det.cantidadDevolucion;
            newDetSer.descripcion = det.descripcion;
            newDetSer.flete = 0;
            newDetSer.idArticulo = det.idArticulo;
            newDetSer.idServicio = det.idServicio;
            newDetSer.descuentos.descuento1 = det.descuentos.descuento1;
            newDetSer.descuentos.descuento2 = det.descuentos.descuento2;
            newDetSer.descuentos.descuento3 = det.descuentos.descuento3;
            newDetSer.descuentos.descuento4 = det.descuentos.descuento4;
            newDetSer.descuentos.descuento5 = det.descuentos.descuento5;
            newDetSer.descuentos.porcentajeDescuento1 = det.descuentos.porcentajeDescuento1;
            newDetSer.descuentos.porcentajeDescuento2 = det.descuentos.porcentajeDescuento2;
            newDetSer.descuentos.porcentajeDescuento3 = det.descuentos.porcentajeDescuento3;
            newDetSer.descuentos.porcentajeDescuento4 = det.descuentos.porcentajeDescuento4;
            newDetSer.descuentos.porcentajeDescuento5 = det.descuentos.porcentajeDescuento5;
            newDetSer.precioUnitario = det.precioUnitario;

            //Llenando los impuestos
            det.impuestos.forEach(imp => {
                var newImp: ImpuestoDetailInfo = new ImpuestoDetailInfo;
                newImp.base = imp.base;
                newImp.importe = imp.importe;
                newImp.impuesto = imp.impuesto;
                newImp.tasaOCuota = imp.tasaOCuota;
                newImp.tipoAplicacion = imp.tipoAplicacion;
                newImp.tipoFactor = imp.tipoFactor;
                newDetSer.impuestos.push(newImp);
            });

            this.devolucion.details.push(newDetSer);
        });
        
        this.articulos = this.devolucion.details.map(d => <DevolucionArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
        this.servicios = this.devolucion.details.map(d => <DevolucionServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
        
     }

    cellEditEndingHandler(sender: any, e: any) {

        let art: any = sender.rows[e.row].dataItem;

        var flex = sender,
            oldCantVal = art.cantidadRef,
            oldUnitVal = art.unidadesRef,
            newVal = flex.activeEditor.value;


        /*****************/
        //Obteniendo el neuvo valor empaquetado
        var cajas: number;
        var botellas: number;
        if (e.col == 3) {
            cajas = flex.activeEditor.value;
            botellas = flex.getCellData(e.row, 4);
        }
        if (e.col == 4) {
            cajas = flex.getCellData(e.row, 3);
            botellas = flex.activeEditor.value;
        }

        let paqueteNew = new Empaquetado(Math.trunc(cajas), art.unidadMedida.factor, Math.round(botellas));
        let paqueteOld = new Empaquetado(Math.trunc(oldCantVal), art.unidadMedida.factor, Math.round(oldUnitVal));

        if (paqueteNew.toDecimal() > paqueteOld.toDecimal()) {
            this.showAlert("No puede devolver una cantidad mayor al saldo", "danger");
            e.cancel = true;
            art.cantidad = oldCantVal;
            art.unidades = oldUnitVal;
        }


    }

    onCellEditEnded(sender: any, e: any) {

    }

    cargarInfAdicional() {

        //Cargar información del almacén origen
        this.almacenService.getDetailByID(this.devolucion.idAlmacen)
            .then(almacen => this.devolucion.almacen = almacen)
            .catch(error => this.handleError(error));

        //Cargar información de la unidad de negocio
        this.unidadNegocioService.getDetailByID(this.devolucion.idUnidadNegocio)
            .then(unidadNegocio => this.devolucion.unidadNegocio = unidadNegocio)
            .catch(error => this.handleError(error));

        this.divisaService.getDetailByID(this.devolucion.idDivisa)
            .then(divisa => {
                this.devolucion.divisa = divisa;
            })
            .catch(error => this.handleError(error));

        this.clienteService.getDetailByID(this.devolucion.idCliente)
            .then(cliente => this.devolucion.cliente = cliente)
            .catch(error => this.handleError(error));


    }


}

