﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder } from '@angular/forms';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';

//
import { Context } from 'framework3/core';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//
import { AreaItem, AlmacenItem, AreaService, AlmacenService } from '../../../inventory';
import { DivisaService, DivisaInfo } from '../../../accounting';
import { UnidadNegocioItem } from '../../models/unidad-negocio';
import { UnidadNegocioService } from '../../services/unidad-negocio.service';
import { ClienteInfo } from '../../models/cliente';

@Component({
    selector: 'sales-nueva-venta',  
    templateUrl: './nueva-venta.component.html'
})
export class NuevaVentaComponent {
    valid = false;
    form: FormGroup;
    idUnidadNegocio: number;
    idArea: number;
    idAlmacen: number;
    idDivisa: number;
    idCliente: number;
    idOrigen: number;
    cliente: ClienteInfo;
    unidadesNegocio: UnidadNegocioItem[] = [];
    areas: AreaItem[] = [];
    almacenes: AlmacenItem[] = [];
    selectedUnidad: UnidadNegocioItem;

    /////////// INPUTS

    /////////// OUTPUS

    @ViewChild(AlertsBarComponent) alertsBar: AlertsBarComponent;

    constructor(private formBuilder: FormBuilder, private router: Router, private route: ActivatedRoute, private context: Context,
        private areaService: AreaService,
        private almacenService: AlmacenService,
        private unidadNegocioService: UnidadNegocioService,
        private divisaService: DivisaService) {

        this.form = formBuilder.group({
            'idUnidadNegocio': [0, AztecaValidators.selected],
            'idArea': [0],
            'idAlmacen': [0],
            'idDivisa': [0, AztecaValidators.selected],
            'idCliente': [0, AztecaValidators.selected]
        });
        //
        this.form.controls['idUnidadNegocio'].valueChanges.subscribe(value => {
            this.selectedUnidad = this.unidadesNegocio.find(u => u.id == value);
            if (this.selectedUnidad) {
                if (this.selectedUnidad.editarAlmacen)                 
                    this.form.controls['idAlmacen'].setValidators(AztecaValidators.selected);
                else
                    this.form.controls['idAlmacen'].setValidators(null);
                this.form.controls['idAlmacen'].updateValueAndValidity();
            }
        });
        //
        this.form.statusChanges.subscribe(_ => setTimeout(_ => this.valid = this.form.valid));

        this.route.params.forEach((params: Params) => {
            this.idOrigen = params['idOrigen'] ? +params['idOrigen'] : 0;
        });

    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadUnidadesNegocio());
        setTimeout(_ => this.cargarDivisaDefault());
    }

    /////////// METHODS

    private loadUnidadesNegocio() {
        this.context.app.showSpinner();
        this.unidadNegocioService.loadListByUser(this.context.app.defaultSite.identity, this.context.session.user.identity)
            .then(unidades => {
                this.unidadesNegocio = unidades;
                this.context.app.hideSpinner();
                this.selectedUnidad = this.unidadesNegocio[0];
                if (this.selectedUnidad.editarAlmacen) this.loadAreas();
            })
            .catch(error => this.handleError(error));
    }

    private loadAreas() {
        this.context.app.showSpinner();
        this.areaService.loadListByUser(this.context.app.defaultSite.identity, this.context.session.user.identity)
            .then(areas => {
                this.areas = areas;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));

    }

    private loadAlmacenes(idArea: number) {
        this.idAlmacen = 0;
        this.context.app.showSpinner();
        this.almacenService.loadListByUser(idArea, this.context.session.user.identity)
            .then(almacenes => {
                this.almacenes = almacenes.filter(almacen => almacen.salida);
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    cargarDivisaDefault() {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(divisa => {
                this.idDivisa = divisa.identity;                
            })
            .catch(error => this.handleError(error));
    }


    /////////// HANDLERS

    onToolbarClick(button: ToolbarButton) {
        if (button.name == 'OK') {
            let idAlmacen = this.idAlmacen;
            if (this.selectedUnidad.afectarInventario) {
                //Si no está permitido que el usuario elija el almacén origen, utilizar el que la unidad de negocios tenga configurado
                if (!this.selectedUnidad.editarAlmacen) idAlmacen = this.selectedUnidad.idAlmacen;
            }
            else {
                idAlmacen = 0;
            }
            //
            this.router.navigate(['../', '0', { idUnidadNegocio: this.idUnidadNegocio, idAlmacen: idAlmacen, idDivisa: this.idDivisa, idCliente: this.idCliente, idOrigen: this.idOrigen }], { relativeTo: this.route });
        }
        else if (button.name == 'CANCEL') {
            this.router.navigate(['../'], { relativeTo: this.route });
        }
    }

    onClienteChanged(cliente: ClienteInfo) {
        this.cliente = cliente;
    }

    handleError(error: any) {
        console.log(error);
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alertsBar.showAlert(message, 'danger');
    }
}
