﻿import { Component, Input, OnInit, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';
//Wijmo
import { Globalize } from 'wijmo/wijmo';
//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
import { AztecaValidators } from 'framework3/controls';
//Inventario
import { ArticuloStockInfo, UnidadMedidaInfo, UnidadMedidaItem, StockInfo, LoteInfo, SerieInfo, UnidadMedidaService, AlmacenService, MetodoValuacion } from '../../../inventory';
//Ventas
import { VentaDetail } from '../../models/venta-detail';
import { VentaArticuloDetail } from '../../models/venta-articulo-detail';
import { ArticuloInfo } from '../../models/articulo';
import { ArticuloService } from '../../services/articulo.service';

import { SucursalService }from'../../services/sucursal.service'

@Component({
    selector: 'venta-articulos-detail-editor',
    templateUrl: './venta-articulos-detail.editor.html'
})
export class VentaArticulosDetailEditor extends BasicEditor {    
    unidadesMedida: UnidadMedidaItem[] = [];
    stockConfig: StockInfo;
    validInput: boolean = false;
    lotes: LoteInfo[] = [];
    series: SerieInfo[] = [];
    precioUnitario: string = '$0.0';
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new VentaArticuloDetail();
    //Objeto original que se editará
    @Input() detail: VentaArticuloDetail = new VentaArticuloDetail();
    @Input() idCliente: number = 0;    
    @Input() idAlmacen: number = 0;

    idZonaVenta: number = 0;

    constructor(protected context: Context, private builder: FormBuilder, private unidadService: UnidadMedidaService,
        private almacenService: AlmacenService, private articuloService: ArticuloService, private sucursalService: SucursalService) {
        super(context);

        this.form = builder.group({
            'idArticulo': [0, AztecaValidators.selected],
            'precio':[0]
        });
    }

    /////////////////// METODOS ///////////////////////////////
    loadStock(idArticulo: number) {
        this.almacenService.loadStock(this.idAlmacen, idArticulo)
            .then(stock => {
                this.stockConfig = stock;
                this.loadUnidadesMedida(this.detailCopy.articulo.unidadMedida.idUnidadMedidaTipo);
            })
            .catch(error => this.handleError(error));
    }

    loadUnidadesMedida(idTipoUnidad: number) {
        //Si el articulo esta controlado por series, no se puede cambiar la unidad de medida, por lo tanto solo se debe utilizar la unidad de medida el articulo,
        //en caso contrario, cargar lista de unidades de medida del mismo tipo
        if (this.stockConfig.manejaSeries) {
            let unidadArticulo = this.detailCopy.articulo.unidadMedida;
            let newUnidad: UnidadMedidaItem = {
                id: unidadArticulo.identity, codigo: unidadArticulo.code, idUnidadMedidaTipo: unidadArticulo.idUnidadMedidaTipo, base: unidadArticulo.base,
                factor: unidadArticulo.factor, fraccionable: unidadArticulo.fraccionable, paquete: unidadArticulo.paquete, activo: true
            };
            let newUnidades = [];
            newUnidades.push(newUnidad);
            this.unidadesMedida = newUnidades;
            this.detailCopy.idUnidadMedida = unidadArticulo.identity;
        }
        else {
            this.unidadService.loadList(idTipoUnidad)
                .then(unidadesMedida => {
                    this.unidadesMedida = unidadesMedida;
                    this.detailCopy.idUnidadMedida = this.detailCopy.articulo.idUnidadMedida;
                })
                .catch(error => this.handleError(error));
        }
    }


    /////////////////// HANDLERS //////////////////////////////
    //ngAfterViewInit() {
    ngOnInit() {
        setTimeout(_ => this.loadSucursal());
    }

    loadSucursal() {
        this.sucursalService.getDetailByID(this.context.app.defaultSite.identity)
            .then(suc => {                
                this.idZonaVenta = suc.idZonaVenta;
            })
            .catch(error => this.handleError(error));
    }

    onCreatingItem() {
        this.detailCopy = this.detail;
		this.form.reset({ sample: '' });
        this.form.markAsPristine();

        (this.context.app.defaultSite.identity);
    }

    onLoadingItem(detail: VentaDetail ) {
        this.detailCopy = <VentaArticuloDetail>this.copyItem(this.detail);
    }

    onUpdatingItem(detail: VentaArticuloDetail) {
        this.unidadService.getDetailFromCache(this.detailCopy.idUnidadMedida)
            .then(unidad => {
                //Si el artículo maneja lotes y ningún método de valuación, el usuario podrá especificar indicar manualmente los lotes y por lo cual generar más de un detalle
                if (!this.stockConfig.manejaSeries && this.stockConfig.manejaLotes && (this.stockConfig.metodoValuacion == MetodoValuacion.NINGUNO || this.stockConfig.metodoValuacion == '')) {
                    let details: VentaDetail[] = [];
                    //Crear una partida por cada lote que el usuario haya seleccionado
                    this.lotes.filter(lote => lote.cantidad > 0 || lote.unidades > 0).forEach(lote => {
                        let detail = new VentaArticuloDetail();
                        detail._clientID = this.createClientID();
                        detail.idArticulo = this.detailCopy.idArticulo;
                        detail.codigo = this.detailCopy.codigo;
                        detail.descripcion = this.detailCopy.descripcion;
                        detail.cantidad = lote.cantidad;
                        detail.unidades = lote.unidades;
                        //detail.costo = lote.costo;
                        detail.idLote = lote.id;
                        detail.idUnidadMedida = this.detailCopy.idUnidadMedida;
                        //
                        detail.articulo = this.detailCopy.articulo;
                        detail.stock = this.stockConfig;
                        detail.unidadMedida = unidad;

                        //Asignacion de importes
                        detail.precioUnitario = this.detailCopy.precioUnitario;
                        detail.precioNeto = this.detailCopy.precioNeto;
                        detail.descuentos = this.detailCopy.descuentos;
                        detail.impuestos = this.detailCopy.impuestos;                    
                        detail.importe = detail.precioNeto * detail.cantidad;
                        //
                        details.push(detail);
                    });
                    this.savingItems.emit(details);
                }
                //Si el artículo maneja series, crear un detalle por cada número de serie
                else if (this.stockConfig.manejaSeries && (this.stockConfig.metodoValuacion == MetodoValuacion.NINGUNO || this.stockConfig.metodoValuacion == '')) {
                    let details: VentaArticuloDetail[] = [];
                    //Crear una partida por cada lote que el usuario haya seleccionado
                    this.series.forEach(item => {
                        let detail = new VentaArticuloDetail();
                        detail._clientID = this.createClientID();
                        detail.idArticulo = this.detailCopy.idArticulo;
                        detail.numeroSerie = item.numeroSerie;
                        detail.codigo = this.detailCopy.codigo;
                        detail.descripcion = this.detailCopy.descripcion;
                        detail.cantidad = 1;
                        detail.unidades = 0;
                        //detail.costoUnitario = item.costo;
                        detail.idLote = item.idLote;
                        detail.idUnidadMedida = this.detailCopy.idUnidadMedida;
                        //
                        detail.articulo = this.detailCopy.articulo;
                        detail.stock = this.stockConfig;
                        detail.unidadMedida = unidad;                        
                        //
                        details.push(detail);
                    });
                    this.savingItems.emit(details);
                }
                else {
                    this.detailCopy.stock = this.stockConfig;
                    this.detailCopy.unidadMedida = unidad;
                    this.detailCopy._clientID = this.createClientID();
                    this.detailCopy.importe = this.detailCopy.precioNeto * this.detailCopy.cantidad;
                    this.savingItem.emit(this.detailCopy);
                }
            })
            .catch(error => this.handleError(error));
    }

    onArticuloChanged(articulo: ArticuloInfo) {
        if (articulo) {
            this.detailCopy.codigo = articulo.code;
            this.detailCopy.descripcion = articulo.nombre;
            this.detailCopy.articulo = articulo;
            this.detailCopy.idEstructuraComercial = articulo.idEstCom;
            this.loadStock(articulo.articuloStock.identity);
            this.articuloService.calcularPrecioCliente(this.detailCopy.idArticulo, this.idCliente, this.context.app.defaultCompany.identity, this.idZonaVenta)
                .then(precio => {
                    this.detailCopy.precioUnitario = precio.precioBase;
                    this.detailCopy.precioNeto = precio.precioNeto;
                    this.detailCopy.descuentos = precio.descuentos;
                    this.detailCopy.impuestos = precio.impuestos;                    
                    this.precioUnitario = Globalize.formatNumber(this.detailCopy.precioUnitario, 'c');
                })
                .catch(error => this.handleError(error));                                        
        }
        else {
            this.unidadesMedida = [];
            this.stockConfig = new StockInfo();
        }
    }

    onInputValidChange(valid: boolean) {
        this.validInput = valid;
    }

    onLotesChange(lotes: LoteInfo[]) {
        this.lotes = lotes;
    }

    onSeriesChange(series: SerieInfo[]) {
        this.series = series;
    }
}
