﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo, FileLink } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, DownloadPopupComponent} from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Pixkua
import { DivisaService, DivisaInfo, ComprobanteFiscalService } from '../../accounting';
import { AlmacenItem, AlmacenInfo, AlmacenService, DetailQuantityEditor, Empaquetado } from '../../inventory';
import { CajaItem, CajaService, CobranzaComponent, CobranzaFormaPagoDetail } from '../../credit';
//Ventas
import { VentaInfo, VentaItem } from '../models/venta';
import { VentaDetail } from '../models/venta-detail';
import { VentaService } from '../services/venta.service';
import { VentaArticulosDetailEditor } from './parts/venta-articulos-detail.editor';
import { VentaServiciosDetailEditor } from './parts/venta-servicios-detail.editor';
import { VentaArticuloDetail} from '../models/venta-articulo-detail';
import { VentaServicioDetail } from '../models/venta-servicio-detail';
import { ClienteService } from '../services/cliente.service';
import { ClienteInfo } from '../models/cliente';
import { UnidadNegocioService } from '../services/unidad-negocio.service';
import { UnidadNegocioInfo } from '../models/unidad-negocio';
import { EmpresaService } from '../services/empresa.service';
import { ImportesInfo } from '../../accounting/models/importes';


@Component({
    selector: 'sales-ventas-editor',
    templateUrl: './ventas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => VentasEditor)
        },
        {
            provide: DocumentService, useExisting: VentaService
        }
    ]
})
export class VentasEditor extends DocumentEditor {
    venta = new VentaInfo();    
    articulos: VentaArticuloDetail[] = [];
    servicios: VentaServicioDetail[] = [];

    condicionesPagoItems: { id: string, nombre: string }[] = [];
    metodoCobranzaEfectivo: string="";

    argsEditorEvent: EditorEventArgs;
    argsCurrentDocument: WorkflowEventArgs;    

    //
    @ViewChild('articulosList') articulosList: EditableListComponent;
    @ViewChild('serviciosList') serviciosList: EditableListComponent;
    @ViewChild('articulosEditor') articulosEditor: VentaArticulosDetailEditor;
    @ViewChild('serviciosEditor') serviciosEditor: VentaServiciosDetailEditor;

    @ViewChild('dlgCobranza') dlgCobranza: CobranzaComponent;
    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;
    @ViewChild(DetailQuantityEditor) quantityEditor: DetailQuantityEditor;

    get importes() {
        return this.importesDesglosados();
    }

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private ventaService: VentaService,
        private divisaService: DivisaService,
        private almacenService: AlmacenService,
        private unidadNegocioService: UnidadNegocioService,
        private clienteService: ClienteService,
        private empresaService: EmpresaService) {

        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'idVendedor': [0],
            'condicionesPago': ['E']
        });
        this.condicionesPagoItems.push({ id: 'E', nombre: 'Contado'});
        this.condicionesPagoItems.push({ id: 'C', nombre: 'Parcialidades o diferido' });
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();        

        this.empresaService.getDetailFromCache(this.context.app.defaultCompany.identity)
            .then(empresa => { 
                this.metodoCobranzaEfectivo = empresa.efectivoMetodoCobranza;
            })
            .catch(error => this.handleError(error));
    }

    onCreatingDocument() {        
        this.articulos = [];
        this.servicios = [];
        //Cargar parámetros que se proporcionan cuando es un nuevo documento
        this.route.params.forEach(params => {
            this.venta.idUnidadNegocio = +params['idUnidadNegocio'];
            this.venta.idAlmacen = +params['idAlmacen'];
            this.venta.idCliente = +params['idCliente'];
            this.venta.idDivisa = +params['idDivisa'];
            if (params['idOrigen']) {
                this.venta.idOrigen = +params['idOrigen'];
            }
        });

        setTimeout(_ => this.cargarInfomacionEntidades());
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.ventaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.venta = doc;
                this.setReadonlyMode();
                this.articulos = this.venta.details.map(d => <VentaArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.servicios = this.venta.details.map(d => <VentaServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.ventaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.venta = doc;
                //
                this.articulos = this.venta.details.map(d => <VentaArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.servicios = this.venta.details.map(d => <VentaServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
                //
                this.articulosEditor.clientIndexStart = this.articulos.length;
                this.serviciosEditor.clientIndexStart = this.servicios.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.ventaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.venta = doc;
                //
                this.articulos = this.venta.details.map(d => <VentaArticuloDetail>d).filter(d => d.idArticulo && d.idArticulo > 0);
                this.servicios = this.venta.details.map(d => <VentaServicioDetail>d).filter(d => d.idServicio && d.idServicio > 0);
                //
                this.articulosEditor.clientIndexStart = this.articulos.length;
                this.serviciosEditor.clientIndexStart = this.servicios.length;
                if (this.venta.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
                setTimeout(_ => this.refreshValidity());
            })
            .catch(error => this.handleError(error));
    }

    combineDetail(): VentaDetail[] {
        let newDetail: VentaDetail[] = [];
        this.servicios.forEach(s => newDetail.push(s));
        this.articulos.forEach(a => newDetail.push(a));
        return newDetail;
    }

    onSavingDocument(args: EditorEventArgs) {
        this.argsEditorEvent = args;
        this.context.app.showSpinner();
        this.venta.details = this.combineDetail();
           
        this.ventaService.update(this.venta)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.venta.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.venta = updatedDoc;
            })
            .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.argsCurrentDocument = args;
        if (args.step.activities.find(a => a.tag == 'CREDIT') && this.venta.condicionesPago == "E" && this.metodoCobranzaEfectivo =="A" ){
            this.dlgCobranza.show(this.venta.total, this.venta.cliente.deudor.identity);
        }
        else {
            this.context.app.showSpinner();
            this.ventaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
                .then(_ => {
                    this.context.app.hideSpinner();
                    //Volver a cargar documento
                    this.onViewingDocument(args.document.id);
                })
                .catch(error => this.handleError(error));
        }
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.ventaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.venta.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PDF') {
            this.context.app.showSpinner();
            this.ventaService.generarPdfFactura(this.venta.identity)
                .then(uri => {
                    this.context.app.hideSpinner();
                    window.open('/file-server/' + uri, '_blank');
                })
                .catch(error => this.handleError(error));
        }

        if (option.name == "DOWNLOAD") {
            this.context.app.showSpinner();
            //Generar PDF
            this.ventaService.generarPdfFactura(this.venta.identity)
                .then(pdf => {
                    //Descargar xml
                    this.ventaService.getXmlFactura(this.venta.identity)
                        .then(xml => {
                            this.context.app.hideSpinner();
                            let links: FileLink[] = [];
                            links.push(pdf);
                            links.push(xml);
                            this.downloadPopup.show(links);
                        })
                        .catch(error => this.handleError(error));

                })
                .catch(error => this.handleError(error));
        }

    }

    ///////////// ARTICULOS /////////////////////////

    onAddingArticulo() {
        this.articulosEditor.detail = new VentaArticuloDetail();
        this.articulosEditor.show();
    }

    onEditingArticulo(detail: VentaArticuloDetail) {
        this.quantityEditor.unidadMedida = detail.unidadMedida;
        this.quantityEditor.cantidad = detail.cantidad;
        this.quantityEditor.unidades = detail.unidades;
        this.quantityEditor.show();
    }

    onDeletingArticulo(detail: VentaArticuloDetail) {
        this.articulosList.deleteItem(detail);
    }

    onArticulosUpdated(updatedItems: VentaArticuloDetail[]) {
        this.articulos = updatedItems;
    }

    onSavingArticulo(detail: VentaArticuloDetail) {
        this.articulosList.updateItem(detail);
    }

    onSavingArticulos(details: VentaArticuloDetail[]) {
        this.articulosList.updateItems(details);
    }

    onUpdateQuantity(value: any) {
        let detail = <VentaArticuloDetail>this.articulosList.currentItem;
        if (detail.unidadMedida.paquete) {
            let cantidad = <Empaquetado>value;
            detail.cantidad = cantidad.paquetes;
            detail.unidades = cantidad.unidades;
        }
        else {
            detail.cantidad = <number>value;
        }
        this.articulosList.updateItem(detail);
    }

    ///////////// SERVICIOS /////////////////////////

    onAddingServicio() {
        this.serviciosEditor.detail = new VentaServicioDetail();
        this.serviciosEditor.show();
    }

    onEditingServicio(detail: VentaServicioDetail) {
        this.serviciosEditor.detail = detail;
        this.serviciosEditor.show();
    }

    onDeletingServicio(detail: VentaServicioDetail) {
        this.serviciosList.deleteItem(detail);
    }

    onServiciosUpdated(updatedServices: VentaServicioDetail[]) {
        this.servicios = updatedServices;
    }

    onSavingServicio(detail: VentaServicioDetail) {
        this.serviciosList.updateItem(detail);
    }

    onSavingServicios(details: VentaServicioDetail[]) {
        this.serviciosList.updateItems(details);
    }

    ////////////  METODOS ///////////////////////////////////////
    

    cobroAceptado(formasPagoCobro: CobranzaFormaPagoDetail) {
        let pago = { formasPago: formasPagoCobro }
        this.context.app.showSpinner();
        this.ventaService.nextStep(this.editor.document.identity, this.argsCurrentDocument.step.idWorkflowStep, pago)
            .then(_ => {
                this.context.app.hideSpinner();
                //this.refreshList();
                this.dlgCobranza.clear();
                this.onViewingDocument(this.editor.document.identity);

                if (!this.argsEditorEvent.closeEditor && this.venta.identity == '') {
                    this.router.navigate(['../', this.editor.document.identity], { relativeTo: this.route });
                }
                else {
                    if (this.argsEditorEvent.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }


            })
            .catch(error => this.handleError(error));
    }

    cargarInfomacionEntidades() {
        //Cargar información del almacén origen
        if (this.venta.idAlmacen > 0) {
            this.almacenService.getDetailByID(this.venta.idAlmacen)
                .then(almacen => this.venta.almacen = almacen)
                .catch(error => this.handleError(error));
        }
        else {
            this.venta.almacen.nombre = 'N/A';
        }

        //Cargar información de la unidad de negocio
        this.unidadNegocioService.getDetailByID(this.venta.idUnidadNegocio)
            .then(unidadNegocio => this.venta.unidadNegocio = unidadNegocio)
            .catch(error => this.handleError(error));

        this.divisaService.getDetailByID(this.venta.idDivisa)
            .then(divisa => {
                this.venta.divisa = divisa;
            })
            .catch(error => this.handleError(error));

        this.clienteService.getDetailByID(this.venta.idCliente)
            .then(cliente => {
                this.venta.cliente = cliente;                
                this.venta.condicionesPago = this.venta.cliente.deudor.condicionesPago;
                setTimeout(_ => this.refreshValidity());
            })
            .catch(error => this.handleError(error));
    }

    private refreshValidity() {
        this.editor.valid = this.documentForm.valid;
    }

    ///metodos locales
    importesDesglosados(): ImportesInfo {
        let desglose: ImportesInfo = new ImportesInfo();
        let elementos = this.combineDetail();

        elementos.forEach(item => {
            desglose.subtotal += item.precioUnitario * item.cantidad;
            desglose.descuentos += (item.descuentos.descuento1 + item.descuentos.descuento2 + item.descuentos.descuento3 + item.descuentos.descuento4 + item.descuentos.descuento5) * item.cantidad;
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "T").length > 0)
                desglose.impuestosTrasladados += item.impuestos.filter(tras => tras.tipoAplicacion == "T").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
            if (item.impuestos.filter(tras => tras.tipoAplicacion == "R").length > 0)
                desglose.impuestosRetenidos += item.impuestos.filter(tras => tras.tipoAplicacion == "R").map(imp => imp.importe * item.cantidad).reduce((a, b) => a + b);
        });

        desglose.total = desglose.subtotal - desglose.descuentos + desglose.impuestosTrasladados - desglose.impuestosRetenidos;

        return desglose;
    }    


}

