﻿//Frameworks
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef, forwardRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormGroup } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Azteca Kernel
import { Context, BaseComponent, CatalogService, PolicyInfo  } from 'framework3/core';
import { AztecaValidators, ToolbarButton, AlertsBarComponent } from 'framework3/controls';
import { TrackableCollection, CatalogEditor, EditorEventArgs } from 'framework3/editors';
//Features
import { ClienteService } from '../services/cliente.service'
import { ClienteInfo, PreRegistro } from '../models/cliente'

@Component({
    selector: 'sales-clientes-registrar-editor',
    templateUrl: './clientes-registrar-editor.maintenance.html'
})
export class ClientesRegistrarEditor extends CatalogEditor  {
    editorForm: FormGroup;
    info: PreRegistro = new PreRegistro();
    pRegistro: PreRegistro = new PreRegistro();
    idCliente: number;
    cliente: ClienteInfo = new ClienteInfo();


    //Inputs
    @Input() data: PreRegistro = new PreRegistro();
    //outputs
    @Output() dataUpdated: EventEmitter<PreRegistro> = new EventEmitter<PreRegistro>();


    @ViewChild('dialog') dialog: ModalDirective;

    constructor(private clienteService: ClienteService, protected context: Context, private formBuilder: FormBuilder, protected router: Router, protected route: ActivatedRoute, protected changeDetector: ChangeDetectorRef) {
        super(context, router, route, changeDetector);
        this.editorForm = formBuilder.group({
            'idCliente': [0]
        });
    }


    show(inf: PreRegistro) {
        this.pRegistro = inf;
        this.info = new PreRegistro();
        this.dialog.config.backdrop = false;
        this.dialog.show();
        setTimeout(_ => {
            this.editorForm.reset();
            this.info = this.data;
            this.data = new PreRegistro();
        });
        Control.invalidateAll();
    }


    onSelectedItemChanged(args: any) {
        if (args != null) {
            this.cliente = args;
            this.idCliente = args.identity;
        }
    }

    onConsultarCliente() {

        this.router.navigate(['sales/clientes-ventas', this.process.id, this.idCliente, { readonly: true }])            

    }

    onUpdate(args: PreRegistro) {

        this.clienteService.registrarDePreregistro(this.idCliente, this.pRegistro.password, this.pRegistro.idPreRegistro, this.pRegistro.correo)
            .then(_=>
                {
                    this.dataUpdated.emit(this.pRegistro);
                    this.dialog.hide();
                }
            )
            .catch(error => this.handleError(error))

    }


}