﻿import { Component, Input, Output, EventEmitter, forwardRef, SimpleChange, ViewChild } from '@angular/core';
import { ControlValueAccessor, Validators, NG_VALUE_ACCESSOR, FormGroup, FormBuilder } from '@angular/forms';
//
import { Context, CatalogItem, BaseComponent, PolicyInfo } from 'framework3/core';
import { AztecaValidators } from 'framework3/controls';
import { BaseControl } from 'framework3/controls/models/base-control';
//Features
import { ArticuloInfo } from '../models/articulo';
import { CuentaSelectorComponent } from '../../accounting/components/cuenta-selector.component';
import { EstComSelectorComponent } from '../controls/est-com-selector.component';
import { FlujoSelectorComponent } from '../../accounting/components/flujo-selector.component';
import { MarcasSelectorComponent } from '../controls/marcas-selector.component';

@Component({
    selector: 'articulos-comercial',
    templateUrl: './articulos-comercial.component.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ArticuloComercialComponent)
        },
        {
            provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => ArticuloComercialComponent), multi: true
        }
    ],
    styles: [
        '.ng-valid-component { border-left: 1px solid #ccc!important;}'
    ]
})

export class ArticuloComercialComponent extends BaseControl implements ControlValueAccessor {
    private form: FormGroup;
    //Indica si la información de la forma es válida
    get valid(): boolean {
        return this.form.valid;
    }
    //ViewChilds
    @ViewChild('selectorCuenta') selectorCuenta: CuentaSelectorComponent;
    @ViewChild('selectorEstCom') selectorEstCom: EstComSelectorComponent;
    @ViewChild('selectorCatFlujo') selectorCatFlujo: FlujoSelectorComponent;
    @ViewChild('selectorMarca') selectorMarca: MarcasSelectorComponent;

    //Inputs
    @Input() articulo: ArticuloInfo = new ArticuloInfo();

    //Outputs
    @Output() articuloChange = new EventEmitter<ArticuloInfo>();
    @Output() validChange = new EventEmitter<boolean>();
    //Campos privados para enlazar a UI

    //
    constructor(private formBuilder: FormBuilder) {
        super();

        this.form = formBuilder.group({
            'cuenta': [''],
            'estCom': [''],
            'catFlujo': [''],
            'marcas': [''],
            'modelo': ['', Validators.required],
            'impuestoIncluido': [false]
        });
    }

    ngOnChanges(changes: SimpleChange) {

    }

    ngAfterViewInit() {
        this.form.statusChanges.subscribe(_ => {
            this.validChange.emit(this.form.valid);
        });
    }


    //////////////////////////////// METODOS ////////////////////////////////////////

    notifiyChange() {
        let newArticulo: ArticuloInfo = JSON.parse(JSON.stringify(this.articulo));
        newArticulo.idCuenta = this.articulo.idCuenta;
        newArticulo.idEstCom = this.articulo.idEstCom;
        newArticulo.idFlujo = this.articulo.idFlujo;
        newArticulo.idMarca = this.articulo.idMarca;
        newArticulo.modelo = this.articulo.modelo;
        newArticulo.impuestoIncluido = this.articulo.impuestoIncluido;
        //
        this.articuloChange.emit(newArticulo);
        this.propagateChange(newArticulo);
    }

    //////////////////////////////// HANDLERS ////////////////////////////////////////

    onCuentaChange(args: any) {
        this.notifiyChange();
    }
    onEstComChange(args: any) {
        this.notifiyChange();
    }
    onCatFlujoChange(args: any) {
        this.notifiyChange();
    }
    onMarcasChange(args: any) {
        this.notifiyChange();
    }
    onModeloChange(args: any) {
        this.notifiyChange();
    }
    onImpuestoIncluidoChange(args: any) {
        this.notifiyChange();
    }

    //////////////////////////////// CONTROL VALUE ACCESOR ////////////////////////////////////////

    writeValue(value: any) {

    }

    propagateChange = (_: any) => { };

    registerOnChange(fn: any) {
        this.propagateChange = fn;
    }

    registerOnTouched() {
    }


}