﻿import { Component, ViewChild, Input } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem, CatalogService } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { UsuariosListComponent } from 'framework3/security';
import { PolicyInfo, UsuarioItem } from 'framework3/core';

//
import { PersonaItem, PersonaInfo, PersonaService } from 'framework3/general';
import { ClienteService } from '../services/cliente.service';


@Component({
    selector: 'sales-clientes-contactos',
    templateUrl: './clientes-contactos.component.html',
    providers: [
        {
            provide: CatalogService, useExisting: PersonaService
        }
    ]
})

export class ClientesContactosComponent extends BaseComponent {
    title: string = '';
    idCliente: number;
    proceso: number;
    @Input() personas: PersonaInfo[] = [];       

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(private context: Context, private clienteService: ClienteService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.proceso = +param['proceso'];
            this.idCliente = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadContactos());
    }

    applyPolicy(policy: PolicyInfo) {

    }       

    loadContactos() {
        this.context.app.showSpinner();
        this.clienteService.loadContactos(this.idCliente)
            .then(contactos => {
                this.personas = contactos;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onAddPersona(){
        //this.router.navigate(['contacts'], { relativeTo: this.route });
        this.router.navigate(['sales/clientes/contacts', this.proceso, this.idCliente, 0, 0])
    }

    onEditPersona(item:any) {
        //this.router.navigate(['contacts'], { relativeTo: this.route });
        this.router.navigate(['sales/clientes/contacts', this.proceso, this.idCliente, 0, item.data.identity])
    }

    onRemovePersonas(item: any) {
        this.clienteService.removeContacto(this.idCliente, item.data.identity)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('El contacto han sido agregado con éxito.', 'success');
                this.loadContactos;
            })
            .catch(error => this.handleError(error));
    }

    onCancel() {
        this.location.back();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}