﻿import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ModalDirective } from 'ngx-bootstrap';


import { BaseComponent, Context, CatalogItem, CatalogService, PolicyInfo } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
//import { PolicyInfo, UsuarioItem } from 'framework3/core';

//
import { PersonaItem, PersonaInfo, PersonaService } from 'framework3/general';
import { ClienteService } from '../services/cliente.service';


@Component({
    selector: 'sales-clientes-contactos-editor',
    templateUrl: './clientes-contactos.editor.html',
    providers: [
        {
            provide: CatalogService, useExisting: PersonaService
        }
    ]
})

export class ClientesContactosEditor extends BaseComponent {
    editorForm: FormGroup;
    title: string = '';
    idCliente: number;
    idContacto: number;
    idPersona: number;
    persona: PersonaInfo= new PersonaInfo;

    //@Input() personas: PersonaItem[] = [];

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    //
    @Output() dataUpdated: EventEmitter<number> = new EventEmitter<number>();
    //
    @ViewChild('dialog') dialog: ModalDirective;
    

    constructor(private context: Context, private route: ActivatedRoute, private formBuilder: FormBuilder, private location: Location, protected changeDetector: ChangeDetectorRef, private personaService: PersonaService, private clienteService: ClienteService) {
        super();
        this.editorForm = formBuilder.group({
            'persona': [''],
        });     
    }
   

    ngOnInit() {
        this.route.params.forEach(param => {            
            this.idCliente = +param['id'];
            this.idContacto = +param['idContacto'];
            this.idPersona = +param['idPersona'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadContacto());
    }

    loadContacto() {
        if (this.idPersona == 0)
            this.persona = new PersonaInfo;
        else
            this.loadPersona();
    }

    loadPersona() {
        this.personaService.getDetailByID(this.idPersona)
            .then(persona=> this.persona = persona)
            .catch(error => this.handleError(error));
    }

    onUpdate() {
        this.clienteService.addContacto(this.idCliente, this.persona)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('El contacto han sido agregado con éxito.', 'success');  
                this.location.back();
            })
            .catch(error => this.handleError(error));
    }

    onCancel() {
        this.location.back();
    }

    applyPolicy(policy: PolicyInfo) {

    }    

    onPersonValidChange() {
        this.changeDetector.detectChanges();
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message =  this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }

}