﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup } from '@angular/forms';
//
import { ModalDirective } from 'ngx-bootstrap';
//
import { BaseComponent, Context, CatalogItem, PolicyInfo, UsuarioItem } from 'framework3/core';
import { SearchRequest, SearchCondition, CatalogListComponent } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//
import { DivisaService, DivisaInfo } from '../../accounting';
//
import { GrupoDescuentoService } from '../services/grupo-descuento.service';
import { GrupoDescuentoInfo, GrupoDescuentoItem } from '../models/grupo-descuento';
import { ArticuloGrupoDescuentoItem } from '../models/articulo-grupo-descuento';


@Component({
    selector: 'sales-grupo-descuentos-articulos',
    templateUrl: './grupos-descuentos-articulos.component.html'
})
export class GruposDescuentosArticulosComponent extends BaseComponent {
    form: FormGroup;
    title: string = 'Artículos del grupo de descuentos';
    articulos: ArticuloGrupoDescuentoItem[];
    idGrupoDescuentos: number;
    grupoDescuentos: GrupoDescuentoInfo;
    divisaDefault: DivisaInfo;
    currentItem = new ArticuloGrupoDescuentoItem();
    pageSize: number = 25;
    pageIndex: number = 0;
    totalRows: number = 0;
    

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(CatalogListComponent) catalogList: CatalogListComponent;
    @ViewChild(ModalDirective) editDialog: ModalDirective;
    //
    constructor(private context: Context, private router: Router, private route: ActivatedRoute, private location: Location, private formBuilder: FormBuilder,
        private grupoDescuentosService: GrupoDescuentoService) {
        super();

        this.form = this.formBuilder.group({            
            'descuento1': [0],
            'descuento2': [0],
            'descuento3': [0],
            'descuento4': [0],
            'descuento5': [0]
        });
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idGrupoDescuentos = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadInitialData());
    }

    ///////////////// METODOS ///////////////////////////////

    loadInitialData() {
        this.loadGrupoDescuentos();
        this.triggerSearch()
    }

    loadGrupoDescuentos() {
        this.grupoDescuentosService.getDetailByID(this.idGrupoDescuentos)
            .then(grupo => {
                this.grupoDescuentos = grupo;
                this.title = 'Articulos del grupo de descuentos: ' + this.grupoDescuentos.nombre;
            })
            .catch(error => this.handleError(error));
    }

    triggerSearch() {
        //Disparar búsqueda inicial. Si el manejo se realiza del lado del servidor, obtener sólo la primer página, si es local obtener toda la información disponible
        let request: SearchRequest = new SearchRequest();
        request.pageIndex = 0;
        request.pageSize = this.pageSize;
        request.paged = true;
        //
        this.onSearchRequested(request);
    }

    ///////////////// HANDLERS ///////////////////////////////

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        this.grupoDescuentosService.loadArticulos(this.context.app.defaultSite.identity, this.idGrupoDescuentos, request)
            .then(result => {
                this.articulos = result.rows;
                this.totalRows = result.totalRows;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onCurrentItemChanged(item: ArticuloGrupoDescuentoItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onEditArticulo(articulo: ArticuloGrupoDescuentoItem) {
        this.currentItem = JSON.parse(JSON.stringify(articulo));
        this.editDialog.config.backdrop = false;
        this.editDialog.show();
    }

    onUpdateItem() {
        this.context.app.showSpinner();
        this.grupoDescuentosService.updateArticulo(this.idGrupoDescuentos, this.currentItem)
            .then(_ => {
                let newArticulos = this.articulos.slice();
                let index = newArticulos.findIndex(articulo => articulo.id == this.currentItem.id);
                newArticulos[index] = this.currentItem;
                this.articulos = newArticulos;
                this.editDialog.hide();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}