﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup } from '@angular/forms';
//
import { ModalDirective } from 'ngx-bootstrap';
//
import { BaseComponent, Context, CatalogItem, PolicyInfo, UsuarioItem } from 'framework3/core';
import { SearchRequest, SearchCondition, CatalogListComponent } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//
import { DivisaService, DivisaInfo } from '../../accounting';
//
import { GrupoDescuentoService } from '../services/grupo-descuento.service';
import { GrupoDescuentoInfo, GrupoDescuentoItem } from '../models/grupo-descuento';
import { ServicioGrupoDescuentoItem } from '../models/servicio-grupo-descuento';


@Component({
    selector: 'sales-grupo-descuentos-servicios',
    templateUrl: './grupos-descuentos-servicios.component.html'
})
export class GruposDescuentosServiciosComponent extends BaseComponent {
    form: FormGroup;
    title: string = 'Servicios del grupo de descuentos';
    servicios: ServicioGrupoDescuentoItem[];
    idGrupoDescuentos: number;
    grupoDescuentos: GrupoDescuentoInfo;
    divisaDefault: DivisaInfo;
    currentItem = new ServicioGrupoDescuentoItem();
    pageSize: number = 25;
    pageIndex: number = 0;
    totalRows: number = 0;


    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(CatalogListComponent) catalogList: CatalogListComponent;
    @ViewChild(ModalDirective) editDialog: ModalDirective;
    //
    constructor(private context: Context, private router: Router, private route: ActivatedRoute, private location: Location, private formBuilder: FormBuilder,
        private grupoDescuentosService: GrupoDescuentoService) {
        super();

        this.form = this.formBuilder.group({
            'descuento1': [0],
            'descuento2': [0],
            'descuento3': [0],
            'descuento4': [0],
            'descuento5': [0]
        });
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idGrupoDescuentos = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadInitialData());
    }

    ///////////////// METODOS ///////////////////////////////

    loadInitialData() {
        this.loadGrupoDescuentos();
        this.triggerSearch()
    }

    loadGrupoDescuentos() {
        this.grupoDescuentosService.getDetailByID(this.idGrupoDescuentos)
            .then(grupo => {
                this.grupoDescuentos = grupo;
                this.title = 'Servicios del grupo de descuentos: ' + this.grupoDescuentos.nombre;
            })
            .catch(error => this.handleError(error));
    }

    triggerSearch() {
        //Disparar búsqueda inicial. Si el manejo se realiza del lado del servidor, obtener sólo la primer página, si es local obtener toda la información disponible
        let request: SearchRequest = new SearchRequest();
        request.pageIndex = 0;
        request.pageSize = this.pageSize;
        request.paged = true;
        //
        this.onSearchRequested(request);
    }

    ///////////////// HANDLERS ///////////////////////////////

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        this.grupoDescuentosService.loadServicios(this.context.app.defaultSite.identity, this.idGrupoDescuentos, request)
            .then(result => {
                this.servicios = result.rows;
                this.totalRows = result.totalRows;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onCurrentItemChanged(item: ServicioGrupoDescuentoItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onEditServicio(servicio: ServicioGrupoDescuentoItem) {
        this.currentItem = JSON.parse(JSON.stringify(servicio));
        this.editDialog.config.backdrop = false;
        this.editDialog.show();
    }

    onUpdateItem() {
        this.context.app.showSpinner();
        this.grupoDescuentosService.updateServicio(this.idGrupoDescuentos, this.currentItem)
            .then(_ => {
                let newServicios = this.servicios.slice();
                let index = newServicios.findIndex(servicio => servicio.id == this.currentItem.id);
                newServicios[index] = this.currentItem;
                this.servicios = newServicios;
                this.editDialog.hide();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}