﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
import { FormBuilder, FormGroup } from '@angular/forms';
//
import { ModalDirective } from 'ngx-bootstrap';
//
import { BaseComponent, Context, CatalogItem, PolicyInfo, UsuarioItem } from 'framework3/core';
import { SearchRequest, SearchCondition, CatalogListComponent } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent, AztecaValidators } from 'framework3/controls';
//
import { DivisaService, DivisaInfo } from '../../accounting';
//
import { ListaPreciosService } from '../services/lista-precios.service';
import { ListaPreciosInfo, ListaPreciosItem } from '../models/lista-precios';
import { ArticuloListaPreciosItem } from '../models/articulo-lista-precios';


@Component({
    selector: 'sales-lista-precios-articulos',
    templateUrl: './listas-precios-articulos.component.html'
})
export class ListasPreciosArticulosComponent extends BaseComponent {
    form: FormGroup;
    title: string = 'Artículos de la lista de precios';    
    articulos: ArticuloListaPreciosItem[];
    idListaPrecios: number;
    listaPrecios: ListaPreciosInfo;
    divisaDefault: DivisaInfo;
    currentItem = new ArticuloListaPreciosItem();
    pageSize: number = 25;
    pageIndex: number = 0;
    totalRows: number = 0;
  
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    @ViewChild(CatalogListComponent) catalogList: CatalogListComponent;
    @ViewChild(ModalDirective) editDialog: ModalDirective;
    //
    constructor(private context: Context, private router: Router, private route: ActivatedRoute, private location: Location, private formBuilder: FormBuilder,
        private listaPreciosService: ListaPreciosService, private divisaService: DivisaService) {
        super();

        this.form = this.formBuilder.group({
            'idDivisa': [0, AztecaValidators.selected],
            'precio': [0]
        });
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idListaPrecios = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadInitialData());        
    }

    ///////////////// METODOS ///////////////////////////////

    loadInitialData() {
        this.loadDivisaDefault();
        this.loadListaPrecios();
        this.triggerSearch()
    }

    loadListaPrecios(){
        this.listaPreciosService.getDetailByID(this.idListaPrecios)
            .then(lista => {
                this.listaPrecios = lista;
                this.title = 'Articulos de la lista de precios: ' + this.listaPrecios.nombre;
            })
            .catch(error => this.handleError(error));
    }

    loadDivisaDefault() {
        this.divisaService.getDetailByEmpresa(this.context.app.defaultCompany.identity)
            .then(divisa => this.divisaDefault = divisa)
            .catch(error => this.handleError(error));
    }

    triggerSearch() {
        //Disparar búsqueda inicial. Si el manejo se realiza del lado del servidor, obtener sólo la primer página, si es local obtener toda la información disponible
        let request: SearchRequest = new SearchRequest();
        request.pageIndex = 0;
        request.pageSize = this.pageSize;
        request.paged = true;
        //
        this.onSearchRequested(request);
    }

    ///////////////// HANDLERS ///////////////////////////////

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        this.listaPreciosService.loadArticulos(this.context.app.defaultSite.identity, this.idListaPrecios, request)
            .then(result => {
                this.articulos = result.rows;
                this.totalRows = result.totalRows;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));    
    }

    onCurrentItemChanged(item: ArticuloListaPreciosItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    applyPolicy(policy: PolicyInfo) {

    }

    onDivisaChanged(divisa: DivisaInfo) {
        if (divisa) {
            this.currentItem.idDivisa = divisa.identity;
            this.currentItem.divisa = divisa.nombre;
        }
    }

    onEditArticulo(articulo: ArticuloListaPreciosItem) {
        this.currentItem = JSON.parse(JSON.stringify(articulo));
        //Si no se ha definido aún la divisa, establecer por default la de la empresa
        if (this.currentItem.idDivisa == 0) this.currentItem.idDivisa = this.divisaDefault.identity;
        this.editDialog.config.backdrop = false;
        this.editDialog.show();
    }

    onUpdateItem() {
        this.context.app.showSpinner();
        this.listaPreciosService.updateArticulo(this.idListaPrecios, this.currentItem)
            .then(_ => {
                let newArticulos = this.articulos.slice();
                let index = newArticulos.findIndex(articulo => articulo.id == this.currentItem.id);
                newArticulos[index] = this.currentItem;
                this.articulos = newArticulos;
                this.editDialog.hide();
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}