﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { ClientesListComponent } from '../controls/clientes-list.component';
import { PolicyInfo, UsuarioItem } from 'framework3/core';

//
import { ListaPreciosService } from '../services/lista-precios.service';
import { ListaPreciosInfo} from '../models/lista-precios';

import { ClienteService } from '../services/cliente.service';
import { ClienteInfo, ClienteItem } from '../models/cliente';

@Component({
    selector: 'sales-listas-precios-clientes',
    templateUrl: './listas-precios-clientes.component.html'
})
export class ListaPreciosClientesComponent extends BaseComponent {
    title: string = 'Clientes asignados a la lista de precios';
    clientes: ClienteItem[] = [];
    idListaPrecios: number;
    listaPrecios: ListaPreciosInfo;
    currentItem: ClienteItem = new ClienteItem();
    pageSize: number = 25;
    pageIndex: number = 0;
    totalRows: number = 0;

    //
    @ViewChild(ClientesListComponent) clientesDialog: ClientesListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private listaPreciosService: ListaPreciosService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idListaPrecios = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.loadInitialData());
    }

    ///////////////// METODOS ///////////////////////////////
    loadInitialData() {
        this.loadListaPrecios();
        this.triggerSearch();
    }

    loadListaPrecios() {
        this.listaPreciosService.getDetailByID(this.idListaPrecios)
            .then(info => {
                this.listaPrecios = info;
                this.title = 'Miembros de la lista de precios - ' + this.listaPrecios.nombre;
            })
            .catch(error => this.handleError(error));
    }

    triggerSearch() {
        //Disparar búsqueda inicial. Si el manejo se realiza del lado del servidor, obtener sólo la primer página, si es local obtener toda la información disponible
        let request: SearchRequest = new SearchRequest();
        request.pageIndex = 0;
        request.pageSize = this.pageSize;
        request.paged = true;
        //
        this.onSearchRequested(request);
    }

    ///////////////// HANDLERS ///////////////////////////////

    onCurrentItemChanged(item: ClienteItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.clientesDialog.show();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();
        this.listaPreciosService.loadClientes(this.idListaPrecios, this.context.app.defaultCompany.identity, this.context.session.user.identity, request)
            .then(result => {
                this.clientes = result.rows;
                this.totalRows = result.totalRows;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedChanged(newClientes: ClienteItem[]) {
        this.listaPreciosService.addClientes(this.idListaPrecios, newClientes)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los clientes han sido agregados con éxito.', 'success');                
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}