﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { UsuariosListComponent } from 'framework3/security';
import { PolicyInfo, UsuarioItem } from 'framework3/core';

//
import { UnidadNegocioService } from '../services/unidad-negocio.service';
import { UnidadNegocioInfo, UnidadNegocioItem } from '../models/unidad-negocio';


@Component({
    selector: 'sales-unidades-negocio-members',
    templateUrl: './unidades-negocio-members.component.html'
})
export class UnidadesNegocioMembersComponent extends BaseComponent {
    title: string = '';
    usuarios: UsuarioItem[];
    idUnidadNegocio: number;
    unidadNegocio: UnidadNegocioInfo;
    currentItem: CatalogItem;

    //
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private unidadNegocioService: UnidadNegocioService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idUnidadNegocio = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadUnidadNegocio());

    }

    ///////////////// METODOS ///////////////////////////////

    loadUnidadNegocio() {
        this.unidadNegocioService.getDetailByID(this.idUnidadNegocio)
            .then(info => {
                this.unidadNegocio = info;
                this.title = 'Miembros de la unidad de negocio - ' + this.unidadNegocio.nombre;
            })
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.unidadNegocioService.getMembers(this.idUnidadNegocio)
            .then(usuarios => {
                this.usuarios = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ///////////////// HANDLERS ///////////////////////////////
    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.unidadNegocioService.addMembers(this.idUnidadNegocio, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser() {
        this.context.app.showSpinner();
        this.unidadNegocioService.removeMember(this.idUnidadNegocio, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}