﻿import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
//Modulos Azteca
import { CoreModule } from 'framework3/core';
import { EditorsModule } from 'framework3/editors';
import { SearchModule } from 'framework3/search';
import { GeneralModule } from 'framework3/general';
import { ControlsModule } from 'framework3/controls';
import { SecurityModule } from 'framework3/security';
import { NotificationModule } from 'framework3/notification';

import { SalesRoutingModule } from './sales.routing.module';
//Bootstrap
import { AlertModule, TabsModule, ModalModule } from 'ngx-bootstrap';
//Wijmo
import { WjCoreModule } from 'wijmo/wijmo.angular2.core';
import { WjGridModule } from 'wijmo/wijmo.angular2.grid';
import { WjGridFilterModule } from 'wijmo/wijmo.angular2.grid.filter';
import { WjInputModule } from 'wijmo/wijmo.angular2.input';

//Pixkua
import { AccountingModule } from '../accounting/accounting.module'
import { InventoryModule } from '../inventory/inventory.module'
import { CreditModule } from '../credit/credit.module'
import { RhModule } from '../rh/rh.module'


/////////////////////////////// Servicios ////////////////////////////
import { EmpresaService } from './services/empresa.service';
import { SucursalService } from './services/sucursal.service';
import { RegionService } from './services/region.service';
import { ZonaVentaService} from './services/zona-venta.service';
import { UnidadNegocioService } from './services/unidad-negocio.service';
import { ClienteService } from './services/cliente.service';
import { ListaPreciosService } from './services/lista-precios.service';
import { GrupoDescuentoService } from './services/grupo-descuento.service';
import { EstructuraComercialService } from './services/estructura-comercial.service';
import { ArticuloService } from './services/articulo.service';
import { MarcasService } from './services/marcas.service';
import { VentaService } from './services/venta.service';
import { ServicioService } from './services/servicio.service';
import { VendedorService } from './services/vendedor.service';
import { DevolucionService } from './services/devolucion.service';
import { TerminalService } from './services/terminal.service';

//////////////////////////// Catalogos ////////////////////////////////
import { EmpresasManager } from './catalogs/empresas.manager';
import { EmpresasEditor } from './catalogs/empresas.editor';
import { SucursalesManager } from './catalogs/sucursales.manager';
import { SucursalesEditor } from './catalogs/sucursales.editor';
import { RegionesManager } from './catalogs/regiones.manager';
import { RegionesEditor } from './catalogs/regiones.editor';
import { ZonasVentaManager } from './catalogs/zonas-venta.manager';
import { ZonasVentaEditor } from './catalogs/zonas-venta.editor';
import { UnidadesNegocioManager } from './catalogs/unidades-negocio.manager';
import { UnidadesNegocioEditor } from './catalogs/unidades-negocio.editor';
import { ClientesManager } from './catalogs/clientes.manager';
import { ClientesEditor } from './catalogs/clientes.editor';
import { ListasPreciosManager } from './catalogs/listas-precios.manager';
import { ListasPreciosEditor } from './catalogs/listas-precios.editor';
import { GruposDescuentosManager } from './catalogs/grupos-descuentos.manager';
import { GruposDescuentosEditor } from './catalogs/grupos-descuentos.editor';
import { EstComManager } from './catalogs/est-com.manager';
import { ArticulosManager } from './catalogs/articulos.manager';
import { ArticulosEditor } from './catalogs/articulos.editor';
import { MarcasManager } from './catalogs/marcas.manager';
import { MarcasEditor } from './catalogs/marcas.editor';
import { ServiciosManager } from './catalogs/servicios.manager';
import { ServiciosEditor } from './catalogs/servicios.editor';
import { VendedoresManager } from './catalogs/vendedores.manager';
import { VendedoresEditor } from './catalogs/vendedores.editor';
import { TerminalesManager } from './catalogs/terminales.manager';
import { TerminalesEditor } from './catalogs/terminales.editor';

//////////////////////////// Controls /////////////////////////////
import { ClientesListComponent } from './controls/clientes-list.component';
import { EstComSelectorComponent } from './controls/est-com-selector.component';
import { MarcasSelectorComponent } from './controls/marcas-selector.component';
import { VendedoresSelectorComponent } from './controls/vendedores-selector.component';
import { ClientesSelectorComponent } from './controls/clientes-selector.component';
import { ArticulosSelectorComponent } from './controls/articulos-selector.component';
import { ServiciosSelectorComponent } from './controls/servicios-selector.component';
import { UnidadesNegocioSelectorComponent } from './controls/unidades-negocio-selector.component';

//////////////////////////// Componentes /////////////////////////////
import { ArticuloNuevoComponent } from './components/nuevo-articulo.component';
import { ArticulosImpuestosComponent } from './components/articulo-impuestos.component';
import { ArticuloListaPreciosComponent } from './components/articulo-lista-precios.component';
import { ArticuloListaPreciosEditorComponent } from './components/articulo-lista-precios-editor.component';
import { ServicioListaPreciosComponent } from './components/servicio-lista-precios.component';
import { ServicioListaPreciosEditorComponent } from './components/servicio-lista-precios-editor.component';
import { ArticuloGrupoDescuentoComponent } from './components/articulo-grupo-descuento.component';
import { ArticuloGrupoDescuentoEditorComponent } from './components/articulo-grupo-descuento-editor.component';
import { ServicioGrupoDescuentoComponent } from './components/servicio-grupo-descuento.component';
import { ServicioGrupoDescuentoEditorComponent } from './components/servicio-grupo-descuento-editor.component';
import { VentasSelectorComponent } from './controls/ventas-selector.component';

//////////////////////////// Mantenimientos ////////////////////////////////
import { ClientesRegistrar } from './maintenances/clientes-registrar.maintenance';
import { ClientesRegistrarEditor } from './maintenances/clientes-registrar-editor.maintenance';
import { Confirmacion } from './maintenances/parts/confirmacion.component';

///////////////////////////// Document /////////////////////////////////
import { VentasManager } from './documents/ventas.manager';
import { VentasEditor } from './documents/ventas.editor';
import { DevolucionesManager } from './documents/devoluciones.manager';
import { DevolucionesEditor } from './documents/devoluciones.editor';

//////////////////////////// Parts /////////////////////////////////////
import { UnidadesNegocioMembersComponent } from './parts/unidades-negocio-members.component';
import { ClientesContactosComponent } from './parts/clientes-contactos.component';
import { ClientesContactosEditor } from './parts/clientes-contactos.editor';
import { ListaPreciosClientesComponent } from './parts/listas-precios-clientes.component';
import { ListasPreciosArticulosComponent } from './parts/listas-precios-articulos.component';
import { ListasPreciosServiciosComponent } from './parts/listas-precios-servicios.component';
import { GruposDescuentosArticulosComponent } from './parts/grupos-descuentos-articulos.component';
import { GruposDescuentosServiciosComponent } from './parts/grupos-descuentos-servicios.component';
import { VentaArticulosDetailEditor } from './documents/parts/venta-articulos-detail.editor';
import { VentaServiciosDetailEditor } from './documents/parts/venta-servicios-detail.editor';
import { NuevaVentaComponent } from './documents/parts/nueva-venta.component';
import { DevolucionDetailEditor } from './documents/parts/devoluciones-detail.editor';
import { ArticuloComercialComponent } from './parts/articulos-comercial.component';


/////////////////////////////////////////////////////////////////////////////////
let COMPONENTES = [EmpresasManager, EmpresasEditor, SucursalesManager, SucursalesEditor, RegionesManager, RegionesEditor, ZonasVentaManager, ZonasVentaEditor, UnidadesNegocioManager,
    UnidadesNegocioEditor, UnidadesNegocioMembersComponent, ClientesManager, ClientesEditor, ClientesContactosComponent, ClientesContactosEditor, ClientesListComponent, EstComSelectorComponent,
    ListasPreciosManager, ListasPreciosEditor, ListaPreciosClientesComponent, GruposDescuentosManager, GruposDescuentosEditor, EstComManager, ArticulosManager, ArticulosEditor,
    MarcasManager, MarcasEditor, MarcasSelectorComponent, ArticuloNuevoComponent, ArticulosImpuestosComponent, ServiciosManager, ServiciosEditor, ArticulosSelectorComponent,
    VendedoresManager, VendedoresEditor, VendedoresSelectorComponent, ClientesSelectorComponent, VentasManager, VentasEditor, VentaArticulosDetailEditor, VentaServiciosDetailEditor,
    UnidadesNegocioSelectorComponent, NuevaVentaComponent, ServiciosSelectorComponent, ListasPreciosArticulosComponent, ListasPreciosServiciosComponent, ArticuloListaPreciosComponent,
    ArticuloListaPreciosEditorComponent, ServicioListaPreciosComponent, ServicioListaPreciosEditorComponent, ArticuloGrupoDescuentoComponent, ArticuloGrupoDescuentoEditorComponent,
    ServicioGrupoDescuentoComponent, ServicioGrupoDescuentoEditorComponent, GruposDescuentosArticulosComponent, GruposDescuentosServiciosComponent,
    DevolucionesEditor, DevolucionesManager, DevolucionDetailEditor, ArticuloComercialComponent, VentasSelectorComponent, TerminalesManager, TerminalesEditor, ClientesRegistrar, ClientesRegistrarEditor, Confirmacion];

let SERVICIOS = [EmpresaService, SucursalService, RegionService, ZonaVentaService, UnidadNegocioService, ClienteService, ListaPreciosService, GrupoDescuentoService, EstructuraComercialService,
    ArticuloService, MarcasService, VentaService, ServicioService, VendedorService, DevolucionService, TerminalService];

@NgModule({
    imports: [
        CommonModule, FormsModule, ReactiveFormsModule,
        AlertModule.forRoot(), TabsModule.forRoot(), ModalModule.forRoot(),
        CoreModule, EditorsModule, SearchModule, GeneralModule, ControlsModule, SecurityModule, NotificationModule,
        AccountingModule, InventoryModule, CreditModule, RhModule,
        WjGridModule, WjGridFilterModule, WjCoreModule, WjInputModule, SalesRoutingModule
    ],
    declarations: COMPONENTES,
    providers: SERVICIOS,
    exports: COMPONENTES
})
export class SalesModule {

}