﻿import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

//////////////////////////// Guards //////////////////////////////////
import { AuthGuard } from 'framework3/core';
import { EditorDeactivateGuard } from 'framework3/editors';

//////////////////////////// Catalogos ////////////////////////////////
import { EmpresasManager } from './catalogs/empresas.manager';
import { EmpresasEditor } from './catalogs/empresas.editor';
import { SucursalesManager } from './catalogs/sucursales.manager';
import { SucursalesEditor } from './catalogs/sucursales.editor';
import { UnidadesNegocioManager } from './catalogs/unidades-negocio.manager';
import { UnidadesNegocioEditor } from './catalogs/unidades-negocio.editor';
import { UnidadesNegocioMembersComponent } from './parts/unidades-negocio-members.component';
import { RegionesManager } from './catalogs/regiones.manager';
import { RegionesEditor } from './catalogs/regiones.editor';
import { ZonasVentaManager } from './catalogs/zonas-venta.manager';
import { ZonasVentaEditor } from './catalogs/zonas-venta.editor';
import { ClientesManager } from './catalogs/clientes.manager';
import { ClientesEditor } from './catalogs/clientes.editor';
import { ListasPreciosManager } from './catalogs/listas-precios.manager';
import { ListasPreciosEditor } from './catalogs/listas-precios.editor';

import { GruposDescuentosManager } from './catalogs/grupos-descuentos.manager';
import { GruposDescuentosEditor } from './catalogs/grupos-descuentos.editor';
import { EstComManager } from './catalogs/est-com.manager';

import { ArticulosManager } from './catalogs/articulos.manager';
import { ArticulosEditor } from './catalogs/articulos.editor';

import { MarcasManager } from './catalogs/marcas.manager';
import { MarcasEditor } from './catalogs/marcas.editor';

import { ServiciosManager } from './catalogs/servicios.manager';
import { ServiciosEditor } from './catalogs/servicios.editor';

import { VendedoresManager } from './catalogs/vendedores.manager';
import { VendedoresEditor } from './catalogs/vendedores.editor';
import { TerminalesManager } from './catalogs/terminales.manager';
import { TerminalesEditor } from './catalogs/terminales.editor';


///////////////////////////// Document /////////////////////////////////
import { VentasManager } from './documents/ventas.manager';
import { VentasEditor } from './documents/ventas.editor';

import { DevolucionesManager } from './documents/devoluciones.manager';
import { DevolucionesEditor } from './documents/devoluciones.editor';
import { DevolucionDetailEditor } from './documents/parts/devoluciones-detail.editor';

//////////////////////////// Mantenimientos ////////////////////////////////
import { ClientesRegistrar } from './maintenances/clientes-registrar.maintenance';
import { ClientesRegistrarEditor } from './maintenances/clientes-registrar-editor.maintenance';

///////////////////////////// Parts /////////////////////////////////
import { ClientesContactosComponent } from './parts/clientes-contactos.component';
import { ClientesContactosEditor } from './parts/clientes-contactos.editor';
import { ListaPreciosClientesComponent } from './parts/listas-precios-clientes.component';
import { ListasPreciosArticulosComponent } from './parts/listas-precios-articulos.component';
import { ListasPreciosServiciosComponent } from './parts/listas-precios-servicios.component';
import { GruposDescuentosArticulosComponent } from './parts/grupos-descuentos-articulos.component';
import { GruposDescuentosServiciosComponent } from './parts/grupos-descuentos-servicios.component';
import { NuevaVentaComponent } from './documents/parts/nueva-venta.component';
import { ArticuloListaPreciosComponent } from './components/articulo-lista-precios.component';
import { ServicioListaPreciosComponent } from './components/servicio-lista-precios.component';
import { ArticuloGrupoDescuentoComponent } from './components/articulo-grupo-descuento.component';
import { ServicioGrupoDescuentoComponent } from './components/servicio-grupo-descuento.component';

///////////////////////////// Rutas /////////////////////////////////

const routes: Routes = [
    {
        path: '',
        canActivateChild: [AuthGuard],
        children: [
            { path: 'empresas-ventas/:proceso', component: EmpresasManager },
            { path: 'empresas-ventas/:proceso/:id', component: EmpresasEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'sucursales-ventas/:proceso', component: SucursalesManager },
            { path: 'sucursales-ventas/:proceso/:idEmpresa/:id', component: SucursalesEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'unidades-negocio/:idSucursal/:proceso/:id', component: UnidadesNegocioEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'unidades-negocio/:idSucursal/:proceso/:id/members', component: UnidadesNegocioMembersComponent },

            { path: 'regiones/:proceso', component: RegionesManager },
            { path: 'regiones/:proceso/:id', component: RegionesEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'zonas-venta/:proceso', component: ZonasVentaManager },
            { path: 'zonas-venta/:proceso/:id', component: ZonasVentaEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'clientes-ventas/:proceso', component: ClientesManager },
            { path: 'clientes-ventas/:proceso/:id', component: ClientesEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'clientes/contacts/:proceso/:id', component: ClientesContactosComponent },
            { path: 'clientes/contacts/:proceso/:id/:idContacto/:idPersona', component: ClientesContactosEditor },

            { path: 'listas-precio/:proceso', component: ListasPreciosManager },
            { path: 'listas-precio/:proceso/:id', component: ListasPreciosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'listas-precio/:proceso/:id/clientes', component: ListaPreciosClientesComponent },
            { path: 'listas-precio/:proceso/:id/articulos', component: ListasPreciosArticulosComponent },
            { path: 'listas-precio/:proceso/:id/servicios', component: ListasPreciosServiciosComponent },

            { path: 'grupos-descuento/:proceso', component: GruposDescuentosManager },
            { path: 'grupos-descuento/:proceso/:id', component: GruposDescuentosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'grupos-descuento/:proceso/:id/clientes', component: GruposDescuentosArticulosComponent },
            { path: 'grupos-descuento/:proceso/:id/articulos', component: GruposDescuentosArticulosComponent },
            { path: 'grupos-descuento/:proceso/:id/servicios', component: GruposDescuentosServiciosComponent },

            { path: 'estructura-comercial/:proceso', component: EstComManager },

            { path: 'articulos-ventas/:proceso', component: ArticulosManager },
            { path: 'articulos-ventas/:proceso/:id', component: ArticulosEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'marcas/:proceso', component: MarcasManager },
            { path: 'marcas/:proceso/:id', component: MarcasEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'terminales/:proceso', component: TerminalesManager },
            { path: 'terminales/:proceso/:id', component: TerminalesEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'servicios/:proceso', component: ServiciosManager },
            { path: 'servicios/:proceso/:id', component: ServiciosEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'vendedores/:proceso', component: VendedoresManager },
            { path: 'vendedores/:proceso/:id', component: VendedoresEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'ventas/:proceso', component: VentasManager },
            { path: 'ventas/:proceso/new', component: NuevaVentaComponent },
            { path: 'ventas/:proceso/:id', component: VentasEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'articulo-listas-precio/:proceso/:idArticulo', component: ArticuloListaPreciosComponent },
            { path: 'servicio-listas-precio/:proceso/:idServicio', component: ServicioListaPreciosComponent },

            { path: 'articulo-grupo-descuento/:proceso/:idArticulo', component: ArticuloGrupoDescuentoComponent },
            { path: 'servicio-grupo-descuento/:proceso/:idServicio', component: ServicioGrupoDescuentoComponent },

            { path: 'devoluciones/:proceso', component: DevolucionesManager },
            { path: 'devoluciones/:proceso/:id', component: DevolucionesEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'clientes-registrar/:proceso', component: ClientesRegistrar },


        ]
    },
];



@NgModule({
    imports: [
        RouterModule.forChild(routes)
    ],
    exports: [
        RouterModule
    ]
})
export class SalesRoutingModule { }

