﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Wijmo
import { CollectionView } from 'wijmo/wijmo'
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { ArticuloInfo } from '../models/articulo'
import { ArticuloStockItem } from '../../inventory/models/articulo-stock'
import { ImpuestoAsignacionInfo } from '../../accounting/models/impuesto-asignacion';
import { ArticuloListaPreciosItem } from '../models/articulo-lista-precios';
import { ArticuloGrupoDescuentoItem } from '../models/articulo-grupo-descuento';
import { PrecioInfo } from '../models/precio';

@Injectable()
export class ArticuloService extends CatalogService<ArticuloInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/ventas/articulos');
    }

    create(): Promise<ArticuloInfo> {
        return this.apiService.get<ArticuloInfo>('create', null)
            .toPromise();
    }

    GetArticuloStock(idSucursal: number, codigo: string): Promise<ArticuloStockItem> {
        let params = new HttpParams()
            .set("idSucursal", idSucursal.toString())
            .set("codigo", codigo);
        return this.apiService.get<ArticuloStockItem>("get-articulo-stock", params)
            .toPromise();
    }

    articuloAsignadoCheck(idArticuloStock: number): Promise<boolean> {
        let params = new HttpParams()
            .set("idArticuloStock", idArticuloStock.toString());
        return this.apiService.get<boolean>("articulo-asignado-check", params)
            .toPromise();
    }

    getArticuloByArticuloStock(idArticuloStock: number): Promise<number> {
        let params = new HttpParams()
            .set("idArticuloStock", idArticuloStock.toString());
        return this.apiService.get<number>("get-articulo-by-articulo-stock", params)
            .toPromise();
    }

    //Lista de Precios

    loadArticuloListaPrecios(idEmpresa: number, idArticulo: number): Promise<ArticuloListaPreciosItem[]> {
        let params = new HttpParams()
            .set("idEmpresa", idEmpresa.toString())
            .set("idArticulo", idArticulo.toString());
        return this.apiService.get<ArticuloListaPreciosItem[]>("load-articulo-lista-precios", params)
            .toPromise();
    }

    updateArticuloListaPrecios(data: ArticuloListaPreciosItem[], idArticulo: number): Promise<any> {
        let params = new HttpParams()
            .set("idArticulo", idArticulo.toString());
        return this.apiService.post<any>('update-articulo-lista-precios', params, data)
            .toPromise();
    }

    //Grupos de Descuentos

    loadArticuloGrupoDescuento(idEmpresa: number, idArticulo: number): Promise<ArticuloGrupoDescuentoItem[]> {
        let params = new HttpParams()
            .set("idEmpresa", idEmpresa.toString())
            .set("idArticulo", idArticulo.toString());
        return this.apiService.get<ArticuloGrupoDescuentoItem[]>("load-articulo-grupo-descuento", params)
            .toPromise();
    }

    updateArticuloGrupoDescuento(data: ArticuloGrupoDescuentoItem[], idArticulo: number): Promise<any> {
        let params = new HttpParams()
            .set("idArticulo", idArticulo.toString());
        return this.apiService.post<any>('update-articulo-grupo-descuento', params, data)
            .toPromise();
    }

    calcularPrecio(idArticulo: number, idListaPrecios: number, idGrupoDescuentos: number, idEmpresa: number,idZonaVenta:number): Promise<PrecioInfo> {
        let params = new HttpParams()
            .set("idArticulo", idArticulo.toString())
            .set("idListaPrecios", idListaPrecios.toString())
            .set("idGrupoDescuentos", idGrupoDescuentos.toString())
            .set("idEmpresa", idEmpresa.toString())
            .set("idZonaVenta",idZonaVenta.toString());
        return this.apiService.get<PrecioInfo>('precio', params)
            .toPromise();
    }

    calcularPrecioCliente(idArticulo: number, idCliente: number, idEmpresa: number, idZonaVenta: number): Promise<PrecioInfo> {
        let params = new HttpParams()
            .set("idArticulo", idArticulo.toString())
            .set("idCliente", idCliente.toString())
            .set("idEmpresa", idEmpresa.toString())
            .set("idZonaVenta", idZonaVenta.toString());
        return this.apiService.get<PrecioInfo>('precio-cliente', params)
            .toPromise();
    }


}