﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService, EmpresaItem } from 'framework3/core';
//Locales
import { ClienteInfo, PreRegistro } from '../models/cliente'
import { PersonaItem, PersonaInfo } from 'framework3/general';


@Injectable()
export class ClienteService extends CatalogService<ClienteInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/ventas/clientes");
    }

    create(): Promise<ClienteInfo> {
        return this.apiService.get<ClienteInfo>('create', null)
            .toPromise();
    }

    loadContactos(idCliente: number): Promise<PersonaInfo[]> {
        let params = new HttpParams()
            .append("idCliente", idCliente.toString());
        return this.apiService.get<PersonaInfo[]>("load-contactos", params)
            .toPromise();
    }
    
    addContacto(idCliente: number, contacto: PersonaInfo): Promise<any> {
        let params = new HttpParams()
            .set('idCliente', idCliente.toString());
        return this.apiService.post<any>('add-contacto', params, contacto)
            .toPromise();
    }

    removeContacto(idCliente: number, idPersona: number): Promise<any> {
        let params = new HttpParams()
            .set('idCliente', idCliente.toString())
            .set('idPersona', idPersona.toString());
        return this.apiService.delete<any>('remove-contacto', params)
            .toPromise();
    }

    enviarRegistro(idCliente: number): Promise<any> {
        let params = new HttpParams()
            .set('idCliente', idCliente.toString());
        return this.apiService.get<any>('enviar-registro', params)
            .toPromise();
    }

    registrar(idCliente: number, password: string ): Promise<any> {
        let params = new HttpParams()
            .set('idCliente', idCliente.toString())
            .set('password', password);
        return this.apiService.get<any>('registrar', params)
            .toPromise();
    }



    getDetailByUser(idUsuario: number, idEmpresa: number): Promise<ClienteInfo> {
        let params = new HttpParams()
            .set('idUsuario', idUsuario.toString())
            .set('idEmpresa', idEmpresa.toString());
        return this.apiService.get<ClienteInfo>('detail-by-user', params)
            .toPromise();
    }

    loadEmpresas(idUsuario: number): Promise<EmpresaItem[]> {
        let params = new HttpParams()
            .set('idUsuario', idUsuario.toString());
        return this.apiService.get<EmpresaItem[]>('empresas', params)
            .toPromise();
    }

    //Pre registro

    registrarDePreregistro(idCliente: number, password: string, idPreRegistro: number, correo: string): Promise<any> {
        let params = new HttpParams()
            .set('idCliente', idCliente.toString())
            .set('password', password)
            .set('idPreRegistro', idPreRegistro.toString())
            .set('correo', correo);
        return this.apiService.get<any>('registrar-de-preregistro', params)
            .toPromise();
    }

    preRegistroEliminar(idPreRegistro: number): Promise<any> {
        let params = new HttpParams()
            .set('idPreRegistro', idPreRegistro.toString());
        return this.apiService.post<any>('pre-registro-delete', params)
            .toPromise();

    }

    preRegistrar(preReg: PreRegistro): Promise<any> {
        let params = new HttpParams()
        return this.apiService.post<any>("pre-registrar", params, preReg)
            .toPromise();
    }

    preRegistros(): Promise<PreRegistro[]> {
        let params = new HttpParams()
        return this.apiService.get<PreRegistro[]>('pre-registros', params)
            .toPromise();
    }

}

