﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService } from 'framework3/core';
//Inventario
import { Empaquetado } from '../../inventory';
//Locales
import { DevolucionInfo, DevolucionItem } from '../models/devolucion';
import { DevolucionDetail } from '../models/devolucion-detail';
import { DevolucionArticuloDetail } from '../models/devolucion-articulo-detail';

@Injectable()
export class DevolucionService extends DocumentService<DevolucionDetail, DevolucionInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/ventas/devoluciones');
    }

    create(): Promise<DevolucionInfo> {
        let newDoc = new DevolucionInfo();
        return Promise.resolve(newDoc);
    }

    update(info: DevolucionInfo): Promise<DevolucionInfo> {
        //Si no tiene asignado la empresa y sucursal, asignar la actual
        if (info.idSucursal == 0) info.idSucursal = this.context.app.defaultSite.identity;
        //Actualizar campo de cantidad basado en las propiedades auxiliares (solo para articulos)
        info.details.filter(detail => detail.idArticulo > 0).map(detail => <DevolucionArticuloDetail>detail).forEach(detail => {
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(Math.trunc(detail.cantidad), detail.unidadMedida.factor, Math.trunc(detail.unidades));
                detail.cantidad = cantidad.toDecimal();
                let cantidadRef = new Empaquetado(detail.cantidadRef, detail.unidadMedida.factor, detail.unidadesRef);
                detail.cantidadRef = cantidadRef.toDecimal();
            }
        });
        //
        let params = new HttpParams();
        return this.apiService.post<DevolucionInfo>('save', params, info)
            .toPromise();
    }

    protected mapDocument(document: DevolucionInfo) {
        document.details.filter(detail => detail.idArticulo > 0).map(detail => <DevolucionArticuloDetail>detail).forEach(detail => {
            detail.fechaCaducidad = moment(detail.fechaCaducidad).toDate();
            //Si la unidad de medida es un paquete, separar unidades
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor);
                detail.cantidad = Math.round(cantidad.paquetes);
                detail.unidades = Math.round(cantidad.unidades);
            }
            else {
                detail.unidades = 0;
            }
        });
    }

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: DevolucionInfo) {

    }
	*/



}