﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { GrupoDescuentoInfo, GrupoDescuentoItem } from '../models/grupo-descuento'
import { ArticuloGrupoDescuentoItem } from '../models/articulo-grupo-descuento';
import { ServicioGrupoDescuentoItem } from '../models/servicio-grupo-descuento';
import { ClienteItem } from '../models/cliente'


import { SearchRequest, SearchResult } from 'framework3/search';

@Injectable()
export class GrupoDescuentoService extends CatalogService<GrupoDescuentoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/ventas/gruposdescuentos");
    }

    create(): Promise<GrupoDescuentoInfo> {
        return Promise.resolve(new GrupoDescuentoInfo());
    }

    loadList(idSucursal: number): Promise<GrupoDescuentoItem[]> {
        let params = new HttpParams()
            .set("idSucursal", idSucursal.toString());
        return this.apiService.get<GrupoDescuentoItem[]>("load-list", params)
            .toPromise();
    }

    loadClientes(idGrupoDescuentos: number, idEmpresa: number, idUsuario: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idGrupoDescuentos', idGrupoDescuentos.toString())
            .set('idEmpresa', idEmpresa.toString())
            .set('idUsuario', idUsuario.toString());
        //
        return this.apiService.post<SearchResult>('clientes', params, request)
            .toPromise();
    }

    addClientes(idGrupoDescuentos: number, clientes: ClienteItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idGrupoDescuentos', idGrupoDescuentos.toString());
        return this.apiService.post<any>('add-clientes', params, clientes)
            .toPromise();
    }

    loadArticulos(idSucursal: number, idGrupoDescuentos: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idGrupoDescuentos', idGrupoDescuentos.toString())
            .set('idSucursal', idSucursal.toString());
        //
        return this.apiService.post<SearchResult>('articulos', params, request)
            .toPromise();
    }

    updateArticulo(idGrupoDescuentos: number, articulo: ArticuloGrupoDescuentoItem): Promise<any> {
        let params = new HttpParams()
            .set('idGrupoDescuentos', idGrupoDescuentos.toString());
        return this.apiService.post<any>('update-articulo', params, articulo)
            .toPromise();
    }

    loadServicios(idSucursal: number, idGrupoDescuentos: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idGrupoDescuentos', idGrupoDescuentos.toString())
            .set('idSucursal', idSucursal.toString());
        //
        return this.apiService.post<SearchResult>('servicios', params, request)
            .toPromise();
    }

    updateServicio(idGrupoDescuentos: number, servicio: ServicioGrupoDescuentoItem): Promise<any> {
        let params = new HttpParams()
            .set('idGrupoDescuentos', idGrupoDescuentos.toString());
        return this.apiService.post<any>('update-servicio', params, servicio)
            .toPromise();
    }


}

