﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
import { SearchRequest, SearchResult} from 'framework3/search';
//Locales
import { ListaPreciosInfo, ListaPreciosItem } from '../models/lista-precios'
import { ClienteItem } from '../models/cliente'
import { ArticuloListaPreciosItem } from '../models/articulo-lista-precios';
import { ServicioListaPreciosItem } from '../models/servicio-lista-precios';


@Injectable()
export class ListaPreciosService extends CatalogService<ListaPreciosInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, "api/ventas/listas-precios");
    }

    create(): Promise<ListaPreciosInfo> {
        return Promise.resolve(new ListaPreciosInfo());
    }

    loadList(idSucursal: number): Promise<ListaPreciosItem[]> {
        let params = new HttpParams()
            .set("idSucursal", idSucursal.toString());
        return this.apiService.get<ListaPreciosItem[]>("load-list", params)
            .toPromise();
    }

    loadClientes(idListaPrecios: number, idEmpresa: number, idUsuario: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idListaPrecios', idListaPrecios.toString())
            .set('idEmpresa', idEmpresa.toString())
            .set('idUsuario', idUsuario.toString());
        //
        return this.apiService.post<SearchResult>('clientes', params, request)
            .toPromise();
    }

    addClientes(idListaPrecios: number, clientes: ClienteItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idListaPrecios', idListaPrecios.toString());
        return this.apiService.post<any>('add-clientes', params, clientes)
            .toPromise();
    }

    loadArticulos(idSucursal: number, idListaPrecios: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idListaPrecios', idListaPrecios.toString())
            .set('idSucursal', idSucursal.toString());
        //
        return this.apiService.post<SearchResult>('articulos', params, request)
            .toPromise();
    }

    updateArticulo(idListaPrecios: number, articulo: ArticuloListaPreciosItem): Promise<any> {
        let params = new HttpParams()
            .set('idListaPrecios', idListaPrecios.toString());
        return this.apiService.post<any>('update-articulo', params, articulo)
            .toPromise();
    }

    loadServicios(idSucursal: number, idListaPrecios: number, request: SearchRequest): Promise<SearchResult> {
        let params = new HttpParams()
            .set('idListaPrecios', idListaPrecios.toString())
            .set('idSucursal', idSucursal.toString());
        //
        return this.apiService.post<SearchResult>('servicios', params, request)
            .toPromise();
    }

    updateServicio(idListaPrecios: number, servicio: ServicioListaPreciosItem): Promise<any> {
        let params = new HttpParams()
            .set('idListaPrecios', idListaPrecios.toString());
        return this.apiService.post<SearchResult>('update-servicio', params, servicio)
            .toPromise();
    }

}

