﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { UnidadNegocioInfo, UnidadNegocioItem } from '../models/unidad-negocio'
import { UsuarioItem } from 'framework3/core';

@Injectable()
export class UnidadNegocioService extends CatalogService<UnidadNegocioInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/ventas/unidades-negocio');
    }

    create(): Promise<UnidadNegocioInfo> {
        return Promise.resolve(new UnidadNegocioInfo());
    }

    loadListByUser(idSucursal: number, idUsuario: number): Promise<UnidadNegocioItem[]> {
        let params = new HttpParams()
            .set('idSucursal', idSucursal.toString())
            .set('idUsuario', idUsuario.toString());
        return this.apiService.get<UnidadNegocioItem[]>('user-access', params)
            .toPromise();
    }

    getMembers(idUnidadNegocio: number): Promise<UsuarioItem[]> {
        let params = new HttpParams()
            .set('idUnidadNegocio', idUnidadNegocio.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)
            .toPromise();
    }

    addMembers(idUnidadNegocio: number, users: UsuarioItem[]): Promise<any> {
        let params = new HttpParams()
            .set('idUnidadNegocio', idUnidadNegocio.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }

    removeMember(idUnidadNegocio: number, idUser: number): Promise<any> {
        let params = new HttpParams()
            .set('idUnidadNegocio', idUnidadNegocio.toString())
            .set('idUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

    loadList(idSucursal: number): Promise<UnidadNegocioItem[]> {
        let params = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<UnidadNegocioItem[]>('load-list', params)
            .toPromise();
    }
}

