﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
import * as moment from 'moment';
import { map } from 'rxjs/operators';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';
//Inventario
import { Empaquetado } from '../../inventory';
//Locales
import { VentaInfo, VentaItem } from '../models/venta';
import { VentaDetail } from '../models/venta-detail';
import { VentaArticuloDetail } from '../models/venta-articulo-detail';


@Injectable()
export class VentaService extends DocumentService<VentaDetail, VentaInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/ventas/ventas');
    }

    create(): Promise<VentaInfo> {
        let newDoc = new VentaInfo();
        return Promise.resolve(newDoc);
    }

    update(info: VentaInfo): Promise<VentaInfo> {
        //Si no tiene asignado la empresa y sucursal, asignar la actual
        if (info.idSucursal == 0) info.idSucursal = this.context.app.defaultSite.identity;
        //Actualizar campo de cantidad basado en las propiedades auxiliares (solo para articulos)
        info.details.filter(detail => detail.idArticulo > 0).map(detail => <VentaArticuloDetail>detail).forEach(detail => {
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor, detail.unidades);
                detail.cantidad = cantidad.toDecimal();
            }
        });
        //
        let params = new HttpParams();
        return this.apiService.post<VentaInfo>('save', params, info)
            .toPromise();
    }

    searchKeyword(idBranch: number, keyword: string): Observable<VentaItem[]> {
        let params = new HttpParams()
            .set("idBranch", idBranch.toString())
            .set("keyword", keyword);
        //
        return this.apiService.get<VentaItem[]>("search-ventas", params);
    }

    protected XXXXmapDocument(document: VentaInfo) {
        document.details.filter(detail => detail.idArticulo > 0).map(detail => <VentaArticuloDetail>detail).forEach(detail => {
            detail.fechaCaducidad = moment(detail.fechaCaducidad).toDate();
            //Si la unidad de medida es un paquete, separar unidades
            if (detail.unidadMedida.paquete && detail.unidadMedida.factor > 1) {
                let cantidad = new Empaquetado(detail.cantidad, detail.unidadMedida.factor);
                detail.cantidad = cantidad.paquetes;
                detail.unidades = cantidad.unidades;
            }
            else {
                detail.unidades = 0;
            }
        });
    }

    generarPdfFactura(idVenta: string): Promise<FileLink> {
        let params = new HttpParams()
            .set("idVenta", idVenta);
        return this.apiService.get<FileLink>("pdf-factura", params)
            .toPromise();
    }

    getXmlFactura(idVenta: string): Promise<FileLink> {
        let params = new HttpParams()
            .set("idVenta", idVenta);
        return this.apiService.get<FileLink>("xml-factura", params)
            .toPromise();
    }

    loadEstadisticaPeriodo(idSucursal: number, fechaInicial: Date, fechaFinal: Date): Promise<any[]> {
        let fi = moment(fechaInicial).format();
        let ff = moment(fechaFinal).format();
        let params = new HttpParams()
            .set('idSucursal', idSucursal.toString())
            .set('fechaInicial', fi)
            .set('fechaFinal', ff);
        return this.apiService.get<any[]>('estadistica-periodo', params)
            .pipe(map(result => {
                result.forEach(row => {
                    row.fechaAfectacion = moment(row.fechaAfectacion).toDate();
                });
                return result;
            }))
            .toPromise();
    }

    imprimirFormatoVenta(idVenta: string): Promise<any> {
        let params = new HttpParams()
            .set("idVenta", idVenta);
        return this.apiService.get<FileLink>("formato-venta", params)
            .toPromise();
    }

    regenerarFactura(idVenta: string) {
        let params: HttpParams = new HttpParams()
            .set("IdVenta", idVenta);
        //
        return this.apiService.post<any>('regenerar-factura', params, null)
            .toPromise();
    }

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/



}