﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { BeneficiarioService} from '../services/beneficiario.service';
import { BeneficiarioInfo, BeneficiarioItem } from '../models/beneficiario';
import { GrupoBeneficiariosSelectorComponent } from '../controls/grupo-beneficiarios-selector.component';
import { GrupoBeneficiariosInfo } from '../models/grupo-beneficiarios';
import { BancoSelectorComponent } from '../../accounting/components/banco-selector.component';
import { validateConfig } from '@angular/router/src/config';

@Component({
    selector: 'tesoreria-beneficiario-editor',
    templateUrl: './beneficiarios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => BeneficiariosEditor)
        },
        {
            provide: CatalogService, useExisting: BeneficiarioService
        }
    ]
})
export class BeneficiariosEditor extends CatalogEditor {
    beneficiario: BeneficiarioInfo = new BeneficiarioInfo();     
    @ViewChild('selGrupoBeneficiario') selGrupo: GrupoBeneficiariosSelectorComponent;
    @ViewChild('selBanco') selBanco: BancoSelectorComponent;
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private beneficiarioService: BeneficiarioService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idGrupoBeneficiario': [0],
            'idCuenta': [''],
            'idBanco': [''],
            'clabe': [''],
            'leyendaCheque': [false],
            'saldo': [''],
            'limiteCredito': [''],
            'diasCredito': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.beneficiarioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.beneficiario = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.beneficiarioService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.beneficiario = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.beneficiarioService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.beneficiario = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.beneficiarioService.update(this.beneficiario)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.beneficiario.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.beneficiario = info;
            })
            .catch(error => this.handleError(error));        
    }

    onSelectTab(args: any) {
        if (args.heading) {
            Control.invalidateAll();
        }
    }

    onGrupoBeneficiarioSelected(grupo: GrupoBeneficiariosInfo) {
        // do something incredible!!
    }
}