﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators, MenuItem } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
//Features
import { CajaService} from '../services/caja.service';
import { CajaInfo, CajaItem} from '../models/caja';
import { CuentaBancariaItem } from '../../accounting/models/cuenta-bancaria';
import { CuentaBancariaService } from '../../accounting/services/cuenta-bancaria.service';

@Component({
    selector: 'tesoreria-caja-editor',
    templateUrl: './cajas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => CajasEditor)
        },
        {
            provide: CatalogService, useExisting: CajaService
        }
    ]
})
export class CajasEditor extends CatalogEditor {
    caja: CajaInfo = new CajaInfo();   
    cuentasBancarias: CuentaBancariaItem[] = [];
    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private cajaService: CajaService, private cuentaBancariaService: CuentaBancariaService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'idChequeraDefault': [''],
            'cuenta': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.applyPolicies();
        this.loadCuentasBancarias();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.cajaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.caja = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.cajaService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.caja = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.cajaService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.caja = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.cajaService.update(this.caja)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.caja.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.caja = info;
            })
            .catch(error => this.handleError(error));        
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'USERS') {
            this.router.navigate(['members'], { relativeTo: this.route });
        }
    }

    loadCuentasBancarias() {
        this.cuentaBancariaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cuentasBeneficiario => {
                this.cuentasBancarias= cuentasBeneficiario;
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

}