﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, CatalogService} from 'framework3/core';
import { CatalogOptions } from 'framework3/custom';
import { AztecaValidators } from 'framework3/controls';
import { CatalogEditor, EditorEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { FormatoTipoService, FormatoItem } from 'framework3/general';
//Features
import { ChequeraService} from '../services/chequera.service';
import { ChequeraInfo, ChequeraItem} from '../models/chequera';

import { CuentaBancariaItem } from '../../accounting/models/cuenta-bancaria';
import { CuentaBancariaService } from '../../accounting/services/cuenta-bancaria.service';


@Component({
    selector: 'tesoreria-chequera-editor',
    templateUrl: './chequeras.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ChequerasEditor)
        },
        {
            provide: CatalogService, useExisting: ChequeraService
        }
    ]
})
export class ChequerasEditor extends CatalogEditor {
    chequera: ChequeraInfo = new ChequeraInfo();        
    cuentasBancarias: CuentaBancariaItem[] = [];
    formatos: FormatoItem[] = [];

    //
    constructor(protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder, protected changeDetector: ChangeDetectorRef,
        protected context: Context, private location: Location, private chequeraService: ChequeraService, private cuentaBancariaService: CuentaBancariaService,
        private formatoService: FormatoTipoService) {

        super(context, router, route, changeDetector);
        this.catalogForm = formBuilder.group({
            'codigo': ['', Validators.required],
            'nombre': ['', Validators.required],
            'cuentaBancaria': [0, Validators.required], //(control) => this.validateCuentaBeneficiario(this.regExpCuentaBeneficiario, control)],
            'idFormato': [0],
            'folioCheque': [0],
            'folioTransferencia': [0],
            'idCuenta': [0],
            'sample_field': ['']
        });
    }

	///////////////////////// METODOS ///////////////////////////////////////

	///////////////////////// HANDLERS ///////////////////////////////////////
    onConfigureCatalog(options: CatalogOptions) {
        this.autogeneratedCode = options.codigoAutogenerado;
        this.loadCuentasBancarias();
        this.loadFormatos();
		this.applyPolicies();
    }

    onCreatingItem() {
        
    }

    onViewingItem(id: number) {
        this.context.app.showSpinner();

        this.chequeraService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.chequera = info;                
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(id: number) {
        this.context.app.showSpinner();
        //
        this.chequeraService.clone(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.chequera = info;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingItem(id: number) {
        this.context.app.showSpinner();

        this.chequeraService.getDetailByID(id)
            .then(info => {
                this.context.app.hideSpinner();
                this.chequera = info;                
            })
            .catch(error => this.handleError(error));
    }

    onSavingItem(args: EditorEventArgs) {
        this.context.app.showSpinner();
        this.chequeraService.update(this.chequera)
            .then(info => {
                this.context.app.hideSpinner();
				this.reset();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                if (!args.closeEditor && this.chequera.identity == 0) {
                    this.router.navigate(['../', info.identity], { relativeTo: this.route });
                }
                else {					
					if (args.closeEditor)
						this.router.navigate(['../'], { relativeTo: this.route });
					else
						this.showAlert('La información ha sido actualizada con éxito', 'success');                
                }
				this.chequera = info;
            })
            .catch(error => this.handleError(error));        
    }

    loadCuentasBancarias() {
        this.cuentaBancariaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cuentasBancarias => {
                this.cuentasBancarias = cuentasBancarias;
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

    loadFormatos() {
        this.formatoService.loadFormatos('CHECKS', this.context.app.defaultCompany.identity)
            .then(formatos => {
                this.formatos = formatos;
                this.formatos.unshift({ id: 0, nombre: '[NINGUNO]', activo: true, codigo: '' });
            })
            .catch(error => this.handleError(error));
    }


}