﻿import { Component, Input, Output, OnInit, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { mergeMap, map } from 'rxjs/operators';
//
import { Context, CatalogItem, BaseComponent } from 'framework3/core';
import { SearchRequest, SortOrder, CatalogSelector, SearchCondition } from 'framework3/search/index';
//
import { BeneficiarioInfo, BeneficiarioItem } from '../models/beneficiario';
import { BeneficiarioService } from '../services/beneficiario.service';

@Component({
    selector: 'treasury-beneficiario-selector',
    templateUrl: './beneficiario-selector.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => BeneficiarioSelectorComponent)
    },
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => BeneficiarioSelectorComponent),
        multi: true
    }]
})
export class BeneficiarioSelectorComponent extends CatalogSelector {
    beneficiarios: any[];
    selectedItem: BeneficiarioInfo = null;

    @Output() itemSelected: EventEmitter<BeneficiarioInfo> = new EventEmitter<BeneficiarioInfo>();

    //Inputs
    @Input() tipoBeneficiario: string;


    constructor(protected context: Context, private service: BeneficiarioService) {
        super(context);

        this.keywordSearchSource = Observable.create(observer => {
            observer.next(this.innerSelector.displayValue);
        })
            .pipe(mergeMap((keyword: string) => this.findGrupo(keyword)));
    }

    loadItem(itemID: number) {
        if (itemID > 0) {
            this.service.getDetailByID(itemID)
                .then(info => {
                    this.selectedItem = info;
                    this.displayValue = info.nombre;
                    this.itemSelected.emit(this.selectedItem);
                })
                .catch(error => this.handleError(error));
        }
        else {
            this.selectedItem = null;
            this.displayValue = "";
        }
    }

    doSearch(request: SearchRequest) {

        if (this.tipoBeneficiario != undefined) {
            let condicion: SearchCondition = { field: "tipoBeneficiario", operator: "=", valueType: "string", value: this.tipoBeneficiario }
            request.conditions.push(condicion);
        }

        this.context.app.showSpinner();

        this.service.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.beneficiarios = response.rows;
                this.totalRows = response.totalRows;
                this.pageIndex = request.pageIndex;
            })
            .catch(error => this.handleError(error));
    }

    findGrupo(keyword: string): Observable<BeneficiarioItem[]> {
        return this.service.searchKeywordBeneficiarios(this.context.app.defaultSite.identity, keyword)
            .pipe(map(benefisiarios => {
                return benefisiarios.filter(beneficiario => (beneficiario.tipoBeneficiario == this.tipoBeneficiario || this.tipoBeneficiario == undefined));
        }));

    }

    onKeywordSearch(keyword: string) {
        this.markUnselected();
    }

}