﻿import { Component, EventEmitter, Input, Output, ViewChild, forwardRef } from '@angular/core';
//
import { BsModalService, BsModalRef } from 'ngx-bootstrap';
//import { Context, SucursalItem, SucursalService } from '../../../core';
import { Context, BaseComponent } from 'framework3/core';
import { DocumentListDialog, SearchRequest, SearchCondition } from 'framework3/search';

import { CuentaPorPagarService } from '../services/cuenta-por-pagar.service'
import { CuentaPorPagarInfo, CuentaPorPagarItem } from '../models/cuenta-por-pagar'
import { elementAt } from 'rxjs-compat/operator/elementAt';
//

@Component({
    selector: 'treasury-cuentas-por-pagar-list',
    templateUrl: './cuenta-por-pagar-list.component.html',
    providers: [{
        provide: BaseComponent,
        useExisting: forwardRef(() => CuentasPorPagarListComponent)
    }]
})
export class CuentasPorPagarListComponent extends DocumentListDialog {
    ordenes: CuentaPorPagarItem[];
    //Inputs    
    @Input() idCentroRecepcion: number = 0;
    @Input() idWorkflowStep: number = 0;
    @Input() idBeneficiario: number = 0;
    @Input() idDivisa: number = 0;
    @Input() idStatus: number = 0;
    @Input() startPeriod: number = 15;

    constructor(protected context: Context, private service: CuentaPorPagarService) {
        super(context);
    }

    onSearchRequested(request: SearchRequest) {
        //Si se especificó status, agregar filtro
        if (this.idWorkflowStep != 0) {
            let condition: SearchCondition = { field: 'IdWorkflowStep', operator: '=', valueType: 'int', value: this.idWorkflowStep };
            request.conditions.push(condition);
        }

        //Si se especificó Beneficiario, agregar filtro
        if (this.idBeneficiario != 0) {
            let condition: SearchCondition = { field: 'IdBeneficiario', operator: '=', valueType: 'int', value: this.idBeneficiario };
            request.conditions.push(condition);
        }

        //Si se especificó divisa, agregar filtro
        if (this.idDivisa != 0) {
            let condition: SearchCondition = { field: 'IdDivisa', operator: '=', valueType: 'int', value: this.idDivisa };
            request.conditions.push(condition);
        }

        //Si se especificó centro de recepcion, agregar filtro
        if (this.idCentroRecepcion != 0) {
            let condition: SearchCondition = { field: 'IdCentroRecepcion', operator: '=', valueType: 'int', value: this.idCentroRecepcion };
            request.conditions.push(condition);
        }
        //
        if (this.idStatus > 0) {
            let condition: SearchCondition = { field: 'IdStatus', operator: '=', valueType: 'int', value: this.idStatus };
            request.conditions.push(condition);
        }

        this.service.search(this.context.app.defaultSite.identity, request)
            .then(result => {
                this.ordenes = result.rows;
                //this.totalRows = result.totalRows; se habilita cuando se agregue el extends a DocumentListDialog
                this.pageIndex = request.pageIndex;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    onSelectedItemsChanged(selecteItems: CuentaPorPagarItem[]) {
        let elements = [];
        selecteItems.forEach(item => {
            this.ordenes.forEach(itemCuentaPorPagar => {
                if (itemCuentaPorPagar.id == item.id) {
                    elements.push(itemCuentaPorPagar);
                }
            });
        });
        this.selectedItemsChanged.emit(elements);
    }

    onSelectedItemChanged(item: CuentaPorPagarItem) {
        if (item != null) {
            this.ordenes.forEach(itemCuentaPorPagar => {
                if (itemCuentaPorPagar.id == item.id) {
                    this.selectedItemChanged.emit(itemCuentaPorPagar);
                }
            });
        }
    }

    //se borra cuando se agregue el extends a DocumentListDialog
    handleError(error: any) {
        console.error(error);
        this.context.app.hideSpinner();
    }


}