﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { MenuItem } from 'framework3/controls';
import { ContentViewerComponent } from 'framework3/controls/components/content-viewer.component';

//Locales
import { ContrareciboInfo, ContrareciboItem } from '../models/contrarecibo';
import { ContrareciboDetail } from '../models/contrarecibo-detail';
import { ContrareciboService } from '../services/contrarecibo.service';

import { ReporteInfo, ParametroInfo } from 'framework3/core';

import { ReportService } from 'framework3/core';

@Component({
    selector: 'tesoreria-contrarecibos-tit-manager',
    templateUrl: './contrarecibos-tit.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => ContrarecibosTitManager)
        },
        {
            provide: DocumentService, useExisting: ContrareciboService
        }
    ]
})
export class ContrarecibosTitManager extends DocumentManager {
    contrarecibosTit: ContrareciboItem[];

    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;
    titleContentView: string = "";
    fileNameContentView: string = "";

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private contrareciboService: ContrareciboService, private reportSvc: ReportService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.contrareciboService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.contrarecibosTit = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.contrareciboService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.contrareciboService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.contrareciboService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {
        if (option.name == 'PRINT') {
            this.context.app.showSpinner();
            let item = <ContrareciboItem>this.manager.currentDocument;
            let informe = new ReporteInfo();
            let parametro = new ParametroInfo();
            parametro.name = "IdContrareciboTit";
            parametro.values.push(item.id);
            informe.parametros.push(parametro);
            informe.path = "/Onili/Tesoreria/ContraRecibo";
            informe.format = "pdf";
            this.reportSvc.loadReporte(informe)
                .then(info => {
                    this.context.app.hideSpinner();
                    let item = <ContrareciboItem>this.manager.currentDocument;
                    this.titleContentView = "Recibo " + item.id;
                    this.fileNameContentView = item.id + ".pdf";
                    this.reportViewer.show("data:application/pdf; base64, " + info.content);
                })
                .catch(error => this.handleError(error));
        }
    }

}

