﻿import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { Router, Route, ActivatedRoute, Params } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import * as moment from 'moment';
//Bootstrap
import { ModalDirective } from 'ngx-bootstrap';
//Wijmo
import { Control, CollectionView, EventArgs } from 'wijmo/wijmo';
import { Menu } from 'wijmo/wijmo.input';
import { FlexGrid, HitTestInfo, CellType, CellRange, CellRangeEventArgs } from 'wijmo/wijmo.grid';
//Azteca Kernel
import { Context, ProcessComponent, PolicyInfo, FileLink } from 'framework3/core';
import { FileInfo } from 'framework3/general';
import { MenuItem, ToolbarButton, AlertsBarComponent, DownloadPopupComponent } from 'framework3/controls';
import { SearchCondition, SearchRequest, SearchResult, ConditionValueType, FilterGroup, DocumentListComponent } from 'framework3/search';
import { CatalogEditor, EditorEventArgs, TrackableCollection } from 'framework3/editors';

import { EstadoCuentaBancarioService } from '../services/estado-cuenta-bancario.service';
import { EstadoCuentaBancarioConciliarItem } from '../models/estado-cuenta-bancario-conciliar-item';
import { AsyncTaskService } from "framework3/queue/services/async-task.service";

@Component({
    selector: 'tesoreria-estados-cuenta-bancarios-conciliar',
    templateUrl: './estados-cuenta-bancarios-conciliar.component.html'
})
export class EstadoCuentaBancarioConciliar extends ProcessComponent{
    form: FormGroup;
    idEstadoCuenta: string;
    conciliando: any[] = [];
    cuenta: string;
    origenes: { id: string, nombre: string }[] = [];
    origen: string;

    previewCollectionMovimientos: TrackableCollection = new TrackableCollection();
    previewCollectionMovimientosRepetidos: TrackableCollection = new TrackableCollection();

    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private formBuilder: FormBuilder,
        private estadoCuentaService: EstadoCuentaBancarioService,
        private asyncTaskSvc: AsyncTaskService
    ) {
        super(context, router, route);
        this.route.params.subscribe((params: Params) => {
            this.idEstadoCuenta = params['idEstadoCuenta'];
        });

        this.form = formBuilder.group({
            'cuenta': ['', Validators.required],
            'origen': ['', Validators.required]
        });

        this.origenes =
            [
            { "id": "P", "nombre": "Pixcua" },
            { "id": "O", "nombre": "Onili" },
            { "id": "C", "nombre": "Contpac" }
            ];

    }

    //Implementacion de politicas
    applyPolicy(policy: PolicyInfo) {

    }

    ngOnInit() {

    }

    onSelectionChanged(args: any) {
        var currentItem: any = new EstadoCuentaBancarioConciliarItem;
        currentItem = this.previewCollectionMovimientos.currentItem;

        this.previewCollectionMovimientosRepetidos.sourceCollection = this.conciliando.filter(item => item.IdEstadosCuentaBancariosDet == currentItem.IdEstadosCuentaBancariosDet);
    }

    onCellEditEnded(flex: any, e: any) {
        let det: any = flex.rows[e.row].dataItem;

    }

    onCellEditEndedB(flex: any, e: any) {
        //let det: any = flex.rows[e.row].dataItem;
        let sels: any[] = [];
        sels = this.previewCollectionMovimientosRepetidos.sourceCollection.filter(item => item.Sel == true);
        if (sels.length > 1) {
            this.alerts.showAlert('Sólo debe de seleccionar un registro', 'danger');
        }
    }


    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'CONSULTAR') {
            if (this.form.valid) {
                this.context.app.showSpinner();
                this.estadoCuentaService.asyncConciliarIngresos(this.idEstadoCuenta, this.origen, this.cuenta + '%')//'102%'
                    .then(edo => {
                        this.asyncTaskSvc.subscribeTask(edo.idAsyncTask).subscribe(info => {
                            this.context.app.hideSpinner();
                            this.conciliando = JSON.parse(info.result);
                            this.previewCollectionMovimientos.sourceCollection = this.conciliando.filter(item => item.Cont == 1);
                        })
                    })
                    .catch(error => this.handleError(error));
            }
            else {
                this.alerts.showAlert('Debe de establecer una cuenta o familia de cuentas', 'danger')
            }

        }
        if (button.name == 'GUARDAR') {
            this.estadoCuentaService.conciliarIngresosSet(this.conciliando)
                .then(result => {
                    this.alerts.showAlert('Informacion guardada exitosamente', 'sucess');
                })
                .catch(error => this.handleError(error));
        }
    }

    getStatusColor(color: number): string {
        if (color == -1)
            return '#ff9900';
        else
            return 'white';
    }


    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.context.getExceptionMessage(error);
        this.alerts.showAlert(message, 'danger');
    }


}