﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control, Globalize } from 'wijmo/wijmo';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo, ChangeState } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton, ContentViewerComponent } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
import { TrackableCollection } from 'framework3/editors';
//Locales
import { EstadoCuentaBancarioInfo, EstadoCuentaBancarioItem } from '../models/estado-cuenta-bancario';
import { EstadoCuentaBancarioDetail } from '../models/estado-cuenta-bancario-detail';
import { EstadoCuentaBancarioService } from '../services/estado-cuenta-bancario.service';
import { EstadoCuentaBancarioDetailEditor } from './parts/estados-cuenta-bancarios-detail.editor';

//Features
import { CuentaBancariaItem } from '../../accounting/models/cuenta-bancaria';
import { CuentaBancariaService } from '../../accounting/services/cuenta-bancaria.service';

import { EstadoCuentaBancarioFormatoInfo } from '../models/estado-cuenta-bancario-formato';
import { DatasetInfo, ParameterItem, TranspilerInfo, DatasetService } from '../../translator/index';


@Component({
    selector: 'tesoreria-estados-cuenta-bancarios-editor',
    templateUrl: './estados-cuenta-bancarios.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EstadosCuentaBancariosEditor)
        },
        {
            provide: DocumentService, useExisting: EstadoCuentaBancarioService
        }
    ]
})
export class EstadosCuentaBancariosEditor extends DocumentEditor {
    estadoCuentaBancario = new EstadoCuentaBancarioInfo();
    cuentasBancarias: CuentaBancariaItem[] = [];
    estadoCuentaFormato: EstadoCuentaBancarioFormatoInfo = new EstadoCuentaBancarioFormatoInfo();
    cvParametros: TrackableCollection;
    interval: any;
    saldoContabilidad: number = 0;
    saldoEstadoCuenta: number = 0;

    titleContentView: string = "";
    fileNameContentView: string = "";


    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: EstadoCuentaBancarioDetailEditor;
    @ViewChild(ContentViewerComponent) reportViewer: ContentViewerComponent;

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private estadoCuentaBancarioService: EstadoCuentaBancarioService,
        private cuentaBancariaService: CuentaBancariaService,
        private datasetSvc: DatasetService) {


        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            //'sample': ['', Validators.required],
            'idDataset': [AztecaValidators.selected],
            archivo: [null, Validators.required],
            'cuentaBancaria': [0, Validators.required],
            'ImporteDepositos': [0],
            'ImporteRetiros': [0],
            'DepositosOnili': [0],
            'RetirosOnili': [0],
            'DepositosPixcua': [0],
            'RetirosPixcua': [0],
            'DepositosContpac': [0],
            'RetirosContpac': [0],
            'saldoEstadoCuenta': [0],
            'saldoContabilidad': [0],
        });
        this.cvParametros = new TrackableCollection();
        this.cvParametros.filter = (item: ParameterItem) => {
            return (!item.hidden && item.state != ChangeState.DELETED);
        }
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.loadCuentasBancarias();
        this.applyPolicies();
    }

    onCreatingDocument() {

    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.estadoCuentaBancarioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.estadoCuentaBancario = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.estadoCuentaBancarioService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.estadoCuentaBancario = doc;
                this.detailsEditor.clientIndexStart = this.estadoCuentaBancario.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.estadoCuentaBancarioService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.estadoCuentaBancario = doc;
                this.detailsEditor.clientIndexStart = this.estadoCuentaBancario.details.length;
                if (this.estadoCuentaBancario.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.context.app.showSpinner();

        this.estadoCuentaFormato.transpiler.datasetInput.parameters.forEach(param => {
            if (param.field == 'IdSucursal') {
                param.value = this.context.app.defaultSite.identity.toString();
            }
            else if (param.field == 'IdCuentaBancaria') {
                param.value = this.estadoCuentaBancario.idCuentaBancaria.toString();
            }
        })

        this.estadoCuentaBancarioService.encolarTranspiler(this.estadoCuentaFormato)
            .then(result => {
                if (result > 0) {
                    this.showAlert("Se ha generado la tarea para subir estado de cuenta", "success");
                    this.interval = setInterval(_ => this.checaStatus(result), 5000);
                } else {
                    this.context.app.hideSpinner();
                    this.showAlert("Se a generado un error al registrar la tarea para subir el estado de cuenta", "error");
                }
                /*
                this.context.app.hideSpinner();
                this.router.navigate(['../'], { relativeTo: this.route });
                */
            })
            .catch(error => this.handleError(error));


        /*
        this.estadoCuentaBancarioService.update(this.estadoCuentaBancario)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                this.reset();
                if (!args.closeEditor && this.estadoCuentaBancario.identity == '') {
                    this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                }
                else {
                    if (args.closeEditor)
                        this.router.navigate(['../'], { relativeTo: this.route });
                    else
                        this.showAlert('La información ha sido actualizada con éxito', 'success');
                }
                this.estadoCuentaBancario = updatedDoc;
            })
            .catch(error => this.handleError(error));
        */
    }

    checaStatus(idLote) {
        this.estadoCuentaBancarioService.getTranspilerStatus(idLote)
            .then(result => {
                if (result.completed) {
                    if (result.errorMessage.length > 0) {
                        clearInterval(this.interval);
                        this.context.app.hideSpinner();
                        this.showAlert("Error al subir estado de cuenta " + result.errorMessage.length, "error");
                    } else {
                        clearInterval(this.interval);
                        this.context.app.hideSpinner();
                        this.showAlert("Se ha subido el estado de cuenta", "success");
                        this.router.navigate(['../'], { relativeTo: this.route });
                    }
                }
            })
            .catch(error => this.handleError(error))
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.estadoCuentaBancarioService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.estadoCuentaBancarioService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.estadoCuentaBancario.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        this.detailsEditor.detail = new EstadoCuentaBancarioDetail();
        this.detailsEditor.show();
    }

    onEditingDetail(detail: EstadoCuentaBancarioDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: EstadoCuentaBancarioDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: EstadoCuentaBancarioDetail[]) {
        this.estadoCuentaBancario.details = updatedDetails;
    }

    onSavingDetail(detail: EstadoCuentaBancarioDetail) {
        this.detailsList.updateItem(detail);
    }

    loadCuentasBancarias() {
        this.cuentaBancariaService.loadListByUser(this.context.app.defaultSite.identity)
            .then(cuentasBancarias => {
                this.cuentasBancarias = cuentasBancarias;
            })
            .catch(error => this.showAlert(error, 'danger'));
    }

    onTranspilerSelected(item: TranspilerInfo) {
        if (item) {
            this.estadoCuentaFormato.transpiler = item;
            this.context.app.showSpinner();
            let promises = [];
            if (item.idDatasetInput > 0) {
                let promiseInput = this.datasetSvc.getDetailByID(item.idDatasetInput)
                    .then(info => {
                        this.estadoCuentaFormato.transpiler.datasetInput = info
                        this.cvParametros.sourceCollection = info.parameters;
                        this.cvParametros.refresh();
                    })
                    .catch(error => this.handleError(error));
                promises.push(promiseInput);
            }
            if (item.idDatasetOutput > 0) {
                let promisOutput = this.datasetSvc.getDetailByID(item.idDatasetOutput)
                    .then(info => this.estadoCuentaFormato.transpiler.datasetOutput = info)
                    .catch(error => this.handleError(error));
                promises.push(promisOutput);
            }
            Promise.all<void>(promises).then(() => {
                this.context.app.hideSpinner();
            });
        }
    }

    onImprimir() {
        this.context.app.showSpinner();
        this.estadoCuentaBancarioService.toPdf(this.estadoCuentaBancario.identity, this.saldoContabilidad, this.saldoEstadoCuenta)
            .then(pdf => {
                this.context.app.hideSpinner();
                //window.open('/file-server/' + pdf.url, '_blank');
                this.titleContentView = "EstadoCuenta_" + this.estadoCuentaBancario.identity + ".pdf";
                this.fileNameContentView = "EstadoCuenta_" + this.estadoCuentaBancario.identity + ".pdf";
                this.reportViewer.show("data:application/pdf; base64, " + pdf);

            })
    }

    get getImporteDepositos() {
        if (!this.estadoCuentaBancario.importeDepositos)
            this.estadoCuentaBancario.importeDepositos = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.importeDepositos, 'c');
    }
    get getImporteRetiros() {
        if (!this.estadoCuentaBancario.importeRetiros)
            this.estadoCuentaBancario.importeRetiros = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.importeRetiros, 'c');
    }
    get getImporteDepositosOnili() {
        if (!this.estadoCuentaBancario.importeConciliadosDepositos)
            this.estadoCuentaBancario.importeConciliadosDepositos = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.importeConciliadosDepositos, 'c');
    }
    get getImporteRetirosOnili() {
        if (!this.estadoCuentaBancario.importeConciliadosRetiros)
            this.estadoCuentaBancario.importeConciliadosRetiros = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.importeConciliadosRetiros, 'c');
    }
    get getImporteDepositosPixcua() {
        if (!this.estadoCuentaBancario.importeConciliadosPixcuaDepositos)
            this.estadoCuentaBancario.importeConciliadosPixcuaDepositos = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.importeConciliadosPixcuaDepositos, 'c');
    }
    get getImporteRetirosPixcua() {
        if (!this.estadoCuentaBancario.importeConciliadosPixcuaRetiros)
            this.estadoCuentaBancario.importeConciliadosPixcuaRetiros = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.importeConciliadosPixcuaRetiros, 'c');
    }
    get getImporteDepositosContpac() {
        if (!this.estadoCuentaBancario.importeConciliadosContpacDepositos)
            this.estadoCuentaBancario.importeConciliadosContpacDepositos = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.importeConciliadosContpacDepositos, 'c');
    }
    get getImporteRetirosContpac() {
        if (!this.estadoCuentaBancario.importeConciliadosContpacRetiros)
            this.estadoCuentaBancario.importeConciliadosContpacRetiros = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.importeConciliadosContpacRetiros, 'c');
    }
    get getSaldoFinalEstadoCuenta() {
        if (!this.estadoCuentaBancario.saldoFinal)
            this.estadoCuentaBancario.saldoFinal = 0;
        return Globalize.formatNumber(this.estadoCuentaBancario.saldoFinal, 'c');
    }
    

}

