﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { MenuItem } from 'framework3/controls';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
//Locales
import { EstadoCuentaBancarioInfo, EstadoCuentaBancarioItem } from '../models/estado-cuenta-bancario';
import { EstadoCuentaBancarioDetail } from '../models/estado-cuenta-bancario-detail';
import { EstadoCuentaBancarioService } from '../services/estado-cuenta-bancario.service';

@Component({
    selector: 'tesoreria-estados-cuenta-bancarios-manager',
    templateUrl: './estados-cuenta-bancarios.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => EstadosCuentaBancariosManager)
        },
        {
            provide: DocumentService, useExisting: EstadoCuentaBancarioService
        }
    ]
})
export class EstadosCuentaBancariosManager extends DocumentManager {
    estadosCuentaBancarios: EstadoCuentaBancarioItem[];

    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private estadoCuentaBancarioService: EstadoCuentaBancarioService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.estadoCuentaBancarioService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.estadosCuentaBancarios = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.estadoCuentaBancarioService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.estadoCuentaBancarioService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.estadoCuentaBancarioService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onMenuItemClick(option: MenuItem) {

        if (option.name == 'CONCILIAR-INGRESOS') {
            this.router.navigate(['treasury/estados-cuenta-bancarios-conciliar/' + this.process.id + '/' + this.manager.currentDocument.id]);
        }
    }


}

