﻿//Frameworks
import { Component, ViewChild, forwardRef, Input } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentInfo, DocumentItem, FileLink } from 'framework3/core';
import { DocumentManager, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest } from 'framework3/search';
import { LoteImpresionInfo, LoteImpresionService, PrinterService } from 'framework3/printing';
//Locales
import { PagoInfo, PagoItem, Cheque } from '../models/pago';
import { PagoDetail } from '../models/pago-detail';
import { PagoService } from '../services/pago.service';
import { MenuItem, DownloadPopupComponent } from 'framework3/controls';
import { ModalDirective } from 'ngx-bootstrap/modal/modal.directive';
import { PolizaPerfilListComponent, PolizaPerfilItem, PolizaInfo, PolizaService } from '../../accounting/index';

@Component({
    selector: 'tesoreria-pagos-manager',
    templateUrl: './pagos.manager.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PagosManager)
        },
        {
            provide: DocumentService, useExisting: PagoService
        }
    ]
})
export class PagosManager extends DocumentManager {
    pagos: PagoItem[];
    pagosToCheque: string[] = [];
    intervalId: any;
    currentPoliza: PolizaInfo = new PolizaInfo();

    @ViewChild('viewDialog') dlgPolizaViewer: ModalDirective;
    @ViewChild(PolizaPerfilListComponent) dlgPolizaPerfil: PolizaPerfilListComponent;
    @ViewChild(DownloadPopupComponent) downloadPopup: DownloadPopupComponent;


    constructor(protected context: Context, protected router: Router, protected route: ActivatedRoute, private pagoService: PagoService, private loteImpresionService: LoteImpresionService, private printerService: PrinterService, private polizaSvc: PolizaService) {
        super(context, router, route);
    }

    onConfigureDocument() {
        this.refreshList();
        this.applyPolicies();
    }

    onSearchRequested(request: SearchRequest) {
        this.context.app.showSpinner();

        this.pagoService.search(this.context.app.defaultSite.identity, request)
            .then(response => {
                this.context.app.hideSpinner();
                //
                this.pagos = response.rows;
                this.totalRows = response.totalRows;
                this.filterGroups = response.filterGroups;
            })
            .catch(error => this.handleError(error));
    }

    onAddingItem() {
        this.router.navigate(['0'], { relativeTo: this.route });
    }

    onViewingItem(document: DocumentItem) {
        this.router.navigate([document.id, { readonly: true }], { relativeTo: this.route });
    }

    onEditingItem(document: DocumentItem) {
        this.router.navigate([document.id], { relativeTo: this.route });
    }

    onDeletingItem(document: DocumentItem) {
        this.pagoService.delete(document.id)
            .then(success => {
                this.refreshList();
                this.showAlert('El documento ha sido eliminado con éxito.', 'success');
            })
            .catch(error => this.handleError(error));
    }

    onCloningItem(document: DocumentItem) {
        this.router.navigate([0, { cloneId: document.id }], { relativeTo: this.route });
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();
        
        this.pagoService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.pagoService.previousStep(document.id, null)
            .then(_ => {
                this.context.app.hideSpinner();
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }
    
    onMenuItemClick(option: MenuItem) {
        if (option.name == 'IMPRIMIR') {
            this.pagosToCheque = [];
            this.context.app.showSpinner();
            var i = 0;
            for (i = 0; i <= this.manager.baseList.documentsView.totalItemCount - 1; i++) {
                this.pagosToCheque[i] = this.manager.baseList.documentsView._view[i].id;
            }
            this.pagoService.getLoteCheques(this.pagosToCheque)
                .then(lote => {
                    this.intervalId = setInterval(_ => this.checaStatus(lote.id), 5000);
                })
                .catch(error => this.handleError(error));
        }
        else if (option.name == "CONTABILIZAR") {
            this.dlgPolizaPerfil.show();
        }
    }

    checaStatus(idLote) {
        //this.context.app.hideSpinner();
        this.loteImpresionService.getStatus(idLote)
            .then(result => {

                if (result == 'C') {
                    this.loteImpresionService.getFiles(idLote)
                        .then(files => {
                            this.context.app.hideSpinner();
                            let links: FileLink[] = [];
                            let Cheques: FileLink = new FileLink;
                            Cheques.uri = files[0];
                            Cheques.fileName = files[0];
                            links.push(Cheques);
                            this.printerService.printTextFile('/file-server/' + files[0]);
                            clearInterval(this.intervalId);
                        })

                }
                /*
                if (result == 'R') {
                    this.showAlert('Generando bloque de impresión, el proceso puede tardar varios minutos', 'success');
                }
                */
                if (result == 'E') {
                    clearInterval(this.intervalId);
                    this.context.app.hideSpinner();
                    this.showAlert('Error al obtener el bloque de impresión de cheques', 'success');
                }
        })
        .catch(error => this.handleError(error))
    }

    onSelectedPolizaPerfil(item: PolizaPerfilItem) {
        this.context.app.showSpinner();
        this.polizaSvc.loadPolizaByDocumento(this.manager.currentDocument.id, item.id, this.context.app.defaultSite.identity)
            .then(result => {
                this.currentPoliza = result;
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.config.backdrop = false;
                this.dlgPolizaViewer.show();
            }).catch(error => this.handleError(error));
    }

    onClickGuardar() {
        this.context.app.showSpinner();
        if (this.currentPoliza.idPolizaOrigen == 0) this.currentPoliza.idPolizaOrigen = 1 //Manual
        this.polizaSvc.update(this.currentPoliza)
            .then(updatedDoc => {
                this.context.app.hideSpinner();
                this.dlgPolizaViewer.hide();
                this.manager.showAlert('La poliza ha sido registrada con éxito', 'success');
            })
            .catch(error => this.handleError(error));
    }

}

