﻿import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
//Locales
import { CuentaPorPagarDetail } from '../../models/cuenta-por-pagar-detail';
import { ConceptoInfo } from '../../models/concepto';
import { ConceptoService } from '../../services/concepto.service';
import { ImpuestoDetailInfo } from '../../../accounting/models/impuesto-detail';

@Component({
    selector: 'tesoreria-cuenta-por-pagar-detail-editor',
    templateUrl: './cuentas-por-pagar-detail.editor.html'
})
export class CuentaPorPagarDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new CuentaPorPagarDetail();
    //Objeto original que se editará
    @Input() detail: CuentaPorPagarDetail = new CuentaPorPagarDetail();

    constructor(protected context: Context, private builder: FormBuilder, private conceptoService: ConceptoService) {
        super(context);

        this.form = builder.group({
            'idConcepto': [0, Validators.required],
            'cantidad': [0, Validators.required],
            'costoUnitario': [0, Validators.required],
        });
    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
		this.form.reset({ sample: '' });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: CuentaPorPagarDetail ) {
        this.detailCopy = <CuentaPorPagarDetail>this.copyItem(this.detail);
    }

    onUpdatingItem(detail: CuentaPorPagarDetail ) {
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
    }

    onConceptoChanged(concepto:ConceptoInfo) {
        this.detailCopy.codigoConcepto = concepto.code;
        this.detailCopy.descripcionConcepto = concepto.nombre;
        //this.detailCopy.descuentos = concepto.descuentos;
        this.detailCopy.impuestos = [];
        concepto.impuestos.forEach(impuesto => {
            let impto = new ImpuestoDetailInfo();
            impto.base = 0;
            impto.idImpuesto = impuesto.idImpuesto;
            impto.impuesto = impuesto.impuesto;
            impto.tasaOCuota = impuesto.tasaOCuota;
            impto.tipoAplicacion = impuesto.tipoAplicacion;
            impto.tipoFactor= impuesto.tipoCalculo;
            this.detailCopy.impuestos.push(impto)
        });
        
    }

    onCantidadChange(value: number) {
        this.detailCopy.importe = this.detailCopy.costoUnitario * value;
    }

    onCostoChange(costoUnitario: number) {
        if (costoUnitario > 0 && this.detailCopy) { //&& this.detailCopy.servicio.libre) {
            this.detailCopy.impuestos.forEach(impuesto => {
                impuesto.base = costoUnitario;
            });
            //this.trasladosCollection.refresh();
            //this.retencionesCollection.refresh();
        }

        let costo = this.conceptoService.calcularCostoLibre(costoUnitario, false, this.detailCopy.impuestos)

        this.detailCopy.costoUnitario = costo.costoBase;        
        this.detailCopy.importe = costo.costoBase * this.detailCopy.cantidad;  
    }


}
