﻿import { Component, Input, Output, EventEmitter } from '@angular/core';
import { FormGroup, FormBuilder, AbstractControl, Validators } from '@angular/forms';
import * as moment from 'moment';

//Kernel
import { Context } from 'framework3/core';
import { BasicEditor, ChangeState } from 'framework3/editors';
//Locales
import { PagoDetail } from '../../models/pago-detail';
import { PagoService } from '../../services/pago.service'

@Component({
    selector: 'tesoreria-pago-detail-editor',
    templateUrl: './pagos-detail.editor.html'
})
export class PagoDetailEditor extends BasicEditor {    
    //Copia temporal que se utilizará para edición y no afecte al objeto original por si el usuario decide cancelar la edición
    detailCopy = new PagoDetail();
    //Objeto original que se editará
    @Input() detail: PagoDetail = new PagoDetail();
    @Input() idBeneficiario: number = 0;
    @Input() idDivisa: number = 0;

    @Output() dataLoaded: EventEmitter<PagoDetail[]> = new EventEmitter<PagoDetail[]>();

    fechasFiltro: { id: string, nombre: string }[] = [];

    fechaSearch: string = 'G';
    fechaIni: Date;
    fechaFin: Date;


    constructor(protected context: Context, private builder: FormBuilder, private pagoService: PagoService) {
        super(context);

        this.form = builder.group({
            'fechaBusqueda': ['', Validators.required],
            'fechaIni': [new Date()],
            'fechaFin': [new Date()],
        });

        this.fechasFiltro =
            [
                { "id": "G", "nombre": "Generación" },
                { "id": "V", "nombre": "Vencimiento" }

            ];

        this.fechaSearch = "G";
        this.fechaIni = moment().subtract(3, 'months').toDate();
        this.fechaFin = moment().add(2, 'week').toDate();

    }

    /////////////////// METODOS ///////////////////////////////


    /////////////////// HANDLERS //////////////////////////////

    onCreatingItem() {
        this.detailCopy = this.detail;
		//this.form.reset({ sample: '' });
        this.form.markAsPristine();
    }

    onLoadingItem(detail: PagoDetail ) {
        this.detailCopy = <PagoDetail>this.copyItem(this.detail);
    }

    onUpdatingItem(detail: PagoDetail) {
        /*
        if (this.detailCopy._clientID == 0) {
            this.detailCopy._clientID = this.createClientID();
        }
        else {
            this.detailCopy.state = ChangeState.MODIFIED;
        }
        this.savingItem.emit(this.detailCopy);
        */
        this.loadCxPbyBeneficiario();
    }

    loadCxPbyBeneficiario() {
        this.context.app.showSpinner()
        this.pagoService.loadCxPByBeneficiario(moment(this.fechaIni).format(), moment(this.fechaFin).format(), this.idBeneficiario, this.fechaSearch, this.idDivisa, this.context.app.defaultSite.identity)
            .then(cxc => {
                this.context.app.hideSpinner();
                //this.details = cxc;
                this.dataLoaded.emit(cxc);
            })
            .catch(error => this.showAlert(error.message, 'danger'));
    }


}
