﻿import { Component, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { Location } from '@angular/common';
//

import { BaseComponent, Context, CatalogItem, PolicyInfo, UsuarioItem } from 'framework3/core';
import { SearchRequest, SearchCondition } from 'framework3/search';
import { ToolbarButton, AlertsBarComponent } from 'framework3/controls';
//
import { UsuariosListComponent } from 'framework3/security';
//
import { OperacionTipoService } from '../services/operacion-tipo.service';
import { OperacionTipoInfo, OperacionTipoItem } from '../models/operacion-tipo';


@Component({
    selector: 'treasury-operaciones-tipos-miembros',
    templateUrl: './operaciones-tipos-miembros.component.html'
})
export class OperacionesTiposMiembrosComponent extends BaseComponent {
    title: string = '';
    usuarios: UsuarioItem[];
    idOperacionTipo: number;
    operacionTipo: OperacionTipoInfo;
    currentItem: CatalogItem;

    //
    @ViewChild(UsuariosListComponent) usuariosDialog: UsuariosListComponent;
    @ViewChild(AlertsBarComponent) alerts: AlertsBarComponent;
    //
    constructor(private context: Context, private operacionTipoService: OperacionTipoService, private router: Router, private route: ActivatedRoute,
        private location: Location) {
        super();
    }

    ngOnInit() {
        this.route.params.forEach(param => {
            this.idOperacionTipo = +param['id'];
        });
    }

    ngAfterViewInit() {
        setTimeout(_ => this.refreshList());
        setTimeout(_ => this.loadOperacionTipo());

    }

    ///////////////// METODOS ///////////////////////////////

    loadOperacionTipo() {
        this.operacionTipoService.getDetailByID(this.idOperacionTipo)
            .then(info => {
                this.operacionTipo = info;
                this.title = 'Miembros del tipo de operacion - ' + this.operacionTipo.nombre;
            })
            .catch(error => this.handleError(error));
    }

    refreshList() {
        this.context.app.showSpinner();
        this.operacionTipoService.getMembers(this.idOperacionTipo)
            .then(usuarios => {
                this.usuarios = usuarios;
                this.context.app.hideSpinner();
            })
            .catch(error => this.handleError(error));
    }

    ///////////////// HANDLERS ///////////////////////////////
    onCurrentItemChanged(item: CatalogItem) {
        this.currentItem = item;
    }

    onToolbarButtonClick(button: ToolbarButton) {
        if (button.name == 'ADD') {
            this.usuariosDialog.show();
        }
        if (button.name == 'REFRESH') {
            this.refreshList();
        }
        if (button.name == 'CLOSE') {
            this.location.back();
        }
    }

    onSelectedUsersChanged(newUsers: UsuarioItem[]) {
        this.operacionTipoService.addMembers(this.idOperacionTipo, newUsers)
            .then(_ => {
                this.context.app.hideSpinner();
                this.alerts.showAlert('Los usuarios han sido agregados con éxito.', 'success');
                this.refreshList();
            })
            .catch(error => this.handleError(error));
    }

    onRemoveUser() {
        this.context.app.showSpinner();
        this.operacionTipoService.removeMember(this.idOperacionTipo, this.currentItem.id)
            .then(_ => {
                this.context.app.hideSpinner();
                this.usuarios = this.usuarios.filter(item => item.id != this.currentItem.id);
            })
            .catch(error => this.handleError(error));
    }

    applyPolicy(policy: PolicyInfo) {

    }

    handleError(error: any) {
        this.context.app.hideSpinner();
        let message = this.getErrorMessage(error);
        this.alerts.showAlert(message, 'danger');
    }
}