﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { CajaInfo, CajaItem } from '../models/caja'
import { UsuarioItem } from 'framework3/core';

@Injectable()
export class CajaService extends CatalogService<CajaInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/tesoreria/cajas');
    }

    create(): Promise<CajaInfo> {
        return Promise.resolve(new CajaInfo());
    }

    //lista de cajas
    loadListByUser(idSucursal: number): Promise<CajaItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<CajaItem[]>('load-list', params)
            .toPromise();
    }

    getMembers(idCaja: number): Promise<UsuarioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idCaja', idCaja.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)
            .toPromise();
    }

    addMembers(idCaja: number, users: UsuarioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCaja', idCaja.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }

    removeMember(idCaja: number, idUser: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idCaja', idCaja.toString())
            .set('idUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

}

