﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { ConceptoInfo } from '../models/concepto'
import { CostoInfo } from '../models/costo'
import { TipoAplicacion, TipoFactor, ImpuestoDetailInfo } from '../../accounting';


@Injectable()
export class ConceptoService extends CatalogService<ConceptoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/tesoreria/conceptos');
    }

    create(): Promise<ConceptoInfo> {
        return Promise.resolve(new ConceptoInfo());
    }

    calcularCostoLibre(costoBase: number, impuestosIncluidos: boolean, impuestos: ImpuestoDetailInfo[]): CostoInfo {
        let costo = new CostoInfo();
        if (impuestosIncluidos) {
            costo.costoBase = this.calcularCostoBase(costoBase, impuestos);
        }
        else {
            costo.costoBase = costoBase;
        }
        //Recalcular importes de impuestos
        impuestos.forEach(impuesto => {
            impuesto.importe = impuesto.base * impuesto.tasaOCuota;
        });
        costo.impuestos = impuestos;
        //
        let totalTraslados = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.TRASLADO).forEach(i => totalTraslados += i.importe);
        //
        let totalRetenciones = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.RETENCION).forEach(i => totalRetenciones += i.importe);
        //
        costo.costoNeto = costo.costoBase + totalTraslados - totalRetenciones;
        return costo;
    }

    calcularCostoBase(costoNeto: number, impuestos: ImpuestoDetailInfo[]): number {
        //Sumar las tasas de traslado
        let tasasTraslados: number = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.TRASLADO && i.tipoFactor == TipoFactor.TASA).forEach(i => tasasTraslados += i.tasaOCuota);
        //Sumar las cuotas de traslado
        let cuotasTraslados: number = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.TRASLADO && i.tipoFactor == TipoFactor.CUOTA).forEach(i => cuotasTraslados += i.tasaOCuota * i.base);

        //Sumar las tasas de las retenciones
        let tasasRetenciones: number = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.RETENCION && i.tipoFactor == TipoFactor.TASA).forEach(i => tasasRetenciones += i.tasaOCuota);

        //Sumar las cuotas de las retenciones
        let cuotasRetenciones: number = 0;
        impuestos.filter(i => i.tipoAplicacion == TipoAplicacion.RETENCION && i.tipoFactor == TipoFactor.CUOTA).forEach(i => cuotasRetenciones += i.tasaOCuota * i.base);

        let costoBase = (costoNeto - cuotasTraslados + cuotasRetenciones) / (1 + (tasasTraslados - tasasRetenciones));

        return Math.round(costoBase * 100) / 100;
    }
}

