﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';
//Locales
import { EstadoCuentaBancarioInfo, EstadoCuentaBancarioItem } from '../models/estado-cuenta-bancario';
import { EstadoCuentaBancarioDetail } from '../models/estado-cuenta-bancario-detail';
import { EstadoCuentaBancarioFormatoInfo } from '../models/estado-cuenta-bancario-formato';
import { EstadoCuentaBancarioConciliarItem } from '../models/estado-cuenta-bancario-conciliar-item';
import { AsyncTaskInfo } from 'framework3/queue/models/async-task';

@Injectable()
export class EstadoCuentaBancarioService extends DocumentService<EstadoCuentaBancarioDetail, EstadoCuentaBancarioInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/tesoreria/estados-cuenta-bancarios');
    }

    create(): Promise<EstadoCuentaBancarioInfo> {
        let newDoc = new EstadoCuentaBancarioInfo();
        return Promise.resolve(newDoc);
    }

    encolarTranspiler(estadoCuentaFormato: EstadoCuentaBancarioFormatoInfo): Promise<any> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<any>('encolar-transpiler', params, estadoCuentaFormato)
            .toPromise();

    }

    getTranspilerStatus(idLote: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set("idLote", idLote.toString());
        return this.apiService.post<any>("transpiler-status", params)
            .toPromise();
    }

    conciliarIngresos(idEstadoCuenta: string, origen: string): Promise<EstadoCuentaBancarioConciliarItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idEstadoCuenta', idEstadoCuenta)
            .set('origen', origen);
        return this.apiService.get<EstadoCuentaBancarioConciliarItem[]>('conciliar-ingresos', params)
            .toPromise();
    }

    conciliarIngresosSet(conciliado: EstadoCuentaBancarioConciliarItem[]): Promise<boolean> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<any>('conciliar-ingresos-set', params, conciliado)
            .toPromise();
    }

    asyncConciliarIngresos(idEstadoCuenta: string, origen: string, cuenta: string): Promise<AsyncTaskInfo> {
        let params: HttpParams = new HttpParams()
            .set('idEstadoCuenta', idEstadoCuenta)
            .set('origen', origen)
            .set('cuenta', cuenta);
        return this.apiService.get<AsyncTaskInfo>('get-async-conciliar-ingresos', params)
            .toPromise();
    }

    toPdf(idEstadoCuenta: string, saldoEmpresa: number, saldoEstadoCuentaBancario: number): Promise<any> {
        let params = new HttpParams()
            .set('idEstadoCuenta', idEstadoCuenta)
            .set('saldoEmpresa', saldoEmpresa.toString())
            .set('saldoEstadoCuentaBancario', saldoEstadoCuentaBancario.toString());
        return this.apiService.post<FileLink>("to-pdf", params).toPromise();
    }


	/*
    sampleMethod(param1: number, param2: string): Promise<EstadoCuentaBancarioInfo> {
        let params = new HttpParams()
			.set('param1', param1.toString())
			.set('param2', param2);
		//
        return this.apiService.get<EstadoCuentaBancarioInfo>('sample-method', params)
            .toPromise();
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: EstadoCuentaBancarioInfo) {

    }
	*/



}