﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs';
import 'rxjs/add/operator/toPromise';
//Azteca Kernel
import { Context, CatalogService } from 'framework3/core';
//Locales
import { OperacionTipoInfo,OperacionTipoItem } from '../models/operacion-tipo'
import { UsuarioItem } from 'framework3/core';


@Injectable()
export class OperacionTipoService extends CatalogService<OperacionTipoInfo>{

    constructor(http: HttpClient, context: Context) {
        super(http, context, 'api/tesoreria/operaciones-tipos');
    }

    create(): Promise<OperacionTipoInfo> {
        return Promise.resolve(new OperacionTipoInfo());
    }

    //lista de tipo de operaciones
    loadListByUser(idSucursal: number): Promise<OperacionTipoItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<OperacionTipoItem[]>('load-list-by-user', params)
            .toPromise();
    }

    getMembers(idCaja: number): Promise<UsuarioItem[]> {
        let params: HttpParams = new HttpParams()
            .set('idOperacionTipo', idCaja.toString());
        return this.apiService.get<UsuarioItem[]>('members', params)
            .toPromise();
    }

    addMembers(idOperacionTipo: number, users: UsuarioItem[]): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idOperacionTipo', idOperacionTipo.toString());
        return this.apiService.post<any>('add-members', params, users)
            .toPromise();
    }

    removeMember(idOperacionTipo: number, idUser: number): Promise<any> {
        let params: HttpParams = new HttpParams()
            .set('idOperacionTipo', idOperacionTipo.toString())
            .set('idUser', idUser.toString());
        return this.apiService.delete<any>('remove-member', params)
            .toPromise();
    }

}

