﻿//Frameworks
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpRequest, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Rx';
import 'rxjs/add/operator/toPromise';
//Azteca Framework
import { Context, DocumentInfo, DocumentItem, DocumentService, FileLink } from 'framework3/core';
import { LoteImpresionInfo } from 'framework3/printing';
//Locales
import { PagoInfo, PagoItem, Cheque } from '../models/pago';
import { PagoDetail } from '../models/pago-detail';

@Injectable()
export class PagoService extends DocumentService<PagoDetail, PagoInfo> {

    constructor(protected http: HttpClient, protected context: Context ) {
        super(http, context, 'api/tesoreria/pagos');
    }

    create(): Promise<PagoInfo> {
        let newDoc = new PagoInfo();
        return Promise.resolve(newDoc);
    }

    //cargar cxp por beneficiario
    loadCxPByBeneficiario(fechaIni: string, fechaFin: string, idBeneficiario: number, fechaFiltro, idDivisa: number, idSucursal: number): Promise<PagoDetail[]> {
        let params: HttpParams = new HttpParams()
            .set('fechaIni', fechaIni.toString())
            .set('fechaFin', fechaFin.toString())
            .set('idBeneficiario', idBeneficiario.toString())
            .set('FechaFiltro', fechaFiltro)
            .set('idDivisa', idDivisa.toString())
            .set('idSucursal', idSucursal.toString());
        return this.apiService.get<PagoDetail[]>('load-cxp-by-beneficiario', params)
            .toPromise();
    }

    toCheques(idPagos: string[]): Promise<string[]> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<string[]>("to-cheques", params,idPagos)
            .toPromise();
    }

    getLoteCheques(idPagos: string[]): Promise<LoteImpresionInfo> {
        let params: HttpParams = new HttpParams();
        return this.apiService.post<LoteImpresionInfo>("lote-cheques", params, idPagos)
            .toPromise();
    }


	/*
    sampleMethod(param1: number, param2: string): Promise<PagoInfo> {
        let params = new HttpParams()
			.set('param1', param1.toString())
			.set('param2', param2);
		//
        return this.apiService.get<PagoInfo>('sample-method', params)
            .toPromise();
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) en los resultados de búsquedas. Eliminar si no es necesario
	protected mapSearchResult(row: any) {
        
    }
	*/

	/*
	//Descomentar si se necesita realizar conversión de fechas (o cualquier otro tipo de dato) al cargar un documento. Eliminar si no es necesario
    protected mapDocument(document: PagoInfo) {

    }
	*/



}