﻿import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

//////////////////////////// Guards //////////////////////////////////
import { AuthGuard } from 'framework3/core';
import { EditorDeactivateGuard } from 'framework3/editors';

//////////////////////////// Catalogos ////////////////////////////////
import { BeneficiariosManager } from './catalogs/beneficiarios.manager';
import { BeneficiariosEditor } from './catalogs/beneficiarios.editor';
import { GruposBeneficiarioManager } from './catalogs/grupos-beneficiario.manager';
import { GruposBeneficiarioEditor } from './catalogs/grupos-beneficiario.editor';
import { CajasManager } from './catalogs/cajas.manager';
import { CajasEditor } from './catalogs/cajas.editor';
import { ChequerasManager } from './catalogs/chequeras.manager';
import { ChequerasEditor } from './catalogs/chequeras.editor';
import { PagosTipoManager } from './catalogs/pagos-tipo.manager';
import { PagosTipoEditor } from './catalogs/pagos-tipo.editor';

import { OperacionesTiposManager } from './catalogs/operaciones-tipos.manager';
import { OperacionesTiposEditor } from './catalogs/operaciones-tipos.editor';

import { EmpresasManager } from './catalogs/empresas.manager';
import { EmpresasEditor } from './catalogs/empresas.editor';
import { SucursalesManager } from './catalogs/sucursales.manager';
import { SucursalesEditor } from './catalogs/sucursales.editor';


///////////////////////////// Document /////////////////////////////////
import { ContrarecibosTitManager } from './documents/contrarecibos-tit.manager';
import { ContrarecibosTitEditor } from './documents/contrarecibos-tit.editor';
import { CuentasPorPagarManager } from './documents/cuentas-por-pagar.manager';
import { CuentasPorPagarEditor } from './documents/cuentas-por-pagar.editor';
import { PagosManager } from './documents/pagos.manager';
import { PagosEditor } from './documents/pagos.editor';
import { EstadosCuentaBancariosManager } from './documents/estados-cuenta-bancarios.manager';
import { EstadosCuentaBancariosEditor } from './documents/estados-cuenta-bancarios.editor';

//Parts//
import { CajaMiembrosComponent } from './parts/caja-miembros.component';
import { OperacionesTiposMiembrosComponent } from './parts/operaciones-tipos-miembros.component';
///////////////////////////// Rutas /////////////////////////////////


/////////////////////////////  Componentes //////////////////////////////
import * as fromCatalogs from './catalogs';
import * as fromDocuments from './documents';
//import * as fromComponents from './components';
import * as fromParts from './parts';


const routes: Routes = [
    {
        path: '',
        canActivateChild: [AuthGuard],
        children: [
            //CATALOGS
            { path: 'beneficiarios/:proceso', component: BeneficiariosManager },
            { path: 'beneficiarios/:proceso/:id', component: BeneficiariosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'grupo-beneficiarios/:proceso', component: GruposBeneficiarioManager },
            { path: 'grupo-beneficiarios/:proceso/:id', component: GruposBeneficiarioEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'cajas/:proceso', component: CajasManager },
            { path: 'cajas/:proceso/:id', component: CajasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'cajas/:proceso/:id/members', component: CajaMiembrosComponent },
            { path: 'chequeras/:proceso', component: ChequerasManager },
            { path: 'chequeras/:proceso/:id', component: ChequerasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'pagos-tipo/:proceso', component: PagosTipoManager },
            { path: 'pagos-tipo/:proceso/:id', component: PagosTipoEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'operaciones-tipo/:proceso', component: OperacionesTiposManager },
            { path: 'operaciones-tipo/:proceso/:id', component: OperacionesTiposEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'operaciones-tipo/:proceso/:id/members', component: OperacionesTiposMiembrosComponent },

            { path: 'conceptos/:proceso', component: fromCatalogs.ConceptosManager },
            { path: 'conceptos/:proceso/:id', component: fromCatalogs.ConceptosEditor, canDeactivate: [EditorDeactivateGuard] },


            { path: 'empresas-tesoreria/:proceso', component: EmpresasManager },
            { path: 'empresas-tesoreria/:proceso/:id', component: EmpresasEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'sucursales-tesoreria/:proceso', component: SucursalesManager },
            { path: 'sucursales-tesoreria/:proceso/:idEmpresa/:id', component: SucursalesEditor, canDeactivate: [EditorDeactivateGuard] },

           //DOCUMENTS
            { path: 'cuentas-por-pagar/:proceso', component: CuentasPorPagarManager },
            { path: 'cuentas-por-pagar/:proceso/:id', component: CuentasPorPagarEditor, canDeactivate: [EditorDeactivateGuard] },

            //DOCUMENTS
            { path: 'contra-recibo/:proceso', component: ContrarecibosTitManager },
            { path: 'contra-recibo/:proceso/:id', component: ContrarecibosTitEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'pagos/:proceso', component: PagosManager },
            { path: 'pagos/:proceso/:id', component: PagosEditor, canDeactivate: [EditorDeactivateGuard] },

            { path: 'estados-cuenta-bancarios/:proceso', component: EstadosCuentaBancariosManager },
            { path: 'estados-cuenta-bancarios/:proceso/:id', component: EstadosCuentaBancariosEditor, canDeactivate: [EditorDeactivateGuard] },
            { path: 'estados-cuenta-bancarios-conciliar/:proceso/:idEstadoCuenta', component: fromDocuments.EstadoCuentaBancarioConciliar }


        ]
    },
];



@NgModule({
    imports: [
        RouterModule.forChild(routes)
    ],
    exports: [
        RouterModule
    ]
})
export class TreasuryRoutingModule { }

