﻿//Frameworks
import { Component, OnInit, ViewChild, Input, forwardRef, ChangeDetectorRef, } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { FormBuilder, Validators, FormControl } from '@angular/forms';
import { Location } from '@angular/common';
//Wijmo
import { Control } from 'wijmo/wijmo';
import { WjInputModule } from 'wijmo/wijmo.angular2.input';
import { Menu } from 'wijmo/wijmo.input';

//Azteca Kernel
import { Context, BaseComponent, DocumentService, DocumentItem, DocumentStatus, DocumentTypeService, DocumentTypeInfo } from 'framework3/core';
import { AztecaValidators, MenuItem, ToolbarButton } from 'framework3/controls';
import { DocumentEditor, EditableListComponent, EditorEventArgs, WorkflowEventArgs } from 'framework3/editors';
import { SearchRequest, SearchCondition, FilterGroup } from 'framework3/search';
//Locales
import { PolizaInfo, PolizaItem } from '../models/poliza';
import { PolizaPerfilItem } from '../models/poliza-perfil';
import { PolizaDetail } from '../models/poliza-detail';
import { PolizaService } from '../services/poliza.service';
import { PolizaDetailEditor } from './parts/polizas-detail.editor';
import { PolizaTipo } from '../models/poliza-tipo';

import { DivisaItem } from '../models/divisa';
import { DivisaService } from '../services/divisa.service';
import { PolizasAsociacionesComponent } from './parts/polizas-asociaciones.component';


@Component({
    selector: 'contabilidad-polizas-editor',
    templateUrl: './polizas.editor.html',
    providers: [
        {
            provide: BaseComponent, useExisting: forwardRef(() => PolizasEditor)
        },
        {
            provide: DocumentService, useExisting: PolizaService
        }
    ]
})
export class PolizasEditor extends DocumentEditor {
    poliza = new PolizaInfo();
    cargos: number;
    abonos: number;
    polizaT = new PolizaTipo();
    

    //
    @ViewChild('detailsList') detailsList: EditableListComponent;
    @ViewChild('detailsEditor') detailsEditor: PolizaDetailEditor;
    @ViewChild('dlgAsociaciones') dlgAsociaciones: PolizasAsociacionesComponent;
    //
    divisas: DivisaItem[] = [];
    polizasPerfiles: PolizaPerfilItem[] = [];
    polizasTipos: PolizaTipo[] = [];

    constructor(
        protected context: Context,
        protected router: Router,
        protected route: ActivatedRoute,
        private location: Location,
        private formBuilder: FormBuilder,
        protected documentTypeService: DocumentTypeService,
        private polizaService: PolizaService,
        private divisaService: DivisaService) {


        super(context, router, route, documentTypeService);

        this.documentForm = formBuilder.group({
            'concepto': [''],
            'divisa': [''],
            'idPolizaPerfil': [0],
            'afectacion': ['', Validators.required],
            'cargos': [''],
            'abonos': [''],
            'idPolizaType':[''],
        });
       
    }

    validatePerfil(identity: string, control: FormControl) {
        if (identity != "")
            return null;
        else
            return 'Asigne el perfil';
    }

    onConfigureDocument(docType: DocumentTypeInfo) {        
        this.applyPolicies();  
        this.loadPerfilesManuales();
        this.loadPolizaTipo();
    }

    onCreatingDocument() {
        this.polizaService.create().then(doc => {
            this.poliza = doc;
            this.poliza.idSucursal = this.context.app.defaultSite.identity;
            console.log("Creandon documebti");
        })
            .catch(error => this.handleError(error));
    }

    onViewingDocument(id: string) {
        this.context.app.showSpinner();

        this.polizaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.poliza = doc;
                this.setReadonlyMode();
            })
            .catch(error => this.handleError(error));
    }

    onCloningDocument(id: string) {
        this.context.app.showSpinner();
        //
        this.polizaService.clone(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.poliza = doc;
                this.detailsEditor.clientIndexStart = this.poliza.details.length;
            })
            .catch(error => this.handleError(error));
    }

    onLoadingDocument(id: string) {
        this.context.app.showSpinner();

        this.polizaService.getDetail(id)
            .then(doc => {
                this.context.app.hideSpinner();
                this.poliza = doc;
                this.detailsEditor.clientIndexStart = this.poliza.details.length;
                if (this.poliza.status != DocumentStatus.DRAFT)
                    this.setReadonlyMode();
                this.sumarizacion();
            })
            .catch(error => this.handleError(error));
    }

    onSavingDocument(args: EditorEventArgs) {
        this.poliza.idPolizaTipo = this.polizaT.idPolizaTipo;
        this.context.app.showSpinner();

        if (this.poliza.idDivisa == 0 || this.poliza.idDivisa == undefined) this.poliza.idDivisa = 100

        if (this.polizaT.idPolizaTipo == 0 || this.polizaT.idPolizaTipo == undefined)
        {
            this.context.app.hideSpinner();
            this.showAlert("No se ha seleccionado un tipo de poliza", 'danger');
        }
        
        if (this.poliza.idPolizaOrigen == 0 || this.poliza.idPolizaOrigen == undefined) this.poliza.idPolizaOrigen = 1 //Manual

        if (this.poliza.idPolizaPerfil == 0 || this.poliza.idPolizaPerfil == undefined)
        {
            this.context.app.hideSpinner();
            this.showAlert("No se ha establecido el perfil de poliza",'danger');
        }
       
        else
            this.polizaService.update(this.poliza)
                .then(updatedDoc => {
                    this.context.app.hideSpinner();
                    //Si es un elemento nuevo y no se cerrará el editor, navegar a nueva ruta
                    this.reset();
                    if (!args.closeEditor && this.poliza.identity == '') {
                        this.router.navigate(['../', updatedDoc.identity], { relativeTo: this.route });
                    }
                    else {
                        if (args.closeEditor)
                            this.router.navigate(['../'], { relativeTo: this.route });
                        else
                            this.showAlert('La información ha sido actualizada con éxito', 'success');
                    }
                    this.poliza = updatedDoc;
                })
                .catch(error => this.handleError(error));
    }

    onApplyingNextStep(args: WorkflowEventArgs) {
        this.context.app.showSpinner();

        this.polizaService.nextStep(args.document.id, args.step.idWorkflowStep, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(args.document.id);
            })
            .catch(error => this.handleError(error));
    }

    onApplyingPreviousStep(document: DocumentItem) {
        this.context.app.showSpinner();

        this.polizaService.previousStep(document.id, null)
            .then(_ => {
                //Volver a cargar documento
                this.onViewingDocument(document.id);
            })
            .catch(error => this.handleError(error));
    }

    onTabSelected(args: any) {
        if (args.heading) {
            //Al seleccionar la pestaña de historial, realizar carga de información
            if (args.heading == 'Historial') {
                this.loadHistory(this.poliza.identity);
            }
            //Actualizar controles de wijmo
            Control.invalidateAll();
        }
    }


    ///////////// DETALLE DEL DOCUMENTO /////////////////////////

    onAddingDetail() {
        //let updatedDetails: PolizaDetail[]
        //updatedDetails = this.poliza.details;        
        //let detail: PolizaDetail = new PolizaDetail();        
        //this.poliza.details.push(detail)
        //this.poliza.details = updatedDetails;
        this.detailsEditor.detail = new PolizaDetail();
        this.detailsEditor.detail.consecutivo = this.poliza.details.length + 1; 
        this.detailsEditor.show();
    }

    onEditingDetail(detail: PolizaDetail) {
        this.detailsEditor.detail = detail;
        this.detailsEditor.show();
    }

    onDeletingDetail(detail: PolizaDetail) {
        this.detailsList.deleteItem(detail);
    }

    onListItemsUpdated(updatedDetails: PolizaDetail[]) {
        this.poliza.details = updatedDetails;
        this.sumarizacion();
    }

    onSavingDetail(detail: PolizaDetail) {
        this.detailsList.updateItem(detail);
    }
       

    sumarizacion(){
        this.cargos = 0;
        this.abonos = 0;
        this.poliza.details.forEach(detail => {
            if (detail.afectador == "C")
                this.cargos += detail.importe;
            else
                this.abonos += detail.importe;
        });
    }

    onMenuItemClick(item: MenuItem) {
        if (item.name == "ASOC") {
            var itemDetail: PolizaDetail = (this.detailsList.currentItem as PolizaDetail); 
            this.dlgAsociaciones.show(this.poliza.identity, this.poliza.fechaSys, itemDetail.idCuenta, itemDetail.consecutivo);
        }
    }

    loadPerfilesManuales() {
        this.context.app.showSpinner();
        this.polizaService.loadPerfiles(this.context.app.defaultSite.identity, this.context.session.user.identity)
            .then(items => {
                this.context.app.hideSpinner();
                this.polizasPerfiles = items.filter(filter=> filter.modalidad=='M');
            })
            .catch(error => this.handleError(error));
    }

    loadPolizaTipo() {
        this.context.app.showSpinner();
        this.polizaService.loadPolizaTipo()
            .then(result => {
                this.context.app.hideSpinner();
                this.polizasTipos = result;
                
            })
    }

}